/*
 ***********************************************************************************************************************
 *
 *  Copyright (c) 2023-2024 Advanced Micro Devices, Inc. All Rights Reserved.
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in all
 *  copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 *  SOFTWARE.
 *
 **********************************************************************************************************************/

#pragma once

// Enumeration of image number formats.
// This enum is auto-generated using convert_pal_format_to_rmt.py and
// should match exactly ChNumFormat in palFormat.h.
enum RMT_NUM_FORMAT
{
    RMT_FORMAT_UNDEFINED                               = 0,
    RMT_FORMAT_X1_UNORM                                = 1,
    RMT_FORMAT_X1_USCALED                              = 2,
    RMT_FORMAT_X4Y4_UNORM                              = 3,
    RMT_FORMAT_X4Y4_USCALED                            = 4,
    RMT_FORMAT_L4A4_UNORM                              = 5,
    RMT_FORMAT_X4Y4Z4W4_UNORM                          = 6,
    RMT_FORMAT_X4Y4Z4W4_USCALED                        = 7,
    RMT_FORMAT_X5Y6Z5_UNORM                            = 8,
    RMT_FORMAT_X5Y6Z5_USCALED                          = 9,
    RMT_FORMAT_X5Y5Z5W1_UNORM                          = 10,
    RMT_FORMAT_X5Y5Z5W1_USCALED                        = 11,
    RMT_FORMAT_X1Y5Z5W5_UNORM                          = 12,
    RMT_FORMAT_X1Y5Z5W5_USCALED                        = 13,
    RMT_FORMAT_X8_UNORM                                = 14,
    RMT_FORMAT_X8_SNORM                                = 15,
    RMT_FORMAT_X8_USCALED                              = 16,
    RMT_FORMAT_X8_SSCALED                              = 17,
    RMT_FORMAT_X8_UINT                                 = 18,
    RMT_FORMAT_X8_SINT                                 = 19,
    RMT_FORMAT_X8_SRGB                                 = 20,
    RMT_FORMAT_A8_UNORM                                = 21,
    RMT_FORMAT_L8_UNORM                                = 22,
    RMT_FORMAT_P8_UNORM                                = 23,
    RMT_FORMAT_X8Y8_UNORM                              = 24,
    RMT_FORMAT_X8Y8_SNORM                              = 25,
    RMT_FORMAT_X8Y8_USCALED                            = 26,
    RMT_FORMAT_X8Y8_SSCALED                            = 27,
    RMT_FORMAT_X8Y8_UINT                               = 28,
    RMT_FORMAT_X8Y8_SINT                               = 29,
    RMT_FORMAT_X8Y8_SRGB                               = 30,
    RMT_FORMAT_L8A8_UNORM                              = 31,
    RMT_FORMAT_X8Y8Z8W8_UNORM                          = 32,
    RMT_FORMAT_X8Y8Z8W8_SNORM                          = 33,
    RMT_FORMAT_X8Y8Z8W8_USCALED                        = 34,
    RMT_FORMAT_X8Y8Z8W8_SSCALED                        = 35,
    RMT_FORMAT_X8Y8Z8W8_UINT                           = 36,
    RMT_FORMAT_X8Y8Z8W8_SINT                           = 37,
    RMT_FORMAT_X8Y8Z8W8_SRGB                           = 38,
    RMT_FORMAT_U8V8_SNORM_L8W8_UNORM                   = 39,
    RMT_FORMAT_X10Y11Z11_FLOAT                         = 40,
    RMT_FORMAT_X11Y11Z10_FLOAT                         = 41,
    RMT_FORMAT_X10Y10Z10W2_UNORM                       = 42,
    RMT_FORMAT_X10Y10Z10W2_SNORM                       = 43,
    RMT_FORMAT_X10Y10Z10W2_USCALED                     = 44,
    RMT_FORMAT_X10Y10Z10W2_SSCALED                     = 45,
    RMT_FORMAT_X10Y10Z10W2_UINT                        = 46,
    RMT_FORMAT_X10Y10Z10W2_SINT                        = 47,
    RMT_FORMAT_X10Y10Z10W2BIAS_UNORM                   = 48,
    RMT_FORMAT_U10V10W10_SNORM_A2_UNORM                = 49,
    RMT_FORMAT_X16_UNORM                               = 50,
    RMT_FORMAT_X16_SNORM                               = 51,
    RMT_FORMAT_X16_USCALED                             = 52,
    RMT_FORMAT_X16_SSCALED                             = 53,
    RMT_FORMAT_X16_UINT                                = 54,
    RMT_FORMAT_X16_SINT                                = 55,
    RMT_FORMAT_X16_FLOAT                               = 56,
    RMT_FORMAT_L16_UNORM                               = 57,
    RMT_FORMAT_X16Y16_UNORM                            = 58,
    RMT_FORMAT_X16Y16_SNORM                            = 59,
    RMT_FORMAT_X16Y16_USCALED                          = 60,
    RMT_FORMAT_X16Y16_SSCALED                          = 61,
    RMT_FORMAT_X16Y16_UINT                             = 62,
    RMT_FORMAT_X16Y16_SINT                             = 63,
    RMT_FORMAT_X16Y16_FLOAT                            = 64,
    RMT_FORMAT_X16Y16Z16W16_UNORM                      = 65,
    RMT_FORMAT_X16Y16Z16W16_SNORM                      = 66,
    RMT_FORMAT_X16Y16Z16W16_USCALED                    = 67,
    RMT_FORMAT_X16Y16Z16W16_SSCALED                    = 68,
    RMT_FORMAT_X16Y16Z16W16_UINT                       = 69,
    RMT_FORMAT_X16Y16Z16W16_SINT                       = 70,
    RMT_FORMAT_X16Y16Z16W16_FLOAT                      = 71,
    RMT_FORMAT_X32_UINT                                = 72,
    RMT_FORMAT_X32_SINT                                = 73,
    RMT_FORMAT_X32_FLOAT                               = 74,
    RMT_FORMAT_X32Y32_UINT                             = 75,
    RMT_FORMAT_X32Y32_SINT                             = 76,
    RMT_FORMAT_X32Y32_FLOAT                            = 77,
    RMT_FORMAT_X32Y32Z32_UINT                          = 78,
    RMT_FORMAT_X32Y32Z32_SINT                          = 79,
    RMT_FORMAT_X32Y32Z32_FLOAT                         = 80,
    RMT_FORMAT_X32Y32Z32W32_UINT                       = 81,
    RMT_FORMAT_X32Y32Z32W32_SINT                       = 82,
    RMT_FORMAT_X32Y32Z32W32_FLOAT                      = 83,
    RMT_FORMAT_D16_UNORM_S8_UINT                       = 84,
    RMT_FORMAT_D32_FLOAT_S8_UINT                       = 85,
    RMT_FORMAT_X9Y9Z9E5_FLOAT                          = 86,
    RMT_FORMAT_BC1_UNORM                               = 87,
    RMT_FORMAT_BC1_SRGB                                = 88,
    RMT_FORMAT_BC2_UNORM                               = 89,
    RMT_FORMAT_BC2_SRGB                                = 90,
    RMT_FORMAT_BC3_UNORM                               = 91,
    RMT_FORMAT_BC3_SRGB                                = 92,
    RMT_FORMAT_BC4_UNORM                               = 93,
    RMT_FORMAT_BC4_SNORM                               = 94,
    RMT_FORMAT_BC5_UNORM                               = 95,
    RMT_FORMAT_BC5_SNORM                               = 96,
    RMT_FORMAT_BC6_UFLOAT                              = 97,
    RMT_FORMAT_BC6_SFLOAT                              = 98,
    RMT_FORMAT_BC7_UNORM                               = 99,
    RMT_FORMAT_BC7_SRGB                                = 100,
    RMT_FORMAT_ETC2X8Y8Z8_UNORM                        = 101,
    RMT_FORMAT_ETC2X8Y8Z8_SRGB                         = 102,
    RMT_FORMAT_ETC2X8Y8Z8W1_UNORM                      = 103,
    RMT_FORMAT_ETC2X8Y8Z8W1_SRGB                       = 104,
    RMT_FORMAT_ETC2X8Y8Z8W8_UNORM                      = 105,
    RMT_FORMAT_ETC2X8Y8Z8W8_SRGB                       = 106,
    RMT_FORMAT_ETC2X11_UNORM                           = 107,
    RMT_FORMAT_ETC2X11_SNORM                           = 108,
    RMT_FORMAT_ETC2X11Y11_UNORM                        = 109,
    RMT_FORMAT_ETC2X11Y11_SNORM                        = 110,
    RMT_FORMAT_ASTCLDR4X4_UNORM                        = 111,
    RMT_FORMAT_ASTCLDR4X4_SRGB                         = 112,
    RMT_FORMAT_ASTCLDR5X4_UNORM                        = 113,
    RMT_FORMAT_ASTCLDR5X4_SRGB                         = 114,
    RMT_FORMAT_ASTCLDR5X5_UNORM                        = 115,
    RMT_FORMAT_ASTCLDR5X5_SRGB                         = 116,
    RMT_FORMAT_ASTCLDR6X5_UNORM                        = 117,
    RMT_FORMAT_ASTCLDR6X5_SRGB                         = 118,
    RMT_FORMAT_ASTCLDR6X6_UNORM                        = 119,
    RMT_FORMAT_ASTCLDR6X6_SRGB                         = 120,
    RMT_FORMAT_ASTCLDR8X5_UNORM                        = 121,
    RMT_FORMAT_ASTCLDR8X5_SRGB                         = 122,
    RMT_FORMAT_ASTCLDR8X6_UNORM                        = 123,
    RMT_FORMAT_ASTCLDR8X6_SRGB                         = 124,
    RMT_FORMAT_ASTCLDR8X8_UNORM                        = 125,
    RMT_FORMAT_ASTCLDR8X8_SRGB                         = 126,
    RMT_FORMAT_ASTCLDR10X5_UNORM                       = 127,
    RMT_FORMAT_ASTCLDR10X5_SRGB                        = 128,
    RMT_FORMAT_ASTCLDR10X6_UNORM                       = 129,
    RMT_FORMAT_ASTCLDR10X6_SRGB                        = 130,
    RMT_FORMAT_ASTCLDR10X8_UNORM                       = 131,
    RMT_FORMAT_ASTCLDR10X8_SRGB                        = 132,
    RMT_FORMAT_ASTCLDR10X10_UNORM                      = 133,
    RMT_FORMAT_ASTCLDR10X10_SRGB                       = 134,
    RMT_FORMAT_ASTCLDR12X10_UNORM                      = 135,
    RMT_FORMAT_ASTCLDR12X10_SRGB                       = 136,
    RMT_FORMAT_ASTCLDR12X12_UNORM                      = 137,
    RMT_FORMAT_ASTCLDR12X12_SRGB                       = 138,
    RMT_FORMAT_ASTCHDR4X4_FLOAT                        = 139,
    RMT_FORMAT_ASTCHDR5X4_FLOAT                        = 140,
    RMT_FORMAT_ASTCHDR5X5_FLOAT                        = 141,
    RMT_FORMAT_ASTCHDR6X5_FLOAT                        = 142,
    RMT_FORMAT_ASTCHDR6X6_FLOAT                        = 143,
    RMT_FORMAT_ASTCHDR8X5_FLOAT                        = 144,
    RMT_FORMAT_ASTCHDR8X6_FLOAT                        = 145,
    RMT_FORMAT_ASTCHDR8X8_FLOAT                        = 146,
    RMT_FORMAT_ASTCHDR10X5_FLOAT                       = 147,
    RMT_FORMAT_ASTCHDR10X6_FLOAT                       = 148,
    RMT_FORMAT_ASTCHDR10X8_FLOAT                       = 149,
    RMT_FORMAT_ASTCHDR10X10_FLOAT                      = 150,
    RMT_FORMAT_ASTCHDR12X10_FLOAT                      = 151,
    RMT_FORMAT_ASTCHDR12X12_FLOAT                      = 152,
    RMT_FORMAT_X8Y8_Z8Y8_UNORM                         = 153,
    RMT_FORMAT_X8Y8_Z8Y8_USCALED                       = 154,
    RMT_FORMAT_Y8X8_Y8Z8_UNORM                         = 155,
    RMT_FORMAT_Y8X8_Y8Z8_USCALED                       = 156,
    RMT_FORMAT_AYUV                                    = 157,
    RMT_FORMAT_UYVY                                    = 158,
    RMT_FORMAT_VYUY                                    = 159,
    RMT_FORMAT_YUY2                                    = 160,
    RMT_FORMAT_YVY2                                    = 161,
    RMT_FORMAT_YV12                                    = 162,
    RMT_FORMAT_NV11                                    = 163,
    RMT_FORMAT_NV12                                    = 164,
    RMT_FORMAT_NV21                                    = 165,
    RMT_FORMAT_P016                                    = 166,
    RMT_FORMAT_P010                                    = 167,
    RMT_FORMAT_P210                                    = 168,
    RMT_FORMAT_X8_MM_UNORM                             = 169,
    RMT_FORMAT_X8_MM_UINT                              = 170,
    RMT_FORMAT_X8Y8_MM_UNORM                           = 171,
    RMT_FORMAT_X8Y8_MM_UINT                            = 172,
    RMT_FORMAT_X16_MM10_UNORM                          = 173,
    RMT_FORMAT_X16_MM10_UINT                           = 174,
    RMT_FORMAT_X16Y16_MM10_UNORM                       = 175,
    RMT_FORMAT_X16Y16_MM10_UINT                        = 176,
    RMT_FORMAT_P208                                    = 177,
    RMT_FORMAT_X16_MM12_UNORM                          = 178,
    RMT_FORMAT_X16_MM12_UINT                           = 179,
    RMT_FORMAT_X16Y16_MM12_UNORM                       = 180,
    RMT_FORMAT_X16Y16_MM12_UINT                        = 181,
    RMT_FORMAT_P012                                    = 182,
    RMT_FORMAT_P212                                    = 183,
    RMT_FORMAT_P412                                    = 184,
    RMT_FORMAT_X10Y10Z10W2_FLOAT                       = 185,
    RMT_FORMAT_Y216                                    = 186,
    RMT_FORMAT_Y210                                    = 187,
    RMT_FORMAT_Y416                                    = 188,
    RMT_FORMAT_Y410                                    = 189,
    Count,
};
