/*
 ***********************************************************************************************************************
 *
 *  Copyright (c) 2025 Advanced Micro Devices, Inc. All Rights Reserved.
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in all
 *  copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 *  SOFTWARE.
 *
 **********************************************************************************************************************/

#pragma once

namespace Pal
{
namespace Gfx12
{
inline namespace Chip
{

typedef enum IT_OpCodeType {
    IT_NOP                                             = 0x00000010,
    IT_SET_BASE                                        = 0x00000011,
    IT_CLEAR_STATE                                     = 0x00000012,
    IT_INDEX_BUFFER_SIZE                               = 0x00000013,
    IT_DISPATCH_DIRECT                                 = 0x00000015,
    IT_DISPATCH_INDIRECT                               = 0x00000016,
    IT_INDIRECT_BUFFER_END                             = 0x00000017,
    IT_INDIRECT_BUFFER_CNST_END                        = 0x00000019,
    IT_RESERVED_1C                                     = 0x0000001c,
    IT_ATOMIC_GDS                                      = 0x0000001d,
    IT_ATOMIC_MEM                                      = 0x0000001e,
    IT_OCCLUSION_QUERY                                 = 0x0000001f,
    IT_SET_PREDICATION                                 = 0x00000020,
    IT_REG_RMW                                         = 0x00000021,
    IT_COND_EXEC                                       = 0x00000022,
    IT_PRED_EXEC                                       = 0x00000023,
    IT_DRAW_INDIRECT                                   = 0x00000024,
    IT_DRAW_INDEX_INDIRECT                             = 0x00000025,
    IT_INDEX_BASE                                      = 0x00000026,
    IT_DRAW_INDEX_2                                    = 0x00000027,
    IT_CONTEXT_CONTROL                                 = 0x00000028,
    IT_RESERVED_29                                     = 0x00000029,
    IT_INDEX_TYPE                                      = 0x0000002a,
    IT_RESERVED_2B                                     = 0x0000002b,
    IT_DRAW_INDIRECT_MULTI                             = 0x0000002c,
    IT_DRAW_INDEX_AUTO                                 = 0x0000002d,
    IT_RESERVED_2E                                     = 0x0000002e,
    IT_NUM_INSTANCES                                   = 0x0000002f,
    IT_DRAW_INDEX_MULTI_AUTO                           = 0x00000030,
    IT_RESERVED_31                                     = 0x00000031,
    IT_INDIRECT_BUFFER_PRIV                            = 0x00000032,
    IT_INDIRECT_BUFFER_CNST                            = 0x00000033,
    IT_COND_INDIRECT_BUFFER_CNST                       = 0x00000033,
    IT_STRMOUT_BUFFER_UPDATE                           = 0x00000034,
    IT_DRAW_INDEX_OFFSET_2                             = 0x00000035,
    IT_DRAW_PREAMBLE                                   = 0x00000036,
    IT_WRITE_DATA                                      = 0x00000037,
    IT_DRAW_INDEX_INDIRECT_MULTI                       = 0x00000038,
    IT_MEM_SEMAPHORE                                   = 0x00000039,
    IT_COPY_DW                                         = 0x0000003b,
    IT_WAIT_REG_MEM                                    = 0x0000003c,
    IT_PERF_COUNTER_WINDOW                             = 0x0000003d,
    IT_RESERVED_3E                                     = 0x0000003e,
    IT_INDIRECT_BUFFER                                 = 0x0000003f,
    IT_COND_INDIRECT_BUFFER                            = 0x0000003f,
    IT_COPY_DATA                                       = 0x00000040,
    IT_CP_DMA                                          = 0x00000041,
    IT_PFP_SYNC_ME                                     = 0x00000042,
    IT_SURFACE_SYNC                                    = 0x00000043,
    IT_ME_INITIALIZE                                   = 0x00000044,
    IT_COND_WRITE                                      = 0x00000045,
    IT_EVENT_WRITE                                     = 0x00000046,
    IT_EVENT_WRITE_EOP                                 = 0x00000047,
    IT_EVENT_WRITE_EOS                                 = 0x00000048,
    IT_RELEASE_MEM                                     = 0x00000049,
    IT_PREAMBLE_CNTL                                   = 0x0000004a,
    IT_DISPATCH_MESH_INDIRECT_MULTI                    = 0x0000004c,
    IT_DISPATCH_TASKMESH_GFX                           = 0x0000004d,
    IT_DISPATCH_MESH_DIRECT                            = 0x0000004e,
    IT_RESERVED_4F                                     = 0x0000004f,
    IT_DMA_DATA                                        = 0x00000050,
    IT_CONTEXT_REG_RMW                                 = 0x00000051,
    IT_GFX_CNTX_UPDATE                                 = 0x00000052,
    IT_BLK_CNTX_UPDATE                                 = 0x00000053,
    IT_PERFMON_CONTROL                                 = 0x00000054,
    IT_INCR_UPDT_STATE                                 = 0x00000055,
    IT_RESERVED_57                                     = 0x00000057,
    IT_ACQUIRE_MEM                                     = 0x00000058,
    IT_REWIND                                          = 0x00000059,
    IT_INTERRUPT                                       = 0x0000005a,
    IT_GEN_PDEPTE                                      = 0x0000005b,
    IT_INDIRECT_BUFFER_PASID                           = 0x0000005c,
    IT_PRIME_UTCL2                                     = 0x0000005d,
    IT_LOAD_UCONFIG_REG                                = 0x0000005e,
    IT_LOAD_SH_REG                                     = 0x0000005f,
    IT_LOAD_CONFIG_REG                                 = 0x00000060,
    IT_LOAD_CONTEXT_REG                                = 0x00000061,
    IT_LOAD_COMPUTE_STATE                              = 0x00000062,
    IT_LOAD_SH_REG_INDEX                               = 0x00000063,
    IT_LOAD_UCONFIG_REG_INDEX                          = 0x00000064,
    IT_RESERVED_65                                     = 0x00000065,
    IT_RESERVED_66                                     = 0x00000066,
    IT_RESERVED_67                                     = 0x00000067,
    IT_SET_CONFIG_REG                                  = 0x00000068,
    IT_SET_CONTEXT_REG                                 = 0x00000069,
    IT_SET_CONTEXT_REG_INDEX                           = 0x0000006a,
    IT_DRAW_RESERVED0                                  = 0x0000006b,
    IT_DRAW_RESERVED1                                  = 0x0000006c,
    IT_DRAW_RESERVED2                                  = 0x0000006d,
    IT_DRAW_RESERVED3                                  = 0x0000006e,
    IT_RESERVED_70                                     = 0x00000070,
    IT_SET_VGPR_REG_DI_MULTI                           = 0x00000071,
    IT_SET_SH_REG_DI                                   = 0x00000072,
    IT_SET_CONTEXT_REG_INDIRECT                        = 0x00000073,
    IT_SET_SH_REG_DI_MULTI                             = 0x00000074,
    IT_RESERVED_75                                     = 0x00000075,
    IT_SET_SH_REG                                      = 0x00000076,
    IT_SET_SH_REG_OFFSET                               = 0x00000077,
    IT_SET_QUEUE_REG                                   = 0x00000078,
    IT_SET_UCONFIG_REG                                 = 0x00000079,
    IT_SET_UCONFIG_REG_INDEX                           = 0x0000007a,
    IT_FORWARD_HEADER                                  = 0x0000007c,
    IT_SCRATCH_RAM_WRITE                               = 0x0000007d,
    IT_SCRATCH_RAM_READ                                = 0x0000007e,
    IT_RESERVED_7F                                     = 0x0000007f,
    IT_LOAD_CONST_RAM                                  = 0x00000080,
    IT_WRITE_CONST_RAM                                 = 0x00000081,
    IT_RESERVED_82                                     = 0x00000082,
    IT_DUMP_CONST_RAM                                  = 0x00000083,
    IT_INCREMENT_CE_COUNTER                            = 0x00000084,
    IT_INCREMENT_DE_COUNTER                            = 0x00000085,
    IT_WAIT_ON_CE_COUNTER                              = 0x00000086,
    IT_RESERVED_87                                     = 0x00000087,
    IT_WAIT_ON_DE_COUNTER_DIFF                         = 0x00000088,
    IT_SWITCH_BUFFER                                   = 0x0000008b,
    IT_RESERVED_8C                                     = 0x0000008c,
    IT_RESERVED_8D                                     = 0x0000008d,
    IT_RESERVED_8E                                     = 0x0000008e,
    IT_RESERVED_8F                                     = 0x0000008f,
    IT_FRAME_CONTROL                                   = 0x00000090,
    IT_INDEX_ATTRIBUTES_INDIRECT                       = 0x00000091,
    IT_WAIT_FOR_WRITE_CONFIRM                          = 0x00000092,
    IT_WAIT_REG_MEM64                                  = 0x00000093,
    IT_COND_PREEMPT                                    = 0x00000094,
    IT_HDP_FLUSH                                       = 0x00000095,
    IT_INVALIDATE_TLBS                                 = 0x00000098,
    IT_RESERVED_99                                     = 0x00000099,
    IT_DMA_DATA_FILL_MULTI                             = 0x0000009a,
    IT_SET_SH_REG_INDEX                                = 0x0000009b,
    IT_DRAW_INDIRECT_COUNT_MULTI                       = 0x0000009c,
    IT_DRAW_INDEX_INDIRECT_COUNT_MULTI                 = 0x0000009d,
    IT_DUMP_CONST_RAM_OFFSET                           = 0x0000009e,
    IT_LOAD_CONTEXT_REG_INDEX                          = 0x0000009f,
    IT_SET_RESOURCES                                   = 0x000000a0,
    IT_MAP_PROCESS                                     = 0x000000a1,
    IT_MAP_QUEUES                                      = 0x000000a2,
    IT_UNMAP_QUEUES                                    = 0x000000a3,
    IT_QUERY_STATUS                                    = 0x000000a4,
    IT_RUN_LIST                                        = 0x000000a5,
    IT_MAP_PROCESS_VM                                  = 0x000000a6,
    IT_DISPATCH_DIRECT_INTERLEAVED                     = 0x000000a7,
    IT_DISPATCH_INDIRECT_INTERLEAVED                   = 0x000000a8,
    IT_DISPATCH_TASK_STATE_INIT                        = 0x000000a9,
    IT_DISPATCH_TASKMESH_DIRECT_ACE                    = 0x000000aa,
    IT_CONTEXT_PUSH                                    = 0x000000ab,
    IT_CONTEXT_POP                                     = 0x000000ac,
    IT_DISPATCH_TASKMESH_INDIRECT_MULTI_ACE            = 0x000000ad,
    IT_EXECUTE_INDIRECT                                = 0x000000ae,
    IT_BUILD_UNTYPED_SRD                               = 0x000000af,
    IT_EVENT_WRITE_ZPASS                               = 0x000000b1,
    IT_RESERVED_B1                                     = 0x000000b1,
    IT_TIMESTAMP                                       = 0x000000b2,
    IT_UBER_DRAWS                                      = 0x000000b3,
    IT_RESERVED_B3                                     = 0x000000b3,
    IT_EXECUTE_INDIRECT_V2                             = 0x000000b4,
    IT_RESERVED_B5                                     = 0x000000b5,
    IT_MARKER                                          = 0x000000b7,
    IT_SET_CONTEXT_REG_PAIRS                           = 0x000000b8,
    IT_SET_CONTEXT_REG_PAIRS_PACKED                    = 0x000000b9,
    IT_SET_SH_REG_PAIRS                                = 0x000000ba,
    IT_SET_SH_REG_PAIRS_PACKED                         = 0x000000bc,
    IT_SET_SH_REG_PAIRS_PACKED_N                       = 0x000000bd,
    IT_SET_UCONFIG_REG_PAIRS                           = 0x000000be,
    IT_RESERVED_BE                                     = 0x000000be,
    IT_RESERVED_BF                                     = 0x000000bf,
    IT_LOAD_BUFFER_FILLED_SIZE                         = 0x000000c0,
    IT_RESERVED_C0                                     = 0x000000c0,
    IT_SET_BUFFER_FILLED_SIZE                          = 0x000000c1,
    IT_RESERVED_C1                                     = 0x000000c1,
    IT_SAVE_BUFFER_FILLED_SIZE                         = 0x000000c2,
    IT_RESERVED_C2                                     = 0x000000c2,
    IT_STRMOUT_STATS_QUERY                             = 0x000000c3,
    IT_RESERVED_C3                                     = 0x000000c3,
    IT_RESERVED_C4                                     = 0x000000c4,
    IT_RESERVED_C5                                     = 0x000000c5,
    IT_RESERVED_C6                                     = 0x000000c6,
    IT_RESERVED_C7                                     = 0x000000c7,
    IT_RESERVED_C8                                     = 0x000000c8,
    IT_RESERVED_C9                                     = 0x000000c9,
    IT_RESERVED_CA                                     = 0x000000ca,
    IT_RESERVED_CB                                     = 0x000000cb,
    IT_RESERVED_CC                                     = 0x000000cc,
    IT_RESERVED_CD                                     = 0x000000cd,
    IT_RESERVED_CE                                     = 0x000000ce,
    IT_RESERVED_CF                                     = 0x000000cf,
    IT_RESERVED_D0                                     = 0x000000d0,
    IT_RESERVED_D1                                     = 0x000000d1,
    IT_RESERVED_D2                                     = 0x000000d2,
    IT_RESERVED_D3                                     = 0x000000d3,
    IT_RESERVED_D5                                     = 0x000000d5,
    IT_RESERVED_D6                                     = 0x000000d6,
    IT_RESERVED_D7                                     = 0x000000d7,
    IT_RESERVED_D8                                     = 0x000000d8,
    IT_RESERVED_D9                                     = 0x000000d9,
    IT_RESERVED_DA                                     = 0x000000da,
    IT_RESERVED_DB                                     = 0x000000db,
    IT_RESERVED_DC                                     = 0x000000dc,
    IT_RESERVED_DD                                     = 0x000000dd,
    IT_RESERVED_DE                                     = 0x000000de,
    IT_RESERVED_DF                                     = 0x000000df,
    IT_RESERVED_E0                                     = 0x000000e0,
    IT_RESERVED_E1                                     = 0x000000e1,
    IT_RESERVED_E2                                     = 0x000000e2,
    IT_RESERVED_E3                                     = 0x000000e3,
    IT_RESERVED_E4                                     = 0x000000e4,
    IT_RESERVED_E5                                     = 0x000000e5,
    IT_RESERVED_E6                                     = 0x000000e6,
    IT_RESERVED_E7                                     = 0x000000e7,
    IT_RESERVED_E8                                     = 0x000000e8,
    IT_RESERVED_E9                                     = 0x000000e9,
    IT_RESERVED_EA                                     = 0x000000ea,
    IT_RESERVED_EB                                     = 0x000000eb,
    IT_RESERVED_EC                                     = 0x000000ec,
    IT_RESERVED_ED                                     = 0x000000ed,
    IT_RESERVED_EE                                     = 0x000000ee,
    IT_UPDATE_DB_SUMMARIZER_TIMEOUT                    = 0x000000ef,
    IT_SET_Q_PREEMPTION_MODE                           = 0x000000f0,
    IT_RESERVED_F0                                     = 0x000000f0,
    IT_RESERVED_F1                                     = 0x000000f1,
    IT_RESERVED_F2                                     = 0x000000f2,
    IT_RESERVED_F3                                     = 0x000000f3,
    IT_RESERVED_F4                                     = 0x000000f4,
    IT_RESERVED_F5                                     = 0x000000f5,
    IT_RESERVED_F6                                     = 0x000000f6,
    IT_RESERVED_F7                                     = 0x000000f7,
    IT_RESERVED_F8                                     = 0x000000f8,
    IT_RESERVED_F9                                     = 0x000000f9,
    IT_RESERVED_FA                                     = 0x000000fa,
    IT_RESERVED_FB                                     = 0x000000fb,
    IT_DRAW_MULTI_PREAMBLE                             = 0x000000fe,
    IT_AQL_PACKET                                      = 0x000000ff,
} IT_OpCodeType;

constexpr unsigned int PM4_TYPE_0                                         = 0;
constexpr unsigned int PM4_TYPE_2                                         = 2;
constexpr unsigned int PM4_TYPE_3                                         = 3;

} // inline namespace Chip
} // namespace Gfx12
} // namespace Pal
