/*
 ***********************************************************************************************************************
 *
 *  Copyright (c) 2025 Advanced Micro Devices, Inc. All Rights Reserved.
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in all
 *  copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 *  SOFTWARE.
 *
 **********************************************************************************************************************/

#pragma once

namespace Pal
{
namespace Gfx12
{
inline namespace Chip
{

typedef enum BinEventCntl {
    BINNER_BREAK_BATCH                                 = 0x00000000,
    BINNER_PIPELINE                                    = 0x00000001,
    BINNER_DROP                                        = 0x00000002,
    BINNER_PIPELINE_BREAK                              = 0x00000003,
} BinEventCntl;

typedef enum BinMapMode {
    BIN_MAP_MODE_NONE                                  = 0x00000000,
    BIN_MAP_MODE_RTA_INDEX                             = 0x00000001,
    BIN_MAP_MODE_POPS                                  = 0x00000002,
} BinMapMode;

typedef enum BinningMode {
    BINNING_ALLOWED                                    = 0x00000000,
    FORCE_BINNING_ON                                   = 0x00000001,
    BINNING_ONE_PRIM_PER_BATCH                         = 0x00000002,
    BINNING_DISABLED                                   = 0x00000003,
} BinningMode;

typedef enum BinSizeExtend {
    BIN_SIZE_32_PIXELS                                 = 0x00000000,
    BIN_SIZE_64_PIXELS                                 = 0x00000001,
    BIN_SIZE_128_PIXELS                                = 0x00000002,
    BIN_SIZE_256_PIXELS                                = 0x00000003,
    BIN_SIZE_512_PIXELS                                = 0x00000004,
} BinSizeExtend;

typedef enum BlendOp {
    BLEND_ZERO                                         = 0x00000000,
    BLEND_ONE                                          = 0x00000001,
    BLEND_SRC_COLOR                                    = 0x00000002,
    BLEND_ONE_MINUS_SRC_COLOR                          = 0x00000003,
    BLEND_SRC_ALPHA                                    = 0x00000004,
    BLEND_ONE_MINUS_SRC_ALPHA                          = 0x00000005,
    BLEND_DST_ALPHA                                    = 0x00000006,
    BLEND_ONE_MINUS_DST_ALPHA                          = 0x00000007,
    BLEND_DST_COLOR                                    = 0x00000008,
    BLEND_ONE_MINUS_DST_COLOR                          = 0x00000009,
    BLEND_SRC_ALPHA_SATURATE                           = 0x0000000a,
    BLEND_CONSTANT_COLOR                               = 0x0000000b,
    BLEND_ONE_MINUS_CONSTANT_COLOR                     = 0x0000000c,
    BLEND_SRC1_COLOR                                   = 0x0000000d,
    BLEND_INV_SRC1_COLOR                               = 0x0000000e,
    BLEND_SRC1_ALPHA                                   = 0x0000000f,
    BLEND_INV_SRC1_ALPHA                               = 0x00000010,
    BLEND_CONSTANT_ALPHA                               = 0x00000011,
    BLEND_ONE_MINUS_CONSTANT_ALPHA                     = 0x00000012,
} BlendOp;

typedef enum BlendOpt {
    FORCE_OPT_AUTO                                     = 0x00000000,
    FORCE_OPT_DISABLE                                  = 0x00000001,
    FORCE_OPT_ENABLE_IF_SRC_A_0                        = 0x00000002,
    FORCE_OPT_ENABLE_IF_SRC_RGB_0                      = 0x00000003,
    FORCE_OPT_ENABLE_IF_SRC_ARGB_0                     = 0x00000004,
    FORCE_OPT_ENABLE_IF_SRC_A_1                        = 0x00000005,
    FORCE_OPT_ENABLE_IF_SRC_RGB_1                      = 0x00000006,
    FORCE_OPT_ENABLE_IF_SRC_ARGB_1                     = 0x00000007,
} BlendOpt;

typedef enum BUF_DATA_FORMAT {
    BUF_DATA_FORMAT_INVALID                            = 0x00000000,
    BUF_DATA_FORMAT_8                                  = 0x00000001,
    BUF_DATA_FORMAT_16                                 = 0x00000002,
    BUF_DATA_FORMAT_8_8                                = 0x00000003,
    BUF_DATA_FORMAT_32                                 = 0x00000004,
    BUF_DATA_FORMAT_16_16                              = 0x00000005,
    BUF_DATA_FORMAT_10_11_11                           = 0x00000006,
    BUF_DATA_FORMAT_11_11_10                           = 0x00000007,
    BUF_DATA_FORMAT_10_10_10_2                         = 0x00000008,
    BUF_DATA_FORMAT_2_10_10_10                         = 0x00000009,
    BUF_DATA_FORMAT_8_8_8_8                            = 0x0000000a,
    BUF_DATA_FORMAT_32_32                              = 0x0000000b,
    BUF_DATA_FORMAT_16_16_16_16                        = 0x0000000c,
    BUF_DATA_FORMAT_32_32_32                           = 0x0000000d,
    BUF_DATA_FORMAT_32_32_32_32                        = 0x0000000e,
    BUF_DATA_FORMAT_RESERVED_15                        = 0x0000000f,
} BUF_DATA_FORMAT;

typedef enum BUF_FMT {
    BUF_FMT_INVALID                                    = 0x00000000,
    BUF_FMT_8_UNORM                                    = 0x00000001,
    BUF_FMT_8_SNORM                                    = 0x00000002,
    BUF_FMT_8_USCALED                                  = 0x00000003,
    BUF_FMT_8_SSCALED                                  = 0x00000004,
    BUF_FMT_8_UINT                                     = 0x00000005,
    BUF_FMT_8_SINT                                     = 0x00000006,
    BUF_FMT_16_UNORM                                   = 0x00000007,
    BUF_FMT_16_SNORM                                   = 0x00000008,
    BUF_FMT_16_USCALED                                 = 0x00000009,
    BUF_FMT_16_SSCALED                                 = 0x0000000a,
    BUF_FMT_16_UINT                                    = 0x0000000b,
    BUF_FMT_16_SINT                                    = 0x0000000c,
    BUF_FMT_16_FLOAT                                   = 0x0000000d,
    BUF_FMT_8_8_UNORM                                  = 0x0000000e,
    BUF_FMT_8_8_SNORM                                  = 0x0000000f,
    BUF_FMT_8_8_USCALED                                = 0x00000010,
    BUF_FMT_8_8_SSCALED                                = 0x00000011,
    BUF_FMT_8_8_UINT                                   = 0x00000012,
    BUF_FMT_8_8_SINT                                   = 0x00000013,
    BUF_FMT_32_UINT                                    = 0x00000014,
    BUF_FMT_32_SINT                                    = 0x00000015,
    BUF_FMT_32_FLOAT                                   = 0x00000016,
    BUF_FMT_16_16_UNORM                                = 0x00000017,
    BUF_FMT_16_16_SNORM                                = 0x00000018,
    BUF_FMT_16_16_USCALED                              = 0x00000019,
    BUF_FMT_16_16_SSCALED                              = 0x0000001a,
    BUF_FMT_16_16_UINT                                 = 0x0000001b,
    BUF_FMT_16_16_SINT                                 = 0x0000001c,
    BUF_FMT_16_16_FLOAT                                = 0x0000001d,
    BUF_FMT_10_11_11_FLOAT                             = 0x0000001e,
    BUF_FMT_11_11_10_FLOAT                             = 0x0000001f,
    BUF_FMT_10_10_10_2_UNORM                           = 0x00000020,
    BUF_FMT_10_10_10_2_SNORM                           = 0x00000021,
    BUF_FMT_10_10_10_2_UINT                            = 0x00000022,
    BUF_FMT_10_10_10_2_SINT                            = 0x00000023,
    BUF_FMT_2_10_10_10_UNORM                           = 0x00000024,
    BUF_FMT_2_10_10_10_SNORM                           = 0x00000025,
    BUF_FMT_2_10_10_10_USCALED                         = 0x00000026,
    BUF_FMT_2_10_10_10_SSCALED                         = 0x00000027,
    BUF_FMT_2_10_10_10_UINT                            = 0x00000028,
    BUF_FMT_2_10_10_10_SINT                            = 0x00000029,
    BUF_FMT_8_8_8_8_UNORM                              = 0x0000002a,
    BUF_FMT_8_8_8_8_SNORM                              = 0x0000002b,
    BUF_FMT_8_8_8_8_USCALED                            = 0x0000002c,
    BUF_FMT_8_8_8_8_SSCALED                            = 0x0000002d,
    BUF_FMT_8_8_8_8_UINT                               = 0x0000002e,
    BUF_FMT_8_8_8_8_SINT                               = 0x0000002f,
    BUF_FMT_32_32_UINT                                 = 0x00000030,
    BUF_FMT_32_32_SINT                                 = 0x00000031,
    BUF_FMT_32_32_FLOAT                                = 0x00000032,
    BUF_FMT_16_16_16_16_UNORM                          = 0x00000033,
    BUF_FMT_16_16_16_16_SNORM                          = 0x00000034,
    BUF_FMT_16_16_16_16_USCALED                        = 0x00000035,
    BUF_FMT_16_16_16_16_SSCALED                        = 0x00000036,
    BUF_FMT_16_16_16_16_UINT                           = 0x00000037,
    BUF_FMT_16_16_16_16_SINT                           = 0x00000038,
    BUF_FMT_16_16_16_16_FLOAT                          = 0x00000039,
    BUF_FMT_32_32_32_UINT                              = 0x0000003a,
    BUF_FMT_32_32_32_SINT                              = 0x0000003b,
    BUF_FMT_32_32_32_FLOAT                             = 0x0000003c,
    BUF_FMT_32_32_32_32_UINT                           = 0x0000003d,
    BUF_FMT_32_32_32_32_SINT                           = 0x0000003e,
    BUF_FMT_32_32_32_32_FLOAT                          = 0x0000003f,
} BUF_FMT;

typedef enum BUF_NUM_FORMAT {
    BUF_NUM_FORMAT_UNORM                               = 0x00000000,
    BUF_NUM_FORMAT_SNORM                               = 0x00000001,
    BUF_NUM_FORMAT_USCALED                             = 0x00000002,
    BUF_NUM_FORMAT_SSCALED                             = 0x00000003,
    BUF_NUM_FORMAT_UINT                                = 0x00000004,
    BUF_NUM_FORMAT_SINT                                = 0x00000005,
    BUF_NUM_FORMAT_RESERVED_6                          = 0x00000006,
    BUF_NUM_FORMAT_FLOAT                               = 0x00000007,
} BUF_NUM_FORMAT;

typedef enum CBMode {
    CB_DISABLE                                         = 0x00000000,
    CB_NORMAL                                          = 0x00000001,
    CB_ELIMINATE_FAST_CLEAR                            = 0x00000002,
    CB_DCC_DECOMPRESS                                  = 0x00000003,
    CB_RESERVED                                        = 0x00000004,
} CBMode;

typedef enum CBPerfClearFilterSel {
    CB_PERF_CLEAR_FILTER_SEL_NONCLEAR                  = 0x00000000,
    CB_PERF_CLEAR_FILTER_SEL_CLEAR                     = 0x00000001,
} CBPerfClearFilterSel;

typedef enum CBPerfOpFilterSel {
    CB_PERF_OP_FILTER_SEL_WRITE_ONLY                   = 0x00000000,
    CB_PERF_OP_FILTER_SEL_NEEDS_DESTINATION            = 0x00000001,
    CB_PERF_OP_FILTER_SEL_RESOLVE                      = 0x00000002,
    CB_PERF_OP_FILTER_SEL_DECOMPRESS                   = 0x00000003,
    CB_PERF_OP_FILTER_SEL_FMASK_DECOMPRESS             = 0x00000004,
    CB_PERF_OP_FILTER_SEL_ELIMINATE_FAST_CLEAR         = 0x00000005,
} CBPerfOpFilterSel;

typedef enum CBPerfSel {
    CB_PERF_SEL_BUSY                                   = 0x00000001,
    CB_PERF_SEL_ACTIVE                                 = 0x00000002,
    CB_PERF_SEL_DRAWN_PIXEL                            = 0x00000003,
    CB_PERF_SEL_DRAWN_PIXEL_ONE_MRT                    = 0x00000004,
    CB_PERF_SEL_DRAWN_PIXEL_BLEND                      = 0x00000005,
    CB_PERF_SEL_DRAWN_FRAGMENT                         = 0x00000006,
    CB_PERF_SEL_DRAWN_QUAD                             = 0x00000007,
    CB_PERF_SEL_DRAWN_BUSY                             = 0x00000008,
    CB_PERF_SEL_DRAWN_QUAD_FRAGMENT_1PPC               = 0x00000009,
    CB_PERF_SEL_DRAWN_QUAD_FRAGMENT_2PPC               = 0x0000000a,
    CB_PERF_SEL_DRAWN_QUAD_FRAGMENT_4PPC               = 0x0000000b,
    CB_PERF_SEL_DRAWN_QUAD_FRAGMENT_8PPC               = 0x0000000c,
    CB_PERF_SEL_TOTAL_READ_REQUEST                     = 0x0000000d,
    CB_PERF_SEL_TOTAL_WRITE_REQUEST                    = 0x0000000e,
    CB_PERF_SEL_DB_CB_EXPORT_VALID_READY               = 0x0000000f,
    CB_PERF_SEL_DB_CB_EXPORT_VALID_READYB              = 0x00000010,
    CB_PERF_SEL_DB_CB_EXPORT_VALIDB_READY              = 0x00000011,
    CB_PERF_SEL_DB_CB_EXPORT_VALIDB_READYB             = 0x00000012,
    CB_PERF_SEL_DUAL_SOURCE_COLOR_QUAD_FRAGMENT        = 0x00000013,
    CB_PERF_SEL_CC_CRW_LB_REQ_READ_STALL_CYC           = 0x0000001e,
    CB_PERF_SEL_CC_CRW_LB_REQ_WRITE_STALL_CYC          = 0x0000001f,
    CB_PERF_SEL_CC_CRW_GLX_CRED_STALL                  = 0x00000020,
    CB_PERF_SEL_CC_CRW_GLX_REQ_READ_REQUEST            = 0x00000028,
    CB_PERF_SEL_CC_CRW_GLX_READ_RETURNS                = 0x00000029,
    CB_PERF_SEL_CC_CRW_GLX_REQ_WRITE_REQUEST           = 0x0000002a,
    CB_PERF_SEL_CC_CRW_GLX_WRITE_RETURNS               = 0x0000002b,
    CB_PERF_SEL_CC_CRW_GLX_SRC_WRITE_CYCLES            = 0x0000002c,
    CB_PERF_SEL_CC_CRW_GLX_READ_REQUESTS_IN_FLIGHT     = 0x0000002d,
    CB_PERF_SEL_CC_CRW_GLX_WRITE_REQUESTS_IN_FLIGHT    = 0x0000002e,
    CB_PERF_SEL_EVENT                                  = 0x00000032,
    CB_PERF_SEL_EVENT_CACHE_FLUSH_TS                   = 0x00000033,
    CB_PERF_SEL_EVENT_CONTEXT_DONE                     = 0x00000034,
    CB_PERF_SEL_EVENT_CACHE_FLUSH                      = 0x00000035,
    CB_PERF_SEL_EVENT_CACHE_FLUSH_AND_INV_TS_EVENT     = 0x00000036,
    CB_PERF_SEL_EVENT_CACHE_FLUSH_AND_INV_EVENT        = 0x00000037,
    CB_PERF_SEL_EVENT_FLUSH_AND_INV_CB_DATA_TS         = 0x00000038,
    CB_PERF_SEL_EVENT_FLUSH_AND_INV_CB_META            = 0x00000039,
    CB_PERF_SEL_EVENT_BOTTOM_OF_PIPE_TS                = 0x0000003a,
    CB_PERF_SEL_STATIC_CLOCK_EN                        = 0x0000003c,
    CB_PERF_SEL_PERFMON_CLOCK_EN                       = 0x0000003d,
    CB_PERF_SEL_BLEND_CLOCK_EN                         = 0x0000003e,
    CB_PERF_SEL_ALPHA_BLEND_CLOCK_EN                   = 0x0000003f,
    CB_PERF_SEL_COLOR_STORE_CLOCK_EN                   = 0x00000040,
    CB_PERF_SEL_COLOR_STORE_RAM_CLOCK_EN               = 0x00000041,
    CB_PERF_SEL_BACKEND_READ_RTN_CLOCK_EN              = 0x00000042,
    CB_PERF_SEL_GRBM_CLOCK_EN                          = 0x00000043,
    CB_PERF_SEL_BACKEND_EVICT_PIPE_CLOCK_EN            = 0x00000044,
    CB_PERF_SEL_BACKEND_EVICT_PIPE_FMRAC_CLOCK_EN      = 0x00000045,
    CB_PERF_SEL_BACKEND_FRAGOP_CLOCK_EN                = 0x00000046,
    CB_PERF_SEL_BACKEND_FRAGOP_SEQ_CLOCK_EN            = 0x00000047,
    CB_PERF_SEL_BACKEND_FRAGOP_FCS_CLOCK_EN            = 0x00000048,
    CB_PERF_SEL_BACKEND_SRC_FIFO_CLOCK_EN              = 0x00000049,
    CB_PERF_SEL_BACKEND_CACHE_CTL_CLOCK_EN             = 0x0000004a,
    CB_PERF_SEL_BACKEND_CACHE_CTL_CAS_CLOCK_EN         = 0x0000004b,
    CB_PERF_SEL_BACKEND_CACHE_CTL_IAS_CLOCK_EN         = 0x0000004c,
    CB_PERF_SEL_BACKEND_CACHE_CTL_PROBE_CLOCK_EN       = 0x0000004d,
    CB_PERF_SEL_BACKEND_CACHE_CTL_RCM_CLOCK_EN         = 0x0000004e,
    CB_PERF_SEL_CRW_CLK_EN                             = 0x0000004f,
    CB_PERF_SEL_FRONTEND_INPUT_CLOCK_EN                = 0x00000050,
    CB_PERF_SEL_FRONTEND_ADDR_CLOCK_EN                 = 0x00000051,
    CB_PERF_SEL_EVENTS_CLK_EN                          = 0x00000052,
    CB_PERF_SEL_CC_TAG_HIT                             = 0x00000054,
    CB_PERF_SEL_CC_TAG_MISS                            = 0x00000055,
    CB_PERF_SEL_CC_BLOCK_MISS                          = 0x00000056,
    CB_PERF_SEL_CC_BLOCK_HIT                           = 0x00000057,
    CB_PERF_SEL_CC_CACHE_READ_OUTPUT_STALL             = 0x00000058,
    CB_PERF_SEL_CC_CACHE_WRITE_OUTPUT_STALL            = 0x00000059,
    CB_PERF_SEL_CC_CACHE_ACK_OUTPUT_STALL              = 0x0000005a,
    CB_PERF_SEL_CC_CACHE_STALL                         = 0x0000005b,
    CB_PERF_SEL_CC_CACHE_FLUSH                         = 0x0000005c,
    CB_PERF_SEL_CC_CACHE_SECTORS_FLUSHED               = 0x0000005d,
    CB_PERF_SEL_CC_CACHE_WA_TO_RMW_CONVERSION          = 0x0000005e,
    CB_PERF_SEL_CC_CACHE_READS_SAVED_DUE_TO_DCC        = 0x00000061,
    CB_PERF_SEL_CCC_COLOR_RESOURCE_PANIC               = 0x00000063,
    CB_PERF_SEL_CCC_FMASK_RESOURCE_PANIC               = 0x00000064,
    CB_PERF_SEL_CCC_FREE_WAYS_PANIC                    = 0x00000065,
    CB_PERF_SEL_CCC_SKID_FIFO_FULL                     = 0x00000066,
    CB_PERF_SEL_CCC_SKID_FIFO_STALL                    = 0x00000067,
    CB_PERF_SEL_CCC_COLOR_RESOURCE_STALL               = 0x00000068,
    CB_PERF_SEL_CCC_FMASK_RESOURCE_STALL               = 0x00000069,
    CB_PERF_SEL_CCC_FREE_WAYS_STALL                    = 0x0000006a,
    CB_PERF_SEL_CCC_COVERED_BYTE_RAM_STALLS            = 0x0000006b,
    CB_PERF_SEL_CC_BLOCKS_READ_ON_MISS                 = 0x0000006c,
    CB_PERF_SEL_CC_BLOCKS_READ_ON_HIT                  = 0x0000006d,
    CB_PERF_SEL_CCC_READ_UNCOMP                        = 0x0000006e,
    CB_PERF_SEL_CCC_READ_COMP                          = 0x0000006f,
    CB_PERF_SEL_BE_SRCFIFO_FULL                        = 0x00000070,
    CB_PERF_SEL_BE_RDLATFIFO_FULL                      = 0x00000071,
    CB_PERF_SEL_RDLAT_FIFO_QUAD_RESIDENCY_STALL        = 0x00000072,
    CB_PERF_SEL_CC_QUADFRAG_VALID_READY                = 0x00000073,
    CB_PERF_SEL_CC_QUADFRAG_VALID_READYB               = 0x00000074,
    CB_PERF_SEL_CC_QUADFRAG_VALIDB_READY               = 0x00000075,
    CB_PERF_SEL_CC_QUADFRAG_VALIDB_READYB              = 0x00000076,
    CB_PERF_SEL_FRAGOP_RAW_STALL                       = 0x00000077,
    CB_PERF_SEL_CC_BB_BLEND_PIXEL_VALID_READY          = 0x00000078,
    CB_PERF_SEL_CC_BB_BLEND_PIXEL_VALID_READYB         = 0x00000079,
    CB_PERF_SEL_CC_BB_BLEND_PIXEL_VALIDB_READY         = 0x0000007a,
    CB_PERF_SEL_CC_BB_BLEND_PIXEL_VALIDB_READYB        = 0x0000007b,
    CB_PERF_SEL_EVICT_WR_UNCOMP_BLOCK                  = 0x00000084,
    CB_PERF_SEL_EVICT_WR_FRAG_COMP_BLOCK               = 0x00000085,
    CB_PERF_SEL_EVICT_WR_SINGLE_BLOCK                  = 0x00000086,
    CB_PERF_SEL_EVICT_WR_UNCOMP_TIDS                   = 0x00000087,
    CB_PERF_SEL_EVICT_WR_FRAG_COMP_TIDS                = 0x00000088,
    CB_PERF_SEL_EVICT_WR_1FRAG_COMP_TIDS               = 0x00000089,
    CB_PERF_SEL_EVICT_MAX_FRAG_DIFF_ACCUM              = 0x0000008a,
    CB_PERF_SEL_EVICT_WR_BLOCKS_FULLY_COVERED          = 0x0000008b,
    CB_PERF_SEL_EVICT_WR_BLOCKS_PARTIALLY_COVERED      = 0x0000008c,
    CB_PERF_SEL_RD_UNCOMP_HALF_BLOCK                   = 0x0000008e,
    CB_PERF_SEL_RD_FRAG_COMP_HALF_BLOCK                = 0x0000008f,
    CB_PERF_SEL_RD_SINGLE_HALF_BLOCK                   = 0x00000090,
    CB_PERF_SEL_RD_CLEAR_HALF_BLOCK                    = 0x00000091,
    CB_PERF_SEL_RBP_EXPORT_8PIX_LIT_BOTH               = 0x00000098,
    CB_PERF_SEL_RBP_EXPORT_8PIX_LIT_LEFT               = 0x00000099,
    CB_PERF_SEL_RBP_EXPORT_8PIX_LIT_RIGHT              = 0x0000009a,
    CB_PERF_SEL_BLEND_QUAD_DST_READ_COULD_HAVE_BEEN_OPTIMIZED = 0x000000b6,
    CB_PERF_SEL_BLEND_QUAD_BLENDING_COULD_HAVE_BEEN_BYPASSED = 0x000000b7,
    CB_PERF_SEL_BLEND_QUAD_COULD_HAVE_BEEN_DISCARDED   = 0x000000b8,
    CB_PERF_SEL_BLEND_OPT_PIXELS_RESULT_EQ_DEST        = 0x000000b9,
    CB_PERF_SEL_BLEND_STALL_AT_OUTPUT                  = 0x000000ba,
    CB_PERF_SEL_BLEND_STALL_ON_CACHE_ACCESS            = 0x000000bb,
    CB_PERF_SEL_BLEND_COLLISION_DUE_TO_CACHE_WRITE     = 0x000000bc,
    CB_PERF_SEL_BLEND_RAW_HAZARD_STALL                 = 0x000000bd,
    CB_PERF_SEL_BE_CS_FILLRATE_1X2                     = 0x000000c0,
    CB_PERF_SEL_BE_CS_FILLRATE_2X1                     = 0x000000c1,
    CB_PERF_SEL_BE_CS_FILLRATE_2X2                     = 0x000000c2,
    CB_PERF_SEL_FORMAT_IS_32_R                         = 0x000000fc,
    CB_PERF_SEL_FORMAT_IS_32_AR                        = 0x000000fd,
    CB_PERF_SEL_FORMAT_IS_32_GR                        = 0x000000fe,
    CB_PERF_SEL_FORMAT_IS_32_ABGR                      = 0x000000ff,
    CB_PERF_SEL_FORMAT_IS_FP16_ABGR                    = 0x00000100,
    CB_PERF_SEL_FORMAT_IS_SIGNED16_ABGR                = 0x00000101,
    CB_PERF_SEL_FORMAT_IS_UNSIGNED16_ABGR              = 0x00000102,
    CB_PERF_SEL_FORMAT_IS_32BPP_8PIX                   = 0x00000103,
    CB_PERF_SEL_FORMAT_IS_16_16_UNSIGNED_8PIX          = 0x00000104,
    CB_PERF_SEL_FORMAT_IS_16_16_SIGNED_8PIX            = 0x00000105,
    CB_PERF_SEL_FORMAT_IS_16_16_FLOAT_8PIX             = 0x00000106,
    CB_PERF_SEL_EXPORT_ADDED_1_FRAGMENT                = 0x00000107,
    CB_PERF_SEL_EXPORT_ADDED_2_FRAGMENTS               = 0x00000108,
    CB_PERF_SEL_EXPORT_ADDED_3_FRAGMENTS               = 0x00000109,
    CB_PERF_SEL_EXPORT_ADDED_4_FRAGMENTS               = 0x0000010a,
    CB_PERF_SEL_EXPORT_ADDED_5_FRAGMENTS               = 0x0000010b,
    CB_PERF_SEL_EXPORT_ADDED_6_FRAGMENTS               = 0x0000010c,
    CB_PERF_SEL_EXPORT_ADDED_7_FRAGMENTS               = 0x0000010d,
    CB_PERF_SEL_EXPORT_BLEND_OPT_DONT_READ_DST         = 0x0000010e,
    CB_PERF_SEL_EXPORT_BLEND_OPT_BLEND_BYPASS          = 0x0000010f,
    CB_PERF_SEL_EXPORT_BLEND_OPT_DISCARD_PIXELS        = 0x00000110,
    CB_PERF_SEL_EXPORT_HAS_1_FRAGMENT_BEFORE_UPDATE    = 0x00000111,
    CB_PERF_SEL_EXPORT_HAS_1_FRAGMENT_AFTER_UPDATE     = 0x00000112,
    CB_PERF_SEL_EXPORT_HAS_2_FRAGMENTS_BEFORE_UPDATE   = 0x00000113,
    CB_PERF_SEL_EXPORT_HAS_2_FRAGMENTS_AFTER_UPDATE    = 0x00000114,
    CB_PERF_SEL_EXPORT_HAS_3_FRAGMENTS_BEFORE_UPDATE   = 0x00000115,
    CB_PERF_SEL_EXPORT_HAS_3_FRAGMENTS_AFTER_UPDATE    = 0x00000116,
    CB_PERF_SEL_EXPORT_HAS_4_FRAGMENTS_BEFORE_UPDATE   = 0x00000117,
    CB_PERF_SEL_EXPORT_HAS_4_FRAGMENTS_AFTER_UPDATE    = 0x00000118,
    CB_PERF_SEL_EXPORT_HAS_5_FRAGMENTS_BEFORE_UPDATE   = 0x00000119,
    CB_PERF_SEL_EXPORT_HAS_5_FRAGMENTS_AFTER_UPDATE    = 0x0000011a,
    CB_PERF_SEL_EXPORT_HAS_6_FRAGMENTS_BEFORE_UPDATE   = 0x0000011b,
    CB_PERF_SEL_EXPORT_HAS_6_FRAGMENTS_AFTER_UPDATE    = 0x0000011c,
    CB_PERF_SEL_EXPORT_HAS_7_FRAGMENTS_BEFORE_UPDATE   = 0x0000011d,
    CB_PERF_SEL_EXPORT_HAS_7_FRAGMENTS_AFTER_UPDATE    = 0x0000011e,
    CB_PERF_SEL_EXPORT_HAS_8_FRAGMENTS_BEFORE_UPDATE   = 0x0000011f,
    CB_PERF_SEL_EXPORT_HAS_8_FRAGMENTS_AFTER_UPDATE    = 0x00000120,
    CB_PERF_SEL_EXPORT_READS_FRAGMENT_0                = 0x00000121,
    CB_PERF_SEL_EXPORT_READS_FRAGMENT_1                = 0x00000122,
    CB_PERF_SEL_EXPORT_READS_FRAGMENT_2                = 0x00000123,
    CB_PERF_SEL_EXPORT_READS_FRAGMENT_3                = 0x00000124,
    CB_PERF_SEL_EXPORT_READS_FRAGMENT_4                = 0x00000125,
    CB_PERF_SEL_EXPORT_READS_FRAGMENT_5                = 0x00000126,
    CB_PERF_SEL_EXPORT_READS_FRAGMENT_6                = 0x00000127,
    CB_PERF_SEL_EXPORT_READS_FRAGMENT_7                = 0x00000128,
    CB_PERF_SEL_EXPORT_REMOVED_1_FRAGMENT              = 0x00000129,
    CB_PERF_SEL_EXPORT_REMOVED_2_FRAGMENTS             = 0x0000012a,
    CB_PERF_SEL_EXPORT_REMOVED_3_FRAGMENTS             = 0x0000012b,
    CB_PERF_SEL_EXPORT_REMOVED_4_FRAGMENTS             = 0x0000012c,
    CB_PERF_SEL_EXPORT_REMOVED_5_FRAGMENTS             = 0x0000012d,
    CB_PERF_SEL_EXPORT_REMOVED_6_FRAGMENTS             = 0x0000012e,
    CB_PERF_SEL_EXPORT_REMOVED_7_FRAGMENTS             = 0x0000012f,
    CB_PERF_SEL_EXPORT_WRITES_FRAGMENT_0               = 0x00000130,
    CB_PERF_SEL_EXPORT_WRITES_FRAGMENT_1               = 0x00000131,
    CB_PERF_SEL_EXPORT_WRITES_FRAGMENT_2               = 0x00000132,
    CB_PERF_SEL_EXPORT_WRITES_FRAGMENT_3               = 0x00000133,
    CB_PERF_SEL_EXPORT_WRITES_FRAGMENT_4               = 0x00000134,
    CB_PERF_SEL_EXPORT_WRITES_FRAGMENT_5               = 0x00000135,
    CB_PERF_SEL_EXPORT_WRITES_FRAGMENT_6               = 0x00000136,
    CB_PERF_SEL_EXPORT_WRITES_FRAGMENT_7               = 0x00000137,
    CB_PERF_SEL_EXPORT_KILLED_BY_COLOR_INVALID         = 0x00000138,
    CB_PERF_SEL_EXPORT_KILLED_BY_DISCARD_PIXEL         = 0x00000139,
    CB_PERF_SEL_EXPORT_KILLED_BY_NULL_SAMPLE_MASK      = 0x0000013a,
    CB_PERF_SEL_EXPORT_KILLED_BY_NULL_TARGET_SHADER_MASK = 0x0000013b,
} CBPerfSel;

constexpr unsigned int MaxCBPerfSel                    = CB_PERF_SEL_EXPORT_KILLED_BY_NULL_TARGET_SHADER_MASK;

typedef enum CBRamList {
    CB_DCG_CCC_CAS_TAG_ARRAY                           = 0x00000000,
    CB_DCG_CCC_CAS_FRAG_PTR                            = 0x00000001,
    CB_DCG_CCC_CAS_COLOR_PTR                           = 0x00000002,
    CB_DCG_CCC_CAS_SURF_PARAM                          = 0x00000003,
    CB_DCG_CCC_CBR                                     = 0x00000004,
    CB_DCG_BACKEND_RDLAT_FIFO                          = 0x00000005,
    CB_DCG_SRC_FIFO                                    = 0x00000006,
    CB_DCG_COLOR_STORE                                 = 0x00000007,
    CB_DCG_COLOR_STORE_DIRTY_BYTE                      = 0x00000008,
    CB_DCG_FMASK_CACHE_STORE                           = 0x00000009,
    CB_DCG_READ_SKID_FIFO                              = 0x0000000a,
} CBRamList;

typedef enum CHA_PERF_SEL {
    CHA_PERF_SEL_BUSY                                  = 0x00000000,
    CHA_PERF_SEL_STALL_CHC0                            = 0x00000001,
    CHA_PERF_SEL_STALL_CHC1                            = 0x00000002,
#if CHIP_HDR_NAVI48
    CHA_PERF_SEL_STALL_CHC2__NV48                      = 0x00000003,
    CHA_PERF_SEL_STALL_CHC3__NV48                      = 0x00000004,
    CHA_PERF_SEL_REQUEST_CHC0__NV48                    = 0x00000005,
    CHA_PERF_SEL_REQUEST_CHC1__NV48                    = 0x00000006,
    CHA_PERF_SEL_REQUEST_CHC2__NV48                    = 0x00000007,
    CHA_PERF_SEL_REQUEST_CHC3__NV48                    = 0x00000008,
    CHA_PERF_SEL_MEM_BURST_COUNT_CHC0__NV48            = 0x00000009,
    CHA_PERF_SEL_MEM_BURST_COUNT_CHC1__NV48            = 0x0000000a,
    CHA_PERF_SEL_MEM_BURST_COUNT_CHC2__NV48            = 0x0000000b,
    CHA_PERF_SEL_MEM_BURST_COUNT_CHC3__NV48            = 0x0000000c,
    CHA_PERF_SEL_IO_BURST_COUNT_CHC0__NV48             = 0x0000000d,
    CHA_PERF_SEL_IO_BURST_COUNT_CHC1__NV48             = 0x0000000e,
    CHA_PERF_SEL_IO_BURST_COUNT_CHC2__NV48             = 0x0000000f,
    CHA_PERF_SEL_IO_BURST_COUNT_CHC3__NV48             = 0x00000010,
    CHA_PERF_SEL_ARB_REQUESTS__NV48                    = 0x00000011,
    CHA_PERF_SEL_REQ_INFLIGHT_LEVEL__NV48              = 0x00000012,
    CHA_PERF_SEL_STALL_RET_CONFLICT_CHC0__NV48         = 0x00000013,
    CHA_PERF_SEL_STALL_RET_CONFLICT_CHC1__NV48         = 0x00000014,
    CHA_PERF_SEL_STALL_RET_CONFLICT_CHC2__NV48         = 0x00000015,
    CHA_PERF_SEL_STALL_RET_CONFLICT_CHC3__NV48         = 0x00000016,
    CHA_PERF_SEL_REQ_PATH_BUSY__NV48                   = 0x00000017,
    CHA_PERF_SEL_RET_PATH_BUSY__NV48                   = 0x00000018,
    CHA_PERF_SEL_CYCLE__NV48                           = 0x00000019,
#endif
} CHA_PERF_SEL;

#if CHIP_HDR_NAVI48
constexpr unsigned int MaxChaPerfSelNv48               = CHA_PERF_SEL_CYCLE__NV48;
#endif

typedef enum CHC_PERF_SEL {
    CHC_PERF_SEL_CYCLE                                 = 0x00000000,
    CHC_PERF_SEL_BUSY                                  = 0x00000001,
    CHC_PERF_SEL_STARVE                                = 0x00000002,
    CHC_PERF_SEL_ARB_RET_LEVEL                         = 0x00000003,
    CHC_PERF_SEL_EA_REQ_READ_LEVEL                     = 0x00000004,
    CHC_PERF_SEL_EA_REQ_WRITE_LEVEL                    = 0x00000005,
    CHC_PERF_SEL_REQ                                   = 0x00000006,
    CHC_PERF_SEL_REQ_ATOMIC_WITH_RET                   = 0x00000007,
    CHC_PERF_SEL_REQ_ATOMIC_WITHOUT_RET                = 0x00000008,
    CHC_PERF_SEL_REQ_NOP_ACK                           = 0x00000009,
    CHC_PERF_SEL_REQ_NOP_RTN0                          = 0x0000000a,
    CHC_PERF_SEL_REQ_READ                              = 0x0000000b,
    CHC_PERF_SEL_REQ_READ_32B                          = 0x0000000c,
    CHC_PERF_SEL_REQ_READ_64B                          = 0x0000000d,
    CHC_PERF_SEL_REQ_READ_128B                         = 0x0000000e,
    CHC_PERF_SEL_REQ_READ_256B                         = 0x0000000f,
    CHC_PERF_SEL_REQ_WRITE                             = 0x00000010,
    CHC_PERF_SEL_REQ_WRITE_32B                         = 0x00000011,
    CHC_PERF_SEL_REQ_WRITE_64B                         = 0x00000012,
    CHC_PERF_SEL_REQ_WRITE_128B                        = 0x00000013,
    CHC_PERF_SEL_REQ_WRITE_256B                        = 0x00000014,
    CHC_PERF_SEL_REQ_WRITE_32B_DATA_BEATS              = 0x00000015,
    CHC_PERF_SEL_REQ_READ_COMP_KEY                     = 0x00000016,
    CHC_PERF_SEL_REQ_FORCE_EXISTING_DATA_DECOMPRESS    = 0x00000017,
    CHC_PERF_SEL_READ_COMP_MODE_0                      = 0x00000018,
    CHC_PERF_SEL_READ_COMP_MODE_1                      = 0x00000019,
    CHC_PERF_SEL_READ_COMP_MODE_2                      = 0x0000001a,
    CHC_PERF_SEL_READ_32B_COMP_MODE_1                  = 0x0000001b,
    CHC_PERF_SEL_READ_64B_COMP_MODE_1                  = 0x0000001c,
    CHC_PERF_SEL_READ_128B_COMP_MODE_1                 = 0x0000001d,
    CHC_PERF_SEL_READ_256B_COMP_MODE_1                 = 0x0000001e,
    CHC_PERF_SEL_WRITE_COMP_MODE_0                     = 0x0000001f,
    CHC_PERF_SEL_WRITE_COMP_MODE_1                     = 0x00000020,
    CHC_PERF_SEL_WRITE_COMP_MODE_2                     = 0x00000021,
    CHC_PERF_SEL_WRITE_32B_COMP_MODE_1                 = 0x00000022,
    CHC_PERF_SEL_WRITE_64B_COMP_MODE_1                 = 0x00000023,
    CHC_PERF_SEL_WRITE_128B_COMP_MODE_1                = 0x00000024,
    CHC_PERF_SEL_WRITE_256B_COMP_MODE_1                = 0x00000025,
    CHC_PERF_SEL_LOOPBACK_TYPE_1                       = 0x00000026,
    CHC_PERF_SEL_LOOPBACK_TYPE_2                       = 0x00000027,
    CHC_PERF_SEL_LOOPBACK_TYPE_3                       = 0x00000028,
    CHC_PERF_SEL_EA_RET_ACK                            = 0x00000029,
    CHC_PERF_SEL_EA_RET_32B                            = 0x0000002a,
    CHC_PERF_SEL_CH_RET_ACK                            = 0x0000002b,
    CHC_PERF_SEL_CH_RET_32B                            = 0x0000002c,
    CHC_PERF_SEL_EA_RDRET_NACK                         = 0x0000002d,
    CHC_PERF_SEL_EA_WRRET_NACK                         = 0x0000002e,
    CHC_PERF_SEL_STALL_OC_EA_REQ_CREDITS_IO            = 0x0000002f,
    CHC_PERF_SEL_STALL_OC_EA_REQ_CREDITS_DRAM          = 0x00000030,
    CHC_PERF_SEL_STALL_OC_EA_DATA_CREDITS_IO           = 0x00000031,
    CHC_PERF_SEL_STALL_OC_EA_DATA_CREDITS_DRAM         = 0x00000032,
    CHC_PERF_SEL_STALL_BUFFER_FULL                     = 0x00000033,
    CHC_PERF_SEL_STALL_LOOPBACK_FIFO_FULL              = 0x00000034,
    CHC_PERF_SEL_REQ_CLIENT0                           = 0x00000035,
    CHC_PERF_SEL_REQ_CLIENT1                           = 0x00000036,
    CHC_PERF_SEL_REQ_CLIENT2                           = 0x00000037,
    CHC_PERF_SEL_REQ_CLIENT3                           = 0x00000038,
    CHC_PERF_SEL_REQ_CLIENT4                           = 0x00000039,
    CHC_PERF_SEL_REQ_CLIENT5                           = 0x0000003a,
    CHC_PERF_SEL_REQ_CLIENT6                           = 0x0000003b,
    CHC_PERF_SEL_REQ_CLIENT7                           = 0x0000003c,
    CHC_PERF_SEL_REQ_CLIENT8                           = 0x0000003d,
    CHC_PERF_SEL_REQ_CLIENT9                           = 0x0000003e,
    CHC_PERF_SEL_REQ_CLIENT10                          = 0x0000003f,
    CHC_PERF_SEL_REQ_CLIENT11                          = 0x00000040,
    CHC_PERF_SEL_REQ_CLIENT12                          = 0x00000041,
    CHC_PERF_SEL_REQ_CLIENT13                          = 0x00000042,
    CHC_PERF_SEL_REQ_CLIENT14                          = 0x00000043,
    CHC_PERF_SEL_REQ_CLIENT15                          = 0x00000044,
    CHC_PERF_SEL_REQ_CLIENT16                          = 0x00000045,
    CHC_PERF_SEL_REQ_CLIENT17                          = 0x00000046,
    CHC_PERF_SEL_REQ_CLIENT18                          = 0x00000047,
    CHC_PERF_SEL_REQ_CLIENT19                          = 0x00000048,
    CHC_PERF_SEL_REQ_CLIENT20                          = 0x00000049,
    CHC_PERF_SEL_REQ_CLIENT21                          = 0x0000004a,
    CHC_PERF_SEL_REQ_CLIENT22                          = 0x0000004b,
    CHC_PERF_SEL_REQ_CLIENT23                          = 0x0000004c,
    CHC_PERF_SEL_OC_DCC_IN_XFC                         = 0x0000004d,
    CHC_PERF_SEL_OC_DCC_IN_COMP                        = 0x0000004e,
    CHC_PERF_SEL_OC_DCC_IN_UNCOMP                      = 0x0000004f,
    CHC_PERF_SEL_OC_DCC_IN_CONST2SINGLE_256B           = 0x00000050,
    CHC_PERF_SEL_OC_DCC_IN_CONST2SINGLE_128B           = 0x00000051,
    CHC_PERF_SEL_OC_DCC_IN_CONST2CLEAR_256B            = 0x00000052,
    CHC_PERF_SEL_OC_DCC_IN_CONST2CLEAR_128B            = 0x00000053,
    CHC_PERF_SEL_OC_DCC_OUT_XFC                        = 0x00000054,
    CHC_PERF_SEL_OC_DCC_OUT_COMP                       = 0x00000055,
    CHC_PERF_SEL_OC_DCC_OUT_UNCOMP                     = 0x00000056,
    CHC_PERF_SEL_OC_DCC_OUT_CONST2SINGLE_256B          = 0x00000057,
    CHC_PERF_SEL_OC_DCC_OUT_CONST2SINGLE_128B          = 0x00000058,
    CHC_PERF_SEL_OC_DCC_OUT_CONST2CLEAR_256B           = 0x00000059,
    CHC_PERF_SEL_OC_DCC_OUT_CONST2CLEAR_128B           = 0x0000005a,
    CHC_PERF_SEL_OC_DCC_ACTIVE                         = 0x0000005b,
    CHC_PERF_SEL_OC_DCC_STALLED                        = 0x0000005c,
    CHC_PERF_SEL_OC_COMPRESS_DISABLE_REQ               = 0x0000005d,
    CHC_PERF_SEL_OC_BYPASS_REQ                         = 0x0000005e,
} CHC_PERF_SEL;

constexpr unsigned int MaxChcPerfSel                   = CHC_PERF_SEL_OC_BYPASS_REQ;

typedef enum CmaskMode {
    CMASK_CLEAR_NONE                                   = 0x00000000,
    CMASK_CLEAR_ONE                                    = 0x00000001,
    CMASK_CLEAR_ALL                                    = 0x00000002,
    CMASK_ANY_EXPANDED                                 = 0x00000003,
    CMASK_ALPHA0_FRAG1                                 = 0x00000004,
    CMASK_ALPHA0_FRAG2                                 = 0x00000005,
    CMASK_ALPHA0_FRAG4                                 = 0x00000006,
    CMASK_ALPHA0_FRAGS                                 = 0x00000007,
    CMASK_ALPHA1_FRAG1                                 = 0x00000008,
    CMASK_ALPHA1_FRAG2                                 = 0x00000009,
    CMASK_ALPHA1_FRAG4                                 = 0x0000000a,
    CMASK_ALPHA1_FRAGS                                 = 0x0000000b,
    CMASK_ALPHAX_FRAG1                                 = 0x0000000c,
    CMASK_ALPHAX_FRAG2                                 = 0x0000000d,
    CMASK_ALPHAX_FRAG4                                 = 0x0000000e,
    CMASK_ALPHAX_FRAGS                                 = 0x0000000f,
} CmaskMode;

typedef enum ColorArray {
    ARRAY_2D_ALT_COLOR                                 = 0x00000000,
    ARRAY_2D_COLOR                                     = 0x00000001,
    ARRAY_3D_SLICE_COLOR                               = 0x00000003,
} ColorArray;

typedef enum ColorFormat {
    COLOR_INVALID                                      = 0x00000000,
    COLOR_8                                            = 0x00000001,
    COLOR_16                                           = 0x00000002,
    COLOR_8_8                                          = 0x00000003,
    COLOR_32                                           = 0x00000004,
    COLOR_16_16                                        = 0x00000005,
    COLOR_10_11_11                                     = 0x00000006,
    COLOR_11_11_10                                     = 0x00000007,
    COLOR_10_10_10_2                                   = 0x00000008,
    COLOR_2_10_10_10                                   = 0x00000009,
    COLOR_8_8_8_8                                      = 0x0000000a,
    COLOR_32_32                                        = 0x0000000b,
    COLOR_16_16_16_16                                  = 0x0000000c,
    COLOR_RESERVED_13                                  = 0x0000000d,
    COLOR_32_32_32_32                                  = 0x0000000e,
    COLOR_RESERVED_15                                  = 0x0000000f,
    COLOR_5_6_5                                        = 0x00000010,
    COLOR_1_5_5_5                                      = 0x00000011,
    COLOR_5_5_5_1                                      = 0x00000012,
    COLOR_4_4_4_4                                      = 0x00000013,
    COLOR_8_24                                         = 0x00000014,
    COLOR_24_8                                         = 0x00000015,
    COLOR_X24_8_32_FLOAT                               = 0x00000016,
    COLOR_RESERVED_23                                  = 0x00000017,
    COLOR_5_9_9_9                                      = 0x00000018,
    COLOR_RESERVED_25                                  = 0x00000019,
    COLOR_RESERVED_26                                  = 0x0000001a,
    COLOR_RESERVED_27                                  = 0x0000001b,
    COLOR_RESERVED_28                                  = 0x0000001c,
    COLOR_RESERVED_29                                  = 0x0000001d,
    COLOR_2_10_10_10_7E3                               = 0x0000001e,
    COLOR_2_10_10_10_6E4                               = 0x0000001f,
} ColorFormat;

typedef enum ColorTransform {
    DCC_CT_AUTO                                        = 0x00000000,
    DCC_CT_NONE                                        = 0x00000001,
    ABGR_TO_A_BG_G_RB                                  = 0x00000002,
    BGRA_TO_BG_G_RB_A                                  = 0x00000003,
} ColorTransform;

typedef enum CombFunc {
    COMB_DST_PLUS_SRC                                  = 0x00000000,
    COMB_SRC_MINUS_DST                                 = 0x00000001,
    COMB_MIN_DST_SRC                                   = 0x00000002,
    COMB_MAX_DST_SRC                                   = 0x00000003,
    COMB_DST_MINUS_SRC                                 = 0x00000004,
} CombFunc;

typedef enum CompareFrag {
    FRAG_NEVER                                         = 0x00000000,
    FRAG_LESS                                          = 0x00000001,
    FRAG_EQUAL                                         = 0x00000002,
    FRAG_LEQUAL                                        = 0x00000003,
    FRAG_GREATER                                       = 0x00000004,
    FRAG_NOTEQUAL                                      = 0x00000005,
    FRAG_GEQUAL                                        = 0x00000006,
    FRAG_ALWAYS                                        = 0x00000007,
} CompareFrag;

typedef enum CompareRef {
    REF_NEVER                                          = 0x00000000,
    REF_LESS                                           = 0x00000001,
    REF_EQUAL                                          = 0x00000002,
    REF_LEQUAL                                         = 0x00000003,
    REF_GREATER                                        = 0x00000004,
    REF_NOTEQUAL                                       = 0x00000005,
    REF_GEQUAL                                         = 0x00000006,
    REF_ALWAYS                                         = 0x00000007,
} CompareRef;

typedef enum ConservativeZExport {
    EXPORT_ANY_Z                                       = 0x00000000,
    EXPORT_LESS_THAN_Z                                 = 0x00000001,
    EXPORT_GREATER_THAN_Z                              = 0x00000002,
    EXPORT_RESERVED                                    = 0x00000003,
} ConservativeZExport;

typedef enum CovToShaderSel {
    INPUT_COVERAGE                                     = 0x00000000,
    INPUT_INNER_COVERAGE                               = 0x00000001,
    INPUT_DEPTH_COVERAGE                               = 0x00000002,
    RAW                                                = 0x00000003,
} CovToShaderSel;

typedef enum CPC_LATENCY_STATS_SEL {
    CPC_LATENCY_STATS_SEL_XACK_MAX                     = 0x00000000,
    CPC_LATENCY_STATS_SEL_XACK_MIN                     = 0x00000001,
    CPC_LATENCY_STATS_SEL_XACK_LAST                    = 0x00000002,
    CPC_LATENCY_STATS_SEL_XNACK_MAX                    = 0x00000003,
    CPC_LATENCY_STATS_SEL_XNACK_MIN                    = 0x00000004,
    CPC_LATENCY_STATS_SEL_XNACK_LAST                   = 0x00000005,
    CPC_LATENCY_STATS_SEL_WRITE_MAX                    = 0x00000006,
    CPC_LATENCY_STATS_SEL_WRITE_MIN                    = 0x00000007,
    CPC_LATENCY_STATS_SEL_WRITE_LAST                   = 0x00000008,
    CPC_LATENCY_STATS_SEL_READ_MAX                     = 0x00000009,
    CPC_LATENCY_STATS_SEL_READ_MIN                     = 0x0000000a,
    CPC_LATENCY_STATS_SEL_READ_LAST                    = 0x0000000b,
    CPC_LATENCY_STATS_SEL_ATOMIC_MAX                   = 0x0000000c,
    CPC_LATENCY_STATS_SEL_ATOMIC_MIN                   = 0x0000000d,
    CPC_LATENCY_STATS_SEL_ATOMIC_LAST                  = 0x0000000e,
    CPC_LATENCY_STATS_SEL_INVAL_MAX                    = 0x0000000f,
    CPC_LATENCY_STATS_SEL_INVAL_MIN                    = 0x00000010,
    CPC_LATENCY_STATS_SEL_INVAL_LAST                   = 0x00000011,
} CPC_LATENCY_STATS_SEL;

typedef enum CPC_PERFCOUNT_SEL {
    CPC_PERF_SEL_ALWAYS_COUNT                          = 0x00000000,
    CPC_PERF_SEL_RCIU_STALL_WAIT_ON_FREE               = 0x00000001,
    CPC_PERF_SEL_RCIU_STALL_PRIV_VIOLATION             = 0x00000002,
    CPC_PERF_SEL_TCIU_STALL_WAIT_ON_FREE               = 0x00000005,
    CPC_PERF_SEL_ME1_STALL_WAIT_ON_RCIU_READY          = 0x00000006,
    CPC_PERF_SEL_ME1_STALL_WAIT_ON_RCIU_READY_PERF     = 0x00000007,
    CPC_PERF_SEL_ME1_STALL_WAIT_ON_RCIU_READ           = 0x00000008,
    CPC_PERF_SEL_ME1_STALL_WAIT_ON_MEM_READ            = 0x00000009,
    CPC_PERF_SEL_ME1_STALL_WAIT_ON_MEM_WRITE           = 0x0000000a,
    CPC_PERF_SEL_ME1_STALL_ON_DATA_FROM_ROQ            = 0x0000000b,
    CPC_PERF_SEL_ME1_STALL_ON_DATA_FROM_ROQ_PERF       = 0x0000000c,
    CPC_PERF_SEL_ME1_BUSY_FOR_PACKET_DECODE            = 0x0000000d,
    CPC_PERF_SEL_ME2_STALL_WAIT_ON_RCIU_READY          = 0x0000000e,
    CPC_PERF_SEL_ME2_STALL_WAIT_ON_RCIU_READY_PERF     = 0x0000000f,
    CPC_PERF_SEL_ME2_STALL_WAIT_ON_RCIU_READ           = 0x00000010,
    CPC_PERF_SEL_ME2_STALL_WAIT_ON_MEM_READ            = 0x00000011,
    CPC_PERF_SEL_ME2_STALL_WAIT_ON_MEM_WRITE           = 0x00000012,
    CPC_PERF_SEL_ME2_STALL_ON_DATA_FROM_ROQ            = 0x00000013,
    CPC_PERF_SEL_ME2_STALL_ON_DATA_FROM_ROQ_PERF       = 0x00000014,
    CPC_PERF_SEL_ME2_BUSY_FOR_PACKET_DECODE            = 0x00000015,
    CPC_PERF_SEL_UTCL2IU_STALL_WAIT_ON_FREE            = 0x00000016,
    CPC_PERF_SEL_UTCL2IU_STALL_WAIT_ON_TAGS            = 0x00000017,
    CPC_PERF_SEL_UTCL1_STALL_ON_TRANSLATION            = 0x00000018,
    CPC_PERF_SEL_CPC_STAT_BUSY                         = 0x00000019,
    CPC_PERF_SEL_CPC_STAT_IDLE                         = 0x0000001a,
    CPC_PERF_SEL_CPC_STAT_STALL                        = 0x0000001b,
    CPC_PERF_SEL_CPC_TCIU_BUSY                         = 0x0000001c,
    CPC_PERF_SEL_CPC_TCIU_IDLE                         = 0x0000001d,
    CPC_PERF_SEL_CPC_UTCL2IU_BUSY                      = 0x0000001e,
    CPC_PERF_SEL_CPC_UTCL2IU_IDLE                      = 0x0000001f,
    CPC_PERF_SEL_CPC_UTCL2IU_STALL                     = 0x00000020,
    CPC_PERF_SEL_ME1_DC0_SPI_BUSY                      = 0x00000021,
    CPC_PERF_SEL_ME2_DC1_SPI_BUSY                      = 0x00000022,
    CPC_PERF_SEL_CPC_GCRIU_BUSY                        = 0x00000023,
    CPC_PERF_SEL_CPC_GCRIU_IDLE                        = 0x00000024,
    CPC_PERF_SEL_CPC_GCRIU_STALL                       = 0x00000025,
    CPC_PERF_SEL_GCRIU_STALL_WAIT_ON_FREE              = 0x00000026,
    CPC_PERF_SEL_ME1_STALL_WAIT_ON_TCIU_READ           = 0x00000027,
    CPC_PERF_SEL_ME2_STALL_WAIT_ON_TCIU_READ           = 0x00000028,
    CPC_PERF_SEL_CPC_UTCL2IU_XACK                      = 0x00000029,
    CPC_PERF_SEL_CPC_UTCL2IU_XNACK                     = 0x0000002a,
    CPC_PERF_SEL_MEC_INSTR_CACHE_HIT                   = 0x0000002b,
    CPC_PERF_SEL_MEC_INSTR_CACHE_MISS                  = 0x0000002c,
    CPC_PERF_SEL_MES_THREAD0                           = 0x0000002d,
    CPC_PERF_SEL_MES_THREAD1                           = 0x0000002e,
    CPC_PERF_SEL_TCIU_STALL_WAIT_ON_TAGS               = 0x0000002f,
    CPC_PERF_SEL_TCIU_WRITE_REQUEST_SENT               = 0x00000030,
    CPC_PERF_SEL_TCIU_READ_REQUEST_SENT                = 0x00000031,
    CPC_PERF_SEL_MEC_THREAD0                           = 0x00000034,
    CPC_PERF_SEL_MEC_THREAD1                           = 0x00000035,
    CPC_PERF_SEL_MEC_THREAD2                           = 0x00000036,
    CPC_PERF_SEL_MEC_THREAD3                           = 0x00000037,
} CPC_PERFCOUNT_SEL;

constexpr unsigned int MaxCpcPerfcountSel              = CPC_PERF_SEL_MEC_THREAD3;

typedef enum CPF_LATENCY_STATS_SEL {
    CPF_LATENCY_STATS_SEL_XACK_MAX                     = 0x00000000,
    CPF_LATENCY_STATS_SEL_XACK_MIN                     = 0x00000001,
    CPF_LATENCY_STATS_SEL_XACK_LAST                    = 0x00000002,
    CPF_LATENCY_STATS_SEL_XNACK_MAX                    = 0x00000003,
    CPF_LATENCY_STATS_SEL_XNACK_MIN                    = 0x00000004,
    CPF_LATENCY_STATS_SEL_XNACK_LAST                   = 0x00000005,
    CPF_LATENCY_STATS_SEL_READ_MAX                     = 0x00000006,
    CPF_LATENCY_STATS_SEL_READ_MIN                     = 0x00000007,
    CPF_LATENCY_STATS_SEL_READ_LAST                    = 0x00000008,
    CPF_LATENCY_STATS_SEL_INVAL_MAX                    = 0x00000009,
    CPF_LATENCY_STATS_SEL_INVAL_MIN                    = 0x0000000a,
    CPF_LATENCY_STATS_SEL_INVAL_LAST                   = 0x0000000b,
} CPF_LATENCY_STATS_SEL;

typedef enum CPF_PERFCOUNTWINDOW_SEL {
    CPF_PERFWINDOW_SEL_CSF                             = 0x00000000,
    CPF_PERFWINDOW_SEL_HQD1                            = 0x00000001,
    CPF_PERFWINDOW_SEL_HQD2                            = 0x00000002,
    CPF_PERFWINDOW_SEL_RDMA                            = 0x00000003,
    CPF_PERFWINDOW_SEL_RWPP                            = 0x00000004,
} CPF_PERFCOUNTWINDOW_SEL;

typedef enum CPF_PERFCOUNT_SEL {
    CPF_PERF_SEL_ALWAYS_COUNT                          = 0x00000000,
    CPF_PERF_SEL_TCIU_STALLED_WAITING_ON_FREE          = 0x00000002,
    CPF_PERF_SEL_TCIU_STALLED_WAITING_ON_TAGS          = 0x00000003,
    CPF_PERF_SEL_CSF_BUSY_FOR_FETCHING_RING            = 0x00000004,
    CPF_PERF_SEL_CSF_BUSY_FOR_FETCHING_IB1             = 0x00000005,
    CPF_PERF_SEL_CSF_BUSY_FOR_FETCHING_IB2             = 0x00000006,
    CPF_PERF_SEL_CSF_BUSY_FOR_FETCHING_STATE           = 0x00000007,
    CPF_PERF_SEL_CSF_STATE_FIFO_NOT_RTR                = 0x0000000a,
    CPF_PERF_SEL_CSF_FETCHING_CMD_BUFFERS              = 0x0000000b,
    CPF_PERF_SEL_GRBM_DWORDS_SENT                      = 0x0000000c,
    CPF_PERF_SEL_DYNAMIC_CLOCK_VALID                   = 0x0000000d,
    CPF_PERF_SEL_REGISTER_CLOCK_VALID                  = 0x0000000e,
    CPF_PERF_SEL_UTCL2IU_STALL_WAIT_ON_FREE            = 0x00000011,
    CPF_PERF_SEL_UTCL2IU_STALL_WAIT_ON_TAGS            = 0x00000012,
    CPF_PERF_SEL_GFX_UTCL1_STALL_ON_TRANSLATION        = 0x00000013,
    CPF_PERF_SEL_CMP_UTCL1_STALL_ON_TRANSLATION        = 0x00000014,
    CPF_PERF_SEL_RCIU_STALL_WAIT_ON_FREE               = 0x00000015,
    CPF_PERF_SEL_TCIU_WRITE_REQUEST_SENT               = 0x00000016,
    CPF_PERF_SEL_TCIU_READ_REQUEST_SENT                = 0x00000017,
    CPF_PERF_SEL_CPF_STAT_BUSY                         = 0x00000018,
    CPF_PERF_SEL_CPF_STAT_IDLE                         = 0x00000019,
    CPF_PERF_SEL_CPF_STAT_STALL                        = 0x0000001a,
    CPF_PERF_SEL_CPF_TCIU_BUSY                         = 0x0000001b,
    CPF_PERF_SEL_CPF_TCIU_IDLE                         = 0x0000001c,
    CPF_PERF_SEL_CPF_TCIU_STALL                        = 0x0000001d,
    CPF_PERF_SEL_CPF_UTCL2IU_BUSY                      = 0x0000001e,
    CPF_PERF_SEL_CPF_UTCL2IU_IDLE                      = 0x0000001f,
    CPF_PERF_SEL_CPF_UTCL2IU_STALL                     = 0x00000020,
    CPF_PERF_SEL_CPF_GCRIU_BUSY                        = 0x00000021,
    CPF_PERF_SEL_CPF_GCRIU_IDLE                        = 0x00000022,
    CPF_PERF_SEL_CPF_GCRIU_STALL                       = 0x00000023,
    CPF_PERF_SEL_GCRIU_STALL_WAIT_ON_FREE              = 0x00000024,
    CPF_PERF_SEL_CSF_BUSY_FOR_FETCHING_DB              = 0x00000025,
    CPF_PERF_SEL_CPF_UTCL2IU_XACK                      = 0x00000026,
    CPF_PERF_SEL_CPF_UTCL2IU_XNACK                     = 0x00000027,
    CPF_PERF_SEL_CP_SDMA_MNGR_DMA_REQ                  = 0x00000028,
    CPF_PERF_SEL_CP_SDMA_MNGR_DMA_DONE                 = 0x00000029,
    CPF_PERF_SEL_CP_SDMA_MNGR_LATENCY                  = 0x0000002a,
    CPF_PERF_SEL_CP_SDMA_MNGR_SDMABUSY                 = 0x0000002b,
} CPF_PERFCOUNT_SEL;

constexpr unsigned int MaxCpfPerfcountSel              = CPF_PERF_SEL_CP_SDMA_MNGR_SDMABUSY;

typedef enum CPF_SCRATCH_REG_ATOMIC_OP {
    CPF_SCRATCH_REG_ATOMIC_ADD                         = 0x00000000,
    CPF_SCRATCH_REG_ATOMIC_SUB                         = 0x00000001,
    CPF_SCRATCH_REG_ATOMIC_OR                          = 0x00000002,
    CPF_SCRATCH_REG_ATOMIC_AND                         = 0x00000003,
    CPF_SCRATCH_REG_ATOMIC_NOT                         = 0x00000004,
    CPF_SCRATCH_REG_ATOMIC_MIN                         = 0x00000005,
    CPF_SCRATCH_REG_ATOMIC_MAX                         = 0x00000006,
    CPF_SCRATCH_REG_ATOMIC_CMPSWAP                     = 0x00000007,
} CPF_SCRATCH_REG_ATOMIC_OP;

typedef enum CPG_LATENCY_STATS_SEL {
    CPG_LATENCY_STATS_SEL_XACK_MAX                     = 0x00000000,
    CPG_LATENCY_STATS_SEL_XACK_MIN                     = 0x00000001,
    CPG_LATENCY_STATS_SEL_XACK_LAST                    = 0x00000002,
    CPG_LATENCY_STATS_SEL_XNACK_MAX                    = 0x00000003,
    CPG_LATENCY_STATS_SEL_XNACK_MIN                    = 0x00000004,
    CPG_LATENCY_STATS_SEL_XNACK_LAST                   = 0x00000005,
    CPG_LATENCY_STATS_SEL_WRITE_MAX                    = 0x00000006,
    CPG_LATENCY_STATS_SEL_WRITE_MIN                    = 0x00000007,
    CPG_LATENCY_STATS_SEL_WRITE_LAST                   = 0x00000008,
    CPG_LATENCY_STATS_SEL_READ_MAX                     = 0x00000009,
    CPG_LATENCY_STATS_SEL_READ_MIN                     = 0x0000000a,
    CPG_LATENCY_STATS_SEL_READ_LAST                    = 0x0000000b,
    CPG_LATENCY_STATS_SEL_ATOMIC_MAX                   = 0x0000000c,
    CPG_LATENCY_STATS_SEL_ATOMIC_MIN                   = 0x0000000d,
    CPG_LATENCY_STATS_SEL_ATOMIC_LAST                  = 0x0000000e,
    CPG_LATENCY_STATS_SEL_INVAL_MAX                    = 0x0000000f,
    CPG_LATENCY_STATS_SEL_INVAL_MIN                    = 0x00000010,
    CPG_LATENCY_STATS_SEL_INVAL_LAST                   = 0x00000011,
} CPG_LATENCY_STATS_SEL;

typedef enum CPG_PERFCOUNTWINDOW_SEL {
    CPG_PERFWINDOW_SEL_PFP                             = 0x00000000,
    CPG_PERFWINDOW_SEL_ME                              = 0x00000001,
    CPG_PERFWINDOW_SEL_CE                              = 0x00000002,
    CPG_PERFWINDOW_SEL_MES                             = 0x00000003,
    CPG_PERFWINDOW_SEL_MEC1                            = 0x00000004,
    CPG_PERFWINDOW_SEL_MEC2                            = 0x00000005,
    CPG_PERFWINDOW_SEL_DFY                             = 0x00000006,
    CPG_PERFWINDOW_SEL_DMA                             = 0x00000007,
    CPG_PERFWINDOW_SEL_SHADOW                          = 0x00000008,
    CPG_PERFWINDOW_SEL_RB                              = 0x00000009,
    CPG_PERFWINDOW_SEL_CEDMA                           = 0x0000000a,
    CPG_PERFWINDOW_SEL_PRT_HDR_RPTR                    = 0x0000000b,
    CPG_PERFWINDOW_SEL_PRT_SMP_RPTR                    = 0x0000000c,
    CPG_PERFWINDOW_SEL_PQ1                             = 0x0000000d,
    CPG_PERFWINDOW_SEL_PQ2                             = 0x0000000e,
    CPG_PERFWINDOW_SEL_PQ3                             = 0x0000000f,
    CPG_PERFWINDOW_SEL_MEMWR                           = 0x00000010,
    CPG_PERFWINDOW_SEL_MEMRD                           = 0x00000011,
    CPG_PERFWINDOW_SEL_VGT0                            = 0x00000012,
    CPG_PERFWINDOW_SEL_VGT1                            = 0x00000013,
    CPG_PERFWINDOW_SEL_APPEND                          = 0x00000014,
    CPG_PERFWINDOW_SEL_QURD                            = 0x00000015,
    CPG_PERFWINDOW_SEL_DDID                            = 0x00000016,
    CPG_PERFWINDOW_SEL_SR                              = 0x00000017,
    CPG_PERFWINDOW_SEL_QU_EOP                          = 0x00000018,
    CPG_PERFWINDOW_SEL_QU_STRM                         = 0x00000019,
    CPG_PERFWINDOW_SEL_QU_PIPE                         = 0x0000001a,
    CPG_PERFWINDOW_SEL_RESERVED1                       = 0x0000001b,
    CPG_PERFWINDOW_SEL_CPC_IC                          = 0x0000001c,
    CPG_PERFWINDOW_SEL_RESERVED2                       = 0x0000001d,
    CPG_PERFWINDOW_SEL_CPG_IC                          = 0x0000001e,
} CPG_PERFCOUNTWINDOW_SEL;

typedef enum CPG_PERFCOUNT_SEL {
    CPG_PERF_SEL_ALWAYS_COUNT                          = 0x00000000,
    CPG_PERF_SEL_RBIU_FIFO_FULL                        = 0x00000001,
    CPG_PERF_SEL_CP_GRBM_DWORDS_SENT                   = 0x00000004,
    CPG_PERF_SEL_ME_PARSER_BUSY                        = 0x00000005,
    CPG_PERF_SEL_COUNT_TYPE0_PACKETS                   = 0x00000006,
    CPG_PERF_SEL_COUNT_TYPE3_PACKETS                   = 0x00000007,
    CPG_PERF_SEL_CP_GRBM_OUT_OF_CREDITS                = 0x00000009,
    CPG_PERF_SEL_CP_PFP_GRBM_OUT_OF_CREDITS            = 0x0000000a,
    CPG_PERF_SEL_CP_GDS_GRBM_OUT_OF_CREDITS            = 0x0000000b,
    CPG_PERF_SEL_RCIU_STALLED_ON_ME_READ               = 0x0000000c,
    CPG_PERF_SEL_RCIU_STALLED_ON_DMA_READ              = 0x0000000d,
    CPG_PERF_SEL_SSU_STALLED_ON_ACTIVE_CNTX            = 0x0000000e,
    CPG_PERF_SEL_SSU_STALLED_ON_CLEAN_SIGNALS          = 0x0000000f,
    CPG_PERF_SEL_QU_STALLED_ON_EOP_DONE_PULSE          = 0x00000010,
    CPG_PERF_SEL_QU_STALLED_ON_EOP_DONE_WR_CONFIRM     = 0x00000011,
    CPG_PERF_SEL_PFP_STALLED_ON_CSF_READY              = 0x00000012,
    CPG_PERF_SEL_PFP_STALLED_ON_MEQ_READY              = 0x00000013,
    CPG_PERF_SEL_PFP_STALLED_ON_RCIU_READY             = 0x00000014,
    CPG_PERF_SEL_PFP_STALLED_FOR_DATA_FROM_ROQ         = 0x00000015,
    CPG_PERF_SEL_ME_STALLED_FOR_DATA_FROM_PFP          = 0x00000016,
    CPG_PERF_SEL_ME_STALLED_FOR_DATA_FROM_STQ          = 0x00000017,
    CPG_PERF_SEL_ME_STALLED_ON_NO_AVAIL_GFX_CNTX       = 0x00000018,
    CPG_PERF_SEL_ME_STALLED_WRITING_TO_RCIU            = 0x00000019,
    CPG_PERF_SEL_ME_STALLED_WRITING_CONSTANTS          = 0x0000001a,
    CPG_PERF_SEL_ME_STALLED_ON_PARTIAL_FLUSH           = 0x0000001b,
    CPG_PERF_SEL_ME_WAIT_ON_CE_COUNTER                 = 0x0000001c,
    CPG_PERF_SEL_ME_WAIT_ON_AVAIL_BUFFER               = 0x0000001d,
    CPG_PERF_SEL_LOAD_STALLED_ON_SET_COHERENCY         = 0x0000001f,
    CPG_PERF_SEL_DYNAMIC_CLK_VALID                     = 0x00000020,
    CPG_PERF_SEL_REGISTER_CLK_VALID                    = 0x00000021,
    CPG_PERF_SEL_CE_STALL_RAM_DUMP                     = 0x00000024,
    CPG_PERF_SEL_CE_STALL_RAM_WRITE                    = 0x00000025,
    CPG_PERF_SEL_CE_STALL_ON_INC_FIFO                  = 0x00000026,
    CPG_PERF_SEL_CE_STALL_ON_WR_RAM_FIFO               = 0x00000027,
    CPG_PERF_SEL_CE_STALL_ON_DATA_FROM_ROQ             = 0x00000029,
    CPG_PERF_SEL_CE_STALL_ON_CE_BUFFER_FLAG            = 0x0000002a,
    CPG_PERF_SEL_CE_STALL_ON_DE_COUNTER                = 0x0000002b,
    CPG_PERF_SEL_TCIU_STALL_WAIT_ON_FREE               = 0x0000002c,
    CPG_PERF_SEL_TCIU_STALL_WAIT_ON_TAGS               = 0x0000002d,
    CPG_PERF_SEL_UTCL2IU_STALL_WAIT_ON_FREE            = 0x0000002e,
    CPG_PERF_SEL_UTCL2IU_STALL_WAIT_ON_TAGS            = 0x0000002f,
    CPG_PERF_SEL_UTCL1_STALL_ON_TRANSLATION            = 0x00000030,
    CPG_PERF_SEL_TCIU_WRITE_REQUEST_SENT               = 0x00000031,
    CPG_PERF_SEL_TCIU_READ_REQUEST_SENT                = 0x00000032,
    CPG_PERF_SEL_CPG_STAT_BUSY                         = 0x00000033,
    CPG_PERF_SEL_CPG_STAT_IDLE                         = 0x00000034,
    CPG_PERF_SEL_CPG_STAT_STALL                        = 0x00000035,
    CPG_PERF_SEL_CPG_TCIU_BUSY                         = 0x00000036,
    CPG_PERF_SEL_CPG_TCIU_IDLE                         = 0x00000037,
    CPG_PERF_SEL_CPG_TCIU_STALL                        = 0x00000038,
    CPG_PERF_SEL_CPG_UTCL2IU_BUSY                      = 0x00000039,
    CPG_PERF_SEL_CPG_UTCL2IU_IDLE                      = 0x0000003a,
    CPG_PERF_SEL_CPG_UTCL2IU_STALL                     = 0x0000003b,
    CPG_PERF_SEL_CPG_GCRIU_BUSY                        = 0x0000003c,
    CPG_PERF_SEL_CPG_GCRIU_IDLE                        = 0x0000003d,
    CPG_PERF_SEL_CPG_GCRIU_STALL                       = 0x0000003e,
    CPG_PERF_SEL_GCRIU_STALL_WAIT_ON_FREE              = 0x0000003f,
    CPG_PERF_SEL_ALL_GFX_PIPES_BUSY                    = 0x00000040,
    CPG_PERF_SEL_CPG_UTCL2IU_XACK                      = 0x00000041,
    CPG_PERF_SEL_CPG_UTCL2IU_XNACK                     = 0x00000042,
    CPG_PERF_SEL_PFP_STALLED_ON_MEQ_DDID_READY         = 0x00000043,
    CPG_PERF_SEL_PFP_INSTR_CACHE_HIT                   = 0x00000044,
    CPG_PERF_SEL_PFP_INSTR_CACHE_MISS                  = 0x00000045,
    CPG_PERF_SEL_CE_INSTR_CACHE_HIT                    = 0x00000046,
    CPG_PERF_SEL_CE_INSTR_CACHE_MISS                   = 0x00000047,
    CPG_PERF_SEL_ME_INSTR_CACHE_HIT                    = 0x00000048,
    CPG_PERF_SEL_ME_INSTR_CACHE_MISS                   = 0x00000049,
    CPG_PERF_SEL_PFP_PACKET_FILTER_HIT_IB1             = 0x0000004a,
    CPG_PERF_SEL_PFP_PACKET_FILTER_MISS_IB1            = 0x0000004b,
    CPG_PERF_SEL_PFP_PACKET_FILTER_HIT_IB2             = 0x0000004c,
    CPG_PERF_SEL_PFP_PACKET_FILTER_MISS_IB2            = 0x0000004d,
    CPG_PERF_SEL_DMA_BUSY                              = 0x0000004e,
    CPG_PERF_SEL_DMA_STARVED                           = 0x0000004f,
    CPG_PERF_SEL_DMA_STALLED                           = 0x00000050,
    CPG_PERF_SEL_DMA_FETCHER_STALLED_ON_ROQ_FULL       = 0x00000051,
    CPG_PERF_SEL_PFP_PWS_STALLED0                      = 0x00000052,
    CPG_PERF_SEL_ME_PWS_STALLED0                       = 0x00000053,
    CPG_PERF_SEL_PFP_VGTDMA_INDR_STRUCT_BYPASS0        = 0x00000054,
    CPG_PERF_SEL_PFP_VGTDMA_INDR_STRUCT_NOT_BYPASS0    = 0x00000055,
    CPG_PERF_SEL_PFP_VGTDMA_DB_ROQ_DATA_STALL0         = 0x00000056,
    CPG_PERF_SEL_PFP_PWS_STALLED1                      = 0x00000057,
    CPG_PERF_SEL_ME_PWS_STALLED1                       = 0x00000058,
    CPG_PERF_SEL_PFP_VGTDMA_INDR_STRUCT_BYPASS1        = 0x00000059,
    CPG_PERF_SEL_PFP_VGTDMA_INDR_STRUCT_NOT_BYPASS1    = 0x0000005a,
    CPG_PERF_SEL_PFP_VGTDMA_DB_ROQ_DATA_STALL1         = 0x0000005b,
    CPG_PERF_SEL_PFP_RS64_THREAD0                      = 0x0000005c,
    CPG_PERF_SEL_ME_RS64_THREAD0                       = 0x0000005d,
    CPG_PERF_SEL_PFP_RS64_THREAD1                      = 0x0000005e,
    CPG_PERF_SEL_ME_RS64_THREAD1                       = 0x0000005f,
} CPG_PERFCOUNT_SEL;

constexpr unsigned int MaxCpgPerfcountSel              = CPG_PERF_SEL_ME_RS64_THREAD1;

typedef enum CP_PERFMON_ENABLE_MODE {
    CP_PERFMON_ENABLE_MODE_ALWAYS_COUNT                = 0x00000000,
    CP_PERFMON_ENABLE_MODE_RESERVED_1                  = 0x00000001,
    CP_PERFMON_ENABLE_MODE_COUNT_CONTEXT_TRUE          = 0x00000002,
    CP_PERFMON_ENABLE_MODE_COUNT_CONTEXT_FALSE         = 0x00000003,
} CP_PERFMON_ENABLE_MODE;

typedef enum CP_PERFMON_STATE {
    CP_PERFMON_STATE_DISABLE_AND_RESET                 = 0x00000000,
    CP_PERFMON_STATE_START_COUNTING                    = 0x00000001,
    CP_PERFMON_STATE_STOP_COUNTING                     = 0x00000002,
    CP_PERFMON_STATE_RESERVED_3                        = 0x00000003,
    CP_PERFMON_STATE_DISABLE_AND_RESET_PHANTOM         = 0x00000004,
    CP_PERFMON_STATE_COUNT_AND_DUMP_PHANTOM            = 0x00000005,
} CP_PERFMON_STATE;

typedef enum DbMemArbWatermarks {
    TRANSFERRED_64_BYTES                               = 0x00000000,
    TRANSFERRED_128_BYTES                              = 0x00000001,
    TRANSFERRED_256_BYTES                              = 0x00000002,
    TRANSFERRED_512_BYTES                              = 0x00000003,
    TRANSFERRED_1024_BYTES                             = 0x00000004,
    TRANSFERRED_2048_BYTES                             = 0x00000005,
    TRANSFERRED_4096_BYTES                             = 0x00000006,
    TRANSFERRED_8192_BYTES                             = 0x00000007,
} DbMemArbWatermarks;

typedef enum DbPSLControl {
    PSLC_AUTO                                          = 0x00000000,
    PSLC_ON_HANG_ONLY                                  = 0x00000001,
    PSLC_ASAP                                          = 0x00000002,
    PSLC_COUNTDOWN                                     = 0x00000003,
} DbPSLControl;

typedef enum DepthArray {
    ARRAY_2D_ALT_DEPTH                                 = 0x00000000,
    ARRAY_2D_DEPTH                                     = 0x00000001,
} DepthArray;

typedef enum DepthFormat {
    DEPTH_INVALID                                      = 0x00000000,
    DEPTH_16                                           = 0x00000001,
    DEPTH_X8_24                                        = 0x00000002,
    DEPTH_8_24                                         = 0x00000003,
    DEPTH_X8_24_FLOAT                                  = 0x00000004,
    DEPTH_8_24_FLOAT                                   = 0x00000005,
    DEPTH_32_FLOAT                                     = 0x00000006,
    DEPTH_X24_8_32_FLOAT                               = 0x00000007,
} DepthFormat;

typedef enum DF_MALL_PERF_SEL {
    DF_MALL_PERF_SEL_ML_MTQ_OCC                        = 0x00000000,
    DF_MALL_PERF_SEL_ML_MRS_OCC                        = 0x00000001,
    DF_MALL_PERF_SEL_ML_REQ0                           = 0x00000002,
    DF_MALL_PERF_SEL_ML_OPTYPES                        = 0x00000003,
    DF_MALL_PERF_SEL_ML_CACHE_LKUP_RSP0                = 0x00000004,
    DF_MALL_PERF_SEL_ML_CACHE_LKUP_RSP1                = 0x00000005,
    DF_MALL_PERF_SEL_ML_CACHE_TAG_PICK_STALL           = 0x00000006,
    DF_MALL_PERF_SEL_ML_UMC_PICK_STALL                 = 0x00000007,
    DF_MALL_PERF_SEL_ML_MISC0                          = 0x00000008,
    DF_MALL_PERF_SEL_ML_MISC1                          = 0x00000009,
    DF_MALL_PERF_SEL_ML_CACHE_LKUP_RSP2                = 0x0000000a,
    DF_MALL_PERF_SEL_ML_CACHE_LKUP_RSP3                = 0x0000000b,
    DF_MALL_PERF_SEL_ML_REQ1                           = 0x0000000c,
    DF_MALL_PERF_SEL_ML_OPTYPES_EXT                    = 0x0000000d,
    DF_MALL_PERF_SEL_ML_MVQ_OCC                        = 0x0000000e,
    DF_MALL_PERF_SEL_ML_SPEC_MEMRD                     = 0x0000000f,
    DF_MALL_PERF_SEL_ML_DCC                            = 0x00000010,
    DF_MALL_PERF_SEL_ML_CVQ_OCC                        = 0x00000011,
    DF_MALL_PERF_SEL_ML_CTQ_OCC                        = 0x00000012,
    DF_MALL_PERF_SEL_ML_CTQ_REQ0                       = 0x00000013,
    DF_MALL_PERF_SEL_ML_CTQ_REQ1                       = 0x00000014,
    DF_MALL_PERF_SEL_ML_CTQ_REQ2                       = 0x00000015,
    DF_MALL_PERF_SEL_ML_CVQ                            = 0x00000016,
    DF_MALL_PERF_SEL_ML_CTQ_PICK                       = 0x00000017,
    DF_MALL_PERF_SEL_ML_UMC_SPM                        = 0x0000002f,
    DF_MALL_PERF_SEL_MALL_SDP_AVG_LAT_TRANS_CNT        = 0x00000030,
    DF_MALL_PERF_SEL_MALL_SDP_AVG_LAT_CYCLE_CNT        = 0x00000031,
    DF_MALL_PERF_SEL_MALL_SDP_LAT_HIST_GT50            = 0x00000032,
    DF_MALL_PERF_SEL_MALL_SDP_LAT_HIST_GT100           = 0x00000033,
    DF_MALL_PERF_SEL_MALL_SDP_LAT_HIST_GT150           = 0x00000034,
    DF_MALL_PERF_SEL_MALL_SDP_LAT_HIST_GT200           = 0x00000035,
    DF_MALL_PERF_SEL_MALL_SDP_LAT_HIST_GT500           = 0x00000036,
    DF_MALL_PERF_SEL_MALL_SDP_LAT_HIST_GT1000          = 0x00000037,
} DF_MALL_PERF_SEL;

constexpr unsigned int MaxDfMallPerfSel                = DF_MALL_PERF_SEL_MALL_SDP_LAT_HIST_GT1000;

typedef enum ForceControl {
    FORCE_OFF                                          = 0x00000000,
    FORCE_ENABLE                                       = 0x00000001,
    FORCE_DISABLE                                      = 0x00000002,
    FORCE_RESERVED                                     = 0x00000003,
} ForceControl;

typedef enum GCRPerfSel {
    GCR_PERF_SEL_NONE                                  = 0x00000000,
    GCR_PERF_SEL_SDMA0_ALL_REQ                         = 0x00000001,
    GCR_PERF_SEL_SDMA0_GL2_RANGE_REQ                   = 0x00000002,
    GCR_PERF_SEL_SDMA0_GL2_RANGE_LT16K_REQ             = 0x00000003,
    GCR_PERF_SEL_SDMA0_GL2_RANGE_16K_REQ               = 0x00000004,
    GCR_PERF_SEL_SDMA0_GL2_RANGE_GT16K_REQ             = 0x00000005,
    GCR_PERF_SEL_SDMA0_GL2_ALL_REQ                     = 0x00000006,
    GCR_PERF_SEL_SDMA0_GL1_RANGE_REQ                   = 0x00000007,
    GCR_PERF_SEL_SDMA0_GL1_RANGE_LT16K_REQ             = 0x00000008,
    GCR_PERF_SEL_SDMA0_GL1_RANGE_16K_REQ               = 0x00000009,
    GCR_PERF_SEL_SDMA0_GL1_RANGE_GT16K_REQ             = 0x0000000a,
    GCR_PERF_SEL_SDMA0_GL1_ALL_REQ                     = 0x0000000b,
    GCR_PERF_SEL_SDMA0_METADATA_REQ                    = 0x0000000c,
    GCR_PERF_SEL_SDMA0_SQC_DATA_REQ                    = 0x0000000d,
    GCR_PERF_SEL_SDMA0_SQC_INST_REQ                    = 0x0000000e,
    GCR_PERF_SEL_SDMA0_TCP_REQ                         = 0x0000000f,
    GCR_PERF_SEL_SDMA0_GL1_TLB_SHOOTDOWN_REQ           = 0x00000010,
    GCR_PERF_SEL_SDMA1_ALL_REQ                         = 0x00000011,
    GCR_PERF_SEL_SDMA1_GL2_RANGE_REQ                   = 0x00000012,
    GCR_PERF_SEL_SDMA1_GL2_RANGE_LT16K_REQ             = 0x00000013,
    GCR_PERF_SEL_SDMA1_GL2_RANGE_16K_REQ               = 0x00000014,
    GCR_PERF_SEL_SDMA1_GL2_RANGE_GT16K_REQ             = 0x00000015,
    GCR_PERF_SEL_SDMA1_GL2_ALL_REQ                     = 0x00000016,
    GCR_PERF_SEL_SDMA1_GL1_RANGE_REQ                   = 0x00000017,
    GCR_PERF_SEL_SDMA1_GL1_RANGE_LT16K_REQ             = 0x00000018,
    GCR_PERF_SEL_SDMA1_GL1_RANGE_16K_REQ               = 0x00000019,
    GCR_PERF_SEL_SDMA1_GL1_RANGE_GT16K_REQ             = 0x0000001a,
    GCR_PERF_SEL_SDMA1_GL1_ALL_REQ                     = 0x0000001b,
    GCR_PERF_SEL_SDMA1_METADATA_REQ                    = 0x0000001c,
    GCR_PERF_SEL_SDMA1_SQC_DATA_REQ                    = 0x0000001d,
    GCR_PERF_SEL_SDMA1_SQC_INST_REQ                    = 0x0000001e,
    GCR_PERF_SEL_SDMA1_TCP_REQ                         = 0x0000001f,
    GCR_PERF_SEL_SDMA1_GL1_TLB_SHOOTDOWN_REQ           = 0x00000020,
    GCR_PERF_SEL_CPC_ALL_REQ                           = 0x00000021,
    GCR_PERF_SEL_CPC_GL2_RANGE_REQ                     = 0x00000022,
    GCR_PERF_SEL_CPC_GL2_RANGE_LT16K_REQ               = 0x00000023,
    GCR_PERF_SEL_CPC_GL2_RANGE_16K_REQ                 = 0x00000024,
    GCR_PERF_SEL_CPC_GL2_RANGE_GT16K_REQ               = 0x00000025,
    GCR_PERF_SEL_CPC_GL2_ALL_REQ                       = 0x00000026,
    GCR_PERF_SEL_CPC_GL1_RANGE_REQ                     = 0x00000027,
    GCR_PERF_SEL_CPC_GL1_RANGE_LT16K_REQ               = 0x00000028,
    GCR_PERF_SEL_CPC_GL1_RANGE_16K_REQ                 = 0x00000029,
    GCR_PERF_SEL_CPC_GL1_RANGE_GT16K_REQ               = 0x0000002a,
    GCR_PERF_SEL_CPC_GL1_ALL_REQ                       = 0x0000002b,
    GCR_PERF_SEL_CPC_METADATA_REQ                      = 0x0000002c,
    GCR_PERF_SEL_CPC_SQC_DATA_REQ                      = 0x0000002d,
    GCR_PERF_SEL_CPC_SQC_INST_REQ                      = 0x0000002e,
    GCR_PERF_SEL_CPC_TCP_REQ                           = 0x0000002f,
    GCR_PERF_SEL_CPC_GL1_TLB_SHOOTDOWN_REQ             = 0x00000030,
    GCR_PERF_SEL_CPG_ALL_REQ                           = 0x00000031,
    GCR_PERF_SEL_CPG_GL2_RANGE_REQ                     = 0x00000032,
    GCR_PERF_SEL_CPG_GL2_RANGE_LT16K_REQ               = 0x00000033,
    GCR_PERF_SEL_CPG_GL2_RANGE_16K_REQ                 = 0x00000034,
    GCR_PERF_SEL_CPG_GL2_RANGE_GT16K_REQ               = 0x00000035,
    GCR_PERF_SEL_CPG_GL2_ALL_REQ                       = 0x00000036,
    GCR_PERF_SEL_CPG_GL1_RANGE_REQ                     = 0x00000037,
    GCR_PERF_SEL_CPG_GL1_RANGE_LT16K_REQ               = 0x00000038,
    GCR_PERF_SEL_CPG_GL1_RANGE_16K_REQ                 = 0x00000039,
    GCR_PERF_SEL_CPG_GL1_RANGE_GT16K_REQ               = 0x0000003a,
    GCR_PERF_SEL_CPG_GL1_ALL_REQ                       = 0x0000003b,
    GCR_PERF_SEL_CPG_METADATA_REQ                      = 0x0000003c,
    GCR_PERF_SEL_CPG_SQC_DATA_REQ                      = 0x0000003d,
    GCR_PERF_SEL_CPG_SQC_INST_REQ                      = 0x0000003e,
    GCR_PERF_SEL_CPG_TCP_REQ                           = 0x0000003f,
    GCR_PERF_SEL_CPG_GL1_TLB_SHOOTDOWN_REQ             = 0x00000040,
    GCR_PERF_SEL_CPF_ALL_REQ                           = 0x00000041,
    GCR_PERF_SEL_CPF_GL2_RANGE_REQ                     = 0x00000042,
    GCR_PERF_SEL_CPF_GL2_RANGE_LT16K_REQ               = 0x00000043,
    GCR_PERF_SEL_CPF_GL2_RANGE_16K_REQ                 = 0x00000044,
    GCR_PERF_SEL_CPF_GL2_RANGE_GT16K_REQ               = 0x00000045,
    GCR_PERF_SEL_CPF_GL2_ALL_REQ                       = 0x00000046,
    GCR_PERF_SEL_CPF_GL1_RANGE_REQ                     = 0x00000047,
    GCR_PERF_SEL_CPF_GL1_RANGE_LT16K_REQ               = 0x00000048,
    GCR_PERF_SEL_CPF_GL1_RANGE_16K_REQ                 = 0x00000049,
    GCR_PERF_SEL_CPF_GL1_RANGE_GT16K_REQ               = 0x0000004a,
    GCR_PERF_SEL_CPF_GL1_ALL_REQ                       = 0x0000004b,
    GCR_PERF_SEL_CPF_METADATA_REQ                      = 0x0000004c,
    GCR_PERF_SEL_CPF_SQC_DATA_REQ                      = 0x0000004d,
    GCR_PERF_SEL_CPF_SQC_INST_REQ                      = 0x0000004e,
    GCR_PERF_SEL_CPF_TCP_REQ                           = 0x0000004f,
    GCR_PERF_SEL_CPF_GL1_TLB_SHOOTDOWN_REQ             = 0x00000050,
    GCR_PERF_SEL_VIRT_REQ                              = 0x00000051,
    GCR_PERF_SEL_PHY_REQ                               = 0x00000052,
    GCR_PERF_SEL_TLB_SHOOTDOWN_HEAVY_REQ               = 0x00000053,
    GCR_PERF_SEL_TLB_SHOOTDOWN_LIGHT_REQ               = 0x00000054,
    GCR_PERF_SEL_ALL_REQ                               = 0x00000055,
    GCR_PERF_SEL_CLK_FOR_PHY_OUTSTANDING_REQ           = 0x00000056,
    GCR_PERF_SEL_CLK_FOR_VIRT_OUTSTANDING_REQ          = 0x00000057,
    GCR_PERF_SEL_CLK_FOR_ALL_OUTSTANDING_REQ           = 0x00000058,
    GCR_PERF_SEL_UTCL2_REQ                             = 0x00000059,
    GCR_PERF_SEL_UTCL2_RET                             = 0x0000005a,
    GCR_PERF_SEL_UTCL2_OUT_OF_CREDIT_EVENT             = 0x0000005b,
    GCR_PERF_SEL_UTCL2_INFLIGHT_REQ                    = 0x0000005c,
    GCR_PERF_SEL_UTCL2_FILTERED_RET                    = 0x0000005d,
    GCR_PERF_SEL_PMM_ABIT_NUM_FLUSH                    = 0x0000005e,
    GCR_PERF_SEL_PMM_ABIT_FLUSH_ONGOING                = 0x0000005f,
    GCR_PERF_SEL_PMM_NUM_INTERRUPT                     = 0x00000060,
    GCR_PERF_SEL_PMM_STALL_PMM_IH_CREDITS              = 0x00000061,
    GCR_PERF_SEL_PMM_INTERRUPT_READY_TO_SEND           = 0x00000062,
    GCR_PERF_SEL_PMM_ABIT_FORCE_FLUSH                  = 0x00000063,
    GCR_PERF_SEL_PMM_ABIT_FLUSH_INTERRUPT              = 0x00000064,
    GCR_PERF_SEL_PMM_ALOG_INTERRUPT                    = 0x00000065,
    GCR_PERF_SEL_PMM_MAM_FLUSH_REQ                     = 0x00000066,
    GCR_PERF_SEL_PMM_MAM_FLUSH_RESP                    = 0x00000067,
    GCR_PERF_SEL_RLC_ALL_REQ                           = 0x00000068,
    GCR_PERF_SEL_RLC_GL2_RANGE_REQ                     = 0x00000069,
    GCR_PERF_SEL_RLC_GL2_RANGE_LT16K_REQ               = 0x0000006a,
    GCR_PERF_SEL_RLC_GL2_RANGE_16K_REQ                 = 0x0000006b,
    GCR_PERF_SEL_RLC_GL2_RANGE_GT16K_REQ               = 0x0000006c,
    GCR_PERF_SEL_RLC_GL2_ALL_REQ                       = 0x0000006d,
    GCR_PERF_SEL_RLC_GL1_RANGE_REQ                     = 0x0000006e,
    GCR_PERF_SEL_RLC_GL1_RANGE_LT16K_REQ               = 0x0000006f,
    GCR_PERF_SEL_RLC_GL1_RANGE_16K_REQ                 = 0x00000070,
    GCR_PERF_SEL_RLC_GL1_RANGE_GT16K_REQ               = 0x00000071,
    GCR_PERF_SEL_RLC_GL1_ALL_REQ                       = 0x00000072,
    GCR_PERF_SEL_RLC_METADATA_REQ                      = 0x00000073,
    GCR_PERF_SEL_RLC_SQC_DATA_REQ                      = 0x00000074,
    GCR_PERF_SEL_RLC_SQC_INST_REQ                      = 0x00000075,
    GCR_PERF_SEL_RLC_TCP_REQ                           = 0x00000076,
    GCR_PERF_SEL_RLC_GL1_TLB_SHOOTDOWN_REQ             = 0x00000077,
    GCR_PERF_SEL_PM_ALL_REQ                            = 0x00000078,
    GCR_PERF_SEL_PM_GL2_RANGE_REQ                      = 0x00000079,
    GCR_PERF_SEL_PM_GL2_RANGE_LT16K_REQ                = 0x0000007a,
    GCR_PERF_SEL_PM_GL2_RANGE_16K_REQ                  = 0x0000007b,
    GCR_PERF_SEL_PM_GL2_RANGE_GT16K_REQ                = 0x0000007c,
    GCR_PERF_SEL_PM_GL2_ALL_REQ                        = 0x0000007d,
    GCR_PERF_SEL_PM_GL1_RANGE_REQ                      = 0x0000007e,
    GCR_PERF_SEL_PM_GL1_RANGE_LT16K_REQ                = 0x0000007f,
    GCR_PERF_SEL_PM_GL1_RANGE_16K_REQ                  = 0x00000080,
    GCR_PERF_SEL_PM_GL1_RANGE_GT16K_REQ                = 0x00000081,
    GCR_PERF_SEL_PM_GL1_ALL_REQ                        = 0x00000082,
    GCR_PERF_SEL_PM_METADATA_REQ                       = 0x00000083,
    GCR_PERF_SEL_PM_SQC_DATA_REQ                       = 0x00000084,
    GCR_PERF_SEL_PM_SQC_INST_REQ                       = 0x00000085,
    GCR_PERF_SEL_PM_TCP_REQ                            = 0x00000086,
    GCR_PERF_SEL_PM_GL1_TLB_SHOOTDOWN_REQ              = 0x00000087,
    GCR_PERF_SEL_PIO_ALL_REQ                           = 0x00000088,
    GCR_PERF_SEL_PIO_GL2_RANGE_REQ                     = 0x00000089,
    GCR_PERF_SEL_PIO_GL2_RANGE_LT16K_REQ               = 0x0000008a,
    GCR_PERF_SEL_PIO_GL2_RANGE_16K_REQ                 = 0x0000008b,
    GCR_PERF_SEL_PIO_GL2_RANGE_GT16K_REQ               = 0x0000008c,
    GCR_PERF_SEL_PIO_GL2_ALL_REQ                       = 0x0000008d,
    GCR_PERF_SEL_PIO_GL1_RANGE_REQ                     = 0x0000008e,
    GCR_PERF_SEL_PIO_GL1_RANGE_LT16K_REQ               = 0x0000008f,
    GCR_PERF_SEL_PIO_GL1_RANGE_16K_REQ                 = 0x00000090,
    GCR_PERF_SEL_PIO_GL1_RANGE_GT16K_REQ               = 0x00000091,
    GCR_PERF_SEL_PIO_GL1_ALL_REQ                       = 0x00000092,
    GCR_PERF_SEL_PIO_METADATA_REQ                      = 0x00000093,
    GCR_PERF_SEL_PIO_SQC_DATA_REQ                      = 0x00000094,
    GCR_PERF_SEL_PIO_SQC_INST_REQ                      = 0x00000095,
    GCR_PERF_SEL_PIO_TCP_REQ                           = 0x00000096,
    GCR_PERF_SEL_PIO_GL1_TLB_SHOOTDOWN_REQ             = 0x00000097,
} GCRPerfSel;

constexpr unsigned int MaxGCRPerfSel                   = GCR_PERF_SEL_PIO_GL1_TLB_SHOOTDOWN_REQ;

typedef enum GCVML2_PERF_SEL {
    GCVML2_PERF_SEL_EVENT_0                            = 0x00000000,
    GCVML2_PERF_SEL_EVENT_1                            = 0x00000001,
    GCVML2_PERF_SEL_EVENT_2                            = 0x00000002,
    GCVML2_PERF_SEL_EVENT_3                            = 0x00000003,
    GCVML2_PERF_SEL_EVENT_4                            = 0x00000004,
    GCVML2_PERF_SEL_EVENT_5                            = 0x00000005,
    GCVML2_PERF_SEL_EVENT_6                            = 0x00000006,
    GCVML2_PERF_SEL_EVENT_7                            = 0x00000007,
    GCVML2_PERF_SEL_EVENT_8                            = 0x00000008,
    GCVML2_PERF_SEL_EVENT_9                            = 0x00000009,
    GCVML2_PERF_SEL_EVENT_10                           = 0x0000000a,
    GCVML2_PERF_SEL_EVENT_11                           = 0x0000000b,
    GCVML2_PERF_SEL_EVENT_12                           = 0x0000000c,
    GCVML2_PERF_SEL_EVENT_13                           = 0x0000000d,
    GCVML2_PERF_SEL_EVENT_14                           = 0x0000000e,
    GCVML2_PERF_SEL_EVENT_15                           = 0x0000000f,
    GCVML2_PERF_SEL_EVENT_16                           = 0x00000010,
    GCVML2_PERF_SEL_EVENT_17                           = 0x00000011,
    GCVML2_PERF_SEL_EVENT_18                           = 0x00000012,
    GCVML2_PERF_SEL_EVENT_19                           = 0x00000013,
    GCVML2_PERF_SEL_EVENT_20                           = 0x00000014,
    GCVML2_PERF_SEL_EVENT_21                           = 0x00000015,
    GCVML2_PERF_SEL_EVENT_22                           = 0x00000016,
    GCVML2_PERF_SEL_EVENT_23                           = 0x00000017,
    GCVML2_PERF_SEL_EVENT_24                           = 0x00000018,
    GCVML2_PERF_SEL_EVENT_25                           = 0x00000019,
    GCVML2_PERF_SEL_EVENT_26                           = 0x0000001a,
    GCVML2_PERF_SEL_EVENT_27                           = 0x0000001b,
    GCVML2_PERF_SEL_EVENT_28                           = 0x0000001c,
    GCVML2_PERF_SEL_EVENT_29                           = 0x0000001d,
    GCVML2_PERF_SEL_EVENT_30                           = 0x0000001e,
    GCVML2_PERF_SEL_EVENT_31                           = 0x0000001f,
    GCVML2_PERF_SEL_EVENT_32                           = 0x00000020,
    GCVML2_PERF_SEL_EVENT_33                           = 0x00000021,
    GCVML2_PERF_SEL_EVENT_34                           = 0x00000022,
    GCVML2_PERF_SEL_EVENT_35                           = 0x00000023,
    GCVML2_PERF_SEL_EVENT_36                           = 0x00000024,
    GCVML2_PERF_SEL_EVENT_37                           = 0x00000025,
    GCVML2_PERF_SEL_EVENT_38                           = 0x00000026,
    GCVML2_PERF_SEL_EVENT_39                           = 0x00000027,
    GCVML2_PERF_SEL_EVENT_40                           = 0x00000028,
    GCVML2_PERF_SEL_EVENT_41                           = 0x00000029,
    GCVML2_PERF_SEL_EVENT_42                           = 0x0000002a,
    GCVML2_PERF_SEL_EVENT_43                           = 0x0000002b,
    GCVML2_PERF_SEL_EVENT_44                           = 0x0000002c,
    GCVML2_PERF_SEL_EVENT_45                           = 0x0000002d,
    GCVML2_PERF_SEL_EVENT_46                           = 0x0000002e,
    GCVML2_PERF_SEL_EVENT_47                           = 0x0000002f,
    GCVML2_PERF_SEL_EVENT_48                           = 0x00000030,
    GCVML2_PERF_SEL_EVENT_49                           = 0x00000031,
    GCVML2_PERF_SEL_EVENT_50                           = 0x00000032,
    GCVML2_PERF_SEL_EVENT_51                           = 0x00000033,
    GCVML2_PERF_SEL_EVENT_52                           = 0x00000034,
    GCVML2_PERF_SEL_EVENT_53                           = 0x00000035,
    GCVML2_PERF_SEL_EVENT_54                           = 0x00000036,
    GCVML2_PERF_SEL_EVENT_55                           = 0x00000037,
    GCVML2_PERF_SEL_EVENT_56                           = 0x00000038,
    GCVML2_PERF_SEL_EVENT_57                           = 0x00000039,
    GCVML2_PERF_SEL_EVENT_58                           = 0x0000003a,
    GCVML2_PERF_SEL_EVENT_59                           = 0x0000003b,
    GCVML2_PERF_SEL_EVENT_60                           = 0x0000003c,
    GCVML2_PERF_SEL_EVENT_61                           = 0x0000003d,
    GCVML2_PERF_SEL_EVENT_62                           = 0x0000003e,
    GCVML2_PERF_SEL_EVENT_63                           = 0x0000003f,
    GCVML2_PERF_SEL_EVENT_64                           = 0x00000040,
    GCVML2_PERF_SEL_EVENT_65                           = 0x00000041,
    GCVML2_PERF_SEL_EVENT_66                           = 0x00000042,
    GCVML2_PERF_SEL_EVENT_67                           = 0x00000043,
    GCVML2_PERF_SEL_EVENT_68                           = 0x00000044,
    GCVML2_PERF_SEL_EVENT_69                           = 0x00000045,
    GCVML2_PERF_SEL_EVENT_70                           = 0x00000046,
    GCVML2_PERF_SEL_EVENT_71                           = 0x00000047,
    GCVML2_PERF_SEL_EVENT_72                           = 0x00000048,
    GCVML2_PERF_SEL_EVENT_73                           = 0x00000049,
    GCVML2_PERF_SEL_EVENT_74                           = 0x0000004a,
    GCVML2_PERF_SEL_EVENT_75                           = 0x0000004b,
    GCVML2_PERF_SEL_EVENT_76                           = 0x0000004c,
    GCVML2_PERF_SEL_EVENT_77                           = 0x0000004d,
    GCVML2_PERF_SEL_EVENT_78                           = 0x0000004e,
    GCVML2_PERF_SEL_EVENT_79                           = 0x0000004f,
    GCVML2_PERF_SEL_EVENT_80                           = 0x00000050,
    GCVML2_PERF_SEL_EVENT_81                           = 0x00000051,
    GCVML2_PERF_SEL_EVENT_82                           = 0x00000052,
    GCVML2_PERF_SEL_EVENT_83                           = 0x00000053,
    GCVML2_PERF_SEL_EVENT_84                           = 0x00000054,
    GCVML2_PERF_SEL_EVENT_85                           = 0x00000055,
    GCVML2_PERF_SEL_EVENT_86                           = 0x00000056,
    GCVML2_PERF_SEL_EVENT_87                           = 0x00000057,
    GCVML2_PERF_SEL_EVENT_88                           = 0x00000058,
    GCVML2_PERF_SEL_EVENT_89                           = 0x00000059,
    GCVML2_PERF_SEL_EVENT_90                           = 0x0000005a,
} GCVML2_PERF_SEL;

constexpr unsigned int MaxGcvml2PerfSel                = GCVML2_PERF_SEL_EVENT_90;

typedef enum GC_EA_CPWD_PERFCOUNT_SEL {
    GC_EA_CPWD_PERF_SEL_NONE                           = 0x00000000,
    GC_EA_CPWD_PERF_SEL_SARB_IO_RD_SIZE_REQ            = 0x00000001,
    GC_EA_CPWD_PERF_SEL_SARB_IO_WR_SIZE_REQ            = 0x00000002,
    GC_EA_CPWD_PERF_SEL_SARB_DRAM_RD_SIZE_REQ          = 0x00000003,
    GC_EA_CPWD_PERF_SEL_SARB_DRAM_WR_SIZE_REQ          = 0x00000004,
    GC_EA_CPWD_PERF_SEL_SARB_MAM_WR_SIZE_REQ           = 0x00000005,
    GC_EA_CPWD_PERF_SEL_SARB_OUTSTANDING_IO_RD         = 0x00000006,
    GC_EA_CPWD_PERF_SEL_IO_RD_CMD_POP                  = 0x00000007,
    GC_EA_CPWD_PERF_SEL_SARB_OUTSTANDING_IO_WR         = 0x00000008,
    GC_EA_CPWD_PERF_SEL_IO_WR_CMD_POP                  = 0x00000009,
    GC_EA_CPWD_PERF_SEL_SARB_OUTSTANDING_DRAM_RD       = 0x0000000a,
    GC_EA_CPWD_PERF_SEL_DRAM_RD_CMD_POP                = 0x0000000b,
    GC_EA_CPWD_PERF_SEL_SARB_OUTSTANDING_DRAM_WR       = 0x0000000c,
    GC_EA_CPWD_PERF_SEL_DRAM_WR_CMD_POP                = 0x0000000d,
    GC_EA_CPWD_PERF_SEL_SARB_OUTSTANDING_ALL           = 0x0000000e,
    GC_EA_CPWD_PERF_SEL_CMD_POP                        = 0x0000000f,
    GC_EA_CPWD_PERF_SEL_SARB_STALLED                   = 0x00000010,
    GC_EA_CPWD_PERF_SEL_SARB_STARVING                  = 0x00000011,
    GC_EA_CPWD_PERF_SEL_SARB_IDLE                      = 0x00000012,
    GC_EA_CPWD_PERF_SEL_RRET_VLD                       = 0x00000013,
    GC_EA_CPWD_PERF_SEL_WRET_VLD                       = 0x00000014,
    GC_EA_CPWD_PERF_SEL_SARB_COHERENT_SIZE_REQ         = 0x00000015,
    GC_EA_CPWD_PERF_SEL_STALL_IO_RD                    = 0x00000016,
    GC_EA_CPWD_PERF_SEL_STALL_IO_WR                    = 0x00000017,
    GC_EA_CPWD_PERF_SEL_STALL_DRAM                     = 0x00000018,
    GC_EA_CPWD_PERF_SEL_STALL_MAM_WR                   = 0x00000019,
    GC_EA_CPWD_PERF_SEL_MAM_ARAM_REQ_VLD               = 0x0000001a,
    GC_EA_CPWD_PERF_SEL_MAM_FLUSH_REQ                  = 0x0000001b,
    GC_EA_CPWD_PERF_SEL_MAM_FLUSH_RESP                 = 0x0000001c,
    GC_EA_CPWD_PERF_SEL_MAM_AFLUSH_INTERRUPT           = 0x0000001d,
    GC_EA_CPWD_PERF_SEL_MAM_AFLUSH_INTERRUPT_STALLED   = 0x0000001e,
    GC_EA_CPWD_PERF_SEL_MAM_AFLUSH_COMPLETED           = 0x0000001f,
    GC_EA_CPWD_PERF_SEL_MAM_AFLUSH_ONGOING             = 0x00000020,
} GC_EA_CPWD_PERFCOUNT_SEL;

constexpr unsigned int MaxGcEaCpwdPerfcountSel         = GC_EA_CPWD_PERF_SEL_MAM_AFLUSH_ONGOING;

typedef enum GC_EA_SE_PERFCOUNT_SEL {
    GC_EA_SE_PERF_SEL_NONE                             = 0x00000000,
    GC_EA_SE_PERF_SEL_SARB_IO_RD_SIZE_REQ              = 0x00000001,
    GC_EA_SE_PERF_SEL_SARB_IO_WR_SIZE_REQ              = 0x00000002,
    GC_EA_SE_PERF_SEL_SARB_DRAM_RD_SIZE_REQ            = 0x00000003,
    GC_EA_SE_PERF_SEL_SARB_DRAM_WR_SIZE_REQ            = 0x00000004,
    GC_EA_SE_PERF_SEL_SARB_MAM_WR_SIZE_REQ             = 0x00000005,
    GC_EA_SE_PERF_SEL_SARB_OUTSTANDING_IO_RD           = 0x00000006,
    GC_EA_SE_PERF_SEL_IO_RD_CMD_POP                    = 0x00000007,
    GC_EA_SE_PERF_SEL_SARB_OUTSTANDING_IO_WR           = 0x00000008,
    GC_EA_SE_PERF_SEL_IO_WR_CMD_POP                    = 0x00000009,
    GC_EA_SE_PERF_SEL_SARB_OUTSTANDING_DRAM_RD         = 0x0000000a,
    GC_EA_SE_PERF_SEL_DRAM_RD_CMD_POP                  = 0x0000000b,
    GC_EA_SE_PERF_SEL_SARB_OUTSTANDING_DRAM_WR         = 0x0000000c,
    GC_EA_SE_PERF_SEL_DRAM_WR_CMD_POP                  = 0x0000000d,
    GC_EA_SE_PERF_SEL_SARB_OUTSTANDING_ALL             = 0x0000000e,
    GC_EA_SE_PERF_SEL_CMD_POP                          = 0x0000000f,
    GC_EA_SE_PERF_SEL_SARB_STALLED                     = 0x00000010,
    GC_EA_SE_PERF_SEL_SARB_STARVING                    = 0x00000011,
    GC_EA_SE_PERF_SEL_SARB_IDLE                        = 0x00000012,
    GC_EA_SE_PERF_SEL_RRET_VLD                         = 0x00000013,
    GC_EA_SE_PERF_SEL_WRET_VLD                         = 0x00000014,
    GC_EA_SE_PERF_SEL_SARB_COHERENT_SIZE_REQ           = 0x00000015,
    GC_EA_SE_PERF_SEL_STALL_IO_RD                      = 0x00000016,
    GC_EA_SE_PERF_SEL_STALL_IO_WR                      = 0x00000017,
    GC_EA_SE_PERF_SEL_STALL_DRAM                       = 0x00000018,
    GC_EA_SE_PERF_SEL_STALL_MAM_WR                     = 0x00000019,
    GC_EA_SE_PERF_SEL_MAM_ARAM_REQ_VLD                 = 0x0000001a,
    GC_EA_SE_PERF_SEL_MAM_FLUSH_REQ                    = 0x0000001b,
    GC_EA_SE_PERF_SEL_MAM_FLUSH_RESP                   = 0x0000001c,
    GC_EA_SE_PERF_SEL_MAM_AFLUSH_INTERRUPT             = 0x0000001d,
    GC_EA_SE_PERF_SEL_MAM_AFLUSH_INTERRUPT_STALLED     = 0x0000001e,
    GC_EA_SE_PERF_SEL_MAM_AFLUSH_COMPLETED             = 0x0000001f,
    GC_EA_SE_PERF_SEL_MAM_AFLUSH_ONGOING               = 0x00000020,
} GC_EA_SE_PERFCOUNT_SEL;

constexpr unsigned int MaxGcEaSePerfcountSel           = GC_EA_SE_PERF_SEL_MAM_AFLUSH_ONGOING;

typedef enum GE1_PERFCOUNT_SELECT {
    ge1_assembler_busy                                 = 0x00000000,
    ge1_assembler_stalled                              = 0x00000001,
    ge1_dma_busy                                       = 0x00000002,
    ge1_dma_lat_bin_0                                  = 0x00000003,
    ge1_dma_lat_bin_1                                  = 0x00000004,
    ge1_dma_lat_bin_2                                  = 0x00000005,
    ge1_dma_lat_bin_3                                  = 0x00000006,
    ge1_dma_lat_bin_4                                  = 0x00000007,
    ge1_dma_lat_bin_5                                  = 0x00000008,
    ge1_dma_lat_bin_6                                  = 0x00000009,
    ge1_dma_lat_bin_7                                  = 0x0000000a,
    ge1_dma_return_cl0                                 = 0x0000000b,
    ge1_dma_return_cl1                                 = 0x0000000c,
    ge1_dma_utcl1_consecutive_retry_event              = 0x0000000d,
    ge1_dma_utcl1_request_event                        = 0x0000000e,
    ge1_dma_utcl1_retry_event                          = 0x0000000f,
    ge1_dma_utcl1_stall_event                          = 0x00000010,
    ge1_dma_utcl1_stall_utcl2_event                    = 0x00000011,
    ge1_dma_utcl1_translation_hit_event                = 0x00000012,
    ge1_dma_utcl1_translation_miss_event               = 0x00000013,
    ge1_assembler_dma_starved                          = 0x00000014,
    ge1_rbiu_di_fifo_stalled_p0                        = 0x00000015,
    ge1_rbiu_di_fifo_starved_p0                        = 0x00000016,
    ge1_rbiu_dr_fifo_stalled_p0                        = 0x00000017,
    ge1_rbiu_dr_fifo_starved_p0                        = 0x00000018,
    ge1_sclk_reg_vld                                   = 0x00000019,
    ge1_stat_busy                                      = 0x0000001a,
    ge1_stat_no_dma_busy                               = 0x0000001b,
    ge1_pipe0_to_pipe1                                 = 0x0000001c,
    ge1_pipe1_to_pipe0                                 = 0x0000001d,
    ge1_dma_return_size_cl0                            = 0x0000001e,
    ge1_dma_return_size_cl1                            = 0x0000001f,
    ge1_small_draws_one_instance                       = 0x00000020,
    ge1_sclk_input_vld                                 = 0x00000021,
    ge1_prim_group_limit_hit                           = 0x00000022,
    ge1_rbiu_di_fifo_stalled_p1                        = 0x00000024,
    ge1_rbiu_di_fifo_starved_p1                        = 0x00000025,
    ge1_rbiu_dr_fifo_stalled_p1                        = 0x00000026,
    ge1_rbiu_dr_fifo_starved_p1                        = 0x00000027,
    ge_ia_ia_clocks                                    = 0x00000028,
    ge_ia_dma_active                                   = 0x00000029,
    ge_ia_dma_blocked                                  = 0x0000002a,
    ge_ia_dma_busy                                     = 0x0000002b,
    ge_ia_dma_num_indices_per_index_type_8             = 0x0000002c,
    ge_ia_dma_num_indices_per_index_type_16            = 0x0000002d,
    ge_ia_dma_num_indices_per_index_type_32            = 0x0000002e,
    ge_ia_dma_num_indices_client0                      = 0x0000002f,
    ge_ia_dma_num_indices_client1                      = 0x00000030,
    ge_ia_dma_indices_out_of_bound                     = 0x00000031,
    ge_ia_index_out_of_range                           = 0x00000032,
    ge_ia_indices_reset                                = 0x00000033,
    ge_ia_dma_ready_no_draw                            = 0x00000034,
    ge_ia_asm_out_null_prim_count                      = 0x00000035,
    ge_ia_asm_out_valid_prim_count                     = 0x00000036,
} GE1_PERFCOUNT_SELECT;

constexpr unsigned int MaxGe1PerfcountSelect           = ge_ia_asm_out_valid_prim_count;

typedef enum GE2_DIST_PERFCOUNT_SELECT {
    ge_dist_hs_done                                    = 0x00000000,
    ge_dist_hs_done_latency_se0                        = 0x00000001,
    ge_dist_hs_done_latency_se1                        = 0x00000002,
    ge_dist_hs_done_latency_se2                        = 0x00000003,
    ge_dist_hs_done_latency_se3                        = 0x00000004,
    ge_dist_hs_done_latency_se4                        = 0x00000005,
    ge_dist_hs_done_latency_se5                        = 0x00000006,
    ge_dist_hs_done_latency_se6                        = 0x00000007,
    ge_dist_hs_done_latency_se7                        = 0x00000008,
    ge_dist_inside_tf_bin_0                            = 0x00000009,
    ge_dist_inside_tf_bin_1                            = 0x0000000a,
    ge_dist_inside_tf_bin_2                            = 0x0000000b,
    ge_dist_inside_tf_bin_3                            = 0x0000000c,
    ge_dist_inside_tf_bin_4                            = 0x0000000d,
    ge_dist_inside_tf_bin_5                            = 0x0000000e,
    ge_dist_inside_tf_bin_6                            = 0x0000000f,
    ge_dist_inside_tf_bin_7                            = 0x00000010,
    ge_dist_inside_tf_bin_8                            = 0x00000011,
    ge_dist_null_patch                                 = 0x00000012,
    ge_dist_sclk_core_vld                              = 0x00000013,
    ge_dist_sclk_wd_te11_vld                           = 0x00000014,
    ge_dist_tfreq_lat_bin_0                            = 0x00000015,
    ge_dist_tfreq_lat_bin_1                            = 0x00000016,
    ge_dist_tfreq_lat_bin_2                            = 0x00000017,
    ge_dist_tfreq_lat_bin_3                            = 0x00000018,
    ge_dist_tfreq_lat_bin_4                            = 0x00000019,
    ge_dist_tfreq_lat_bin_5                            = 0x0000001a,
    ge_dist_tfreq_lat_bin_6                            = 0x0000001b,
    ge_dist_tfreq_lat_bin_7                            = 0x0000001c,
    ge_dist_tfreq_utcl1_consecutive_retry_event        = 0x0000001d,
    ge_dist_tfreq_utcl1_request_event                  = 0x0000001e,
    ge_dist_tfreq_utcl1_retry_event                    = 0x0000001f,
    ge_dist_tfreq_utcl1_stall_event                    = 0x00000020,
    ge_dist_tfreq_utcl1_stall_utcl2_event              = 0x00000021,
    ge_dist_tfreq_utcl1_translation_hit_event          = 0x00000022,
    ge_dist_tfreq_utcl1_translation_miss_event         = 0x00000023,
    ge_dist_pc_feorder_fifo_full                       = 0x00000024,
    ge_dist_pc_ge_manager_busy                         = 0x00000025,
    ge_dist_sclk_input_vld                             = 0x00000026,
    ge_dist_wd_te11_busy                               = 0x00000027,
    ge_dist_te11_starved                               = 0x00000028,
    ge_dist_switch_mode_stall                          = 0x00000029,
    ge_all_tf_eq                                       = 0x0000002a,
    ge_all_tf2                                         = 0x0000002b,
    ge_all_tf3                                         = 0x0000002c,
    ge_all_tf4                                         = 0x0000002d,
    ge_all_tf5                                         = 0x0000002e,
    ge_all_tf6                                         = 0x0000002f,
    ge_se0_te11_starved_on_hs_done                     = 0x00000030,
    ge_se1_te11_starved_on_hs_done                     = 0x00000031,
    ge_se2_te11_starved_on_hs_done                     = 0x00000032,
    ge_se3_te11_starved_on_hs_done                     = 0x00000033,
    ge_se4_te11_starved_on_hs_done                     = 0x00000034,
    ge_se5_te11_starved_on_hs_done                     = 0x00000035,
    ge_se6_te11_starved_on_hs_done                     = 0x00000036,
    ge_se7_te11_starved_on_hs_done                     = 0x00000037,
    ge_dist_op_fifo_full_starve                        = 0x00000038,
    ge_dist_hs_done_se0                                = 0x00000039,
    ge_dist_hs_done_se1                                = 0x0000003a,
    ge_dist_hs_done_se2                                = 0x0000003b,
    ge_dist_hs_done_se3                                = 0x0000003c,
    ge_dist_hs_done_se4                                = 0x0000003d,
    ge_dist_hs_done_se5                                = 0x0000003e,
    ge_dist_hs_done_se6                                = 0x0000003f,
    ge_dist_hs_done_se7                                = 0x00000040,
    ge_dist_hs_done_latency                            = 0x00000041,
    ge_dist_distributer_busy                           = 0x00000042,
    ge_tf_ret_data_stalling_hs_done                    = 0x00000043,
    ge_num_of_no_dist_patches                          = 0x00000044,
    ge_num_of_donut_dist_patches                       = 0x00000045,
    ge_num_of_patch_dist_patches                       = 0x00000046,
    ge_num_of_se_switches_due_to_patch_accum           = 0x00000047,
    ge_num_of_se_switches_due_to_donut                 = 0x00000048,
    ge_num_of_se_switches_due_to_trap                  = 0x00000049,
    ge_num_of_hs_dealloc_events                        = 0x0000004a,
    ge_agm_gcr_req                                     = 0x0000004b,
    ge_agm_gcr_tag_stall                               = 0x0000004c,
    ge_agm_gcr_crd_stall                               = 0x0000004d,
    ge_agm_gcr_stall                                   = 0x0000004e,
    ge_agm_gcr_latency                                 = 0x0000004f,
    ge_distclk_vld                                     = 0x00000050,
    ge_dist_indx_fifos_full_and_empty                  = 0x00000051,
    ge_hs_done_all_tf0_se0                             = 0x00000052,
    ge_hs_done_all_tf0_se1                             = 0x00000053,
    ge_hs_done_all_tf0_se2                             = 0x00000054,
    ge_hs_done_all_tf0_se3                             = 0x00000055,
    ge_hs_done_all_tf0_se4                             = 0x00000056,
    ge_hs_done_all_tf0_se5                             = 0x00000057,
    ge_hs_done_all_tf0_se6                             = 0x00000058,
    ge_hs_done_all_tf0_se7                             = 0x00000059,
    ge_hs_done_all_tf1_se0                             = 0x0000005a,
    ge_hs_done_all_tf1_se1                             = 0x0000005b,
    ge_hs_done_all_tf1_se2                             = 0x0000005c,
    ge_hs_done_all_tf1_se3                             = 0x0000005d,
    ge_hs_done_all_tf1_se4                             = 0x0000005e,
    ge_hs_done_all_tf1_se5                             = 0x0000005f,
    ge_hs_done_all_tf1_se6                             = 0x00000060,
    ge_hs_done_all_tf1_se7                             = 0x00000061,
    ge_agm_gcr_req_outstanding                         = 0x00000062,
    ge_agm_gcr_req_amount                              = 0x00000063,
    ge_agm_gcr_combine                                 = 0x00000064,
    ge_wd_clocks                                       = 0x00000065,
    ge_wd_gsgrp_done_latency_se0                       = 0x00000066,
    ge_wd_gsgrp_done_latency_se1                       = 0x00000067,
    ge_wd_gsgrp_done_latency_se2                       = 0x00000068,
    ge_wd_gsgrp_done_latency_se3                       = 0x00000069,
    ge_wd_gsgrp_done_latency_se4                       = 0x0000006a,
    ge_wd_gsgrp_done_latency_se5                       = 0x0000006b,
    ge_wd_gsgrp_done_latency_se6                       = 0x0000006c,
    ge_wd_gsgrp_done_latency_se7                       = 0x0000006d,
    ge_wd_gsgrp_done_latency_se8                       = 0x0000006e,
    ge_wd_gsgrp_done_se0                               = 0x0000006f,
    ge_wd_gsgrp_done_se1                               = 0x00000070,
    ge_wd_gsgrp_done_se2                               = 0x00000071,
    ge_wd_gsgrp_done_se3                               = 0x00000072,
    ge_wd_gsgrp_done_se4                               = 0x00000073,
    ge_wd_gsgrp_done_se5                               = 0x00000074,
    ge_wd_gsgrp_done_se6                               = 0x00000075,
    ge_wd_gsgrp_done_se7                               = 0x00000076,
    ge_wd_gsgrp_done_se8                               = 0x00000077,
    ge_wd_gsgrp_done_starved_se0                       = 0x00000078,
    ge_wd_gsgrp_done_starved_se1                       = 0x00000079,
    ge_wd_gsgrp_done_starved_se2                       = 0x0000007a,
    ge_wd_gsgrp_done_starved_se3                       = 0x0000007b,
    ge_wd_gsgrp_done_starved_se4                       = 0x0000007c,
    ge_wd_gsgrp_done_starved_se5                       = 0x0000007d,
    ge_wd_gsgrp_done_starved_se6                       = 0x0000007e,
    ge_wd_gsgrp_done_starved_se7                       = 0x0000007f,
    ge_wd_gsgrp_done_starved_se8                       = 0x00000080,
    ge_wd_pc_dealloc_starved_se0                       = 0x00000081,
    ge_wd_pc_dealloc_starved_se1                       = 0x00000082,
    ge_wd_pc_dealloc_starved_se2                       = 0x00000083,
    ge_wd_pc_dealloc_starved_se3                       = 0x00000084,
    ge_wd_pc_dealloc_starved_se4                       = 0x00000085,
    ge_wd_pc_dealloc_starved_se5                       = 0x00000086,
    ge_wd_pc_dealloc_starved_se6                       = 0x00000087,
    ge_wd_pc_dealloc_starved_se7                       = 0x00000088,
    ge_wd_pc_dealloc_starved_se8                       = 0x00000089,
    ge_wd_ph_dealloc_starved_se0                       = 0x0000008a,
    ge_wd_ph_dealloc_starved_se1                       = 0x0000008b,
    ge_wd_ph_dealloc_starved_se2                       = 0x0000008c,
    ge_wd_ph_dealloc_starved_se3                       = 0x0000008d,
    ge_wd_ph_dealloc_starved_se4                       = 0x0000008e,
    ge_wd_ph_dealloc_starved_se5                       = 0x0000008f,
    ge_wd_ph_dealloc_starved_se6                       = 0x00000090,
    ge_wd_ph_dealloc_starved_se7                       = 0x00000091,
    ge_wd_ph_dealloc_starved_se8                       = 0x00000092,
    ge_wd_hs_dealloc_starved_se0                       = 0x00000093,
    ge_wd_hs_dealloc_starved_se1                       = 0x00000094,
    ge_wd_hs_dealloc_starved_se2                       = 0x00000095,
    ge_wd_hs_dealloc_starved_se3                       = 0x00000096,
    ge_wd_hs_dealloc_starved_se4                       = 0x00000097,
    ge_wd_hs_dealloc_starved_se5                       = 0x00000098,
    ge_wd_hs_dealloc_starved_se6                       = 0x00000099,
    ge_wd_hs_dealloc_starved_se7                       = 0x0000009a,
    ge_wd_hs_dealloc_starved_se8                       = 0x0000009b,
    ge_wd_hs_dealloc_latency_se0                       = 0x0000009c,
    ge_wd_hs_dealloc_latency_se1                       = 0x0000009d,
    ge_wd_hs_dealloc_latency_se2                       = 0x0000009e,
    ge_wd_hs_dealloc_latency_se3                       = 0x0000009f,
    ge_wd_hs_dealloc_latency_se4                       = 0x000000a0,
    ge_wd_hs_dealloc_latency_se5                       = 0x000000a1,
    ge_wd_hs_dealloc_latency_se6                       = 0x000000a2,
    ge_wd_hs_dealloc_latency_se7                       = 0x000000a3,
    ge_wd_hs_dealloc_latency_se8                       = 0x000000a4,
    ge_wd_hs_dealloc_se0                               = 0x000000a5,
    ge_wd_hs_dealloc_se1                               = 0x000000a6,
    ge_wd_hs_dealloc_se2                               = 0x000000a7,
    ge_wd_hs_dealloc_se3                               = 0x000000a8,
    ge_wd_hs_dealloc_se4                               = 0x000000a9,
    ge_wd_hs_dealloc_se5                               = 0x000000aa,
    ge_wd_hs_dealloc_se6                               = 0x000000ab,
    ge_wd_hs_dealloc_se7                               = 0x000000ac,
    ge_wd_hs_dealloc_se8                               = 0x000000ad,
    ge_wd_utcl_requests                                = 0x000000ae,
    ge_wd_utcl_ret_latency                             = 0x000000af,
    ge_wd_gs_thrt_stall_bin0                           = 0x000000b0,
    ge_wd_gs_thrt_stall_bin1                           = 0x000000b1,
    ge_wd_gs_thrt_stall_bin2                           = 0x000000b2,
    ge_wd_gs_thrt_stall_bin3                           = 0x000000b3,
    ge_wd_phmq_fsm_starved_se0                         = 0x000000b4,
    ge_wd_phmq_fsm_starved_se1                         = 0x000000b5,
    ge_wd_phmq_fsm_starved_se2                         = 0x000000b6,
    ge_wd_phmq_fsm_starved_se3                         = 0x000000b7,
    ge_wd_phmq_fsm_starved_se4                         = 0x000000b8,
    ge_wd_phmq_fsm_starved_se5                         = 0x000000b9,
    ge_wd_phmq_fsm_starved_se6                         = 0x000000ba,
    ge_wd_phmq_fsm_starved_se7                         = 0x000000bb,
    ge_wd_phmq_fsm_starved_se8                         = 0x000000bc,
} GE2_DIST_PERFCOUNT_SELECT;

constexpr unsigned int MaxGe2DistPerfcountSelect       = ge_wd_phmq_fsm_starved_se8;

typedef enum GE2_SE_PERFCOUNT_SELECT {
    ge_se_ds_prims                                     = 0x00000000,
    ge_se_esvert_stalled_gsprim                        = 0x00000001,
    ge_se_hs_tfm_stall                                 = 0x00000002,
    ge_se_hs_tgs_active_high_water_mark                = 0x00000003,
    ge_se_hs_thread_groups                             = 0x00000004,
    ge_se_reused_es_indices                            = 0x00000005,
    ge_se_sclk_ngg_vld                                 = 0x00000006,
    ge_se_sclk_te11_vld                                = 0x00000007,
    ge_se_spi_esvert_eov                               = 0x00000008,
    ge_se_spi_esvert_stalled                           = 0x00000009,
    ge_se_spi_esvert_starved_busy                      = 0x0000000a,
    ge_se_spi_esvert_valid                             = 0x0000000b,
    ge_se_spi_gsprim_cont                              = 0x0000000c,
    ge_se_spi_gsprim_eov                               = 0x0000000d,
    ge_se_spi_gsprim_stalled                           = 0x0000000e,
    ge_se_spi_gsprim_starved_busy                      = 0x0000000f,
    ge_se_spi_gsprim_valid                             = 0x00000010,
    ge_se_spi_gssubgrp_is_event                        = 0x00000011,
    ge_se_spi_gssubgrp_send                            = 0x00000012,
    ge_se_spi_hsvert_eov                               = 0x00000013,
    ge_se_spi_hsvert_stalled                           = 0x00000014,
    ge_se_spi_hsvert_starved_busy                      = 0x00000015,
    ge_se_spi_hsvert_valid                             = 0x00000016,
    ge_se_spi_hsgrp_is_event                           = 0x00000017,
    ge_se_spi_hsgrp_send                               = 0x00000018,
    ge_se_spi_lsvert_eov                               = 0x00000019,
    ge_se_spi_lsvert_stalled                           = 0x0000001a,
    ge_se_spi_lsvert_starved_busy                      = 0x0000001b,
    ge_se_spi_lsvert_valid                             = 0x0000001c,
    ge_se_spi_hsvert_fifo_full_stall                   = 0x0000001d,
    ge_se_spi_tgrp_fifo_stall                          = 0x0000001e,
    ge_spi_hsgrp_spi_stall                             = 0x0000001f,
    ge_se_spi_gssubgrp_event_window_active             = 0x00000020,
    ge_se_hs_input_stall                               = 0x00000021,
    ge_se_sending_vert_or_prim                         = 0x00000022,
    ge_se_sclk_input_vld                               = 0x00000023,
    ge_spi_lswave_fifo_full_stall                      = 0x00000024,
    ge_spi_hswave_fifo_full_stall                      = 0x00000025,
    ge_hs_tif_stall                                    = 0x00000026,
    ge_csb_spi_bp                                      = 0x00000027,
    ge_ngg_starving_for_wave_id                        = 0x00000028,
    ge_pa0_csb_eop                                     = 0x00000029,
    ge_ngg_starved_idle                                = 0x0000002a,
    ge_gsprim_send                                     = 0x0000002b,
    ge_esvert_send                                     = 0x0000002c,
    ge_ngg_starved_after_work                          = 0x0000002d,
    ge_ngg_subgrp_fifo_stall                           = 0x0000002e,
    ge_ngg_ord_id_req_stall                            = 0x0000002f,
    ge_ngg_indx_bus_stall                              = 0x00000030,
    ge_hs_stall_tfmm_fifo_full                         = 0x00000031,
    ge_gs_issue_rtr_stalled                            = 0x00000032,
    ge_gsprim_stalled_esvert                           = 0x00000033,
    ge_gsthread_stalled                                = 0x00000034,
    ge_ngg_attr_grp_alloc                              = 0x00000035,
    ge_ngg_attr_discard_alloc                          = 0x00000036,
    ge_ngg_pc_space_not_avail                          = 0x00000037,
    ge_ngg_agm_req_stall                               = 0x00000038,
    ge_ngg_spi_esvert_partial_eov                      = 0x00000039,
    ge_ngg_spi_gsprim_partial_eov                      = 0x0000003a,
    ge_spi_gsgrp_valid                                 = 0x0000003b,
    ge_ngg_attr_grp_latency                            = 0x0000003c,
    ge_ngg_reuse_prim_limit_hit                        = 0x0000003d,
    ge_ngg_reuse_vert_limit_hit                        = 0x0000003e,
    ge_te11_con_stall                                  = 0x0000003f,
    ge_te11_compactor_starved                          = 0x00000040,
    ge_ngg_stall_tess_off_tess_on                      = 0x00000041,
    ge_ngg_stall_tess_on_tess_off                      = 0x00000042,
    ge_merged_lses_vert_stalled                        = 0x00000043,
    ge_merged_hsgs_vert_stalled                        = 0x00000044,
    ge_merged_hsgs_grp_stalled                         = 0x00000045,
    ge_merge_lses_fifo_blocked                         = 0x00000046,
    ge_merge_hsgs_fifo_blocked                         = 0x00000047,
    ge_merge_lses_vert_switch                          = 0x00000048,
    ge_merge_hsgs_vert_switch                          = 0x00000049,
    ge_merge_hsgs_grp_switch                           = 0x0000004a,
    ge_merge_gsgrp_rdy_pending_verts                   = 0x0000004b,
    ge_merge_hsgrp_rdy_pending_verts                   = 0x0000004c,
    ge_se_ds_cache_hits                                = 0x0000004d,
    ge_se_api_vs_verts                                 = 0x0000004e,
    ge_se_api_ds_verts                                 = 0x0000004f,
    ge_se_combined_busy                                = 0x00000050,
    ge_spi_lsvert_send                                 = 0x00000051,
    ge_spi_hsvert_send                                 = 0x00000052,
    ge_ngg_attr_grp_wasted                             = 0x00000053,
    ge_spi_gssubgrp_stalled                            = 0x00000054,
    ge_ngg_attr_null_dealloc                           = 0x00000055,
    ge_ngg_busy_base                                   = 0x00000056,
    ge_spi_hsthdgrp_stalled_offchip_unavail            = 0x00000057,
    ge_se_clocks                                       = 0x00000058,
    ge_se_tess_patches                                 = 0x00000059,
    ge_se_verts                                        = 0x0000005a,
    ge_se_prims                                        = 0x0000005b,
    ge_se_2cy_indx_data_xfer                           = 0x0000005c,
    ge_se_draw_sync_stall                              = 0x0000005d,
    ge_se_event_sync_stall                             = 0x0000005e,
    ge_se_ph_alloc_pos_buf_full_stall                  = 0x0000005f,
    ge_se_ph_alloc_prim_buf_full_stall                 = 0x00000060,
    ge_se_ph_dealloc                                   = 0x00000061,
    ge_se_ph_alloc_subgrp_fifo_stall                   = 0x00000062,
    ge_se_ph_dealloc_latency                           = 0x00000063,
    ge_se_no_pc_export                                 = 0x00000064,
    ge_se_ngg_subgrps_throttled                        = 0x00000065,
    ge_se_ngg_subgrp_throttle_latency                  = 0x00000066,
    ge_se_hs_stall_offchip_unavail                     = 0x00000067,
} GE2_SE_PERFCOUNT_SELECT;

constexpr unsigned int MaxGe2SePerfcountSelect         = ge_se_hs_stall_offchip_unavail;

typedef enum GE_DIST_EOP {
    GE_DIST_EOP_ZERO                                   = 0x00000000,
    GE_DIST_DRAW_EOP                                   = 0x00000001,
    GE_DIST_SUPPRESSED_EOP                             = 0x00000002,
    GE_DIST_NULL_EOP                                   = 0x00000003,
} GE_DIST_EOP;

typedef enum GL0V_CACHE_POLICIES {
    GL0V_CACHE_POLICY_MISS_LRU                         = 0x00000000,
    GL0V_CACHE_POLICY_MISS_EVICT                       = 0x00000001,
    GL0V_CACHE_POLICY_HIT_LRU                          = 0x00000002,
    GL0V_CACHE_POLICY_HIT_INVAL                        = 0x00000003,
    GL0V_CACHE_POLICY_MISS_INVAL                       = 0x00000004,
} GL0V_CACHE_POLICIES;

typedef enum GL1A_PERF_SEL {
    GL1A_PERF_SEL_BUSY                                 = 0x00000000,
    GL1A_PERF_SEL_STALL_GL1C0                          = 0x00000001,
    GL1A_PERF_SEL_STALL_GL1C1                          = 0x00000002,
    GL1A_PERF_SEL_STALL_GL1C2                          = 0x00000003,
    GL1A_PERF_SEL_STALL_GL1C3                          = 0x00000004,
    GL1A_PERF_SEL_REQUEST_GL1C0                        = 0x00000005,
    GL1A_PERF_SEL_REQUEST_GL1C1                        = 0x00000006,
    GL1A_PERF_SEL_REQUEST_GL1C2                        = 0x00000007,
    GL1A_PERF_SEL_REQUEST_GL1C3                        = 0x00000008,
    GL1A_PERF_SEL_BURST_COUNT_GL1C0                    = 0x00000009,
    GL1A_PERF_SEL_BURST_COUNT_GL1C1                    = 0x0000000a,
    GL1A_PERF_SEL_BURST_COUNT_GL1C2                    = 0x0000000b,
    GL1A_PERF_SEL_BURST_COUNT_GL1C3                    = 0x0000000c,
    GL1A_PERF_SEL_ARB_REQUESTS                         = 0x0000000d,
    GL1A_PERF_SEL_REQ_INFLIGHT_LEVEL                   = 0x0000000e,
    GL1A_PERF_SEL_STALL_RET_CONFLICT_GL1C0             = 0x0000000f,
    GL1A_PERF_SEL_STALL_RET_CONFLICT_GL1C1             = 0x00000010,
    GL1A_PERF_SEL_STALL_RET_CONFLICT_GL1C2             = 0x00000011,
    GL1A_PERF_SEL_STALL_RET_CONFLICT_GL1C3             = 0x00000012,
    GL1A_PERF_SEL_REQ_PATH_BUSY                        = 0x00000013,
    GL1A_PERF_SEL_RET_PATH_BUSY                        = 0x00000014,
    GL1A_PERF_SEL_CYCLE                                = 0x00000015,
} GL1A_PERF_SEL;

constexpr unsigned int MaxGl1aPerfSel                  = GL1A_PERF_SEL_CYCLE;

typedef enum GL1C_PERF_SEL {
    GL1C_PERF_SEL_CYCLE                                = 0x00000000,
    GL1C_PERF_SEL_BUSY                                 = 0x00000001,
    GL1C_PERF_SEL_STARVE                               = 0x00000002,
    GL1C_PERF_SEL_ARB_RET_LEVEL                        = 0x00000003,
    GL1C_PERF_SEL_GL2_REQ_READ                         = 0x00000004,
    GL1C_PERF_SEL_GL2_REQ_READ_TCP                     = 0x00000005,
    GL1C_PERF_SEL_GL2_REQ_READ_CB                      = 0x00000006,
    GL1C_PERF_SEL_GL2_REQ_READ_DB                      = 0x00000007,
    GL1C_PERF_SEL_GL2_REQ_READ_32B                     = 0x00000008,
    GL1C_PERF_SEL_GL2_REQ_READ_64B                     = 0x00000009,
    GL1C_PERF_SEL_GL2_REQ_READ_128B                    = 0x0000000a,
    GL1C_PERF_SEL_GL2_REQ_READ_256B                    = 0x0000000b,
    GL1C_PERF_SEL_GL2_REQ_READ_LEVEL                   = 0x0000000c,
    GL1C_PERF_SEL_GL2_REQ_READ_LEVEL_TCP               = 0x0000000d,
    GL1C_PERF_SEL_GL2_REQ_READ_LEVEL_CB                = 0x0000000e,
    GL1C_PERF_SEL_GL2_REQ_READ_LEVEL_DB                = 0x0000000f,
    GL1C_PERF_SEL_GL2_REQ_WRITE                        = 0x00000010,
    GL1C_PERF_SEL_GL2_REQ_WRITE_TCP                    = 0x00000011,
    GL1C_PERF_SEL_GL2_REQ_WRITE_CB                     = 0x00000012,
    GL1C_PERF_SEL_GL2_REQ_WRITE_DB                     = 0x00000013,
    GL1C_PERF_SEL_GL2_REQ_WRITE_0B                     = 0x00000014,
    GL1C_PERF_SEL_GL2_REQ_WRITE_32B                    = 0x00000015,
    GL1C_PERF_SEL_GL2_REQ_WRITE_64B                    = 0x00000016,
    GL1C_PERF_SEL_GL2_REQ_WRITE_128B                   = 0x00000017,
    GL1C_PERF_SEL_GL2_REQ_WRITE_256B                   = 0x00000018,
    GL1C_PERF_SEL_GL2_REQ_WRITE_LEVEL                  = 0x00000019,
    GL1C_PERF_SEL_GL2_REQ_WRITE_LEVEL_TCP              = 0x0000001a,
    GL1C_PERF_SEL_GL2_REQ_WRITE_LEVEL_CB               = 0x0000001b,
    GL1C_PERF_SEL_GL2_REQ_WRITE_LEVEL_DB               = 0x0000001c,
    GL1C_PERF_SEL_GL2_REQ_WRITE_32B_DATA_BEATS         = 0x0000001d,
    GL1C_PERF_SEL_REQ                                  = 0x0000001e,
    GL1C_PERF_SEL_REQ_ATOMIC_WITH_RET                  = 0x0000001f,
    GL1C_PERF_SEL_REQ_ATOMIC_WITHOUT_RET               = 0x00000020,
    GL1C_PERF_SEL_REQ_NOP_ACK                          = 0x00000021,
    GL1C_PERF_SEL_REQ_NOP_RTN0                         = 0x00000022,
    GL1C_PERF_SEL_REQ_READ                             = 0x00000023,
    GL1C_PERF_SEL_REQ_READ_32B                         = 0x00000024,
    GL1C_PERF_SEL_REQ_READ_64B                         = 0x00000025,
    GL1C_PERF_SEL_REQ_READ_128B                        = 0x00000026,
    GL1C_PERF_SEL_REQ_READ_256B                        = 0x00000027,
    GL1C_PERF_SEL_REQ_WRITE                            = 0x00000028,
    GL1C_PERF_SEL_REQ_WRITE_0B                         = 0x00000029,
    GL1C_PERF_SEL_REQ_WRITE_32B                        = 0x0000002a,
    GL1C_PERF_SEL_REQ_WRITE_64B                        = 0x0000002b,
    GL1C_PERF_SEL_REQ_WRITE_128B                       = 0x0000002c,
    GL1C_PERF_SEL_REQ_WRITE_256B                       = 0x0000002d,
    GL1C_PERF_SEL_REQ_WRITE_32B_DATA_BEATS             = 0x0000002e,
    GL1C_PERF_SEL_REQ_LOAD_RESERVE_32B                 = 0x0000002f,
    GL1C_PERF_SEL_REQ_LOAD_RESERVE_64B                 = 0x00000030,
    GL1C_PERF_SEL_REQ_READ_COMP_KEY                    = 0x00000031,
    GL1C_PERF_SEL_REQ_FORCE_EXISTING_DATA_DECOMPRESS   = 0x00000032,
    GL1C_PERF_SEL_READ_COMP_MODE_0                     = 0x00000033,
    GL1C_PERF_SEL_READ_COMP_MODE_1                     = 0x00000034,
    GL1C_PERF_SEL_READ_COMP_MODE_2                     = 0x00000035,
    GL1C_PERF_SEL_READ_32B_COMP_MODE_1                 = 0x00000036,
    GL1C_PERF_SEL_READ_64B_COMP_MODE_1                 = 0x00000037,
    GL1C_PERF_SEL_READ_128B_COMP_MODE_1                = 0x00000038,
    GL1C_PERF_SEL_READ_256B_COMP_MODE_1                = 0x00000039,
    GL1C_PERF_SEL_WRITE_COMP_MODE_0                    = 0x0000003a,
    GL1C_PERF_SEL_WRITE_COMP_MODE_1                    = 0x0000003b,
    GL1C_PERF_SEL_WRITE_COMP_MODE_2                    = 0x0000003c,
    GL1C_PERF_SEL_WRITE_0B_COMP_MODE_1                 = 0x0000003d,
    GL1C_PERF_SEL_WRITE_32B_COMP_MODE_1                = 0x0000003e,
    GL1C_PERF_SEL_WRITE_64B_COMP_MODE_1                = 0x0000003f,
    GL1C_PERF_SEL_WRITE_128B_COMP_MODE_1               = 0x00000040,
    GL1C_PERF_SEL_WRITE_256B_COMP_MODE_1               = 0x00000041,
    GL1C_PERF_SEL_GL2_RET_ACK                          = 0x00000042,
    GL1C_PERF_SEL_GL2_RET_32B                          = 0x00000043,
    GL1C_PERF_SEL_GL1_RET_ACK                          = 0x00000044,
    GL1C_PERF_SEL_GL1_RET_32B                          = 0x00000045,
    GL1C_PERF_SEL_STALL_GL1_GL2_REQ_CREDITS            = 0x00000046,
    GL1C_PERF_SEL_STALL_GL1_GL2_DATA_CREDITS           = 0x00000047,
    GL1C_PERF_SEL_STALL_BUFFER_FULL                    = 0x00000048,
    GL1C_PERF_SEL_STALL_GCR_SHOOTDOWN_PENDING          = 0x00000049,
    GL1C_PERF_SEL_STALL_VM                             = 0x0000004a,
    GL1C_PERF_SEL_REQ_CLIENT0                          = 0x0000004b,
    GL1C_PERF_SEL_REQ_CLIENT1                          = 0x0000004c,
    GL1C_PERF_SEL_REQ_CLIENT2                          = 0x0000004d,
    GL1C_PERF_SEL_REQ_CLIENT3                          = 0x0000004e,
    GL1C_PERF_SEL_REQ_CLIENT4                          = 0x0000004f,
    GL1C_PERF_SEL_REQ_CLIENT5                          = 0x00000050,
    GL1C_PERF_SEL_REQ_CLIENT6                          = 0x00000051,
    GL1C_PERF_SEL_REQ_CLIENT7                          = 0x00000052,
    GL1C_PERF_SEL_REQ_CLIENT8                          = 0x00000053,
    GL1C_PERF_SEL_REQ_CLIENT9                          = 0x00000054,
    GL1C_PERF_SEL_REQ_CLIENT10                         = 0x00000055,
    GL1C_PERF_SEL_REQ_CLIENT11                         = 0x00000056,
    GL1C_PERF_SEL_REQ_CLIENT12                         = 0x00000057,
    GL1C_PERF_SEL_REQ_CLIENT13                         = 0x00000058,
    GL1C_PERF_SEL_REQ_CLIENT14                         = 0x00000059,
    GL1C_PERF_SEL_REQ_CLIENT15                         = 0x0000005a,
    GL1C_PERF_SEL_REQ_CLIENT16                         = 0x0000005b,
    GL1C_PERF_SEL_REQ_CLIENT17                         = 0x0000005c,
    GL1C_PERF_SEL_REQ_CLIENT18                         = 0x0000005d,
    GL1C_PERF_SEL_REQ_CLIENT19                         = 0x0000005e,
    GL1C_PERF_SEL_REQ_CLIENT20                         = 0x0000005f,
    GL1C_PERF_SEL_REQ_CLIENT21                         = 0x00000060,
    GL1C_PERF_SEL_REQ_CLIENT22                         = 0x00000061,
    GL1C_PERF_SEL_REQ_CLIENT23                         = 0x00000062,
    GL1C_PERF_SEL_REQ_CLIENT24                         = 0x00000063,
    GL1C_PERF_SEL_REQ_CLIENT25                         = 0x00000064,
    GL1C_PERF_SEL_REQ_CLIENT26                         = 0x00000065,
    GL1C_PERF_SEL_REQ_CLIENT27                         = 0x00000066,
    GL1C_PERF_SEL_UTCL0_REQUEST                        = 0x00000067,
    GL1C_PERF_SEL_UTCL0_TRANSLATION_HIT                = 0x00000068,
    GL1C_PERF_SEL_UTCL0_TRANSLATION_MISS               = 0x00000069,
    GL1C_PERF_SEL_UTCL0_PERMISSION_MISS                = 0x0000006a,
    GL1C_PERF_SEL_UTCL0_MISS_UNDER_MISS                = 0x0000006b,
    GL1C_PERF_SEL_UTCL0_LFIFO_FULL                     = 0x0000006c,
    GL1C_PERF_SEL_UTCL0_STALL_INFLIGHT_MAX             = 0x0000006d,
    GL1C_PERF_SEL_UTCL0_STALL_LFIFO_NOT_RES            = 0x0000006e,
    GL1C_PERF_SEL_UTCL0_STALL_LRU_INFLIGHT             = 0x0000006f,
    GL1C_PERF_SEL_UTCL0_STALL_MISSFIFO_FULL            = 0x00000070,
    GL1C_PERF_SEL_UTCL0_STALL_UTCL1_REQ_OUT_OF_CREDITS = 0x00000071,
    GL1C_PERF_SEL_UTCL0_UTCL1_PERM_FAULT               = 0x00000072,
    GL1C_PERF_SEL_CLIENT_UTCL0_INFLIGHT                = 0x00000073,
    GL1C_PERF_SEL_UTCL0_UTCL1_INFLIGHT                 = 0x00000074,
    GL1C_PERF_SEL_UTCL0_INTERNAL_RETRY_REQ             = 0x00000075,
    GL1C_PERF_SEL_UTCL0_UTCL1_XNACK_RETRY_FAULT        = 0x00000076,
    GL1C_PERF_SEL_UTCL0_UTCL1_XNACK_PRT_FAULT          = 0x00000077,
    GL1C_PERF_SEL_UTCL0_UTCL1_XNACK_NO_RETRY_FAULT     = 0x00000078,
    GL1C_PERF_SEL_UTCL0_GPA3_REQUEST                   = 0x00000079,
} GL1C_PERF_SEL;

constexpr unsigned int MaxGl1cPerfSel                  = GL1C_PERF_SEL_UTCL0_GPA3_REQUEST;

typedef enum GL1XA_PERF_SEL {
    GL1XA_PERF_SEL_BUSY                                = 0x00000000,
    GL1XA_PERF_SEL_STALL_GL1XC0                        = 0x00000001,
    GL1XA_PERF_SEL_STALL_GL1XC1                        = 0x00000002,
    GL1XA_PERF_SEL_STALL_GL1XC2                        = 0x00000003,
    GL1XA_PERF_SEL_STALL_GL1XC3                        = 0x00000004,
    GL1XA_PERF_SEL_REQUEST_GL1XC0                      = 0x00000005,
    GL1XA_PERF_SEL_REQUEST_GL1XC1                      = 0x00000006,
    GL1XA_PERF_SEL_REQUEST_GL1XC2                      = 0x00000007,
    GL1XA_PERF_SEL_REQUEST_GL1XC3                      = 0x00000008,
    GL1XA_PERF_SEL_BURST_COUNT_GL1XC0                  = 0x00000009,
    GL1XA_PERF_SEL_BURST_COUNT_GL1XC1                  = 0x0000000a,
    GL1XA_PERF_SEL_BURST_COUNT_GL1XC2                  = 0x0000000b,
    GL1XA_PERF_SEL_BURST_COUNT_GL1XC3                  = 0x0000000c,
    GL1XA_PERF_SEL_ARB_REQUESTS                        = 0x0000000d,
    GL1XA_PERF_SEL_REQ_INFLIGHT_LEVEL                  = 0x0000000e,
    GL1XA_PERF_SEL_STALL_RET_CONFLICT_GL1XC0           = 0x0000000f,
    GL1XA_PERF_SEL_STALL_RET_CONFLICT_GL1XC1           = 0x00000010,
    GL1XA_PERF_SEL_STALL_RET_CONFLICT_GL1XC2           = 0x00000011,
    GL1XA_PERF_SEL_STALL_RET_CONFLICT_GL1XC3           = 0x00000012,
    GL1XA_PERF_SEL_REQ_PATH_BUSY                       = 0x00000013,
    GL1XA_PERF_SEL_RET_PATH_BUSY                       = 0x00000014,
    GL1XA_PERF_SEL_CYCLE                               = 0x00000015,
} GL1XA_PERF_SEL;

constexpr unsigned int MaxGl1xaPerfSel                 = GL1XA_PERF_SEL_CYCLE;

typedef enum GL1XC_PERF_SEL {
    GL1XC_PERF_SEL_CYCLE                               = 0x00000000,
    GL1XC_PERF_SEL_BUSY                                = 0x00000001,
    GL1XC_PERF_SEL_STARVE                              = 0x00000002,
    GL1XC_PERF_SEL_ARB_RET_LEVEL                       = 0x00000003,
    GL1XC_PERF_SEL_GL2_REQ_READ                        = 0x00000004,
    GL1XC_PERF_SEL_GL2_REQ_READ_32B                    = 0x00000005,
    GL1XC_PERF_SEL_GL2_REQ_READ_64B                    = 0x00000006,
    GL1XC_PERF_SEL_GL2_REQ_READ_128B                   = 0x00000007,
    GL1XC_PERF_SEL_GL2_REQ_READ_256B                   = 0x00000008,
    GL1XC_PERF_SEL_GL2_REQ_READ_LEVEL                  = 0x00000009,
    GL1XC_PERF_SEL_GL2_REQ_WRITE                       = 0x0000000a,
    GL1XC_PERF_SEL_GL2_REQ_WRITE_0B                    = 0x0000000b,
    GL1XC_PERF_SEL_GL2_REQ_WRITE_32B                   = 0x0000000c,
    GL1XC_PERF_SEL_GL2_REQ_WRITE_64B                   = 0x0000000d,
    GL1XC_PERF_SEL_GL2_REQ_WRITE_128B                  = 0x0000000e,
    GL1XC_PERF_SEL_GL2_REQ_WRITE_256B                  = 0x0000000f,
    GL1XC_PERF_SEL_GL2_REQ_WRITE_LEVEL                 = 0x00000010,
    GL1XC_PERF_SEL_GL2_REQ_WRITE_32B_DATA_BEATS        = 0x00000011,
    GL1XC_PERF_SEL_REQ                                 = 0x00000012,
    GL1XC_PERF_SEL_REQ_ATOMIC_WITH_RET                 = 0x00000013,
    GL1XC_PERF_SEL_REQ_ATOMIC_WITHOUT_RET              = 0x00000014,
    GL1XC_PERF_SEL_REQ_NOP_ACK                         = 0x00000015,
    GL1XC_PERF_SEL_REQ_NOP_RTN0                        = 0x00000016,
    GL1XC_PERF_SEL_REQ_READ                            = 0x00000017,
    GL1XC_PERF_SEL_REQ_READ_32B                        = 0x00000018,
    GL1XC_PERF_SEL_REQ_READ_64B                        = 0x00000019,
    GL1XC_PERF_SEL_REQ_READ_128B                       = 0x0000001a,
    GL1XC_PERF_SEL_REQ_READ_256B                       = 0x0000001b,
    GL1XC_PERF_SEL_REQ_WRITE                           = 0x0000001c,
    GL1XC_PERF_SEL_REQ_WRITE_0B                        = 0x0000001d,
    GL1XC_PERF_SEL_REQ_WRITE_32B                       = 0x0000001e,
    GL1XC_PERF_SEL_REQ_WRITE_64B                       = 0x0000001f,
    GL1XC_PERF_SEL_REQ_WRITE_128B                      = 0x00000020,
    GL1XC_PERF_SEL_REQ_WRITE_256B                      = 0x00000021,
    GL1XC_PERF_SEL_REQ_WRITE_32B_DATA_BEATS            = 0x00000022,
    GL1XC_PERF_SEL_REQ_LOAD_RESERVE_32B                = 0x00000023,
    GL1XC_PERF_SEL_REQ_LOAD_RESERVE_64B                = 0x00000024,
    GL1XC_PERF_SEL_REQ_READ_COMP_KEY                   = 0x00000025,
    GL1XC_PERF_SEL_REQ_FORCE_EXISTING_DATA_DECOMPRESS  = 0x00000026,
    GL1XC_PERF_SEL_READ_COMP_MODE_0                    = 0x00000027,
    GL1XC_PERF_SEL_READ_COMP_MODE_1                    = 0x00000028,
    GL1XC_PERF_SEL_READ_COMP_MODE_2                    = 0x00000029,
    GL1XC_PERF_SEL_READ_32B_COMP_MODE_1                = 0x0000002a,
    GL1XC_PERF_SEL_READ_64B_COMP_MODE_1                = 0x0000002b,
    GL1XC_PERF_SEL_READ_128B_COMP_MODE_1               = 0x0000002c,
    GL1XC_PERF_SEL_READ_256B_COMP_MODE_1               = 0x0000002d,
    GL1XC_PERF_SEL_WRITE_COMP_MODE_0                   = 0x0000002e,
    GL1XC_PERF_SEL_WRITE_COMP_MODE_1                   = 0x0000002f,
    GL1XC_PERF_SEL_WRITE_COMP_MODE_2                   = 0x00000030,
    GL1XC_PERF_SEL_WRITE_0B_COMP_MODE_1                = 0x00000031,
    GL1XC_PERF_SEL_WRITE_32B_COMP_MODE_1               = 0x00000032,
    GL1XC_PERF_SEL_WRITE_64B_COMP_MODE_1               = 0x00000033,
    GL1XC_PERF_SEL_WRITE_128B_COMP_MODE_1              = 0x00000034,
    GL1XC_PERF_SEL_WRITE_256B_COMP_MODE_1              = 0x00000035,
    GL1XC_PERF_SEL_GL2_RET_ACK                         = 0x00000036,
    GL1XC_PERF_SEL_GL2_RET_32B                         = 0x00000037,
    GL1XC_PERF_SEL_GL1_RET_ACK                         = 0x00000038,
    GL1XC_PERF_SEL_GL1_RET_32B                         = 0x00000039,
    GL1XC_PERF_SEL_STALL_GL1_GL2_REQ_CREDITS           = 0x0000003a,
    GL1XC_PERF_SEL_STALL_GL1_GL2_DATA_CREDITS          = 0x0000003b,
    GL1XC_PERF_SEL_STALL_BUFFER_FULL                   = 0x0000003c,
    GL1XC_PERF_SEL_STALL_GCR_SHOOTDOWN_PENDING         = 0x0000003d,
    GL1XC_PERF_SEL_STALL_VM                            = 0x0000003e,
    GL1XC_PERF_SEL_REQ_CLIENT0                         = 0x0000003f,
    GL1XC_PERF_SEL_REQ_CLIENT1                         = 0x00000040,
    GL1XC_PERF_SEL_REQ_CLIENT2                         = 0x00000041,
    GL1XC_PERF_SEL_REQ_CLIENT3                         = 0x00000042,
    GL1XC_PERF_SEL_REQ_CLIENT4                         = 0x00000043,
    GL1XC_PERF_SEL_REQ_CLIENT5                         = 0x00000044,
    GL1XC_PERF_SEL_REQ_CLIENT6                         = 0x00000045,
    GL1XC_PERF_SEL_REQ_CLIENT7                         = 0x00000046,
    GL1XC_PERF_SEL_REQ_CLIENT8                         = 0x00000047,
    GL1XC_PERF_SEL_REQ_CLIENT9                         = 0x00000048,
    GL1XC_PERF_SEL_REQ_CLIENT10                        = 0x00000049,
    GL1XC_PERF_SEL_REQ_CLIENT11                        = 0x0000004a,
    GL1XC_PERF_SEL_REQ_CLIENT12                        = 0x0000004b,
    GL1XC_PERF_SEL_REQ_CLIENT13                        = 0x0000004c,
    GL1XC_PERF_SEL_REQ_CLIENT14                        = 0x0000004d,
    GL1XC_PERF_SEL_REQ_CLIENT15                        = 0x0000004e,
    GL1XC_PERF_SEL_REQ_CLIENT16                        = 0x0000004f,
    GL1XC_PERF_SEL_REQ_CLIENT17                        = 0x00000050,
    GL1XC_PERF_SEL_REQ_CLIENT18                        = 0x00000051,
    GL1XC_PERF_SEL_REQ_CLIENT19                        = 0x00000052,
    GL1XC_PERF_SEL_REQ_CLIENT20                        = 0x00000053,
    GL1XC_PERF_SEL_REQ_CLIENT21                        = 0x00000054,
    GL1XC_PERF_SEL_REQ_CLIENT22                        = 0x00000055,
    GL1XC_PERF_SEL_REQ_CLIENT23                        = 0x00000056,
    GL1XC_PERF_SEL_REQ_CLIENT24                        = 0x00000057,
    GL1XC_PERF_SEL_REQ_CLIENT25                        = 0x00000058,
    GL1XC_PERF_SEL_REQ_CLIENT26                        = 0x00000059,
    GL1XC_PERF_SEL_REQ_CLIENT27                        = 0x0000005a,
    GL1XC_PERF_SEL_UTCL0_REQUEST                       = 0x0000005b,
    GL1XC_PERF_SEL_UTCL0_TRANSLATION_HIT               = 0x0000005c,
    GL1XC_PERF_SEL_UTCL0_TRANSLATION_MISS              = 0x0000005d,
    GL1XC_PERF_SEL_UTCL0_PERMISSION_MISS               = 0x0000005e,
    GL1XC_PERF_SEL_UTCL0_MISS_UNDER_MISS               = 0x0000005f,
    GL1XC_PERF_SEL_UTCL0_LFIFO_FULL                    = 0x00000060,
    GL1XC_PERF_SEL_UTCL0_STALL_INFLIGHT_MAX            = 0x00000061,
    GL1XC_PERF_SEL_UTCL0_STALL_LFIFO_NOT_RES           = 0x00000062,
    GL1XC_PERF_SEL_UTCL0_STALL_LRU_INFLIGHT            = 0x00000063,
    GL1XC_PERF_SEL_UTCL0_STALL_MISSFIFO_FULL           = 0x00000064,
    GL1XC_PERF_SEL_UTCL0_STALL_UTCL1_REQ_OUT_OF_CREDITS = 0x00000065,
    GL1XC_PERF_SEL_UTCL0_UTCL1_PERM_FAULT              = 0x00000066,
    GL1XC_PERF_SEL_CLIENT_UTCL0_INFLIGHT               = 0x00000067,
    GL1XC_PERF_SEL_UTCL0_UTCL1_INFLIGHT                = 0x00000068,
    GL1XC_PERF_SEL_UTCL0_INTERNAL_RETRY_REQ            = 0x00000069,
    GL1XC_PERF_SEL_UTCL0_UTCL1_XNACK_RETRY_FAULT       = 0x0000006a,
    GL1XC_PERF_SEL_UTCL0_UTCL1_XNACK_PRT_FAULT         = 0x0000006b,
    GL1XC_PERF_SEL_UTCL0_UTCL1_XNACK_NO_RETRY_FAULT    = 0x0000006c,
    GL1XC_PERF_SEL_UTCL0_GPA3_REQUEST                  = 0x0000006d,
} GL1XC_PERF_SEL;

constexpr unsigned int MaxGl1xcPerfSel                 = GL1XC_PERF_SEL_UTCL0_GPA3_REQUEST;

typedef enum GL1_CACHE_POLICIES {
    GL1_CACHE_POLICY_MISS_LRU                          = 0x00000000,
    GL1_CACHE_POLICY_MISS_EVICT                        = 0x00000001,
    GL1_CACHE_POLICY_HIT_LRU                           = 0x00000002,
    GL1_CACHE_POLICY_HIT_EVICT                         = 0x00000003,
} GL1_CACHE_POLICIES;

typedef enum GL1_CACHE_STORE_POLICIES {
    GL1_CACHE_STORE_POLICY_BYPASS                      = 0x00000000,
} GL1_CACHE_STORE_POLICIES;

typedef enum GL2A_PERF_SEL {
    GL2A_PERF_SEL_NONE                                 = 0x00000000,
    GL2A_PERF_SEL_CYCLE                                = 0x00000001,
    GL2A_PERF_SEL_BUSY                                 = 0x00000002,
    GL2A_PERF_SEL_REQ_GL2C0                            = 0x00000003,
    GL2A_PERF_SEL_REQ_GL2C1                            = 0x00000004,
    GL2A_PERF_SEL_REQ_GL2C2                            = 0x00000005,
    GL2A_PERF_SEL_REQ_GL2C3                            = 0x00000006,
    GL2A_PERF_SEL_REQ_GL2C4                            = 0x00000007,
    GL2A_PERF_SEL_REQ_GL2C5                            = 0x00000008,
    GL2A_PERF_SEL_REQ_GL2C6                            = 0x00000009,
    GL2A_PERF_SEL_REQ_GL2C7                            = 0x0000000a,
    GL2A_PERF_SEL_REQ_BURST_GL2C0                      = 0x00000013,
    GL2A_PERF_SEL_REQ_BURST_GL2C1                      = 0x00000014,
    GL2A_PERF_SEL_REQ_BURST_GL2C2                      = 0x00000015,
    GL2A_PERF_SEL_REQ_BURST_GL2C3                      = 0x00000016,
    GL2A_PERF_SEL_REQ_BURST_GL2C4                      = 0x00000017,
    GL2A_PERF_SEL_REQ_BURST_GL2C5                      = 0x00000018,
    GL2A_PERF_SEL_REQ_BURST_GL2C6                      = 0x00000019,
    GL2A_PERF_SEL_REQ_BURST_GL2C7                      = 0x0000001a,
    GL2A_PERF_SEL_REQ_STALL_GL2C0                      = 0x0000001b,
    GL2A_PERF_SEL_REQ_STALL_GL2C1                      = 0x0000001c,
    GL2A_PERF_SEL_REQ_STALL_GL2C2                      = 0x0000001d,
    GL2A_PERF_SEL_REQ_STALL_GL2C3                      = 0x0000001e,
    GL2A_PERF_SEL_REQ_STALL_GL2C4                      = 0x0000001f,
    GL2A_PERF_SEL_REQ_STALL_GL2C5                      = 0x00000020,
    GL2A_PERF_SEL_REQ_STALL_GL2C6                      = 0x00000021,
    GL2A_PERF_SEL_REQ_STALL_GL2C7                      = 0x00000022,
    GL2A_PERF_SEL_RTN_STALL_GL2C0                      = 0x00000023,
    GL2A_PERF_SEL_RTN_STALL_GL2C1                      = 0x00000024,
    GL2A_PERF_SEL_RTN_STALL_GL2C2                      = 0x00000025,
    GL2A_PERF_SEL_RTN_STALL_GL2C3                      = 0x00000026,
    GL2A_PERF_SEL_RTN_STALL_GL2C4                      = 0x00000027,
    GL2A_PERF_SEL_RTN_STALL_GL2C5                      = 0x00000028,
    GL2A_PERF_SEL_RTN_STALL_GL2C6                      = 0x00000029,
    GL2A_PERF_SEL_RTN_STALL_GL2C7                      = 0x0000002a,
    GL2A_PERF_SEL_RTN_CLIENT0                          = 0x0000002b,
    GL2A_PERF_SEL_RTN_CLIENT1                          = 0x0000002c,
    GL2A_PERF_SEL_RTN_CLIENT2                          = 0x0000002d,
    GL2A_PERF_SEL_RTN_CLIENT3                          = 0x0000002e,
    GL2A_PERF_SEL_RTN_CLIENT4                          = 0x0000002f,
    GL2A_PERF_SEL_RTN_CLIENT5                          = 0x00000030,
    GL2A_PERF_SEL_RTN_CLIENT6                          = 0x00000031,
    GL2A_PERF_SEL_RTN_CLIENT7                          = 0x00000032,
    GL2A_PERF_SEL_RTN_CLIENT8                          = 0x00000033,
    GL2A_PERF_SEL_RTN_CLIENT9                          = 0x00000034,
    GL2A_PERF_SEL_RTN_CLIENT10                         = 0x00000035,
    GL2A_PERF_SEL_RTN_CLIENT11                         = 0x00000036,
    GL2A_PERF_SEL_RTN_CLIENT12                         = 0x00000037,
    GL2A_PERF_SEL_RTN_CLIENT13                         = 0x00000038,
    GL2A_PERF_SEL_RTN_CLIENT14                         = 0x00000039,
    GL2A_PERF_SEL_RTN_CLIENT15                         = 0x0000003a,
    GL2A_PERF_SEL_RTN_ARB_COLLISION_CLIENT0            = 0x0000003b,
    GL2A_PERF_SEL_RTN_ARB_COLLISION_CLIENT1            = 0x0000003c,
    GL2A_PERF_SEL_RTN_ARB_COLLISION_CLIENT2            = 0x0000003d,
    GL2A_PERF_SEL_RTN_ARB_COLLISION_CLIENT3            = 0x0000003e,
    GL2A_PERF_SEL_RTN_ARB_COLLISION_CLIENT4            = 0x0000003f,
    GL2A_PERF_SEL_RTN_ARB_COLLISION_CLIENT5            = 0x00000040,
    GL2A_PERF_SEL_RTN_ARB_COLLISION_CLIENT6            = 0x00000041,
    GL2A_PERF_SEL_RTN_ARB_COLLISION_CLIENT7            = 0x00000042,
    GL2A_PERF_SEL_RTN_ARB_COLLISION_CLIENT8            = 0x00000043,
    GL2A_PERF_SEL_RTN_ARB_COLLISION_CLIENT9            = 0x00000044,
    GL2A_PERF_SEL_RTN_ARB_COLLISION_CLIENT10           = 0x00000045,
    GL2A_PERF_SEL_RTN_ARB_COLLISION_CLIENT11           = 0x00000046,
    GL2A_PERF_SEL_RTN_ARB_COLLISION_CLIENT12           = 0x00000047,
    GL2A_PERF_SEL_RTN_ARB_COLLISION_CLIENT13           = 0x00000048,
    GL2A_PERF_SEL_RTN_ARB_COLLISION_CLIENT14           = 0x00000049,
    GL2A_PERF_SEL_RTN_ARB_COLLISION_CLIENT15           = 0x0000004a,
    GL2A_PERF_SEL_REQ_BURST_CLIENT0                    = 0x0000004b,
    GL2A_PERF_SEL_REQ_BURST_CLIENT1                    = 0x0000004c,
    GL2A_PERF_SEL_REQ_BURST_CLIENT2                    = 0x0000004d,
    GL2A_PERF_SEL_REQ_BURST_CLIENT3                    = 0x0000004e,
    GL2A_PERF_SEL_REQ_BURST_CLIENT4                    = 0x0000004f,
    GL2A_PERF_SEL_REQ_BURST_CLIENT5                    = 0x00000050,
    GL2A_PERF_SEL_REQ_BURST_CLIENT6                    = 0x00000051,
    GL2A_PERF_SEL_REQ_BURST_CLIENT7                    = 0x00000052,
    GL2A_PERF_SEL_REQ_BURST_CLIENT8                    = 0x00000053,
    GL2A_PERF_SEL_REQ_BURST_CLIENT9                    = 0x00000054,
    GL2A_PERF_SEL_REQ_BURST_CLIENT10                   = 0x00000055,
    GL2A_PERF_SEL_REQ_BURST_CLIENT11                   = 0x00000056,
    GL2A_PERF_SEL_REQ_BURST_CLIENT12                   = 0x00000057,
    GL2A_PERF_SEL_REQ_BURST_CLIENT13                   = 0x00000058,
    GL2A_PERF_SEL_REQ_BURST_CLIENT14                   = 0x00000059,
    GL2A_PERF_SEL_REQ_BURST_CLIENT15                   = 0x0000005a,
    GL2A_PERF_SEL_RTN_CREDIT_STALL_CLIENT0             = 0x0000005b,
    GL2A_PERF_SEL_RTN_CREDIT_STALL_CLIENT1             = 0x0000005c,
    GL2A_PERF_SEL_RTN_CREDIT_STALL_CLIENT2             = 0x0000005d,
    GL2A_PERF_SEL_RTN_CREDIT_STALL_CLIENT3             = 0x0000005e,
    GL2A_PERF_SEL_RTN_CREDIT_STALL_CLIENT4             = 0x0000005f,
    GL2A_PERF_SEL_RTN_CREDIT_STALL_CLIENT5             = 0x00000060,
    GL2A_PERF_SEL_RTN_CREDIT_STALL_CLIENT6             = 0x00000061,
    GL2A_PERF_SEL_RTN_CREDIT_STALL_CLIENT7             = 0x00000062,
    GL2A_PERF_SEL_RTN_CREDIT_STALL_CLIENT8             = 0x00000063,
    GL2A_PERF_SEL_RTN_CREDIT_STALL_CLIENT9             = 0x00000064,
    GL2A_PERF_SEL_RTN_CREDIT_STALL_CLIENT10            = 0x00000065,
    GL2A_PERF_SEL_RTN_CREDIT_STALL_CLIENT11            = 0x00000066,
    GL2A_PERF_SEL_RTN_CREDIT_STALL_CLIENT12            = 0x00000067,
    GL2A_PERF_SEL_RTN_CREDIT_STALL_CLIENT13            = 0x00000068,
    GL2A_PERF_SEL_RTN_CREDIT_STALL_CLIENT14            = 0x00000069,
    GL2A_PERF_SEL_RTN_CREDIT_STALL_CLIENT15            = 0x0000006a,
    GL2A_PERF_SEL_REQ_HI_PRIO_GL2C0                    = 0x0000006b,
    GL2A_PERF_SEL_REQ_HI_PRIO_GL2C1                    = 0x0000006c,
    GL2A_PERF_SEL_REQ_HI_PRIO_GL2C2                    = 0x0000006d,
    GL2A_PERF_SEL_REQ_HI_PRIO_GL2C3                    = 0x0000006e,
    GL2A_PERF_SEL_REQ_HI_PRIO_GL2C4                    = 0x0000006f,
    GL2A_PERF_SEL_REQ_HI_PRIO_GL2C5                    = 0x00000070,
    GL2A_PERF_SEL_REQ_HI_PRIO_GL2C6                    = 0x00000071,
    GL2A_PERF_SEL_REQ_HI_PRIO_GL2C7                    = 0x00000072,
} GL2A_PERF_SEL;

constexpr unsigned int MaxGl2aPerfSel                  = GL2A_PERF_SEL_REQ_HI_PRIO_GL2C7;

typedef enum GL2C_PERF_SEL {
    GL2C_PERF_SEL_NONE                                 = 0x00000000,
    GL2C_PERF_SEL_CYCLE                                = 0x00000001,
    GL2C_PERF_SEL_BUSY                                 = 0x00000002,
    GL2C_PERF_SEL_REQ                                  = 0x00000003,
    GL2C_PERF_SEL_HIGH_PRIORITY_REQ                    = 0x00000004,
    GL2C_PERF_SEL_READ                                 = 0x00000005,
    GL2C_PERF_SEL_WRITE                                = 0x00000006,
    GL2C_PERF_SEL_IO_READ                              = 0x00000007,
    GL2C_PERF_SEL_IO_WRITE                             = 0x00000008,
    GL2C_PERF_SEL_ATOMIC                               = 0x00000009,
    GL2C_PERF_SEL_ATOMIC_SECTORS                       = 0x0000000a,
    GL2C_PERF_SEL_BLOCKED                              = 0x0000000b,
    GL2C_PERF_SEL_NOP_ACK                              = 0x0000000c,
    GL2C_PERF_SEL_NOP_RTN0                             = 0x0000000d,
    GL2C_PERF_SEL_OP_INV_START                         = 0x0000000e,
    GL2C_PERF_SEL_OP_WB_START                          = 0x0000000f,
    GL2C_PERF_SEL_OP_WBINV_START                       = 0x00000010,
    GL2C_PERF_SEL_CLIENT0_REQ                          = 0x00000011,
    GL2C_PERF_SEL_CLIENT1_REQ                          = 0x00000012,
    GL2C_PERF_SEL_CLIENT2_REQ                          = 0x00000013,
    GL2C_PERF_SEL_CLIENT3_REQ                          = 0x00000014,
    GL2C_PERF_SEL_CLIENT4_REQ                          = 0x00000015,
    GL2C_PERF_SEL_CLIENT5_REQ                          = 0x00000016,
    GL2C_PERF_SEL_CLIENT6_REQ                          = 0x00000017,
    GL2C_PERF_SEL_CLIENT7_REQ                          = 0x00000018,
    GL2C_PERF_SEL_CLIENT8_REQ                          = 0x00000019,
    GL2C_PERF_SEL_CLIENT9_REQ                          = 0x0000001a,
    GL2C_PERF_SEL_CLIENT10_REQ                         = 0x0000001b,
    GL2C_PERF_SEL_CLIENT11_REQ                         = 0x0000001c,
    GL2C_PERF_SEL_CLIENT12_REQ                         = 0x0000001d,
    GL2C_PERF_SEL_CLIENT13_REQ                         = 0x0000001e,
    GL2C_PERF_SEL_CLIENT14_REQ                         = 0x0000001f,
    GL2C_PERF_SEL_CLIENT15_REQ                         = 0x00000020,
    GL2C_PERF_SEL_UC_REQ                               = 0x00000021,
    GL2C_PERF_SEL_RT_REQ                               = 0x00000022,
    GL2C_PERF_SEL_NT_REQ                               = 0x00000023,
    GL2C_PERF_SEL_FM_REQ                               = 0x00000024,
    GL2C_PERF_SEL_FW_REQ                               = 0x00000025,
    GL2C_PERF_SEL_LU_REQ                               = 0x00000026,
    GL2C_PERF_SEL_CC_REQ                               = 0x00000027,
    GL2C_PERF_SEL_CC_OPS_COMPLETED                     = 0x00000028,
    GL2C_PERF_SEL_HIT                                  = 0x00000029,
    GL2C_PERF_SEL_MISS                                 = 0x0000002a,
    GL2C_PERF_SEL_FULL_HIT                             = 0x0000002b,
    GL2C_PERF_SEL_PARTIAL_32B_HIT                      = 0x0000002c,
    GL2C_PERF_SEL_PARTIAL_64B_HIT                      = 0x0000002d,
    GL2C_PERF_SEL_PARTIAL_96B_HIT                      = 0x0000002e,
    GL2C_PERF_SEL_PARTIAL_128B_HIT                     = 0x0000002f,
    GL2C_PERF_SEL_PARTIAL_160B_HIT                     = 0x00000030,
    GL2C_PERF_SEL_PARTIAL_192B_HIT                     = 0x00000031,
    GL2C_PERF_SEL_PARTIAL_224B_HIT                     = 0x00000032,
    GL2C_PERF_SEL_DEWRITE_ALLOCATE_HIT                 = 0x00000033,
    GL2C_PERF_SEL_FULLY_WRITTEN_HIT                    = 0x00000034,
    GL2C_PERF_SEL_UNCACHED_WRITE                       = 0x00000035,
    GL2C_PERF_SEL_WRITEBACK                            = 0x00000036,
    GL2C_PERF_SEL_NORMAL_WRITEBACK                     = 0x00000037,
    GL2C_PERF_SEL_EVICT                                = 0x00000038,
    GL2C_PERF_SEL_NORMAL_EVICT                         = 0x00000039,
    GL2C_PERF_SEL_REQ_TO_MISS_QUEUE                    = 0x0000003a,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT0             = 0x0000003b,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT1             = 0x0000003c,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT2             = 0x0000003d,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT3             = 0x0000003e,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT4             = 0x0000003f,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT5             = 0x00000040,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT6             = 0x00000041,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT7             = 0x00000042,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT8             = 0x00000043,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT9             = 0x00000044,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT10            = 0x00000045,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT11            = 0x00000046,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT12            = 0x00000047,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT13            = 0x00000048,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT14            = 0x00000049,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT15            = 0x0000004a,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_HIGH_PRI_QUEUE      = 0x0000004b,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_IO_QUEUE            = 0x0000004c,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_UC_QUEUE            = 0x0000004d,
    GL2C_PERF_SEL_READ_32_REQ                          = 0x0000004e,
    GL2C_PERF_SEL_READ_64_REQ                          = 0x0000004f,
    GL2C_PERF_SEL_READ_128_REQ                         = 0x00000050,
    GL2C_PERF_SEL_READ_256_REQ                         = 0x00000051,
    GL2C_PERF_SEL_WRITE_32_REQ                         = 0x00000052,
    GL2C_PERF_SEL_WRITE_64_REQ                         = 0x00000053,
    GL2C_PERF_SEL_WRITE_128_REQ                        = 0x00000054,
    GL2C_PERF_SEL_WRITE_256_REQ                        = 0x00000055,
    GL2C_PERF_SEL_READ_COMPRESSED_32_REQ               = 0x00000056,
    GL2C_PERF_SEL_READ_COMPRESSED_64_REQ               = 0x00000057,
    GL2C_PERF_SEL_READ_COMPRESSED_128_REQ              = 0x00000058,
    GL2C_PERF_SEL_READ_COMPRESSED_256_REQ              = 0x00000059,
    GL2C_PERF_SEL_WRITE_COMPRESSED_128_REQ             = 0x0000005a,
    GL2C_PERF_SEL_WRITE_COMPRESSED_256_REQ             = 0x0000005b,
    GL2C_PERF_SEL_WRITE_COMPRESSED_32B_SRC_DATA_BEATS  = 0x0000005c,
    GL2C_PERF_SEL_WRITE_COMPRESSED_64B_SRC_DATA_BEATS  = 0x0000005d,
    GL2C_PERF_SEL_READ_SECTOR_MISS                     = 0x0000005e,
    GL2C_PERF_SEL_READ_SECTOR_MISS_DUPLICATE           = 0x0000005f,
    GL2C_PERF_SEL_READ_SECTORS                         = 0x00000060,
    GL2C_PERF_SEL_READ_TAG_MISS                        = 0x00000061,
    GL2C_PERF_SEL_SECTORS                              = 0x00000062,
    GL2C_PERF_SEL_SECTORS_FILL                         = 0x00000063,
    GL2C_PERF_SEL_SECTORS_WRITTEN                      = 0x00000064,
    GL2C_PERF_SEL_WRITE_SECTORS                        = 0x00000065,
    GL2C_PERF_SEL_READ_COMP_MODE_0                     = 0x00000066,
    GL2C_PERF_SEL_READ_COMP_MODE_1                     = 0x00000067,
    GL2C_PERF_SEL_READ_COMP_MODE_2                     = 0x00000068,
    GL2C_PERF_SEL_WRITE_COMP_MODE_0                    = 0x00000069,
    GL2C_PERF_SEL_WRITE_COMP_MODE_1                    = 0x0000006a,
    GL2C_PERF_SEL_WRITE_COMP_MODE_2                    = 0x0000006b,
    GL2C_PERF_SEL_EA_WRREQ                             = 0x0000006c,
    GL2C_PERF_SEL_EA_WRREQ_TCP                         = 0x0000006d,
    GL2C_PERF_SEL_EA_WRREQ_CB                          = 0x0000006e,
    GL2C_PERF_SEL_EA_WRREQ_DB                          = 0x0000006f,
    GL2C_PERF_SEL_EA_WRREQ_SNOOP                       = 0x00000070,
    GL2C_PERF_SEL_EA_WRREQ_32B                         = 0x00000071,
    GL2C_PERF_SEL_EA_WRREQ_64B                         = 0x00000072,
    GL2C_PERF_SEL_EA_WRREQ_128B                        = 0x00000073,
    GL2C_PERF_SEL_EA_WRREQ_256B                        = 0x00000074,
    GL2C_PERF_SEL_EA_WRREQ_COMPRESSED_128B             = 0x00000075,
    GL2C_PERF_SEL_EA_WRREQ_COMPRESSED_256B             = 0x00000076,
    GL2C_PERF_SEL_EA_WRDATA_COMPRESSED_32B             = 0x00000077,
    GL2C_PERF_SEL_EA_WRDATA_COMPRESSED_64B             = 0x00000078,
    GL2C_PERF_SEL_EA_WR_UNCACHED_32B                   = 0x00000079,
    GL2C_PERF_SEL_EA_WRREQ_STALL                       = 0x0000007a,
    GL2C_PERF_SEL_EA_WRREQ_IO_CREDIT_STALL             = 0x0000007b,
    GL2C_PERF_SEL_EA_WRREQ_DRAM_CREDIT_STALL           = 0x0000007c,
    GL2C_PERF_SEL_EA_DATA_IO_CREDIT_STALL              = 0x0000007d,
    GL2C_PERF_SEL_EA_DATA_DRAM_CREDIT_STALL            = 0x0000007e,
    GL2C_PERF_SEL_TOO_MANY_EA_WRREQS_STALL             = 0x0000007f,
    GL2C_PERF_SEL_EA_WRREQ_LEVEL                       = 0x00000080,
    GL2C_PERF_SEL_EA_WRREQ_LEVEL_TCP                   = 0x00000081,
    GL2C_PERF_SEL_EA_WRREQ_LEVEL_CB                    = 0x00000082,
    GL2C_PERF_SEL_EA_WRREQ_LEVEL_DB                    = 0x00000083,
    GL2C_PERF_SEL_EA_ATOMIC                            = 0x00000084,
    GL2C_PERF_SEL_EA_ATOMIC_TCP                        = 0x00000085,
    GL2C_PERF_SEL_EA_ATOMIC_CB                         = 0x00000086,
    GL2C_PERF_SEL_EA_ATOMIC_DB                         = 0x00000087,
    GL2C_PERF_SEL_EA_ATOMIC_LEVEL                      = 0x00000088,
    GL2C_PERF_SEL_EA_ATOMIC_LEVEL_TCP                  = 0x00000089,
    GL2C_PERF_SEL_EA_ATOMIC_LEVEL_CB                   = 0x0000008a,
    GL2C_PERF_SEL_EA_ATOMIC_LEVEL_DB                   = 0x0000008b,
    GL2C_PERF_SEL_EA_RDREQ                             = 0x0000008c,
    GL2C_PERF_SEL_EA_RDREQ_TCP                         = 0x0000008d,
    GL2C_PERF_SEL_EA_RDREQ_CB                          = 0x0000008e,
    GL2C_PERF_SEL_EA_RDREQ_DB                          = 0x0000008f,
    GL2C_PERF_SEL_EA_RDREQ_SNOOP                       = 0x00000090,
    GL2C_PERF_SEL_EA_RDREQ_SPLIT                       = 0x00000091,
    GL2C_PERF_SEL_EA_RDREQ_32B                         = 0x00000092,
    GL2C_PERF_SEL_EA_RDREQ_64B                         = 0x00000093,
    GL2C_PERF_SEL_EA_RDREQ_128B                        = 0x00000094,
    GL2C_PERF_SEL_EA_RDREQ_256B                        = 0x00000095,
    GL2C_PERF_SEL_EA_RDREQ_COMPRESSED_128B             = 0x00000096,
    GL2C_PERF_SEL_EA_RDREQ_COMPRESSED_256B             = 0x00000097,
    GL2C_PERF_SEL_EA_RDDATA_COMPRESSED_32B             = 0x00000098,
    GL2C_PERF_SEL_EA_RDDATA_COMPRESSED_64B             = 0x00000099,
    GL2C_PERF_SEL_EA_RDDATA_32B                        = 0x0000009a,
    GL2C_PERF_SEL_EA_RD_UNCACHED_32B                   = 0x0000009b,
    GL2C_PERF_SEL_EA_RD_COMPRESSED_32B                 = 0x0000009c,
    GL2C_PERF_SEL_EA_RDREQ_IO_CREDIT_STALL             = 0x0000009d,
    GL2C_PERF_SEL_EA_RDREQ_DRAM_CREDIT_STALL           = 0x0000009e,
    GL2C_PERF_SEL_EA_REQ_IO_CREDIT_STALL               = 0x0000009f,
    GL2C_PERF_SEL_EA_REQ_DRAM_CREDIT_STALL             = 0x000000a0,
    GL2C_PERF_SEL_EA_RDREQ_LEVEL                       = 0x000000a1,
    GL2C_PERF_SEL_EA_RDREQ_LEVEL_TCP                   = 0x000000a2,
    GL2C_PERF_SEL_EA_RDREQ_LEVEL_CB                    = 0x000000a3,
    GL2C_PERF_SEL_EA_RDREQ_LEVEL_DB                    = 0x000000a4,
    GL2C_PERF_SEL_EA_RDREQ_DRAM                        = 0x000000a5,
    GL2C_PERF_SEL_EA_WRREQ_DRAM                        = 0x000000a6,
    GL2C_PERF_SEL_EA_RDREQ_DRAM_32B                    = 0x000000a7,
    GL2C_PERF_SEL_EA_WRREQ_DRAM_32B                    = 0x000000a8,
    GL2C_PERF_SEL_EA_IO_READ                           = 0x000000a9,
    GL2C_PERF_SEL_EA_IO_WRITE                          = 0x000000aa,
    GL2C_PERF_SEL_EA_OUTSTANDING                       = 0x000000ab,
    GL2C_PERF_SEL_EA_RDRET_VALID                       = 0x000000ac,
    GL2C_PERF_SEL_EA_WRRET_ACK                         = 0x000000ad,
    GL2C_PERF_SEL_LATENCY_FIFO_FULL                    = 0x000000ae,
    GL2C_PERF_SEL_SRC_FIFO_FULL                        = 0x000000af,
    GL2C_PERF_SEL_TAG_STALL                            = 0x000000b0,
    GL2C_PERF_SEL_TAG_WRITEBACK_FIFO_FULL_STALL        = 0x000000b1,
    GL2C_PERF_SEL_TAG_CACHED_EA_FIFO_FULL_STALL        = 0x000000b2,
    GL2C_PERF_SEL_TAG_MISS_NOTHING_REPLACEABLE_STALL   = 0x000000b3,
    GL2C_PERF_SEL_TAG_HIT_INV_DIRTY_STALL              = 0x000000b4,
    GL2C_PERF_SEL_TAG_UNCACHED_WRITE_ATOMIC_FIFO_FULL_STALL = 0x000000b5,
    GL2C_PERF_SEL_TAG_NO_UNCACHED_WRITE_ATOMIC_ENTRIES_STALL = 0x000000b6,
    GL2C_PERF_SEL_TAG_READ_DST_STALL                   = 0x000000b7,
    GL2C_PERF_SEL_SELF_INITIATED_INV                   = 0x000000b8,
    GL2C_PERF_SEL_SELF_INITIATED_WBINV                 = 0x000000b9,
    GL2C_PERF_SEL_SELF_INITIATED_DISCARD               = 0x000000ba,
    GL2C_PERF_SEL_READ_RETURN_TIMEOUT                  = 0x000000bb,
    GL2C_PERF_SEL_WRITEBACK_READ_TIMEOUT               = 0x000000bc,
    GL2C_PERF_SEL_READ_RETURN_FULL_BUBBLE              = 0x000000bd,
    GL2C_PERF_SEL_BUBBLE                               = 0x000000be,
    GL2C_PERF_SEL_IB_REQ                               = 0x000000bf,
    GL2C_PERF_SEL_IB_STALL                             = 0x000000c0,
    GL2C_PERF_SEL_IB_TAG_STALL                         = 0x000000c1,
    GL2C_PERF_SEL_RETURN_ACK                           = 0x000000c2,
    GL2C_PERF_SEL_RETURN_DATA                          = 0x000000c3,
    GL2C_PERF_SEL_EA_RDRET_NACK                        = 0x000000c4,
    GL2C_PERF_SEL_EA_WRRET_NACK                        = 0x000000c5,
    GL2C_PERF_SEL_GL2A_LEVEL                           = 0x000000c6,
    GL2C_PERF_SEL_ALL_GL2C_OP_WB_OR_INV_START          = 0x000000c7,
    GL2C_PERF_SEL_GCR_INV                              = 0x000000c8,
    GL2C_PERF_SEL_GCR_WB                               = 0x000000c9,
    GL2C_PERF_SEL_GCR_DISCARD                          = 0x000000ca,
    GL2C_PERF_SEL_GCR_RANGE                            = 0x000000cb,
    GL2C_PERF_SEL_GCR_ALL                              = 0x000000cc,
    GL2C_PERF_SEL_GCR_VOL                              = 0x000000cd,
    GL2C_PERF_SEL_GCR_UNSHARED                         = 0x000000ce,
    GL2C_PERF_SEL_GCR_GL2_INV_ALL                      = 0x000000cf,
    GL2C_PERF_SEL_GCR_GL2_WB_ALL                       = 0x000000d0,
    GL2C_PERF_SEL_GCR_GL2_INV_RANGE                    = 0x000000d1,
    GL2C_PERF_SEL_GCR_GL2_WB_RANGE                     = 0x000000d2,
    GL2C_PERF_SEL_GCR_GL2_WB_INV_RANGE                 = 0x000000d3,
    GL2C_PERF_SEL_ALL_GCR_INV_EVICT                    = 0x000000d4,
    GL2C_PERF_SEL_ALL_GCR_WB_OR_INV_CYCLE              = 0x000000d5,
    GL2C_PERF_SEL_ALL_GCR_WB_WRITEBACK                 = 0x000000d6,
    GL2C_PERF_SEL_GCR_INVL2_VOL_START                  = 0x000000d7,
    GL2C_PERF_SEL_GCR_WBL2_VOL_START                   = 0x000000d8,
    GL2C_PERF_SEL_GCR_WBINVL2_CYCLE                    = 0x000000d9,
    GL2C_PERF_SEL_GCR_WBINVL2_EVICT                    = 0x000000da,
    GL2C_PERF_SEL_GCR_WBINVL2_START                    = 0x000000db,
    GL2C_PERF_SEL_OC_COMPRESS_DISABLE_REQ              = 0x000000dc,
    GL2C_PERF_SEL_OC_BYPASS_REQ                        = 0x000000dd,
    GL2C_PERF_SEL_COMPRESS_IN_XFC                      = 0x000000de,
    GL2C_PERF_SEL_COMPRESS_IN_COMP                     = 0x000000df,
    GL2C_PERF_SEL_COMPRESS_IN_UNCOMP                   = 0x000000e0,
    GL2C_PERF_SEL_COMPRESS_IN_CONST2SINGLE_256B        = 0x000000e1,
    GL2C_PERF_SEL_COMPRESS_IN_CONST2SINGLE_128B        = 0x000000e2,
    GL2C_PERF_SEL_COMPRESS_IN_CONST2CLEAR_256B         = 0x000000e3,
    GL2C_PERF_SEL_COMPRESS_IN_CONST2CLEAR_128B         = 0x000000e4,
    GL2C_PERF_SEL_COMPRESS_OUT_XFC                     = 0x000000e5,
    GL2C_PERF_SEL_COMPRESS_OUT_COMP                    = 0x000000e6,
    GL2C_PERF_SEL_COMPRESS_OUT_UNCOMP                  = 0x000000e7,
    GL2C_PERF_SEL_COMPRESS_OUT_CONST2SINGLE_256B       = 0x000000e8,
    GL2C_PERF_SEL_COMPRESS_OUT_CONST2SINGLE_128B       = 0x000000e9,
    GL2C_PERF_SEL_COMPRESS_OUT_CONST2CLEAR_256B        = 0x000000ea,
    GL2C_PERF_SEL_COMPRESS_OUT_CONST2CLEAR_128B        = 0x000000eb,
    GL2C_PERF_SEL_COMPRESS_ACTIVE                      = 0x000000ec,
    GL2C_PERF_SEL_COMPRESS_STALLED                     = 0x000000ed,
    GL2C_PERF_SEL_DECOMPRESS_ACTIVE                    = 0x000000ee,
    GL2C_PERF_SEL_DECOMPRESS_STALLED                   = 0x000000ef,
    GL2C_PERF_SEL_DECOMPRESS_DATA_IN                   = 0x000000f0,
    GL2C_PERF_SEL_DECOMPRESS_DATA_OUT                  = 0x000000f1,
    GL2C_PERF_SEL_DECOMPRESS_DATA_IN_READ_COMPRESSED   = 0x000000f2,
    GL2C_PERF_SEL_DECOMPRESS_DATA_OUT_READ_COMPRESSED  = 0x000000f3,
    GL2C_PERF_SEL_EXEC_BYPASS_DATA_32B                 = 0x000000f4,
    GL2C_PERF_SEL_EXEC_BYPASS_DATA_READ_COMPRESSED_32B = 0x000000f5,
    GL2C_PERF_SEL_INPUT_DECOMPRESS_128B_LEVEL          = 0x000000f6,
    GL2C_PERF_SEL_INPUT_DECOMPRESS_128B_REQ            = 0x000000f7,
    GL2C_PERF_SEL_INPUT_DECOMPRESS_256B_LEVEL          = 0x000000f8,
    GL2C_PERF_SEL_INPUT_DECOMPRESS_256B_REQ            = 0x000000f9,
} GL2C_PERF_SEL;

constexpr unsigned int MaxGl2cPerfSel                  = GL2C_PERF_SEL_INPUT_DECOMPRESS_256B_REQ;

typedef enum GL2_CACHE_POLICIES {
    GL2_CACHE_POLICY_LRU                               = 0x00000000,
    GL2_CACHE_POLICY_STREAM                            = 0x00000001,
    GL2_CACHE_POLICY_NOA                               = 0x00000002,
    GL2_CACHE_POLICY_BYPASS                            = 0x00000003,
} GL2_CACHE_POLICIES;

typedef enum GL2_OP {
    GL2_OP_READ                                        = 0x00000000,
    GL2_OP_ATOMIC_FMIN_RTN_32                          = 0x00000002,
    GL2_OP_ATOMIC_FMAX_RTN_32                          = 0x00000003,
    GL2_OP_ATOMIC_PK_ADD_FP16_RTN                      = 0x00000004,
    GL2_OP_ATOMIC_FADD_RTN_32                          = 0x00000005,
    GL2_OP_ATOMIC_PK_ADD_BF16_RTN                      = 0x00000006,
    GL2_OP_ATOMIC_SWAP_RTN_32                          = 0x00000007,
    GL2_OP_ATOMIC_CMPSWAP_RTN_32                       = 0x00000008,
    GL2_OP_ATOMIC_ADD_RTN_32                           = 0x0000000f,
    GL2_OP_ATOMIC_SUB_RTN_32                           = 0x00000010,
    GL2_OP_ATOMIC_SMIN_RTN_32                          = 0x00000011,
    GL2_OP_ATOMIC_UMIN_RTN_32                          = 0x00000012,
    GL2_OP_ATOMIC_SMAX_RTN_32                          = 0x00000013,
    GL2_OP_ATOMIC_UMAX_RTN_32                          = 0x00000014,
    GL2_OP_ATOMIC_AND_RTN_32                           = 0x00000015,
    GL2_OP_ATOMIC_OR_RTN_32                            = 0x00000016,
    GL2_OP_ATOMIC_XOR_RTN_32                           = 0x00000017,
    GL2_OP_ATOMIC_INC_RTN_32                           = 0x00000018,
    GL2_OP_ATOMIC_DEC_RTN_32                           = 0x00000019,
    GL2_OP_ATOMIC_CLAMP_SUB_RTN_32                     = 0x0000001a,
    GL2_OP_ATOMIC_COND_SUB_RTN_32                      = 0x0000001b,
    GL2_OP_UTC_PROBE                                   = 0x0000001d,
    GL2_OP_LOAD_RESERVE                                = 0x0000001e,
    GL2_OP_WRITE                                       = 0x00000020,
    GL2_OP_ATOMIC_ORDERED_ADD_RTN_B64                  = 0x00000026,
    GL2_OP_ATOMIC_SWAP_RTN_64                          = 0x00000027,
    GL2_OP_ATOMIC_CMPSWAP_RTN_64                       = 0x00000028,
    GL2_OP_ATOMIC_ADD_RTN_64                           = 0x0000002f,
    GL2_OP_ATOMIC_SUB_RTN_64                           = 0x00000030,
    GL2_OP_ATOMIC_SMIN_RTN_64                          = 0x00000031,
    GL2_OP_ATOMIC_UMIN_RTN_64                          = 0x00000032,
    GL2_OP_ATOMIC_SMAX_RTN_64                          = 0x00000033,
    GL2_OP_ATOMIC_UMAX_RTN_64                          = 0x00000034,
    GL2_OP_ATOMIC_AND_RTN_64                           = 0x00000035,
    GL2_OP_ATOMIC_OR_RTN_64                            = 0x00000036,
    GL2_OP_ATOMIC_XOR_RTN_64                           = 0x00000037,
    GL2_OP_ATOMIC_INC_RTN_64                           = 0x00000038,
    GL2_OP_ATOMIC_DEC_RTN_64                           = 0x00000039,
    GL2_OP_WRITE_ZERO_SIZE                             = 0x0000003b,
    GL2_OP_GL2_INV                                     = 0x0000003d,
    GL2_OP_ATOMIC_STORE_COND_RTN                       = 0x0000003e,
    GL2_OP_ATOMIC_FMIN_32                              = 0x00000042,
    GL2_OP_ATOMIC_FMAX_32                              = 0x00000043,
    GL2_OP_ATOMIC_PK_ADD_FP16                          = 0x00000044,
    GL2_OP_ATOMIC_FADD_32                              = 0x00000045,
    GL2_OP_ATOMIC_PK_ADD_BF16                          = 0x00000046,
    GL2_OP_ATOMIC_SWAP_32                              = 0x00000047,
    GL2_OP_ATOMIC_CMPSWAP_32                           = 0x00000048,
    GL2_OP_ATOMIC_UMIN_8                               = 0x0000004c,
    GL2_OP_ATOMIC_ADD_32                               = 0x0000004f,
    GL2_OP_ATOMIC_SUB_32                               = 0x00000050,
    GL2_OP_ATOMIC_SMIN_32                              = 0x00000051,
    GL2_OP_ATOMIC_UMIN_32                              = 0x00000052,
    GL2_OP_ATOMIC_SMAX_32                              = 0x00000053,
    GL2_OP_ATOMIC_UMAX_32                              = 0x00000054,
    GL2_OP_ATOMIC_AND_32                               = 0x00000055,
    GL2_OP_ATOMIC_OR_32                                = 0x00000056,
    GL2_OP_ATOMIC_XOR_32                               = 0x00000057,
    GL2_OP_ATOMIC_INC_32                               = 0x00000058,
    GL2_OP_ATOMIC_DEC_32                               = 0x00000059,
    GL2_OP_NOP_RTN0                                    = 0x0000005b,
    GL2_OP_GL2_WB                                      = 0x0000005d,
    GL2_OP_FORCE_EXISTING_DATA_TO_DECOMPRESS           = 0x0000005e,
    GL2_OP_ATOMIC_SWAP_64                              = 0x00000067,
    GL2_OP_ATOMIC_CMPSWAP_64                           = 0x00000068,
    GL2_OP_ATOMIC_ADD_64                               = 0x0000006f,
    GL2_OP_ATOMIC_SUB_64                               = 0x00000070,
    GL2_OP_ATOMIC_SMIN_64                              = 0x00000071,
    GL2_OP_ATOMIC_UMIN_64                              = 0x00000072,
    GL2_OP_ATOMIC_SMAX_64                              = 0x00000073,
    GL2_OP_ATOMIC_UMAX_64                              = 0x00000074,
    GL2_OP_ATOMIC_AND_64                               = 0x00000075,
    GL2_OP_ATOMIC_OR_64                                = 0x00000076,
    GL2_OP_ATOMIC_XOR_64                               = 0x00000077,
    GL2_OP_ATOMIC_INC_64                               = 0x00000078,
    GL2_OP_ATOMIC_DEC_64                               = 0x00000079,
    GL2_OP_ATOMIC_UMAX_8                               = 0x0000007a,
    GL2_OP_NOP_ACK                                     = 0x0000007b,
    GL2_OP_GL2_WBINV                                   = 0x0000007d,
    GL2_OP_READ_COMPRESSION_KEY                        = 0x0000007e,
} GL2_OP;

typedef enum GL2_OP_MASKS {
    GL2_OP_MASK_64                                     = 0x00000020,
    GL2_OP_MASK_NO_RTN                                 = 0x00000040,
} GL2_OP_MASKS;

typedef enum GRBMH_PERF_SEL {
    GRBMH_PERF_SEL_COUNT                               = 0x00000000,
    GRBMH_PERF_SEL_USER_DEFINED                        = 0x00000001,
    GRBMH_PERF_SEL_CB_BUSY                             = 0x00000002,
    GRBMH_PERF_SEL_CB_CLEAN                            = 0x00000003,
    GRBMH_PERF_SEL_DB_BUSY                             = 0x00000004,
    GRBMH_PERF_SEL_DB_CLEAN                            = 0x00000005,
    GRBMH_PERF_SEL_SC_BUSY                             = 0x00000006,
    GRBMH_PERF_SEL_SC_CLEAN                            = 0x00000007,
    GRBMH_PERF_SEL_WGS_BUSY                            = 0x00000008,
    GRBMH_PERF_SEL_SPI_BUSY                            = 0x00000009,
    GRBMH_PERF_SEL_SX_BUSY                             = 0x0000000a,
    GRBMH_PERF_SEL_TA_BUSY                             = 0x0000000b,
    GRBMH_PERF_SEL_EA_BUSY                             = 0x0000000c,
    GRBMH_PERF_SEL_EA_LINK_BUSY                        = 0x0000000d,
    GRBMH_PERF_SEL_PA_BUSY                             = 0x0000000e,
    GRBMH_PERF_SEL_BCI_BUSY                            = 0x0000000f,
    GRBMH_PERF_SEL_GL2A_BUSY                           = 0x00000010,
    GRBMH_PERF_SEL_GL2C_BUSY                           = 0x00000011,
    GRBMH_PERF_SEL_UTCL1_BUSY                          = 0x00000012,
    GRBMH_PERF_SEL_TCP_BUSY                            = 0x00000013,
    GRBMH_PERF_SEL_GL1A_BUSY                           = 0x00000014,
    GRBMH_PERF_SEL_GL1CC_BUSY                          = 0x00000015,
    GRBMH_PERF_SEL_GL1XCC_BUSY                         = 0x00000016,
    GRBMH_PERF_SEL_PC_BUSY                             = 0x00000017,
    GRBMH_PERF_SEL_GE_BUSY                             = 0x00000018,
    GRBMH_PERF_SEL_RLC_BUSY                            = 0x00000019,
} GRBMH_PERF_SEL;

constexpr unsigned int MaxGrbmhPerfSel                 = GRBMH_PERF_SEL_RLC_BUSY;

typedef enum GRBM_PERF_SEL {
    GRBM_PERF_SEL_COUNT                                = 0x00000000,
    GRBM_PERF_SEL_USER_DEFINED                         = 0x00000001,
    GRBM_PERF_SEL_GUI_ACTIVE                           = 0x00000002,
    GRBM_PERF_SEL_CP_BUSY                              = 0x00000003,
    GRBM_PERF_SEL_CP_COHER_BUSY                        = 0x00000004,
    GRBM_PERF_SEL_CP_DMA_BUSY                          = 0x00000005,
    GRBM_PERF_SEL_CB_BUSY                              = 0x00000006,
    GRBM_PERF_SEL_DB_BUSY                              = 0x00000007,
    GRBM_PERF_SEL_PA_BUSY                              = 0x00000008,
    GRBM_PERF_SEL_SC_BUSY                              = 0x00000009,
    GRBM_PERF_SEL_SPI_BUSY                             = 0x0000000b,
    GRBM_PERF_SEL_SX_BUSY                              = 0x0000000c,
    GRBM_PERF_SEL_TA_BUSY                              = 0x0000000d,
    GRBM_PERF_SEL_CB_CLEAN                             = 0x0000000e,
    GRBM_PERF_SEL_DB_CLEAN                             = 0x0000000f,
    GRBM_PERF_SEL_SC_CLEAN                             = 0x00000010,
    GRBM_PERF_SEL_BCI_BUSY                             = 0x0000001a,
    GRBM_PERF_SEL_RLC_BUSY                             = 0x0000001b,
    GRBM_PERF_SEL_TCP_BUSY                             = 0x0000001c,
    GRBM_PERF_SEL_CPG_BUSY                             = 0x0000001d,
    GRBM_PERF_SEL_CPC_BUSY                             = 0x0000001e,
    GRBM_PERF_SEL_CPF_BUSY                             = 0x0000001f,
    GRBM_PERF_SEL_GE_BUSY                              = 0x00000020,
    GRBM_PERF_SEL_GE_NO_DMA_BUSY                       = 0x00000021,
    GRBM_PERF_SEL_UTCL2_BUSY                           = 0x00000022,
    GRBM_PERF_SEL_EA_BUSY                              = 0x00000023,
    GRBM_PERF_SEL_UTCL1_BUSY                           = 0x00000027,
    GRBM_PERF_SEL_GL2CC_BUSY                           = 0x00000028,
    GRBM_PERF_SEL_SDMA_BUSY                            = 0x00000029,
    GRBM_PERF_SEL_CH_BUSY                              = 0x0000002a,
    GRBM_PERF_SEL_PMM_BUSY                             = 0x0000002c,
    GRBM_PERF_SEL_GL1CC_BUSY                           = 0x0000002e,
    GRBM_PERF_SEL_ANY_ACTIVE_F_BUSY                    = 0x0000002f,
    GRBM_PERF_SEL_GL1XCC_BUSY                          = 0x00000030,
    GRBM_PERF_SEL_PC_BUSY                              = 0x00000031,
    GRBM_PERF_SEL_WGS_BUSY                             = 0x00000033,
} GRBM_PERF_SEL;

constexpr unsigned int MaxGrbmPerfSel                  = GRBM_PERF_SEL_WGS_BUSY;

typedef enum IMG_DATA_FORMAT {
    IMG_DATA_FORMAT_INVALID                            = 0x00000000,
    IMG_DATA_FORMAT_8                                  = 0x00000001,
    IMG_DATA_FORMAT_16                                 = 0x00000002,
    IMG_DATA_FORMAT_8_8                                = 0x00000003,
    IMG_DATA_FORMAT_32                                 = 0x00000004,
    IMG_DATA_FORMAT_16_16                              = 0x00000005,
    IMG_DATA_FORMAT_10_11_11                           = 0x00000006,
    IMG_DATA_FORMAT_11_11_10                           = 0x00000007,
    IMG_DATA_FORMAT_10_10_10_2                         = 0x00000008,
    IMG_DATA_FORMAT_2_10_10_10                         = 0x00000009,
    IMG_DATA_FORMAT_8_8_8_8                            = 0x0000000a,
    IMG_DATA_FORMAT_32_32                              = 0x0000000b,
    IMG_DATA_FORMAT_16_16_16_16                        = 0x0000000c,
    IMG_DATA_FORMAT_32_32_32                           = 0x0000000d,
    IMG_DATA_FORMAT_32_32_32_32                        = 0x0000000e,
    IMG_DATA_FORMAT_RESERVED_15                        = 0x0000000f,
    IMG_DATA_FORMAT_5_6_5                              = 0x00000010,
    IMG_DATA_FORMAT_1_5_5_5                            = 0x00000011,
    IMG_DATA_FORMAT_5_5_5_1                            = 0x00000012,
    IMG_DATA_FORMAT_4_4_4_4                            = 0x00000013,
    IMG_DATA_FORMAT_8_24                               = 0x00000014,
    IMG_DATA_FORMAT_24_8                               = 0x00000015,
    IMG_DATA_FORMAT_X24_8_32                           = 0x00000016,
    IMG_DATA_FORMAT_RESERVED_23                        = 0x00000017,
    IMG_DATA_FORMAT_5_9_9_9                            = 0x00000018,
    IMG_DATA_FORMAT_GB_GR                              = 0x00000019,
    IMG_DATA_FORMAT_BG_RG                              = 0x0000001a,
    IMG_DATA_FORMAT_4_4                                = 0x0000001b,
    IMG_DATA_FORMAT_1                                  = 0x0000001c,
    IMG_DATA_FORMAT_1_REVERSED                         = 0x0000001d,
    IMG_DATA_FORMAT_7E3                                = 0x0000001e,
    IMG_DATA_FORMAT_6E4                                = 0x0000001f,
    IMG_DATA_FORMAT_BC1                                = 0x00000020,
    IMG_DATA_FORMAT_BC2                                = 0x00000021,
    IMG_DATA_FORMAT_BC3                                = 0x00000022,
    IMG_DATA_FORMAT_BC4                                = 0x00000023,
    IMG_DATA_FORMAT_BC5                                = 0x00000024,
    IMG_DATA_FORMAT_BC6                                = 0x00000025,
    IMG_DATA_FORMAT_BC7                                = 0x00000026,
    IMG_DATA_FORMAT_RESERVED_39                        = 0x00000027,
    IMG_DATA_FORMAT_RESERVED_40                        = 0x00000028,
    IMG_DATA_FORMAT_RESERVED_41                        = 0x00000029,
    IMG_DATA_FORMAT_RESERVED_42                        = 0x0000002a,
    IMG_DATA_FORMAT_RESERVED_43                        = 0x0000002b,
    IMG_DATA_FORMAT_RESERVED_44                        = 0x0000002c,
    IMG_DATA_FORMAT_RESERVED_45                        = 0x0000002d,
    IMG_DATA_FORMAT_RESERVED_46                        = 0x0000002e,
    IMG_DATA_FORMAT_YCBCR                              = 0x0000002f,
    IMG_DATA_FORMAT_LOD_5P3_USCALED                    = 0x00000030,
    IMG_DATA_FORMAT_MM_8                               = 0x00000031,
    IMG_DATA_FORMAT_MM_8_8                             = 0x00000032,
    IMG_DATA_FORMAT_MM_8_8_8_8                         = 0x00000033,
    IMG_DATA_FORMAT_MM_VYUY8                           = 0x00000034,
    IMG_DATA_FORMAT_MM_10_11_11                        = 0x00000035,
    IMG_DATA_FORMAT_MM_2_10_10_10                      = 0x00000036,
    IMG_DATA_FORMAT_MM_10_IN_16                        = 0x00000037,
    IMG_DATA_FORMAT_MM_10_IN_16_16                     = 0x00000038,
    IMG_DATA_FORMAT_MM_10_IN_16_16_16_16               = 0x00000039,
    IMG_DATA_FORMAT_MM_12_IN_16                        = 0x0000003a,
    IMG_DATA_FORMAT_MM_12_IN_16_16                     = 0x0000003b,
    IMG_DATA_FORMAT_MM_12_IN_16_16_16_16               = 0x0000003c,
    IMG_DATA_FORMAT_MM_16_16_16_16                     = 0x0000003d,
    IMG_DATA_FORMAT_RESERVED_62                        = 0x0000003e,
    IMG_DATA_FORMAT_RESERVED_63                        = 0x0000003f,
} IMG_DATA_FORMAT;

typedef enum IMG_FMT {
    IMG_FMT_INVALID                                    = 0x00000000,
    IMG_FMT_8_UNORM                                    = 0x00000001,
    IMG_FMT_8_SNORM                                    = 0x00000002,
    IMG_FMT_8_USCALED                                  = 0x00000003,
    IMG_FMT_8_SSCALED                                  = 0x00000004,
    IMG_FMT_8_UINT                                     = 0x00000005,
    IMG_FMT_8_SINT                                     = 0x00000006,
    IMG_FMT_16_UNORM                                   = 0x00000007,
    IMG_FMT_16_SNORM                                   = 0x00000008,
    IMG_FMT_16_USCALED                                 = 0x00000009,
    IMG_FMT_16_SSCALED                                 = 0x0000000a,
    IMG_FMT_16_UINT                                    = 0x0000000b,
    IMG_FMT_16_SINT                                    = 0x0000000c,
    IMG_FMT_16_FLOAT                                   = 0x0000000d,
    IMG_FMT_8_8_UNORM                                  = 0x0000000e,
    IMG_FMT_8_8_SNORM                                  = 0x0000000f,
    IMG_FMT_8_8_USCALED                                = 0x00000010,
    IMG_FMT_8_8_SSCALED                                = 0x00000011,
    IMG_FMT_8_8_UINT                                   = 0x00000012,
    IMG_FMT_8_8_SINT                                   = 0x00000013,
    IMG_FMT_32_UINT                                    = 0x00000014,
    IMG_FMT_32_SINT                                    = 0x00000015,
    IMG_FMT_32_FLOAT                                   = 0x00000016,
    IMG_FMT_16_16_UNORM                                = 0x00000017,
    IMG_FMT_16_16_SNORM                                = 0x00000018,
    IMG_FMT_16_16_USCALED                              = 0x00000019,
    IMG_FMT_16_16_SSCALED                              = 0x0000001a,
    IMG_FMT_16_16_UINT                                 = 0x0000001b,
    IMG_FMT_16_16_SINT                                 = 0x0000001c,
    IMG_FMT_16_16_FLOAT                                = 0x0000001d,
    IMG_FMT_10_11_11_FLOAT                             = 0x0000001e,
    IMG_FMT_11_11_10_FLOAT                             = 0x0000001f,
    IMG_FMT_10_10_10_2_UNORM                           = 0x00000020,
    IMG_FMT_10_10_10_2_SNORM                           = 0x00000021,
    IMG_FMT_10_10_10_2_UINT                            = 0x00000022,
    IMG_FMT_10_10_10_2_SINT                            = 0x00000023,
    IMG_FMT_2_10_10_10_UNORM                           = 0x00000024,
    IMG_FMT_2_10_10_10_SNORM                           = 0x00000025,
    IMG_FMT_2_10_10_10_USCALED                         = 0x00000026,
    IMG_FMT_2_10_10_10_SSCALED                         = 0x00000027,
    IMG_FMT_2_10_10_10_UINT                            = 0x00000028,
    IMG_FMT_2_10_10_10_SINT                            = 0x00000029,
    IMG_FMT_8_8_8_8_UNORM                              = 0x0000002a,
    IMG_FMT_8_8_8_8_SNORM                              = 0x0000002b,
    IMG_FMT_8_8_8_8_USCALED                            = 0x0000002c,
    IMG_FMT_8_8_8_8_SSCALED                            = 0x0000002d,
    IMG_FMT_8_8_8_8_UINT                               = 0x0000002e,
    IMG_FMT_8_8_8_8_SINT                               = 0x0000002f,
    IMG_FMT_32_32_UINT                                 = 0x00000030,
    IMG_FMT_32_32_SINT                                 = 0x00000031,
    IMG_FMT_32_32_FLOAT                                = 0x00000032,
    IMG_FMT_16_16_16_16_UNORM                          = 0x00000033,
    IMG_FMT_16_16_16_16_SNORM                          = 0x00000034,
    IMG_FMT_16_16_16_16_USCALED                        = 0x00000035,
    IMG_FMT_16_16_16_16_SSCALED                        = 0x00000036,
    IMG_FMT_16_16_16_16_UINT                           = 0x00000037,
    IMG_FMT_16_16_16_16_SINT                           = 0x00000038,
    IMG_FMT_16_16_16_16_FLOAT                          = 0x00000039,
    IMG_FMT_32_32_32_UINT                              = 0x0000003a,
    IMG_FMT_32_32_32_SINT                              = 0x0000003b,
    IMG_FMT_32_32_32_FLOAT                             = 0x0000003c,
    IMG_FMT_32_32_32_32_UINT                           = 0x0000003d,
    IMG_FMT_32_32_32_32_SINT                           = 0x0000003e,
    IMG_FMT_32_32_32_32_FLOAT                          = 0x0000003f,
    IMG_FMT_8_SRGB                                     = 0x00000040,
    IMG_FMT_8_8_SRGB                                   = 0x00000041,
    IMG_FMT_8_8_8_8_SRGB                               = 0x00000042,
    IMG_FMT_5_9_9_9_FLOAT                              = 0x00000043,
    IMG_FMT_5_6_5_UNORM                                = 0x00000044,
    IMG_FMT_1_5_5_5_UNORM                              = 0x00000045,
    IMG_FMT_5_5_5_1_UNORM                              = 0x00000046,
    IMG_FMT_4_4_4_4_UNORM                              = 0x00000047,
    IMG_FMT_4_4_UNORM                                  = 0x00000048,
    IMG_FMT_1_UNORM                                    = 0x00000049,
    IMG_FMT_1_REVERSED_UNORM                           = 0x0000004a,
    IMG_FMT_32_FLOAT_CLAMP                             = 0x0000004b,
    IMG_FMT_8_24_UNORM                                 = 0x0000004c,
    IMG_FMT_8_24_UINT                                  = 0x0000004d,
    IMG_FMT_24_8_UNORM                                 = 0x0000004e,
    IMG_FMT_24_8_UINT                                  = 0x0000004f,
    IMG_FMT_X24_8_32_UINT                              = 0x00000050,
    IMG_FMT_X24_8_32_FLOAT                             = 0x00000051,
    IMG_FMT_GB_GR_UNORM                                = 0x00000052,
    IMG_FMT_GB_GR_SNORM                                = 0x00000053,
    IMG_FMT_GB_GR_UINT                                 = 0x00000054,
    IMG_FMT_GB_GR_SRGB                                 = 0x00000055,
    IMG_FMT_BG_RG_UNORM                                = 0x00000056,
    IMG_FMT_BG_RG_SNORM                                = 0x00000057,
    IMG_FMT_BG_RG_UINT                                 = 0x00000058,
    IMG_FMT_BG_RG_SRGB                                 = 0x00000059,
    IMG_FMT_MM_10_IN_16_UNORM                          = 0x0000005a,
    IMG_FMT_MM_10_IN_16_UINT                           = 0x0000005b,
    IMG_FMT_MM_10_IN_16_16_UNORM                       = 0x0000005c,
    IMG_FMT_MM_10_IN_16_16_UINT                        = 0x0000005d,
    IMG_FMT_MM_10_IN_16_16_16_16_UNORM                 = 0x0000005e,
    IMG_FMT_MM_10_IN_16_16_16_16_UINT                  = 0x0000005f,
    IMG_FMT_RESERVED_96                                = 0x00000060,
    IMG_FMT_RESERVED_97                                = 0x00000061,
    IMG_FMT_RESERVED_98                                = 0x00000062,
    IMG_FMT_RESERVED_99                                = 0x00000063,
    IMG_FMT_RESERVED_100                               = 0x00000064,
    IMG_FMT_RESERVED_101                               = 0x00000065,
    IMG_FMT_RESERVED_102                               = 0x00000066,
    IMG_FMT_RESERVED_103                               = 0x00000067,
    IMG_FMT_RESERVED_104                               = 0x00000068,
    IMG_FMT_RESERVED_105                               = 0x00000069,
    IMG_FMT_RESERVED_106                               = 0x0000006a,
    IMG_FMT_RESERVED_107                               = 0x0000006b,
    IMG_FMT_RESERVED_108                               = 0x0000006c,
    IMG_FMT_BC1_UNORM                                  = 0x0000006d,
    IMG_FMT_BC1_SRGB                                   = 0x0000006e,
    IMG_FMT_BC2_UNORM                                  = 0x0000006f,
    IMG_FMT_BC2_SRGB                                   = 0x00000070,
    IMG_FMT_BC3_UNORM                                  = 0x00000071,
    IMG_FMT_BC3_SRGB                                   = 0x00000072,
    IMG_FMT_BC4_UNORM                                  = 0x00000073,
    IMG_FMT_BC4_SNORM                                  = 0x00000074,
    IMG_FMT_BC5_UNORM                                  = 0x00000075,
    IMG_FMT_BC5_SNORM                                  = 0x00000076,
    IMG_FMT_BC6_UFLOAT                                 = 0x00000077,
    IMG_FMT_BC6_SFLOAT                                 = 0x00000078,
    IMG_FMT_BC7_UNORM                                  = 0x00000079,
    IMG_FMT_BC7_SRGB                                   = 0x0000007a,
    IMG_FMT_RESERVED_123                               = 0x0000007b,
    IMG_FMT_RESERVED_124                               = 0x0000007c,
    IMG_FMT_RESERVED_125                               = 0x0000007d,
    IMG_FMT_RESERVED_126                               = 0x0000007e,
    IMG_FMT_RESERVED_127                               = 0x0000007f,
    IMG_FMT_RESERVED_128                               = 0x00000080,
    IMG_FMT_RESERVED_129                               = 0x00000081,
    IMG_FMT_RESERVED_130                               = 0x00000082,
    IMG_FMT_RESERVED_131                               = 0x00000083,
    IMG_FMT_RESERVED_132                               = 0x00000084,
    IMG_FMT_RESERVED_133                               = 0x00000085,
    IMG_FMT_RESERVED_134                               = 0x00000086,
    IMG_FMT_RESERVED_135                               = 0x00000087,
    IMG_FMT_RESERVED_136                               = 0x00000088,
    IMG_FMT_RESERVED_137                               = 0x00000089,
    IMG_FMT_RESERVED_138                               = 0x0000008a,
    IMG_FMT_RESERVED_139                               = 0x0000008b,
    IMG_FMT_RESERVED_140                               = 0x0000008c,
    IMG_FMT_RESERVED_141                               = 0x0000008d,
    IMG_FMT_RESERVED_142                               = 0x0000008e,
    IMG_FMT_RESERVED_143                               = 0x0000008f,
    IMG_FMT_RESERVED_144                               = 0x00000090,
    IMG_FMT_RESERVED_145                               = 0x00000091,
    IMG_FMT_RESERVED_146                               = 0x00000092,
    IMG_FMT_RESERVED_147                               = 0x00000093,
    IMG_FMT_RESERVED_148                               = 0x00000094,
    IMG_FMT_RESERVED_149                               = 0x00000095,
    IMG_FMT_RESERVED_150                               = 0x00000096,
    IMG_FMT_RESERVED_151                               = 0x00000097,
    IMG_FMT_RESERVED_152                               = 0x00000098,
    IMG_FMT_RESERVED_153                               = 0x00000099,
    IMG_FMT_RESERVED_154                               = 0x0000009a,
    IMG_FMT_RESERVED_155                               = 0x0000009b,
    IMG_FMT_RESERVED_156                               = 0x0000009c,
    IMG_FMT_RESERVED_157                               = 0x0000009d,
    IMG_FMT_RESERVED_158                               = 0x0000009e,
    IMG_FMT_RESERVED_159                               = 0x0000009f,
    IMG_FMT_RESERVED_160                               = 0x000000a0,
    IMG_FMT_RESERVED_161                               = 0x000000a1,
    IMG_FMT_RESERVED_162                               = 0x000000a2,
    IMG_FMT_RESERVED_163                               = 0x000000a3,
    IMG_FMT_RESERVED_164                               = 0x000000a4,
    IMG_FMT_RESERVED_165                               = 0x000000a5,
    IMG_FMT_RESERVED_166                               = 0x000000a6,
    IMG_FMT_RESERVED_167                               = 0x000000a7,
    IMG_FMT_RESERVED_168                               = 0x000000a8,
    IMG_FMT_RESERVED_169                               = 0x000000a9,
    IMG_FMT_RESERVED_170                               = 0x000000aa,
    IMG_FMT_RESERVED_171                               = 0x000000ab,
    IMG_FMT_RESERVED_172                               = 0x000000ac,
    IMG_FMT_RESERVED_173                               = 0x000000ad,
    IMG_FMT_RESERVED_174                               = 0x000000ae,
    IMG_FMT_RESERVED_175                               = 0x000000af,
    IMG_FMT_RESERVED_176                               = 0x000000b0,
    IMG_FMT_RESERVED_177                               = 0x000000b1,
    IMG_FMT_RESERVED_178                               = 0x000000b2,
    IMG_FMT_RESERVED_179                               = 0x000000b3,
    IMG_FMT_RESERVED_180                               = 0x000000b4,
    IMG_FMT_RESERVED_181                               = 0x000000b5,
    IMG_FMT_RESERVED_182                               = 0x000000b6,
    IMG_FMT_RESERVED_183                               = 0x000000b7,
    IMG_FMT_RESERVED_184                               = 0x000000b8,
    IMG_FMT_RESERVED_185                               = 0x000000b9,
    IMG_FMT_RESERVED_186                               = 0x000000ba,
    IMG_FMT_RESERVED_187                               = 0x000000bb,
    IMG_FMT_RESERVED_188                               = 0x000000bc,
    IMG_FMT_RESERVED_189                               = 0x000000bd,
    IMG_FMT_RESERVED_190                               = 0x000000be,
    IMG_FMT_RESERVED_191                               = 0x000000bf,
    IMG_FMT_RESERVED_192                               = 0x000000c0,
    IMG_FMT_RESERVED_193                               = 0x000000c1,
    IMG_FMT_RESERVED_194                               = 0x000000c2,
    IMG_FMT_RESERVED_195                               = 0x000000c3,
    IMG_FMT_RESERVED_196                               = 0x000000c4,
    IMG_FMT_RESERVED_197                               = 0x000000c5,
    IMG_FMT_RESERVED_198                               = 0x000000c6,
    IMG_FMT_RESERVED_199                               = 0x000000c7,
    IMG_FMT_RESERVED_200                               = 0x000000c8,
    IMG_FMT_RESERVED_201                               = 0x000000c9,
    IMG_FMT_RESERVED_202                               = 0x000000ca,
    IMG_FMT_RESERVED_203                               = 0x000000cb,
    IMG_FMT_RESERVED_204                               = 0x000000cc,
    IMG_FMT_YCBCR_UNORM                                = 0x000000cd,
    IMG_FMT_YCBCR_SRGB                                 = 0x000000ce,
    IMG_FMT_MM_8_UNORM                                 = 0x000000cf,
    IMG_FMT_MM_8_UINT                                  = 0x000000d0,
    IMG_FMT_MM_8_8_UNORM                               = 0x000000d1,
    IMG_FMT_MM_8_8_UINT                                = 0x000000d2,
    IMG_FMT_MM_8_8_8_8_UNORM                           = 0x000000d3,
    IMG_FMT_MM_8_8_8_8_UINT                            = 0x000000d4,
    IMG_FMT_MM_VYUY8_UNORM                             = 0x000000d5,
    IMG_FMT_MM_VYUY8_UINT                              = 0x000000d6,
    IMG_FMT_MM_10_11_11_UNORM                          = 0x000000d7,
    IMG_FMT_MM_10_11_11_UINT                           = 0x000000d8,
    IMG_FMT_MM_2_10_10_10_UNORM                        = 0x000000d9,
    IMG_FMT_MM_2_10_10_10_UINT                         = 0x000000da,
    IMG_FMT_MM_16_16_16_16_UNORM                       = 0x000000db,
    IMG_FMT_MM_16_16_16_16_UINT                        = 0x000000dc,
    IMG_FMT_MM_12_IN_16_UNORM                          = 0x000000dd,
    IMG_FMT_MM_12_IN_16_UINT                           = 0x000000de,
    IMG_FMT_MM_12_IN_16_16_UNORM                       = 0x000000df,
    IMG_FMT_MM_12_IN_16_16_UINT                        = 0x000000e0,
    IMG_FMT_MM_12_IN_16_16_16_16_UNORM                 = 0x000000e1,
    IMG_FMT_MM_12_IN_16_16_16_16_UINT                  = 0x000000e2,
    IMG_FMT_6E4_FLOAT                                  = 0x000000e3,
    IMG_FMT_7E3_FLOAT                                  = 0x000000e4,
    IMG_FMT_RESERVED_229                               = 0x000000e5,
    IMG_FMT_RESERVED_230                               = 0x000000e6,
    IMG_FMT_RESERVED_231                               = 0x000000e7,
    IMG_FMT_RESERVED_232                               = 0x000000e8,
    IMG_FMT_RESERVED_233                               = 0x000000e9,
    IMG_FMT_RESERVED_234                               = 0x000000ea,
    IMG_FMT_RESERVED_235                               = 0x000000eb,
    IMG_FMT_RESERVED_236                               = 0x000000ec,
    IMG_FMT_RESERVED_237                               = 0x000000ed,
    IMG_FMT_RESERVED_238                               = 0x000000ee,
    IMG_FMT_RESERVED_239                               = 0x000000ef,
    IMG_FMT_RESERVED_240                               = 0x000000f0,
    IMG_FMT_RESERVED_241                               = 0x000000f1,
    IMG_FMT_RESERVED_242                               = 0x000000f2,
    IMG_FMT_RESERVED_243                               = 0x000000f3,
    IMG_FMT_RESERVED_244                               = 0x000000f4,
    IMG_FMT_RESERVED_245                               = 0x000000f5,
    IMG_FMT_RESERVED_246                               = 0x000000f6,
    IMG_FMT_RESERVED_247                               = 0x000000f7,
    IMG_FMT_RESERVED_248                               = 0x000000f8,
    IMG_FMT_RESERVED_249                               = 0x000000f9,
    IMG_FMT_RESERVED_250                               = 0x000000fa,
    IMG_FMT_RESERVED_251                               = 0x000000fb,
    IMG_FMT_RESERVED_252                               = 0x000000fc,
    IMG_FMT_RESERVED_253                               = 0x000000fd,
    IMG_FMT_RESERVED_254                               = 0x000000fe,
    IMG_FMT_RESERVED_255                               = 0x000000ff,
} IMG_FMT;

typedef enum IMG_NUM_FORMAT {
    IMG_NUM_FORMAT_UNORM                               = 0x00000000,
    IMG_NUM_FORMAT_SNORM                               = 0x00000001,
    IMG_NUM_FORMAT_USCALED                             = 0x00000002,
    IMG_NUM_FORMAT_SSCALED                             = 0x00000003,
    IMG_NUM_FORMAT_UINT                                = 0x00000004,
    IMG_NUM_FORMAT_SINT                                = 0x00000005,
    IMG_NUM_FORMAT_SRGB                                = 0x00000006,
    IMG_NUM_FORMAT_FLOAT                               = 0x00000007,
} IMG_NUM_FORMAT;

typedef enum IMG_NUM_FORMAT_FMASK {
    IMG_NUM_FORMAT_FMASK_8_2_1                         = 0x00000000,
    IMG_NUM_FORMAT_FMASK_8_4_1                         = 0x00000001,
    IMG_NUM_FORMAT_FMASK_8_8_1                         = 0x00000002,
    IMG_NUM_FORMAT_FMASK_8_2_2                         = 0x00000003,
    IMG_NUM_FORMAT_FMASK_8_4_2                         = 0x00000004,
    IMG_NUM_FORMAT_FMASK_8_4_4                         = 0x00000005,
    IMG_NUM_FORMAT_FMASK_16_16_1                       = 0x00000006,
    IMG_NUM_FORMAT_FMASK_16_8_2                        = 0x00000007,
    IMG_NUM_FORMAT_FMASK_32_16_2                       = 0x00000008,
    IMG_NUM_FORMAT_FMASK_32_8_4                        = 0x00000009,
    IMG_NUM_FORMAT_FMASK_32_8_8                        = 0x0000000a,
    IMG_NUM_FORMAT_FMASK_64_16_4                       = 0x0000000b,
    IMG_NUM_FORMAT_FMASK_64_16_8                       = 0x0000000c,
    IMG_NUM_FORMAT_FMASK_RESERVED_13                   = 0x0000000d,
    IMG_NUM_FORMAT_FMASK_RESERVED_14                   = 0x0000000e,
    IMG_NUM_FORMAT_FMASK_RESERVED_15                   = 0x0000000f,
} IMG_NUM_FORMAT_FMASK;

typedef enum IMG_NUM_FORMAT_N_IN_16 {
    IMG_NUM_FORMAT_N_IN_16_RESERVED_0                  = 0x00000000,
    IMG_NUM_FORMAT_N_IN_16_UNORM_10                    = 0x00000001,
    IMG_NUM_FORMAT_N_IN_16_UNORM_9                     = 0x00000002,
    IMG_NUM_FORMAT_N_IN_16_RESERVED_3                  = 0x00000003,
    IMG_NUM_FORMAT_N_IN_16_UINT_10                     = 0x00000004,
    IMG_NUM_FORMAT_N_IN_16_UINT_9                      = 0x00000005,
    IMG_NUM_FORMAT_N_IN_16_RESERVED_6                  = 0x00000006,
    IMG_NUM_FORMAT_N_IN_16_UNORM_UINT_10               = 0x00000007,
    IMG_NUM_FORMAT_N_IN_16_UNORM_UINT_9                = 0x00000008,
    IMG_NUM_FORMAT_N_IN_16_RESERVED_9                  = 0x00000009,
    IMG_NUM_FORMAT_N_IN_16_RESERVED_10                 = 0x0000000a,
    IMG_NUM_FORMAT_N_IN_16_RESERVED_11                 = 0x0000000b,
    IMG_NUM_FORMAT_N_IN_16_RESERVED_12                 = 0x0000000c,
    IMG_NUM_FORMAT_N_IN_16_RESERVED_13                 = 0x0000000d,
    IMG_NUM_FORMAT_N_IN_16_RESERVED_14                 = 0x0000000e,
    IMG_NUM_FORMAT_N_IN_16_RESERVED_15                 = 0x0000000f,
} IMG_NUM_FORMAT_N_IN_16;

typedef enum MTYPE {
    MTYPE_C_RW_US                                      = 0x00000000,
    MTYPE_RESERVED_1                                   = 0x00000001,
    MTYPE_C_RO_S                                       = 0x00000002,
    MTYPE_UC                                           = 0x00000003,
    MTYPE_C_RW_S                                       = 0x00000004,
    MTYPE_RESERVED_5                                   = 0x00000005,
    MTYPE_C_RO_US                                      = 0x00000006,
    MTYPE_RESERVED_7                                   = 0x00000007,
} MTYPE;

typedef enum OreoMode {
    OMODE_BLEND                                        = 0x00000000,
    OMODE_O_THEN_B                                     = 0x00000001,
    OMODE_P_THEN_O_THEN_B                              = 0x00000002,
    OMODE_RESERVED_3                                   = 0x00000003,
} OreoMode;

typedef enum PC_PERFCNT_SEL {
    PC_PERF_SC_PC_PTR_SEND0                            = 0x00000000,
    PC_PERF_SC_PC_PTR_VALID0                           = 0x00000001,
    PC_PERF_SC_FPOSG0                                  = 0x00000002,
    PC_PERF_PKR0_FPOSG_EQ1                             = 0x00000003,
    PC_PERF_PKR0_FPOSG_GT1                             = 0x00000004,
    PC_PERF_PKR0_FPOSG_GT16                            = 0x00000005,
    PC_PERF_PKR0_FPOSG_GT64                            = 0x00000006,
    PC_PERF_PKR0_FPOSG_GT128                           = 0x00000007,
    PC_PERF_PKR0_FPOSG_OUT_OF_WAVE                     = 0x00000008,
    PC_PERF_SC_FPOSG_WAIT0                             = 0x00000009,
    PC_PERF_SC_WAIT_SYNC0                              = 0x0000000a,
    PC_PERF_SC_PQ_FREEZE0                              = 0x0000000b,
    PC_PERF_PKR0_NUM_PROBES                            = 0x0000000c,
    PC_PERF_PKR0_PRIMS_PER_PROBE_EQ1                   = 0x0000000d,
    PC_PERF_PKR0_PRIMS_PER_PROBE_GT1                   = 0x0000000e,
    PC_PERF_PKR0_PRIMS_PER_PROBE_GT2                   = 0x0000000f,
    PC_PERF_PKR0_PRIMS_PER_PROBE_GT4                   = 0x00000010,
    PC_PERF_PKR0_PRIMS_PER_PROBE_GT8                   = 0x00000011,
    PC_PERF_PKR0_NUM_WAVES                             = 0x00000012,
    PC_PERF_PKR0_PRIMS_PER_WAVE_EQ1                    = 0x00000013,
    PC_PERF_PKR0_PRIMS_PER_WAVE_GT1                    = 0x00000014,
    PC_PERF_PKR0_PRIMS_PER_WAVE_GT2                    = 0x00000015,
    PC_PERF_PKR0_PRIMS_PER_WAVE_GT4                    = 0x00000016,
    PC_PERF_PKR0_PRIMS_PER_WAVE_GT8                    = 0x00000017,
    PC_PERF_PKR0_PROBES_PER_WAVE_EQ1                   = 0x00000018,
    PC_PERF_PKR0_PROBES_PER_WAVE_GT1                   = 0x00000019,
    PC_PERF_PKR0_PROBES_PER_WAVE_GT2                   = 0x0000001a,
    PC_PERF_PKR0_PROBES_PER_WAVE_GT4                   = 0x0000001b,
    PC_PERF_PKR0_PROBES_PER_WAVE_GT8                   = 0x0000001c,
    PC_PERF_PKR0_PRIMS_PER_WAVE_REUSE                  = 0x0000001d,
    PC_PERF_SC_PC_PTR_SEND1                            = 0x0000001e,
    PC_PERF_SC_PC_PTR_VALID1                           = 0x0000001f,
    PC_PERF_SC_FPOSG1                                  = 0x00000020,
    PC_PERF_PKR1_FPOSG_EQ1                             = 0x00000021,
    PC_PERF_PKR1_FPOSG_GT1                             = 0x00000022,
    PC_PERF_PKR1_FPOSG_GT16                            = 0x00000023,
    PC_PERF_PKR1_FPOSG_GT64                            = 0x00000024,
    PC_PERF_PKR1_FPOSG_GT128                           = 0x00000025,
    PC_PERF_PKR1_FPOSG_OUT_OF_WAVE                     = 0x00000026,
    PC_PERF_SC_FPOSG_WAIT1                             = 0x00000027,
    PC_PERF_SC_WAIT_SYNC1                              = 0x00000028,
    PC_PERF_SC_PQ_FREEZE1                              = 0x00000029,
    PC_PERF_PKR1_NUM_PROBES                            = 0x0000002a,
    PC_PERF_PKR1_PRIMS_PER_PROBE_EQ1                   = 0x0000002b,
    PC_PERF_PKR1_PRIMS_PER_PROBE_GT1                   = 0x0000002c,
    PC_PERF_PKR1_PRIMS_PER_PROBE_GT2                   = 0x0000002d,
    PC_PERF_PKR1_PRIMS_PER_PROBE_GT4                   = 0x0000002e,
    PC_PERF_PKR1_PRIMS_PER_PROBE_GT8                   = 0x0000002f,
    PC_PERF_PKR1_NUM_WAVES                             = 0x00000030,
    PC_PERF_PKR1_PRIMS_PER_WAVE_EQ1                    = 0x00000031,
    PC_PERF_PKR1_PRIMS_PER_WAVE_GT1                    = 0x00000032,
    PC_PERF_PKR1_PRIMS_PER_WAVE_GT2                    = 0x00000033,
    PC_PERF_PKR1_PRIMS_PER_WAVE_GT4                    = 0x00000034,
    PC_PERF_PKR1_PRIMS_PER_WAVE_GT8                    = 0x00000035,
    PC_PERF_PKR1_PROBES_PER_WAVE_EQ1                   = 0x00000036,
    PC_PERF_PKR1_PROBES_PER_WAVE_GT1                   = 0x00000037,
    PC_PERF_PKR1_PROBES_PER_WAVE_GT2                   = 0x00000038,
    PC_PERF_PKR1_PROBES_PER_WAVE_GT4                   = 0x00000039,
    PC_PERF_PKR1_PROBES_PER_WAVE_GT8                   = 0x0000003a,
    PC_PERF_PKR1_PRIMS_PER_WAVE_REUSE                  = 0x0000003b,
    PC_PERF_SC_PC_PTR_SEND2                            = 0x0000003c,
    PC_PERF_SC_PC_PTR_VALID2                           = 0x0000003d,
    PC_PERF_SC_FPOSG2                                  = 0x0000003e,
    PC_PERF_PKR2_FPOSG_EQ1                             = 0x0000003f,
    PC_PERF_PKR2_FPOSG_GT1                             = 0x00000040,
    PC_PERF_PKR2_FPOSG_GT16                            = 0x00000041,
    PC_PERF_PKR2_FPOSG_GT64                            = 0x00000042,
    PC_PERF_PKR2_FPOSG_GT128                           = 0x00000043,
    PC_PERF_PKR2_FPOSG_OUT_OF_WAVE                     = 0x00000044,
    PC_PERF_SC_FPOSG_WAIT2                             = 0x00000045,
    PC_PERF_SC_WAIT_SYNC2                              = 0x00000046,
    PC_PERF_SC_PQ_FREEZE2                              = 0x00000047,
    PC_PERF_PKR2_NUM_PROBES                            = 0x00000048,
    PC_PERF_PKR2_PRIMS_PER_PROBE_EQ1                   = 0x00000049,
    PC_PERF_PKR2_PRIMS_PER_PROBE_GT1                   = 0x0000004a,
    PC_PERF_PKR2_PRIMS_PER_PROBE_GT2                   = 0x0000004b,
    PC_PERF_PKR2_PRIMS_PER_PROBE_GT4                   = 0x0000004c,
    PC_PERF_PKR2_PRIMS_PER_PROBE_GT8                   = 0x0000004d,
    PC_PERF_PKR2_NUM_WAVES                             = 0x0000004e,
    PC_PERF_PKR2_PRIMS_PER_WAVE_EQ1                    = 0x0000004f,
    PC_PERF_PKR2_PRIMS_PER_WAVE_GT1                    = 0x00000050,
    PC_PERF_PKR2_PRIMS_PER_WAVE_GT2                    = 0x00000051,
    PC_PERF_PKR2_PRIMS_PER_WAVE_GT4                    = 0x00000052,
    PC_PERF_PKR2_PRIMS_PER_WAVE_GT8                    = 0x00000053,
    PC_PERF_PKR2_PROBES_PER_WAVE_EQ1                   = 0x00000054,
    PC_PERF_PKR2_PROBES_PER_WAVE_GT1                   = 0x00000055,
    PC_PERF_PKR2_PROBES_PER_WAVE_GT2                   = 0x00000056,
    PC_PERF_PKR2_PROBES_PER_WAVE_GT4                   = 0x00000057,
    PC_PERF_PKR2_PROBES_PER_WAVE_GT8                   = 0x00000058,
    PC_PERF_PKR2_PRIMS_PER_WAVE_REUSE                  = 0x00000059,
    PC_PERF_SC_PC_PTR_SEND3                            = 0x0000005a,
    PC_PERF_SC_PC_PTR_VALID3                           = 0x0000005b,
    PC_PERF_SC_FPOSG3                                  = 0x0000005c,
    PC_PERF_PKR3_FPOSG_EQ1                             = 0x0000005d,
    PC_PERF_PKR3_FPOSG_GT1                             = 0x0000005e,
    PC_PERF_PKR3_FPOSG_GT16                            = 0x0000005f,
    PC_PERF_PKR3_FPOSG_GT64                            = 0x00000060,
    PC_PERF_PKR3_FPOSG_GT128                           = 0x00000061,
    PC_PERF_PKR3_FPOSG_OUT_OF_WAVE                     = 0x00000062,
    PC_PERF_SC_FPOSG_WAIT3                             = 0x00000063,
    PC_PERF_SC_WAIT_SYNC3                              = 0x00000064,
    PC_PERF_SC_PQ_FREEZE3                              = 0x00000065,
    PC_PERF_PKR3_NUM_PROBES                            = 0x00000066,
    PC_PERF_PKR3_PRIMS_PER_PROBE_EQ1                   = 0x00000067,
    PC_PERF_PKR3_PRIMS_PER_PROBE_GT1                   = 0x00000068,
    PC_PERF_PKR3_PRIMS_PER_PROBE_GT2                   = 0x00000069,
    PC_PERF_PKR3_PRIMS_PER_PROBE_GT4                   = 0x0000006a,
    PC_PERF_PKR3_PRIMS_PER_PROBE_GT8                   = 0x0000006b,
    PC_PERF_PKR3_NUM_WAVES                             = 0x0000006c,
    PC_PERF_PKR3_PRIMS_PER_WAVE_EQ1                    = 0x0000006d,
    PC_PERF_PKR3_PRIMS_PER_WAVE_GT1                    = 0x0000006e,
    PC_PERF_PKR3_PRIMS_PER_WAVE_GT2                    = 0x0000006f,
    PC_PERF_PKR3_PRIMS_PER_WAVE_GT4                    = 0x00000070,
    PC_PERF_PKR3_PRIMS_PER_WAVE_GT8                    = 0x00000071,
    PC_PERF_PKR3_PROBES_PER_WAVE_EQ1                   = 0x00000072,
    PC_PERF_PKR3_PROBES_PER_WAVE_GT1                   = 0x00000073,
    PC_PERF_PKR3_PROBES_PER_WAVE_GT2                   = 0x00000074,
    PC_PERF_PKR3_PROBES_PER_WAVE_GT4                   = 0x00000075,
    PC_PERF_PKR3_PROBES_PER_WAVE_GT8                   = 0x00000076,
    PC_PERF_PKR3_PRIMS_PER_WAVE_REUSE                  = 0x00000077,
    PC_PERF_SC_MW_FREEZE                               = 0x00000078,
    PC_PERF_SC_NUM_PROBES                              = 0x00000079,
    PC_PERF_SC_NUM_WAVES                               = 0x0000007a,
    PC_PERF_SC_NUM_SPLIT_WAVES                         = 0x0000007b,
    PC_PERF_GE_GSDONE                                  = 0x0000007c,
    PC_PERF_PKR0_GSDONE_WHILE_IDLE                     = 0x0000007d,
    PC_PERF_PKR1_GSDONE_WHILE_IDLE                     = 0x0000007e,
    PC_PERF_PKR2_GSDONE_WHILE_IDLE                     = 0x0000007f,
    PC_PERF_PKR3_GSDONE_WHILE_IDLE                     = 0x00000080,
    PC_PERF_MW_PC_SPI_PROBE_FREEZE                     = 0x00000081,
    PC_PERF_MW_PC_SPI_PROBE_OUT_OF_CREDIT              = 0x00000082,
    PC_PERF_MW_RTN_ADDR_FREEZE                         = 0x00000083,
    PC_PERF_MW_PROBE_CNT_FREEZE                        = 0x00000084,
    PC_PERF_MW_GL1H_REQ_FREEZE                         = 0x00000085,
    PC_PERF_MW_GL1H_NUM_REQS                           = 0x00000086,
    PC_PERF_MW_DLINE_ALLOC                             = 0x00000087,
    PC_PERF_MW_DLINE_DEALLOC                           = 0x00000088,
    PC_PERF_MW_TAGLINE_ALLOC                           = 0x00000089,
    PC_PERF_MW_TAGLINE_DEALLOC                         = 0x0000008a,
    PC_PERF_MW_PHY_DLINE_FULL_STALL                    = 0x0000008b,
    PC_PERF_MW_CACHE_CNTL_FULL_STALL                   = 0x0000008c,
    PC_PERF_MW_STAMP_LIMIT_STALL                       = 0x0000008d,
    PC_PERF_MW_CACHE_MISS                              = 0x0000008e,
    PC_PERF_MW_CACHE_HIT                               = 0x0000008f,
    PC_PERF_MW_CACHE_REUSE                             = 0x00000090,
    PC_PERF_MW_DEALLOC_HIT                             = 0x00000091,
    PC_PERF_PC_MEM_BANK_CONF0                          = 0x00000092,
    PC_PERF_PC_MEM_BANK_CONF1                          = 0x00000093,
    PC_PERF_PC_LDS_VERTEX_REUSE0                       = 0x00000094,
    PC_PERF_PC_LDS_CNTL_VALID0                         = 0x00000095,
    PC_PERF_PC_LDS_VERTEX_REUSE1                       = 0x00000096,
    PC_PERF_PC_LDS_CNTL_VALID1                         = 0x00000097,
    PC_PERF_GRBM_BUSY                                  = 0x00000098,
    PC_PERF_GL1_RTN_CNT_GTE1                           = 0x00000099,
    PC_PERF_GL1_RTN_CNT_GT512                          = 0x0000009a,
    PC_PERF_GL1_RTN_CNT_GT768                          = 0x0000009b,
    PC_PERF_LWC0_PROBE_ORDER_STALL                     = 0x0000009c,
    PC_PERF_LWC0_PC_MEM_READ_STALL                     = 0x0000009d,
    PC_PERF_LWC0_PKR2_SA_BDRY_CROSSING                 = 0x0000009e,
    PC_PERF_LWC0_PKR3_SA_BDRY_CROSSING                 = 0x0000009f,
    PC_PERF_LWC1_PROBE_ORDER_STALL                     = 0x000000a0,
    PC_PERF_LWC1_PC_MEM_READ_STALL                     = 0x000000a1,
    PC_PERF_LWC1_PKR0_SA_BDRY_CROSSING                 = 0x000000a2,
    PC_PERF_LWC1_PKR1_SA_BDRY_CROSSING                 = 0x000000a3,
    PC_PERF_NUM_PSWAVE                                 = 0x000000a4,
} PC_PERFCNT_SEL;

constexpr unsigned int MaxPcPerfcntSel                 = PC_PERF_NUM_PSWAVE;

typedef enum PerfCounter_Vals {
    DB_PERF_SEL_DB_SC_quad_sends                       = 0x0000001d,
    DB_PERF_SEL_DB_SC_quad_busy                        = 0x0000001e,
    DB_PERF_SEL_DB_SC_quad_stalls                      = 0x0000001f,
    DB_PERF_SEL_DB_SC_quad_tiles                       = 0x00000020,
    DB_PERF_SEL_DB_SC_quad_lit_quad                    = 0x00000021,
    DB_PERF_SEL_DB_CB_export_events                    = 0x00000022,
    DB_PERF_SEL_SX_DB_quad_sends                       = 0x00000025,
    DB_PERF_SEL_SX_DB_quad_busy                        = 0x00000026,
    DB_PERF_SEL_SX_DB_quad_stalls                      = 0x00000027,
    DB_PERF_SEL_SX_DB_quad_quads                       = 0x00000028,
    DB_PERF_SEL_SX_DB_quad_pixels                      = 0x00000029,
    DB_PERF_SEL_SX_DB_quad_exports                     = 0x0000002a,
    DB_PERF_SEL_SH_quads_outstanding_sum               = 0x0000002b,
    DB_PERF_SEL_DB_CB_export_sends                     = 0x0000002c,
    DB_PERF_SEL_DB_CB_export_busy                      = 0x0000002d,
    DB_PERF_SEL_DB_CB_export_stalls                    = 0x0000002e,
    DB_PERF_SEL_DB_CB_export_quads                     = 0x0000002f,
    DB_PERF_SEL_quad_rd_sends                          = 0x00000032,
    DB_PERF_SEL_quad_rd_busy                           = 0x00000033,
    DB_PERF_SEL_quad_rd_mi_stall                       = 0x00000034,
    DB_PERF_SEL_quad_rd_panic                          = 0x00000038,
    DB_PERF_SEL_quad_rd_outstanding_sum                = 0x00000039,
    DB_PERF_SEL_quad_rdret_sends                       = 0x0000003a,
    DB_PERF_SEL_quad_rdret_busy                        = 0x0000003b,
    DB_PERF_SEL_quad_wr_busy                           = 0x00000040,
    DB_PERF_SEL_quad_wr_mi_stall                       = 0x00000041,
    DB_PERF_SEL_quad_wr_acks                           = 0x00000043,
    DB_PERF_SEL_mi_quad_wr_outstanding_sum             = 0x00000044,
    DB_PERF_SEL_Sample_Tile_Cache_sends                = 0x00000051,
    DB_PERF_SEL_Sample_Tile_Cache_busy                 = 0x00000052,
    DB_PERF_SEL_Sample_Tile_Cache_starves              = 0x00000053,
    DB_PERF_SEL_Sample_Tile_Cache_dtile_locked         = 0x00000054,
    DB_PERF_SEL_Sample_Tile_Cache_alloc_stall          = 0x00000055,
    DB_PERF_SEL_Sample_Tile_Cache_misses               = 0x00000056,
    DB_PERF_SEL_Sample_Tile_Cache_hits                 = 0x00000057,
    DB_PERF_SEL_Sample_Tile_Cache_noop_tile            = 0x00000059,
    DB_PERF_SEL_Sample_Tile_Cache_detailed_noop        = 0x0000005a,
    DB_PERF_SEL_Sample_Tile_Cache_event                = 0x0000005b,
    DB_PERF_SEL_Sample_Tile_Cache_mem_return_starve    = 0x0000005e,
    DB_PERF_SEL_Stencil_Cache_misses                   = 0x0000005f,
    DB_PERF_SEL_Stencil_Cache_hits                     = 0x00000060,
    DB_PERF_SEL_Stencil_Cache_starves                  = 0x00000062,
    DB_PERF_SEL_Z_Cache_separate_Z_misses              = 0x00000064,
    DB_PERF_SEL_Z_Cache_separate_Z_hits                = 0x00000065,
    DB_PERF_SEL_Z_Cache_separate_Z_starves             = 0x00000067,
    DB_PERF_SEL_Z_Cache_pmask_misses                   = 0x00000068,
    DB_PERF_SEL_Z_Cache_pmask_hits                     = 0x00000069,
    DB_PERF_SEL_Z_Cache_pmask_starves                  = 0x0000006b,
    DB_PERF_SEL_Plane_Cache_misses                     = 0x0000006d,
    DB_PERF_SEL_Plane_Cache_hits                       = 0x0000006e,
    DB_PERF_SEL_Plane_Cache_starves                    = 0x00000070,
    DB_PERF_SEL_earlyZ_waiting_for_postZ_done          = 0x00000087,
    DB_PERF_SEL_reZ_waiting_for_postZ_done             = 0x00000088,
    DB_PERF_SEL_dk_tile_sends                          = 0x00000089,
    DB_PERF_SEL_dk_tile_busy                           = 0x0000008a,
    DB_PERF_SEL_dk_tile_quad_starves                   = 0x0000008b,
    DB_PERF_SEL_dk_tile_stalls                         = 0x0000008c,
    DB_PERF_SEL_dk_squad_sends                         = 0x0000008d,
    DB_PERF_SEL_dk_squad_busy                          = 0x0000008e,
    DB_PERF_SEL_dk_squad_stalls                        = 0x0000008f,
    DB_PERF_SEL_Op_Pipe_Busy                           = 0x00000090,
    DB_PERF_SEL_Op_Pipe_MC_Read_stall                  = 0x00000091,
    DB_PERF_SEL_qc_busy                                = 0x00000092,
    DB_PERF_SEL_qc_xfc                                 = 0x00000093,
    DB_PERF_SEL_qc_conflicts                           = 0x00000094,
    DB_PERF_SEL_qc_full_stall                          = 0x00000095,
    DB_PERF_SEL_qc_in_preZ_tile_stalls_postZ           = 0x00000096,
    DB_PERF_SEL_qc_in_postZ_tile_stalls_preZ           = 0x00000097,
    DB_PERF_SEL_tsc_insert_summarize_stall             = 0x00000098,
    DB_PERF_SEL_tl_busy                                = 0x00000099,
    DB_PERF_SEL_tl_dtc_read_starved                    = 0x0000009a,
    DB_PERF_SEL_tl_z_fetch_stall                       = 0x0000009b,
    DB_PERF_SEL_tl_stencil_stall                       = 0x0000009c,
    DB_PERF_SEL_tl_z_decompress_stall                  = 0x0000009d,
    DB_PERF_SEL_tl_stencil_locked_stall                = 0x0000009e,
    DB_PERF_SEL_tl_events                              = 0x0000009f,
    DB_PERF_SEL_tl_summarize_squads                    = 0x000000a0,
    DB_PERF_SEL_tl_preZ_squads                         = 0x000000a3,
    DB_PERF_SEL_tl_postZ_squads                        = 0x000000a4,
    DB_PERF_SEL_tl_preZ_noop_squads                    = 0x000000a5,
    DB_PERF_SEL_tl_postZ_noop_squads                   = 0x000000a6,
    DB_PERF_SEL_tl_tile_ops                            = 0x000000a7,
    DB_PERF_SEL_tl_in_xfc                              = 0x000000a8,
    DB_PERF_SEL_tl_in_single_stencil_expand_stall      = 0x000000a9,
    DB_PERF_SEL_tl_in_fast_z_stall                     = 0x000000aa,
    DB_PERF_SEL_tl_out_xfc                             = 0x000000ab,
    DB_PERF_SEL_tl_out_squads                          = 0x000000ac,
    DB_PERF_SEL_PostZ_Samples_passing_Z                = 0x000000ae,
    DB_PERF_SEL_PostZ_Samples_failing_Z                = 0x000000af,
    DB_PERF_SEL_PostZ_Samples_failing_S                = 0x000000b0,
    DB_PERF_SEL_PreZ_Samples_passing_Z                 = 0x000000b1,
    DB_PERF_SEL_PreZ_Samples_failing_Z                 = 0x000000b2,
    DB_PERF_SEL_PreZ_Samples_failing_S                 = 0x000000b3,
    DB_PERF_SEL_ts_tc_update_stall                     = 0x000000b4,
    DB_PERF_SEL_sc_kick_start                          = 0x000000b5,
    DB_PERF_SEL_sc_kick_end                            = 0x000000b6,
    DB_PERF_SEL_clock_reg_active                       = 0x000000b7,
    DB_PERF_SEL_clock_main_active                      = 0x000000b8,
    DB_PERF_SEL_clock_mem_export_active                = 0x000000b9,
    DB_PERF_SEL_esr_ps_out_busy                        = 0x000000ba,
    DB_PERF_SEL_esr_ps_lqf_busy                        = 0x000000bb,
    DB_PERF_SEL_esr_ps_lqf_stall                       = 0x000000bc,
    DB_PERF_SEL_etr_out_send                           = 0x000000bd,
    DB_PERF_SEL_etr_out_busy                           = 0x000000be,
    DB_PERF_SEL_etr_out_ltile_probe_fifo_full_stall    = 0x000000bf,
    DB_PERF_SEL_etr_out_esr_stall                      = 0x000000c1,
    DB_PERF_SEL_esr_ps_vic_busy                        = 0x000000c2,
    DB_PERF_SEL_esr_ps_vic_stall                       = 0x000000c3,
    DB_PERF_SEL_esr_eot_fwd_busy                       = 0x000000c4,
    DB_PERF_SEL_esr_eot_fwd_holding_squad              = 0x000000c5,
    DB_PERF_SEL_esr_eot_fwd_forward                    = 0x000000c6,
    DB_PERF_SEL_esr_sqq_zi_busy                        = 0x000000c7,
    DB_PERF_SEL_esr_sqq_zi_stall                       = 0x000000c8,
    DB_PERF_SEL_postzl_sq_pt_busy                      = 0x000000c9,
    DB_PERF_SEL_postzl_sq_pt_stall                     = 0x000000ca,
    DB_PERF_SEL_postzl_se_busy                         = 0x000000cb,
    DB_PERF_SEL_postzl_se_stall                        = 0x000000cc,
    DB_PERF_SEL_postzl_partial_launch                  = 0x000000cd,
    DB_PERF_SEL_postzl_full_launch                     = 0x000000ce,
    DB_PERF_SEL_postzl_partial_waiting                 = 0x000000cf,
    DB_PERF_SEL_postzl_tile_mem_stall                  = 0x000000d0,
    DB_PERF_SEL_postzl_tile_init_stall                 = 0x000000d1,
    DB_PERF_SEL_prezl_tile_mem_stall                   = 0x000000d2,
    DB_PERF_SEL_dtt_sm_clash_stall                     = 0x000000d4,
    DB_PERF_SEL_dtt_sm_slot_stall                      = 0x000000d5,
    DB_PERF_SEL_dtt_sm_miss_stall                      = 0x000000d6,
    DB_PERF_SEL_mi_wrreq_busy                          = 0x000000d9,
    DB_PERF_SEL_mi_wrreq_stall                         = 0x000000da,
    DB_PERF_SEL_recomp_tile_to_1zplane_no_fastop       = 0x000000db,
    DB_PERF_SEL_dkg_tile_rate_tile                     = 0x000000dc,
    DB_PERF_SEL_prezl_src_in_sends                     = 0x000000dd,
    DB_PERF_SEL_prezl_src_in_stall                     = 0x000000de,
    DB_PERF_SEL_prezl_src_in_squads                    = 0x000000df,
    DB_PERF_SEL_prezl_src_in_squads_unrolled           = 0x000000e0,
    DB_PERF_SEL_prezl_src_in_tile_rate                 = 0x000000e1,
    DB_PERF_SEL_prezl_src_in_tile_rate_unrolled        = 0x000000e2,
    DB_PERF_SEL_prezl_src_out_stall                    = 0x000000e3,
    DB_PERF_SEL_postzl_src_in_sends                    = 0x000000e4,
    DB_PERF_SEL_postzl_src_in_stall                    = 0x000000e5,
    DB_PERF_SEL_postzl_src_in_squads                   = 0x000000e6,
    DB_PERF_SEL_postzl_src_in_squads_unrolled          = 0x000000e7,
    DB_PERF_SEL_postzl_src_in_tile_rate                = 0x000000e8,
    DB_PERF_SEL_postzl_src_in_tile_rate_unrolled       = 0x000000e9,
    DB_PERF_SEL_postzl_src_out_stall                   = 0x000000ea,
    DB_PERF_SEL_esr_ps_src_in_sends                    = 0x000000eb,
    DB_PERF_SEL_esr_ps_src_in_stall                    = 0x000000ec,
    DB_PERF_SEL_esr_ps_src_in_squads                   = 0x000000ed,
    DB_PERF_SEL_esr_ps_src_in_squads_unrolled          = 0x000000ee,
    DB_PERF_SEL_esr_ps_src_in_tile_rate                = 0x000000ef,
    DB_PERF_SEL_esr_ps_src_in_tile_rate_unrolled       = 0x000000f0,
    DB_PERF_SEL_esr_ps_src_in_tile_rate_unrolled_to_pixel_rate = 0x000000f1,
    DB_PERF_SEL_esr_ps_src_out_stall                   = 0x000000f2,
    DB_PERF_SEL_PreZ_Samples_failing_DB                = 0x000000f4,
    DB_PERF_SEL_PostZ_Samples_failing_DB               = 0x000000f5,
    DB_PERF_SEL_tiles_z_fully_summarized               = 0x000000f8,
    DB_PERF_SEL_tiles_stencil_fully_summarized         = 0x000000f9,
    DB_PERF_SEL_Sample_Tile_Cache_tiles_z_clear_on_expclear = 0x000000fa,
    DB_PERF_SEL_Sample_Tile_Cache_tiles_s_clear_on_expclear = 0x000000fb,
    DB_PERF_SEL_Sample_Tile_Cache_tiles_decomp_on_expclear = 0x000000fc,
    DB_PERF_SEL_Sample_Tile_Cache_tiles_compressed_to_decompressed = 0x000000fd,
    DB_PERF_SEL_Op_Pipe_Prez_Busy                      = 0x000000fe,
    DB_PERF_SEL_Op_Pipe_Postz_Busy                     = 0x000000ff,
    DB_PERF_SEL_DB_CB_export_export_quads              = 0x00000105,
    DB_PERF_SEL_DB_CB_export_double_format             = 0x00000106,
    DB_PERF_SEL_DB_CB_export_fast_format               = 0x00000107,
    DB_PERF_SEL_DB_CB_export_slow_format               = 0x00000108,
    DB_PERF_SEL_CB_DB_rdreq_sends                      = 0x00000109,
    DB_PERF_SEL_CB_DB_rdreq_prt_sends                  = 0x0000010a,
    DB_PERF_SEL_CB_DB_wrreq_sends                      = 0x0000010b,
    DB_PERF_SEL_CB_DB_wrreq_prt_sends                  = 0x0000010c,
    DB_PERF_SEL_DB_CB_rdret_ack                        = 0x0000010d,
    DB_PERF_SEL_DB_CB_rdret_nack                       = 0x0000010e,
    DB_PERF_SEL_DB_CB_wrret_ack                        = 0x0000010f,
    DB_PERF_SEL_DB_CB_wrret_nack                       = 0x00000110,
    DB_PERF_SEL_MI_quad_req_wrack_counter_stall        = 0x00000112,
    DB_PERF_SEL_MI_zpc_req_wrack_counter_stall         = 0x00000113,
    DB_PERF_SEL_DB_CB_export_is_event_FLUSH_AND_INV_DB_DATA_TS = 0x00000116,
    DB_PERF_SEL_DB_CB_export_is_event_FLUSH_AND_INV_CB_PIXEL_DATA = 0x00000117,
    DB_PERF_SEL_DB_CB_export_is_event_BOTTOM_OF_PIPE_TS = 0x00000118,
    DB_PERF_SEL_DB_CB_export_waiting_for_perfcounter_stop_event = 0x00000119,
    DB_PERF_SEL_DB_CB_export_fmt_32bpp_8pix            = 0x0000011a,
    DB_PERF_SEL_DB_CB_export_fmt_16_16_unsigned_8pix   = 0x0000011b,
    DB_PERF_SEL_DB_CB_export_fmt_16_16_signed_8pix     = 0x0000011c,
    DB_PERF_SEL_DB_CB_export_fmt_16_16_float_8pix      = 0x0000011d,
    DB_PERF_SEL_DB_CB_export_num_pixels_need_blending  = 0x0000011e,
    DB_PERF_SEL_DB_CB_context_dones                    = 0x0000011f,
    DB_PERF_SEL_DB_CB_eop_dones                        = 0x00000120,
    DB_PERF_SEL_SX_DB_quad_all_pixels_killed           = 0x00000121,
    DB_PERF_SEL_SX_DB_quad_all_pixels_enabled          = 0x00000122,
    DB_PERF_SEL_SX_DB_quad_need_blending_and_dst_read  = 0x00000123,
    DB_PERF_SEL_SC_DB_quad_quads                       = 0x00000125,
    DB_PERF_SEL_DB_SC_quad_quads_with_1_pixel          = 0x00000126,
    DB_PERF_SEL_DB_SC_quad_quads_with_2_pixels         = 0x00000127,
    DB_PERF_SEL_DB_SC_quad_quads_with_3_pixels         = 0x00000128,
    DB_PERF_SEL_DB_SC_quad_quads_with_4_pixels         = 0x00000129,
    DB_PERF_SEL_DB_SC_quad_double_quad                 = 0x0000012a,
    DB_PERF_SEL_SX_DB_quad_export_quads                = 0x0000012b,
    DB_PERF_SEL_SX_DB_quad_double_format               = 0x0000012c,
    DB_PERF_SEL_SX_DB_quad_fast_format                 = 0x0000012d,
    DB_PERF_SEL_SX_DB_quad_slow_format                 = 0x0000012e,
    DB_PERF_SEL_SC_DB_stile_sends                      = 0x0000012f,
    DB_PERF_SEL_SC_DB_stile_busy                       = 0x00000130,
    DB_PERF_SEL_SC_DB_stile_ptiles                     = 0x00000131,
    DB_PERF_SEL_SC_DB_quad_quads_pipe0                 = 0x00000133,
    DB_PERF_SEL_SC_DB_quad_quads_pipe1                 = 0x00000134,
    DB_PERF_SEL_noz_waiting_for_postz_done             = 0x00000136,
    DB_PERF_SEL_DB_CB_export_quads_vrs_rate_1x1        = 0x00000137,
    DB_PERF_SEL_DB_CB_export_quads_vrs_rate_2x1        = 0x00000138,
    DB_PERF_SEL_DB_CB_export_quads_vrs_rate_1x2        = 0x00000139,
    DB_PERF_SEL_DB_CB_export_quads_vrs_rate_2x2        = 0x0000013a,
    DB_PERF_SEL_RMI_rd_z_32byte_req                    = 0x0000013c,
    DB_PERF_SEL_RMI_rd_s_32byte_req                    = 0x0000013d,
    DB_PERF_SEL_RMI_wr_z_32byte_req                    = 0x0000013f,
    DB_PERF_SEL_RMI_wr_s_32byte_req                    = 0x00000140,
    DB_PERF_SEL_RMI_wr_zpc_32byte_req                  = 0x00000141,
    DB_PERF_SEL_RMI_rd_z_32byte_ret                    = 0x00000143,
    DB_PERF_SEL_RMI_rd_s_32byte_ret                    = 0x00000144,
    DB_PERF_SEL_RMI_wr_z_32byte_ack                    = 0x00000146,
    DB_PERF_SEL_RMI_wr_s_32byte_ack                    = 0x00000147,
    DB_PERF_SEL_RMI_wr_zpc_32byte_ack                  = 0x00000148,
    DB_PERF_SEL_esr_vic_sqq_busy                       = 0x00000149,
    DB_PERF_SEL_esr_vic_sqq_stall                      = 0x0000014a,
    DB_PERF_SEL_esr_psi_vic_tile_rate                  = 0x0000014b,
    DB_PERF_SEL_esr_vic_footprint_match_2x2            = 0x0000014c,
    DB_PERF_SEL_esr_vic_footprint_match_2x1            = 0x0000014d,
    DB_PERF_SEL_esr_vic_footprint_match_1x2            = 0x0000014e,
    DB_PERF_SEL_DB_SC_quad_num_null_2x2_coarse_pixels  = 0x0000014f,
    DB_PERF_SEL_DB_SC_quad_num_null_2x1_coarse_pixels  = 0x00000150,
    DB_PERF_SEL_DB_SC_quad_num_null_1x2_coarse_pixels  = 0x00000151,
    DB_PERF_SEL_esr_ps_woc_1squadIn_2squadOut          = 0x00000154,
    DB_PERF_SEL_esr_ps_woc_2squadIn_1squadOut          = 0x00000155,
    DB_PERF_SEL_prez_ps_invoked_pixel_cnt              = 0x00000156,
    DB_PERF_SEL_postz_ps_invoked_pixel_cnt             = 0x00000157,
    DB_PERF_SEL_ts_events_pws_enable                   = 0x00000158,
    DB_PERF_SEL_ps_events_pws_enable                   = 0x00000159,
    DB_PERF_SEL_cs_events_pws_enable                   = 0x0000015a,
    DB_PERF_SEL_DB_SC_quad_noz_tiles                   = 0x0000015b,
    DB_PERF_SEL_DB_SC_quad_lit_noz_quad                = 0x0000015c,
    DB_PERF_SEL_DB_SC_quad_conflicts                   = 0x0000015d,
    DB_PERF_SEL_SC_DB_sgroup_sends                     = 0x0000015e,
    DB_PERF_SEL_SC_DB_sgroup_busy                      = 0x0000015f,
    DB_PERF_SEL_SC_DB_sgroup_sgroups                   = 0x00000160,
    DB_PERF_SEL_SC_DB_sgroup_fquads                    = 0x00000161,
    DB_PERF_SEL_SC_DB_sgroup_stiles                    = 0x00000162,
    DB_PERF_SEL_SC_DB_wave_sends                       = 0x00000165,
    DB_PERF_SEL_SC_DB_wave_busy                        = 0x00000166,
    DB_PERF_SEL_SC_DB_wave_quads                       = 0x00000167,
    DB_PERF_SEL_SC_DB_wave_id_wrapped                  = 0x00000168,
    DB_PERF_SEL_DB_SC_wave_sends                       = 0x00000169,
    DB_PERF_SEL_DB_SC_wave_busy                        = 0x0000016a,
    DB_PERF_SEL_DB_SC_wave_stalls                      = 0x0000016b,
    DB_PERF_SEL_DB_SC_wave_conflict                    = 0x0000016c,
    DB_PERF_SEL_DB_SC_wave_hard_conflict               = 0x0000016d,
    DB_PERF_SEL_DB_SC_wave_id_wrapped                  = 0x0000016e,
    DB_PERF_SEL_SX_DB_quad_waves                       = 0x0000016f,
    DB_PERF_SEL_pws_stall                              = 0x00000170,
    DB_PERF_SEL_pws_liveness_stall_dtt_tag             = 0x00000171,
    DB_PERF_SEL_OREO_TT_load                           = 0x00000173,
    DB_PERF_SEL_OREO_TT_read                           = 0x00000174,
    DB_PERF_SEL_OREO_TT_stalls                         = 0x00000175,
    DB_PERF_SEL_OREO_ST_load                           = 0x00000176,
    DB_PERF_SEL_OREO_ST_read                           = 0x00000177,
    DB_PERF_SEL_OREO_ST_stalls                         = 0x00000178,
    DB_PERF_SEL_OREO_WT_load                           = 0x00000179,
    DB_PERF_SEL_OREO_WT_read                           = 0x0000017a,
    DB_PERF_SEL_OREO_SB_misses                         = 0x0000017b,
    DB_PERF_SEL_OREO_SB_hits                           = 0x0000017c,
    DB_PERF_SEL_OREO_SB_evicts                         = 0x0000017d,
    DB_PERF_SEL_OREO_SB_stalls                         = 0x0000017e,
    DB_PERF_SEL_OREO_SB_opaque_conflict                = 0x0000017f,
    DB_PERF_SEL_OREO_Events_load                       = 0x00000180,
    DB_PERF_SEL_OREO_Events_transition                 = 0x00000181,
    DB_PERF_SEL_OREO_Events_non_transition             = 0x00000182,
    DB_PERF_SEL_OREO_Events_delayed                    = 0x00000183,
    DB_PERF_SEL_OREO_Events_stalls                     = 0x00000184,
    DB_PERF_SEL_mi_rd_send                             = 0x00000185,
    DB_PERF_SEL_mi_rd_stall                            = 0x00000186,
    DB_PERF_SEL_mi_rd_64B_req_s                        = 0x00000187,
    DB_PERF_SEL_mi_rd_64B_req_z                        = 0x00000188,
    DB_PERF_SEL_mi_rdret_send                          = 0x00000189,
    DB_PERF_SEL_mi_rdret_send_s                        = 0x0000018a,
    DB_PERF_SEL_mi_rdret_send_z                        = 0x0000018b,
    DB_PERF_SEL_mi_rd_outstanding_sum                  = 0x0000018c,
    DB_PERF_SEL_mi_wr_send                             = 0x0000018d,
    DB_PERF_SEL_mi_wr_stall                            = 0x0000018e,
    DB_PERF_SEL_mi_wr_64B_req_s                        = 0x0000018f,
    DB_PERF_SEL_mi_wr_64B_req_z                        = 0x00000190,
    DB_PERF_SEL_mi_wr_64B_req_zpc                      = 0x00000191,
    DB_PERF_SEL_mi_wrack_send                          = 0x00000192,
    DB_PERF_SEL_mi_wrack_send_s                        = 0x00000193,
    DB_PERF_SEL_mi_wrack_send_z                        = 0x00000194,
    DB_PERF_SEL_mi_wrack_send_zpc                      = 0x00000195,
    DB_PERF_SEL_mi_wr_outstanding_sum                  = 0x00000196,
    DB_PERF_SEL_rr_z_uncomp                            = 0x00000197,
    DB_PERF_SEL_rr_z_planar_1                          = 0x00000198,
    DB_PERF_SEL_rr_z_planar_2                          = 0x00000199,
    DB_PERF_SEL_rr_z_planar_3                          = 0x0000019a,
    DB_PERF_SEL_rr_z_planar_4                          = 0x0000019b,
    DB_PERF_SEL_rr_z_single_0                          = 0x0000019c,
    DB_PERF_SEL_rr_z_single_1                          = 0x0000019d,
    DB_PERF_SEL_rr_z_single_2                          = 0x0000019e,
    DB_PERF_SEL_rr_s_uncomp                            = 0x0000019f,
    DB_PERF_SEL_rr_s_single_0                          = 0x000001a0,
    DB_PERF_SEL_rr_s_single_1                          = 0x000001a1,
    DB_PERF_SEL_rr_s_single_2                          = 0x000001a2,
    DB_PERF_SEL_flush_s_single_0                       = 0x000001a3,
    DB_PERF_SEL_flush_s_single_1                       = 0x000001a4,
    DB_PERF_SEL_flush_s_single_2                       = 0x000001a5,
    DB_PERF_SEL_flush_s_uncomp                         = 0x000001a6,
    DB_PERF_SEL_flush_z_single_0                       = 0x000001a7,
    DB_PERF_SEL_flush_z_single_1                       = 0x000001a8,
    DB_PERF_SEL_flush_z_single_2                       = 0x000001a9,
    DB_PERF_SEL_flush_z_uncomp                         = 0x000001aa,
    DB_PERF_SEL_flush_z_pl_comp_1plane                 = 0x000001ab,
    DB_PERF_SEL_flush_z_pl_comp_2plane                 = 0x000001ac,
    DB_PERF_SEL_flush_z_pl_comp_3plane                 = 0x000001ad,
    DB_PERF_SEL_flush_z_pl_comp_4plane                 = 0x000001ae,
    DB_PERF_SEL_Sample_Tile_Cache_stag_miss_stall      = 0x000001af,
    DB_PERF_SEL_Sample_Tile_Cache_stag_resource_stall  = 0x000001b0,
    DB_PERF_SEL_Sample_Tile_Cache_stag_locked          = 0x000001b1,
    DB_PERF_SEL_Sample_Tile_Cache_zcompressed_2_expanding = 0x000001b2,
    DB_PERF_SEL_db_busy                                = 0x000001b3,
    DB_PERF_SEL_db_blocked                             = 0x000001b4,
    DB_PERF_SEL_etr_blocked                            = 0x000001b5,
    DB_PERF_SEL_str_blocked                            = 0x000001b6,
    DB_PERF_SEL_prezl_blocked                          = 0x000001b7,
    DB_PERF_SEL_osb_owh_blocked                        = 0x000001b8,
    DB_PERF_SEL_osb_TT_blocked                         = 0x000001b9,
} PerfCounter_Vals;

constexpr unsigned int MaxPerfcounterVals              = DB_PERF_SEL_osb_TT_blocked;

typedef enum PERFMON_CNTOFF_AND_OR {
    PERFMON_CNTOFF_OR                                  = 0x00000000,
    PERFMON_CNTOFF_AND                                 = 0x00000001,
} PERFMON_CNTOFF_AND_OR;

typedef enum PERFMON_CNTOFF_INT_EN {
    PERFMON_CNTOFF_INT_DISABLE                         = 0x00000000,
    PERFMON_CNTOFF_INT_ENABLE                          = 0x00000001,
} PERFMON_CNTOFF_INT_EN;

typedef enum PERFMON_CNTOFF_INT_TYPE {
    PERFMON_CNTOFF_INT_TYPE_LEVEL                      = 0x00000000,
    PERFMON_CNTOFF_INT_TYPE_PULSE                      = 0x00000001,
} PERFMON_CNTOFF_INT_TYPE;

typedef enum PERFMON_COUNTER_MODE {
    PERFMON_COUNTER_MODE_ACCUM                         = 0x00000000,
    PERFMON_COUNTER_MODE_ACTIVE_CYCLES                 = 0x00000001,
    PERFMON_COUNTER_MODE_MAX                           = 0x00000002,
    PERFMON_COUNTER_MODE_DIRTY                         = 0x00000003,
    PERFMON_COUNTER_MODE_SAMPLE                        = 0x00000004,
    PERFMON_COUNTER_MODE_CYCLES_SINCE_FIRST_EVENT      = 0x00000005,
    PERFMON_COUNTER_MODE_CYCLES_SINCE_LAST_EVENT       = 0x00000006,
    PERFMON_COUNTER_MODE_CYCLES_GE_HI                  = 0x00000007,
    PERFMON_COUNTER_MODE_CYCLES_EQ_HI                  = 0x00000008,
    PERFMON_COUNTER_MODE_INACTIVE_CYCLES               = 0x00000009,
    PERFMON_COUNTER_MODE_RESERVED                      = 0x0000000f,
} PERFMON_COUNTER_MODE;

typedef enum PERFMON_SPM_MODE {
    PERFMON_SPM_MODE_OFF                               = 0x00000000,
    PERFMON_SPM_MODE_16BIT_CLAMP                       = 0x00000001,
    PERFMON_SPM_MODE_16BIT_NO_CLAMP                    = 0x00000002,
    PERFMON_SPM_MODE_32BIT_CLAMP                       = 0x00000003,
    PERFMON_SPM_MODE_32BIT_NO_CLAMP                    = 0x00000004,
    PERFMON_SPM_MODE_RESERVED_5                        = 0x00000005,
    PERFMON_SPM_MODE_RESERVED_6                        = 0x00000006,
    PERFMON_SPM_MODE_RESERVED_7                        = 0x00000007,
    PERFMON_SPM_MODE_TEST_MODE_0                       = 0x00000008,
    PERFMON_SPM_MODE_TEST_MODE_1                       = 0x00000009,
    PERFMON_SPM_MODE_TEST_MODE_2                       = 0x0000000a,
} PERFMON_SPM_MODE;

typedef enum PERFMON_STATE {
    PERFMON_STATE_RESET                                = 0x00000000,
    PERFMON_STATE_START                                = 0x00000001,
    PERFMON_STATE_FREEZE                               = 0x00000002,
    PERFMON_STATE_HW                                   = 0x00000003,
} PERFMON_STATE;

typedef enum PH_PERFCNT_SEL {
    PH_PERF_SEL_SC0_SRPS_WINDOW_VALID                  = 0x00000000,
    PH_PERF_SEL_SC0_ARB_XFC_ALL_EVENT_OR_PRIM_CYCLES   = 0x00000001,
    PH_PERF_SEL_SC0_ARB_XFC_ONLY_PRIM_CYCLES           = 0x00000002,
    PH_PERF_SEL_SC0_ARB_XFC_ONLY_ONE_INC_PER_PRIM      = 0x00000003,
    PH_PERF_SEL_SC0_ARB_STALLED_FROM_BELOW             = 0x00000004,
    PH_PERF_SEL_SC0_ARB_STARVED_FROM_ABOVE             = 0x00000005,
    PH_PERF_SEL_SC0_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_NOT_EMPTY = 0x00000006,
    PH_PERF_SEL_SC0_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_FULL = 0x00000007,
    PH_PERF_SEL_SC0_ARB_BUSY                           = 0x00000008,
    PH_PERF_SEL_SC0_ARB_PA_BUSY_SOP                    = 0x00000009,
    PH_PERF_SEL_SC0_ARB_EOP_POP_SYNC_POP               = 0x0000000a,
    PH_PERF_SEL_SC0_ARB_EVENT_SYNC_POP                 = 0x0000000b,
    PH_PERF_SEL_SC0_PS_ENG_MULTICYCLE_BUBBLE           = 0x0000000c,
    PH_PERF_SEL_SC0_EOP_SYNC_WINDOW                    = 0x0000000d,
    PH_PERF_SEL_SC0_BUSY_PROCESSING_MULTICYCLE_PRIM    = 0x0000000e,
    PH_PERF_SEL_SC0_BUSY_CNT_NOT_ZERO                  = 0x0000000f,
    PH_PERF_SEL_SC0_SEND                               = 0x00000010,
    PH_PERF_SEL_SC0_CREDIT_AT_ZERO_WITH_PENDING_SEND   = 0x00000011,
    PH_PERF_SEL_SC0_CREDIT_AT_MAX                      = 0x00000012,
    PH_PERF_SEL_SC0_CREDIT_AT_MAX_NO_PENDING_SEND      = 0x00000013,
    PH_PERF_SEL_SC0_GFX_PIPE0_TO_1_TRANSITION          = 0x00000014,
    PH_PERF_SEL_SC0_GFX_PIPE1_TO_0_TRANSITION          = 0x00000015,
    PH_PERF_SEL_SC0_GFX_PIPE_PRIM_PROVOKED_TRANSITION  = 0x00000016,
    PH_PERF_SEL_SC0_GFX_PIPE_EVENT_PROVOKED_TRANSITION = 0x00000017,
    PH_PERF_SEL_SC0_PA0_DATA_FIFO_RD                   = 0x00000018,
    PH_PERF_SEL_SC0_PA0_DATA_FIFO_WE                   = 0x00000019,
    PH_PERF_SEL_SC0_PA0_FIFO_EMPTY                     = 0x0000001a,
    PH_PERF_SEL_SC0_PA0_FIFO_FULL                      = 0x0000001b,
    PH_PERF_SEL_SC0_PA0_NULL_WE                        = 0x0000001c,
    PH_PERF_SEL_SC0_PA0_EVENT_WE                       = 0x0000001d,
    PH_PERF_SEL_SC0_PA0_FPOV_WE                        = 0x0000001e,
    PH_PERF_SEL_SC0_PA0_FPOP_WE                        = 0x0000001f,
    PH_PERF_SEL_SC0_PA0_EOP_WE                         = 0x00000020,
    PH_PERF_SEL_SC0_PA0_DATA_FIFO_EOP_RD               = 0x00000021,
    PH_PERF_SEL_SC0_PA0_EOPG_WE                        = 0x00000022,
    PH_PERF_SEL_SC0_PA0_DEALLOC_WE                     = 0x00000023,
    PH_PERF_SEL_SC0_PA1_DATA_FIFO_RD                   = 0x00000024,
    PH_PERF_SEL_SC0_PA1_DATA_FIFO_WE                   = 0x00000025,
    PH_PERF_SEL_SC0_PA1_FIFO_EMPTY                     = 0x00000026,
    PH_PERF_SEL_SC0_PA1_FIFO_FULL                      = 0x00000027,
    PH_PERF_SEL_SC0_PA1_NULL_WE                        = 0x00000028,
    PH_PERF_SEL_SC0_PA1_EVENT_WE                       = 0x00000029,
    PH_PERF_SEL_SC0_PA1_FPOV_WE                        = 0x0000002a,
    PH_PERF_SEL_SC0_PA1_FPOP_WE                        = 0x0000002b,
    PH_PERF_SEL_SC0_PA1_EOP_WE                         = 0x0000002c,
    PH_PERF_SEL_SC0_PA1_DATA_FIFO_EOP_RD               = 0x0000002d,
    PH_PERF_SEL_SC0_PA1_EOPG_WE                        = 0x0000002e,
    PH_PERF_SEL_SC0_PA1_DEALLOC_WE                     = 0x0000002f,
    PH_PERF_SEL_SC0_PA2_DATA_FIFO_RD                   = 0x00000030,
    PH_PERF_SEL_SC0_PA2_DATA_FIFO_WE                   = 0x00000031,
    PH_PERF_SEL_SC0_PA2_FIFO_EMPTY                     = 0x00000032,
    PH_PERF_SEL_SC0_PA2_FIFO_FULL                      = 0x00000033,
    PH_PERF_SEL_SC0_PA2_NULL_WE                        = 0x00000034,
    PH_PERF_SEL_SC0_PA2_EVENT_WE                       = 0x00000035,
    PH_PERF_SEL_SC0_PA2_FPOV_WE                        = 0x00000036,
    PH_PERF_SEL_SC0_PA2_FPOP_WE                        = 0x00000037,
    PH_PERF_SEL_SC0_PA2_EOP_WE                         = 0x00000038,
    PH_PERF_SEL_SC0_PA2_DATA_FIFO_EOP_RD               = 0x00000039,
    PH_PERF_SEL_SC0_PA2_EOPG_WE                        = 0x0000003a,
    PH_PERF_SEL_SC0_PA2_DEALLOC_WE                     = 0x0000003b,
    PH_PERF_SEL_SC0_PA3_DATA_FIFO_RD                   = 0x0000003c,
    PH_PERF_SEL_SC0_PA3_DATA_FIFO_WE                   = 0x0000003d,
    PH_PERF_SEL_SC0_PA3_FIFO_EMPTY                     = 0x0000003e,
    PH_PERF_SEL_SC0_PA3_FIFO_FULL                      = 0x0000003f,
    PH_PERF_SEL_SC0_PA3_NULL_WE                        = 0x00000040,
    PH_PERF_SEL_SC0_PA3_EVENT_WE                       = 0x00000041,
    PH_PERF_SEL_SC0_PA3_FPOV_WE                        = 0x00000042,
    PH_PERF_SEL_SC0_PA3_FPOP_WE                        = 0x00000043,
    PH_PERF_SEL_SC0_PA3_EOP_WE                         = 0x00000044,
    PH_PERF_SEL_SC0_PA3_DATA_FIFO_EOP_RD               = 0x00000045,
    PH_PERF_SEL_SC0_PA3_EOPG_WE                        = 0x00000046,
    PH_PERF_SEL_SC0_PA3_DEALLOC_WE                     = 0x00000047,
    PH_PERF_SEL_SC0_PA4_DATA_FIFO_RD                   = 0x00000048,
    PH_PERF_SEL_SC0_PA4_DATA_FIFO_WE                   = 0x00000049,
    PH_PERF_SEL_SC0_PA4_FIFO_EMPTY                     = 0x0000004a,
    PH_PERF_SEL_SC0_PA4_FIFO_FULL                      = 0x0000004b,
    PH_PERF_SEL_SC0_PA4_NULL_WE                        = 0x0000004c,
    PH_PERF_SEL_SC0_PA4_EVENT_WE                       = 0x0000004d,
    PH_PERF_SEL_SC0_PA4_FPOV_WE                        = 0x0000004e,
    PH_PERF_SEL_SC0_PA4_FPOP_WE                        = 0x0000004f,
    PH_PERF_SEL_SC0_PA4_EOP_WE                         = 0x00000050,
    PH_PERF_SEL_SC0_PA4_DATA_FIFO_EOP_RD               = 0x00000051,
    PH_PERF_SEL_SC0_PA4_EOPG_WE                        = 0x00000052,
    PH_PERF_SEL_SC0_PA4_DEALLOC_WE                     = 0x00000053,
    PH_PERF_SEL_SC0_PA5_DATA_FIFO_RD                   = 0x00000054,
    PH_PERF_SEL_SC0_PA5_DATA_FIFO_WE                   = 0x00000055,
    PH_PERF_SEL_SC0_PA5_FIFO_EMPTY                     = 0x00000056,
    PH_PERF_SEL_SC0_PA5_FIFO_FULL                      = 0x00000057,
    PH_PERF_SEL_SC0_PA5_NULL_WE                        = 0x00000058,
    PH_PERF_SEL_SC0_PA5_EVENT_WE                       = 0x00000059,
    PH_PERF_SEL_SC0_PA5_FPOV_WE                        = 0x0000005a,
    PH_PERF_SEL_SC0_PA5_FPOP_WE                        = 0x0000005b,
    PH_PERF_SEL_SC0_PA5_EOP_WE                         = 0x0000005c,
    PH_PERF_SEL_SC0_PA5_DATA_FIFO_EOP_RD               = 0x0000005d,
    PH_PERF_SEL_SC0_PA5_EOPG_WE                        = 0x0000005e,
    PH_PERF_SEL_SC0_PA5_DEALLOC_WE                     = 0x0000005f,
    PH_PERF_SEL_SC0_PA6_DATA_FIFO_RD                   = 0x00000060,
    PH_PERF_SEL_SC0_PA6_DATA_FIFO_WE                   = 0x00000061,
    PH_PERF_SEL_SC0_PA6_FIFO_EMPTY                     = 0x00000062,
    PH_PERF_SEL_SC0_PA6_FIFO_FULL                      = 0x00000063,
    PH_PERF_SEL_SC0_PA6_NULL_WE                        = 0x00000064,
    PH_PERF_SEL_SC0_PA6_EVENT_WE                       = 0x00000065,
    PH_PERF_SEL_SC0_PA6_FPOV_WE                        = 0x00000066,
    PH_PERF_SEL_SC0_PA6_FPOP_WE                        = 0x00000067,
    PH_PERF_SEL_SC0_PA6_EOP_WE                         = 0x00000068,
    PH_PERF_SEL_SC0_PA6_DATA_FIFO_EOP_RD               = 0x00000069,
    PH_PERF_SEL_SC0_PA6_EOPG_WE                        = 0x0000006a,
    PH_PERF_SEL_SC0_PA6_DEALLOC_WE                     = 0x0000006b,
    PH_PERF_SEL_SC0_PA7_DATA_FIFO_RD                   = 0x0000006c,
    PH_PERF_SEL_SC0_PA7_DATA_FIFO_WE                   = 0x0000006d,
    PH_PERF_SEL_SC0_PA7_FIFO_EMPTY                     = 0x0000006e,
    PH_PERF_SEL_SC0_PA7_FIFO_FULL                      = 0x0000006f,
    PH_PERF_SEL_SC0_PA7_NULL_WE                        = 0x00000070,
    PH_PERF_SEL_SC0_PA7_EVENT_WE                       = 0x00000071,
    PH_PERF_SEL_SC0_PA7_FPOV_WE                        = 0x00000072,
    PH_PERF_SEL_SC0_PA7_FPOP_WE                        = 0x00000073,
    PH_PERF_SEL_SC0_PA7_EOP_WE                         = 0x00000074,
    PH_PERF_SEL_SC0_PA7_DATA_FIFO_EOP_RD               = 0x00000075,
    PH_PERF_SEL_SC0_PA7_EOPG_WE                        = 0x00000076,
    PH_PERF_SEL_SC0_PA7_DEALLOC_WE                     = 0x00000077,
    PH_PERF_SEL_SC1_SRPS_WINDOW_VALID                  = 0x00000078,
    PH_PERF_SEL_SC1_ARB_XFC_ALL_EVENT_OR_PRIM_CYCLES   = 0x00000079,
    PH_PERF_SEL_SC1_ARB_XFC_ONLY_PRIM_CYCLES           = 0x0000007a,
    PH_PERF_SEL_SC1_ARB_XFC_ONLY_ONE_INC_PER_PRIM      = 0x0000007b,
    PH_PERF_SEL_SC1_ARB_STALLED_FROM_BELOW             = 0x0000007c,
    PH_PERF_SEL_SC1_ARB_STARVED_FROM_ABOVE             = 0x0000007d,
    PH_PERF_SEL_SC1_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_NOT_EMPTY = 0x0000007e,
    PH_PERF_SEL_SC1_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_FULL = 0x0000007f,
    PH_PERF_SEL_SC1_ARB_BUSY                           = 0x00000080,
    PH_PERF_SEL_SC1_ARB_PA_BUSY_SOP                    = 0x00000081,
    PH_PERF_SEL_SC1_ARB_EOP_POP_SYNC_POP               = 0x00000082,
    PH_PERF_SEL_SC1_ARB_EVENT_SYNC_POP                 = 0x00000083,
    PH_PERF_SEL_SC1_PS_ENG_MULTICYCLE_BUBBLE           = 0x00000084,
    PH_PERF_SEL_SC1_EOP_SYNC_WINDOW                    = 0x00000085,
    PH_PERF_SEL_SC1_BUSY_PROCESSING_MULTICYCLE_PRIM    = 0x00000086,
    PH_PERF_SEL_SC1_BUSY_CNT_NOT_ZERO                  = 0x00000087,
    PH_PERF_SEL_SC1_SEND                               = 0x00000088,
    PH_PERF_SEL_SC1_CREDIT_AT_ZERO_WITH_PENDING_SEND   = 0x00000089,
    PH_PERF_SEL_SC1_CREDIT_AT_MAX                      = 0x0000008a,
    PH_PERF_SEL_SC1_CREDIT_AT_MAX_NO_PENDING_SEND      = 0x0000008b,
    PH_PERF_SEL_SC1_GFX_PIPE0_TO_1_TRANSITION          = 0x0000008c,
    PH_PERF_SEL_SC1_GFX_PIPE1_TO_0_TRANSITION          = 0x0000008d,
    PH_PERF_SEL_SC1_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION = 0x0000008e,
    PH_PERF_SEL_SC1_GFX_PIPE_EVENT_PROVOKED_TRANSITION = 0x0000008f,
    PH_PERF_SEL_SC1_PA0_DATA_FIFO_RD                   = 0x00000090,
    PH_PERF_SEL_SC1_PA0_DATA_FIFO_WE                   = 0x00000091,
    PH_PERF_SEL_SC1_PA0_FIFO_EMPTY                     = 0x00000092,
    PH_PERF_SEL_SC1_PA0_FIFO_FULL                      = 0x00000093,
    PH_PERF_SEL_SC1_PA0_NULL_WE                        = 0x00000094,
    PH_PERF_SEL_SC1_PA0_EVENT_WE                       = 0x00000095,
    PH_PERF_SEL_SC1_PA0_FPOV_WE                        = 0x00000096,
    PH_PERF_SEL_SC1_PA0_FPOP_WE                        = 0x00000097,
    PH_PERF_SEL_SC1_PA0_EOP_WE                         = 0x00000098,
    PH_PERF_SEL_SC1_PA0_DATA_FIFO_EOP_RD               = 0x00000099,
    PH_PERF_SEL_SC1_PA0_EOPG_WE                        = 0x0000009a,
    PH_PERF_SEL_SC1_PA0_DEALLOC_WE                     = 0x0000009b,
    PH_PERF_SEL_SC1_PA1_DATA_FIFO_RD                   = 0x0000009c,
    PH_PERF_SEL_SC1_PA1_DATA_FIFO_WE                   = 0x0000009d,
    PH_PERF_SEL_SC1_PA1_FIFO_EMPTY                     = 0x0000009e,
    PH_PERF_SEL_SC1_PA1_FIFO_FULL                      = 0x0000009f,
    PH_PERF_SEL_SC1_PA1_NULL_WE                        = 0x000000a0,
    PH_PERF_SEL_SC1_PA1_EVENT_WE                       = 0x000000a1,
    PH_PERF_SEL_SC1_PA1_FPOV_WE                        = 0x000000a2,
    PH_PERF_SEL_SC1_PA1_FPOP_WE                        = 0x000000a3,
    PH_PERF_SEL_SC1_PA1_EOP_WE                         = 0x000000a4,
    PH_PERF_SEL_SC1_PA1_DATA_FIFO_EOP_RD               = 0x000000a5,
    PH_PERF_SEL_SC1_PA1_EOPG_WE                        = 0x000000a6,
    PH_PERF_SEL_SC1_PA1_DEALLOC_WE                     = 0x000000a7,
    PH_PERF_SEL_SC1_PA2_DATA_FIFO_RD                   = 0x000000a8,
    PH_PERF_SEL_SC1_PA2_DATA_FIFO_WE                   = 0x000000a9,
    PH_PERF_SEL_SC1_PA2_FIFO_EMPTY                     = 0x000000aa,
    PH_PERF_SEL_SC1_PA2_FIFO_FULL                      = 0x000000ab,
    PH_PERF_SEL_SC1_PA2_NULL_WE                        = 0x000000ac,
    PH_PERF_SEL_SC1_PA2_EVENT_WE                       = 0x000000ad,
    PH_PERF_SEL_SC1_PA2_FPOV_WE                        = 0x000000ae,
    PH_PERF_SEL_SC1_PA2_FPOP_WE                        = 0x000000af,
    PH_PERF_SEL_SC1_PA2_EOP_WE                         = 0x000000b0,
    PH_PERF_SEL_SC1_PA2_DATA_FIFO_EOP_RD               = 0x000000b1,
    PH_PERF_SEL_SC1_PA2_EOPG_WE                        = 0x000000b2,
    PH_PERF_SEL_SC1_PA2_DEALLOC_WE                     = 0x000000b3,
    PH_PERF_SEL_SC1_PA3_DATA_FIFO_RD                   = 0x000000b4,
    PH_PERF_SEL_SC1_PA3_DATA_FIFO_WE                   = 0x000000b5,
    PH_PERF_SEL_SC1_PA3_FIFO_EMPTY                     = 0x000000b6,
    PH_PERF_SEL_SC1_PA3_FIFO_FULL                      = 0x000000b7,
    PH_PERF_SEL_SC1_PA3_NULL_WE                        = 0x000000b8,
    PH_PERF_SEL_SC1_PA3_EVENT_WE                       = 0x000000b9,
    PH_PERF_SEL_SC1_PA3_FPOV_WE                        = 0x000000ba,
    PH_PERF_SEL_SC1_PA3_FPOP_WE                        = 0x000000bb,
    PH_PERF_SEL_SC1_PA3_EOP_WE                         = 0x000000bc,
    PH_PERF_SEL_SC1_PA3_DATA_FIFO_EOP_RD               = 0x000000bd,
    PH_PERF_SEL_SC1_PA3_EOPG_WE                        = 0x000000be,
    PH_PERF_SEL_SC1_PA3_DEALLOC_WE                     = 0x000000bf,
    PH_PERF_SEL_SC1_PA4_DATA_FIFO_RD                   = 0x000000c0,
    PH_PERF_SEL_SC1_PA4_DATA_FIFO_WE                   = 0x000000c1,
    PH_PERF_SEL_SC1_PA4_FIFO_EMPTY                     = 0x000000c2,
    PH_PERF_SEL_SC1_PA4_FIFO_FULL                      = 0x000000c3,
    PH_PERF_SEL_SC1_PA4_NULL_WE                        = 0x000000c4,
    PH_PERF_SEL_SC1_PA4_EVENT_WE                       = 0x000000c5,
    PH_PERF_SEL_SC1_PA4_FPOV_WE                        = 0x000000c6,
    PH_PERF_SEL_SC1_PA4_FPOP_WE                        = 0x000000c7,
    PH_PERF_SEL_SC1_PA4_EOP_WE                         = 0x000000c8,
    PH_PERF_SEL_SC1_PA4_DATA_FIFO_EOP_RD               = 0x000000c9,
    PH_PERF_SEL_SC1_PA4_EOPG_WE                        = 0x000000ca,
    PH_PERF_SEL_SC1_PA4_DEALLOC_WE                     = 0x000000cb,
    PH_PERF_SEL_SC1_PA5_DATA_FIFO_RD                   = 0x000000cc,
    PH_PERF_SEL_SC1_PA5_DATA_FIFO_WE                   = 0x000000cd,
    PH_PERF_SEL_SC1_PA5_FIFO_EMPTY                     = 0x000000ce,
    PH_PERF_SEL_SC1_PA5_FIFO_FULL                      = 0x000000cf,
    PH_PERF_SEL_SC1_PA5_NULL_WE                        = 0x000000d0,
    PH_PERF_SEL_SC1_PA5_EVENT_WE                       = 0x000000d1,
    PH_PERF_SEL_SC1_PA5_FPOV_WE                        = 0x000000d2,
    PH_PERF_SEL_SC1_PA5_FPOP_WE                        = 0x000000d3,
    PH_PERF_SEL_SC1_PA5_EOP_WE                         = 0x000000d4,
    PH_PERF_SEL_SC1_PA5_DATA_FIFO_EOP_RD               = 0x000000d5,
    PH_PERF_SEL_SC1_PA5_EOPG_WE                        = 0x000000d6,
    PH_PERF_SEL_SC1_PA5_DEALLOC_WE                     = 0x000000d7,
    PH_PERF_SEL_SC1_PA6_DATA_FIFO_RD                   = 0x000000d8,
    PH_PERF_SEL_SC1_PA6_DATA_FIFO_WE                   = 0x000000d9,
    PH_PERF_SEL_SC1_PA6_FIFO_EMPTY                     = 0x000000da,
    PH_PERF_SEL_SC1_PA6_FIFO_FULL                      = 0x000000db,
    PH_PERF_SEL_SC1_PA6_NULL_WE                        = 0x000000dc,
    PH_PERF_SEL_SC1_PA6_EVENT_WE                       = 0x000000dd,
    PH_PERF_SEL_SC1_PA6_FPOV_WE                        = 0x000000de,
    PH_PERF_SEL_SC1_PA6_FPOP_WE                        = 0x000000df,
    PH_PERF_SEL_SC1_PA6_EOP_WE                         = 0x000000e0,
    PH_PERF_SEL_SC1_PA6_DATA_FIFO_EOP_RD               = 0x000000e1,
    PH_PERF_SEL_SC1_PA6_EOPG_WE                        = 0x000000e2,
    PH_PERF_SEL_SC1_PA6_DEALLOC_WE                     = 0x000000e3,
    PH_PERF_SEL_SC1_PA7_DATA_FIFO_RD                   = 0x000000e4,
    PH_PERF_SEL_SC1_PA7_DATA_FIFO_WE                   = 0x000000e5,
    PH_PERF_SEL_SC1_PA7_FIFO_EMPTY                     = 0x000000e6,
    PH_PERF_SEL_SC1_PA7_FIFO_FULL                      = 0x000000e7,
    PH_PERF_SEL_SC1_PA7_NULL_WE                        = 0x000000e8,
    PH_PERF_SEL_SC1_PA7_EVENT_WE                       = 0x000000e9,
    PH_PERF_SEL_SC1_PA7_FPOV_WE                        = 0x000000ea,
    PH_PERF_SEL_SC1_PA7_FPOP_WE                        = 0x000000eb,
    PH_PERF_SEL_SC1_PA7_EOP_WE                         = 0x000000ec,
    PH_PERF_SEL_SC1_PA7_DATA_FIFO_EOP_RD               = 0x000000ed,
    PH_PERF_SEL_SC1_PA7_EOPG_WE                        = 0x000000ee,
    PH_PERF_SEL_SC1_PA7_DEALLOC_WE                     = 0x000000ef,
    PH_PERF_SEL_SC2_SRPS_WINDOW_VALID                  = 0x000000f0,
    PH_PERF_SEL_SC2_ARB_XFC_ALL_EVENT_OR_PRIM_CYCLES   = 0x000000f1,
    PH_PERF_SEL_SC2_ARB_XFC_ONLY_PRIM_CYCLES           = 0x000000f2,
    PH_PERF_SEL_SC2_ARB_XFC_ONLY_ONE_INC_PER_PRIM      = 0x000000f3,
    PH_PERF_SEL_SC2_ARB_STALLED_FROM_BELOW             = 0x000000f4,
    PH_PERF_SEL_SC2_ARB_STARVED_FROM_ABOVE             = 0x000000f5,
    PH_PERF_SEL_SC2_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_NOT_EMPTY = 0x000000f6,
    PH_PERF_SEL_SC2_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_FULL = 0x000000f7,
    PH_PERF_SEL_SC2_ARB_BUSY                           = 0x000000f8,
    PH_PERF_SEL_SC2_ARB_PA_BUSY_SOP                    = 0x000000f9,
    PH_PERF_SEL_SC2_ARB_EOP_POP_SYNC_POP               = 0x000000fa,
    PH_PERF_SEL_SC2_ARB_EVENT_SYNC_POP                 = 0x000000fb,
    PH_PERF_SEL_SC2_PS_ENG_MULTICYCLE_BUBBLE           = 0x000000fc,
    PH_PERF_SEL_SC2_EOP_SYNC_WINDOW                    = 0x000000fd,
    PH_PERF_SEL_SC2_BUSY_PROCESSING_MULTICYCLE_PRIM    = 0x000000fe,
    PH_PERF_SEL_SC2_BUSY_CNT_NOT_ZERO                  = 0x000000ff,
    PH_PERF_SEL_SC2_SEND                               = 0x00000100,
    PH_PERF_SEL_SC2_CREDIT_AT_ZERO_WITH_PENDING_SEND   = 0x00000101,
    PH_PERF_SEL_SC2_CREDIT_AT_MAX                      = 0x00000102,
    PH_PERF_SEL_SC2_CREDIT_AT_MAX_NO_PENDING_SEND      = 0x00000103,
    PH_PERF_SEL_SC2_GFX_PIPE0_TO_1_TRANSITION          = 0x00000104,
    PH_PERF_SEL_SC2_GFX_PIPE1_TO_0_TRANSITION          = 0x00000105,
    PH_PERF_SEL_SC2_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION = 0x00000106,
    PH_PERF_SEL_SC2_GFX_PIPE_EVENT_PROVOKED_TRANSITION = 0x00000107,
    PH_PERF_SEL_SC2_PA0_DATA_FIFO_RD                   = 0x00000108,
    PH_PERF_SEL_SC2_PA0_DATA_FIFO_WE                   = 0x00000109,
    PH_PERF_SEL_SC2_PA0_FIFO_EMPTY                     = 0x0000010a,
    PH_PERF_SEL_SC2_PA0_FIFO_FULL                      = 0x0000010b,
    PH_PERF_SEL_SC2_PA0_NULL_WE                        = 0x0000010c,
    PH_PERF_SEL_SC2_PA0_EVENT_WE                       = 0x0000010d,
    PH_PERF_SEL_SC2_PA0_FPOV_WE                        = 0x0000010e,
    PH_PERF_SEL_SC2_PA0_FPOP_WE                        = 0x0000010f,
    PH_PERF_SEL_SC2_PA0_EOP_WE                         = 0x00000110,
    PH_PERF_SEL_SC2_PA0_DATA_FIFO_EOP_RD               = 0x00000111,
    PH_PERF_SEL_SC2_PA0_EOPG_WE                        = 0x00000112,
    PH_PERF_SEL_SC2_PA0_DEALLOC_WE                     = 0x00000113,
    PH_PERF_SEL_SC2_PA1_DATA_FIFO_RD                   = 0x00000114,
    PH_PERF_SEL_SC2_PA1_DATA_FIFO_WE                   = 0x00000115,
    PH_PERF_SEL_SC2_PA1_FIFO_EMPTY                     = 0x00000116,
    PH_PERF_SEL_SC2_PA1_FIFO_FULL                      = 0x00000117,
    PH_PERF_SEL_SC2_PA1_NULL_WE                        = 0x00000118,
    PH_PERF_SEL_SC2_PA1_EVENT_WE                       = 0x00000119,
    PH_PERF_SEL_SC2_PA1_FPOV_WE                        = 0x0000011a,
    PH_PERF_SEL_SC2_PA1_FPOP_WE                        = 0x0000011b,
    PH_PERF_SEL_SC2_PA1_EOP_WE                         = 0x0000011c,
    PH_PERF_SEL_SC2_PA1_DATA_FIFO_EOP_RD               = 0x0000011d,
    PH_PERF_SEL_SC2_PA1_EOPG_WE                        = 0x0000011e,
    PH_PERF_SEL_SC2_PA1_DEALLOC_WE                     = 0x0000011f,
    PH_PERF_SEL_SC2_PA2_DATA_FIFO_RD                   = 0x00000120,
    PH_PERF_SEL_SC2_PA2_DATA_FIFO_WE                   = 0x00000121,
    PH_PERF_SEL_SC2_PA2_FIFO_EMPTY                     = 0x00000122,
    PH_PERF_SEL_SC2_PA2_FIFO_FULL                      = 0x00000123,
    PH_PERF_SEL_SC2_PA2_NULL_WE                        = 0x00000124,
    PH_PERF_SEL_SC2_PA2_EVENT_WE                       = 0x00000125,
    PH_PERF_SEL_SC2_PA2_FPOV_WE                        = 0x00000126,
    PH_PERF_SEL_SC2_PA2_FPOP_WE                        = 0x00000127,
    PH_PERF_SEL_SC2_PA2_EOP_WE                         = 0x00000128,
    PH_PERF_SEL_SC2_PA2_DATA_FIFO_EOP_RD               = 0x00000129,
    PH_PERF_SEL_SC2_PA2_EOPG_WE                        = 0x0000012a,
    PH_PERF_SEL_SC2_PA2_DEALLOC_WE                     = 0x0000012b,
    PH_PERF_SEL_SC2_PA3_DATA_FIFO_RD                   = 0x0000012c,
    PH_PERF_SEL_SC2_PA3_DATA_FIFO_WE                   = 0x0000012d,
    PH_PERF_SEL_SC2_PA3_FIFO_EMPTY                     = 0x0000012e,
    PH_PERF_SEL_SC2_PA3_FIFO_FULL                      = 0x0000012f,
    PH_PERF_SEL_SC2_PA3_NULL_WE                        = 0x00000130,
    PH_PERF_SEL_SC2_PA3_EVENT_WE                       = 0x00000131,
    PH_PERF_SEL_SC2_PA3_FPOV_WE                        = 0x00000132,
    PH_PERF_SEL_SC2_PA3_FPOP_WE                        = 0x00000133,
    PH_PERF_SEL_SC2_PA3_EOP_WE                         = 0x00000134,
    PH_PERF_SEL_SC2_PA3_DATA_FIFO_EOP_RD               = 0x00000135,
    PH_PERF_SEL_SC2_PA3_EOPG_WE                        = 0x00000136,
    PH_PERF_SEL_SC2_PA3_DEALLOC_WE                     = 0x00000137,
    PH_PERF_SEL_SC2_PA4_DATA_FIFO_RD                   = 0x00000138,
    PH_PERF_SEL_SC2_PA4_DATA_FIFO_WE                   = 0x00000139,
    PH_PERF_SEL_SC2_PA4_FIFO_EMPTY                     = 0x0000013a,
    PH_PERF_SEL_SC2_PA4_FIFO_FULL                      = 0x0000013b,
    PH_PERF_SEL_SC2_PA4_NULL_WE                        = 0x0000013c,
    PH_PERF_SEL_SC2_PA4_EVENT_WE                       = 0x0000013d,
    PH_PERF_SEL_SC2_PA4_FPOV_WE                        = 0x0000013e,
    PH_PERF_SEL_SC2_PA4_FPOP_WE                        = 0x0000013f,
    PH_PERF_SEL_SC2_PA4_EOP_WE                         = 0x00000140,
    PH_PERF_SEL_SC2_PA4_DATA_FIFO_EOP_RD               = 0x00000141,
    PH_PERF_SEL_SC2_PA4_EOPG_WE                        = 0x00000142,
    PH_PERF_SEL_SC2_PA4_DEALLOC_WE                     = 0x00000143,
    PH_PERF_SEL_SC2_PA5_DATA_FIFO_RD                   = 0x00000144,
    PH_PERF_SEL_SC2_PA5_DATA_FIFO_WE                   = 0x00000145,
    PH_PERF_SEL_SC2_PA5_FIFO_EMPTY                     = 0x00000146,
    PH_PERF_SEL_SC2_PA5_FIFO_FULL                      = 0x00000147,
    PH_PERF_SEL_SC2_PA5_NULL_WE                        = 0x00000148,
    PH_PERF_SEL_SC2_PA5_EVENT_WE                       = 0x00000149,
    PH_PERF_SEL_SC2_PA5_FPOV_WE                        = 0x0000014a,
    PH_PERF_SEL_SC2_PA5_FPOP_WE                        = 0x0000014b,
    PH_PERF_SEL_SC2_PA5_EOP_WE                         = 0x0000014c,
    PH_PERF_SEL_SC2_PA5_DATA_FIFO_EOP_RD               = 0x0000014d,
    PH_PERF_SEL_SC2_PA5_EOPG_WE                        = 0x0000014e,
    PH_PERF_SEL_SC2_PA5_DEALLOC_WE                     = 0x0000014f,
    PH_PERF_SEL_SC2_PA6_DATA_FIFO_RD                   = 0x00000150,
    PH_PERF_SEL_SC2_PA6_DATA_FIFO_WE                   = 0x00000151,
    PH_PERF_SEL_SC2_PA6_FIFO_EMPTY                     = 0x00000152,
    PH_PERF_SEL_SC2_PA6_FIFO_FULL                      = 0x00000153,
    PH_PERF_SEL_SC2_PA6_NULL_WE                        = 0x00000154,
    PH_PERF_SEL_SC2_PA6_EVENT_WE                       = 0x00000155,
    PH_PERF_SEL_SC2_PA6_FPOV_WE                        = 0x00000156,
    PH_PERF_SEL_SC2_PA6_FPOP_WE                        = 0x00000157,
    PH_PERF_SEL_SC2_PA6_EOP_WE                         = 0x00000158,
    PH_PERF_SEL_SC2_PA6_DATA_FIFO_EOP_RD               = 0x00000159,
    PH_PERF_SEL_SC2_PA6_EOPG_WE                        = 0x0000015a,
    PH_PERF_SEL_SC2_PA6_DEALLOC_WE                     = 0x0000015b,
    PH_PERF_SEL_SC2_PA7_DATA_FIFO_RD                   = 0x0000015c,
    PH_PERF_SEL_SC2_PA7_DATA_FIFO_WE                   = 0x0000015d,
    PH_PERF_SEL_SC2_PA7_FIFO_EMPTY                     = 0x0000015e,
    PH_PERF_SEL_SC2_PA7_FIFO_FULL                      = 0x0000015f,
    PH_PERF_SEL_SC2_PA7_NULL_WE                        = 0x00000160,
    PH_PERF_SEL_SC2_PA7_EVENT_WE                       = 0x00000161,
    PH_PERF_SEL_SC2_PA7_FPOV_WE                        = 0x00000162,
    PH_PERF_SEL_SC2_PA7_FPOP_WE                        = 0x00000163,
    PH_PERF_SEL_SC2_PA7_EOP_WE                         = 0x00000164,
    PH_PERF_SEL_SC2_PA7_DATA_FIFO_EOP_RD               = 0x00000165,
    PH_PERF_SEL_SC2_PA7_EOPG_WE                        = 0x00000166,
    PH_PERF_SEL_SC2_PA7_DEALLOC_WE                     = 0x00000167,
    PH_PERF_SEL_SC3_SRPS_WINDOW_VALID                  = 0x00000168,
    PH_PERF_SEL_SC3_ARB_XFC_ALL_EVENT_OR_PRIM_CYCLES   = 0x00000169,
    PH_PERF_SEL_SC3_ARB_XFC_ONLY_PRIM_CYCLES           = 0x0000016a,
    PH_PERF_SEL_SC3_ARB_XFC_ONLY_ONE_INC_PER_PRIM      = 0x0000016b,
    PH_PERF_SEL_SC3_ARB_STALLED_FROM_BELOW             = 0x0000016c,
    PH_PERF_SEL_SC3_ARB_STARVED_FROM_ABOVE             = 0x0000016d,
    PH_PERF_SEL_SC3_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_NOT_EMPTY = 0x0000016e,
    PH_PERF_SEL_SC3_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_FULL = 0x0000016f,
    PH_PERF_SEL_SC3_ARB_BUSY                           = 0x00000170,
    PH_PERF_SEL_SC3_ARB_PA_BUSY_SOP                    = 0x00000171,
    PH_PERF_SEL_SC3_ARB_EOP_POP_SYNC_POP               = 0x00000172,
    PH_PERF_SEL_SC3_ARB_EVENT_SYNC_POP                 = 0x00000173,
    PH_PERF_SEL_SC3_PS_ENG_MULTICYCLE_BUBBLE           = 0x00000174,
    PH_PERF_SEL_SC3_EOP_SYNC_WINDOW                    = 0x00000175,
    PH_PERF_SEL_SC3_BUSY_PROCESSING_MULTICYCLE_PRIM    = 0x00000176,
    PH_PERF_SEL_SC3_BUSY_CNT_NOT_ZERO                  = 0x00000177,
    PH_PERF_SEL_SC3_SEND                               = 0x00000178,
    PH_PERF_SEL_SC3_CREDIT_AT_ZERO_WITH_PENDING_SEND   = 0x00000179,
    PH_PERF_SEL_SC3_CREDIT_AT_MAX                      = 0x0000017a,
    PH_PERF_SEL_SC3_CREDIT_AT_MAX_NO_PENDING_SEND      = 0x0000017b,
    PH_PERF_SEL_SC3_GFX_PIPE0_TO_1_TRANSITION          = 0x0000017c,
    PH_PERF_SEL_SC3_GFX_PIPE1_TO_0_TRANSITION          = 0x0000017d,
    PH_PERF_SEL_SC3_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION = 0x0000017e,
    PH_PERF_SEL_SC3_GFX_PIPE_EVENT_PROVOKED_TRANSITION = 0x0000017f,
    PH_PERF_SEL_SC3_PA0_DATA_FIFO_RD                   = 0x00000180,
    PH_PERF_SEL_SC3_PA0_DATA_FIFO_WE                   = 0x00000181,
    PH_PERF_SEL_SC3_PA0_FIFO_EMPTY                     = 0x00000182,
    PH_PERF_SEL_SC3_PA0_FIFO_FULL                      = 0x00000183,
    PH_PERF_SEL_SC3_PA0_NULL_WE                        = 0x00000184,
    PH_PERF_SEL_SC3_PA0_EVENT_WE                       = 0x00000185,
    PH_PERF_SEL_SC3_PA0_FPOV_WE                        = 0x00000186,
    PH_PERF_SEL_SC3_PA0_FPOP_WE                        = 0x00000187,
    PH_PERF_SEL_SC3_PA0_EOP_WE                         = 0x00000188,
    PH_PERF_SEL_SC3_PA0_DATA_FIFO_EOP_RD               = 0x00000189,
    PH_PERF_SEL_SC3_PA0_EOPG_WE                        = 0x0000018a,
    PH_PERF_SEL_SC3_PA0_DEALLOC_WE                     = 0x0000018b,
    PH_PERF_SEL_SC3_PA1_DATA_FIFO_RD                   = 0x0000018c,
    PH_PERF_SEL_SC3_PA1_DATA_FIFO_WE                   = 0x0000018d,
    PH_PERF_SEL_SC3_PA1_FIFO_EMPTY                     = 0x0000018e,
    PH_PERF_SEL_SC3_PA1_FIFO_FULL                      = 0x0000018f,
    PH_PERF_SEL_SC3_PA1_NULL_WE                        = 0x00000190,
    PH_PERF_SEL_SC3_PA1_EVENT_WE                       = 0x00000191,
    PH_PERF_SEL_SC3_PA1_FPOV_WE                        = 0x00000192,
    PH_PERF_SEL_SC3_PA1_FPOP_WE                        = 0x00000193,
    PH_PERF_SEL_SC3_PA1_EOP_WE                         = 0x00000194,
    PH_PERF_SEL_SC3_PA1_DATA_FIFO_EOP_RD               = 0x00000195,
    PH_PERF_SEL_SC3_PA1_EOPG_WE                        = 0x00000196,
    PH_PERF_SEL_SC3_PA1_DEALLOC_WE                     = 0x00000197,
    PH_PERF_SEL_SC3_PA2_DATA_FIFO_RD                   = 0x00000198,
    PH_PERF_SEL_SC3_PA2_DATA_FIFO_WE                   = 0x00000199,
    PH_PERF_SEL_SC3_PA2_FIFO_EMPTY                     = 0x0000019a,
    PH_PERF_SEL_SC3_PA2_FIFO_FULL                      = 0x0000019b,
    PH_PERF_SEL_SC3_PA2_NULL_WE                        = 0x0000019c,
    PH_PERF_SEL_SC3_PA2_EVENT_WE                       = 0x0000019d,
    PH_PERF_SEL_SC3_PA2_FPOV_WE                        = 0x0000019e,
    PH_PERF_SEL_SC3_PA2_FPOP_WE                        = 0x0000019f,
    PH_PERF_SEL_SC3_PA2_EOP_WE                         = 0x000001a0,
    PH_PERF_SEL_SC3_PA2_DATA_FIFO_EOP_RD               = 0x000001a1,
    PH_PERF_SEL_SC3_PA2_EOPG_WE                        = 0x000001a2,
    PH_PERF_SEL_SC3_PA2_DEALLOC_WE                     = 0x000001a3,
    PH_PERF_SEL_SC3_PA3_DATA_FIFO_RD                   = 0x000001a4,
    PH_PERF_SEL_SC3_PA3_DATA_FIFO_WE                   = 0x000001a5,
    PH_PERF_SEL_SC3_PA3_FIFO_EMPTY                     = 0x000001a6,
    PH_PERF_SEL_SC3_PA3_FIFO_FULL                      = 0x000001a7,
    PH_PERF_SEL_SC3_PA3_NULL_WE                        = 0x000001a8,
    PH_PERF_SEL_SC3_PA3_EVENT_WE                       = 0x000001a9,
    PH_PERF_SEL_SC3_PA3_FPOV_WE                        = 0x000001aa,
    PH_PERF_SEL_SC3_PA3_FPOP_WE                        = 0x000001ab,
    PH_PERF_SEL_SC3_PA3_EOP_WE                         = 0x000001ac,
    PH_PERF_SEL_SC3_PA3_DATA_FIFO_EOP_RD               = 0x000001ad,
    PH_PERF_SEL_SC3_PA3_EOPG_WE                        = 0x000001ae,
    PH_PERF_SEL_SC3_PA3_DEALLOC_WE                     = 0x000001af,
    PH_PERF_SEL_SC3_PA4_DATA_FIFO_RD                   = 0x000001b0,
    PH_PERF_SEL_SC3_PA4_DATA_FIFO_WE                   = 0x000001b1,
    PH_PERF_SEL_SC3_PA4_FIFO_EMPTY                     = 0x000001b2,
    PH_PERF_SEL_SC3_PA4_FIFO_FULL                      = 0x000001b3,
    PH_PERF_SEL_SC3_PA4_NULL_WE                        = 0x000001b4,
    PH_PERF_SEL_SC3_PA4_EVENT_WE                       = 0x000001b5,
    PH_PERF_SEL_SC3_PA4_FPOV_WE                        = 0x000001b6,
    PH_PERF_SEL_SC3_PA4_FPOP_WE                        = 0x000001b7,
    PH_PERF_SEL_SC3_PA4_EOP_WE                         = 0x000001b8,
    PH_PERF_SEL_SC3_PA4_DATA_FIFO_EOP_RD               = 0x000001b9,
    PH_PERF_SEL_SC3_PA4_EOPG_WE                        = 0x000001ba,
    PH_PERF_SEL_SC3_PA4_DEALLOC_WE                     = 0x000001bb,
    PH_PERF_SEL_SC3_PA5_DATA_FIFO_RD                   = 0x000001bc,
    PH_PERF_SEL_SC3_PA5_DATA_FIFO_WE                   = 0x000001bd,
    PH_PERF_SEL_SC3_PA5_FIFO_EMPTY                     = 0x000001be,
    PH_PERF_SEL_SC3_PA5_FIFO_FULL                      = 0x000001bf,
    PH_PERF_SEL_SC3_PA5_NULL_WE                        = 0x000001c0,
    PH_PERF_SEL_SC3_PA5_EVENT_WE                       = 0x000001c1,
    PH_PERF_SEL_SC3_PA5_FPOV_WE                        = 0x000001c2,
    PH_PERF_SEL_SC3_PA5_FPOP_WE                        = 0x000001c3,
    PH_PERF_SEL_SC3_PA5_EOP_WE                         = 0x000001c4,
    PH_PERF_SEL_SC3_PA5_DATA_FIFO_EOP_RD               = 0x000001c5,
    PH_PERF_SEL_SC3_PA5_EOPG_WE                        = 0x000001c6,
    PH_PERF_SEL_SC3_PA5_DEALLOC_WE                     = 0x000001c7,
    PH_PERF_SEL_SC3_PA6_DATA_FIFO_RD                   = 0x000001c8,
    PH_PERF_SEL_SC3_PA6_DATA_FIFO_WE                   = 0x000001c9,
    PH_PERF_SEL_SC3_PA6_FIFO_EMPTY                     = 0x000001ca,
    PH_PERF_SEL_SC3_PA6_FIFO_FULL                      = 0x000001cb,
    PH_PERF_SEL_SC3_PA6_NULL_WE                        = 0x000001cc,
    PH_PERF_SEL_SC3_PA6_EVENT_WE                       = 0x000001cd,
    PH_PERF_SEL_SC3_PA6_FPOV_WE                        = 0x000001ce,
    PH_PERF_SEL_SC3_PA6_FPOP_WE                        = 0x000001cf,
    PH_PERF_SEL_SC3_PA6_EOP_WE                         = 0x000001d0,
    PH_PERF_SEL_SC3_PA6_DATA_FIFO_EOP_RD               = 0x000001d1,
    PH_PERF_SEL_SC3_PA6_EOPG_WE                        = 0x000001d2,
    PH_PERF_SEL_SC3_PA6_DEALLOC_WE                     = 0x000001d3,
    PH_PERF_SEL_SC3_PA7_DATA_FIFO_RD                   = 0x000001d4,
    PH_PERF_SEL_SC3_PA7_DATA_FIFO_WE                   = 0x000001d5,
    PH_PERF_SEL_SC3_PA7_FIFO_EMPTY                     = 0x000001d6,
    PH_PERF_SEL_SC3_PA7_FIFO_FULL                      = 0x000001d7,
    PH_PERF_SEL_SC3_PA7_NULL_WE                        = 0x000001d8,
    PH_PERF_SEL_SC3_PA7_EVENT_WE                       = 0x000001d9,
    PH_PERF_SEL_SC3_PA7_FPOV_WE                        = 0x000001da,
    PH_PERF_SEL_SC3_PA7_FPOP_WE                        = 0x000001db,
    PH_PERF_SEL_SC3_PA7_EOP_WE                         = 0x000001dc,
    PH_PERF_SEL_SC3_PA7_DATA_FIFO_EOP_RD               = 0x000001dd,
    PH_PERF_SEL_SC3_PA7_EOPG_WE                        = 0x000001de,
    PH_PERF_SEL_SC3_PA7_DEALLOC_WE                     = 0x000001df,
    PH_PERF_SEL_SC4_SRPS_WINDOW_VALID                  = 0x000001e0,
    PH_PERF_SEL_SC4_ARB_XFC_ALL_EVENT_OR_PRIM_CYCLES   = 0x000001e1,
    PH_PERF_SEL_SC4_ARB_XFC_ONLY_PRIM_CYCLES           = 0x000001e2,
    PH_PERF_SEL_SC4_ARB_XFC_ONLY_ONE_INC_PER_PRIM      = 0x000001e3,
    PH_PERF_SEL_SC4_ARB_STALLED_FROM_BELOW             = 0x000001e4,
    PH_PERF_SEL_SC4_ARB_STARVED_FROM_ABOVE             = 0x000001e5,
    PH_PERF_SEL_SC4_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_NOT_EMPTY = 0x000001e6,
    PH_PERF_SEL_SC4_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_FULL = 0x000001e7,
    PH_PERF_SEL_SC4_ARB_BUSY                           = 0x000001e8,
    PH_PERF_SEL_SC4_ARB_PA_BUSY_SOP                    = 0x000001e9,
    PH_PERF_SEL_SC4_ARB_EOP_POP_SYNC_POP               = 0x000001ea,
    PH_PERF_SEL_SC4_ARB_EVENT_SYNC_POP                 = 0x000001eb,
    PH_PERF_SEL_SC4_PS_ENG_MULTICYCLE_BUBBLE           = 0x000001ec,
    PH_PERF_SEL_SC4_EOP_SYNC_WINDOW                    = 0x000001ed,
    PH_PERF_SEL_SC4_BUSY_PROCESSING_MULTICYCLE_PRIM    = 0x000001ee,
    PH_PERF_SEL_SC4_BUSY_CNT_NOT_ZERO                  = 0x000001ef,
    PH_PERF_SEL_SC4_SEND                               = 0x000001f0,
    PH_PERF_SEL_SC4_CREDIT_AT_ZERO_WITH_PENDING_SEND   = 0x000001f1,
    PH_PERF_SEL_SC4_CREDIT_AT_MAX                      = 0x000001f2,
    PH_PERF_SEL_SC4_CREDIT_AT_MAX_NO_PENDING_SEND      = 0x000001f3,
    PH_PERF_SEL_SC4_GFX_PIPE0_TO_1_TRANSITION          = 0x000001f4,
    PH_PERF_SEL_SC4_GFX_PIPE1_TO_0_TRANSITION          = 0x000001f5,
    PH_PERF_SEL_SC4_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION = 0x000001f6,
    PH_PERF_SEL_SC4_GFX_PIPE_EVENT_PROVOKED_TRANSITION = 0x000001f7,
    PH_PERF_SEL_SC4_PA0_DATA_FIFO_RD                   = 0x000001f8,
    PH_PERF_SEL_SC4_PA0_DATA_FIFO_WE                   = 0x000001f9,
    PH_PERF_SEL_SC4_PA0_FIFO_EMPTY                     = 0x000001fa,
    PH_PERF_SEL_SC4_PA0_FIFO_FULL                      = 0x000001fb,
    PH_PERF_SEL_SC4_PA0_NULL_WE                        = 0x000001fc,
    PH_PERF_SEL_SC4_PA0_EVENT_WE                       = 0x000001fd,
    PH_PERF_SEL_SC4_PA0_FPOV_WE                        = 0x000001fe,
    PH_PERF_SEL_SC4_PA0_FPOP_WE                        = 0x000001ff,
    PH_PERF_SEL_SC4_PA0_EOP_WE                         = 0x00000200,
    PH_PERF_SEL_SC4_PA0_DATA_FIFO_EOP_RD               = 0x00000201,
    PH_PERF_SEL_SC4_PA0_EOPG_WE                        = 0x00000202,
    PH_PERF_SEL_SC4_PA0_DEALLOC_WE                     = 0x00000203,
    PH_PERF_SEL_SC4_PA1_DATA_FIFO_RD                   = 0x00000204,
    PH_PERF_SEL_SC4_PA1_DATA_FIFO_WE                   = 0x00000205,
    PH_PERF_SEL_SC4_PA1_FIFO_EMPTY                     = 0x00000206,
    PH_PERF_SEL_SC4_PA1_FIFO_FULL                      = 0x00000207,
    PH_PERF_SEL_SC4_PA1_NULL_WE                        = 0x00000208,
    PH_PERF_SEL_SC4_PA1_EVENT_WE                       = 0x00000209,
    PH_PERF_SEL_SC4_PA1_FPOV_WE                        = 0x0000020a,
    PH_PERF_SEL_SC4_PA1_FPOP_WE                        = 0x0000020b,
    PH_PERF_SEL_SC4_PA1_EOP_WE                         = 0x0000020c,
    PH_PERF_SEL_SC4_PA1_DATA_FIFO_EOP_RD               = 0x0000020d,
    PH_PERF_SEL_SC4_PA1_EOPG_WE                        = 0x0000020e,
    PH_PERF_SEL_SC4_PA1_DEALLOC_WE                     = 0x0000020f,
    PH_PERF_SEL_SC4_PA2_DATA_FIFO_RD                   = 0x00000210,
    PH_PERF_SEL_SC4_PA2_DATA_FIFO_WE                   = 0x00000211,
    PH_PERF_SEL_SC4_PA2_FIFO_EMPTY                     = 0x00000212,
    PH_PERF_SEL_SC4_PA2_FIFO_FULL                      = 0x00000213,
    PH_PERF_SEL_SC4_PA2_NULL_WE                        = 0x00000214,
    PH_PERF_SEL_SC4_PA2_EVENT_WE                       = 0x00000215,
    PH_PERF_SEL_SC4_PA2_FPOV_WE                        = 0x00000216,
    PH_PERF_SEL_SC4_PA2_FPOP_WE                        = 0x00000217,
    PH_PERF_SEL_SC4_PA2_EOP_WE                         = 0x00000218,
    PH_PERF_SEL_SC4_PA2_DATA_FIFO_EOP_RD               = 0x00000219,
    PH_PERF_SEL_SC4_PA2_EOPG_WE                        = 0x0000021a,
    PH_PERF_SEL_SC4_PA2_DEALLOC_WE                     = 0x0000021b,
    PH_PERF_SEL_SC4_PA3_DATA_FIFO_RD                   = 0x0000021c,
    PH_PERF_SEL_SC4_PA3_DATA_FIFO_WE                   = 0x0000021d,
    PH_PERF_SEL_SC4_PA3_FIFO_EMPTY                     = 0x0000021e,
    PH_PERF_SEL_SC4_PA3_FIFO_FULL                      = 0x0000021f,
    PH_PERF_SEL_SC4_PA3_NULL_WE                        = 0x00000220,
    PH_PERF_SEL_SC4_PA3_EVENT_WE                       = 0x00000221,
    PH_PERF_SEL_SC4_PA3_FPOV_WE                        = 0x00000222,
    PH_PERF_SEL_SC4_PA3_FPOP_WE                        = 0x00000223,
    PH_PERF_SEL_SC4_PA3_EOP_WE                         = 0x00000224,
    PH_PERF_SEL_SC4_PA3_DATA_FIFO_EOP_RD               = 0x00000225,
    PH_PERF_SEL_SC4_PA3_EOPG_WE                        = 0x00000226,
    PH_PERF_SEL_SC4_PA3_DEALLOC_WE                     = 0x00000227,
    PH_PERF_SEL_SC4_PA4_DATA_FIFO_RD                   = 0x00000228,
    PH_PERF_SEL_SC4_PA4_DATA_FIFO_WE                   = 0x00000229,
    PH_PERF_SEL_SC4_PA4_FIFO_EMPTY                     = 0x0000022a,
    PH_PERF_SEL_SC4_PA4_FIFO_FULL                      = 0x0000022b,
    PH_PERF_SEL_SC4_PA4_NULL_WE                        = 0x0000022c,
    PH_PERF_SEL_SC4_PA4_EVENT_WE                       = 0x0000022d,
    PH_PERF_SEL_SC4_PA4_FPOV_WE                        = 0x0000022e,
    PH_PERF_SEL_SC4_PA4_FPOP_WE                        = 0x0000022f,
    PH_PERF_SEL_SC4_PA4_EOP_WE                         = 0x00000230,
    PH_PERF_SEL_SC4_PA4_DATA_FIFO_EOP_RD               = 0x00000231,
    PH_PERF_SEL_SC4_PA4_EOPG_WE                        = 0x00000232,
    PH_PERF_SEL_SC4_PA4_DEALLOC_WE                     = 0x00000233,
    PH_PERF_SEL_SC4_PA5_DATA_FIFO_RD                   = 0x00000234,
    PH_PERF_SEL_SC4_PA5_DATA_FIFO_WE                   = 0x00000235,
    PH_PERF_SEL_SC4_PA5_FIFO_EMPTY                     = 0x00000236,
    PH_PERF_SEL_SC4_PA5_FIFO_FULL                      = 0x00000237,
    PH_PERF_SEL_SC4_PA5_NULL_WE                        = 0x00000238,
    PH_PERF_SEL_SC4_PA5_EVENT_WE                       = 0x00000239,
    PH_PERF_SEL_SC4_PA5_FPOV_WE                        = 0x0000023a,
    PH_PERF_SEL_SC4_PA5_FPOP_WE                        = 0x0000023b,
    PH_PERF_SEL_SC4_PA5_EOP_WE                         = 0x0000023c,
    PH_PERF_SEL_SC4_PA5_DATA_FIFO_EOP_RD               = 0x0000023d,
    PH_PERF_SEL_SC4_PA5_EOPG_WE                        = 0x0000023e,
    PH_PERF_SEL_SC4_PA5_DEALLOC_WE                     = 0x0000023f,
    PH_PERF_SEL_SC4_PA6_DATA_FIFO_RD                   = 0x00000240,
    PH_PERF_SEL_SC4_PA6_DATA_FIFO_WE                   = 0x00000241,
    PH_PERF_SEL_SC4_PA6_FIFO_EMPTY                     = 0x00000242,
    PH_PERF_SEL_SC4_PA6_FIFO_FULL                      = 0x00000243,
    PH_PERF_SEL_SC4_PA6_NULL_WE                        = 0x00000244,
    PH_PERF_SEL_SC4_PA6_EVENT_WE                       = 0x00000245,
    PH_PERF_SEL_SC4_PA6_FPOV_WE                        = 0x00000246,
    PH_PERF_SEL_SC4_PA6_FPOP_WE                        = 0x00000247,
    PH_PERF_SEL_SC4_PA6_EOP_WE                         = 0x00000248,
    PH_PERF_SEL_SC4_PA6_DATA_FIFO_EOP_RD               = 0x00000249,
    PH_PERF_SEL_SC4_PA6_EOPG_WE                        = 0x0000024a,
    PH_PERF_SEL_SC4_PA6_DEALLOC_WE                     = 0x0000024b,
    PH_PERF_SEL_SC4_PA7_DATA_FIFO_RD                   = 0x0000024c,
    PH_PERF_SEL_SC4_PA7_DATA_FIFO_WE                   = 0x0000024d,
    PH_PERF_SEL_SC4_PA7_FIFO_EMPTY                     = 0x0000024e,
    PH_PERF_SEL_SC4_PA7_FIFO_FULL                      = 0x0000024f,
    PH_PERF_SEL_SC4_PA7_NULL_WE                        = 0x00000250,
    PH_PERF_SEL_SC4_PA7_EVENT_WE                       = 0x00000251,
    PH_PERF_SEL_SC4_PA7_FPOV_WE                        = 0x00000252,
    PH_PERF_SEL_SC4_PA7_FPOP_WE                        = 0x00000253,
    PH_PERF_SEL_SC4_PA7_EOP_WE                         = 0x00000254,
    PH_PERF_SEL_SC4_PA7_DATA_FIFO_EOP_RD               = 0x00000255,
    PH_PERF_SEL_SC4_PA7_EOPG_WE                        = 0x00000256,
    PH_PERF_SEL_SC4_PA7_DEALLOC_WE                     = 0x00000257,
    PH_PERF_SEL_SC5_SRPS_WINDOW_VALID                  = 0x00000258,
    PH_PERF_SEL_SC5_ARB_XFC_ALL_EVENT_OR_PRIM_CYCLES   = 0x00000259,
    PH_PERF_SEL_SC5_ARB_XFC_ONLY_PRIM_CYCLES           = 0x0000025a,
    PH_PERF_SEL_SC5_ARB_XFC_ONLY_ONE_INC_PER_PRIM      = 0x0000025b,
    PH_PERF_SEL_SC5_ARB_STALLED_FROM_BELOW             = 0x0000025c,
    PH_PERF_SEL_SC5_ARB_STARVED_FROM_ABOVE             = 0x0000025d,
    PH_PERF_SEL_SC5_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_NOT_EMPTY = 0x0000025e,
    PH_PERF_SEL_SC5_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_FULL = 0x0000025f,
    PH_PERF_SEL_SC5_ARB_BUSY                           = 0x00000260,
    PH_PERF_SEL_SC5_ARB_PA_BUSY_SOP                    = 0x00000261,
    PH_PERF_SEL_SC5_ARB_EOP_POP_SYNC_POP               = 0x00000262,
    PH_PERF_SEL_SC5_ARB_EVENT_SYNC_POP                 = 0x00000263,
    PH_PERF_SEL_SC5_PS_ENG_MULTICYCLE_BUBBLE           = 0x00000264,
    PH_PERF_SEL_SC5_EOP_SYNC_WINDOW                    = 0x00000265,
    PH_PERF_SEL_SC5_BUSY_PROCESSING_MULTICYCLE_PRIM    = 0x00000266,
    PH_PERF_SEL_SC5_BUSY_CNT_NOT_ZERO                  = 0x00000267,
    PH_PERF_SEL_SC5_SEND                               = 0x00000268,
    PH_PERF_SEL_SC5_CREDIT_AT_ZERO_WITH_PENDING_SEND   = 0x00000269,
    PH_PERF_SEL_SC5_CREDIT_AT_MAX                      = 0x0000026a,
    PH_PERF_SEL_SC5_CREDIT_AT_MAX_NO_PENDING_SEND      = 0x0000026b,
    PH_PERF_SEL_SC5_GFX_PIPE0_TO_1_TRANSITION          = 0x0000026c,
    PH_PERF_SEL_SC5_GFX_PIPE1_TO_0_TRANSITION          = 0x0000026d,
    PH_PERF_SEL_SC5_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION = 0x0000026e,
    PH_PERF_SEL_SC5_GFX_PIPE_EVENT_PROVOKED_TRANSITION = 0x0000026f,
    PH_PERF_SEL_SC5_PA0_DATA_FIFO_RD                   = 0x00000270,
    PH_PERF_SEL_SC5_PA0_DATA_FIFO_WE                   = 0x00000271,
    PH_PERF_SEL_SC5_PA0_FIFO_EMPTY                     = 0x00000272,
    PH_PERF_SEL_SC5_PA0_FIFO_FULL                      = 0x00000273,
    PH_PERF_SEL_SC5_PA0_NULL_WE                        = 0x00000274,
    PH_PERF_SEL_SC5_PA0_EVENT_WE                       = 0x00000275,
    PH_PERF_SEL_SC5_PA0_FPOV_WE                        = 0x00000276,
    PH_PERF_SEL_SC5_PA0_FPOP_WE                        = 0x00000277,
    PH_PERF_SEL_SC5_PA0_EOP_WE                         = 0x00000278,
    PH_PERF_SEL_SC5_PA0_DATA_FIFO_EOP_RD               = 0x00000279,
    PH_PERF_SEL_SC5_PA0_EOPG_WE                        = 0x0000027a,
    PH_PERF_SEL_SC5_PA0_DEALLOC_WE                     = 0x0000027b,
    PH_PERF_SEL_SC5_PA1_DATA_FIFO_RD                   = 0x0000027c,
    PH_PERF_SEL_SC5_PA1_DATA_FIFO_WE                   = 0x0000027d,
    PH_PERF_SEL_SC5_PA1_FIFO_EMPTY                     = 0x0000027e,
    PH_PERF_SEL_SC5_PA1_FIFO_FULL                      = 0x0000027f,
    PH_PERF_SEL_SC5_PA1_NULL_WE                        = 0x00000280,
    PH_PERF_SEL_SC5_PA1_EVENT_WE                       = 0x00000281,
    PH_PERF_SEL_SC5_PA1_FPOV_WE                        = 0x00000282,
    PH_PERF_SEL_SC5_PA1_FPOP_WE                        = 0x00000283,
    PH_PERF_SEL_SC5_PA1_EOP_WE                         = 0x00000284,
    PH_PERF_SEL_SC5_PA1_DATA_FIFO_EOP_RD               = 0x00000285,
    PH_PERF_SEL_SC5_PA1_EOPG_WE                        = 0x00000286,
    PH_PERF_SEL_SC5_PA1_DEALLOC_WE                     = 0x00000287,
    PH_PERF_SEL_SC5_PA2_DATA_FIFO_RD                   = 0x00000288,
    PH_PERF_SEL_SC5_PA2_DATA_FIFO_WE                   = 0x00000289,
    PH_PERF_SEL_SC5_PA2_FIFO_EMPTY                     = 0x0000028a,
    PH_PERF_SEL_SC5_PA2_FIFO_FULL                      = 0x0000028b,
    PH_PERF_SEL_SC5_PA2_NULL_WE                        = 0x0000028c,
    PH_PERF_SEL_SC5_PA2_EVENT_WE                       = 0x0000028d,
    PH_PERF_SEL_SC5_PA2_FPOV_WE                        = 0x0000028e,
    PH_PERF_SEL_SC5_PA2_FPOP_WE                        = 0x0000028f,
    PH_PERF_SEL_SC5_PA2_EOP_WE                         = 0x00000290,
    PH_PERF_SEL_SC5_PA2_DATA_FIFO_EOP_RD               = 0x00000291,
    PH_PERF_SEL_SC5_PA2_EOPG_WE                        = 0x00000292,
    PH_PERF_SEL_SC5_PA2_DEALLOC_WE                     = 0x00000293,
    PH_PERF_SEL_SC5_PA3_DATA_FIFO_RD                   = 0x00000294,
    PH_PERF_SEL_SC5_PA3_DATA_FIFO_WE                   = 0x00000295,
    PH_PERF_SEL_SC5_PA3_FIFO_EMPTY                     = 0x00000296,
    PH_PERF_SEL_SC5_PA3_FIFO_FULL                      = 0x00000297,
    PH_PERF_SEL_SC5_PA3_NULL_WE                        = 0x00000298,
    PH_PERF_SEL_SC5_PA3_EVENT_WE                       = 0x00000299,
    PH_PERF_SEL_SC5_PA3_FPOV_WE                        = 0x0000029a,
    PH_PERF_SEL_SC5_PA3_FPOP_WE                        = 0x0000029b,
    PH_PERF_SEL_SC5_PA3_EOP_WE                         = 0x0000029c,
    PH_PERF_SEL_SC5_PA3_DATA_FIFO_EOP_RD               = 0x0000029d,
    PH_PERF_SEL_SC5_PA3_EOPG_WE                        = 0x0000029e,
    PH_PERF_SEL_SC5_PA3_DEALLOC_WE                     = 0x0000029f,
    PH_PERF_SEL_SC5_PA4_DATA_FIFO_RD                   = 0x000002a0,
    PH_PERF_SEL_SC5_PA4_DATA_FIFO_WE                   = 0x000002a1,
    PH_PERF_SEL_SC5_PA4_FIFO_EMPTY                     = 0x000002a2,
    PH_PERF_SEL_SC5_PA4_FIFO_FULL                      = 0x000002a3,
    PH_PERF_SEL_SC5_PA4_NULL_WE                        = 0x000002a4,
    PH_PERF_SEL_SC5_PA4_EVENT_WE                       = 0x000002a5,
    PH_PERF_SEL_SC5_PA4_FPOV_WE                        = 0x000002a6,
    PH_PERF_SEL_SC5_PA4_FPOP_WE                        = 0x000002a7,
    PH_PERF_SEL_SC5_PA4_EOP_WE                         = 0x000002a8,
    PH_PERF_SEL_SC5_PA4_DATA_FIFO_EOP_RD               = 0x000002a9,
    PH_PERF_SEL_SC5_PA4_EOPG_WE                        = 0x000002aa,
    PH_PERF_SEL_SC5_PA4_DEALLOC_WE                     = 0x000002ab,
    PH_PERF_SEL_SC5_PA5_DATA_FIFO_RD                   = 0x000002ac,
    PH_PERF_SEL_SC5_PA5_DATA_FIFO_WE                   = 0x000002ad,
    PH_PERF_SEL_SC5_PA5_FIFO_EMPTY                     = 0x000002ae,
    PH_PERF_SEL_SC5_PA5_FIFO_FULL                      = 0x000002af,
    PH_PERF_SEL_SC5_PA5_NULL_WE                        = 0x000002b0,
    PH_PERF_SEL_SC5_PA5_EVENT_WE                       = 0x000002b1,
    PH_PERF_SEL_SC5_PA5_FPOV_WE                        = 0x000002b2,
    PH_PERF_SEL_SC5_PA5_FPOP_WE                        = 0x000002b3,
    PH_PERF_SEL_SC5_PA5_EOP_WE                         = 0x000002b4,
    PH_PERF_SEL_SC5_PA5_DATA_FIFO_EOP_RD               = 0x000002b5,
    PH_PERF_SEL_SC5_PA5_EOPG_WE                        = 0x000002b6,
    PH_PERF_SEL_SC5_PA5_DEALLOC_WE                     = 0x000002b7,
    PH_PERF_SEL_SC5_PA6_DATA_FIFO_RD                   = 0x000002b8,
    PH_PERF_SEL_SC5_PA6_DATA_FIFO_WE                   = 0x000002b9,
    PH_PERF_SEL_SC5_PA6_FIFO_EMPTY                     = 0x000002ba,
    PH_PERF_SEL_SC5_PA6_FIFO_FULL                      = 0x000002bb,
    PH_PERF_SEL_SC5_PA6_NULL_WE                        = 0x000002bc,
    PH_PERF_SEL_SC5_PA6_EVENT_WE                       = 0x000002bd,
    PH_PERF_SEL_SC5_PA6_FPOV_WE                        = 0x000002be,
    PH_PERF_SEL_SC5_PA6_FPOP_WE                        = 0x000002bf,
    PH_PERF_SEL_SC5_PA6_EOP_WE                         = 0x000002c0,
    PH_PERF_SEL_SC5_PA6_DATA_FIFO_EOP_RD               = 0x000002c1,
    PH_PERF_SEL_SC5_PA6_EOPG_WE                        = 0x000002c2,
    PH_PERF_SEL_SC5_PA6_DEALLOC_WE                     = 0x000002c3,
    PH_PERF_SEL_SC5_PA7_DATA_FIFO_RD                   = 0x000002c4,
    PH_PERF_SEL_SC5_PA7_DATA_FIFO_WE                   = 0x000002c5,
    PH_PERF_SEL_SC5_PA7_FIFO_EMPTY                     = 0x000002c6,
    PH_PERF_SEL_SC5_PA7_FIFO_FULL                      = 0x000002c7,
    PH_PERF_SEL_SC5_PA7_NULL_WE                        = 0x000002c8,
    PH_PERF_SEL_SC5_PA7_EVENT_WE                       = 0x000002c9,
    PH_PERF_SEL_SC5_PA7_FPOV_WE                        = 0x000002ca,
    PH_PERF_SEL_SC5_PA7_FPOP_WE                        = 0x000002cb,
    PH_PERF_SEL_SC5_PA7_EOP_WE                         = 0x000002cc,
    PH_PERF_SEL_SC5_PA7_DATA_FIFO_EOP_RD               = 0x000002cd,
    PH_PERF_SEL_SC5_PA7_EOPG_WE                        = 0x000002ce,
    PH_PERF_SEL_SC5_PA7_DEALLOC_WE                     = 0x000002cf,
    PH_PERF_SEL_SC6_SRPS_WINDOW_VALID                  = 0x000002d0,
    PH_PERF_SEL_SC6_ARB_XFC_ALL_EVENT_OR_PRIM_CYCLES   = 0x000002d1,
    PH_PERF_SEL_SC6_ARB_XFC_ONLY_PRIM_CYCLES           = 0x000002d2,
    PH_PERF_SEL_SC6_ARB_XFC_ONLY_ONE_INC_PER_PRIM      = 0x000002d3,
    PH_PERF_SEL_SC6_ARB_STALLED_FROM_BELOW             = 0x000002d4,
    PH_PERF_SEL_SC6_ARB_STARVED_FROM_ABOVE             = 0x000002d5,
    PH_PERF_SEL_SC6_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_NOT_EMPTY = 0x000002d6,
    PH_PERF_SEL_SC6_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_FULL = 0x000002d7,
    PH_PERF_SEL_SC6_ARB_BUSY                           = 0x000002d8,
    PH_PERF_SEL_SC6_ARB_PA_BUSY_SOP                    = 0x000002d9,
    PH_PERF_SEL_SC6_ARB_EOP_POP_SYNC_POP               = 0x000002da,
    PH_PERF_SEL_SC6_ARB_EVENT_SYNC_POP                 = 0x000002db,
    PH_PERF_SEL_SC6_PS_ENG_MULTICYCLE_BUBBLE           = 0x000002dc,
    PH_PERF_SEL_SC6_EOP_SYNC_WINDOW                    = 0x000002dd,
    PH_PERF_SEL_SC6_BUSY_PROCESSING_MULTICYCLE_PRIM    = 0x000002de,
    PH_PERF_SEL_SC6_BUSY_CNT_NOT_ZERO                  = 0x000002df,
    PH_PERF_SEL_SC6_SEND                               = 0x000002e0,
    PH_PERF_SEL_SC6_CREDIT_AT_ZERO_WITH_PENDING_SEND   = 0x000002e1,
    PH_PERF_SEL_SC6_CREDIT_AT_MAX                      = 0x000002e2,
    PH_PERF_SEL_SC6_CREDIT_AT_MAX_NO_PENDING_SEND      = 0x000002e3,
    PH_PERF_SEL_SC6_GFX_PIPE0_TO_1_TRANSITION          = 0x000002e4,
    PH_PERF_SEL_SC6_GFX_PIPE1_TO_0_TRANSITION          = 0x000002e5,
    PH_PERF_SEL_SC6_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION = 0x000002e6,
    PH_PERF_SEL_SC6_GFX_PIPE_EVENT_PROVOKED_TRANSITION = 0x000002e7,
    PH_PERF_SEL_SC6_PA0_DATA_FIFO_RD                   = 0x000002e8,
    PH_PERF_SEL_SC6_PA0_DATA_FIFO_WE                   = 0x000002e9,
    PH_PERF_SEL_SC6_PA0_FIFO_EMPTY                     = 0x000002ea,
    PH_PERF_SEL_SC6_PA0_FIFO_FULL                      = 0x000002eb,
    PH_PERF_SEL_SC6_PA0_NULL_WE                        = 0x000002ec,
    PH_PERF_SEL_SC6_PA0_EVENT_WE                       = 0x000002ed,
    PH_PERF_SEL_SC6_PA0_FPOV_WE                        = 0x000002ee,
    PH_PERF_SEL_SC6_PA0_FPOP_WE                        = 0x000002ef,
    PH_PERF_SEL_SC6_PA0_EOP_WE                         = 0x000002f0,
    PH_PERF_SEL_SC6_PA0_DATA_FIFO_EOP_RD               = 0x000002f1,
    PH_PERF_SEL_SC6_PA0_EOPG_WE                        = 0x000002f2,
    PH_PERF_SEL_SC6_PA0_DEALLOC_WE                     = 0x000002f3,
    PH_PERF_SEL_SC6_PA1_DATA_FIFO_RD                   = 0x000002f4,
    PH_PERF_SEL_SC6_PA1_DATA_FIFO_WE                   = 0x000002f5,
    PH_PERF_SEL_SC6_PA1_FIFO_EMPTY                     = 0x000002f6,
    PH_PERF_SEL_SC6_PA1_FIFO_FULL                      = 0x000002f7,
    PH_PERF_SEL_SC6_PA1_NULL_WE                        = 0x000002f8,
    PH_PERF_SEL_SC6_PA1_EVENT_WE                       = 0x000002f9,
    PH_PERF_SEL_SC6_PA1_FPOV_WE                        = 0x000002fa,
    PH_PERF_SEL_SC6_PA1_FPOP_WE                        = 0x000002fb,
    PH_PERF_SEL_SC6_PA1_EOP_WE                         = 0x000002fc,
    PH_PERF_SEL_SC6_PA1_DATA_FIFO_EOP_RD               = 0x000002fd,
    PH_PERF_SEL_SC6_PA1_EOPG_WE                        = 0x000002fe,
    PH_PERF_SEL_SC6_PA1_DEALLOC_WE                     = 0x000002ff,
    PH_PERF_SEL_SC6_PA2_DATA_FIFO_RD                   = 0x00000300,
    PH_PERF_SEL_SC6_PA2_DATA_FIFO_WE                   = 0x00000301,
    PH_PERF_SEL_SC6_PA2_FIFO_EMPTY                     = 0x00000302,
    PH_PERF_SEL_SC6_PA2_FIFO_FULL                      = 0x00000303,
    PH_PERF_SEL_SC6_PA2_NULL_WE                        = 0x00000304,
    PH_PERF_SEL_SC6_PA2_EVENT_WE                       = 0x00000305,
    PH_PERF_SEL_SC6_PA2_FPOV_WE                        = 0x00000306,
    PH_PERF_SEL_SC6_PA2_FPOP_WE                        = 0x00000307,
    PH_PERF_SEL_SC6_PA2_EOP_WE                         = 0x00000308,
    PH_PERF_SEL_SC6_PA2_DATA_FIFO_EOP_RD               = 0x00000309,
    PH_PERF_SEL_SC6_PA2_EOPG_WE                        = 0x0000030a,
    PH_PERF_SEL_SC6_PA2_DEALLOC_WE                     = 0x0000030b,
    PH_PERF_SEL_SC6_PA3_DATA_FIFO_RD                   = 0x0000030c,
    PH_PERF_SEL_SC6_PA3_DATA_FIFO_WE                   = 0x0000030d,
    PH_PERF_SEL_SC6_PA3_FIFO_EMPTY                     = 0x0000030e,
    PH_PERF_SEL_SC6_PA3_FIFO_FULL                      = 0x0000030f,
    PH_PERF_SEL_SC6_PA3_NULL_WE                        = 0x00000310,
    PH_PERF_SEL_SC6_PA3_EVENT_WE                       = 0x00000311,
    PH_PERF_SEL_SC6_PA3_FPOV_WE                        = 0x00000312,
    PH_PERF_SEL_SC6_PA3_FPOP_WE                        = 0x00000313,
    PH_PERF_SEL_SC6_PA3_EOP_WE                         = 0x00000314,
    PH_PERF_SEL_SC6_PA3_DATA_FIFO_EOP_RD               = 0x00000315,
    PH_PERF_SEL_SC6_PA3_EOPG_WE                        = 0x00000316,
    PH_PERF_SEL_SC6_PA3_DEALLOC_WE                     = 0x00000317,
    PH_PERF_SEL_SC6_PA4_DATA_FIFO_RD                   = 0x00000318,
    PH_PERF_SEL_SC6_PA4_DATA_FIFO_WE                   = 0x00000319,
    PH_PERF_SEL_SC6_PA4_FIFO_EMPTY                     = 0x0000031a,
    PH_PERF_SEL_SC6_PA4_FIFO_FULL                      = 0x0000031b,
    PH_PERF_SEL_SC6_PA4_NULL_WE                        = 0x0000031c,
    PH_PERF_SEL_SC6_PA4_EVENT_WE                       = 0x0000031d,
    PH_PERF_SEL_SC6_PA4_FPOV_WE                        = 0x0000031e,
    PH_PERF_SEL_SC6_PA4_FPOP_WE                        = 0x0000031f,
    PH_PERF_SEL_SC6_PA4_EOP_WE                         = 0x00000320,
    PH_PERF_SEL_SC6_PA4_DATA_FIFO_EOP_RD               = 0x00000321,
    PH_PERF_SEL_SC6_PA4_EOPG_WE                        = 0x00000322,
    PH_PERF_SEL_SC6_PA4_DEALLOC_WE                     = 0x00000323,
    PH_PERF_SEL_SC6_PA5_DATA_FIFO_RD                   = 0x00000324,
    PH_PERF_SEL_SC6_PA5_DATA_FIFO_WE                   = 0x00000325,
    PH_PERF_SEL_SC6_PA5_FIFO_EMPTY                     = 0x00000326,
    PH_PERF_SEL_SC6_PA5_FIFO_FULL                      = 0x00000327,
    PH_PERF_SEL_SC6_PA5_NULL_WE                        = 0x00000328,
    PH_PERF_SEL_SC6_PA5_EVENT_WE                       = 0x00000329,
    PH_PERF_SEL_SC6_PA5_FPOV_WE                        = 0x0000032a,
    PH_PERF_SEL_SC6_PA5_FPOP_WE                        = 0x0000032b,
    PH_PERF_SEL_SC6_PA5_EOP_WE                         = 0x0000032c,
    PH_PERF_SEL_SC6_PA5_DATA_FIFO_EOP_RD               = 0x0000032d,
    PH_PERF_SEL_SC6_PA5_EOPG_WE                        = 0x0000032e,
    PH_PERF_SEL_SC6_PA5_DEALLOC_WE                     = 0x0000032f,
    PH_PERF_SEL_SC6_PA6_DATA_FIFO_RD                   = 0x00000330,
    PH_PERF_SEL_SC6_PA6_DATA_FIFO_WE                   = 0x00000331,
    PH_PERF_SEL_SC6_PA6_FIFO_EMPTY                     = 0x00000332,
    PH_PERF_SEL_SC6_PA6_FIFO_FULL                      = 0x00000333,
    PH_PERF_SEL_SC6_PA6_NULL_WE                        = 0x00000334,
    PH_PERF_SEL_SC6_PA6_EVENT_WE                       = 0x00000335,
    PH_PERF_SEL_SC6_PA6_FPOV_WE                        = 0x00000336,
    PH_PERF_SEL_SC6_PA6_FPOP_WE                        = 0x00000337,
    PH_PERF_SEL_SC6_PA6_EOP_WE                         = 0x00000338,
    PH_PERF_SEL_SC6_PA6_DATA_FIFO_EOP_RD               = 0x00000339,
    PH_PERF_SEL_SC6_PA6_EOPG_WE                        = 0x0000033a,
    PH_PERF_SEL_SC6_PA6_DEALLOC_WE                     = 0x0000033b,
    PH_PERF_SEL_SC6_PA7_DATA_FIFO_RD                   = 0x0000033c,
    PH_PERF_SEL_SC6_PA7_DATA_FIFO_WE                   = 0x0000033d,
    PH_PERF_SEL_SC6_PA7_FIFO_EMPTY                     = 0x0000033e,
    PH_PERF_SEL_SC6_PA7_FIFO_FULL                      = 0x0000033f,
    PH_PERF_SEL_SC6_PA7_NULL_WE                        = 0x00000340,
    PH_PERF_SEL_SC6_PA7_EVENT_WE                       = 0x00000341,
    PH_PERF_SEL_SC6_PA7_FPOV_WE                        = 0x00000342,
    PH_PERF_SEL_SC6_PA7_FPOP_WE                        = 0x00000343,
    PH_PERF_SEL_SC6_PA7_EOP_WE                         = 0x00000344,
    PH_PERF_SEL_SC6_PA7_DATA_FIFO_EOP_RD               = 0x00000345,
    PH_PERF_SEL_SC6_PA7_EOPG_WE                        = 0x00000346,
    PH_PERF_SEL_SC6_PA7_DEALLOC_WE                     = 0x00000347,
    PH_PERF_SEL_SC7_SRPS_WINDOW_VALID                  = 0x00000348,
    PH_PERF_SEL_SC7_ARB_XFC_ALL_EVENT_OR_PRIM_CYCLES   = 0x00000349,
    PH_PERF_SEL_SC7_ARB_XFC_ONLY_PRIM_CYCLES           = 0x0000034a,
    PH_PERF_SEL_SC7_ARB_XFC_ONLY_ONE_INC_PER_PRIM      = 0x0000034b,
    PH_PERF_SEL_SC7_ARB_STALLED_FROM_BELOW             = 0x0000034c,
    PH_PERF_SEL_SC7_ARB_STARVED_FROM_ABOVE             = 0x0000034d,
    PH_PERF_SEL_SC7_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_NOT_EMPTY = 0x0000034e,
    PH_PERF_SEL_SC7_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_FULL = 0x0000034f,
    PH_PERF_SEL_SC7_ARB_BUSY                           = 0x00000350,
    PH_PERF_SEL_SC7_ARB_PA_BUSY_SOP                    = 0x00000351,
    PH_PERF_SEL_SC7_ARB_EOP_POP_SYNC_POP               = 0x00000352,
    PH_PERF_SEL_SC7_ARB_EVENT_SYNC_POP                 = 0x00000353,
    PH_PERF_SEL_SC7_PS_ENG_MULTICYCLE_BUBBLE           = 0x00000354,
    PH_PERF_SEL_SC7_EOP_SYNC_WINDOW                    = 0x00000355,
    PH_PERF_SEL_SC7_BUSY_PROCESSING_MULTICYCLE_PRIM    = 0x00000356,
    PH_PERF_SEL_SC7_BUSY_CNT_NOT_ZERO                  = 0x00000357,
    PH_PERF_SEL_SC7_SEND                               = 0x00000358,
    PH_PERF_SEL_SC7_CREDIT_AT_ZERO_WITH_PENDING_SEND   = 0x00000359,
    PH_PERF_SEL_SC7_CREDIT_AT_MAX                      = 0x0000035a,
    PH_PERF_SEL_SC7_CREDIT_AT_MAX_NO_PENDING_SEND      = 0x0000035b,
    PH_PERF_SEL_SC7_GFX_PIPE0_TO_1_TRANSITION          = 0x0000035c,
    PH_PERF_SEL_SC7_GFX_PIPE1_TO_0_TRANSITION          = 0x0000035d,
    PH_PERF_SEL_SC7_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION = 0x0000035e,
    PH_PERF_SEL_SC7_GFX_PIPE_EVENT_PROVOKED_TRANSITION = 0x0000035f,
    PH_PERF_SEL_SC7_PA0_DATA_FIFO_RD                   = 0x00000360,
    PH_PERF_SEL_SC7_PA0_DATA_FIFO_WE                   = 0x00000361,
    PH_PERF_SEL_SC7_PA0_FIFO_EMPTY                     = 0x00000362,
    PH_PERF_SEL_SC7_PA0_FIFO_FULL                      = 0x00000363,
    PH_PERF_SEL_SC7_PA0_NULL_WE                        = 0x00000364,
    PH_PERF_SEL_SC7_PA0_EVENT_WE                       = 0x00000365,
    PH_PERF_SEL_SC7_PA0_FPOV_WE                        = 0x00000366,
    PH_PERF_SEL_SC7_PA0_FPOP_WE                        = 0x00000367,
    PH_PERF_SEL_SC7_PA0_EOP_WE                         = 0x00000368,
    PH_PERF_SEL_SC7_PA0_DATA_FIFO_EOP_RD               = 0x00000369,
    PH_PERF_SEL_SC7_PA0_EOPG_WE                        = 0x0000036a,
    PH_PERF_SEL_SC7_PA0_DEALLOC_WE                     = 0x0000036b,
    PH_PERF_SEL_SC7_PA1_DATA_FIFO_RD                   = 0x0000036c,
    PH_PERF_SEL_SC7_PA1_DATA_FIFO_WE                   = 0x0000036d,
    PH_PERF_SEL_SC7_PA1_FIFO_EMPTY                     = 0x0000036e,
    PH_PERF_SEL_SC7_PA1_FIFO_FULL                      = 0x0000036f,
    PH_PERF_SEL_SC7_PA1_NULL_WE                        = 0x00000370,
    PH_PERF_SEL_SC7_PA1_EVENT_WE                       = 0x00000371,
    PH_PERF_SEL_SC7_PA1_FPOV_WE                        = 0x00000372,
    PH_PERF_SEL_SC7_PA1_FPOP_WE                        = 0x00000373,
    PH_PERF_SEL_SC7_PA1_EOP_WE                         = 0x00000374,
    PH_PERF_SEL_SC7_PA1_DATA_FIFO_EOP_RD               = 0x00000375,
    PH_PERF_SEL_SC7_PA1_EOPG_WE                        = 0x00000376,
    PH_PERF_SEL_SC7_PA1_DEALLOC_WE                     = 0x00000377,
    PH_PERF_SEL_SC7_PA2_DATA_FIFO_RD                   = 0x00000378,
    PH_PERF_SEL_SC7_PA2_DATA_FIFO_WE                   = 0x00000379,
    PH_PERF_SEL_SC7_PA2_FIFO_EMPTY                     = 0x0000037a,
    PH_PERF_SEL_SC7_PA2_FIFO_FULL                      = 0x0000037b,
    PH_PERF_SEL_SC7_PA2_NULL_WE                        = 0x0000037c,
    PH_PERF_SEL_SC7_PA2_EVENT_WE                       = 0x0000037d,
    PH_PERF_SEL_SC7_PA2_FPOV_WE                        = 0x0000037e,
    PH_PERF_SEL_SC7_PA2_FPOP_WE                        = 0x0000037f,
    PH_PERF_SEL_SC7_PA2_EOP_WE                         = 0x00000380,
    PH_PERF_SEL_SC7_PA2_DATA_FIFO_EOP_RD               = 0x00000381,
    PH_PERF_SEL_SC7_PA2_EOPG_WE                        = 0x00000382,
    PH_PERF_SEL_SC7_PA2_DEALLOC_WE                     = 0x00000383,
    PH_PERF_SEL_SC7_PA3_DATA_FIFO_RD                   = 0x00000384,
    PH_PERF_SEL_SC7_PA3_DATA_FIFO_WE                   = 0x00000385,
    PH_PERF_SEL_SC7_PA3_FIFO_EMPTY                     = 0x00000386,
    PH_PERF_SEL_SC7_PA3_FIFO_FULL                      = 0x00000387,
    PH_PERF_SEL_SC7_PA3_NULL_WE                        = 0x00000388,
    PH_PERF_SEL_SC7_PA3_EVENT_WE                       = 0x00000389,
    PH_PERF_SEL_SC7_PA3_FPOV_WE                        = 0x0000038a,
    PH_PERF_SEL_SC7_PA3_FPOP_WE                        = 0x0000038b,
    PH_PERF_SEL_SC7_PA3_EOP_WE                         = 0x0000038c,
    PH_PERF_SEL_SC7_PA3_DATA_FIFO_EOP_RD               = 0x0000038d,
    PH_PERF_SEL_SC7_PA3_EOPG_WE                        = 0x0000038e,
    PH_PERF_SEL_SC7_PA3_DEALLOC_WE                     = 0x0000038f,
    PH_PERF_SEL_SC7_PA4_DATA_FIFO_RD                   = 0x00000390,
    PH_PERF_SEL_SC7_PA4_DATA_FIFO_WE                   = 0x00000391,
    PH_PERF_SEL_SC7_PA4_FIFO_EMPTY                     = 0x00000392,
    PH_PERF_SEL_SC7_PA4_FIFO_FULL                      = 0x00000393,
    PH_PERF_SEL_SC7_PA4_NULL_WE                        = 0x00000394,
    PH_PERF_SEL_SC7_PA4_EVENT_WE                       = 0x00000395,
    PH_PERF_SEL_SC7_PA4_FPOV_WE                        = 0x00000396,
    PH_PERF_SEL_SC7_PA4_FPOP_WE                        = 0x00000397,
    PH_PERF_SEL_SC7_PA4_EOP_WE                         = 0x00000398,
    PH_PERF_SEL_SC7_PA4_DATA_FIFO_EOP_RD               = 0x00000399,
    PH_PERF_SEL_SC7_PA4_EOPG_WE                        = 0x0000039a,
    PH_PERF_SEL_SC7_PA4_DEALLOC_WE                     = 0x0000039b,
    PH_PERF_SEL_SC7_PA5_DATA_FIFO_RD                   = 0x0000039c,
    PH_PERF_SEL_SC7_PA5_DATA_FIFO_WE                   = 0x0000039d,
    PH_PERF_SEL_SC7_PA5_FIFO_EMPTY                     = 0x0000039e,
    PH_PERF_SEL_SC7_PA5_FIFO_FULL                      = 0x0000039f,
    PH_PERF_SEL_SC7_PA5_NULL_WE                        = 0x000003a0,
    PH_PERF_SEL_SC7_PA5_EVENT_WE                       = 0x000003a1,
    PH_PERF_SEL_SC7_PA5_FPOV_WE                        = 0x000003a2,
    PH_PERF_SEL_SC7_PA5_FPOP_WE                        = 0x000003a3,
    PH_PERF_SEL_SC7_PA5_EOP_WE                         = 0x000003a4,
    PH_PERF_SEL_SC7_PA5_DATA_FIFO_EOP_RD               = 0x000003a5,
    PH_PERF_SEL_SC7_PA5_EOPG_WE                        = 0x000003a6,
    PH_PERF_SEL_SC7_PA5_DEALLOC_WE                     = 0x000003a7,
    PH_PERF_SEL_SC7_PA6_DATA_FIFO_RD                   = 0x000003a8,
    PH_PERF_SEL_SC7_PA6_DATA_FIFO_WE                   = 0x000003a9,
    PH_PERF_SEL_SC7_PA6_FIFO_EMPTY                     = 0x000003aa,
    PH_PERF_SEL_SC7_PA6_FIFO_FULL                      = 0x000003ab,
    PH_PERF_SEL_SC7_PA6_NULL_WE                        = 0x000003ac,
    PH_PERF_SEL_SC7_PA6_EVENT_WE                       = 0x000003ad,
    PH_PERF_SEL_SC7_PA6_FPOV_WE                        = 0x000003ae,
    PH_PERF_SEL_SC7_PA6_FPOP_WE                        = 0x000003af,
    PH_PERF_SEL_SC7_PA6_EOP_WE                         = 0x000003b0,
    PH_PERF_SEL_SC7_PA6_DATA_FIFO_EOP_RD               = 0x000003b1,
    PH_PERF_SEL_SC7_PA6_EOPG_WE                        = 0x000003b2,
    PH_PERF_SEL_SC7_PA6_DEALLOC_WE                     = 0x000003b3,
    PH_PERF_SEL_SC7_PA7_DATA_FIFO_RD                   = 0x000003b4,
    PH_PERF_SEL_SC7_PA7_DATA_FIFO_WE                   = 0x000003b5,
    PH_PERF_SEL_SC7_PA7_FIFO_EMPTY                     = 0x000003b6,
    PH_PERF_SEL_SC7_PA7_FIFO_FULL                      = 0x000003b7,
    PH_PERF_SEL_SC7_PA7_NULL_WE                        = 0x000003b8,
    PH_PERF_SEL_SC7_PA7_EVENT_WE                       = 0x000003b9,
    PH_PERF_SEL_SC7_PA7_FPOV_WE                        = 0x000003ba,
    PH_PERF_SEL_SC7_PA7_FPOP_WE                        = 0x000003bb,
    PH_PERF_SEL_SC7_PA7_EOP_WE                         = 0x000003bc,
    PH_PERF_SEL_SC7_PA7_DATA_FIFO_EOP_RD               = 0x000003bd,
    PH_PERF_SEL_SC7_PA7_EOPG_WE                        = 0x000003be,
    PH_PERF_SEL_SC7_PA7_DEALLOC_WE                     = 0x000003bf,
    PH_PERF_SEL_1_SC_ARB_STALLED_FROM_BELOW            = 0x000003c0,
    PH_PERF_SEL_2_SC_ARB_STALLED_FROM_BELOW            = 0x000003c1,
    PH_PERF_SEL_3_SC_ARB_STALLED_FROM_BELOW            = 0x000003c2,
    PH_PERF_SEL_4_SC_ARB_STALLED_FROM_BELOW            = 0x000003c3,
    PH_PERF_SEL_5_SC_ARB_STALLED_FROM_BELOW            = 0x000003c4,
    PH_PERF_SEL_6_SC_ARB_STALLED_FROM_BELOW            = 0x000003c5,
    PH_PERF_SEL_7_SC_ARB_STALLED_FROM_BELOW            = 0x000003c6,
    PH_PERF_SEL_8_SC_ARB_STALLED_FROM_BELOW            = 0x000003c7,
    PH_PERF_SEL_1_SC_ARB_STARVED_FROM_ABOVE            = 0x000003c8,
    PH_PERF_SEL_2_SC_ARB_STARVED_FROM_ABOVE            = 0x000003c9,
    PH_PERF_SEL_3_SC_ARB_STARVED_FROM_ABOVE            = 0x000003ca,
    PH_PERF_SEL_4_SC_ARB_STARVED_FROM_ABOVE            = 0x000003cb,
    PH_PERF_SEL_5_SC_ARB_STARVED_FROM_ABOVE            = 0x000003cc,
    PH_PERF_SEL_6_SC_ARB_STARVED_FROM_ABOVE            = 0x000003cd,
    PH_PERF_SEL_7_SC_ARB_STARVED_FROM_ABOVE            = 0x000003ce,
    PH_PERF_SEL_8_SC_ARB_STARVED_FROM_ABOVE            = 0x000003cf,
    PH_PERF_SEL_1_SC_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_NOT_EMPTY = 0x000003d0,
    PH_PERF_SEL_2_SC_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_NOT_EMPTY = 0x000003d1,
    PH_PERF_SEL_3_SC_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_NOT_EMPTY = 0x000003d2,
    PH_PERF_SEL_4_SC_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_NOT_EMPTY = 0x000003d3,
    PH_PERF_SEL_5_SC_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_NOT_EMPTY = 0x000003d4,
    PH_PERF_SEL_6_SC_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_NOT_EMPTY = 0x000003d5,
    PH_PERF_SEL_7_SC_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_NOT_EMPTY = 0x000003d6,
    PH_PERF_SEL_8_SC_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_NOT_EMPTY = 0x000003d7,
    PH_PERF_SEL_1_SC_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_FULL = 0x000003d8,
    PH_PERF_SEL_2_SC_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_FULL = 0x000003d9,
    PH_PERF_SEL_3_SC_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_FULL = 0x000003da,
    PH_PERF_SEL_4_SC_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_FULL = 0x000003db,
    PH_PERF_SEL_5_SC_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_FULL = 0x000003dc,
    PH_PERF_SEL_6_SC_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_FULL = 0x000003dd,
    PH_PERF_SEL_7_SC_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_FULL = 0x000003de,
    PH_PERF_SEL_8_SC_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_FULL = 0x000003df,
    PH_PERF_SC0_FIFO_STATUS_0                          = 0x000003e0,
    PH_PERF_SC0_FIFO_STATUS_1                          = 0x000003e1,
    PH_PERF_SC0_FIFO_STATUS_2                          = 0x000003e2,
    PH_PERF_SC0_FIFO_STATUS_3                          = 0x000003e3,
    PH_PERF_SC1_FIFO_STATUS_0                          = 0x000003e4,
    PH_PERF_SC1_FIFO_STATUS_1                          = 0x000003e5,
    PH_PERF_SC1_FIFO_STATUS_2                          = 0x000003e6,
    PH_PERF_SC1_FIFO_STATUS_3                          = 0x000003e7,
    PH_PERF_SC2_FIFO_STATUS_0                          = 0x000003e8,
    PH_PERF_SC2_FIFO_STATUS_1                          = 0x000003e9,
    PH_PERF_SC2_FIFO_STATUS_2                          = 0x000003ea,
    PH_PERF_SC2_FIFO_STATUS_3                          = 0x000003eb,
    PH_PERF_SC3_FIFO_STATUS_0                          = 0x000003ec,
    PH_PERF_SC3_FIFO_STATUS_1                          = 0x000003ed,
    PH_PERF_SC3_FIFO_STATUS_2                          = 0x000003ee,
    PH_PERF_SC3_FIFO_STATUS_3                          = 0x000003ef,
    PH_PERF_SC4_FIFO_STATUS_0                          = 0x000003f0,
    PH_PERF_SC4_FIFO_STATUS_1                          = 0x000003f1,
    PH_PERF_SC4_FIFO_STATUS_2                          = 0x000003f2,
    PH_PERF_SC4_FIFO_STATUS_3                          = 0x000003f3,
    PH_PERF_SC5_FIFO_STATUS_0                          = 0x000003f4,
    PH_PERF_SC5_FIFO_STATUS_1                          = 0x000003f5,
    PH_PERF_SC5_FIFO_STATUS_2                          = 0x000003f6,
    PH_PERF_SC5_FIFO_STATUS_3                          = 0x000003f7,
    PH_PERF_SC6_FIFO_STATUS_0                          = 0x000003f8,
    PH_PERF_SC6_FIFO_STATUS_1                          = 0x000003f9,
    PH_PERF_SC6_FIFO_STATUS_2                          = 0x000003fa,
    PH_PERF_SC6_FIFO_STATUS_3                          = 0x000003fb,
    PH_PERF_SC7_FIFO_STATUS_0                          = 0x000003fc,
    PH_PERF_SC7_FIFO_STATUS_1                          = 0x000003fd,
    PH_PERF_SC7_FIFO_STATUS_2                          = 0x000003fe,
    PH_PERF_SC7_FIFO_STATUS_3                          = 0x000003ff,
} PH_PERFCNT_SEL;

constexpr unsigned int MaxPhPerfcntSel                 = PH_PERF_SC7_FIFO_STATUS_3;

typedef enum PipeConfig {
    ADDR_SURF_P2                                       = 0x00000000,
    ADDR_SURF_P2_RESERVED0                             = 0x00000001,
    ADDR_SURF_P2_RESERVED1                             = 0x00000002,
    ADDR_SURF_P2_RESERVED2                             = 0x00000003,
    ADDR_SURF_P4_8x16                                  = 0x00000004,
    ADDR_SURF_P4_16x16                                 = 0x00000005,
    ADDR_SURF_P4_16x32                                 = 0x00000006,
    ADDR_SURF_P4_32x32                                 = 0x00000007,
    ADDR_SURF_P8_16x16_8x16                            = 0x00000008,
    ADDR_SURF_P8_16x32_8x16                            = 0x00000009,
    ADDR_SURF_P8_32x32_8x16                            = 0x0000000a,
    ADDR_SURF_P8_16x32_16x16                           = 0x0000000b,
    ADDR_SURF_P8_32x32_16x16                           = 0x0000000c,
    ADDR_SURF_P8_32x32_16x32                           = 0x0000000d,
    ADDR_SURF_P8_32x64_32x32                           = 0x0000000e,
    ADDR_SURF_P8_RESERVED0                             = 0x0000000f,
    ADDR_SURF_P16_32x32_8x16                           = 0x00000010,
    ADDR_SURF_P16_32x32_16x16                          = 0x00000011,
    ADDR_SURF_P16                                      = 0x00000012,
} PipeConfig;

typedef enum PixelPipeCounterId {
    PIXEL_PIPE_OCCLUSION_COUNT_0                       = 0x00000000,
    PIXEL_PIPE_OCCLUSION_COUNT_1                       = 0x00000001,
    PIXEL_PIPE_OCCLUSION_COUNT_2                       = 0x00000002,
    PIXEL_PIPE_OCCLUSION_COUNT_3                       = 0x00000003,
    PIXEL_PIPE_SCREEN_MIN_EXTENTS_0                    = 0x00000004,
    PIXEL_PIPE_SCREEN_MAX_EXTENTS_0                    = 0x00000005,
    PIXEL_PIPE_SCREEN_MIN_EXTENTS_1                    = 0x00000006,
    PIXEL_PIPE_SCREEN_MAX_EXTENTS_1                    = 0x00000007,
} PixelPipeCounterId;

typedef enum PixelPipeStride {
    PIXEL_PIPE_STRIDE_32_BITS                          = 0x00000000,
    PIXEL_PIPE_STRIDE_64_BITS                          = 0x00000001,
    PIXEL_PIPE_STRIDE_128_BITS                         = 0x00000002,
    PIXEL_PIPE_STRIDE_256_BITS                         = 0x00000003,
} PixelPipeStride;

typedef enum QuadExportFormat {
    EXPORT_UNUSED                                      = 0x00000000,
    EXPORT_32_R                                        = 0x00000001,
    EXPORT_32_GR                                       = 0x00000002,
    EXPORT_32_AR                                       = 0x00000003,
    EXPORT_FP16_ABGR                                   = 0x00000004,
    EXPORT_UNSIGNED16_ABGR                             = 0x00000005,
    EXPORT_SIGNED16_ABGR                               = 0x00000006,
    EXPORT_32_ABGR                                     = 0x00000007,
    EXPORT_32BPP_8PIX                                  = 0x00000008,
    EXPORT_16_16_UNSIGNED_8PIX                         = 0x00000009,
    EXPORT_16_16_SIGNED_8PIX                           = 0x0000000a,
    EXPORT_16_16_FLOAT_8PIX                            = 0x0000000b,
} QuadExportFormat;

typedef enum QuadExportFormatOld {
    EXPORT_4P_32BPC_ABGR                               = 0x00000000,
    EXPORT_4P_16BPC_ABGR                               = 0x00000001,
    EXPORT_4P_32BPC_GR                                 = 0x00000002,
    EXPORT_4P_32BPC_AR                                 = 0x00000003,
    EXPORT_2P_32BPC_ABGR                               = 0x00000004,
    EXPORT_8P_32BPC_R                                  = 0x00000005,
} QuadExportFormatOld;

typedef enum ReadPolicy {
    CACHE_LRU_RD                                       = 0x00000000,
    CACHE_STREAM_RD                                    = 0x00000001,
    CACHE_NOA                                          = 0x00000002,
    RESERVED_RDPOLICY                                  = 0x00000003,
} ReadPolicy;

typedef enum RLC_PERFCOUNTER_SEL {
    RLC_PERF_SEL_POWER_FEATURE_0                       = 0x00000000,
    RLC_PERF_SEL_POWER_FEATURE_1                       = 0x00000001,
    RLC_PERF_SEL_CP_INTERRUPT                          = 0x00000002,
    RLC_PERF_SEL_GRBM_INTERRUPT                        = 0x00000003,
    RLC_PERF_SEL_SPM_INTERRUPT                         = 0x00000004,
    RLC_PERF_SEL_IH_INTERRUPT                          = 0x00000005,
    RLC_PERF_SEL_SERDES_COMMAND_WRITE                  = 0x00000006,
} RLC_PERFCOUNTER_SEL;

constexpr unsigned int MaxRlcPerfcounterSel            = RLC_PERF_SEL_SERDES_COMMAND_WRITE;

typedef enum RLC_PERFMON_STATE {
    RLC_PERFMON_STATE_RESET                            = 0x00000000,
    RLC_PERFMON_STATE_ENABLE                           = 0x00000001,
    RLC_PERFMON_STATE_DISABLE                          = 0x00000002,
    RLC_PERFMON_STATE_RESERVED_3                       = 0x00000003,
    RLC_PERFMON_STATE_RESERVED_4                       = 0x00000004,
    RLC_PERFMON_STATE_RESERVED_5                       = 0x00000005,
    RLC_PERFMON_STATE_RESERVED_6                       = 0x00000006,
    RLC_PERFMON_STATE_ROLLOVER                         = 0x00000007,
} RLC_PERFMON_STATE;

typedef enum RMIPerfSel {
    RMI_PERF_SEL_NONE                                  = 0x00000000,
    RMI_PERF_SEL_BUSY                                  = 0x00000001,
    RMI_PERF_SEL_REG_CLK_VLD                           = 0x00000002,
    RMI_PERF_SEL_DYN_CLK_CMN_VLD                       = 0x00000003,
    RMI_PERF_SEL_DYN_CLK_RB_VLD                        = 0x00000004,
    RMI_PERF_SEL_DYN_CLK_PERF_VLD                      = 0x00000005,
    RMI_PERF_SEL_PERF_WINDOW                           = 0x00000006,
    RMI_PERF_SEL_EVENT_SEND                            = 0x00000007,
    RMI_PERF_SEL_RB_RMI_WRREQ_ALL_CID                  = 0x00000008,
    RMI_PERF_SEL_RB_RMI_WRREQ_TO_WRRET_BUSY            = 0x00000009,
    RMI_PERF_SEL_RB_RMI_WRREQ_CID0                     = 0x0000000a,
    RMI_PERF_SEL_RB_RMI_WRREQ_CID1                     = 0x0000000b,
    RMI_PERF_SEL_RB_RMI_WRREQ_CID2                     = 0x0000000c,
    RMI_PERF_SEL_RB_RMI_WRREQ_CID3                     = 0x0000000d,
    RMI_PERF_SEL_RB_RMI_WRREQ_CID4                     = 0x0000000e,
    RMI_PERF_SEL_RB_RMI_WRREQ_CID5                     = 0x0000000f,
    RMI_PERF_SEL_RB_RMI_WRREQ_CID6                     = 0x00000010,
    RMI_PERF_SEL_RB_RMI_WRREQ_CID7                     = 0x00000011,
    RMI_PERF_SEL_RB_RMI_32BWRREQ_INFLIGHT_ALL_ORONE_CID = 0x00000012,
    RMI_PERF_SEL_RB_RMI_WRREQ_BURST_LENGTH_ALL_ORONE_CID = 0x00000013,
    RMI_PERF_SEL_RB_RMI_WRREQ_BURST_ALL_ORONE_CID      = 0x00000014,
    RMI_PERF_SEL_RB_RMI_WRREQ_RESIDENCY                = 0x00000015,
    RMI_PERF_SEL_RMI_RB_WRRET_VALID_ALL_CID            = 0x00000016,
    RMI_PERF_SEL_RMI_RB_WRRET_VALID_CID0               = 0x00000017,
    RMI_PERF_SEL_RMI_RB_WRRET_VALID_CID1               = 0x00000018,
    RMI_PERF_SEL_RMI_RB_WRRET_VALID_CID2               = 0x00000019,
    RMI_PERF_SEL_RMI_RB_WRRET_VALID_CID3               = 0x0000001a,
    RMI_PERF_SEL_RMI_RB_WRRET_VALID_CID4               = 0x0000001b,
    RMI_PERF_SEL_RMI_RB_WRRET_VALID_CID5               = 0x0000001c,
    RMI_PERF_SEL_RMI_RB_WRRET_VALID_CID6               = 0x0000001d,
    RMI_PERF_SEL_RMI_RB_WRRET_VALID_CID7               = 0x0000001e,
    RMI_PERF_SEL_RMI_RB_WRRET_VALID_NACK0              = 0x0000001f,
    RMI_PERF_SEL_RMI_RB_WRRET_VALID_NACK1              = 0x00000020,
    RMI_PERF_SEL_RMI_RB_WRRET_VALID_NACK2              = 0x00000021,
    RMI_PERF_SEL_RMI_RB_WRRET_VALID_NACK3              = 0x00000022,
    RMI_PERF_SEL_RB_RMI_32BRDREQ_ALL_CID               = 0x00000023,
    RMI_PERF_SEL_RB_RMI_RDREQ_ALL_CID                  = 0x00000024,
    RMI_PERF_SEL_RB_RMI_RDREQ_TO_RDRET_BUSY            = 0x00000025,
    RMI_PERF_SEL_RB_RMI_32BRDREQ_CID0                  = 0x00000026,
    RMI_PERF_SEL_RB_RMI_32BRDREQ_CID1                  = 0x00000027,
    RMI_PERF_SEL_RB_RMI_32BRDREQ_CID2                  = 0x00000028,
    RMI_PERF_SEL_RB_RMI_32BRDREQ_CID3                  = 0x00000029,
    RMI_PERF_SEL_RB_RMI_32BRDREQ_CID4                  = 0x0000002a,
    RMI_PERF_SEL_RB_RMI_32BRDREQ_CID5                  = 0x0000002b,
    RMI_PERF_SEL_RB_RMI_32BRDREQ_CID6                  = 0x0000002c,
    RMI_PERF_SEL_RB_RMI_32BRDREQ_CID7                  = 0x0000002d,
    RMI_PERF_SEL_RB_RMI_RDREQ_CID0                     = 0x0000002e,
    RMI_PERF_SEL_RB_RMI_RDREQ_CID1                     = 0x0000002f,
    RMI_PERF_SEL_RB_RMI_RDREQ_CID2                     = 0x00000030,
    RMI_PERF_SEL_RB_RMI_RDREQ_CID3                     = 0x00000031,
    RMI_PERF_SEL_RB_RMI_RDREQ_CID4                     = 0x00000032,
    RMI_PERF_SEL_RB_RMI_RDREQ_CID5                     = 0x00000033,
    RMI_PERF_SEL_RB_RMI_RDREQ_CID6                     = 0x00000034,
    RMI_PERF_SEL_RB_RMI_RDREQ_CID7                     = 0x00000035,
    RMI_PERF_SEL_RB_RMI_32BRDREQ_INFLIGHT_ALL_ORONE_CID = 0x00000036,
    RMI_PERF_SEL_RB_RMI_RDREQ_BURST_LENGTH_ALL_ORONE_CID = 0x00000037,
    RMI_PERF_SEL_RB_RMI_RDREQ_BURST_ALL_ORONE_CID      = 0x00000038,
    RMI_PERF_SEL_RB_RMI_RDREQ_RESIDENCY                = 0x00000039,
    RMI_PERF_SEL_RMI_RB_32BRDRET_VALID_ALL_CID         = 0x0000003a,
    RMI_PERF_SEL_RMI_RB_32BRDRET_VALID_CID0            = 0x0000003b,
    RMI_PERF_SEL_RMI_RB_32BRDRET_VALID_CID1            = 0x0000003c,
    RMI_PERF_SEL_RMI_RB_32BRDRET_VALID_CID2            = 0x0000003d,
    RMI_PERF_SEL_RMI_RB_32BRDRET_VALID_CID3            = 0x0000003e,
    RMI_PERF_SEL_RMI_RB_32BRDRET_VALID_CID4            = 0x0000003f,
    RMI_PERF_SEL_RMI_RB_32BRDRET_VALID_CID5            = 0x00000040,
    RMI_PERF_SEL_RMI_RB_32BRDRET_VALID_CID6            = 0x00000041,
    RMI_PERF_SEL_RMI_RB_32BRDRET_VALID_CID7            = 0x00000042,
    RMI_PERF_SEL_RMI_RB_32BRDRET_VALID_NACK0           = 0x00000043,
    RMI_PERF_SEL_RMI_RB_32BRDRET_VALID_NACK1           = 0x00000044,
    RMI_PERF_SEL_RMI_RB_32BRDRET_VALID_NACK2           = 0x00000045,
    RMI_PERF_SEL_RMI_RB_32BRDRET_VALID_NACK3           = 0x00000046,
    RMI_PERF_SEL_RB_RMI_WR_FIFO_MAX                    = 0x00000047,
    RMI_PERF_SEL_RB_RMI_WR_FIFO_EMPTY                  = 0x00000048,
    RMI_PERF_SEL_RB_RMI_WR_IDLE                        = 0x00000049,
    RMI_PERF_SEL_RB_RMI_WR_STARVE                      = 0x0000004a,
    RMI_PERF_SEL_RB_RMI_WR_STALL                       = 0x0000004b,
    RMI_PERF_SEL_RB_RMI_WR_BUSY                        = 0x0000004c,
    RMI_PERF_SEL_RB_RMI_WR_INTF_BUSY                   = 0x0000004d,
    RMI_PERF_SEL_RB_RMI_RD_FIFO_MAX                    = 0x0000004e,
    RMI_PERF_SEL_RB_RMI_RD_FIFO_EMPTY                  = 0x0000004f,
    RMI_PERF_SEL_RB_RMI_RD_IDLE                        = 0x00000050,
    RMI_PERF_SEL_RB_RMI_RD_STARVE                      = 0x00000051,
    RMI_PERF_SEL_RB_RMI_RD_STALL                       = 0x00000052,
    RMI_PERF_SEL_RB_RMI_RD_BUSY                        = 0x00000053,
    RMI_PERF_SEL_RB_RMI_RD_INTF_BUSY                   = 0x00000054,
    RMI_PERF_SEL_RMI_TC_64BWRREQ_ALL_ORONE_CID         = 0x00000055,
    RMI_PERF_SEL_RMI_TC_64BRDREQ_ALL_ORONE_CID         = 0x00000056,
    RMI_PERF_SEL_RMI_TC_WRREQ_ALL_CID                  = 0x00000057,
    RMI_PERF_SEL_RMI_TC_REQ_BUSY                       = 0x00000058,
    RMI_PERF_SEL_RMI_TC_WRREQ_CID0                     = 0x00000059,
    RMI_PERF_SEL_RMI_TC_WRREQ_CID1                     = 0x0000005a,
    RMI_PERF_SEL_RMI_TC_WRREQ_CID2                     = 0x0000005b,
    RMI_PERF_SEL_RMI_TC_WRREQ_CID3                     = 0x0000005c,
    RMI_PERF_SEL_RMI_TC_WRREQ_CID4                     = 0x0000005d,
    RMI_PERF_SEL_RMI_TC_WRREQ_CID5                     = 0x0000005e,
    RMI_PERF_SEL_RMI_TC_WRREQ_CID6                     = 0x0000005f,
    RMI_PERF_SEL_RMI_TC_WRREQ_CID7                     = 0x00000060,
    RMI_PERF_SEL_RMI_TC_WRREQ_INFLIGHT_ALL_CID         = 0x00000061,
    RMI_PERF_SEL_TC_RMI_WRRET_VALID_ALL_CID            = 0x00000062,
    RMI_PERF_SEL_RMI_TC_RDREQ_ALL_CID                  = 0x00000063,
    RMI_PERF_SEL_RMI_TC_RDREQ_CID0                     = 0x00000064,
    RMI_PERF_SEL_RMI_TC_RDREQ_CID1                     = 0x00000065,
    RMI_PERF_SEL_RMI_TC_RDREQ_CID2                     = 0x00000066,
    RMI_PERF_SEL_RMI_TC_RDREQ_CID3                     = 0x00000067,
    RMI_PERF_SEL_RMI_TC_RDREQ_CID4                     = 0x00000068,
    RMI_PERF_SEL_RMI_TC_RDREQ_CID5                     = 0x00000069,
    RMI_PERF_SEL_RMI_TC_RDREQ_CID6                     = 0x0000006a,
    RMI_PERF_SEL_RMI_TC_RDREQ_CID7                     = 0x0000006b,
    RMI_PERF_SEL_RMI_TC_STALL_RDREQ                    = 0x0000006c,
    RMI_PERF_SEL_RMI_TC_STALL_WRREQ                    = 0x0000006d,
    RMI_PERF_SEL_RMI_TC_STALL_ALLREQ                   = 0x0000006e,
    RMI_PERF_SEL_RMI_TC_CREDIT_FULL_NO_PENDING_SEND    = 0x0000006f,
    RMI_PERF_SEL_RMI_TC_CREDIT_ZERO_PENDING_SEND       = 0x00000070,
    RMI_PERF_SEL_RMI_TC_RDREQ_INFLIGHT_ALL_CID         = 0x00000071,
    RMI_PERF_SEL_TC_RMI_RDRET_VALID_ALL_CID            = 0x00000072,
    RMI_PERF_SEL_TCIW_INFLIGHT_COUNT                   = 0x00000073,
    RMI_PERF_SEL_TCIW_REQ                              = 0x00000074,
    RMI_PERF_SEL_TCIW_BUSY                             = 0x00000075,
    RMI_PERF_SEL_DEMUX_TCIW_FORMATTER_RTS_RTR          = 0x00000076,
    RMI_PERF_SEL_DEMUX_TCIW_FORMATTER_RTSB_RTR         = 0x00000077,
    RMI_PERF_SEL_DEMUX_TCIW_FORMATTER_RTS_RTRB         = 0x00000078,
    RMI_PERF_SEL_DEMUX_TCIW_FORMATTER_RTSB_RTRB        = 0x00000079,
    RMI_PERF_SEL_REORDER_FIFO_REQ                      = 0x0000007a,
    RMI_PERF_SEL_REORDER_FIFO_BUSY                     = 0x0000007b,
    RMI_PERF_SEL_RMI_RB_EARLY_WRACK_ALL_CID            = 0x0000007c,
    RMI_PERF_SEL_RMI_RB_EARLY_WRACK_CID0               = 0x0000007d,
    RMI_PERF_SEL_RMI_RB_EARLY_WRACK_CID1               = 0x0000007e,
    RMI_PERF_SEL_RMI_RB_EARLY_WRACK_CID2               = 0x0000007f,
    RMI_PERF_SEL_RMI_RB_EARLY_WRACK_CID3               = 0x00000080,
    RMI_PERF_SEL_RMI_RB_EARLY_WRACK_CID4               = 0x00000081,
    RMI_PERF_SEL_RMI_RB_EARLY_WRACK_CID5               = 0x00000082,
    RMI_PERF_SEL_RMI_RB_EARLY_WRACK_CID6               = 0x00000083,
    RMI_PERF_SEL_RMI_RB_EARLY_WRACK_CID7               = 0x00000084,
    RMI_PERF_SEL_CONSUMER_PROBEGEN_READ_RTS_RTR        = 0x00000085,
    RMI_PERF_SEL_CONSUMER_PROBEGEN_WRITE_RTS_RTR       = 0x00000086,
    RMI_PERF_SEL_CONSUMER_PROBEGEN_IN0_RTS_RTR         = 0x00000087,
    RMI_PERF_SEL_CONSUMER_PROBEGEN_IN1_RTS_RTR         = 0x00000088,
    RMI_PERF_SEL_CONSUMER_PROBEGEN_CB_RTS_RTR          = 0x00000089,
    RMI_PERF_SEL_CONSUMER_PROBEGEN_DB_RTS_RTR          = 0x0000008a,
} RMIPerfSel;

constexpr unsigned int MaxRMIPerfSel                   = RMI_PERF_SEL_CONSUMER_PROBEGEN_DB_RTS_RTR;

typedef enum RoundMode {
    ROUND_BY_HALF                                      = 0x00000000,
    ROUND_TRUNCATE                                     = 0x00000001,
} RoundMode;

typedef enum SCOPE {
    SCOPE_CU                                           = 0x00000000,
    SCOPE_SE                                           = 0x00000001,
    SCOPE_DEV                                          = 0x00000002,
    SCOPE_SYS                                          = 0x00000003,
} SCOPE;

typedef enum SC_ConservativeZExport {
    SC_EXPORT_ANY_Z                                    = 0x00000000,
    SC_EXPORT_LESS_THAN_Z                              = 0x00000001,
    SC_EXPORT_GREATER_THAN_Z                           = 0x00000002,
    SC_EXPORT_RESERVED                                 = 0x00000003,
} SC_ConservativeZExport;

typedef enum SC_ForceControl {
    SC_FORCE_OFF                                       = 0x00000000,
    SC_FORCE_ENABLE                                    = 0x00000001,
    SC_FORCE_DISABLE                                   = 0x00000002,
    SC_FORCE_RESERVED                                  = 0x00000003,
} SC_ForceControl;

typedef enum SC_PERFCNT_SEL {
    SC_SRPS_WINDOW_VALID                               = 0x00000000,
    SC_PSSW_WINDOW_VALID                               = 0x00000001,
    SC_TPQZ_WINDOW_VALID                               = 0x00000002,
    SC_QZQP_WINDOW_VALID                               = 0x00000003,
    SC_TRPK_WINDOW_VALID                               = 0x00000004,
    SC_SRPS_WINDOW_VALID_BUSY                          = 0x00000005,
    SC_PSSW_WINDOW_VALID_BUSY                          = 0x00000006,
    SC_TPQZ_WINDOW_VALID_BUSY                          = 0x00000007,
    SC_QZQP_WINDOW_VALID_BUSY                          = 0x00000008,
    SC_TRPK_WINDOW_VALID_BUSY                          = 0x00000009,
    SC_STARVED_BY_PA                                   = 0x0000000a,
    SC_STALLED_BY_PRIMFIFO                             = 0x0000000b,
    SC_STALLED_BY_DB_TILE                              = 0x0000000c,
    SC_STARVED_BY_DB_TILE                              = 0x0000000d,
    SC_STALLED_BY_TILEORDERFIFO                        = 0x0000000e,
    SC_STALLED_BY_TILEFIFO                             = 0x0000000f,
    SC_STALLED_BY_DB_QUAD                              = 0x00000010,
    SC_STARVED_BY_DB_QUAD                              = 0x00000011,
    SC_STALLED_BY_QUADFIFO                             = 0x00000012,
    SC_STALLED_BY_BCI                                  = 0x00000013,
    SC_STALLED_BY_SPI                                  = 0x00000014,
    SC_SCISSOR_DISCARD                                 = 0x00000015,
    SC_BB_DISCARD                                      = 0x00000016,
    SC_SUPERTILE_COUNT                                 = 0x00000017,
    SC_SUPERTILE_PER_PRIM_H0                           = 0x00000018,
    SC_SUPERTILE_PER_PRIM_H1                           = 0x00000019,
    SC_SUPERTILE_PER_PRIM_H2                           = 0x0000001a,
    SC_SUPERTILE_PER_PRIM_H3                           = 0x0000001b,
    SC_SUPERTILE_PER_PRIM_H4                           = 0x0000001c,
    SC_SUPERTILE_PER_PRIM_H5                           = 0x0000001d,
    SC_SUPERTILE_PER_PRIM_H6                           = 0x0000001e,
    SC_SUPERTILE_PER_PRIM_H7                           = 0x0000001f,
    SC_SUPERTILE_PER_PRIM_H8                           = 0x00000020,
    SC_SUPERTILE_PER_PRIM_H9                           = 0x00000021,
    SC_SUPERTILE_PER_PRIM_H10                          = 0x00000022,
    SC_SUPERTILE_PER_PRIM_H11                          = 0x00000023,
    SC_SUPERTILE_PER_PRIM_H12                          = 0x00000024,
    SC_SUPERTILE_PER_PRIM_H13                          = 0x00000025,
    SC_SUPERTILE_PER_PRIM_H14                          = 0x00000026,
    SC_SUPERTILE_PER_PRIM_H15                          = 0x00000027,
    SC_SUPERTILE_PER_PRIM_H16                          = 0x00000028,
    SC_TILE_PER_PRIM_H0                                = 0x00000029,
    SC_TILE_PER_PRIM_H1                                = 0x0000002a,
    SC_TILE_PER_PRIM_H2                                = 0x0000002b,
    SC_TILE_PER_PRIM_H3                                = 0x0000002c,
    SC_TILE_PER_PRIM_H4                                = 0x0000002d,
    SC_TILE_PER_PRIM_H5                                = 0x0000002e,
    SC_TILE_PER_PRIM_H6                                = 0x0000002f,
    SC_TILE_PER_PRIM_H7                                = 0x00000030,
    SC_TILE_PER_PRIM_H8                                = 0x00000031,
    SC_TILE_PER_PRIM_H9                                = 0x00000032,
    SC_TILE_PER_PRIM_H10                               = 0x00000033,
    SC_TILE_PER_PRIM_H11                               = 0x00000034,
    SC_TILE_PER_PRIM_H12                               = 0x00000035,
    SC_TILE_PER_PRIM_H13                               = 0x00000036,
    SC_TILE_PER_PRIM_H14                               = 0x00000037,
    SC_TILE_PER_PRIM_H15                               = 0x00000038,
    SC_TILE_PER_PRIM_H16                               = 0x00000039,
    SC_TILE_PER_SUPERTILE_H0                           = 0x0000003a,
    SC_TILE_PER_SUPERTILE_H1                           = 0x0000003b,
    SC_TILE_PER_SUPERTILE_H2                           = 0x0000003c,
    SC_TILE_PER_SUPERTILE_H3                           = 0x0000003d,
    SC_TILE_PER_SUPERTILE_H4                           = 0x0000003e,
    SC_TILE_PER_SUPERTILE_H5                           = 0x0000003f,
    SC_TILE_PER_SUPERTILE_H6                           = 0x00000040,
    SC_TILE_PER_SUPERTILE_H7                           = 0x00000041,
    SC_TILE_PER_SUPERTILE_H8                           = 0x00000042,
    SC_TILE_PER_SUPERTILE_H9                           = 0x00000043,
    SC_TILE_PER_SUPERTILE_H10                          = 0x00000044,
    SC_TILE_PER_SUPERTILE_H11                          = 0x00000045,
    SC_TILE_PER_SUPERTILE_H12                          = 0x00000046,
    SC_TILE_PER_SUPERTILE_H13                          = 0x00000047,
    SC_TILE_PER_SUPERTILE_H14                          = 0x00000048,
    SC_TILE_PER_SUPERTILE_H15                          = 0x00000049,
    SC_TILE_PER_SUPERTILE_H16                          = 0x0000004a,
    SC_TILE_PICKED_H1                                  = 0x0000004b,
    SC_PERF_SEL_RESERVED_76                            = 0x0000004c,
    SC_PERF_SEL_RESERVED_77                            = 0x0000004d,
    SC_PERF_SEL_RESERVED_78                            = 0x0000004e,
    SC_QZ0_TILE_COUNT                                  = 0x0000004f,
    SC_QZ1_TILE_COUNT                                  = 0x00000050,
    SC_QZ2_TILE_COUNT                                  = 0x00000051,
    SC_QZ3_TILE_COUNT                                  = 0x00000052,
    SC_QZ0_TILE_COVERED_COUNT                          = 0x00000053,
    SC_QZ1_TILE_COVERED_COUNT                          = 0x00000054,
    SC_QZ2_TILE_COVERED_COUNT                          = 0x00000055,
    SC_QZ3_TILE_COVERED_COUNT                          = 0x00000056,
    SC_QZ0_TILE_NOT_COVERED_COUNT                      = 0x00000057,
    SC_QZ1_TILE_NOT_COVERED_COUNT                      = 0x00000058,
    SC_QZ2_TILE_NOT_COVERED_COUNT                      = 0x00000059,
    SC_QZ3_TILE_NOT_COVERED_COUNT                      = 0x0000005a,
    SC_QZ0_QUAD_PER_TILE_H0                            = 0x0000005b,
    SC_QZ0_QUAD_PER_TILE_H1                            = 0x0000005c,
    SC_QZ0_QUAD_PER_TILE_H2                            = 0x0000005d,
    SC_QZ0_QUAD_PER_TILE_H3                            = 0x0000005e,
    SC_QZ0_QUAD_PER_TILE_H4                            = 0x0000005f,
    SC_QZ0_QUAD_PER_TILE_H5                            = 0x00000060,
    SC_QZ0_QUAD_PER_TILE_H6                            = 0x00000061,
    SC_QZ0_QUAD_PER_TILE_H7                            = 0x00000062,
    SC_QZ0_QUAD_PER_TILE_H8                            = 0x00000063,
    SC_QZ0_QUAD_PER_TILE_H9                            = 0x00000064,
    SC_QZ0_QUAD_PER_TILE_H10                           = 0x00000065,
    SC_QZ0_QUAD_PER_TILE_H11                           = 0x00000066,
    SC_QZ0_QUAD_PER_TILE_H12                           = 0x00000067,
    SC_QZ0_QUAD_PER_TILE_H13                           = 0x00000068,
    SC_QZ0_QUAD_PER_TILE_H14                           = 0x00000069,
    SC_QZ0_QUAD_PER_TILE_H15                           = 0x0000006a,
    SC_QZ0_QUAD_PER_TILE_H16                           = 0x0000006b,
    SC_QZ1_QUAD_PER_TILE_H0                            = 0x0000006c,
    SC_QZ1_QUAD_PER_TILE_H1                            = 0x0000006d,
    SC_QZ1_QUAD_PER_TILE_H2                            = 0x0000006e,
    SC_QZ1_QUAD_PER_TILE_H3                            = 0x0000006f,
    SC_QZ1_QUAD_PER_TILE_H4                            = 0x00000070,
    SC_QZ1_QUAD_PER_TILE_H5                            = 0x00000071,
    SC_QZ1_QUAD_PER_TILE_H6                            = 0x00000072,
    SC_QZ1_QUAD_PER_TILE_H7                            = 0x00000073,
    SC_QZ1_QUAD_PER_TILE_H8                            = 0x00000074,
    SC_QZ1_QUAD_PER_TILE_H9                            = 0x00000075,
    SC_QZ1_QUAD_PER_TILE_H10                           = 0x00000076,
    SC_QZ1_QUAD_PER_TILE_H11                           = 0x00000077,
    SC_QZ1_QUAD_PER_TILE_H12                           = 0x00000078,
    SC_QZ1_QUAD_PER_TILE_H13                           = 0x00000079,
    SC_QZ1_QUAD_PER_TILE_H14                           = 0x0000007a,
    SC_QZ1_QUAD_PER_TILE_H15                           = 0x0000007b,
    SC_QZ1_QUAD_PER_TILE_H16                           = 0x0000007c,
    SC_QZ2_QUAD_PER_TILE_H0                            = 0x0000007d,
    SC_QZ2_QUAD_PER_TILE_H1                            = 0x0000007e,
    SC_QZ2_QUAD_PER_TILE_H2                            = 0x0000007f,
    SC_QZ2_QUAD_PER_TILE_H3                            = 0x00000080,
    SC_QZ2_QUAD_PER_TILE_H4                            = 0x00000081,
    SC_QZ2_QUAD_PER_TILE_H5                            = 0x00000082,
    SC_QZ2_QUAD_PER_TILE_H6                            = 0x00000083,
    SC_QZ2_QUAD_PER_TILE_H7                            = 0x00000084,
    SC_QZ2_QUAD_PER_TILE_H8                            = 0x00000085,
    SC_QZ2_QUAD_PER_TILE_H9                            = 0x00000086,
    SC_QZ2_QUAD_PER_TILE_H10                           = 0x00000087,
    SC_QZ2_QUAD_PER_TILE_H11                           = 0x00000088,
    SC_QZ2_QUAD_PER_TILE_H12                           = 0x00000089,
    SC_QZ2_QUAD_PER_TILE_H13                           = 0x0000008a,
    SC_QZ2_QUAD_PER_TILE_H14                           = 0x0000008b,
    SC_QZ2_QUAD_PER_TILE_H15                           = 0x0000008c,
    SC_QZ2_QUAD_PER_TILE_H16                           = 0x0000008d,
    SC_QZ3_QUAD_PER_TILE_H0                            = 0x0000008e,
    SC_QZ3_QUAD_PER_TILE_H1                            = 0x0000008f,
    SC_QZ3_QUAD_PER_TILE_H2                            = 0x00000090,
    SC_QZ3_QUAD_PER_TILE_H3                            = 0x00000091,
    SC_QZ3_QUAD_PER_TILE_H4                            = 0x00000092,
    SC_QZ3_QUAD_PER_TILE_H5                            = 0x00000093,
    SC_QZ3_QUAD_PER_TILE_H6                            = 0x00000094,
    SC_QZ3_QUAD_PER_TILE_H7                            = 0x00000095,
    SC_QZ3_QUAD_PER_TILE_H8                            = 0x00000096,
    SC_QZ3_QUAD_PER_TILE_H9                            = 0x00000097,
    SC_QZ3_QUAD_PER_TILE_H10                           = 0x00000098,
    SC_QZ3_QUAD_PER_TILE_H11                           = 0x00000099,
    SC_QZ3_QUAD_PER_TILE_H12                           = 0x0000009a,
    SC_QZ3_QUAD_PER_TILE_H13                           = 0x0000009b,
    SC_QZ3_QUAD_PER_TILE_H14                           = 0x0000009c,
    SC_QZ3_QUAD_PER_TILE_H15                           = 0x0000009d,
    SC_QZ3_QUAD_PER_TILE_H16                           = 0x0000009e,
    SC_QZ0_QUAD_COUNT                                  = 0x0000009f,
    SC_QZ1_QUAD_COUNT                                  = 0x000000a0,
    SC_QZ2_QUAD_COUNT                                  = 0x000000a1,
    SC_QZ3_QUAD_COUNT                                  = 0x000000a2,
    SC_P0_HIZ_TILE_COUNT                               = 0x000000a3,
    SC_PERF_SEL_RESERVED_164                           = 0x000000a4,
    SC_PERF_SEL_RESERVED_165                           = 0x000000a5,
    SC_PERF_SEL_RESERVED_166                           = 0x000000a6,
    SC_P0_HIZ_QUAD_PER_TILE_H0                         = 0x000000a7,
    SC_P0_HIZ_QUAD_PER_TILE_H1                         = 0x000000a8,
    SC_P0_HIZ_QUAD_PER_TILE_H2                         = 0x000000a9,
    SC_P0_HIZ_QUAD_PER_TILE_H3                         = 0x000000aa,
    SC_P0_HIZ_QUAD_PER_TILE_H4                         = 0x000000ab,
    SC_P0_HIZ_QUAD_PER_TILE_H5                         = 0x000000ac,
    SC_P0_HIZ_QUAD_PER_TILE_H6                         = 0x000000ad,
    SC_P0_HIZ_QUAD_PER_TILE_H7                         = 0x000000ae,
    SC_P0_HIZ_QUAD_PER_TILE_H8                         = 0x000000af,
    SC_P0_HIZ_QUAD_PER_TILE_H9                         = 0x000000b0,
    SC_P0_HIZ_QUAD_PER_TILE_H10                        = 0x000000b1,
    SC_P0_HIZ_QUAD_PER_TILE_H11                        = 0x000000b2,
    SC_P0_HIZ_QUAD_PER_TILE_H12                        = 0x000000b3,
    SC_P0_HIZ_QUAD_PER_TILE_H13                        = 0x000000b4,
    SC_P0_HIZ_QUAD_PER_TILE_H14                        = 0x000000b5,
    SC_P0_HIZ_QUAD_PER_TILE_H15                        = 0x000000b6,
    SC_P0_HIZ_QUAD_PER_TILE_H16                        = 0x000000b7,
    SC_PERF_SEL_RESERVED_184                           = 0x000000b8,
    SC_PERF_SEL_RESERVED_185                           = 0x000000b9,
    SC_PERF_SEL_RESERVED_186                           = 0x000000ba,
    SC_PERF_SEL_RESERVED_187                           = 0x000000bb,
    SC_PERF_SEL_RESERVED_188                           = 0x000000bc,
    SC_PERF_SEL_RESERVED_189                           = 0x000000bd,
    SC_PERF_SEL_RESERVED_190                           = 0x000000be,
    SC_PERF_SEL_RESERVED_191                           = 0x000000bf,
    SC_PERF_SEL_RESERVED_192                           = 0x000000c0,
    SC_PERF_SEL_RESERVED_193                           = 0x000000c1,
    SC_PERF_SEL_RESERVED_194                           = 0x000000c2,
    SC_PERF_SEL_RESERVED_195                           = 0x000000c3,
    SC_PERF_SEL_RESERVED_196                           = 0x000000c4,
    SC_PERF_SEL_RESERVED_197                           = 0x000000c5,
    SC_PERF_SEL_RESERVED_198                           = 0x000000c6,
    SC_PERF_SEL_RESERVED_199                           = 0x000000c7,
    SC_PERF_SEL_RESERVED_200                           = 0x000000c8,
    SC_PERF_SEL_RESERVED_201                           = 0x000000c9,
    SC_PERF_SEL_RESERVED_202                           = 0x000000ca,
    SC_PERF_SEL_RESERVED_203                           = 0x000000cb,
    SC_PERF_SEL_RESERVED_204                           = 0x000000cc,
    SC_PERF_SEL_RESERVED_205                           = 0x000000cd,
    SC_PERF_SEL_RESERVED_206                           = 0x000000ce,
    SC_PERF_SEL_RESERVED_207                           = 0x000000cf,
    SC_PERF_SEL_RESERVED_208                           = 0x000000d0,
    SC_PERF_SEL_RESERVED_209                           = 0x000000d1,
    SC_PERF_SEL_RESERVED_210                           = 0x000000d2,
    SC_PERF_SEL_RESERVED_211                           = 0x000000d3,
    SC_PERF_SEL_RESERVED_212                           = 0x000000d4,
    SC_PERF_SEL_RESERVED_213                           = 0x000000d5,
    SC_PERF_SEL_RESERVED_214                           = 0x000000d6,
    SC_PERF_SEL_RESERVED_215                           = 0x000000d7,
    SC_PERF_SEL_RESERVED_216                           = 0x000000d8,
    SC_PERF_SEL_RESERVED_217                           = 0x000000d9,
    SC_PERF_SEL_RESERVED_218                           = 0x000000da,
    SC_PERF_SEL_RESERVED_219                           = 0x000000db,
    SC_PERF_SEL_RESERVED_220                           = 0x000000dc,
    SC_PERF_SEL_RESERVED_221                           = 0x000000dd,
    SC_PERF_SEL_RESERVED_222                           = 0x000000de,
    SC_PERF_SEL_RESERVED_223                           = 0x000000df,
    SC_PERF_SEL_RESERVED_224                           = 0x000000e0,
    SC_PERF_SEL_RESERVED_225                           = 0x000000e1,
    SC_PERF_SEL_RESERVED_226                           = 0x000000e2,
    SC_PERF_SEL_RESERVED_227                           = 0x000000e3,
    SC_PERF_SEL_RESERVED_228                           = 0x000000e4,
    SC_PERF_SEL_RESERVED_229                           = 0x000000e5,
    SC_PERF_SEL_RESERVED_230                           = 0x000000e6,
    SC_PERF_SEL_RESERVED_231                           = 0x000000e7,
    SC_PERF_SEL_RESERVED_232                           = 0x000000e8,
    SC_PERF_SEL_RESERVED_233                           = 0x000000e9,
    SC_PERF_SEL_RESERVED_234                           = 0x000000ea,
    SC_P0_HIZ_QUAD_COUNT                               = 0x000000eb,
    SC_PERF_SEL_RESERVED_236                           = 0x000000ec,
    SC_PERF_SEL_RESERVED_237                           = 0x000000ed,
    SC_PERF_SEL_RESERVED_238                           = 0x000000ee,
    SC_P0_DETAIL_QUAD_COUNT                            = 0x000000ef,
    SC_PERF_SEL_RESERVED_240                           = 0x000000f0,
    SC_PERF_SEL_RESERVED_241                           = 0x000000f1,
    SC_PERF_SEL_RESERVED_242                           = 0x000000f2,
    SC_P0_DETAIL_QUAD_WITH_1_PIX                       = 0x000000f3,
    SC_P0_DETAIL_QUAD_WITH_2_PIX                       = 0x000000f4,
    SC_P0_DETAIL_QUAD_WITH_3_PIX                       = 0x000000f5,
    SC_P0_DETAIL_QUAD_WITH_4_PIX                       = 0x000000f6,
    SC_PERF_SEL_RESERVED_247                           = 0x000000f7,
    SC_PERF_SEL_RESERVED_248                           = 0x000000f8,
    SC_PERF_SEL_RESERVED_249                           = 0x000000f9,
    SC_PERF_SEL_RESERVED_250                           = 0x000000fa,
    SC_PERF_SEL_RESERVED_251                           = 0x000000fb,
    SC_PERF_SEL_RESERVED_252                           = 0x000000fc,
    SC_PERF_SEL_RESERVED_253                           = 0x000000fd,
    SC_PERF_SEL_RESERVED_254                           = 0x000000fe,
    SC_PERF_SEL_RESERVED_255                           = 0x000000ff,
    SC_PERF_SEL_RESERVED_256                           = 0x00000100,
    SC_PERF_SEL_RESERVED_257                           = 0x00000101,
    SC_PERF_SEL_RESERVED_258                           = 0x00000102,
    SC_EARLYZ_QUAD_COUNT                               = 0x00000103,
    SC_EARLYZ_QUAD_WITH_1_PIX                          = 0x00000104,
    SC_EARLYZ_QUAD_WITH_2_PIX                          = 0x00000105,
    SC_EARLYZ_QUAD_WITH_3_PIX                          = 0x00000106,
    SC_EARLYZ_QUAD_WITH_4_PIX                          = 0x00000107,
    SC_PKR_QUAD_PER_ROW_H1                             = 0x00000108,
    SC_PKR_QUAD_PER_ROW_H2                             = 0x00000109,
    SC_PKR_4X2_QUAD_SPLIT                              = 0x0000010a,
    SC_PKR_4X2_FILL_QUAD                               = 0x0000010b,
    SC_PKR_END_OF_VECTOR                               = 0x0000010c,
    SC_PKR_CONTROL_XFER                                = 0x0000010d,
    SC_PKR_DBHANG_FORCE_EOV                            = 0x0000010e,
    SC_REG_SCLK_BUSY                                   = 0x0000010f,
    SC_GRP0_DYN_SCLK_BUSY                              = 0x00000110,
    SC_GRP1_DYN_SCLK_BUSY                              = 0x00000111,
    SC_GRP2_DYN_SCLK_BUSY                              = 0x00000112,
    SC_GRP3_DYN_SCLK_BUSY                              = 0x00000113,
    SC_GRP4_DYN_SCLK_BUSY                              = 0x00000114,
    SC_PA0_SC_DATA_FIFO_RD                             = 0x00000115,
    SC_PA0_SC_DATA_FIFO_WE                             = 0x00000116,
    SC_PERF_SEL_RESERVED_279                           = 0x00000117,
    SC_PERF_SEL_RESERVED_280                           = 0x00000118,
    SC_PS_ARB_XFC_ALL_EVENT_OR_PRIM_CYCLES             = 0x00000119,
    SC_PS_ARB_XFC_ONLY_PRIM_CYCLES                     = 0x0000011a,
    SC_PS_ARB_XFC_ONLY_ONE_INC_PER_PRIM                = 0x0000011b,
    SC_PS_ARB_STALLED_FROM_BELOW                       = 0x0000011c,
    SC_PS_ARB_STARVED_FROM_ABOVE                       = 0x0000011d,
    SC_PS_ARB_SC_BUSY                                  = 0x0000011e,
    SC_PS_ARB_PA_SC_BUSY                               = 0x0000011f,
    SC_PERF_SEL_RESERVED_288                           = 0x00000120,
    SC_PERF_SEL_RESERVED_289                           = 0x00000121,
    SC_PERF_SEL_RESERVED_290                           = 0x00000122,
    SC_PERF_SEL_RESERVED_291                           = 0x00000123,
    SC_PA_SC_DEALLOC_2_0_WE                            = 0x00000124,
    SC_PERF_SEL_RESERVED_293                           = 0x00000125,
    SC_PERF_SEL_RESERVED_294                           = 0x00000126,
    SC_PERF_SEL_RESERVED_295                           = 0x00000127,
    SC_PERF_SEL_RESERVED_296                           = 0x00000128,
    SC_PERF_SEL_RESERVED_297                           = 0x00000129,
    SC_PERF_SEL_RESERVED_298                           = 0x0000012a,
    SC_PERF_SEL_RESERVED_299                           = 0x0000012b,
    SC_PA0_SC_EOP_WE                                   = 0x0000012c,
    SC_PERF_SEL_RESERVED_301                           = 0x0000012d,
    SC_PA0_SC_EVENT_WE                                 = 0x0000012e,
    SC_PERF_SEL_RESERVED_303                           = 0x0000012f,
    SC_PERF_SEL_RESERVED_304                           = 0x00000130,
    SC_PERF_SEL_RESERVED_305                           = 0x00000131,
    SC_PERF_SEL_RESERVED_306                           = 0x00000132,
    SC_PERF_SEL_RESERVED_307                           = 0x00000133,
    SC_PERF_SEL_RESERVED_308                           = 0x00000134,
    SC_PERF_SEL_RESERVED_309                           = 0x00000135,
    SC_PERF_SEL_RESERVED_310                           = 0x00000136,
    SC_PERF_SEL_RESERVED_311                           = 0x00000137,
    SC_PERF_SEL_RESERVED_312                           = 0x00000138,
    SC_PERF_SEL_RESERVED_313                           = 0x00000139,
    SC_PERF_SEL_RESERVED_314                           = 0x0000013a,
    SC_PERF_SEL_RESERVED_315                           = 0x0000013b,
    SC_PERF_SEL_RESERVED_316                           = 0x0000013c,
    SC_PERF_SEL_RESERVED_317                           = 0x0000013d,
    SC_PA_SC_FPOV_WE                                   = 0x0000013e,
    SC_PERF_SEL_RESERVED_319                           = 0x0000013f,
    SC_PERF_SEL_RESERVED_320                           = 0x00000140,
    SC_PERF_SEL_RESERVED_321                           = 0x00000141,
    SC_PERF_SEL_RESERVED_322                           = 0x00000142,
    SC_PERF_SEL_RESERVED_323                           = 0x00000143,
    SC_PERF_SEL_RESERVED_324                           = 0x00000144,
    SC_PERF_SEL_RESERVED_325                           = 0x00000145,
    SC_SPI_DEALLOC_4_0                                 = 0x00000146,
    SC_SPI_DEALLOC_7_5                                 = 0x00000147,
    SC_PERF_SEL_RESERVED_328                           = 0x00000148,
    SC_PERF_SEL_RESERVED_329                           = 0x00000149,
    SC_PERF_SEL_RESERVED_330                           = 0x0000014a,
    SC_PERF_SEL_RESERVED_331                           = 0x0000014b,
    SC_PERF_SEL_RESERVED_332                           = 0x0000014c,
    SC_PERF_SEL_RESERVED_333                           = 0x0000014d,
    SC_PERF_SEL_RESERVED_334                           = 0x0000014e,
    SC_PERF_SEL_RESERVED_335                           = 0x0000014f,
    SC_PERF_SEL_RESERVED_336                           = 0x00000150,
    SC_PERF_SEL_RESERVED_337                           = 0x00000151,
    SC_SPI_FPOV_4_0                                    = 0x00000152,
    SC_SPI_FPOV_7_5                                    = 0x00000153,
    SC_PERF_SEL_RESERVED_340                           = 0x00000154,
    SC_PERF_SEL_RESERVED_341                           = 0x00000155,
    SC_SPI_EVENT                                       = 0x00000156,
    SC_PS_TS_EVENT_FIFO_PUSH                           = 0x00000157,
    SC_PS_TS_EVENT_FIFO_POP                            = 0x00000158,
    SC_PS_CTX_DONE_FIFO_PUSH                           = 0x00000159,
    SC_PS_CTX_DONE_FIFO_POP                            = 0x0000015a,
    SC_PERF_SEL_RESERVED_347                           = 0x0000015b,
    SC_PERF_SEL_RESERVED_348                           = 0x0000015c,
    SC_PA0_SC_NULL_WE                                  = 0x0000015d,
    SC_PA0_SC_NULL_DEALLOC_WE                          = 0x0000015e,
    SC_PERF_SEL_RESERVED_351                           = 0x0000015f,
    SC_PA0_SC_DATA_FIFO_EOP_RD                         = 0x00000160,
    SC_PA0_SC_DEALLOC_2_0_RD                           = 0x00000161,
    SC_PERF_SEL_RESERVED_354                           = 0x00000162,
    SC_PERF_SEL_RESERVED_355                           = 0x00000163,
    SC_PERF_SEL_RESERVED_356                           = 0x00000164,
    SC_PERF_SEL_RESERVED_357                           = 0x00000165,
    SC_PERF_SEL_RESERVED_358                           = 0x00000166,
    SC_PERF_SEL_RESERVED_359                           = 0x00000167,
    SC_PERF_SEL_RESERVED_360                           = 0x00000168,
    SC_PERF_SEL_RESERVED_361                           = 0x00000169,
    SC_PERF_SEL_RESERVED_362                           = 0x0000016a,
    SC_PERF_SEL_RESERVED_363                           = 0x0000016b,
    SC_PERF_SEL_RESERVED_364                           = 0x0000016c,
    SC_PERF_SEL_RESERVED_365                           = 0x0000016d,
    SC_PERF_SEL_RESERVED_366                           = 0x0000016e,
    SC_PERF_SEL_RESERVED_367                           = 0x0000016f,
    SC_PERF_SEL_RESERVED_368                           = 0x00000170,
    SC_PERF_SEL_RESERVED_369                           = 0x00000171,
    SC_PERF_SEL_RESERVED_370                           = 0x00000172,
    SC_PERF_SEL_RESERVED_371                           = 0x00000173,
    SC_PERF_SEL_RESERVED_372                           = 0x00000174,
    SC_PS_PA0_SC_FIFO_EMPTY                            = 0x00000175,
    SC_PS_PA0_SC_FIFO_FULL                             = 0x00000176,
    SC_PERF_SEL_RESERVED_375                           = 0x00000177,
    SC_PERF_SEL_RESERVED_376                           = 0x00000178,
    SC_PERF_SEL_RESERVED_377                           = 0x00000179,
    SC_PERF_SEL_RESERVED_378                           = 0x0000017a,
    SC_PERF_SEL_RESERVED_379                           = 0x0000017b,
    SC_PERF_SEL_RESERVED_380                           = 0x0000017c,
    SC_PERF_SEL_RESERVED_381                           = 0x0000017d,
    SC_PERF_SEL_RESERVED_382                           = 0x0000017e,
    SC_PERF_SEL_RESERVED_383                           = 0x0000017f,
    SC_PERF_SEL_RESERVED_384                           = 0x00000180,
    SC_PERF_SEL_RESERVED_385                           = 0x00000181,
    SC_BUSY_CNT_NOT_ZERO                               = 0x00000182,
    SC_BM_BUSY                                         = 0x00000183,
    SC_BACKEND_BUSY                                    = 0x00000184,
    SC_SCF_SCB_INTERFACE_BUSY                          = 0x00000185,
    SC_SCB_BUSY                                        = 0x00000186,
    SC_STARVED_BY_PA_WITH_UNSELECTED_PA_NOT_EMPTY      = 0x00000187,
    SC_STARVED_BY_PA_WITH_UNSELECTED_PA_FULL           = 0x00000188,
    SC_PBB_BIN_HIST_NUM_PRIMS                          = 0x00000189,
    SC_PBB_BATCH_HIST_NUM_PRIMS                        = 0x0000018a,
    SC_PBB_BIN_HIST_NUM_CONTEXTS                       = 0x0000018b,
    SC_PBB_BATCH_HIST_NUM_CONTEXTS                     = 0x0000018c,
    SC_PBB_BIN_HIST_NUM_PERSISTENT_STATES              = 0x0000018d,
    SC_PBB_BATCH_HIST_NUM_PERSISTENT_STATES            = 0x0000018e,
    SC_PBB_BATCH_HIST_NUM_PS_WAVE_BREAKS               = 0x0000018f,
    SC_PBB_BATCH_HIST_NUM_TRIV_REJECTED_PRIMS          = 0x00000190,
    SC_PBB_BATCH_HIST_NUM_ROWS_PER_PRIM                = 0x00000191,
    SC_PBB_BATCH_HIST_NUM_COLUMNS_PER_ROW              = 0x00000192,
    SC_PBB_BUSY                                        = 0x00000193,
    SC_PBB_BUSY_AND_NO_SENDS                           = 0x00000194,
    SC_PBB_STALLS_PA_DUE_TO_NO_TILES                   = 0x00000195,
    SC_PBB_NUM_BINS                                    = 0x00000196,
    SC_PBB_END_OF_BIN                                  = 0x00000197,
    SC_PBB_END_OF_BATCH                                = 0x00000198,
    SC_PBB_PRIMBIN_PROCESSED                           = 0x00000199,
    SC_PBB_PRIM_ADDED_TO_BATCH                         = 0x0000019a,
    SC_PBB_NONBINNED_PRIM                              = 0x0000019b,
    SC_PBB_TOTAL_REAL_PRIMS_OUT_OF_PBB                 = 0x0000019c,
    SC_PBB_TOTAL_NULL_PRIMS_OUT_OF_PBB                 = 0x0000019d,
    SC_PBB_IDLE_CLK_DUE_TO_ROW_TO_COLUMN_TRANSITION    = 0x0000019e,
    SC_PBB_IDLE_CLK_DUE_TO_FALSE_POSITIVE_ON_ROW       = 0x0000019f,
    SC_PBB_IDLE_CLK_DUE_TO_FALSE_POSITIVE_ON_COLUMN    = 0x000001a0,
    SC_PBB_BATCH_BREAK_DUE_TO_PERSISTENT_STATE         = 0x000001a1,
    SC_PBB_BATCH_BREAK_DUE_TO_CONTEXT_STATE            = 0x000001a2,
    SC_PBB_BATCH_BREAK_DUE_TO_PRIM                     = 0x000001a3,
    SC_PBB_BATCH_BREAK_DUE_TO_PC_STORAGE               = 0x000001a4,
    SC_PBB_BATCH_BREAK_DUE_TO_EVENT                    = 0x000001a5,
    SC_PBB_BATCH_BREAK_DUE_TO_FPOV_LIMIT               = 0x000001a6,
    SC_PERF_SEL_RESERVED_423                           = 0x000001a7,
    SC_PERF_SEL_RESERVED_424                           = 0x000001a8,
    SC_PERF_SEL_RESERVED_425                           = 0x000001a9,
    SC_PERF_SEL_RESERVED_426                           = 0x000001aa,
    SC_PERF_SEL_RESERVED_427                           = 0x000001ab,
    SC_PERF_SEL_RESERVED_428                           = 0x000001ac,
    SC_PERF_SEL_RESERVED_429                           = 0x000001ad,
    SC_PERF_SEL_RESERVED_430                           = 0x000001ae,
    SC_PERF_SEL_RESERVED_431                           = 0x000001af,
    SC_PERF_SEL_RESERVED_432                           = 0x000001b0,
    SC_PERF_SEL_RESERVED_433                           = 0x000001b1,
    SC_PERF_SEL_RESERVED_434                           = 0x000001b2,
    SC_PERF_SEL_RESERVED_435                           = 0x000001b3,
    SC_PERF_SEL_RESERVED_436                           = 0x000001b4,
    SC_GRP5_DYN_SCLK_BUSY                              = 0x000001b5,
    SC_GRP6_DYN_SCLK_BUSY                              = 0x000001b6,
    SC_GRP7_DYN_SCLK_BUSY                              = 0x000001b7,
    SC_GRP8_DYN_SCLK_BUSY                              = 0x000001b8,
    SC_GRP9_DYN_SCLK_BUSY                              = 0x000001b9,
    SC_PS_TO_BE_SCLK_GATE_STALL                        = 0x000001ba,
    SC_PA_TO_PBB_SCLK_GATE_STALL_STALL                 = 0x000001bb,
    SC_PK_BUSY                                         = 0x000001bc,
    SC_PK_MAX_DEALLOC_FORCE_EOV                        = 0x000001bd,
    SC_PK_DEALLOC_WAVE_BREAK                           = 0x000001be,
    SC_SPI_SEND                                        = 0x000001bf,
    SC_SPI_CREDIT_AT_ZERO_WITH_PENDING_SEND            = 0x000001c0,
    SC_SPI_CREDIT_AT_MAX                               = 0x000001c1,
    SC_SPI_CREDIT_AT_MAX_NO_PENDING_SEND               = 0x000001c2,
    SC_BCI_SEND                                        = 0x000001c3,
    SC_BCI_CREDIT_AT_ZERO_WITH_PENDING_SEND            = 0x000001c4,
    SC_BCI_CREDIT_AT_MAX                               = 0x000001c5,
    SC_BCI_CREDIT_AT_MAX_NO_PENDING_SEND               = 0x000001c6,
    SC_SPIBC_FULL_FREEZE                               = 0x000001c7,
    SC_PW_BM_PASS_EMPTY_PRIM                           = 0x000001c8,
    SC_SUPERTILE_COUNT_EXCLUDE_PASS_EMPTY_PRIM         = 0x000001c9,
    SC_SUPERTILE_PER_PRIM_EXCLUDE_PASS_EMPTY_PRIM_H0   = 0x000001ca,
    SC_SUPERTILE_PER_PRIM_EXCLUDE_PASS_EMPTY_PRIM_H1   = 0x000001cb,
    SC_SUPERTILE_PER_PRIM_EXCLUDE_PASS_EMPTY_PRIM_H2   = 0x000001cc,
    SC_SUPERTILE_PER_PRIM_EXCLUDE_PASS_EMPTY_PRIM_H3   = 0x000001cd,
    SC_SUPERTILE_PER_PRIM_EXCLUDE_PASS_EMPTY_PRIM_H4   = 0x000001ce,
    SC_SUPERTILE_PER_PRIM_EXCLUDE_PASS_EMPTY_PRIM_H5   = 0x000001cf,
    SC_SUPERTILE_PER_PRIM_EXCLUDE_PASS_EMPTY_PRIM_H6   = 0x000001d0,
    SC_SUPERTILE_PER_PRIM_EXCLUDE_PASS_EMPTY_PRIM_H7   = 0x000001d1,
    SC_SUPERTILE_PER_PRIM_EXCLUDE_PASS_EMPTY_PRIM_H8   = 0x000001d2,
    SC_SUPERTILE_PER_PRIM_EXCLUDE_PASS_EMPTY_PRIM_H9   = 0x000001d3,
    SC_SUPERTILE_PER_PRIM_EXCLUDE_PASS_EMPTY_PRIM_H10  = 0x000001d4,
    SC_SUPERTILE_PER_PRIM_EXCLUDE_PASS_EMPTY_PRIM_H11  = 0x000001d5,
    SC_SUPERTILE_PER_PRIM_EXCLUDE_PASS_EMPTY_PRIM_H12  = 0x000001d6,
    SC_SUPERTILE_PER_PRIM_EXCLUDE_PASS_EMPTY_PRIM_H13  = 0x000001d7,
    SC_SUPERTILE_PER_PRIM_EXCLUDE_PASS_EMPTY_PRIM_H14  = 0x000001d8,
    SC_SUPERTILE_PER_PRIM_EXCLUDE_PASS_EMPTY_PRIM_H15  = 0x000001d9,
    SC_SUPERTILE_PER_PRIM_EXCLUDE_PASS_EMPTY_PRIM_H16  = 0x000001da,
    SC_DB0_TILE_INTERFACE_BUSY                         = 0x000001db,
    SC_DB0_TILE_INTERFACE_SEND                         = 0x000001dc,
    SC_DB0_TILE_INTERFACE_SEND_EVENT                   = 0x000001dd,
    SC_PERF_SEL_RESERVED_478                           = 0x000001de,
    SC_PERF_SEL_RESERVED_479                           = 0x000001df,
    SC_DB0_TILE_INTERFACE_CREDIT_AT_ZERO_WITH_PENDING_SEND = 0x000001e0,
    SC_DB0_TILE_INTERFACE_CREDIT_AT_MAX                = 0x000001e1,
    SC_DB0_TILE_INTERFACE_CREDIT_AT_MAX_WITH_NO_PENDING_SEND = 0x000001e2,
    SC_PERF_SEL_RESERVED_483                           = 0x000001e3,
    SC_PERF_SEL_RESERVED_484                           = 0x000001e4,
    SC_PERF_SEL_RESERVED_485                           = 0x000001e5,
    SC_PERF_SEL_RESERVED_486                           = 0x000001e6,
    SC_PERF_SEL_RESERVED_487                           = 0x000001e7,
    SC_PERF_SEL_RESERVED_488                           = 0x000001e8,
    SC_PERF_SEL_RESERVED_489                           = 0x000001e9,
    SC_PERF_SEL_RESERVED_490                           = 0x000001ea,
    SC_BACKEND_PRIM_FIFO_FULL                          = 0x000001eb,
    SC_PBB_BATCH_BREAK_DUE_TO_TIMEOUT_COUNTER          = 0x000001ec,
    SC_PBB_BATCH_BREAK_DUE_TO_NONBINNED_BATCH          = 0x000001ed,
    SC_PBB_BATCH_BREAK_DUE_TO_DEBUG_DATA_PER_DRAW_DISPATCH = 0x000001ee,
    SC_PBB_BATCH_BREAK_DUE_TO_OVERRIDE_REGISTER_PERSISTENT = 0x000001ef,
    SC_PBB_BATCH_BREAK_DUE_TO_OVERRIDE_REGISTER_CONTEXT = 0x000001f0,
    SC_PBB_BATCH_BREAK_DUE_TO_OVERRIDE_REGISTER_FPOV   = 0x000001f1,
    SC_PBB_BATCH_BREAK_DUE_TO_NEW_SC_MODE              = 0x000001f2,
    SC_PBB_BATCH_BREAK_DUE_TO_BINNING_MODE_CHANGE      = 0x000001f3,
    SC_PBB_BATCH_BREAK_DUE_TO_PIPELINE_EVENT_COUNT     = 0x000001f4,
    SC_PBB_BATCH_BREAK_DUE_TO_PIPE_RESET               = 0x000001f5,
    SC_PBB_BATCH_BREAK_DUE_TO_GFX_PIPE_CHANGE          = 0x000001f6,
    SC_STALLED_BY_DB0_TILEFIFO                         = 0x000001f7,
    SC_DB0_QUAD_INTF_SEND                              = 0x000001f8,
    SC_DB0_QUAD_INTF_BUSY                              = 0x000001f9,
    SC_DB0_QUAD_INTF_STALLED_BY_DB                     = 0x000001fa,
    SC_DB0_QUAD_INTF_CREDIT_AT_MAX                     = 0x000001fb,
    SC_DB0_QUAD_INTF_IDLE                              = 0x000001fc,
    SC_PERF_SEL_RESERVED_509                           = 0x000001fd,
    SC_PERF_SEL_RESERVED_510                           = 0x000001fe,
    SC_PERF_SEL_RESERVED_511                           = 0x000001ff,
    SC_PERF_SEL_RESERVED_512                           = 0x00000200,
    SC_PERF_SEL_RESERVED_513                           = 0x00000201,
    SC_PERF_SEL_RESERVED_514                           = 0x00000202,
    SC_PKR_WAVE_BREAK_OUTSIDE_REGION                   = 0x00000203,
    SC_PKR_WAVE_BREAK_FULL_TILE                        = 0x00000204,
    SC_RESERVED_60                                     = 0x00000205,
    SC_PBB_EMPTY_INPUT_CYCLE_WHEN_BATCH_OPEN           = 0x00000206,
    SC_PBB_BATCH_BREAK_DUE_TO_NULL_PRIM_BREAK_BATCH_LIMIT = 0x00000207,
    SC_DB0_WE_STALLED_BY_RSLT_FIFO_FULL                = 0x00000208,
    SC_DB0_WE_TILE_MASK_RETURN_FIFO_FULL_WITH_WE_RSLT_FIFO_STALL = 0x00000209,
    SC_DB0_TILE_MASK_FIFO_FULL                         = 0x0000020a,
    SC_PERF_SEL_RESERVED_523                           = 0x0000020b,
    SC_PERF_SEL_RESERVED_524                           = 0x0000020c,
    SC_PERF_SEL_RESERVED_525                           = 0x0000020d,
    SC_PS_PM_PBB_TO_PSE_FIFO_WE_STALL_BY_PFF_PW_FULL   = 0x0000020e,
    SC_PS_PM_PBB_TO_PSE_FIFO_WE_STALL_BY_ZFF_PW_FULL   = 0x0000020f,
    SC_PS_PM_PBB_TO_PSE_FIFO_WE_STALL_BY_PBB_TO_PSE_FIFO_FULL = 0x00000210,
    SC_PS_PM_PFF_PW_FULL                               = 0x00000211,
    SC_PS_PM_ZFF_PW_FULL                               = 0x00000212,
    SC_PS_PM_PBB_TO_PSE_FIFO_FULL                      = 0x00000213,
    SC_PERF_SEL_RESERVED_532                           = 0x00000214,
    SC_PERF_SEL_RESERVED_533                           = 0x00000215,
    SC_PERF_SEL_RESERVED_534                           = 0x00000216,
    SC_PK_PM_4X2_SPLIT_WAVE_BRK_1H                     = 0x00000217,
    SC_PK_PM_PKR_FILL_4X2_WAVE_BRK_1H                  = 0x00000218,
    SC_PK_PM_SPLIT_OR_FILL_4X2_WAVE_BRK_1H             = 0x00000219,
    SC_PK_PM_END_OF_VECTOR_WAVE_BRK_1H                 = 0x0000021a,
    SC_PERF_SEL_RESERVED_539                           = 0x0000021b,
    SC_PK_PM_CTL_ONLY_CMD_WAVE_BRK_1H                  = 0x0000021c,
    SC_PK_PM_AVOID_DEALLOC_ADD_WAVE_BRK_1H             = 0x0000021d,
    SC_PK_PM_FD_CONFLICT_WAVE_BRK_1H                   = 0x0000021e,
    SC_PK_PM_FORCE_PARTIAL_FOR_DEALLOC_WAVE_BRK_1H     = 0x0000021f,
    SC_PK_PM_AE_CONFLICT_WAVE_BRK_1H                   = 0x00000220,
    SC_PK_PM_EOP_OR_LAD_WAVE_BRK_1H                    = 0x00000221,
    SC_PK_PM_FULL_TILE_WAVE_BRK_1H                     = 0x00000222,
    SC_PK_PM_OREO_CONFLICT_QUAD_FORCE_EOV_WAVE_BRK_1H  = 0x00000223,
    SC_PK_PM_MAX_DEALLOC_FORCE_EOV_WAVE_BRK_1H         = 0x00000224,
    SC_PK_PM_WAVE_BREAK_OUTSIDE_REGION_WAVE_BRK_1H     = 0x00000225,
    SC_PK_PM_MAX_CLK_CNT_FORCE_EOV_WAVE_BRK_1H         = 0x00000226,
    SC_PK_PM_MAX_REZ_CNT_FORCE_EOV_WAVE_BRK_1H         = 0x00000227,
    SC_PK_PM_VRS_RATE_X_00_Y_00_QUAD                   = 0x00000228,
    SC_PK_PM_VRS_RATE_X_00_Y_01_QUAD                   = 0x00000229,
    SC_PK_PM_VRS_RATE_X_00_Y_10_QUAD                   = 0x0000022a,
    SC_PK_PM_VRS_RATE_X_00_Y_11_QUAD                   = 0x0000022b,
    SC_PK_PM_VRS_RATE_X_01_Y_00_QUAD                   = 0x0000022c,
    SC_PK_PM_VRS_RATE_X_01_Y_01_QUAD                   = 0x0000022d,
    SC_PK_PM_VRS_RATE_X_01_Y_10_QUAD                   = 0x0000022e,
    SC_PK_PM_VRS_RATE_X_01_Y_11_QUAD                   = 0x0000022f,
    SC_PK_PM_VRS_RATE_X_10_Y_00_QUAD                   = 0x00000230,
    SC_PK_PM_VRS_RATE_X_10_Y_01_QUAD                   = 0x00000231,
    SC_PK_PM_VRS_RATE_X_10_Y_10_QUAD                   = 0x00000232,
    SC_PK_PM_VRS_RATE_X_10_Y_11_QUAD                   = 0x00000233,
    SC_PK_PM_VRS_RATE_X_11_Y_00_QUAD                   = 0x00000234,
    SC_PK_PM_VRS_RATE_X_11_Y_01_QUAD                   = 0x00000235,
    SC_PK_PM_VRS_RATE_X_11_Y_10_QUAD                   = 0x00000236,
    SC_PK_PM_VRS_RATE_X_11_Y_11_QUAD                   = 0x00000237,
    SC_PERF_SEL_RESERVED_568                           = 0x00000238,
    SC_PBB_RESERVED                                    = 0x00000239,
    SC_BM_BE0_STALLED                                  = 0x0000023a,
    SC_BM_BE1_STALLED                                  = 0x0000023b,
    SC_BM_BE2_STALLED                                  = 0x0000023c,
    SC_BM_BE3_STALLED                                  = 0x0000023d,
    SC_BM_MULTI_ACCUM_1_BE_STALLED                     = 0x0000023e,
    SC_BM_MULTI_ACCUM_2_BE_STALLED                     = 0x0000023f,
    SC_BM_MULTI_ACCUM_3_BE_STALLED                     = 0x00000240,
    SC_BM_MULTI_ACCUM_4_BE_STALLED                     = 0x00000241,
    SC_PBB_READ_PH0                                    = 0x00000242,
    SC_PBB_READ_DEALLOC_4_0                            = 0x00000243,
    SC_PBB_READ_DEALLOC_7_5                            = 0x00000244,
    SC_PBB_READ_FPOG_4_0                               = 0x00000245,
    SC_PBB_READ_FPOG_7_5                               = 0x00000246,
    SC_VRC_SECTOR_HIT                                  = 0x00000247,
    SC_VRC_TAG_MISS                                    = 0x00000248,
    SC_VRC_SECTOR_MISS                                 = 0x00000249,
    SC_VRC_LRU_EVICT_STALL                             = 0x0000024a,
    SC_VRC_LRU_EVICT_SCHEDULED_EVICT_STALL             = 0x0000024b,
    SC_VRC_LRU_EVICT_PENDING_EVICT_STALL               = 0x0000024c,
    SC_VRC_REEVICTION_STALL                            = 0x0000024d,
    SC_VRC_EVICT_NONZERO_INFLIGHT_STALL                = 0x0000024e,
    SC_VRC_REPLACE_SCHEDULED_EVICT_STALL               = 0x0000024f,
    SC_VRC_REPLACE_PENDING_EVICT_STALL                 = 0x00000250,
    SC_VRC_REPLACE_FLUSH_IN_PROGRESS_STALL             = 0x00000251,
    SC_VRC_INFLIGHT_COUNTER_MAXIMUM_STALL              = 0x00000252,
    SC_VRC_READ_OUTPUT_STALL                           = 0x00000253,
    SC_VRC_WRITE_OUTPUT_STALL                          = 0x00000254,
    SC_VRC_ACK_OUTPUT_STALL                            = 0x00000255,
    SC_VRC_FLUSH_EVICT_STALL                           = 0x00000256,
    SC_VRC_FLUSH_REFLUSH_STALL                         = 0x00000257,
    SC_VRC_FLUSH_FIP_HIT_STALL                         = 0x00000258,
    SC_VRC_FLUSH_WRREQ_DRAIN_STALL                     = 0x00000259,
    SC_VRC_FLUSH_DONE_STALL                            = 0x0000025a,
    SC_VRC_FLUSH_STALL                                 = 0x0000025b,
    SC_VRC_STALL                                       = 0x0000025c,
    SC_VRC_FLUSH                                       = 0x0000025d,
    SC_VRC_SECTORS_FLUSHED                             = 0x0000025e,
    SC_VRC_DIRTY_SECTORS_FLUSHED                       = 0x0000025f,
    SC_VRC_TAGS_FLUSHED                                = 0x00000260,
    SC_VRC_HPF_REQ                                     = 0x00000261,
    SC_VRC_HPF_EVENT                                   = 0x00000262,
    SC_VRC_HPF_STALLED                                 = 0x00000263,
    SC_VRC_PROBE_ACK_TILES                             = 0x00000264,
    SC_VRC_GL1X_RD_REQ                                 = 0x00000265,
    SC_VRC_GL1X_WR_REQ                                 = 0x00000266,
    SC_VRC_GL1X_SRC_XFR                                = 0x00000267,
    SC_VRC_GL1X_RD_RET                                 = 0x00000268,
    SC_VRC_GL1X_WR_ACK                                 = 0x00000269,
    SC_VRC_GL1X_RD_XNACK                               = 0x0000026a,
    SC_VRC_GL1X_WR_XNACK                               = 0x0000026b,
    SC_VRC_GL1X_REQ_STALLED                            = 0x0000026c,
    SC_VRC_GL1X_SRC_STALLED                            = 0x0000026d,
    SC_VRC_RATEMEM_WE_CNT                              = 0x0000026e,
    SC_VRC_RATEMEM_RE_CNT                              = 0x0000026f,
    SC_VRC_HINTMEM_WE_CNT                              = 0x00000270,
    SC_VRC_HINTMEM_RE_CNT                              = 0x00000271,
    SC_VRC_BUSY                                        = 0x00000272,
    SC_GL1X_BUSY                                       = 0x00000273,
    SC_BE_VRS_RD_REQ                                   = 0x00000274,
    SC_BE_VRS_RD_REQ_STALLED                           = 0x00000275,
    SC_BE_VRS_RD_REQ_HIT                               = 0x00000276,
    SC_BE_VRS_RD_RET                                   = 0x00000277,
    SC_BE_VRS_RD_RET_STALLED                           = 0x00000278,
    SC_BE_VRS_FB_RET                                   = 0x00000279,
    SC_BE_VRS_FB_RET_STALLED                           = 0x0000027a,
    SC_BE_VRS_FB_RET_HIT                               = 0x0000027b,
    SC_VRS_BE_BUSY                                     = 0x0000027c,
    SC_PWS_CS_EVENTS_PWS_ENABLE                        = 0x0000027d,
    SC_PWS_PS_EVENTS_PWS_ENABLE                        = 0x0000027e,
    SC_PWS_TS_EVENTS_PWS_ENABLE                        = 0x0000027f,
    SC_PWS_STALLED                                     = 0x00000280,
    SC_PWS_P0_CS_SYNC_COMPLETE                         = 0x00000281,
    SC_PWS_P0_PS_SYNC_COMPLETE                         = 0x00000282,
    SC_PWS_P0_TS_SYNC_COMPLETE                         = 0x00000283,
    SC_PWS_P1_CS_SYNC_COMPLETE                         = 0x00000284,
    SC_PWS_P1_PS_SYNC_COMPLETE                         = 0x00000285,
    SC_PWS_P1_TS_SYNC_COMPLETE                         = 0x00000286,
    SC_PKR_PC_NO_CREDITS                               = 0x00000287,
    SC_PKR_PC_STALLED                                  = 0x00000288,
    SC_PKR_PC_SEND                                     = 0x00000289,
    SC_PKR_PC_SEND_PRIM_VALID_1                        = 0x0000028a,
    SC_PKR_PC_SEND_PRIM_VALID_0                        = 0x0000028b,
    SC_PKR_PC_SEND_TRUE_PRIM                           = 0x0000028c,
    SC_PKR_PC_SEND_EOV                                 = 0x0000028d,
    SC_PKR_PC_SEND_EVENT                               = 0x0000028e,
    SC_PKR_DB_WAVE_STALL                               = 0x0000028f,
    SC_PKR_PSINVOC_SEDC_FIFO_FULL                      = 0x00000290,
    SC_PKR_OREO_STALLED_BY_NO_VALID_WAIVE_ID           = 0x00000291,
    SC_PKR_SPI_QUAD_COUNT                              = 0x00000292,
    SC_PKR_DB_OREO_WAVE_QUAD_COUNT                     = 0x00000293,
    SC_PKR_BCI_QUAD_NEW_PRIM                           = 0x00000294,
    SC_SPI_WAVE_STALLED_BY_SPI                         = 0x00000295,
    SC_HISZ_TEST0_STILE_SEND                           = 0x00000296,
    SC_HISZ_TEST0_STILE_STALL                          = 0x00000297,
    SC_HISZ_TEST0_STILE_FAST_Z_SET_OP                  = 0x00000298,
    SC_HISZ_TEST0_STILE_STENCIL_FAST_SET_OP            = 0x00000299,
    SC_HISZ_TEST0_STILE_CULLED                         = 0x0000029a,
    SC_HISZ_TEST0_STILE_HIER_KILL                      = 0x0000029b,
    SC_HISZ_TEST0_STILE_TILE_RATE                      = 0x0000029c,
    SC_HISZ_TEST0_STILE_FAST_OP                        = 0x0000029d,
    SC_HISZ_TEST0_STILE_NO_OP                          = 0x0000029e,
    SC_HISZ_TEST0_STILE_SSAA_KILL                      = 0x0000029f,
    SC_HISZ_TEST0_STILE_Z_TILE_RATE                    = 0x000002a0,
    SC_HISZ_TEST0_STILE_S_TILE_RATE                    = 0x000002a1,
    SC_HISZ_TEST0_STILE_C_TILE_RATE                    = 0x000002a2,
    SC_HISZ_TEST0_HIS_STILE_CULLED                     = 0x000002a3,
    SC_HISZ_TEST0_HIZ_STILE_CULLED                     = 0x000002a4,
    SC_HISZ_TEST0_HCA_UPD_STALLED                      = 0x000002a5,
    SC_HISZ_TEST0_DEPTH_BOUNDS_STILE_CULLED            = 0x000002a6,
    SC_HISZ_TEST0_HCA_RET_STARVED                      = 0x000002a7,
    SC_HISZ_TEST1_STILE_SEND                           = 0x000002a8,
    SC_HISZ_TEST1_STILE_STALL                          = 0x000002a9,
    SC_HISZ_TEST1_STILE_FAST_Z_SET_OP                  = 0x000002aa,
    SC_HISZ_TEST1_STILE_STENCIL_FAST_SET_OP            = 0x000002ab,
    SC_HISZ_TEST1_STILE_CULLED                         = 0x000002ac,
    SC_HISZ_TEST1_STILE_HIER_KILL                      = 0x000002ad,
    SC_HISZ_TEST1_STILE_TILE_RATE                      = 0x000002ae,
    SC_HISZ_TEST1_STILE_FAST_OP                        = 0x000002af,
    SC_HISZ_TEST1_STILE_NO_OP                          = 0x000002b0,
    SC_HISZ_TEST1_STILE_SSAA_KILL                      = 0x000002b1,
    SC_HISZ_TEST1_STILE_Z_TILE_RATE                    = 0x000002b2,
    SC_HISZ_TEST1_STILE_S_TILE_RATE                    = 0x000002b3,
    SC_HISZ_TEST1_STILE_C_TILE_RATE                    = 0x000002b4,
    SC_HISZ_TEST1_HIS_STILE_CULLED                     = 0x000002b5,
    SC_HISZ_TEST1_HIZ_STILE_CULLED                     = 0x000002b6,
    SC_HISZ_TEST1_HCA_UPD_STALLED                      = 0x000002b7,
    SC_HISZ_TEST1_DEPTH_BOUNDS_STILE_CULLED            = 0x000002b8,
    SC_HISZ_TEST1_HCA_RET_STARVED                      = 0x000002b9,
    SC_HISZ_TEST2_STILE_SEND                           = 0x000002ba,
    SC_HISZ_TEST2_STILE_STALL                          = 0x000002bb,
    SC_HISZ_TEST2_STILE_FAST_Z_SET_OP                  = 0x000002bc,
    SC_HISZ_TEST2_STILE_STENCIL_FAST_SET_OP            = 0x000002bd,
    SC_HISZ_TEST2_STILE_CULLED                         = 0x000002be,
    SC_HISZ_TEST2_STILE_HIER_KILL                      = 0x000002bf,
    SC_HISZ_TEST2_STILE_TILE_RATE                      = 0x000002c0,
    SC_HISZ_TEST2_STILE_FAST_OP                        = 0x000002c1,
    SC_HISZ_TEST2_STILE_NO_OP                          = 0x000002c2,
    SC_HISZ_TEST2_STILE_SSAA_KILL                      = 0x000002c3,
    SC_HISZ_TEST2_STILE_Z_TILE_RATE                    = 0x000002c4,
    SC_HISZ_TEST2_STILE_S_TILE_RATE                    = 0x000002c5,
    SC_HISZ_TEST2_STILE_C_TILE_RATE                    = 0x000002c6,
    SC_HISZ_TEST2_HIS_STILE_CULLED                     = 0x000002c7,
    SC_HISZ_TEST2_HIZ_STILE_CULLED                     = 0x000002c8,
    SC_HISZ_TEST2_HCA_UPD_STALLED                      = 0x000002c9,
    SC_HISZ_TEST2_DEPTH_BOUNDS_STILE_CULLED            = 0x000002ca,
    SC_HISZ_TEST2_HCA_RET_STARVED                      = 0x000002cb,
    SC_HISZ_TEST3_STILE_SEND                           = 0x000002cc,
    SC_HISZ_TEST3_STILE_STALL                          = 0x000002cd,
    SC_HISZ_TEST3_STILE_FAST_Z_SET_OP                  = 0x000002ce,
    SC_HISZ_TEST3_STILE_STENCIL_FAST_SET_OP            = 0x000002cf,
    SC_HISZ_TEST3_STILE_CULLED                         = 0x000002d0,
    SC_HISZ_TEST3_STILE_HIER_KILL                      = 0x000002d1,
    SC_HISZ_TEST3_STILE_TILE_RATE                      = 0x000002d2,
    SC_HISZ_TEST3_STILE_FAST_OP                        = 0x000002d3,
    SC_HISZ_TEST3_STILE_NO_OP                          = 0x000002d4,
    SC_HISZ_TEST3_STILE_SSAA_KILL                      = 0x000002d5,
    SC_HISZ_TEST3_STILE_Z_TILE_RATE                    = 0x000002d6,
    SC_HISZ_TEST3_STILE_S_TILE_RATE                    = 0x000002d7,
    SC_HISZ_TEST3_STILE_C_TILE_RATE                    = 0x000002d8,
    SC_HISZ_TEST3_HIS_STILE_CULLED                     = 0x000002d9,
    SC_HISZ_TEST3_HIZ_STILE_CULLED                     = 0x000002da,
    SC_HISZ_TEST3_HCA_UPD_STALLED                      = 0x000002db,
    SC_HISZ_TEST3_DEPTH_BOUNDS_STILE_CULLED            = 0x000002dc,
    SC_HISZ_TEST3_HCA_RET_STARVED                      = 0x000002dd,
    SC_TILE_PICKED_H2                                  = 0x000002de,
    SC_TILE_PICKED_H3                                  = 0x000002df,
    SC_TILE_PICKED_H4                                  = 0x000002e0,
    SC_TILE_PICKED_H5                                  = 0x000002e1,
    SC_HSC_SECTOR_HIT                                  = 0x000002e2,
    SC_HSC_TAG_MISS                                    = 0x000002e3,
    SC_HSC_SECTOR_MISS                                 = 0x000002e4,
    SC_HSC_LRU_EVICT_STALL                             = 0x000002e5,
    SC_HSC_LRU_EVICT_SCHEDULED_EVICT_STALL             = 0x000002e6,
    SC_HSC_LRU_EVICT_PENDING_EVICT_STALL               = 0x000002e7,
    SC_HSC_REEVICTION_STALL                            = 0x000002e8,
    SC_HSC_EVICT_NONZERO_INFLIGHT_STALL                = 0x000002e9,
    SC_HSC_REPLACE_SCHEDULED_EVICT_STALL               = 0x000002ea,
    SC_HSC_REPLACE_PENDING_EVICT_STALL                 = 0x000002eb,
    SC_HSC_REPLACE_FLUSH_IN_PROGRESS_STALL             = 0x000002ec,
    SC_HSC_INFLIGHT_COUNTER_MAXIMUM_STALL              = 0x000002ed,
    SC_HSC_READ_OUTPUT_STALL                           = 0x000002ee,
    SC_HSC_WRITE_OUTPUT_STALL                          = 0x000002ef,
    SC_HSC_ACK_OUTPUT_STALL                            = 0x000002f0,
    SC_HSC_FLUSH_EVICT_STALL                           = 0x000002f1,
    SC_HSC_FLUSH_REFLUSH_STALL                         = 0x000002f2,
    SC_HSC_FLUSH_FIP_HIT_STALL                         = 0x000002f3,
    SC_HSC_FLUSH_WRREQ_DRAIN_STALL                     = 0x000002f4,
    SC_HSC_FLUSH_DONE_STALL                            = 0x000002f5,
    SC_HSC_FLUSH_STALL                                 = 0x000002f6,
    SC_HSC_STALL                                       = 0x000002f7,
    SC_HSC_FLUSH                                       = 0x000002f8,
    SC_HSC_SECTORS_FLUSHED                             = 0x000002f9,
    SC_HSC_DIRTY_SECTORS_FLUSHED                       = 0x000002fa,
    SC_HSC_TAGS_FLUSHED                                = 0x000002fb,
    SC_HSC_HPF_REQ                                     = 0x000002fc,
    SC_HSC_HPF_EVENT                                   = 0x000002fd,
    SC_HSC_HPF_STALLED                                 = 0x000002fe,
    SC_HSC_PROBE_ACK_TILES                             = 0x000002ff,
    SC_HSC_GL1X_RD_REQ                                 = 0x00000300,
    SC_HSC_GL1X_WR_REQ                                 = 0x00000301,
    SC_HSC_GL1X_SRC_XFR                                = 0x00000302,
    SC_HSC_GL1X_RD_RET                                 = 0x00000303,
    SC_HSC_GL1X_WR_ACK                                 = 0x00000304,
    SC_HSC_GL1X_RD_XNACK                               = 0x00000305,
    SC_HSC_GL1X_WR_XNACK                               = 0x00000306,
    SC_HSC_GL1X_REQ_STALLED                            = 0x00000307,
    SC_HSC_GL1X_SRC_STALLED                            = 0x00000308,
    SC_HSC_SURFMEM_WE_CNT                              = 0x00000309,
    SC_HSC_SURFMEM_RE_CNT                              = 0x0000030a,
    SC_HSC_BUSY                                        = 0x0000030b,
    SC_HZC_SECTOR_HIT                                  = 0x0000030c,
    SC_HZC_TAG_MISS                                    = 0x0000030d,
    SC_HZC_SECTOR_MISS                                 = 0x0000030e,
    SC_HZC_LRU_EVICT_STALL                             = 0x0000030f,
    SC_HZC_LRU_EVICT_SCHEDULED_EVICT_STALL             = 0x00000310,
    SC_HZC_LRU_EVICT_PENDING_EVICT_STALL               = 0x00000311,
    SC_HZC_REEVICTION_STALL                            = 0x00000312,
    SC_HZC_EVICT_NONZERO_INFLIGHT_STALL                = 0x00000313,
    SC_HZC_REPLACE_SCHEDULED_EVICT_STALL               = 0x00000314,
    SC_HZC_REPLACE_PENDING_EVICT_STALL                 = 0x00000315,
    SC_HZC_REPLACE_FLUSH_IN_PROGRESS_STALL             = 0x00000316,
    SC_HZC_INFLIGHT_COUNTER_MAXIMUM_STALL              = 0x00000317,
    SC_HZC_READ_OUTPUT_STALL                           = 0x00000318,
    SC_HZC_WRITE_OUTPUT_STALL                          = 0x00000319,
    SC_HZC_ACK_OUTPUT_STALL                            = 0x0000031a,
    SC_HZC_FLUSH_EVICT_STALL                           = 0x0000031b,
    SC_HZC_FLUSH_REFLUSH_STALL                         = 0x0000031c,
    SC_HZC_FLUSH_FIP_HIT_STALL                         = 0x0000031d,
    SC_HZC_FLUSH_WRREQ_DRAIN_STALL                     = 0x0000031e,
    SC_HZC_FLUSH_DONE_STALL                            = 0x0000031f,
    SC_HZC_FLUSH_STALL                                 = 0x00000320,
    SC_HZC_STALL                                       = 0x00000321,
    SC_HZC_FLUSH                                       = 0x00000322,
    SC_HZC_SECTORS_FLUSHED                             = 0x00000323,
    SC_HZC_DIRTY_SECTORS_FLUSHED                       = 0x00000324,
    SC_HZC_TAGS_FLUSHED                                = 0x00000325,
    SC_HZC_HPF_REQ                                     = 0x00000326,
    SC_HZC_HPF_EVENT                                   = 0x00000327,
    SC_HZC_HPF_STALLED                                 = 0x00000328,
    SC_HZC_PROBE_ACK_TILES                             = 0x00000329,
    SC_HZC_GL1X_RD_REQ                                 = 0x0000032a,
    SC_HZC_GL1X_WR_REQ                                 = 0x0000032b,
    SC_HZC_GL1X_SRC_XFR                                = 0x0000032c,
    SC_HZC_GL1X_RD_RET                                 = 0x0000032d,
    SC_HZC_GL1X_WR_ACK                                 = 0x0000032e,
    SC_HZC_GL1X_RD_XNACK                               = 0x0000032f,
    SC_HZC_GL1X_WR_XNACK                               = 0x00000330,
    SC_HZC_GL1X_REQ_STALLED                            = 0x00000331,
    SC_HZC_GL1X_SRC_STALLED                            = 0x00000332,
    SC_HZC_SURFMEM_WE_CNT                              = 0x00000333,
    SC_HZC_SURFMEM_RE_CNT                              = 0x00000334,
    SC_HZC_BUSY                                        = 0x00000335,
} SC_PERFCNT_SEL;

constexpr unsigned int MaxScPerfcntSel                 = SC_HZC_BUSY;

typedef enum SDMA_PERFMON_SEL {
    SDMA_PERFMON_SEL_CYCLE                             = 0x00000000,
    SDMA_PERFMON_SEL_IDLE                              = 0x00000001,
    SDMA_PERFMON_SEL_BUSY                              = 0x00000002,
    SDMA_PERFMON_SEL_CE_READ_REQUEST                   = 0x00000003,
    SDMA_PERFMON_SEL_CE_READ_RETURN                    = 0x00000004,
    SDMA_PERFMON_SEL_CE_WRITE_ADDR_REQUEST             = 0x00000005,
    SDMA_PERFMON_SEL_CE_WRITE_DATA_REQUEST             = 0x00000006,
    SDMA_PERFMON_SEL_CE_WRITE_RETURN                   = 0x00000007,
    SDMA_PERFMON_SEL_REG_IDLE                          = 0x00000008,
    SDMA_PERFMON_SEL_REG_BUSY                          = 0x00000009,
    SDMA_PERFMON_SEL_RB_EMPTY                          = 0x0000000a,
    SDMA_PERFMON_SEL_RB_FULL                           = 0x0000000b,
    SDMA_PERFMON_SEL_RB_WPTR_POLL_READ                 = 0x0000000c,
    SDMA_PERFMON_SEL_RB_RPTR_WB                        = 0x0000000d,
    SDMA_PERFMON_SEL_RB_CMD_IDLE                       = 0x0000000e,
    SDMA_PERFMON_SEL_RB_CMD_FULL                       = 0x0000000f,
    SDMA_PERFMON_SEL_IB_CMD_IDLE                       = 0x00000010,
    SDMA_PERFMON_SEL_IB_CMD_FULL                       = 0x00000011,
    SDMA_PERFMON_SEL_SRBM_REG_SEND                     = 0x00000012,
    SDMA_PERFMON_SEL_INT_IDLE                          = 0x00000013,
    SDMA_PERFMON_SEL_INT_REQ_COUNT                     = 0x00000014,
    SDMA_PERFMON_SEL_NUM_PACKET                        = 0x00000015,
    SDMA_PERFMON_SEL_DMA_CE_IDLE_STAT_0                = 0x00000016,
    SDMA_PERFMON_SEL_DMA_CE_IDLE_STAT_1                = 0x00000017,
    SDMA_PERFMON_SEL_DMA_CE_IDLE_STAT_2                = 0x00000018,
    SDMA_PERFMON_SEL_DMA_CE_IDLE_STAT_3                = 0x00000019,
    SDMA_PERFMON_SEL_DMA_CE_IDLE_STAT_4                = 0x0000001a,
    SDMA_PERFMON_SEL_DMA_CE_IDLE_STAT_5                = 0x0000001b,
    SDMA_PERFMON_SEL_DMA_CE_IDLE_STAT_6                = 0x0000001c,
    SDMA_PERFMON_SEL_DMA_CE_IDLE_STAT_7                = 0x0000001d,
    SDMA_PERFMON_SEL_DMA_CE_IDLE_STAT_8                = 0x0000001e,
    SDMA_PERFMON_SEL_DMA_CE_IDLE_STAT_9                = 0x0000001f,
    SDMA_PERFMON_SEL_MCU_CACHE0_HIT                    = 0x00000020,
    SDMA_PERFMON_SEL_MCU_CACHE0_MISS                   = 0x00000021,
    SDMA_PERFMON_SEL_MCU_CACHE1_HIT                    = 0x00000022,
    SDMA_PERFMON_SEL_MCU_CACHE1_MISS                   = 0x00000023,
    SDMA_PERFMON_SEL_MCU_GSK_GRBM_RD                   = 0x00000024,
    SDMA_PERFMON_SEL_MCU_GSK_GRBM_WR                   = 0x00000025,
    SDMA_PERFMON_SEL_MCU_GSK_MEM_RD                    = 0x00000026,
    SDMA_PERFMON_SEL_MCU_GSK_MEM_WR                    = 0x00000027,
    SDMA_PERFMON_SEL_PROC_CNTL_IDLE                    = 0x00000028,
    SDMA_PERFMON_SEL_INSTR_FETCH_IDLE                  = 0x00000029,
    SDMA_PERFMON_SEL_CE_AFIFO_FULL                     = 0x0000002a,
    SDMA_PERFMON_SEL_CE_INFO_FULL                      = 0x0000002b,
    SDMA_PERFMON_SEL_CE_INFO1_FULL                     = 0x0000002c,
    SDMA_PERFMON_SEL_CE_RD_STALL                       = 0x0000002d,
    SDMA_PERFMON_SEL_QUEUE0_SELECT                     = 0x0000002e,
    SDMA_PERFMON_SEL_QUEUE1_SELECT                     = 0x0000002f,
    SDMA_PERFMON_SEL_QUEUE2_SELECT                     = 0x00000030,
    SDMA_PERFMON_SEL_QUEUE3_SELECT                     = 0x00000031,
    SDMA_PERFMON_SEL_QUEUE4_SELECT                     = 0x00000032,
    SDMA_PERFMON_SEL_QUEUE5_SELECT                     = 0x00000033,
    SDMA_PERFMON_SEL_QUEUE6_SELECT                     = 0x00000034,
    SDMA_PERFMON_SEL_QUEUE7_SELECT                     = 0x00000035,
    SDMA_PERFMON_SEL_CTX_CHANGE                        = 0x00000036,
    SDMA_PERFMON_SEL_CTX_CHANGE_EXPIRED                = 0x00000037,
    SDMA_PERFMON_SEL_CTX_CHANGE_EXCEPTION              = 0x00000038,
    SDMA_PERFMON_SEL_DOORBELL                          = 0x00000039,
    SDMA_PERFMON_SEL_L63_MEMREQ_RD_IN_SE_VLD           = 0x0000003a,
    SDMA_PERFMON_SEL_L62_MEMREQ_RD_OUT_SE_VLD          = 0x0000003b,
    SDMA_PERFMON_SEL_L61_MEMREQ_RD_IN_REGSCH_VLD       = 0x0000003c,
    SDMA_PERFMON_SEL_L60_MEMREQ_RD_OUT_REGSCH_VLD      = 0x0000003d,
    SDMA_PERFMON_SEL_L59_MEMREQ_RD_IN_MCU_VLD          = 0x0000003e,
    SDMA_PERFMON_SEL_L58_MEMREQ_RD_OUT_MCU_VLD         = 0x0000003f,
    SDMA_PERFMON_SEL_L57_MEMREQ_RD_IN_CMDFETCH_VLD     = 0x00000040,
    SDMA_PERFMON_SEL_L56_MEMREQ_RD_OUT_CMDFETCH_VLD    = 0x00000041,
    SDMA_PERFMON_SEL_L55_MEMREQ_WR_IN_SE_VLD           = 0x00000042,
    SDMA_PERFMON_SEL_L54_MEMREQ_WR_OUT_SE_VLD          = 0x00000043,
    SDMA_PERFMON_SEL_L53_MEMREQ_WR_IN_REGSCH_VLD       = 0x00000044,
    SDMA_PERFMON_SEL_L52_MEMREQ_WR_OUT_REGSCH_VLD      = 0x00000045,
    SDMA_PERFMON_SEL_L51_MEMREQ_WR_IN_MCU_VLD          = 0x00000046,
    SDMA_PERFMON_SEL_L50_MEMREQ_WR_OUT_MCU_VLD         = 0x00000047,
    SDMA_PERFMON_SEL_L49_MEMREQ_RD_IN_CE_VLD           = 0x00000048,
    SDMA_PERFMON_SEL_L48_MEMREQ_RD_OUT_CE_VLD          = 0x00000049,
    SDMA_PERFMON_SEL_L47_MEMREQ_RD_IN_GU_VLD           = 0x0000004a,
    SDMA_PERFMON_SEL_L46_MEMREQ_RD_OUT_GU_VLD          = 0x0000004b,
    SDMA_PERFMON_SEL_L45_MEMREQ_RD_IN_INSTR_VLD        = 0x0000004c,
    SDMA_PERFMON_SEL_L44_MEMREQ_RD_OUT_INSTR_VLD       = 0x0000004d,
    SDMA_PERFMON_SEL_L43_MEMREQ_WR_IN_CE_VLD           = 0x0000004e,
    SDMA_PERFMON_SEL_L42_MEMREQ_WR_OUT_CE_VLD          = 0x0000004f,
    SDMA_PERFMON_SEL_L41_MEMREQ_WR_IN_CE_DATA_VLD      = 0x00000050,
    SDMA_PERFMON_SEL_L40_MEMREQ_RD_IN_VLD              = 0x00000051,
    SDMA_PERFMON_SEL_L39_MEMREQ_WR_IN_VLD              = 0x00000052,
    SDMA_PERFMON_SEL_L38_MEMREQ_RD_OUT_RTS             = 0x00000053,
    SDMA_PERFMON_SEL_L37_MEMREQ_WR_OUT_RTS             = 0x00000054,
    SDMA_PERFMON_SEL_L36_L1_INV_IDLE                   = 0x00000055,
    SDMA_PERFMON_SEL_L35_RD_REQRET_IDLE                = 0x00000056,
    SDMA_PERFMON_SEL_L34_WR_REQRET_IDLE                = 0x00000057,
    SDMA_PERFMON_SEL_L33_L2_INTF_IDLE                  = 0x00000058,
    SDMA_PERFMON_SEL_L32_L2_INTF_RD_IDLE               = 0x00000059,
    SDMA_PERFMON_SEL_L31_L2_INTF_WR_IDLE               = 0x0000005a,
    SDMA_PERFMON_SEL_L30_L2_1CYC_INTF_IDLE             = 0x0000005b,
    SDMA_PERFMON_SEL_L29_L2_5CYC_CACHE0_INTF_IDLE      = 0x0000005c,
    SDMA_PERFMON_SEL_L28_L2_5CYC_CACHE1_INTF_IDLE      = 0x0000005d,
    SDMA_PERFMON_SEL_L27_INV_GPUVM_MODE0_1_VLD         = 0x0000005e,
    SDMA_PERFMON_SEL_L26_INV_GPUVM_MODE2_VLD           = 0x0000005f,
    SDMA_PERFMON_SEL_L25_INV_GPUVM_MODE3_VLD           = 0x00000060,
    SDMA_PERFMON_SEL_L24_INV_GPUVM_ALL                 = 0x00000061,
    SDMA_PERFMON_SEL_L23_INV_CPF_FLUSH_VLD             = 0x00000062,
    SDMA_PERFMON_SEL_L22_INV_CPF_NFLUSH_VLD            = 0x00000063,
    SDMA_PERFMON_SEL_L21_INV_CPF_ALL                   = 0x00000064,
    SDMA_PERFMON_SEL_L20_INV_PROC_VLD                  = 0x00000065,
    SDMA_PERFMON_SEL_L19_SDMA_UTCL2_REQ_FREE           = 0x00000066,
    SDMA_PERFMON_SEL_L18_SDMA_UTCL2_5CYC_REQ_FREE      = 0x00000067,
    SDMA_PERFMON_SEL_L17_L2_REQ_RD                     = 0x00000068,
    SDMA_PERFMON_SEL_L16_L2_REQ_WR                     = 0x00000069,
    SDMA_PERFMON_SEL_L15_L2_REQ_CACHE0                 = 0x0000006a,
    SDMA_PERFMON_SEL_L14_L2_REQ_CACHE1                 = 0x0000006b,
    SDMA_PERFMON_SEL_L13_L2_RET_VALID_DATA_RD          = 0x0000006c,
    SDMA_PERFMON_SEL_L12_L2_RET_VALID_DATA_WR          = 0x0000006d,
    SDMA_PERFMON_SEL_L11_L2_RET_VALID_CACHE0           = 0x0000006e,
    SDMA_PERFMON_SEL_L10_L2_RET_VALID_CACHE1           = 0x0000006f,
    SDMA_PERFMON_SEL_L9_L2_RET_VALID_0NACK             = 0x00000070,
    SDMA_PERFMON_SEL_L8_RETURN_PAGE_EXCEPTION          = 0x00000071,
    SDMA_PERFMON_SEL_L7_INSTR_CACHE_IN_VLD             = 0x00000072,
    SDMA_PERFMON_SEL_L6_INSTR_CACHE_MISS_CAM           = 0x00000073,
    SDMA_PERFMON_SEL_L5_GU_CACHE_IN_RD_VLD             = 0x00000074,
    SDMA_PERFMON_SEL_L4_GU_CACHE_IN_WR_VLD             = 0x00000075,
    SDMA_PERFMON_SEL_L3_GU_CACHE_IN_VLD                = 0x00000076,
    SDMA_PERFMON_SEL_L2_GU_CACHE_MISS_CAM              = 0x00000077,
    SDMA_PERFMON_SEL_L1_CE_UTCL1_IN_RD_VLD             = 0x00000078,
    SDMA_PERFMON_SEL_L0_CE_UTCL1_IN_WR_VLD             = 0x00000079,
    SDMA_PERFMON_SEL_TLBI_SEND                         = 0x0000007a,
    SDMA_PERFMON_SEL_TLBI_RTN                          = 0x0000007b,
    SDMA_PERFMON_SEL_GCR_SEND                          = 0x0000007c,
    SDMA_PERFMON_SEL_GCR_RTN                           = 0x0000007d,
} SDMA_PERFMON_SEL;

constexpr unsigned int MaxSdmaPerfmonSel               = SDMA_PERFMON_SEL_GCR_RTN;

typedef enum SH_MEM_ADDRESS_MODE {
    SH_MEM_ADDRESS_MODE_64                             = 0x00000000,
    SH_MEM_ADDRESS_MODE_32                             = 0x00000001,
} SH_MEM_ADDRESS_MODE;

typedef enum SH_MEM_ALIGNMENT_MODE {
    SH_MEM_ALIGNMENT_MODE_DWORD                        = 0x00000000,
    SH_MEM_ALIGNMENT_MODE_DWORD_STRICT                 = 0x00000001,
    SH_MEM_ALIGNMENT_MODE_STRICT                       = 0x00000002,
    SH_MEM_ALIGNMENT_MODE_UNALIGNED                    = 0x00000003,
} SH_MEM_ALIGNMENT_MODE;

typedef enum SPI_FOG_MODE {
    SPI_FOG_NONE                                       = 0x00000000,
    SPI_FOG_EXP                                        = 0x00000001,
    SPI_FOG_EXP2                                       = 0x00000002,
    SPI_FOG_LINEAR                                     = 0x00000003,
} SPI_FOG_MODE;

typedef enum SPI_LB_WAVES_SELECT {
    HS_GS                                              = 0x00000000,
    PS                                                 = 0x00000001,
    CS_NA                                              = 0x00000002,
    SPI_LB_WAVES_RSVD                                  = 0x00000003,
} SPI_LB_WAVES_SELECT;

typedef enum SPI_PERFCNT_SEL {
    SPI_PERF_GS_WINDOW_VALID                           = 0x00000001,
    SPI_PERF_GS_BUSY                                   = 0x00000002,
    SPI_PERF_GS_CRAWLER_STALL                          = 0x00000003,
    SPI_PERF_GS_EVENT_WAVE                             = 0x00000004,
    SPI_PERF_GS_WAVE                                   = 0x00000005,
    SPI_PERF_GS_PERS_UPD_FULL0                         = 0x00000006,
    SPI_PERF_GS_PERS_UPD_FULL1                         = 0x00000007,
    SPI_PERF_GS_FIRST_SUBGRP                           = 0x00000008,
    SPI_PERF_GS_HS_DEALLOC                             = 0x00000009,
    SPI_PERF_GS_NGG_SE_LATE_ALLOC_LIMIT                = 0x0000000a,
    SPI_PERF_GS_POS0_STALL                             = 0x0000000b,
    SPI_PERF_GS_POS1_STALL                             = 0x0000000c,
    SPI_PERF_GS_INDX0_STALL                            = 0x0000000d,
    SPI_PERF_GS_INDX1_STALL                            = 0x0000000e,
    SPI_PERF_GS_PWS_STALL                              = 0x0000000f,
    SPI_PERF_GS_GRP_LIFETIME                           = 0x00000010,
    SPI_PERF_GS_WAVE_IN_FLIGHT                         = 0x00000011,
    SPI_PERF_GS_GRP_LIFETIME_SAMPLE                    = 0x00000012,
    SPI_PERF_HS_WINDOW_VALID                           = 0x00000015,
    SPI_PERF_HS_BUSY                                   = 0x00000016,
    SPI_PERF_HS_CRAWLER_STALL                          = 0x00000017,
    SPI_PERF_HS_FIRST_WAVE                             = 0x00000018,
    SPI_PERF_HS_EVENT_WAVE                             = 0x0000001a,
    SPI_PERF_HS_WAVE                                   = 0x0000001b,
    SPI_PERF_HS_PERS_UPD_FULL0                         = 0x0000001c,
    SPI_PERF_HS_PERS_UPD_FULL1                         = 0x0000001d,
    SPI_PERF_HS_PWS_STALL                              = 0x0000001e,
    SPI_PERF_HS_WAVE_IN_FLIGHT                         = 0x0000001f,
    SPI_PERF_CSGN_WINDOW_VALID                         = 0x00000025,
    SPI_PERF_CSGN_BUSY                                 = 0x00000026,
    SPI_PERF_CSGN_NUM_THREADGROUPS                     = 0x00000027,
    SPI_PERF_CSGN_CRAWLER_STALL                        = 0x00000028,
    SPI_PERF_CSGN_EVENT_WAVE                           = 0x00000029,
    SPI_PERF_CSGN_WAVE                                 = 0x0000002a,
    SPI_PERF_CSGN_PWS_STALL                            = 0x0000002b,
    SPI_PERF_CSGN_WAVE_IN_FLIGHT                       = 0x0000002c,
    SPI_PERF_CSN_WINDOW_VALID                          = 0x0000002d,
    SPI_PERF_CSN_BUSY                                  = 0x0000002e,
    SPI_PERF_CSN_NUM_THREADGROUPS                      = 0x0000002f,
    SPI_PERF_CSN_CRAWLER_STALL                         = 0x00000030,
    SPI_PERF_CSN_EVENT_WAVE                            = 0x00000031,
    SPI_PERF_CSN_WAVE                                  = 0x00000032,
    SPI_PERF_CSN_WAVE_IN_FLIGHT                        = 0x00000033,
    SPI_PERF_PS0_WINDOW_VALID                          = 0x00000035,
    SPI_PERF_PS1_WINDOW_VALID                          = 0x00000036,
    SPI_PERF_PS2_WINDOW_VALID                          = 0x00000037,
    SPI_PERF_PS3_WINDOW_VALID                          = 0x00000038,
    SPI_PERF_PS0_BUSY                                  = 0x00000039,
    SPI_PERF_PS1_BUSY                                  = 0x0000003a,
    SPI_PERF_PS2_BUSY                                  = 0x0000003b,
    SPI_PERF_PS3_BUSY                                  = 0x0000003c,
    SPI_PERF_PS0_ACTIVE                                = 0x0000003d,
    SPI_PERF_PS1_ACTIVE                                = 0x0000003e,
    SPI_PERF_PS2_ACTIVE                                = 0x0000003f,
    SPI_PERF_PS3_ACTIVE                                = 0x00000040,
    SPI_PERF_PS0_DEALLOC                               = 0x00000041,
    SPI_PERF_PS1_DEALLOC                               = 0x00000042,
    SPI_PERF_PS2_DEALLOC                               = 0x00000043,
    SPI_PERF_PS3_DEALLOC                               = 0x00000044,
    SPI_PERF_PS0_EVENT_WAVE                            = 0x00000045,
    SPI_PERF_PS1_EVENT_WAVE                            = 0x00000046,
    SPI_PERF_PS2_EVENT_WAVE                            = 0x00000047,
    SPI_PERF_PS3_EVENT_WAVE                            = 0x00000048,
    SPI_PERF_PS0_WAVE                                  = 0x00000049,
    SPI_PERF_PS1_WAVE                                  = 0x0000004a,
    SPI_PERF_PS2_WAVE                                  = 0x0000004b,
    SPI_PERF_PS3_WAVE                                  = 0x0000004c,
    SPI_PERF_PS0_OPT_WAVE                              = 0x0000004d,
    SPI_PERF_PS1_OPT_WAVE                              = 0x0000004e,
    SPI_PERF_PS2_OPT_WAVE                              = 0x0000004f,
    SPI_PERF_PS3_OPT_WAVE                              = 0x00000050,
    SPI_PERF_PS0_PRIM_BIN0                             = 0x00000051,
    SPI_PERF_PS1_PRIM_BIN0                             = 0x00000052,
    SPI_PERF_PS2_PRIM_BIN0                             = 0x00000053,
    SPI_PERF_PS3_PRIM_BIN0                             = 0x00000054,
    SPI_PERF_PS0_PRIM_BIN1                             = 0x00000055,
    SPI_PERF_PS1_PRIM_BIN1                             = 0x00000056,
    SPI_PERF_PS2_PRIM_BIN1                             = 0x00000057,
    SPI_PERF_PS3_PRIM_BIN1                             = 0x00000058,
    SPI_PERF_PS0_CRAWLER_STALL                         = 0x00000059,
    SPI_PERF_PS1_CRAWLER_STALL                         = 0x0000005a,
    SPI_PERF_PS2_CRAWLER_STALL                         = 0x0000005b,
    SPI_PERF_PS3_CRAWLER_STALL                         = 0x0000005c,
    SPI_PERF_PS_PERS_UPD_FULL0                         = 0x0000005d,
    SPI_PERF_PS_PERS_UPD_FULL1                         = 0x0000005e,
    SPI_PERF_PS0_2_WAVE_GROUPS                         = 0x0000005f,
    SPI_PERF_PS1_2_WAVE_GROUPS                         = 0x00000060,
    SPI_PERF_PS2_2_WAVE_GROUPS                         = 0x00000061,
    SPI_PERF_PS3_2_WAVE_GROUPS                         = 0x00000062,
    SPI_PERF_PS0_WAVE_GROUP_CLOCK_DELAY                = 0x00000063,
    SPI_PERF_PS1_WAVE_GROUP_CLOCK_DELAY                = 0x00000064,
    SPI_PERF_PS2_WAVE_GROUP_CLOCK_DELAY                = 0x00000065,
    SPI_PERF_PS3_WAVE_GROUP_CLOCK_DELAY                = 0x00000066,
    SPI_PERF_PS0_WAVE_GROUP_TIMEOUTS                   = 0x00000067,
    SPI_PERF_PS1_WAVE_GROUP_TIMEOUTS                   = 0x00000068,
    SPI_PERF_PS2_WAVE_GROUP_TIMEOUTS                   = 0x00000069,
    SPI_PERF_PS3_WAVE_GROUP_TIMEOUTS                   = 0x0000006a,
    SPI_PERF_PS_PWS_STALL                              = 0x0000006b,
    SPI_PERF_PS0_LDS_DONE_FULL                         = 0x0000006c,
    SPI_PERF_PS1_LDS_DONE_FULL                         = 0x0000006d,
    SPI_PERF_PS2_LDS_DONE_FULL                         = 0x0000006e,
    SPI_PERF_PS3_LDS_DONE_FULL                         = 0x0000006f,
    SPI_PERF_PS0_DEALLOC_FULL                          = 0x00000070,
    SPI_PERF_PS1_DEALLOC_FULL                          = 0x00000071,
    SPI_PERF_PS2_DEALLOC_FULL                          = 0x00000072,
    SPI_PERF_PS3_DEALLOC_FULL                          = 0x00000073,
    SPI_PERF_PS0_WAVE_IN_FLIGHT                        = 0x00000074,
    SPI_PERF_PS1_WAVE_IN_FLIGHT                        = 0x00000075,
    SPI_PERF_PS2_WAVE_IN_FLIGHT                        = 0x00000076,
    SPI_PERF_PS3_WAVE_IN_FLIGHT                        = 0x00000077,
    SPI_PERF_RA_GS_LDS_OCCUPANCY                       = 0x00000085,
    SPI_PERF_RA_GS_VGPR_OCCUPANCY                      = 0x00000086,
    SPI_PERF_RA_PS_LDS_OCCUPANCY                       = 0x00000087,
    SPI_PERF_RA_PS_VGPR_OCCUPANCY                      = 0x00000088,
    SPI_PERF_RA_SPI_THROTTLE                           = 0x00000089,
    SPI_PERF_RA_PH_THROTTLE                            = 0x0000008a,
    SPI_PERF_RA_PC_PROBE_STALL_PS                      = 0x0000008b,
    SPI_PERF_RA_PC_PSWAVE_STALL_PS                     = 0x0000008c,
    SPI_PERF_RA_PIPE_REQ_BIN2                          = 0x0000008d,
    SPI_PERF_RA_TASK_REQ_BIN3                          = 0x0000008e,
    SPI_PERF_RA_WR_CTL_FULL                            = 0x0000008f,
    SPI_PERF_RA_REQ_NO_ALLOC                           = 0x00000090,
    SPI_PERF_RA_REQ_NO_ALLOC_PS                        = 0x00000091,
    SPI_PERF_RA_REQ_NO_ALLOC_GS                        = 0x00000092,
    SPI_PERF_RA_REQ_NO_ALLOC_HS                        = 0x00000093,
    SPI_PERF_RA_REQ_NO_ALLOC_CSG                       = 0x00000094,
    SPI_PERF_RA_REQ_NO_ALLOC_CSN                       = 0x00000095,
    SPI_PERF_RA_RES_STALL_PS                           = 0x00000096,
    SPI_PERF_RA_RES_STALL_GS                           = 0x00000097,
    SPI_PERF_RA_RES_STALL_HS                           = 0x00000098,
    SPI_PERF_RA_RES_STALL_CSG                          = 0x00000099,
    SPI_PERF_RA_RES_STALL_CSN                          = 0x0000009a,
    SPI_PERF_RA_TMP_STALL_PS                           = 0x0000009b,
    SPI_PERF_RA_TMP_STALL_GS                           = 0x0000009c,
    SPI_PERF_RA_TMP_STALL_HS                           = 0x0000009d,
    SPI_PERF_RA_TMP_STALL_CSG                          = 0x0000009e,
    SPI_PERF_RA_TMP_STALL_CSN                          = 0x0000009f,
    SPI_PERF_RA_WAVE_SIMD_FULL_PS                      = 0x000000a0,
    SPI_PERF_RA_WAVE_SIMD_FULL_GS                      = 0x000000a1,
    SPI_PERF_RA_WAVE_SIMD_FULL_HS                      = 0x000000a2,
    SPI_PERF_RA_WAVE_SIMD_FULL_CSG                     = 0x000000a3,
    SPI_PERF_RA_WAVE_SIMD_FULL_CSN                     = 0x000000a4,
    SPI_PERF_RA_VGPR_SIMD_FULL_PS                      = 0x000000a5,
    SPI_PERF_RA_VGPR_SIMD_FULL_GS                      = 0x000000a6,
    SPI_PERF_RA_VGPR_SIMD_FULL_HS                      = 0x000000a7,
    SPI_PERF_RA_VGPR_SIMD_FULL_CSG                     = 0x000000a8,
    SPI_PERF_RA_VGPR_SIMD_FULL_CSN                     = 0x000000a9,
    SPI_PERF_RA_LDS_CU_FULL_PS                         = 0x000000aa,
    SPI_PERF_RA_LDS_CU_FULL_HS                         = 0x000000ab,
    SPI_PERF_RA_LDS_CU_FULL_GS                         = 0x000000ac,
    SPI_PERF_RA_LDS_CU_FULL_CSG                        = 0x000000ad,
    SPI_PERF_RA_LDS_CU_FULL_CSN                        = 0x000000ae,
    SPI_PERF_RA_BAR_CU_FULL_PS                         = 0x000000af,
    SPI_PERF_RA_BAR_CU_FULL_GS                         = 0x000000b0,
    SPI_PERF_RA_BAR_CU_FULL_HS                         = 0x000000b1,
    SPI_PERF_RA_BAR_CU_FULL_CSG                        = 0x000000b2,
    SPI_PERF_RA_BAR_CU_FULL_CSN                        = 0x000000b3,
    SPI_PERF_RA_BULKY_CU_FULL_CSG                      = 0x000000b4,
    SPI_PERF_RA_BULKY_CU_FULL_CSN                      = 0x000000b5,
    SPI_PERF_RA_TGLIM_CU_FULL_CSG                      = 0x000000b6,
    SPI_PERF_RA_TGLIM_CU_FULL_CSN                      = 0x000000b7,
    SPI_PERF_RA_WVLIM_STALL_PS                         = 0x000000b8,
    SPI_PERF_RA_WVLIM_STALL_GS                         = 0x000000b9,
    SPI_PERF_RA_WVLIM_STALL_HS                         = 0x000000ba,
    SPI_PERF_RA_WVLIM_STALL_CSG                        = 0x000000bb,
    SPI_PERF_RA_WVLIM_STALL_CSN                        = 0x000000bc,
    SPI_PERF_RA_RSV_UPD                                = 0x000000c1,
    SPI_PERF_RA_PRE_ALLOC_STALL                        = 0x000000c2,
    SPI_PERF_RA_GFX_UNDER_TUNNEL                       = 0x000000c3,
    SPI_PERF_RA_CSC_UNDER_TUNNEL                       = 0x000000c4,
    SPI_PERF_RA_WVALLOC_STALL                          = 0x000000c5,
    SPI_PERF_RA_ACCUM0_SIMD_FULL_PS                    = 0x000000c6,
    SPI_PERF_RA_ACCUM1_SIMD_FULL_PS                    = 0x000000c7,
    SPI_PERF_RA_ACCUM2_SIMD_FULL_PS                    = 0x000000c8,
    SPI_PERF_RA_ACCUM3_SIMD_FULL_PS                    = 0x000000c9,
    SPI_PERF_RA_ACCUM0_SIMD_FULL_GS                    = 0x000000ca,
    SPI_PERF_RA_ACCUM1_SIMD_FULL_GS                    = 0x000000cb,
    SPI_PERF_RA_ACCUM2_SIMD_FULL_GS                    = 0x000000cc,
    SPI_PERF_RA_ACCUM3_SIMD_FULL_GS                    = 0x000000cd,
    SPI_PERF_RA_ACCUM0_SIMD_FULL_HS                    = 0x000000ce,
    SPI_PERF_RA_ACCUM1_SIMD_FULL_HS                    = 0x000000cf,
    SPI_PERF_RA_ACCUM2_SIMD_FULL_HS                    = 0x000000d0,
    SPI_PERF_RA_ACCUM3_SIMD_FULL_HS                    = 0x000000d1,
    SPI_PERF_RA_ACCUM0_SIMD_FULL_CSG                   = 0x000000d2,
    SPI_PERF_RA_ACCUM1_SIMD_FULL_CSG                   = 0x000000d3,
    SPI_PERF_RA_ACCUM2_SIMD_FULL_CSG                   = 0x000000d4,
    SPI_PERF_RA_ACCUM3_SIMD_FULL_CSG                   = 0x000000d5,
    SPI_PERF_RA_ACCUM0_SIMD_FULL_CSN                   = 0x000000d6,
    SPI_PERF_RA_ACCUM1_SIMD_FULL_CSN                   = 0x000000d7,
    SPI_PERF_RA_ACCUM2_SIMD_FULL_CSN                   = 0x000000d8,
    SPI_PERF_RA_ACCUM3_SIMD_FULL_CSN                   = 0x000000d9,
    SPI_PERF_EXP_ARB_COL_CNT                           = 0x000000da,
    SPI_PERF_EXP_ARB_POS_CNT                           = 0x000000db,
    SPI_PERF_EXP_ARB_GDS_CNT                           = 0x000000dc,
    SPI_PERF_EXP_ARB_IDX_CNT                           = 0x000000dd,
    SPI_PERF_EXP_WITH_CONFLICT                         = 0x000000de,
    SPI_PERF_EXP_WITH_CONFLICT_CLEAR                   = 0x000000df,
    SPI_PERF_GS_EXP_DONE                               = 0x000000e0,
    SPI_PERF_PS_EXP_DONE                               = 0x000000e1,
    SPI_PERF_PS_EXP_ARB_CONFLICT                       = 0x000000e2,
    SPI_PERF_GS_SCBD_IDX_CLEANUP                       = 0x000000e3,
    SPI_PERF_GS_SCBD_POS_CLEANUP                       = 0x000000e4,
    SPI_PERF_PS_EXP_ALLOC                              = 0x000000e5,
    SPI_PERF_PS0_WAVEID_STARVED                        = 0x000000e6,
    SPI_PERF_PS1_WAVEID_STARVED                        = 0x000000e7,
    SPI_PERF_PS2_WAVEID_STARVED                        = 0x000000e8,
    SPI_PERF_PS3_WAVEID_STARVED                        = 0x000000e9,
    SPI_PERF_PS0_EXP_ALLOC_WITH_CONFLICT               = 0x000000ea,
    SPI_PERF_PS1_EXP_ALLOC_WITH_CONFLICT               = 0x000000eb,
    SPI_PERF_PS2_EXP_ALLOC_WITH_CONFLICT               = 0x000000ec,
    SPI_PERF_PS3_EXP_ALLOC_WITH_CONFLICT               = 0x000000ed,
    SPI_PERF_NUM_PS_COL_SA0SQ0_EXPORTS                 = 0x000000ee,
    SPI_PERF_NUM_PS_COL_SA0SQ1_EXPORTS                 = 0x000000ef,
    SPI_PERF_NUM_PS_COL_SA1SQ0_EXPORTS                 = 0x000000f0,
    SPI_PERF_NUM_PS_COL_SA1SQ1_EXPORTS                 = 0x000000f1,
    SPI_PERF_NUM_POS_SA0SQ0_EXPORTS                    = 0x000000f2,
    SPI_PERF_NUM_POS_SA0SQ1_EXPORTS                    = 0x000000f3,
    SPI_PERF_NUM_POS_SA1SQ0_EXPORTS                    = 0x000000f4,
    SPI_PERF_NUM_POS_SA1SQ1_EXPORTS                    = 0x000000f5,
    SPI_PERF_NUM_GDS_SA0SQ0_EXPORTS                    = 0x000000f6,
    SPI_PERF_NUM_GDS_SA0SQ1_EXPORTS                    = 0x000000f7,
    SPI_PERF_NUM_GDS_SA1SQ0_EXPORTS                    = 0x000000f8,
    SPI_PERF_NUM_GDS_SA1SQ1_EXPORTS                    = 0x000000f9,
    SPI_PERF_NUM_EXPGRANT_EXPORTS                      = 0x000000fa,
    SPI_PERF_GS_ALLOC_IDX                              = 0x000000fb,
    SPI_PERF_GS_ALLOC_POS                              = 0x000000fc,
    SPI_PERF_PIX_ALLOC_PEND_CNT                        = 0x000000fd,
    SPI_PERF_EXPORT_SCB0_STALL                         = 0x000000fe,
    SPI_PERF_EXPORT_SCB1_STALL                         = 0x000000ff,
    SPI_PERF_EXPORT_SCB2_STALL                         = 0x00000100,
    SPI_PERF_EXPORT_SCB3_STALL                         = 0x00000101,
    SPI_PERF_EXPORT_DB0_STALL                          = 0x00000102,
    SPI_PERF_EXPORT_DB1_STALL                          = 0x00000103,
    SPI_PERF_EXPORT_DB2_STALL                          = 0x00000104,
    SPI_PERF_EXPORT_DB3_STALL                          = 0x00000105,
    SPI_PERF_EXPORT_DB4_STALL                          = 0x00000106,
    SPI_PERF_EXPORT_DB5_STALL                          = 0x00000107,
    SPI_PERF_EXPORT_DB6_STALL                          = 0x00000108,
    SPI_PERF_EXPORT_DB7_STALL                          = 0x00000109,
    SPI_PERF_GS_NGG_SE_SEND_GS_ALLOC                   = 0x0000010a,
    SPI_PERF_GS_NGG_STALL_MSG_VAL                      = 0x0000010b,
    SPI_PERF_SWC_PS_WR                                 = 0x0000010c,
    SPI_PERF_SWC_GS_WR                                 = 0x0000010d,
    SPI_PERF_SWC_HS_WR                                 = 0x0000010e,
    SPI_PERF_SWC_CSGN_WR                               = 0x0000010f,
    SPI_PERF_SWC_CSN_WR                                = 0x00000110,
    SPI_PERF_VWC_PS_WR                                 = 0x00000111,
    SPI_PERF_VWC_ES_WR                                 = 0x00000112,
    SPI_PERF_VWC_GS_WR                                 = 0x00000113,
    SPI_PERF_VWC_LS_WR                                 = 0x00000114,
    SPI_PERF_VWC_HS_WR                                 = 0x00000115,
    SPI_PERF_VWC_CSGN_WR                               = 0x00000116,
    SPI_PERF_VWC_CSN_WR                                = 0x00000117,
    SPI_PERF_EXP_THROT_UPSTEP                          = 0x00000118,
    SPI_PERF_EXP_THROT_DOWNSTEP                        = 0x00000119,
    SPI_PERF_EXP_THROT_CAUSALITY_DETECTED              = 0x0000011a,
    SPI_PERF_BUSY                                      = 0x0000011b,
    SPI_PERF_ALL_PS_WAVE                               = 0x0000011c,
    SPI_PERF_ALL_PS_WAVE_IN_FLIGHT                     = 0x0000011d,
    SPI_PERF_ALL_WAVE                                  = 0x0000011e,
    SPI_PERF_ALL_WAVE_IN_FLIGHT                        = 0x0000011f,
    SPI_PERF_RA_REQ_ALLOC                              = 0x00000120,
    SPI_PERF_VGPR_INIT                                 = 0x00000121,
    SPI_PERF_SGPR_INIT                                 = 0x00000122,
    SPI_PERF_VGPR_ALLOC_LEVEL                          = 0x00000123,
    SPI_PERF_LDS_ALLOC_LEVEL                           = 0x00000124,
    SPI_PERF_GFX_TEMP_ALLOC_LEVEL                      = 0x00000125,
    SPI_PERF_CSG_TEMP_ALLOC_LEVEL                      = 0x00000126,
    SPI_PERF_CSN_TEMP_ALLOC_LEVEL                      = 0x00000127,
    SPI_PERF_ALL_WAVE_RESTORED                         = 0x00000128,
    SPI_PERF_ALL_WAVE_SAVED                            = 0x00000129,
    SPI_PERF_ALL_WAVE_W32                              = 0x0000012a,
    SPI_PERF_ALL_WAVE_W64                              = 0x0000012b,
    SPI_PERF_ALL_WAVE_ITEMS                            = 0x0000012c,
    SPI_PERF_ALL_WAVE_ITEMS_W32                        = 0x0000012d,
    SPI_PERF_ALL_WAVE_ITEMS_W64                        = 0x0000012e,
    SPI_PERF_RA_REQ_ALLOC_WGP_TAKEOVER_STALL           = 0x0000012f,
    SPI_PERF_RA_REQ_ALLOC_WGP_TAKEOVER_LEVEL           = 0x00000130,
    SPI_PERF_RA_REQ_ALLOC_DYN_VGPR_STALL               = 0x00000131,
    SPI_PERF_RA_REQ_ALLOC_DYN_VGPR_CU_LEVEL            = 0x00000132,
    SPI_PERF_GLG_LOCKED_CUS_PS                         = 0x00000133,
    SPI_PERF_GLG_LOCKED_CUS_GS                         = 0x00000134,
    SPI_PERF_GLG_LOCKED_CUS_HS                         = 0x00000135,
    SPI_PERF_GLG_LOCKED_CUS_CS0                        = 0x00000136,
    SPI_PERF_GLG_LOCKED_CUS_CS1                        = 0x00000137,
    SPI_PERF_GLG_NUM_LOCKING_PERIODS_GFX               = 0x00000138,
    SPI_PERF_GLG_NUM_LOCKING_PERIODS_CSC               = 0x00000139,
    SPI_PERF_GLG_NUM_SUCCESS_ALLOCS_GFX                = 0x0000013a,
    SPI_PERF_GLG_NUM_SUCCESS_ALLOCS_CSC                = 0x0000013b,
    SPI_PERF_GLG_NUM_TRIGGER                           = 0x0000013c,
    SPI_PERF_GLG_NUM_GS_ACTIVATE                       = 0x0000013d,
    SPI_PERF_GLG_NUM_MRT_ACTIVATE                      = 0x0000013e,
} SPI_PERFCNT_SEL;

constexpr unsigned int MaxSpiPerfcntSel                = SPI_PERF_GLG_NUM_MRT_ACTIVATE;

typedef enum SPI_PNT_SPRITE_OVERRIDE {
    SPI_PNT_SPRITE_SEL_0                               = 0x00000000,
    SPI_PNT_SPRITE_SEL_1                               = 0x00000001,
    SPI_PNT_SPRITE_SEL_S                               = 0x00000002,
    SPI_PNT_SPRITE_SEL_T                               = 0x00000003,
    SPI_PNT_SPRITE_SEL_NONE                            = 0x00000004,
} SPI_PNT_SPRITE_OVERRIDE;

typedef enum SPI_PS_LDS_GROUP_SIZE {
    SPI_PS_LDS_GROUP_1                                 = 0x00000000,
    SPI_PS_LDS_GROUP_2                                 = 0x00000001,
    SPI_PS_LDS_GROUP_4                                 = 0x00000002,
} SPI_PS_LDS_GROUP_SIZE;

typedef enum SPI_SAMPLE_CNTL {
    CENTROIDS_ONLY                                     = 0x00000000,
    CENTERS_ONLY                                       = 0x00000001,
    CENTROIDS_AND_CENTERS                              = 0x00000002,
    UNDEF                                              = 0x00000003,
} SPI_SAMPLE_CNTL;

typedef enum SPI_SHADER_EX_FORMAT {
    SPI_SHADER_ZERO                                    = 0x00000000,
    SPI_SHADER_32_R                                    = 0x00000001,
    SPI_SHADER_32_GR                                   = 0x00000002,
    SPI_SHADER_32_AR                                   = 0x00000003,
    SPI_SHADER_FP16_ABGR                               = 0x00000004,
    SPI_SHADER_UNORM16_ABGR                            = 0x00000005,
    SPI_SHADER_SNORM16_ABGR                            = 0x00000006,
    SPI_SHADER_UINT16_ABGR                             = 0x00000007,
    SPI_SHADER_SINT16_ABGR                             = 0x00000008,
    SPI_SHADER_32_ABGR                                 = 0x00000009,
} SPI_SHADER_EX_FORMAT;

typedef enum SPI_SHADER_FORMAT {
    SPI_SHADER_NONE                                    = 0x00000000,
    SPI_SHADER_1COMP                                   = 0x00000001,
    SPI_SHADER_2COMP                                   = 0x00000002,
    SPI_SHADER_4COMPRESS                               = 0x00000003,
    SPI_SHADER_4COMP                                   = 0x00000004,
} SPI_SHADER_FORMAT;

typedef enum SPM_PERFMON_STATE {
    STRM_PERFMON_STATE_DISABLE_AND_RESET               = 0x00000000,
    STRM_PERFMON_STATE_START_COUNTING                  = 0x00000001,
    STRM_PERFMON_STATE_STOP_COUNTING                   = 0x00000002,
    STRM_PERFMON_STATE_RESERVED_3                      = 0x00000003,
    STRM_PERFMON_STATE_DISABLE_AND_RESET_PHANTOM       = 0x00000004,
    STRM_PERFMON_STATE_COUNT_AND_DUMP_PHANTOM          = 0x00000005,
} SPM_PERFMON_STATE;

typedef enum SQG_PERF_SEL {
    SQG_PERF_SEL_NONE                                  = 0x00000000,
    SQG_PERF_SEL_MSG_BUS_BUSY                          = 0x00000001,
    SQG_PERF_SEL_EXP_REQ0_BUS_BUSY                     = 0x00000002,
    SQG_PERF_SEL_EXP_REQ1_BUS_BUSY                     = 0x00000003,
    SQG_PERF_SEL_EXP_BUS0_BUSY                         = 0x00000004,
    SQG_PERF_SEL_EXP_BUS1_BUSY                         = 0x00000005,
    SQG_PERF_SEL_TTRACE_WRITE_DATA                     = 0x00000006,
    SQG_PERF_SEL_TTRACE_STALL                          = 0x00000007,
    SQG_PERF_SEL_TTRACE_LOST_PACKETS                   = 0x00000008,
    SQG_PERF_SEL_WAVES_INITIAL_PREFETCH                = 0x00000009,
    SQG_PERF_SEL_EVENTS                                = 0x0000000a,
    SQG_PERF_SEL_WAVES_RESTORED                        = 0x0000000b,
    SQG_PERF_SEL_WAVES_SAVED                           = 0x0000000c,
    SQG_PERF_SEL_ACCUM_PREV                            = 0x0000000d,
    SQG_PERF_SEL_CYCLES                                = 0x0000000e,
    SQG_PERF_SEL_BUSY_CYCLES                           = 0x0000000f,
    SQG_PERF_SEL_WAVE_CYCLES                           = 0x00000010,
    SQG_PERF_SEL_MSG                                   = 0x00000011,
    SQG_PERF_SEL_MSG_INTERRUPT                         = 0x00000012,
    SQG_PERF_SEL_WAVES                                 = 0x00000013,
    SQG_PERF_SEL_WAVES_32                              = 0x00000014,
    SQG_PERF_SEL_WAVES_64                              = 0x00000015,
    SQG_PERF_SEL_ITEMS                                 = 0x00000016,
    SQG_PERF_SEL_WAVE32_ITEMS                          = 0x00000017,
    SQG_PERF_SEL_WAVE64_ITEMS                          = 0x00000018,
    SQG_PERF_SEL_PS_QUADS                              = 0x00000019,
    SQG_PERF_SEL_WAVES_EQ_64                           = 0x0000001a,
    SQG_PERF_SEL_WAVES_EQ_32                           = 0x0000001b,
    SQG_PERF_SEL_WAVES_LT_64                           = 0x0000001c,
    SQG_PERF_SEL_WAVES_LT_48                           = 0x0000001d,
    SQG_PERF_SEL_WAVES_LT_32                           = 0x0000001e,
    SQG_PERF_SEL_WAVES_LT_16                           = 0x0000001f,
    SQG_PERF_SEL_REFCLKS                               = 0x00000020,
    SQG_PERF_SEL_WAVES_WGP_TAKEOVER                    = 0x00000021,
    SQG_PERF_SEL_WAVES_DYN_VGPR                        = 0x00000022,
    SQG_PERF_SEL_ITEMS_PS                              = 0x00000023,
    SQG_PERF_SEL_ITEMS_GS                              = 0x00000024,
    SQG_PERF_SEL_ITEMS_HS                              = 0x00000025,
    SQG_PERF_SEL_ITEMS_CS                              = 0x00000026,
    SQG_PERF_SEL_WAVES_VEC32                           = 0x00000027,
    SQG_PERF_SEL_WAVES_PS_VEC32                        = 0x00000028,
    SQG_PERF_SEL_WAVES_GS_VEC32                        = 0x00000029,
    SQG_PERF_SEL_WAVES_HS_VEC32                        = 0x0000002a,
    SQG_PERF_SEL_WAVES_CS_VEC32                        = 0x0000002b,
    SQG_PERF_SEL_LEVEL_WGP_ACTIVE                      = 0x0000002c,
    SQG_PERF_SEL_DUMMY_LAST                            = 0x0000002d,
} SQG_PERF_SEL;

constexpr unsigned int MaxSqgPerfSel                   = SQG_PERF_SEL_DUMMY_LAST;

typedef enum SQ_IMG_FILTER_TYPE {
    SQ_IMG_FILTER_MODE_BLEND                           = 0x00000000,
    SQ_IMG_FILTER_MODE_MIN                             = 0x00000001,
    SQ_IMG_FILTER_MODE_MAX                             = 0x00000002,
} SQ_IMG_FILTER_TYPE;

typedef enum SQ_LLC_CTL {
    SQ_LLC_0                                           = 0x00000000,
    SQ_LLC_1                                           = 0x00000001,
    SQ_LLC_RSVD_2                                      = 0x00000002,
    SQ_LLC_BYPASS                                      = 0x00000003,
} SQ_LLC_CTL;

typedef enum SQ_OOB_SELECT {
    SQ_OOB_INDEX_AND_OFFSET                            = 0x00000000,
    SQ_OOB_INDEX_ONLY                                  = 0x00000001,
    SQ_OOB_NUM_RECORDS_0                               = 0x00000002,
    SQ_OOB_COMPLETE                                    = 0x00000003,
} SQ_OOB_SELECT;

typedef enum SQ_PERF_SEL {
    SQ_PERF_SEL_NONE                                   = 0x00000000,
    SQ_PERF_SEL_ACCUM_PREV                             = 0x00000001,
    SQ_PERF_SEL_CYCLES                                 = 0x00000002,
    SQ_PERF_SEL_BUSY_CYCLES                            = 0x00000003,
    SQ_PERF_SEL_WAVES                                  = 0x00000004,
    SQ_PERF_SEL_WAVES_32                               = 0x00000005,
    SQ_PERF_SEL_WAVES_64                               = 0x00000006,
    SQ_PERF_SEL_LEVEL_WAVES                            = 0x00000007,
    SQ_PERF_SEL_ITEMS                                  = 0x00000008,
    SQ_PERF_SEL_WAVE32_ITEMS                           = 0x00000009,
    SQ_PERF_SEL_WAVE64_ITEMS                           = 0x0000000a,
    SQ_PERF_SEL_PS_QUADS                               = 0x0000000b,
    SQ_PERF_SEL_EVENTS                                 = 0x0000000c,
    SQ_PERF_SEL_WAVES_EQ_32                            = 0x0000000d,
    SQ_PERF_SEL_WAVES_EQ_64                            = 0x0000000e,
    SQ_PERF_SEL_WAVES_LT_64                            = 0x0000000f,
    SQ_PERF_SEL_WAVES_LT_48                            = 0x00000010,
    SQ_PERF_SEL_WAVES_LT_32                            = 0x00000011,
    SQ_PERF_SEL_WAVES_LT_16                            = 0x00000012,
    SQ_PERF_SEL_WAVES_SAVED                            = 0x00000014,
    SQ_PERF_SEL_MSG                                    = 0x00000015,
    SQ_PERF_SEL_MSG_INTERRUPT                          = 0x00000016,
    SQ_PERF_SEL_WAVES_INITIAL_PREFETCH                 = 0x00000017,
    SQ_PERF_SEL_WAVE_CYCLES                            = 0x00000018,
    SQ_PERF_SEL_WAVE_READY                             = 0x00000019,
    SQ_PERF_SEL_WAIT_INST_ANY                          = 0x0000001a,
    SQ_PERF_SEL_WAIT_ANY                               = 0x0000001b,
    SQ_PERF_SEL_WAIT_CNT_ANY                           = 0x0000001c,
    SQ_PERF_SEL_WAIT_CNT_LOAD                          = 0x0000001d,
    SQ_PERF_SEL_WAIT_CNT_STORE                         = 0x0000001e,
    SQ_PERF_SEL_WAIT_TTRACE                            = 0x0000001f,
    SQ_PERF_SEL_WAIT_IFETCH                            = 0x00000020,
    SQ_PERF_SEL_WAIT_BARRIER                           = 0x00000021,
    SQ_PERF_SEL_WAIT_EXP_ALLOC                         = 0x00000022,
    SQ_PERF_SEL_WAIT_SLEEP                             = 0x00000023,
    SQ_PERF_SEL_WAIT_DELAY_ALU                         = 0x00000024,
    SQ_PERF_SEL_WAIT_DEPCTR                            = 0x00000025,
    SQ_PERF_SEL_WAIT_OTHER                             = 0x00000026,
    SQ_PERF_SEL_INSTS_ALL                              = 0x00000027,
    SQ_PERF_SEL_INSTS_BRANCH                           = 0x00000028,
    SQ_PERF_SEL_INSTS_CBRANCH_NOT_TAKEN                = 0x00000029,
    SQ_PERF_SEL_INSTS_CBRANCH_TAKEN                    = 0x0000002a,
    SQ_PERF_SEL_INSTS_EXP                              = 0x0000002b,
    SQ_PERF_SEL_INSTS_FLAT                             = 0x0000002c,
    SQ_PERF_SEL_INSTS_LDS                              = 0x0000002d,
    SQ_PERF_SEL_INSTS_SALU                             = 0x0000002e,
    SQ_PERF_SEL_INSTS_SMEM                             = 0x0000002f,
    SQ_PERF_SEL_INSTS_SMEM_NORM                        = 0x00000030,
    SQ_PERF_SEL_INSTS_SENDMSG                          = 0x00000031,
    SQ_PERF_SEL_INSTS_VALU                             = 0x00000032,
    SQ_PERF_SEL_INSTS_VALU_TRANS32                     = 0x00000033,
    SQ_PERF_SEL_INSTS_VALU_NO_COEXEC                   = 0x00000034,
    SQ_PERF_SEL_INSTS_TEX                              = 0x00000035,
    SQ_PERF_SEL_INSTS_TEX_LOAD                         = 0x00000036,
    SQ_PERF_SEL_INSTS_TEX_STORE                        = 0x00000037,
    SQ_PERF_SEL_INSTS_DELAY_ALU                        = 0x00000038,
    SQ_PERF_SEL_INSTS_INTERNAL                         = 0x00000039,
    SQ_PERF_SEL_INSTS_VEC32                            = 0x0000003a,
    SQ_PERF_SEL_INSTS_VEC32_FLAT                       = 0x0000003b,
    SQ_PERF_SEL_INSTS_VEC32_LDS                        = 0x0000003c,
    SQ_PERF_SEL_INSTS_VEC32_VALU                       = 0x0000003d,
    SQ_PERF_SEL_WAVE32_INSTS_EXP                       = 0x0000003e,
    SQ_PERF_SEL_INSTS_VEC32_VALU_TRANS32               = 0x0000003f,
    SQ_PERF_SEL_INSTS_VEC32_VALU_NO_COEXEC             = 0x00000040,
    SQ_PERF_SEL_INSTS_VEC32_TEX                        = 0x00000041,
    SQ_PERF_SEL_INSTS_VEC32_TEX_LOAD                   = 0x00000042,
    SQ_PERF_SEL_INSTS_VEC32_TEX_STORE                  = 0x00000043,
    SQ_PERF_SEL_ITEM_CYCLES_VALU                       = 0x00000044,
    SQ_PERF_SEL_VALU_READWRITELANE_CYCLES              = 0x00000045,
    SQ_PERF_SEL_WAVE32_INSTS                           = 0x00000046,
    SQ_PERF_SEL_WAVE64_INSTS                           = 0x00000047,
    SQ_PERF_SEL_INSTS_VALU_EXEC_SKIPPED                = 0x00000048,
    SQ_PERF_SEL_WAVE64_HALF_SKIP                       = 0x00000049,
    SQ_PERF_SEL_INST_LEVEL_EXP                         = 0x0000004a,
    SQ_PERF_SEL_INST_LEVEL_LDS                         = 0x0000004b,
    SQ_PERF_SEL_INST_LEVEL_SMEM                        = 0x0000004c,
    SQ_PERF_SEL_INST_LEVEL_TEX_LOAD                    = 0x0000004d,
    SQ_PERF_SEL_INST_LEVEL_TEX_STORE                   = 0x0000004e,
    SQ_PERF_SEL_IFETCH_REQS                            = 0x0000004f,
    SQ_PERF_SEL_IFETCH_LEVEL                           = 0x00000050,
    SQ_PERF_SEL_LDS_DIRECT_CMD_FIFO_FULL_STALL         = 0x00000051,
    SQ_PERF_SEL_VALU_SGATHER_STALL                     = 0x00000052,
    SQ_PERF_SEL_VALU_FWD_BUFFER_FULL_STALL             = 0x00000053,
    SQ_PERF_SEL_VALU_SGPR_RD_FIFO_FULL_STALL           = 0x00000054,
    SQ_PERF_SEL_VALU_SGATHER_FULL_STALL                = 0x00000055,
    SQ_PERF_SEL_SALU_SGATHER_STALL                     = 0x00000056,
    SQ_PERF_SEL_SALU_SGPR_RD_FIFO_FULL_STALL           = 0x00000057,
    SQ_PERF_SEL_SALU_GATHER_FULL_STALL                 = 0x00000058,
    SQ_PERF_SEL_INST_ISSUE_SMEM_STALL                  = 0x00000059,
    SQ_PERF_SEL_INST_ISSUE_ALL_STALL                   = 0x0000005a,
    SQ_PERF_SEL_INST_ISSUE_VALU_STALL                  = 0x0000005b,
    SQ_PERF_SEL_INST_ISSUE_SALU_STALL                  = 0x0000005c,
    SQ_PERF_SEL_INST_ISSUE_TEX_STALL                   = 0x0000005d,
    SQ_PERF_SEL_INST_ISSUE_LDS_STALL                   = 0x0000005e,
    SQ_PERF_SEL_INST_ISSUE_EXP_STALL                   = 0x00000060,
    SQ_PERF_SEL_INST_WAITCNT_STALL                     = 0x00000061,
    SQ_PERF_SEL_INST_BARRIER_STALL                     = 0x00000062,
    SQ_PERF_SEL_INST_CYCLES_VALU                       = 0x00000063,
    SQ_PERF_SEL_INST_CYCLES_VALU_TRANS32               = 0x00000064,
    SQ_PERF_SEL_INST_CYCLES_VALU_NO_COEXEC             = 0x00000065,
    SQ_PERF_SEL_INST_CYCLES_VMEM                       = 0x00000066,
    SQ_PERF_SEL_INST_CYCLES_VMEM_LOAD                  = 0x00000067,
    SQ_PERF_SEL_INST_CYCLES_VMEM_STORE                 = 0x00000068,
    SQ_PERF_SEL_INST_CYCLES_LDS                        = 0x00000069,
    SQ_PERF_SEL_INST_CYCLES_TEX                        = 0x0000006a,
    SQ_PERF_SEL_INST_CYCLES_FLAT                       = 0x0000006b,
    SQ_PERF_SEL_INST_CYCLES_EXP                        = 0x0000006c,
    SQ_PERF_SEL_VALU_STARVE                            = 0x0000006d,
    SQ_PERF_SEL_VMEM_ARB_FIFO_FULL                     = 0x0000006e,
    SQ_PERF_SEL_MSG_FIFO_FULL_STALL                    = 0x0000006f,
    SQ_PERF_SEL_EXP_REQ_FIFO_FULL                      = 0x00000070,
    SQ_PERF_SEL_VMEM_BUS_ACTIVE                        = 0x00000071,
    SQ_PERF_SEL_VMEM_BUS_STALL                         = 0x00000072,
    SQ_PERF_SEL_VMEM_BUS_STALL_TA_ADDR_FIFO_FULL       = 0x00000073,
    SQ_PERF_SEL_VMEM_BUS_STALL_TA_CMD_FIFO_FULL        = 0x00000074,
    SQ_PERF_SEL_VMEM_BUS_STALL_LDS_ADDR_FIFO_FULL      = 0x00000075,
    SQ_PERF_SEL_VMEM_BUS_STALL_LDS_CMD_FIFO_FULL       = 0x00000076,
    SQ_PERF_SEL_VMEM_STARVE_TA_ADDR_EMPTY              = 0x00000077,
    SQ_PERF_SEL_VMEM_STARVE_LDS_ADDR_EMPTY             = 0x00000078,
    SQ_PERF_SEL_SALU_PIPE_STALL                        = 0x00000079,
    SQ_PERF_SEL_SMEM_DCACHE_RETURN_CYCLES              = 0x0000007a,
    SQ_PERF_SEL_MSG_BUS_BUSY                           = 0x0000007b,
    SQ_PERF_SEL_EXP_REQ_BUS_STALL                      = 0x0000007c,
    SQ_PERF_SEL_EXP_REQ0_BUS_BUSY                      = 0x0000007d,
    SQ_PERF_SEL_EXP_REQ1_BUS_BUSY                      = 0x0000007e,
    SQ_PERF_SEL_EXP_BUS0_BUSY                          = 0x0000007f,
    SQ_PERF_SEL_EXP_BUS1_BUSY                          = 0x00000080,
    SQ_PERF_SEL_INST_CACHE_REQ_STALL                   = 0x00000081,
    SQ_PERF_SEL_USER0                                  = 0x00000082,
    SQ_PERF_SEL_USER1                                  = 0x00000083,
    SQ_PERF_SEL_USER2                                  = 0x00000084,
    SQ_PERF_SEL_USER3                                  = 0x00000085,
    SQ_PERF_SEL_USER4                                  = 0x00000086,
    SQ_PERF_SEL_USER5                                  = 0x00000087,
    SQ_PERF_SEL_USER6                                  = 0x00000088,
    SQ_PERF_SEL_USER7                                  = 0x00000089,
    SQ_PERF_SEL_USER8                                  = 0x0000008a,
    SQ_PERF_SEL_USER9                                  = 0x0000008b,
    SQ_PERF_SEL_USER10                                 = 0x0000008c,
    SQ_PERF_SEL_USER11                                 = 0x0000008d,
    SQ_PERF_SEL_USER12                                 = 0x0000008e,
    SQ_PERF_SEL_USER13                                 = 0x0000008f,
    SQ_PERF_SEL_USER14                                 = 0x00000090,
    SQ_PERF_SEL_USER15                                 = 0x00000091,
    SQ_PERF_SEL_USER_LEVEL0                            = 0x00000092,
    SQ_PERF_SEL_USER_LEVEL1                            = 0x00000093,
    SQ_PERF_SEL_USER_LEVEL2                            = 0x00000094,
    SQ_PERF_SEL_USER_LEVEL3                            = 0x00000095,
    SQ_PERF_SEL_USER_LEVEL4                            = 0x00000096,
    SQ_PERF_SEL_USER_LEVEL5                            = 0x00000097,
    SQ_PERF_SEL_USER_LEVEL6                            = 0x00000098,
    SQ_PERF_SEL_USER_LEVEL7                            = 0x00000099,
    SQ_PERF_SEL_USER_LEVEL8                            = 0x0000009a,
    SQ_PERF_SEL_USER_LEVEL9                            = 0x0000009b,
    SQ_PERF_SEL_USER_LEVEL10                           = 0x0000009c,
    SQ_PERF_SEL_USER_LEVEL11                           = 0x0000009d,
    SQ_PERF_SEL_USER_LEVEL12                           = 0x0000009e,
    SQ_PERF_SEL_USER_LEVEL13                           = 0x0000009f,
    SQ_PERF_SEL_USER_LEVEL14                           = 0x000000a0,
    SQ_PERF_SEL_USER_LEVEL15                           = 0x000000a1,
    SQ_PERF_SEL_VALU_RETURN_SDST                       = 0x000000a2,
    SQ_PERF_SEL_VMEM_VGPR_READ_STALLED_BY_EXPORT       = 0x000000a3,
    SQ_PERF_SEL_INSTS_VALU_TRANS                       = 0x000000a4,
    SQ_PERF_SEL_INSTS_LDS_DIRECT_LOAD                  = 0x000000a5,
    SQ_PERF_SEL_INSTS_LDS_PARAM_LOAD                   = 0x000000a6,
    SQ_PERF_SEL_INSTS_VEC32_LDS_PARAM_DIRECT           = 0x000000a7,
    SQ_PERF_SEL_INSTS_VALU_ONE_CYCLE_WAVE64            = 0x000000a8,
    SQ_PERF_SEL_INSTS_VALU_VINTERP                     = 0x000000a9,
    SQ_PERF_SEL_INSTS_VEC32_VALU_VINTERP               = 0x000000aa,
    SQ_PERF_SEL_OVERFLOW_PREV                          = 0x000000ab,
    SQ_PERF_SEL_INSTS_VALU_1_PASS                      = 0x000000ad,
    SQ_PERF_SEL_INSTS_VALU_2_PASS                      = 0x000000ae,
    SQ_PERF_SEL_INSTS_VALU_4_PASS                      = 0x000000af,
    SQ_PERF_SEL_INSTS_VALU_DP                          = 0x000000b0,
    SQ_PERF_SEL_SP_CONST_CYCLES                        = 0x000000b1,
    SQ_PERF_SEL_SP_CONST_STALL_CYCLES                  = 0x000000b2,
    SQ_PERF_SEL_ITEMS_VALU                             = 0x000000b3,
    SQ_PERF_SEL_ITEMS_MAX_VALU                         = 0x000000b4,
    SQ_PERF_SEL_ITEM_CYCLES_VMEM                       = 0x000000b5,
    SQ_PERF_SEL_INSTS_DELAY_ALU_COISSUE                = 0x000000b6,
    SQ_PERF_SEL_INSTS_FLAT_LOAD                        = 0x000000b7,
    SQ_PERF_SEL_INSTS_FLAT_STORE                       = 0x000000b8,
    SQ_PERF_SEL_INSTS_VALU_ONE_CYCLE_WAVE64_16BIT      = 0x000000b9,
    SQ_PERF_SEL_INSTS_VALU_ONE_CYCLE_WAVE64_32BIT      = 0x000000ba,
    SQ_PERF_SEL_INSTS_NON_VALU_EXEC_SKIPPED            = 0x000000bb,
    SQ_PERF_SEL_INSTS_BARRIER_LOCK                     = 0x000000bc,
    SQ_PERF_SEL_INSTS_WAKEUP                           = 0x000000bd,
    SQ_PERF_SEL_IS_CACHE_REQ                           = 0x000000be,
    SQ_PERF_SEL_INSTS_SALU_PS                          = 0x000000bf,
    SQ_PERF_SEL_INSTS_SALU_GS                          = 0x000000c0,
    SQ_PERF_SEL_INSTS_SALU_HS                          = 0x000000c1,
    SQ_PERF_SEL_INSTS_SALU_CS                          = 0x000000c2,
    SQ_PERF_SEL_INSTS_SMEM_PS                          = 0x000000c3,
    SQ_PERF_SEL_INSTS_SMEM_GS                          = 0x000000c4,
    SQ_PERF_SEL_INSTS_SMEM_HS                          = 0x000000c5,
    SQ_PERF_SEL_INSTS_SMEM_CS                          = 0x000000c6,
    SQ_PERF_SEL_INSTS_VEC32_TEX_PS                     = 0x000000c7,
    SQ_PERF_SEL_INSTS_VEC32_TEX_GS                     = 0x000000c8,
    SQ_PERF_SEL_INSTS_VEC32_TEX_HS                     = 0x000000c9,
    SQ_PERF_SEL_INSTS_VEC32_TEX_CS                     = 0x000000ca,
    SQ_PERF_SEL_INSTS_VEC32_VALU_PS                    = 0x000000cb,
    SQ_PERF_SEL_INSTS_VEC32_VALU_GS                    = 0x000000cc,
    SQ_PERF_SEL_INSTS_VEC32_VALU_HS                    = 0x000000cd,
    SQ_PERF_SEL_INSTS_VEC32_VALU_CS                    = 0x000000ce,
    SQ_PERF_SEL_WAIT_CNT_SAMPLE                        = 0x000000cf,
    SQ_PERF_SEL_WAIT_CNT_BVH                           = 0x000000d0,
    SQ_PERF_SEL_WAIT_CNT_KM                            = 0x000000d1,
    SQ_PERF_SEL_WAIT_CNT_DS                            = 0x000000d2,
    SQ_PERF_SEL_WAIT_CNT_EXP                           = 0x000000d3,
    SQ_PERF_SEL_INSTS_SALU_FLOAT                       = 0x000000d4,
    SQ_PERF_SEL_INSTS_VGPR_ALLOC                       = 0x000000d5,
    SQ_PERF_SEL_INSTS_VGPR_ALLOC_FAIL                  = 0x000000d6,
    SQ_PERF_SEL_INSTS_LOCK                             = 0x000000d7,
    SQ_PERF_SEL_INSTS_VALU_COISSUE                     = 0x000000d8,
    SQ_PERF_SEL_INSTS_VEC32_LEVEL_LDS_LOAD             = 0x000000d9,
    SQ_PERF_SEL_INSTS_VEC32_LEVEL_LDS_STORE            = 0x000000da,
    SQ_PERF_SEL_IS_CACHE_MISS                          = 0x000000db,
    SQ_PERF_SEL_IS_CACHE_DUP_MISS                      = 0x000000dc,
    SQ_PERF_SEL_INST_CYCLES_VMEM_ATOMIC                = 0x000000dd,
    SQ_PERF_SEL_INSTS_TEX_BLOCK_LOAD                   = 0x000000de,
    SQ_PERF_SEL_INSTS_TEX_BVH                          = 0x000000df,
    SQ_PERF_SEL_INSTS_TEX_SAMPLE                       = 0x000000e0,
    SQ_PERF_SEL_INSTS_TEX_ATOMIC_RTN                   = 0x000000e1,
    SQ_PERF_SEL_INSTS_TEX_BLOCK_STORE                  = 0x000000e2,
    SQ_PERF_SEL_INSTS_TEX_ATOMIC_NORTN                 = 0x000000e3,
    SQ_PERF_SEL_INSTS_GLOBAL_SCRATCH                   = 0x000000e4,
    SQ_PERF_SEL_INSTS_WMMA_LOAD                        = 0x000000e5,
    SQ_PERF_SEL_INSTS_FLAT_ATOMIC                      = 0x000000e6,
    SQ_PERF_SEL_INSTS_EXP_MRT                          = 0x000000e7,
    SQ_PERF_SEL_INSTS_EXP_Z                            = 0x000000e8,
    SQ_PERF_SEL_INSTS_VEC32_VALU_WMMA                  = 0x000000e9,
    SQ_PERF_SEL_INSTS_VEC32_LDS_LOAD                   = 0x000000ea,
    SQ_PERF_SEL_INSTS_VEC32_LDS_ATOMIC_RTN             = 0x000000eb,
    SQ_PERF_SEL_INSTS_VEC32_LDS_STORE                  = 0x000000ec,
    SQ_PERF_SEL_INSTS_VEC32_LDS_ATOMIC_NORTN           = 0x000000ed,
    SQ_PERF_SEL_INSTS_VEC32_LDS_BVH                    = 0x000000ee,
    SQ_PERF_SEL_INSTS_VEC32_LDS_OTHER                  = 0x000000ef,
    SQ_PERF_SEL_INSTS_VEC32_TEX_BVH                    = 0x000000f0,
    SQ_PERF_SEL_INSTS_VEC32_TEX_SAMPLE                 = 0x000000f1,
    SQ_PERF_SEL_INSTS_VEC32_TEX_ATOMIC                 = 0x000000f2,
    SQ_PERF_SEL_INSTS_VEC32_FLAT_LOAD                  = 0x000000f3,
    SQ_PERF_SEL_INSTS_VEC32_FLAT_STORE                 = 0x000000f4,
    SQ_PERF_SEL_INSTS_VEC32_FLAT_ATOMIC                = 0x000000f5,
    SQ_PERF_SEL_INSTS_VEC32_GLOBAL_SCRATCH             = 0x000000f6,
    SQ_PERF_SEL_INSTS_VEC32_GLOBAL_SCRATCH_LOAD        = 0x000000f7,
    SQ_PERF_SEL_INSTS_VEC32_GLOBAL_SCRATCH_STORE       = 0x000000f8,
    SQ_PERF_SEL_INSTS_VEC32_GLOBAL_SCRATCH_ATOMIC      = 0x000000f9,
    SQ_PERF_SEL_INSTS_VEC32_LEVEL_LDS                  = 0x000000fa,
    SQ_PERF_SEL_ACTIVE_WGP_ISSUE                       = 0x000000fb,
    SQ_PERF_SEL_INSTS_VEC32_LEVEL_LDS_PARAM_DIRECT     = 0x000000fc,
    SQ_PERF_SEL_INST_CYCLES_VALU_CORE                  = 0x000000fd,
    SQ_PERF_SEL_INST_CYCLES_VALU_SIDE                  = 0x000000fe,
    SQ_PERF_SEL_INST_CYCLES_VALU_DP                    = 0x000000ff,
    SQ_PERF_SEL_INST_CYCLES_VALU_WMMA                  = 0x00000100,
    SQ_PERF_SEL_WAVES_WGP_TAKEOVER                     = 0x00000101,
    SQ_PERF_SEL_WAVES_DYN_VGPR                         = 0x00000102,
    SQ_PERF_SEL_ITEMS_PS                               = 0x00000103,
    SQ_PERF_SEL_ITEMS_GS                               = 0x00000104,
    SQ_PERF_SEL_ITEMS_HS                               = 0x00000105,
    SQ_PERF_SEL_ITEMS_CS                               = 0x00000106,
    SQ_PERF_SEL_WAVES_VEC32                            = 0x00000107,
    SQ_PERF_SEL_WAVES_PS_VEC32                         = 0x00000108,
    SQ_PERF_SEL_WAVES_GS_VEC32                         = 0x00000109,
    SQ_PERF_SEL_WAVES_HS_VEC32                         = 0x0000010a,
    SQ_PERF_SEL_WAVES_CS_VEC32                         = 0x0000010b,
    SQ_PERF_SEL_DUMMY_END                              = 0x0000010c,
    SQ_PERF_SEL_DUMMY_LAST                             = 0x0000011f,
    SQC_PERF_SEL_LDS_BANK_CONFLICT                     = 0x00000120,
    SQC_PERF_SEL_LDS_ADDR_CONFLICT                     = 0x00000121,
    SQC_PERF_SEL_LDS_UNALIGNED_STALL                   = 0x00000122,
    SQC_PERF_SEL_LDS_MEM_VIOLATIONS                    = 0x00000123,
    SQC_PERF_SEL_LDS_ATOMIC_RETURN                     = 0x00000124,
    SQC_PERF_SEL_LDS_IDX_ACTIVE                        = 0x00000125,
    SQC_PERF_SEL_LDS_ADDR_STALL                        = 0x00000126,
    SQC_PERF_SEL_LDS_ADDR_ACTIVE                       = 0x00000127,
    SQC_PERF_SEL_LDS_PC_LDS_WRITE_STALL_TD             = 0x00000128,
    SQC_PERF_SEL_LDS_SPI_VGPR_WRITE_STALL_TD           = 0x00000129,
    SQC_PERF_SEL_LDS_LDS_VGPR_WRITE_STALL              = 0x0000012a,
    SQC_PERF_SEL_LDS_FP_ADD_CYCLES                     = 0x0000012b,
    SQC_PERF_SEL_ICACHE_BUSY_CYCLES                    = 0x0000012c,
    SQC_PERF_SEL_ICACHE_REQ                            = 0x0000012d,
    SQC_PERF_SEL_ICACHE_HITS                           = 0x0000012e,
    SQC_PERF_SEL_ICACHE_MISSES                         = 0x0000012f,
    SQC_PERF_SEL_ICACHE_MISSES_DUPLICATE               = 0x00000130,
    SQC_PERF_SEL_ICACHE_INVAL_INST                     = 0x00000131,
    SQC_PERF_SEL_ICACHE_INVAL_ASYNC                    = 0x00000132,
    SQC_PERF_SEL_ICACHE_INFLIGHT_LEVEL                 = 0x00000133,
    SQC_PERF_SEL_DCACHE_INFLIGHT_LEVEL                 = 0x00000134,
    SQC_PERF_SEL_TC_INFLIGHT_LEVEL                     = 0x00000135,
    SQC_PERF_SEL_ICACHE_TC_INFLIGHT_LEVEL              = 0x00000136,
    SQC_PERF_SEL_DCACHE_TC_INFLIGHT_LEVEL              = 0x00000137,
    SQC_PERF_SEL_ICACHE_INPUT_VALID_READYB             = 0x00000138,
    SQC_PERF_SEL_DCACHE_INPUT_VALID_READYB             = 0x00000139,
    SQC_PERF_SEL_TC_REQ                                = 0x0000013a,
    SQC_PERF_SEL_TC_INST_REQ                           = 0x0000013b,
    SQC_PERF_SEL_TC_DATA_READ_REQ                      = 0x0000013c,
    SQC_PERF_SEL_TC_STALL                              = 0x0000013d,
    SQC_PERF_SEL_TC_STARVE                             = 0x0000013e,
    SQC_PERF_SEL_ICACHE_INPUT_STALL_ARB_NO_GRANT       = 0x0000013f,
    SQC_PERF_SEL_ICACHE_INPUT_STALL_BANK_READYB        = 0x00000140,
    SQC_PERF_SEL_ICACHE_CACHE_STALLED                  = 0x00000141,
    SQC_PERF_SEL_ICACHE_CACHE_STALL_INFLIGHT_MAX       = 0x00000142,
    SQC_PERF_SEL_ICACHE_STALL_OUTXBAR_ARB_NO_GRANT     = 0x00000143,
    SQC_PERF_SEL_DCACHE_BUSY_CYCLES                    = 0x00000144,
    SQC_PERF_SEL_DCACHE_REQ                            = 0x00000145,
    SQC_PERF_SEL_DCACHE_HITS                           = 0x00000146,
    SQC_PERF_SEL_DCACHE_MISSES                         = 0x00000147,
    SQC_PERF_SEL_DCACHE_MISSES_DUPLICATE               = 0x00000148,
    SQC_PERF_SEL_DCACHE_INVAL_INST                     = 0x00000149,
    SQC_PERF_SEL_DCACHE_INVAL_ASYNC                    = 0x0000014a,
    SQC_PERF_SEL_DCACHE_HIT_LRU_READ                   = 0x0000014b,
    SQC_PERF_SEL_DCACHE_INPUT_STALL_ARB_NO_GRANT       = 0x0000014c,
    SQC_PERF_SEL_DCACHE_INPUT_STALL_BANK_READYB        = 0x0000014d,
    SQC_PERF_SEL_DCACHE_CACHE_STALLED                  = 0x0000014e,
    SQC_PERF_SEL_DCACHE_CACHE_STALL_INFLIGHT_MAX       = 0x0000014f,
    SQC_PERF_SEL_DCACHE_CACHE_STALL_OUTPUT             = 0x00000150,
    SQC_PERF_SEL_DCACHE_STALL_OUTXBAR_ARB_NO_GRANT     = 0x00000151,
    SQC_PERF_SEL_DCACHE_REQ_READ_1                     = 0x00000152,
    SQC_PERF_SEL_DCACHE_REQ_READ_2                     = 0x00000153,
    SQC_PERF_SEL_DCACHE_REQ_READ_4                     = 0x00000154,
    SQC_PERF_SEL_DCACHE_REQ_READ_8                     = 0x00000155,
    SQC_PERF_SEL_DCACHE_REQ_READ_16                    = 0x00000156,
    SQC_PERF_SEL_DCACHE_REQ_ATC_PROBE                  = 0x00000157,
    SQC_PERF_SEL_SQ_DCACHE_REQS                        = 0x00000158,
    SQC_PERF_SEL_DCACHE_FLAT_REQ                       = 0x00000159,
    SQC_PERF_SEL_TD_VGPR_BUSY                          = 0x0000015a,
    SQC_PERF_SEL_LDS_VGPR_BUSY                         = 0x0000015b,
    SQC_PERF_SEL_LDS_TD_VGPR_CONF_STALL                = 0x0000015c,
    SQC_PERF_SEL_ICACHE_GCR                            = 0x0000015d,
    SQC_PERF_SEL_ICACHE_GCR_HITS                       = 0x0000015e,
    SQC_PERF_SEL_DCACHE_GCR                            = 0x0000015f,
    SQC_PERF_SEL_DCACHE_GCR_HITS                       = 0x00000160,
    SQC_PERF_SEL_ICACHE_GCR_INVALIDATE                 = 0x00000161,
    SQC_PERF_SEL_DCACHE_GCR_INVALIDATE                 = 0x00000162,
    SQC_PERF_SEL_DCACHE_SPI_RETURN_STALL               = 0x00000163,
    SQC_PERF_SEL_ICACHE_PREFETCH_REQ_CACHELINES        = 0x00000164,
    SQC_PERF_SEL_DCACHE_PREFETCH_REQ_CACHELINES        = 0x00000165,
    SQC_PERF_SEL_ICACHE_PREFETCH_MISSES                = 0x00000166,
    SQC_PERF_SEL_DCACHE_PREFETCH_MISSES                = 0x00000167,
    SQC_PERF_SEL_LDS_BANK_CONFLICT_LOAD_CNT            = 0x00000168,
    SQC_PERF_SEL_LDS_BANK_CONFLICT_STORE_CNT           = 0x00000169,
    SQC_PERF_SEL_LDS_BANK_CONFLICT_ATOMIC_CNT          = 0x0000016a,
    SQC_PERF_SEL_LDS_ACTIVE_LOAD_CNT                   = 0x0000016b,
    SQC_PERF_SEL_LDS_ACTIVE_STORE_CNT                  = 0x0000016c,
    SQC_PERF_SEL_LDS_ACTIVE_ATOMIC_CNT                 = 0x0000016d,
    SQC_PERF_SEL_LDS_STORE_DWORDS                      = 0x0000016e,
    SQC_PERF_SEL_LDS_LOAD_DWORDS                       = 0x0000016f,
    SQC_PERF_SEL_LDS_ATOMIC_DWORDS                     = 0x00000170,
    SQC_PERF_SEL_LDS_EXECUTION_STALL                   = 0x00000171,
    SQC_PERF_SEL_SMEM_LOAD_DWORDS                      = 0x00000172,
    SQC_PERF_SEL_DUMMY_LAST                            = 0x00000173,
    SP_PERF_SEL_DST_BUF_ALLOC_STALL                    = 0x000001c0,
    SP_PERF_SEL_DST_BUF_WB_CONF_W_TD_LDS               = 0x000001c1,
    SP_PERF_SEL_DST_BUF_WB_CONF_W_SPI                  = 0x000001c2,
    SP_PERF_SEL_DST_BUF_EVEN_DIRTY                     = 0x000001c3,
    SP_PERF_SEL_DST_BUF_ODD_DIRTY                      = 0x000001c4,
    SP_PERF_SEL_SRC_CACHE_HIT_B0                       = 0x000001c5,
    SP_PERF_SEL_SRC_CACHE_HIT_B1                       = 0x000001c6,
    SP_PERF_SEL_SRC_CACHE_HIT_B2                       = 0x000001c7,
    SP_PERF_SEL_SRC_CACHE_HIT_B3                       = 0x000001c8,
    SP_PERF_SEL_SRC_CACHE_PROBE_B0                     = 0x000001c9,
    SP_PERF_SEL_SRC_CACHE_PROBE_B1                     = 0x000001ca,
    SP_PERF_SEL_SRC_CACHE_PROBE_B2                     = 0x000001cb,
    SP_PERF_SEL_SRC_CACHE_PROBE_B3                     = 0x000001cc,
    SP_PERF_SEL_SRC_CACHE_VGPR_RD_B0                   = 0x000001cd,
    SP_PERF_SEL_SRC_CACHE_VGPR_RD_B1                   = 0x000001ce,
    SP_PERF_SEL_SRC_CACHE_VGPR_RD_B2                   = 0x000001cf,
    SP_PERF_SEL_SRC_CACHE_VGPR_RD_B3                   = 0x000001d0,
    SP_PERF_SEL_SRC_CACHE_RECYCLE_HIT_B0               = 0x000001d1,
    SP_PERF_SEL_SRC_CACHE_RECYCLE_HIT_B1               = 0x000001d2,
    SP_PERF_SEL_SRC_CACHE_RECYCLE_HIT_B2               = 0x000001d3,
    SP_PERF_SEL_SRC_CACHE_RECYCLE_HIT_B3               = 0x000001d4,
    SP_PERF_SEL_SRC_CACHE_PROBE_SUCCESS_B0             = 0x000001d5,
    SP_PERF_SEL_SRC_CACHE_PROBE_SUCCESS_B1             = 0x000001d6,
    SP_PERF_SEL_SRC_CACHE_PROBE_SUCCESS_B2             = 0x000001d7,
    SP_PERF_SEL_SRC_CACHE_PROBE_SUCCESS_B3             = 0x000001d8,
    SP_PERF_SEL_VALU_PENDING_QUEUE_STALL               = 0x000001d9,
    SP_PERF_SEL_VALU_OPERAND                           = 0x000001da,
    SP_PERF_SEL_VALU_VGPR_OPERAND                      = 0x000001db,
    SP_PERF_SEL_VALU_OPERAND_FROM_DST_BUF              = 0x000001dc,
    SP_PERF_SEL_VALU_EXEC_MASK_CHANGE                  = 0x000001dd,
    SP_PERF_SEL_VALU_COEXEC_WITH_TRANS                 = 0x000001de,
    SP_PERF_SEL_VALU_SGPR_FWD_BUF_FULL                 = 0x000001df,
    SP_PERF_SEL_VALU_STALL                             = 0x000001e0,
    SP_PERF_SEL_VALU_STALL_VGPR_NOT_READY              = 0x000001e1,
    SP_PERF_SEL_VALU_STALL_SGPR_NOT_READY              = 0x000001e2,
    SP_PERF_SEL_VALU_STALL_VDST_FWD                    = 0x000001e3,
    SP_PERF_SEL_VALU_STALL_SDST_FWD                    = 0x000001e4,
    SP_PERF_SEL_VALU_STALL_DST_STALL                   = 0x000001e5,
    SP_PERF_SEL_VALU_FAST_OP_STALL_VGPR_NOT_READY      = 0x000001e6,
    SP_PERF_SEL_VGPR_VMEM_RD                           = 0x000001e7,
    SP_PERF_SEL_VGPR_EXP_RD                            = 0x000001e8,
    SP_PERF_SEL_VGPR_SPI_WR                            = 0x000001e9,
    SP_PERF_SEL_VGPR_TDLDS_DATA_WR                     = 0x000001ea,
    SP_PERF_SEL_VGPR_WR                                = 0x000001eb,
    SP_PERF_SEL_VGPR_RD                                = 0x000001ec,
    SP_PERF_SEL_VGPR_WR_KILL                           = 0x000001ed,
    SP_PERF_SEL_VALU_VGPR_RD_CONFLICT_EXP              = 0x000001ee,
    SP_PERF_SEL_VALU_VGPR_RD_CONFLICT_LDS              = 0x000001ef,
    SP_PERF_SEL_VALU_VGPR_RD_CONFLICT_TEX              = 0x000001f0,
    SP_PERF_SEL_DUMMY_LAST                             = 0x000001f1,
    SQ_PERF_SEL_NONE2                                  = 0x000001ff,
} SQ_PERF_SEL;

constexpr unsigned int MaxSqPerfSel                    = SQ_PERF_SEL_NONE2;

typedef enum SQ_RSRC_BUF_TYPE {
    SQ_RSRC_BUF                                        = 0x00000000,
    SQ_RSRC_BUF_RSVD_1                                 = 0x00000001,
    SQ_RSRC_BUF_RSVD_2                                 = 0x00000002,
    SQ_RSRC_BUF_RSVD_3                                 = 0x00000003,
} SQ_RSRC_BUF_TYPE;

typedef enum SQ_RSRC_FLAT_TYPE {
    SQ_RSRC_FLAT_RSVD_0                                = 0x00000000,
    SQ_RSRC_FLAT                                       = 0x00000001,
    SQ_RSRC_FLAT_RSVD_2                                = 0x00000002,
    SQ_RSRC_FLAT_RSVD_3                                = 0x00000003,
} SQ_RSRC_FLAT_TYPE;

typedef enum SQ_RSRC_IMG_TYPE {
    SQ_RSRC_IMG_RSVD_0                                 = 0x00000000,
    SQ_RSRC_IMG_RSVD_1                                 = 0x00000001,
    SQ_RSRC_IMG_RSVD_2                                 = 0x00000002,
    SQ_RSRC_IMG_RSVD_3                                 = 0x00000003,
    SQ_RSRC_IMG_RSVD_4                                 = 0x00000004,
    SQ_RSRC_IMG_RSVD_5                                 = 0x00000005,
    SQ_RSRC_IMG_RSVD_6                                 = 0x00000006,
    SQ_RSRC_IMG_RSVD_7                                 = 0x00000007,
    SQ_RSRC_IMG_1D                                     = 0x00000008,
    SQ_RSRC_IMG_2D                                     = 0x00000009,
    SQ_RSRC_IMG_3D                                     = 0x0000000a,
    SQ_RSRC_IMG_CUBE                                   = 0x0000000b,
    SQ_RSRC_IMG_1D_ARRAY                               = 0x0000000c,
    SQ_RSRC_IMG_2D_ARRAY                               = 0x0000000d,
    SQ_RSRC_IMG_2D_MSAA                                = 0x0000000e,
    SQ_RSRC_IMG_2D_MSAA_ARRAY                          = 0x0000000f,
} SQ_RSRC_IMG_TYPE;

typedef enum SQ_SEL_XYZW01 {
    SQ_SEL_0                                           = 0x00000000,
    SQ_SEL_1                                           = 0x00000001,
    SQ_SEL_N_BC_1                                      = 0x00000002,
    SQ_SEL_RESERVED_1                                  = 0x00000003,
    SQ_SEL_X                                           = 0x00000004,
    SQ_SEL_Y                                           = 0x00000005,
    SQ_SEL_Z                                           = 0x00000006,
    SQ_SEL_W                                           = 0x00000007,
} SQ_SEL_XYZW01;

typedef enum SQ_TEX_ANISO_RATIO {
    SQ_TEX_ANISO_RATIO_1                               = 0x00000000,
    SQ_TEX_ANISO_RATIO_2                               = 0x00000001,
    SQ_TEX_ANISO_RATIO_4                               = 0x00000002,
    SQ_TEX_ANISO_RATIO_8                               = 0x00000003,
    SQ_TEX_ANISO_RATIO_16                              = 0x00000004,
} SQ_TEX_ANISO_RATIO;

typedef enum SQ_TEX_BORDER_COLOR {
    SQ_TEX_BORDER_COLOR_TRANS_BLACK                    = 0x00000000,
    SQ_TEX_BORDER_COLOR_OPAQUE_BLACK                   = 0x00000001,
    SQ_TEX_BORDER_COLOR_OPAQUE_WHITE                   = 0x00000002,
    SQ_TEX_BORDER_COLOR_REGISTER                       = 0x00000003,
} SQ_TEX_BORDER_COLOR;

typedef enum SQ_TEX_CLAMP {
    SQ_TEX_WRAP                                        = 0x00000000,
    SQ_TEX_MIRROR                                      = 0x00000001,
    SQ_TEX_CLAMP_LAST_TEXEL                            = 0x00000002,
    SQ_TEX_MIRROR_ONCE_LAST_TEXEL                      = 0x00000003,
    SQ_TEX_CLAMP_HALF_BORDER                           = 0x00000004,
    SQ_TEX_MIRROR_ONCE_HALF_BORDER                     = 0x00000005,
    SQ_TEX_CLAMP_BORDER                                = 0x00000006,
    SQ_TEX_MIRROR_ONCE_BORDER                          = 0x00000007,
} SQ_TEX_CLAMP;

typedef enum SQ_TEX_DEPTH_COMPARE {
    SQ_TEX_DEPTH_COMPARE_NEVER                         = 0x00000000,
    SQ_TEX_DEPTH_COMPARE_LESS                          = 0x00000001,
    SQ_TEX_DEPTH_COMPARE_EQUAL                         = 0x00000002,
    SQ_TEX_DEPTH_COMPARE_LESSEQUAL                     = 0x00000003,
    SQ_TEX_DEPTH_COMPARE_GREATER                       = 0x00000004,
    SQ_TEX_DEPTH_COMPARE_NOTEQUAL                      = 0x00000005,
    SQ_TEX_DEPTH_COMPARE_GREATEREQUAL                  = 0x00000006,
    SQ_TEX_DEPTH_COMPARE_ALWAYS                        = 0x00000007,
} SQ_TEX_DEPTH_COMPARE;

typedef enum SQ_TEX_MIP_FILTER {
    SQ_TEX_MIP_FILTER_NONE                             = 0x00000000,
    SQ_TEX_MIP_FILTER_POINT                            = 0x00000001,
    SQ_TEX_MIP_FILTER_LINEAR                           = 0x00000002,
    SQ_TEX_MIP_FILTER_POINT_ANISO_ADJ                  = 0x00000003,
} SQ_TEX_MIP_FILTER;

typedef enum SQ_TEX_XY_FILTER {
    SQ_TEX_XY_FILTER_POINT                             = 0x00000000,
    SQ_TEX_XY_FILTER_BILINEAR                          = 0x00000001,
    SQ_TEX_XY_FILTER_ANISO_POINT                       = 0x00000002,
    SQ_TEX_XY_FILTER_ANISO_BILINEAR                    = 0x00000003,
} SQ_TEX_XY_FILTER;

typedef enum SQ_TEX_Z_FILTER {
    SQ_TEX_Z_FILTER_NONE                               = 0x00000000,
    SQ_TEX_Z_FILTER_POINT                              = 0x00000001,
    SQ_TEX_Z_FILTER_LINEAR                             = 0x00000002,
} SQ_TEX_Z_FILTER;

typedef enum SQ_TT_MODE {
    SQ_TT_MODE_OFF                                     = 0x00000000,
    SQ_TT_MODE_ON                                      = 0x00000001,
    SQ_TT_MODE_GLOBAL                                  = 0x00000002,
    SQ_TT_MODE_DETAIL                                  = 0x00000003,
} SQ_TT_MODE;

typedef enum SQ_TT_RT_FREQ {
    SQ_TT_RT_FREQ_NEVER                                = 0x00000000,
    SQ_TT_RT_FREQ_1024_CLK                             = 0x00000001,
    SQ_TT_RT_FREQ_4096_CLK                             = 0x00000002,
} SQ_TT_RT_FREQ;

typedef enum SQ_TT_TOKEN_MASK_INST_EXCLUDE {
    SQ_TT_INST_EXCLUDE_VMEM_OTHER_SIMD_BIT             = 0x00000001,
    SQ_TT_INST_EXCLUDE_EXPGNT234_BIT                   = 0x00000002,
} SQ_TT_TOKEN_MASK_INST_EXCLUDE;

typedef enum SQ_TT_TOKEN_MASK_INST_EXCLUDE_SHIFT {
    SQ_TT_INST_EXCLUDE_VMEM_OTHER_SIMD_SHIFT           = 0x00000000,
    SQ_TT_INST_EXCLUDE_EXPGNT234_SHIFT                 = 0x00000001,
} SQ_TT_TOKEN_MASK_INST_EXCLUDE_SHIFT;

typedef enum SQ_TT_TOKEN_MASK_REG_EXCLUDE {
    SQ_TT_REG_EXCLUDE_USER_DATA_BIT                    = 0x00000001,
    SQ_TT_REG_EXCLUDE_CP_ME_MC_RADDR_BIT               = 0x00000002,
    SQ_TT_REG_EXCLUDE_GRBM_COMPUTE_EXCLUDE_BIT         = 0x00000004,
} SQ_TT_TOKEN_MASK_REG_EXCLUDE;

typedef enum SQ_TT_TOKEN_MASK_REG_EXCLUDE_SHIFT {
    SQ_TT_REG_EXCLUDE_USER_DATA_SHIFT                  = 0x00000000,
    SQ_TT_REG_EXCLUDE_CP_ME_MC_RADDR_SHIFT             = 0x00000001,
    SQ_TT_REG_EXCLUDE_GRBM_COMPUTE_EXCLUDE_SHIFT       = 0x00000002,
} SQ_TT_TOKEN_MASK_REG_EXCLUDE_SHIFT;

typedef enum SQ_TT_TOKEN_MASK_REG_INCLUDE {
    SQ_TT_TOKEN_MASK_SQDEC_BIT                         = 0x00000001,
    SQ_TT_TOKEN_MASK_SHDEC_BIT                         = 0x00000002,
    SQ_TT_TOKEN_MASK_GFXUDEC_BIT                       = 0x00000004,
    SQ_TT_TOKEN_MASK_COMP_BIT                          = 0x00000008,
    SQ_TT_TOKEN_MASK_CONTEXT_BIT                       = 0x00000010,
    SQ_TT_TOKEN_MASK_CONFIG_BIT                        = 0x00000020,
    SQ_TT_TOKEN_MASK_ALL_BIT                           = 0x00000040,
    SQ_TT_TOKEN_MASK_RSVD_BIT                          = 0x00000080,
} SQ_TT_TOKEN_MASK_REG_INCLUDE;

typedef enum SQ_TT_TOKEN_MASK_REG_INCLUDE_SHIFT {
    SQ_TT_TOKEN_MASK_SQDEC_SHIFT                       = 0x00000000,
    SQ_TT_TOKEN_MASK_SHDEC_SHIFT                       = 0x00000001,
    SQ_TT_TOKEN_MASK_GFXUDEC_SHIFT                     = 0x00000002,
    SQ_TT_TOKEN_MASK_COMP_SHIFT                        = 0x00000003,
    SQ_TT_TOKEN_MASK_CONTEXT_SHIFT                     = 0x00000004,
    SQ_TT_TOKEN_MASK_CONFIG_SHIFT                      = 0x00000005,
    SQ_TT_TOKEN_MASK_ALL_SHIFT                         = 0x00000006,
    SQ_TT_TOKEN_MASK_RSVD_SHIFT                        = 0x00000007,
} SQ_TT_TOKEN_MASK_REG_INCLUDE_SHIFT;

typedef enum SQ_TT_TOKEN_MASK_TOKEN_EXCLUDE_SHIFT {
    SQ_TT_TOKEN_EXCLUDE_VMEMEXEC_SHIFT                 = 0x00000000,
    SQ_TT_TOKEN_EXCLUDE_ALUEXEC_SHIFT                  = 0x00000001,
    SQ_TT_TOKEN_EXCLUDE_VALUINST_SHIFT                 = 0x00000002,
    SQ_TT_TOKEN_EXCLUDE_WAVERDY_SHIFT                  = 0x00000003,
    SQ_TT_TOKEN_EXCLUDE_WAVESTARTEND_SHIFT             = 0x00000004,
    SQ_TT_TOKEN_EXCLUDE_IMMEDIATE_SHIFT                = 0x00000005,
    SQ_TT_TOKEN_EXCLUDE_REG_SHIFT                      = 0x00000006,
    SQ_TT_TOKEN_EXCLUDE_EVENT_SHIFT                    = 0x00000007,
    SQ_TT_TOKEN_EXCLUDE_INST_SHIFT                     = 0x00000008,
    SQ_TT_TOKEN_EXCLUDE_UTILCTR_SHIFT                  = 0x00000009,
    SQ_TT_TOKEN_EXCLUDE_WAVEALLOC_SHIFT                = 0x0000000a,
    SQ_TT_TOKEN_EXCLUDE_REALTIME                       = 0x0000000b,
} SQ_TT_TOKEN_MASK_TOKEN_EXCLUDE_SHIFT;

typedef enum SQ_TT_UTIL_TIMER {
    SQ_TT_UTIL_TIMER_100_CLK                           = 0x00000000,
    SQ_TT_UTIL_TIMER_250_CLK                           = 0x00000001,
} SQ_TT_UTIL_TIMER;

typedef enum SQ_TT_WAVESTART_MODE {
    SQ_TT_WAVESTART_MODE_SHORT                         = 0x00000000,
    SQ_TT_WAVESTART_MODE_ALLOC                         = 0x00000001,
    SQ_TT_WAVESTART_MODE_PBB_ID                        = 0x00000002,
    SQ_TT_WAVESTART_MODE_WG_ID                         = 0x00000003,
} SQ_TT_WAVESTART_MODE;

typedef enum SQ_TT_WTYPE_INCLUDE {
    SQ_TT_WTYPE_INCLUDE_PS_BIT                         = 0x00000001,
    SQ_TT_WTYPE_INCLUDE_RSVD0_BIT                      = 0x00000002,
    SQ_TT_WTYPE_INCLUDE_GS_BIT                         = 0x00000004,
    SQ_TT_WTYPE_INCLUDE_RSVD1_BIT                      = 0x00000008,
    SQ_TT_WTYPE_INCLUDE_HS_BIT                         = 0x00000010,
    SQ_TT_WTYPE_INCLUDE_RSVD2_BIT                      = 0x00000020,
    SQ_TT_WTYPE_INCLUDE_CS_BIT                         = 0x00000040,
} SQ_TT_WTYPE_INCLUDE;

typedef enum SQ_TT_WTYPE_INCLUDE_SHIFT {
    SQ_TT_WTYPE_INCLUDE_PS_SHIFT                       = 0x00000000,
    SQ_TT_WTYPE_INCLUDE_RSVD0_SHIFT                    = 0x00000001,
    SQ_TT_WTYPE_INCLUDE_GS_SHIFT                       = 0x00000002,
    SQ_TT_WTYPE_INCLUDE_RSVD1_SHIFT                    = 0x00000003,
    SQ_TT_WTYPE_INCLUDE_HS_SHIFT                       = 0x00000004,
    SQ_TT_WTYPE_INCLUDE_RSVD2_SHIFT                    = 0x00000005,
    SQ_TT_WTYPE_INCLUDE_CS_SHIFT                       = 0x00000006,
} SQ_TT_WTYPE_INCLUDE_SHIFT;

typedef enum StencilFormat {
    STENCIL_INVALID                                    = 0x00000000,
    STENCIL_8                                          = 0x00000001,
} StencilFormat;

typedef enum StencilOp {
    STENCIL_KEEP                                       = 0x00000000,
    STENCIL_ZERO                                       = 0x00000001,
    STENCIL_ONES                                       = 0x00000002,
    STENCIL_REPLACE_TEST                               = 0x00000003,
    STENCIL_REPLACE_OP                                 = 0x00000004,
    STENCIL_ADD_CLAMP                                  = 0x00000005,
    STENCIL_SUB_CLAMP                                  = 0x00000006,
    STENCIL_INVERT                                     = 0x00000007,
    STENCIL_ADD_WRAP                                   = 0x00000008,
    STENCIL_SUB_WRAP                                   = 0x00000009,
    STENCIL_AND                                        = 0x0000000a,
    STENCIL_OR                                         = 0x0000000b,
    STENCIL_XOR                                        = 0x0000000c,
    STENCIL_NAND                                       = 0x0000000d,
    STENCIL_NOR                                        = 0x0000000e,
    STENCIL_XNOR                                       = 0x0000000f,
} StencilOp;

typedef enum SurfaceArray {
    ARRAY_1D                                           = 0x00000000,
    ARRAY_2D                                           = 0x00000001,
    ARRAY_3D                                           = 0x00000002,
    ARRAY_3D_SLICE                                     = 0x00000003,
} SurfaceArray;

typedef enum SurfaceEndian {
    ENDIAN_NONE                                        = 0x00000000,
    ENDIAN_8IN16                                       = 0x00000001,
    ENDIAN_8IN32                                       = 0x00000002,
    ENDIAN_8IN64                                       = 0x00000003,
} SurfaceEndian;

typedef enum SurfaceFormat {
    FMT_INVALID                                        = 0x00000000,
    FMT_8                                              = 0x00000001,
    FMT_16                                             = 0x00000002,
    FMT_8_8                                            = 0x00000003,
    FMT_32                                             = 0x00000004,
    FMT_16_16                                          = 0x00000005,
    FMT_10_11_11                                       = 0x00000006,
    FMT_11_11_10                                       = 0x00000007,
    FMT_10_10_10_2                                     = 0x00000008,
    FMT_2_10_10_10                                     = 0x00000009,
    FMT_8_8_8_8                                        = 0x0000000a,
    FMT_32_32                                          = 0x0000000b,
    FMT_16_16_16_16                                    = 0x0000000c,
    FMT_32_32_32                                       = 0x0000000d,
    FMT_32_32_32_32                                    = 0x0000000e,
    FMT_RESERVED_15                                    = 0x0000000f,
    FMT_5_6_5                                          = 0x00000010,
    FMT_1_5_5_5                                        = 0x00000011,
    FMT_5_5_5_1                                        = 0x00000012,
    FMT_4_4_4_4                                        = 0x00000013,
    FMT_8_24                                           = 0x00000014,
    FMT_24_8                                           = 0x00000015,
    FMT_X24_8_32_FLOAT                                 = 0x00000016,
    FMT_RESERVED_23                                    = 0x00000017,
    FMT_11_11_10_FLOAT                                 = 0x00000018,
    FMT_16_FLOAT                                       = 0x00000019,
    FMT_32_FLOAT                                       = 0x0000001a,
    FMT_16_16_FLOAT                                    = 0x0000001b,
    FMT_8_24_FLOAT                                     = 0x0000001c,
    FMT_24_8_FLOAT                                     = 0x0000001d,
    FMT_32_32_FLOAT                                    = 0x0000001e,
    FMT_10_11_11_FLOAT                                 = 0x0000001f,
    FMT_16_16_16_16_FLOAT                              = 0x00000020,
    FMT_3_3_2                                          = 0x00000021,
    FMT_6_5_5                                          = 0x00000022,
    FMT_32_32_32_32_FLOAT                              = 0x00000023,
    FMT_RESERVED_36                                    = 0x00000024,
    FMT_1                                              = 0x00000025,
    FMT_1_REVERSED                                     = 0x00000026,
    FMT_GB_GR                                          = 0x00000027,
    FMT_BG_RG                                          = 0x00000028,
    FMT_32_AS_8                                        = 0x00000029,
    FMT_32_AS_8_8                                      = 0x0000002a,
    FMT_5_9_9_9_SHAREDEXP                              = 0x0000002b,
    FMT_8_8_8                                          = 0x0000002c,
    FMT_16_16_16                                       = 0x0000002d,
    FMT_16_16_16_FLOAT                                 = 0x0000002e,
    FMT_4_4                                            = 0x0000002f,
    FMT_32_32_32_FLOAT                                 = 0x00000030,
    FMT_BC1                                            = 0x00000031,
    FMT_BC2                                            = 0x00000032,
    FMT_BC3                                            = 0x00000033,
    FMT_BC4                                            = 0x00000034,
    FMT_BC5                                            = 0x00000035,
    FMT_BC6                                            = 0x00000036,
    FMT_BC7                                            = 0x00000037,
    FMT_32_AS_32_32_32_32                              = 0x00000038,
    FMT_APC3                                           = 0x00000039,
    FMT_APC4                                           = 0x0000003a,
    FMT_APC5                                           = 0x0000003b,
    FMT_APC6                                           = 0x0000003c,
    FMT_APC7                                           = 0x0000003d,
    FMT_CTX1                                           = 0x0000003e,
    FMT_RESERVED_63                                    = 0x0000003f,
} SurfaceFormat;

typedef enum SurfaceNumber {
    NUMBER_UNORM                                       = 0x00000000,
    NUMBER_SNORM                                       = 0x00000001,
    NUMBER_USCALED                                     = 0x00000002,
    NUMBER_SSCALED                                     = 0x00000003,
    NUMBER_UINT                                        = 0x00000004,
    NUMBER_SINT                                        = 0x00000005,
    NUMBER_SRGB                                        = 0x00000006,
    NUMBER_FLOAT                                       = 0x00000007,
} SurfaceNumber;

typedef enum SurfaceSwap {
    SWAP_STD                                           = 0x00000000,
    SWAP_ALT                                           = 0x00000001,
    SWAP_STD_REV                                       = 0x00000002,
    SWAP_ALT_REV                                       = 0x00000003,
} SurfaceSwap;

typedef enum SurfaceTiling {
    ARRAY_LINEAR                                       = 0x00000000,
    ARRAY_TILED                                        = 0x00000001,
} SurfaceTiling;

typedef enum SU_PERFCNT_SEL {
    PERF_PAPC_PASX_REQ                                 = 0x00000000,
    PERF_PAPC_PASX_VTX_KILL_DISCARD                    = 0x00000006,
    PERF_PAPC_PASX_VTX_NAN_DISCARD                     = 0x00000007,
    PERF_CLPR_INPUT_PRIM                               = 0x00000008,
    PERF_CLPR_INPUT_NULL_PRIM                          = 0x00000009,
    PERF_CLPR_INPUT_EVENT                              = 0x0000000a,
    PERF_CLPR_INPUT_FIRST_OF_SUBGROUP                  = 0x0000000b,
    PERF_CLPR_INPUT_END_OF_PACKET                      = 0x0000000c,
    PERF_CLPR_INPUT_EXTENDED_EVENT                     = 0x0000000d,
    PERF_PAPC_CLPR_CULL_PRIM                           = 0x0000000e,
    PERF_PAPC_CLPR_VVUCP_CULL_PRIM                     = 0x0000000f,
    PERF_PAPC_CLPR_VV_CULL_PRIM                        = 0x00000010,
    PERF_PAPC_CLPR_UCP_CULL_PRIM                       = 0x00000011,
    PERF_PAPC_CLPR_VTX_KILL_CULL_PRIM                  = 0x00000012,
    PERF_PAPC_CLPR_VTX_NAN_CULL_PRIM                   = 0x00000013,
    PERF_PAPC_CLPR_CULL_TO_NULL_PRIM                   = 0x00000014,
    PERF_PAPC_CLIPGA_VTE_KILL_PRIM                     = 0x00000030,
    PERF_PAPC_SU_INPUT_PRIM                            = 0x00000031,
    PERF_PAPC_SU_INPUT_NULL_PRIM                       = 0x00000033,
    PERF_PAPC_SU_INPUT_PRIM_DUAL                       = 0x00000034,
    PERF_PAPC_SU_ZERO_AREA_CULL_PRIM                   = 0x00000036,
    PERF_PAPC_SU_BACK_FACE_CULL_PRIM                   = 0x00000037,
    PERF_PAPC_SU_FRONT_FACE_CULL_PRIM                  = 0x00000038,
    PERF_PAPC_SU_POLYMODE_FACE_CULL                    = 0x00000039,
    PERF_PAPC_SU_POLYMODE_BACK_CULL                    = 0x0000003a,
    PERF_PAPC_SU_POLYMODE_FRONT_CULL                   = 0x0000003b,
    PERF_PAPC_SU_POLYMODE_INVALID_FILL                 = 0x0000003c,
    PERF_PAPC_SU_OUTPUT_PRIM                           = 0x0000003d,
    PERF_PAPC_SU_OUTPUT_NULL_PRIM                      = 0x0000003f,
    PERF_PAPC_SU_OUTPUT_EVENT_FLAG                     = 0x00000040,
    PERF_PAPC_SU_OUTPUT_FIRST_PRIM_SLOT                = 0x00000041,
    PERF_PAPC_SU_OUTPUT_END_OF_PACKET                  = 0x00000042,
    PERF_PAPC_SU_OUTPUT_POLYMODE_FACE                  = 0x00000043,
    PERF_PAPC_SU_OUTPUT_POLYMODE_BACK                  = 0x00000044,
    PERF_PAPC_SU_OUTPUT_POLYMODE_FRONT                 = 0x00000045,
    PERF_PAPC_SU_OUTPUT_PRIM_DUAL                      = 0x00000049,
    PERF_PAPC_SU_OUTPUT_POLYMODE_DUAL                  = 0x0000004b,
    PERF_PAPC_PASX_REQ_IDLE                            = 0x0000004d,
    PERF_PAPC_PASX_REQ_BUSY                            = 0x0000004e,
    PERF_PAPC_PASX_REQ_STALLED                         = 0x0000004f,
    PERF_PAPC_PASX_REC_IDLE                            = 0x00000050,
    PERF_PAPC_PASX_REC_BUSY                            = 0x00000051,
    PERF_PAPC_PASX_REC_STARVED_SX                      = 0x00000052,
    PERF_PAPC_PASX_REC_STALLED                         = 0x00000053,
    PERF_PAPC_PASX_REC_STALLED_POS_MEM                 = 0x00000054,
    PERF_PAPC_PASX_REC_STALLED_CCGSM_IN                = 0x00000055,
    PERF_PAPC_CCGSM_IDLE                               = 0x00000056,
    PERF_PAPC_CCGSM_BUSY                               = 0x00000057,
    PERF_PAPC_CCGSM_STALLED                            = 0x00000058,
    PERF_PAPC_CLIPGA_IDLE                              = 0x00000064,
    PERF_PAPC_CLIPGA_BUSY                              = 0x00000065,
    PERF_PAPC_CLIPGA_STARVED_VTE_CLIP                  = 0x00000066,
    PERF_PAPC_CLIPGA_STALLED                           = 0x00000067,
    PERF_PAPC_CLIP_IDLE                                = 0x00000068,
    PERF_PAPC_CLIP_BUSY                                = 0x00000069,
    PERF_PAPC_SU_IDLE                                  = 0x0000006a,
    PERF_PAPC_SU_BUSY                                  = 0x0000006b,
    PERF_PAPC_SU_STARVED_CLIP                          = 0x0000006c,
    PERF_PAPC_SU_STALLED_SC                            = 0x0000006d,
    PERF_PAPC_CL_DYN_SCLK_VLD                          = 0x0000006e,
    PERF_PAPC_SU_DYN_SCLK_VLD                          = 0x0000006f,
    PERF_PAPC_PA_REG_SCLK_VLD                          = 0x00000070,
    PERF_PAPC_SU_SE0_PRIM_FILTER_CULL                  = 0x00000078,
    PERF_PAPC_SU_SE1_PRIM_FILTER_CULL                  = 0x00000079,
    PERF_PAPC_SU_SE0_OUTPUT_PRIM                       = 0x0000007b,
    PERF_PAPC_SU_SE1_OUTPUT_PRIM                       = 0x0000007c,
    PERF_PAPC_SU_ALL_OUTPUT_PRIM                       = 0x0000007d,
    PERF_PAPC_SU_SE0_OUTPUT_NULL_PRIM                  = 0x0000007e,
    PERF_PAPC_SU_SE1_OUTPUT_NULL_PRIM                  = 0x0000007f,
    PERF_PAPC_SU_ALL_OUTPUT_NULL_PRIM                  = 0x00000080,
    PERF_PAPC_SU_CULLED_PRIM                           = 0x00000087,
    PERF_PAPC_SU_OUTPUT_EOPG                           = 0x00000088,
    PERF_PAPC_SU_SE2_PRIM_FILTER_CULL                  = 0x00000089,
    PERF_PAPC_SU_SE3_PRIM_FILTER_CULL                  = 0x0000008a,
    PERF_PAPC_SU_SE2_OUTPUT_PRIM                       = 0x0000008b,
    PERF_PAPC_SU_SE3_OUTPUT_PRIM                       = 0x0000008c,
    PERF_PAPC_SU_SE2_OUTPUT_NULL_PRIM                  = 0x0000008d,
    PERF_PAPC_SU_SE3_OUTPUT_NULL_PRIM                  = 0x0000008e,
    PERF_SU_SMALL_PRIM_FILTER_CULL_CNT                 = 0x00000099,
    PERF_SMALL_PRIM_CULL_PRIM_1X1                      = 0x0000009a,
    PERF_SMALL_PRIM_CULL_PRIM_2X1                      = 0x0000009b,
    PERF_SMALL_PRIM_CULL_PRIM_1X2                      = 0x0000009c,
    PERF_SMALL_PRIM_CULL_PRIM_2X2                      = 0x0000009d,
    PERF_SMALL_PRIM_CULL_PRIM_3X1                      = 0x0000009e,
    PERF_SMALL_PRIM_CULL_PRIM_1X3                      = 0x0000009f,
    PERF_SMALL_PRIM_CULL_PRIM_3X2                      = 0x000000a0,
    PERF_SMALL_PRIM_CULL_PRIM_2X3                      = 0x000000a1,
    PERF_SMALL_PRIM_CULL_PRIM_NX1                      = 0x000000a2,
    PERF_SMALL_PRIM_CULL_PRIM_1XN                      = 0x000000a3,
    PERF_SMALL_PRIM_CULL_PRIM_NX2                      = 0x000000a4,
    PERF_SMALL_PRIM_CULL_PRIM_2XN                      = 0x000000a5,
    PERF_PA_VERTEX_FIFO_FULL                           = 0x000000b1,
    PERF_PA_PRIMIC_TO_CLPRIM_FIFO_FULL                 = 0x000000b2,
    PERF_PA_FETCH_TO_PRIMIC_P_FIFO_FULL                = 0x000000b3,
    PERF_ENGG_CSB_MACHINE_IS_STARVED                   = 0x000000b7,
    PERF_ENGG_CSB_MACHINE_STALLED_BY_CSB_MEMORY        = 0x000000b8,
    PERF_ENGG_CSB_MACHINE_STALLED_BY_SPI               = 0x000000b9,
    PERF_ENGG_CSB_GE_INPUT_FIFO_FULL                   = 0x000000ba,
    PERF_ENGG_CSB_PAYLOAD_INPUT_FIFO_FULL              = 0x000000bc,
    PERF_ENGG_CSB_GE_INPUT_FIFO_POP_BIT                = 0x000000bd,
    PERF_ENGG_CSB_PRIM_COUNT_EQ0                       = 0x000000be,
    PERF_ENGG_CSB_NULL_SUBGROUP                        = 0x000000bf,
    PERF_ENGG_CSB_GE_SENDING_SUBGROUP                  = 0x000000c0,
    PERF_ENGG_CSB_GE_MEMORY_FULL                       = 0x000000c1,
    PERF_ENGG_CSB_GE_MEMORY_EMPTY                      = 0x000000c2,
    PERF_ENGG_CSB_SPI_MEMORY_FULL                      = 0x000000c3,
    PERF_ENGG_CSB_SPI_MEMORY_EMPTY                     = 0x000000c4,
    PERF_ENGG_INDEX_REQ_NULL_REQUEST                   = 0x000000e0,
    PERF_ENGG_INDEX_RET_0_NEW_VERTS_THIS_PRIM          = 0x000000e1,
    PERF_ENGG_INDEX_RET_1_NEW_VERTS_THIS_PRIM          = 0x000000e2,
    PERF_ENGG_INDEX_RET_2_NEW_VERTS_THIS_PRIM          = 0x000000e3,
    PERF_ENGG_INDEX_RET_3_NEW_VERTS_THIS_PRIM          = 0x000000e4,
    PERF_ENGG_INDEX_REQ_STARVED                        = 0x000000e5,
    PERF_ENGG_INDEX_REQ_IDLE_AND_STALLED_BY_REQ2RTN_FIFO_FULL = 0x000000e6,
    PERF_ENGG_INDEX_REQ_BUSY_AND_STALLED_BY_REQ2RTN_FIFO_FULL = 0x000000e7,
    PERF_ENGG_INDEX_REQ_STALLED_BY_SX_CREDITS          = 0x000000e8,
    PERF_ENGG_INDEX_RET_REQ2RTN_FIFO_FULL              = 0x000000e9,
    PERF_ENGG_INDEX_RET_REQ2RTN_FIFO_EMPTY             = 0x000000ea,
    PERF_ENGG_INDEX_RET_SX_RECEIVE_FIFO_FULL           = 0x000000eb,
    PERF_ENGG_INDEX_RET_SXRX_STARVED_BY_CSB            = 0x000000ec,
    PERF_ENGG_INDEX_RET_SXRX_STARVED_BY_PRIMS          = 0x000000ed,
    PERF_ENGG_INDEX_RET_SXRX_STALLED_BY_PRIM_INDICES_CSB_FIFO = 0x000000ee,
    PERF_ENGG_INDEX_RET_SXRX_STALLED_BY_PRIM_INDICES_FIFO = 0x000000ef,
    PERF_ENGG_INDEX_RET_SXRX_READING_EVENT             = 0x000000f0,
    PERF_ENGG_INDEX_RET_SXRX_READING_NULL_SUBGROUP     = 0x000000f1,
    PERF_ENGG_INDEX_RET_SXRX_READING_SUBGROUP_PRIMCOUNT_EQ0 = 0x000000f2,
    PERF_ENGG_INDEX_RET_SXRX_READING_QDWORD_0_VALID_PRIMS_NOPL = 0x000000f3,
    PERF_ENGG_INDEX_RET_SXRX_READING_QDWORD_1_VALID_PRIMS_NOPL = 0x000000f4,
    PERF_ENGG_INDEX_RET_SXRX_READING_QDWORD_2_VALID_PRIMS_NOPL = 0x000000f5,
    PERF_ENGG_INDEX_RET_SXRX_READING_QDWORD_3_VALID_PRIMS_NOPL = 0x000000f6,
    PERF_ENGG_INDEX_RET_SXRX_READING_QDWORD_4_VALID_PRIMS_NOPL = 0x000000f7,
    PERF_ENGG_INDEX_RET_SXRX_READING_QDWORD_0_VALID_PRIMS_PL = 0x000000f8,
    PERF_ENGG_INDEX_RET_SXRX_READING_QDWORD_1_VALID_PRIMS_PL = 0x000000f9,
    PERF_ENGG_INDEX_RET_SXRX_READING_QDWORD_2_VALID_PRIMS_PL = 0x000000fa,
    PERF_ENGG_INDEX_RET_SXRX_READING_QDWORD_3_VALID_PRIMS_PL = 0x000000fb,
    PERF_ENGG_INDEX_RET_SXRX_READING_QDWORD_4_VALID_PRIMS_PL = 0x000000fc,
    PERF_ENGG_INDEX_PRIM_IF_STALLED_BY_FULL_FETCH_TO_PRIMIC_P_FIFO = 0x000000fd,
    PERF_ENGG_INDEX_PRIM_IF_STALLED_BY_FULL_FETCH_TO_PRIMIC_S_FIFO = 0x000000fe,
    PERF_ENGG_INDEX_PRIM_IF_STARVED_BY_NO_CSB          = 0x000000ff,
    PERF_ENGG_INDEX_PRIM_IF_STARVED_BY_NO_PRIM         = 0x00000100,
    PERF_ENGG_INDEX_PRIM_IF_FETCH_TO_PRIMIC_P_FIFO_WRITE = 0x00000101,
    PERF_ENGG_INDEX_PRIM_IF_FETCH_TO_PRIMIC_P_FIFO_NO_WRITE = 0x00000102,
    PERF_ENGG_POS_REQ_STARVED                          = 0x00000103,
    PERF_ENGG_INDEX_RET_SXRX_NULL_DROPPER_STALLED_BY_FULL_PRIM_FIFO = 0x00000104,
    PERF_ENGG_BUSY                                     = 0x00000105,
    PERF_OUTPUT_PRIM_1_SC                              = 0x00000107,
    PERF_OUTPUT_PRIM_2_SC                              = 0x00000108,
    PERF_OUTPUT_PRIM_3_SC                              = 0x00000109,
    PERF_OUTPUT_PRIM_4_SC                              = 0x0000010a,
    PERF_OUTPUT_PRIM_5_SC                              = 0x0000010b,
    PERF_OUTPUT_PRIM_6_SC                              = 0x0000010c,
    PERF_OUTPUT_PRIM_7_SC                              = 0x0000010d,
    PERF_OUTPUT_PRIM_8_SC                              = 0x0000010e,
    PERF_OUTPUT_PRIM_9_SC                              = 0x0000010f,
    PERF_PAF_INPUT_VERTS                               = 0x00000114,
    PERF_PASX_MISC_VECTOR                              = 0x00000115,
    PERF_PASX_CCDIST0_VECTOR                           = 0x00000116,
    PERF_PASX_CCDIST1_VECTOR                           = 0x00000117,
    PERF_PASX_STEREO_POS_VECTOR                        = 0x00000118,
    PERF_CLPR_INPUT_SEND                               = 0x00000119,
    PERF_SU_INPUT_SEND                                 = 0x0000011a,
    PERF_SU_OUTPUT_SEND                                = 0x0000011b,
    PERF_PAPC_SU_SE4_PRIM_FILTER_CULL                  = 0x0000011c,
    PERF_PAPC_SU_SE5_PRIM_FILTER_CULL                  = 0x0000011d,
    PERF_PAPC_SU_SE4_OUTPUT_PRIM                       = 0x0000011e,
    PERF_PAPC_SU_SE5_OUTPUT_PRIM                       = 0x0000011f,
    PERF_PAPC_SU_SE4_OUTPUT_NULL_PRIM                  = 0x00000120,
    PERF_PAPC_SU_SE5_OUTPUT_NULL_PRIM                  = 0x00000121,
    PERF_PAPC_SU_SE4_STALLED_SC                        = 0x00000122,
    PERF_PAPC_SU_SE5_STALLED_SC                        = 0x00000123,
    PERF_ENGG_INDEX_RET0_NEW_VERTS                     = 0x00000124,
    PERF_ENGG_INDEX_RET1_NEW_VERTS                     = 0x00000125,
    PERF_ENGG_INDEX_RET2_NEW_VERTS                     = 0x00000126,
    PERF_ENGG_INDEX_RET3_NEW_VERTS                     = 0x00000127,
    PERF_ENGG_INDEX_RET4_NEW_VERTS                     = 0x00000128,
    PERF_ENGG_INDEX_RET5_NEW_VERTS                     = 0x00000129,
    PERF_ENGG_INDEX_RET6_NEW_VERTS                     = 0x0000012a,
    PERF_ENGG_INDEX_RET7_NEW_VERTS                     = 0x0000012b,
    PERF_ENGG_INDEX_RET8_NEW_VERTS                     = 0x0000012c,
    PERF_ENGG_INDEX_RET9_NEW_VERTS                     = 0x0000012d,
    PERF_ENGG_INDEX_RET10_NEW_VERTS                    = 0x0000012e,
    PERF_ENGG_INDEX_RET11_NEW_VERTS                    = 0x0000012f,
    PERF_ENGG_INDEX_RET12_NEW_VERTS                    = 0x00000130,
    PERF_CLPR_BACK_PRIM                                = 0x00000135,
    PERF_PA_BUSY                                       = 0x00000136,
    PERF_PAF_BUSY                                      = 0x00000137,
    PERF_PAF_PRIM_SHADER_OUT_PRIMS                     = 0x00000138,
    PERF_PAF_PRIM_SHADER_OUT_VERTS                     = 0x00000139,
    PERF_PAF_SU_TOTAL_CULL_PRIM                        = 0x0000013a,
    PERF_PAF_POS_DATA_WRITE                            = 0x0000013b,
    PERF_PAF_PRIM_DATA_WRITE                           = 0x0000013c,
    PERF_PAF_MEM_WR_TAG_STALL                          = 0x0000013d,
    PERF_PAF_MEM_WR_MAX_SUBGRP_STALL                   = 0x0000013e,
    PERF_PAF_MEM_WR_REQ_TAG0                           = 0x0000013f,
    PERF_PAF_MEM_WR_INFLIGHT_TAG0                      = 0x00000140,
    PERF_PAF_GEWD_CSB_STALL                            = 0x00000141,
    PERF_PAF_GL1X_REQ_STALL                            = 0x00000142,
    PERF_PAF_PAB_EVENT_ADDR_STALL                      = 0x00000143,
    PERF_PAF_PRIM_W_LE_ZERO                            = 0x00000144,
    PERF_PAF_PRIM_VTE_POS_CLAMP                        = 0x00000145,
    PERF_PAB_CLPR_INPUT_PRIM                           = 0x00000208,
    PERF_PAB_CLPR_INPUT_NULL_PRIM                      = 0x00000209,
    PERF_PAB_CLPR_INPUT_EVENT                          = 0x0000020a,
    PERF_PAB_CLPR_INPUT_FIRST_OF_SUBGROUP              = 0x0000020b,
    PERF_PAB_CLPR_INPUT_END_OF_PACKET                  = 0x0000020c,
    PERF_PAB_CLPR_INPUT_EXTENDED_EVENT                 = 0x0000020d,
    PERF_PAB_PAPC_CLPR_CULL_PRIM                       = 0x0000020e,
    PERF_PAB_PAPC_CLPR_VVUCP_CULL_PRIM                 = 0x0000020f,
    PERF_PAB_PAPC_CLPR_VV_CULL_PRIM                    = 0x00000210,
    PERF_PAB_PAPC_CLPR_UCP_CULL_PRIM                   = 0x00000211,
    PERF_PAB_PAPC_CLPR_CULL_TO_NULL_PRIM               = 0x00000214,
    PERF_PAB_PAPC_CLPR_VVUCP_CLIP_PRIM                 = 0x00000215,
    PERF_PAB_PAPC_CLPR_VV_CLIP_PRIM                    = 0x00000216,
    PERF_PAB_PAPC_CLPR_UCP_CLIP_PRIM                   = 0x00000217,
    PERF_PAB_PAPC_CLPR_POINT_CLIP_CANDIDATE            = 0x00000218,
    PERF_PAB_PAPC_CLPR_CLIP_PLANE_CNT_1                = 0x00000219,
    PERF_PAB_PAPC_CLPR_CLIP_PLANE_CNT_2                = 0x0000021a,
    PERF_PAB_PAPC_CLPR_CLIP_PLANE_CNT_3                = 0x0000021b,
    PERF_PAB_PAPC_CLPR_CLIP_PLANE_CNT_4                = 0x0000021c,
    PERF_PAB_PAPC_CLPR_CLIP_PLANE_CNT_5_8              = 0x0000021d,
    PERF_PAB_CLPR_CLIP_PLANE_CNT_9_PLUS                = 0x0000021e,
    PERF_PAB_PAPC_CLPR_CLIP_PLANE_NEAR                 = 0x0000021f,
    PERF_PAB_PAPC_CLPR_CLIP_PLANE_FAR                  = 0x00000220,
    PERF_PAB_PAPC_CLPR_CLIP_PLANE_LEFT                 = 0x00000221,
    PERF_PAB_PAPC_CLPR_CLIP_PLANE_RIGHT                = 0x00000222,
    PERF_PAB_PAPC_CLPR_CLIP_PLANE_TOP                  = 0x00000223,
    PERF_PAB_PAPC_CLPR_CLIP_PLANE_BOTTOM               = 0x00000224,
    PERF_PAB_PAPC_CLPR_RASTER_KILL_CULL_PRIM           = 0x00000226,
    PERF_PAB_PAPC_CLSM_NULL_PRIM                       = 0x00000227,
    PERF_PAB_PAPC_CLSM_TOTALLY_VISIBLE_PRIM            = 0x00000228,
    PERF_PAB_PAPC_CLSM_CULL_TO_NULL_PRIM               = 0x00000229,
    PERF_PAB_PAPC_CLSM_OUT_PRIM_CNT_1                  = 0x0000022a,
    PERF_PAB_PAPC_CLSM_OUT_PRIM_CNT_2                  = 0x0000022b,
    PERF_PAB_PAPC_CLSM_OUT_PRIM_CNT_3                  = 0x0000022c,
    PERF_PAB_PAPC_CLSM_OUT_PRIM_CNT_4                  = 0x0000022d,
    PERF_PAB_PAPC_CLSM_OUT_PRIM_CNT_5_8                = 0x0000022e,
    PERF_PAB_PAPC_CLSM_OUT_PRIM_CNT_9_PLUS             = 0x0000022f,
    PERF_PAB_PAPC_SU_INPUT_PRIM                        = 0x00000231,
    PERF_PAB_PAPC_SU_INPUT_CLIP_PRIM                   = 0x00000232,
    PERF_PAB_PAPC_SU_INPUT_NULL_PRIM                   = 0x00000233,
    PERF_PAB_PAPC_SU_INPUT_PRIM_DUAL                   = 0x00000234,
    PERF_PAB_PAPC_SU_INPUT_CLIP_PRIM_DUAL              = 0x00000235,
    PERF_PAB_PAPC_SU_ZERO_AREA_CULL_PRIM               = 0x00000236,
    PERF_PAB_PAPC_SU_BACK_FACE_CULL_PRIM               = 0x00000237,
    PERF_PAB_PAPC_SU_FRONT_FACE_CULL_PRIM              = 0x00000238,
    PERF_PAB_PAPC_SU_POLYMODE_FACE_CULL                = 0x00000239,
    PERF_PAB_PAPC_SU_POLYMODE_BACK_CULL                = 0x0000023a,
    PERF_PAB_PAPC_SU_POLYMODE_FRONT_CULL               = 0x0000023b,
    PERF_PAB_PAPC_SU_POLYMODE_INVALID_FILL             = 0x0000023c,
    PERF_PAB_PAPC_SU_OUTPUT_PRIM                       = 0x0000023d,
    PERF_PAB_PAPC_SU_OUTPUT_CLIP_PRIM                  = 0x0000023e,
    PERF_PAB_PAPC_SU_OUTPUT_NULL_PRIM                  = 0x0000023f,
    PERF_PAB_PAPC_SU_OUTPUT_EVENT_FLAG                 = 0x00000240,
    PERF_PAB_PAPC_SU_OUTPUT_FIRST_PRIM_SLOT            = 0x00000241,
    PERF_PAB_PAPC_SU_OUTPUT_END_OF_PACKET              = 0x00000242,
    PERF_PAB_PAPC_SU_OUTPUT_POLYMODE_FACE              = 0x00000243,
    PERF_PAB_PAPC_SU_OUTPUT_POLYMODE_BACK              = 0x00000244,
    PERF_PAB_PAPC_SU_OUTPUT_POLYMODE_FRONT             = 0x00000245,
    PERF_PAB_PAPC_SU_OUT_CLIP_POLYMODE_FACE            = 0x00000246,
    PERF_PAB_PAPC_SU_OUT_CLIP_POLYMODE_BACK            = 0x00000247,
    PERF_PAB_PAPC_SU_OUT_CLIP_POLYMODE_FRONT           = 0x00000248,
    PERF_PAB_PAPC_SU_OUTPUT_PRIM_DUAL                  = 0x00000249,
    PERF_PAB_PAPC_SU_OUTPUT_CLIP_PRIM_DUAL             = 0x0000024a,
    PERF_PAB_PAPC_SU_OUTPUT_POLYMODE_DUAL              = 0x0000024b,
    PERF_PAB_PAPC_SU_OUTPUT_CLIP_POLYMODE_DUAL         = 0x0000024c,
    PERF_PAB_PAPC_CCGSM_IDLE                           = 0x00000256,
    PERF_PAB_PAPC_CCGSM_BUSY                           = 0x00000257,
    PERF_PAB_PAPC_CCGSM_STALLED                        = 0x00000258,
    PERF_PAB_PAPC_CLPRIM_IDLE                          = 0x00000259,
    PERF_PAB_PAPC_CLPRIM_BUSY                          = 0x0000025a,
    PERF_PAB_PAPC_CLPRIM_STALLED                       = 0x0000025b,
    PERF_PAB_PAPC_CLPRIM_STARVED_CCGSM                 = 0x0000025c,
    PERF_PAB_PAPC_CLIPSM_IDLE                          = 0x0000025d,
    PERF_PAB_PAPC_CLIPSM_BUSY                          = 0x0000025e,
    PERF_PAB_PAPC_CLIPSM_WAIT_CLIP_VERT_ENGH           = 0x0000025f,
    PERF_PAB_PAPC_CLIPSM_WAIT_HIGH_PRI_SEQ             = 0x00000260,
    PERF_PAB_PAPC_CLIPSM_WAIT_CLIPGA                   = 0x00000261,
    PERF_PAB_PAPC_CLIPSM_WAIT_AVAIL_VTE_CLIP           = 0x00000262,
    PERF_PAB_PAPC_CLIPSM_WAIT_CLIP_OUTSM               = 0x00000263,
    PERF_PAB_PAPC_CLIPGA_IDLE                          = 0x00000264,
    PERF_PAB_PAPC_CLIPGA_BUSY                          = 0x00000265,
    PERF_PAB_PAPC_CLIPGA_STARVED_VTE_CLIP              = 0x00000266,
    PERF_PAB_PAPC_CLIPGA_STALLED                       = 0x00000267,
    PERF_PAB_PAPC_CLIP_IDLE                            = 0x00000268,
    PERF_PAB_PAPC_CLIP_BUSY                            = 0x00000269,
    PERF_PAB_PAPC_SU_IDLE                              = 0x0000026a,
    PERF_PAB_PAPC_SU_BUSY                              = 0x0000026b,
    PERF_PAB_PAPC_SU_STARVED_CLIP                      = 0x0000026c,
    PERF_PAB_PAPC_SU_STALLED_SC                        = 0x0000026d,
    PERF_PAB_PAPC_CL_DYN_SCLK_VLD                      = 0x0000026e,
    PERF_PAB_PAPC_SU_DYN_SCLK_VLD                      = 0x0000026f,
    PERF_PAB_PAPC_PA_REG_SCLK_VLD                      = 0x00000270,
    PERF_PAB_PAPC_SU_SE0_STALLED_SC                    = 0x00000283,
    PERF_PAB_PAPC_SU_SE1_STALLED_SC                    = 0x00000284,
    PERF_PAB_PAPC_SU_ALL_STALLED_SC                    = 0x00000285,
    PERF_PAB_PAPC_CLSM_CLIPPING_PRIM                   = 0x00000286,
    PERF_PAB_PAPC_SU_CULLED_PRIM                       = 0x00000287,
    PERF_PAB_PAPC_SU_SE2_STALLED_SC                    = 0x00000297,
    PERF_PAB_PAPC_SU_SE3_STALLED_SC                    = 0x00000298,
    PERF_PAB_SU_SMALL_PRIM_FILTER_CULL_CNT             = 0x00000299,
    PERF_PAB_SMALL_PRIM_CULL_PRIM_1X1                  = 0x0000029a,
    PERF_PAB_SMALL_PRIM_CULL_PRIM_2X1                  = 0x0000029b,
    PERF_PAB_SMALL_PRIM_CULL_PRIM_1X2                  = 0x0000029c,
    PERF_PAB_SMALL_PRIM_CULL_PRIM_2X2                  = 0x0000029d,
    PERF_PAB_SMALL_PRIM_CULL_PRIM_3X1                  = 0x0000029e,
    PERF_PAB_SMALL_PRIM_CULL_PRIM_1X3                  = 0x0000029f,
    PERF_PAB_SMALL_PRIM_CULL_PRIM_3X2                  = 0x000002a0,
    PERF_PAB_SMALL_PRIM_CULL_PRIM_2X3                  = 0x000002a1,
    PERF_PAB_SMALL_PRIM_CULL_PRIM_NX1                  = 0x000002a2,
    PERF_PAB_SMALL_PRIM_CULL_PRIM_1XN                  = 0x000002a3,
    PERF_PAB_SMALL_PRIM_CULL_PRIM_NX2                  = 0x000002a4,
    PERF_PAB_SMALL_PRIM_CULL_PRIM_2XN                  = 0x000002a5,
    PERF_PAB_SC0_QUALIFIED_SEND_BUSY_EVENT             = 0x000002a9,
    PERF_PAB_SC0_QUALIFIED_SEND_NOT_BUSY_EVENT         = 0x000002aa,
    PERF_PAB_SC1_QUALIFIED_SEND_BUSY_EVENT             = 0x000002ab,
    PERF_PAB_SC1_QUALIFIED_SEND_NOT_BUSY_EVENT         = 0x000002ac,
    PERF_PAB_SC2_QUALIFIED_SEND_BUSY_EVENT             = 0x000002ad,
    PERF_PAB_SC2_QUALIFIED_SEND_NOT_BUSY_EVENT         = 0x000002ae,
    PERF_PAB_SC3_QUALIFIED_SEND_BUSY_EVENT             = 0x000002af,
    PERF_PAB_SC3_QUALIFIED_SEND_NOT_BUSY_EVENT         = 0x000002b0,
    PERF_PAB_PA_VERTEX_FIFO_FULL                       = 0x000002b1,
    PERF_PAB_PA_PRIMIC_TO_CLPRIM_FIFO_FULL             = 0x000002b2,
    PERF_PAB_PA_FETCH_TO_PRIMIC_P_FIFO_FULL            = 0x000002b3,
    PERF_PAB_CLIPSM_CULL_PRIMS_CNT                     = 0x00000306,
    PERF_PAB_PASX_POS_VECTOR                           = 0x00000314,
    PERF_PAB_PASX_MISC_VECTOR                          = 0x00000315,
    PERF_PAB_PASX_CCDIST0_VECTOR                       = 0x00000316,
    PERF_PAB_PASX_CCDIST1_VECTOR                       = 0x00000317,
    PERF_PAB_CLPR_INPUT_SEND                           = 0x00000319,
    PERF_PAB_SU_INPUT_SEND                             = 0x0000031a,
    PERF_PAB_SU_OUTPUT_SEND                            = 0x0000031b,
    PERF_PAB_ENGG_INDEX_RET0_NEW_VERTS                 = 0x00000324,
    PERF_PAB_ENGG_INDEX_RET1_NEW_VERTS                 = 0x00000325,
    PERF_PAB_ENGG_INDEX_RET2_NEW_VERTS                 = 0x00000326,
    PERF_PAB_ENGG_INDEX_RET3_NEW_VERTS                 = 0x00000327,
    PERF_PAB_ENGG_INDEX_RET4_NEW_VERTS                 = 0x00000328,
    PERF_PAB_ENGG_INDEX_RET5_NEW_VERTS                 = 0x00000329,
    PERF_PAB_ENGG_INDEX_RET6_NEW_VERTS                 = 0x0000032a,
    PERF_PAB_ENGG_INDEX_RET7_NEW_VERTS                 = 0x0000032b,
    PERF_PAB_ENGG_INDEX_RET8_NEW_VERTS                 = 0x0000032c,
    PERF_PAB_ENGG_INDEX_RET9_NEW_VERTS                 = 0x0000032d,
    PERF_PAB_ENGG_INDEX_RET10_NEW_VERTS                = 0x0000032e,
    PERF_PAB_ENGG_INDEX_RET11_NEW_VERTS                = 0x0000032f,
    PERF_PAB_ENGG_INDEX_RET12_NEW_VERTS                = 0x00000330,
    PERF_PAB_CLPR_BACK_PRIM                            = 0x00000335,
    PERF_PAB_BUSY                                      = 0x00000336,
    PERF_PAB_SU_TOTAL_CULL_PRIM                        = 0x00000337,
    PERF_PAB_GL1X_REQ_STALL                            = 0x00000338,
    PERF_PAB_MEM_RD_TAG_STALL                          = 0x00000339,
    PERF_PAB_MEM_RD_REQ_TAG0                           = 0x0000033a,
    PERF_PAB_MEM_RD_INFLIGHT_TAG0                      = 0x0000033b,
    PERF_PAB_POS_BANK_CONFLICT                         = 0x0000033c,
} SU_PERFCNT_SEL;

constexpr unsigned int MaxSuPerfcntSel                 = PERF_PAB_POS_BANK_CONFLICT;

typedef enum SWIZZLE_MODE_ENUM {
    SW_LINEAR                                          = 0x00000000,
    SW_256B_2D                                         = 0x00000001,
    SW_4KB_2D                                          = 0x00000002,
    SW_64KB_2D                                         = 0x00000003,
    SW_256KB_2D                                        = 0x00000004,
    SW_4KB_3D                                          = 0x00000005,
    SW_64KB_3D                                         = 0x00000006,
    SW_256KB_3D                                        = 0x00000007,
} SWIZZLE_MODE_ENUM;

typedef enum SWIZZLE_TYPE_ENUM {
    SW_D_2D                                            = 0x00000000,
    SW_S_3D                                            = 0x00000001,
    SW_L                                               = 0x00000002,
} SWIZZLE_TYPE_ENUM;

typedef enum SX_BLEND_OPT {
    BLEND_OPT_PRESERVE_NONE_IGNORE_ALL                 = 0x00000000,
    BLEND_OPT_PRESERVE_ALL_IGNORE_NONE                 = 0x00000001,
    BLEND_OPT_PRESERVE_C1_IGNORE_C0                    = 0x00000002,
    BLEND_OPT_PRESERVE_C0_IGNORE_C1                    = 0x00000003,
    BLEND_OPT_PRESERVE_A1_IGNORE_A0                    = 0x00000004,
    BLEND_OPT_PRESERVE_A0_IGNORE_A1                    = 0x00000005,
    BLEND_OPT_PRESERVE_NONE_IGNORE_A0                  = 0x00000006,
    BLEND_OPT_PRESERVE_NONE_IGNORE_NONE                = 0x00000007,
} SX_BLEND_OPT;

typedef enum SX_DOWNCONVERT_FORMAT {
    SX_RT_EXPORT_NO_CONVERSION                         = 0x00000000,
    SX_RT_EXPORT_32_R                                  = 0x00000001,
    SX_RT_EXPORT_32_A                                  = 0x00000002,
    SX_RT_EXPORT_10_11_11                              = 0x00000003,
    SX_RT_EXPORT_2_10_10_10                            = 0x00000004,
    SX_RT_EXPORT_8_8_8_8                               = 0x00000005,
    SX_RT_EXPORT_5_6_5                                 = 0x00000006,
    SX_RT_EXPORT_1_5_5_5                               = 0x00000007,
    SX_RT_EXPORT_4_4_4_4                               = 0x00000008,
    SX_RT_EXPORT_16_16_GR                              = 0x00000009,
    SX_RT_EXPORT_16_16_AR                              = 0x0000000a,
    SX_RT_EXPORT_9_9_9_E5                              = 0x0000000b,
    SX_RT_EXPORT_2_10_10_10_7E3                        = 0x0000000c,
    SX_RT_EXPORT_2_10_10_10_6E4                        = 0x0000000d,
} SX_DOWNCONVERT_FORMAT;

typedef enum SX_OPT_COMB_FCN {
    OPT_COMB_NONE                                      = 0x00000000,
    OPT_COMB_ADD                                       = 0x00000001,
    OPT_COMB_SUBTRACT                                  = 0x00000002,
    OPT_COMB_MIN                                       = 0x00000003,
    OPT_COMB_MAX                                       = 0x00000004,
    OPT_COMB_REVSUBTRACT                               = 0x00000005,
    OPT_COMB_BLEND_DISABLED                            = 0x00000006,
    OPT_COMB_SAFE_ADD                                  = 0x00000007,
} SX_OPT_COMB_FCN;

typedef enum SX_PERFCOUNTER_VALS {
    SX_PERF_SEL_PA_IDLE_CYCLES                         = 0x00000000,
    SX_PERF_SEL_PA_REQ                                 = 0x00000001,
    SX_PERF_SEL_PA_POS                                 = 0x00000002,
    SX_PERF_SEL_CLOCK                                  = 0x00000003,
    SX_PERF_SEL_GATE_EN1                               = 0x00000004,
    SX_PERF_SEL_GATE_EN2                               = 0x00000005,
    SX_PERF_SEL_GATE_EN3                               = 0x00000006,
    SX_PERF_SEL_GATE_EN4                               = 0x00000007,
    SX_PERF_SEL_SH_POS_STARVE                          = 0x00000008,
    SX_PERF_SEL_SH_COLOR_STARVE                        = 0x00000009,
    SX_PERF_SEL_SH_POS_STALL                           = 0x0000000a,
    SX_PERF_SEL_SH_COLOR_STALL                         = 0x0000000b,
    SX_PERF_SEL_DB0_PIXELS                             = 0x0000000c,
    SX_PERF_SEL_DB0_HALF_QUADS                         = 0x0000000d,
    SX_PERF_SEL_DB0_PIXEL_STALL                        = 0x0000000e,
    SX_PERF_SEL_DB0_PIXEL_IDLE                         = 0x0000000f,
    SX_PERF_SEL_DB0_PRED_PIXELS                        = 0x00000010,
    SX_PERF_SEL_DB1_PIXELS                             = 0x00000011,
    SX_PERF_SEL_DB1_HALF_QUADS                         = 0x00000012,
    SX_PERF_SEL_DB1_PIXEL_STALL                        = 0x00000013,
    SX_PERF_SEL_DB1_PIXEL_IDLE                         = 0x00000014,
    SX_PERF_SEL_DB1_PRED_PIXELS                        = 0x00000015,
    SX_PERF_SEL_DB2_PIXELS                             = 0x00000016,
    SX_PERF_SEL_DB2_HALF_QUADS                         = 0x00000017,
    SX_PERF_SEL_DB2_PIXEL_STALL                        = 0x00000018,
    SX_PERF_SEL_DB2_PIXEL_IDLE                         = 0x00000019,
    SX_PERF_SEL_DB2_PRED_PIXELS                        = 0x0000001a,
    SX_PERF_SEL_DB3_PIXELS                             = 0x0000001b,
    SX_PERF_SEL_DB3_HALF_QUADS                         = 0x0000001c,
    SX_PERF_SEL_DB3_PIXEL_STALL                        = 0x0000001d,
    SX_PERF_SEL_DB3_PIXEL_IDLE                         = 0x0000001e,
    SX_PERF_SEL_DB3_PRED_PIXELS                        = 0x0000001f,
    SX_PERF_SEL_COL_BUSY                               = 0x00000020,
    SX_PERF_SEL_POS_BUSY                               = 0x00000021,
    SX_PERF_SEL_DB0_MRT_BLEND_BYPASS                   = 0x00000022,
    SX_PERF_SEL_DB0_MRT_DONT_RD_DEST                   = 0x00000023,
    SX_PERF_SEL_DB0_MRT_DISCARD_SRC                    = 0x00000024,
    SX_PERF_SEL_DB0_MRT_SINGLE_QUADS                   = 0x00000025,
    SX_PERF_SEL_DB0_MRT_DOUBLE_QUADS                   = 0x00000026,
    SX_PERF_SEL_DB1_MRT_BLEND_BYPASS                   = 0x00000027,
    SX_PERF_SEL_DB1_MRT_DONT_RD_DEST                   = 0x00000028,
    SX_PERF_SEL_DB1_MRT_DISCARD_SRC                    = 0x00000029,
    SX_PERF_SEL_DB1_MRT_SINGLE_QUADS                   = 0x0000002a,
    SX_PERF_SEL_DB1_MRT_DOUBLE_QUADS                   = 0x0000002b,
    SX_PERF_SEL_DB2_MRT_BLEND_BYPASS                   = 0x0000002c,
    SX_PERF_SEL_DB2_MRT_DONT_RD_DEST                   = 0x0000002d,
    SX_PERF_SEL_DB2_MRT_DISCARD_SRC                    = 0x0000002e,
    SX_PERF_SEL_DB2_MRT_SINGLE_QUADS                   = 0x0000002f,
    SX_PERF_SEL_DB2_MRT_DOUBLE_QUADS                   = 0x00000030,
    SX_PERF_SEL_DB3_MRT_BLEND_BYPASS                   = 0x00000031,
    SX_PERF_SEL_DB3_MRT_DONT_RD_DEST                   = 0x00000032,
    SX_PERF_SEL_DB3_MRT_DISCARD_SRC                    = 0x00000033,
    SX_PERF_SEL_DB3_MRT_SINGLE_QUADS                   = 0x00000034,
    SX_PERF_SEL_DB3_MRT_DOUBLE_QUADS                   = 0x00000035,
    SX_PERF_SEL_PA_REQ_LATENCY                         = 0x00000036,
    SX_PERF_SEL_POS_SCBD_STALL                         = 0x00000037,
    SX_PERF_SEL_CLOCK_DROP_STALL                       = 0x00000038,
    SX_PERF_SEL_GATE_EN5                               = 0x00000039,
    SX_PERF_SEL_GATE_EN6                               = 0x0000003a,
    SX_PERF_SEL_DB0_SIZE                               = 0x0000003b,
    SX_PERF_SEL_DB1_SIZE                               = 0x0000003c,
    SX_PERF_SEL_DB2_SIZE                               = 0x0000003d,
    SX_PERF_SEL_DB3_SIZE                               = 0x0000003e,
    SX_PERF_SEL_IDX_STALL_CYCLES                       = 0x0000003f,
    SX_PERF_SEL_IDX_IDLE_CYCLES                        = 0x00000040,
    SX_PERF_SEL_IDX_REQ                                = 0x00000041,
    SX_PERF_SEL_IDX_RET                                = 0x00000042,
    SX_PERF_SEL_IDX_REQ_LATENCY                        = 0x00000043,
    SX_PERF_SEL_IDX_SCBD_STALL                         = 0x00000044,
    SX_PERF_SEL_GATE_EN7                               = 0x00000045,
    SX_PERF_SEL_GATE_EN8                               = 0x00000046,
    SX_PERF_SEL_SH_IDX_STARVE                          = 0x00000047,
    SX_PERF_SEL_IDX_BUSY                               = 0x00000048,
    SX_PERF_SEL_PA_POS_BANK_CONF                       = 0x00000049,
    SX_PERF_SEL_DB0_END_OF_WAVE                        = 0x0000004a,
    SX_PERF_SEL_DB0_4X2_DISCARD                        = 0x0000004b,
    SX_PERF_SEL_DB1_END_OF_WAVE                        = 0x0000004c,
    SX_PERF_SEL_DB1_4X2_DISCARD                        = 0x0000004d,
    SX_PERF_SEL_DB2_END_OF_WAVE                        = 0x0000004e,
    SX_PERF_SEL_DB2_4X2_DISCARD                        = 0x0000004f,
    SX_PERF_SEL_DB3_END_OF_WAVE                        = 0x00000050,
    SX_PERF_SEL_DB3_4X2_DISCARD                        = 0x00000051,
} SX_PERFCOUNTER_VALS;

constexpr unsigned int MaxSxPerfcounterVals            = SX_PERF_SEL_DB3_4X2_DISCARD;

typedef enum TA_PERFCOUNT_SEL {
    TA_PERF_SEL_NULL                                   = 0x00000000,
    TA_PERF_SEL_image_sampler_has_offset_instructions  = 0x00000001,
    TA_PERF_SEL_image_sampler_has_bias_instructions    = 0x00000002,
    TA_PERF_SEL_image_sampler_has_reference_instructions = 0x00000003,
    TA_PERF_SEL_image_sampler_has_ds_instructions      = 0x00000004,
    TA_PERF_SEL_image_sampler_has_dt_instructions      = 0x00000005,
    TA_PERF_SEL_image_sampler_has_dr_instructions      = 0x00000006,
    TA_PERF_SEL_gradient_busy                          = 0x00000007,
    TA_PERF_SEL_gradient_fifo_busy                     = 0x00000008,
    TA_PERF_SEL_lod_busy                               = 0x00000009,
    TA_PERF_SEL_lod_fifo_busy                          = 0x0000000a,
    TA_PERF_SEL_addresser_busy                         = 0x0000000b,
    TA_PERF_SEL_addresser_fifo_busy                    = 0x0000000c,
    TA_PERF_SEL_aligner_busy                           = 0x0000000d,
    TA_PERF_SEL_write_path_busy                        = 0x0000000e,
    TA_PERF_SEL_ta_busy                                = 0x0000000f,
    TA_PERF_SEL_image_sampler_1_input_vgpr_instructions = 0x00000010,
    TA_PERF_SEL_image_sampler_2_input_vgpr_instructions = 0x00000011,
    TA_PERF_SEL_image_sampler_3_input_vgpr_instructions = 0x00000012,
    TA_PERF_SEL_image_sampler_4_input_vgpr_instructions = 0x00000013,
    TA_PERF_SEL_image_sampler_5_input_vgpr_instructions = 0x00000014,
    TA_PERF_SEL_image_sampler_6_input_vgpr_instructions = 0x00000015,
    TA_PERF_SEL_image_sampler_7_input_vgpr_instructions = 0x00000016,
    TA_PERF_SEL_image_sampler_8_input_vgpr_instructions = 0x00000017,
    TA_PERF_SEL_image_sampler_9_input_vgpr_instructions = 0x00000018,
    TA_PERF_SEL_image_sampler_10_input_vgpr_instructions = 0x00000019,
    TA_PERF_SEL_image_sampler_11_input_vgpr_instructions = 0x0000001a,
    TA_PERF_SEL_image_sampler_12_input_vgpr_instructions = 0x0000001b,
    TA_PERF_SEL_image_sampler_has_t_instructions       = 0x0000001c,
    TA_PERF_SEL_image_sampler_has_r_instructions       = 0x0000001d,
    TA_PERF_SEL_image_sampler_has_q_instructions       = 0x0000001e,
    TA_PERF_SEL_total_wavefronts                       = 0x00000020,
    TA_PERF_SEL_gradient_cycles                        = 0x00000021,
    TA_PERF_SEL_walker_cycles                          = 0x00000022,
    TA_PERF_SEL_aligner_cycles                         = 0x00000023,
    TA_PERF_SEL_image_wavefronts                       = 0x00000024,
    TA_PERF_SEL_image_read_wavefronts                  = 0x00000025,
    TA_PERF_SEL_image_store_wavefronts                 = 0x00000026,
    TA_PERF_SEL_image_atomic_wavefronts                = 0x00000027,
    TA_PERF_SEL_image_sampler_total_cycles             = 0x00000028,
    TA_PERF_SEL_image_nosampler_total_cycles           = 0x00000029,
    TA_PERF_SEL_flat_total_cycles                      = 0x0000002a,
    TA_PERF_SEL_bvh_total_cycles                       = 0x0000002b,
    TA_PERF_SEL_buffer_wavefronts                      = 0x0000002c,
    TA_PERF_SEL_buffer_load_wavefronts                 = 0x0000002d,
    TA_PERF_SEL_buffer_store_wavefronts                = 0x0000002e,
    TA_PERF_SEL_buffer_atomic_wavefronts               = 0x0000002f,
    TA_PERF_SEL_image_bvh_wavefronts                   = 0x00000030,
    TA_PERF_SEL_buffer_total_cycles                    = 0x00000031,
    TA_PERF_SEL_buffer_1_address_input_vgpr_instructions = 0x00000032,
    TA_PERF_SEL_buffer_2_address_input_vgpr_instructions = 0x00000033,
    TA_PERF_SEL_buffer_has_index_instructions          = 0x00000034,
    TA_PERF_SEL_buffer_has_offset_instructions         = 0x00000035,
    TA_PERF_SEL_addr_stalled_by_tc_cycles              = 0x00000036,
    TA_PERF_SEL_addr_stalled_by_td_cycles              = 0x00000037,
    TA_PERF_SEL_image_sampler_wavefronts               = 0x00000038,
    TA_PERF_SEL_addresser_stalled_by_aligner_only_cycles = 0x00000039,
    TA_PERF_SEL_addresser_stalled_cycles               = 0x0000003a,
    TA_PERF_SEL_aniso_stalled_by_addresser_only_cycles = 0x0000003b,
    TA_PERF_SEL_aniso_stalled_cycles                   = 0x0000003c,
    TA_PERF_SEL_deriv_stalled_by_aniso_only_cycles     = 0x0000003d,
    TA_PERF_SEL_deriv_stalled_cycles                   = 0x0000003e,
    TA_PERF_SEL_aniso_gt1_cycle_quads                  = 0x0000003f,
    TA_PERF_SEL_color_1_cycle_quads                    = 0x00000040,
    TA_PERF_SEL_color_2_cycle_quads                    = 0x00000041,
    TA_PERF_SEL_color_3_cycle_quads                    = 0x00000042,
    TA_PERF_SEL_mip_1_cycle_quads                      = 0x00000044,
    TA_PERF_SEL_mip_2_cycle_quads                      = 0x00000045,
    TA_PERF_SEL_vol_1_cycle_quads                      = 0x00000046,
    TA_PERF_SEL_vol_2_cycle_quads                      = 0x00000047,
    TA_PERF_SEL_sampler_op_quads                       = 0x00000048,
    TA_PERF_SEL_mipmap_lod_0_samples                   = 0x00000049,
    TA_PERF_SEL_mipmap_lod_1_samples                   = 0x0000004a,
    TA_PERF_SEL_mipmap_lod_2_samples                   = 0x0000004b,
    TA_PERF_SEL_mipmap_lod_3_samples                   = 0x0000004c,
    TA_PERF_SEL_mipmap_lod_4_samples                   = 0x0000004d,
    TA_PERF_SEL_mipmap_lod_5_samples                   = 0x0000004e,
    TA_PERF_SEL_mipmap_lod_6_samples                   = 0x0000004f,
    TA_PERF_SEL_mipmap_lod_7_samples                   = 0x00000050,
    TA_PERF_SEL_mipmap_lod_8_samples                   = 0x00000051,
    TA_PERF_SEL_mipmap_lod_9_samples                   = 0x00000052,
    TA_PERF_SEL_mipmap_lod_10_samples                  = 0x00000053,
    TA_PERF_SEL_mipmap_lod_11_samples                  = 0x00000054,
    TA_PERF_SEL_mipmap_lod_12_samples                  = 0x00000055,
    TA_PERF_SEL_mipmap_lod_13_samples                  = 0x00000056,
    TA_PERF_SEL_mipmap_lod_14_samples                  = 0x00000057,
    TA_PERF_SEL_mipmap_invalid_samples                 = 0x00000058,
    TA_PERF_SEL_aniso_1_cycle_quads                    = 0x00000059,
    TA_PERF_SEL_aniso_2_cycle_quads                    = 0x0000005a,
    TA_PERF_SEL_aniso_4_cycle_quads                    = 0x0000005b,
    TA_PERF_SEL_aniso_6_cycle_quads                    = 0x0000005c,
    TA_PERF_SEL_aniso_8_cycle_quads                    = 0x0000005d,
    TA_PERF_SEL_aniso_10_cycle_quads                   = 0x0000005e,
    TA_PERF_SEL_aniso_12_cycle_quads                   = 0x0000005f,
    TA_PERF_SEL_aniso_14_cycle_quads                   = 0x00000060,
    TA_PERF_SEL_aniso_16_cycle_quads                   = 0x00000061,
    TA_PERF_SEL_store_write_data_input_cycles          = 0x00000062,
    TA_PERF_SEL_store_write_data_output_cycles         = 0x00000063,
    TA_PERF_SEL_flat_wavefronts                        = 0x00000064,
    TA_PERF_SEL_flat_load_wavefronts                   = 0x00000065,
    TA_PERF_SEL_flat_store_wavefronts                  = 0x00000066,
    TA_PERF_SEL_flat_atomic_wavefronts                 = 0x00000067,
    TA_PERF_SEL_flat_1_address_input_vgpr_instructions = 0x00000068,
    TA_PERF_SEL_register_clk_valid_cycles              = 0x00000069,
    TA_PERF_SEL_non_harvestable_clk_enabled_cycles     = 0x0000006a,
    TA_PERF_SEL_harvestable_clk_enabled_cycles         = 0x0000006b,
    TA_PERF_SEL_flat_2_address_input_vgpr_instructions = 0x0000006c,
    TA_PERF_SEL_boundary_non_harvestable_clk_enabled_cycles = 0x0000006d,
    TA_PERF_SEL_boundary_harvestable_clk_enabled_cycles = 0x0000006e,
    TA_PERF_SEL_mipmap_lod_15_samples                  = 0x00000070,
    TA_PERF_SEL_mipmap_lod_16_samples                  = 0x00000071,
    TA_PERF_SEL_store_2_write_data_vgpr_instructions   = 0x00000072,
    TA_PERF_SEL_store_3_write_data_vgpr_instructions   = 0x00000073,
    TA_PERF_SEL_store_4_write_data_vgpr_instructions   = 0x00000074,
    TA_PERF_SEL_store_has_x_instructions               = 0x00000075,
    TA_PERF_SEL_store_has_y_instructions               = 0x00000076,
    TA_PERF_SEL_store_has_z_instructions               = 0x00000077,
    TA_PERF_SEL_store_has_w_instructions               = 0x00000078,
    TA_PERF_SEL_image_nosampler_has_t_instructions     = 0x00000079,
    TA_PERF_SEL_image_nosampler_has_r_instructions     = 0x0000007a,
    TA_PERF_SEL_image_nosampler_has_q_instructions     = 0x0000007b,
    TA_PERF_SEL_image_nosampler_1_address_input_vgpr_instructions = 0x0000007c,
    TA_PERF_SEL_image_nosampler_2_address_input_vgpr_instructions = 0x0000007d,
    TA_PERF_SEL_image_nosampler_3_address_input_vgpr_instructions = 0x0000007e,
    TA_PERF_SEL_image_nosampler_4_address_input_vgpr_instructions = 0x0000007f,
    TA_PERF_SEL_in_busy                                = 0x00000080,
    TA_PERF_SEL_in_fifos_busy                          = 0x00000081,
    TA_PERF_SEL_in_cfifo_busy                          = 0x00000082,
    TA_PERF_SEL_in_qfifo_busy                          = 0x00000083,
    TA_PERF_SEL_in_wfifo_busy                          = 0x00000084,
    TA_PERF_SEL_in_rfifo_busy                          = 0x00000085,
    TA_PERF_SEL_bf_busy                                = 0x00000086,
    TA_PERF_SEL_ns_busy                                = 0x00000087,
    TA_PERF_SEL_smp_busy_ns_idle                       = 0x00000088,
    TA_PERF_SEL_smp_idle_ns_busy                       = 0x00000089,
    TA_PERF_SEL_vmemcmd_cycles                         = 0x00000090,
    TA_PERF_SEL_vmemreq_cycles                         = 0x00000091,
    TA_PERF_SEL_in_waiting_on_req_cycles               = 0x00000092,
    TA_PERF_SEL_in_addr_cycles                         = 0x00000096,
    TA_PERF_SEL_in_data_cycles                         = 0x00000097,
    TA_PERF_SEL_num_of_bvh_valid_64B_HP_box            = 0x0000009a,
    TA_PERF_SEL_num_of_bvh_usernode0                   = 0x0000009b,
    TA_PERF_SEL_num_of_bvh_usernode1                   = 0x0000009c,
    TA_PERF_SEL_num_of_bvh_invalidated_64B_HP_box      = 0x0000009d,
    TA_PERF_SEL_point_sampled_quads                    = 0x000000a0,
    TA_PERF_SEL_atomic_2_write_data_vgpr_instructions  = 0x000000a2,
    TA_PERF_SEL_atomic_4_write_data_vgpr_instructions  = 0x000000a3,
    TA_PERF_SEL_atomic_write_data_input_cycles         = 0x000000a4,
    TA_PERF_SEL_atomic_write_data_output_cycles        = 0x000000a5,
    TA_PERF_SEL_bvh_2_valid_nodes                      = 0x000000a6,
    TA_PERF_SEL_bvh_rt30_1_valid_node                  = 0x000000a7,
    TA_PERF_SEL_bvh_rt30_0_valid_nodes                 = 0x000000a8,
    TA_PERF_SEL_bvh_nodes_2_cycles                     = 0x000000a9,
    TA_PERF_SEL_num_unlit_nodes_ta_opt                 = 0x000000ad,
    TA_PERF_SEL_num_nodes_invalidated_due_to_bad_input = 0x000000ae,
    TA_PERF_SEL_num_nodes_invalidated_due_to_oob       = 0x000000af,
    TA_PERF_SEL_num_of_bvh_valid_first_tri             = 0x000000b0,
    TA_PERF_SEL_num_of_bvh_valid_second_tri            = 0x000000b1,
    TA_PERF_SEL_num_of_bvh_valid_third_tri             = 0x000000b2,
    TA_PERF_SEL_num_of_bvh_valid_fourth_tri            = 0x000000b3,
    TA_PERF_SEL_num_of_bvh_valid_fp16_box              = 0x000000b4,
    TA_PERF_SEL_num_of_bvh_valid_fp32_box              = 0x000000b5,
    TA_PERF_SEL_num_of_bvh_invalidated_first_tri       = 0x000000b6,
    TA_PERF_SEL_num_of_bvh_invalidated_second_tri      = 0x000000b7,
    TA_PERF_SEL_num_of_bvh_invalidated_third_tri       = 0x000000b8,
    TA_PERF_SEL_num_of_bvh_invalidated_fourth_tri      = 0x000000b9,
    TA_PERF_SEL_num_of_bvh_invalidated_fp16_box        = 0x000000ba,
    TA_PERF_SEL_num_of_bvh_invalidated_fp32_box        = 0x000000bb,
    TA_PERF_SEL_image_bvh_8_input_vgpr_instructions    = 0x000000bc,
    TA_PERF_SEL_image_bvh_9_input_vgpr_instructions    = 0x000000bd,
    TA_PERF_SEL_image_bvh_11_input_vgpr_instructions   = 0x000000be,
    TA_PERF_SEL_image_bvh_12_input_vgpr_instructions   = 0x000000bf,
    TA_PERF_SEL_image_sampler_1_op_burst               = 0x000000c0,
    TA_PERF_SEL_image_sampler_2to3_op_burst            = 0x000000c1,
    TA_PERF_SEL_image_sampler_4to7_op_burst            = 0x000000c2,
    TA_PERF_SEL_image_sampler_ge8_op_burst             = 0x000000c3,
    TA_PERF_SEL_image_linked_1_op_burst                = 0x000000c4,
    TA_PERF_SEL_image_linked_2to3_op_burst             = 0x000000c5,
    TA_PERF_SEL_image_linked_4to7_op_burst             = 0x000000c6,
    TA_PERF_SEL_image_linked_ge8_op_burst              = 0x000000c7,
    TA_PERF_SEL_image_bvh_1_op_burst                   = 0x000000c8,
    TA_PERF_SEL_image_bvh_2to3_op_burst                = 0x000000c9,
    TA_PERF_SEL_image_bvh_4to7_op_burst                = 0x000000ca,
    TA_PERF_SEL_image_bvh_ge8_op_burst                 = 0x000000cb,
    TA_PERF_SEL_image_nosampler_1_op_burst             = 0x000000cc,
    TA_PERF_SEL_image_nosampler_2to3_op_burst          = 0x000000cd,
    TA_PERF_SEL_image_nosampler_4to31_op_burst         = 0x000000ce,
    TA_PERF_SEL_image_nosampler_ge32_op_burst          = 0x000000cf,
    TA_PERF_SEL_buffer_flat_1_op_burst                 = 0x000000d0,
    TA_PERF_SEL_buffer_flat_2to3_op_burst              = 0x000000d1,
    TA_PERF_SEL_buffer_flat_4to31_op_burst             = 0x000000d2,
    TA_PERF_SEL_buffer_flat_ge32_op_burst              = 0x000000d3,
    TA_PERF_SEL_write_1_op_burst                       = 0x000000d4,
    TA_PERF_SEL_write_2to3_op_burst                    = 0x000000d5,
    TA_PERF_SEL_write_4to31_op_burst                   = 0x000000d6,
    TA_PERF_SEL_write_ge32_op_burst                    = 0x000000d7,
    TA_PERF_SEL_ibubble_1_cycle_burst                  = 0x000000d8,
    TA_PERF_SEL_ibubble_2to3_cycle_burst               = 0x000000d9,
    TA_PERF_SEL_ibubble_4to15_cycle_burst              = 0x000000da,
    TA_PERF_SEL_ibubble_16to31_cycle_burst             = 0x000000db,
    TA_PERF_SEL_ibubble_32to63_cycle_burst             = 0x000000dc,
    TA_PERF_SEL_ibubble_ge64_cycle_burst               = 0x000000dd,
    TA_PERF_SEL_num_of_bvh_valid_instance              = 0x000000de,
    TA_PERF_SEL_num_of_bvh_invalidated_instance        = 0x000000df,
    TA_PERF_SEL_sampler_clk_valid_cycles               = 0x000000e0,
    TA_PERF_SEL_nonsampler_clk_valid_cycles            = 0x000000e1,
    TA_PERF_SEL_buffer_flat_clk_valid_cycles           = 0x000000e2,
    TA_PERF_SEL_write_data_clk_valid_cycles            = 0x000000e3,
    TA_PERF_SEL_gradient_clk_valid_cycles              = 0x000000e4,
    TA_PERF_SEL_lod_aniso_clk_valid_cycles             = 0x000000e5,
    TA_PERF_SEL_sampler_addressing_clk_valid_cycles    = 0x000000e6,
    TA_PERF_SEL_sync_sampler_sstate_fifo_clk_valid_cycles = 0x000000e7,
    TA_PERF_SEL_sync_sampler_cstate_fifo_clk_valid_cycles = 0x000000e8,
    TA_PERF_SEL_sync_nonsampler_fifo_clk_valid_cycles  = 0x000000e9,
    TA_PERF_SEL_aligner_clk_valid_cycles               = 0x000000ea,
    TA_PERF_SEL_tcreq_clk_valid_cycles                 = 0x000000eb,
    TA_PERF_SEL_load_wavefronts                        = 0x000000f0,
    TA_PERF_SEL_store_wavefronts                       = 0x000000f1,
    TA_PERF_SEL_atomic_wavefronts                      = 0x000000f2,
    TA_PERF_SEL_total_threads                          = 0x000000f3,
    TA_PERF_SEL_load_threads                           = 0x000000f4,
    TA_PERF_SEL_store_threads                          = 0x000000f5,
    TA_PERF_SEL_atomic_threads                         = 0x000000f6,
    TA_PERF_SEL_ta_active                              = 0x000000f7,
    TA_PERF_SEL_ta_blocked                             = 0x000000f8,
    TA_PERF_SEL_image_sample_point                     = 0x000000f9,
    TA_PERF_SEL_image_sample_bilinear                  = 0x000000fa,
    TA_PERF_SEL_image_sample_trilinear                 = 0x000000fb,
    TA_PERF_SEL_image_sample_aniso                     = 0x000000fc,
    TA_PERF_SEL_image_sample_aniso_quads               = 0x000000fd,
    TA_PERF_SEL_image_sample_aniso_miplevel_per_quad   = 0x000000fe,
} TA_PERFCOUNT_SEL;

constexpr unsigned int MaxTaPerfcountSel               = TA_PERF_SEL_image_sample_aniso_miplevel_per_quad;

typedef enum TA_TC_ADDR_MODES {
    TA_TC_ADDR_MODE_DEFAULT                            = 0x00000000,
    TA_TC_ADDR_MODE_COMP0                              = 0x00000001,
    TA_TC_ADDR_MODE_COMP1                              = 0x00000002,
    TA_TC_ADDR_MODE_COMP2                              = 0x00000003,
    TA_TC_ADDR_MODE_COMP3                              = 0x00000004,
    TA_TC_ADDR_MODE_UNALIGNED                          = 0x00000005,
    TA_TC_ADDR_MODE_BORDER_COLOR                       = 0x00000006,
} TA_TC_ADDR_MODES;

typedef enum TA_TC_REQ_MODES {
    TA_TC_REQ_MODE_BORDER                              = 0x00000000,
    TA_TC_REQ_MODE_TEX2                                = 0x00000001,
    TA_TC_REQ_MODE_TEX1                                = 0x00000002,
    TA_TC_REQ_MODE_TEX0                                = 0x00000003,
    TA_TC_REQ_MODE_NORMAL                              = 0x00000004,
    TA_TC_REQ_MODE_DWORD                               = 0x00000005,
    TA_TC_REQ_MODE_BYTE                                = 0x00000006,
    TA_TC_REQ_MODE_BYTE_NV                             = 0x00000007,
} TA_TC_REQ_MODES;

typedef enum TCC_MTYPE {
    MTYPE_NC                                           = 0x00000000,
    MTYPE_WC                                           = 0x00000001,
    MTYPE_CC                                           = 0x00000002,
} TCC_MTYPE;

typedef enum TCP_CACHE_POLICIES {
    TCP_CACHE_POLICY_MISS_LRU                          = 0x00000000,
    TCP_CACHE_POLICY_MISS_EVICT                        = 0x00000001,
    TCP_CACHE_POLICY_HIT_LRU                           = 0x00000002,
    TCP_CACHE_POLICY_HIT_INVAL                         = 0x00000003,
    TCP_CACHE_POLICY_MISS_INVAL                        = 0x00000004,
} TCP_CACHE_POLICIES;

typedef enum TCP_CACHE_STORE_POLICIES {
    TCP_CACHE_STORE_POLICY_WT_LRU                      = 0x00000000,
    TCP_CACHE_STORE_POLICY_WT_EVICT                    = 0x00000001,
} TCP_CACHE_STORE_POLICIES;

typedef enum TCP_COMPRESSION_BYPASS {
    TCP_COMPRESSION_BYPASS_DIS                         = 0x00000000,
    TCP_COMPRESSION_BYPASS_EN                          = 0x00000001,
} TCP_COMPRESSION_BYPASS;

typedef enum TCP_COMPRESSION_OVERRIDE {
    TCP_COMPRESSION_OVERRIDE_DIS                       = 0x00000000,
    TCP_COMPRESSION_OVERRIDE_EN                        = 0x00000001,
} TCP_COMPRESSION_OVERRIDE;

typedef enum TCP_DSM_DATA_SEL {
    TCP_DSM_DISABLE                                    = 0x00000000,
    TCP_DSM_SEL0                                       = 0x00000001,
    TCP_DSM_SEL1                                       = 0x00000002,
    TCP_DSM_SEL_BOTH                                   = 0x00000003,
} TCP_DSM_DATA_SEL;

typedef enum TCP_DSM_INJECT_SEL {
    TCP_DSM_INJECT_SEL0                                = 0x00000000,
    TCP_DSM_INJECT_SEL1                                = 0x00000001,
    TCP_DSM_INJECT_SEL2                                = 0x00000002,
    TCP_DSM_INJECT_SEL3                                = 0x00000003,
} TCP_DSM_INJECT_SEL;

typedef enum TCP_DSM_SINGLE_WRITE {
    TCP_DSM_SINGLE_WRITE_DIS                           = 0x00000000,
    TCP_DSM_SINGLE_WRITE_EN                            = 0x00000001,
} TCP_DSM_SINGLE_WRITE;

typedef enum TCP_OPCODE_TYPE {
    TCP_OPCODE_READ                                    = 0x00000000,
    TCP_OPCODE_WRITE                                   = 0x00000001,
    TCP_OPCODE_ATOMIC                                  = 0x00000002,
    TCP_OPCODE_INV                                     = 0x00000003,
    TCP_OPCODE_ATOMIC_CMPSWAP                          = 0x00000004,
    TCP_OPCODE_SAMPLER                                 = 0x00000005,
    TCP_OPCODE_LOAD                                    = 0x00000006,
    TCP_OPCODE_GATHERH                                 = 0x00000007,
} TCP_OPCODE_TYPE;

typedef enum TCP_PERFCOUNT_SELECT {
    TCP_PERF_SEL_GATE_EN1                              = 0x00000000,
    TCP_PERF_SEL_GATE_EN2                              = 0x00000001,
    TCP_PERF_SEL_TA_REQ                                = 0x00000002,
    TCP_PERF_SEL_TA_REQ_STATE_READ                     = 0x00000003,
    TCP_PERF_SEL_TA_REQ_READ                           = 0x00000004,
    TCP_PERF_SEL_TA_REQ_WRITE                          = 0x00000005,
    TCP_PERF_SEL_TA_REQ_ATOMIC_WITH_RET                = 0x00000006,
    TCP_PERF_SEL_TA_REQ_ATOMIC_WITHOUT_RET             = 0x00000007,
    TCP_PERF_SEL_TA_REQ_GL0_INV                        = 0x00000008,
    TCP_PERF_SEL_REQ                                   = 0x00000009,
    TCP_PERF_SEL_REQ_READ                              = 0x0000000a,
    TCP_PERF_SEL_REQ_READ_HIT_INVAL                    = 0x0000000b,
    TCP_PERF_SEL_REQ_READ_HIT_LRU                      = 0x0000000c,
    TCP_PERF_SEL_REQ_READ_MISS_INVAL                   = 0x0000000d,
    TCP_PERF_SEL_REQ_WRITE                             = 0x0000000e,
    TCP_PERF_SEL_REQ_WRITE_MISS_EVICT                  = 0x0000000f,
    TCP_PERF_SEL_REQ_NON_READ                          = 0x00000010,
    TCP_PERF_SEL_REQ_MISS                              = 0x00000011,
    TCP_PERF_SEL_REQ_TAGBANK0_SET0                     = 0x00000012,
    TCP_PERF_SEL_REQ_TAGBANK0_SET1                     = 0x00000013,
    TCP_PERF_SEL_REQ_TAGBANK1_SET0                     = 0x00000014,
    TCP_PERF_SEL_REQ_TAGBANK1_SET1                     = 0x00000015,
    TCP_PERF_SEL_REQ_TAGBANK2_SET0                     = 0x00000016,
    TCP_PERF_SEL_REQ_TAGBANK2_SET1                     = 0x00000017,
    TCP_PERF_SEL_REQ_TAGBANK3_SET0                     = 0x00000018,
    TCP_PERF_SEL_REQ_TAGBANK3_SET1                     = 0x00000019,
    TCP_PERF_SEL_REQ_MISS_TAGBANK0                     = 0x0000001a,
    TCP_PERF_SEL_REQ_MISS_TAGBANK1                     = 0x0000001b,
    TCP_PERF_SEL_REQ_MISS_TAGBANK2                     = 0x0000001c,
    TCP_PERF_SEL_REQ_MISS_TAGBANK3                     = 0x0000001d,
    TCP_PERF_SEL_GL1_REQ_READ                          = 0x0000001e,
    TCP_PERF_SEL_GL1_REQ_READ_128B                     = 0x0000001f,
    TCP_PERF_SEL_GL1_REQ_READ_64B                      = 0x00000020,
    TCP_PERF_SEL_GL1_REQ_WRITE                         = 0x00000021,
    TCP_PERF_SEL_GL1_REQ_ATOMIC_WITH_RET               = 0x00000022,
    TCP_PERF_SEL_GL1_REQ_ATOMIC_WITHOUT_RET            = 0x00000023,
    TCP_PERF_SEL_GL1_READ_LATENCY                      = 0x00000024,
    TCP_PERF_SEL_GL1_WRITE_LATENCY                     = 0x00000025,
    TCP_PERF_SEL_TCP_LATENCY                           = 0x00000026,
    TCP_PERF_SEL_TCP_TA_REQ_STALL                      = 0x00000027,
    TCP_PERF_SEL_TA_TCP_REQ_STARVE                     = 0x00000028,
    TCP_PERF_SEL_DATA_FIFO_STALL                       = 0x00000029,
    TCP_PERF_SEL_LOD_STALL                             = 0x0000002a,
    TCP_PERF_SEL_POWER_STALL                           = 0x0000002b,
    TCP_PERF_SEL_ALLOC_STALL                           = 0x0000002c,
    TCP_PERF_SEL_READ_TAGCONFLICT_STALL                = 0x0000002e,
    TCP_PERF_SEL_WRITE_TAGCONFLICT_STALL               = 0x0000002f,
    TCP_PERF_SEL_ATOMIC_TAGCONFLICT_STALL              = 0x00000030,
    TCP_PERF_SEL_LFIFO_STALL                           = 0x00000031,
    TCP_PERF_SEL_MEM_REQ_FIFO_STALL                    = 0x00000032,
    TCP_PERF_SEL_GL1_TCP_BACK_PRESSURE                 = 0x00000033,
    TCP_PERF_SEL_GL1_TCP_RDRET_STALL                   = 0x00000034,
    TCP_PERF_SEL_GL1_GRANT_READ_STALL                  = 0x00000035,
    TCP_PERF_SEL_GL1_PENDING_STALL                     = 0x00000036,
    TCP_PERF_SEL_TD_DATA_CYCLE_STALL                   = 0x00000037,
    TCP_PERF_SEL_COMP_TEX_LOAD_STALL                   = 0x00000038,
    TCP_PERF_SEL_READ_DATACONFLICT_STALL               = 0x00000039,
    TCP_PERF_SEL_WRITE_DATACONFLICT_STALL              = 0x0000003a,
    TCP_PERF_SEL_TD_TCP_STALL                          = 0x0000003b,
    TCP_PERF_SEL_TA_REQ_BUFFERNOP                      = 0x0000003c,
    TCP_PERF_SEL_WRITECOMBINE_ENDCLAUSE                = 0x0000003d,
    TCP_PERF_SEL_TAGFAKE_EOW                           = 0x0000003e,
    TCP_PERF_SEL_REQ_TAG_MATCH_AND_NOT_VALID           = 0x0000003f,
    TCP_PERF_SEL_BURST_BIN_WRITECOMBINE_0              = 0x00000040,
    TCP_PERF_SEL_BURST_BIN_WRITECOMBINE_1to2           = 0x00000041,
    TCP_PERF_SEL_BURST_BIN_WRITECOMBINE_3to4           = 0x00000042,
    TCP_PERF_SEL_BURST_BIN_WRITECOMBINE_5to8           = 0x00000043,
    TCP_PERF_SEL_BURST_BIN_WRITECOMBINE_9to16          = 0x00000044,
    TCP_PERF_SEL_BURST_BIN_READHIT_0                   = 0x00000046,
    TCP_PERF_SEL_BURST_BIN_READHIT_1                   = 0x00000047,
    TCP_PERF_SEL_BURST_BIN_READHIT_2to4                = 0x00000048,
    TCP_PERF_SEL_BURST_BIN_READHIT_5to8                = 0x00000049,
    TCP_PERF_SEL_BURST_BIN_READHIT_9to16               = 0x0000004a,
    TCP_PERF_SEL_BURST_BIN_READHIT_gt16                = 0x0000004b,
    TCP_PERF_SEL_TA_TC_REQ_EN_SUM                      = 0x0000004c,
    TCP_PERF_SEL_GL1_REQ_LU                            = 0x0000004d,
    TCP_PERF_SEL_REQ_TAG_MATCH_AND_LU_INVALIDATE       = 0x0000004e,
    TCP_PERF_SEL_GL1_REQ_READ_COMPRESSION_KEY          = 0x00000051,
    TCP_PERF_SEL_GL1_REQ_FORCE_TO_DECOMPRESS           = 0x00000052,
    TCP_PERF_SEL_LFIFO_MAIN_WRITES                     = 0x00000053,
    TCP_PERF_SEL_LFIFO_EXT_WRITES                      = 0x00000054,
    TCP_PERF_SEL_TC_TD_DATA_WRITES                     = 0x00000055,
    TCP_PERF_SEL_VEC32_REQ                             = 0x00000056,
    TCP_PERF_SEL_VEC32_REQ_READ                        = 0x00000057,
    TCP_PERF_SEL_VEC32_REQ_WRITE                       = 0x00000058,
    TCP_PERF_SEL_VEC32_REQ_ATOMIC                      = 0x00000059,
    TCP_PERF_SEL_SECTORS                               = 0x0000005a,
    TCP_PERF_SEL_SECTORS_READ                          = 0x0000005b,
    TCP_PERF_SEL_SECTORS_WRITE                         = 0x0000005c,
    TCP_PERF_SEL_SECTOR_MISS                           = 0x0000005d,
    TCP_PERF_SEL_SECTOR_MISS_DUPLICATE                 = 0x0000005e,
    TCP_PERF_SEL_TOTAL_CONFLICT_STALLS                 = 0x0000005f,
    TCP_PERF_SEL_TCP_BUSY                              = 0x00000060,
    TCP_PERF_SEL_REQ_READ_MISS_LRU                     = 0x00000061,
    TCP_PERF_SEL_REQ_WRITE_MISS_LRU                    = 0x00000062,
    TCP_PERF_SEL_REQ_WRITE_MISS_INVAL                  = 0x00000063,
} TCP_PERFCOUNT_SELECT;

constexpr unsigned int MaxTcpPerfcountSelect           = TCP_PERF_SEL_REQ_WRITE_MISS_INVAL;

typedef enum TCP_WATCH_MODES {
    TCP_WATCH_MODE_READ                                = 0x00000000,
    TCP_WATCH_MODE_NONREAD                             = 0x00000001,
    TCP_WATCH_MODE_ATOMIC                              = 0x00000002,
    TCP_WATCH_MODE_ALL                                 = 0x00000003,
} TCP_WATCH_MODES;

typedef enum TCP_WRITE_COMPRESSION_DISABLE {
    TCP_WRITE_COMPRESSION_DISABLE_DIS                  = 0x00000000,
    TCP_WRITE_COMPRESSION_DISABLE_EN                   = 0x00000001,
} TCP_WRITE_COMPRESSION_DISABLE;

typedef enum TC_EA_CID {
    TC_EA_CID_RT                                       = 0x00000000,
    TC_EA_CID_FMASK                                    = 0x00000001,
    TC_EA_CID_DCC                                      = 0x00000002,
    TC_EA_CID_TCPMETA                                  = 0x00000003,
    TC_EA_CID_Z                                        = 0x00000004,
    TC_EA_CID_STENCIL                                  = 0x00000005,
    TC_EA_CID_HTILE                                    = 0x00000006,
    TC_EA_CID_MISC                                     = 0x00000007,
    TC_EA_CID_TCP                                      = 0x00000008,
    TC_EA_CID_SQC                                      = 0x00000009,
    TC_EA_CID_CPF                                      = 0x0000000a,
    TC_EA_CID_CPG                                      = 0x0000000b,
    TC_EA_CID_IA                                       = 0x0000000c,
    TC_EA_CID_WD                                       = 0x0000000d,
    TC_EA_CID_PA                                       = 0x0000000e,
    TC_EA_CID_UTCL2_TPI                                = 0x0000000f,
} TC_EA_CID;

typedef enum TC_MICRO_TILE_MODE {
    MICRO_TILE_MODE_LINEAR                             = 0x00000000,
    MICRO_TILE_MODE_RENDER_TARGET                      = 0x00000001,
    MICRO_TILE_MODE_STD_2D                             = 0x00000002,
    MICRO_TILE_MODE_STD_3D                             = 0x00000003,
    MICRO_TILE_MODE_DISPLAY_2D                         = 0x00000004,
    MICRO_TILE_MODE_DISPLAY_3D                         = 0x00000005,
    MICRO_TILE_MODE_Z                                  = 0x00000006,
    MICRO_TILE_MODE_Z_VAR                              = 0x00000007,
} TC_MICRO_TILE_MODE;

typedef enum TC_NACKS {
    TC_NACK_NO_FAULT                                   = 0x00000000,
    TC_NACK_PAGE_FAULT                                 = 0x00000001,
    TC_NACK_PROTECTION_FAULT                           = 0x00000002,
    TC_NACK_DATA_ERROR                                 = 0x00000003,
} TC_NACKS;

typedef enum TC_OP {
    TC_OP_READ                                         = 0x00000000,
    TC_OP_ATOMIC_FCMPSWAP_RTN_32                       = 0x00000001,
    TC_OP_ATOMIC_FMIN_RTN_32                           = 0x00000002,
    TC_OP_ATOMIC_FMAX_RTN_32                           = 0x00000003,
    TC_OP_RESERVED_FOP_RTN_32_0                        = 0x00000004,
    TC_OP_RESERVED_FADD_RTN_32                         = 0x00000005,
    TC_OP_RESERVED_FOP_RTN_32_2                        = 0x00000006,
    TC_OP_ATOMIC_SWAP_RTN_32                           = 0x00000007,
    TC_OP_ATOMIC_CMPSWAP_RTN_32                        = 0x00000008,
    TC_OP_ATOMIC_FCMPSWAP_FLUSH_DENORM_RTN_32          = 0x00000009,
    TC_OP_ATOMIC_FMIN_FLUSH_DENORM_RTN_32              = 0x0000000a,
    TC_OP_ATOMIC_FMAX_FLUSH_DENORM_RTN_32              = 0x0000000b,
    TC_OP_PROBE_FILTER                                 = 0x0000000c,
    TC_OP_ATOMIC_FADD_FLUSH_DENORM_RTN_32              = 0x0000000d,
    TC_OP_RESERVED_FOP_FLUSH_DENORM_RTN_32_2           = 0x0000000e,
    TC_OP_ATOMIC_ADD_RTN_32                            = 0x0000000f,
    TC_OP_ATOMIC_SUB_RTN_32                            = 0x00000010,
    TC_OP_ATOMIC_SMIN_RTN_32                           = 0x00000011,
    TC_OP_ATOMIC_UMIN_RTN_32                           = 0x00000012,
    TC_OP_ATOMIC_SMAX_RTN_32                           = 0x00000013,
    TC_OP_ATOMIC_UMAX_RTN_32                           = 0x00000014,
    TC_OP_ATOMIC_AND_RTN_32                            = 0x00000015,
    TC_OP_ATOMIC_OR_RTN_32                             = 0x00000016,
    TC_OP_ATOMIC_XOR_RTN_32                            = 0x00000017,
    TC_OP_ATOMIC_INC_RTN_32                            = 0x00000018,
    TC_OP_ATOMIC_DEC_RTN_32                            = 0x00000019,
    TC_OP_WBINVL1_VOL                                  = 0x0000001a,
    TC_OP_WBINVL1_SD                                   = 0x0000001b,
    TC_OP_RESERVED_NON_FLOAT_RTN_32_0                  = 0x0000001c,
    TC_OP_RESERVED_NON_FLOAT_RTN_32_1                  = 0x0000001d,
    TC_OP_RESERVED_NON_FLOAT_RTN_32_2                  = 0x0000001e,
    TC_OP_RESERVED_NON_FLOAT_RTN_32_3                  = 0x0000001f,
    TC_OP_WRITE                                        = 0x00000020,
    TC_OP_ATOMIC_FCMPSWAP_RTN_64                       = 0x00000021,
    TC_OP_ATOMIC_FMIN_RTN_64                           = 0x00000022,
    TC_OP_ATOMIC_FMAX_RTN_64                           = 0x00000023,
    TC_OP_RESERVED_FOP_RTN_64_0                        = 0x00000024,
    TC_OP_RESERVED_FOP_RTN_64_1                        = 0x00000025,
    TC_OP_RESERVED_FOP_RTN_64_2                        = 0x00000026,
    TC_OP_ATOMIC_SWAP_RTN_64                           = 0x00000027,
    TC_OP_ATOMIC_CMPSWAP_RTN_64                        = 0x00000028,
    TC_OP_ATOMIC_FCMPSWAP_FLUSH_DENORM_RTN_64          = 0x00000029,
    TC_OP_ATOMIC_FMIN_FLUSH_DENORM_RTN_64              = 0x0000002a,
    TC_OP_ATOMIC_FMAX_FLUSH_DENORM_RTN_64              = 0x0000002b,
    TC_OP_WBINVL2_SD                                   = 0x0000002c,
    TC_OP_RESERVED_FOP_FLUSH_DENORM_RTN_64_0           = 0x0000002d,
    TC_OP_RESERVED_FOP_FLUSH_DENORM_RTN_64_1           = 0x0000002e,
    TC_OP_ATOMIC_ADD_RTN_64                            = 0x0000002f,
    TC_OP_ATOMIC_SUB_RTN_64                            = 0x00000030,
    TC_OP_ATOMIC_SMIN_RTN_64                           = 0x00000031,
    TC_OP_ATOMIC_UMIN_RTN_64                           = 0x00000032,
    TC_OP_ATOMIC_SMAX_RTN_64                           = 0x00000033,
    TC_OP_ATOMIC_UMAX_RTN_64                           = 0x00000034,
    TC_OP_ATOMIC_AND_RTN_64                            = 0x00000035,
    TC_OP_ATOMIC_OR_RTN_64                             = 0x00000036,
    TC_OP_ATOMIC_XOR_RTN_64                            = 0x00000037,
    TC_OP_ATOMIC_INC_RTN_64                            = 0x00000038,
    TC_OP_ATOMIC_DEC_RTN_64                            = 0x00000039,
    TC_OP_WBL2_NC                                      = 0x0000003a,
    TC_OP_WBL2_WC                                      = 0x0000003b,
    TC_OP_RESERVED_NON_FLOAT_RTN_64_1                  = 0x0000003c,
    TC_OP_RESERVED_NON_FLOAT_RTN_64_2                  = 0x0000003d,
    TC_OP_RESERVED_NON_FLOAT_RTN_64_3                  = 0x0000003e,
    TC_OP_RESERVED_NON_FLOAT_RTN_64_4                  = 0x0000003f,
    TC_OP_WBINVL1                                      = 0x00000040,
    TC_OP_ATOMIC_FCMPSWAP_32                           = 0x00000041,
    TC_OP_ATOMIC_FMIN_32                               = 0x00000042,
    TC_OP_ATOMIC_FMAX_32                               = 0x00000043,
    TC_OP_RESERVED_FOP_32_0                            = 0x00000044,
    TC_OP_RESERVED_FADD_32                             = 0x00000045,
    TC_OP_RESERVED_FOP_32_2                            = 0x00000046,
    TC_OP_ATOMIC_SWAP_32                               = 0x00000047,
    TC_OP_ATOMIC_CMPSWAP_32                            = 0x00000048,
    TC_OP_ATOMIC_FCMPSWAP_FLUSH_DENORM_32              = 0x00000049,
    TC_OP_ATOMIC_FMIN_FLUSH_DENORM_32                  = 0x0000004a,
    TC_OP_ATOMIC_FMAX_FLUSH_DENORM_32                  = 0x0000004b,
    TC_OP_INV_METADATA                                 = 0x0000004c,
    TC_OP_ATOMIC_FADD_FLUSH_DENORM_32                  = 0x0000004d,
    TC_OP_RESERVED_FOP_FLUSH_DENORM_32_2               = 0x0000004e,
    TC_OP_ATOMIC_ADD_32                                = 0x0000004f,
    TC_OP_ATOMIC_SUB_32                                = 0x00000050,
    TC_OP_ATOMIC_SMIN_32                               = 0x00000051,
    TC_OP_ATOMIC_UMIN_32                               = 0x00000052,
    TC_OP_ATOMIC_SMAX_32                               = 0x00000053,
    TC_OP_ATOMIC_UMAX_32                               = 0x00000054,
    TC_OP_ATOMIC_AND_32                                = 0x00000055,
    TC_OP_ATOMIC_OR_32                                 = 0x00000056,
    TC_OP_ATOMIC_XOR_32                                = 0x00000057,
    TC_OP_ATOMIC_INC_32                                = 0x00000058,
    TC_OP_ATOMIC_DEC_32                                = 0x00000059,
    TC_OP_INVL2_NC                                     = 0x0000005a,
    TC_OP_NOP_RTN0                                     = 0x0000005b,
    TC_OP_RESERVED_NON_FLOAT_32_1                      = 0x0000005c,
    TC_OP_RESERVED_NON_FLOAT_32_2                      = 0x0000005d,
    TC_OP_RESERVED_NON_FLOAT_32_3                      = 0x0000005e,
    TC_OP_RESERVED_NON_FLOAT_32_4                      = 0x0000005f,
    TC_OP_WBINVL2                                      = 0x00000060,
    TC_OP_ATOMIC_FCMPSWAP_64                           = 0x00000061,
    TC_OP_ATOMIC_FMIN_64                               = 0x00000062,
    TC_OP_ATOMIC_FMAX_64                               = 0x00000063,
    TC_OP_RESERVED_FOP_64_0                            = 0x00000064,
    TC_OP_RESERVED_FOP_64_1                            = 0x00000065,
    TC_OP_RESERVED_FOP_64_2                            = 0x00000066,
    TC_OP_ATOMIC_SWAP_64                               = 0x00000067,
    TC_OP_ATOMIC_CMPSWAP_64                            = 0x00000068,
    TC_OP_ATOMIC_FCMPSWAP_FLUSH_DENORM_64              = 0x00000069,
    TC_OP_ATOMIC_FMIN_FLUSH_DENORM_64                  = 0x0000006a,
    TC_OP_ATOMIC_FMAX_FLUSH_DENORM_64                  = 0x0000006b,
    TC_OP_RESERVED_FOP_FLUSH_DENORM_64_0               = 0x0000006c,
    TC_OP_RESERVED_FOP_FLUSH_DENORM_64_1               = 0x0000006d,
    TC_OP_RESERVED_FOP_FLUSH_DENORM_64_2               = 0x0000006e,
    TC_OP_ATOMIC_ADD_64                                = 0x0000006f,
    TC_OP_ATOMIC_SUB_64                                = 0x00000070,
    TC_OP_ATOMIC_SMIN_64                               = 0x00000071,
    TC_OP_ATOMIC_UMIN_64                               = 0x00000072,
    TC_OP_ATOMIC_SMAX_64                               = 0x00000073,
    TC_OP_ATOMIC_UMAX_64                               = 0x00000074,
    TC_OP_ATOMIC_AND_64                                = 0x00000075,
    TC_OP_ATOMIC_OR_64                                 = 0x00000076,
    TC_OP_ATOMIC_XOR_64                                = 0x00000077,
    TC_OP_ATOMIC_INC_64                                = 0x00000078,
    TC_OP_ATOMIC_DEC_64                                = 0x00000079,
    TC_OP_WBINVL2_NC                                   = 0x0000007a,
    TC_OP_NOP_ACK                                      = 0x0000007b,
    TC_OP_RESERVED_NON_FLOAT_64_1                      = 0x0000007c,
    TC_OP_RESERVED_NON_FLOAT_64_2                      = 0x0000007d,
    TC_OP_RESERVED_NON_FLOAT_64_3                      = 0x0000007e,
    TC_OP_RESERVED_NON_FLOAT_64_4                      = 0x0000007f,
} TC_OP;

typedef enum TC_OP_MASKS {
    TC_OP_MASK_FLUSH_DENROM                            = 0x00000008,
    TC_OP_MASK_64                                      = 0x00000020,
    TC_OP_MASK_NO_RTN                                  = 0x00000040,
} TC_OP_MASKS;

typedef enum TD_PERFCOUNT_SEL {
    TD_PERF_SEL_none                                   = 0x00000000,
    TD_PERF_SEL_td_busy                                = 0x00000001,
    TD_PERF_SEL_input_busy                             = 0x00000002,
    TD_PERF_SEL_sampler_lerp_busy                      = 0x00000003,
    TD_PERF_SEL_sampler_out_busy                       = 0x00000004,
    TD_PERF_SEL_nofilter_busy                          = 0x00000005,
    TD_PERF_SEL_ray_tracing_bvh4_busy                  = 0x00000006,
    TD_PERF_SEL_sampler_core_sclk_en                   = 0x00000007,
    TD_PERF_SEL_sampler_preformatter_sclk_en           = 0x00000008,
    TD_PERF_SEL_sampler_bilerp_sclk_en                 = 0x00000009,
    TD_PERF_SEL_sampler_bypass_sclk_en                 = 0x0000000a,
    TD_PERF_SEL_sampler_minmax_sclk_en                 = 0x0000000b,
    TD_PERF_SEL_sampler_accum_sclk_en                  = 0x0000000c,
    TD_PERF_SEL_sampler_format_flt_sclk_en             = 0x0000000d,
    TD_PERF_SEL_sampler_format_fxdpt_sclk_en           = 0x0000000e,
    TD_PERF_SEL_sampler_out_sclk_en                    = 0x0000000f,
    TD_PERF_SEL_nofilter_sclk_en                       = 0x00000010,
    TD_PERF_SEL_nofilter_d32_sclk_en                   = 0x00000011,
    TD_PERF_SEL_nofilter_d16_sclk_en                   = 0x00000012,
    TD_PERF_SEL_ray_tracing_bvh4_sclk_en               = 0x00000016,
    TD_PERF_SEL_ray_tracing_bvh4_ip_sclk_en_slot0      = 0x00000017,
    TD_PERF_SEL_ray_tracing_bvh4_box_sclk_en_slot0     = 0x00000018,
    TD_PERF_SEL_ray_tracing_bvh4_tri_sclk_en_slot0     = 0x00000019,
    TD_PERF_SEL_sampler_sclk_on_nofilter_sclk_off      = 0x0000001a,
    TD_PERF_SEL_nofilter_sclk_on_sampler_sclk_off      = 0x0000001b,
    TD_PERF_SEL_all_pipes_sclk_on_at_same_time         = 0x0000001c,
    TD_PERF_SEL_sampler_and_nofilter_sclk_on_bvh4_sclk_off = 0x0000001d,
    TD_PERF_SEL_sampler_and_bvh4_sclk_on_nofilter_sclk_off = 0x0000001e,
    TD_PERF_SEL_nofilter_and_bvh4_sclk_on_sampler_sclk_off = 0x0000001f,
    TD_PERF_SEL_core_state_ram_max_cnt                 = 0x00000020,
    TD_PERF_SEL_core_state_rams_read                   = 0x00000021,
    TD_PERF_SEL_weight_data_rams_read                  = 0x00000022,
    TD_PERF_SEL_reference_data_rams_read               = 0x00000023,
    TD_PERF_SEL_tc_td_ram_fifo_full                    = 0x00000024,
    TD_PERF_SEL_tc_td_ram_fifo_max_cnt                 = 0x00000025,
    TD_PERF_SEL_tc_td_data_fifo_full                   = 0x00000026,
    TD_PERF_SEL_input_state_fifo_full                  = 0x00000027,
    TD_PERF_SEL_ta_data_stall                          = 0x00000028,
    TD_PERF_SEL_tc_data_stall                          = 0x00000029,
    TD_PERF_SEL_tc_ram_stall                           = 0x0000002a,
    TD_PERF_SEL_lds_stall                              = 0x0000002b,
    TD_PERF_SEL_sampler_pkr_full                       = 0x0000002c,
    TD_PERF_SEL_sampler_pkr_full_due_to_arb            = 0x0000002d,
    TD_PERF_SEL_nofilter_pkr_full                      = 0x0000002e,
    TD_PERF_SEL_nofilter_pkr_full_due_to_arb           = 0x0000002f,
    TD_PERF_SEL_ray_tracing_bvh4_pkr_full_slot0        = 0x00000030,
    TD_PERF_SEL_ray_tracing_bvh4_pkr_full_due_to_arb_slot0 = 0x00000031,
    TD_PERF_SEL_gather4_instr                          = 0x00000032,
    TD_PERF_SEL_gather4h_instr                         = 0x00000033,
    TD_PERF_SEL_getlod_instr                           = 0x00000034,
    TD_PERF_SEL_sample_instr                           = 0x00000036,
    TD_PERF_SEL_sample_c_instr                         = 0x00000037,
    TD_PERF_SEL_load_instr                             = 0x00000038,
    TD_PERF_SEL_ps_load_instr                          = 0x00000039,
    TD_PERF_SEL_write_ack_instr                        = 0x0000003a,
    TD_PERF_SEL_d16_en_instr                           = 0x0000003b,
    TD_PERF_SEL_bypassLerp_instr                       = 0x0000003c,
    TD_PERF_SEL_min_max_filter_instr                   = 0x0000003d,
    TD_PERF_SEL_one_comp_return_instr                  = 0x0000003e,
    TD_PERF_SEL_two_comp_return_instr                  = 0x0000003f,
    TD_PERF_SEL_three_comp_return_instr                = 0x00000040,
    TD_PERF_SEL_four_comp_return_instr                 = 0x00000041,
    TD_PERF_SEL_user_defined_border                    = 0x00000042,
    TD_PERF_SEL_white_border                           = 0x00000043,
    TD_PERF_SEL_opaque_black_border                    = 0x00000044,
    TD_PERF_SEL_lod_warn_from_ta                       = 0x00000045,
    TD_PERF_SEL_instruction_dest_is_lds                = 0x00000046,
    TD_PERF_SEL_td_cycling_of_nofilter_instr_2cycles   = 0x00000047,
    TD_PERF_SEL_td_cycling_of_nofilter_instr_4cycles   = 0x00000048,
    TD_PERF_SEL_tc_cycling_of_nofilter_instr_2cycles   = 0x00000049,
    TD_PERF_SEL_tc_cycling_of_nofilter_instr_4cycles   = 0x0000004a,
    TD_PERF_SEL_out_of_order_instr                     = 0x0000004b,
    TD_PERF_SEL_total_num_instr                        = 0x0000004c,
    TD_PERF_SEL_total_num_instr_with_perf_wdw          = 0x0000004d,
    TD_PERF_SEL_total_num_sampler_instr                = 0x0000004e,
    TD_PERF_SEL_total_num_sampler_instr_with_perf_wdw  = 0x0000004f,
    TD_PERF_SEL_total_num_nofilter_instr               = 0x00000050,
    TD_PERF_SEL_total_num_nofilter_instr_with_perf_wdw = 0x00000051,
    TD_PERF_SEL_total_num_ray_tracing_bvh4_instr       = 0x00000052,
    TD_PERF_SEL_total_num_ray_tracing_bvh4_instr_with_perf_wdw = 0x00000053,
    TD_PERF_SEL_mixmode_instr                          = 0x00000054,
    TD_PERF_SEL_mixmode_resource                       = 0x00000055,
    TD_PERF_SEL_status_packet                          = 0x00000056,
    TD_PERF_SEL_done_scoreboard_max_stored_cnt         = 0x00000059,
    TD_PERF_SEL_done_scoreboard_not_empty              = 0x0000005b,
    TD_PERF_SEL_done_scoreboard_is_full                = 0x0000005c,
    TD_PERF_SEL_done_scoreboard_bp                     = 0x0000005d,
    TD_PERF_SEL_nofilter_formatters_turned_on          = 0x0000005f,
    TD_PERF_SEL_nofilter_insert_extra_comps            = 0x00000060,
    TD_PERF_SEL_nofilter_popcount_dmask_gt_num_comp_of_fmt = 0x00000061,
    TD_PERF_SEL_nofilter_popcount_dmask_lt_num_comp_of_fmt = 0x00000062,
    TD_PERF_SEL_msaa_load_instr                        = 0x00000063,
    TD_PERF_SEL_blend_prt_with_prt_default_0           = 0x00000064,
    TD_PERF_SEL_resmap_instr                           = 0x00000066,
    TD_PERF_SEL_prt_ack_instr                          = 0x00000067,
    TD_PERF_SEL_resmap_with_volume_filtering           = 0x00000068,
    TD_PERF_SEL_resmap_with_aniso_filtering            = 0x00000069,
    TD_PERF_SEL_resmap_with_no_more_filtering          = 0x0000006a,
    TD_PERF_SEL_resmap_with_cubemap_corner             = 0x0000006b,
    TD_PERF_SEL_ray_tracing_bvh4_threads_per_instruction_is_0_slot0 = 0x0000006c,
    TD_PERF_SEL_ray_tracing_bvh4_threads_per_instruction_is_1_slot0 = 0x0000006d,
    TD_PERF_SEL_ray_tracing_bvh4_threads_per_instruction_is_2_slot0 = 0x0000006e,
    TD_PERF_SEL_ray_tracing_bvh4_threads_per_instruction_is_3to4_slot0 = 0x0000006f,
    TD_PERF_SEL_ray_tracing_bvh4_threads_per_instruction_is_5to8_slot0 = 0x00000070,
    TD_PERF_SEL_ray_tracing_bvh4_threads_per_instruction_is_9to16_slot0 = 0x00000071,
    TD_PERF_SEL_ray_tracing_bvh4_threads_per_instruction_is_17to31_slot0 = 0x00000072,
    TD_PERF_SEL_ray_tracing_bvh4_threads_per_instruction_is_32_slot0 = 0x00000073,
    TD_PERF_SEL_ray_tracing_bvh4_fp16_box_node         = 0x00000074,
    TD_PERF_SEL_ray_tracing_bvh4_fp32_box_node         = 0x00000075,
    TD_PERF_SEL_ray_tracing_bvh4_tri_node              = 0x00000076,
    TD_PERF_SEL_ray_tracing_bvh4_dropped_box_node      = 0x00000077,
    TD_PERF_SEL_ray_tracing_bvh4_dropped_tri_node      = 0x00000078,
    TD_PERF_SEL_ray_tracing_bvh4_invalid_box_node      = 0x00000079,
    TD_PERF_SEL_ray_tracing_bvh4_invalid_tri_node      = 0x0000007a,
    TD_PERF_SEL_ray_tracing_bvh4_box_sort_en           = 0x0000007b,
    TD_PERF_SEL_ray_tracing_bvh4_box_grow_val_nonzero  = 0x0000007c,
    TD_PERF_SEL_ray_tracing_bvh4_num_box_with_inf_or_nan_vtx_slot0 = 0x0000007d,
    TD_PERF_SEL_ray_tracing_bvh4_num_tri_with_inf_or_nan_vtx_slot0 = 0x0000007e,
    TD_PERF_SEL_ray_tracing_bvh4_num_box_that_squashed_a_nan_slot0 = 0x0000007f,
    TD_PERF_SEL_ray_tracing_bvh4_num_box_misses_slot0  = 0x00000080,
    TD_PERF_SEL_ray_tracing_bvh4_num_tri_misses_slot0  = 0x00000081,
    TD_PERF_SEL_ray_tracing_bvh4_num_tri_tie_breakers_slot0 = 0x00000082,
    TD_PERF_SEL_burst_bin_preempting_nofilter_1        = 0x00000083,
    TD_PERF_SEL_burst_bin_preempting_nofilter_2to4     = 0x00000084,
    TD_PERF_SEL_burst_bin_preempting_nofilter_5to7     = 0x00000085,
    TD_PERF_SEL_burst_bin_preempting_nofilter_8to16    = 0x00000086,
    TD_PERF_SEL_burst_bin_preempting_nofilter_gt16     = 0x00000087,
    TD_PERF_SEL_burst_bin_sampler_1                    = 0x00000088,
    TD_PERF_SEL_burst_bin_sampler_2to8                 = 0x00000089,
    TD_PERF_SEL_burst_bin_sampler_9to16                = 0x0000008a,
    TD_PERF_SEL_burst_bin_sampler_gt16                 = 0x0000008b,
    TD_PERF_SEL_burst_bin_gather_1                     = 0x0000008c,
    TD_PERF_SEL_burst_bin_gather_2to8                  = 0x0000008d,
    TD_PERF_SEL_burst_bin_gather_9to16                 = 0x0000008e,
    TD_PERF_SEL_burst_bin_gather_gt16                  = 0x0000008f,
    TD_PERF_SEL_burst_bin_nofilter_1                   = 0x00000090,
    TD_PERF_SEL_burst_bin_nofilter_2to4                = 0x00000091,
    TD_PERF_SEL_burst_bin_nofilter_5to7                = 0x00000092,
    TD_PERF_SEL_burst_bin_nofilter_8to16               = 0x00000093,
    TD_PERF_SEL_burst_bin_nofilter_gt16                = 0x00000094,
    TD_PERF_SEL_burst_bin_bvh4_1                       = 0x00000095,
    TD_PERF_SEL_burst_bin_bvh4_2to8                    = 0x00000096,
    TD_PERF_SEL_burst_bin_bvh4_9to16                   = 0x00000097,
    TD_PERF_SEL_burst_bin_bvh4_gt16                    = 0x00000098,
    TD_PERF_SEL_burst_bin_bvh4_box_nodes_1_slot0       = 0x00000099,
    TD_PERF_SEL_burst_bin_bvh4_box_nodes_2to4_slot0    = 0x0000009a,
    TD_PERF_SEL_burst_bin_bvh4_box_nodes_5to7_slot0    = 0x0000009b,
    TD_PERF_SEL_burst_bin_bvh4_box_nodes_8to16_slot0   = 0x0000009c,
    TD_PERF_SEL_burst_bin_bvh4_box_nodes_gt16_slot0    = 0x0000009d,
    TD_PERF_SEL_burst_bin_bvh4_tri_nodes_1_slot0       = 0x0000009e,
    TD_PERF_SEL_burst_bin_bvh4_tri_nodes_2to8_slot0    = 0x0000009f,
    TD_PERF_SEL_burst_bin_bvh4_tri_nodes_9to16_slot0   = 0x000000a0,
    TD_PERF_SEL_burst_bin_bvh4_tri_nodes_gt16_slot0    = 0x000000a1,
    TD_PERF_SEL_burst_bin_bvh4_dropped_nodes_1_slot0   = 0x000000a2,
    TD_PERF_SEL_burst_bin_bvh4_dropped_nodes_2to8_slot0 = 0x000000a3,
    TD_PERF_SEL_burst_bin_bvh4_dropped_nodes_9to16_slot0 = 0x000000a4,
    TD_PERF_SEL_burst_bin_bvh4_dropped_nodes_gt16_slot0 = 0x000000a5,
    TD_PERF_SEL_burst_bin_bvh4_invalid_nodes_1_slot0   = 0x000000a6,
    TD_PERF_SEL_burst_bin_bvh4_invalid_nodes_2to8_slot0 = 0x000000a7,
    TD_PERF_SEL_burst_bin_bvh4_invalid_nodes_9to16_slot0 = 0x000000a8,
    TD_PERF_SEL_burst_bin_bvh4_invalid_nodes_gt16_slot0 = 0x000000a9,
    TD_PERF_SEL_bubble_bin_ta_waiting_for_tc_data_0    = 0x000000aa,
    TD_PERF_SEL_bubble_bin_ta_waiting_for_tc_data_1    = 0x000000ab,
    TD_PERF_SEL_bubble_bin_ta_waiting_for_tc_data_2to31 = 0x000000ac,
    TD_PERF_SEL_bubble_bin_ta_waiting_for_tc_data_32to127 = 0x000000ad,
    TD_PERF_SEL_bubble_bin_ta_waiting_for_tc_data_128to511 = 0x000000ae,
    TD_PERF_SEL_bubble_bin_ta_waiting_for_tc_data_gt511 = 0x000000af,
    TD_PERF_SEL_bubble_bin_lds_stall_1to3              = 0x000000b0,
    TD_PERF_SEL_bubble_bin_lds_stall_4to7              = 0x000000b1,
    TD_PERF_SEL_bubble_bin_lds_stall_8to15             = 0x000000b2,
    TD_PERF_SEL_bubble_bin_lds_stall_gt15              = 0x000000b3,
    TD_PERF_SEL_preempting_nofilter_max_cnt            = 0x000000b4,
    TD_PERF_SEL_sampler_lerp0_active                   = 0x000000b5,
    TD_PERF_SEL_sampler_lerp1_active                   = 0x000000b6,
    TD_PERF_SEL_sampler_lerp2_active                   = 0x000000b7,
    TD_PERF_SEL_sampler_lerp3_active                   = 0x000000b8,
    TD_PERF_SEL_sampler_lerp4_active                   = 0x000000b9,
    TD_PERF_SEL_sampler_lerp5_active                   = 0x000000ba,
    TD_PERF_SEL_sampler_lerp6_active                   = 0x000000bb,
    TD_PERF_SEL_sampler_lerp7_active                   = 0x000000bc,
    TD_PERF_SEL_nofilter_total_num_comps_to_lds        = 0x000000bd,
    TD_PERF_SEL_nofilter_byte_cycling_4cycles          = 0x000000be,
    TD_PERF_SEL_nofilter_byte_cycling_8cycles          = 0x000000bf,
    TD_PERF_SEL_nofilter_byte_cycling_16cycles         = 0x000000c0,
    TD_PERF_SEL_nofilter_dword_cycling_2cycles         = 0x000000c1,
    TD_PERF_SEL_nofilter_dword_cycling_4cycles         = 0x000000c2,
    TD_PERF_SEL_input_bp_due_to_done_scoreboard_full   = 0x000000c3,
    TD_PERF_SEL_ray_tracing_bvh4_instr_invld_thread_cnt_slot0 = 0x000000c4,
    TD_PERF_SEL_ray_tracing_bvh4_box_sort_closest_child_slot0 = 0x000000c5,
    TD_PERF_SEL_ray_tracing_bvh4_box_sort_largest_first_slot0 = 0x000000c6,
    TD_PERF_SEL_ray_tracing_bvh4_box_sort_closest_midpoint_slot0 = 0x000000c7,
    TD_PERF_SEL_store_preempts_a_load                  = 0x000000c8,
    TD_PERF_SEL_sample_2x_instr                        = 0x000000c9,
    TD_PERF_SEL_gather4_2x_instr                       = 0x000000ca,
    TD_PERF_SEL_gather4h_2x_instr                      = 0x000000cb,
    TD_PERF_SEL_getlod_2x_instr                        = 0x000000cc,
    TD_PERF_SEL_resmap_2x_instr                        = 0x000000cd,
    TD_PERF_SEL_2x_sampler_op_with_1_unlit_quad        = 0x000000ce,
    TD_PERF_SEL_2x_sampler_op_with_both_quads_unlit    = 0x000000cf,
    TD_PERF_SEL_tri_proc_node_override_slot0           = 0x000000d0,
    TD_PERF_SEL_tri_run_intersect_ahs_slot0            = 0x000000d1,
    TD_PERF_SEL_tri_run_ahs_slot0                      = 0x000000d2,
    TD_PERF_SEL_ray_tracing_bvh4_instr_invld_thread_cnt_slot1 = 0x000000d3,
    TD_PERF_SEL_ray_tracing_bvh4_threads_per_instruction_is_0_slot1 = 0x000000d4,
    TD_PERF_SEL_ray_tracing_bvh4_threads_per_instruction_is_1_slot1 = 0x000000d5,
    TD_PERF_SEL_ray_tracing_bvh4_threads_per_instruction_is_2_slot1 = 0x000000d6,
    TD_PERF_SEL_ray_tracing_bvh4_threads_per_instruction_is_3to4_slot1 = 0x000000d7,
    TD_PERF_SEL_ray_tracing_bvh4_threads_per_instruction_is_5to8_slot1 = 0x000000d8,
    TD_PERF_SEL_ray_tracing_bvh4_threads_per_instruction_is_9to16_slot1 = 0x000000d9,
    TD_PERF_SEL_ray_tracing_bvh4_threads_per_instruction_is_17to31_slot1 = 0x000000da,
    TD_PERF_SEL_ray_tracing_bvh4_threads_per_instruction_is_32_slot1 = 0x000000db,
    TD_PERF_SEL_ray_tracing_bvh4_num_box_with_inf_or_nan_vtx_slot1 = 0x000000dc,
    TD_PERF_SEL_ray_tracing_bvh4_num_tri_with_inf_or_nan_vtx_slot1 = 0x000000dd,
    TD_PERF_SEL_ray_tracing_bvh4_num_box_that_squashed_a_nan_slot1 = 0x000000de,
    TD_PERF_SEL_ray_tracing_bvh4_num_box_misses_slot1  = 0x000000df,
    TD_PERF_SEL_ray_tracing_bvh4_num_tri_misses_slot1  = 0x000000e0,
    TD_PERF_SEL_ray_tracing_bvh4_num_tri_tie_breakers_slot1 = 0x000000e1,
    TD_PERF_SEL_ray_tracing_bvh4_pkr_full_slot1        = 0x000000e2,
    TD_PERF_SEL_ray_tracing_bvh4_pkr_full_due_to_arb_slot1 = 0x000000e3,
    TD_PERF_SEL_ray_tracing_bvh4_box_sort_closest_child_slot1 = 0x000000e4,
    TD_PERF_SEL_ray_tracing_bvh4_box_sort_largest_first_slot1 = 0x000000e5,
    TD_PERF_SEL_ray_tracing_bvh4_box_sort_closest_midpoint_slot1 = 0x000000e6,
    TD_PERF_SEL_tri_proc_node_override_slot1           = 0x000000e7,
    TD_PERF_SEL_tri_run_intersect_ahs_slot1            = 0x000000e8,
    TD_PERF_SEL_tri_run_ahs_slot1                      = 0x000000e9,
    TD_PERF_SEL_ray_tracing_bvh4_hp64_box_node         = 0x000000ea,
    TD_PERF_SEL_ray_tracing_bvh4_total_vld_nodes       = 0x000000eb,
    TD_PERF_SEL_ray_tracing_bvh4_thread_uses_no_slots  = 0x000000ec,
    TD_PERF_SEL_ray_tracing_bvh4_thread_uses_slot0_only = 0x000000ed,
    TD_PERF_SEL_ray_tracing_bvh4_thread_uses_slot1_only = 0x000000ee,
    TD_PERF_SEL_ray_tracing_bvh4_thread_uses_both_slots = 0x000000ef,
    TD_PERF_SEL_ray_tracing_bvh4_inst_transform_op     = 0x000000f0,
    TD_PERF_SEL_instance_mask_culled                   = 0x000000f1,
    TD_PERF_SEL_box_opaque_culled                      = 0x000000f2,
    TD_PERF_SEL_box_non_opaque_culled                  = 0x000000f3,
    TD_PERF_SEL_box_with_triangle_children_only_culled = 0x000000f4,
    TD_PERF_SEL_box_with_procedural_children_only_culled = 0x000000f5,
    TD_PERF_SEL_triangle_opaque_culled                 = 0x000000f6,
    TD_PERF_SEL_triangle_non_opaque_culled             = 0x000000f7,
    TD_PERF_SEL_triangle_front_facing_culled           = 0x000000f8,
    TD_PERF_SEL_triangle_back_facing_culled            = 0x000000f9,
    TD_PERF_SEL_ray_tracing_bvh4_ip_sclk_en_slot1      = 0x000000fa,
    TD_PERF_SEL_ray_tracing_bvh4_box_sclk_en_slot1     = 0x000000fb,
    TD_PERF_SEL_ray_tracing_bvh4_tri_sclk_en_slot1     = 0x000000fc,
    TD_PERF_SEL_burst_bin_bvh4_box_nodes_1_slot1       = 0x000000fd,
    TD_PERF_SEL_burst_bin_bvh4_box_nodes_2to4_slot1    = 0x000000fe,
    TD_PERF_SEL_burst_bin_bvh4_box_nodes_5to7_slot1    = 0x000000ff,
    TD_PERF_SEL_burst_bin_bvh4_box_nodes_8to16_slot1   = 0x00000100,
    TD_PERF_SEL_burst_bin_bvh4_box_nodes_gt16_slot1    = 0x00000101,
    TD_PERF_SEL_burst_bin_bvh4_tri_nodes_1_slot1       = 0x00000102,
    TD_PERF_SEL_burst_bin_bvh4_tri_nodes_2to8_slot1    = 0x00000103,
    TD_PERF_SEL_burst_bin_bvh4_tri_nodes_9to16_slot1   = 0x00000104,
    TD_PERF_SEL_burst_bin_bvh4_tri_nodes_gt16_slot1    = 0x00000105,
    TD_PERF_SEL_burst_bin_bvh4_dropped_nodes_1_slot1   = 0x00000106,
    TD_PERF_SEL_burst_bin_bvh4_dropped_nodes_2to8_slot1 = 0x00000107,
    TD_PERF_SEL_burst_bin_bvh4_dropped_nodes_9to16_slot1 = 0x00000108,
    TD_PERF_SEL_burst_bin_bvh4_dropped_nodes_gt16_slot1 = 0x00000109,
    TD_PERF_SEL_burst_bin_bvh4_invalid_nodes_1_slot1   = 0x0000010a,
    TD_PERF_SEL_burst_bin_bvh4_invalid_nodes_2to8_slot1 = 0x0000010b,
    TD_PERF_SEL_burst_bin_bvh4_invalid_nodes_9to16_slot1 = 0x0000010c,
    TD_PERF_SEL_burst_bin_bvh4_invalid_nodes_gt16_slot1 = 0x0000010d,
    TD_PERF_SEL_ray_tracing_obb_nodes                  = 0x0000010e,
    TD_PERF_SEL_ray_tracing_intersectable_instance_valid_children = 0x0000010f,
} TD_PERFCOUNT_SEL;

constexpr unsigned int MaxTdPerfcountSel               = TD_PERF_SEL_ray_tracing_intersectable_instance_valid_children;

typedef enum TEX_BC_SWIZZLE {
    TEX_BC_Swizzle_XYZW                                = 0x00000000,
    TEX_BC_Swizzle_XWYZ                                = 0x00000001,
    TEX_BC_Swizzle_WZYX                                = 0x00000002,
    TEX_BC_Swizzle_WXYZ                                = 0x00000003,
    TEX_BC_Swizzle_ZYXW                                = 0x00000004,
    TEX_BC_Swizzle_YXWZ                                = 0x00000005,
} TEX_BC_SWIZZLE;

typedef enum TEX_BORDER_COLOR_TYPE {
    TEX_BorderColor_TransparentBlack                   = 0x00000000,
    TEX_BorderColor_OpaqueBlack                        = 0x00000001,
    TEX_BorderColor_OpaqueWhite                        = 0x00000002,
    TEX_BorderColor_Register                           = 0x00000003,
} TEX_BORDER_COLOR_TYPE;

typedef enum TEX_CHROMA_KEY {
    TEX_ChromaKey_Disabled                             = 0x00000000,
    TEX_ChromaKey_Kill                                 = 0x00000001,
    TEX_ChromaKey_Blend                                = 0x00000002,
    TEX_ChromaKey_RESERVED_3                           = 0x00000003,
} TEX_CHROMA_KEY;

typedef enum TEX_CLAMP {
    TEX_Clamp_Repeat                                   = 0x00000000,
    TEX_Clamp_Mirror                                   = 0x00000001,
    TEX_Clamp_ClampToLast                              = 0x00000002,
    TEX_Clamp_MirrorOnceToLast                         = 0x00000003,
    TEX_Clamp_ClampHalfToBorder                        = 0x00000004,
    TEX_Clamp_MirrorOnceHalfToBorder                   = 0x00000005,
    TEX_Clamp_ClampToBorder                            = 0x00000006,
    TEX_Clamp_MirrorOnceToBorder                       = 0x00000007,
} TEX_CLAMP;

typedef enum TEX_COORD_TYPE {
    TEX_CoordType_Unnormalized                         = 0x00000000,
    TEX_CoordType_Normalized                           = 0x00000001,
} TEX_COORD_TYPE;

typedef enum TEX_DEPTH_COMPARE_FUNCTION {
    TEX_DepthCompareFunction_Never                     = 0x00000000,
    TEX_DepthCompareFunction_Less                      = 0x00000001,
    TEX_DepthCompareFunction_Equal                     = 0x00000002,
    TEX_DepthCompareFunction_LessEqual                 = 0x00000003,
    TEX_DepthCompareFunction_Greater                   = 0x00000004,
    TEX_DepthCompareFunction_NotEqual                  = 0x00000005,
    TEX_DepthCompareFunction_GreaterEqual              = 0x00000006,
    TEX_DepthCompareFunction_Always                    = 0x00000007,
} TEX_DEPTH_COMPARE_FUNCTION;

typedef enum TEX_DIM {
    TEX_Dim_1D                                         = 0x00000000,
    TEX_Dim_2D                                         = 0x00000001,
    TEX_Dim_3D                                         = 0x00000002,
    TEX_Dim_CubeMap                                    = 0x00000003,
    TEX_Dim_1DArray                                    = 0x00000004,
    TEX_Dim_2DArray                                    = 0x00000005,
    TEX_Dim_2D_MSAA                                    = 0x00000006,
    TEX_Dim_2DArray_MSAA                               = 0x00000007,
} TEX_DIM;

typedef enum TEX_FORMAT_COMP {
    TEX_FormatComp_Unsigned                            = 0x00000000,
    TEX_FormatComp_Signed                              = 0x00000001,
    TEX_FormatComp_UnsignedBiased                      = 0x00000002,
    TEX_FormatComp_RESERVED_3                          = 0x00000003,
} TEX_FORMAT_COMP;

typedef enum TEX_MAX_ANISO_RATIO {
    TEX_MaxAnisoRatio_1to1                             = 0x00000000,
    TEX_MaxAnisoRatio_2to1                             = 0x00000001,
    TEX_MaxAnisoRatio_4to1                             = 0x00000002,
    TEX_MaxAnisoRatio_8to1                             = 0x00000003,
    TEX_MaxAnisoRatio_16to1                            = 0x00000004,
    TEX_MaxAnisoRatio_RESERVED_5                       = 0x00000005,
    TEX_MaxAnisoRatio_RESERVED_6                       = 0x00000006,
    TEX_MaxAnisoRatio_RESERVED_7                       = 0x00000007,
} TEX_MAX_ANISO_RATIO;

typedef enum TEX_MIP_FILTER {
    TEX_MipFilter_None                                 = 0x00000000,
    TEX_MipFilter_Point                                = 0x00000001,
    TEX_MipFilter_Linear                               = 0x00000002,
    TEX_MipFilter_Point_Aniso_Adj                      = 0x00000003,
} TEX_MIP_FILTER;

typedef enum TEX_REQUEST_SIZE {
    TEX_RequestSize_32B                                = 0x00000000,
    TEX_RequestSize_64B                                = 0x00000001,
    TEX_RequestSize_128B                               = 0x00000002,
    TEX_RequestSize_2X64B                              = 0x00000003,
} TEX_REQUEST_SIZE;

typedef enum TEX_SAMPLER_TYPE {
    TEX_SamplerType_Invalid                            = 0x00000000,
    TEX_SamplerType_Valid                              = 0x00000001,
} TEX_SAMPLER_TYPE;

typedef enum TEX_XY_FILTER {
    TEX_XYFilter_Point                                 = 0x00000000,
    TEX_XYFilter_Linear                                = 0x00000001,
    TEX_XYFilter_AnisoPoint                            = 0x00000002,
    TEX_XYFilter_AnisoLinear                           = 0x00000003,
} TEX_XY_FILTER;

typedef enum TEX_Z_FILTER {
    TEX_ZFilter_None                                   = 0x00000000,
    TEX_ZFilter_Point                                  = 0x00000001,
    TEX_ZFilter_Linear                                 = 0x00000002,
    TEX_ZFilter_RESERVED_3                             = 0x00000003,
} TEX_Z_FILTER;

typedef enum UMC_PERFCOUNT_SELECT {
    UMC_PERF_SEL_NONE                                  = 0x00000000,
    UMC_PERF_SEL_SdpPh                                 = 0x00000001,
    UMC_PERF_SEL_SdpPm                                 = 0x00000002,
    UMC_PERF_SEL_SdpPc                                 = 0x00000003,
    UMC_PERF_SEL_ActCmd                                = 0x00000005,
    UMC_PERF_SEL_PchgCmd                               = 0x00000006,
    UMC_PERF_SEL_SpecPchgCmd                           = 0x00000007,
    UMC_PERF_SEL_PchgAllCmd                            = 0x00000008,
    UMC_PERF_SEL_OtherPchgCmd                          = 0x00000009,
    UMC_PERF_SEL_CasCmd                                = 0x0000000a,
    UMC_PERF_SEL_CasWithAP                             = 0x0000000b,
    UMC_PERF_SEL_TurnRndRdWr                           = 0x0000000d,
    UMC_PERF_SEL_CmdSlotClks                           = 0x00000013,
    UMC_PERF_SEL_DataSlotClks                          = 0x00000014,
    UMC_PERF_SEL_CmdError                              = 0x00000016,
    UMC_PERF_SEL_DataError                             = 0x00000017,
    UMC_PERF_SEL_RetryParity                           = 0x00000018,
    UMC_PERF_SEL_RetryCRC                              = 0x00000019,
    UMC_PERF_SEL_BypassSat                             = 0x0000001a,
    UMC_PERF_SEL_PrechagePwrDownClks                   = 0x0000001b,
    UMC_PERF_SEL_ThermalThrottleClks                   = 0x0000001c,
    UMC_PERF_SEL_DcqOccupancy                          = 0x0000001d,
    UMC_PERF_SEL_SelfRefreshClks                       = 0x00000022,
    UMC_PERF_SEL_PendReqCnt                            = 0x00000024,
    UMC_PERF_SEL_AutoRefCnt                            = 0x00000025,
    UMC_PERF_SEL_BeqRdWrCmd                            = 0x00000026,
    UMC_PERF_SEL_BeqEdcErr                             = 0x00000027,
    UMC_PERF_SEL_ArbRdWrPhase                          = 0x0000002b,
    UMC_PERF_SEL_ArbRefReqClks                         = 0x0000002c,
    UMC_PERF_SEL_UrgRefStBkClks                        = 0x0000002d,
    UMC_PERF_SEL_RtCalEvtCnt                           = 0x00000030,
    UMC_PERF_SEL_RtCalBlockClks                        = 0x00000031,
    UMC_PERF_SEL_SdpCoherent                           = 0x00000033,
    UMC_PERF_SEL_SdpChngPri                            = 0x00000034,
    UMC_PERF_SEL_SdpCancel                             = 0x00000035,
    UMC_PERF_SEL_SdpBurstLength                        = 0x00000036,
    UMC_PERF_SEL_SdpBurstCnt                           = 0x00000037,
    UMC_PERF_SEL_SdpGapLen                             = 0x00000038,
    UMC_PERF_SEL_SdpGapCnt                             = 0x00000039,
    UMC_PERF_SEL_CasCmdWSM                             = 0x0000003a,
    UMC_PERF_SEL_CasCmdWDM                             = 0x0000003b,
    UMC_PERF_SEL_CasBurstLength                        = 0x0000003c,
    UMC_PERF_SEL_CasBurstCnt                           = 0x0000003d,
    UMC_PERF_SEL_CasCmdCW                              = 0x0000003e,
    UMC_PERF_SEL_TurnRndRdWr_noRd                      = 0x0000003f,
    UMC_PERF_SEL_TurnRndRdWr_RdThresh                  = 0x00000040,
    UMC_PERF_SEL_TurnRndWrRd_noWr                      = 0x00000041,
    UMC_PERF_SEL_TurnRndWrRd_WrThresh                  = 0x00000042,
    UMC_PERF_SEL_TurnRndWrRd_RdUrg                     = 0x00000043,
    UMC_PERF_SEL_PrechargePwrDownEntry                 = 0x00000044,
    UMC_PERF_SEL_ThermalThrottleEntry                  = 0x00000045,
    UMC_PERF_SEL_PgtActiveBanksCnt                     = 0x00000046,
    UMC_PERF_SEL_SelfRefreshEntry                      = 0x00000047,
    UMC_PERF_SEL_RdRspBusyNoCrdt                       = 0x00000048,
    UMC_PERF_SEL_RdstallRdRspBufNotRdy                 = 0x00000049,
    UMC_PERF_SEL_RdrspBusyVdciNotRdy                   = 0x0000004a,
    UMC_PERF_SEL_DfiLpReq                              = 0x0000004b,
    UMC_PERF_SEL_DfiLpCyc                              = 0x0000004c,
    UMC_PERF_SEL_BubbleOverall                         = 0x0000004d,
    UMC_PERF_SEL_BeqEdcErrB0                           = 0x0000004e,
    UMC_PERF_SEL_BeqEdcErrB1                           = 0x0000004f,
    UMC_PERF_SEL_BeqEdcErrB2                           = 0x00000050,
    UMC_PERF_SEL_BeqEdcErrB3                           = 0x00000051,
    UMC_PERF_SEL_TempOverThresh                        = 0x00000052,
    UMC_PERF_SEL_TempCnt                               = 0x00000053,
    UMC_PERF_SEL_ReqFullEntry                          = 0x00000054,
    UMC_PERF_SEL_NumReqs                               = 0x00000055,
    UMC_PERF_SEL_CasWithDeAlloc                        = 0x00000056,
    UMC_PERF_SEL_RfmCmd                                = 0x00000057,
    UMC_PERF_SEL_SdpNewEntry                           = 0x00000058,
    UMC_PERF_SEL_SdpHitEntry                           = 0x00000059,
    UMC_PERF_SEL_PatrolScrub                           = 0x0000005a,
    UMC_PERF_SEL_RedirectScrub                         = 0x0000005b,
    UMC_PERF_SEL_PoisonScrub                           = 0x0000005c,
    UMC_PERF_SEL_MemClear                              = 0x0000005d,
    UMC_PERF_SEL_DroppedRedirectScrub                  = 0x0000005e,
    UMC_PERF_SEL_DroppedPoisonScrub                    = 0x0000005f,
    UMC_PERF_SEL_DroppedScrubWrite                     = 0x00000060,
    UMC_PERF_SEL_RfmBlockClk                           = 0x00000061,
    UMC_PERF_SEL_NoRefOverThres                        = 0x00000062,
    UMC_PERF_SEL_StoredRefOverThres                    = 0x00000063,
    UMC_PERF_SEL_PptRunning                            = 0x00000064,
    UMC_PERF_SEL_PptTrig                               = 0x00000065,
    UMC_PERF_SEL_PptMinWin                             = 0x00000066,
    UMC_PERF_SEL_ClockCount                            = 0x000000ff,
} UMC_PERFCOUNT_SELECT;

constexpr unsigned int MaxUmcPerfcountSelect           = UMC_PERF_SEL_ClockCount;

typedef enum UTCL1PerfSel {
    UTCL1_PERF_SEL_NONE                                = 0x00000000,
    UTCL1_PERF_SEL_REQS                                = 0x00000001,
    UTCL1_PERF_SEL_HITS                                = 0x00000002,
    UTCL1_PERF_SEL_MISSES                              = 0x00000003,
    UTCL1_PERF_SEL_MH_RECENT_BUF_HIT                   = 0x00000004,
    UTCL1_PERF_SEL_MH_DUPLICATE_DETECT                 = 0x00000005,
    UTCL1_PERF_SEL_UTCL2_REQS                          = 0x00000006,
    UTCL1_PERF_SEL_UTCL2_RET_XNACK_RETRY               = 0x00000007,
    UTCL1_PERF_SEL_UTCL2_RET_FAULT                     = 0x00000008,
    UTCL1_PERF_SEL_STALL_UTCL2_CREDITS                 = 0x00000009,
    UTCL1_PERF_SEL_STALL_MH_FULL                       = 0x0000000a,
    UTCL1_PERF_SEL_UTCL2_REQS_OUTSTANDING_ACCUM        = 0x0000000b,
    UTCL1_PERF_SEL_UTCL2_RET_CNT                       = 0x0000000c,
    UTCL1_PERF_SEL_RTNS                                = 0x0000000d,
    UTCL1_PERF_SEL_XLAT_REQ_BUSY                       = 0x0000000e,
    UTCL1_PERF_SEL_RANGE_INVREQS                       = 0x0000000f,
    UTCL1_PERF_SEL_INV_ALL_VMID_INVREQS                = 0x00000010,
    UTCL1_PERF_SEL_BYPASS_REQS                         = 0x00000011,
    UTCL1_PERF_SEL_HIT_INV_FILTER_REQS                 = 0x00000012,
    UTCL1_PERF_SEL_UTCL2_RET_PERM_FAULT                = 0x00000013,
    UTCL1_PERF_SEL_UTCL2_RET_PRT_FAULT                 = 0x00000014,
    UTCL1_PERF_SEL_CP_INVREQS                          = 0x00000015,
    UTCL1_PERF_SEL_UTCL2_UTCL1_INVREQS                 = 0x00000016,
    UTCL1_PERF_SEL_NUM_UTCL2_RTN_SIZE_4K_32K           = 0x00000017,
    UTCL1_PERF_SEL_NUM_UTCL2_RTN_SIZE_64K_128K         = 0x00000018,
    UTCL1_PERF_SEL_NUM_UTCL2_RTN_SIZE_256K             = 0x00000019,
    UTCL1_PERF_SEL_NUM_UTCL2_RTN_SIZE_512K             = 0x0000001a,
    UTCL1_PERF_SEL_NUM_UTCL2_RTN_SIZE_1M               = 0x0000001b,
    UTCL1_PERF_SEL_NUM_UTCL2_RTN_SIZE_2M               = 0x0000001c,
    UTCL1_PERF_SEL_NUM_UTCL2_RTN_SIZE_4M               = 0x0000001d,
    UTCL1_PERF_SEL_NUM_UTCL2_RTN_SIZE_8M               = 0x0000001e,
    UTCL1_PERF_SEL_NUM_UTCL2_RTN_SIZE_16M              = 0x0000001f,
    UTCL1_PERF_SEL_NUM_UTCL2_RTN_SIZE_32M_INF          = 0x00000020,
    UTCL1_PERF_SEL_UTCL2_REQ_SQUASHED_NUM              = 0x00000021,
    UTCL1_PERF_SEL_REQ_NUM_CACHE_CORE_0                = 0x00000022,
    UTCL1_PERF_SEL_REQ_NUM_CACHE_CORE_1                = 0x00000023,
    UTCL1_PERF_SEL_REQ_NUM_CACHE_CORE_2                = 0x00000024,
    UTCL1_PERF_SEL_REQ_NUM_CACHE_CORE_3                = 0x00000025,
    UTCL1_PERF_SEL_STALL_CYCLES_CACHE_CORE_0           = 0x00000026,
    UTCL1_PERF_SEL_STALL_CYCLES_CACHE_CORE_1           = 0x00000027,
    UTCL1_PERF_SEL_STALL_CYCLES_CACHE_CORE_2           = 0x00000028,
    UTCL1_PERF_SEL_STALL_CYCLES_CACHE_CORE_3           = 0x00000029,
    UTCL1_PERF_SEL_UTCL1_UTCL2_INVACKS                 = 0x0000002a,
    UTCL1_PERF_SEL_UTCL0_UTCL1_INVACKS                 = 0x0000002b,
    UTCL1_PERF_SEL_HITS_PG_SIZE_1                      = 0x0000002c,
    UTCL1_PERF_SEL_HITS_PG_SIZE_2                      = 0x0000002d,
    UTCL1_PERF_SEL_HITS_PG_SIZE_3                      = 0x0000002e,
    UTCL1_PERF_SEL_HITS_PG_SIZE_4                      = 0x0000002f,
    UTCL1_PERF_SEL_REQ_TO_MISS_HNDLR_0                 = 0x00000030,
    UTCL1_PERF_SEL_REQ_TO_MISS_HNDLR_1                 = 0x00000031,
    UTCL1_PERF_SEL_REQ_TO_MISS_HNDLR_2                 = 0x00000032,
    UTCL1_PERF_SEL_REQ_TO_MISS_HNDLR_3                 = 0x00000033,
    UTCL1_PERF_SEL_AVG_INV_LATENCY                     = 0x00000034,
    UTCL1_PERF_SEL_NUM_OF_CYCLES_RQ_EXISTS_TO_CC0      = 0x00000035,
    UTCL1_PERF_SEL_NUM_OF_CYCLES_RQ_EXISTS_TO_CC1      = 0x00000036,
    UTCL1_PERF_SEL_NUM_OF_CYCLES_RQ_EXISTS_TO_CC2      = 0x00000037,
    UTCL1_PERF_SEL_NUM_OF_CYCLES_RQ_EXISTS_TO_CC3      = 0x00000038,
    UTCL1_PERF_SEL_NUM_OF_CYCLES_W_COLLISION_CC0       = 0x00000039,
    UTCL1_PERF_SEL_NUM_OF_CYCLES_W_COLLISION_CC1       = 0x0000003a,
    UTCL1_PERF_SEL_NUM_OF_CYCLES_W_COLLISION_CC2       = 0x0000003b,
    UTCL1_PERF_SEL_NUM_OF_CYCLES_W_COLLISION_CC3       = 0x0000003c,
    UTCL1_PERF_SEL_EVICTIONS_NUM_CC0                   = 0x0000003d,
    UTCL1_PERF_SEL_EVICTIONS_NUM_CC1                   = 0x0000003e,
    UTCL1_PERF_SEL_EVICTIONS_NUM_CC2                   = 0x0000003f,
    UTCL1_PERF_SEL_EVICTIONS_NUM_CC3                   = 0x00000040,
    UTCL1_PERF_SEL_MH_B2B_DUPLICATE_DETECT             = 0x00000041,
    UTCL1_PERF_SEL_MH_B2B_UTCL2_REQ_SQUASHED           = 0x00000042,
    UTCL1_PERF_SEL_ALOG_INTERRUPT                      = 0x00000043,
    UTCL1_PERF_SEL_ALOG_INTERRUPT_DROPPED              = 0x00000044,
    UTCL1_PERF_SEL_ALOG_CACHE_REQ                      = 0x00000045,
    UTCL1_PERF_SEL_ALOG_CACHE_HIT                      = 0x00000046,
    UTCL1_PERF_SEL_ALOG_STALL_PMM_CREDITS              = 0x00000047,
} UTCL1PerfSel;

constexpr unsigned int MaxUTCL1PerfSel                 = UTCL1_PERF_SEL_ALOG_STALL_PMM_CREDITS;

typedef enum VGT_DETECT_ONE {
    ENABLE_TF1_OPT                                     = 0x00000000,
    DISABLE_TF1_OPT                                    = 0x00000001,
} VGT_DETECT_ONE;

typedef enum VGT_DETECT_ZERO {
    ENABLE_TF0_OPT                                     = 0x00000000,
    DISABLE_TF0_OPT                                    = 0x00000001,
} VGT_DETECT_ZERO;

typedef enum VGT_DIST_MODE {
    NO_DIST                                            = 0x00000000,
    PATCHES                                            = 0x00000001,
    DONUTS                                             = 0x00000002,
    TRAPEZOIDS                                         = 0x00000003,
} VGT_DIST_MODE;

typedef enum VGT_DI_INDEX_SIZE {
    DI_INDEX_SIZE_16_BIT                               = 0x00000000,
    DI_INDEX_SIZE_32_BIT                               = 0x00000001,
    DI_INDEX_SIZE_8_BIT                                = 0x00000002,
} VGT_DI_INDEX_SIZE;

typedef enum VGT_DI_PRIM_TYPE {
    DI_PT_NONE                                         = 0x00000000,
    DI_PT_POINTLIST                                    = 0x00000001,
    DI_PT_LINELIST                                     = 0x00000002,
    DI_PT_LINESTRIP                                    = 0x00000003,
    DI_PT_TRILIST                                      = 0x00000004,
    DI_PT_TRIFAN                                       = 0x00000005,
    DI_PT_TRISTRIP                                     = 0x00000006,
    DI_PT_2D_RECTANGLE                                 = 0x00000007,
    DI_PT_UNUSED_1                                     = 0x00000008,
    DI_PT_PATCH                                        = 0x00000009,
    DI_PT_LINELIST_ADJ                                 = 0x0000000a,
    DI_PT_LINESTRIP_ADJ                                = 0x0000000b,
    DI_PT_TRILIST_ADJ                                  = 0x0000000c,
    DI_PT_TRISTRIP_ADJ                                 = 0x0000000d,
    DI_PT_UNUSED_3                                     = 0x0000000e,
    DI_PT_UNUSED_4                                     = 0x0000000f,
    DI_PT_UNUSED_5                                     = 0x00000010,
    DI_PT_RECTLIST                                     = 0x00000011,
    DI_PT_LINELOOP                                     = 0x00000012,
    DI_PT_QUADLIST                                     = 0x00000013,
    DI_PT_QUADSTRIP                                    = 0x00000014,
    DI_PT_POLYGON                                      = 0x00000015,
} VGT_DI_PRIM_TYPE;

typedef enum VGT_DI_SOURCE_SELECT {
    DI_SRC_SEL_DMA                                     = 0x00000000,
    DI_SRC_SEL_IMMEDIATE                               = 0x00000001,
    DI_SRC_SEL_AUTO_INDEX                              = 0x00000002,
    DI_SRC_SEL_RESERVED                                = 0x00000003,
} VGT_DI_SOURCE_SELECT;

typedef enum VGT_DMA_BUF_TYPE {
    VGT_DMA_BUF_MEM                                    = 0x00000000,
    VGT_DMA_BUF_RING                                   = 0x00000001,
    VGT_DMA_BUF_SETUP                                  = 0x00000002,
    VGT_DMA_PTR_UPDATE                                 = 0x00000003,
} VGT_DMA_BUF_TYPE;

typedef enum VGT_DMA_SWAP_MODE {
    VGT_DMA_SWAP_NONE                                  = 0x00000000,
    VGT_DMA_SWAP_16_BIT                                = 0x00000001,
    VGT_DMA_SWAP_32_BIT                                = 0x00000002,
    VGT_DMA_SWAP_WORD                                  = 0x00000003,
} VGT_DMA_SWAP_MODE;

typedef enum VGT_EVENT_TYPE {
    Reserved_0x00                                      = 0x00000000,
    SAMPLE_STREAMOUTSTATS1                             = 0x00000001,
    SAMPLE_STREAMOUTSTATS2                             = 0x00000002,
    SAMPLE_STREAMOUTSTATS3                             = 0x00000003,
    CACHE_FLUSH_TS                                     = 0x00000004,
    CONTEXT_DONE                                       = 0x00000005,
    CACHE_FLUSH                                        = 0x00000006,
    CS_PARTIAL_FLUSH                                   = 0x00000007,
    VGT_STREAMOUT_SYNC                                 = 0x00000008,
    EVENT_STATE_CHANGE                                 = 0x00000009,
    VGT_STREAMOUT_RESET                                = 0x0000000a,
    END_OF_PIPE_INCR_DE                                = 0x0000000b,
    END_OF_PIPE_IB_END                                 = 0x0000000c,
    RST_PIX_CNT                                        = 0x0000000d,
    BREAK_BATCH                                        = 0x0000000e,
    VS_PARTIAL_FLUSH                                   = 0x0000000f,
    PS_PARTIAL_FLUSH                                   = 0x00000010,
    FLUSH_HS_OUTPUT                                    = 0x00000011,
    FLUSH_DFSM                                         = 0x00000012,
    RESET_TO_LOWEST_VGT                                = 0x00000013,
    CACHE_FLUSH_AND_INV_TS_EVENT                       = 0x00000014,
    WAIT_SYNC                                          = 0x00000015,
    CACHE_FLUSH_AND_INV_EVENT                          = 0x00000016,
    PERFCOUNTER_START                                  = 0x00000017,
    PERFCOUNTER_STOP                                   = 0x00000018,
    PIPELINESTAT_START                                 = 0x00000019,
    PIPELINESTAT_STOP                                  = 0x0000001a,
    PERFCOUNTER_SAMPLE                                 = 0x0000001b,
    FLUSH_ES_OUTPUT                                    = 0x0000001c,
    BIN_CONF_OVERRIDE_CHECK                            = 0x0000001d,
    SAMPLE_PIPELINESTAT                                = 0x0000001e,
    SO_VGTSTREAMOUT_FLUSH                              = 0x0000001f,
    SAMPLE_STREAMOUTSTATS                              = 0x00000020,
    RESET_VTX_CNT                                      = 0x00000021,
    BLOCK_CONTEXT_DONE                                 = 0x00000022,
    CS_CONTEXT_DONE                                    = 0x00000023,
    VGT_FLUSH                                          = 0x00000024,
    TGID_ROLLOVER                                      = 0x00000025,
    SQ_NON_EVENT                                       = 0x00000026,
    SC_SEND_DB_VPZ                                     = 0x00000027,
    BOTTOM_OF_PIPE_TS                                  = 0x00000028,
    FLUSH_SX_TS                                        = 0x00000029,
    DB_CACHE_FLUSH_AND_INV                             = 0x0000002a,
    FLUSH_AND_INV_DB_DATA_TS                           = 0x0000002b,
    FLUSH_AND_INV_DB_META                              = 0x0000002c,
    FLUSH_AND_INV_CB_DATA_TS                           = 0x0000002d,
    FLUSH_AND_INV_CB_META                              = 0x0000002e,
    CS_DONE                                            = 0x0000002f,
    PS_DONE                                            = 0x00000030,
    FLUSH_AND_INV_CB_PIXEL_DATA                        = 0x00000031,
    SX_CB_RAT_ACK_REQUEST                              = 0x00000032,
    THREAD_TRACE_START                                 = 0x00000033,
    THREAD_TRACE_STOP                                  = 0x00000034,
    THREAD_TRACE_MARKER                                = 0x00000035,
    THREAD_TRACE_DRAW                                  = 0x00000036,
    THREAD_TRACE_FINISH                                = 0x00000037,
    PIXEL_PIPE_STAT_CONTROL                            = 0x00000038,
    PIXEL_PIPE_STAT_DUMP                               = 0x00000039,
    PIXEL_PIPE_STAT_RESET                              = 0x0000003a,
    CONTEXT_SUSPEND                                    = 0x0000003b,
    OFFCHIP_HS_DEALLOC                                 = 0x0000003c,
    ENABLE_NGG_PIPELINE                                = 0x0000003d,
    ENABLE_PIPELINE_NOT_USED                           = 0x0000003e,
    DRAW_DONE                                          = 0x0000003f,
} VGT_EVENT_TYPE;

typedef enum VGT_GROUP_CONV_SEL {
    VGT_GRP_INDEX_16                                   = 0x00000000,
    VGT_GRP_INDEX_32                                   = 0x00000001,
    VGT_GRP_UINT_16                                    = 0x00000002,
    VGT_GRP_UINT_32                                    = 0x00000003,
    VGT_GRP_SINT_16                                    = 0x00000004,
    VGT_GRP_SINT_32                                    = 0x00000005,
    VGT_GRP_FLOAT_32                                   = 0x00000006,
    VGT_GRP_AUTO_PRIM                                  = 0x00000007,
    VGT_GRP_FIX_1_23_TO_FLOAT                          = 0x00000008,
} VGT_GROUP_CONV_SEL;

typedef enum VGT_GS_MODE_TYPE {
    GS_OFF                                             = 0x00000000,
    GS_SCENARIO_A                                      = 0x00000001,
    GS_SCENARIO_B                                      = 0x00000002,
    GS_SCENARIO_G                                      = 0x00000003,
    GS_SCENARIO_C                                      = 0x00000004,
    SPRITE_EN                                          = 0x00000005,
} VGT_GS_MODE_TYPE;

typedef enum VGT_GS_OUTPRIM_TYPE {
    POINTLIST                                          = 0x00000000,
    LINESTRIP                                          = 0x00000001,
    TRISTRIP                                           = 0x00000002,
    RECT_2D                                            = 0x00000003,
    RECTLIST                                           = 0x00000004,
} VGT_GS_OUTPRIM_TYPE;

typedef enum VGT_INDEX_TYPE_MODE {
    VGT_INDEX_16                                       = 0x00000000,
    VGT_INDEX_32                                       = 0x00000001,
    VGT_INDEX_8                                        = 0x00000002,
} VGT_INDEX_TYPE_MODE;

typedef enum VGT_OUTPATH_SELECT {
    VGT_OUTPATH_VTX_REUSE                              = 0x00000000,
    VGT_OUTPATH_GS_BLOCK                               = 0x00000001,
    VGT_OUTPATH_HS_BLOCK                               = 0x00000002,
    VGT_OUTPATH_PRIM_GEN                               = 0x00000003,
    VGT_OUTPATH_TE_PRIM_GEN                            = 0x00000004,
    VGT_OUTPATH_TE_GS_BLOCK                            = 0x00000005,
    VGT_OUTPATH_TE_OUTPUT                              = 0x00000006,
} VGT_OUTPATH_SELECT;

typedef enum VGT_OUT_PRIM_TYPE {
    VGT_OUT_POINT                                      = 0x00000000,
    VGT_OUT_LINE                                       = 0x00000001,
    VGT_OUT_TRI                                        = 0x00000002,
    VGT_OUT_2D_RECT                                    = 0x00000003,
    VGT_OUT_RECT_V0                                    = 0x00000004,
    VGT_OUT_DUMMY_1                                    = 0x00000005,
    VGT_OUT_DUMMY_2                                    = 0x00000006,
    VGT_OUT_DUMMY_3                                    = 0x00000007,
    VGT_OUT_PATCH                                      = 0x00000008,
    VGT_OUT_LINE_ADJ                                   = 0x00000009,
    VGT_OUT_TRI_ADJ                                    = 0x0000000a,
} VGT_OUT_PRIM_TYPE;

typedef enum VGT_RDREQ_POLICY {
    VGT_POLICY_LRU                                     = 0x00000000,
    VGT_POLICY_STREAM                                  = 0x00000001,
    VGT_POLICY_BYPASS                                  = 0x00000002,
} VGT_RDREQ_POLICY;

typedef enum VGT_SPEC_DATA_READ {
    VGT_SPEC_DATA_READ_AUTO                            = 0x00000000,
    VGT_SPEC_DATA_READ_FORCE_ON                        = 0x00000001,
    VGT_SPEC_DATA_READ_FORCE_OFF                       = 0x00000002,
} VGT_SPEC_DATA_READ;

typedef enum VGT_STAGES_GS_EN {
    GS_STAGE_OFF                                       = 0x00000000,
    GS_STAGE_ON                                        = 0x00000001,
} VGT_STAGES_GS_EN;

typedef enum VGT_STAGES_HS_EN {
    HS_STAGE_OFF                                       = 0x00000000,
    HS_STAGE_ON                                        = 0x00000001,
} VGT_STAGES_HS_EN;

typedef enum VGT_TEMPORAL {
    VGT_TEMPORAL_NORMAL                                = 0x00000000,
    VGT_TEMPORAL_STREAM                                = 0x00000001,
    VGT_TEMPORAL_HIGH_PRIORITY                         = 0x00000002,
    VGT_TEMPORAL_DISCARD                               = 0x00000003,
} VGT_TEMPORAL;

typedef enum VGT_TESS_PARTITION {
    PART_INTEGER                                       = 0x00000000,
    PART_POW2                                          = 0x00000001,
    PART_FRAC_ODD                                      = 0x00000002,
    PART_FRAC_EVEN                                     = 0x00000003,
} VGT_TESS_PARTITION;

typedef enum VGT_TESS_TOPOLOGY {
    OUTPUT_POINT                                       = 0x00000000,
    OUTPUT_LINE                                        = 0x00000001,
    OUTPUT_TRIANGLE_CW                                 = 0x00000002,
    OUTPUT_TRIANGLE_CCW                                = 0x00000003,
} VGT_TESS_TOPOLOGY;

typedef enum VGT_TESS_TYPE {
    TESS_ISOLINE                                       = 0x00000000,
    TESS_TRIANGLE                                      = 0x00000001,
    TESS_QUAD                                          = 0x00000002,
} VGT_TESS_TYPE;

typedef enum VRSCombinerModeSC {
    SC_VRS_COMB_MODE_PASSTHRU                          = 0x00000000,
    SC_VRS_COMB_MODE_OVERRIDE                          = 0x00000001,
    SC_VRS_COMB_MODE_MIN                               = 0x00000002,
    SC_VRS_COMB_MODE_MAX                               = 0x00000003,
    SC_VRS_COMB_MODE_SATURATE                          = 0x00000004,
} VRSCombinerModeSC;

typedef enum VRSrate {
    VRS_SHADING_RATE_1X1                               = 0x00000000,
    VRS_SHADING_RATE_1X2                               = 0x00000001,
    VRS_SHADING_RATE_UNDEFINED0                        = 0x00000002,
    VRS_SHADING_RATE_UNDEFINED1                        = 0x00000003,
    VRS_SHADING_RATE_2X1                               = 0x00000004,
    VRS_SHADING_RATE_2X2                               = 0x00000005,
    VRS_SHADING_RATE_2X4                               = 0x00000006,
    VRS_SHADING_RATE_UNDEFINED2                        = 0x00000007,
    VRS_SHADING_RATE_UNDEFINED3                        = 0x00000008,
    VRS_SHADING_RATE_4X2                               = 0x00000009,
    VRS_SHADING_RATE_4X4                               = 0x0000000a,
    VRS_SHADING_RATE_UNDEFINED4                        = 0x0000000b,
    VRS_SHADING_RATE_16X_SSAA                          = 0x0000000c,
    VRS_SHADING_RATE_8X_SSAA                           = 0x0000000d,
    VRS_SHADING_RATE_4X_SSAA                           = 0x0000000e,
    VRS_SHADING_RATE_2X_SSAA                           = 0x0000000f,
} VRSrate;

typedef enum WD_IA_DRAW_REG_XFER {
    WD_IA_DRAW_REG_XFER_VGT_INSTANCE_BASE_ID           = 0x00000000,
    WD_IA_DRAW_REG_XFER_VGT_MULTI_PRIM_IB_RESET_EN     = 0x00000001,
    WD_IA_DRAW_REG_XFER_VGT_GS_OUT_PRIM_TYPE           = 0x00000002,
    WD_IA_DRAW_REG_XFER_GE_CNTL                        = 0x00000003,
    WD_IA_DRAW_REG_XFER_VGT_PRIMITIVE_TYPE             = 0x00000004,
    WD_IA_DRAW_REG_XFER_GFX_PIPE_CONTROL               = 0x00000005,
    WD_IA_DRAW_REG_XFER_GE_USER_VGPR_EN                = 0x00000006,
    WD_IA_DRAW_REG_XFER_FL_MS_WG_DIM                   = 0x00000007,
    WD_IA_DRAW_REG_XFER_FL_MS_WG_DIM_1                 = 0x00000008,
    WD_IA_DRAW_REG_XFER_FL_MS_EXP_ALLOC                = 0x00000009,
    WD_IA_DRAW_REG_XFER_FL_MS_TG_SIZE                  = 0x0000000a,
    WD_IA_DRAW_REG_XFER_VGT_DRAW_PAYLOAD_CNTL          = 0x0000000b,
    WD_IA_DRAW_REG_XFER_GE_STEREO_CNTL                 = 0x0000000c,
    WD_IA_DRAW_REG_XFER_VGT_PRIMITIVEID_RESET          = 0x0000000d,
    WD_IA_DRAW_REG_XFER_VGT_PRIMITIVEID_EN             = 0x0000000e,
    WD_IA_DRAW_REG_XFER_GE_USER_VGPR1                  = 0x0000000f,
    WD_IA_DRAW_REG_XFER_GE_USER_VGPR2                  = 0x00000010,
    WD_IA_DRAW_REG_XFER_GE_USER_VGPR3                  = 0x00000011,
    WD_IA_DRAW_REG_XFER_GE_VRS_RATE                    = 0x00000012,
    WD_IA_DRAW_REG_XFER_GE_PC_ALLOC                    = 0x00000013,
    WD_IA_DRAW_REG_XFER_SPI_SHADER_GS_OUT_CONFIG_PS    = 0x00000014,
    WD_IA_DRAW_REG_XFER_GE_GS_THROTTLE                 = 0x00000015,
} WD_IA_DRAW_REG_XFER;

typedef enum WD_IA_DRAW_SOURCE {
    WD_IA_DRAW_SOURCE_DMA                              = 0x00000000,
    WD_IA_DRAW_SOURCE_IMMD                             = 0x00000001,
    WD_IA_DRAW_SOURCE_AUTO                             = 0x00000002,
    WD_IA_DRAW_SOURCE_OPAQ                             = 0x00000003,
} WD_IA_DRAW_SOURCE;

typedef enum WD_IA_DRAW_TYPE {
    WD_IA_DRAW_TYPE_DI_MM0                             = 0x00000000,
    WD_IA_DRAW_TYPE_INDX_OFF                           = 0x00000001,
    WD_IA_DRAW_TYPE_EVENT_INIT                         = 0x00000002,
    WD_IA_DRAW_TYPE_EVENT_ADDR                         = 0x00000003,
    WD_IA_DRAW_TYPE_REG_XFER                           = 0x00000004,
    WD_IA_DRAW_TYPE_MIN_INDX                           = 0x00000005,
    WD_IA_DRAW_TYPE_MAX_INDX                           = 0x00000006,
    WD_IA_DRAW_TYPE_IMM_DATA                           = 0x00000007,
} WD_IA_DRAW_TYPE;

typedef enum WGS_PERFCOUNTWINDOW_SEL {
    WGS_PERFWINDOW_SEL_MES                             = 0x00000000,
    WGS_PERFWINDOW_SEL_MEC1                            = 0x00000001,
    WGS_PERFWINDOW_SEL_MEC2                            = 0x00000002,
    WGS_PERFWINDOW_SEL_SR                              = 0x00000003,
    WGS_PERFWINDOW_SEL_WGS_IC                          = 0x00000004,
} WGS_PERFCOUNTWINDOW_SEL;

typedef enum WGS_PERFCOUNT_SEL {
    WGS_PERF_SEL_ALWAYS_COUNT                          = 0x00000000,
    WGS_PERF_SEL_RCIU_STALL_WAIT_ON_FREE               = 0x00000001,
    WGS_PERF_SEL_RCIU_STALL_PRIV_VIOLATION             = 0x00000002,
    WGS_PERF_SEL_TCIU_STALL_WAIT_ON_FREE               = 0x00000005,
    WGS_PERF_SEL_ME1_STALL_WAIT_ON_RCIU_READY          = 0x00000006,
    WGS_PERF_SEL_ME1_STALL_WAIT_ON_RCIU_READY_PERF     = 0x00000007,
    WGS_PERF_SEL_ME1_STALL_WAIT_ON_RCIU_READ           = 0x00000008,
    WGS_PERF_SEL_ME1_STALL_WAIT_ON_MEM_READ            = 0x00000009,
    WGS_PERF_SEL_ME1_STALL_WAIT_ON_MEM_WRITE           = 0x0000000a,
    WGS_PERF_SEL_ME1_STALL_ON_DATA_FROM_ROQ            = 0x0000000b,
    WGS_PERF_SEL_ME1_STALL_ON_DATA_FROM_ROQ_PERF       = 0x0000000c,
    WGS_PERF_SEL_ME1_BUSY_FOR_PACKET_DECODE            = 0x0000000d,
    WGS_PERF_SEL_ME2_STALL_WAIT_ON_RCIU_READY          = 0x0000000e,
    WGS_PERF_SEL_ME2_STALL_WAIT_ON_RCIU_READY_PERF     = 0x0000000f,
    WGS_PERF_SEL_ME2_STALL_WAIT_ON_RCIU_READ           = 0x00000010,
    WGS_PERF_SEL_ME2_STALL_WAIT_ON_MEM_READ            = 0x00000011,
    WGS_PERF_SEL_ME2_STALL_WAIT_ON_MEM_WRITE           = 0x00000012,
    WGS_PERF_SEL_ME2_STALL_ON_DATA_FROM_ROQ            = 0x00000013,
    WGS_PERF_SEL_ME2_STALL_ON_DATA_FROM_ROQ_PERF       = 0x00000014,
    WGS_PERF_SEL_ME2_BUSY_FOR_PACKET_DECODE            = 0x00000015,
    WGS_PERF_SEL_UTCL2IU_STALL_WAIT_ON_FREE            = 0x00000016,
    WGS_PERF_SEL_UTCL2IU_STALL_WAIT_ON_TAGS            = 0x00000017,
    WGS_PERF_SEL_UTCL1_STALL_ON_TRANSLATION            = 0x00000018,
    WGS_PERF_SEL_WGS_STAT_BUSY                         = 0x00000019,
    WGS_PERF_SEL_WGS_STAT_IDLE                         = 0x0000001a,
    WGS_PERF_SEL_WGS_STAT_STALL                        = 0x0000001b,
    WGS_PERF_SEL_WGS_TCIU_BUSY                         = 0x0000001c,
    WGS_PERF_SEL_WGS_TCIU_IDLE                         = 0x0000001d,
    WGS_PERF_SEL_WGS_UTCL2IU_BUSY                      = 0x0000001e,
    WGS_PERF_SEL_WGS_UTCL2IU_IDLE                      = 0x0000001f,
    WGS_PERF_SEL_WGS_UTCL2IU_STALL                     = 0x00000020,
    WGS_PERF_SEL_ME1_DC0_SPI_BUSY                      = 0x00000021,
    WGS_PERF_SEL_ME2_DC1_SPI_BUSY                      = 0x00000022,
    WGS_PERF_SEL_WGS_GCRIU_BUSY                        = 0x00000023,
    WGS_PERF_SEL_WGS_GCRIU_IDLE                        = 0x00000024,
    WGS_PERF_SEL_WGS_GCRIU_STALL                       = 0x00000025,
    WGS_PERF_SEL_GCRIU_STALL_WAIT_ON_FREE              = 0x00000026,
    WGS_PERF_SEL_ME1_STALL_WAIT_ON_TCIU_READ           = 0x00000027,
    WGS_PERF_SEL_ME2_STALL_WAIT_ON_TCIU_READ           = 0x00000028,
    WGS_PERF_SEL_WGS_UTCL2IU_XACK                      = 0x00000029,
    WGS_PERF_SEL_WGS_UTCL2IU_XNACK                     = 0x0000002a,
    WGS_PERF_SEL_MEC_INSTR_CACHE_HIT                   = 0x0000002b,
    WGS_PERF_SEL_MEC_INSTR_CACHE_MISS                  = 0x0000002c,
    WGS_PERF_SEL_MES_THREAD0                           = 0x0000002d,
    WGS_PERF_SEL_MES_THREAD1                           = 0x0000002e,
    WGS_PERF_SEL_TCIU_STALL_WAIT_ON_TAGS               = 0x0000002f,
    WGS_PERF_SEL_TCIU_WRITE_REQUEST_SENT               = 0x00000030,
    WGS_PERF_SEL_TCIU_READ_REQUEST_SENT                = 0x00000031,
    WGS_PERF_SEL_MEC_THREAD0                           = 0x00000034,
    WGS_PERF_SEL_MEC_THREAD1                           = 0x00000035,
    WGS_PERF_SEL_MEC_THREAD2                           = 0x00000036,
    WGS_PERF_SEL_MEC_THREAD3                           = 0x00000037,
} WGS_PERFCOUNT_SEL;

constexpr unsigned int MaxWgsPerfcountSel              = WGS_PERF_SEL_MEC_THREAD3;

typedef enum WGS_PERFMON_ENABLE_MODE {
    WGS_PERFMON_ENABLE_MODE_ALWAYS_COUNT               = 0x00000000,
    WGS_PERFMON_ENABLE_MODE_RESERVED_1                 = 0x00000001,
    WGS_PERFMON_ENABLE_MODE_COUNT_CONTEXT_TRUE         = 0x00000002,
    WGS_PERFMON_ENABLE_MODE_COUNT_CONTEXT_FALSE        = 0x00000003,
} WGS_PERFMON_ENABLE_MODE;

typedef enum WGS_PERFMON_STATE {
    WGS_PERFMON_STATE_DISABLE_AND_RESET                = 0x00000000,
    WGS_PERFMON_STATE_START_COUNTING                   = 0x00000001,
    WGS_PERFMON_STATE_STOP_COUNTING                    = 0x00000002,
    WGS_PERFMON_STATE_RESERVED_3                       = 0x00000003,
    WGS_PERFMON_STATE_DISABLE_AND_RESET_PHANTOM        = 0x00000004,
    WGS_PERFMON_STATE_COUNT_AND_DUMP_PHANTOM           = 0x00000005,
} WGS_PERFMON_STATE;

typedef enum WritePolicy {
    CACHE_LRU_WR                                       = 0x00000000,
    CACHE_STREAM                                       = 0x00000001,
    CACHE_NOA_WR                                       = 0x00000002,
    CACHE_BYPASS                                       = 0x00000003,
} WritePolicy;

typedef enum ZFormat {
    Z_INVALID                                          = 0x00000000,
    Z_16                                               = 0x00000001,
    Z_24                                               = 0x00000002,
    Z_32_FLOAT                                         = 0x00000003,
} ZFormat;

typedef enum ZLimitSumm {
    FORCE_SUMM_OFF                                     = 0x00000000,
    FORCE_SUMM_MINZ                                    = 0x00000001,
    FORCE_SUMM_MAXZ                                    = 0x00000002,
    FORCE_SUMM_BOTH                                    = 0x00000003,
} ZLimitSumm;

typedef enum ZModeForce {
    NO_FORCE                                           = 0x00000000,
    FORCE_EARLY_Z                                      = 0x00000001,
    FORCE_LATE_Z                                       = 0x00000002,
    FORCE_RE_Z                                         = 0x00000003,
} ZModeForce;

typedef enum ZOrder {
    LATE_Z                                             = 0x00000000,
    EARLY_Z_THEN_LATE_Z                                = 0x00000001,
    RE_Z                                               = 0x00000002,
    EARLY_Z_THEN_RE_Z                                  = 0x00000003,
} ZOrder;

typedef enum ZSamplePosition {
    Z_SAMPLE_CENTER                                    = 0x00000000,
    Z_SAMPLE_CENTROID                                  = 0x00000001,
} ZSamplePosition;

constexpr unsigned int CONFIG_SPACE1_END                                  = 0x00002bff;
constexpr unsigned int CONFIG_SPACE1_START                                = 0x00002000;
constexpr unsigned int CONFIG_SPACE2_END                                  = 0x00009fff;
constexpr unsigned int CONFIG_SPACE2_START                                = 0x00003000;
constexpr unsigned int CONFIG_SPACE_END                                   = 0x00009fff;
constexpr unsigned int CONFIG_SPACE_START                                 = 0x00002000;
constexpr unsigned int CONTEXT_SPACE_END                                  = 0x0000a3ff;
constexpr unsigned int CONTEXT_SPACE_START                                = 0x0000a000;
constexpr unsigned int CSCNTL_ADDR_WIDTH                                  = 0x00000007;
constexpr unsigned int CSCNTL_DATA_WIDTH                                  = 0x00000020;
constexpr unsigned int CSCNTL_TYPE_WIDTH                                  = 0x00000002;
constexpr unsigned int CSDATA_ADDR_WIDTH                                  = 0x00000007;
constexpr unsigned int CSDATA_DATA_WIDTH                                  = 0x00000020;
constexpr unsigned int CSDATA_TYPE_WIDTH                                  = 0x00000002;
constexpr unsigned int GSTHREADID_SIZE                                    = 0x00000002;
constexpr unsigned int INST_ID_ECC_INTERRUPT_MSG                          = 0xfffffff0;
constexpr unsigned int INST_ID_HOST_REG_TRAP_MSG                          = 0xfffffffe;
constexpr unsigned int INST_ID_HW_TRAP                                    = 0xfffffff2;
constexpr unsigned int INST_ID_HW_TRAP_GET_TBA                            = 0xfffffff5;
constexpr unsigned int INST_ID_KILL_SEQ                                   = 0xfffffff3;
constexpr unsigned int INST_ID_PRIV_START                                 = 0x80000000;
constexpr unsigned int INST_ID_SPI_WREXEC                                 = 0xfffffff4;
constexpr unsigned int INST_ID_TTRACE_NEW_PC_MSG                          = 0xfffffff1;
constexpr unsigned int IQ_DEQUEUE_RETRY                                   = 0x00000004;
constexpr unsigned int IQ_INTR_TYPE_IB                                    = 0x00000001;
constexpr unsigned int IQ_INTR_TYPE_MQD                                   = 0x00000002;
constexpr unsigned int IQ_INTR_TYPE_PQ                                    = 0x00000000;
constexpr unsigned int IQ_OFFLOAD_RETRY                                   = 0x00000001;
constexpr unsigned int IQ_QUEUE_SLEEP                                     = 0x00000000;
constexpr unsigned int IQ_SCH_WAVE_MSG                                    = 0x00000002;
constexpr unsigned int PERSISTENT_SPACE_END                               = 0x00002fff;
constexpr unsigned int PERSISTENT_SPACE_START                             = 0x00002c00;
constexpr unsigned int PFVF_SQDEC_BEGIN                                   = 0x0000a9e0;
constexpr unsigned int PFVF_SQDEC_END                                     = 0x0000a9ff;
constexpr unsigned int ROM_SIGNATURE                                      = 0x0000aa55;
constexpr unsigned int SIMM16_WAITCNT_DEPCTR_HOLD_CNT_SIZE                = 0x00000001;
constexpr unsigned int SIMM16_WAITCNT_DEPCTR_HOLD_CNT_START               = 0x00000007;
constexpr unsigned int SIMM16_WAITCNT_DEPCTR_SA_SDST_SIZE                 = 0x00000001;
constexpr unsigned int SIMM16_WAITCNT_DEPCTR_SA_SDST_START                = 0x00000000;
constexpr unsigned int SIMM16_WAITCNT_DEPCTR_VA_SDST_SIZE                 = 0x00000003;
constexpr unsigned int SIMM16_WAITCNT_DEPCTR_VA_SDST_START                = 0x00000009;
constexpr unsigned int SIMM16_WAITCNT_DEPCTR_VA_SSRC_SIZE                 = 0x00000001;
constexpr unsigned int SIMM16_WAITCNT_DEPCTR_VA_SSRC_START                = 0x00000008;
constexpr unsigned int SIMM16_WAITCNT_DEPCTR_VA_VCC_SIZE                  = 0x00000001;
constexpr unsigned int SIMM16_WAITCNT_DEPCTR_VA_VCC_START                 = 0x00000001;
constexpr unsigned int SIMM16_WAITCNT_DEPCTR_VA_VDST_SIZE                 = 0x00000004;
constexpr unsigned int SIMM16_WAITCNT_DEPCTR_VA_VDST_START                = 0x0000000c;
constexpr unsigned int SIMM16_WAITCNT_DEPCTR_VM_VSRC_SIZE                 = 0x00000003;
constexpr unsigned int SIMM16_WAITCNT_DEPCTR_VM_VSRC_START                = 0x00000002;
constexpr unsigned int SIMM16_WAITCNT_EXP_CNT_SIZE                        = 0x00000003;
constexpr unsigned int SIMM16_WAITCNT_EXP_CNT_START                       = 0x00000000;
constexpr unsigned int SIMM16_WAITCNT_LGKM_CNT_SIZE                       = 0x00000006;
constexpr unsigned int SIMM16_WAITCNT_LGKM_CNT_START                      = 0x00000004;
constexpr unsigned int SIMM16_WAITCNT_VM_CNT_SIZE                         = 0x00000006;
constexpr unsigned int SIMM16_WAITCNT_VM_CNT_START                        = 0x0000000a;
constexpr unsigned int SIMM16_WAIT_EVENT_EXP_RDY_SIZE                     = 0x00000001;
constexpr unsigned int SIMM16_WAIT_EVENT_EXP_RDY_START                    = 0x00000001;
constexpr unsigned int SQDEC_BEGIN                                        = 0x00002300;
constexpr unsigned int SQDEC_END                                          = 0x000023ff;
constexpr unsigned int SQGFXUDEC_BEGIN                                    = 0x0000c330;
constexpr unsigned int SQGFXUDEC_END                                      = 0x0000c380;
constexpr unsigned int SQIND_GLOBAL_REGS_OFFSET                           = 0x00000000;
constexpr unsigned int SQIND_GLOBAL_REGS_SIZE                             = 0x00000008;
constexpr unsigned int SQIND_LOCAL_REGS_OFFSET                            = 0x00000008;
constexpr unsigned int SQIND_LOCAL_REGS_SIZE                              = 0x00000008;
constexpr unsigned int SQIND_WAVE_HOST_REGS_OFFSET                        = 0x00000140;
constexpr unsigned int SQIND_WAVE_HOST_REGS_SIZE                          = 0x000000c0;
constexpr unsigned int SQIND_WAVE_HW_REGS_OFFSET                          = 0x00000100;
constexpr unsigned int SQIND_WAVE_HW_REGS_SIZE                            = 0x00000040;
constexpr unsigned int SQIND_WAVE_SGPRS_OFFSET                            = 0x00000200;
constexpr unsigned int SQIND_WAVE_SGPRS_SIZE                              = 0x00000200;
constexpr unsigned int SQIND_WAVE_VGPRS_OFFSET                            = 0x00000400;
constexpr unsigned int SQIND_WAVE_VGPRS_SIZE                              = 0x00000400;
constexpr unsigned int SQPERFDDEC_BEGIN                                   = 0x0000d1c0;
constexpr unsigned int SQPERFDDEC_END                                     = 0x0000d240;
constexpr unsigned int SQPERFSDEC_BEGIN                                   = 0x0000d9c0;
constexpr unsigned int SQPERFSDEC_END                                     = 0x0000da40;
constexpr unsigned int SQPWRDEC_BEGIN                                     = 0x0000f08c;
constexpr unsigned int SQPWRDEC_END                                       = 0x0000f094;
constexpr unsigned int SQ_ARB_STATE_ISSUED_BRMSG                          = 0x00000000;
constexpr unsigned int SQ_ARB_STATE_ISSUED_EXPORT                         = 0x00000001;
constexpr unsigned int SQ_ARB_STATE_ISSUED_LDS                            = 0x00000003;
constexpr unsigned int SQ_ARB_STATE_ISSUED_LDS_DIRECT                     = 0x00000002;
constexpr unsigned int SQ_ARB_STATE_ISSUED_SCALAR                         = 0x00000005;
constexpr unsigned int SQ_ARB_STATE_ISSUED_TEX                            = 0x00000004;
constexpr unsigned int SQ_ARB_STATE_ISSUED_VALU                           = 0x00000006;
constexpr unsigned int SQ_ARB_STATE_STALLED_BRMSG                         = 0x00000008;
constexpr unsigned int SQ_ARB_STATE_STALLED_EXPORT                        = 0x00000009;
constexpr unsigned int SQ_ARB_STATE_STALLED_LDS                           = 0x0000000b;
constexpr unsigned int SQ_ARB_STATE_STALLED_LDS_DIRECT                    = 0x0000000a;
constexpr unsigned int SQ_ARB_STATE_STALLED_SCALAR                        = 0x0000000d;
constexpr unsigned int SQ_ARB_STATE_STALLED_TEX                           = 0x0000000c;
constexpr unsigned int SQ_ARB_STATE_STALLED_VALU                          = 0x0000000e;
constexpr unsigned int SQ_DISPATCHER_GFX_CNT_PER_RING                     = 0x00000008;
constexpr unsigned int SQ_DISPATCHER_GFX_MIN                              = 0x00000010;
constexpr unsigned int SQ_EDC_FUE_CNTL_LDS                                = 0x00000005;
constexpr unsigned int SQ_EDC_FUE_CNTL_SIMD0                              = 0x00000000;
constexpr unsigned int SQ_EDC_FUE_CNTL_SIMD1                              = 0x00000001;
constexpr unsigned int SQ_EDC_FUE_CNTL_SIMD2                              = 0x00000002;
constexpr unsigned int SQ_EDC_FUE_CNTL_SIMD3                              = 0x00000003;
constexpr unsigned int SQ_EDC_FUE_CNTL_SQ                                 = 0x00000004;
constexpr unsigned int SQ_EDC_FUE_CNTL_TA                                 = 0x00000007;
constexpr unsigned int SQ_EDC_FUE_CNTL_TCP                                = 0x00000008;
constexpr unsigned int SQ_EDC_FUE_CNTL_TD                                 = 0x00000006;
constexpr unsigned int SQ_EX_EXCP_ADDR_WATCH                              = 0x00000007;
constexpr unsigned int SQ_EX_EXCP_ALU_FLOAT_DIV0                          = 0x00000002;
constexpr unsigned int SQ_EX_EXCP_ALU_INEXACT                             = 0x00000005;
constexpr unsigned int SQ_EX_EXCP_ALU_INPUT_DENORM                        = 0x00000001;
constexpr unsigned int SQ_EX_EXCP_ALU_INT_DIV0                            = 0x00000006;
constexpr unsigned int SQ_EX_EXCP_ALU_INVALID                             = 0x00000000;
constexpr unsigned int SQ_EX_EXCP_ALU_OVERFLOW                            = 0x00000003;
constexpr unsigned int SQ_EX_EXCP_ALU_UNDERFLOW                           = 0x00000004;
constexpr unsigned int SQ_EX_EXCP_VALU_BASE                               = 0x00000000;
constexpr unsigned int SQ_EX_EXCP_VALU_SIZE                               = 0x00000007;
constexpr unsigned int SQ_FLAT                                            = 0x00000000;
constexpr unsigned int SQ_GFXDEC_BEGIN                                    = 0x0000a000;
constexpr unsigned int SQ_GFXDEC_END                                      = 0x0000c000;
constexpr unsigned int SQ_GFXDEC_STATE_ID_SHIFT                           = 0x0000000a;
constexpr unsigned int SQ_GLOBAL                                          = 0x00000002;
constexpr unsigned int SQ_MAX_PGM_SGPRS                                   = 0x00000068;
constexpr unsigned int SQ_MAX_PGM_VGPRS                                   = 0x00000100;
constexpr unsigned int SQ_SCRATCH                                         = 0x00000001;
constexpr unsigned int SQ_WAVE_IB_DEP_HOLD_CNT_SIZE                       = 0x00000001;
constexpr unsigned int SQ_WAVE_IB_DEP_LDS_DIR_SIZE                        = 0x00000003;
constexpr unsigned int SQ_WAVE_IB_DEP_SA_EXEC_SIZE                        = 0x00000002;
constexpr unsigned int SQ_WAVE_IB_DEP_SA_M0_SIZE                          = 0x00000001;
constexpr unsigned int SQ_WAVE_IB_DEP_SA_SDST_SIZE                        = 0x00000004;
constexpr unsigned int SQ_WAVE_IB_DEP_VA_EXEC_SIZE                        = 0x00000002;
constexpr unsigned int SQ_WAVE_IB_DEP_VA_SDST_SIZE                        = 0x00000004;
constexpr unsigned int SQ_WAVE_IB_DEP_VA_SSRC_SIZE                        = 0x00000003;
constexpr unsigned int SQ_WAVE_IB_DEP_VA_VCC_SIZE                         = 0x00000003;
constexpr unsigned int SQ_WAVE_IB_DEP_VA_VDST_SIZE                        = 0x00000005;
constexpr unsigned int SQ_WAVE_IB_DEP_VM_VSRC_SIZE                        = 0x00000004;
constexpr unsigned int SQ_WAVE_TYPE_PS0                                   = 0x00000000;
constexpr unsigned int SRCID_NONSECURE_CP                                 = 0x00000001;
constexpr unsigned int SRCID_NONSECURE_CP_RCIU                            = 0x00000001;
constexpr unsigned int SRCID_RLC                                          = 0x00000000;
constexpr unsigned int SRCID_RLCV                                         = 0x00000006;
constexpr unsigned int SRCID_SECURE_CP                                    = 0x00000007;
constexpr unsigned int SRCID_SECURE_CP_RCIU                               = 0x00000007;
constexpr unsigned int UCONFIG_SPACE_END                                  = 0x0000ffff;
constexpr unsigned int UCONFIG_SPACE_START                                = 0x0000c000;
constexpr unsigned int VMID_SZ                                            = 0x00000004;

enum PerfCtrId
{
    CBPerfSelId                              = 0,
    ChaPerfSelId                             = 1,
    ChcPerfSelId                             = 2,
    CpcPerfcountSelId                        = 3,
    CpfPerfcountSelId                        = 4,
    CpgPerfcountSelId                        = 5,
    DfMallPerfSelId                          = 6,
    GCRPerfSelId                             = 7,
    GcEaCpwdPerfcountSelId                   = 8,
    GcEaSePerfcountSelId                     = 9,
    Gcvml2PerfSelId                          = 10,
    Ge1PerfcountSelectId                     = 11,
    Ge2DistPerfcountSelectId                 = 12,
    Ge2SePerfcountSelectId                   = 13,
    Gl1aPerfSelId                            = 14,
    Gl1cPerfSelId                            = 15,
    Gl1xaPerfSelId                           = 16,
    Gl1xcPerfSelId                           = 17,
    Gl2aPerfSelId                            = 18,
    Gl2cPerfSelId                            = 19,
    GrbmPerfSelId                            = 20,
    GrbmhPerfSelId                           = 21,
    PcPerfcntSelId                           = 22,
    PerfcounterValsId                        = 23,
    PhPerfcntSelId                           = 24,
    RMIPerfSelId                             = 25,
    RlcPerfcounterSelId                      = 26,
    ScPerfcntSelId                           = 27,
    SdmaPerfmonSelId                         = 28,
    SpiPerfcntSelId                          = 29,
    SqPerfSelId                              = 30,
    SqgPerfSelId                             = 31,
    SuPerfcntSelId                           = 32,
    SxPerfcounterValsId                      = 33,
    TaPerfcountSelId                         = 34,
    TcpPerfcountSelectId                     = 35,
    TdPerfcountSelId                         = 36,
    UTCL1PerfSelId                           = 37,
    UmcPerfcountSelectId                     = 38,
    WgsPerfcountSelId                        = 39,
    MaxPerfCtrId                             = 40,
};

#if CHIP_HDR_NAVI48
constexpr unsigned int Nv48MaxPerfEventIds[MaxPerfCtrId] =
{
    CB_PERF_SEL_EXPORT_KILLED_BY_NULL_TARGET_SHADER_MASK,
    CHA_PERF_SEL_CYCLE__NV48,
    CHC_PERF_SEL_OC_BYPASS_REQ,
    CPC_PERF_SEL_MEC_THREAD3,
    CPF_PERF_SEL_CP_SDMA_MNGR_SDMABUSY,
    CPG_PERF_SEL_ME_RS64_THREAD1,
    DF_MALL_PERF_SEL_MALL_SDP_LAT_HIST_GT1000,
    GCR_PERF_SEL_PIO_GL1_TLB_SHOOTDOWN_REQ,
    GC_EA_CPWD_PERF_SEL_MAM_AFLUSH_ONGOING,
    GC_EA_SE_PERF_SEL_MAM_AFLUSH_ONGOING,
    GCVML2_PERF_SEL_EVENT_90,
    ge_ia_asm_out_valid_prim_count,
    ge_wd_phmq_fsm_starved_se8,
    ge_se_hs_stall_offchip_unavail,
    GL1A_PERF_SEL_CYCLE,
    GL1C_PERF_SEL_UTCL0_GPA3_REQUEST,
    GL1XA_PERF_SEL_CYCLE,
    GL1XC_PERF_SEL_UTCL0_GPA3_REQUEST,
    GL2A_PERF_SEL_REQ_HI_PRIO_GL2C7,
    GL2C_PERF_SEL_INPUT_DECOMPRESS_256B_REQ,
    GRBM_PERF_SEL_WGS_BUSY,
    GRBMH_PERF_SEL_RLC_BUSY,
    PC_PERF_NUM_PSWAVE,
    DB_PERF_SEL_osb_TT_blocked,
    PH_PERF_SC7_FIFO_STATUS_3,
    RMI_PERF_SEL_CONSUMER_PROBEGEN_DB_RTS_RTR,
    RLC_PERF_SEL_SERDES_COMMAND_WRITE,
    SC_HZC_BUSY,
    SDMA_PERFMON_SEL_GCR_RTN,
    SPI_PERF_GLG_NUM_MRT_ACTIVATE,
    SQ_PERF_SEL_NONE2,
    SQG_PERF_SEL_DUMMY_LAST,
    PERF_PAB_POS_BANK_CONFLICT,
    SX_PERF_SEL_DB3_4X2_DISCARD,
    TA_PERF_SEL_image_sample_aniso_miplevel_per_quad,
    TCP_PERF_SEL_REQ_WRITE_MISS_INVAL,
    TD_PERF_SEL_ray_tracing_intersectable_instance_valid_children,
    UTCL1_PERF_SEL_ALOG_STALL_PMM_CREDITS,
    UMC_PERF_SEL_ClockCount,
    WGS_PERF_SEL_MEC_THREAD3,
};
#endif
} // inline namespace Chip
} // namespace Gfx12
} // namespace Pal
