/*
 ***********************************************************************************************************************
 *
 *  Copyright (c) 2025 Advanced Micro Devices, Inc. All Rights Reserved.
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in all
 *  copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 *  SOFTWARE.
 *
 **********************************************************************************************************************/

#pragma once

//
// Make sure the necessary endian defines are there.
//
#ifndef LITTLEENDIAN_CPU
#error "LITTLEENDIAN_CPU must be defined"
#endif

namespace Pal
{
namespace Gfx12
{
inline namespace Chip
{
union CB_BLEND0_CONTROL {
    struct {
        unsigned int COLOR_SRCBLEND                                               :  5;
        unsigned int COLOR_COMB_FCN                                               :  3;
        unsigned int COLOR_DESTBLEND                                              :  5;
        unsigned int                                                              :  3;
        unsigned int ALPHA_SRCBLEND                                               :  5;
        unsigned int ALPHA_COMB_FCN                                               :  3;
        unsigned int ALPHA_DESTBLEND                                              :  5;
        unsigned int SEPARATE_ALPHA_BLEND                                         :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int DISABLE_ROP3                                                 :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_BLEND1_CONTROL {
    struct {
        unsigned int COLOR_SRCBLEND                                               :  5;
        unsigned int COLOR_COMB_FCN                                               :  3;
        unsigned int COLOR_DESTBLEND                                              :  5;
        unsigned int                                                              :  3;
        unsigned int ALPHA_SRCBLEND                                               :  5;
        unsigned int ALPHA_COMB_FCN                                               :  3;
        unsigned int ALPHA_DESTBLEND                                              :  5;
        unsigned int SEPARATE_ALPHA_BLEND                                         :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int DISABLE_ROP3                                                 :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_BLEND2_CONTROL {
    struct {
        unsigned int COLOR_SRCBLEND                                               :  5;
        unsigned int COLOR_COMB_FCN                                               :  3;
        unsigned int COLOR_DESTBLEND                                              :  5;
        unsigned int                                                              :  3;
        unsigned int ALPHA_SRCBLEND                                               :  5;
        unsigned int ALPHA_COMB_FCN                                               :  3;
        unsigned int ALPHA_DESTBLEND                                              :  5;
        unsigned int SEPARATE_ALPHA_BLEND                                         :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int DISABLE_ROP3                                                 :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_BLEND3_CONTROL {
    struct {
        unsigned int COLOR_SRCBLEND                                               :  5;
        unsigned int COLOR_COMB_FCN                                               :  3;
        unsigned int COLOR_DESTBLEND                                              :  5;
        unsigned int                                                              :  3;
        unsigned int ALPHA_SRCBLEND                                               :  5;
        unsigned int ALPHA_COMB_FCN                                               :  3;
        unsigned int ALPHA_DESTBLEND                                              :  5;
        unsigned int SEPARATE_ALPHA_BLEND                                         :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int DISABLE_ROP3                                                 :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_BLEND4_CONTROL {
    struct {
        unsigned int COLOR_SRCBLEND                                               :  5;
        unsigned int COLOR_COMB_FCN                                               :  3;
        unsigned int COLOR_DESTBLEND                                              :  5;
        unsigned int                                                              :  3;
        unsigned int ALPHA_SRCBLEND                                               :  5;
        unsigned int ALPHA_COMB_FCN                                               :  3;
        unsigned int ALPHA_DESTBLEND                                              :  5;
        unsigned int SEPARATE_ALPHA_BLEND                                         :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int DISABLE_ROP3                                                 :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_BLEND5_CONTROL {
    struct {
        unsigned int COLOR_SRCBLEND                                               :  5;
        unsigned int COLOR_COMB_FCN                                               :  3;
        unsigned int COLOR_DESTBLEND                                              :  5;
        unsigned int                                                              :  3;
        unsigned int ALPHA_SRCBLEND                                               :  5;
        unsigned int ALPHA_COMB_FCN                                               :  3;
        unsigned int ALPHA_DESTBLEND                                              :  5;
        unsigned int SEPARATE_ALPHA_BLEND                                         :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int DISABLE_ROP3                                                 :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_BLEND6_CONTROL {
    struct {
        unsigned int COLOR_SRCBLEND                                               :  5;
        unsigned int COLOR_COMB_FCN                                               :  3;
        unsigned int COLOR_DESTBLEND                                              :  5;
        unsigned int                                                              :  3;
        unsigned int ALPHA_SRCBLEND                                               :  5;
        unsigned int ALPHA_COMB_FCN                                               :  3;
        unsigned int ALPHA_DESTBLEND                                              :  5;
        unsigned int SEPARATE_ALPHA_BLEND                                         :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int DISABLE_ROP3                                                 :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_BLEND7_CONTROL {
    struct {
        unsigned int COLOR_SRCBLEND                                               :  5;
        unsigned int COLOR_COMB_FCN                                               :  3;
        unsigned int COLOR_DESTBLEND                                              :  5;
        unsigned int                                                              :  3;
        unsigned int ALPHA_SRCBLEND                                               :  5;
        unsigned int ALPHA_COMB_FCN                                               :  3;
        unsigned int ALPHA_DESTBLEND                                              :  5;
        unsigned int SEPARATE_ALPHA_BLEND                                         :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int DISABLE_ROP3                                                 :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_BLEND_ALPHA {
    struct {
        unsigned int BLEND_ALPHA                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_BLEND_BLUE {
    struct {
        unsigned int BLEND_BLUE                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_BLEND_GREEN {
    struct {
        unsigned int BLEND_GREEN                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_BLEND_RED {
    struct {
        unsigned int BLEND_RED                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_CACHE_EVICT_POINTS {
    struct {
        unsigned int CC_COLOR_EVICT_POINT                                         :  8;
        unsigned int CC_FMASK_EVICT_POINT                                         :  8;
        unsigned int                                                              :  8;
        unsigned int CC_CACHE_EVICT_POINT                                         :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR0_ATTRIB {
    struct {
        unsigned int NUM_FRAGMENTS                                                :  2;
        unsigned int FORCE_DST_ALPHA_1                                            :  1;
        unsigned int FORCE_LIMIT_COLOR_SECTOR_TO_256B_MAX                         :  1;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR0_ATTRIB2 {
    struct {
        unsigned int MIP0_HEIGHT                                                  : 16;
        unsigned int MIP0_WIDTH                                                   : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR0_ATTRIB3 {
    struct {
        unsigned int MIP0_DEPTH                                                   : 14;
        unsigned int                                                              :  1;
        unsigned int COLOR_SW_MODE                                                :  3;
        unsigned int                                                              :  1;
        unsigned int MAX_MIP                                                      :  5;
        unsigned int RESOURCE_TYPE                                                :  2;
        unsigned int SPECULATIVE_READ                                             :  2;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR0_BASE {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR0_BASE_EXT {
    struct {
        unsigned int BASE_256B                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR0_FDCC_CONTROL {
    struct {
        unsigned int                                                              :  2;
        unsigned int MAX_UNCOMPRESSED_BLOCK_SIZE                                  :  1;
        unsigned int                                                              :  2;
        unsigned int MAX_COMPRESSED_BLOCK_SIZE                                    :  2;
        unsigned int                                                              : 17;
        unsigned int FRAGMENT_COMPRESS_DISABLE                                    :  1;
        unsigned int                                                              :  1;
        unsigned int COMPRESSION_MODE                                             :  2;
        unsigned int ENABLE_MAX_COMP_FRAG_OVERRIDE                                :  1;
        unsigned int MAX_COMP_FRAGS                                               :  2;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR0_INFO {
    struct {
        unsigned int FORMAT                                                       :  5;
        unsigned int                                                              :  2;
        unsigned int LINEAR_GENERAL                                               :  1;
        unsigned int NUMBER_TYPE                                                  :  3;
        unsigned int COMP_SWAP                                                    :  2;
        unsigned int                                                              :  2;
        unsigned int BLEND_CLAMP                                                  :  1;
        unsigned int BLEND_BYPASS                                                 :  1;
        unsigned int SIMPLE_FLOAT                                                 :  1;
        unsigned int ROUND_MODE                                                   :  1;
        unsigned int DISABLE_WA_FOR_PARTIAL_TARGET_MASK_ALL                       :  1;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR0_VIEW {
    struct {
        unsigned int SLICE_START                                                  : 14;
        unsigned int SLICE_MAX                                                    : 14;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR0_VIEW2 {
    struct {
        unsigned int MIP_LEVEL                                                    :  5;
        unsigned int                                                              : 27;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR1_ATTRIB {
    struct {
        unsigned int NUM_FRAGMENTS                                                :  2;
        unsigned int FORCE_DST_ALPHA_1                                            :  1;
        unsigned int FORCE_LIMIT_COLOR_SECTOR_TO_256B_MAX                         :  1;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR1_ATTRIB2 {
    struct {
        unsigned int MIP0_HEIGHT                                                  : 16;
        unsigned int MIP0_WIDTH                                                   : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR1_ATTRIB3 {
    struct {
        unsigned int MIP0_DEPTH                                                   : 14;
        unsigned int                                                              :  1;
        unsigned int COLOR_SW_MODE                                                :  3;
        unsigned int                                                              :  1;
        unsigned int MAX_MIP                                                      :  5;
        unsigned int RESOURCE_TYPE                                                :  2;
        unsigned int SPECULATIVE_READ                                             :  2;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR1_BASE {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR1_BASE_EXT {
    struct {
        unsigned int BASE_256B                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR1_FDCC_CONTROL {
    struct {
        unsigned int                                                              :  2;
        unsigned int MAX_UNCOMPRESSED_BLOCK_SIZE                                  :  1;
        unsigned int                                                              :  2;
        unsigned int MAX_COMPRESSED_BLOCK_SIZE                                    :  2;
        unsigned int                                                              : 17;
        unsigned int FRAGMENT_COMPRESS_DISABLE                                    :  1;
        unsigned int                                                              :  1;
        unsigned int COMPRESSION_MODE                                             :  2;
        unsigned int ENABLE_MAX_COMP_FRAG_OVERRIDE                                :  1;
        unsigned int MAX_COMP_FRAGS                                               :  2;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR1_INFO {
    struct {
        unsigned int FORMAT                                                       :  5;
        unsigned int                                                              :  2;
        unsigned int LINEAR_GENERAL                                               :  1;
        unsigned int NUMBER_TYPE                                                  :  3;
        unsigned int COMP_SWAP                                                    :  2;
        unsigned int                                                              :  2;
        unsigned int BLEND_CLAMP                                                  :  1;
        unsigned int BLEND_BYPASS                                                 :  1;
        unsigned int SIMPLE_FLOAT                                                 :  1;
        unsigned int ROUND_MODE                                                   :  1;
        unsigned int DISABLE_WA_FOR_PARTIAL_TARGET_MASK_ALL                       :  1;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR1_VIEW {
    struct {
        unsigned int SLICE_START                                                  : 14;
        unsigned int SLICE_MAX                                                    : 14;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR1_VIEW2 {
    struct {
        unsigned int MIP_LEVEL                                                    :  5;
        unsigned int                                                              : 27;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR2_ATTRIB {
    struct {
        unsigned int NUM_FRAGMENTS                                                :  2;
        unsigned int FORCE_DST_ALPHA_1                                            :  1;
        unsigned int FORCE_LIMIT_COLOR_SECTOR_TO_256B_MAX                         :  1;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR2_ATTRIB2 {
    struct {
        unsigned int MIP0_HEIGHT                                                  : 16;
        unsigned int MIP0_WIDTH                                                   : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR2_ATTRIB3 {
    struct {
        unsigned int MIP0_DEPTH                                                   : 14;
        unsigned int                                                              :  1;
        unsigned int COLOR_SW_MODE                                                :  3;
        unsigned int                                                              :  1;
        unsigned int MAX_MIP                                                      :  5;
        unsigned int RESOURCE_TYPE                                                :  2;
        unsigned int SPECULATIVE_READ                                             :  2;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR2_BASE {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR2_BASE_EXT {
    struct {
        unsigned int BASE_256B                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR2_FDCC_CONTROL {
    struct {
        unsigned int                                                              :  2;
        unsigned int MAX_UNCOMPRESSED_BLOCK_SIZE                                  :  1;
        unsigned int                                                              :  2;
        unsigned int MAX_COMPRESSED_BLOCK_SIZE                                    :  2;
        unsigned int                                                              : 17;
        unsigned int FRAGMENT_COMPRESS_DISABLE                                    :  1;
        unsigned int                                                              :  1;
        unsigned int COMPRESSION_MODE                                             :  2;
        unsigned int ENABLE_MAX_COMP_FRAG_OVERRIDE                                :  1;
        unsigned int MAX_COMP_FRAGS                                               :  2;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR2_INFO {
    struct {
        unsigned int FORMAT                                                       :  5;
        unsigned int                                                              :  2;
        unsigned int LINEAR_GENERAL                                               :  1;
        unsigned int NUMBER_TYPE                                                  :  3;
        unsigned int COMP_SWAP                                                    :  2;
        unsigned int                                                              :  2;
        unsigned int BLEND_CLAMP                                                  :  1;
        unsigned int BLEND_BYPASS                                                 :  1;
        unsigned int SIMPLE_FLOAT                                                 :  1;
        unsigned int ROUND_MODE                                                   :  1;
        unsigned int DISABLE_WA_FOR_PARTIAL_TARGET_MASK_ALL                       :  1;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR2_VIEW {
    struct {
        unsigned int SLICE_START                                                  : 14;
        unsigned int SLICE_MAX                                                    : 14;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR2_VIEW2 {
    struct {
        unsigned int MIP_LEVEL                                                    :  5;
        unsigned int                                                              : 27;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR3_ATTRIB {
    struct {
        unsigned int NUM_FRAGMENTS                                                :  2;
        unsigned int FORCE_DST_ALPHA_1                                            :  1;
        unsigned int FORCE_LIMIT_COLOR_SECTOR_TO_256B_MAX                         :  1;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR3_ATTRIB2 {
    struct {
        unsigned int MIP0_HEIGHT                                                  : 16;
        unsigned int MIP0_WIDTH                                                   : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR3_ATTRIB3 {
    struct {
        unsigned int MIP0_DEPTH                                                   : 14;
        unsigned int                                                              :  1;
        unsigned int COLOR_SW_MODE                                                :  3;
        unsigned int                                                              :  1;
        unsigned int MAX_MIP                                                      :  5;
        unsigned int RESOURCE_TYPE                                                :  2;
        unsigned int SPECULATIVE_READ                                             :  2;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR3_BASE {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR3_BASE_EXT {
    struct {
        unsigned int BASE_256B                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR3_FDCC_CONTROL {
    struct {
        unsigned int                                                              :  2;
        unsigned int MAX_UNCOMPRESSED_BLOCK_SIZE                                  :  1;
        unsigned int                                                              :  2;
        unsigned int MAX_COMPRESSED_BLOCK_SIZE                                    :  2;
        unsigned int                                                              : 17;
        unsigned int FRAGMENT_COMPRESS_DISABLE                                    :  1;
        unsigned int                                                              :  1;
        unsigned int COMPRESSION_MODE                                             :  2;
        unsigned int ENABLE_MAX_COMP_FRAG_OVERRIDE                                :  1;
        unsigned int MAX_COMP_FRAGS                                               :  2;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR3_INFO {
    struct {
        unsigned int FORMAT                                                       :  5;
        unsigned int                                                              :  2;
        unsigned int LINEAR_GENERAL                                               :  1;
        unsigned int NUMBER_TYPE                                                  :  3;
        unsigned int COMP_SWAP                                                    :  2;
        unsigned int                                                              :  2;
        unsigned int BLEND_CLAMP                                                  :  1;
        unsigned int BLEND_BYPASS                                                 :  1;
        unsigned int SIMPLE_FLOAT                                                 :  1;
        unsigned int ROUND_MODE                                                   :  1;
        unsigned int DISABLE_WA_FOR_PARTIAL_TARGET_MASK_ALL                       :  1;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR3_VIEW {
    struct {
        unsigned int SLICE_START                                                  : 14;
        unsigned int SLICE_MAX                                                    : 14;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR3_VIEW2 {
    struct {
        unsigned int MIP_LEVEL                                                    :  5;
        unsigned int                                                              : 27;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR4_ATTRIB {
    struct {
        unsigned int NUM_FRAGMENTS                                                :  2;
        unsigned int FORCE_DST_ALPHA_1                                            :  1;
        unsigned int FORCE_LIMIT_COLOR_SECTOR_TO_256B_MAX                         :  1;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR4_ATTRIB2 {
    struct {
        unsigned int MIP0_HEIGHT                                                  : 16;
        unsigned int MIP0_WIDTH                                                   : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR4_ATTRIB3 {
    struct {
        unsigned int MIP0_DEPTH                                                   : 14;
        unsigned int                                                              :  1;
        unsigned int COLOR_SW_MODE                                                :  3;
        unsigned int                                                              :  1;
        unsigned int MAX_MIP                                                      :  5;
        unsigned int RESOURCE_TYPE                                                :  2;
        unsigned int SPECULATIVE_READ                                             :  2;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR4_BASE {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR4_BASE_EXT {
    struct {
        unsigned int BASE_256B                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR4_FDCC_CONTROL {
    struct {
        unsigned int                                                              :  2;
        unsigned int MAX_UNCOMPRESSED_BLOCK_SIZE                                  :  1;
        unsigned int                                                              :  2;
        unsigned int MAX_COMPRESSED_BLOCK_SIZE                                    :  2;
        unsigned int                                                              : 17;
        unsigned int FRAGMENT_COMPRESS_DISABLE                                    :  1;
        unsigned int                                                              :  1;
        unsigned int COMPRESSION_MODE                                             :  2;
        unsigned int ENABLE_MAX_COMP_FRAG_OVERRIDE                                :  1;
        unsigned int MAX_COMP_FRAGS                                               :  2;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR4_INFO {
    struct {
        unsigned int FORMAT                                                       :  5;
        unsigned int                                                              :  2;
        unsigned int LINEAR_GENERAL                                               :  1;
        unsigned int NUMBER_TYPE                                                  :  3;
        unsigned int COMP_SWAP                                                    :  2;
        unsigned int                                                              :  2;
        unsigned int BLEND_CLAMP                                                  :  1;
        unsigned int BLEND_BYPASS                                                 :  1;
        unsigned int SIMPLE_FLOAT                                                 :  1;
        unsigned int ROUND_MODE                                                   :  1;
        unsigned int DISABLE_WA_FOR_PARTIAL_TARGET_MASK_ALL                       :  1;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR4_VIEW {
    struct {
        unsigned int SLICE_START                                                  : 14;
        unsigned int SLICE_MAX                                                    : 14;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR4_VIEW2 {
    struct {
        unsigned int MIP_LEVEL                                                    :  5;
        unsigned int                                                              : 27;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR5_ATTRIB {
    struct {
        unsigned int NUM_FRAGMENTS                                                :  2;
        unsigned int FORCE_DST_ALPHA_1                                            :  1;
        unsigned int FORCE_LIMIT_COLOR_SECTOR_TO_256B_MAX                         :  1;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR5_ATTRIB2 {
    struct {
        unsigned int MIP0_HEIGHT                                                  : 16;
        unsigned int MIP0_WIDTH                                                   : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR5_ATTRIB3 {
    struct {
        unsigned int MIP0_DEPTH                                                   : 14;
        unsigned int                                                              :  1;
        unsigned int COLOR_SW_MODE                                                :  3;
        unsigned int                                                              :  1;
        unsigned int MAX_MIP                                                      :  5;
        unsigned int RESOURCE_TYPE                                                :  2;
        unsigned int SPECULATIVE_READ                                             :  2;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR5_BASE {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR5_BASE_EXT {
    struct {
        unsigned int BASE_256B                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR5_FDCC_CONTROL {
    struct {
        unsigned int                                                              :  2;
        unsigned int MAX_UNCOMPRESSED_BLOCK_SIZE                                  :  1;
        unsigned int                                                              :  2;
        unsigned int MAX_COMPRESSED_BLOCK_SIZE                                    :  2;
        unsigned int                                                              : 17;
        unsigned int FRAGMENT_COMPRESS_DISABLE                                    :  1;
        unsigned int                                                              :  1;
        unsigned int COMPRESSION_MODE                                             :  2;
        unsigned int ENABLE_MAX_COMP_FRAG_OVERRIDE                                :  1;
        unsigned int MAX_COMP_FRAGS                                               :  2;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR5_INFO {
    struct {
        unsigned int FORMAT                                                       :  5;
        unsigned int                                                              :  2;
        unsigned int LINEAR_GENERAL                                               :  1;
        unsigned int NUMBER_TYPE                                                  :  3;
        unsigned int COMP_SWAP                                                    :  2;
        unsigned int                                                              :  2;
        unsigned int BLEND_CLAMP                                                  :  1;
        unsigned int BLEND_BYPASS                                                 :  1;
        unsigned int SIMPLE_FLOAT                                                 :  1;
        unsigned int ROUND_MODE                                                   :  1;
        unsigned int DISABLE_WA_FOR_PARTIAL_TARGET_MASK_ALL                       :  1;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR5_VIEW {
    struct {
        unsigned int SLICE_START                                                  : 14;
        unsigned int SLICE_MAX                                                    : 14;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR5_VIEW2 {
    struct {
        unsigned int MIP_LEVEL                                                    :  5;
        unsigned int                                                              : 27;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR6_ATTRIB {
    struct {
        unsigned int NUM_FRAGMENTS                                                :  2;
        unsigned int FORCE_DST_ALPHA_1                                            :  1;
        unsigned int FORCE_LIMIT_COLOR_SECTOR_TO_256B_MAX                         :  1;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR6_ATTRIB2 {
    struct {
        unsigned int MIP0_HEIGHT                                                  : 16;
        unsigned int MIP0_WIDTH                                                   : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR6_ATTRIB3 {
    struct {
        unsigned int MIP0_DEPTH                                                   : 14;
        unsigned int                                                              :  1;
        unsigned int COLOR_SW_MODE                                                :  3;
        unsigned int                                                              :  1;
        unsigned int MAX_MIP                                                      :  5;
        unsigned int RESOURCE_TYPE                                                :  2;
        unsigned int SPECULATIVE_READ                                             :  2;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR6_BASE {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR6_BASE_EXT {
    struct {
        unsigned int BASE_256B                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR6_FDCC_CONTROL {
    struct {
        unsigned int                                                              :  2;
        unsigned int MAX_UNCOMPRESSED_BLOCK_SIZE                                  :  1;
        unsigned int                                                              :  2;
        unsigned int MAX_COMPRESSED_BLOCK_SIZE                                    :  2;
        unsigned int                                                              : 17;
        unsigned int FRAGMENT_COMPRESS_DISABLE                                    :  1;
        unsigned int                                                              :  1;
        unsigned int COMPRESSION_MODE                                             :  2;
        unsigned int ENABLE_MAX_COMP_FRAG_OVERRIDE                                :  1;
        unsigned int MAX_COMP_FRAGS                                               :  2;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR6_INFO {
    struct {
        unsigned int FORMAT                                                       :  5;
        unsigned int                                                              :  2;
        unsigned int LINEAR_GENERAL                                               :  1;
        unsigned int NUMBER_TYPE                                                  :  3;
        unsigned int COMP_SWAP                                                    :  2;
        unsigned int                                                              :  2;
        unsigned int BLEND_CLAMP                                                  :  1;
        unsigned int BLEND_BYPASS                                                 :  1;
        unsigned int SIMPLE_FLOAT                                                 :  1;
        unsigned int ROUND_MODE                                                   :  1;
        unsigned int DISABLE_WA_FOR_PARTIAL_TARGET_MASK_ALL                       :  1;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR6_VIEW {
    struct {
        unsigned int SLICE_START                                                  : 14;
        unsigned int SLICE_MAX                                                    : 14;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR6_VIEW2 {
    struct {
        unsigned int MIP_LEVEL                                                    :  5;
        unsigned int                                                              : 27;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR7_ATTRIB {
    struct {
        unsigned int NUM_FRAGMENTS                                                :  2;
        unsigned int FORCE_DST_ALPHA_1                                            :  1;
        unsigned int FORCE_LIMIT_COLOR_SECTOR_TO_256B_MAX                         :  1;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR7_ATTRIB2 {
    struct {
        unsigned int MIP0_HEIGHT                                                  : 16;
        unsigned int MIP0_WIDTH                                                   : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR7_ATTRIB3 {
    struct {
        unsigned int MIP0_DEPTH                                                   : 14;
        unsigned int                                                              :  1;
        unsigned int COLOR_SW_MODE                                                :  3;
        unsigned int                                                              :  1;
        unsigned int MAX_MIP                                                      :  5;
        unsigned int RESOURCE_TYPE                                                :  2;
        unsigned int SPECULATIVE_READ                                             :  2;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR7_BASE {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR7_BASE_EXT {
    struct {
        unsigned int BASE_256B                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR7_FDCC_CONTROL {
    struct {
        unsigned int                                                              :  2;
        unsigned int MAX_UNCOMPRESSED_BLOCK_SIZE                                  :  1;
        unsigned int                                                              :  2;
        unsigned int MAX_COMPRESSED_BLOCK_SIZE                                    :  2;
        unsigned int                                                              : 17;
        unsigned int FRAGMENT_COMPRESS_DISABLE                                    :  1;
        unsigned int                                                              :  1;
        unsigned int COMPRESSION_MODE                                             :  2;
        unsigned int ENABLE_MAX_COMP_FRAG_OVERRIDE                                :  1;
        unsigned int MAX_COMP_FRAGS                                               :  2;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR7_INFO {
    struct {
        unsigned int FORMAT                                                       :  5;
        unsigned int                                                              :  2;
        unsigned int LINEAR_GENERAL                                               :  1;
        unsigned int NUMBER_TYPE                                                  :  3;
        unsigned int COMP_SWAP                                                    :  2;
        unsigned int                                                              :  2;
        unsigned int BLEND_CLAMP                                                  :  1;
        unsigned int BLEND_BYPASS                                                 :  1;
        unsigned int SIMPLE_FLOAT                                                 :  1;
        unsigned int ROUND_MODE                                                   :  1;
        unsigned int DISABLE_WA_FOR_PARTIAL_TARGET_MASK_ALL                       :  1;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR7_VIEW {
    struct {
        unsigned int SLICE_START                                                  : 14;
        unsigned int SLICE_MAX                                                    : 14;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR7_VIEW2 {
    struct {
        unsigned int MIP_LEVEL                                                    :  5;
        unsigned int                                                              : 27;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_COLOR_CONTROL {
    struct {
        unsigned int DISABLE_DUAL_QUAD                                            :  1;
        unsigned int                                                              :  2;
        unsigned int DEGAMMA_ENABLE                                               :  1;
        unsigned int MODE                                                         :  3;
        unsigned int                                                              :  9;
        unsigned int ROP3                                                         :  8;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_FGCG_SRAM_OVERRIDE {
    struct {
        unsigned int DISABLE_FGCG                                                 : 11;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_HW_CONTROL {
    struct {
        unsigned int DISABLE_GRBM_BUSY_CNTR                                       :  1;
        unsigned int DISABLE_VRS_FILLRATE_OPTIMIZATION                            :  1;
        unsigned int                                                              :  4;
        unsigned int GLX_CREDITS                                                  :  4;
        unsigned int                                                              :  2;
        unsigned int NUM_CCC_SKID_FIFO_ENTRIES                                    :  3;
        unsigned int DISABLE_EVICT_ILLEGAL_KEY_OVERRIDE                           :  1;
        unsigned int FORCE_EVICT_ALL_VALID                                        :  1;
        unsigned int                                                              :  1;
        unsigned int FORCE_WAIT_EOP_DONE_FLUSH                                    :  1;
        unsigned int FORCE_NEEDS_DST                                              :  1;
        unsigned int DISABLE_USE_OF_SET_HASH                                      :  1;
        unsigned int DISABLE_BLEND_OPT_RESULT_EQ_DEST                             :  1;
        unsigned int ENABLE_SINGLE_KEY_WR_OPT                                     :  1;
        unsigned int DISABLE_POWER_OPT_HC                                         :  1;
        unsigned int DISABLE_BLEND_OPT_DONT_RD_DST                                :  1;
        unsigned int DISABLE_BLEND_OPT_BYPASS                                     :  1;
        unsigned int DISABLE_BLEND_OPT_DISCARD_PIXEL                              :  1;
        unsigned int DISABLE_BLEND_OPT_WHEN_DISABLED_SRCALPHA_IS_USED             :  1;
        unsigned int DISABLE_MULTICYCLE_WRITES                                    :  1;
        unsigned int DISABLE_HOLE_COLLAPSE                                        :  1;
        unsigned int DISABLE_FMASK_REREAD_OPT                                     :  1;
        unsigned int EN_KEY_OVERRIDE                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_HW_CONTROL_1 {
    struct {
        unsigned int CC_CACHE_NUM_TAGS                                            :  6;
        unsigned int                                                              : 15;
        unsigned int DISABLE_SRC_FIFO_BYP                                         :  1;
        unsigned int DISABLE_RDLAT_FIFO_BYP                                       :  1;
        unsigned int ENABLE_COMPRESSION_ON_BYPASS                                 :  1;
        unsigned int COLOR_SCOPE                                                  :  2;
        unsigned int GLX_NOFILL                                                   :  1;
        unsigned int GLX_VQID                                                     :  4;
        unsigned int GLX_PERF_CNTR_EN                                             :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_HW_CONTROL_2 {
    struct {
        unsigned int RESERVED                                                     :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_HW_CONTROL_3 {
    struct {
        unsigned int                                                              :  3;
        unsigned int FORCE_GLX_REQ_CLKEN_HIGH                                     :  1;
        unsigned int FORCE_GLX_SRC_CLKEN_HIGH                                     :  1;
        unsigned int                                                              :  1;
        unsigned int SPLIT_ALL_FAST_MODE_TRANSFERS                                :  1;
        unsigned int DISABLE_SHADER_BLEND_OPTS                                    :  1;
        unsigned int                                                              : 13;
        unsigned int DISABLE_FMASK_OPT_WA_AND_FULLY_COVERED                       :  1;
        unsigned int                                                              :  3;
        unsigned int DISABLE_READ_DAW_OPT_OVERRIDE_ROH_COMP                       :  1;
        unsigned int DISABLE_READ_DAW_OPT_INSERT_BUBBLES_B2B                      :  1;
        unsigned int DISABLE_READ_DAW_CLEAR_KEY_OVERRIDE_FROM_ILLEGAL_DECODE      :  1;
        unsigned int DISABLE_READ_DAW_WAIT_SECOND_64B                             :  1;
        unsigned int DISABLE_READ_DAW_OPT_256B_CLEARS_BECOME_1FRAG                :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_HW_CONTROL_4 {
    struct {
        unsigned int ENABLE_READ_RESIDENCY_TIMEOUT_CNTR                           :  1;
        unsigned int THRESHOLD_READ_RESIDENCY_TIMEOUT_CNTR                        :  2;
        unsigned int                                                              : 11;
        unsigned int DISABLE_FRAGOP_MULTI_FRAGMENT                                :  1;
        unsigned int                                                              :  1;
        unsigned int ENABLE_FRAGOP_STALLING_ON_RAW_HAZARD                         :  1;
        unsigned int ENABLE_FRAGOP_STALLING_ON_COARSE_RAW_HAZARD                  :  1;
        unsigned int ENABLE_FRAGOP_STALLING_ON_DS_RAW_HAZARD                      :  1;
        unsigned int                                                              : 13;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_HW_MEM_ARBITER_CTL {
    struct {
        unsigned int ARB_MODE                                                     :  1;
        unsigned int READ_CYC_WEIGHT                                              :  5;
        unsigned int                                                              :  1;
        unsigned int READ_CRED_CNT_MAX                                            :  5;
        unsigned int                                                              :  1;
        unsigned int WRITE_CYC_WEIGHT                                             :  5;
        unsigned int                                                              :  1;
        unsigned int WRITE_CRED_CNT_MAX                                           :  5;
        unsigned int                                                              :  3;
        unsigned int WRITE_CREDIT_MODE                                            :  1;
        unsigned int READ_CREDIT_MODE                                             :  1;
        unsigned int                                                              :  3;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_MEM0_INFO {
    struct {
        unsigned int TEMPORAL_READ                                                :  3;
        unsigned int TEMPORAL_WRITE                                               :  3;
        unsigned int                                                              : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_MEM1_INFO {
    struct {
        unsigned int TEMPORAL_READ                                                :  3;
        unsigned int TEMPORAL_WRITE                                               :  3;
        unsigned int                                                              : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_MEM2_INFO {
    struct {
        unsigned int TEMPORAL_READ                                                :  3;
        unsigned int TEMPORAL_WRITE                                               :  3;
        unsigned int                                                              : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_MEM3_INFO {
    struct {
        unsigned int TEMPORAL_READ                                                :  3;
        unsigned int TEMPORAL_WRITE                                               :  3;
        unsigned int                                                              : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_MEM4_INFO {
    struct {
        unsigned int TEMPORAL_READ                                                :  3;
        unsigned int TEMPORAL_WRITE                                               :  3;
        unsigned int                                                              : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_MEM5_INFO {
    struct {
        unsigned int TEMPORAL_READ                                                :  3;
        unsigned int TEMPORAL_WRITE                                               :  3;
        unsigned int                                                              : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_MEM6_INFO {
    struct {
        unsigned int TEMPORAL_READ                                                :  3;
        unsigned int TEMPORAL_WRITE                                               :  3;
        unsigned int                                                              : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_MEM7_INFO {
    struct {
        unsigned int TEMPORAL_READ                                                :  3;
        unsigned int TEMPORAL_WRITE                                               :  3;
        unsigned int                                                              : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 18;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 18;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_PERFCOUNTER3_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_PERFCOUNTER3_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 18;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_PERFCOUNTER_FILTER {
    struct {
        unsigned int OP_FILTER_ENABLE                                             :  1;
        unsigned int OP_FILTER_SEL                                                :  3;
        unsigned int FORMAT_FILTER_ENABLE                                         :  1;
        unsigned int FORMAT_FILTER_SEL                                            :  5;
        unsigned int CLEAR_FILTER_ENABLE                                          :  1;
        unsigned int CLEAR_FILTER_SEL                                             :  1;
        unsigned int MRT_FILTER_ENABLE                                            :  1;
        unsigned int MRT_FILTER_SEL                                               :  3;
        unsigned int                                                              :  1;
        unsigned int NUM_SAMPLES_FILTER_ENABLE                                    :  1;
        unsigned int NUM_SAMPLES_FILTER_SEL                                       :  3;
        unsigned int NUM_FRAGMENTS_FILTER_ENABLE                                  :  1;
        unsigned int NUM_FRAGMENTS_FILTER_SEL                                     :  2;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_READ_RESP_KEY_OVERRIDE {
    struct {
        unsigned int KEY_IDX                                                      :  8;
        unsigned int NEW_KEY_VALUE                                                :  8;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_RMI_GL2_CACHE_CONTROL {
    struct {
        unsigned int                                                              :  2;
        unsigned int COLOR_WR_POLICY                                              :  2;
        unsigned int                                                              : 18;
        unsigned int COLOR_RD_POLICY                                              :  2;
        unsigned int                                                              :  3;
        unsigned int COLOR_L3_BYPASS                                              :  1;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_SHADER_MASK {
    struct {
        unsigned int OUTPUT0_ENABLE                                               :  4;
        unsigned int OUTPUT1_ENABLE                                               :  4;
        unsigned int OUTPUT2_ENABLE                                               :  4;
        unsigned int OUTPUT3_ENABLE                                               :  4;
        unsigned int OUTPUT4_ENABLE                                               :  4;
        unsigned int OUTPUT5_ENABLE                                               :  4;
        unsigned int OUTPUT6_ENABLE                                               :  4;
        unsigned int OUTPUT7_ENABLE                                               :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CB_TARGET_MASK {
    struct {
        unsigned int TARGET0_ENABLE                                               :  4;
        unsigned int TARGET1_ENABLE                                               :  4;
        unsigned int TARGET2_ENABLE                                               :  4;
        unsigned int TARGET3_ENABLE                                               :  4;
        unsigned int TARGET4_ENABLE                                               :  4;
        unsigned int TARGET5_ENABLE                                               :  4;
        unsigned int TARGET6_ENABLE                                               :  4;
        unsigned int TARGET7_ENABLE                                               :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHA_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHA_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHA_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHA_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHA_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHA_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHA_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHA_PERFCOUNTER1_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHA_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHA_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHA_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHA_PERFCOUNTER2_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHA_PERFCOUNTER3_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHA_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHA_PERFCOUNTER3_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHA_PERFCOUNTER3_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHC_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHC_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHC_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHC_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHC_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHC_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHC_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHC_PERFCOUNTER1_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHC_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHC_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHC_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHC_PERFCOUNTER2_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHC_PERFCOUNTER3_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHC_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHC_PERFCOUNTER3_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CHC_PERFCOUNTER3_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COHER_DEST_BASE_0 {
    struct {
        unsigned int DEST_BASE_256B                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COHER_DEST_BASE_1 {
    struct {
        unsigned int DEST_BASE_256B                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COHER_DEST_BASE_2 {
    struct {
        unsigned int DEST_BASE_256B                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COHER_DEST_BASE_3 {
    struct {
        unsigned int DEST_BASE_256B                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COHER_DEST_BASE_HI_0 {
    struct {
        unsigned int DEST_BASE_HI_256B                                            :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COHER_DEST_BASE_HI_1 {
    struct {
        unsigned int DEST_BASE_HI_256B                                            :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COHER_DEST_BASE_HI_2 {
    struct {
        unsigned int DEST_BASE_HI_256B                                            :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COHER_DEST_BASE_HI_3 {
    struct {
        unsigned int DEST_BASE_HI_256B                                            :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_DDID_INDEX {
    struct {
        unsigned int INDEX                                                        : 11;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_DESTINATION_EN_SE0 {
    struct {
        unsigned int CU_EN                                                        : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_DESTINATION_EN_SE1 {
    struct {
        unsigned int CU_EN                                                        : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_DESTINATION_EN_SE2 {
    struct {
        unsigned int CU_EN                                                        : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_DESTINATION_EN_SE3 {
    struct {
        unsigned int CU_EN                                                        : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_DIM_X {
    struct {
        unsigned int SIZE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_DIM_Y {
    struct {
        unsigned int SIZE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_DIM_Z {
    struct {
        unsigned int SIZE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_DISPATCH_END {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_DISPATCH_ID {
    struct {
        unsigned int DISPATCH_ID                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_DISPATCH_INITIATOR {
    struct {
        unsigned int COMPUTE_SHADER_EN                                            :  1;
        unsigned int PARTIAL_TG_EN                                                :  1;
        unsigned int FORCE_START_AT_000                                           :  1;
        unsigned int ORDERED_APPEND_ENBL                                          :  1;
        unsigned int ORDERED_APPEND_MODE                                          :  1;
        unsigned int USE_THREAD_DIMENSIONS                                        :  1;
        unsigned int ORDER_MODE                                                   :  1;
        unsigned int                                                              :  3;
        unsigned int SCALAR_L1_INV_VOL                                            :  1;
        unsigned int VECTOR_L1_INV_VOL                                            :  1;
        unsigned int PING_PONG_EN                                                 :  1;
        unsigned int TUNNEL_ENABLE                                                :  1;
        unsigned int RESTORE                                                      :  1;
        unsigned int CS_W32_EN                                                    :  1;
        unsigned int AMP_SHADER_EN                                                :  1;
        unsigned int DISABLE_DISP_PREMPT_EN                                       :  1;
        unsigned int INTERLEAVE_2D_EN                                             :  1;
        unsigned int WGS_DISPATCH                                                 :  1;
        unsigned int                                                              :  9;
        unsigned int TTRACE_QUEUE_ID                                              :  3;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_DISPATCH_INTERLEAVE {
    struct {
        unsigned int INTERLEAVE_1D                                                : 10;
        unsigned int                                                              :  6;
        unsigned int INTERLEAVE_2D_X_SIZE                                         :  4;
        unsigned int                                                              :  4;
        unsigned int INTERLEAVE_2D_Y_SIZE                                         :  4;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_DISPATCH_PKT_ADDR_HI {
    struct {
        unsigned int DATA                                                         :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_DISPATCH_PKT_ADDR_LO {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_DISPATCH_SCRATCH_BASE_HI {
    struct {
        unsigned int DATA                                                         :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_DISPATCH_SCRATCH_BASE_LO {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_DISPATCH_TUNNEL {
    struct {
        unsigned int OFF_DELAY                                                    : 13;
        unsigned int                                                              : 19;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_MISC_RESERVED {
    struct {
        unsigned int SEND_SEID                                                    :  4;
        unsigned int RESERVED4                                                    :  1;
        unsigned int WAVE_ID_BASE                                                 : 12;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_NOWHERE {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_NUM_THREAD_X {
    struct {
        unsigned int NUM_THREAD_FULL                                              : 13;
        unsigned int INTERLEAVE_BITS_X                                            :  3;
        unsigned int NUM_THREAD_PARTIAL                                           : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_NUM_THREAD_Y {
    struct {
        unsigned int NUM_THREAD_FULL                                              : 13;
        unsigned int INTERLEAVE_BITS_Y                                            :  3;
        unsigned int NUM_THREAD_PARTIAL                                           : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_NUM_THREAD_Z {
    struct {
        unsigned int NUM_THREAD_FULL                                              : 16;
        unsigned int NUM_THREAD_PARTIAL                                           : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_PERFCOUNT_ENABLE {
    struct {
        unsigned int PERFCOUNT_ENABLE                                             :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_PGM_HI {
    struct {
        unsigned int DATA                                                         :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_PGM_LO {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_PGM_RSRC1 {
    struct {
        unsigned int VGPRS                                                        :  6;
        unsigned int SGPRS                                                        :  4;
        unsigned int PRIORITY                                                     :  2;
        unsigned int FLOAT_MODE                                                   :  8;
        unsigned int PRIV                                                         :  1;
        unsigned int WG_RR_EN                                                     :  1;
        unsigned int DEBUG_MODE                                                   :  1;
        unsigned int DISABLE_PERF                                                 :  1;
        unsigned int BULKY                                                        :  1;
        unsigned int CDBG_USER                                                    :  1;
        unsigned int FP16_OVFL                                                    :  1;
        unsigned int                                                              :  2;
        unsigned int WGP_MODE                                                     :  1;
        unsigned int MEM_ORDERED                                                  :  1;
        unsigned int FWD_PROGRESS                                                 :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_PGM_RSRC2 {
    struct {
        unsigned int SCRATCH_EN                                                   :  1;
        unsigned int USER_SGPR                                                    :  5;
        unsigned int DYNAMIC_VGPR                                                 :  1;
        unsigned int TGID_X_EN                                                    :  1;
        unsigned int TGID_Y_EN                                                    :  1;
        unsigned int TGID_Z_EN                                                    :  1;
        unsigned int TG_SIZE_EN                                                   :  1;
        unsigned int TIDIG_COMP_CNT                                               :  2;
        unsigned int EXCP_EN_MSB                                                  :  2;
        unsigned int LDS_SIZE                                                     :  9;
        unsigned int EXCP_EN                                                      :  7;
        unsigned int WGP_TAKEOVER                                                 :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_PGM_RSRC3 {
    struct {
        unsigned int SHARED_VGPR_CNT                                              :  4;
        unsigned int INST_PREF_SIZE                                               :  8;
        unsigned int                                                              :  1;
        unsigned int GLG_EN                                                       :  1;
        unsigned int                                                              : 17;
        unsigned int IMAGE_OP                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_PIPELINESTAT_ENABLE {
    struct {
        unsigned int PIPELINESTAT_ENABLE                                          :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_PRESCALED_DIM_X {
    struct {
        unsigned int SIZE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_PRESCALED_DIM_Y {
    struct {
        unsigned int SIZE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_PRESCALED_DIM_Z {
    struct {
        unsigned int SIZE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_RELAUNCH {
    struct {
        unsigned int PAYLOAD                                                      : 30;
        unsigned int IS_EVENT                                                     :  1;
        unsigned int IS_STATE                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_RELAUNCH2 {
    struct {
        unsigned int PAYLOAD                                                      : 30;
        unsigned int IS_EVENT                                                     :  1;
        unsigned int IS_STATE                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_REQ_CTRL {
    struct {
        unsigned int SOFT_GROUPING_EN                                             :  1;
        unsigned int NUMBER_OF_REQUESTS_PER_CU                                    :  4;
        unsigned int SOFT_GROUPING_ALLOCATION_TIMEOUT                             :  4;
        unsigned int HARD_LOCK_HYSTERESIS                                         :  1;
        unsigned int HARD_LOCK_LOW_THRESHOLD                                      :  5;
        unsigned int PRODUCER_REQUEST_LOCKOUT                                     :  1;
        unsigned int GLOBAL_SCANNING_EN                                           :  1;
        unsigned int ALLOCATION_RATE_THROTTLING_THRESHOLD                         :  3;
        unsigned int DEDICATED_PREALLOCATION_BUFFER_LIMIT                         :  7;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_RESOURCE_LIMITS {
    struct {
        unsigned int WAVES_PER_SH                                                 : 10;
        unsigned int                                                              :  2;
        unsigned int TG_PER_CU                                                    :  4;
        unsigned int LOCK_THRESHOLD                                               :  6;
        unsigned int SIMD_DEST_CNTL                                               :  1;
        unsigned int FORCE_SIMD_DIST                                              :  1;
        unsigned int CU_GROUP_COUNT                                               :  3;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_RESTART_X {
    struct {
        unsigned int RESTART                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_RESTART_Y {
    struct {
        unsigned int RESTART                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_RESTART_Z {
    struct {
        unsigned int RESTART                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_SHADER_CHKSUM {
    struct {
        unsigned int CHECKSUM                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_START_X {
    struct {
        unsigned int START                                                        : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_START_Y {
    struct {
        unsigned int START                                                        : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_START_Z {
    struct {
        unsigned int START                                                        : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_STATIC_THREAD_MGMT_SE0 {
    struct {
        unsigned int SA0_CU_EN                                                    : 16;
        unsigned int SA1_CU_EN                                                    : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_STATIC_THREAD_MGMT_SE1 {
    struct {
        unsigned int SA0_CU_EN                                                    : 16;
        unsigned int SA1_CU_EN                                                    : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_STATIC_THREAD_MGMT_SE2 {
    struct {
        unsigned int SA0_CU_EN                                                    : 16;
        unsigned int SA1_CU_EN                                                    : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_STATIC_THREAD_MGMT_SE3 {
    struct {
        unsigned int SA0_CU_EN                                                    : 16;
        unsigned int SA1_CU_EN                                                    : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_STATIC_THREAD_MGMT_SE4 {
    struct {
        unsigned int SA0_CU_EN                                                    : 16;
        unsigned int SA1_CU_EN                                                    : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_STATIC_THREAD_MGMT_SE5 {
    struct {
        unsigned int SA0_CU_EN                                                    : 16;
        unsigned int SA1_CU_EN                                                    : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_STATIC_THREAD_MGMT_SE6 {
    struct {
        unsigned int SA0_CU_EN                                                    : 16;
        unsigned int SA1_CU_EN                                                    : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_STATIC_THREAD_MGMT_SE7 {
    struct {
        unsigned int SA0_CU_EN                                                    : 16;
        unsigned int SA1_CU_EN                                                    : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_STATIC_THREAD_MGMT_SE8 {
    struct {
        unsigned int SA0_CU_EN                                                    : 16;
        unsigned int SA1_CU_EN                                                    : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_THREADGROUP_ID {
    struct {
        unsigned int THREADGROUP_ID                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_THREAD_TRACE_ENABLE {
    struct {
        unsigned int THREAD_TRACE_ENABLE                                          :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_TMPRING_SIZE {
    struct {
        unsigned int WAVES                                                        : 12;
        unsigned int WAVESIZE                                                     : 18;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_USER_ACCUM_0 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_USER_ACCUM_1 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_USER_ACCUM_2 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_USER_ACCUM_3 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_USER_DATA_0 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_USER_DATA_1 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_USER_DATA_2 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_USER_DATA_3 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_USER_DATA_4 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_USER_DATA_5 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_USER_DATA_6 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_USER_DATA_7 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_USER_DATA_8 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_USER_DATA_9 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_USER_DATA_10 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_USER_DATA_11 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_USER_DATA_12 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_USER_DATA_13 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_USER_DATA_14 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_USER_DATA_15 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_VMID {
    struct {
        unsigned int DATA                                                         :  4;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_WAVE_RESTORE_ADDR_HI {
    struct {
        unsigned int ADDR                                                         : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union COMPUTE_WAVE_RESTORE_ADDR_LO {
    struct {
        unsigned int ADDR                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CPC_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CPC_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CPC_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int CNTR_MODE1                                                   :  4;
        unsigned int CNTR_MODE0                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CPC_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int CNTR_MODE3                                                   :  4;
        unsigned int CNTR_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CPC_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CPC_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CPC_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 18;
        unsigned int CNTR_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CPF_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CPF_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CPF_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int CNTR_MODE1                                                   :  4;
        unsigned int CNTR_MODE0                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CPF_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int CNTR_MODE3                                                   :  4;
        unsigned int CNTR_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CPF_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CPF_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CPF_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 18;
        unsigned int CNTR_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CPG_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CPG_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CPG_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int CNTR_MODE1                                                   :  4;
        unsigned int CNTR_MODE0                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CPG_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int CNTR_MODE3                                                   :  4;
        unsigned int CNTR_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CPG_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CPG_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CPG_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 18;
        unsigned int CNTR_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_CNTL {
    struct {
        unsigned int UTCL1_FAULT_CONTROL                                          :  1;
        unsigned int WATCH_CONTROL                                                :  1;
        unsigned int                                                              :  2;
        unsigned int MIN_AVAILSZ                                                  :  2;
        unsigned int SPECULATIVE_DATA_READ                                        :  2;
        unsigned int                                                              :  8;
        unsigned int BUFFER_DEPTH                                                 :  9;
        unsigned int                                                              :  3;
        unsigned int PIO_FIFO_EMPTY                                               :  1;
        unsigned int PIO_FIFO_FULL                                                :  1;
        unsigned int PIO_COUNT                                                    :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_ME_CMD_ADDR_HI {
    struct {
        unsigned int ADDR_HI                                                      : 16;
        unsigned int RSVD                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_ME_CMD_ADDR_LO {
    struct {
        unsigned int RSVD                                                         :  2;
        unsigned int ADDR_LO                                                      : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_ME_COMMAND {
    struct {
        unsigned int BYTE_COUNT                                                   : 26;
        unsigned int SAS                                                          :  1;
        unsigned int DAS                                                          :  1;
        unsigned int SAIC                                                         :  1;
        unsigned int DAIC                                                         :  1;
        unsigned int RAW_WAIT                                                     :  1;
        unsigned int DIS_WC                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_ME_CONTROL {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int                                                              :  6;
        unsigned int MEMLOG_CLEAR                                                 :  1;
        unsigned int                                                              :  2;
        unsigned int SRC_CACHE_POLICY                                             :  2;
        unsigned int                                                              :  5;
        unsigned int DST_SELECT                                                   :  2;
        unsigned int                                                              :  3;
        unsigned int DST_CACHE_POLICY                                             :  2;
        unsigned int                                                              :  2;
        unsigned int SRC_SELECT                                                   :  2;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_ME_DST_ADDR {
    struct {
        unsigned int DST_ADDR                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_ME_DST_ADDR_HI {
    struct {
        unsigned int DST_ADDR_HI                                                  : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_ME_SRC_ADDR {
    struct {
        unsigned int SRC_ADDR                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_ME_SRC_ADDR_HI {
    struct {
        unsigned int SRC_ADDR_HI                                                  : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_PFP_CMD_ADDR_HI {
    struct {
        unsigned int ADDR_HI                                                      : 16;
        unsigned int RSVD                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_PFP_CMD_ADDR_LO {
    struct {
        unsigned int RSVD                                                         :  2;
        unsigned int ADDR_LO                                                      : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_PFP_COMMAND {
    struct {
        unsigned int BYTE_COUNT                                                   : 26;
        unsigned int SAS                                                          :  1;
        unsigned int DAS                                                          :  1;
        unsigned int SAIC                                                         :  1;
        unsigned int DAIC                                                         :  1;
        unsigned int RAW_WAIT                                                     :  1;
        unsigned int DIS_WC                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_PFP_CONTROL {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int                                                              :  6;
        unsigned int MEMLOG_CLEAR                                                 :  1;
        unsigned int                                                              :  2;
        unsigned int SRC_CACHE_POLICY                                             :  2;
        unsigned int                                                              :  5;
        unsigned int DST_SELECT                                                   :  2;
        unsigned int                                                              :  3;
        unsigned int DST_CACHE_POLICY                                             :  2;
        unsigned int                                                              :  2;
        unsigned int SRC_SELECT                                                   :  2;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_PFP_DST_ADDR {
    struct {
        unsigned int DST_ADDR                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_PFP_DST_ADDR_HI {
    struct {
        unsigned int DST_ADDR_HI                                                  : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_PFP_SRC_ADDR {
    struct {
        unsigned int SRC_ADDR                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_PFP_SRC_ADDR_HI {
    struct {
        unsigned int SRC_ADDR_HI                                                  : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_PIO_COMMAND {
    struct {
        unsigned int BYTE_COUNT                                                   : 26;
        unsigned int SAS                                                          :  1;
        unsigned int DAS                                                          :  1;
        unsigned int SAIC                                                         :  1;
        unsigned int DAIC                                                         :  1;
        unsigned int RAW_WAIT                                                     :  1;
        unsigned int DIS_WC                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_PIO_CONTROL {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int                                                              :  6;
        unsigned int MEMLOG_CLEAR                                                 :  1;
        unsigned int                                                              :  2;
        unsigned int SRC_CACHE_POLICY                                             :  2;
        unsigned int                                                              :  5;
        unsigned int DST_SELECT                                                   :  2;
        unsigned int                                                              :  3;
        unsigned int DST_CACHE_POLICY                                             :  2;
        unsigned int                                                              :  2;
        unsigned int SRC_SELECT                                                   :  2;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_PIO_DST_ADDR {
    struct {
        unsigned int DST_ADDR                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_PIO_DST_ADDR_HI {
    struct {
        unsigned int DST_ADDR_HI                                                  : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_PIO_SRC_ADDR {
    struct {
        unsigned int SRC_ADDR                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_PIO_SRC_ADDR_HI {
    struct {
        unsigned int SRC_ADDR_HI                                                  : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_READ_TAGS {
    struct {
        unsigned int DMA_READ_TAG                                                 : 26;
        unsigned int                                                              :  2;
        unsigned int DMA_READ_TAG_VALID                                           :  1;
        unsigned int                                                              :  3;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_WATCH0_ADDR_HI {
    struct {
        unsigned int ADDR_HI                                                      : 16;
        unsigned int RSVD                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_WATCH0_ADDR_LO {
    struct {
        unsigned int RSVD                                                         :  8;
        unsigned int ADDR_LO                                                      : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_WATCH0_CNTL {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int RSVD1                                                        :  4;
        unsigned int WATCH_READS                                                  :  1;
        unsigned int WATCH_WRITES                                                 :  1;
        unsigned int ANY_VMID                                                     :  1;
        unsigned int RSVD2                                                        : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_WATCH0_MASK {
    struct {
        unsigned int RSVD                                                         :  8;
        unsigned int MASK                                                         : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_WATCH1_ADDR_HI {
    struct {
        unsigned int ADDR_HI                                                      : 16;
        unsigned int RSVD                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_WATCH1_ADDR_LO {
    struct {
        unsigned int RSVD                                                         :  8;
        unsigned int ADDR_LO                                                      : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_WATCH1_CNTL {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int RSVD1                                                        :  4;
        unsigned int WATCH_READS                                                  :  1;
        unsigned int WATCH_WRITES                                                 :  1;
        unsigned int ANY_VMID                                                     :  1;
        unsigned int RSVD2                                                        : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_WATCH1_MASK {
    struct {
        unsigned int RSVD                                                         :  8;
        unsigned int MASK                                                         : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_WATCH2_ADDR_HI {
    struct {
        unsigned int ADDR_HI                                                      : 16;
        unsigned int RSVD                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_WATCH2_ADDR_LO {
    struct {
        unsigned int RSVD                                                         :  8;
        unsigned int ADDR_LO                                                      : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_WATCH2_CNTL {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int RSVD1                                                        :  4;
        unsigned int WATCH_READS                                                  :  1;
        unsigned int WATCH_WRITES                                                 :  1;
        unsigned int ANY_VMID                                                     :  1;
        unsigned int RSVD2                                                        : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_WATCH2_MASK {
    struct {
        unsigned int RSVD                                                         :  8;
        unsigned int MASK                                                         : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_WATCH3_ADDR_HI {
    struct {
        unsigned int ADDR_HI                                                      : 16;
        unsigned int RSVD                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_WATCH3_ADDR_LO {
    struct {
        unsigned int RSVD                                                         :  8;
        unsigned int ADDR_LO                                                      : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_WATCH3_CNTL {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int RSVD1                                                        :  4;
        unsigned int WATCH_READS                                                  :  1;
        unsigned int WATCH_WRITES                                                 :  1;
        unsigned int ANY_VMID                                                     :  1;
        unsigned int RSVD2                                                        : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_WATCH3_MASK {
    struct {
        unsigned int RSVD                                                         :  8;
        unsigned int MASK                                                         : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_WATCH_STAT {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int QUEUE_ID                                                     :  3;
        unsigned int                                                              :  1;
        unsigned int CLIENT_ID                                                    :  3;
        unsigned int                                                              :  1;
        unsigned int PIPE                                                         :  2;
        unsigned int                                                              :  2;
        unsigned int WATCH_ID                                                     :  2;
        unsigned int                                                              :  2;
        unsigned int RD_WR                                                        :  1;
        unsigned int                                                              : 10;
        unsigned int TRAP_FLAG                                                    :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_WATCH_STAT_ADDR_HI {
    struct {
        unsigned int ADDR_HI                                                      : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_DMA_WATCH_STAT_ADDR_LO {
    struct {
        unsigned int                                                              :  2;
        unsigned int ADDR_LO                                                      : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_EOP_DONE_ADDR_HI {
    struct {
        unsigned int ADDR_HI                                                      : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_EOP_DONE_ADDR_LO {
    struct {
        unsigned int                                                              :  2;
        unsigned int ADDR_LO                                                      : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_EOP_DONE_CNTX_ID {
    struct {
        unsigned int CNTX_ID                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_EOP_DONE_DATA_CNTL {
    struct {
        unsigned int                                                              : 16;
        unsigned int DST_SEL                                                      :  2;
        unsigned int                                                              :  2;
        unsigned int ACTION_PIPE_ID                                               :  2;
        unsigned int ACTION_ID                                                    :  2;
        unsigned int INT_SEL                                                      :  3;
        unsigned int                                                              :  2;
        unsigned int DATA_SEL                                                     :  3;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_EOP_DONE_DATA_HI {
    struct {
        unsigned int DATA_HI                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_EOP_DONE_DATA_LO {
    struct {
        unsigned int DATA_LO                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_EOP_DONE_EVENT_CNTL {
    struct {
        unsigned int                                                              : 12;
        unsigned int GCR_CNTL                                                     : 13;
        unsigned int CACHE_POLICY                                                 :  2;
        unsigned int                                                              :  1;
        unsigned int EXECUTE                                                      :  1;
        unsigned int                                                              :  1;
        unsigned int GLK_INV                                                      :  1;
        unsigned int PWS_ENABLE                                                   :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_EOP_LAST_FENCE_HI {
    struct {
        unsigned int LAST_FENCE_HI                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_EOP_LAST_FENCE_LO {
    struct {
        unsigned int LAST_FENCE_LO                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE0_BASE0 {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE0_BASE1 {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE0_CNTL0 {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE0_CNTL1 {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE0_MASK0 {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE0_MASK1 {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE1_BASE0 {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE1_BASE1 {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE1_CNTL0 {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE1_CNTL1 {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE1_MASK0 {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE1_MASK1 {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE2_BASE0 {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE2_BASE1 {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE2_CNTL0 {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE2_CNTL1 {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE2_MASK0 {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE2_MASK1 {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE3_BASE0 {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE3_BASE1 {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE3_CNTL0 {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE3_CNTL1 {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE3_MASK0 {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE3_MASK1 {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE4_BASE0 {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE4_BASE1 {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE4_CNTL0 {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE4_CNTL1 {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE4_MASK0 {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE4_MASK1 {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE5_BASE0 {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE5_BASE1 {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE5_CNTL0 {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE5_CNTL1 {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE5_MASK0 {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE5_MASK1 {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE6_BASE0 {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE6_BASE1 {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE6_CNTL0 {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE6_CNTL1 {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE6_MASK0 {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE6_MASK1 {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE7_BASE0 {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE7_BASE1 {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE7_CNTL0 {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE7_CNTL1 {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE7_MASK0 {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE7_MASK1 {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE8_BASE0 {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE8_BASE1 {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE8_CNTL0 {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE8_CNTL1 {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE8_MASK0 {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE8_MASK1 {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE9_BASE0 {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE9_BASE1 {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE9_CNTL0 {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE9_CNTL1 {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE9_MASK0 {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE9_MASK1 {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE10_BASE0 {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE10_BASE1 {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE10_CNTL0 {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE10_CNTL1 {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE10_MASK0 {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE10_MASK1 {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE11_BASE0 {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE11_BASE1 {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE11_CNTL0 {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE11_CNTL1 {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE11_MASK0 {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE11_MASK1 {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE12_BASE0 {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE12_BASE1 {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE12_CNTL0 {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE12_CNTL1 {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE12_MASK0 {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE12_MASK1 {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE13_BASE0 {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE13_BASE1 {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE13_CNTL0 {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE13_CNTL1 {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE13_MASK0 {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE13_MASK1 {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE14_BASE0 {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE14_BASE1 {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE14_CNTL0 {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE14_CNTL1 {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE14_MASK0 {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE14_MASK1 {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE15_BASE0 {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE15_BASE1 {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE15_CNTL0 {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE15_CNTL1 {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE15_MASK0 {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_DC_APERTURE15_MASK1 {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_PERFCOUNT_CNTL0 {
    struct {
        unsigned int EVENT_SEL                                                    :  5;
        unsigned int                                                              : 27;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_GFX_RS64_PERFCOUNT_CNTL1 {
    struct {
        unsigned int EVENT_SEL                                                    :  5;
        unsigned int                                                              : 27;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_ACTIVE {
    struct {
        unsigned int ACTIVE                                                       :  1;
        unsigned int BUSY_GATE                                                    :  1;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_AQL_CONTROL {
    struct {
        unsigned int CONTROL0                                                     : 15;
        unsigned int CONTROL0_EN                                                  :  1;
        unsigned int CONTROL1                                                     : 15;
        unsigned int CONTROL1_EN                                                  :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_ATOMIC0_PREOP_HI {
    struct {
        unsigned int ATOMIC0_PREOP_HI                                             : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_ATOMIC0_PREOP_LO {
    struct {
        unsigned int ATOMIC0_PREOP_LO                                             : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_ATOMIC1_PREOP_HI {
    struct {
        unsigned int ATOMIC1_PREOP_HI                                             : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_ATOMIC1_PREOP_LO {
    struct {
        unsigned int ATOMIC1_PREOP_LO                                             : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_CNTL_STACK_OFFSET {
    struct {
        unsigned int                                                              :  2;
        unsigned int OFFSET                                                       : 14;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_CNTL_STACK_SIZE {
    struct {
        unsigned int                                                              : 12;
        unsigned int SIZE                                                         :  4;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_CTX_SAVE_BASE_ADDR_HI {
    struct {
        unsigned int ADDR_HI                                                      : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_CTX_SAVE_BASE_ADDR_LO {
    struct {
        unsigned int                                                              : 12;
        unsigned int ADDR                                                         : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_CTX_SAVE_CONTROL {
    struct {
        unsigned int                                                              :  3;
        unsigned int POLICY                                                       :  2;
        unsigned int                                                              : 18;
        unsigned int EXE_DISABLE                                                  :  1;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_CTX_SAVE_SIZE {
    struct {
        unsigned int                                                              : 12;
        unsigned int SIZE                                                         : 14;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_DDID_DELTA_RPT_COUNT {
    struct {
        unsigned int COUNT                                                        :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_DDID_INFLIGHT_COUNT {
    struct {
        unsigned int COUNT                                                        : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_DDID_RPTR {
    struct {
        unsigned int RPTR                                                         : 11;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_DDID_WPTR {
    struct {
        unsigned int WPTR                                                         : 11;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_DEQUEUE_REQUEST {
    struct {
        unsigned int DEQUEUE_REQ                                                  :  4;
        unsigned int IQ_REQ_PEND                                                  :  1;
        unsigned int                                                              :  3;
        unsigned int DEQUEUE_INT                                                  :  1;
        unsigned int IQ_REQ_PEND_EN                                               :  1;
        unsigned int DEQUEUE_REQ_EN                                               :  1;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_DEQUEUE_STATUS {
    struct {
        unsigned int DEQUEUE_STAT                                                 :  4;
        unsigned int SUSPEND_REQ_PEND                                             :  1;
        unsigned int                                                              :  4;
        unsigned int SUSPEND_REQ_PEND_EN                                          :  1;
        unsigned int DEQUEUE_STAT_EN                                              :  1;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_DMA_OFFLOAD {
    struct {
        unsigned int DMA_OFFLOAD                                                  :  1;
        unsigned int DMA_OFFLOAD_EN                                               :  1;
        unsigned int AQL_OFFLOAD                                                  :  1;
        unsigned int AQL_OFFLOAD_EN                                               :  1;
        unsigned int EOP_OFFLOAD                                                  :  1;
        unsigned int EOP_OFFLOAD_EN                                               :  1;
        unsigned int                                                              : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_EOP_BASE_ADDR {
    struct {
        unsigned int BASE_ADDR                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_EOP_BASE_ADDR_HI {
    struct {
        unsigned int BASE_ADDR_HI                                                 :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_EOP_CONTROL {
    struct {
        unsigned int EOP_SIZE                                                     :  6;
        unsigned int                                                              :  2;
        unsigned int PROCESSING_EOP                                               :  1;
        unsigned int                                                              :  3;
        unsigned int PROCESS_EOP_EN                                               :  1;
        unsigned int PROCESSING_EOPIB                                             :  1;
        unsigned int PROCESS_EOPIB_EN                                             :  1;
        unsigned int                                                              :  6;
        unsigned int HALT_FETCHER                                                 :  1;
        unsigned int HALT_FETCHER_EN                                              :  1;
        unsigned int EXE_DISABLE                                                  :  1;
        unsigned int CACHE_POLICY                                                 :  2;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_EOP_EVENTS {
    struct {
        unsigned int EVENT_COUNT                                                  : 12;
        unsigned int                                                              :  4;
        unsigned int CS_PARTIAL_FLUSH_PEND                                        :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_EOP_RPTR {
    struct {
        unsigned int RPTR                                                         : 13;
        unsigned int                                                              : 15;
        unsigned int RESET_FETCHER                                                :  1;
        unsigned int DEQUEUE_PEND                                                 :  1;
        unsigned int RPTR_EQ_CSMD_WPTR                                            :  1;
        unsigned int INIT_FETCHER                                                 :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_EOP_WPTR {
    struct {
        unsigned int WPTR                                                         : 13;
        unsigned int                                                              :  2;
        unsigned int EOP_EMPTY                                                    :  1;
        unsigned int EOP_AVAIL                                                    : 13;
        unsigned int                                                              :  3;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_EOP_WPTR_MEM {
    struct {
        unsigned int WPTR                                                         : 13;
        unsigned int                                                              : 19;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_ERROR {
    struct {
        unsigned int EDC_ERROR_ID                                                 :  4;
        unsigned int SUA_ERROR                                                    :  1;
        unsigned int AQL_ERROR                                                    :  1;
        unsigned int                                                              :  2;
        unsigned int PQ_UTCL1_ERROR                                               :  1;
        unsigned int IB_UTCL1_ERROR                                               :  1;
        unsigned int EOP_UTCL1_ERROR                                              :  1;
        unsigned int IQ_UTCL1_ERROR                                               :  1;
        unsigned int RRPT_UTCL1_ERROR                                             :  1;
        unsigned int WPP_UTCL1_ERROR                                              :  1;
        unsigned int                                                              :  1;
        unsigned int DMA_SRC_UTCL1_ERROR                                          :  1;
        unsigned int DMA_DST_UTCL1_ERROR                                          :  1;
        unsigned int SR_UTCL1_ERROR                                               :  1;
        unsigned int QU_UTCL1_ERROR                                               :  1;
        unsigned int TC_UTCL1_ERROR                                               :  1;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_GDS_RESOURCE_STATE {
    struct {
        unsigned int OA_REQUIRED                                                  :  1;
        unsigned int OA_ACQUIRED                                                  :  1;
        unsigned int                                                              :  2;
        unsigned int GWS_SIZE                                                     :  6;
        unsigned int                                                              :  2;
        unsigned int GWS_PNTR                                                     :  6;
        unsigned int                                                              : 14;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_GFX_CONTROL {
    struct {
        unsigned int MESSAGE                                                      :  4;
        unsigned int MISC                                                         : 11;
        unsigned int DB_UPDATED_MSG_EN                                            :  1;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_GFX_STATUS {
    struct {
        unsigned int STATUS                                                       : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_HQ_CONTROL0 {
    struct {
        unsigned int CONTROL                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_HQ_CONTROL1 {
    struct {
        unsigned int CONTROL                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_HQ_SCHEDULER0 {
    struct {
        unsigned int CWSR                                                         :  1;
        unsigned int SAVE_STATUS                                                  :  1;
        unsigned int RSRV                                                         :  1;
        unsigned int STATIC_QUEUE                                                 :  3;
        unsigned int QUEUE_RUN_ONCE                                               :  1;
        unsigned int SCRATCH_RAM_INIT                                             :  1;
        unsigned int TCL2_DIRTY                                                   :  1;
        unsigned int C_INHERIT_VMID                                               :  1;
        unsigned int QUEUE_SCHEDULER_TYPE                                         :  3;
        unsigned int C_QUEUE_USE_GWS                                              :  1;
        unsigned int C_QUEUE_DEBUG_EN                                             :  1;
        unsigned int QUEUE_SLOT_CONNECTED                                         :  1;
        unsigned int                                                              :  4;
        unsigned int MES_INTERRUPT_ENABLED                                        :  1;
        unsigned int MES_INTERRUPT_PIPE                                           :  2;
        unsigned int                                                              :  1;
        unsigned int CONCURRENT_PROCESS_COUNT                                     :  4;
        unsigned int                                                              :  2;
        unsigned int QUEUE_IDLE                                                   :  1;
        unsigned int DB_UPDATED_MSG_EN                                            :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_HQ_SCHEDULER1 {
    struct {
        unsigned int SCHEDULER                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_HQ_STATUS0 {
    struct {
        unsigned int CWSR                                                         :  1;
        unsigned int SAVE_STATUS                                                  :  1;
        unsigned int RSRV                                                         :  1;
        unsigned int STATIC_QUEUE                                                 :  3;
        unsigned int QUEUE_RUN_ONCE                                               :  1;
        unsigned int SCRATCH_RAM_INIT                                             :  1;
        unsigned int TCL2_DIRTY                                                   :  1;
        unsigned int C_INHERIT_VMID                                               :  1;
        unsigned int QUEUE_SCHEDULER_TYPE                                         :  3;
        unsigned int C_QUEUE_USE_GWS                                              :  1;
        unsigned int C_QUEUE_DEBUG_EN                                             :  1;
        unsigned int QUEUE_SLOT_CONNECTED                                         :  1;
        unsigned int                                                              :  4;
        unsigned int MES_INTERRUPT_ENABLED                                        :  1;
        unsigned int MES_INTERRUPT_PIPE                                           :  2;
        unsigned int                                                              :  1;
        unsigned int CONCURRENT_PROCESS_COUNT                                     :  4;
        unsigned int                                                              :  2;
        unsigned int QUEUE_IDLE                                                   :  1;
        unsigned int DB_UPDATED_MSG_EN                                            :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_HQ_STATUS1 {
    struct {
        unsigned int STATUS                                                       : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_IB_BASE_ADDR {
    struct {
        unsigned int                                                              :  2;
        unsigned int IB_BASE_ADDR                                                 : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_IB_BASE_ADDR_HI {
    struct {
        unsigned int IB_BASE_ADDR_HI                                              : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_IB_CONTROL {
    struct {
        unsigned int IB_SIZE                                                      : 20;
        unsigned int MIN_IB_AVAIL_SIZE                                            :  2;
        unsigned int                                                              :  1;
        unsigned int IB_EXE_DISABLE                                               :  1;
        unsigned int IB_CACHE_POLICY                                              :  2;
        unsigned int                                                              :  4;
        unsigned int IB_PRIV_STATE                                                :  1;
        unsigned int PROCESSING_IB                                                :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_IB_RPTR {
    struct {
        unsigned int CONSUMED_OFFSET                                              : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_IQ_RPTR {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_IQ_TIMER {
    struct {
        unsigned int WAIT_TIME                                                    :  8;
        unsigned int RETRY_TYPE                                                   :  3;
        unsigned int IMMEDIATE_EXPIRE                                             :  1;
        unsigned int INTERRUPT_TYPE                                               :  2;
        unsigned int CLOCK_COUNT                                                  :  2;
        unsigned int INTERRUPT_SIZE                                               :  6;
        unsigned int QUANTUM_TIMER                                                :  1;
        unsigned int EXE_DISABLE                                                  :  1;
        unsigned int CACHE_POLICY                                                 :  2;
        unsigned int                                                              :  1;
        unsigned int QUEUE_TYPE                                                   :  1;
        unsigned int REARM_TIMER                                                  :  1;
        unsigned int PROCESS_IQ_EN                                                :  1;
        unsigned int PROCESSING_IQ                                                :  1;
        unsigned int ACTIVE                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_MSG_TYPE {
    struct {
        unsigned int ACTION                                                       :  3;
        unsigned int                                                              :  1;
        unsigned int SAVE_STATE                                                   :  3;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_OFFLOAD {
    struct {
        unsigned int DMA_OFFLOAD                                                  :  1;
        unsigned int DMA_OFFLOAD_EN                                               :  1;
        unsigned int AQL_OFFLOAD                                                  :  1;
        unsigned int AQL_OFFLOAD_EN                                               :  1;
        unsigned int EOP_OFFLOAD                                                  :  1;
        unsigned int EOP_OFFLOAD_EN                                               :  1;
        unsigned int                                                              : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_PERSISTENT_STATE {
    struct {
        unsigned int PRELOAD_REQ                                                  :  1;
        unsigned int TMZ_CONNECT_OVERRIDE                                         :  1;
        unsigned int                                                              :  2;
        unsigned int WGS_QUEUE                                                    :  1;
        unsigned int                                                              :  2;
        unsigned int SUSPEND_STATUS                                               :  1;
        unsigned int PRELOAD_SIZE                                                 : 10;
        unsigned int TMZ_SWITCH_EXEMPT                                            :  1;
        unsigned int TMZ_MATCH_DIS                                                :  1;
        unsigned int WPP_CLAMP_EN                                                 :  1;
        unsigned int WPP_SWITCH_QOS_EN                                            :  1;
        unsigned int IQ_SWITCH_QOS_EN                                             :  1;
        unsigned int IB_SWITCH_QOS_EN                                             :  1;
        unsigned int EOP_SWITCH_QOS_EN                                            :  1;
        unsigned int PQ_SWITCH_QOS_EN                                             :  1;
        unsigned int TC_OFFLOAD_QOS_EN                                            :  1;
        unsigned int CACHE_FULL_PACKET_EN                                         :  1;
        unsigned int RESTORE_ACTIVE                                               :  1;
        unsigned int RELAUNCH_WAVES                                               :  1;
        unsigned int QSWITCH_MODE                                                 :  1;
        unsigned int DISP_ACTIVE                                                  :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_PIPE_PRIORITY {
    struct {
        unsigned int PIPE_PRIORITY                                                :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_PQ_BASE {
    struct {
        unsigned int ADDR                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_PQ_BASE_HI {
    struct {
        unsigned int ADDR_HI                                                      :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_PQ_CONTROL {
    struct {
        unsigned int QUEUE_SIZE                                                   :  6;
        unsigned int WPTR_CARRY                                                   :  1;
        unsigned int RPTR_CARRY                                                   :  1;
        unsigned int RPTR_BLOCK_SIZE                                              :  6;
        unsigned int QUEUE_FULL_EN                                                :  1;
        unsigned int PQ_EMPTY                                                     :  1;
        unsigned int                                                              :  2;
        unsigned int SLOT_BASED_WPTR                                              :  2;
        unsigned int MIN_AVAIL_SIZE                                               :  2;
        unsigned int                                                              :  1;
        unsigned int EXE_DISABLE                                                  :  1;
        unsigned int CACHE_POLICY                                                 :  2;
        unsigned int                                                              :  1;
        unsigned int NO_UPDATE_RPTR                                               :  1;
        unsigned int UNORD_DISPATCH                                               :  1;
        unsigned int TUNNEL_DISPATCH                                              :  1;
        unsigned int PRIV_STATE                                                   :  1;
        unsigned int KMD_QUEUE                                                    :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_PQ_DOORBELL_CONTROL {
    struct {
        unsigned int DOORBELL_MODE                                                :  1;
        unsigned int DOORBELL_BIF_DROP                                            :  1;
        unsigned int DOORBELL_OFFSET                                              : 26;
        unsigned int DOORBELL_SOURCE                                              :  1;
        unsigned int DOORBELL_SCHD_HIT                                            :  1;
        unsigned int DOORBELL_EN                                                  :  1;
        unsigned int DOORBELL_HIT                                                 :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_PQ_RPTR {
    struct {
        unsigned int CONSUMED_OFFSET                                              : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_PQ_RPTR_REPORT_ADDR {
    struct {
        unsigned int                                                              :  2;
        unsigned int RPTR_REPORT_ADDR                                             : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_PQ_RPTR_REPORT_ADDR_HI {
    struct {
        unsigned int RPTR_REPORT_ADDR_HI                                          : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_PQ_WPTR_HI {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_PQ_WPTR_LO {
    struct {
        unsigned int OFFSET                                                       : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_PQ_WPTR_POLL_ADDR {
    struct {
        unsigned int                                                              :  3;
        unsigned int WPTR_ADDR                                                    : 29;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_PQ_WPTR_POLL_ADDR_HI {
    struct {
        unsigned int WPTR_ADDR_HI                                                 : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_QUANTUM {
    struct {
        unsigned int QUANTUM_EN                                                   :  1;
        unsigned int                                                              :  3;
        unsigned int QUANTUM_SCALE                                                :  1;
        unsigned int                                                              :  3;
        unsigned int QUANTUM_DURATION                                             :  6;
        unsigned int                                                              : 17;
        unsigned int QUANTUM_ACTIVE                                               :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_QUEUE_PRIORITY {
    struct {
        unsigned int PRIORITY_LEVEL                                               :  4;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_SECURE_REQ0 {
    struct {
        unsigned int REQ_TYPE                                                     :  8;
        unsigned int RSP_TYPE                                                     :  8;
        unsigned int                                                              : 14;
        unsigned int RSP_EN                                                       :  1;
        unsigned int REQ_EN                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_SUSPEND_CNTL_STACK_DW_CNT {
    struct {
        unsigned int CNT                                                          : 14;
        unsigned int                                                              : 18;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_SUSPEND_CNTL_STACK_OFFSET {
    struct {
        unsigned int                                                              :  2;
        unsigned int OFFSET                                                       : 14;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_SUSPEND_WG_STATE_OFFSET {
    struct {
        unsigned int                                                              :  2;
        unsigned int OFFSET                                                       : 24;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_VMID {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int                                                              :  4;
        unsigned int IB_VMID                                                      :  4;
        unsigned int                                                              :  4;
        unsigned int VQID                                                         : 10;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_HQD_WG_STATE_OFFSET {
    struct {
        unsigned int                                                              :  2;
        unsigned int OFFSET                                                       : 24;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_INT_CNTL {
    struct {
        unsigned int                                                              :  8;
        unsigned int RESUME_INT_ENABLE                                            :  1;
        unsigned int SUSPEND_INT_ENABLE                                           :  1;
        unsigned int DMA_WATCH_INT_ENABLE                                         :  1;
        unsigned int CP_VM_DOORBELL_WR_INT_ENABLE                                 :  1;
        unsigned int                                                              :  2;
        unsigned int CP_ECC_ERROR_INT_ENABLE                                      :  1;
        unsigned int                                                              :  1;
        unsigned int GPF_INT_ENABLE                                               :  1;
        unsigned int WRM_POLL_TIMEOUT_INT_ENABLE                                  :  1;
        unsigned int CMP_BUSY_INT_ENABLE                                          :  1;
        unsigned int CNTX_BUSY_INT_ENABLE                                         :  1;
        unsigned int CNTX_EMPTY_INT_ENABLE                                        :  1;
        unsigned int GFX_IDLE_INT_ENABLE                                          :  1;
        unsigned int PRIV_INSTR_INT_ENABLE                                        :  1;
        unsigned int PRIV_REG_INT_ENABLE                                          :  1;
        unsigned int OPCODE_ERROR_INT_ENABLE                                      :  1;
        unsigned int                                                              :  1;
        unsigned int TIME_STAMP_INT_ENABLE                                        :  1;
        unsigned int RESERVED_BIT_ERROR_INT_ENABLE                                :  1;
        unsigned int                                                              :  1;
        unsigned int GENERIC2_INT_ENABLE                                          :  1;
        unsigned int GENERIC1_INT_ENABLE                                          :  1;
        unsigned int GENERIC0_INT_ENABLE                                          :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_INT_CNTL_RING0 {
    struct {
        unsigned int                                                              :  8;
        unsigned int RESUME_INT_ENABLE                                            :  1;
        unsigned int SUSPEND_INT_ENABLE                                           :  1;
        unsigned int DMA_WATCH_INT_ENABLE                                         :  1;
        unsigned int CP_VM_DOORBELL_WR_INT_ENABLE                                 :  1;
        unsigned int                                                              :  2;
        unsigned int CP_ECC_ERROR_INT_ENABLE                                      :  1;
        unsigned int                                                              :  1;
        unsigned int GPF_INT_ENABLE                                               :  1;
        unsigned int WRM_POLL_TIMEOUT_INT_ENABLE                                  :  1;
        unsigned int CMP_BUSY_INT_ENABLE                                          :  1;
        unsigned int CNTX_BUSY_INT_ENABLE                                         :  1;
        unsigned int CNTX_EMPTY_INT_ENABLE                                        :  1;
        unsigned int GFX_IDLE_INT_ENABLE                                          :  1;
        unsigned int PRIV_INSTR_INT_ENABLE                                        :  1;
        unsigned int PRIV_REG_INT_ENABLE                                          :  1;
        unsigned int OPCODE_ERROR_INT_ENABLE                                      :  1;
        unsigned int                                                              :  1;
        unsigned int TIME_STAMP_INT_ENABLE                                        :  1;
        unsigned int RESERVED_BIT_ERROR_INT_ENABLE                                :  1;
        unsigned int                                                              :  1;
        unsigned int GENERIC2_INT_ENABLE                                          :  1;
        unsigned int GENERIC1_INT_ENABLE                                          :  1;
        unsigned int GENERIC0_INT_ENABLE                                          :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_INT_STATUS {
    struct {
        unsigned int                                                              :  8;
        unsigned int RESUME_INT_STAT                                              :  1;
        unsigned int SUSPEND_INT_STAT                                             :  1;
        unsigned int DMA_WATCH_INT_STAT                                           :  1;
        unsigned int CP_VM_DOORBELL_WR_INT_STAT                                   :  1;
        unsigned int                                                              :  2;
        unsigned int CP_ECC_ERROR_INT_STAT                                        :  1;
        unsigned int                                                              :  1;
        unsigned int GPF_INT_STAT                                                 :  1;
        unsigned int WRM_POLL_TIMEOUT_INT_STAT                                    :  1;
        unsigned int CMP_BUSY_INT_STAT                                            :  1;
        unsigned int CNTX_BUSY_INT_STAT                                           :  1;
        unsigned int CNTX_EMPTY_INT_STAT                                          :  1;
        unsigned int GFX_IDLE_INT_STAT                                            :  1;
        unsigned int PRIV_INSTR_INT_STAT                                          :  1;
        unsigned int PRIV_REG_INT_STAT                                            :  1;
        unsigned int OPCODE_ERROR_INT_STAT                                        :  1;
        unsigned int                                                              :  1;
        unsigned int TIME_STAMP_INT_STAT                                          :  1;
        unsigned int RESERVED_BIT_ERROR_INT_STAT                                  :  1;
        unsigned int                                                              :  1;
        unsigned int GENERIC2_INT_STAT                                            :  1;
        unsigned int GENERIC1_INT_STAT                                            :  1;
        unsigned int GENERIC0_INT_STAT                                            :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_INT_STATUS_RING0 {
    struct {
        unsigned int                                                              :  8;
        unsigned int RESUME_INT_STAT                                              :  1;
        unsigned int SUSPEND_INT_STAT                                             :  1;
        unsigned int DMA_WATCH_INT_STAT                                           :  1;
        unsigned int CP_VM_DOORBELL_WR_INT_STAT                                   :  1;
        unsigned int                                                              :  2;
        unsigned int CP_ECC_ERROR_INT_STAT                                        :  1;
        unsigned int                                                              :  1;
        unsigned int GPF_INT_STAT                                                 :  1;
        unsigned int WRM_POLL_TIMEOUT_INT_STAT                                    :  1;
        unsigned int CMP_BUSY_INT_STAT                                            :  1;
        unsigned int GCNTX_BUSY_INT_STAT                                          :  1;
        unsigned int CNTX_EMPTY_INT_STAT                                          :  1;
        unsigned int GFX_IDLE_INT_STAT                                            :  1;
        unsigned int PRIV_INSTR_INT_STAT                                          :  1;
        unsigned int PRIV_REG_INT_STAT                                            :  1;
        unsigned int OPCODE_ERROR_INT_STAT                                        :  1;
        unsigned int                                                              :  1;
        unsigned int TIME_STAMP_INT_STAT                                          :  1;
        unsigned int RESERVED_BIT_ERROR_INT_STAT                                  :  1;
        unsigned int                                                              :  1;
        unsigned int GENERIC2_INT_STAT                                            :  1;
        unsigned int GENERIC1_INT_STAT                                            :  1;
        unsigned int GENERIC0_INT_STAT                                            :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MEC_RS64_PERFCOUNT_CNTL {
    struct {
        unsigned int EVENT_SEL                                                    :  5;
        unsigned int                                                              : 27;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MES_PERFCOUNT_CNTL {
    struct {
        unsigned int EVENT_SEL                                                    :  5;
        unsigned int                                                              : 27;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME_COHER_BASE {
    struct {
        unsigned int COHER_BASE_256B                                              : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME_COHER_BASE_HI {
    struct {
        unsigned int COHER_BASE_HI_256B                                           :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME_COHER_CNTL {
    struct {
        unsigned int DEST_BASE_0_ENA                                              :  1;
        unsigned int DEST_BASE_1_ENA                                              :  1;
        unsigned int                                                              :  4;
        unsigned int CB0_DEST_BASE_ENA                                            :  1;
        unsigned int CB1_DEST_BASE_ENA                                            :  1;
        unsigned int CB2_DEST_BASE_ENA                                            :  1;
        unsigned int CB3_DEST_BASE_ENA                                            :  1;
        unsigned int CB4_DEST_BASE_ENA                                            :  1;
        unsigned int CB5_DEST_BASE_ENA                                            :  1;
        unsigned int CB6_DEST_BASE_ENA                                            :  1;
        unsigned int CB7_DEST_BASE_ENA                                            :  1;
        unsigned int DB_DEST_BASE_ENA                                             :  1;
        unsigned int                                                              :  4;
        unsigned int DEST_BASE_2_ENA                                              :  1;
        unsigned int                                                              :  1;
        unsigned int DEST_BASE_3_ENA                                              :  1;
        unsigned int                                                              : 10;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME_COHER_SIZE {
    struct {
        unsigned int COHER_SIZE_256B                                              : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME_COHER_SIZE_HI {
    struct {
        unsigned int COHER_SIZE_HI_256B                                           :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_ME_COHER_STATUS {
    struct {
        unsigned int MATCHING_GFX_CNTX                                            :  8;
        unsigned int                                                              : 23;
        unsigned int STATUS                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MQD_BASE_ADDR {
    struct {
        unsigned int                                                              :  2;
        unsigned int BASE_ADDR                                                    : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MQD_BASE_ADDR_HI {
    struct {
        unsigned int BASE_ADDR_HI                                                 : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_MQD_CONTROL {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int                                                              :  4;
        unsigned int PRIV_STATE                                                   :  1;
        unsigned int                                                              :  3;
        unsigned int PROCESSING_MQD                                               :  1;
        unsigned int PROCESSING_MQD_EN                                            :  1;
        unsigned int                                                              :  9;
        unsigned int EXE_DISABLE                                                  :  1;
        unsigned int CACHE_POLICY                                                 :  2;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_PERFMON_CNTL {
    struct {
        unsigned int PERFMON_STATE                                                :  4;
        unsigned int SPM_PERFMON_STATE                                            :  4;
        unsigned int PERFMON_ENABLE_MODE                                          :  2;
        unsigned int PERFMON_SAMPLE_ENABLE                                        :  1;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_PERFMON_CNTX_CNTL {
    struct {
        unsigned int                                                              : 31;
        unsigned int PERFMON_ENABLE                                               :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_PIPEID {
    struct {
        unsigned int PIPE_ID                                                      :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_PIPE_STATS_ADDR_HI {
    struct {
        unsigned int PIPE_STATS_ADDR_HI                                           : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_PIPE_STATS_ADDR_LO {
    struct {
        unsigned int                                                              :  2;
        unsigned int PIPE_STATS_ADDR_LO                                           : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_PIPE_STATS_CONTROL {
    struct {
        unsigned int                                                              : 25;
        unsigned int CACHE_POLICY                                                 :  2;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_SC_PSINVOC_COUNT0_HI {
    struct {
        unsigned int PSINVOC_COUNT0_HI                                            : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_SC_PSINVOC_COUNT0_LO {
    struct {
        unsigned int PSINVOC_COUNT0_LO                                            : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_SC_PSINVOC_COUNT1_HI {
    struct {
        unsigned int OBSOLETE                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_SC_PSINVOC_COUNT1_LO {
    struct {
        unsigned int OBSOLETE                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_STAT {
    struct {
        unsigned int                                                              :  5;
        unsigned int ROQ_DB_BUSY                                                  :  1;
        unsigned int ROQ_CE_DB_BUSY                                               :  1;
        unsigned int                                                              :  2;
        unsigned int ROQ_RING_BUSY                                                :  1;
        unsigned int ROQ_INDIRECT1_BUSY                                           :  1;
        unsigned int ROQ_INDIRECT2_BUSY                                           :  1;
        unsigned int ROQ_STATE_BUSY                                               :  1;
        unsigned int DC_BUSY                                                      :  1;
        unsigned int UTCL2IU_BUSY                                                 :  1;
        unsigned int PFP_BUSY                                                     :  1;
        unsigned int MEQ_BUSY                                                     :  1;
        unsigned int ME_BUSY                                                      :  1;
        unsigned int QUERY_BUSY                                                   :  1;
        unsigned int                                                              :  1;
        unsigned int INTERRUPT_BUSY                                               :  1;
        unsigned int SURFACE_SYNC_BUSY                                            :  1;
        unsigned int DMA_BUSY                                                     :  1;
        unsigned int RCIU_BUSY                                                    :  1;
        unsigned int SCRATCH_RAM_BUSY                                             :  1;
        unsigned int GCRIU_BUSY                                                   :  1;
        unsigned int CE_BUSY                                                      :  1;
        unsigned int TCIU_BUSY                                                    :  1;
        unsigned int ROQ_CE_RING_BUSY                                             :  1;
        unsigned int ROQ_CE_INDIRECT1_BUSY                                        :  1;
        unsigned int ROQ_CE_INDIRECT2_BUSY                                        :  1;
        unsigned int CP_BUSY                                                      :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_VGT_ASINVOC_COUNT_HI {
    struct {
        unsigned int ASINVOC_COUNT_HI                                             : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_VGT_ASINVOC_COUNT_LO {
    struct {
        unsigned int ASINVOC_COUNT_LO                                             : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_VGT_CSINVOC_COUNT_HI {
    struct {
        unsigned int CSINVOC_COUNT_HI                                             : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_VGT_CSINVOC_COUNT_LO {
    struct {
        unsigned int CSINVOC_COUNT_LO                                             : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_VGT_DSINVOC_COUNT_HI {
    struct {
        unsigned int DSINVOC_COUNT_HI                                             : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_VGT_DSINVOC_COUNT_LO {
    struct {
        unsigned int DSINVOC_COUNT_LO                                             : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_VGT_GSINVOC_COUNT_HI {
    struct {
        unsigned int GSINVOC_COUNT_HI                                             : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_VGT_GSINVOC_COUNT_LO {
    struct {
        unsigned int GSINVOC_COUNT_LO                                             : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_VGT_GSPRIM_COUNT_HI {
    struct {
        unsigned int GSPRIM_COUNT_HI                                              : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_VGT_GSPRIM_COUNT_LO {
    struct {
        unsigned int GSPRIM_COUNT_LO                                              : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_VGT_HSINVOC_COUNT_HI {
    struct {
        unsigned int HSINVOC_COUNT_HI                                             : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_VGT_HSINVOC_COUNT_LO {
    struct {
        unsigned int HSINVOC_COUNT_LO                                             : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_VGT_IAPRIM_COUNT_HI {
    struct {
        unsigned int IAPRIM_COUNT_HI                                              : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_VGT_IAPRIM_COUNT_LO {
    struct {
        unsigned int IAPRIM_COUNT_LO                                              : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_VGT_IAVERT_COUNT_HI {
    struct {
        unsigned int IAVERT_COUNT_HI                                              : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_VGT_IAVERT_COUNT_LO {
    struct {
        unsigned int IAVERT_COUNT_LO                                              : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_VGT_VSINVOC_COUNT_HI {
    struct {
        unsigned int VSINVOC_COUNT_HI                                             : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union CP_VGT_VSINVOC_COUNT_LO {
    struct {
        unsigned int VSINVOC_COUNT_LO                                             : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_ALPHA_TO_MASK {
    struct {
        unsigned int ALPHA_TO_MASK_ENABLE                                         :  1;
        unsigned int                                                              :  7;
        unsigned int ALPHA_TO_MASK_OFFSET0                                        :  2;
        unsigned int ALPHA_TO_MASK_OFFSET1                                        :  2;
        unsigned int ALPHA_TO_MASK_OFFSET2                                        :  2;
        unsigned int ALPHA_TO_MASK_OFFSET3                                        :  2;
        unsigned int OFFSET_ROUND                                                 :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_ARB_CONFIG {
    struct {
        unsigned int ARB_MODE                                                     :  2;
        unsigned int CREDITS_MAX_RD                                               :  5;
        unsigned int CREDITS_WEIGHT_RD                                            :  5;
        unsigned int CREDITS_MAX_WR                                               :  5;
        unsigned int CREDITS_WEIGHT_WR                                            :  5;
        unsigned int                                                              : 10;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_CGTT_CLK_CTRL_0 {
    struct {
        unsigned int SOFT_OVERRIDE0                                               :  1;
        unsigned int SOFT_OVERRIDE1                                               :  1;
        unsigned int SOFT_OVERRIDE2                                               :  1;
        unsigned int SOFT_OVERRIDE3                                               :  1;
        unsigned int SOFT_OVERRIDE4                                               :  1;
        unsigned int SOFT_OVERRIDE5                                               :  1;
        unsigned int SOFT_OVERRIDE6                                               :  1;
        unsigned int                                                              :  1;
        unsigned int SOFT_OVERRIDE8                                               :  1;
        unsigned int SOFT_OVERRIDE9                                               :  1;
        unsigned int RESERVED                                                     : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_COUNT_CONTROL {
    struct {
        unsigned int                                                              :  1;
        unsigned int PERFECT_ZPASS_COUNTS                                         :  1;
        unsigned int DISABLE_CONSERVATIVE_ZPASS_COUNTS                            :  1;
        unsigned int ENHANCED_CONSERVATIVE_ZPASS_COUNTS                           :  1;
        unsigned int                                                              :  4;
        unsigned int ZPASS_ENABLE                                                 :  4;
        unsigned int ZFAIL_ENABLE                                                 :  4;
        unsigned int SFAIL_ENABLE                                                 :  4;
        unsigned int DBFAIL_ENABLE                                                :  4;
        unsigned int SLICE_EVEN_ENABLE                                            :  4;
        unsigned int SLICE_ODD_ENABLE                                             :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_CREDIT_LIMIT {
    struct {
        unsigned int DB_SC_UPDATE_CREDITS                                         :  5;
        unsigned int DB_SC_QUAD_CREDITS                                           :  5;
        unsigned int DB_CB_EXPORT_CREDITS                                         :  3;
        unsigned int DB_SC_WAVE_CREDITS                                           :  5;
        unsigned int DB_SC_FREE_WAVE_CREDITS                                      :  5;
        unsigned int                                                              :  9;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_DEPTH_BOUNDS_MAX {
    struct {
        unsigned int MAX                                                          : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_DEPTH_BOUNDS_MIN {
    struct {
        unsigned int MIN                                                          : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_DEPTH_CONTROL {
    struct {
        unsigned int STENCIL_ENABLE                                               :  1;
        unsigned int Z_ENABLE                                                     :  1;
        unsigned int Z_WRITE_ENABLE                                               :  1;
        unsigned int DEPTH_BOUNDS_ENABLE                                          :  1;
        unsigned int ZFUNC                                                        :  3;
        unsigned int BACKFACE_ENABLE                                              :  1;
        unsigned int STENCILFUNC                                                  :  3;
        unsigned int                                                              :  9;
        unsigned int STENCILFUNC_BF                                               :  3;
        unsigned int                                                              :  7;
        unsigned int RESERVED_FIELD_30                                            :  1;
        unsigned int RESERVED_FIELD_31                                            :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_DEPTH_SIZE_XY {
    struct {
        unsigned int X_MAX                                                        : 16;
        unsigned int Y_MAX                                                        : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_DEPTH_VIEW {
    struct {
        unsigned int SLICE_START                                                  : 14;
        unsigned int                                                              :  2;
        unsigned int SLICE_MAX                                                    : 14;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_DEPTH_VIEW1 {
    struct {
        unsigned int                                                              : 24;
        unsigned int Z_READ_ONLY                                                  :  1;
        unsigned int STENCIL_READ_ONLY                                            :  1;
        unsigned int MIPID                                                        :  5;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_DFD_INDIRECT_DAT {
    struct {
        unsigned int DFD_DATA                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_DFD_INDIRECT_SEL {
    struct {
        unsigned int DFD_INDEX                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_EQAA {
    struct {
        unsigned int RESERVED_FIELD_1                                             :  3;
        unsigned int                                                              :  5;
        unsigned int MASK_EXPORT_NUM_SAMPLES                                      :  3;
        unsigned int                                                              :  1;
        unsigned int ALPHA_TO_MASK_NUM_SAMPLES                                    :  3;
        unsigned int                                                              :  1;
        unsigned int HIGH_QUALITY_INTERSECTIONS                                   :  1;
        unsigned int RESERVED_FIELD_17                                            :  1;
        unsigned int RESERVED_FIELD_18                                            :  1;
        unsigned int RESERVED_FIELD_19                                            :  1;
        unsigned int STATIC_ANCHOR_ASSOCIATIONS                                   :  1;
        unsigned int RESERVED_FIELD_21                                            :  1;
        unsigned int                                                              :  2;
        unsigned int OVERRASTERIZATION_AMOUNT                                     :  3;
        unsigned int ENABLE_POSTZ_OVERRASTERIZATION                               :  1;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_EXCEPTION_CONTROL {
    struct {
        unsigned int EARLY_Z_PANIC_DISABLE                                        :  1;
        unsigned int LATE_Z_PANIC_DISABLE                                         :  1;
        unsigned int RE_Z_PANIC_DISABLE                                           :  1;
        unsigned int AUTO_FLUSH_QUAD                                              :  1;
        unsigned int FORCE_SUMMARIZE                                              :  4;
        unsigned int LQUAD_FIFO_LO_WATERMARK                                      :  8;
        unsigned int LQUAD_FIFO_HI_WATERMARK                                      :  8;
        unsigned int CAM_FREE_WATERMARK                                           :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_FGCG_INTERFACES_CLK_CTRL {
    struct {
        unsigned int DB_SC_QUAD_OVERRIDE                                          :  1;
        unsigned int                                                              :  1;
        unsigned int DB_CB_EXPORT_OVERRIDE                                        :  1;
        unsigned int DB_GL1_COMP_REQ_OVERRIDE                                     :  1;
        unsigned int DB_GL1_SRC_OVERRIDE                                          :  1;
        unsigned int DB_SC_UPDATE_OVERRIDE                                        :  1;
        unsigned int DB_CB_RMIRET_OVERRIDE                                        :  1;
        unsigned int DB_SC_WAVE_OVERRIDE                                          :  1;
        unsigned int DB_SC_FREE_WAVE_OVERRIDE                                     :  1;
        unsigned int                                                              : 23;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_FGCG_SRAMS_CLK_CTRL {
    struct {
        unsigned int OVERRIDE0                                                    :  1;
        unsigned int OVERRIDE1                                                    :  1;
        unsigned int OVERRIDE2                                                    :  1;
        unsigned int OVERRIDE3                                                    :  1;
        unsigned int OVERRIDE4                                                    :  1;
        unsigned int OVERRIDE5                                                    :  1;
        unsigned int OVERRIDE6                                                    :  1;
        unsigned int OVERRIDE7                                                    :  1;
        unsigned int OVERRIDE8                                                    :  1;
        unsigned int OVERRIDE9                                                    :  1;
        unsigned int OVERRIDE10                                                   :  1;
        unsigned int OVERRIDE11                                                   :  1;
        unsigned int OVERRIDE12                                                   :  1;
        unsigned int OVERRIDE13                                                   :  1;
        unsigned int OVERRIDE14                                                   :  1;
        unsigned int OVERRIDE15                                                   :  1;
        unsigned int OVERRIDE16                                                   :  1;
        unsigned int OVERRIDE17                                                   :  1;
        unsigned int OVERRIDE18                                                   :  1;
        unsigned int OVERRIDE19                                                   :  1;
        unsigned int OVERRIDE20                                                   :  1;
        unsigned int                                                              :  1;
        unsigned int OVERRIDE22                                                   :  1;
        unsigned int OVERRIDE23                                                   :  1;
        unsigned int OVERRIDE24                                                   :  1;
        unsigned int                                                              :  1;
        unsigned int OVERRIDE26                                                   :  1;
        unsigned int OVERRIDE27                                                   :  1;
        unsigned int OVERRIDE28                                                   :  1;
        unsigned int OVERRIDE29                                                   :  1;
        unsigned int OVERRIDE30                                                   :  1;
        unsigned int OVERRIDE31                                                   :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_FIFO_DEPTH1 {
    struct {
        unsigned int MI_RDREQ_FIFO_DEPTH                                          :  8;
        unsigned int MI_WRREQ_FIFO_DEPTH                                          :  8;
        unsigned int                                                              :  8;
        unsigned int QC_DEPTH                                                     :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_FIFO_DEPTH2 {
    struct {
        unsigned int EQUAD_FIFO_DEPTH                                             :  8;
        unsigned int ETILE_OP_FIFO_DEPTH                                          :  8;
        unsigned int LQUAD_FIFO_DEPTH                                             :  9;
        unsigned int LTILE_OP_FIFO_DEPTH                                          :  7;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_FIFO_DEPTH3 {
    struct {
        unsigned int LTILE_PROBE_FIFO_DEPTH                                       :  8;
        unsigned int OSB_WAVE_TABLE_DEPTH                                         :  8;
        unsigned int OREO_WAVE_HIDE_DEPTH                                         :  8;
        unsigned int QUAD_READ_REQS                                               :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_FIFO_DEPTH4 {
    struct {
        unsigned int OSB_SQUAD_TABLE_DEPTH                                        :  8;
        unsigned int OSB_TILE_TABLE_DEPTH                                         :  8;
        unsigned int OSB_SCORE_BOARD_DEPTH                                        :  8;
        unsigned int OSB_EVENT_FIFO_DEPTH                                         :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_FREE_CACHELINES {
    struct {
        unsigned int FREE_DTILE_DEPTH                                             :  8;
        unsigned int FREE_PLANE_DEPTH                                             :  8;
        unsigned int FREE_Z_DEPTH                                                 :  8;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_GL1_INTERFACE_CONTROL {
    struct {
        unsigned int Z_SPECULATIVE_READ                                           :  2;
        unsigned int STENCIL_SPECULATIVE_READ                                     :  2;
        unsigned int Z_COMPRESSION_MODE                                           :  2;
        unsigned int STENCIL_COMPRESSION_MODE                                     :  2;
        unsigned int OCCLUSION_COMPRESSION_MODE                                   :  2;
        unsigned int                                                              : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_HTILE_SURFACE {
    struct {
        unsigned int RESERVED_FIELD_1                                             :  1;
        unsigned int                                                              :  1;
        unsigned int RESERVED_FIELD_2                                             :  1;
        unsigned int RESERVED_FIELD_3                                             :  1;
        unsigned int RESERVED_FIELD_4                                             :  6;
        unsigned int RESERVED_FIELD_5                                             :  6;
        unsigned int DST_OUTSIDE_ZERO_TO_ONE                                      :  1;
        unsigned int RESERVED_FIELD_6                                             :  1;
        unsigned int                                                              : 14;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_MEM_ARB_WATERMARKS {
    struct {
        unsigned int CLIENT0_WATERMARK                                            :  3;
        unsigned int                                                              :  5;
        unsigned int CLIENT1_WATERMARK                                            :  3;
        unsigned int                                                              :  5;
        unsigned int CLIENT2_WATERMARK                                            :  3;
        unsigned int                                                              :  5;
        unsigned int CLIENT3_WATERMARK                                            :  3;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_MEM_CONFIG {
    struct {
        unsigned int Z_SCOPE                                                      :  2;
        unsigned int STENCIL_SCOPE                                                :  2;
        unsigned int OCCLUSION_SCOPE                                              :  2;
        unsigned int ENABLE_OVERRIDE_COMP_MODE_Z                                  :  1;
        unsigned int ENABLE_OVERRIDE_COMP_MODE_STENCIL                            :  1;
        unsigned int ENABLE_OVERRIDE_COMP_MODE_OCCLUSION                          :  1;
        unsigned int Z_OVERRIDE_COMPRESSION_MODE                                  :  2;
        unsigned int STENCIL_OVERRIDE_COMPRESSION_MODE                            :  2;
        unsigned int OCCLUSION_OVERRIDE_COMPRESSION_MODE                          :  2;
        unsigned int FL_DISABLE_SINGLE_COMPRESS                                   :  1;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_MEM_TEMPORAL {
    struct {
        unsigned int Z_TEMPORAL_READ                                              :  3;
        unsigned int Z_TEMPORAL_WRITE                                             :  3;
        unsigned int STENCIL_TEMPORAL_READ                                        :  3;
        unsigned int STENCIL_TEMPORAL_WRITE                                       :  3;
        unsigned int OCCLUSION_TEMPORAL_WRITE                                     :  3;
        unsigned int                                                              : 17;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_OCCLUSION_COUNT0_HI {
    struct {
        unsigned int COUNT_HI                                                     : 31;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_OCCLUSION_COUNT0_LOW {
    struct {
        unsigned int COUNT_LOW                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_OCCLUSION_COUNT1_HI {
    struct {
        unsigned int COUNT_HI                                                     : 31;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_OCCLUSION_COUNT1_LOW {
    struct {
        unsigned int COUNT_LOW                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_OCCLUSION_COUNT2_HI {
    struct {
        unsigned int COUNT_HI                                                     : 31;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_OCCLUSION_COUNT2_LOW {
    struct {
        unsigned int COUNT_LOW                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_OCCLUSION_COUNT3_HI {
    struct {
        unsigned int COUNT_HI                                                     : 31;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_OCCLUSION_COUNT3_LOW {
    struct {
        unsigned int COUNT_LOW                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_PERFCOUNTER1_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_PERFCOUNTER2_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_PERFCOUNTER3_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_PERFCOUNTER3_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_PERFCOUNTER3_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_RENDER_CONTROL {
    struct {
        unsigned int                                                              :  1;
        unsigned int STENCIL_CLEAR_ENABLE                                         :  1;
        unsigned int RESERVED_FIELD_1                                             :  1;
        unsigned int RESERVED_FIELD_2                                             :  1;
        unsigned int RESERVED_FIELD_4                                             :  1;
        unsigned int STENCIL_COMPRESS_DISABLE                                     :  1;
        unsigned int DEPTH_COMPRESS_DISABLE                                       :  1;
        unsigned int COPY_CENTROID                                                :  1;
        unsigned int COPY_SAMPLE                                                  :  4;
        unsigned int DECOMPRESS_ENABLE                                            :  1;
        unsigned int                                                              :  3;
        unsigned int OREO_MODE                                                    :  2;
        unsigned int FORCE_OREO_MODE                                              :  1;
        unsigned int FORCE_EXPORT_ORDER                                           :  1;
        unsigned int MAX_ALLOWED_STILES_IN_WAVE                                   :  4;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_RENDER_OVERRIDE {
    struct {
        unsigned int FORCE_HIZ_ENABLE                                             :  2;
        unsigned int FORCE_HIS_ENABLE0                                            :  2;
        unsigned int FORCE_HIS_ENABLE1                                            :  2;
        unsigned int FORCE_SHADER_Z_ORDER                                         :  1;
        unsigned int FAST_Z_DISABLE                                               :  1;
        unsigned int FAST_STENCIL_DISABLE                                         :  1;
        unsigned int NOOP_CULL_DISABLE                                            :  1;
        unsigned int FORCE_COLOR_KILL                                             :  1;
        unsigned int FORCE_Z_READ                                                 :  1;
        unsigned int FORCE_STENCIL_READ                                           :  1;
        unsigned int FORCE_FULL_Z_RANGE                                           :  2;
        unsigned int FORCE_Z_ALLOC                                                :  1;
        unsigned int                                                              :  3;
        unsigned int FORCE_Z_LIMIT_SUMM                                           :  2;
        unsigned int MAX_TILES_IN_DTT                                             :  5;
        unsigned int DISABLE_TILE_RATE_TILES                                      :  1;
        unsigned int FORCE_Z_DIRTY                                                :  1;
        unsigned int FORCE_STENCIL_DIRTY                                          :  1;
        unsigned int FORCE_Z_VALID                                                :  1;
        unsigned int FORCE_STENCIL_VALID                                          :  1;
        unsigned int PRESERVE_COMPRESSION                                         :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_RENDER_OVERRIDE2 {
    struct {
        unsigned int PARTIAL_SQUAD_LAUNCH_CONTROL                                 :  2;
        unsigned int PARTIAL_SQUAD_LAUNCH_COUNTDOWN                               :  3;
        unsigned int RESERVED_FIELD_5                                             :  1;
        unsigned int RESERVED_FIELD_6                                             :  1;
        unsigned int DISABLE_COLOR_ON_VALIDATION                                  :  1;
        unsigned int DECOMPRESS_Z_ON_FLUSH                                        :  1;
        unsigned int RESERVED_FIELD_1                                             :  1;
        unsigned int DEPTH_BOUNDS_HIER_DEPTH_DISABLE                              :  1;
        unsigned int FORCE_SUMM_Z_RANGE_TO_MAX                                    :  1;
        unsigned int FORCE_SUMM_STENCIL_RANGE_TO_MAX                              :  1;
        unsigned int RESERVED_FIELD_2                                             :  8;
        unsigned int PRESERVE_ZRANGE                                              :  1;
        unsigned int                                                              :  1;
        unsigned int DISABLE_FAST_PASS                                            :  1;
        unsigned int                                                              :  1;
        unsigned int ALLOW_PARTIAL_RES_HIER_KILL                                  :  1;
        unsigned int                                                              :  1;
        unsigned int CENTROID_COMPUTATION_MODE                                    :  2;
        unsigned int DISABLE_NOZ                                                  :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_RING_CONTROL {
    struct {
        unsigned int COUNTER_CONTROL                                              :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_SHADER_CONTROL {
    struct {
        unsigned int Z_EXPORT_ENABLE                                              :  1;
        unsigned int STENCIL_TEST_VAL_EXPORT_ENABLE                               :  1;
        unsigned int STENCIL_OP_VAL_EXPORT_ENABLE                                 :  1;
        unsigned int                                                              :  1;
        unsigned int Z_ORDER                                                      :  2;
        unsigned int KILL_ENABLE                                                  :  1;
        unsigned int COVERAGE_TO_MASK_ENABLE                                      :  1;
        unsigned int MASK_EXPORT_ENABLE                                           :  1;
        unsigned int EXEC_ON_HIER_FAIL                                            :  1;
        unsigned int EXEC_ON_NOOP                                                 :  1;
        unsigned int ALPHA_TO_MASK_DISABLE                                        :  1;
        unsigned int DEPTH_BEFORE_SHADER                                          :  1;
        unsigned int CONSERVATIVE_Z_EXPORT                                        :  2;
        unsigned int DUAL_QUAD_DISABLE                                            :  1;
        unsigned int PRIMITIVE_ORDERED_PIXEL_SHADER                               :  1;
        unsigned int                                                              :  6;
        unsigned int PRE_SHADER_DEPTH_COVERAGE_ENABLE                             :  1;
        unsigned int OREO_BLEND_ENABLE                                            :  1;
        unsigned int OVERRIDE_INTRINSIC_RATE_ENABLE                               :  1;
        unsigned int OVERRIDE_INTRINSIC_RATE                                      :  3;
        unsigned int                                                              :  3;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_SPI_VRS_CENTER_LOCATION {
    struct {
        unsigned int CENTER_X_OFFSET_1X1                                          :  4;
        unsigned int CENTER_Y_OFFSET_1X1                                          :  4;
        unsigned int CENTER_X_OFFSET_2X1                                          :  4;
        unsigned int CENTER_Y_OFFSET_2X1                                          :  4;
        unsigned int CENTER_X_OFFSET_1X2                                          :  4;
        unsigned int CENTER_Y_OFFSET_1X2                                          :  4;
        unsigned int CENTER_X_OFFSET_2X2                                          :  4;
        unsigned int CENTER_Y_OFFSET_2X2                                          :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_SRESULTS_COMPARE_STATE0 {
    struct {
        unsigned int COMPAREFUNC0                                                 :  3;
        unsigned int                                                              :  1;
        unsigned int COMPAREVALUE0                                                :  8;
        unsigned int COMPAREMASK0                                                 :  8;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_SRESULTS_COMPARE_STATE1 {
    struct {
        unsigned int COMPAREFUNC1                                                 :  3;
        unsigned int                                                              :  1;
        unsigned int COMPAREVALUE1                                                :  8;
        unsigned int COMPAREMASK1                                                 :  8;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_STENCIL_CONTROL {
    struct {
        unsigned int STENCILFAIL                                                  :  4;
        unsigned int STENCILZPASS                                                 :  4;
        unsigned int STENCILZFAIL                                                 :  4;
        unsigned int STENCILFAIL_BF                                               :  4;
        unsigned int STENCILZPASS_BF                                              :  4;
        unsigned int STENCILZFAIL_BF                                              :  4;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_STENCIL_INFO {
    struct {
        unsigned int FORMAT                                                       :  1;
        unsigned int                                                              :  3;
        unsigned int SW_MODE                                                      :  5;
        unsigned int RESERVED_FIELD_2                                             :  2;
        unsigned int RESERVED_FIELD_11                                            :  1;
        unsigned int RESERVED_FIELD_12                                            :  1;
        unsigned int RESERVED_FIELD_1                                             :  3;
        unsigned int                                                              :  4;
        unsigned int RESERVED_FIELD_20                                            :  1;
        unsigned int                                                              :  6;
        unsigned int RESERVED_FIELD_27                                            :  1;
        unsigned int                                                              :  1;
        unsigned int TILE_STENCIL_DISABLE                                         :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_STENCIL_OPVAL {
    struct {
        unsigned int OPVAL                                                        :  8;
        unsigned int OPVAL_BF                                                     :  8;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_STENCIL_READ_BASE {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_STENCIL_READ_BASE_HI {
    struct {
        unsigned int BASE_HI                                                      :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_STENCIL_READ_MASK {
    struct {
        unsigned int TESTMASK                                                     :  8;
        unsigned int TESTMASK_BF                                                  :  8;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_STENCIL_REF {
    struct {
        unsigned int TESTVAL                                                      :  8;
        unsigned int TESTVAL_BF                                                   :  8;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_STENCIL_WRITE_BASE {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_STENCIL_WRITE_BASE_HI {
    struct {
        unsigned int BASE_HI                                                      :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_STENCIL_WRITE_MASK {
    struct {
        unsigned int WRITEMASK                                                    :  8;
        unsigned int WRITEMASK_BF                                                 :  8;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_SUMMARIZER_TIMEOUTS {
    struct {
        unsigned int SUMM_CNTL_EVICT_TIMEOUT                                      : 12;
        unsigned int                                                              :  4;
        unsigned int SUMM_EVICT_TIMEOUT                                           : 12;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_VIEWPORT_CONTROL {
    struct {
        unsigned int DISABLE_VIEWPORT_CLAMP                                       :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_WATERMARKS {
    struct {
        unsigned int DEPTH_FREE                                                   :  8;
        unsigned int DEPTH_FLUSH                                                  :  8;
        unsigned int DEPTH_PENDING_FREE                                           :  8;
        unsigned int DEPTH_CACHELINE_FREE                                         :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_Z_INFO {
    struct {
        unsigned int FORMAT                                                       :  2;
        unsigned int NUM_SAMPLES                                                  :  2;
        unsigned int SW_MODE                                                      :  5;
        unsigned int RESERVED_FIELD_2                                             :  2;
        unsigned int RESERVED_FIELD_11                                            :  1;
        unsigned int RESERVED_FIELD_12                                            :  1;
        unsigned int RESERVED_FIELD_1                                             :  2;
        unsigned int MAXMIP                                                       :  5;
        unsigned int RESERVED_FIELD_20                                            :  1;
        unsigned int                                                              :  2;
        unsigned int DECOMPRESS_ON_N_ZPLANES                                      :  4;
        unsigned int RESERVED_FIELD_27                                            :  1;
        unsigned int RESERVED_FIELD_28                                            :  1;
        unsigned int TILE_SURFACE_ENABLE                                          :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_Z_READ_BASE {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_Z_READ_BASE_HI {
    struct {
        unsigned int BASE_HI                                                      :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_Z_WRITE_BASE {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DB_Z_WRITE_BASE_HI {
    struct {
        unsigned int BASE_HI                                                      :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtlHi0 {
    struct {
        unsigned int EventSelectHi                                                :  6;
        unsigned int                                                              :  2;
        unsigned int Filter                                                       : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtlHi1 {
    struct {
        unsigned int EventSelectHi                                                :  6;
        unsigned int                                                              :  2;
        unsigned int Filter                                                       : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtlHi2 {
    struct {
        unsigned int EventSelectHi                                                :  6;
        unsigned int                                                              :  2;
        unsigned int Filter                                                       : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtlHi3 {
    struct {
        unsigned int EventSelectHi                                                :  6;
        unsigned int                                                              :  2;
        unsigned int Filter                                                       : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtlHi4 {
    struct {
        unsigned int EventSelectHi                                                :  6;
        unsigned int                                                              :  2;
        unsigned int Filter                                                       : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtlHi5 {
    struct {
        unsigned int EventSelectHi                                                :  6;
        unsigned int                                                              :  2;
        unsigned int Filter                                                       : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtlHi6 {
    struct {
        unsigned int EventSelectHi                                                :  6;
        unsigned int                                                              :  2;
        unsigned int Filter                                                       : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtlHi7 {
    struct {
        unsigned int EventSelectHi                                                :  6;
        unsigned int                                                              :  2;
        unsigned int Filter                                                       : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtlHi8 {
    struct {
        unsigned int EventSelectHi                                                :  6;
        unsigned int                                                              :  2;
        unsigned int Filter                                                       : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtlHi9 {
    struct {
        unsigned int EventSelectHi                                                :  6;
        unsigned int                                                              :  2;
        unsigned int Filter                                                       : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtlHi10 {
    struct {
        unsigned int EventSelectHi                                                :  6;
        unsigned int                                                              :  2;
        unsigned int Filter                                                       : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtlHi11 {
    struct {
        unsigned int EventSelectHi                                                :  6;
        unsigned int                                                              :  2;
        unsigned int Filter                                                       : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtlHi12 {
    struct {
        unsigned int EventSelectHi                                                :  6;
        unsigned int                                                              :  2;
        unsigned int Filter                                                       : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtlHi13 {
    struct {
        unsigned int EventSelectHi                                                :  6;
        unsigned int                                                              :  2;
        unsigned int Filter                                                       : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtlHi14 {
    struct {
        unsigned int EventSelectHi                                                :  6;
        unsigned int                                                              :  2;
        unsigned int Filter                                                       : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtlHi15 {
    struct {
        unsigned int EventSelectHi                                                :  6;
        unsigned int                                                              :  2;
        unsigned int Filter                                                       : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtlLo0 {
    struct {
        unsigned int EventSelectLo                                                :  8;
        unsigned int UnitMaskLo                                                   :  8;
        unsigned int                                                              :  3;
        unsigned int DsmTrig                                                      :  1;
        unsigned int Mp1Int                                                       :  1;
        unsigned int BypassEn                                                     :  1;
        unsigned int En                                                           :  1;
        unsigned int                                                              :  1;
        unsigned int UnitMaskHi                                                   :  4;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtlLo1 {
    struct {
        unsigned int EventSelectLo                                                :  8;
        unsigned int UnitMaskLo                                                   :  8;
        unsigned int                                                              :  3;
        unsigned int DsmTrig                                                      :  1;
        unsigned int Mp1Int                                                       :  1;
        unsigned int BypassEn                                                     :  1;
        unsigned int En                                                           :  1;
        unsigned int                                                              :  1;
        unsigned int UnitMaskHi                                                   :  4;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtlLo2 {
    struct {
        unsigned int EventSelectLo                                                :  8;
        unsigned int UnitMaskLo                                                   :  8;
        unsigned int                                                              :  3;
        unsigned int DsmTrig                                                      :  1;
        unsigned int Mp1Int                                                       :  1;
        unsigned int BypassEn                                                     :  1;
        unsigned int En                                                           :  1;
        unsigned int                                                              :  1;
        unsigned int UnitMaskHi                                                   :  4;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtlLo3 {
    struct {
        unsigned int EventSelectLo                                                :  8;
        unsigned int UnitMaskLo                                                   :  8;
        unsigned int                                                              :  3;
        unsigned int DsmTrig                                                      :  1;
        unsigned int Mp1Int                                                       :  1;
        unsigned int BypassEn                                                     :  1;
        unsigned int En                                                           :  1;
        unsigned int                                                              :  1;
        unsigned int UnitMaskHi                                                   :  4;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtlLo4 {
    struct {
        unsigned int EventSelectLo                                                :  8;
        unsigned int UnitMaskLo                                                   :  8;
        unsigned int                                                              :  3;
        unsigned int DsmTrig                                                      :  1;
        unsigned int Mp1Int                                                       :  1;
        unsigned int BypassEn                                                     :  1;
        unsigned int En                                                           :  1;
        unsigned int                                                              :  1;
        unsigned int UnitMaskHi                                                   :  4;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtlLo5 {
    struct {
        unsigned int EventSelectLo                                                :  8;
        unsigned int UnitMaskLo                                                   :  8;
        unsigned int                                                              :  3;
        unsigned int DsmTrig                                                      :  1;
        unsigned int Mp1Int                                                       :  1;
        unsigned int BypassEn                                                     :  1;
        unsigned int En                                                           :  1;
        unsigned int                                                              :  1;
        unsigned int UnitMaskHi                                                   :  4;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtlLo6 {
    struct {
        unsigned int EventSelectLo                                                :  8;
        unsigned int UnitMaskLo                                                   :  8;
        unsigned int                                                              :  3;
        unsigned int DsmTrig                                                      :  1;
        unsigned int Mp1Int                                                       :  1;
        unsigned int BypassEn                                                     :  1;
        unsigned int En                                                           :  1;
        unsigned int                                                              :  1;
        unsigned int UnitMaskHi                                                   :  4;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtlLo7 {
    struct {
        unsigned int EventSelectLo                                                :  8;
        unsigned int UnitMaskLo                                                   :  8;
        unsigned int                                                              :  3;
        unsigned int DsmTrig                                                      :  1;
        unsigned int Mp1Int                                                       :  1;
        unsigned int BypassEn                                                     :  1;
        unsigned int En                                                           :  1;
        unsigned int                                                              :  1;
        unsigned int UnitMaskHi                                                   :  4;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtlLo8 {
    struct {
        unsigned int EventSelectLo                                                :  8;
        unsigned int UnitMaskLo                                                   :  8;
        unsigned int                                                              :  3;
        unsigned int DsmTrig                                                      :  1;
        unsigned int Mp1Int                                                       :  1;
        unsigned int BypassEn                                                     :  1;
        unsigned int En                                                           :  1;
        unsigned int                                                              :  1;
        unsigned int UnitMaskHi                                                   :  4;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtlLo9 {
    struct {
        unsigned int EventSelectLo                                                :  8;
        unsigned int UnitMaskLo                                                   :  8;
        unsigned int                                                              :  3;
        unsigned int DsmTrig                                                      :  1;
        unsigned int Mp1Int                                                       :  1;
        unsigned int BypassEn                                                     :  1;
        unsigned int En                                                           :  1;
        unsigned int                                                              :  1;
        unsigned int UnitMaskHi                                                   :  4;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtlLo10 {
    struct {
        unsigned int EventSelectLo                                                :  8;
        unsigned int UnitMaskLo                                                   :  8;
        unsigned int                                                              :  3;
        unsigned int DsmTrig                                                      :  1;
        unsigned int Mp1Int                                                       :  1;
        unsigned int BypassEn                                                     :  1;
        unsigned int En                                                           :  1;
        unsigned int                                                              :  1;
        unsigned int UnitMaskHi                                                   :  4;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtlLo11 {
    struct {
        unsigned int EventSelectLo                                                :  8;
        unsigned int UnitMaskLo                                                   :  8;
        unsigned int                                                              :  3;
        unsigned int DsmTrig                                                      :  1;
        unsigned int Mp1Int                                                       :  1;
        unsigned int BypassEn                                                     :  1;
        unsigned int En                                                           :  1;
        unsigned int                                                              :  1;
        unsigned int UnitMaskHi                                                   :  4;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtlLo12 {
    struct {
        unsigned int EventSelectLo                                                :  8;
        unsigned int UnitMaskLo                                                   :  8;
        unsigned int                                                              :  3;
        unsigned int DsmTrig                                                      :  1;
        unsigned int Mp1Int                                                       :  1;
        unsigned int BypassEn                                                     :  1;
        unsigned int En                                                           :  1;
        unsigned int                                                              :  1;
        unsigned int UnitMaskHi                                                   :  4;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtlLo13 {
    struct {
        unsigned int EventSelectLo                                                :  8;
        unsigned int UnitMaskLo                                                   :  8;
        unsigned int                                                              :  3;
        unsigned int DsmTrig                                                      :  1;
        unsigned int Mp1Int                                                       :  1;
        unsigned int BypassEn                                                     :  1;
        unsigned int En                                                           :  1;
        unsigned int                                                              :  1;
        unsigned int UnitMaskHi                                                   :  4;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtlLo14 {
    struct {
        unsigned int EventSelectLo                                                :  8;
        unsigned int UnitMaskLo                                                   :  8;
        unsigned int                                                              :  3;
        unsigned int DsmTrig                                                      :  1;
        unsigned int Mp1Int                                                       :  1;
        unsigned int BypassEn                                                     :  1;
        unsigned int En                                                           :  1;
        unsigned int                                                              :  1;
        unsigned int UnitMaskHi                                                   :  4;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtlLo15 {
    struct {
        unsigned int EventSelectLo                                                :  8;
        unsigned int UnitMaskLo                                                   :  8;
        unsigned int                                                              :  3;
        unsigned int DsmTrig                                                      :  1;
        unsigned int Mp1Int                                                       :  1;
        unsigned int BypassEn                                                     :  1;
        unsigned int En                                                           :  1;
        unsigned int                                                              :  1;
        unsigned int UnitMaskHi                                                   :  4;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtrHi0 {
    struct {
        unsigned int CTR_47_32                                                    : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtrHi1 {
    struct {
        unsigned int CTR_47_32                                                    : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtrHi2 {
    struct {
        unsigned int CTR_47_32                                                    : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtrHi3 {
    struct {
        unsigned int CTR_47_32                                                    : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtrHi4 {
    struct {
        unsigned int CTR_47_32                                                    : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtrHi5 {
    struct {
        unsigned int CTR_47_32                                                    : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtrHi6 {
    struct {
        unsigned int CTR_47_32                                                    : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtrHi7 {
    struct {
        unsigned int CTR_47_32                                                    : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtrHi8 {
    struct {
        unsigned int CTR_47_32                                                    : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtrHi9 {
    struct {
        unsigned int CTR_47_32                                                    : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtrHi10 {
    struct {
        unsigned int CTR_47_32                                                    : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtrHi11 {
    struct {
        unsigned int CTR_47_32                                                    : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtrHi12 {
    struct {
        unsigned int CTR_47_32                                                    : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtrHi13 {
    struct {
        unsigned int CTR_47_32                                                    : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtrHi14 {
    struct {
        unsigned int CTR_47_32                                                    : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtrHi15 {
    struct {
        unsigned int CTR_47_32                                                    : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtrLo0 {
    struct {
        unsigned int CTR_31_0                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtrLo1 {
    struct {
        unsigned int CTR_31_0                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtrLo2 {
    struct {
        unsigned int CTR_31_0                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtrLo3 {
    struct {
        unsigned int CTR_31_0                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtrLo4 {
    struct {
        unsigned int CTR_31_0                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtrLo5 {
    struct {
        unsigned int CTR_31_0                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtrLo6 {
    struct {
        unsigned int CTR_31_0                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtrLo7 {
    struct {
        unsigned int CTR_31_0                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtrLo8 {
    struct {
        unsigned int CTR_31_0                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtrLo9 {
    struct {
        unsigned int CTR_31_0                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtrLo10 {
    struct {
        unsigned int CTR_31_0                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtrLo11 {
    struct {
        unsigned int CTR_31_0                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtrLo12 {
    struct {
        unsigned int CTR_31_0                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtrLo13 {
    struct {
        unsigned int CTR_31_0                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtrLo14 {
    struct {
        unsigned int CTR_31_0                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union DF_PIE_AON_PerfMonCtrLo15 {
    struct {
        unsigned int CTR_31_0                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_ADDR_CONFIG {
    struct {
        unsigned int NUM_PIPES                                                    :  3;
        unsigned int PIPE_INTERLEAVE_SIZE                                         :  3;
        unsigned int MAX_COMPRESSED_FRAGS                                         :  2;
        unsigned int NUM_PKRS                                                     :  3;
        unsigned int                                                              :  8;
        unsigned int NUM_SHADER_ENGINES                                           :  4;
        unsigned int                                                              :  3;
        unsigned int NUM_RB_PER_SE                                                :  2;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GB_ADDR_CONFIG_READ {
    struct {
        unsigned int NUM_PIPES                                                    :  3;
        unsigned int PIPE_INTERLEAVE_SIZE                                         :  3;
        unsigned int MAX_COMPRESSED_FRAGS                                         :  2;
        unsigned int NUM_PKRS                                                     :  3;
        unsigned int                                                              :  8;
        unsigned int NUM_SHADER_ENGINES                                           :  4;
        unsigned int                                                              :  3;
        unsigned int NUM_RB_PER_SE                                                :  2;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_L2_PERFCOUNTER0_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_L2_PERFCOUNTER1_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_L2_PERFCOUNTER2_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_L2_PERFCOUNTER3_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_L2_PERFCOUNTER4_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_L2_PERFCOUNTER5_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_L2_PERFCOUNTER6_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_L2_PERFCOUNTER7_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_L2_PERFCOUNTER_HI {
    struct {
        unsigned int COUNTER_HI                                                   : 16;
        unsigned int COMPARE_VALUE                                                : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_L2_PERFCOUNTER_LO {
    struct {
        unsigned int COUNTER_LO                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCMC_VM_L2_PERFCOUNTER_RSLT_CNTL {
    struct {
        unsigned int PERF_COUNTER_SELECT                                          :  4;
        unsigned int                                                              :  4;
        unsigned int START_TRIGGER                                                :  8;
        unsigned int STOP_TRIGGER                                                 :  8;
        unsigned int ENABLE_ANY                                                   :  1;
        unsigned int CLEAR_ALL                                                    :  1;
        unsigned int STOP_ALL_ON_SATURATE                                         :  1;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCR_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCR_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCR_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCR_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCR_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCR_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCR_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCR_PERFCOUNTER1_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCVML2_PERFCOUNTER2_0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCVML2_PERFCOUNTER2_0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCVML2_PERFCOUNTER2_0_MODE {
    struct {
        unsigned int COMPARE_MODE0                                                :  2;
        unsigned int COMPARE_MODE1                                                :  2;
        unsigned int COMPARE_MODE2                                                :  2;
        unsigned int COMPARE_MODE3                                                :  2;
        unsigned int COMPARE_VALUE0                                               :  4;
        unsigned int COMPARE_VALUE1                                               :  4;
        unsigned int COMPARE_VALUE2                                               :  4;
        unsigned int COMPARE_VALUE3                                               :  4;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCVML2_PERFCOUNTER2_0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCVML2_PERFCOUNTER2_0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCVML2_PERFCOUNTER2_1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCVML2_PERFCOUNTER2_1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCVML2_PERFCOUNTER2_1_MODE {
    struct {
        unsigned int COMPARE_MODE0                                                :  2;
        unsigned int COMPARE_MODE1                                                :  2;
        unsigned int COMPARE_MODE2                                                :  2;
        unsigned int COMPARE_MODE3                                                :  2;
        unsigned int COMPARE_VALUE0                                               :  4;
        unsigned int COMPARE_VALUE1                                               :  4;
        unsigned int COMPARE_VALUE2                                               :  4;
        unsigned int COMPARE_VALUE3                                               :  4;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCVML2_PERFCOUNTER2_1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GCVML2_PERFCOUNTER2_1_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_EA_CPWD_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_EA_CPWD_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_EA_CPWD_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_EA_CPWD_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_EA_CPWD_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_EA_CPWD_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_EA_CPWD_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 18;
        unsigned int COUNTER_MODE                                                 :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_EA_SE_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_EA_SE_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_EA_SE_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_EA_SE_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_EA_SE_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_EA_SE_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GC_EA_SE_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 18;
        unsigned int COUNTER_MODE                                                 :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE1_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE1_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE1_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL0                                                    : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE0                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE1_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE1_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE1_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE1_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL0                                                    : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE0                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE1_PERFCOUNTER1_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE1_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE1_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE1_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL0                                                    : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE0                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE1_PERFCOUNTER2_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE1_PERFCOUNTER3_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE1_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE1_PERFCOUNTER3_SELECT {
    struct {
        unsigned int PERF_SEL0                                                    : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE0                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE1_PERFCOUNTER3_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE2_DIST_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE2_DIST_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE2_DIST_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL0                                                    : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE0                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE2_DIST_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE2_DIST_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE2_DIST_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE2_DIST_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL0                                                    : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE0                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE2_DIST_PERFCOUNTER1_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE2_DIST_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE2_DIST_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE2_DIST_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL0                                                    : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE0                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE2_DIST_PERFCOUNTER2_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE2_DIST_PERFCOUNTER3_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE2_DIST_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE2_DIST_PERFCOUNTER3_SELECT {
    struct {
        unsigned int PERF_SEL0                                                    : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE0                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE2_DIST_PERFCOUNTER3_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE2_SE_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE2_SE_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE2_SE_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL0                                                    : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE0                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE2_SE_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE2_SE_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE2_SE_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE2_SE_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL0                                                    : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE0                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE2_SE_PERFCOUNTER1_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE2_SE_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE2_SE_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE2_SE_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL0                                                    : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE0                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE2_SE_PERFCOUNTER2_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE2_SE_PERFCOUNTER3_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE2_SE_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE2_SE_PERFCOUNTER3_SELECT {
    struct {
        unsigned int PERF_SEL0                                                    : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE0                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE2_SE_PERFCOUNTER3_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_CNTL {
    struct {
        unsigned int PRIMS_PER_SUBGRP                                             :  9;
        unsigned int VERTS_PER_SUBGRP                                             :  9;
        unsigned int BREAK_SUBGRP_AT_EOI                                          :  1;
        unsigned int PACKET_TO_ONE_PA                                             :  1;
        unsigned int BREAK_PRIMGRP_AT_EOI                                         :  1;
        unsigned int PRIM_GRP_SIZE                                                :  9;
        unsigned int GCR_DISABLE                                                  :  1;
        unsigned int DIS_PG_SIZE_ADJUST_FOR_STRIP                                 :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_GS_FAST_LAUNCH_WG_DIM {
    struct {
        unsigned int GS_FL_DIM_X                                                  : 16;
        unsigned int GS_FL_DIM_Y                                                  : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_GS_FAST_LAUNCH_WG_DIM_1 {
    struct {
        unsigned int GS_FL_DIM_Z                                                  : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_GS_ORDERED_ID_BASE {
    struct {
        unsigned int BASE                                                         : 12;
        unsigned int                                                              : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_GS_THROTTLE {
    struct {
        unsigned int T0                                                           :  3;
        unsigned int T1                                                           :  3;
        unsigned int T2                                                           :  3;
        unsigned int STALL_CYCLES                                                 :  7;
        unsigned int FACTOR1                                                      :  3;
        unsigned int FACTOR2                                                      :  3;
        unsigned int ENABLE_THROTTLE                                              :  1;
        unsigned int NUM_INIT_GRPS                                                :  8;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_IA_ENHANCE {
    struct {
        unsigned int MISC                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_INDX_OFFSET {
    struct {
        unsigned int INDX_OFFSET                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_MAX_OUTPUT_PER_SUBGROUP {
    struct {
        unsigned int MAX_VERTS_PER_SUBGROUP                                       : 10;
        unsigned int                                                              : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_MAX_VTX_INDX {
    struct {
        unsigned int MAX_INDX                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_MIN_VTX_INDX {
    struct {
        unsigned int MIN_INDX                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_MULTI_PRIM_IB_RESET_EN {
    struct {
        unsigned int RESET_EN                                                     :  1;
        unsigned int MATCH_ALL_BITS                                               :  1;
        unsigned int DISABLE_FOR_AUTO_INDEX                                       :  1;
        unsigned int                                                              : 29;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_NGG_SUBGRP_CNTL {
    struct {
        unsigned int PRIM_AMP_FACTOR                                              :  9;
        unsigned int THDS_PER_SUBGRP                                              :  9;
        unsigned int                                                              : 14;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_PA_IF_SAFE_REG {
    struct {
        unsigned int GE_PA_CSB                                                    : 10;
        unsigned int GE_PA_PAYLOAD                                                : 10;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_POS_RING_BASE {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_POS_RING_SIZE {
    struct {
        unsigned int MEM_SIZE                                                     : 14;
        unsigned int                                                              : 18;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_PRIM_RING_BASE {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_PRIM_RING_SIZE {
    struct {
        unsigned int MEM_SIZE                                                     : 11;
        unsigned int                                                              :  5;
        unsigned int SCOPE                                                        :  2;
        unsigned int PAF_TEMPORAL                                                 :  3;
        unsigned int PAB_TEMPORAL                                                 :  3;
        unsigned int SPEC_DATA_READ                                               :  2;
        unsigned int FORCE_SE_SCOPE                                               :  1;
        unsigned int PAB_NOFILL                                                   :  1;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_PRIV_CONTROL {
    struct {
        unsigned int RESERVED                                                     :  1;
        unsigned int CLAMP_PRIMGRP_SIZE                                           :  9;
        unsigned int                                                              :  5;
        unsigned int FGCG_OVERRIDE                                                :  1;
        unsigned int CLAMP_HS_OFFCHIP_PER_SE_OVERRIDE                             :  1;
        unsigned int                                                              :  1;
        unsigned int MIN_ATTR_GRPS                                                :  4;
        unsigned int                                                              : 10;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_RATE_CNTL_1 {
    struct {
        unsigned int ADD_X_CLKS_LS_VERT                                           :  4;
        unsigned int AFTER_Y_TRANS_LS_VERT                                        :  4;
        unsigned int ADD_X_CLKS_HS_VERT                                           :  4;
        unsigned int AFTER_Y_TRANS_HS_VERT                                        :  4;
        unsigned int ADD_X_CLKS_ES_VERT                                           :  4;
        unsigned int AFTER_Y_TRANS_ES_VERT                                        :  4;
        unsigned int ADD_X_CLKS_GS_PRIM                                           :  4;
        unsigned int AFTER_Y_TRANS_GS_PRIM                                        :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_RATE_CNTL_2 {
    struct {
        unsigned int                                                              : 16;
        unsigned int ADD_X_CLKS_MERGED_HS_GS                                      :  4;
        unsigned int ADD_X_CLKS_MERGED_LS_ES                                      :  4;
        unsigned int MERGED_HS_GS_MODE                                            :  1;
        unsigned int MERGED_LS_ES_MODE                                            :  1;
        unsigned int ENABLE_RATE_CNTL                                             :  1;
        unsigned int SWAP_PRIORITY                                                :  1;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_SE_CNTL_STATUS {
    struct {
        unsigned int TE_BUSY                                                      :  1;
        unsigned int NGG_BUSY                                                     :  1;
        unsigned int HS_BUSY                                                      :  1;
        unsigned int                                                              : 29;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_SE_ENHANCE {
    struct {
        unsigned int MISC                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_SPI_IF_SAFE_REG {
    struct {
        unsigned int GE_SPI_LS_ES_DATA                                            :  6;
        unsigned int GE_SPI_HS_GS_DATA                                            :  6;
        unsigned int GE_SPI_GRP                                                   :  6;
        unsigned int                                                              : 14;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_STATUS {
    struct {
        unsigned int PERFCOUNTER_STATUS                                           :  1;
        unsigned int THREAD_TRACE_STATUS                                          :  1;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_STEREO_CNTL {
    struct {
        unsigned int RT_SLICE                                                     :  3;
        unsigned int VIEWPORT                                                     :  4;
        unsigned int UNUSED                                                       :  1;
        unsigned int EN_STEREO                                                    :  1;
        unsigned int                                                              : 23;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_USER_VGPR1 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_USER_VGPR2 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_USER_VGPR3 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_USER_VGPR_EN {
    struct {
        unsigned int EN_USER_VGPR1                                                :  1;
        unsigned int EN_USER_VGPR2                                                :  1;
        unsigned int EN_USER_VGPR3                                                :  1;
        unsigned int                                                              : 29;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_VRS_RATE {
    struct {
        unsigned int RATE_X                                                       :  2;
        unsigned int                                                              :  2;
        unsigned int RATE_Y                                                       :  2;
        unsigned int                                                              : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_WD_CNTL_STATUS {
    struct {
        unsigned int DIST_BUSY                                                    :  1;
        unsigned int DIST_BE_BUSY                                                 :  1;
        unsigned int GE_UTCL1_BUSY                                                :  1;
        unsigned int WD_TE11_BUSY                                                 :  1;
        unsigned int PC_MANAGER_BUSY                                              :  1;
        unsigned int WLC_BUSY                                                     :  1;
        unsigned int                                                              : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GE_WD_ENHANCE {
    struct {
        unsigned int MISC                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1A_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1A_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1A_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1A_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1A_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1A_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1A_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1A_PERFCOUNTER1_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1A_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1A_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1A_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1A_PERFCOUNTER2_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1A_PERFCOUNTER3_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1A_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1A_PERFCOUNTER3_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1A_PERFCOUNTER3_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1C_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1C_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1C_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1C_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1C_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1C_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1C_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1C_PERFCOUNTER1_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1C_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1C_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1C_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1C_PERFCOUNTER2_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1C_PERFCOUNTER3_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1C_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1C_PERFCOUNTER3_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1C_PERFCOUNTER3_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1XA_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1XA_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1XA_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1XA_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1XA_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1XA_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1XA_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1XA_PERFCOUNTER1_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1XA_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1XA_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1XA_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1XA_PERFCOUNTER2_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1XA_PERFCOUNTER3_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1XA_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1XA_PERFCOUNTER3_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1XA_PERFCOUNTER3_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1XC_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1XC_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1XC_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1XC_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1XC_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1XC_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1XC_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1XC_PERFCOUNTER1_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1XC_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1XC_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1XC_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1XC_PERFCOUNTER2_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1XC_PERFCOUNTER3_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1XC_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1XC_PERFCOUNTER3_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL1XC_PERFCOUNTER3_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2A_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2A_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2A_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2A_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2A_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2A_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2A_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2A_PERFCOUNTER1_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2A_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2A_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2A_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2A_PERFCOUNTER2_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2A_PERFCOUNTER3_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2A_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2A_PERFCOUNTER3_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2A_PERFCOUNTER3_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2C_CTRL {
    struct {
        unsigned int CACHE_SIZE                                                   :  2;
        unsigned int RATE                                                         :  2;
        unsigned int WRITEBACK_MARGIN                                             :  4;
        unsigned int DCC_COMP_TRANSFER_SIZE_ENABLE                                :  3;
        unsigned int DCC_COMP_SKIP_LOW_COMP_RATIOS                                :  1;
        unsigned int SRC_FIFO_SIZE                                                :  4;
        unsigned int LATENCY_FIFO_SIZE                                            :  4;
        unsigned int                                                              :  1;
        unsigned int LINEAR_SET_HASH                                              :  1;
        unsigned int FORCE_HIT_QUEUE_POP                                          :  2;
        unsigned int                                                              :  3;
        unsigned int IGNORE_FULLY_WRITTEN                                         :  1;
        unsigned int SINGLE_GL2                                                   :  1;
        unsigned int UNSUPPORTED_DF_ATOMIC_OPS                                    :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2C_CTRL2 {
    struct {
        unsigned int FILL_SIZE_32                                                 :  2;
        unsigned int FILL_SIZE_64                                                 :  2;
        unsigned int ADDR_MATCH_DISABLE                                           :  1;
        unsigned int FILL_SIZE_128                                                :  1;
        unsigned int SC_TO_HI_PRIORITY                                            :  1;
        unsigned int HIT_UNDER_MISS_DISABLE                                       :  1;
        unsigned int RO_DISABLE                                                   :  1;
        unsigned int UNCOMP_RET_LATENCY_MODE                                      :  1;
        unsigned int GCR_ARB_CTRL                                                 :  3;
        unsigned int GCR_ALL_SET                                                  :  1;
        unsigned int DCC_COMPRESSION_DISABLE                                      :  1;
        unsigned int DF_COMPRESSION_MODE_OVERRIDE                                 :  2;
        unsigned int COMPRESSED_WRITE_SAFE_MODE                                   :  1;
        unsigned int USE_EA_EARLYWRRET_ON_WRITEBACK                               :  1;
        unsigned int WRITEBACK_ALL_WAIT_FOR_ALL_EA_WRITE_COMPLETE                 :  1;
        unsigned int MAX_MIN_CTRL                                                 :  2;
        unsigned int DECOMPRESS_WRITE_COMPRESSION_DISABLE                         :  1;
        unsigned int FORCE_WRITE_DECOMPRESSION                                    :  1;
        unsigned int DCC_CLEAR_ERRORS                                             :  1;
        unsigned int                                                              :  1;
        unsigned int DCC_COMP_TO_SINGLE_EN                                        :  1;
        unsigned int DCC_COMP_TO_CONSTANT_EN                                      :  1;
        unsigned int DISABLE_HI_PRIORITY                                          :  1;
        unsigned int OC_OVERRIDE_UNCOMP_LOGICAL_SIZE_DISABLE                      :  1;
        unsigned int DISABLE_CACHE_RAM_READ_FILTER                                :  1;
        unsigned int DISABLE_TAG_RAM_READ_FILTER                                  :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2C_CTRL3 {
    struct {
        unsigned int COMP_STREAM_OVERRIDE                                         :  1;
        unsigned int LAST_USE_MODE                                                :  1;
        unsigned int FORCE_UNCOMP_READ                                            :  2;
        unsigned int LAST_USE_SAFE_MODE                                           :  1;
        unsigned int BANK_LINEAR_HASH_MODE                                        :  1;
        unsigned int ENABLE_64B_LAST_USE                                          :  1;
        unsigned int UNCACHED_TO_UC_QUEUE                                         :  1;
        unsigned int IO_CHANNEL_ENABLE                                            :  1;
        unsigned int                                                              :  2;
        unsigned int BANK_LINEAR_HASH_ENABLE                                      :  1;
        unsigned int HASH_256B_ENABLE                                             :  1;
        unsigned int DECOMP_NBC_IND64_DISABLE                                     :  1;
        unsigned int FORCE_READ_ON_WRITE_OP                                       :  1;
        unsigned int FGCG_OVERRIDE                                                :  1;
        unsigned int FORCE_MTYPE_UC                                               :  1;
        unsigned int                                                              :  1;
        unsigned int WRITE_SET_SECTOR_FULLY_WRITTEN                               :  1;
        unsigned int EA_READ_SIZE_LIMIT                                           :  1;
        unsigned int EA_WRITE_SIZE_LIMIT                                          :  1;
        unsigned int WB_OPT_ENABLE                                                :  1;
        unsigned int WB_OPT_BURST_MAX_COUNT                                       :  2;
        unsigned int SET_GROUP_LINEAR_HASH_ENABLE                                 :  1;
        unsigned int EA_GMI_DISABLE                                               :  1;
        unsigned int COMP_STREAM_OVERRIDE_BYPASS                                  :  1;
        unsigned int INF_NAN_CLAMP                                                :  1;
        unsigned int SCRATCH                                                      :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2C_CTRL4 {
    struct {
        unsigned int                                                              :  1;
        unsigned int SPA_CHANNEL_ENABLE                                           :  1;
        unsigned int                                                              :  1;
        unsigned int WRITEBACK_FIFO_STALL_ENABLE                                  :  1;
        unsigned int OC_UNCOMP_128B_COMPRESS_EN_DISABLE                           :  1;
        unsigned int                                                              :  1;
        unsigned int TAG_MGCG_MODE                                                :  1;
        unsigned int CORE_MGCG_MODE                                               :  1;
        unsigned int EXECUTE_MGCG_MODE                                            :  1;
        unsigned int EA_NACK_DISABLE                                              :  1;
        unsigned int FED_SAFE_MODE                                                :  1;
        unsigned int FLUSH_SET_COUNTER_MASK_DISABLE                               :  1;
        unsigned int LFIFO_VMISS_DISABLE                                          :  1;
        unsigned int EA_COMPRESSED_NACK_DISABLE                                   :  1;
        unsigned int DCC_FORCE_BYPASS                                             :  1;
        unsigned int DCC_CLEAR_128B_CONSTANT_ENCODE_EN                            :  1;
        unsigned int LFIFO_HASH_MODE                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2C_CTRL5 {
    struct {
        unsigned int CB_SUPPORTED_COMP_SCHEME                                     :  3;
        unsigned int                                                              :  1;
        unsigned int DB_SUPPORTED_COMP_SCHEME                                     :  3;
        unsigned int                                                              :  1;
        unsigned int CONCAT_GT_128B_DISABLE                                       :  1;
        unsigned int FORCE_UNCOMP_READ_ON_UNCOMPRESSED_CL                         :  1;
        unsigned int VGT_GS_MAX_WAVE_ID_WIDTH                                     :  5;
        unsigned int CID_REMAP_ENABLE                                             :  1;
        unsigned int PERF_CNTR_EN_OVERRIDE                                        :  1;
        unsigned int COMP_BYPASS_READ_MODE                                        :  1;
        unsigned int UNCACHED_COMP_WRITE_PASSTHROUGH_EN                           :  1;
        unsigned int LAST_USE_SAFE_MODE_COMP                                      :  1;
        unsigned int UNCOMP_KEY_COMP_WRITE_EN                                     :  1;
        unsigned int                                                              : 11;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2C_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2C_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2C_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2C_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2C_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2C_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2C_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2C_PERFCOUNTER1_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2C_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2C_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2C_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2C_PERFCOUNTER2_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2C_PERFCOUNTER3_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2C_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2C_PERFCOUNTER3_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GL2C_PERFCOUNTER3_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE2                                                   :  4;
        unsigned int PERF_MODE3                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBMH_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBMH_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBMH_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  6;
        unsigned int GL1CC_BUSY_USER_DEFINED_MASK                                 :  1;
        unsigned int GL1XCC_BUSY_USER_DEFINED_MASK                                :  1;
        unsigned int SQG_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int SC_CLEAN_USER_DEFINED_MASK                                   :  1;
        unsigned int WGS_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int DB_CLEAN_USER_DEFINED_MASK                                   :  1;
        unsigned int CB_CLEAN_USER_DEFINED_MASK                                   :  1;
        unsigned int TA_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int SX_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int GL2C_BUSY_USER_DEFINED_MASK                                  :  1;
        unsigned int SPI_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int SC_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int PA_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int GL2A_BUSY_USER_DEFINED_MASK                                  :  1;
        unsigned int DB_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int CB_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int PC_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int EA_LINK_BUSY_USER_DEFINED_MASK                               :  1;
        unsigned int GL1A_BUSY_USER_DEFINED_MASK                                  :  1;
        unsigned int BCI_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int RLC_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int TCP_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int GE_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int UTCL1_BUSY_USER_DEFINED_MASK                                 :  1;
        unsigned int EA_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBMH_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBMH_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBMH_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  6;
        unsigned int GL1CC_BUSY_USER_DEFINED_MASK                                 :  1;
        unsigned int GL1XCC_BUSY_USER_DEFINED_MASK                                :  1;
        unsigned int SQG_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int SC_CLEAN_USER_DEFINED_MASK                                   :  1;
        unsigned int WGS_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int DB_CLEAN_USER_DEFINED_MASK                                   :  1;
        unsigned int CB_CLEAN_USER_DEFINED_MASK                                   :  1;
        unsigned int TA_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int SX_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int GL2C_BUSY_USER_DEFINED_MASK                                  :  1;
        unsigned int SPI_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int SC_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int PA_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int GL2A_BUSY_USER_DEFINED_MASK                                  :  1;
        unsigned int DB_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int CB_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int PC_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int EA_LINK_BUSY_USER_DEFINED_MASK                               :  1;
        unsigned int GL1A_BUSY_USER_DEFINED_MASK                                  :  1;
        unsigned int BCI_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int RLC_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int TCP_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int GE_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int UTCL1_BUSY_USER_DEFINED_MASK                                 :  1;
        unsigned int EA_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBMH_SYNC {
    struct {
        unsigned int GFX_PIPE0_PERFMON_SYNC                                       :  1;
        unsigned int                                                              : 15;
        unsigned int GFX_PIPE0_SYNC                                               :  1;
        unsigned int                                                              : 14;
        unsigned int GFX_SYNC_SET_CLR                                             :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_GFX_INDEX {
    struct {
        unsigned int INSTANCE_INDEX                                               :  7;
        unsigned int                                                              :  1;
        unsigned int SA_INDEX                                                     :  2;
        unsigned int                                                              :  6;
        unsigned int SE_INDEX                                                     :  4;
        unsigned int                                                              :  9;
        unsigned int SA_BROADCAST_WRITES                                          :  1;
        unsigned int INSTANCE_BROADCAST_WRITES                                    :  1;
        unsigned int SE_BROADCAST_WRITES                                          :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_GFX_INDEX_SR_DATA {
    struct {
        unsigned int INSTANCE_INDEX                                               :  7;
        unsigned int                                                              :  1;
        unsigned int SA_INDEX                                                     :  2;
        unsigned int                                                              :  6;
        unsigned int SE_INDEX                                                     :  4;
        unsigned int                                                              :  9;
        unsigned int SA_BROADCAST_WRITES                                          :  1;
        unsigned int INSTANCE_BROADCAST_WRITES                                    :  1;
        unsigned int SE_BROADCAST_WRITES                                          :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_GFX_INDEX_SR_SELECT {
    struct {
        unsigned int INDEX                                                        :  3;
        unsigned int                                                              : 28;
        unsigned int VF_PF                                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_NOWHERE {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_NOWHERE_2 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  6;
        unsigned int                                                              :  3;
        unsigned int SC_CLEAN_USER_DEFINED_MASK                                   :  1;
        unsigned int DB_CLEAN_USER_DEFINED_MASK                                   :  1;
        unsigned int CB_CLEAN_USER_DEFINED_MASK                                   :  1;
        unsigned int                                                              :  1;
        unsigned int TA_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int SX_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int                                                              :  1;
        unsigned int SPI_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int SC_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int PA_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int GRBM_BUSY_USER_DEFINED_MASK                                  :  1;
        unsigned int DB_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int CB_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int CP_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int                                                              :  2;
        unsigned int BCI_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int RLC_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int TCP_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int GE_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int UTCL2_BUSY_USER_DEFINED_MASK                                 :  1;
        unsigned int EA_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_PERFCOUNTER0_SELECT_HI {
    struct {
        unsigned int                                                              :  1;
        unsigned int UTCL1_BUSY_USER_DEFINED_MASK                                 :  1;
        unsigned int GL2CC_BUSY_USER_DEFINED_MASK                                 :  1;
        unsigned int SDMA_BUSY_USER_DEFINED_MASK                                  :  1;
        unsigned int CH_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int                                                              :  1;
        unsigned int PMM_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int                                                              :  1;
        unsigned int GL1CC_BUSY_USER_DEFINED_MASK                                 :  1;
        unsigned int GL1XCC_BUSY_USER_DEFINED_MASK                                :  1;
        unsigned int PC_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int                                                              :  1;
        unsigned int WGS_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int EA_STAT_LINK_BUSY_USER_DEFINED_MASK                          :  1;
        unsigned int                                                              : 18;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  6;
        unsigned int                                                              :  3;
        unsigned int SC_CLEAN_USER_DEFINED_MASK                                   :  1;
        unsigned int DB_CLEAN_USER_DEFINED_MASK                                   :  1;
        unsigned int CB_CLEAN_USER_DEFINED_MASK                                   :  1;
        unsigned int                                                              :  1;
        unsigned int TA_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int SX_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int                                                              :  1;
        unsigned int SPI_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int SC_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int PA_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int GRBM_BUSY_USER_DEFINED_MASK                                  :  1;
        unsigned int DB_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int CB_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int CP_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int                                                              :  2;
        unsigned int BCI_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int RLC_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int TCP_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int GE_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int UTCL2_BUSY_USER_DEFINED_MASK                                 :  1;
        unsigned int EA_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union GRBM_PERFCOUNTER1_SELECT_HI {
    struct {
        unsigned int                                                              :  1;
        unsigned int UTCL1_BUSY_USER_DEFINED_MASK                                 :  1;
        unsigned int GL2CC_BUSY_USER_DEFINED_MASK                                 :  1;
        unsigned int SDMA_BUSY_USER_DEFINED_MASK                                  :  1;
        unsigned int CH_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int                                                              :  1;
        unsigned int PMM_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int                                                              :  1;
        unsigned int GL1CC_BUSY_USER_DEFINED_MASK                                 :  1;
        unsigned int GL1XCC_BUSY_USER_DEFINED_MASK                                :  1;
        unsigned int PC_BUSY_USER_DEFINED_MASK                                    :  1;
        unsigned int                                                              :  1;
        unsigned int WGS_BUSY_USER_DEFINED_MASK                                   :  1;
        unsigned int EA_STAT_LINK_BUSY_USER_DEFINED_MASK                          :  1;
        unsigned int                                                              : 18;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union IA_UTCL1_CNTL {
    struct {
        unsigned int XNACK_REDO_TIMER_CNT                                         : 20;
        unsigned int                                                              :  3;
        unsigned int VMID_RESET_MODE                                              :  1;
        unsigned int DROP_MODE                                                    :  1;
        unsigned int BYPASS                                                       :  1;
        unsigned int INVALIDATE                                                   :  1;
        unsigned int FRAG_LIMIT_MODE                                              :  1;
        unsigned int FORCE_SNOOP                                                  :  1;
        unsigned int MTYPE_OVERRIDE                                               :  1;
        unsigned int LLC_NOALLOC_OVERRIDE                                         :  1;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union IA_UTCL1_STATUS {
    struct {
        unsigned int FAULT_DETECTED                                               :  1;
        unsigned int RETRY_DETECTED                                               :  1;
        unsigned int PRT_DETECTED                                                 :  1;
        unsigned int                                                              :  1;
        unsigned int FAULT_VMID                                                   :  4;
        unsigned int FAULT_UTCL1ID                                                :  6;
        unsigned int FAULT_INSTANCEID                                             :  2;
        unsigned int RETRY_UTCL1ID                                                :  6;
        unsigned int RETRY_INSTANCEID                                             :  2;
        unsigned int PRT_UTCL1ID                                                  :  6;
        unsigned int PRT_INSTANCEID                                               :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union IA_UTCL1_STATUS_2 {
    struct {
        unsigned int FAULT_DETECTED                                               :  1;
        unsigned int RETRY_DETECTED                                               :  1;
        unsigned int PRT_DETECTED                                                 :  1;
        unsigned int                                                              :  1;
        unsigned int FAULT_VMID                                                   :  4;
        unsigned int FAULT_UTCL1ID                                                :  6;
        unsigned int FAULT_INSTANCEID                                             :  2;
        unsigned int RETRY_UTCL1ID                                                :  6;
        unsigned int RETRY_INSTANCEID                                             :  2;
        unsigned int PRT_UTCL1ID                                                  :  6;
        unsigned int PRT_INSTANCEID                                               :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union MP1_SMN_FPS_CNT {
    struct {
        unsigned int COUNT                                                        : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_CLIP_CNTL {
    struct {
        unsigned int UCP_ENA_0                                                    :  1;
        unsigned int UCP_ENA_1                                                    :  1;
        unsigned int UCP_ENA_2                                                    :  1;
        unsigned int UCP_ENA_3                                                    :  1;
        unsigned int UCP_ENA_4                                                    :  1;
        unsigned int UCP_ENA_5                                                    :  1;
        unsigned int                                                              :  7;
        unsigned int PS_UCP_Y_SCALE_NEG                                           :  1;
        unsigned int PS_UCP_MODE                                                  :  2;
        unsigned int CLIP_DISABLE                                                 :  1;
        unsigned int UCP_CULL_ONLY_ENA                                            :  1;
        unsigned int BOUNDARY_EDGE_FLAG_ENA                                       :  1;
        unsigned int DX_CLIP_SPACE_DEF                                            :  1;
        unsigned int DIS_CLIP_ERR_DETECT                                          :  1;
        unsigned int VTX_KILL_OR                                                  :  1;
        unsigned int DX_RASTERIZATION_KILL                                        :  1;
        unsigned int                                                              :  1;
        unsigned int DX_LINEAR_ATTR_CLIP_ENA                                      :  1;
        unsigned int VTE_VPORT_PROVOKE_DISABLE                                    :  1;
        unsigned int ZCLIP_NEAR_DISABLE                                           :  1;
        unsigned int ZCLIP_FAR_DISABLE                                            :  1;
        unsigned int ZCLIP_PROG_NEAR_ENA                                          :  1;
        unsigned int                                                              :  3;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_CNTL_STATUS {
    struct {
        unsigned int                                                              : 31;
        unsigned int CL_BUSY                                                      :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_ENHANCE {
    struct {
        unsigned int CLIP_VTX_REORDER_ENA                                         :  1;
        unsigned int NUM_CLIP_SEQ                                                 :  2;
        unsigned int CLIPPED_PRIM_SEQ_STALL                                       :  1;
        unsigned int VE_NAN_PROC_DISABLE                                          :  1;
        unsigned int XTRA_DEBUG_REG_SEL                                           :  1;
        unsigned int IGNORE_PIPELINE_RESET                                        :  1;
        unsigned int KILL_INNER_EDGE_FLAGS                                        :  1;
        unsigned int NGG_PA_TO_ALL_SC                                             :  1;
        unsigned int TC_LATENCY_TIME_STAMP_RESOLUTION                             :  2;
        unsigned int NGG_BYPASS_PRIM_FILTER                                       :  1;
        unsigned int NGG_SIDEBAND_MEMORY_DEPTH                                    :  2;
        unsigned int NGG_PRIM_INDICES_FIFO_DEPTH                                  :  3;
        unsigned int PROG_NEAR_CLIP_PLANE_ENABLE                                  :  1;
        unsigned int POLY_INNER_EDGE_FLAG_DISABLE                                 :  1;
        unsigned int TC_REQUEST_PERF_CNTR_ENABLE                                  :  1;
        unsigned int DISABLE_PA_PH_INTF_FINE_CLOCK_GATE                           :  1;
        unsigned int DISABLE_PA_SX_REQ_INTF_FINE_CLOCK_GATE                       :  1;
        unsigned int ENABLE_PA_RATE_CNTL                                          :  1;
        unsigned int CLAMP_NEGATIVE_BB_TO_ZERO                                    :  1;
        unsigned int PA_W_GL1X_SRC_CLK_OVERRIDE                                   :  1;
        unsigned int PA_W_GL1X_REQ_CLK_OVERRIDE                                   :  1;
        unsigned int PA_R_GL1X_REQ_CLK_OVERRIDE                                   :  1;
        unsigned int PAF_GEWD_CSB_CLK_OVERRIDE                                    :  1;
        unsigned int BROADCAST_PMODE_PRIMS                                        :  1;
        unsigned int BROADCAST_PERP_ENDCAP_PRIMS                                  :  1;
        unsigned int ECO_SPARE1                                                   :  1;
        unsigned int ECO_SPARE0                                                   :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_GB_HORZ_CLIP_ADJ {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_GB_HORZ_DISC_ADJ {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_GB_VERT_CLIP_ADJ {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_GB_VERT_DISC_ADJ {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_NANINF_CNTL {
    struct {
        unsigned int VTE_XY_INF_DISCARD                                           :  1;
        unsigned int VTE_Z_INF_DISCARD                                            :  1;
        unsigned int VTE_W_INF_DISCARD                                            :  1;
        unsigned int VTE_0XNANINF_IS_0                                            :  1;
        unsigned int VTE_XY_NAN_RETAIN                                            :  1;
        unsigned int VTE_Z_NAN_RETAIN                                             :  1;
        unsigned int VTE_W_NAN_RETAIN                                             :  1;
        unsigned int VTE_W_RECIP_NAN_IS_0                                         :  1;
        unsigned int VS_XY_NAN_TO_INF                                             :  1;
        unsigned int VS_XY_INF_RETAIN                                             :  1;
        unsigned int VS_Z_NAN_TO_INF                                              :  1;
        unsigned int VS_Z_INF_RETAIN                                              :  1;
        unsigned int VS_W_NAN_TO_INF                                              :  1;
        unsigned int VS_W_INF_RETAIN                                              :  1;
        unsigned int VS_CLIP_DIST_INF_DISCARD                                     :  1;
        unsigned int                                                              :  5;
        unsigned int VTE_NO_OUTPUT_NEG_0                                          :  1;
        unsigned int                                                              : 11;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_NGG_CNTL {
    struct {
        unsigned int VERTEX_REUSE_OFF                                             :  1;
        unsigned int INDEX_BUF_EDGE_FLAG_ENA                                      :  1;
        unsigned int VERTEX_REUSE_DEPTH                                           :  8;
        unsigned int                                                              : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_POINT_CULL_RAD {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_POINT_SIZE {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_POINT_X_RAD {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_POINT_Y_RAD {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_PROG_NEAR_CLIP_Z {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_0_W {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_0_X {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_0_Y {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_0_Z {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_1_W {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_1_X {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_1_Y {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_1_Z {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_2_W {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_2_X {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_2_Y {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_2_Z {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_3_W {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_3_X {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_3_Y {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_3_Z {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_4_W {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_4_X {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_4_Y {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_4_Z {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_5_W {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_5_X {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_5_Y {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_UCP_5_Z {
    struct {
        unsigned int DATA_REGISTER                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XOFFSET {
    struct {
        unsigned int VPORT_XOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XOFFSET_1 {
    struct {
        unsigned int VPORT_XOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XOFFSET_2 {
    struct {
        unsigned int VPORT_XOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XOFFSET_3 {
    struct {
        unsigned int VPORT_XOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XOFFSET_4 {
    struct {
        unsigned int VPORT_XOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XOFFSET_5 {
    struct {
        unsigned int VPORT_XOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XOFFSET_6 {
    struct {
        unsigned int VPORT_XOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XOFFSET_7 {
    struct {
        unsigned int VPORT_XOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XOFFSET_8 {
    struct {
        unsigned int VPORT_XOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XOFFSET_9 {
    struct {
        unsigned int VPORT_XOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XOFFSET_10 {
    struct {
        unsigned int VPORT_XOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XOFFSET_11 {
    struct {
        unsigned int VPORT_XOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XOFFSET_12 {
    struct {
        unsigned int VPORT_XOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XOFFSET_13 {
    struct {
        unsigned int VPORT_XOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XOFFSET_14 {
    struct {
        unsigned int VPORT_XOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XOFFSET_15 {
    struct {
        unsigned int VPORT_XOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XSCALE {
    struct {
        unsigned int VPORT_XSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XSCALE_1 {
    struct {
        unsigned int VPORT_XSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XSCALE_2 {
    struct {
        unsigned int VPORT_XSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XSCALE_3 {
    struct {
        unsigned int VPORT_XSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XSCALE_4 {
    struct {
        unsigned int VPORT_XSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XSCALE_5 {
    struct {
        unsigned int VPORT_XSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XSCALE_6 {
    struct {
        unsigned int VPORT_XSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XSCALE_7 {
    struct {
        unsigned int VPORT_XSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XSCALE_8 {
    struct {
        unsigned int VPORT_XSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XSCALE_9 {
    struct {
        unsigned int VPORT_XSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XSCALE_10 {
    struct {
        unsigned int VPORT_XSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XSCALE_11 {
    struct {
        unsigned int VPORT_XSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XSCALE_12 {
    struct {
        unsigned int VPORT_XSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XSCALE_13 {
    struct {
        unsigned int VPORT_XSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XSCALE_14 {
    struct {
        unsigned int VPORT_XSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_XSCALE_15 {
    struct {
        unsigned int VPORT_XSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YOFFSET {
    struct {
        unsigned int VPORT_YOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YOFFSET_1 {
    struct {
        unsigned int VPORT_YOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YOFFSET_2 {
    struct {
        unsigned int VPORT_YOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YOFFSET_3 {
    struct {
        unsigned int VPORT_YOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YOFFSET_4 {
    struct {
        unsigned int VPORT_YOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YOFFSET_5 {
    struct {
        unsigned int VPORT_YOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YOFFSET_6 {
    struct {
        unsigned int VPORT_YOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YOFFSET_7 {
    struct {
        unsigned int VPORT_YOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YOFFSET_8 {
    struct {
        unsigned int VPORT_YOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YOFFSET_9 {
    struct {
        unsigned int VPORT_YOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YOFFSET_10 {
    struct {
        unsigned int VPORT_YOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YOFFSET_11 {
    struct {
        unsigned int VPORT_YOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YOFFSET_12 {
    struct {
        unsigned int VPORT_YOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YOFFSET_13 {
    struct {
        unsigned int VPORT_YOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YOFFSET_14 {
    struct {
        unsigned int VPORT_YOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YOFFSET_15 {
    struct {
        unsigned int VPORT_YOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YSCALE {
    struct {
        unsigned int VPORT_YSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YSCALE_1 {
    struct {
        unsigned int VPORT_YSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YSCALE_2 {
    struct {
        unsigned int VPORT_YSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YSCALE_3 {
    struct {
        unsigned int VPORT_YSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YSCALE_4 {
    struct {
        unsigned int VPORT_YSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YSCALE_5 {
    struct {
        unsigned int VPORT_YSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YSCALE_6 {
    struct {
        unsigned int VPORT_YSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YSCALE_7 {
    struct {
        unsigned int VPORT_YSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YSCALE_8 {
    struct {
        unsigned int VPORT_YSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YSCALE_9 {
    struct {
        unsigned int VPORT_YSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YSCALE_10 {
    struct {
        unsigned int VPORT_YSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YSCALE_11 {
    struct {
        unsigned int VPORT_YSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YSCALE_12 {
    struct {
        unsigned int VPORT_YSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YSCALE_13 {
    struct {
        unsigned int VPORT_YSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YSCALE_14 {
    struct {
        unsigned int VPORT_YSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_YSCALE_15 {
    struct {
        unsigned int VPORT_YSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZOFFSET {
    struct {
        unsigned int VPORT_ZOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZOFFSET_1 {
    struct {
        unsigned int VPORT_ZOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZOFFSET_2 {
    struct {
        unsigned int VPORT_ZOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZOFFSET_3 {
    struct {
        unsigned int VPORT_ZOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZOFFSET_4 {
    struct {
        unsigned int VPORT_ZOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZOFFSET_5 {
    struct {
        unsigned int VPORT_ZOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZOFFSET_6 {
    struct {
        unsigned int VPORT_ZOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZOFFSET_7 {
    struct {
        unsigned int VPORT_ZOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZOFFSET_8 {
    struct {
        unsigned int VPORT_ZOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZOFFSET_9 {
    struct {
        unsigned int VPORT_ZOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZOFFSET_10 {
    struct {
        unsigned int VPORT_ZOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZOFFSET_11 {
    struct {
        unsigned int VPORT_ZOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZOFFSET_12 {
    struct {
        unsigned int VPORT_ZOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZOFFSET_13 {
    struct {
        unsigned int VPORT_ZOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZOFFSET_14 {
    struct {
        unsigned int VPORT_ZOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZOFFSET_15 {
    struct {
        unsigned int VPORT_ZOFFSET                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZSCALE {
    struct {
        unsigned int VPORT_ZSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZSCALE_1 {
    struct {
        unsigned int VPORT_ZSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZSCALE_2 {
    struct {
        unsigned int VPORT_ZSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZSCALE_3 {
    struct {
        unsigned int VPORT_ZSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZSCALE_4 {
    struct {
        unsigned int VPORT_ZSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZSCALE_5 {
    struct {
        unsigned int VPORT_ZSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZSCALE_6 {
    struct {
        unsigned int VPORT_ZSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZSCALE_7 {
    struct {
        unsigned int VPORT_ZSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZSCALE_8 {
    struct {
        unsigned int VPORT_ZSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZSCALE_9 {
    struct {
        unsigned int VPORT_ZSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZSCALE_10 {
    struct {
        unsigned int VPORT_ZSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZSCALE_11 {
    struct {
        unsigned int VPORT_ZSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZSCALE_12 {
    struct {
        unsigned int VPORT_ZSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZSCALE_13 {
    struct {
        unsigned int VPORT_ZSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZSCALE_14 {
    struct {
        unsigned int VPORT_ZSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VPORT_ZSCALE_15 {
    struct {
        unsigned int VPORT_ZSCALE                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VRS_CNTL {
    struct {
        unsigned int VERTEX_RATE_COMBINER_MODE                                    :  3;
        unsigned int PRIMITIVE_RATE_COMBINER_MODE                                 :  3;
        unsigned int HTILE_RATE_COMBINER_MODE                                     :  3;
        unsigned int SAMPLE_ITER_COMBINER_MODE                                    :  3;
        unsigned int                                                              :  1;
        unsigned int EXPOSE_VRS_PIXELS_MASK                                       :  1;
        unsigned int CMASK_RATE_HINT_FORCE_ZERO                                   :  1;
        unsigned int SAMPLE_COVERAGE_ENCODING                                     :  1;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VS_OUT_CNTL {
    struct {
        unsigned int CLIP_DIST_ENA_0                                              :  1;
        unsigned int CLIP_DIST_ENA_1                                              :  1;
        unsigned int CLIP_DIST_ENA_2                                              :  1;
        unsigned int CLIP_DIST_ENA_3                                              :  1;
        unsigned int CLIP_DIST_ENA_4                                              :  1;
        unsigned int CLIP_DIST_ENA_5                                              :  1;
        unsigned int CLIP_DIST_ENA_6                                              :  1;
        unsigned int CLIP_DIST_ENA_7                                              :  1;
        unsigned int CULL_DIST_ENA_0                                              :  1;
        unsigned int CULL_DIST_ENA_1                                              :  1;
        unsigned int CULL_DIST_ENA_2                                              :  1;
        unsigned int CULL_DIST_ENA_3                                              :  1;
        unsigned int CULL_DIST_ENA_4                                              :  1;
        unsigned int CULL_DIST_ENA_5                                              :  1;
        unsigned int CULL_DIST_ENA_6                                              :  1;
        unsigned int CULL_DIST_ENA_7                                              :  1;
        unsigned int USE_VTX_POINT_SIZE                                           :  1;
        unsigned int USE_VTX_EDGE_FLAG                                            :  1;
        unsigned int USE_VTX_RENDER_TARGET_INDX                                   :  1;
        unsigned int USE_VTX_VIEWPORT_INDX                                        :  1;
        unsigned int USE_VTX_KILL_FLAG                                            :  1;
        unsigned int VS_OUT_MISC_VEC_ENA                                          :  1;
        unsigned int VS_OUT_CCDIST0_VEC_ENA                                       :  1;
        unsigned int VS_OUT_CCDIST1_VEC_ENA                                       :  1;
        unsigned int VS_OUT_MISC_SIDE_BUS_ENA                                     :  1;
        unsigned int                                                              :  2;
        unsigned int USE_VTX_LINE_WIDTH                                           :  1;
        unsigned int USE_VTX_VRS_RATE                                             :  1;
        unsigned int BYPASS_VTX_RATE_COMBINER                                     :  1;
        unsigned int BYPASS_PRIM_RATE_COMBINER                                    :  1;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_CL_VTE_CNTL {
    struct {
        unsigned int VPORT_X_SCALE_ENA                                            :  1;
        unsigned int VPORT_X_OFFSET_ENA                                           :  1;
        unsigned int VPORT_Y_SCALE_ENA                                            :  1;
        unsigned int VPORT_Y_OFFSET_ENA                                           :  1;
        unsigned int VPORT_Z_SCALE_ENA                                            :  1;
        unsigned int VPORT_Z_OFFSET_ENA                                           :  1;
        unsigned int                                                              :  2;
        unsigned int VTX_XY_FMT                                                   :  1;
        unsigned int VTX_Z_FMT                                                    :  1;
        unsigned int VTX_W0_FMT                                                   :  1;
        unsigned int PERFCOUNTER_REF                                              :  1;
        unsigned int                                                              : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_ENHANCE {
    struct {
        unsigned int ECO_SPARE0                                                   :  1;
        unsigned int ECO_SPARE1                                                   :  1;
        unsigned int ECO_SPARE2                                                   :  1;
        unsigned int ECO_SPARE3                                                   :  1;
        unsigned int DISABLE_PH_SC_INTF_FINE_CLOCK_GATE                           :  1;
        unsigned int DISABLE_FOPKT                                                :  1;
        unsigned int DISABLE_FOPKT_SCAN_POST_RESET                                :  1;
        unsigned int DISABLE_PH_SC_INTF_CLKEN_CLOCK_GATE                          :  1;
        unsigned int                                                              :  1;
        unsigned int DISABLE_PH_PERF_REG_FGCG                                     :  1;
        unsigned int ENABLE_PH_INTF_CLKEN_STRETCH                                 :  3;
        unsigned int DISABLE_USE_LAST_PH_ARBITER_PERFCOUNTER_SAMPLE_EVENT         :  1;
        unsigned int USE_PERFCOUNTER_START_STOP_EVENTS                            :  1;
        unsigned int FORCE_PH_PERFCOUNTER_SAMPLE_ENABLE_ON                        :  1;
        unsigned int PH_SPI_GE_THROTTLE_MODE                                      :  1;
        unsigned int PH_SPI_GE_THROTTLE_MODE_DISABLE                              :  1;
        unsigned int PH_SPI_GE_THROTTLE_PERFCOUNTER_COUNT_MODE                    :  1;
        unsigned int                                                              : 13;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_INTERFACE_FIFO_SIZE {
    struct {
        unsigned int PA_PH_IF_FIFO_SIZE                                           : 10;
        unsigned int                                                              :  6;
        unsigned int PH_SC_IF_FIFO_SIZE                                           :  6;
        unsigned int                                                              : 10;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER1_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER2_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER3_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER3_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER3_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER4_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER4_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER4_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER5_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER5_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER5_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER6_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER6_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER6_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER7_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER7_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_PH_PERFCOUNTER7_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_RATE_CNTL {
    struct {
        unsigned int VERTEX_RATE                                                  :  4;
        unsigned int PRIM_RATE                                                    :  4;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_AA_CONFIG {
    struct {
        unsigned int MSAA_NUM_SAMPLES                                             :  3;
        unsigned int                                                              :  1;
        unsigned int AA_MASK_CENTROID_DTMN                                        :  1;
        unsigned int                                                              : 15;
        unsigned int MSAA_EXPOSED_SAMPLES                                         :  3;
        unsigned int                                                              :  1;
        unsigned int DETAIL_TO_EXPOSED_MODE                                       :  2;
        unsigned int                                                              :  4;
        unsigned int PS_ITER_SAMPLES                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_AA_MASK_X0Y0_X1Y0 {
    struct {
        unsigned int AA_MASK_X0Y0                                                 : 16;
        unsigned int AA_MASK_X1Y0                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_AA_MASK_X0Y1_X1Y1 {
    struct {
        unsigned int AA_MASK_X0Y1                                                 : 16;
        unsigned int AA_MASK_X1Y1                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_AA_SAMPLE_LOCS_PIXEL_X0Y0_0 {
    struct {
        unsigned int S0_X                                                         :  4;
        unsigned int S0_Y                                                         :  4;
        unsigned int S1_X                                                         :  4;
        unsigned int S1_Y                                                         :  4;
        unsigned int S2_X                                                         :  4;
        unsigned int S2_Y                                                         :  4;
        unsigned int S3_X                                                         :  4;
        unsigned int S3_Y                                                         :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_AA_SAMPLE_LOCS_PIXEL_X0Y0_1 {
    struct {
        unsigned int S4_X                                                         :  4;
        unsigned int S4_Y                                                         :  4;
        unsigned int S5_X                                                         :  4;
        unsigned int S5_Y                                                         :  4;
        unsigned int S6_X                                                         :  4;
        unsigned int S6_Y                                                         :  4;
        unsigned int S7_X                                                         :  4;
        unsigned int S7_Y                                                         :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_AA_SAMPLE_LOCS_PIXEL_X0Y0_2 {
    struct {
        unsigned int S8_X                                                         :  4;
        unsigned int S8_Y                                                         :  4;
        unsigned int S9_X                                                         :  4;
        unsigned int S9_Y                                                         :  4;
        unsigned int S10_X                                                        :  4;
        unsigned int S10_Y                                                        :  4;
        unsigned int S11_X                                                        :  4;
        unsigned int S11_Y                                                        :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_AA_SAMPLE_LOCS_PIXEL_X0Y0_3 {
    struct {
        unsigned int S12_X                                                        :  4;
        unsigned int S12_Y                                                        :  4;
        unsigned int S13_X                                                        :  4;
        unsigned int S13_Y                                                        :  4;
        unsigned int S14_X                                                        :  4;
        unsigned int S14_Y                                                        :  4;
        unsigned int S15_X                                                        :  4;
        unsigned int S15_Y                                                        :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_AA_SAMPLE_LOCS_PIXEL_X0Y1_0 {
    struct {
        unsigned int S0_X                                                         :  4;
        unsigned int S0_Y                                                         :  4;
        unsigned int S1_X                                                         :  4;
        unsigned int S1_Y                                                         :  4;
        unsigned int S2_X                                                         :  4;
        unsigned int S2_Y                                                         :  4;
        unsigned int S3_X                                                         :  4;
        unsigned int S3_Y                                                         :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_AA_SAMPLE_LOCS_PIXEL_X0Y1_1 {
    struct {
        unsigned int S4_X                                                         :  4;
        unsigned int S4_Y                                                         :  4;
        unsigned int S5_X                                                         :  4;
        unsigned int S5_Y                                                         :  4;
        unsigned int S6_X                                                         :  4;
        unsigned int S6_Y                                                         :  4;
        unsigned int S7_X                                                         :  4;
        unsigned int S7_Y                                                         :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_AA_SAMPLE_LOCS_PIXEL_X0Y1_2 {
    struct {
        unsigned int S8_X                                                         :  4;
        unsigned int S8_Y                                                         :  4;
        unsigned int S9_X                                                         :  4;
        unsigned int S9_Y                                                         :  4;
        unsigned int S10_X                                                        :  4;
        unsigned int S10_Y                                                        :  4;
        unsigned int S11_X                                                        :  4;
        unsigned int S11_Y                                                        :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_AA_SAMPLE_LOCS_PIXEL_X0Y1_3 {
    struct {
        unsigned int S12_X                                                        :  4;
        unsigned int S12_Y                                                        :  4;
        unsigned int S13_X                                                        :  4;
        unsigned int S13_Y                                                        :  4;
        unsigned int S14_X                                                        :  4;
        unsigned int S14_Y                                                        :  4;
        unsigned int S15_X                                                        :  4;
        unsigned int S15_Y                                                        :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_AA_SAMPLE_LOCS_PIXEL_X1Y0_0 {
    struct {
        unsigned int S0_X                                                         :  4;
        unsigned int S0_Y                                                         :  4;
        unsigned int S1_X                                                         :  4;
        unsigned int S1_Y                                                         :  4;
        unsigned int S2_X                                                         :  4;
        unsigned int S2_Y                                                         :  4;
        unsigned int S3_X                                                         :  4;
        unsigned int S3_Y                                                         :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_AA_SAMPLE_LOCS_PIXEL_X1Y0_1 {
    struct {
        unsigned int S4_X                                                         :  4;
        unsigned int S4_Y                                                         :  4;
        unsigned int S5_X                                                         :  4;
        unsigned int S5_Y                                                         :  4;
        unsigned int S6_X                                                         :  4;
        unsigned int S6_Y                                                         :  4;
        unsigned int S7_X                                                         :  4;
        unsigned int S7_Y                                                         :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_AA_SAMPLE_LOCS_PIXEL_X1Y0_2 {
    struct {
        unsigned int S8_X                                                         :  4;
        unsigned int S8_Y                                                         :  4;
        unsigned int S9_X                                                         :  4;
        unsigned int S9_Y                                                         :  4;
        unsigned int S10_X                                                        :  4;
        unsigned int S10_Y                                                        :  4;
        unsigned int S11_X                                                        :  4;
        unsigned int S11_Y                                                        :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_AA_SAMPLE_LOCS_PIXEL_X1Y0_3 {
    struct {
        unsigned int S12_X                                                        :  4;
        unsigned int S12_Y                                                        :  4;
        unsigned int S13_X                                                        :  4;
        unsigned int S13_Y                                                        :  4;
        unsigned int S14_X                                                        :  4;
        unsigned int S14_Y                                                        :  4;
        unsigned int S15_X                                                        :  4;
        unsigned int S15_Y                                                        :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_AA_SAMPLE_LOCS_PIXEL_X1Y1_0 {
    struct {
        unsigned int S0_X                                                         :  4;
        unsigned int S0_Y                                                         :  4;
        unsigned int S1_X                                                         :  4;
        unsigned int S1_Y                                                         :  4;
        unsigned int S2_X                                                         :  4;
        unsigned int S2_Y                                                         :  4;
        unsigned int S3_X                                                         :  4;
        unsigned int S3_Y                                                         :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_AA_SAMPLE_LOCS_PIXEL_X1Y1_1 {
    struct {
        unsigned int S4_X                                                         :  4;
        unsigned int S4_Y                                                         :  4;
        unsigned int S5_X                                                         :  4;
        unsigned int S5_Y                                                         :  4;
        unsigned int S6_X                                                         :  4;
        unsigned int S6_Y                                                         :  4;
        unsigned int S7_X                                                         :  4;
        unsigned int S7_Y                                                         :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_AA_SAMPLE_LOCS_PIXEL_X1Y1_2 {
    struct {
        unsigned int S8_X                                                         :  4;
        unsigned int S8_Y                                                         :  4;
        unsigned int S9_X                                                         :  4;
        unsigned int S9_Y                                                         :  4;
        unsigned int S10_X                                                        :  4;
        unsigned int S10_Y                                                        :  4;
        unsigned int S11_X                                                        :  4;
        unsigned int S11_Y                                                        :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_AA_SAMPLE_LOCS_PIXEL_X1Y1_3 {
    struct {
        unsigned int S12_X                                                        :  4;
        unsigned int S12_Y                                                        :  4;
        unsigned int S13_X                                                        :  4;
        unsigned int S13_Y                                                        :  4;
        unsigned int S14_X                                                        :  4;
        unsigned int S14_Y                                                        :  4;
        unsigned int S15_X                                                        :  4;
        unsigned int S15_Y                                                        :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_ATM_CNTL {
    struct {
        unsigned int SC_PC_IF_SIZE                                                :  6;
        unsigned int                                                              :  1;
        unsigned int DISABLE_SC_PC_IF_FGCG_EN                                     :  1;
        unsigned int MAX_ATTRIBUTES_IN_WAVE                                       :  8;
        unsigned int DISABLE_MAX_ATTRIBUTES                                       :  1;
        unsigned int SELECT_MAX_ATTRIBUTES                                        :  1;
        unsigned int                                                              : 14;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_BINNER_CNTL_0 {
    struct {
        unsigned int BINNING_MODE                                                 :  2;
        unsigned int BIN_SIZE_X                                                   :  1;
        unsigned int BIN_SIZE_Y                                                   :  1;
        unsigned int BIN_SIZE_X_EXTEND                                            :  3;
        unsigned int BIN_SIZE_Y_EXTEND                                            :  3;
        unsigned int CONTEXT_STATES_PER_BIN                                       :  3;
        unsigned int PERSISTENT_STATES_PER_BIN                                    :  5;
        unsigned int DISABLE_START_OF_PRIM                                        :  1;
        unsigned int FPOVS_PER_BATCH                                              :  8;
        unsigned int OPTIMAL_BIN_SELECTION                                        :  1;
        unsigned int FLUSH_ON_BINNING_TRANSITION                                  :  1;
        unsigned int BIN_MAPPING_MODE                                             :  2;
        unsigned int RESERVED_31                                                  :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_BINNER_CNTL_1 {
    struct {
        unsigned int MAX_ALLOC_COUNT                                              : 16;
        unsigned int MAX_PRIM_PER_BATCH                                           : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_BINNER_CNTL_2 {
    struct {
        unsigned int BIN_SIZE_X_MULT_BY_1P5X                                      :  1;
        unsigned int BIN_SIZE_Y_MULT_BY_1P5X                                      :  1;
        unsigned int ENABLE_LIGHT_VOLUME_RENDERING_OPTIMIZATION                   :  1;
        unsigned int DUAL_LIGHT_SHAFT_IN_DRAW                                     :  1;
        unsigned int RESERVED_LIGHT_SHAFT_DRAW_CALL_LIMIT                         :  3;
        unsigned int CONTEXT_DONE_EVENTS_PER_BIN                                  :  4;
        unsigned int ZPP_ENABLED                                                  :  1;
        unsigned int ZPP_OPTIMIZATION_ENABLED                                     :  1;
        unsigned int ZPP_AREA_THRESHOLD                                           :  8;
        unsigned int DISABLE_NOPCEXPORT_BREAKBATCH_CONDITION                      :  1;
        unsigned int SBB_ENABLE                                                   :  1;
        unsigned int ENABLE_PING_PONG_BIN_ORDER                                   :  1;
        unsigned int PING_PONG_BIN_ORDER_FLIP                                     :  2;
        unsigned int LIGHT_SHAFT_DRAW_CALL_LIMIT                                  :  5;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_BINNER_CNTL_OVERRIDE {
    struct {
        unsigned int BINNING_MODE                                                 :  2;
        unsigned int                                                              :  8;
        unsigned int CONTEXT_STATES_PER_BIN                                       :  3;
        unsigned int PERSISTENT_STATES_PER_BIN                                    :  5;
        unsigned int                                                              :  1;
        unsigned int FPOVS_PER_BATCH                                              :  8;
        unsigned int DIRECT_OVERRIDE_MODE                                         :  1;
        unsigned int OVERRIDE                                                     :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_BINNER_DYNAMIC_BATCH_LIMIT {
    struct {
        unsigned int LIMIT                                                        : 11;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_BINNER_EVENT_CNTL_0 {
    struct {
        unsigned int RESERVED_0                                                   :  2;
        unsigned int SAMPLE_STREAMOUTSTATS1                                       :  2;
        unsigned int SAMPLE_STREAMOUTSTATS2                                       :  2;
        unsigned int SAMPLE_STREAMOUTSTATS3                                       :  2;
        unsigned int CACHE_FLUSH_TS                                               :  2;
        unsigned int CONTEXT_DONE                                                 :  2;
        unsigned int CACHE_FLUSH                                                  :  2;
        unsigned int CS_PARTIAL_FLUSH                                             :  2;
        unsigned int VGT_STREAMOUT_SYNC                                           :  2;
        unsigned int EVENT_STATE_CHANGE                                           :  2;
        unsigned int VGT_STREAMOUT_RESET                                          :  2;
        unsigned int END_OF_PIPE_INCR_DE                                          :  2;
        unsigned int END_OF_PIPE_IB_END                                           :  2;
        unsigned int RST_PIX_CNT                                                  :  2;
        unsigned int BREAK_BATCH                                                  :  2;
        unsigned int VS_PARTIAL_FLUSH                                             :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_BINNER_EVENT_CNTL_1 {
    struct {
        unsigned int PS_PARTIAL_FLUSH                                             :  2;
        unsigned int FLUSH_HS_OUTPUT                                              :  2;
        unsigned int FLUSH_DFSM                                                   :  2;
        unsigned int RESET_TO_LOWEST_VGT                                          :  2;
        unsigned int CACHE_FLUSH_AND_INV_TS_EVENT                                 :  2;
        unsigned int WAIT_SYNC                                                    :  2;
        unsigned int CACHE_FLUSH_AND_INV_EVENT                                    :  2;
        unsigned int PERFCOUNTER_START                                            :  2;
        unsigned int PERFCOUNTER_STOP                                             :  2;
        unsigned int PIPELINESTAT_START                                           :  2;
        unsigned int PIPELINESTAT_STOP                                            :  2;
        unsigned int PERFCOUNTER_SAMPLE                                           :  2;
        unsigned int FLUSH_ES_OUTPUT                                              :  2;
        unsigned int BIN_CONF_OVERRIDE_CHECK                                      :  2;
        unsigned int SAMPLE_PIPELINESTAT                                          :  2;
        unsigned int SO_VGTSTREAMOUT_FLUSH                                        :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_BINNER_EVENT_CNTL_2 {
    struct {
        unsigned int SAMPLE_STREAMOUTSTATS                                        :  2;
        unsigned int RESET_VTX_CNT                                                :  2;
        unsigned int BLOCK_CONTEXT_DONE                                           :  2;
        unsigned int RESERVED_35                                                  :  2;
        unsigned int VGT_FLUSH                                                    :  2;
        unsigned int TGID_ROLLOVER                                                :  2;
        unsigned int SQ_NON_EVENT                                                 :  2;
        unsigned int SC_SEND_DB_VPZ                                               :  2;
        unsigned int BOTTOM_OF_PIPE_TS                                            :  2;
        unsigned int RESERVED_41                                                  :  2;
        unsigned int DB_CACHE_FLUSH_AND_INV                                       :  2;
        unsigned int FLUSH_AND_INV_DB_DATA_TS                                     :  2;
        unsigned int FLUSH_AND_INV_DB_META                                        :  2;
        unsigned int FLUSH_AND_INV_CB_DATA_TS                                     :  2;
        unsigned int FLUSH_AND_INV_CB_META                                        :  2;
        unsigned int CS_DONE                                                      :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_BINNER_EVENT_CNTL_3 {
    struct {
        unsigned int PS_DONE                                                      :  2;
        unsigned int FLUSH_AND_INV_CB_PIXEL_DATA                                  :  2;
        unsigned int RESERVED_50                                                  :  2;
        unsigned int THREAD_TRACE_START                                           :  2;
        unsigned int THREAD_TRACE_STOP                                            :  2;
        unsigned int THREAD_TRACE_MARKER                                          :  2;
        unsigned int THREAD_TRACE_DRAW                                            :  2;
        unsigned int THREAD_TRACE_FINISH                                          :  2;
        unsigned int PIXEL_PIPE_STAT_CONTROL                                      :  2;
        unsigned int PIXEL_PIPE_STAT_DUMP                                         :  2;
        unsigned int PIXEL_PIPE_STAT_RESET                                        :  2;
        unsigned int CONTEXT_SUSPEND                                              :  2;
        unsigned int OFFCHIP_HS_DEALLOC                                           :  2;
        unsigned int ENABLE_NGG_PIPELINE                                          :  2;
        unsigned int ENABLE_PIPELINE_NOT_USED                                     :  2;
        unsigned int DRAW_DONE                                                    :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_BINNER_OUTPUT_TIMEOUT_CNTL {
    struct {
        unsigned int THRESHOLD                                                    : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_BINNER_OUTPUT_TIMEOUT_COUNTER {
    struct {
        unsigned int THRESHOLD                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_BINNER_PERF_CNTL_0 {
    struct {
        unsigned int BIN_HIST_NUM_PRIMS_THRESHOLD                                 : 10;
        unsigned int BATCH_HIST_NUM_PRIMS_THRESHOLD                               : 10;
        unsigned int BIN_HIST_NUM_CONTEXT_THRESHOLD                               :  3;
        unsigned int BATCH_HIST_NUM_CONTEXT_THRESHOLD                             :  3;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_BINNER_PERF_CNTL_1 {
    struct {
        unsigned int BIN_HIST_NUM_PERSISTENT_STATE_THRESHOLD                      :  5;
        unsigned int BATCH_HIST_NUM_PERSISTENT_STATE_THRESHOLD                    :  5;
        unsigned int BATCH_HIST_NUM_TRIV_REJECTED_PRIMS_THRESHOLD                 : 16;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_BINNER_PERF_CNTL_2 {
    struct {
        unsigned int BATCH_HIST_NUM_ROWS_PER_PRIM_THRESHOLD                       : 11;
        unsigned int BATCH_HIST_NUM_COLUMNS_PER_ROW_THRESHOLD                     : 11;
        unsigned int                                                              : 10;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_BINNER_PERF_CNTL_3 {
    struct {
        unsigned int BATCH_HIST_NUM_PS_WAVE_BREAKS_THRESHOLD                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_BINNER_TIMEOUT_COUNTER {
    struct {
        unsigned int THRESHOLD                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_CENTROID_PRIORITY_0 {
    struct {
        unsigned int DISTANCE_0                                                   :  4;
        unsigned int DISTANCE_1                                                   :  4;
        unsigned int DISTANCE_2                                                   :  4;
        unsigned int DISTANCE_3                                                   :  4;
        unsigned int DISTANCE_4                                                   :  4;
        unsigned int DISTANCE_5                                                   :  4;
        unsigned int DISTANCE_6                                                   :  4;
        unsigned int DISTANCE_7                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_CENTROID_PRIORITY_1 {
    struct {
        unsigned int DISTANCE_8                                                   :  4;
        unsigned int DISTANCE_9                                                   :  4;
        unsigned int DISTANCE_10                                                  :  4;
        unsigned int DISTANCE_11                                                  :  4;
        unsigned int DISTANCE_12                                                  :  4;
        unsigned int DISTANCE_13                                                  :  4;
        unsigned int DISTANCE_14                                                  :  4;
        unsigned int DISTANCE_15                                                  :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_CLIPRECT_0_BR {
    struct {
        unsigned int BR_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int BR_Y                                                         : 15;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_CLIPRECT_0_EXT {
    struct {
        unsigned int BR_X_EXT                                                     :  2;
        unsigned int BR_Y_EXT                                                     :  2;
        unsigned int TL_X_EXT                                                     :  2;
        unsigned int TL_Y_EXT                                                     :  2;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_CLIPRECT_0_TL {
    struct {
        unsigned int TL_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int TL_Y                                                         : 15;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_CLIPRECT_1_BR {
    struct {
        unsigned int BR_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int BR_Y                                                         : 15;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_CLIPRECT_1_EXT {
    struct {
        unsigned int BR_X_EXT                                                     :  2;
        unsigned int BR_Y_EXT                                                     :  2;
        unsigned int TL_X_EXT                                                     :  2;
        unsigned int TL_Y_EXT                                                     :  2;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_CLIPRECT_1_TL {
    struct {
        unsigned int TL_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int TL_Y                                                         : 15;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_CLIPRECT_2_BR {
    struct {
        unsigned int BR_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int BR_Y                                                         : 15;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_CLIPRECT_2_EXT {
    struct {
        unsigned int BR_X_EXT                                                     :  2;
        unsigned int BR_Y_EXT                                                     :  2;
        unsigned int TL_X_EXT                                                     :  2;
        unsigned int TL_Y_EXT                                                     :  2;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_CLIPRECT_2_TL {
    struct {
        unsigned int TL_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int TL_Y                                                         : 15;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_CLIPRECT_3_BR {
    struct {
        unsigned int BR_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int BR_Y                                                         : 15;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_CLIPRECT_3_EXT {
    struct {
        unsigned int BR_X_EXT                                                     :  2;
        unsigned int BR_Y_EXT                                                     :  2;
        unsigned int TL_X_EXT                                                     :  2;
        unsigned int TL_Y_EXT                                                     :  2;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_CLIPRECT_3_TL {
    struct {
        unsigned int TL_X                                                         : 15;
        unsigned int                                                              :  1;
        unsigned int TL_Y                                                         : 15;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_CLIPRECT_RULE {
    struct {
        unsigned int CLIP_RULE                                                    : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_CONSERVATIVE_RASTERIZATION_CNTL {
    struct {
        unsigned int OVER_RAST_ENABLE                                             :  1;
        unsigned int OVER_RAST_SAMPLE_SELECT                                      :  4;
        unsigned int UNDER_RAST_ENABLE                                            :  1;
        unsigned int UNDER_RAST_SAMPLE_SELECT                                     :  4;
        unsigned int PBB_UNCERTAINTY_REGION_ENABLE                                :  1;
        unsigned int ZMM_TRI_EXTENT                                               :  1;
        unsigned int ZMM_TRI_OFFSET                                               :  1;
        unsigned int OVERRIDE_OVER_RAST_INNER_TO_NORMAL                           :  1;
        unsigned int OVERRIDE_UNDER_RAST_INNER_TO_NORMAL                          :  1;
        unsigned int DEGENERATE_OVERRIDE_INNER_TO_NORMAL_DISABLE                  :  1;
        unsigned int UNCERTAINTY_REGION_MODE                                      :  2;
        unsigned int OUTER_UNCERTAINTY_EDGERULE_OVERRIDE                          :  1;
        unsigned int INNER_UNCERTAINTY_EDGERULE_OVERRIDE                          :  1;
        unsigned int NULL_SQUAD_AA_MASK_ENABLE                                    :  1;
        unsigned int COVERAGE_AA_MASK_ENABLE                                      :  1;
        unsigned int PREZ_AA_MASK_ENABLE                                          :  1;
        unsigned int POSTZ_AA_MASK_ENABLE                                         :  1;
        unsigned int CENTROID_SAMPLE_OVERRIDE                                     :  1;
        unsigned int UNCERTAINTY_REGION_MULT                                      :  2;
        unsigned int UNCERTAINTY_REGION_PBB_MULT                                  :  2;
        unsigned int                                                              :  3;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_DSM_CNTL {
    struct {
        unsigned int FORCE_EOV_REZ_0                                              :  1;
        unsigned int FORCE_EOV_REZ_1                                              :  1;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_EDGERULE {
    struct {
        unsigned int ER_TRI                                                       :  4;
        unsigned int ER_POINT                                                     :  4;
        unsigned int ER_RECT                                                      :  4;
        unsigned int ER_LINE_LR                                                   :  6;
        unsigned int ER_LINE_RL                                                   :  6;
        unsigned int ER_LINE_TB                                                   :  4;
        unsigned int ER_LINE_BT                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_ENHANCE {
    struct {
        unsigned int ENABLE_PA_SC_OUT_OF_ORDER                                    :  1;
        unsigned int DISABLE_SC_DB_TILE_FIX                                       :  1;
        unsigned int DISABLE_AA_MASK_FULL_FIX                                     :  1;
        unsigned int ENABLE_1XMSAA_SAMPLE_LOCATIONS                               :  1;
        unsigned int ENABLE_1XMSAA_SAMPLE_LOC_CENTROID                            :  1;
        unsigned int DISABLE_SCISSOR_FIX                                          :  1;
        unsigned int SEND_UNLIT_STILES_TO_PACKER                                  :  1;
        unsigned int DISABLE_DUALGRAD_PERF_OPTIMIZATION                           :  1;
        unsigned int DISABLE_SC_PROCESS_RESET_PRIM                                :  1;
        unsigned int DISABLE_SC_PROCESS_RESET_SUPERTILE                           :  1;
        unsigned int DISABLE_SC_PROCESS_RESET_TILE                                :  1;
        unsigned int DISABLE_PA_SC_GUIDANCE                                       :  1;
        unsigned int DISABLE_EOV_ALL_CTRL_ONLY_COMBINATIONS                       :  1;
        unsigned int ENABLE_MULTICYCLE_BUBBLE_FREEZE                              :  1;
        unsigned int DISABLE_OUT_OF_ORDER_PA_SC_GUIDANCE                          :  1;
        unsigned int ENABLE_OUT_OF_ORDER_POLY_MODE                                :  1;
        unsigned int DISABLE_OUT_OF_ORDER_EOP_SYNC_NULL_PRIMS_LAST                :  1;
        unsigned int DISABLE_OUT_OF_ORDER_THRESHOLD_SWITCHING                     :  1;
        unsigned int ENABLE_OUT_OF_ORDER_THRESHOLD_SWITCH_AT_EOPG_ONLY            :  1;
        unsigned int DISABLE_OUT_OF_ORDER_DESIRED_FIFO_EMPTY_SWITCHING            :  1;
        unsigned int DISABLE_OUT_OF_ORDER_SELECTED_FIFO_EMPTY_SWITCHING           :  1;
        unsigned int DISABLE_OUT_OF_ORDER_EMPTY_SWITCHING_HYSTERYSIS              :  1;
        unsigned int ENABLE_OUT_OF_ORDER_DESIRED_FIFO_IS_NEXT_FEID                :  1;
        unsigned int DISABLE_OOO_NO_EOPG_SKEW_DESIRED_FIFO_IS_CURRENT_FIFO        :  1;
        unsigned int OOO_DISABLE_EOP_ON_FIRST_LIVE_PRIM_HIT                       :  1;
        unsigned int OOO_DISABLE_EOPG_SKEW_THRESHOLD_SWITCHING                    :  1;
        unsigned int DISABLE_EOP_LINE_STIPPLE_RESET                               :  1;
        unsigned int DISABLE_VPZ_EOP_LINE_STIPPLE_RESET                           :  1;
        unsigned int IOO_DISABLE_SCAN_UNSELECTED_FIFOS_FOR_DUAL_GFX_RING_CHANGE   :  1;
        unsigned int OOO_USE_ABSOLUTE_FIFO_COUNT_IN_THRESHOLD_SWITCHING           :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_ENHANCE_1 {
    struct {
        unsigned int REALIGN_DQUADS_OVERRIDE_ENABLE                               :  1;
        unsigned int REALIGN_DQUADS_OVERRIDE                                      :  2;
        unsigned int DISABLE_SC_BINNING                                           :  1;
        unsigned int BYPASS_PBB                                                   :  1;
        unsigned int DISABLE_NONBINNED_LIVE_PRIM_DG1_LS0_CL0_EOPKT_POKE           :  1;
        unsigned int ECO_SPARE1                                                   :  1;
        unsigned int ECO_SPARE2                                                   :  1;
        unsigned int ECO_SPARE3                                                   :  1;
        unsigned int DISABLE_SC_PROCESS_RESET_PBB                                 :  1;
        unsigned int DISABLE_PBB_SCISSOR_OPT                                      :  1;
        unsigned int ENABLE_DFSM_FLUSH_EVENT_TO_FLUSH_POPS_CAM                    :  1;
        unsigned int                                                              :  1;
        unsigned int DEBUG_PIXEL_PICKER_COUNT_PIXELS                              :  1;
        unsigned int DISABLE_SC_DB_STILE_INTF_FINE_CLOCK_GATE                     :  1;
        unsigned int                                                              :  1;
        unsigned int DISABLE_PACKER_ODC_ENHANCE                                   :  1;
        unsigned int                                                              :  1;
        unsigned int OPTIMAL_BIN_SELECTION                                        :  1;
        unsigned int DISABLE_FORCE_SOP_ALL_EVENTS                                 :  1;
        unsigned int DISABLE_PBB_CLK_OPTIMIZATION                                 :  1;
        unsigned int DISABLE_PBB_SCISSOR_CLK_OPTIMIZATION                         :  1;
        unsigned int DISABLE_PBB_BINNING_CLK_OPTIMIZATION                         :  1;
        unsigned int DISABLE_INTF_CG                                              :  1;
        unsigned int IOO_DISABLE_EOP_ON_FIRST_LIVE_PRIM_HIT                       :  1;
        unsigned int DISABLE_SHADER_PROFILING_FOR_POWER                           :  1;
        unsigned int FLUSH_ON_BINNING_TRANSITION                                  :  1;
        unsigned int DISABLE_QUAD_PROC_FDCE_ENHANCE                               :  1;
        unsigned int DISABLE_SC_PS_PA_ARBITER_FIX                                 :  1;
        unsigned int DISABLE_SC_PS_PA_ARBITER_FIX_1                               :  1;
        unsigned int PASS_VPZ_EVENT_TO_SPI                                        :  1;
        unsigned int RESERVED_31                                                  :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_ENHANCE_2 {
    struct {
        unsigned int                                                              :  2;
        unsigned int DISABLE_SC_BCI_QUAD_INTF_FINE_CLOCK_GATE                     :  1;
        unsigned int DISABLE_SC_BCI_PRIM_INTF_FINE_CLOCK_GATE                     :  1;
        unsigned int ENABLE_LPOV_WAVE_BREAK                                       :  1;
        unsigned int ENABLE_FPOV_WAVE_BREAK                                       :  1;
        unsigned int                                                              :  1;
        unsigned int ENABLE_SC_SEND_DB_VPZ_FOR_EN_PRIM_PAYLOAD                    :  1;
        unsigned int DISABLE_BREAK_BATCH_ON_GFX_PIPE_SWITCH                       :  1;
        unsigned int DISABLE_FULL_TILE_WAVE_BREAK                                 :  1;
        unsigned int ENABLE_VPZ_INJECTION_BEFORE_NULL_PRIMS                       :  1;
        unsigned int PBB_TIMEOUT_THRESHOLD_MODE                                   :  1;
        unsigned int DISABLE_PACKER_GRAD_FDCE_ENHANCE                             :  1;
        unsigned int DISABLE_SC_SPI_INTF_EARLY_WAKEUP                             :  1;
        unsigned int DISABLE_SC_BCI_INTF_EARLY_WAKEUP                             :  1;
        unsigned int DISABLE_EXPOSED_GT_DETAIL_RATE_TILE_COV_ADJ                  :  1;
        unsigned int RESERVED_16                                                  :  1;
        unsigned int PBB_MAIN_CLK_REG_BUSY_WAKEUP                                 :  1;
        unsigned int DISABLE_BREAK_BATCH_ON_GFX_PIPELINE_RESET                    :  1;
        unsigned int                                                              :  3;
        unsigned int RESERVED_22                                                  :  1;
        unsigned int PROCESS_RESET_FORCE_STILE_MASK_TO_ZERO                       :  1;
        unsigned int                                                              :  2;
        unsigned int BREAK_WHEN_ONE_NULL_PRIM_BATCH                               :  1;
        unsigned int NULL_PRIM_BREAK_BATCH_LIMIT                                  :  3;
        unsigned int DISABLE_MAX_DEALLOC_FORCE_EOV_RESET_N_WAVES_COUNT            :  1;
        unsigned int RSVD                                                         :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_ENHANCE_3 {
    struct {
        unsigned int FORCE_USE_OF_SC_CENTROID_DATA                                :  1;
        unsigned int ECO_SPARE2                                                   :  1;
        unsigned int DISABLE_RB_MASK_COPY_FOR_NONP2_SA_PAIR_HARVEST               :  1;
        unsigned int FORCE_PBB_WORKLOAD_MODE_TO_ZERO                              :  1;
        unsigned int DISABLE_PKR_BCI_QUAD_NEW_PRIM_DATA_LOAD_OPTIMIZATION         :  1;
        unsigned int DISABLE_CP_CONTEXT_DONE_PERFCOUNT_SAMPLE_EN                  :  1;
        unsigned int ENABLE_SINGLE_PA_EOPKT_FIRST_PHASE_FILTER                    :  1;
        unsigned int ENABLE_SINGLE_PA_EOPKT_LAST_PHASE_FILTER                     :  1;
        unsigned int ENABLE_SINGLE_PA_EOPKT_LAST_PHASE_FILTER_FOR_PBB_BINNED_PRIMS :  1;
        unsigned int DISABLE_SET_VPZ_DIRTY_EOPKT_LAST_PHASE_ONLY                  :  1;
        unsigned int DISABLE_PBB_EOP_OPTIMIZATION_WITH_SAME_CONTEXT_BATCHES       :  1;
        unsigned int DISABLE_FAST_NULL_PRIM_OPTIMIZATION                          :  1;
        unsigned int USE_PBB_PRIM_STORAGE_WHEN_STALLED                            :  1;
        unsigned int DISABLE_LIGHT_VOLUME_RENDERING_OPTIMIZATION                  :  1;
        unsigned int DISABLE_ZPRE_PASS_OPTIMIZATION                               :  1;
        unsigned int DISABLE_EVENT_INCLUSION_IN_CONTEXT_STATES_PER_BIN            :  1;
        unsigned int DISABLE_PIXEL_WAIT_SYNC_COUNTERS                             :  1;
        unsigned int DISABLE_SC_CPG_PSINVOC_SEDC_ISOLATION_ACCUM                  :  1;
        unsigned int                                                              :  2;
        unsigned int DISABLE_PKR_FORCE_EOV_MAX_REZ_CNT_FOR_SPI_BACKPRESSURE_ONLY  :  1;
        unsigned int DISABLE_PKR_FORCE_EOV_MAX_CLK_CNT_FOR_SPI_BACKPRESSURE_ONLY  :  1;
        unsigned int DO_NOT_INCLUDE_OREO_WAVEID_IN_FORCE_EOV_MAX_CNT_DISABLE      :  1;
        unsigned int DISABLE_PWS_PRE_DEPTH_WAIT_SYNC_VPZ_INSERTION                :  1;
        unsigned int PKR_CNT_FORCE_EOV_AT_QS_EMPTY_ONLY                           :  1;
        unsigned int PKR_S0_FORCE_EOV_STALL                                       :  1;
        unsigned int PKR_S1_FORCE_EOV_STALL                                       :  1;
        unsigned int APPLY_AA_MASK_AT_EXPOSED_RATE_FOR_VRS_COURSE_QUADS_WITH_CR   :  1;
        unsigned int ECO_SPARE0                                                   :  1;
        unsigned int ECO_SPARE1                                                   :  1;
        unsigned int DISABLE_SC_GL1X_SRC_FINE_CLOCK_GATE                          :  1;
        unsigned int DISABLE_SC_GL1X_REQ_FINE_CLOCK_GATE                          :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_FIFO_DEPTH_CNTL {
    struct {
        unsigned int DEPTH                                                        : 10;
        unsigned int                                                              : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_FIFO_SIZE {
    struct {
        unsigned int SC_FRONTEND_PRIM_FIFO_SIZE                                   :  6;
        unsigned int SC_BACKEND_PRIM_FIFO_SIZE                                    :  9;
        unsigned int SC_HIZ_TILE_FIFO_SIZE                                        :  6;
        unsigned int SC_EARLYZ_TILE_FIFO_SIZE                                     : 11;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_FORCE_EOV_MAX_CNTS {
    struct {
        unsigned int FORCE_EOV_MAX_CLK_CNT                                        : 16;
        unsigned int FORCE_EOV_MAX_REZ_CNT                                        : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_GENERIC_SCISSOR_BR {
    struct {
        unsigned int BR_X                                                         : 16;
        unsigned int BR_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_GENERIC_SCISSOR_TL {
    struct {
        unsigned int TL_X                                                         : 16;
        unsigned int TL_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_HISZ_CONTROL {
    struct {
        unsigned int ROUND                                                        :  3;
        unsigned int CONSERVATIVE_Z_EXPORT                                        :  2;
        unsigned int                                                              : 27;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_HISZ_RENDER_OVERRIDE {
    struct {
        unsigned int FORCE_HIZ_ENABLE                                             :  2;
        unsigned int FORCE_HIS_ENABLE                                             :  2;
        unsigned int FAST_Z_DISABLE                                               :  1;
        unsigned int FAST_STENCIL_DISABLE                                         :  1;
        unsigned int FORCE_FULL_Z_RANGE                                           :  2;
        unsigned int DISABLE_TILE_RATE_TILES                                      :  1;
        unsigned int DEPTH_BOUNDS_HIER_DEPTH_DISABLE                              :  1;
        unsigned int PRESERVE_ZRANGE                                              :  1;
        unsigned int DISABLE_FAST_PASS                                            :  1;
        unsigned int DISABLE_SINGLE_STENCIL                                       :  1;
        unsigned int                                                              : 19;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_HIS_BASE {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_HIS_BASE_EXT {
    struct {
        unsigned int BASE_256B                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_HIS_INFO {
    struct {
        unsigned int SURFACE_ENABLE                                               :  1;
        unsigned int SW_MODE                                                      :  3;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_HIS_SIZE_XY {
    struct {
        unsigned int X_MAX                                                        : 13;
        unsigned int                                                              :  3;
        unsigned int Y_MAX                                                        : 13;
        unsigned int                                                              :  3;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_HIS_SURFACE_CNTL {
    struct {
        unsigned int                                                              :  5;
        unsigned int HSC_OUTSTANDING_CONTEXT_FILTERING_DISABLE                    :  1;
        unsigned int HSC_CONTEXT_DONE_SYNC_DISABLE                                :  1;
        unsigned int HSC_DB_CONTEXT_DONE_SYNC_DISABLE                             :  1;
        unsigned int HSC_FLUSH_EVENT_MASK_DISABLE                                 :  5;
        unsigned int HSC_PREFETCH_DISABLE                                         :  1;
        unsigned int HSC_FLUSH_NO_INV_DISABLE                                     :  1;
        unsigned int HSC_NONSTALLING_FLUSH_DISABLE                                :  1;
        unsigned int HSC_PARTIAL_FLUSH_DISABLE                                    :  1;
        unsigned int                                                              :  1;
        unsigned int HSC_EOP_SYNC_DISABLE                                         :  1;
        unsigned int HSC_MAX_TAGS                                                 :  7;
        unsigned int HSC_EVICT_POINT                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_HIZ_BASE {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_HIZ_BASE_EXT {
    struct {
        unsigned int BASE_256B                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_HIZ_INFO {
    struct {
        unsigned int SURFACE_ENABLE                                               :  1;
        unsigned int FORMAT                                                       :  1;
        unsigned int SW_MODE                                                      :  3;
        unsigned int                                                              : 20;
        unsigned int DST_OUTSIDE_ZERO_TO_ONE                                      :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_HIZ_SIZE_XY {
    struct {
        unsigned int X_MAX                                                        : 13;
        unsigned int                                                              :  3;
        unsigned int Y_MAX                                                        : 13;
        unsigned int                                                              :  3;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_HIZ_SURFACE_CNTL {
    struct {
        unsigned int                                                              :  5;
        unsigned int HZC_OUTSTANDING_CONTEXT_FILTERING_DISABLE                    :  1;
        unsigned int HZC_CONTEXT_DONE_SYNC_DISABLE                                :  1;
        unsigned int HZC_DB_CONTEXT_DONE_SYNC_DISABLE                             :  1;
        unsigned int HZC_FLUSH_EVENT_MASK_DISABLE                                 :  5;
        unsigned int HZC_PREFETCH_DISABLE                                         :  1;
        unsigned int HZC_FLUSH_NO_INV_DISABLE                                     :  1;
        unsigned int HZC_NONSTALLING_FLUSH_DISABLE                                :  1;
        unsigned int HZC_PARTIAL_FLUSH_DISABLE                                    :  1;
        unsigned int                                                              :  1;
        unsigned int HZC_EOP_SYNC_DISABLE                                         :  1;
        unsigned int HZC_MAX_TAGS                                                 :  7;
        unsigned int HZC_EVICT_POINT                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_HP3D_TRAP_SCREEN_COUNT {
    struct {
        unsigned int COUNT                                                        : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_HP3D_TRAP_SCREEN_H {
    struct {
        unsigned int X_COORD                                                      : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_HP3D_TRAP_SCREEN_HV_EN {
    struct {
        unsigned int ENABLE_HV_PRE_SHADER                                         :  1;
        unsigned int FORCE_PRE_SHADER_ALL_PIXELS                                  :  1;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_HP3D_TRAP_SCREEN_HV_LOCK {
    struct {
        unsigned int DISABLE_NON_PRIV_WRITES                                      :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_HP3D_TRAP_SCREEN_OCCURRENCE {
    struct {
        unsigned int COUNT                                                        : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_HP3D_TRAP_SCREEN_V {
    struct {
        unsigned int Y_COORD                                                      : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_IF_FIFO_SIZE {
    struct {
        unsigned int SC_DB_TILE_IF_FIFO_SIZE                                      :  6;
        unsigned int SC_DB_QUAD_IF_FIFO_SIZE                                      :  6;
        unsigned int SC_SPI_IF_FIFO_SIZE                                          :  6;
        unsigned int SC_BCI_IF_FIFO_SIZE                                          :  6;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_LINE_CNTL {
    struct {
        unsigned int                                                              :  9;
        unsigned int EXPAND_LINE_WIDTH                                            :  1;
        unsigned int LAST_PIXEL                                                   :  1;
        unsigned int PERPENDICULAR_ENDCAP_ENA                                     :  1;
        unsigned int DX10_DIAMOND_TEST_ENA                                        :  1;
        unsigned int EXTRA_DX_DY_PRECISION                                        :  1;
        unsigned int                                                              : 18;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_LINE_STIPPLE {
    struct {
        unsigned int LINE_PATTERN                                                 : 16;
        unsigned int REPEAT_COUNT                                                 :  8;
        unsigned int                                                              :  4;
        unsigned int PATTERN_BIT_ORDER                                            :  1;
        unsigned int                                                              :  3;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_LINE_STIPPLE_RESET {
    struct {
        unsigned int AUTO_RESET_CNTL                                              :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_LINE_STIPPLE_STATE {
    struct {
        unsigned int CURRENT_PTR                                                  :  4;
        unsigned int                                                              :  4;
        unsigned int CURRENT_COUNT                                                :  8;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_MODE_CNTL_0 {
    struct {
        unsigned int MSAA_ENABLE                                                  :  1;
        unsigned int VPORT_SCISSOR_ENABLE                                         :  1;
        unsigned int LINE_STIPPLE_ENABLE                                          :  1;
        unsigned int SEND_UNLIT_STILES_TO_PKR                                     :  1;
        unsigned int                                                              :  1;
        unsigned int ALTERNATE_RBS_PER_TILE                                       :  1;
        unsigned int COARSE_TILE_STARTS_ON_EVEN_RB                                :  1;
        unsigned int IMPLICIT_VPORT_SCISSOR_ENABLE                                :  1;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_MODE_CNTL_1 {
    struct {
        unsigned int WALK_SIZE                                                    :  1;
        unsigned int WALK_ALIGNMENT                                               :  1;
        unsigned int WALK_ALIGN8_PRIM_FITS_ST                                     :  1;
        unsigned int WALK_FENCE_ENABLE                                            :  1;
        unsigned int WALK_FENCE_SIZE                                              :  3;
        unsigned int SUPERTILE_WALK_ORDER_ENABLE                                  :  1;
        unsigned int TILE_WALK_ORDER_ENABLE                                       :  1;
        unsigned int TILE_COVER_DISABLE                                           :  1;
        unsigned int TILE_COVER_NO_SCISSOR                                        :  1;
        unsigned int ZMM_LINE_EXTENT                                              :  1;
        unsigned int ZMM_LINE_OFFSET                                              :  1;
        unsigned int ZMM_RECT_EXTENT                                              :  1;
        unsigned int KILL_PIX_POST_HI_Z                                           :  1;
        unsigned int KILL_PIX_POST_DETAIL_MASK                                    :  1;
        unsigned int PS_ITER_SAMPLE                                               :  1;
        unsigned int MULTI_SHADER_ENGINE_PRIM_DISCARD_ENABLE                      :  1;
        unsigned int MULTI_GPU_SUPERTILE_ENABLE                                   :  1;
        unsigned int GPU_ID_OVERRIDE_ENABLE                                       :  1;
        unsigned int GPU_ID_OVERRIDE                                              :  4;
        unsigned int MULTI_GPU_PRIM_DISCARD_ENABLE                                :  1;
        unsigned int FORCE_EOV_CNTDWN_ENABLE                                      :  1;
        unsigned int FORCE_EOV_REZ_ENABLE                                         :  1;
        unsigned int OUT_OF_ORDER_PRIMITIVE_ENABLE                                :  1;
        unsigned int OUT_OF_ORDER_WATER_MARK                                      :  3;
        unsigned int DISABLE_4X_TILE_PICKING                                      :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_NGG_MODE_CNTL {
    struct {
        unsigned int MAX_DEALLOCS_IN_WAVE                                         : 11;
        unsigned int                                                              :  1;
        unsigned int DISABLE_FPOG_AND_DEALLOC_CONFLICT                            :  1;
        unsigned int DISABLE_MAX_DEALLOC                                          :  1;
        unsigned int DISABLE_MAX_ATTRIBUTES                                       :  1;
        unsigned int                                                              :  1;
        unsigned int MAX_FPOVS_IN_WAVE                                            :  8;
        unsigned int MAX_ATTRIBUTES_IN_WAVE                                       :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_P3D_TRAP_SCREEN_COUNT {
    struct {
        unsigned int COUNT                                                        : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_P3D_TRAP_SCREEN_H {
    struct {
        unsigned int X_COORD                                                      : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_P3D_TRAP_SCREEN_HV_EN {
    struct {
        unsigned int ENABLE_HV_PRE_SHADER                                         :  1;
        unsigned int FORCE_PRE_SHADER_ALL_PIXELS                                  :  1;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_P3D_TRAP_SCREEN_HV_LOCK {
    struct {
        unsigned int DISABLE_NON_PRIV_WRITES                                      :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_P3D_TRAP_SCREEN_OCCURRENCE {
    struct {
        unsigned int COUNT                                                        : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_P3D_TRAP_SCREEN_V {
    struct {
        unsigned int Y_COORD                                                      : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PACKER_WAVE_ID_CNTL {
    struct {
        unsigned int WAVES_IN_FLIGHT_LIMIT                                        :  9;
        unsigned int                                                              :  1;
        unsigned int SC_DB_WAVE_IF_FIFO_SIZE                                      :  6;
        unsigned int DISABLE_SC_DB_WAVE_IF_FGCG_EN                                :  1;
        unsigned int SC_SPI_WAVE_IF_FIFO_SIZE                                     :  6;
        unsigned int DISABLE_SC_SPI_WAVE_IF_FGCG_EN                               :  1;
        unsigned int DEBUG_CONFLICT_QUAD                                          :  4;
        unsigned int                                                              :  3;
        unsigned int DISABLE_OREO_CONFLICT_QUAD                                   :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PBB_OVERRIDE_FLAG {
    struct {
        unsigned int OVERRIDE                                                     :  1;
        unsigned int PIPE_ID                                                      :  1;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER3_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER3_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER4_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER4_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER4_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER5_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER5_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER5_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER6_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER6_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER6_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER7_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER7_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PERFCOUNTER7_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_PKR_WAVE_TABLE_CNTL {
    struct {
        unsigned int SIZE                                                         :  6;
        unsigned int                                                              : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_RASTER_CONFIG {
    struct {
        unsigned int RB_MAP_PKR0                                                  :  2;
        unsigned int RB_MAP_PKR1                                                  :  2;
        unsigned int RB_XSEL2                                                     :  2;
        unsigned int RB_XSEL                                                      :  1;
        unsigned int RB_YSEL                                                      :  1;
        unsigned int PKR_MAP                                                      :  2;
        unsigned int PKR_XSEL                                                     :  2;
        unsigned int PKR_YSEL                                                     :  2;
        unsigned int PKR_XSEL2                                                    :  2;
        unsigned int SC_MAP                                                       :  2;
        unsigned int SC_XSEL                                                      :  2;
        unsigned int SC_YSEL                                                      :  2;
        unsigned int                                                              :  2;
        unsigned int SE_MAP                                                       :  2;
        unsigned int SE_XSEL                                                      :  2;
        unsigned int SE_YSEL                                                      :  2;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_RASTER_CONFIG_1 {
    struct {
        unsigned int SE_PAIR_MAP                                                  :  2;
        unsigned int SE_PAIR_XSEL                                                 :  2;
        unsigned int SE_PAIR_YSEL                                                 :  2;
        unsigned int                                                              : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_SAMPLE_PROPERTIES {
    struct {
        unsigned int MAX_SAMPLE_DIST                                              :  4;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_SCREEN_SCISSOR_BR {
    struct {
        unsigned int BR_X                                                         : 16;
        unsigned int BR_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_SCREEN_SCISSOR_TL {
    struct {
        unsigned int TL_X                                                         : 16;
        unsigned int TL_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_SHADER_CONTROL {
    struct {
        unsigned int REALIGN_DQUADS_AFTER_N_WAVES                                 :  2;
        unsigned int LOAD_COLLISION_WAVEID                                        :  1;
        unsigned int LOAD_INTRAWAVE_COLLISION                                     :  1;
        unsigned int                                                              :  1;
        unsigned int WAVE_BREAK_REGION_SIZE                                       :  2;
        unsigned int DISABLE_OREO_CONFLICT_QUAD                                   :  1;
        unsigned int PS_ITER_SAMPLE                                               :  1;
        unsigned int                                                              : 23;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_TILE_STEERING_CREST_OVERRIDE {
    struct {
        unsigned int ONE_RB_MODE_ENABLE                                           :  1;
        unsigned int SE_SELECT                                                    :  2;
        unsigned int                                                              :  2;
        unsigned int RB_SELECT                                                    :  2;
        unsigned int                                                              :  1;
        unsigned int SA_SELECT                                                    :  3;
        unsigned int                                                              : 20;
        unsigned int FORCE_TILE_STEERING_OVERRIDE_USE                             :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_TILE_STEERING_OVERRIDE {
    struct {
        unsigned int ENABLE                                                       :  1;
        unsigned int                                                              : 11;
        unsigned int NUM_SC                                                       :  2;
        unsigned int                                                              :  2;
        unsigned int NUM_RB_PER_SC                                                :  2;
        unsigned int                                                              :  2;
        unsigned int NUM_PACKER_PER_SC                                            :  2;
        unsigned int                                                              : 10;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_TRAP_SCREEN_COUNT {
    struct {
        unsigned int COUNT                                                        : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_TRAP_SCREEN_H {
    struct {
        unsigned int X_COORD                                                      : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_TRAP_SCREEN_HV_EN {
    struct {
        unsigned int ENABLE_HV_PRE_SHADER                                         :  1;
        unsigned int FORCE_PRE_SHADER_ALL_PIXELS                                  :  1;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_TRAP_SCREEN_HV_LOCK {
    struct {
        unsigned int DISABLE_NON_PRIV_WRITES                                      :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_TRAP_SCREEN_OCCURRENCE {
    struct {
        unsigned int COUNT                                                        : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_TRAP_SCREEN_V {
    struct {
        unsigned int Y_COORD                                                      : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_0_BR {
    struct {
        unsigned int BR_X                                                         : 16;
        unsigned int BR_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_0_TL {
    struct {
        unsigned int TL_X                                                         : 16;
        unsigned int TL_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_1_BR {
    struct {
        unsigned int BR_X                                                         : 16;
        unsigned int BR_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_1_TL {
    struct {
        unsigned int TL_X                                                         : 16;
        unsigned int TL_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_2_BR {
    struct {
        unsigned int BR_X                                                         : 16;
        unsigned int BR_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_2_TL {
    struct {
        unsigned int TL_X                                                         : 16;
        unsigned int TL_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_3_BR {
    struct {
        unsigned int BR_X                                                         : 16;
        unsigned int BR_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_3_TL {
    struct {
        unsigned int TL_X                                                         : 16;
        unsigned int TL_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_4_BR {
    struct {
        unsigned int BR_X                                                         : 16;
        unsigned int BR_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_4_TL {
    struct {
        unsigned int TL_X                                                         : 16;
        unsigned int TL_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_5_BR {
    struct {
        unsigned int BR_X                                                         : 16;
        unsigned int BR_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_5_TL {
    struct {
        unsigned int TL_X                                                         : 16;
        unsigned int TL_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_6_BR {
    struct {
        unsigned int BR_X                                                         : 16;
        unsigned int BR_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_6_TL {
    struct {
        unsigned int TL_X                                                         : 16;
        unsigned int TL_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_7_BR {
    struct {
        unsigned int BR_X                                                         : 16;
        unsigned int BR_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_7_TL {
    struct {
        unsigned int TL_X                                                         : 16;
        unsigned int TL_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_8_BR {
    struct {
        unsigned int BR_X                                                         : 16;
        unsigned int BR_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_8_TL {
    struct {
        unsigned int TL_X                                                         : 16;
        unsigned int TL_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_9_BR {
    struct {
        unsigned int BR_X                                                         : 16;
        unsigned int BR_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_9_TL {
    struct {
        unsigned int TL_X                                                         : 16;
        unsigned int TL_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_10_BR {
    struct {
        unsigned int BR_X                                                         : 16;
        unsigned int BR_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_10_TL {
    struct {
        unsigned int TL_X                                                         : 16;
        unsigned int TL_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_11_BR {
    struct {
        unsigned int BR_X                                                         : 16;
        unsigned int BR_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_11_TL {
    struct {
        unsigned int TL_X                                                         : 16;
        unsigned int TL_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_12_BR {
    struct {
        unsigned int BR_X                                                         : 16;
        unsigned int BR_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_12_TL {
    struct {
        unsigned int TL_X                                                         : 16;
        unsigned int TL_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_13_BR {
    struct {
        unsigned int BR_X                                                         : 16;
        unsigned int BR_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_13_TL {
    struct {
        unsigned int TL_X                                                         : 16;
        unsigned int TL_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_14_BR {
    struct {
        unsigned int BR_X                                                         : 16;
        unsigned int BR_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_14_TL {
    struct {
        unsigned int TL_X                                                         : 16;
        unsigned int TL_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_15_BR {
    struct {
        unsigned int BR_X                                                         : 16;
        unsigned int BR_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_15_TL {
    struct {
        unsigned int TL_X                                                         : 16;
        unsigned int TL_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_0_BR {
    struct {
        unsigned int BR_X                                                         : 16;
        unsigned int BR_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_0_TL {
    struct {
        unsigned int TL_X                                                         : 16;
        unsigned int TL_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_1_BR {
    struct {
        unsigned int BR_X                                                         : 16;
        unsigned int BR_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_1_TL {
    struct {
        unsigned int TL_X                                                         : 16;
        unsigned int TL_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_2_BR {
    struct {
        unsigned int BR_X                                                         : 16;
        unsigned int BR_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_2_TL {
    struct {
        unsigned int TL_X                                                         : 16;
        unsigned int TL_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_3_BR {
    struct {
        unsigned int BR_X                                                         : 16;
        unsigned int BR_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_3_TL {
    struct {
        unsigned int TL_X                                                         : 16;
        unsigned int TL_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_4_BR {
    struct {
        unsigned int BR_X                                                         : 16;
        unsigned int BR_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_4_TL {
    struct {
        unsigned int TL_X                                                         : 16;
        unsigned int TL_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_5_BR {
    struct {
        unsigned int BR_X                                                         : 16;
        unsigned int BR_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_5_TL {
    struct {
        unsigned int TL_X                                                         : 16;
        unsigned int TL_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_6_BR {
    struct {
        unsigned int BR_X                                                         : 16;
        unsigned int BR_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_6_TL {
    struct {
        unsigned int TL_X                                                         : 16;
        unsigned int TL_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_7_BR {
    struct {
        unsigned int BR_X                                                         : 16;
        unsigned int BR_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_7_TL {
    struct {
        unsigned int TL_X                                                         : 16;
        unsigned int TL_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_8_BR {
    struct {
        unsigned int BR_X                                                         : 16;
        unsigned int BR_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_8_TL {
    struct {
        unsigned int TL_X                                                         : 16;
        unsigned int TL_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_9_BR {
    struct {
        unsigned int BR_X                                                         : 16;
        unsigned int BR_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_9_TL {
    struct {
        unsigned int TL_X                                                         : 16;
        unsigned int TL_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_10_BR {
    struct {
        unsigned int BR_X                                                         : 16;
        unsigned int BR_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_10_TL {
    struct {
        unsigned int TL_X                                                         : 16;
        unsigned int TL_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_11_BR {
    struct {
        unsigned int BR_X                                                         : 16;
        unsigned int BR_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_11_TL {
    struct {
        unsigned int TL_X                                                         : 16;
        unsigned int TL_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_12_BR {
    struct {
        unsigned int BR_X                                                         : 16;
        unsigned int BR_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_12_TL {
    struct {
        unsigned int TL_X                                                         : 16;
        unsigned int TL_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_13_BR {
    struct {
        unsigned int BR_X                                                         : 16;
        unsigned int BR_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_13_TL {
    struct {
        unsigned int TL_X                                                         : 16;
        unsigned int TL_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_14_BR {
    struct {
        unsigned int BR_X                                                         : 16;
        unsigned int BR_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_14_TL {
    struct {
        unsigned int TL_X                                                         : 16;
        unsigned int TL_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_15_BR {
    struct {
        unsigned int BR_X                                                         : 16;
        unsigned int BR_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_SCISSOR_15_TL {
    struct {
        unsigned int TL_X                                                         : 16;
        unsigned int TL_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMAX_0 {
    struct {
        unsigned int VPORT_ZMAX                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMAX_1 {
    struct {
        unsigned int VPORT_ZMAX                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMAX_2 {
    struct {
        unsigned int VPORT_ZMAX                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMAX_3 {
    struct {
        unsigned int VPORT_ZMAX                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMAX_4 {
    struct {
        unsigned int VPORT_ZMAX                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMAX_5 {
    struct {
        unsigned int VPORT_ZMAX                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMAX_6 {
    struct {
        unsigned int VPORT_ZMAX                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMAX_7 {
    struct {
        unsigned int VPORT_ZMAX                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMAX_8 {
    struct {
        unsigned int VPORT_ZMAX                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMAX_9 {
    struct {
        unsigned int VPORT_ZMAX                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMAX_10 {
    struct {
        unsigned int VPORT_ZMAX                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMAX_11 {
    struct {
        unsigned int VPORT_ZMAX                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMAX_12 {
    struct {
        unsigned int VPORT_ZMAX                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMAX_13 {
    struct {
        unsigned int VPORT_ZMAX                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMAX_14 {
    struct {
        unsigned int VPORT_ZMAX                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMAX_15 {
    struct {
        unsigned int VPORT_ZMAX                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMIN_0 {
    struct {
        unsigned int VPORT_ZMIN                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMIN_1 {
    struct {
        unsigned int VPORT_ZMIN                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMIN_2 {
    struct {
        unsigned int VPORT_ZMIN                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMIN_3 {
    struct {
        unsigned int VPORT_ZMIN                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMIN_4 {
    struct {
        unsigned int VPORT_ZMIN                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMIN_5 {
    struct {
        unsigned int VPORT_ZMIN                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMIN_6 {
    struct {
        unsigned int VPORT_ZMIN                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMIN_7 {
    struct {
        unsigned int VPORT_ZMIN                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMIN_8 {
    struct {
        unsigned int VPORT_ZMIN                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMIN_9 {
    struct {
        unsigned int VPORT_ZMIN                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMIN_10 {
    struct {
        unsigned int VPORT_ZMIN                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMIN_11 {
    struct {
        unsigned int VPORT_ZMIN                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMIN_12 {
    struct {
        unsigned int VPORT_ZMIN                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMIN_13 {
    struct {
        unsigned int VPORT_ZMIN                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMIN_14 {
    struct {
        unsigned int VPORT_ZMIN                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VPORT_ZMIN_15 {
    struct {
        unsigned int VPORT_ZMIN                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VRS_INFO {
    struct {
        unsigned int RATE_SW_MODE                                                 :  3;
        unsigned int FEEDBACK_SW_MODE                                             :  3;
        unsigned int                                                              : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VRS_OVERRIDE_CNTL {
    struct {
        unsigned int VRS_OVERRIDE_RATE_COMBINER_MODE                              :  3;
        unsigned int                                                              :  1;
        unsigned int VRS_RATE                                                     :  4;
        unsigned int                                                              :  4;
        unsigned int VRS_SURFACE_ENABLE                                           :  1;
        unsigned int RATE_HINT_WRITE_BACK_ENABLE                                  :  1;
        unsigned int VRS_FEEDBACK_RATE_OVERRIDE                                   :  1;
        unsigned int                                                              : 17;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VRS_RATE_BASE {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VRS_RATE_BASE_EXT {
    struct {
        unsigned int BASE_256B                                                    :  8;
        unsigned int                                                              : 20;
        unsigned int TB_SYNC_SIM_ID                                               :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VRS_RATE_FEEDBACK_BASE {
    struct {
        unsigned int BASE_256B                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VRS_RATE_FEEDBACK_BASE_EXT {
    struct {
        unsigned int BASE_256B                                                    :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VRS_RATE_FEEDBACK_SIZE_XY {
    struct {
        unsigned int X_MAX                                                        : 13;
        unsigned int                                                              :  3;
        unsigned int Y_MAX                                                        : 13;
        unsigned int                                                              :  3;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VRS_RATE_SIZE_XY {
    struct {
        unsigned int X_MAX                                                        : 13;
        unsigned int                                                              :  3;
        unsigned int Y_MAX                                                        : 13;
        unsigned int                                                              :  3;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VRS_SURFACE_CNTL {
    struct {
        unsigned int                                                              :  6;
        unsigned int VRC_CONTEXT_DONE_SYNC_DISABLE                                :  1;
        unsigned int VRS_FEEDBACK_RATE_OVERRIDE                                   :  1;
        unsigned int VRC_FLUSH_EVENT_MASK_DISABLE                                 :  5;
        unsigned int VRC_PREFETCH_DISABLE                                         :  1;
        unsigned int VRC_FLUSH_NO_INV_DISABLE                                     :  1;
        unsigned int VRC_NONSTALLING_FLUSH_DISABLE                                :  1;
        unsigned int VRC_PARTIAL_FLUSH_DISABLE                                    :  1;
        unsigned int                                                              :  1;
        unsigned int VRC_EOP_SYNC_DISABLE                                         :  1;
        unsigned int VRC_MAX_TAGS                                                 :  7;
        unsigned int VRC_EVICT_POINT                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_VRS_SURFACE_CNTL_1 {
    struct {
        unsigned int FORCE_SC_VRS_RATE_FINE                                       :  1;
        unsigned int FORCE_SC_VRS_RATE_FINE_SHADER_KILL_ENABLE                    :  1;
        unsigned int FORCE_SC_VRS_RATE_FINE_MASK_OPS_ENABLE                       :  1;
        unsigned int FORCE_SC_VRS_RATE_FINE_INTERFACE_RATE_16XAA                  :  1;
        unsigned int FORCE_SC_VRS_RATE_FINE_Z_OR_STENCIL                          :  1;
        unsigned int FORCE_SC_VRS_RATE_FINE_PRE_SHADER_DEPTH_COVERAGE_ENABLED     :  1;
        unsigned int FORCE_SC_VRS_RATE_FINE_POST_DEPTH_IMPORT                     :  1;
        unsigned int FORCE_SC_VRS_RATE_FINE_POPS                                  :  1;
        unsigned int FORCE_SC_VRS_RATE_FINE_INTERFACE_RATE_8XAA                   :  1;
        unsigned int FORCE_SC_VRS_RATE_FINE_INTRINSIC_RATE_GT_4XAA                :  1;
        unsigned int USE_ONLY_VRS_RATE_FINE_CFG                                   :  1;
        unsigned int                                                              :  1;
        unsigned int DISABLE_SSAA_VRS_RATE_NORMALIZATION                          :  1;
        unsigned int                                                              :  2;
        unsigned int DISABLE_PS_ITER_RATE_COMBINER_PASSTHRU_OVERRIDE              :  1;
        unsigned int                                                              :  3;
        unsigned int DISABLE_CMASK_RATE_HINT_FORCE_ZERO_OVERRIDE                  :  1;
        unsigned int DISABLE_SSAA_DETAIL_TO_EXPOSED_RATE_CLAMPING                 :  1;
        unsigned int VRS_ECO_SPARE_0                                              :  1;
        unsigned int VRS_ECO_SPARE_1                                              :  1;
        unsigned int VRS_ECO_SPARE_2                                              :  1;
        unsigned int VRS_ECO_SPARE_3                                              :  1;
        unsigned int VRS_ECO_SPARE_4                                              :  1;
        unsigned int VRS_ECO_SPARE_5                                              :  1;
        unsigned int VRS_ECO_SPARE_6                                              :  1;
        unsigned int VRS_ECO_SPARE_7                                              :  1;
        unsigned int VRS_ECO_SPARE_8                                              :  1;
        unsigned int VRS_ECO_SPARE_9                                              :  1;
        unsigned int VRS_ECO_SPARE_10                                             :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_WINDOW_OFFSET {
    struct {
        unsigned int WINDOW_X_OFFSET                                              : 16;
        unsigned int WINDOW_Y_OFFSET                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_WINDOW_SCISSOR_BR {
    struct {
        unsigned int BR_X                                                         : 16;
        unsigned int BR_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SC_WINDOW_SCISSOR_TL {
    struct {
        unsigned int TL_X                                                         : 16;
        unsigned int TL_Y                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_STATE_STEREO_X {
    struct {
        unsigned int STEREO_X_OFFSET                                              : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_STEREO_CNTL {
    struct {
        unsigned int                                                              :  1;
        unsigned int STEREO_MODE                                                  :  4;
        unsigned int RT_SLICE_MODE                                                :  3;
        unsigned int RT_SLICE_OFFSET                                              :  4;
        unsigned int                                                              :  4;
        unsigned int VP_ID_MODE                                                   :  3;
        unsigned int VP_ID_OFFSET                                                 :  4;
        unsigned int                                                              :  9;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_CNTL_STATUS {
    struct {
        unsigned int                                                              : 31;
        unsigned int SU_BUSY                                                      :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_HARDWARE_SCREEN_OFFSET {
    struct {
        unsigned int HW_SCREEN_OFFSET_X                                           : 12;
        unsigned int                                                              :  4;
        unsigned int HW_SCREEN_OFFSET_Y                                           : 12;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_LINE_CNTL {
    struct {
        unsigned int WIDTH                                                        : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_LINE_STIPPLE_CNTL {
    struct {
        unsigned int LINE_STIPPLE_RESET                                           :  2;
        unsigned int EXPAND_FULL_LENGTH                                           :  1;
        unsigned int FRACTIONAL_ACCUM                                             :  1;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_LINE_STIPPLE_SCALE {
    struct {
        unsigned int LINE_STIPPLE_SCALE                                           : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_LINE_STIPPLE_VALUE {
    struct {
        unsigned int LINE_STIPPLE_VALUE                                           : 24;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_OVER_RASTERIZATION_CNTL {
    struct {
        unsigned int DISCARD_0_AREA_TRIANGLES                                     :  1;
        unsigned int DISCARD_0_AREA_LINES                                         :  1;
        unsigned int DISCARD_0_AREA_POINTS                                        :  1;
        unsigned int DISCARD_0_AREA_RECTANGLES                                    :  1;
        unsigned int USE_PROVOKING_ZW                                             :  1;
        unsigned int                                                              : 27;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_PERFCOUNTER1_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_PERFCOUNTER2_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_PERFCOUNTER3_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_PERFCOUNTER3_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_PERFCOUNTER3_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_POINT_MINMAX {
    struct {
        unsigned int MIN_SIZE                                                     : 16;
        unsigned int MAX_SIZE                                                     : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_POINT_SIZE {
    struct {
        unsigned int HEIGHT                                                       : 16;
        unsigned int WIDTH                                                        : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_POLY_OFFSET_BACK_OFFSET {
    struct {
        unsigned int OFFSET                                                       : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_POLY_OFFSET_BACK_SCALE {
    struct {
        unsigned int SCALE                                                        : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_POLY_OFFSET_CLAMP {
    struct {
        unsigned int CLAMP                                                        : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_POLY_OFFSET_DB_FMT_CNTL {
    struct {
        unsigned int POLY_OFFSET_NEG_NUM_DB_BITS                                  :  8;
        unsigned int POLY_OFFSET_DB_IS_FLOAT_FMT                                  :  1;
        unsigned int                                                              : 23;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_POLY_OFFSET_FRONT_OFFSET {
    struct {
        unsigned int OFFSET                                                       : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_POLY_OFFSET_FRONT_SCALE {
    struct {
        unsigned int SCALE                                                        : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_PRIM_FILTER_CNTL {
    struct {
        unsigned int TRIANGLE_FILTER_DISABLE                                      :  1;
        unsigned int LINE_FILTER_DISABLE                                          :  1;
        unsigned int POINT_FILTER_DISABLE                                         :  1;
        unsigned int RECTANGLE_FILTER_DISABLE                                     :  1;
        unsigned int TRIANGLE_EXPAND_ENA                                          :  1;
        unsigned int LINE_EXPAND_ENA                                              :  1;
        unsigned int POINT_EXPAND_ENA                                             :  1;
        unsigned int RECTANGLE_EXPAND_ENA                                         :  1;
        unsigned int PRIM_EXPAND_CONSTANT                                         :  8;
        unsigned int                                                              : 14;
        unsigned int XMAX_RIGHT_EXCLUSION                                         :  1;
        unsigned int YMAX_BOTTOM_EXCLUSION                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_SC_MODE_CNTL {
    struct {
        unsigned int CULL_FRONT                                                   :  1;
        unsigned int CULL_BACK                                                    :  1;
        unsigned int FACE                                                         :  1;
        unsigned int POLY_MODE                                                    :  2;
        unsigned int POLYMODE_FRONT_PTYPE                                         :  3;
        unsigned int POLYMODE_BACK_PTYPE                                          :  3;
        unsigned int POLY_OFFSET_FRONT_ENABLE                                     :  1;
        unsigned int POLY_OFFSET_BACK_ENABLE                                      :  1;
        unsigned int POLY_OFFSET_PARA_ENABLE                                      :  1;
        unsigned int                                                              :  2;
        unsigned int VTX_WINDOW_OFFSET_ENABLE                                     :  1;
        unsigned int                                                              :  2;
        unsigned int PROVOKING_VTX_LAST                                           :  1;
        unsigned int PERSP_CORR_DIS                                               :  1;
        unsigned int MULTI_PRIM_IB_ENA                                            :  1;
        unsigned int RIGHT_TRIANGLE_ALTERNATE_GRADIENT_REF                        :  1;
        unsigned int NEW_QUAD_DECOMPOSITION                                       :  1;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_SMALL_PRIM_FILTER_CNTL {
    struct {
        unsigned int SMALL_PRIM_FILTER_ENABLE                                     :  1;
        unsigned int TRIANGLE_FILTER_DISABLE                                      :  1;
        unsigned int LINE_FILTER_DISABLE                                          :  1;
        unsigned int POINT_FILTER_DISABLE                                         :  1;
        unsigned int RECTANGLE_FILTER_DISABLE                                     :  1;
        unsigned int                                                              :  1;
        unsigned int SC_1XMSAA_COMPATIBLE_DISABLE                                 :  1;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PA_SU_VTX_CNTL {
    struct {
        unsigned int PIX_CENTER                                                   :  1;
        unsigned int ROUND_MODE                                                   :  2;
        unsigned int QUANT_MODE                                                   :  3;
        unsigned int                                                              : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PC_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PC_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PC_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PC_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PC_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PC_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PC_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PC_PERFCOUNTER1_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PC_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PC_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PC_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PC_PERFCOUNTER2_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PC_PERFCOUNTER3_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PC_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PC_PERFCOUNTER3_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PC_PERFCOUNTER3_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PerfMonCtl1 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int BankSel                                                      :  8;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PerfMonCtl2 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int BankSel                                                      :  8;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PerfMonCtl3 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int BankSel                                                      :  8;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PerfMonCtl4 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int BankSel                                                      :  8;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PerfMonCtl5 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int BankSel                                                      :  8;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PerfMonCtl6 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int BankSel                                                      :  8;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PerfMonCtl7 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int BankSel                                                      :  8;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PerfMonCtl8 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int BankSel                                                      :  8;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PerfMonCtl9 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int BankSel                                                      :  8;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PerfMonCtl10 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int BankSel                                                      :  8;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PerfMonCtl11 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int BankSel                                                      :  8;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PerfMonCtl12 {
    struct {
        unsigned int EventSelect                                                  :  8;
        unsigned int RdWrMask                                                     :  2;
        unsigned int PriorityMask                                                 :  4;
        unsigned int ReqSizeMask                                                  :  2;
        unsigned int BankSel                                                      :  8;
        unsigned int VCSel                                                        :  5;
        unsigned int SubChanMask                                                  :  2;
        unsigned int Enable                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PerfMonCtlClk {
    struct {
        unsigned int GlblResetMsk                                                 : 13;
        unsigned int                                                              :  9;
        unsigned int ClkGate                                                      :  1;
        unsigned int SpmEn                                                        :  1;
        unsigned int GlblReset                                                    :  1;
        unsigned int GlblMonEn                                                    :  1;
        unsigned int NumCounters                                                  :  4;
        unsigned int                                                              :  1;
        unsigned int CtrClkEn                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PerfMonCtr1_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              :  1;
        unsigned int ThreshCntEn                                                  :  2;
        unsigned int ThreshCnt                                                    : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PerfMonCtr1_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PerfMonCtr2_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              :  1;
        unsigned int ThreshCntEn                                                  :  2;
        unsigned int ThreshCnt                                                    : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PerfMonCtr2_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PerfMonCtr3_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              :  1;
        unsigned int ThreshCntEn                                                  :  2;
        unsigned int ThreshCnt                                                    : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PerfMonCtr3_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PerfMonCtr4_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              :  1;
        unsigned int ThreshCntEn                                                  :  2;
        unsigned int ThreshCnt                                                    : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PerfMonCtr4_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PerfMonCtr5_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              :  1;
        unsigned int ThreshCntEn                                                  :  2;
        unsigned int ThreshCnt                                                    : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PerfMonCtr5_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PerfMonCtr6_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              :  1;
        unsigned int ThreshCntEn                                                  :  2;
        unsigned int ThreshCnt                                                    : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PerfMonCtr6_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PerfMonCtr7_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              :  1;
        unsigned int ThreshCntEn                                                  :  2;
        unsigned int ThreshCnt                                                    : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PerfMonCtr7_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PerfMonCtr8_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              :  1;
        unsigned int ThreshCntEn                                                  :  2;
        unsigned int ThreshCnt                                                    : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PerfMonCtr8_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PerfMonCtr9_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              :  1;
        unsigned int ThreshCntEn                                                  :  2;
        unsigned int ThreshCnt                                                    : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PerfMonCtr9_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PerfMonCtr10_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              :  1;
        unsigned int ThreshCntEn                                                  :  2;
        unsigned int ThreshCnt                                                    : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PerfMonCtr10_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PerfMonCtr11_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              :  1;
        unsigned int ThreshCntEn                                                  :  2;
        unsigned int ThreshCnt                                                    : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PerfMonCtr11_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PerfMonCtr12_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              :  1;
        unsigned int ThreshCntEn                                                  :  2;
        unsigned int ThreshCnt                                                    : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PerfMonCtr12_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PerfMonCtrClk_Hi {
    struct {
        unsigned int Data                                                         : 16;
        unsigned int Overflow                                                     :  1;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union PerfMonCtrClk_Lo {
    struct {
        unsigned int Data                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERFCOUNTER_SELECT                                           :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERFCOUNTER_SELECT                                           :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_PERFMON_CNTL {
    struct {
        unsigned int PERFMON_STATE                                                :  3;
        unsigned int RESERVED_9_3                                                 :  7;
        unsigned int PERFMON_SAMPLE_ENABLE                                        :  1;
        unsigned int RESERVED                                                     : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_ACCUM_CTRL {
    struct {
        unsigned int StrobeResetPerfMonitors                                      :  1;
        unsigned int StrobeStartAccumulation                                      :  1;
        unsigned int StrobeRearmAccum                                             :  1;
        unsigned int StrobeResetSpmBlock                                          :  1;
        unsigned int StrobeStartSpm                                               :  4;
        unsigned int StrobeRearmSwaAccum                                          :  1;
        unsigned int StrobeStartSwa                                               :  1;
        unsigned int StrobePerfmonSampleWires                                     :  1;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_ACCUM_CTRLRAM_ADDR {
    struct {
        unsigned int ADDR                                                         : 11;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_ACCUM_CTRLRAM_ADDR_OFFSET {
    struct {
        unsigned int global_offset                                                :  8;
        unsigned int spmwithaccum_se_offset                                       :  8;
        unsigned int spmwithaccum_global_offset                                   :  8;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_ACCUM_CTRLRAM_DATA {
    struct {
        unsigned int DATA                                                         :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_ACCUM_DATARAM_32BITCNTRS_REGIONS {
    struct {
        unsigned int spp_addr_region                                              :  8;
        unsigned int swa_addr_region                                              :  8;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_ACCUM_DATARAM_ADDR {
    struct {
        unsigned int ADDR                                                         :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_ACCUM_DATARAM_DATA {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_ACCUM_DATARAM_WRCOUNT {
    struct {
        unsigned int DataRamWrCount                                               : 19;
        unsigned int                                                              : 13;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_ACCUM_MODE {
    struct {
        unsigned int EnableAccum                                                  :  1;
        unsigned int EnableSpmWithAccumMode                                       :  1;
        unsigned int EnableSPPMode                                                :  1;
        unsigned int AutoResetPerfmonDisable                                      :  1;
        unsigned int RESERVED_4                                                   :  1;
        unsigned int AutoAccumEn                                                  :  1;
        unsigned int SwaAutoAccumEn                                               :  1;
        unsigned int AutoSpmEn                                                    :  1;
        unsigned int SwaAutoSpmEn                                                 :  1;
        unsigned int Globals_LoadOverride                                         :  1;
        unsigned int Globals_SwaLoadOverride                                      :  1;
        unsigned int SE0_LoadOverride                                             :  1;
        unsigned int SE0_SwaLoadOverride                                          :  1;
        unsigned int SE1_LoadOverride                                             :  1;
        unsigned int SE1_SwaLoadOverride                                          :  1;
        unsigned int SE2_LoadOverride                                             :  1;
        unsigned int SE2_SwaLoadOverride                                          :  1;
        unsigned int SE3_LoadOverride                                             :  1;
        unsigned int SE3_SwaLoadOverride                                          :  1;
        unsigned int RESERVED_20_19                                               :  2;
        unsigned int RESERVED_22_21                                               :  2;
        unsigned int RESERVED_24_23                                               :  2;
        unsigned int RESERVED_26_25                                               :  2;
        unsigned int RESERVED_31_27                                               :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_ACCUM_SAMPLES_REQUESTED {
    struct {
        unsigned int SamplesRequested                                             :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_ACCUM_STATUS {
    struct {
        unsigned int NumbSamplesCompleted                                         :  8;
        unsigned int AccumDone                                                    :  1;
        unsigned int SpmDone                                                      :  1;
        unsigned int AccumOverflow                                                :  1;
        unsigned int AccumArmed                                                   :  1;
        unsigned int SequenceInProgress                                           :  1;
        unsigned int FinalSequenceInProgress                                      :  1;
        unsigned int AllFifosEmpty                                                :  1;
        unsigned int FSMIsIdle                                                    :  1;
        unsigned int SwaAccumDone                                                 :  1;
        unsigned int SwaSpmDone                                                   :  1;
        unsigned int SwaAccumOverflow                                             :  1;
        unsigned int SwaAccumArmed                                                :  1;
        unsigned int AllSegsDone                                                  :  1;
        unsigned int RearmSwaPending                                              :  1;
        unsigned int RearmSppPending                                              :  1;
        unsigned int MultiSampleAborted                                           :  1;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_ACCUM_SWA_DATARAM_ADDR {
    struct {
        unsigned int ADDR                                                         :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_ACCUM_SWA_DATARAM_DATA {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_ACCUM_THRESHOLD {
    struct {
        unsigned int Threshold                                                    : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_GFXCLOCK_HIGHCOUNT {
    struct {
        unsigned int GFXCLOCK_HIGHCOUNT                                           : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_GFXCLOCK_LOWCOUNT {
    struct {
        unsigned int GFXCLOCK_LOWCOUNT                                            : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_GLOBAL_BLK_EN_MASK_IND_ADDR {
    struct {
        unsigned int ADDR                                                         : 12;
        unsigned int                                                              : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_GLOBAL_BLK_EN_MASK_IND_DATA {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_GLOBAL_DELAY_IND_ADDR {
    struct {
        unsigned int ADDR                                                         : 12;
        unsigned int                                                              : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_GLOBAL_DELAY_IND_DATA {
    struct {
        unsigned int DATA                                                         :  6;
        unsigned int                                                              : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_GLOBAL_MUXSEL_ADDR {
    struct {
        unsigned int ADDR                                                         : 10;
        unsigned int                                                              : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_GLOBAL_MUXSEL_DATA {
    struct {
        unsigned int SEL0                                                         : 16;
        unsigned int SEL1                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_GLOBAL_USER_DATA_0 {
    struct {
        unsigned int LO                                                           : 16;
        unsigned int HI                                                           : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_GLOBAL_USER_DATA_1 {
    struct {
        unsigned int LO                                                           : 16;
        unsigned int HI                                                           : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_GLOBAL_USER_DATA_2 {
    struct {
        unsigned int LO                                                           : 16;
        unsigned int HI                                                           : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_GLOBAL_USER_DATA_3 {
    struct {
        unsigned int LO                                                           : 16;
        unsigned int HI                                                           : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_GTS_TRIGGER_VALUE_HI {
    struct {
        unsigned int VALUE_HI                                                     : 24;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_GTS_TRIGGER_VALUE_LO {
    struct {
        unsigned int VALUE_LO                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_INT_CNTL {
    struct {
        unsigned int RLC_SPM_INT_CNTL                                             :  1;
        unsigned int RESERVED                                                     : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_INT_INFO_1 {
    struct {
        unsigned int INTERRUPT_INFO_1                                             : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_INT_INFO_2 {
    struct {
        unsigned int INTERRUPT_INFO_2                                             : 16;
        unsigned int INTERRUPT_ID                                                 :  8;
        unsigned int RESERVED                                                     :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_INT_STATUS {
    struct {
        unsigned int RLC_SPM_INT_STATUS                                           :  1;
        unsigned int RESERVED                                                     : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_MC_CNTL {
    struct {
        unsigned int RLC_SPM_VMID                                                 :  4;
        unsigned int RLC_SPM_SDR                                                  :  2;
        unsigned int RLC_SPM_PERF_CNTR                                            :  1;
        unsigned int RLC_SPM_FED                                                  :  1;
        unsigned int RLC_SPM_TEMPORAL                                             :  3;
        unsigned int RLC_SPM_COMP                                                 :  2;
        unsigned int RLC_SPM_COMP_OVER                                            :  1;
        unsigned int RLC_SPM_RO                                                   :  1;
        unsigned int RLC_SPM_NOFILL                                               :  1;
        unsigned int RESERVED                                                     : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_MODE {
    struct {
        unsigned int MODE                                                         :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_PAUSE {
    struct {
        unsigned int PAUSE                                                        :  1;
        unsigned int PAUSED                                                       :  1;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_PERFMON_CNTL {
    struct {
        unsigned int RESERVED                                                     : 12;
        unsigned int PERFMON_RING_MODE                                            :  2;
        unsigned int PERFMON_SAMPLE_INTERVAL_START_MODE                           :  1;
        unsigned int PERFMON_SAMPLE_INTERVAL_TYPE                                 :  1;
        unsigned int PERFMON_SAMPLE_INTERVAL                                      : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_PERFMON_RING_BASE_HI {
    struct {
        unsigned int RING_BASE_HI                                                 : 16;
        unsigned int RESERVED                                                     : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_PERFMON_RING_BASE_LO {
    struct {
        unsigned int RING_BASE_LO                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_PERFMON_RING_SIZE {
    struct {
        unsigned int RING_BASE_SIZE                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_PERFMON_SEGMENT_SIZE {
    struct {
        unsigned int TOTAL_NUM_SEGMENT                                            : 16;
        unsigned int GLOBAL_NUM_SEGMENT                                           :  8;
        unsigned int SE_NUM_SEGMENT                                               :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_RING_RDPTR {
    struct {
        unsigned int PERFMON_RING_RDPTR                                           : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_RING_WRPTR {
    struct {
        unsigned int RESERVED                                                     :  5;
        unsigned int PERFMON_RING_WRPTR                                           : 27;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_RSPM_CMD {
    struct {
        unsigned int CMD                                                          :  4;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_RSPM_CMD_ACK {
    struct {
        unsigned int SE0_ACK                                                      :  1;
        unsigned int SE1_ACK                                                      :  1;
        unsigned int SE2_ACK                                                      :  1;
        unsigned int SE3_ACK                                                      :  1;
        unsigned int SE4_ACK                                                      :  1;
        unsigned int SE5_ACK                                                      :  1;
        unsigned int SE6_ACK                                                      :  1;
        unsigned int SE7_ACK                                                      :  1;
        unsigned int SPM_ACK                                                      :  1;
        unsigned int                                                              : 23;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_RSPM_REQ_DATA {
    struct {
        unsigned int DATA                                                         :  4;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_RSPM_REQ_OP {
    struct {
        unsigned int OP                                                           :  4;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_RSPM_RET_DATA {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_RSPM_RET_OP {
    struct {
        unsigned int OP                                                           :  4;
        unsigned int                                                              :  4;
        unsigned int VALID                                                        :  1;
        unsigned int                                                              : 23;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_SAMPLE_CNT {
    struct {
        unsigned int COUNT                                                        : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_SEGMENT_THRESHOLD {
    struct {
        unsigned int NUM_SEGMENT_THRESHOLD                                        :  8;
        unsigned int RESERVED                                                     : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_SE_BLK_EN_MASK_IND_ADDR {
    struct {
        unsigned int ADDR                                                         : 12;
        unsigned int                                                              : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_SE_BLK_EN_MASK_IND_DATA {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_SE_DELAY_IND_ADDR {
    struct {
        unsigned int ADDR                                                         : 12;
        unsigned int                                                              : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_SE_DELAY_IND_DATA {
    struct {
        unsigned int DATA                                                         :  6;
        unsigned int                                                              : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_SE_MUXSEL_ADDR {
    struct {
        unsigned int ADDR                                                         : 10;
        unsigned int                                                              : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_SE_MUXSEL_DATA {
    struct {
        unsigned int SEL0                                                         : 16;
        unsigned int SEL1                                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_SE_RSPM_REQ_DATA {
    struct {
        unsigned int DATA                                                         :  4;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_SE_RSPM_REQ_OP {
    struct {
        unsigned int OP                                                           :  4;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_SE_RSPM_RET_DATA {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_SE_RSPM_RET_OP {
    struct {
        unsigned int OP                                                           :  4;
        unsigned int                                                              :  4;
        unsigned int VALID                                                        :  1;
        unsigned int                                                              : 23;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_SE_USER_DATA_0 {
    struct {
        unsigned int LO                                                           : 16;
        unsigned int HI                                                           : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_SE_USER_DATA_1 {
    struct {
        unsigned int LO                                                           : 16;
        unsigned int HI                                                           : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_SE_USER_DATA_2 {
    struct {
        unsigned int LO                                                           : 16;
        unsigned int HI                                                           : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_SE_USER_DATA_3 {
    struct {
        unsigned int LO                                                           : 16;
        unsigned int HI                                                           : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_SPARE {
    struct {
        unsigned int SPARE                                                        : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_STATUS {
    struct {
        unsigned int CTL_BUSY                                                     :  1;
        unsigned int RSPM_REG_BUSY                                                :  1;
        unsigned int SPM_RSPM_BUSY                                                :  1;
        unsigned int SPM_RSPM_IO_BUSY                                             :  1;
        unsigned int SE_RSPM_IO_BUSY                                              :  8;
        unsigned int                                                              :  3;
        unsigned int ACCUM_BUSY                                                   :  1;
        unsigned int FSM_MASTER_STATE                                             :  4;
        unsigned int FSM_MEMORY_STATE                                             :  4;
        unsigned int CTL_REQ_STATE                                                :  2;
        unsigned int CTL_RET_STATE                                                :  1;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_THREAD_TRACE_CTRL {
    struct {
        unsigned int THREAD_TRACE_INT_EN                                          :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_UTCL1_CNTL {
    struct {
        unsigned int XNACK_REDO_TIMER_CNT                                         : 20;
        unsigned int RESERVED_23_20                                               :  4;
        unsigned int DROP_MODE                                                    :  1;
        unsigned int BYPASS                                                       :  1;
        unsigned int INVALIDATE                                                   :  1;
        unsigned int FRAG_LIMIT_MODE                                              :  1;
        unsigned int FORCE_SNOOP                                                  :  1;
        unsigned int RESERVED                                                     :  3;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_UTCL1_ERROR_1 {
    struct {
        unsigned int Translated_ReqError                                          :  2;
        unsigned int Translated_ReqErrorVmid                                      :  4;
        unsigned int Translated_ReqErrorAddr_MSB                                  :  4;
        unsigned int                                                              : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RLC_SPM_UTCL1_ERROR_2 {
    struct {
        unsigned int Translated_ReqErrorAddr_LSB                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 18;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_PERFCOUNTER2_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_PERFCOUNTER3_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_PERFCOUNTER3_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 18;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RMI_PERF_COUNTER_CNTL {
    struct {
        unsigned int TRANS_BASED_PERF_EN_SEL                                      :  2;
        unsigned int EVENT_BASED_PERF_EN_SEL                                      :  2;
        unsigned int TC_PERF_EN_SEL                                               :  2;
        unsigned int PERF_EVENT_WINDOW_MASK0                                      :  2;
        unsigned int PERF_EVENT_WINDOW_MASK1                                      :  2;
        unsigned int PERF_COUNTER_CID                                             :  4;
        unsigned int PERF_COUNTER_VMID                                            :  5;
        unsigned int PERF_COUNTER_BURST_LENGTH_THRESHOLD                          :  6;
        unsigned int PERF_SOFT_RESET                                              :  1;
        unsigned int PERF_CNTR_SPM_SEL                                            :  1;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RPB_PERFCOUNTER0_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RPB_PERFCOUNTER1_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RPB_PERFCOUNTER2_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RPB_PERFCOUNTER3_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RPB_PERFCOUNTER_HI {
    struct {
        unsigned int COUNTER_HI                                                   : 16;
        unsigned int COMPARE_VALUE                                                : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RPB_PERFCOUNTER_LO {
    struct {
        unsigned int COUNTER_LO                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RPB_PERFCOUNTER_RSLT_CNTL {
    struct {
        unsigned int PERF_COUNTER_SELECT                                          :  4;
        unsigned int                                                              :  4;
        unsigned int START_TRIGGER                                                :  8;
        unsigned int STOP_TRIGGER                                                 :  8;
        unsigned int ENABLE_ANY                                                   :  1;
        unsigned int CLEAR_ALL                                                    :  1;
        unsigned int STOP_ALL_ON_SATURATE                                         :  1;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union RPB_PERF_COUNTER_CNTL {
    struct {
        unsigned int PERF_COUNTER_SELECT                                          :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SCRATCH_REG0 {
    struct {
        unsigned int SCRATCH_REG0                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SCRATCH_REG1 {
    struct {
        unsigned int SCRATCH_REG1                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SCRATCH_REG2 {
    struct {
        unsigned int SCRATCH_REG2                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SCRATCH_REG3 {
    struct {
        unsigned int SCRATCH_REG3                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SCRATCH_REG4 {
    struct {
        unsigned int SCRATCH_REG4                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SCRATCH_REG5 {
    struct {
        unsigned int SCRATCH_REG5                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SCRATCH_REG6 {
    struct {
        unsigned int SCRATCH_REG6                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SCRATCH_REG7 {
    struct {
        unsigned int SCRATCH_REG7                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SCRATCH_REGISTER0 {
    struct {
        unsigned int ScratchPad0                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SCRATCH_REGISTER1 {
    struct {
        unsigned int ScratchPad1                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SCRATCH_REGISTER2 {
    struct {
        unsigned int ScratchPad2                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SCRATCH_REGISTER3 {
    struct {
        unsigned int ScratchPad3                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SCRATCH_REGISTER4 {
    struct {
        unsigned int ScratchPad4                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SCRATCH_REGISTER5 {
    struct {
        unsigned int ScratchPad5                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SCRATCH_REGISTER6 {
    struct {
        unsigned int ScratchPad6                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SCRATCH_REGISTER7 {
    struct {
        unsigned int ScratchPad7                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SCRATCH_REG_ATOMIC {
    struct {
        unsigned int IMMED                                                        : 24;
        unsigned int ID                                                           :  3;
        unsigned int reserved27                                                   :  1;
        unsigned int OP                                                           :  3;
        unsigned int reserved31                                                   :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SCRATCH_REG_CMPSWAP_ATOMIC {
    struct {
        unsigned int IMMED_COMPARE                                                : 12;
        unsigned int IMMED_REPLACE                                                : 12;
        unsigned int ID                                                           :  3;
        unsigned int reserved27                                                   :  1;
        unsigned int OP                                                           :  3;
        unsigned int reserved31                                                   :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SC_MEM_SCOPE {
    struct {
        unsigned int VRS_RATE_SCOPE                                               :  2;
        unsigned int VRS_FEEDBACK_SCOPE                                           :  2;
        unsigned int HIZ_SCOPE                                                    :  2;
        unsigned int HIS_SCOPE                                                    :  2;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SC_MEM_SPEC_READ {
    struct {
        unsigned int VRS_SPECULATIVE_READ                                         :  2;
        unsigned int HIZ_SPECULATIVE_READ                                         :  2;
        unsigned int HIS_SPECULATIVE_READ                                         :  2;
        unsigned int                                                              : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SC_MEM_TEMPORAL {
    struct {
        unsigned int VRS_TEMPORAL_READ                                            :  3;
        unsigned int VRS_TEMPORAL_WRITE                                           :  3;
        unsigned int HIZ_TEMPORAL_READ                                            :  3;
        unsigned int HIZ_TEMPORAL_WRITE                                           :  3;
        unsigned int HIS_TEMPORAL_READ                                            :  3;
        unsigned int HIS_TEMPORAL_WRITE                                           :  3;
        unsigned int                                                              : 14;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA0_PERFCNT_MISC_CNTL {
    struct {
        unsigned int CMD_OP                                                       : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA0_PERFCNT_PERFCOUNTER0_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA0_PERFCNT_PERFCOUNTER1_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA0_PERFCNT_PERFCOUNTER_HI {
    struct {
        unsigned int COUNTER_HI                                                   : 16;
        unsigned int COMPARE_VALUE                                                : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA0_PERFCNT_PERFCOUNTER_LO {
    struct {
        unsigned int COUNTER_LO                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA0_PERFCNT_PERFCOUNTER_RSLT_CNTL {
    struct {
        unsigned int PERF_COUNTER_SELECT                                          :  4;
        unsigned int                                                              :  4;
        unsigned int START_TRIGGER                                                :  8;
        unsigned int STOP_TRIGGER                                                 :  8;
        unsigned int ENABLE_ANY                                                   :  1;
        unsigned int CLEAR_ALL                                                    :  1;
        unsigned int STOP_ALL_ON_SATURATE                                         :  1;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA0_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA0_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA0_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA0_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA0_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA0_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA0_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA0_PERFCOUNTER1_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA1_PERFCNT_MISC_CNTL {
    struct {
        unsigned int CMD_OP                                                       : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA1_PERFCNT_PERFCOUNTER0_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA1_PERFCNT_PERFCOUNTER1_CFG {
    struct {
        unsigned int PERF_SEL                                                     :  8;
        unsigned int PERF_SEL_END                                                 :  8;
        unsigned int                                                              :  8;
        unsigned int PERF_MODE                                                    :  4;
        unsigned int ENABLE                                                       :  1;
        unsigned int CLEAR                                                        :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA1_PERFCNT_PERFCOUNTER_HI {
    struct {
        unsigned int COUNTER_HI                                                   : 16;
        unsigned int COMPARE_VALUE                                                : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA1_PERFCNT_PERFCOUNTER_LO {
    struct {
        unsigned int COUNTER_LO                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA1_PERFCNT_PERFCOUNTER_RSLT_CNTL {
    struct {
        unsigned int PERF_COUNTER_SELECT                                          :  4;
        unsigned int                                                              :  4;
        unsigned int START_TRIGGER                                                :  8;
        unsigned int STOP_TRIGGER                                                 :  8;
        unsigned int ENABLE_ANY                                                   :  1;
        unsigned int CLEAR_ALL                                                    :  1;
        unsigned int STOP_ALL_ON_SATURATE                                         :  1;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA1_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA1_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA1_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA1_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA1_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA1_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA1_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SDMA1_PERFCOUNTER1_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_ARB_CNTL_0 {
    struct {
        unsigned int EXP_ARB_COL_WT                                               :  4;
        unsigned int EXP_ARB_POS_WT                                               :  4;
        unsigned int EXP_ARB_GDS_WT                                               :  4;
        unsigned int                                                              : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_ARB_CYCLES_0 {
    struct {
        unsigned int TS0_DURATION                                                 : 16;
        unsigned int TS1_DURATION                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_ARB_CYCLES_1 {
    struct {
        unsigned int TS2_DURATION                                                 : 16;
        unsigned int TS3_DURATION                                                 : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_ARB_PRIORITY {
    struct {
        unsigned int PIPE_ORDER_TS0                                               :  3;
        unsigned int PIPE_ORDER_TS1                                               :  3;
        unsigned int PIPE_ORDER_TS2                                               :  3;
        unsigned int PIPE_ORDER_TS3                                               :  3;
        unsigned int TS0_DUR_MULT                                                 :  2;
        unsigned int TS1_DUR_MULT                                                 :  2;
        unsigned int TS2_DUR_MULT                                                 :  2;
        unsigned int TS3_DUR_MULT                                                 :  2;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_ATTRIBUTE_RING_BASE {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_ATTRIBUTE_RING_SIZE {
    struct {
        unsigned int MEM_SIZE                                                     :  8;
        unsigned int                                                              :  8;
        unsigned int BIG_PAGE                                                     :  1;
        unsigned int L1_POLICY                                                    :  2;
        unsigned int L2_POLICY                                                    :  2;
        unsigned int LLC_NOALLOC                                                  :  1;
        unsigned int GL1_PERF_COUNTER_DISABLE                                     :  1;
        unsigned int                                                              :  9;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_BARYC_CNTL {
    struct {
        unsigned int PERSP_CENTER_CNTL                                            :  1;
        unsigned int                                                              :  3;
        unsigned int PERSP_CENTROID_CNTL                                          :  1;
        unsigned int                                                              :  3;
        unsigned int LINEAR_CENTER_CNTL                                           :  1;
        unsigned int                                                              :  3;
        unsigned int LINEAR_CENTROID_CNTL                                         :  1;
        unsigned int                                                              :  3;
        unsigned int POS_FLOAT_LOCATION                                           :  2;
        unsigned int                                                              :  2;
        unsigned int POS_FLOAT_ULC                                                :  1;
        unsigned int                                                              :  3;
        unsigned int FRONT_FACE_ALL_BITS                                          :  1;
        unsigned int                                                              :  7;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_BARYC_SSAA_CNTL {
    struct {
        unsigned int CENTER_SSAA_MODE                                             :  1;
        unsigned int CENTROID_SSAA_MODE                                           :  1;
        unsigned int COVERED_CENTROID_IS_CENTER                                   :  1;
        unsigned int                                                              : 29;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_COMPUTE_QUEUE_RESET {
    struct {
        unsigned int RESET                                                        :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_COMPUTE_WF_CTX_SAVE {
    struct {
        unsigned int INITIATE                                                     :  1;
        unsigned int                                                              :  1;
        unsigned int DONE_INTERRUPT_EN                                            :  1;
        unsigned int                                                              : 28;
        unsigned int SAVE_BUSY                                                    :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_COMPUTE_WF_CTX_SAVE_STATUS {
    struct {
        unsigned int PIPE0_QUEUE0_SAVE_BUSY                                       :  1;
        unsigned int PIPE0_QUEUE1_SAVE_BUSY                                       :  1;
        unsigned int PIPE0_QUEUE2_SAVE_BUSY                                       :  1;
        unsigned int PIPE0_QUEUE3_SAVE_BUSY                                       :  1;
        unsigned int PIPE0_QUEUE4_SAVE_BUSY                                       :  1;
        unsigned int PIPE0_QUEUE5_SAVE_BUSY                                       :  1;
        unsigned int PIPE0_QUEUE6_SAVE_BUSY                                       :  1;
        unsigned int PIPE0_QUEUE7_SAVE_BUSY                                       :  1;
        unsigned int PIPE1_QUEUE0_SAVE_BUSY                                       :  1;
        unsigned int PIPE1_QUEUE1_SAVE_BUSY                                       :  1;
        unsigned int PIPE1_QUEUE2_SAVE_BUSY                                       :  1;
        unsigned int PIPE1_QUEUE3_SAVE_BUSY                                       :  1;
        unsigned int PIPE1_QUEUE4_SAVE_BUSY                                       :  1;
        unsigned int PIPE1_QUEUE5_SAVE_BUSY                                       :  1;
        unsigned int PIPE1_QUEUE6_SAVE_BUSY                                       :  1;
        unsigned int PIPE1_QUEUE7_SAVE_BUSY                                       :  1;
        unsigned int PIPE2_QUEUE0_SAVE_BUSY                                       :  1;
        unsigned int PIPE2_QUEUE1_SAVE_BUSY                                       :  1;
        unsigned int PIPE2_QUEUE2_SAVE_BUSY                                       :  1;
        unsigned int PIPE2_QUEUE3_SAVE_BUSY                                       :  1;
        unsigned int PIPE2_QUEUE4_SAVE_BUSY                                       :  1;
        unsigned int PIPE2_QUEUE5_SAVE_BUSY                                       :  1;
        unsigned int PIPE2_QUEUE6_SAVE_BUSY                                       :  1;
        unsigned int PIPE2_QUEUE7_SAVE_BUSY                                       :  1;
        unsigned int PIPE3_QUEUE0_SAVE_BUSY                                       :  1;
        unsigned int PIPE3_QUEUE1_SAVE_BUSY                                       :  1;
        unsigned int PIPE3_QUEUE2_SAVE_BUSY                                       :  1;
        unsigned int PIPE3_QUEUE3_SAVE_BUSY                                       :  1;
        unsigned int PIPE3_QUEUE4_SAVE_BUSY                                       :  1;
        unsigned int PIPE3_QUEUE5_SAVE_BUSY                                       :  1;
        unsigned int PIPE3_QUEUE6_SAVE_BUSY                                       :  1;
        unsigned int PIPE3_QUEUE7_SAVE_BUSY                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_COMPUTE_WGS_CONTROL {
    struct {
        unsigned int WGS_ENABLE                                                   :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_CONFIG_CNTL {
    struct {
        unsigned int GPR_WRITE_PRIORITY                                           : 21;
        unsigned int EXP_PRIORITY_ORDER                                           :  3;
        unsigned int ENABLE_SQG_TOP_EVENTS                                        :  1;
        unsigned int ENABLE_SQG_BOP_EVENTS                                        :  1;
        unsigned int                                                              :  2;
        unsigned int ALLOC_ARB_LRU_ENA                                            :  1;
        unsigned int EXP_ARB_LRU_ENA                                              :  1;
        unsigned int PS_PKR_PRIORITY_CNTL                                         :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_CONFIG_CNTL_1 {
    struct {
        unsigned int VTX_DONE_DELAY                                               :  4;
        unsigned int INTERP_ONE_PRIM_PER_ROW                                      :  1;
        unsigned int PC_LIMIT_ENABLE                                              :  2;
        unsigned int PC_LIMIT_STRICT                                              :  1;
        unsigned int PS_GROUP_TIMEOUT_MODE                                        :  1;
        unsigned int OREO_EXPALLOC_STALL                                          :  1;
        unsigned int LBPW_CU_CHK_CNT                                              :  4;
        unsigned int CSC_PWR_SAVE_DISABLE                                         :  1;
        unsigned int CSG_PWR_SAVE_DISABLE                                         :  1;
        unsigned int MAX_VTX_SYNC_CNT                                             :  5;
        unsigned int EN_USER_ACCUM                                                :  1;
        unsigned int SA_SCREEN_MAP                                                :  1;
        unsigned int PS_GROUP_TIMEOUT                                             :  9;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_CONFIG_CNTL_2 {
    struct {
        unsigned int CONTEXT_SAVE_WAIT_GDS_REQUEST_CYCLE_OVHD                     :  4;
        unsigned int CONTEXT_SAVE_WAIT_GDS_GRANT_CYCLE_OVHD                       :  4;
        unsigned int PWS_CSG_WAIT_DISABLE                                         :  1;
        unsigned int PWS_HS_WAIT_DISABLE                                          :  1;
        unsigned int PWS_GS_WAIT_DISABLE                                          :  1;
        unsigned int PWS_PS_WAIT_DISABLE                                          :  1;
        unsigned int CSC_HALT_ACK_DELAY                                           :  5;
        unsigned int SPP_TIMEOUT_CTR                                              :  4;
        unsigned int PC_CONTEXT_DONE_SYNC_ENABLE                                  :  1;
        unsigned int                                                              : 10;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_CONFIG_CU_MASK_CS0 {
    struct {
        unsigned int CU_EN_SA0                                                    : 16;
        unsigned int CU_EN_SA1                                                    : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_CONFIG_CU_MASK_CS1 {
    struct {
        unsigned int CU_EN_SA0                                                    : 16;
        unsigned int CU_EN_SA1                                                    : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_CONFIG_CU_MASK_CS2 {
    struct {
        unsigned int CU_EN_SA0                                                    : 16;
        unsigned int CU_EN_SA1                                                    : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_CONFIG_CU_MASK_CS3 {
    struct {
        unsigned int CU_EN_SA0                                                    : 16;
        unsigned int CU_EN_SA1                                                    : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_CONFIG_CU_MASK_CS4 {
    struct {
        unsigned int CU_EN_SA0                                                    : 16;
        unsigned int CU_EN_SA1                                                    : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_CONFIG_CU_MASK_CS5 {
    struct {
        unsigned int CU_EN_SA0                                                    : 16;
        unsigned int CU_EN_SA1                                                    : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_CONFIG_CU_MASK_CS6 {
    struct {
        unsigned int CU_EN_SA0                                                    : 16;
        unsigned int CU_EN_SA1                                                    : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_CONFIG_CU_MASK_CS7 {
    struct {
        unsigned int CU_EN_SA0                                                    : 16;
        unsigned int CU_EN_SA1                                                    : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_CONFIG_CU_MASK_GFX0 {
    struct {
        unsigned int HS_CU_EN                                                     : 16;
        unsigned int GS_CU_EN                                                     : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_CONFIG_CU_MASK_GFX1 {
    struct {
        unsigned int PS_CU_EN                                                     : 16;
        unsigned int CSG_CU_EN                                                    : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_CONFIG_CU_MASK_HP3D0 {
    struct {
        unsigned int HS_CU_EN                                                     : 16;
        unsigned int GS_CU_EN                                                     : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_CONFIG_CU_MASK_HP3D1 {
    struct {
        unsigned int PS_CU_EN                                                     : 16;
        unsigned int CSG_CU_EN                                                    : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_CONFIG_PS_CU_EN {
    struct {
        unsigned int PKR_OFFSET                                                   :  4;
        unsigned int PKR2_OFFSET                                                  :  4;
        unsigned int PKR3_OFFSET                                                  :  4;
        unsigned int                                                              : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_CREST_MODE {
    struct {
        unsigned int ENABLE_CREST                                                 :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_CSQ_WF_ACTIVE_COUNT_0 {
    struct {
        unsigned int COUNT                                                        : 11;
        unsigned int                                                              :  5;
        unsigned int EVENTS                                                       : 11;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_CSQ_WF_ACTIVE_COUNT_1 {
    struct {
        unsigned int COUNT                                                        : 11;
        unsigned int                                                              :  5;
        unsigned int EVENTS                                                       : 11;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_CSQ_WF_ACTIVE_COUNT_2 {
    struct {
        unsigned int COUNT                                                        : 11;
        unsigned int                                                              :  5;
        unsigned int EVENTS                                                       : 11;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_CSQ_WF_ACTIVE_COUNT_3 {
    struct {
        unsigned int COUNT                                                        : 11;
        unsigned int                                                              :  5;
        unsigned int EVENTS                                                       : 11;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_CSQ_WF_ACTIVE_STATUS {
    struct {
        unsigned int ACTIVE                                                       : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_CS_CRAWLER_CONFIG {
    struct {
        unsigned int CSG_DEPTH                                                    :  6;
        unsigned int CSC_DEPTH                                                    :  6;
        unsigned int                                                              : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_DSM_CNTL {
    struct {
        unsigned int SPI_SR_MEM_DSM_IRRITATOR_DATA                                :  2;
        unsigned int SPI_SR_MEM_ENABLE_SINGLE_WRITE                               :  1;
        unsigned int                                                              : 29;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_DSM_CNTL2 {
    struct {
        unsigned int SPI_SR_MEM_ENABLE_ERROR_INJECT                               :  2;
        unsigned int SPI_SR_MEM_SELECT_INJECT_DELAY                               :  1;
        unsigned int SPI_SR_MEM_INJECT_DELAY                                      :  6;
        unsigned int                                                              : 23;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_EDC_CNT {
    struct {
        unsigned int SPI_SR_MEM_SED_COUNT                                         :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_EXP_THROTTLE_CTRL {
    struct {
        unsigned int ENABLE                                                       :  1;
        unsigned int PERIOD                                                       :  4;
        unsigned int UPSTEP                                                       :  4;
        unsigned int DOWNSTEP                                                     :  4;
        unsigned int LOW_STALL_MON_HIST_COUNT                                     :  3;
        unsigned int HIGH_STALL_MON_HIST_COUNT                                    :  3;
        unsigned int EXP_STALL_THRESHOLD                                          :  7;
        unsigned int SKEW_COUNT                                                   :  3;
        unsigned int THROTTLE_RESET                                               :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_FEATURE_CTRL {
    struct {
        unsigned int TUNNELING_WAVE_LIMIT                                         :  4;
        unsigned int RA_PROBE_IGNORE                                              :  1;
        unsigned int PS_THROTTLE_MAX_WAVE_LIMIT                                   :  6;
        unsigned int RA_PROBE_SKEW_WIF_CTRL                                       :  2;
        unsigned int RA_PROBE_SKEW_OOO_CTRL                                       :  1;
        unsigned int RA_PROBE_SKEW_DISABLE                                        :  1;
        unsigned int                                                              : 17;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_GDS_CREDITS {
    struct {
        unsigned int DS_DATA_CREDITS                                              :  8;
        unsigned int DS_CMD_CREDITS                                               :  8;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_GFX_CNTL {
    struct {
        unsigned int RESET_COUNTS                                                 :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_GFX_CRAWLER_CONFIG {
    struct {
        unsigned int PS_DEPTH                                                     :  5;
        unsigned int GS_DEPTH                                                     :  6;
        unsigned int HS_DEPTH                                                     :  6;
        unsigned int PS_ALLOC_DEPTH                                               :  5;
        unsigned int PS_LDS_DONE_DEPTH                                            :  3;
        unsigned int PS_LDS_DONE_CNTL                                             :  1;
        unsigned int RA_PSWAVE_CREDITS                                            :  3;
        unsigned int                                                              :  3;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_GFX_SCRATCH_BASE_HI {
    struct {
        unsigned int DATA                                                         :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_GFX_SCRATCH_BASE_LO {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_GRP_LAUNCH_GUARANTEE_CTRL {
    struct {
        unsigned int NUM_MRT_THRESHOLD                                            :  3;
        unsigned int GFX_PENDING_THRESHOLD                                        :  3;
        unsigned int PRIORITY_LOST_THRESHOLD                                      :  4;
        unsigned int ALLOC_SUCCESS_THRESHOLD                                      :  4;
        unsigned int GFX_WAVE_THRESHOLD_HIGH                                      :  5;
        unsigned int CS_WAVE_THRESHOLD_HIGH                                       :  5;
        unsigned int CU_MASK_ROTATE_PERIODS                                       :  2;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_GRP_LAUNCH_GUARANTEE_ENABLE {
    struct {
        unsigned int ENABLE                                                       :  1;
        unsigned int HS_ASSIST_EN                                                 :  1;
        unsigned int GS_ASSIST_EN                                                 :  1;
        unsigned int MRT_ASSIST_EN                                                :  1;
        unsigned int CS_GLG_DISABLE                                               :  1;
        unsigned int GFX_NUM_LOCK_WGP                                             :  3;
        unsigned int CS_NUM_LOCK_WGP                                              :  3;
        unsigned int LOCK_PERIOD                                                  :  4;
        unsigned int LOCK_MAINT_COUNT                                             :  3;
        unsigned int                                                              : 14;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_GS_THROTTLE_CNTL1 {
    struct {
        unsigned int PH_POLL_INTERVAL                                             :  4;
        unsigned int PH_THROTTLE_BASE                                             :  4;
        unsigned int PH_THROTTLE_STEP_SIZE                                        :  4;
        unsigned int SPI_VGPR_THRESHOLD                                           :  4;
        unsigned int SPI_LDS_THRESHOLD                                            :  4;
        unsigned int SPI_POLL_INTERVAL                                            :  4;
        unsigned int SPI_THROTTLE_BASE                                            :  4;
        unsigned int SPI_THROTTLE_STEP_SIZE                                       :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_GS_THROTTLE_CNTL2 {
    struct {
        unsigned int SPI_THROTTLE_MODE                                            :  2;
        unsigned int GRP_LIFETIME_THRESHOLD                                       :  4;
        unsigned int GRP_LIFETIME_THRESHOLD_FACTOR                                :  2;
        unsigned int GRP_LIFETIME_PENALTY1                                        :  3;
        unsigned int GRP_LIFETIME_PENALTY2                                        :  3;
        unsigned int PS_STALL_THRESHOLD                                           :  2;
        unsigned int PH_MODE                                                      :  1;
        unsigned int RESERVED                                                     : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_INTERP_CONTROL_0 {
    struct {
        unsigned int FLAT_SHADE_ENA                                               :  1;
        unsigned int PNT_SPRITE_ENA                                               :  1;
        unsigned int PNT_SPRITE_OVRD_X                                            :  3;
        unsigned int PNT_SPRITE_OVRD_Y                                            :  3;
        unsigned int PNT_SPRITE_OVRD_Z                                            :  3;
        unsigned int PNT_SPRITE_OVRD_W                                            :  3;
        unsigned int PNT_SPRITE_TOP_1                                             :  1;
        unsigned int                                                              : 17;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_LB_CTR_CTRL {
    struct {
        unsigned int LOAD                                                         :  1;
        unsigned int WAVES_SELECT                                                 :  2;
        unsigned int CLEAR_ON_READ                                                :  1;
        unsigned int RESET_COUNTS                                                 :  1;
        unsigned int                                                              : 27;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_LB_DATA_PERWGP_WAVE_CS {
    struct {
        unsigned int ACTIVE                                                       : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_LB_DATA_PERWGP_WAVE_HSGS {
    struct {
        unsigned int WGP_USED_HS                                                  : 16;
        unsigned int WGP_USED_GS                                                  : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_LB_DATA_PERWGP_WAVE_PS {
    struct {
        unsigned int WGP_USED_PS                                                  : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_LB_DATA_REG {
    struct {
        unsigned int CNT_DATA                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_LB_DATA_WAVES {
    struct {
        unsigned int COUNT0                                                       : 16;
        unsigned int COUNT1                                                       : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_LB_WGP_MASK {
    struct {
        unsigned int WGP_MASK                                                     : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_P0_TRAP_SCREEN_GPR_MIN {
    struct {
        unsigned int VGPR_MIN                                                     :  6;
        unsigned int SGPR_MIN                                                     :  4;
        unsigned int                                                              : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_P0_TRAP_SCREEN_PSBA_HI {
    struct {
        unsigned int MEM_BASE                                                     :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_P0_TRAP_SCREEN_PSBA_LO {
    struct {
        unsigned int MEM_BASE                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_P0_TRAP_SCREEN_PSMA_HI {
    struct {
        unsigned int MEM_BASE                                                     :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_P0_TRAP_SCREEN_PSMA_LO {
    struct {
        unsigned int MEM_BASE                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_P1_TRAP_SCREEN_GPR_MIN {
    struct {
        unsigned int VGPR_MIN                                                     :  6;
        unsigned int SGPR_MIN                                                     :  4;
        unsigned int                                                              : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_P1_TRAP_SCREEN_PSBA_HI {
    struct {
        unsigned int MEM_BASE                                                     :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_P1_TRAP_SCREEN_PSBA_LO {
    struct {
        unsigned int MEM_BASE                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_P1_TRAP_SCREEN_PSMA_HI {
    struct {
        unsigned int MEM_BASE                                                     :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_P1_TRAP_SCREEN_PSMA_LO {
    struct {
        unsigned int MEM_BASE                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER1_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER2_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER3_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER3_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER3_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER4_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER4_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER4_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER4_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER5_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER5_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER5_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER5_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PERFCOUNTER_BINS {
    struct {
        unsigned int BIN0_MIN                                                     :  4;
        unsigned int BIN0_MAX                                                     :  4;
        unsigned int BIN1_MIN                                                     :  4;
        unsigned int BIN1_MAX                                                     :  4;
        unsigned int BIN2_MIN                                                     :  4;
        unsigned int BIN2_MAX                                                     :  4;
        unsigned int BIN3_MIN                                                     :  4;
        unsigned int BIN3_MAX                                                     :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PG_ENABLE_STATIC_WGP_MASK {
    struct {
        unsigned int WGP_MASK                                                     : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PQEV_CTRL {
    struct {
        unsigned int SCAN_PERIOD                                                  : 10;
        unsigned int QUEUE_DURATION                                               :  6;
        unsigned int COMPUTE_PIPE_EN                                              :  8;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_ADDR {
    struct {
        unsigned int PERSP_SAMPLE_ENA                                             :  1;
        unsigned int PERSP_CENTER_ENA                                             :  1;
        unsigned int PERSP_CENTROID_ENA                                           :  1;
        unsigned int PERSP_PULL_MODEL_ENA                                         :  1;
        unsigned int LINEAR_SAMPLE_ENA                                            :  1;
        unsigned int LINEAR_CENTER_ENA                                            :  1;
        unsigned int LINEAR_CENTROID_ENA                                          :  1;
        unsigned int LINE_STIPPLE_TEX_ENA                                         :  1;
        unsigned int POS_X_FLOAT_ENA                                              :  1;
        unsigned int POS_Y_FLOAT_ENA                                              :  1;
        unsigned int POS_Z_FLOAT_ENA                                              :  1;
        unsigned int POS_W_FLOAT_ENA                                              :  1;
        unsigned int FRONT_FACE_ENA                                               :  1;
        unsigned int ANCILLARY_ENA                                                :  1;
        unsigned int SAMPLE_COVERAGE_ENA                                          :  1;
        unsigned int POS_FIXED_PT_ENA                                             :  1;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_0 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int ROTATE_PC_PTR                                                :  1;
        unsigned int PRIM_ATTR                                                    :  1;
        unsigned int                                                              :  4;
        unsigned int PT_SPRITE_TEX                                                :  1;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int PT_SPRITE_TEX_ATTR1                                          :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_1 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int ROTATE_PC_PTR                                                :  1;
        unsigned int PRIM_ATTR                                                    :  1;
        unsigned int                                                              :  4;
        unsigned int PT_SPRITE_TEX                                                :  1;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int PT_SPRITE_TEX_ATTR1                                          :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_2 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int ROTATE_PC_PTR                                                :  1;
        unsigned int PRIM_ATTR                                                    :  1;
        unsigned int                                                              :  4;
        unsigned int PT_SPRITE_TEX                                                :  1;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int PT_SPRITE_TEX_ATTR1                                          :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_3 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int ROTATE_PC_PTR                                                :  1;
        unsigned int PRIM_ATTR                                                    :  1;
        unsigned int                                                              :  4;
        unsigned int PT_SPRITE_TEX                                                :  1;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int PT_SPRITE_TEX_ATTR1                                          :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_4 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int ROTATE_PC_PTR                                                :  1;
        unsigned int PRIM_ATTR                                                    :  1;
        unsigned int                                                              :  4;
        unsigned int PT_SPRITE_TEX                                                :  1;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int PT_SPRITE_TEX_ATTR1                                          :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_5 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int ROTATE_PC_PTR                                                :  1;
        unsigned int PRIM_ATTR                                                    :  1;
        unsigned int                                                              :  4;
        unsigned int PT_SPRITE_TEX                                                :  1;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int PT_SPRITE_TEX_ATTR1                                          :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_6 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int ROTATE_PC_PTR                                                :  1;
        unsigned int PRIM_ATTR                                                    :  1;
        unsigned int                                                              :  4;
        unsigned int PT_SPRITE_TEX                                                :  1;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int PT_SPRITE_TEX_ATTR1                                          :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_7 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int ROTATE_PC_PTR                                                :  1;
        unsigned int PRIM_ATTR                                                    :  1;
        unsigned int                                                              :  4;
        unsigned int PT_SPRITE_TEX                                                :  1;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int PT_SPRITE_TEX_ATTR1                                          :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_8 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int ROTATE_PC_PTR                                                :  1;
        unsigned int PRIM_ATTR                                                    :  1;
        unsigned int                                                              :  4;
        unsigned int PT_SPRITE_TEX                                                :  1;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int PT_SPRITE_TEX_ATTR1                                          :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_9 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int ROTATE_PC_PTR                                                :  1;
        unsigned int PRIM_ATTR                                                    :  1;
        unsigned int                                                              :  4;
        unsigned int PT_SPRITE_TEX                                                :  1;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int PT_SPRITE_TEX_ATTR1                                          :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_10 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int ROTATE_PC_PTR                                                :  1;
        unsigned int PRIM_ATTR                                                    :  1;
        unsigned int                                                              :  4;
        unsigned int PT_SPRITE_TEX                                                :  1;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int PT_SPRITE_TEX_ATTR1                                          :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_11 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int ROTATE_PC_PTR                                                :  1;
        unsigned int PRIM_ATTR                                                    :  1;
        unsigned int                                                              :  4;
        unsigned int PT_SPRITE_TEX                                                :  1;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int PT_SPRITE_TEX_ATTR1                                          :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_12 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int ROTATE_PC_PTR                                                :  1;
        unsigned int PRIM_ATTR                                                    :  1;
        unsigned int                                                              :  4;
        unsigned int PT_SPRITE_TEX                                                :  1;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int PT_SPRITE_TEX_ATTR1                                          :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_13 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int ROTATE_PC_PTR                                                :  1;
        unsigned int PRIM_ATTR                                                    :  1;
        unsigned int                                                              :  4;
        unsigned int PT_SPRITE_TEX                                                :  1;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int PT_SPRITE_TEX_ATTR1                                          :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_14 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int ROTATE_PC_PTR                                                :  1;
        unsigned int PRIM_ATTR                                                    :  1;
        unsigned int                                                              :  4;
        unsigned int PT_SPRITE_TEX                                                :  1;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int PT_SPRITE_TEX_ATTR1                                          :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_15 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int ROTATE_PC_PTR                                                :  1;
        unsigned int PRIM_ATTR                                                    :  1;
        unsigned int                                                              :  4;
        unsigned int PT_SPRITE_TEX                                                :  1;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int PT_SPRITE_TEX_ATTR1                                          :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_16 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int ROTATE_PC_PTR                                                :  1;
        unsigned int PRIM_ATTR                                                    :  1;
        unsigned int                                                              :  4;
        unsigned int PT_SPRITE_TEX                                                :  1;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int PT_SPRITE_TEX_ATTR1                                          :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_17 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int ROTATE_PC_PTR                                                :  1;
        unsigned int PRIM_ATTR                                                    :  1;
        unsigned int                                                              :  4;
        unsigned int PT_SPRITE_TEX                                                :  1;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int PT_SPRITE_TEX_ATTR1                                          :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_18 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int ROTATE_PC_PTR                                                :  1;
        unsigned int PRIM_ATTR                                                    :  1;
        unsigned int                                                              :  4;
        unsigned int PT_SPRITE_TEX                                                :  1;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int PT_SPRITE_TEX_ATTR1                                          :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_19 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int ROTATE_PC_PTR                                                :  1;
        unsigned int PRIM_ATTR                                                    :  1;
        unsigned int                                                              :  4;
        unsigned int PT_SPRITE_TEX                                                :  1;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int PT_SPRITE_TEX_ATTR1                                          :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_20 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int ROTATE_PC_PTR                                                :  1;
        unsigned int PRIM_ATTR                                                    :  1;
        unsigned int                                                              :  5;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int                                                              :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_21 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int ROTATE_PC_PTR                                                :  1;
        unsigned int PRIM_ATTR                                                    :  1;
        unsigned int                                                              :  5;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int                                                              :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_22 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int ROTATE_PC_PTR                                                :  1;
        unsigned int PRIM_ATTR                                                    :  1;
        unsigned int                                                              :  5;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int                                                              :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_23 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int ROTATE_PC_PTR                                                :  1;
        unsigned int PRIM_ATTR                                                    :  1;
        unsigned int                                                              :  5;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int                                                              :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_24 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int ROTATE_PC_PTR                                                :  1;
        unsigned int PRIM_ATTR                                                    :  1;
        unsigned int                                                              :  5;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int                                                              :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_25 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int ROTATE_PC_PTR                                                :  1;
        unsigned int PRIM_ATTR                                                    :  1;
        unsigned int                                                              :  5;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int                                                              :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_26 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int ROTATE_PC_PTR                                                :  1;
        unsigned int PRIM_ATTR                                                    :  1;
        unsigned int                                                              :  5;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int                                                              :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_27 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int ROTATE_PC_PTR                                                :  1;
        unsigned int PRIM_ATTR                                                    :  1;
        unsigned int                                                              :  5;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int                                                              :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_28 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int ROTATE_PC_PTR                                                :  1;
        unsigned int PRIM_ATTR                                                    :  1;
        unsigned int                                                              :  5;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int                                                              :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_29 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int ROTATE_PC_PTR                                                :  1;
        unsigned int PRIM_ATTR                                                    :  1;
        unsigned int                                                              :  5;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int                                                              :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_30 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int ROTATE_PC_PTR                                                :  1;
        unsigned int PRIM_ATTR                                                    :  1;
        unsigned int                                                              :  5;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int                                                              :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_CNTL_31 {
    struct {
        unsigned int OFFSET                                                       :  6;
        unsigned int                                                              :  2;
        unsigned int DEFAULT_VAL                                                  :  2;
        unsigned int FLAT_SHADE                                                   :  1;
        unsigned int ROTATE_PC_PTR                                                :  1;
        unsigned int PRIM_ATTR                                                    :  1;
        unsigned int                                                              :  5;
        unsigned int DUP                                                          :  1;
        unsigned int FP16_INTERP_MODE                                             :  1;
        unsigned int USE_DEFAULT_ATTR1                                            :  1;
        unsigned int DEFAULT_VAL_ATTR1                                            :  2;
        unsigned int                                                              :  1;
        unsigned int ATTR0_VALID                                                  :  1;
        unsigned int ATTR1_VALID                                                  :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_INPUT_ENA {
    struct {
        unsigned int PERSP_SAMPLE_ENA                                             :  1;
        unsigned int PERSP_CENTER_ENA                                             :  1;
        unsigned int PERSP_CENTROID_ENA                                           :  1;
        unsigned int PERSP_PULL_MODEL_ENA                                         :  1;
        unsigned int LINEAR_SAMPLE_ENA                                            :  1;
        unsigned int LINEAR_CENTER_ENA                                            :  1;
        unsigned int LINEAR_CENTROID_ENA                                          :  1;
        unsigned int LINE_STIPPLE_TEX_ENA                                         :  1;
        unsigned int POS_X_FLOAT_ENA                                              :  1;
        unsigned int POS_Y_FLOAT_ENA                                              :  1;
        unsigned int POS_Z_FLOAT_ENA                                              :  1;
        unsigned int POS_W_FLOAT_ENA                                              :  1;
        unsigned int FRONT_FACE_ENA                                               :  1;
        unsigned int ANCILLARY_ENA                                                :  1;
        unsigned int SAMPLE_COVERAGE_ENA                                          :  1;
        unsigned int POS_FIXED_PT_ENA                                             :  1;
        unsigned int COVERAGE_TO_SHADER_SELECT                                    :  2;
        unsigned int                                                              : 14;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_IN_CONTROL {
    struct {
        unsigned int                                                              :  6;
        unsigned int PARAM_GEN                                                    :  1;
        unsigned int                                                              :  7;
        unsigned int BC_OPTIMIZE_DISABLE                                          :  1;
        unsigned int PS_W32_EN                                                    :  1;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_PS_MAX_WAVE_ID {
    struct {
        unsigned int MAX_WAVE_ID                                                  : 12;
        unsigned int                                                              :  4;
        unsigned int MAX_COLLISION_WAVE_ID                                        : 10;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_CU_0 {
    struct {
        unsigned int VGPR                                                         :  4;
        unsigned int SGPR                                                         :  4;
        unsigned int LDS                                                          :  4;
        unsigned int WAVES                                                        :  3;
        unsigned int BARRIERS                                                     :  4;
        unsigned int                                                              : 13;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_CU_1 {
    struct {
        unsigned int VGPR                                                         :  4;
        unsigned int SGPR                                                         :  4;
        unsigned int LDS                                                          :  4;
        unsigned int WAVES                                                        :  3;
        unsigned int BARRIERS                                                     :  4;
        unsigned int                                                              : 13;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_CU_2 {
    struct {
        unsigned int VGPR                                                         :  4;
        unsigned int SGPR                                                         :  4;
        unsigned int LDS                                                          :  4;
        unsigned int WAVES                                                        :  3;
        unsigned int BARRIERS                                                     :  4;
        unsigned int                                                              : 13;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_CU_3 {
    struct {
        unsigned int VGPR                                                         :  4;
        unsigned int SGPR                                                         :  4;
        unsigned int LDS                                                          :  4;
        unsigned int WAVES                                                        :  3;
        unsigned int BARRIERS                                                     :  4;
        unsigned int                                                              : 13;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_CU_4 {
    struct {
        unsigned int VGPR                                                         :  4;
        unsigned int SGPR                                                         :  4;
        unsigned int LDS                                                          :  4;
        unsigned int WAVES                                                        :  3;
        unsigned int BARRIERS                                                     :  4;
        unsigned int                                                              : 13;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_CU_5 {
    struct {
        unsigned int VGPR                                                         :  4;
        unsigned int SGPR                                                         :  4;
        unsigned int LDS                                                          :  4;
        unsigned int WAVES                                                        :  3;
        unsigned int BARRIERS                                                     :  4;
        unsigned int                                                              : 13;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_CU_6 {
    struct {
        unsigned int VGPR                                                         :  4;
        unsigned int SGPR                                                         :  4;
        unsigned int LDS                                                          :  4;
        unsigned int WAVES                                                        :  3;
        unsigned int BARRIERS                                                     :  4;
        unsigned int                                                              : 13;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_CU_7 {
    struct {
        unsigned int VGPR                                                         :  4;
        unsigned int SGPR                                                         :  4;
        unsigned int LDS                                                          :  4;
        unsigned int WAVES                                                        :  3;
        unsigned int BARRIERS                                                     :  4;
        unsigned int                                                              : 13;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_CU_8 {
    struct {
        unsigned int VGPR                                                         :  4;
        unsigned int SGPR                                                         :  4;
        unsigned int LDS                                                          :  4;
        unsigned int WAVES                                                        :  3;
        unsigned int BARRIERS                                                     :  4;
        unsigned int                                                              : 13;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_CU_9 {
    struct {
        unsigned int VGPR                                                         :  4;
        unsigned int SGPR                                                         :  4;
        unsigned int LDS                                                          :  4;
        unsigned int WAVES                                                        :  3;
        unsigned int BARRIERS                                                     :  4;
        unsigned int                                                              : 13;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_CU_10 {
    struct {
        unsigned int VGPR                                                         :  4;
        unsigned int SGPR                                                         :  4;
        unsigned int LDS                                                          :  4;
        unsigned int WAVES                                                        :  3;
        unsigned int BARRIERS                                                     :  4;
        unsigned int                                                              : 13;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_CU_11 {
    struct {
        unsigned int VGPR                                                         :  4;
        unsigned int SGPR                                                         :  4;
        unsigned int LDS                                                          :  4;
        unsigned int WAVES                                                        :  3;
        unsigned int BARRIERS                                                     :  4;
        unsigned int                                                              : 13;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_CU_12 {
    struct {
        unsigned int VGPR                                                         :  4;
        unsigned int SGPR                                                         :  4;
        unsigned int LDS                                                          :  4;
        unsigned int WAVES                                                        :  3;
        unsigned int BARRIERS                                                     :  4;
        unsigned int                                                              : 13;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_CU_13 {
    struct {
        unsigned int VGPR                                                         :  4;
        unsigned int SGPR                                                         :  4;
        unsigned int LDS                                                          :  4;
        unsigned int WAVES                                                        :  3;
        unsigned int BARRIERS                                                     :  4;
        unsigned int                                                              : 13;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_CU_14 {
    struct {
        unsigned int VGPR                                                         :  4;
        unsigned int SGPR                                                         :  4;
        unsigned int LDS                                                          :  4;
        unsigned int WAVES                                                        :  3;
        unsigned int BARRIERS                                                     :  4;
        unsigned int                                                              : 13;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_CU_15 {
    struct {
        unsigned int VGPR                                                         :  4;
        unsigned int SGPR                                                         :  4;
        unsigned int LDS                                                          :  4;
        unsigned int WAVES                                                        :  3;
        unsigned int BARRIERS                                                     :  4;
        unsigned int                                                              : 13;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_EN_CU_0 {
    struct {
        unsigned int EN                                                           :  1;
        unsigned int TYPE_MASK                                                    : 15;
        unsigned int QUEUE_MASK                                                   :  8;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_EN_CU_1 {
    struct {
        unsigned int EN                                                           :  1;
        unsigned int TYPE_MASK                                                    : 15;
        unsigned int QUEUE_MASK                                                   :  8;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_EN_CU_2 {
    struct {
        unsigned int EN                                                           :  1;
        unsigned int TYPE_MASK                                                    : 15;
        unsigned int QUEUE_MASK                                                   :  8;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_EN_CU_3 {
    struct {
        unsigned int EN                                                           :  1;
        unsigned int TYPE_MASK                                                    : 15;
        unsigned int QUEUE_MASK                                                   :  8;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_EN_CU_4 {
    struct {
        unsigned int EN                                                           :  1;
        unsigned int TYPE_MASK                                                    : 15;
        unsigned int QUEUE_MASK                                                   :  8;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_EN_CU_5 {
    struct {
        unsigned int EN                                                           :  1;
        unsigned int TYPE_MASK                                                    : 15;
        unsigned int QUEUE_MASK                                                   :  8;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_EN_CU_6 {
    struct {
        unsigned int EN                                                           :  1;
        unsigned int TYPE_MASK                                                    : 15;
        unsigned int QUEUE_MASK                                                   :  8;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_EN_CU_7 {
    struct {
        unsigned int EN                                                           :  1;
        unsigned int TYPE_MASK                                                    : 15;
        unsigned int QUEUE_MASK                                                   :  8;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_EN_CU_8 {
    struct {
        unsigned int EN                                                           :  1;
        unsigned int TYPE_MASK                                                    : 15;
        unsigned int QUEUE_MASK                                                   :  8;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_EN_CU_9 {
    struct {
        unsigned int EN                                                           :  1;
        unsigned int TYPE_MASK                                                    : 15;
        unsigned int QUEUE_MASK                                                   :  8;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_EN_CU_10 {
    struct {
        unsigned int EN                                                           :  1;
        unsigned int TYPE_MASK                                                    : 15;
        unsigned int QUEUE_MASK                                                   :  8;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_EN_CU_11 {
    struct {
        unsigned int EN                                                           :  1;
        unsigned int TYPE_MASK                                                    : 15;
        unsigned int QUEUE_MASK                                                   :  8;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_EN_CU_12 {
    struct {
        unsigned int EN                                                           :  1;
        unsigned int TYPE_MASK                                                    : 15;
        unsigned int QUEUE_MASK                                                   :  8;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_EN_CU_13 {
    struct {
        unsigned int EN                                                           :  1;
        unsigned int TYPE_MASK                                                    : 15;
        unsigned int QUEUE_MASK                                                   :  8;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_EN_CU_14 {
    struct {
        unsigned int EN                                                           :  1;
        unsigned int TYPE_MASK                                                    : 15;
        unsigned int QUEUE_MASK                                                   :  8;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_RESOURCE_RESERVE_EN_CU_15 {
    struct {
        unsigned int EN                                                           :  1;
        unsigned int TYPE_MASK                                                    : 15;
        unsigned int QUEUE_MASK                                                   :  8;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SAVE_RESTORE_STATUS {
    struct {
        unsigned int PERFCOUNTER_EN                                               :  1;
        unsigned int THREAD_TRACE_EN                                              :  1;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SCRATCH_ADDR_STATUS {
    struct {
        unsigned int WRITE_DIS                                                    :  1;
        unsigned int OVERFLOW_DETECTED                                            :  1;
        unsigned int ME_ID                                                        :  2;
        unsigned int PIPE_ID                                                      :  2;
        unsigned int                                                              : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_COL_FORMAT {
    struct {
        unsigned int COL0_EXPORT_FORMAT                                           :  4;
        unsigned int COL1_EXPORT_FORMAT                                           :  4;
        unsigned int COL2_EXPORT_FORMAT                                           :  4;
        unsigned int COL3_EXPORT_FORMAT                                           :  4;
        unsigned int COL4_EXPORT_FORMAT                                           :  4;
        unsigned int COL5_EXPORT_FORMAT                                           :  4;
        unsigned int COL6_EXPORT_FORMAT                                           :  4;
        unsigned int COL7_EXPORT_FORMAT                                           :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_GS_MESHLET_CTRL {
    struct {
        unsigned int INTERLEAVE_BITS_X                                            :  4;
        unsigned int INTERLEAVE_BITS_Y                                            :  4;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_GS_MESHLET_DIM {
    struct {
        unsigned int MESHLET_NUM_THREAD_X                                         :  8;
        unsigned int MESHLET_NUM_THREAD_Y                                         :  8;
        unsigned int MESHLET_NUM_THREAD_Z                                         :  8;
        unsigned int MESHLET_THREADGROUP_SIZE                                     :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_GS_MESHLET_EXP_ALLOC {
    struct {
        unsigned int MAX_EXP_VERTS                                                :  9;
        unsigned int MAX_EXP_PRIMS                                                :  9;
        unsigned int                                                              : 14;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_GS_OUT_CONFIG_PS {
    struct {
        unsigned int VS_EXPORT_COUNT                                              :  5;
        unsigned int PRIM_EXPORT_COUNT                                            :  5;
        unsigned int NO_PC_EXPORT                                                 :  1;
        unsigned int NUM_INTERP                                                   :  6;
        unsigned int NUM_PRIM_INTERP                                              :  5;
        unsigned int                                                              : 10;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_GS_OUT_CONFIG_PS_GS {
    struct {
        unsigned int VS_EXPORT_COUNT                                              :  5;
        unsigned int PRIM_EXPORT_COUNT                                            :  5;
        unsigned int NO_PC_EXPORT                                                 :  1;
        unsigned int NUM_INTERP                                                   :  6;
        unsigned int NUM_PRIM_INTERP                                              :  5;
        unsigned int                                                              : 10;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_IDX_FORMAT {
    struct {
        unsigned int IDX0_EXPORT_FORMAT                                           :  4;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_CHKSUM_GS {
    struct {
        unsigned int CHECKSUM                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_CHKSUM_HS {
    struct {
        unsigned int CHECKSUM                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_CHKSUM_PS {
    struct {
        unsigned int CHECKSUM                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_HI_ES {
    struct {
        unsigned int MEM_BASE                                                     :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_HI_GS {
    struct {
        unsigned int MEM_BASE                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_HI_HS {
    struct {
        unsigned int MEM_BASE                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_HI_LS {
    struct {
        unsigned int MEM_BASE                                                     :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_HI_PS {
    struct {
        unsigned int MEM_BASE                                                     :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_LO_ES {
    struct {
        unsigned int MEM_BASE                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_LO_GS {
    struct {
        unsigned int MEM_BASE                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_LO_HS {
    struct {
        unsigned int MEM_BASE                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_LO_LS {
    struct {
        unsigned int MEM_BASE                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_LO_PS {
    struct {
        unsigned int MEM_BASE                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_RSRC1_GS {
    struct {
        unsigned int VGPRS                                                        :  6;
        unsigned int SGPRS                                                        :  4;
        unsigned int PRIORITY                                                     :  2;
        unsigned int FLOAT_MODE                                                   :  8;
        unsigned int PRIV                                                         :  1;
        unsigned int WG_RR_EN                                                     :  1;
        unsigned int DEBUG_MODE                                                   :  1;
        unsigned int DISABLE_PERF                                                 :  1;
        unsigned int CU_GROUP_ENABLE                                              :  1;
        unsigned int                                                              :  1;
        unsigned int FWD_PROGRESS                                                 :  1;
        unsigned int WGP_MODE                                                     :  1;
        unsigned int CDBG_USER                                                    :  1;
        unsigned int GS_VGPR_COMP_CNT                                             :  2;
        unsigned int FP16_OVFL                                                    :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_RSRC1_HS {
    struct {
        unsigned int VGPRS                                                        :  6;
        unsigned int SGPRS                                                        :  4;
        unsigned int PRIORITY                                                     :  2;
        unsigned int FLOAT_MODE                                                   :  8;
        unsigned int PRIV                                                         :  1;
        unsigned int WG_RR_EN                                                     :  1;
        unsigned int DEBUG_MODE                                                   :  1;
        unsigned int DISABLE_PERF                                                 :  1;
        unsigned int                                                              :  1;
        unsigned int FWD_PROGRESS                                                 :  1;
        unsigned int WGP_MODE                                                     :  1;
        unsigned int CDBG_USER                                                    :  1;
        unsigned int LS_VGPR_COMP_CNT                                             :  2;
        unsigned int FP16_OVFL                                                    :  1;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_RSRC1_PS {
    struct {
        unsigned int VGPRS                                                        :  6;
        unsigned int SGPRS                                                        :  4;
        unsigned int PRIORITY                                                     :  2;
        unsigned int FLOAT_MODE                                                   :  8;
        unsigned int PRIV                                                         :  1;
        unsigned int WG_RR_EN                                                     :  1;
        unsigned int DEBUG_MODE                                                   :  1;
        unsigned int DISABLE_PERF                                                 :  1;
        unsigned int CU_GROUP_DISABLE                                             :  1;
        unsigned int                                                              :  1;
        unsigned int FWD_PROGRESS                                                 :  1;
        unsigned int LOAD_PROVOKING_VTX                                           :  1;
        unsigned int CDBG_USER                                                    :  1;
        unsigned int FP16_OVFL                                                    :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_RSRC2_GS {
    struct {
        unsigned int SCRATCH_EN                                                   :  1;
        unsigned int USER_SGPR                                                    :  5;
        unsigned int TRAP_PRESENT                                                 :  1;
        unsigned int EXCP_EN                                                      :  9;
        unsigned int ES_VGPR_COMP_CNT                                             :  2;
        unsigned int OC_LDS_EN                                                    :  1;
        unsigned int LDS_SIZE                                                     :  8;
        unsigned int USER_SGPR_MSB                                                :  1;
        unsigned int SHARED_VGPR_CNT                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_RSRC2_HS {
    struct {
        unsigned int SCRATCH_EN                                                   :  1;
        unsigned int USER_SGPR                                                    :  5;
        unsigned int TRAP_PRESENT                                                 :  1;
        unsigned int OC_LDS_EN                                                    :  1;
        unsigned int TG_SIZE_EN                                                   :  1;
        unsigned int EXCP_EN                                                      :  9;
        unsigned int LDS_SIZE                                                     :  9;
        unsigned int USER_SGPR_MSB                                                :  1;
        unsigned int SHARED_VGPR_CNT                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_RSRC2_PS {
    struct {
        unsigned int SCRATCH_EN                                                   :  1;
        unsigned int USER_SGPR                                                    :  5;
        unsigned int TRAP_PRESENT                                                 :  1;
        unsigned int WAVE_CNT_EN                                                  :  1;
        unsigned int EXTRA_LDS_SIZE                                               :  8;
        unsigned int EXCP_EN                                                      :  9;
        unsigned int LOAD_COLLISION_WAVEID                                        :  1;
        unsigned int LOAD_INTRAWAVE_COLLISION                                     :  1;
        unsigned int USER_SGPR_MSB                                                :  1;
        unsigned int SHARED_VGPR_CNT                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_RSRC3_GS {
    struct {
        unsigned int CU_EN                                                        : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_RSRC3_HS {
    struct {
        unsigned int CU_EN                                                        : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_RSRC3_PS {
    struct {
        unsigned int CU_EN                                                        : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_RSRC4_GS {
    struct {
        unsigned int WAVE_LIMIT                                                   : 10;
        unsigned int GLG_EN_OVERRIDE                                              :  1;
        unsigned int GLG_FORCE_DISABLE                                            :  1;
        unsigned int                                                              :  2;
        unsigned int PH_THROTTLE_EN                                               :  1;
        unsigned int SPI_THROTTLE_EN                                              :  1;
        unsigned int SPI_SHADER_LATE_ALLOC_GS                                     :  7;
        unsigned int INST_PREF_SIZE                                               :  8;
        unsigned int IMAGE_OP                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_RSRC4_HS {
    struct {
        unsigned int WAVE_LIMIT                                                   : 10;
        unsigned int GLG_EN_OVERRIDE                                              :  1;
        unsigned int GLG_FORCE_DISABLE                                            :  1;
        unsigned int                                                              :  4;
        unsigned int INST_PREF_SIZE                                               :  8;
        unsigned int                                                              :  7;
        unsigned int IMAGE_OP                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_PGM_RSRC4_PS {
    struct {
        unsigned int WAVE_LIMIT                                                   : 10;
        unsigned int LDS_GROUP_SIZE                                               :  2;
        unsigned int                                                              :  4;
        unsigned int INST_PREF_SIZE                                               :  8;
        unsigned int                                                              :  7;
        unsigned int IMAGE_OP                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_POS_FORMAT {
    struct {
        unsigned int POS0_EXPORT_FORMAT                                           :  4;
        unsigned int POS1_EXPORT_FORMAT                                           :  4;
        unsigned int POS2_EXPORT_FORMAT                                           :  4;
        unsigned int POS3_EXPORT_FORMAT                                           :  4;
        unsigned int POS4_EXPORT_FORMAT                                           :  4;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_REQ_CTRL_ESGS {
    struct {
        unsigned int SOFT_GROUPING_EN                                             :  1;
        unsigned int NUMBER_OF_REQUESTS_PER_CU                                    :  4;
        unsigned int SOFT_GROUPING_ALLOCATION_TIMEOUT                             :  4;
        unsigned int HARD_LOCK_HYSTERESIS                                         :  1;
        unsigned int HARD_LOCK_LOW_THRESHOLD                                      :  5;
        unsigned int PRODUCER_REQUEST_LOCKOUT                                     :  1;
        unsigned int GLOBAL_SCANNING_EN                                           :  1;
        unsigned int ALLOCATION_RATE_THROTTLING_THRESHOLD                         :  3;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_REQ_CTRL_LSHS {
    struct {
        unsigned int SOFT_GROUPING_EN                                             :  1;
        unsigned int NUMBER_OF_REQUESTS_PER_CU                                    :  4;
        unsigned int SOFT_GROUPING_ALLOCATION_TIMEOUT                             :  4;
        unsigned int HARD_LOCK_HYSTERESIS                                         :  1;
        unsigned int HARD_LOCK_LOW_THRESHOLD                                      :  5;
        unsigned int PRODUCER_REQUEST_LOCKOUT                                     :  1;
        unsigned int GLOBAL_SCANNING_EN                                           :  1;
        unsigned int ALLOCATION_RATE_THROTTLING_THRESHOLD                         :  3;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_REQ_CTRL_PS {
    struct {
        unsigned int SOFT_GROUPING_EN                                             :  1;
        unsigned int NUMBER_OF_REQUESTS_PER_CU                                    :  4;
        unsigned int SOFT_GROUPING_ALLOCATION_TIMEOUT                             :  4;
        unsigned int HARD_LOCK_HYSTERESIS                                         :  1;
        unsigned int HARD_LOCK_LOW_THRESHOLD                                      :  5;
        unsigned int PRODUCER_REQUEST_LOCKOUT                                     :  1;
        unsigned int GLOBAL_SCANNING_EN                                           :  1;
        unsigned int ALLOCATION_RATE_THROTTLING_THRESHOLD                         :  3;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_RSRC_LIMIT_CTRL {
    struct {
        unsigned int WAVES_PER_SIMD32                                             :  5;
        unsigned int VGPR_PER_SIMD32                                              :  7;
        unsigned int VGPR_WRAP_DISABLE                                            :  1;
        unsigned int BARRIER_LIMIT                                                :  6;
        unsigned int BARRIER_LIMIT_HIERARCHY_LEVEL                                :  1;
        unsigned int LDS_LIMIT                                                    :  8;
        unsigned int LDS_LIMIT_HIERARCHY_LEVEL                                    :  1;
        unsigned int                                                              :  2;
        unsigned int PERFORMANCE_LIMIT_ENABLE                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_ACCUM_ESGS_0 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_ACCUM_ESGS_1 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_ACCUM_ESGS_2 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_ACCUM_ESGS_3 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_ACCUM_LSHS_0 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_ACCUM_LSHS_1 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_ACCUM_LSHS_2 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_ACCUM_LSHS_3 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_ACCUM_PS_0 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_ACCUM_PS_1 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_ACCUM_PS_2 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_ACCUM_PS_3 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_ADDR_HI_GS {
    struct {
        unsigned int MEM_BASE                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_ADDR_HI_HS {
    struct {
        unsigned int MEM_BASE                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_ADDR_LO_GS {
    struct {
        unsigned int MEM_BASE                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_ADDR_LO_HS {
    struct {
        unsigned int MEM_BASE                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_0 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_1 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_2 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_3 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_4 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_5 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_6 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_7 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_8 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_9 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_10 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_11 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_12 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_13 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_14 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_15 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_16 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_17 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_18 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_19 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_20 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_21 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_22 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_23 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_24 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_25 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_26 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_27 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_28 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_29 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_30 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_GS_31 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_0 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_1 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_2 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_3 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_4 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_5 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_6 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_7 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_8 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_9 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_10 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_11 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_12 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_13 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_14 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_15 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_16 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_17 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_18 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_19 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_20 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_21 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_22 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_23 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_24 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_25 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_26 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_27 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_28 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_29 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_30 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_HS_31 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_0 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_1 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_2 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_3 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_4 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_5 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_6 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_7 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_8 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_9 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_10 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_11 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_12 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_13 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_14 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_15 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_16 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_17 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_18 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_19 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_20 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_21 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_22 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_23 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_24 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_25 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_26 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_27 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_28 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_29 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_30 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_USER_DATA_PS_31 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SHADER_Z_FORMAT {
    struct {
        unsigned int Z_EXPORT_FORMAT                                              :  4;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SQG_EVENT_CTL {
    struct {
        unsigned int ENABLE_SQG_TOP_EVENTS                                        :  1;
        unsigned int ENABLE_SQG_BOP_EVENTS                                        :  1;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SX_EXPORT_BUFFER_SIZES {
    struct {
        unsigned int COLOR_BUFFER_SIZE                                            : 16;
        unsigned int POSITION_BUFFER_SIZE                                         : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_SX_SCOREBOARD_BUFFER_SIZES {
    struct {
        unsigned int COLOR_SCOREBOARD_SIZE                                        : 16;
        unsigned int POSITION_SCOREBOARD_SIZE                                     : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_TMPRING_SIZE {
    struct {
        unsigned int WAVES                                                        : 12;
        unsigned int WAVESIZE                                                     : 18;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_USER_ACCUM_VMID_CNTL {
    struct {
        unsigned int EN_USER_ACCUM                                                :  4;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WCL_PIPE_PERCENT_CS0 {
    struct {
        unsigned int VALUE                                                        :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WCL_PIPE_PERCENT_CS1 {
    struct {
        unsigned int VALUE                                                        :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WCL_PIPE_PERCENT_CS2 {
    struct {
        unsigned int VALUE                                                        :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WCL_PIPE_PERCENT_CS3 {
    struct {
        unsigned int VALUE                                                        :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WCL_PIPE_PERCENT_CS4 {
    struct {
        unsigned int VALUE                                                        :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WCL_PIPE_PERCENT_CS5 {
    struct {
        unsigned int VALUE                                                        :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WCL_PIPE_PERCENT_CS6 {
    struct {
        unsigned int VALUE                                                        :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WCL_PIPE_PERCENT_CS7 {
    struct {
        unsigned int VALUE                                                        :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WCL_PIPE_PERCENT_GFX {
    struct {
        unsigned int VALUE                                                        :  7;
        unsigned int                                                              :  5;
        unsigned int HS_GRP_VALUE                                                 :  5;
        unsigned int                                                              :  5;
        unsigned int GS_GRP_VALUE                                                 :  5;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WCL_PIPE_PERCENT_HP3D {
    struct {
        unsigned int VALUE                                                        :  7;
        unsigned int                                                              :  5;
        unsigned int HS_GRP_VALUE                                                 :  5;
        unsigned int                                                              :  5;
        unsigned int GS_GRP_VALUE                                                 :  5;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_ACTIVE_COUNT_GFX {
    struct {
        unsigned int WF_ALLOCATED                                                 :  8;
        unsigned int WF_ACTIVE                                                    : 16;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_ACTIVE_COUNT_HPG {
    struct {
        unsigned int WF_ALLOCATED                                                 :  8;
        unsigned int WF_ACTIVE                                                    : 16;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_CNTL {
    struct {
        unsigned int SAMPLE_PERIOD                                                :  4;
        unsigned int EN                                                           :  1;
        unsigned int                                                              : 27;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_LIMIT_0 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int EN_WARN                                                      :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_LIMIT_2 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int EN_WARN                                                      :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_LIMIT_3 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int EN_WARN                                                      :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_STATUS_0 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int INT_SENT                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_STATUS_2 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int INT_SENT                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_STATUS_4 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int INT_SENT                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_STATUS_6 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int INT_SENT                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_STATUS_7 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int INT_SENT                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_STATUS_9 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int INT_SENT                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_STATUS_11 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int INT_SENT                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_STATUS_13 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int INT_SENT                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_STATUS_14 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int INT_SENT                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_STATUS_15 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int INT_SENT                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_STATUS_16 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int INT_SENT                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_STATUS_17 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int INT_SENT                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_STATUS_18 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int INT_SENT                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_STATUS_19 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int INT_SENT                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_STATUS_20 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int INT_SENT                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WF_LIFETIME_STATUS_21 {
    struct {
        unsigned int MAX_CNT                                                      : 31;
        unsigned int INT_SENT                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SPI_WGP_WORK_PENDING {
    struct {
        unsigned int SPI_WGP_WORK_PENDING                                         : 16;
        unsigned int RESERVED                                                     : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQG_CONFIG {
    struct {
        unsigned int SQG_ICPFT_EN                                                 :  1;
        unsigned int SQG_ICPFT_CLR                                                :  1;
        unsigned int                                                              : 14;
        unsigned int XNACK_INTR_MASK                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQG_GL1X_CTRL {
    struct {
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int                                                              : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQG_GL1X_STATUS {
    struct {
        unsigned int ACK_ERR_DETECTED                                             :  1;
        unsigned int XNACK_ERR_DETECTED                                           :  1;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQG_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQG_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQG_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              : 11;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQG_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQG_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQG_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              : 11;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQG_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQG_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQG_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              : 11;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQG_PERFCOUNTER3_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQG_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQG_PERFCOUNTER3_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              : 11;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQG_PERFCOUNTER4_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQG_PERFCOUNTER4_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQG_PERFCOUNTER4_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              : 11;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQG_PERFCOUNTER5_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQG_PERFCOUNTER5_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQG_PERFCOUNTER5_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              : 11;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQG_PERFCOUNTER6_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQG_PERFCOUNTER6_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQG_PERFCOUNTER6_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              : 11;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQG_PERFCOUNTER7_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQG_PERFCOUNTER7_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQG_PERFCOUNTER7_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              : 11;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQG_PERFCOUNTER_CTRL {
    struct {
        unsigned int PS_EN                                                        :  1;
        unsigned int                                                              :  1;
        unsigned int GS_EN                                                        :  1;
        unsigned int                                                              :  1;
        unsigned int HS_EN                                                        :  1;
        unsigned int                                                              :  1;
        unsigned int CS_EN                                                        :  1;
        unsigned int                                                              :  7;
        unsigned int DISABLE_ME0PIPE0_PERF                                        :  1;
        unsigned int DISABLE_ME0PIPE1_PERF                                        :  1;
        unsigned int DISABLE_ME1PIPE0_PERF                                        :  1;
        unsigned int DISABLE_ME1PIPE1_PERF                                        :  1;
        unsigned int DISABLE_ME1PIPE2_PERF                                        :  1;
        unsigned int DISABLE_ME1PIPE3_PERF                                        :  1;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQG_PERFCOUNTER_CTRL2 {
    struct {
        unsigned int FORCE_EN                                                     :  1;
        unsigned int VMID_EN                                                      : 16;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQG_PERF_SAMPLE_FINISH {
    struct {
        unsigned int STATUS                                                       :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQG_STATUS {
    struct {
        unsigned int REG_BUSY                                                     :  1;
        unsigned int POWEROFF_RESTORE                                             :  1;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQG_THREAD_TRACE_CONFIG {
    struct {
        unsigned int ALL_VMID                                                     :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_ALU_CLK_CTRL {
    struct {
        unsigned int FORCE_WGP_ON_SA0                                             : 16;
        unsigned int FORCE_WGP_ON_SA1                                             : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_ARB_CONFIG {
    struct {
        unsigned int WG_RR_INTERVAL                                               :  2;
        unsigned int                                                              :  2;
        unsigned int FWD_PROG_INTERVAL                                            :  2;
        unsigned int                                                              : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_CLK_CTRL {
    struct {
        unsigned int                                                              :  2;
        unsigned int SQ_SPI_MSG_FGCG_OVERRIDE                                     :  1;
        unsigned int SQ_SPI_EXPREQ_FGCG_OVERRIDE                                  :  1;
        unsigned int SQ_SX_EXPCMD_FGCG_OVERRIDE                                   :  1;
        unsigned int SQ_SQC_TTRACE_FGCG_OVERRIDE                                  :  1;
        unsigned int WCLK_OVERRIDE                                                :  1;
        unsigned int PERFMON_OVERRIDE                                             :  1;
        unsigned int OVERRIDE_LDS_IDX_BUSY                                        :  1;
        unsigned int OVERRIDE_LDS_DIRECT_BUSY                                     :  1;
        unsigned int WCLK_SLEEP_VMEM_OVERRIDE                                     :  1;
        unsigned int WCLK_SLEEP_EXPALLOC_OVERRIDE                                 :  1;
        unsigned int SQ_SP_CMD_FGCG_OVERRIDE                                      :  1;
        unsigned int SQ_SP_CONST_FGCG_OVERRIDE                                    :  1;
        unsigned int SQ_SP_EXP_FGCG_OVERRIDE                                      :  1;
        unsigned int SQ_SP_VMEM_FGCG_OVERRIDE                                     :  1;
        unsigned int SQ_LDS_DIRECT_FGCG_OVERRIDE                                  :  1;
        unsigned int IS_WAVECLK_IB_WCLK_OVERRIDE                                  :  1;
        unsigned int ISC_SET_FGCG_OVERRIDE                                        :  1;
        unsigned int ISC_CTRL_FGCG_OVERRIDE                                       :  1;
        unsigned int ISC_WAVE_CTRL_FGCG_OVERRIDE                                  :  1;
        unsigned int IB_IBUF_FGCG_OVERRIDE                                        :  1;
        unsigned int IB_WINFO_FGCG_OVERRIDE                                       :  1;
        unsigned int IB_MISC_FGCG_OVERRIDE                                        :  1;
        unsigned int EX_SALU_FGCG_OVERRIDE                                        :  1;
        unsigned int EX_VALU_FGCG_OVERRIDE                                        :  1;
        unsigned int EX_BRMSG_FGCG_OVERRIDE                                       :  1;
        unsigned int SQ_SP_ICG_FGCG_OVERRIDE                                      :  1;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_CMD {
    struct {
        unsigned int CMD                                                          :  4;
        unsigned int MODE                                                         :  3;
        unsigned int CHECK_VMID                                                   :  1;
        unsigned int DATA                                                         :  4;
        unsigned int                                                              :  4;
        unsigned int WAVE_ID                                                      :  5;
        unsigned int                                                              :  3;
        unsigned int QUEUE_ID                                                     :  3;
        unsigned int                                                              :  1;
        unsigned int VM_ID                                                        :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_CONFIG {
    struct {
        unsigned int ECO_SPARE                                                    :  8;
        unsigned int NEW_TRANS_ARB_SCHEME                                         :  1;
        unsigned int DISABLE_VMEM_EXEC_ZERO_SKIP                                  :  1;
        unsigned int DISABLE_SGPR_RD_KILL                                         :  1;
        unsigned int                                                              :  7;
        unsigned int ENABLE_HIPRIO_ON_EXP_RDY_GS                                  :  1;
        unsigned int PRIO_VAL_ON_EXP_RDY_GS                                       :  2;
        unsigned int WCLK_HYSTERESIS_CNT                                          :  2;
        unsigned int DISABLE_ILLEGAL_OPCODE_DETECTION                             :  1;
        unsigned int DISABLE_ILLEGAL_TO_NOP_DETECTION                             :  1;
        unsigned int DISABLE_ILLEGAL_EXPORT_DETECTION                             :  1;
        unsigned int DISABLE_ILLEGAL_CLAUSE_DETECTION                             :  1;
        unsigned int DISABLE_END_CLAUSE_TX                                        :  1;
        unsigned int                                                              :  2;
        unsigned int DISABLE_SP_SINGLE_ISSUE_WAVE64_TRANS                         :  1;
        unsigned int DISABLE_ISC_PREFETCH_LIMITER                                 :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_DSM_CNTL {
    struct {
        unsigned int WAVEFRONT_STALL_0                                            :  1;
        unsigned int WAVEFRONT_STALL_1                                            :  1;
        unsigned int SPI_BACKPRESSURE_0                                           :  1;
        unsigned int SPI_BACKPRESSURE_1                                           :  1;
        unsigned int                                                              :  4;
        unsigned int SEL_DSM_SGPR_IRRITATOR_DATA0                                 :  1;
        unsigned int SEL_DSM_SGPR_IRRITATOR_DATA1                                 :  1;
        unsigned int SGPR_ENABLE_SINGLE_WRITE                                     :  1;
        unsigned int                                                              :  5;
        unsigned int SEL_DSM_LDS_IRRITATOR_DATA0                                  :  1;
        unsigned int SEL_DSM_LDS_IRRITATOR_DATA1                                  :  1;
        unsigned int LDS_ENABLE_SINGLE_WRITE01                                    :  1;
        unsigned int SEL_DSM_LDS_IRRITATOR_DATA2                                  :  1;
        unsigned int SEL_DSM_LDS_IRRITATOR_DATA3                                  :  1;
        unsigned int LDS_ENABLE_SINGLE_WRITE23                                    :  1;
        unsigned int                                                              :  2;
        unsigned int SEL_DSM_SP_IRRITATOR_DATA0                                   :  1;
        unsigned int SEL_DSM_SP_IRRITATOR_DATA1                                   :  1;
        unsigned int SP_ENABLE_SINGLE_WRITE                                       :  1;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_DSM_CNTL2 {
    struct {
        unsigned int SGPR_ENABLE_ERROR_INJECT                                     :  2;
        unsigned int SGPR_SELECT_INJECT_DELAY                                     :  1;
        unsigned int LDS_D_ENABLE_ERROR_INJECT                                    :  2;
        unsigned int LDS_D_SELECT_INJECT_DELAY                                    :  1;
        unsigned int LDS_I_ENABLE_ERROR_INJECT                                    :  2;
        unsigned int LDS_I_SELECT_INJECT_DELAY                                    :  1;
        unsigned int SP_ENABLE_ERROR_INJECT                                       :  2;
        unsigned int SP_SELECT_INJECT_DELAY                                       :  1;
        unsigned int                                                              :  2;
        unsigned int LDS_INJECT_DELAY                                             :  6;
        unsigned int SP_INJECT_DELAY                                              :  6;
        unsigned int SQ_INJECT_DELAY                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_DYN_VGPR {
    struct {
        unsigned int WAVE_LIMIT                                                   :  4;
        unsigned int FWD_PROGRESS                                                 :  1;
        unsigned int MAX_BLOCK_ALLOC                                              :  3;
        unsigned int BLOCK_SIZE                                                   :  1;
        unsigned int                                                              : 23;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_FIFO_SIZES {
    struct {
        unsigned int INTERRUPT_FIFO_SIZE                                          :  4;
        unsigned int                                                              :  4;
        unsigned int TTRACE_FIFO_SIZE                                             :  2;
        unsigned int                                                              :  2;
        unsigned int EXPORT_BUF_GS_RESERVED                                       :  2;
        unsigned int EXPORT_BUF_PS_RESERVED                                       :  2;
        unsigned int EXPORT_BUF_REDUCE                                            :  2;
        unsigned int VMEM_DATA_FIFO_SIZE                                          :  2;
        unsigned int EXPORT_BUF_PRIMPOS_LIMIT                                     :  2;
        unsigned int                                                              : 10;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_IND_DATA {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_IND_INDEX {
    struct {
        unsigned int WAVE_ID                                                      :  5;
        unsigned int WORKITEM_ID                                                  :  6;
        unsigned int AUTO_INCR                                                    :  1;
        unsigned int                                                              :  4;
        unsigned int INDEX                                                        : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_INTERRUPT_AUTO_MASK {
    struct {
        unsigned int MASK                                                         : 24;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_INTERRUPT_MSG_CTRL {
    struct {
        unsigned int STALL                                                        :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_LDS_CLK_CTRL {
    struct {
        unsigned int FORCE_WGP_ON_SA0                                             : 16;
        unsigned int FORCE_WGP_ON_SA1                                             : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              : 11;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              : 11;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              : 11;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER3_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              : 11;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER4_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER4_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              : 11;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER5_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER5_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              : 11;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER6_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER6_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              : 11;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER7_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER7_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              : 11;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER8_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              : 11;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER9_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              : 11;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER10_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              : 11;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER11_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              : 11;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER12_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              : 11;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER13_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              : 11;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER14_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              : 11;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER15_SELECT {
    struct {
        unsigned int PERF_SEL                                                     :  9;
        unsigned int                                                              : 11;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER_CTRL {
    struct {
        unsigned int PS_EN                                                        :  1;
        unsigned int                                                              :  1;
        unsigned int GS_EN                                                        :  1;
        unsigned int                                                              :  1;
        unsigned int HS_EN                                                        :  1;
        unsigned int                                                              :  1;
        unsigned int CS_EN                                                        :  1;
        unsigned int                                                              :  7;
        unsigned int DISABLE_ME0PIPE0_PERF                                        :  1;
        unsigned int DISABLE_ME0PIPE1_PERF                                        :  1;
        unsigned int DISABLE_ME1PIPE0_PERF                                        :  1;
        unsigned int DISABLE_ME1PIPE1_PERF                                        :  1;
        unsigned int DISABLE_ME1PIPE2_PERF                                        :  1;
        unsigned int DISABLE_ME1PIPE3_PERF                                        :  1;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERFCOUNTER_CTRL2 {
    struct {
        unsigned int FORCE_EN                                                     :  1;
        unsigned int VMID_EN                                                      : 16;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERF_SNAPSHOT_CTRL {
    struct {
        unsigned int TIMER_ON_OFF                                                 :  1;
        unsigned int VMID_MASK                                                    : 16;
        unsigned int COUNT_SEL                                                    :  1;
        unsigned int COUNT_INTERVAL                                               :  4;
        unsigned int TEST_MODE                                                    :  1;
        unsigned int                                                              :  9;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERF_SNAPSHOT_DATA {
    struct {
        unsigned int VALID                                                        :  1;
        unsigned int WAVE_ISSUE                                                   :  1;
        unsigned int INST_TYPE                                                    :  4;
        unsigned int NO_ISSUE_REASON                                              :  3;
        unsigned int WAVE_ID                                                      :  5;
        unsigned int                                                              : 18;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERF_SNAPSHOT_DATA1 {
    struct {
        unsigned int WAVE_CNT                                                     :  6;
        unsigned int ARB_STATE_ISSUED_BRMSG                                       :  1;
        unsigned int ARB_STATE_ISSUED_EXPORT                                      :  1;
        unsigned int ARB_STATE_ISSUED_LDS_DIRECT                                  :  1;
        unsigned int ARB_STATE_ISSUED_LDS                                         :  1;
        unsigned int ARB_STATE_ISSUED_TEX                                         :  1;
        unsigned int ARB_STATE_ISSUED_SCALAR                                      :  1;
        unsigned int ARB_STATE_ISSUED_VALU                                        :  1;
        unsigned int ARB_STATE_ISSUED_RESERVED                                    :  1;
        unsigned int ARB_STATE_STALLED_BRMSG                                      :  1;
        unsigned int ARB_STATE_STALLED_EXPORT                                     :  1;
        unsigned int ARB_STATE_STALLED_LDS_DIRECT                                 :  1;
        unsigned int ARB_STATE_STALLED_LDS                                        :  1;
        unsigned int ARB_STATE_STALLED_TEX                                        :  1;
        unsigned int ARB_STATE_STALLED_SCALAR                                     :  1;
        unsigned int ARB_STATE_STALLED_VALU                                       :  1;
        unsigned int ARB_STATE_STALLED_RESERVED                                   :  1;
        unsigned int                                                              : 10;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERF_SNAPSHOT_DATA2 {
    struct {
        unsigned int LOAD_CNT                                                     :  6;
        unsigned int STORE_CNT                                                    :  6;
        unsigned int BVH_CNT                                                      :  3;
        unsigned int SAMPLE_CNT                                                   :  6;
        unsigned int DS_CNT                                                       :  6;
        unsigned int KM_CNT                                                       :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERF_SNAPSHOT_PC_HI {
    struct {
        unsigned int PC_HI                                                        : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_PERF_SNAPSHOT_PC_LO {
    struct {
        unsigned int PC_LO                                                        : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_RANDOM_WAVE_PRI {
    struct {
        unsigned int RET                                                          :  7;
        unsigned int RUI                                                          :  3;
        unsigned int RNG                                                          : 14;
        unsigned int                                                              :  7;
        unsigned int FORCE_IB_ARB_PRIO_MSK_VALID                                  :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_RUNTIME_CONFIG {
    struct {
        unsigned int UNUSED_REGISTER                                              :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_SHADER_CYCLES_HI {
    struct {
        unsigned int CYCLES_HI                                                    : 28;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_SHADER_CYCLES_LO {
    struct {
        unsigned int CYCLES_LO                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_SHADER_TBA_HI {
    struct {
        unsigned int ADDR_HI                                                      :  8;
        unsigned int                                                              : 23;
        unsigned int TRAP_EN                                                      :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_SHADER_TBA_LO {
    struct {
        unsigned int ADDR_LO                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_SHADER_TMA_HI {
    struct {
        unsigned int ADDR_HI                                                      :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_SHADER_TMA_LO {
    struct {
        unsigned int ADDR_LO                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_TEX_CLK_CTRL {
    struct {
        unsigned int FORCE_WGP_ON_SA0                                             : 16;
        unsigned int FORCE_WGP_ON_SA1                                             : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_BUF0_BASE_HI {
    struct {
        unsigned int BASE_HI                                                      : 13;
        unsigned int                                                              : 19;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_BUF0_BASE_LO {
    struct {
        unsigned int BASE_LO                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_BUF0_SIZE {
    struct {
        unsigned int SIZE                                                         : 22;
        unsigned int                                                              : 10;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_BUF1_BASE_HI {
    struct {
        unsigned int BASE_HI                                                      : 13;
        unsigned int                                                              : 19;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_BUF1_BASE_LO {
    struct {
        unsigned int BASE_LO                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_BUF1_SIZE {
    struct {
        unsigned int SIZE                                                         : 22;
        unsigned int                                                              : 10;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_CTRL {
    struct {
        unsigned int MODE                                                         :  2;
        unsigned int                                                              :  1;
        unsigned int GL1_PERF_EN                                                  :  1;
        unsigned int INTERRUPT_EN                                                 :  1;
        unsigned int DOUBLE_BUFFER                                                :  1;
        unsigned int HIWATER                                                      :  3;
        unsigned int REG_AT_HWM                                                   :  2;
        unsigned int SPI_STALL_EN                                                 :  1;
        unsigned int SQ_STALL_EN                                                  :  1;
        unsigned int STALL_ALL_SIMDS                                              :  1;
        unsigned int UTIL_TIMER                                                   :  1;
        unsigned int WAVESTART_MODE                                               :  2;
        unsigned int                                                              :  1;
        unsigned int SYNC_COUNT_MARKERS                                           :  1;
        unsigned int SYNC_COUNT_DRAWS                                             :  1;
        unsigned int LOWATER_OFFSET                                               :  3;
        unsigned int GL1X_PREFETCH_PAGE                                           :  4;
        unsigned int                                                              :  1;
        unsigned int AUTO_FLUSH_PADDING_DIS                                       :  1;
        unsigned int AUTO_FLUSH_MODE                                              :  1;
        unsigned int NCP_REG_TOKEN_EN                                             :  1;
        unsigned int DRAW_EVENT_EN                                                :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_DROPPED_CNTR {
    struct {
        unsigned int CNTR                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_GFX_DRAW_CNTR {
    struct {
        unsigned int CNTR                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_GFX_MARKER_CNTR {
    struct {
        unsigned int CNTR                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_HALT {
    struct {
        unsigned int ENTER_CGCG                                                   :  1;
        unsigned int CGCG_READY                                                   :  1;
        unsigned int ENTER_POWEROFF                                               :  1;
        unsigned int POWEROFF_READY                                               :  1;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_HP3D_DRAW_CNTR {
    struct {
        unsigned int CNTR                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_HP3D_MARKER_CNTR {
    struct {
        unsigned int CNTR                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_MASK {
    struct {
        unsigned int SIMD_SEL                                                     :  2;
        unsigned int                                                              :  2;
        unsigned int WGP_SEL                                                      :  4;
        unsigned int                                                              :  1;
        unsigned int SA_SEL                                                       :  1;
        unsigned int WTYPE_INCLUDE                                                :  7;
        unsigned int EXCLUDE_NONDETAIL_SHADERDATA                                 :  1;
        unsigned int EXCLUDE_NONDETAIL_WAVESTART_EXT                              :  1;
        unsigned int EXCLUDE_NONDETAIL_ALLOC                                      :  1;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_POWEROFF_RESTORE_1 {
    struct {
        unsigned int STATES                                                       : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_STATUS {
    struct {
        unsigned int FINISH_PENDING                                               : 12;
        unsigned int FINISH_DONE                                                  : 12;
        unsigned int WRITE_ERROR                                                  :  1;
        unsigned int BUSY                                                         :  1;
        unsigned int                                                              :  2;
        unsigned int OWNER_VMID                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_STATUS2 {
    struct {
        unsigned int BUF0_FULL                                                    :  1;
        unsigned int BUF1_FULL                                                    :  1;
        unsigned int                                                              :  2;
        unsigned int PACKET_LOST_BUF_NO_LOCKDOWN                                  :  1;
        unsigned int                                                              :  3;
        unsigned int BUF_ISSUE_STATUS                                             :  5;
        unsigned int BUF_ISSUE                                                    :  1;
        unsigned int WRITE_BUF_FULL                                               :  1;
        unsigned int                                                              : 17;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_TOKEN_MASK {
    struct {
        unsigned int TOKEN_EXCLUDE                                                : 12;
        unsigned int TTRACE_EXEC                                                  :  1;
        unsigned int BOP_EVENTS_TOKEN_INCLUDE                                     :  1;
        unsigned int EXCLUDE_BARRIER_WAIT                                         :  1;
        unsigned int                                                              :  1;
        unsigned int REG_INCLUDE                                                  :  8;
        unsigned int INST_EXCLUDE                                                 :  2;
        unsigned int REG_EXCLUDE                                                  :  3;
        unsigned int                                                              :  2;
        unsigned int REG_DETAIL_ALL                                               :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_USERDATA_0 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_USERDATA_1 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_USERDATA_2 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_USERDATA_3 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_USERDATA_4 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_USERDATA_5 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_USERDATA_6 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_USERDATA_7 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_THREAD_TRACE_WPTR {
    struct {
        unsigned int OFFSET                                                       : 29;
        unsigned int                                                              :  2;
        unsigned int BUFFER_ID                                                    :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WATCH0_ADDR_H {
    struct {
        unsigned int ADDR                                                         : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WATCH0_ADDR_L {
    struct {
        unsigned int                                                              :  6;
        unsigned int ADDR                                                         : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WATCH0_CNTL {
    struct {
        unsigned int MASK                                                         : 24;
        unsigned int VMID                                                         :  4;
        unsigned int                                                              :  3;
        unsigned int VALID                                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WATCH1_ADDR_H {
    struct {
        unsigned int ADDR                                                         : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WATCH1_ADDR_L {
    struct {
        unsigned int                                                              :  6;
        unsigned int ADDR                                                         : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WATCH1_CNTL {
    struct {
        unsigned int MASK                                                         : 24;
        unsigned int VMID                                                         :  4;
        unsigned int                                                              :  3;
        unsigned int VALID                                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WATCH2_ADDR_H {
    struct {
        unsigned int ADDR                                                         : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WATCH2_ADDR_L {
    struct {
        unsigned int                                                              :  6;
        unsigned int ADDR                                                         : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WATCH2_CNTL {
    struct {
        unsigned int MASK                                                         : 24;
        unsigned int VMID                                                         :  4;
        unsigned int                                                              :  3;
        unsigned int VALID                                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WATCH3_ADDR_H {
    struct {
        unsigned int ADDR                                                         : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WATCH3_ADDR_L {
    struct {
        unsigned int                                                              :  6;
        unsigned int ADDR                                                         : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WATCH3_CNTL {
    struct {
        unsigned int MASK                                                         : 24;
        unsigned int VMID                                                         :  4;
        unsigned int                                                              :  3;
        unsigned int VALID                                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WAVE_ACTIVE {
    struct {
        unsigned int WAVE_SLOT                                                    : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WAVE_DVGPR_ALLOC_HI {
    struct {
        unsigned int SEGMENT4                                                     :  7;
        unsigned int                                                              :  1;
        unsigned int SEGMENT5                                                     :  7;
        unsigned int                                                              :  1;
        unsigned int SEGMENT6                                                     :  7;
        unsigned int                                                              :  1;
        unsigned int SEGMENT7                                                     :  7;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WAVE_DVGPR_ALLOC_LO {
    struct {
        unsigned int SEGMENT0                                                     :  7;
        unsigned int                                                              :  1;
        unsigned int SEGMENT1                                                     :  7;
        unsigned int                                                              :  1;
        unsigned int SEGMENT2                                                     :  7;
        unsigned int                                                              :  1;
        unsigned int SEGMENT3                                                     :  7;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WAVE_EXCP_FLAG_PRIV {
    struct {
        unsigned int ADDR_WATCH                                                   :  4;
        unsigned int MEMVIOL                                                      :  1;
        unsigned int SAVE_CONTEXT                                                 :  1;
        unsigned int ILLEGAL_INST                                                 :  1;
        unsigned int HOST_TRAP                                                    :  1;
        unsigned int WAVE_START                                                   :  1;
        unsigned int WAVE_END                                                     :  1;
        unsigned int PERF_SNAPSHOT                                                :  1;
        unsigned int TRAP_AFTER_INST                                              :  1;
        unsigned int XNACK_ERROR                                                  :  1;
        unsigned int                                                              : 17;
        unsigned int FIRST_MEMVIOL_SOURCE                                         :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WAVE_EXCP_FLAG_USER {
    struct {
        unsigned int ALU_INVALID                                                  :  1;
        unsigned int ALU_INPUT_DENORM                                             :  1;
        unsigned int ALU_FLOAT_DIV0                                               :  1;
        unsigned int ALU_OVERFLOW                                                 :  1;
        unsigned int ALU_UNDERFLOW                                                :  1;
        unsigned int ALU_INEXACT                                                  :  1;
        unsigned int ALU_INT_DIV0                                                 :  1;
        unsigned int                                                              : 23;
        unsigned int BUFFER_OOB                                                   :  1;
        unsigned int LOD_CLAMPED                                                  :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WAVE_EXEC_HI {
    struct {
        unsigned int EXEC_HI                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WAVE_EXEC_LO {
    struct {
        unsigned int EXEC_LO                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WAVE_FLUSH_IB {
    struct {
        unsigned int UNUSED                                                       : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WAVE_GPR_ALLOC {
    struct {
        unsigned int VGPR_BASE                                                    :  9;
        unsigned int                                                              :  3;
        unsigned int VGPR_SIZE                                                    :  8;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WAVE_HW_ID1 {
    struct {
        unsigned int WAVE_ID                                                      :  5;
        unsigned int                                                              :  3;
        unsigned int SIMD_ID                                                      :  2;
        unsigned int WGP_ID                                                       :  4;
        unsigned int                                                              :  2;
        unsigned int SA_ID                                                        :  1;
        unsigned int                                                              :  1;
        unsigned int SE_ID                                                        :  3;
        unsigned int                                                              :  8;
        unsigned int DP_RATE                                                      :  3;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WAVE_HW_ID2 {
    struct {
        unsigned int QUEUE_ID                                                     :  4;
        unsigned int PIPE_ID                                                      :  2;
        unsigned int                                                              :  2;
        unsigned int ME_ID                                                        :  2;
        unsigned int                                                              :  2;
        unsigned int STATE_ID                                                     :  3;
        unsigned int                                                              :  1;
        unsigned int WG_ID                                                        :  5;
        unsigned int                                                              :  3;
        unsigned int VM_ID                                                        :  4;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WAVE_IB_STS {
    struct {
        unsigned int EXP_CNT                                                      :  3;
        unsigned int DS_CNT                                                       :  6;
        unsigned int LOAD_CNT                                                     :  6;
        unsigned int SAMPLE_CNT                                                   :  6;
        unsigned int BVH_CNT                                                      :  3;
        unsigned int STORE_CNT                                                    :  6;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WAVE_IB_STS2 {
    struct {
        unsigned int KM_CNT                                                       :  5;
        unsigned int                                                              : 23;
        unsigned int INST_PREFETCH                                                :  2;
        unsigned int FWD_PROGRESS                                                 :  1;
        unsigned int TTRACE_EN_SPI                                                :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WAVE_LDS_ALLOC {
    struct {
        unsigned int LDS_BASE                                                     :  8;
        unsigned int                                                              :  4;
        unsigned int LDS_SIZE                                                     :  9;
        unsigned int                                                              :  3;
        unsigned int VGPR_SHARED_SIZE                                             :  4;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WAVE_M0 {
    struct {
        unsigned int M0                                                           : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WAVE_MODE {
    struct {
        unsigned int FP_ROUND                                                     :  4;
        unsigned int FP_DENORM                                                    :  4;
        unsigned int                                                              : 15;
        unsigned int FP16_OVFL                                                    :  1;
        unsigned int SCALAR_PREFETCH_EN                                           :  1;
        unsigned int                                                              :  2;
        unsigned int DISABLE_PERF                                                 :  1;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WAVE_PC_HI {
    struct {
        unsigned int PC_HI                                                        : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WAVE_PC_LO {
    struct {
        unsigned int PC_LO                                                        : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WAVE_SCHED_MODE {
    struct {
        unsigned int DEP_MODE                                                     :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WAVE_SCRATCH_BASE_HI {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WAVE_SCRATCH_BASE_LO {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WAVE_STATE_PRIV {
    struct {
        unsigned int WG_RR_EN                                                     :  1;
        unsigned int SLEEP_WAKEUP                                                 :  1;
        unsigned int BARRIER_COMPLETE                                             :  1;
        unsigned int NAMED_BARRIER_COMPLETE                                       :  1;
        unsigned int NAMED_BARRIER_ID                                             :  5;
        unsigned int SCC                                                          :  1;
        unsigned int SYS_PRIO                                                     :  2;
        unsigned int USER_PRIO                                                    :  2;
        unsigned int HALT                                                         :  1;
        unsigned int POISON_ERR                                                   :  1;
        unsigned int COND_DBG_USER                                                :  1;
        unsigned int COND_DBG_SYS                                                 :  1;
        unsigned int SCRATCH_EN                                                   :  1;
        unsigned int PERF_EN                                                      :  1;
        unsigned int TTRACE_EN                                                    :  1;
        unsigned int                                                              : 11;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WAVE_STATUS {
    struct {
        unsigned int                                                              :  5;
        unsigned int PRIV                                                         :  1;
        unsigned int TRAP_EN                                                      :  1;
        unsigned int                                                              :  1;
        unsigned int EXPORT_RDY                                                   :  1;
        unsigned int EXECZ                                                        :  1;
        unsigned int VCCZ                                                         :  1;
        unsigned int IN_WG                                                        :  1;
        unsigned int                                                              :  2;
        unsigned int TRAP                                                         :  1;
        unsigned int TRAP_BARRIER_COMPLETE                                        :  1;
        unsigned int VALID                                                        :  1;
        unsigned int                                                              :  1;
        unsigned int SKIP_EXPORT                                                  :  1;
        unsigned int                                                              :  3;
        unsigned int OREO_CONFLICT                                                :  1;
        unsigned int FATAL_HALT                                                   :  1;
        unsigned int NO_VGPRS                                                     :  1;
        unsigned int LDS_PARAM_READY                                              :  1;
        unsigned int MUST_GS_ALLOC                                                :  1;
        unsigned int MUST_EXPORT                                                  :  1;
        unsigned int IDLE                                                         :  1;
        unsigned int WAVE64                                                       :  1;
        unsigned int DVGPR_EN                                                     :  1;
        unsigned int WGP_TAKEOVER                                                 :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WAVE_TRAP_CTRL {
    struct {
        unsigned int ALU_INVALID                                                  :  1;
        unsigned int ALU_INPUT_DENORM                                             :  1;
        unsigned int ALU_FLOAT_DIV0                                               :  1;
        unsigned int ALU_OVERFLOW                                                 :  1;
        unsigned int ALU_UNDERFLOW                                                :  1;
        unsigned int ALU_INEXACT                                                  :  1;
        unsigned int ALU_INT_DIV0                                                 :  1;
        unsigned int ADDR_WATCH                                                   :  1;
        unsigned int WAVE_END                                                     :  1;
        unsigned int TRAP_AFTER_INST                                              :  1;
        unsigned int                                                              : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WAVE_TTMP0 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WAVE_TTMP1 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WAVE_TTMP2 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WAVE_TTMP3 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WAVE_TTMP4 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WAVE_TTMP5 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WAVE_TTMP6 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WAVE_TTMP7 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WAVE_TTMP8 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WAVE_TTMP9 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WAVE_TTMP10 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WAVE_TTMP11 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WAVE_TTMP12 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WAVE_TTMP13 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WAVE_TTMP14 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WAVE_TTMP15 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SQ_WAVE_VALID_AND_IDLE {
    struct {
        unsigned int WAVE_SLOT                                                    : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_BLEND_OPT_CONTROL {
    struct {
        unsigned int MRT0_COLOR_OPT_DISABLE                                       :  1;
        unsigned int MRT0_ALPHA_OPT_DISABLE                                       :  1;
        unsigned int                                                              :  2;
        unsigned int MRT1_COLOR_OPT_DISABLE                                       :  1;
        unsigned int MRT1_ALPHA_OPT_DISABLE                                       :  1;
        unsigned int                                                              :  2;
        unsigned int MRT2_COLOR_OPT_DISABLE                                       :  1;
        unsigned int MRT2_ALPHA_OPT_DISABLE                                       :  1;
        unsigned int                                                              :  2;
        unsigned int MRT3_COLOR_OPT_DISABLE                                       :  1;
        unsigned int MRT3_ALPHA_OPT_DISABLE                                       :  1;
        unsigned int                                                              :  2;
        unsigned int MRT4_COLOR_OPT_DISABLE                                       :  1;
        unsigned int MRT4_ALPHA_OPT_DISABLE                                       :  1;
        unsigned int                                                              :  2;
        unsigned int MRT5_COLOR_OPT_DISABLE                                       :  1;
        unsigned int MRT5_ALPHA_OPT_DISABLE                                       :  1;
        unsigned int                                                              :  2;
        unsigned int MRT6_COLOR_OPT_DISABLE                                       :  1;
        unsigned int MRT6_ALPHA_OPT_DISABLE                                       :  1;
        unsigned int                                                              :  2;
        unsigned int MRT7_COLOR_OPT_DISABLE                                       :  1;
        unsigned int MRT7_ALPHA_OPT_DISABLE                                       :  1;
        unsigned int                                                              :  1;
        unsigned int PIXEN_ZERO_OPT_DISABLE                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_BLEND_OPT_EPSILON {
    struct {
        unsigned int MRT0_EPSILON                                                 :  4;
        unsigned int MRT1_EPSILON                                                 :  4;
        unsigned int MRT2_EPSILON                                                 :  4;
        unsigned int MRT3_EPSILON                                                 :  4;
        unsigned int MRT4_EPSILON                                                 :  4;
        unsigned int MRT5_EPSILON                                                 :  4;
        unsigned int MRT6_EPSILON                                                 :  4;
        unsigned int MRT7_EPSILON                                                 :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_MRT0_BLEND_OPT {
    struct {
        unsigned int COLOR_SRC_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int COLOR_DST_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int COLOR_COMB_FCN                                               :  3;
        unsigned int                                                              :  5;
        unsigned int ALPHA_SRC_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int ALPHA_DST_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int ALPHA_COMB_FCN                                               :  3;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_MRT1_BLEND_OPT {
    struct {
        unsigned int COLOR_SRC_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int COLOR_DST_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int COLOR_COMB_FCN                                               :  3;
        unsigned int                                                              :  5;
        unsigned int ALPHA_SRC_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int ALPHA_DST_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int ALPHA_COMB_FCN                                               :  3;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_MRT2_BLEND_OPT {
    struct {
        unsigned int COLOR_SRC_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int COLOR_DST_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int COLOR_COMB_FCN                                               :  3;
        unsigned int                                                              :  5;
        unsigned int ALPHA_SRC_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int ALPHA_DST_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int ALPHA_COMB_FCN                                               :  3;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_MRT3_BLEND_OPT {
    struct {
        unsigned int COLOR_SRC_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int COLOR_DST_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int COLOR_COMB_FCN                                               :  3;
        unsigned int                                                              :  5;
        unsigned int ALPHA_SRC_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int ALPHA_DST_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int ALPHA_COMB_FCN                                               :  3;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_MRT4_BLEND_OPT {
    struct {
        unsigned int COLOR_SRC_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int COLOR_DST_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int COLOR_COMB_FCN                                               :  3;
        unsigned int                                                              :  5;
        unsigned int ALPHA_SRC_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int ALPHA_DST_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int ALPHA_COMB_FCN                                               :  3;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_MRT5_BLEND_OPT {
    struct {
        unsigned int COLOR_SRC_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int COLOR_DST_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int COLOR_COMB_FCN                                               :  3;
        unsigned int                                                              :  5;
        unsigned int ALPHA_SRC_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int ALPHA_DST_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int ALPHA_COMB_FCN                                               :  3;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_MRT6_BLEND_OPT {
    struct {
        unsigned int COLOR_SRC_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int COLOR_DST_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int COLOR_COMB_FCN                                               :  3;
        unsigned int                                                              :  5;
        unsigned int ALPHA_SRC_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int ALPHA_DST_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int ALPHA_COMB_FCN                                               :  3;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_MRT7_BLEND_OPT {
    struct {
        unsigned int COLOR_SRC_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int COLOR_DST_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int COLOR_COMB_FCN                                               :  3;
        unsigned int                                                              :  5;
        unsigned int ALPHA_SRC_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int ALPHA_DST_OPT                                                :  3;
        unsigned int                                                              :  1;
        unsigned int ALPHA_COMB_FCN                                               :  3;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_PERFCOUNTER1_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_PERFCOUNTER2_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_PERFCOUNTER3_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_PERFCOUNTER3_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_PERFCOUNTER3_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_PS_DOWNCONVERT {
    struct {
        unsigned int MRT0                                                         :  4;
        unsigned int MRT1                                                         :  4;
        unsigned int MRT2                                                         :  4;
        unsigned int MRT3                                                         :  4;
        unsigned int MRT4                                                         :  4;
        unsigned int MRT5                                                         :  4;
        unsigned int MRT6                                                         :  4;
        unsigned int MRT7                                                         :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union SX_PS_DOWNCONVERT_CONTROL {
    struct {
        unsigned int MRT0_FMT_MAPPING_DISABLE                                     :  1;
        unsigned int MRT1_FMT_MAPPING_DISABLE                                     :  1;
        unsigned int MRT2_FMT_MAPPING_DISABLE                                     :  1;
        unsigned int MRT3_FMT_MAPPING_DISABLE                                     :  1;
        unsigned int MRT4_FMT_MAPPING_DISABLE                                     :  1;
        unsigned int MRT5_FMT_MAPPING_DISABLE                                     :  1;
        unsigned int MRT6_FMT_MAPPING_DISABLE                                     :  1;
        unsigned int MRT7_FMT_MAPPING_DISABLE                                     :  1;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TA_BC_BASE_ADDR {
    struct {
        unsigned int ADDRESS                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TA_BC_BASE_ADDR_HI {
    struct {
        unsigned int ADDRESS                                                      :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TA_CNTL {
    struct {
        unsigned int TA_SQ_XNACK_FGCG_DISABLE                                     :  1;
        unsigned int                                                              :  1;
        unsigned int TA_INPUT_RDATA_PER_BANK_FGCG_OVERRIDE                        :  1;
        unsigned int TA_INPUT_CFIFO_VEC64_OPT_OVERRIDE                            :  1;
        unsigned int                                                              : 12;
        unsigned int ALIGNER_CREDIT                                               :  5;
        unsigned int                                                              :  1;
        unsigned int TD_FIFO_CREDIT                                               : 10;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TA_CNTL2 {
    struct {
        unsigned int STORE_COMPONENT_MODE                                         :  1;
        unsigned int                                                              :  3;
        unsigned int MAX_RQ_ID                                                    :  3;
        unsigned int                                                              : 10;
        unsigned int ELEMSIZE_HASH_DIS                                            :  1;
        unsigned int TRUNCATE_COORD_MODE                                          :  1;
        unsigned int ELIMINATE_UNLIT_QUAD_DIS                                     :  1;
        unsigned int PRTPLUS_ACCUM_MODE                                           :  2;
        unsigned int                                                              : 10;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TA_CNTL_AUX {
    struct {
        unsigned int SCOAL_DSWIZZLE_N                                             :  1;
        unsigned int DEPTH_AS_PITCH_DIS                                           :  1;
        unsigned int CORNER_SAMPLES_MIN_DIM                                       :  1;
        unsigned int OVERRIDE_QUAD_MODE_DIS                                       :  1;
        unsigned int DERIV_ADJUST_DIS                                             :  1;
        unsigned int TFAULT_EN_OVERRIDE                                           :  1;
        unsigned int GATHERH_DST_SEL                                              :  1;
        unsigned int DISABLE_GATHER4_BC_SWIZZLE                                   :  1;
        unsigned int ANISO_MAG_STEP_CLAMP                                         :  1;
        unsigned int AUTO_ALIGN_FORMAT                                            :  1;
        unsigned int ANISO_HALF_THRESH                                            :  2;
        unsigned int ANISO_ERROR_FP_VBIAS                                         :  1;
        unsigned int ANISO_STEP_ORDER                                             :  1;
        unsigned int ANISO_STEP                                                   :  1;
        unsigned int MINMAG_UNNORM                                                :  1;
        unsigned int ANISO_WEIGHT_MODE                                            :  1;
        unsigned int ANISO_RATIO_LUT                                              :  1;
        unsigned int ANISO_TAP                                                    :  1;
        unsigned int                                                              :  1;
        unsigned int DETERMINISM_RESERVED_DISABLE                                 :  1;
        unsigned int DETERMINISM_OPCODE_STRICT_DISABLE                            :  1;
        unsigned int DETERMINISM_MISC_DISABLE                                     :  1;
        unsigned int DETERMINISM_SAMPLE_C_DFMT_DISABLE                            :  1;
        unsigned int DETERMINISM_SAMPLER_MSAA_DISABLE                             :  1;
        unsigned int DETERMINISM_WRITEOP_READFMT_DISABLE                          :  1;
        unsigned int DETERMINISM_DFMT_NFMT_DISABLE                                :  1;
        unsigned int                                                              :  1;
        unsigned int CUBEMAP_SLICE_CLAMP                                          :  1;
        unsigned int TRUNC_SMALL_NEG                                              :  1;
        unsigned int ARRAY_ROUND_MODE                                             :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TA_CS_BC_BASE_ADDR {
    struct {
        unsigned int ADDRESS                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TA_CS_BC_BASE_ADDR_HI {
    struct {
        unsigned int ADDRESS                                                      :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TA_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TA_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TA_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TA_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TA_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TA_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TA_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TA_SCRATCH {
    struct {
        unsigned int SCRATCH                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TA_STATUS {
    struct {
        unsigned int                                                              : 12;
        unsigned int FG_PFIFO_EMPTYB                                              :  1;
        unsigned int FG_LFIFO_EMPTYB                                              :  1;
        unsigned int FG_SFIFO_EMPTYB                                              :  1;
        unsigned int                                                              :  1;
        unsigned int FL_PFIFO_EMPTYB                                              :  1;
        unsigned int FL_LFIFO_EMPTYB                                              :  1;
        unsigned int FL_SFIFO_EMPTYB                                              :  1;
        unsigned int                                                              :  1;
        unsigned int FA_PFIFO_EMPTYB                                              :  1;
        unsigned int FA_LFIFO_EMPTYB                                              :  1;
        unsigned int FA_SFIFO_EMPTYB                                              :  1;
        unsigned int                                                              :  1;
        unsigned int IN_BUSY                                                      :  1;
        unsigned int FG_BUSY                                                      :  1;
        unsigned int LA_BUSY                                                      :  1;
        unsigned int FL_BUSY                                                      :  1;
        unsigned int TA_BUSY                                                      :  1;
        unsigned int FA_BUSY                                                      :  1;
        unsigned int AL_BUSY                                                      :  1;
        unsigned int BUSY                                                         :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCP_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCP_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCP_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCP_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCP_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCP_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCP_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCP_PERFCOUNTER1_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCP_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCP_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCP_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 18;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCP_PERFCOUNTER3_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCP_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCP_PERFCOUNTER3_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 18;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCP_PERFCOUNTER_FILTER {
    struct {
        unsigned int BUFFER                                                       :  1;
        unsigned int FLAT                                                         :  1;
        unsigned int DIM                                                          :  3;
        unsigned int DATA_FORMAT                                                  :  7;
        unsigned int COMPRESSION_ENABLE                                           :  1;
        unsigned int NUM_FORMAT                                                   :  4;
        unsigned int SW_MODE                                                      :  5;
        unsigned int NUM_SAMPLES                                                  :  2;
        unsigned int OPCODE_TYPE                                                  :  3;
        unsigned int TMPRL                                                        :  3;
        unsigned int SCOPE                                                        :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCP_PERFCOUNTER_FILTER2 {
    struct {
        unsigned int REQ_MODE                                                     :  3;
        unsigned int                                                              : 29;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TCP_PERFCOUNTER_FILTER_EN {
    struct {
        unsigned int BUFFER                                                       :  1;
        unsigned int FLAT                                                         :  1;
        unsigned int DIM                                                          :  1;
        unsigned int DATA_FORMAT                                                  :  1;
        unsigned int NUM_FORMAT                                                   :  1;
        unsigned int SW_MODE                                                      :  1;
        unsigned int NUM_SAMPLES                                                  :  1;
        unsigned int OPCODE_TYPE                                                  :  1;
        unsigned int TMPRL                                                        :  1;
        unsigned int                                                              :  2;
        unsigned int COMPRESSION_ENABLE                                           :  1;
        unsigned int REQ_MODE                                                     :  1;
        unsigned int SCOPE                                                        :  1;
        unsigned int                                                              : 18;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TD_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TD_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TD_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int PERF_MODE1                                                   :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TD_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE3                                                   :  4;
        unsigned int PERF_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TD_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TD_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union TD_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 10;
        unsigned int CNTR_MODE                                                    :  4;
        unsigned int                                                              :  4;
        unsigned int PERF_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UTCL1_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UTCL1_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UTCL1_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 18;
        unsigned int COUNTER_MODE                                                 :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UTCL1_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UTCL1_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UTCL1_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 18;
        unsigned int COUNTER_MODE                                                 :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UTCL1_PERFCOUNTER2_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UTCL1_PERFCOUNTER2_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UTCL1_PERFCOUNTER2_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 18;
        unsigned int COUNTER_MODE                                                 :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UTCL1_PERFCOUNTER3_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UTCL1_PERFCOUNTER3_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union UTCL1_PERFCOUNTER3_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 18;
        unsigned int COUNTER_MODE                                                 :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_DMA_BASE {
    struct {
        unsigned int BASE_ADDR                                                    : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_DMA_BASE_HI {
    struct {
        unsigned int BASE_ADDR                                                    : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_DMA_DATA_FIFO_DEPTH {
    struct {
        unsigned int DMA_DATA_FIFO_DEPTH                                          : 10;
        unsigned int                                                              : 22;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_DMA_INDEX_TYPE {
    struct {
        unsigned int INDEX_TYPE                                                   :  2;
        unsigned int                                                              :  4;
        unsigned int RDREQ_POLICY                                                 :  2;
        unsigned int                                                              :  1;
        unsigned int NOT_EOP                                                      :  1;
        unsigned int                                                              :  1;
        unsigned int MTYPE                                                        :  3;
        unsigned int DISABLE_INSTANCE_PACKING                                     :  1;
        unsigned int TEMPORAL                                                     :  2;
        unsigned int SPEC_DATA_READ                                               :  2;
        unsigned int                                                              : 13;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_DMA_MAX_SIZE {
    struct {
        unsigned int MAX_SIZE                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_DMA_NUM_INSTANCES {
    struct {
        unsigned int NUM_INSTANCES                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_DMA_REQ_FIFO_DEPTH {
    struct {
        unsigned int DMA_REQ_FIFO_DEPTH                                           :  6;
        unsigned int                                                              : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_DMA_SIZE {
    struct {
        unsigned int NUM_INDICES                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_DRAW_INITIATOR {
    struct {
        unsigned int SOURCE_SELECT                                                :  2;
        unsigned int                                                              :  3;
        unsigned int NOT_EOP                                                      :  1;
        unsigned int USE_OPAQUE                                                   :  1;
        unsigned int                                                              : 22;
        unsigned int REG_RT_INDEX                                                 :  3;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_DRAW_INIT_FIFO_DEPTH {
    struct {
        unsigned int DRAW_INIT_FIFO_DEPTH                                         :  6;
        unsigned int                                                              : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_DRAW_PAYLOAD_CNTL {
    struct {
        unsigned int                                                              :  1;
        unsigned int EN_REG_RT_INDEX                                              :  1;
        unsigned int                                                              :  1;
        unsigned int EN_PRIM_PAYLOAD                                              :  1;
        unsigned int EN_DRAW_VP                                                   :  1;
        unsigned int UNUSED                                                       :  1;
        unsigned int EN_VRS_RATE                                                  :  1;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_EVENT_ADDRESS_REG {
    struct {
        unsigned int ADDRESS_LOW                                                  : 28;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_EVENT_INITIATOR {
    struct {
        unsigned int EVENT_TYPE                                                   :  6;
        unsigned int                                                              :  4;
        unsigned int ADDRESS_HI                                                   : 17;
        unsigned int EXTENDED_EVENT                                               :  1;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_GS_INSTANCE_CNT {
    struct {
        unsigned int ENABLE                                                       :  1;
        unsigned int                                                              :  1;
        unsigned int CNT                                                          :  6;
        unsigned int                                                              : 23;
        unsigned int EN_MAX_VERT_OUT_PER_GS_INSTANCE                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_GS_MAX_VERT_OUT {
    struct {
        unsigned int MAX_VERT_OUT                                                 : 11;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_GS_MAX_WAVE_ID {
    struct {
        unsigned int MAX_WAVE_ID                                                  : 12;
        unsigned int                                                              : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_GS_OUT_PRIM_TYPE {
    struct {
        unsigned int OUTPRIM_TYPE                                                 :  6;
        unsigned int                                                              : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_HOS_MAX_TESS_LEVEL {
    struct {
        unsigned int MAX_TESS                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_HOS_MIN_TESS_LEVEL {
    struct {
        unsigned int MIN_TESS                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_HS_OFFCHIP_PARAM {
    struct {
        unsigned int OFFCHIP_BUFFERING                                            : 10;
        unsigned int OFFCHIP_GRANULARITY                                          :  2;
        unsigned int                                                              : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_INDEX_TYPE {
    struct {
        unsigned int INDEX_TYPE                                                   :  2;
        unsigned int                                                              : 12;
        unsigned int DISABLE_INSTANCE_PACKING                                     :  1;
        unsigned int                                                              : 17;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_INSTANCE_BASE_ID {
    struct {
        unsigned int INSTANCE_BASE_ID                                             : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_LS_HS_CONFIG {
    struct {
        unsigned int NUM_PATCHES                                                  :  8;
        unsigned int                                                              :  6;
        unsigned int HS_NUM_OUTPUT_CP                                             :  6;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_MC_LAT_CNTL {
    struct {
        unsigned int MC_TIME_STAMP_RES                                            :  4;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_MULTI_PRIM_IB_RESET_INDX {
    struct {
        unsigned int RESET_INDX                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_NUM_INDICES {
    struct {
        unsigned int NUM_INDICES                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_NUM_INSTANCES {
    struct {
        unsigned int NUM_INSTANCES                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_PRIMITIVEID_EN {
    struct {
        unsigned int                                                              :  2;
        unsigned int NGG_DISABLE_PROVOK_REUSE                                     :  1;
        unsigned int                                                              : 29;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_PRIMITIVEID_RESET {
    struct {
        unsigned int VALUE                                                        : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_PRIMITIVE_TYPE {
    struct {
        unsigned int PRIM_TYPE                                                    :  6;
        unsigned int NUM_INPUT_CP                                                 :  6;
        unsigned int PRIMS_PER_SUBGROUP                                           :  9;
        unsigned int                                                              : 11;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_REUSE_OFF {
    struct {
        unsigned int REUSE_OFF                                                    :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_SHADER_STAGES_EN {
    struct {
        unsigned int                                                              :  2;
        unsigned int HS_EN                                                        :  1;
        unsigned int                                                              :  2;
        unsigned int GS_EN                                                        :  1;
        unsigned int                                                              : 13;
        unsigned int GS_FAST_LAUNCH                                               :  1;
        unsigned int                                                              :  1;
        unsigned int HS_W32_EN                                                    :  1;
        unsigned int GS_W32_EN                                                    :  1;
        unsigned int                                                              :  1;
        unsigned int NGG_WAVE_ID_EN                                               :  1;
        unsigned int                                                              :  1;
        unsigned int PRIMGEN_PASSTHRU_NO_MSG                                      :  1;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_STRMOUT_DRAW_OPAQUE_BUFFER_FILLED_SIZE {
    struct {
        unsigned int SIZE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_STRMOUT_DRAW_OPAQUE_OFFSET {
    struct {
        unsigned int OFFSET                                                       : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_STRMOUT_DRAW_OPAQUE_VERTEX_STRIDE {
    struct {
        unsigned int VERTEX_STRIDE                                                :  9;
        unsigned int                                                              : 23;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_TESS_DISTRIBUTION {
    struct {
        unsigned int ACCUM_ISOLINE                                                :  8;
        unsigned int ACCUM_TRI                                                    :  8;
        unsigned int ACCUM_QUAD                                                   :  8;
        unsigned int DONUT_SPLIT                                                  :  5;
        unsigned int TRAP_SPLIT                                                   :  3;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_TF_MEMORY_BASE {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_TF_MEMORY_BASE_HI {
    struct {
        unsigned int BASE_HI                                                      :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_TF_PARAM {
    struct {
        unsigned int TYPE                                                         :  2;
        unsigned int PARTITIONING                                                 :  3;
        unsigned int TOPOLOGY                                                     :  3;
        unsigned int                                                              :  6;
        unsigned int DISABLE_DONUTS                                               :  1;
        unsigned int TEMPORAL                                                     :  2;
        unsigned int DISTRIBUTION_MODE                                            :  2;
        unsigned int DETECT_ONE                                                   :  1;
        unsigned int DETECT_ZERO                                                  :  1;
        unsigned int                                                              :  2;
        unsigned int MTYPE                                                        :  3;
        unsigned int                                                              :  2;
        unsigned int SPEC_DATA_READ                                               :  2;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union VGT_TF_RING_SIZE {
    struct {
        unsigned int SIZE                                                         : 17;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WD_UTCL1_CNTL {
    struct {
        unsigned int XNACK_REDO_TIMER_CNT                                         : 20;
        unsigned int                                                              :  3;
        unsigned int VMID_RESET_MODE                                              :  1;
        unsigned int DROP_MODE                                                    :  1;
        unsigned int BYPASS                                                       :  1;
        unsigned int INVALIDATE                                                   :  1;
        unsigned int FRAG_LIMIT_MODE                                              :  1;
        unsigned int FORCE_SNOOP                                                  :  1;
        unsigned int MTYPE_OVERRIDE                                               :  1;
        unsigned int LLC_NOALLOC_OVERRIDE                                         :  1;
        unsigned int                                                              :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WD_UTCL1_STATUS {
    struct {
        unsigned int FAULT_DETECTED                                               :  1;
        unsigned int RETRY_DETECTED                                               :  1;
        unsigned int PRT_DETECTED                                                 :  1;
        unsigned int                                                              :  1;
        unsigned int FAULT_VMID                                                   :  4;
        unsigned int FAULT_UTCL1ID                                                :  6;
        unsigned int FAULT_INSTANCEID                                             :  2;
        unsigned int RETRY_UTCL1ID                                                :  6;
        unsigned int RETRY_INSTANCEID                                             :  2;
        unsigned int PRT_UTCL1ID                                                  :  6;
        unsigned int PRT_INSTANCEID                                               :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_BUSY_HYSTERESIS {
    struct {
        unsigned int CAC_ACTIVE                                                   :  8;
        unsigned int CPC_BUSY                                                     :  8;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_BUSY_STAT {
    struct {
        unsigned int MEC1_LOAD_BUSY                                               :  1;
        unsigned int                                                              :  1;
        unsigned int MEC1_MUTEX_BUSY                                              :  1;
        unsigned int MEC1_MESSAGE_BUSY                                            :  1;
        unsigned int MEC1_EOP_QUEUE_BUSY                                          :  1;
        unsigned int MEC1_IQ_QUEUE_BUSY                                           :  1;
        unsigned int MEC1_IB_QUEUE_BUSY                                           :  1;
        unsigned int MEC1_TC_BUSY                                                 :  1;
        unsigned int MEC1_DMA_BUSY                                                :  1;
        unsigned int MEC1_PARTIAL_FLUSH_BUSY                                      :  1;
        unsigned int MEC1_PIPE0_BUSY                                              :  1;
        unsigned int MEC1_PIPE1_BUSY                                              :  1;
        unsigned int MEC1_PIPE2_BUSY                                              :  1;
        unsigned int MEC1_PIPE3_BUSY                                              :  1;
        unsigned int                                                              :  2;
        unsigned int MEC2_LOAD_BUSY                                               :  1;
        unsigned int                                                              :  1;
        unsigned int MEC2_MUTEX_BUSY                                              :  1;
        unsigned int MEC2_MESSAGE_BUSY                                            :  1;
        unsigned int MEC2_EOP_QUEUE_BUSY                                          :  1;
        unsigned int MEC2_IQ_QUEUE_BUSY                                           :  1;
        unsigned int MEC2_IB_QUEUE_BUSY                                           :  1;
        unsigned int MEC2_TC_BUSY                                                 :  1;
        unsigned int MEC2_DMA_BUSY                                                :  1;
        unsigned int MEC2_PARTIAL_FLUSH_BUSY                                      :  1;
        unsigned int MEC2_PIPE0_BUSY                                              :  1;
        unsigned int MEC2_PIPE1_BUSY                                              :  1;
        unsigned int MEC2_PIPE2_BUSY                                              :  1;
        unsigned int MEC2_PIPE3_BUSY                                              :  1;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_BUSY_STAT2 {
    struct {
        unsigned int MES_LOAD_BUSY                                                :  1;
        unsigned int                                                              :  1;
        unsigned int MES_MUTEX_BUSY                                               :  1;
        unsigned int MES_MESSAGE_BUSY                                             :  1;
        unsigned int                                                              :  3;
        unsigned int MES_TC_BUSY                                                  :  1;
        unsigned int MES_DMA_BUSY                                                 :  1;
        unsigned int                                                              :  1;
        unsigned int MES_PIPE0_BUSY                                               :  1;
        unsigned int MES_PIPE1_BUSY                                               :  1;
        unsigned int MES_PIPE2_BUSY                                               :  1;
        unsigned int MES_PIPE3_BUSY                                               :  1;
        unsigned int                                                              :  4;
        unsigned int MEC1_PIPE0_DC_BUSY                                           :  1;
        unsigned int MEC1_PIPE1_DC_BUSY                                           :  1;
        unsigned int MEC1_PIPE2_DC_BUSY                                           :  1;
        unsigned int MEC1_PIPE3_DC_BUSY                                           :  1;
        unsigned int                                                              : 10;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_DDID_INDEX {
    struct {
        unsigned int INDEX                                                        : 11;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_DESTINATION_EN_SE0 {
    struct {
        unsigned int CU_EN                                                        : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_DESTINATION_EN_SE1 {
    struct {
        unsigned int CU_EN                                                        : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_DESTINATION_EN_SE2 {
    struct {
        unsigned int CU_EN                                                        : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_DESTINATION_EN_SE3 {
    struct {
        unsigned int CU_EN                                                        : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_DIM_X {
    struct {
        unsigned int SIZE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_DIM_Y {
    struct {
        unsigned int SIZE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_DIM_Z {
    struct {
        unsigned int SIZE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_DISPATCH_END {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_DISPATCH_ID {
    struct {
        unsigned int DISPATCH_ID                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_DISPATCH_INITIATOR {
    struct {
        unsigned int COMPUTE_SHADER_EN                                            :  1;
        unsigned int PARTIAL_TG_EN                                                :  1;
        unsigned int FORCE_START_AT_000                                           :  1;
        unsigned int ORDERED_APPEND_ENBL                                          :  1;
        unsigned int ORDERED_APPEND_MODE                                          :  1;
        unsigned int USE_THREAD_DIMENSIONS                                        :  1;
        unsigned int ORDER_MODE                                                   :  1;
        unsigned int                                                              :  3;
        unsigned int SCALAR_L1_INV_VOL                                            :  1;
        unsigned int VECTOR_L1_INV_VOL                                            :  1;
        unsigned int PING_PONG_EN                                                 :  1;
        unsigned int TUNNEL_ENABLE                                                :  1;
        unsigned int RESTORE                                                      :  1;
        unsigned int CS_W32_EN                                                    :  1;
        unsigned int AMP_SHADER_EN                                                :  1;
        unsigned int DISABLE_DISP_PREMPT_EN                                       :  1;
        unsigned int INTERLEAVE_2D_EN                                             :  1;
        unsigned int WGS_DISPATCH                                                 :  1;
        unsigned int                                                              :  9;
        unsigned int TTRACE_QUEUE_ID                                              :  3;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_DISPATCH_INTERLEAVE {
    struct {
        unsigned int INTERLEAVE_1D                                                : 10;
        unsigned int                                                              :  6;
        unsigned int INTERLEAVE_2D_X_SIZE                                         :  4;
        unsigned int                                                              :  4;
        unsigned int INTERLEAVE_2D_Y_SIZE                                         :  4;
        unsigned int                                                              :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_DISPATCH_PKT_ADDR_HI {
    struct {
        unsigned int DATA                                                         :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_DISPATCH_PKT_ADDR_LO {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_DISPATCH_SCRATCH_BASE_HI {
    struct {
        unsigned int DATA                                                         :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_DISPATCH_SCRATCH_BASE_LO {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_DISPATCH_TUNNEL {
    struct {
        unsigned int OFF_DELAY                                                    : 13;
        unsigned int                                                              : 19;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_MISC_RESERVED {
    struct {
        unsigned int SEND_SEID                                                    :  4;
        unsigned int RESERVED4                                                    :  1;
        unsigned int WAVE_ID_BASE                                                 : 12;
        unsigned int                                                              : 15;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_NOWHERE {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_NUM_THREAD_X {
    struct {
        unsigned int NUM_THREAD_FULL                                              : 13;
        unsigned int INTERLEAVE_BITS_X                                            :  3;
        unsigned int NUM_THREAD_PARTIAL                                           : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_NUM_THREAD_Y {
    struct {
        unsigned int NUM_THREAD_FULL                                              : 13;
        unsigned int INTERLEAVE_BITS_Y                                            :  3;
        unsigned int NUM_THREAD_PARTIAL                                           : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_NUM_THREAD_Z {
    struct {
        unsigned int NUM_THREAD_FULL                                              : 16;
        unsigned int NUM_THREAD_PARTIAL                                           : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_PERFCOUNT_ENABLE {
    struct {
        unsigned int PERFCOUNT_ENABLE                                             :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_PGM_HI {
    struct {
        unsigned int DATA                                                         :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_PGM_LO {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_PGM_RSRC1 {
    struct {
        unsigned int VGPRS                                                        :  6;
        unsigned int SGPRS                                                        :  4;
        unsigned int PRIORITY                                                     :  2;
        unsigned int FLOAT_MODE                                                   :  8;
        unsigned int PRIV                                                         :  1;
        unsigned int WG_RR_EN                                                     :  1;
        unsigned int DEBUG_MODE                                                   :  1;
        unsigned int DISABLE_PERF                                                 :  1;
        unsigned int BULKY                                                        :  1;
        unsigned int CDBG_USER                                                    :  1;
        unsigned int FP16_OVFL                                                    :  1;
        unsigned int                                                              :  2;
        unsigned int WGP_MODE                                                     :  1;
        unsigned int                                                              :  1;
        unsigned int FWD_PROGRESS                                                 :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_PGM_RSRC2 {
    struct {
        unsigned int SCRATCH_EN                                                   :  1;
        unsigned int USER_SGPR                                                    :  5;
        unsigned int DYNAMIC_VGPR                                                 :  1;
        unsigned int TGID_X_EN                                                    :  1;
        unsigned int TGID_Y_EN                                                    :  1;
        unsigned int TGID_Z_EN                                                    :  1;
        unsigned int TG_SIZE_EN                                                   :  1;
        unsigned int TIDIG_COMP_CNT                                               :  2;
        unsigned int EXCP_EN_MSB                                                  :  2;
        unsigned int LDS_SIZE                                                     :  9;
        unsigned int EXCP_EN                                                      :  7;
        unsigned int WGP_TAKEOVER                                                 :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_PGM_RSRC3 {
    struct {
        unsigned int SHARED_VGPR_CNT                                              :  4;
        unsigned int INST_PREF_SIZE                                               :  8;
        unsigned int                                                              :  1;
        unsigned int GLG_EN                                                       :  1;
        unsigned int                                                              : 17;
        unsigned int IMAGE_OP                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_PIPELINESTAT_ENABLE {
    struct {
        unsigned int PIPELINESTAT_ENABLE                                          :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_PRESCALED_DIM_X {
    struct {
        unsigned int SIZE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_PRESCALED_DIM_Y {
    struct {
        unsigned int SIZE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_PRESCALED_DIM_Z {
    struct {
        unsigned int SIZE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_RELAUNCH {
    struct {
        unsigned int PAYLOAD                                                      : 30;
        unsigned int IS_EVENT                                                     :  1;
        unsigned int IS_STATE                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_RELAUNCH2 {
    struct {
        unsigned int PAYLOAD                                                      : 30;
        unsigned int IS_EVENT                                                     :  1;
        unsigned int IS_STATE                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_REQ_CTRL {
    struct {
        unsigned int SOFT_GROUPING_EN                                             :  1;
        unsigned int NUMBER_OF_REQUESTS_PER_CU                                    :  4;
        unsigned int SOFT_GROUPING_ALLOCATION_TIMEOUT                             :  4;
        unsigned int HARD_LOCK_HYSTERESIS                                         :  1;
        unsigned int HARD_LOCK_LOW_THRESHOLD                                      :  5;
        unsigned int PRODUCER_REQUEST_LOCKOUT                                     :  1;
        unsigned int GLOBAL_SCANNING_EN                                           :  1;
        unsigned int ALLOCATION_RATE_THROTTLING_THRESHOLD                         :  3;
        unsigned int DEDICATED_PREALLOCATION_BUFFER_LIMIT                         :  7;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_RESOURCE_LIMITS {
    struct {
        unsigned int WAVES_PER_SH                                                 : 10;
        unsigned int                                                              :  2;
        unsigned int TG_PER_CU                                                    :  4;
        unsigned int LOCK_THRESHOLD                                               :  6;
        unsigned int SIMD_DEST_CNTL                                               :  1;
        unsigned int FORCE_SIMD_DIST                                              :  1;
        unsigned int CU_GROUP_COUNT                                               :  3;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_RESTART_X {
    struct {
        unsigned int RESTART                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_RESTART_Y {
    struct {
        unsigned int RESTART                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_RESTART_Z {
    struct {
        unsigned int RESTART                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_SHADER_CHKSUM {
    struct {
        unsigned int CHECKSUM                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_START_X {
    struct {
        unsigned int START                                                        : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_START_Y {
    struct {
        unsigned int START                                                        : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_START_Z {
    struct {
        unsigned int START                                                        : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_STATIC_THREAD_MGMT_SE0 {
    struct {
        unsigned int SA0_CU_EN                                                    : 16;
        unsigned int SA1_CU_EN                                                    : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_STATIC_THREAD_MGMT_SE1 {
    struct {
        unsigned int SA0_CU_EN                                                    : 16;
        unsigned int SA1_CU_EN                                                    : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_STATIC_THREAD_MGMT_SE2 {
    struct {
        unsigned int SA0_CU_EN                                                    : 16;
        unsigned int SA1_CU_EN                                                    : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_STATIC_THREAD_MGMT_SE3 {
    struct {
        unsigned int SA0_CU_EN                                                    : 16;
        unsigned int SA1_CU_EN                                                    : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_STATIC_THREAD_MGMT_SE4 {
    struct {
        unsigned int SA0_CU_EN                                                    : 16;
        unsigned int SA1_CU_EN                                                    : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_STATIC_THREAD_MGMT_SE5 {
    struct {
        unsigned int SA0_CU_EN                                                    : 16;
        unsigned int SA1_CU_EN                                                    : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_STATIC_THREAD_MGMT_SE6 {
    struct {
        unsigned int SA0_CU_EN                                                    : 16;
        unsigned int SA1_CU_EN                                                    : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_STATIC_THREAD_MGMT_SE7 {
    struct {
        unsigned int SA0_CU_EN                                                    : 16;
        unsigned int SA1_CU_EN                                                    : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_STATIC_THREAD_MGMT_SE8 {
    struct {
        unsigned int SA0_CU_EN                                                    : 16;
        unsigned int SA1_CU_EN                                                    : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_THREADGROUP_ID {
    struct {
        unsigned int THREADGROUP_ID                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_THREAD_TRACE_ENABLE {
    struct {
        unsigned int THREAD_TRACE_ENABLE                                          :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_TMPRING_SIZE {
    struct {
        unsigned int WAVES                                                        : 12;
        unsigned int WAVESIZE                                                     : 18;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_USER_ACCUM_0 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_USER_ACCUM_1 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_USER_ACCUM_2 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_USER_ACCUM_3 {
    struct {
        unsigned int CONTRIBUTION                                                 :  7;
        unsigned int                                                              : 25;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_USER_DATA_0 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_USER_DATA_1 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_USER_DATA_2 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_USER_DATA_3 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_USER_DATA_4 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_USER_DATA_5 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_USER_DATA_6 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_USER_DATA_7 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_USER_DATA_8 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_USER_DATA_9 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_USER_DATA_10 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_USER_DATA_11 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_USER_DATA_12 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_USER_DATA_13 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_USER_DATA_14 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_USER_DATA_15 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_VMID {
    struct {
        unsigned int DATA                                                         :  4;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_WAVE_RESTORE_ADDR_HI {
    struct {
        unsigned int ADDR                                                         : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_COMPUTE_WAVE_RESTORE_ADDR_LO {
    struct {
        unsigned int ADDR                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DATA1 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DATA2 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DATA3 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DATA4 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DATA5 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DATA6 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DATA7 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DATA8 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DATA9 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DATA10 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DATA11 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DATA12 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DATA13 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DATA14 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DATA15 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DATA16 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE0_BASE {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE0_CNTL {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE0_MASK {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE1_BASE {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE1_CNTL {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE1_MASK {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE2_BASE {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE2_CNTL {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE2_MASK {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE3_BASE {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE3_CNTL {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE3_MASK {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE4_BASE {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE4_CNTL {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE4_MASK {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE5_BASE {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE5_CNTL {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE5_MASK {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE6_BASE {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE6_CNTL {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE6_MASK {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE7_BASE {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE7_CNTL {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE7_MASK {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE8_BASE {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE8_CNTL {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE8_MASK {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE9_BASE {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE9_CNTL {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE9_MASK {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE10_BASE {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE10_CNTL {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE10_MASK {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE11_BASE {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE11_CNTL {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE11_MASK {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE12_BASE {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE12_CNTL {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE12_MASK {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE13_BASE {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE13_CNTL {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE13_MASK {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE14_BASE {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE14_CNTL {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE14_MASK {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE15_BASE {
    struct {
        unsigned int BASE                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE15_CNTL {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int BYPASS_MODE                                                  :  1;
        unsigned int ENABLE                                                       :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_APERTURE15_MASK {
    struct {
        unsigned int MASK                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_BASE_CNTL {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_BASE_HI {
    struct {
        unsigned int DC_BASE_HI                                                   : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_BASE_LO {
    struct {
        unsigned int                                                              : 16;
        unsigned int DC_BASE_LO                                                   : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DC_OP_CNTL {
    struct {
        unsigned int INVALIDATE_DCACHE                                            :  1;
        unsigned int INVALIDATE_DCACHE_COMPLETE                                   :  1;
        unsigned int BYPASS_ALL                                                   :  1;
        unsigned int                                                              : 29;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DDID_BASE_ADDR_HI {
    struct {
        unsigned int BASE_ADDR_HI                                                 : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DDID_BASE_ADDR_LO {
    struct {
        unsigned int                                                              :  6;
        unsigned int BASE_ADDR_LO                                                 : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DDID_CNTL {
    struct {
        unsigned int THRESHOLD                                                    :  8;
        unsigned int                                                              :  8;
        unsigned int SIZE                                                         :  1;
        unsigned int                                                              :  2;
        unsigned int NO_RING_MEMORY                                               :  1;
        unsigned int                                                              :  8;
        unsigned int POLICY                                                       :  2;
        unsigned int MODE                                                         :  1;
        unsigned int ENABLE                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DM_INDEX_ADDR {
    struct {
        unsigned int ADDR                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_DM_INDEX_DATA {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_ENDIAN_SWAP {
    struct {
        unsigned int ENDIAN_SWAP                                                  :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_ERROR {
    struct {
        unsigned int RESERVED                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_GEN_BASE_HI {
    struct {
        unsigned int BASE_HI                                                      : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_GEN_BASE_LO {
    struct {
        unsigned int                                                              : 12;
        unsigned int BASE_LO                                                      : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_GP0_HI {
    struct {
        unsigned int M_RET_ADDR                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_GP0_LO {
    struct {
        unsigned int PG_VIRT_HALTED                                               :  1;
        unsigned int DATA                                                         : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_GP1_HI {
    struct {
        unsigned int RD_WR_SELECT_HI                                              : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_GP1_LO {
    struct {
        unsigned int RD_WR_SELECT_LO                                              : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_GP2_HI {
    struct {
        unsigned int STACK_PNTR_HI                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_GP2_LO {
    struct {
        unsigned int STACK_PNTR_LO                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_GP3_HI {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_GP3_LO {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_GP4_HI {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_GP4_LO {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_GP5_HI {
    struct {
        unsigned int M_RET_ADDR                                                   : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_GP5_LO {
    struct {
        unsigned int PG_VIRT_HALTED                                               :  1;
        unsigned int DATA                                                         : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_GP6_HI {
    struct {
        unsigned int RD_WR_SELECT_HI                                              : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_GP6_LO {
    struct {
        unsigned int RD_WR_SELECT_LO                                              : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_GP7_HI {
    struct {
        unsigned int STACK_PNTR_HI                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_GP7_LO {
    struct {
        unsigned int STACK_PNTR_LO                                                : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_GP8_HI {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_GP8_LO {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_GP9_HI {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_GP9_LO {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_HALT_HYST_COUNT {
    struct {
        unsigned int COUNT                                                        :  4;
        unsigned int                                                              : 28;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_IC_BASE_CNTL {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int                                                              :  1;
        unsigned int PER_PIPE                                                     :  1;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 12;
        unsigned int EXE_DISABLE                                                  :  1;
        unsigned int CACHE_POLICY                                                 :  2;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_IC_BASE_HI {
    struct {
        unsigned int IC_BASE_HI                                                   : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_IC_BASE_LO {
    struct {
        unsigned int                                                              : 12;
        unsigned int IC_BASE_LO                                                   : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_IC_OP_CNTL {
    struct {
        unsigned int INVALIDATE_CACHE                                             :  1;
        unsigned int INVALIDATE_CACHE_COMPLETE                                    :  1;
        unsigned int HALT_ON_PRIME                                                :  1;
        unsigned int PRIME_START_PC                                               :  1;
        unsigned int PRIME_ICACHE                                                 :  1;
        unsigned int ICACHE_PRIMED                                                :  1;
        unsigned int                                                              : 26;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_IH_FREE_COUNT {
    struct {
        unsigned int COUNT                                                        :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_INT_CNTXID_HI {
    struct {
        unsigned int DATA                                                         :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_INT_CNTXID_LO {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_INT_INFO {
    struct {
        unsigned int RING_ID                                                      :  8;
        unsigned int VMID                                                         :  4;
        unsigned int                                                              : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_INT_STATUS {
    struct {
        unsigned int SOURCE_ID                                                    :  8;
        unsigned int SEND                                                         :  1;
        unsigned int                                                              : 23;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_IQ_TIMER {
    struct {
        unsigned int WAIT_TIME                                                    :  8;
        unsigned int RETRY_TYPE                                                   :  3;
        unsigned int IMMEDIATE_EXPIRE                                             :  1;
        unsigned int INTERRUPT_TYPE                                               :  2;
        unsigned int CLOCK_COUNT                                                  :  2;
        unsigned int RESERVED3                                                    :  6;
        unsigned int QUANTUM_TIMER                                                :  1;
        unsigned int RESERVED2                                                    :  4;
        unsigned int QUEUE_TYPE                                                   :  1;
        unsigned int REARM_TIMER                                                  :  1;
        unsigned int RESERVED1                                                    :  2;
        unsigned int ACTIVE                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_IQ_TIMER_MESSAGE {
    struct {
        unsigned int MESSAGE                                                      :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_IQ_WAIT_TIME1 {
    struct {
        unsigned int IB_OFFLOAD                                                   :  8;
        unsigned int ATOMIC_OFFLOAD                                               :  8;
        unsigned int WRM_OFFLOAD                                                  :  8;
        unsigned int GWS                                                          :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_IQ_WAIT_TIME2 {
    struct {
        unsigned int QUE_SLEEP                                                    :  8;
        unsigned int SCH_WAVE                                                     :  8;
        unsigned int                                                              :  8;
        unsigned int DEQ_RETRY                                                    :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_IQ_WAIT_TIME3 {
    struct {
        unsigned int SUSPEND_QUE                                                  :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_LATENCY_STATS_DATA {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_LATENCY_STATS_SELECT {
    struct {
        unsigned int INDEX                                                        :  5;
        unsigned int                                                              : 25;
        unsigned int CLEAR                                                        :  1;
        unsigned int ENABLE                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_LOCAL_APERTURE {
    struct {
        unsigned int APERTURE                                                     :  3;
        unsigned int                                                              :  3;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_LOCAL_BASE0_HI {
    struct {
        unsigned int BASE0_HI                                                     : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_LOCAL_BASE0_LO {
    struct {
        unsigned int                                                              : 16;
        unsigned int BASE0_LO                                                     : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_LOCAL_INSTR_APERTURE {
    struct {
        unsigned int APERTURE                                                     :  3;
        unsigned int                                                              :  3;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_LOCAL_INSTR_BASE_HI {
    struct {
        unsigned int BASE_HI                                                      : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_LOCAL_INSTR_BASE_LO {
    struct {
        unsigned int                                                              : 16;
        unsigned int BASE_LO                                                      : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_LOCAL_INSTR_MASK_HI {
    struct {
        unsigned int MASK_HI                                                      : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_LOCAL_INSTR_MASK_LO {
    struct {
        unsigned int                                                              : 16;
        unsigned int MASK_LO                                                      : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_LOCAL_MASK0_HI {
    struct {
        unsigned int MASK0_HI                                                     : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_LOCAL_MASK0_LO {
    struct {
        unsigned int                                                              : 16;
        unsigned int MASK0_LO                                                     : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_LOCAL_SCRATCH_APERTURE {
    struct {
        unsigned int APERTURE                                                     :  3;
        unsigned int                                                              : 29;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_LOCAL_SCRATCH_BASE_HI {
    struct {
        unsigned int BASE_HI                                                      : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_LOCAL_SCRATCH_BASE_LO {
    struct {
        unsigned int                                                              : 16;
        unsigned int BASE_LO                                                      : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_MDBASE_HI {
    struct {
        unsigned int BASE_HI                                                      : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_MDBASE_LO {
    struct {
        unsigned int                                                              : 16;
        unsigned int BASE_LO                                                      : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_MDBOUND_HI {
    struct {
        unsigned int BOUND_HI                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_MDBOUND_LO {
    struct {
        unsigned int BOUND_LO                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_ME1_HEADER_DUMP {
    struct {
        unsigned int HEADER_DUMP                                                  : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_ME1_PIPE0_PRIORITY {
    struct {
        unsigned int PRIORITY                                                     :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_ME1_PIPE1_PRIORITY {
    struct {
        unsigned int PRIORITY                                                     :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_ME1_PIPE2_PRIORITY {
    struct {
        unsigned int PRIORITY                                                     :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_ME1_PIPE3_PRIORITY {
    struct {
        unsigned int PRIORITY                                                     :  2;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_ME1_PIPE_PRIORITY_CNTS {
    struct {
        unsigned int PRIORITY1_CNT                                                :  8;
        unsigned int PRIORITY2A_CNT                                               :  8;
        unsigned int PRIORITY2B_CNT                                               :  8;
        unsigned int PRIORITY3_CNT                                                :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_METADATA_BASE_ADDR_HI {
    struct {
        unsigned int MD_BASE_HI                                                   : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_METADATA_BASE_ADDR_LO {
    struct {
        unsigned int                                                              :  2;
        unsigned int MD_BASE_LO                                                   : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_METADATA_CNTL {
    struct {
        unsigned int VMID                                                         :  4;
        unsigned int EVENT_MODE                                                   :  2;
        unsigned int SCOPE                                                        :  2;
        unsigned int TEMPORAL                                                     :  3;
        unsigned int EVENT1                                                       :  1;
        unsigned int QUEUE_ID                                                     :  3;
        unsigned int                                                              : 17;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_MGCG_SYNC_CNTL {
    struct {
        unsigned int COOLDOWN_PERIOD                                              :  8;
        unsigned int WARMUP_PERIOD                                                :  8;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_MIBOUND_HI {
    struct {
        unsigned int BOUND_HI                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_MIBOUND_LO {
    struct {
        unsigned int BOUND_LO                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_MIE_HI {
    struct {
        unsigned int MEC_INT                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_MIE_LO {
    struct {
        unsigned int MEC_INT                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_MIP_HI {
    struct {
        unsigned int MIP_HI                                                       : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_MIP_LO {
    struct {
        unsigned int MIP_LO                                                       : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_MTIMECMP_HI {
    struct {
        unsigned int TIME_HI                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_MTIMECMP_LO {
    struct {
        unsigned int TIME_LO                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_MTVEC_HI {
    struct {
        unsigned int ADDR_LO                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_MTVEC_LO {
    struct {
        unsigned int ADDR_LO                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_OS_PIPES {
    struct {
        unsigned int OS_PIPES                                                     :  8;
        unsigned int                                                              : 24;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_PERFCOUNTER0_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_PERFCOUNTER0_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_PERFCOUNTER0_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int PERF_SEL1                                                    : 10;
        unsigned int SPM_MODE                                                     :  4;
        unsigned int CNTR_MODE1                                                   :  4;
        unsigned int CNTR_MODE0                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_PERFCOUNTER0_SELECT1 {
    struct {
        unsigned int PERF_SEL2                                                    : 10;
        unsigned int PERF_SEL3                                                    : 10;
        unsigned int                                                              :  4;
        unsigned int CNTR_MODE3                                                   :  4;
        unsigned int CNTR_MODE2                                                   :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_PERFCOUNTER1_HI {
    struct {
        unsigned int PERFCOUNTER_HI                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_PERFCOUNTER1_LO {
    struct {
        unsigned int PERFCOUNTER_LO                                               : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_PERFCOUNTER1_SELECT {
    struct {
        unsigned int PERF_SEL                                                     : 10;
        unsigned int                                                              : 18;
        unsigned int CNTR_MODE                                                    :  4;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_PERFMON_CNTL {
    struct {
        unsigned int PERFMON_STATE                                                :  4;
        unsigned int SPM_PERFMON_STATE                                            :  4;
        unsigned int PERFMON_ENABLE_MODE                                          :  2;
        unsigned int PERFMON_SAMPLE_ENABLE                                        :  1;
        unsigned int                                                              : 21;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_RS64_CNTL {
    struct {
        unsigned int                                                              :  4;
        unsigned int MEC_INVALIDATE_ICACHE                                        :  1;
        unsigned int                                                              : 11;
        unsigned int RSRV8                                                        :  1;
        unsigned int RSRV7                                                        :  1;
        unsigned int RSRV6                                                        :  1;
        unsigned int RSRV5                                                        :  1;
        unsigned int                                                              :  6;
        unsigned int RSRV4                                                        :  1;
        unsigned int RSRV3                                                        :  1;
        unsigned int RSRV2                                                        :  1;
        unsigned int RSRV1                                                        :  1;
        unsigned int MEC_HALT                                                     :  1;
        unsigned int MEC_STEP                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_RS64_EXCEPTION_STATUS {
    struct {
        unsigned int RS64_EXCEPTION_ILLEGAL_INSTRUCTION                           :  1;
        unsigned int RS64_EXCEPTION_MISALIGNED_ADDR                               :  1;
        unsigned int RS64_EXCEPTION_UNALIGNED_INSTRUTCION                         :  1;
        unsigned int RS64_EXCEPTION_PAGE_FAULT                                    :  1;
        unsigned int RS64_EXCEPTION_INSTRUCTION_ADDR                              : 23;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_RS64_INSTR_PNTR {
    struct {
        unsigned int INSTR_PNTR                                                   : 20;
        unsigned int                                                              : 12;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_RS64_INTERRUPT {
    struct {
        unsigned int MEC_INT                                                      : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_RS64_PENDING_INTERRUPT {
    struct {
        unsigned int PENDING_INTERRUPT                                            : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_RS64_PERFCOUNT_CNTL {
    struct {
        unsigned int EVENT_SEL                                                    :  5;
        unsigned int                                                              : 27;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_RS64_PRGRM_CNTR_START {
    struct {
        unsigned int IP_START                                                     : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_RS64_PRGRM_CNTR_START_HI {
    struct {
        unsigned int IP_START                                                     : 30;
        unsigned int                                                              :  2;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_RS64_PRIV_LEVEL {
    struct {
        unsigned int PRIV_LEVEL                                                   :  1;
        unsigned int GRBM_PRIV_LEVEL                                              :  1;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_RS64_PRIV_LEVEL_VIOLATION_STATUS {
    struct {
        unsigned int RS64_EXCEPTION_ILLEGAL_INSTRUCTION                           :  1;
        unsigned int RS64_EXCEPTION_MISALIGNED_ADDR                               :  1;
        unsigned int RS64_EXCEPTION_UNALIGNED_INSTRUTCION                         :  1;
        unsigned int RS64_EXCEPTION_PAGE_FAULT                                    :  1;
        unsigned int RS64_EXCEPTION_INSTRUCTION_ADDR                              : 23;
        unsigned int                                                              :  5;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_RS64_THREAD_CNTL {
    struct {
        unsigned int MEC_STATUS                                                   :  4;
        unsigned int MEC_INVALIDATE_ICACHE                                        :  1;
        unsigned int                                                              : 11;
        unsigned int MEC_PIPE_RESET                                               :  1;
        unsigned int RSRV7                                                        :  1;
        unsigned int RSRV6                                                        :  1;
        unsigned int RSRV5                                                        :  1;
        unsigned int                                                              :  6;
        unsigned int MEC_PIPE_ACTIVE                                              :  1;
        unsigned int RSRV4                                                        :  1;
        unsigned int RSRV3                                                        :  1;
        unsigned int RSRV2                                                        :  1;
        unsigned int SPARE                                                        :  1;
        unsigned int RSRV1                                                        :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_SCRATCH_DATA {
    struct {
        unsigned int SCRATCH_DATA                                                 : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_SCRATCH_INDEX {
    struct {
        unsigned int SCRATCH_INDEX                                                :  9;
        unsigned int                                                              : 22;
        unsigned int SCRATCH_INDEX_64BIT_MODE                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_SECURE_REG0 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_SECURE_REG1 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_SECURE_REG2 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_SECURE_REG3 {
    struct {
        unsigned int DATA                                                         : 32;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_STALLED_STAT1 {
    struct {
        unsigned int                                                              :  3;
        unsigned int RCIU_TX_FREE_STALL                                           :  1;
        unsigned int RCIU_PRIV_VIOLATION                                          :  1;
        unsigned int                                                              :  1;
        unsigned int TCIU_TX_FREE_STALL                                           :  1;
        unsigned int TCIU_WAITING_ON_TAGS                                         :  1;
        unsigned int MEC1_DECODING_PACKET                                         :  1;
        unsigned int MEC1_WAIT_ON_RCIU                                            :  1;
        unsigned int MEC1_WAIT_ON_RCIU_READ                                       :  1;
        unsigned int                                                              :  2;
        unsigned int MEC1_WAIT_ON_ROQ_DATA                                        :  1;
        unsigned int                                                              :  2;
        unsigned int MEC2_DECODING_PACKET                                         :  1;
        unsigned int MEC2_WAIT_ON_RCIU                                            :  1;
        unsigned int MEC2_WAIT_ON_RCIU_READ                                       :  1;
        unsigned int                                                              :  2;
        unsigned int MEC2_WAIT_ON_ROQ_DATA                                        :  1;
        unsigned int UTCL2IU_WAITING_ON_FREE                                      :  1;
        unsigned int UTCL2IU_WAITING_ON_TAGS                                      :  1;
        unsigned int UTCL1_WAITING_ON_TRANS                                       :  1;
        unsigned int GCRIU_WAITING_ON_FREE                                        :  1;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_STATUS {
    struct {
        unsigned int MEC1_BUSY                                                    :  1;
        unsigned int MEC2_BUSY                                                    :  1;
        unsigned int DC0_BUSY                                                     :  1;
        unsigned int DC1_BUSY                                                     :  1;
        unsigned int RCIU1_BUSY                                                   :  1;
        unsigned int RCIU2_BUSY                                                   :  1;
        unsigned int ROQ1_BUSY                                                    :  1;
        unsigned int ROQ2_BUSY                                                    :  1;
        unsigned int TMZ_STATE                                                    :  1;
        unsigned int VMID_TMZ                                                     :  1;
        unsigned int TCIU_BUSY                                                    :  1;
        unsigned int SCRATCH_RAM_BUSY                                             :  1;
        unsigned int QU_BUSY                                                      :  1;
        unsigned int UTCL2IU_BUSY                                                 :  1;
        unsigned int SAVE_RESTORE_BUSY                                            :  1;
        unsigned int GCRIU_BUSY                                                   :  1;
        unsigned int MES_BUSY                                                     :  1;
        unsigned int MES_SCRATCH_RAM_BUSY                                         :  1;
        unsigned int RCIU3_BUSY                                                   :  1;
        unsigned int MES_INSTRUCTION_CACHE_BUSY                                   :  1;
        unsigned int MES_DATA_CACHE_BUSY                                          :  1;
        unsigned int MEC_DATA_CACHE_BUSY                                          :  1;
        unsigned int GL1_XNACK                                                    :  2;
        unsigned int RSRV                                                         :  7;
        unsigned int WGS_BUSY                                                     :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_SUSPEND_CNTL_STACK_OFFSET {
    struct {
        unsigned int                                                              :  2;
        unsigned int OFFSET                                                       : 14;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_SUSPEND_CNTL_STACK_SIZE {
    struct {
        unsigned int                                                              : 12;
        unsigned int SIZE                                                         :  4;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_SUSPEND_CTX_SAVE_BASE_ADDR_HI {
    struct {
        unsigned int ADDR_HI                                                      : 16;
        unsigned int                                                              : 16;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_SUSPEND_CTX_SAVE_BASE_ADDR_LO {
    struct {
        unsigned int                                                              : 12;
        unsigned int ADDR                                                         : 20;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_SUSPEND_CTX_SAVE_CONTROL {
    struct {
        unsigned int                                                              :  3;
        unsigned int POLICY                                                       :  2;
        unsigned int                                                              : 18;
        unsigned int EXE_DISABLE                                                  :  1;
        unsigned int                                                              :  8;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_SUSPEND_CTX_SAVE_SIZE {
    struct {
        unsigned int                                                              : 12;
        unsigned int SIZE                                                         : 14;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_SUSPEND_RESUME_REQ {
    struct {
        unsigned int SUSPEND_REQ                                                  :  1;
        unsigned int RESUME_REQ                                                   :  1;
        unsigned int                                                              : 30;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_SUSPEND_WG_STATE_OFFSET {
    struct {
        unsigned int                                                              :  2;
        unsigned int OFFSET                                                       : 24;
        unsigned int                                                              :  6;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_TC_PERF_COUNTER_WINDOW_SELECT {
    struct {
        unsigned int INDEX                                                        :  5;
        unsigned int                                                              : 25;
        unsigned int ALWAYS                                                       :  1;
        unsigned int ENABLE                                                       :  1;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

union WGS_TMZ_CNTL {
    struct {
        unsigned int PTE_DISABLE                                                  :  1;
        unsigned int                                                              : 31;
    } bits, bitfields;

    unsigned int u32All;
    signed int   i32All;
    float        f32All;
};

} // inline namespace Chip
} // namespace Gfx12
} // namespace Pal
