/*
 ***********************************************************************************************************************
 *
 *  Copyright (c) 2025 Advanced Micro Devices, Inc. All Rights Reserved.
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in all
 *  copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 *  SOFTWARE.
 *
 **********************************************************************************************************************/

#pragma once

namespace Pal
{
namespace Gfx9
{
inline namespace Chip
{

typedef enum BinEventCntl {
    BINNER_BREAK_BATCH                                 = 0x00000000,
    BINNER_PIPELINE                                    = 0x00000001,
    BINNER_DROP                                        = 0x00000002,
    BINNER_PIPELINE_BREAK                              = 0x00000003,
} BinEventCntl;

typedef enum BinMapMode {
    BIN_MAP_MODE_NONE                                  = 0x00000000,
    BIN_MAP_MODE_RTA_INDEX                             = 0x00000001,
    BIN_MAP_MODE_POPS                                  = 0x00000002,
} BinMapMode;

typedef enum BinningMode {
    BINNING_ALLOWED                                    = 0x00000000,
    FORCE_BINNING_ON                                   = 0x00000001,
    DISABLE_BINNING_USE_NEW_SC__GFX10                  = 0x00000002,
    DISABLE_BINNING_USE_LEGACY_SC__GFX10               = 0x00000003,
    BINNING_ONE_PRIM_PER_BATCH__GFX11                  = 0x00000002,
    BINNING_DISABLED__GFX11                            = 0x00000003,
} BinningMode;

typedef enum BinSizeExtend {
    BIN_SIZE_32_PIXELS                                 = 0x00000000,
    BIN_SIZE_64_PIXELS                                 = 0x00000001,
    BIN_SIZE_128_PIXELS                                = 0x00000002,
    BIN_SIZE_256_PIXELS                                = 0x00000003,
    BIN_SIZE_512_PIXELS                                = 0x00000004,
} BinSizeExtend;

typedef enum BlendOp {
    BLEND_ZERO                                         = 0x00000000,
    BLEND_ONE                                          = 0x00000001,
    BLEND_SRC_COLOR                                    = 0x00000002,
    BLEND_ONE_MINUS_SRC_COLOR                          = 0x00000003,
    BLEND_SRC_ALPHA                                    = 0x00000004,
    BLEND_ONE_MINUS_SRC_ALPHA                          = 0x00000005,
    BLEND_DST_ALPHA                                    = 0x00000006,
    BLEND_ONE_MINUS_DST_ALPHA                          = 0x00000007,
    BLEND_DST_COLOR                                    = 0x00000008,
    BLEND_ONE_MINUS_DST_COLOR                          = 0x00000009,
    BLEND_SRC_ALPHA_SATURATE                           = 0x0000000a,
    BLEND_BOTH_SRC_ALPHA__GFX10                        = 0x0000000b,
    BLEND_BOTH_INV_SRC_ALPHA__GFX10                    = 0x0000000c,
    BLEND_CONSTANT_COLOR__GFX10                        = 0x0000000d,
    BLEND_ONE_MINUS_CONSTANT_COLOR__GFX10              = 0x0000000e,
    BLEND_SRC1_COLOR__GFX10                            = 0x0000000f,
    BLEND_INV_SRC1_COLOR__GFX10                        = 0x00000010,
    BLEND_SRC1_ALPHA__GFX10                            = 0x00000011,
    BLEND_INV_SRC1_ALPHA__GFX10                        = 0x00000012,
    BLEND_CONSTANT_ALPHA__GFX10                        = 0x00000013,
    BLEND_ONE_MINUS_CONSTANT_ALPHA__GFX10              = 0x00000014,
    BLEND_CONSTANT_COLOR__GFX11                        = 0x0000000b,
    BLEND_ONE_MINUS_CONSTANT_COLOR__GFX11              = 0x0000000c,
    BLEND_SRC1_COLOR__GFX11                            = 0x0000000d,
    BLEND_INV_SRC1_COLOR__GFX11                        = 0x0000000e,
    BLEND_SRC1_ALPHA__GFX11                            = 0x0000000f,
    BLEND_INV_SRC1_ALPHA__GFX11                        = 0x00000010,
    BLEND_CONSTANT_ALPHA__GFX11                        = 0x00000011,
    BLEND_ONE_MINUS_CONSTANT_ALPHA__GFX11              = 0x00000012,
} BlendOp;

typedef enum BlendOpt {
    FORCE_OPT_AUTO                                     = 0x00000000,
    FORCE_OPT_DISABLE                                  = 0x00000001,
    FORCE_OPT_ENABLE_IF_SRC_A_0                        = 0x00000002,
    FORCE_OPT_ENABLE_IF_SRC_RGB_0                      = 0x00000003,
    FORCE_OPT_ENABLE_IF_SRC_ARGB_0                     = 0x00000004,
    FORCE_OPT_ENABLE_IF_SRC_A_1                        = 0x00000005,
    FORCE_OPT_ENABLE_IF_SRC_RGB_1                      = 0x00000006,
    FORCE_OPT_ENABLE_IF_SRC_ARGB_1                     = 0x00000007,
} BlendOpt;

typedef enum BUF_DATA_FORMAT {
    BUF_DATA_FORMAT_INVALID                            = 0x00000000,
    BUF_DATA_FORMAT_8                                  = 0x00000001,
    BUF_DATA_FORMAT_16                                 = 0x00000002,
    BUF_DATA_FORMAT_8_8                                = 0x00000003,
    BUF_DATA_FORMAT_32                                 = 0x00000004,
    BUF_DATA_FORMAT_16_16                              = 0x00000005,
    BUF_DATA_FORMAT_10_11_11                           = 0x00000006,
    BUF_DATA_FORMAT_11_11_10                           = 0x00000007,
    BUF_DATA_FORMAT_10_10_10_2                         = 0x00000008,
    BUF_DATA_FORMAT_2_10_10_10                         = 0x00000009,
    BUF_DATA_FORMAT_8_8_8_8                            = 0x0000000a,
    BUF_DATA_FORMAT_32_32                              = 0x0000000b,
    BUF_DATA_FORMAT_16_16_16_16                        = 0x0000000c,
    BUF_DATA_FORMAT_32_32_32                           = 0x0000000d,
    BUF_DATA_FORMAT_32_32_32_32                        = 0x0000000e,
    BUF_DATA_FORMAT_RESERVED_15                        = 0x0000000f,
} BUF_DATA_FORMAT;

typedef enum BUF_FMT {
    BUF_FMT_INVALID                                    = 0x00000000,
    BUF_FMT_8_UNORM                                    = 0x00000001,
    BUF_FMT_8_SNORM                                    = 0x00000002,
    BUF_FMT_8_USCALED                                  = 0x00000003,
    BUF_FMT_8_SSCALED                                  = 0x00000004,
    BUF_FMT_8_UINT                                     = 0x00000005,
    BUF_FMT_8_SINT                                     = 0x00000006,
    BUF_FMT_16_UNORM                                   = 0x00000007,
    BUF_FMT_16_SNORM                                   = 0x00000008,
    BUF_FMT_16_USCALED                                 = 0x00000009,
    BUF_FMT_16_SSCALED                                 = 0x0000000a,
    BUF_FMT_16_UINT                                    = 0x0000000b,
    BUF_FMT_16_SINT                                    = 0x0000000c,
    BUF_FMT_16_FLOAT                                   = 0x0000000d,
    BUF_FMT_8_8_UNORM                                  = 0x0000000e,
    BUF_FMT_8_8_SNORM                                  = 0x0000000f,
    BUF_FMT_8_8_USCALED                                = 0x00000010,
    BUF_FMT_8_8_SSCALED                                = 0x00000011,
    BUF_FMT_8_8_UINT                                   = 0x00000012,
    BUF_FMT_8_8_SINT                                   = 0x00000013,
    BUF_FMT_32_UINT                                    = 0x00000014,
    BUF_FMT_32_SINT                                    = 0x00000015,
    BUF_FMT_32_FLOAT                                   = 0x00000016,
    BUF_FMT_16_16_UNORM                                = 0x00000017,
    BUF_FMT_16_16_SNORM                                = 0x00000018,
    BUF_FMT_16_16_USCALED                              = 0x00000019,
    BUF_FMT_16_16_SSCALED                              = 0x0000001a,
    BUF_FMT_16_16_UINT                                 = 0x0000001b,
    BUF_FMT_16_16_SINT                                 = 0x0000001c,
    BUF_FMT_16_16_FLOAT                                = 0x0000001d,
    BUF_FMT_10_11_11_UNORM__GFX10                      = 0x0000001e,
    BUF_FMT_10_11_11_SNORM__GFX10                      = 0x0000001f,
    BUF_FMT_10_11_11_USCALED__GFX10                    = 0x00000020,
    BUF_FMT_10_11_11_SSCALED__GFX10                    = 0x00000021,
    BUF_FMT_10_11_11_UINT__GFX10                       = 0x00000022,
    BUF_FMT_10_11_11_SINT__GFX10                       = 0x00000023,
    BUF_FMT_10_11_11_FLOAT__GFX10                      = 0x00000024,
    BUF_FMT_11_11_10_UNORM__GFX10                      = 0x00000025,
    BUF_FMT_11_11_10_SNORM__GFX10                      = 0x00000026,
    BUF_FMT_11_11_10_USCALED__GFX10                    = 0x00000027,
    BUF_FMT_11_11_10_SSCALED__GFX10                    = 0x00000028,
    BUF_FMT_11_11_10_UINT__GFX10                       = 0x00000029,
    BUF_FMT_11_11_10_SINT__GFX10                       = 0x0000002a,
    BUF_FMT_11_11_10_FLOAT__GFX10                      = 0x0000002b,
    BUF_FMT_10_10_10_2_UNORM__GFX10                    = 0x0000002c,
    BUF_FMT_10_10_10_2_SNORM__GFX10                    = 0x0000002d,
    BUF_FMT_10_10_10_2_USCALED__GFX10                  = 0x0000002e,
    BUF_FMT_10_10_10_2_SSCALED__GFX10                  = 0x0000002f,
    BUF_FMT_10_10_10_2_UINT__GFX10                     = 0x00000030,
    BUF_FMT_10_10_10_2_SINT__GFX10                     = 0x00000031,
    BUF_FMT_2_10_10_10_UNORM__GFX10                    = 0x00000032,
    BUF_FMT_2_10_10_10_SNORM__GFX10                    = 0x00000033,
    BUF_FMT_2_10_10_10_USCALED__GFX10                  = 0x00000034,
    BUF_FMT_2_10_10_10_SSCALED__GFX10                  = 0x00000035,
    BUF_FMT_2_10_10_10_UINT__GFX10                     = 0x00000036,
    BUF_FMT_2_10_10_10_SINT__GFX10                     = 0x00000037,
    BUF_FMT_8_8_8_8_UNORM__GFX10                       = 0x00000038,
    BUF_FMT_8_8_8_8_SNORM__GFX10                       = 0x00000039,
    BUF_FMT_8_8_8_8_USCALED__GFX10                     = 0x0000003a,
    BUF_FMT_8_8_8_8_SSCALED__GFX10                     = 0x0000003b,
    BUF_FMT_8_8_8_8_UINT__GFX10                        = 0x0000003c,
    BUF_FMT_8_8_8_8_SINT__GFX10                        = 0x0000003d,
    BUF_FMT_32_32_UINT__GFX10                          = 0x0000003e,
    BUF_FMT_32_32_SINT__GFX10                          = 0x0000003f,
    BUF_FMT_32_32_FLOAT__GFX10                         = 0x00000040,
    BUF_FMT_16_16_16_16_UNORM__GFX10                   = 0x00000041,
    BUF_FMT_16_16_16_16_SNORM__GFX10                   = 0x00000042,
    BUF_FMT_16_16_16_16_USCALED__GFX10                 = 0x00000043,
    BUF_FMT_16_16_16_16_SSCALED__GFX10                 = 0x00000044,
    BUF_FMT_16_16_16_16_UINT__GFX10                    = 0x00000045,
    BUF_FMT_16_16_16_16_SINT__GFX10                    = 0x00000046,
    BUF_FMT_16_16_16_16_FLOAT__GFX10                   = 0x00000047,
    BUF_FMT_32_32_32_UINT__GFX10                       = 0x00000048,
    BUF_FMT_32_32_32_SINT__GFX10                       = 0x00000049,
    BUF_FMT_32_32_32_FLOAT__GFX10                      = 0x0000004a,
    BUF_FMT_32_32_32_32_UINT__GFX10                    = 0x0000004b,
    BUF_FMT_32_32_32_32_SINT__GFX10                    = 0x0000004c,
    BUF_FMT_32_32_32_32_FLOAT__GFX10                   = 0x0000004d,
    BUF_FMT_RESERVED_78__GFX10                         = 0x0000004e,
    BUF_FMT_RESERVED_79__GFX10                         = 0x0000004f,
    BUF_FMT_RESERVED_80__GFX10                         = 0x00000050,
    BUF_FMT_RESERVED_81__GFX10                         = 0x00000051,
    BUF_FMT_RESERVED_82__GFX10                         = 0x00000052,
    BUF_FMT_RESERVED_83__GFX10                         = 0x00000053,
    BUF_FMT_RESERVED_84__GFX10                         = 0x00000054,
    BUF_FMT_RESERVED_85__GFX10                         = 0x00000055,
    BUF_FMT_RESERVED_86__GFX10                         = 0x00000056,
    BUF_FMT_RESERVED_87__GFX10                         = 0x00000057,
    BUF_FMT_RESERVED_88__GFX10                         = 0x00000058,
    BUF_FMT_RESERVED_89__GFX10                         = 0x00000059,
    BUF_FMT_RESERVED_90__GFX10                         = 0x0000005a,
    BUF_FMT_RESERVED_91__GFX10                         = 0x0000005b,
    BUF_FMT_RESERVED_92__GFX10                         = 0x0000005c,
    BUF_FMT_RESERVED_93__GFX10                         = 0x0000005d,
    BUF_FMT_RESERVED_94__GFX10                         = 0x0000005e,
    BUF_FMT_RESERVED_95__GFX10                         = 0x0000005f,
    BUF_FMT_RESERVED_96__GFX10                         = 0x00000060,
    BUF_FMT_RESERVED_97__GFX10                         = 0x00000061,
    BUF_FMT_RESERVED_98__GFX10                         = 0x00000062,
    BUF_FMT_RESERVED_99__GFX10                         = 0x00000063,
    BUF_FMT_RESERVED_100__GFX10                        = 0x00000064,
    BUF_FMT_RESERVED_101__GFX10                        = 0x00000065,
    BUF_FMT_RESERVED_102__GFX10                        = 0x00000066,
    BUF_FMT_RESERVED_103__GFX10                        = 0x00000067,
    BUF_FMT_RESERVED_104__GFX10                        = 0x00000068,
    BUF_FMT_RESERVED_105__GFX10                        = 0x00000069,
    BUF_FMT_RESERVED_106__GFX10                        = 0x0000006a,
    BUF_FMT_RESERVED_107__GFX10                        = 0x0000006b,
    BUF_FMT_RESERVED_108__GFX10                        = 0x0000006c,
    BUF_FMT_RESERVED_109__GFX10                        = 0x0000006d,
    BUF_FMT_RESERVED_110__GFX10                        = 0x0000006e,
    BUF_FMT_RESERVED_111__GFX10                        = 0x0000006f,
    BUF_FMT_RESERVED_112__GFX10                        = 0x00000070,
    BUF_FMT_RESERVED_113__GFX10                        = 0x00000071,
    BUF_FMT_RESERVED_114__GFX10                        = 0x00000072,
    BUF_FMT_RESERVED_115__GFX10                        = 0x00000073,
    BUF_FMT_RESERVED_116__GFX10                        = 0x00000074,
    BUF_FMT_RESERVED_117__GFX10                        = 0x00000075,
    BUF_FMT_RESERVED_118__GFX10                        = 0x00000076,
    BUF_FMT_RESERVED_119__GFX10                        = 0x00000077,
    BUF_FMT_RESERVED_120__GFX10                        = 0x00000078,
    BUF_FMT_RESERVED_121__GFX10                        = 0x00000079,
    BUF_FMT_RESERVED_122__GFX10                        = 0x0000007a,
    BUF_FMT_RESERVED_123__GFX10                        = 0x0000007b,
    BUF_FMT_RESERVED_124__GFX10                        = 0x0000007c,
    BUF_FMT_RESERVED_125__GFX10                        = 0x0000007d,
    BUF_FMT_RESERVED_126__GFX10                        = 0x0000007e,
    BUF_FMT_RESERVED_127__GFX10                        = 0x0000007f,
    BUF_FMT_10_11_11_FLOAT__GFX11                      = 0x0000001e,
    BUF_FMT_11_11_10_FLOAT__GFX11                      = 0x0000001f,
    BUF_FMT_10_10_10_2_UNORM__GFX11                    = 0x00000020,
    BUF_FMT_10_10_10_2_SNORM__GFX11                    = 0x00000021,
    BUF_FMT_10_10_10_2_UINT__GFX11                     = 0x00000022,
    BUF_FMT_10_10_10_2_SINT__GFX11                     = 0x00000023,
    BUF_FMT_2_10_10_10_UNORM__GFX11                    = 0x00000024,
    BUF_FMT_2_10_10_10_SNORM__GFX11                    = 0x00000025,
    BUF_FMT_2_10_10_10_USCALED__GFX11                  = 0x00000026,
    BUF_FMT_2_10_10_10_SSCALED__GFX11                  = 0x00000027,
    BUF_FMT_2_10_10_10_UINT__GFX11                     = 0x00000028,
    BUF_FMT_2_10_10_10_SINT__GFX11                     = 0x00000029,
    BUF_FMT_8_8_8_8_UNORM__GFX11                       = 0x0000002a,
    BUF_FMT_8_8_8_8_SNORM__GFX11                       = 0x0000002b,
    BUF_FMT_8_8_8_8_USCALED__GFX11                     = 0x0000002c,
    BUF_FMT_8_8_8_8_SSCALED__GFX11                     = 0x0000002d,
    BUF_FMT_8_8_8_8_UINT__GFX11                        = 0x0000002e,
    BUF_FMT_8_8_8_8_SINT__GFX11                        = 0x0000002f,
    BUF_FMT_32_32_UINT__GFX11                          = 0x00000030,
    BUF_FMT_32_32_SINT__GFX11                          = 0x00000031,
    BUF_FMT_32_32_FLOAT__GFX11                         = 0x00000032,
    BUF_FMT_16_16_16_16_UNORM__GFX11                   = 0x00000033,
    BUF_FMT_16_16_16_16_SNORM__GFX11                   = 0x00000034,
    BUF_FMT_16_16_16_16_USCALED__GFX11                 = 0x00000035,
    BUF_FMT_16_16_16_16_SSCALED__GFX11                 = 0x00000036,
    BUF_FMT_16_16_16_16_UINT__GFX11                    = 0x00000037,
    BUF_FMT_16_16_16_16_SINT__GFX11                    = 0x00000038,
    BUF_FMT_16_16_16_16_FLOAT__GFX11                   = 0x00000039,
    BUF_FMT_32_32_32_UINT__GFX11                       = 0x0000003a,
    BUF_FMT_32_32_32_SINT__GFX11                       = 0x0000003b,
    BUF_FMT_32_32_32_FLOAT__GFX11                      = 0x0000003c,
    BUF_FMT_32_32_32_32_UINT__GFX11                    = 0x0000003d,
    BUF_FMT_32_32_32_32_SINT__GFX11                    = 0x0000003e,
    BUF_FMT_32_32_32_32_FLOAT__GFX11                   = 0x0000003f,
} BUF_FMT;

typedef enum BUF_NUM_FORMAT {
    BUF_NUM_FORMAT_UNORM                               = 0x00000000,
    BUF_NUM_FORMAT_SNORM                               = 0x00000001,
    BUF_NUM_FORMAT_USCALED                             = 0x00000002,
    BUF_NUM_FORMAT_SSCALED                             = 0x00000003,
    BUF_NUM_FORMAT_UINT                                = 0x00000004,
    BUF_NUM_FORMAT_SINT                                = 0x00000005,
    BUF_NUM_FORMAT_FLOAT                               = 0x00000007,
    BUF_NUM_FORMAT_SNORM_NZ__GFX10                     = 0x00000006,
    BUF_NUM_FORMAT_RESERVED_6__GFX11                   = 0x00000006,
} BUF_NUM_FORMAT;

typedef enum CBMode {
    CB_DISABLE                                         = 0x00000000,
    CB_NORMAL                                          = 0x00000001,
    CB_ELIMINATE_FAST_CLEAR                            = 0x00000002,
    CB_RESOLVE__GFX10                                  = 0x00000003,
    CB_DECOMPRESS__GFX10                               = 0x00000004,
    CB_FMASK_DECOMPRESS__GFX10                         = 0x00000005,
    CB_DCC_DECOMPRESS__GFX10                           = 0x00000006,
    CB_RESERVED__GFX10                                 = 0x00000007,
    CB_DCC_DECOMPRESS__GFX11                           = 0x00000003,
    CB_RESERVED__GFX11                                 = 0x00000004,
} CBMode;

typedef enum CBPerfClearFilterSel {
    CB_PERF_CLEAR_FILTER_SEL_NONCLEAR                  = 0x00000000,
    CB_PERF_CLEAR_FILTER_SEL_CLEAR                     = 0x00000001,
} CBPerfClearFilterSel;

typedef enum CBPerfOpFilterSel {
    CB_PERF_OP_FILTER_SEL_WRITE_ONLY                   = 0x00000000,
    CB_PERF_OP_FILTER_SEL_NEEDS_DESTINATION            = 0x00000001,
    CB_PERF_OP_FILTER_SEL_RESOLVE                      = 0x00000002,
    CB_PERF_OP_FILTER_SEL_DECOMPRESS                   = 0x00000003,
    CB_PERF_OP_FILTER_SEL_FMASK_DECOMPRESS             = 0x00000004,
    CB_PERF_OP_FILTER_SEL_ELIMINATE_FAST_CLEAR         = 0x00000005,
} CBPerfOpFilterSel;

typedef enum CBPerfSel {
    CB_PERF_SEL_BUSY                                   = 0x00000001,
    CB_PERF_SEL_NONE__GFX10                            = 0x00000000,
    CB_PERF_SEL_CORE_SCLK_VLD__GFX10                   = 0x00000002,
    CB_PERF_SEL_REG_SCLK0_VLD__GFX10                   = 0x00000003,
    CB_PERF_SEL_REG_SCLK1_VLD__GFX10                   = 0x00000004,
    CB_PERF_SEL_DRAWN_QUAD__GFX10                      = 0x00000005,
    CB_PERF_SEL_DRAWN_PIXEL__GFX10                     = 0x00000006,
    CB_PERF_SEL_DRAWN_QUAD_FRAGMENT__GFX10             = 0x00000007,
    CB_PERF_SEL_DRAWN_TILE__GFX10                      = 0x00000008,
    CB_PERF_SEL_DB_CB_TILE_VALID_READY__GFX10          = 0x00000009,
    CB_PERF_SEL_DB_CB_TILE_VALID_READYB__GFX10         = 0x0000000a,
    CB_PERF_SEL_DB_CB_TILE_VALIDB_READY__GFX10         = 0x0000000b,
    CB_PERF_SEL_DB_CB_TILE_VALIDB_READYB__GFX10        = 0x0000000c,
    CB_PERF_SEL_CM_FC_TILE_VALID_READY__GFX10          = 0x0000000d,
    CB_PERF_SEL_CM_FC_TILE_VALID_READYB__GFX10         = 0x0000000e,
    CB_PERF_SEL_CM_FC_TILE_VALIDB_READY__GFX10         = 0x0000000f,
    CB_PERF_SEL_CM_FC_TILE_VALIDB_READYB__GFX10        = 0x00000010,
    CB_PERF_SEL_MERGE_TILE_ONLY_VALID_READY__GFX10     = 0x00000011,
    CB_PERF_SEL_MERGE_TILE_ONLY_VALID_READYB__GFX10    = 0x00000012,
    CB_PERF_SEL_DB_CB_LQUAD_VALID_READY__GFX10         = 0x00000013,
    CB_PERF_SEL_DB_CB_LQUAD_VALID_READYB__GFX10        = 0x00000014,
    CB_PERF_SEL_DB_CB_LQUAD_VALIDB_READY__GFX10        = 0x00000015,
    CB_PERF_SEL_DB_CB_LQUAD_VALIDB_READYB__GFX10       = 0x00000016,
    CB_PERF_SEL_LQUAD_NO_TILE__GFX10                   = 0x00000017,
    CB_PERF_SEL_LQUAD_FORMAT_IS_EXPORT_32_R__GFX10     = 0x00000018,
    CB_PERF_SEL_LQUAD_FORMAT_IS_EXPORT_32_AR__GFX10    = 0x00000019,
    CB_PERF_SEL_LQUAD_FORMAT_IS_EXPORT_32_GR__GFX10    = 0x0000001a,
    CB_PERF_SEL_LQUAD_FORMAT_IS_EXPORT_32_ABGR__GFX10  = 0x0000001b,
    CB_PERF_SEL_LQUAD_FORMAT_IS_EXPORT_FP16_ABGR__GFX10 = 0x0000001c,
    CB_PERF_SEL_LQUAD_FORMAT_IS_EXPORT_SIGNED16_ABGR__GFX10 = 0x0000001d,
    CB_PERF_SEL_LQUAD_FORMAT_IS_EXPORT_UNSIGNED16_ABGR__GFX10 = 0x0000001e,
    CB_PERF_SEL_QUAD_KILLED_BY_EXTRA_PIXEL_EXPORT__GFX10 = 0x0000001f,
    CB_PERF_SEL_QUAD_KILLED_BY_COLOR_INVALID__GFX10    = 0x00000020,
    CB_PERF_SEL_QUAD_KILLED_BY_NULL_TARGET_SHADER_MASK__GFX10 = 0x00000021,
    CB_PERF_SEL_QUAD_KILLED_BY_NULL_SAMPLE_MASK__GFX10 = 0x00000022,
    CB_PERF_SEL_QUAD_KILLED_BY_DISCARD_PIXEL__GFX10    = 0x00000023,
    CB_PERF_SEL_FC_CLEAR_QUAD_VALID_READY__GFX10       = 0x00000024,
    CB_PERF_SEL_FC_CLEAR_QUAD_VALID_READYB__GFX10      = 0x00000025,
    CB_PERF_SEL_FC_CLEAR_QUAD_VALIDB_READY__GFX10      = 0x00000026,
    CB_PERF_SEL_FC_CLEAR_QUAD_VALIDB_READYB__GFX10     = 0x00000027,
    CB_PERF_SEL_FOP_IN_VALID_READY__GFX10              = 0x00000028,
    CB_PERF_SEL_FOP_IN_VALID_READYB__GFX10             = 0x00000029,
    CB_PERF_SEL_FOP_IN_VALIDB_READY__GFX10             = 0x0000002a,
    CB_PERF_SEL_FOP_IN_VALIDB_READYB__GFX10            = 0x0000002b,
    CB_PERF_SEL_FC_CC_QUADFRAG_VALID_READY__GFX10      = 0x0000002c,
    CB_PERF_SEL_FC_CC_QUADFRAG_VALID_READYB__GFX10     = 0x0000002d,
    CB_PERF_SEL_FC_CC_QUADFRAG_VALIDB_READY__GFX10     = 0x0000002e,
    CB_PERF_SEL_FC_CC_QUADFRAG_VALIDB_READYB__GFX10    = 0x0000002f,
    CB_PERF_SEL_CC_IB_SR_FRAG_VALID_READY__GFX10       = 0x00000030,
    CB_PERF_SEL_CC_IB_SR_FRAG_VALID_READYB__GFX10      = 0x00000031,
    CB_PERF_SEL_CC_IB_SR_FRAG_VALIDB_READY__GFX10      = 0x00000032,
    CB_PERF_SEL_CC_IB_SR_FRAG_VALIDB_READYB__GFX10     = 0x00000033,
    CB_PERF_SEL_CC_IB_TB_FRAG_VALID_READY__GFX10       = 0x00000034,
    CB_PERF_SEL_CC_IB_TB_FRAG_VALID_READYB__GFX10      = 0x00000035,
    CB_PERF_SEL_CC_IB_TB_FRAG_VALIDB_READY__GFX10      = 0x00000036,
    CB_PERF_SEL_CC_IB_TB_FRAG_VALIDB_READYB__GFX10     = 0x00000037,
    CB_PERF_SEL_CC_RB_BC_EVENFRAG_VALID_READY__GFX10   = 0x00000038,
    CB_PERF_SEL_CC_RB_BC_EVENFRAG_VALID_READYB__GFX10  = 0x00000039,
    CB_PERF_SEL_CC_RB_BC_EVENFRAG_VALIDB_READY__GFX10  = 0x0000003a,
    CB_PERF_SEL_CC_RB_BC_EVENFRAG_VALIDB_READYB__GFX10 = 0x0000003b,
    CB_PERF_SEL_CC_RB_BC_ODDFRAG_VALID_READY__GFX10    = 0x0000003c,
    CB_PERF_SEL_CC_RB_BC_ODDFRAG_VALID_READYB__GFX10   = 0x0000003d,
    CB_PERF_SEL_CC_RB_BC_ODDFRAG_VALIDB_READY__GFX10   = 0x0000003e,
    CB_PERF_SEL_CC_RB_BC_ODDFRAG_VALIDB_READYB__GFX10  = 0x0000003f,
    CB_PERF_SEL_CC_BC_CS_FRAG_VALID__GFX10             = 0x00000040,
    CB_PERF_SEL_CM_CACHE_HIT__GFX10                    = 0x00000041,
    CB_PERF_SEL_CM_CACHE_TAG_MISS__GFX10               = 0x00000042,
    CB_PERF_SEL_CM_CACHE_SECTOR_MISS__GFX10            = 0x00000043,
    CB_PERF_SEL_CM_CACHE_REEVICTION_STALL__GFX10       = 0x00000044,
    CB_PERF_SEL_CM_CACHE_EVICT_NONZERO_INFLIGHT_STALL__GFX10 = 0x00000045,
    CB_PERF_SEL_CM_CACHE_REPLACE_PENDING_EVICT_STALL__GFX10 = 0x00000046,
    CB_PERF_SEL_CM_CACHE_INFLIGHT_COUNTER_MAXIMUM_STALL__GFX10 = 0x00000047,
    CB_PERF_SEL_CM_CACHE_READ_OUTPUT_STALL__GFX10      = 0x00000048,
    CB_PERF_SEL_CM_CACHE_WRITE_OUTPUT_STALL__GFX10     = 0x00000049,
    CB_PERF_SEL_CM_CACHE_ACK_OUTPUT_STALL__GFX10       = 0x0000004a,
    CB_PERF_SEL_CM_CACHE_STALL__GFX10                  = 0x0000004b,
    CB_PERF_SEL_CM_CACHE_FLUSH__GFX10                  = 0x0000004c,
    CB_PERF_SEL_CM_CACHE_TAGS_FLUSHED__GFX10           = 0x0000004d,
    CB_PERF_SEL_CM_CACHE_SECTORS_FLUSHED__GFX10        = 0x0000004e,
    CB_PERF_SEL_CM_CACHE_DIRTY_SECTORS_FLUSHED__GFX10  = 0x0000004f,
    CB_PERF_SEL_FC_CACHE_HIT__GFX10                    = 0x00000050,
    CB_PERF_SEL_FC_CACHE_TAG_MISS__GFX10               = 0x00000051,
    CB_PERF_SEL_FC_CACHE_SECTOR_MISS__GFX10            = 0x00000052,
    CB_PERF_SEL_FC_CACHE_REEVICTION_STALL__GFX10       = 0x00000053,
    CB_PERF_SEL_FC_CACHE_EVICT_NONZERO_INFLIGHT_STALL__GFX10 = 0x00000054,
    CB_PERF_SEL_FC_CACHE_REPLACE_PENDING_EVICT_STALL__GFX10 = 0x00000055,
    CB_PERF_SEL_FC_CACHE_INFLIGHT_COUNTER_MAXIMUM_STALL__GFX10 = 0x00000056,
    CB_PERF_SEL_FC_CACHE_READ_OUTPUT_STALL__GFX10      = 0x00000057,
    CB_PERF_SEL_FC_CACHE_WRITE_OUTPUT_STALL__GFX10     = 0x00000058,
    CB_PERF_SEL_FC_CACHE_ACK_OUTPUT_STALL__GFX10       = 0x00000059,
    CB_PERF_SEL_FC_CACHE_STALL__GFX10                  = 0x0000005a,
    CB_PERF_SEL_FC_CACHE_FLUSH__GFX10                  = 0x0000005b,
    CB_PERF_SEL_FC_CACHE_TAGS_FLUSHED__GFX10           = 0x0000005c,
    CB_PERF_SEL_FC_CACHE_SECTORS_FLUSHED__GFX10        = 0x0000005d,
    CB_PERF_SEL_FC_CACHE_DIRTY_SECTORS_FLUSHED__GFX10  = 0x0000005e,
    CB_PERF_SEL_CC_CACHE_HIT__GFX10                    = 0x0000005f,
    CB_PERF_SEL_CC_CACHE_TAG_MISS__GFX10               = 0x00000060,
    CB_PERF_SEL_CC_CACHE_SECTOR_MISS__GFX10            = 0x00000061,
    CB_PERF_SEL_CC_CACHE_REEVICTION_STALL__GFX10       = 0x00000062,
    CB_PERF_SEL_CC_CACHE_EVICT_NONZERO_INFLIGHT_STALL__GFX10 = 0x00000063,
    CB_PERF_SEL_CC_CACHE_REPLACE_PENDING_EVICT_STALL__GFX10 = 0x00000064,
    CB_PERF_SEL_CC_CACHE_INFLIGHT_COUNTER_MAXIMUM_STALL__GFX10 = 0x00000065,
    CB_PERF_SEL_CC_CACHE_READ_OUTPUT_STALL__GFX10      = 0x00000066,
    CB_PERF_SEL_CC_CACHE_WRITE_OUTPUT_STALL__GFX10     = 0x00000067,
    CB_PERF_SEL_CC_CACHE_ACK_OUTPUT_STALL__GFX10       = 0x00000068,
    CB_PERF_SEL_CC_CACHE_STALL__GFX10                  = 0x00000069,
    CB_PERF_SEL_CC_CACHE_FLUSH__GFX10                  = 0x0000006a,
    CB_PERF_SEL_CC_CACHE_TAGS_FLUSHED__GFX10           = 0x0000006b,
    CB_PERF_SEL_CC_CACHE_SECTORS_FLUSHED__GFX10        = 0x0000006c,
    CB_PERF_SEL_CC_CACHE_DIRTY_SECTORS_FLUSHED__GFX10  = 0x0000006d,
    CB_PERF_SEL_CC_CACHE_WA_TO_RMW_CONVERSION__GFX10   = 0x0000006e,
    CB_PERF_SEL_CC_CACHE_READS_SAVED_DUE_TO_DCC__GFX10 = 0x0000006f,
    CB_PERF_SEL_CB_TAP_WRREQ_VALID_READY__GFX10        = 0x00000070,
    CB_PERF_SEL_CB_TAP_WRREQ_VALID_READYB__GFX10       = 0x00000071,
    CB_PERF_SEL_CB_TAP_WRREQ_VALIDB_READY__GFX10       = 0x00000072,
    CB_PERF_SEL_CB_TAP_WRREQ_VALIDB_READYB__GFX10      = 0x00000073,
    CB_PERF_SEL_CM_MC_WRITE_REQUEST__GFX10             = 0x00000074,
    CB_PERF_SEL_FC_MC_WRITE_REQUEST__GFX10             = 0x00000075,
    CB_PERF_SEL_CC_MC_WRITE_REQUEST__GFX10             = 0x00000076,
    CB_PERF_SEL_CM_MC_WRITE_REQUESTS_IN_FLIGHT__GFX10  = 0x00000077,
    CB_PERF_SEL_FC_MC_WRITE_REQUESTS_IN_FLIGHT__GFX10  = 0x00000078,
    CB_PERF_SEL_CC_MC_WRITE_REQUESTS_IN_FLIGHT__GFX10  = 0x00000079,
    CB_PERF_SEL_CB_TAP_RDREQ_VALID_READY__GFX10        = 0x0000007a,
    CB_PERF_SEL_CB_TAP_RDREQ_VALID_READYB__GFX10       = 0x0000007b,
    CB_PERF_SEL_CB_TAP_RDREQ_VALIDB_READY__GFX10       = 0x0000007c,
    CB_PERF_SEL_CB_TAP_RDREQ_VALIDB_READYB__GFX10      = 0x0000007d,
    CB_PERF_SEL_CM_MC_READ_REQUEST__GFX10              = 0x0000007e,
    CB_PERF_SEL_FC_MC_READ_REQUEST__GFX10              = 0x0000007f,
    CB_PERF_SEL_CC_MC_READ_REQUEST__GFX10              = 0x00000080,
    CB_PERF_SEL_CM_MC_READ_REQUESTS_IN_FLIGHT__GFX10   = 0x00000081,
    CB_PERF_SEL_FC_MC_READ_REQUESTS_IN_FLIGHT__GFX10   = 0x00000082,
    CB_PERF_SEL_CC_MC_READ_REQUESTS_IN_FLIGHT__GFX10   = 0x00000083,
    CB_PERF_SEL_CM_TQ_FULL__GFX10                      = 0x00000084,
    CB_PERF_SEL_CM_TQ_FIFO_TILE_RESIDENCY_STALL__GFX10 = 0x00000085,
    CB_PERF_SEL_FC_QUAD_RDLAT_FIFO_FULL__GFX10         = 0x00000086,
    CB_PERF_SEL_FC_TILE_RDLAT_FIFO_FULL__GFX10         = 0x00000087,
    CB_PERF_SEL_FC_RDLAT_FIFO_QUAD_RESIDENCY_STALL__GFX10 = 0x00000088,
    CB_PERF_SEL_FOP_FMASK_RAW_STALL__GFX10             = 0x00000089,
    CB_PERF_SEL_FOP_FMASK_BYPASS_STALL__GFX10          = 0x0000008a,
    CB_PERF_SEL_CC_SF_FULL__GFX10                      = 0x0000008b,
    CB_PERF_SEL_CC_RB_FULL__GFX10                      = 0x0000008c,
    CB_PERF_SEL_CC_EVENFIFO_QUAD_RESIDENCY_STALL__GFX10 = 0x0000008d,
    CB_PERF_SEL_CC_ODDFIFO_QUAD_RESIDENCY_STALL__GFX10 = 0x0000008e,
    CB_PERF_SEL_BLENDER_RAW_HAZARD_STALL__GFX10        = 0x0000008f,
    CB_PERF_SEL_EVENT__GFX10                           = 0x00000090,
    CB_PERF_SEL_EVENT_CACHE_FLUSH_TS__GFX10            = 0x00000091,
    CB_PERF_SEL_EVENT_CONTEXT_DONE__GFX10              = 0x00000092,
    CB_PERF_SEL_EVENT_CACHE_FLUSH__GFX10               = 0x00000093,
    CB_PERF_SEL_EVENT_CACHE_FLUSH_AND_INV_TS_EVENT__GFX10 = 0x00000094,
    CB_PERF_SEL_EVENT_CACHE_FLUSH_AND_INV_EVENT__GFX10 = 0x00000095,
    CB_PERF_SEL_EVENT_FLUSH_AND_INV_CB_DATA_TS__GFX10  = 0x00000096,
    CB_PERF_SEL_EVENT_FLUSH_AND_INV_CB_META__GFX10     = 0x00000097,
    CB_PERF_SEL_CC_SURFACE_SYNC__GFX10                 = 0x00000098,
    CB_PERF_SEL_CMASK_READ_DATA_0xC__GFX10             = 0x00000099,
    CB_PERF_SEL_CMASK_READ_DATA_0xD__GFX10             = 0x0000009a,
    CB_PERF_SEL_CMASK_READ_DATA_0xE__GFX10             = 0x0000009b,
    CB_PERF_SEL_CMASK_READ_DATA_0xF__GFX10             = 0x0000009c,
    CB_PERF_SEL_CMASK_WRITE_DATA_0xC__GFX10            = 0x0000009d,
    CB_PERF_SEL_CMASK_WRITE_DATA_0xD__GFX10            = 0x0000009e,
    CB_PERF_SEL_CMASK_WRITE_DATA_0xE__GFX10            = 0x0000009f,
    CB_PERF_SEL_CMASK_WRITE_DATA_0xF__GFX10            = 0x000000a0,
    CB_PERF_SEL_TWO_PROBE_QUAD_FRAGMENT__GFX10         = 0x000000a1,
    CB_PERF_SEL_EXPORT_32_ABGR_QUAD_FRAGMENT__GFX10    = 0x000000a2,
    CB_PERF_SEL_DUAL_SOURCE_COLOR_QUAD_FRAGMENT__GFX10 = 0x000000a3,
    CB_PERF_SEL_QUAD_HAS_1_FRAGMENT_BEFORE_UPDATE__GFX10 = 0x000000a4,
    CB_PERF_SEL_QUAD_HAS_2_FRAGMENTS_BEFORE_UPDATE__GFX10 = 0x000000a5,
    CB_PERF_SEL_QUAD_HAS_3_FRAGMENTS_BEFORE_UPDATE__GFX10 = 0x000000a6,
    CB_PERF_SEL_QUAD_HAS_4_FRAGMENTS_BEFORE_UPDATE__GFX10 = 0x000000a7,
    CB_PERF_SEL_QUAD_HAS_5_FRAGMENTS_BEFORE_UPDATE__GFX10 = 0x000000a8,
    CB_PERF_SEL_QUAD_HAS_6_FRAGMENTS_BEFORE_UPDATE__GFX10 = 0x000000a9,
    CB_PERF_SEL_QUAD_HAS_7_FRAGMENTS_BEFORE_UPDATE__GFX10 = 0x000000aa,
    CB_PERF_SEL_QUAD_HAS_8_FRAGMENTS_BEFORE_UPDATE__GFX10 = 0x000000ab,
    CB_PERF_SEL_QUAD_HAS_1_FRAGMENT_AFTER_UPDATE__GFX10 = 0x000000ac,
    CB_PERF_SEL_QUAD_HAS_2_FRAGMENTS_AFTER_UPDATE__GFX10 = 0x000000ad,
    CB_PERF_SEL_QUAD_HAS_3_FRAGMENTS_AFTER_UPDATE__GFX10 = 0x000000ae,
    CB_PERF_SEL_QUAD_HAS_4_FRAGMENTS_AFTER_UPDATE__GFX10 = 0x000000af,
    CB_PERF_SEL_QUAD_HAS_5_FRAGMENTS_AFTER_UPDATE__GFX10 = 0x000000b0,
    CB_PERF_SEL_QUAD_HAS_6_FRAGMENTS_AFTER_UPDATE__GFX10 = 0x000000b1,
    CB_PERF_SEL_QUAD_HAS_7_FRAGMENTS_AFTER_UPDATE__GFX10 = 0x000000b2,
    CB_PERF_SEL_QUAD_HAS_8_FRAGMENTS_AFTER_UPDATE__GFX10 = 0x000000b3,
    CB_PERF_SEL_QUAD_ADDED_1_FRAGMENT__GFX10           = 0x000000b4,
    CB_PERF_SEL_QUAD_ADDED_2_FRAGMENTS__GFX10          = 0x000000b5,
    CB_PERF_SEL_QUAD_ADDED_3_FRAGMENTS__GFX10          = 0x000000b6,
    CB_PERF_SEL_QUAD_ADDED_4_FRAGMENTS__GFX10          = 0x000000b7,
    CB_PERF_SEL_QUAD_ADDED_5_FRAGMENTS__GFX10          = 0x000000b8,
    CB_PERF_SEL_QUAD_ADDED_6_FRAGMENTS__GFX10          = 0x000000b9,
    CB_PERF_SEL_QUAD_ADDED_7_FRAGMENTS__GFX10          = 0x000000ba,
    CB_PERF_SEL_QUAD_REMOVED_1_FRAGMENT__GFX10         = 0x000000bb,
    CB_PERF_SEL_QUAD_REMOVED_2_FRAGMENTS__GFX10        = 0x000000bc,
    CB_PERF_SEL_QUAD_REMOVED_3_FRAGMENTS__GFX10        = 0x000000bd,
    CB_PERF_SEL_QUAD_REMOVED_4_FRAGMENTS__GFX10        = 0x000000be,
    CB_PERF_SEL_QUAD_REMOVED_5_FRAGMENTS__GFX10        = 0x000000bf,
    CB_PERF_SEL_QUAD_REMOVED_6_FRAGMENTS__GFX10        = 0x000000c0,
    CB_PERF_SEL_QUAD_REMOVED_7_FRAGMENTS__GFX10        = 0x000000c1,
    CB_PERF_SEL_QUAD_READS_FRAGMENT_0__GFX10           = 0x000000c2,
    CB_PERF_SEL_QUAD_READS_FRAGMENT_1__GFX10           = 0x000000c3,
    CB_PERF_SEL_QUAD_READS_FRAGMENT_2__GFX10           = 0x000000c4,
    CB_PERF_SEL_QUAD_READS_FRAGMENT_3__GFX10           = 0x000000c5,
    CB_PERF_SEL_QUAD_READS_FRAGMENT_4__GFX10           = 0x000000c6,
    CB_PERF_SEL_QUAD_READS_FRAGMENT_5__GFX10           = 0x000000c7,
    CB_PERF_SEL_QUAD_READS_FRAGMENT_6__GFX10           = 0x000000c8,
    CB_PERF_SEL_QUAD_READS_FRAGMENT_7__GFX10           = 0x000000c9,
    CB_PERF_SEL_QUAD_WRITES_FRAGMENT_0__GFX10          = 0x000000ca,
    CB_PERF_SEL_QUAD_WRITES_FRAGMENT_1__GFX10          = 0x000000cb,
    CB_PERF_SEL_QUAD_WRITES_FRAGMENT_2__GFX10          = 0x000000cc,
    CB_PERF_SEL_QUAD_WRITES_FRAGMENT_3__GFX10          = 0x000000cd,
    CB_PERF_SEL_QUAD_WRITES_FRAGMENT_4__GFX10          = 0x000000ce,
    CB_PERF_SEL_QUAD_WRITES_FRAGMENT_5__GFX10          = 0x000000cf,
    CB_PERF_SEL_QUAD_WRITES_FRAGMENT_6__GFX10          = 0x000000d0,
    CB_PERF_SEL_QUAD_WRITES_FRAGMENT_7__GFX10          = 0x000000d1,
    CB_PERF_SEL_QUAD_BLEND_OPT_DONT_READ_DST__GFX10    = 0x000000d2,
    CB_PERF_SEL_QUAD_BLEND_OPT_BLEND_BYPASS__GFX10     = 0x000000d3,
    CB_PERF_SEL_QUAD_BLEND_OPT_DISCARD_PIXELS__GFX10   = 0x000000d4,
    CB_PERF_SEL_QUAD_DST_READ_COULD_HAVE_BEEN_OPTIMIZED__GFX10 = 0x000000d5,
    CB_PERF_SEL_QUAD_BLENDING_COULD_HAVE_BEEN_BYPASSED__GFX10 = 0x000000d6,
    CB_PERF_SEL_QUAD_COULD_HAVE_BEEN_DISCARDED__GFX10  = 0x000000d7,
    CB_PERF_SEL_BLEND_OPT_PIXELS_RESULT_EQ_DEST__GFX10 = 0x000000d8,
    CB_PERF_SEL_DRAWN_BUSY__GFX10                      = 0x000000d9,
    CB_PERF_SEL_TILE_TO_CMR_REGION_BUSY__GFX10         = 0x000000da,
    CB_PERF_SEL_CMR_TO_FCR_REGION_BUSY__GFX10          = 0x000000db,
    CB_PERF_SEL_FCR_TO_CCR_REGION_BUSY__GFX10          = 0x000000dc,
    CB_PERF_SEL_CCR_TO_CCW_REGION_BUSY__GFX10          = 0x000000dd,
    CB_PERF_SEL_FC_PF_SLOW_MODE_QUAD_EMPTY_HALF_DROPPED__GFX10 = 0x000000de,
    CB_PERF_SEL_FC_SEQUENCER_CLEAR__GFX10              = 0x000000df,
    CB_PERF_SEL_FC_SEQUENCER_ELIMINATE_FAST_CLEAR__GFX10 = 0x000000e0,
    CB_PERF_SEL_FC_SEQUENCER_FMASK_DECOMPRESS__GFX10   = 0x000000e1,
    CB_PERF_SEL_FC_SEQUENCER_FMASK_COMPRESSION_DISABLE__GFX10 = 0x000000e2,
    CB_PERF_SEL_FC_KEYID_RDLAT_FIFO_FULL__GFX10        = 0x000000e3,
    CB_PERF_SEL_FC_DOC_IS_STALLED__GFX10               = 0x000000e4,
    CB_PERF_SEL_FC_DOC_MRTS_NOT_COMBINED__GFX10        = 0x000000e5,
    CB_PERF_SEL_FC_DOC_MRTS_COMBINED__GFX10            = 0x000000e6,
    CB_PERF_SEL_FC_DOC_QTILE_CAM_MISS__GFX10           = 0x000000e7,
    CB_PERF_SEL_FC_DOC_QTILE_CAM_HIT__GFX10            = 0x000000e8,
    CB_PERF_SEL_FC_DOC_CLINE_CAM_MISS__GFX10           = 0x000000e9,
    CB_PERF_SEL_FC_DOC_CLINE_CAM_HIT__GFX10            = 0x000000ea,
    CB_PERF_SEL_FC_DOC_QUAD_PTR_FIFO_IS_FULL__GFX10    = 0x000000eb,
    CB_PERF_SEL_FC_DOC_OVERWROTE_1_SECTOR__GFX10       = 0x000000ec,
    CB_PERF_SEL_FC_DOC_OVERWROTE_2_SECTORS__GFX10      = 0x000000ed,
    CB_PERF_SEL_FC_DOC_OVERWROTE_3_SECTORS__GFX10      = 0x000000ee,
    CB_PERF_SEL_FC_DOC_OVERWROTE_4_SECTORS__GFX10      = 0x000000ef,
    CB_PERF_SEL_FC_DOC_TOTAL_OVERWRITTEN_SECTORS__GFX10 = 0x000000f0,
    CB_PERF_SEL_FC_DCC_CACHE_HIT__GFX10                = 0x000000f1,
    CB_PERF_SEL_FC_DCC_CACHE_TAG_MISS__GFX10           = 0x000000f2,
    CB_PERF_SEL_FC_DCC_CACHE_SECTOR_MISS__GFX10        = 0x000000f3,
    CB_PERF_SEL_FC_DCC_CACHE_REEVICTION_STALL__GFX10   = 0x000000f4,
    CB_PERF_SEL_FC_DCC_CACHE_EVICT_NONZERO_INFLIGHT_STALL__GFX10 = 0x000000f5,
    CB_PERF_SEL_FC_DCC_CACHE_REPLACE_PENDING_EVICT_STALL__GFX10 = 0x000000f6,
    CB_PERF_SEL_FC_DCC_CACHE_INFLIGHT_COUNTER_MAXIMUM_STALL__GFX10 = 0x000000f7,
    CB_PERF_SEL_FC_DCC_CACHE_READ_OUTPUT_STALL__GFX10  = 0x000000f8,
    CB_PERF_SEL_FC_DCC_CACHE_WRITE_OUTPUT_STALL__GFX10 = 0x000000f9,
    CB_PERF_SEL_FC_DCC_CACHE_ACK_OUTPUT_STALL__GFX10   = 0x000000fa,
    CB_PERF_SEL_FC_DCC_CACHE_STALL__GFX10              = 0x000000fb,
    CB_PERF_SEL_FC_DCC_CACHE_FLUSH__GFX10              = 0x000000fc,
    CB_PERF_SEL_FC_DCC_CACHE_SECTORS_FLUSHED__GFX10    = 0x000000fd,
    CB_PERF_SEL_FC_DCC_CACHE_DIRTY_SECTORS_FLUSHED__GFX10 = 0x000000fe,
    CB_PERF_SEL_FC_DCC_CACHE_TAGS_FLUSHED__GFX10       = 0x000000ff,
    CB_PERF_SEL_CC_DCC_BEYOND_TILE_SPLIT__GFX10        = 0x00000100,
    CB_PERF_SEL_FC_MC_DCC_WRITE_REQUEST__GFX10         = 0x00000101,
    CB_PERF_SEL_FC_MC_DCC_WRITE_REQUESTS_IN_FLIGHT__GFX10 = 0x00000102,
    CB_PERF_SEL_FC_MC_DCC_READ_REQUEST__GFX10          = 0x00000103,
    CB_PERF_SEL_FC_MC_DCC_READ_REQUESTS_IN_FLIGHT__GFX10 = 0x00000104,
    CB_PERF_SEL_CC_DCC_RDREQ_STALL__GFX10              = 0x00000105,
    CB_PERF_SEL_CC_DCC_DECOMPRESS_TIDS_IN__GFX10       = 0x00000106,
    CB_PERF_SEL_CC_DCC_DECOMPRESS_TIDS_OUT__GFX10      = 0x00000107,
    CB_PERF_SEL_CC_DCC_COMPRESS_TIDS_IN__GFX10         = 0x00000108,
    CB_PERF_SEL_CC_DCC_COMPRESS_TIDS_OUT__GFX10        = 0x00000109,
    CB_PERF_SEL_FC_DCC_KEY_VALUE__CLEAR__GFX10         = 0x0000010a,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__4_BLOCKS__2TO1__GFX10 = 0x0000010b,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__3BLOCKS_2TO1__1BLOCK_2TO2__GFX10 = 0x0000010c,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__2BLOCKS_2TO1__1BLOCK_2TO2__1BLOCK_2TO1__GFX10 = 0x0000010d,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_2TO1__1BLOCK_2TO2__2BLOCKS_2TO1__GFX10 = 0x0000010e,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_2TO2__3BLOCKS_2TO1__GFX10 = 0x0000010f,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__2BLOCKS_2TO1__2BLOCKS_2TO2__GFX10 = 0x00000110,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_2TO1__2BLOCKS_2TO2__1BLOCK_2TO1__GFX10 = 0x00000111,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_2TO1__1BLOCK_2TO2__1BLOCK_2TO1__1BLOCK_2TO2__GFX10 = 0x00000112,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_2TO2__1BLOCK_2TO1__1BLOCK_2TO2__1BLOCK_2TO1__GFX10 = 0x00000113,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__2BLOCKS_2TO2__2BLOCKS_2TO1__GFX10 = 0x00000114,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_2TO2__2BLOCKS_2TO1__1BLOCK_2TO2__GFX10 = 0x00000115,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_2TO1__3BLOCKS_2TO2__GFX10 = 0x00000116,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_2TO2__1BLOCK_2TO1__2BLOCKS_2TO2__GFX10 = 0x00000117,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__2BLOCKS_2TO2__1BLOCK_2TO1__1BLOCK_2TO2__GFX10 = 0x00000118,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__3BLOCKS_2TO2__1BLOCK_2TO1__GFX10 = 0x00000119,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__2BLOCKS_4TO1__GFX10  = 0x0000011a,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_4TO1__1BLOCK_4TO2__GFX10 = 0x0000011b,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_4TO1__1BLOCK_4TO3__GFX10 = 0x0000011c,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_4TO1__1BLOCK_4TO4__GFX10 = 0x0000011d,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_4TO2__1BLOCK_4TO1__GFX10 = 0x0000011e,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__2BLOCKS_4TO2__GFX10  = 0x0000011f,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_4TO2__1BLOCK_4TO3__GFX10 = 0x00000120,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_4TO2__1BLOCK_4TO4__GFX10 = 0x00000121,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_4TO3__1BLOCK_4TO1__GFX10 = 0x00000122,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_4TO3__1BLOCK_4TO2__GFX10 = 0x00000123,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__2BLOCKS_4TO3__GFX10  = 0x00000124,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_4TO3__1BLOCK_4TO4__GFX10 = 0x00000125,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_4TO4__1BLOCK_4TO1__GFX10 = 0x00000126,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_4TO4__1BLOCK_4TO2__GFX10 = 0x00000127,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_4TO4__1BLOCK_4TO3__GFX10 = 0x00000128,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__2BLOCKS_2TO1__1BLOCK_4TO1__GFX10 = 0x00000129,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__2BLOCKS_2TO1__1BLOCK_4TO2__GFX10 = 0x0000012a,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__2BLOCKS_2TO1__1BLOCK_4TO3__GFX10 = 0x0000012b,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__2BLOCKS_2TO1__1BLOCK_4TO4__GFX10 = 0x0000012c,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_2TO1__1BLOCK_2TO2__1BLOCK_4TO1__GFX10 = 0x0000012d,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_2TO1__1BLOCK_2TO2__1BLOCK_4TO2__GFX10 = 0x0000012e,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_2TO1__1BLOCK_2TO2__1BLOCK_4TO3__GFX10 = 0x0000012f,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_2TO1__1BLOCK_2TO2__1BLOCK_4TO4__GFX10 = 0x00000130,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_2TO2__1BLOCK_2TO1__1BLOCK_4TO1__GFX10 = 0x00000131,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_2TO2__1BLOCK_2TO1__1BLOCK_4TO2__GFX10 = 0x00000132,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_2TO2__1BLOCK_2TO1__1BLOCK_4TO3__GFX10 = 0x00000133,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_2TO2__1BLOCK_2TO1__1BLOCK_4TO4__GFX10 = 0x00000134,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__2BLOCKS_2TO2__1BLOCK_4TO1__GFX10 = 0x00000135,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__2BLOCKS_2TO2__1BLOCK_4TO2__GFX10 = 0x00000136,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__2BLOCKS_2TO2__1BLOCK_4TO3__GFX10 = 0x00000137,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_2TO1__1BLOCK_4TO1__1BLOCK_2TO1__GFX10 = 0x00000138,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_2TO1__1BLOCK_4TO2__1BLOCK_2TO1__GFX10 = 0x00000139,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_2TO1__1BLOCK_4TO3__1BLOCK_2TO1__GFX10 = 0x0000013a,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_2TO1__1BLOCK_4TO4__1BLOCK_2TO1__GFX10 = 0x0000013b,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_2TO2__1BLOCK_4TO1__1BLOCK_2TO1__GFX10 = 0x0000013c,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_2TO2__1BLOCK_4TO2__1BLOCK_2TO1__GFX10 = 0x0000013d,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_2TO2__1BLOCK_4TO3__1BLOCK_2TO1__GFX10 = 0x0000013e,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_2TO2__1BLOCK_4TO4__1BLOCK_2TO1__GFX10 = 0x0000013f,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_2TO1__1BLOCK_4TO1__1BLOCK_2TO2__GFX10 = 0x00000140,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_2TO1__1BLOCK_4TO2__1BLOCK_2TO2__GFX10 = 0x00000141,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_2TO1__1BLOCK_4TO3__1BLOCK_2TO2__GFX10 = 0x00000142,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_2TO1__1BLOCK_4TO4__1BLOCK_2TO2__GFX10 = 0x00000143,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_2TO2__1BLOCK_4TO1__1BLOCK_2TO2__GFX10 = 0x00000144,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_2TO2__1BLOCK_4TO2__1BLOCK_2TO2__GFX10 = 0x00000145,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_2TO2__1BLOCK_4TO3__1BLOCK_2TO2__GFX10 = 0x00000146,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_4TO1__2BLOCKS_2TO1__GFX10 = 0x00000147,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_4TO2__2BLOCKS_2TO1__GFX10 = 0x00000148,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_4TO3__2BLOCKS_2TO1__GFX10 = 0x00000149,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_4TO4__2BLOCKS_2TO1__GFX10 = 0x0000014a,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_4TO1__2BLOCKS_2TO2__GFX10 = 0x0000014b,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_4TO2__2BLOCKS_2TO2__GFX10 = 0x0000014c,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_4TO3__2BLOCKS_2TO2__GFX10 = 0x0000014d,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_4TO1__1BLOCK_2TO1__1BLOCK_2TO2__GFX10 = 0x0000014e,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_4TO2__1BLOCK_2TO1__1BLOCK_2TO2__GFX10 = 0x0000014f,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_4TO3__1BLOCK_2TO1__1BLOCK_2TO2__GFX10 = 0x00000150,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_4TO4__1BLOCK_2TO1__1BLOCK_2TO2__GFX10 = 0x00000151,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_4TO1__1BLOCK_2TO2__1BLOCK_2TO1__GFX10 = 0x00000152,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_4TO2__1BLOCK_2TO2__1BLOCK_2TO1__GFX10 = 0x00000153,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_4TO3__1BLOCK_2TO2__1BLOCK_2TO1__GFX10 = 0x00000154,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_4TO4__1BLOCK_2TO2__1BLOCK_2TO1__GFX10 = 0x00000155,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_2TO1__1BLOCK_6TO1__GFX10 = 0x00000156,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_2TO1__1BLOCK_6TO2__GFX10 = 0x00000157,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_2TO1__1BLOCK_6TO3__GFX10 = 0x00000158,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_2TO1__1BLOCK_6TO4__GFX10 = 0x00000159,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_2TO1__1BLOCK_6TO5__GFX10 = 0x0000015a,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_2TO1__1BLOCK_6TO6__GFX10 = 0x0000015b,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_2TO1__INV0__GFX10 = 0x0000015c,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_2TO1__INV1__GFX10 = 0x0000015d,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_2TO2__1BLOCK_6TO1__GFX10 = 0x0000015e,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_2TO2__1BLOCK_6TO2__GFX10 = 0x0000015f,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_2TO2__1BLOCK_6TO3__GFX10 = 0x00000160,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_2TO2__1BLOCK_6TO4__GFX10 = 0x00000161,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_2TO2__1BLOCK_6TO5__GFX10 = 0x00000162,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_2TO2__INV0__GFX10 = 0x00000163,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_2TO2__INV1__GFX10 = 0x00000164,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_6TO1__1BLOCK_2TO1__GFX10 = 0x00000165,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_6TO2__1BLOCK_2TO1__GFX10 = 0x00000166,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_6TO3__1BLOCK_2TO1__GFX10 = 0x00000167,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_6TO4__1BLOCK_2TO1__GFX10 = 0x00000168,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_6TO5__1BLOCK_2TO1__GFX10 = 0x00000169,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_6TO6__1BLOCK_2TO1__GFX10 = 0x0000016a,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__INV0__1BLOCK_2TO1__GFX10 = 0x0000016b,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__INV1__1BLOCK_2TO1__GFX10 = 0x0000016c,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_6TO1__1BLOCK_2TO2__GFX10 = 0x0000016d,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_6TO2__1BLOCK_2TO2__GFX10 = 0x0000016e,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_6TO3__1BLOCK_2TO2__GFX10 = 0x0000016f,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_6TO4__1BLOCK_2TO2__GFX10 = 0x00000170,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_6TO5__1BLOCK_2TO2__GFX10 = 0x00000171,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__INV0__1BLOCK_2TO2__GFX10 = 0x00000172,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__INV1__1BLOCK_2TO2__GFX10 = 0x00000173,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_8TO1__GFX10   = 0x00000174,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_8TO2__GFX10   = 0x00000175,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_8TO3__GFX10   = 0x00000176,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_8TO4__GFX10   = 0x00000177,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_8TO5__GFX10   = 0x00000178,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_8TO6__GFX10   = 0x00000179,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__1BLOCK_8TO7__GFX10   = 0x0000017a,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__UNCOMPRESSED__GFX10  = 0x0000017b,
    CB_PERF_SEL_CC_DCC_COMPRESS_RATIO_2TO1__GFX10      = 0x0000017c,
    CB_PERF_SEL_CC_DCC_COMPRESS_RATIO_4TO1__GFX10      = 0x0000017d,
    CB_PERF_SEL_CC_DCC_COMPRESS_RATIO_4TO2__GFX10      = 0x0000017e,
    CB_PERF_SEL_CC_DCC_COMPRESS_RATIO_4TO3__GFX10      = 0x0000017f,
    CB_PERF_SEL_CC_DCC_COMPRESS_RATIO_6TO1__GFX10      = 0x00000180,
    CB_PERF_SEL_CC_DCC_COMPRESS_RATIO_6TO2__GFX10      = 0x00000181,
    CB_PERF_SEL_CC_DCC_COMPRESS_RATIO_6TO3__GFX10      = 0x00000182,
    CB_PERF_SEL_CC_DCC_COMPRESS_RATIO_6TO4__GFX10      = 0x00000183,
    CB_PERF_SEL_CC_DCC_COMPRESS_RATIO_6TO5__GFX10      = 0x00000184,
    CB_PERF_SEL_CC_DCC_COMPRESS_RATIO_8TO1__GFX10      = 0x00000185,
    CB_PERF_SEL_CC_DCC_COMPRESS_RATIO_8TO2__GFX10      = 0x00000186,
    CB_PERF_SEL_CC_DCC_COMPRESS_RATIO_8TO3__GFX10      = 0x00000187,
    CB_PERF_SEL_CC_DCC_COMPRESS_RATIO_8TO4__GFX10      = 0x00000188,
    CB_PERF_SEL_CC_DCC_COMPRESS_RATIO_8TO5__GFX10      = 0x00000189,
    CB_PERF_SEL_CC_DCC_COMPRESS_RATIO_8TO6__GFX10      = 0x0000018a,
    CB_PERF_SEL_CC_DCC_COMPRESS_RATIO_8TO7__GFX10      = 0x0000018b,
    CB_PERF_SEL_RBP_EXPORT_8PIX_LIT_BOTH__GFX10        = 0x0000018c,
    CB_PERF_SEL_RBP_EXPORT_8PIX_LIT_LEFT__GFX10        = 0x0000018d,
    CB_PERF_SEL_RBP_EXPORT_8PIX_LIT_RIGHT__GFX10       = 0x0000018e,
    CB_PERF_SEL_RBP_SPLIT_MICROTILE__GFX10             = 0x0000018f,
    CB_PERF_SEL_RBP_SPLIT_AA_SAMPLE_MASK__GFX10        = 0x00000190,
    CB_PERF_SEL_RBP_SPLIT_PARTIAL_TARGET_MASK__GFX10   = 0x00000191,
    CB_PERF_SEL_RBP_SPLIT_LINEAR_ADDRESSING__GFX10     = 0x00000192,
    CB_PERF_SEL_RBP_SPLIT_AA_NO_FMASK_COMPRESS__GFX10  = 0x00000193,
    CB_PERF_SEL_RBP_INSERT_MISSING_LAST_QUAD__GFX10    = 0x00000194,
    CB_PERF_SEL_NACK_CM_READ__GFX10                    = 0x00000195,
    CB_PERF_SEL_NACK_CM_WRITE__GFX10                   = 0x00000196,
    CB_PERF_SEL_NACK_FC_READ__GFX10                    = 0x00000197,
    CB_PERF_SEL_NACK_FC_WRITE__GFX10                   = 0x00000198,
    CB_PERF_SEL_NACK_DC_READ__GFX10                    = 0x00000199,
    CB_PERF_SEL_NACK_DC_WRITE__GFX10                   = 0x0000019a,
    CB_PERF_SEL_NACK_CC_READ__GFX10                    = 0x0000019b,
    CB_PERF_SEL_NACK_CC_WRITE__GFX10                   = 0x0000019c,
    CB_PERF_SEL_CM_MC_EARLY_WRITE_RETURN__GFX10        = 0x0000019d,
    CB_PERF_SEL_FC_MC_EARLY_WRITE_RETURN__GFX10        = 0x0000019e,
    CB_PERF_SEL_DC_MC_EARLY_WRITE_RETURN__GFX10        = 0x0000019f,
    CB_PERF_SEL_CC_MC_EARLY_WRITE_RETURN__GFX10        = 0x000001a0,
    CB_PERF_SEL_CM_MC_EARLY_WRITE_REQUESTS_IN_FLIGHT__GFX10 = 0x000001a1,
    CB_PERF_SEL_FC_MC_EARLY_WRITE_REQUESTS_IN_FLIGHT__GFX10 = 0x000001a2,
    CB_PERF_SEL_DC_MC_EARLY_WRITE_REQUESTS_IN_FLIGHT__GFX10 = 0x000001a3,
    CB_PERF_SEL_CC_MC_EARLY_WRITE_REQUESTS_IN_FLIGHT__GFX10 = 0x000001a4,
    CB_PERF_SEL_CM_MC_WRITE_ACK64B__GFX10              = 0x000001a5,
    CB_PERF_SEL_FC_MC_WRITE_ACK64B__GFX10              = 0x000001a6,
    CB_PERF_SEL_DC_MC_WRITE_ACK64B__GFX10              = 0x000001a7,
    CB_PERF_SEL_CC_MC_WRITE_ACK64B__GFX10              = 0x000001a8,
    CB_PERF_SEL_EVENT_BOTTOM_OF_PIPE_TS__GFX10         = 0x000001a9,
    CB_PERF_SEL_EVENT_FLUSH_AND_INV_DB_DATA_TS__GFX10  = 0x000001aa,
    CB_PERF_SEL_EVENT_FLUSH_AND_INV_CB_PIXEL_DATA__GFX10 = 0x000001ab,
    CB_PERF_SEL_DB_CB_TILE_TILENOTEVENT__GFX10         = 0x000001ac,
    CB_PERF_SEL_LQUAD_FORMAT_IS_EXPORT_32BPP_8PIX__GFX10 = 0x000001ad,
    CB_PERF_SEL_LQUAD_FORMAT_IS_EXPORT_16_16_UNSIGNED_8PIX__GFX10 = 0x000001ae,
    CB_PERF_SEL_LQUAD_FORMAT_IS_EXPORT_16_16_SIGNED_8PIX__GFX10 = 0x000001af,
    CB_PERF_SEL_LQUAD_FORMAT_IS_EXPORT_16_16_FLOAT_8PIX__GFX10 = 0x000001b0,
    CB_PERF_SEL_MERGE_PIXELS_WITH_BLEND_ENABLED__GFX10 = 0x000001b1,
    CB_PERF_SEL_DB_CB_CONTEXT_DONE__GFX10              = 0x000001b2,
    CB_PERF_SEL_DB_CB_EOP_DONE__GFX10                  = 0x000001b3,
    CB_PERF_SEL_CC_MC_WRITE_REQUEST_PARTIAL__GFX10     = 0x000001b4,
    CB_PERF_SEL_CC_BB_BLEND_PIXEL_VLD__GFX10           = 0x000001b5,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__CONST_CLEAR_REG__GFX10 = 0x000001b6,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__CONST_CLEAR_SINGLE__GFX10 = 0x000001b7,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__CONST_CLEAR_AC00__GFX10 = 0x000001b8,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__CONST_CLEAR_AC01__GFX10 = 0x000001b9,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__CONST_CLEAR_AC10__GFX10 = 0x000001ba,
    CB_PERF_SEL_CC_DCC_KEY_VALUE__CONST_CLEAR_AC11__GFX10 = 0x000001bb,
    CB_PERF_SEL_TQ_STUTTER_STALL__GFX10                = 0x000001bc,
    CB_PERF_SEL_FC_TILE_STUTTER_STALL__GFX10           = 0x000001bd,
    CB_PERF_SEL_FC_QUAD_STUTTER_STALL__GFX10           = 0x000001be,
    CB_PERF_SEL_FC_KEYID_STUTTER_STALL__GFX10          = 0x000001bf,
    CB_PERF_SEL_CC_EVENFIFO_STUTTER_STALL__GFX10       = 0x000001c0,
    CB_PERF_SEL_CC_ODDFIFO_STUTTER_STALL__GFX10        = 0x000001c1,
    CB_PERF_SEL_CC_CACHE_256BS_SAVED_DUE_TO_QSB__GFX10 = 0x000001c2,
    CB_PERF_SEL_FC_CACHE_FMASK_NO_FETCH__GFX10         = 0x000001c3,
    CB_PERF_SEL_CC_CACHE_SECTOR_HIT__GFX10             = 0x000001c4,
    CB_PERF_SEL_DRAWN_BUSY__GFX11                      = 0x00000002,
    CB_PERF_SEL_DRAWN_PIXEL__GFX11                     = 0x00000003,
    CB_PERF_SEL_DRAWN_QUAD__GFX11                      = 0x00000004,
    CB_PERF_SEL_DRAWN_QUAD_FRAGMENT__GFX11             = 0x00000005,
    CB_PERF_SEL_CC_DCC_DECOMPRESS_TIDS_IN__GFX11       = 0x0000000b,
    CB_PERF_SEL_CC_DCC_DECOMPRESS_TIDS_OUT__GFX11      = 0x0000000c,
    CB_PERF_SEL_CC_DCC_COMPRESS_TIDS_IN__GFX11         = 0x0000000d,
    CB_PERF_SEL_CC_DCC_COMPRESS_TIDS_OUT__GFX11        = 0x0000000e,
    CB_PERF_SEL_DB_CB_EXPORT_VALID_READY__GFX11        = 0x0000000f,
    CB_PERF_SEL_DB_CB_EXPORT_VALID_READYB__GFX11       = 0x00000010,
    CB_PERF_SEL_DB_CB_EXPORT_VALIDB_READY__GFX11       = 0x00000011,
    CB_PERF_SEL_DB_CB_EXPORT_VALIDB_READYB__GFX11      = 0x00000012,
    CB_PERF_SEL_CC_MA_WRITE_REQUEST__GFX11             = 0x00000013,
    CB_PERF_SEL_CC_MA_WRITE_REQUESTS_IN_FLIGHT__GFX11  = 0x00000014,
    CB_PERF_SEL_CC_MA_READ_REQUEST__GFX11              = 0x00000015,
    CB_PERF_SEL_CC_MA_READ_REQUESTS_IN_FLIGHT__GFX11   = 0x00000016,
    CB_PERF_SEL_FDCC_FMASK_DECOMPRESS_BYTES_OUT__GFX11 = 0x00000017,
    CB_PERF_SEL_FDCC_FMASK_COMPRESS_BYTES_IN__GFX11    = 0x00000018,
    CB_PERF_SEL_CC_FDCC_COMPRESS_FRAG_TIDS_IN__GFX11   = 0x00000019,
    CB_PERF_SEL_CC_FDCC_DECOMPRESS_FRAG_TIDS_OUT__GFX11 = 0x0000001a,
    CB_PERF_SEL_DC_MA_WRITE_REQUEST__GFX11             = 0x0000001e,
    CB_PERF_SEL_DC_MA_WRITE_REQUESTS_IN_FLIGHT__GFX11  = 0x0000001f,
    CB_PERF_SEL_DC_MA_READ_REQUEST__GFX11              = 0x00000020,
    CB_PERF_SEL_DC_MA_READ_REQUESTS_IN_FLIGHT__GFX11   = 0x00000021,
    CB_PERF_SEL_CB_RMI_WRREQ_VALID_READY__GFX11        = 0x00000022,
    CB_PERF_SEL_CB_RMI_WRREQ_VALID_READYB__GFX11       = 0x00000023,
    CB_PERF_SEL_CB_RMI_WRREQ_VALIDB_READY__GFX11       = 0x00000024,
    CB_PERF_SEL_CB_RMI_WRREQ_VALIDB_READYB__GFX11      = 0x00000025,
    CB_PERF_SEL_CB_RMI_RDREQ_VALID_READY__GFX11        = 0x00000026,
    CB_PERF_SEL_CB_RMI_RDREQ_VALID_READYB__GFX11       = 0x00000027,
    CB_PERF_SEL_CB_RMI_RDREQ_VALIDB_READY__GFX11       = 0x00000028,
    CB_PERF_SEL_CB_RMI_RDREQ_VALIDB_READYB__GFX11      = 0x00000029,
    CB_PERF_SEL_NACK_CC_READ__GFX11                    = 0x0000002a,
    CB_PERF_SEL_NACK_CC_WRITE__GFX11                   = 0x0000002b,
    CB_PERF_SEL_DC_MA_EARLY_WRITE_RETURN__GFX11        = 0x0000002c,
    CB_PERF_SEL_CC_MA_EARLY_WRITE_RETURN__GFX11        = 0x0000002d,
    CB_PERF_SEL_DC_MA_EARLY_WRITE_REQUESTS_IN_FLIGHT__GFX11 = 0x0000002e,
    CB_PERF_SEL_CC_MA_EARLY_WRITE_REQUESTS_IN_FLIGHT__GFX11 = 0x0000002f,
    CB_PERF_SEL_CC_DCC_RDREQ_STALL__GFX11              = 0x00000030,
    CB_PERF_SEL_EVENT__GFX11                           = 0x00000032,
    CB_PERF_SEL_EVENT_CACHE_FLUSH_TS__GFX11            = 0x00000033,
    CB_PERF_SEL_EVENT_CONTEXT_DONE__GFX11              = 0x00000034,
    CB_PERF_SEL_EVENT_CACHE_FLUSH__GFX11               = 0x00000035,
    CB_PERF_SEL_EVENT_CACHE_FLUSH_AND_INV_TS_EVENT__GFX11 = 0x00000036,
    CB_PERF_SEL_EVENT_CACHE_FLUSH_AND_INV_EVENT__GFX11 = 0x00000037,
    CB_PERF_SEL_EVENT_FLUSH_AND_INV_CB_DATA_TS__GFX11  = 0x00000038,
    CB_PERF_SEL_EVENT_FLUSH_AND_INV_CB_META__GFX11     = 0x00000039,
    CB_PERF_SEL_EVENT_BOTTOM_OF_PIPE_TS__GFX11         = 0x0000003a,
    CB_PERF_SEL_STATIC_CLOCK_EN__GFX11                 = 0x0000003c,
    CB_PERF_SEL_PERFMON_CLOCK_EN__GFX11                = 0x0000003d,
    CB_PERF_SEL_BLEND_CLOCK_EN__GFX11                  = 0x0000003e,
    CB_PERF_SEL_COLOR_STORE_CLOCK_EN__GFX11            = 0x0000003f,
    CB_PERF_SEL_BACKEND_READ_CLOCK_EN__GFX11           = 0x00000040,
    CB_PERF_SEL_GRBM_CLOCK_EN__GFX11                   = 0x00000041,
    CB_PERF_SEL_MEMARB_CLOCK_EN__GFX11                 = 0x00000042,
    CB_PERF_SEL_BACKEND_EVICT_PIPE_CLOCK_EN__GFX11     = 0x00000043,
    CB_PERF_SEL_BACKEND_FRAGOP_CLOCK_EN__GFX11         = 0x00000044,
    CB_PERF_SEL_BACKEND_SRC_FIFO_CLOCK_EN__GFX11       = 0x00000045,
    CB_PERF_SEL_BACKEND_CACHE_CTL_CLOCK_EN__GFX11      = 0x00000046,
    CB_PERF_SEL_FRONTEND_INPUT_CLOCK_EN__GFX11         = 0x00000047,
    CB_PERF_SEL_FRONTEND_ADDR_CLOCK_EN__GFX11          = 0x00000048,
    CB_PERF_SEL_FRONTEND_FDCC_CLOCK_EN__GFX11          = 0x00000049,
    CB_PERF_SEL_FRONTEND_SAMPLE_MASK_TRACKER_CLOCK_EN__GFX11 = 0x0000004a,
    CB_PERF_SEL_EVENTS_CLK_EN__GFX11                   = 0x0000004b,
    CB_PERF_SEL_CC_TAG_HIT__GFX11                      = 0x00000050,
    CB_PERF_SEL_CC_CACHE_TAG_MISS__GFX11               = 0x00000051,
    CB_PERF_SEL_CC_CACHE_SECTOR_MISS__GFX11            = 0x00000052,
    CB_PERF_SEL_CC_CACHE_SECTOR_HIT__GFX11             = 0x00000053,
    CB_PERF_SEL_CC_CACHE_READ_OUTPUT_STALL__GFX11      = 0x00000058,
    CB_PERF_SEL_CC_CACHE_WRITE_OUTPUT_STALL__GFX11     = 0x00000059,
    CB_PERF_SEL_CC_CACHE_ACK_OUTPUT_STALL__GFX11       = 0x0000005a,
    CB_PERF_SEL_CC_CACHE_STALL__GFX11                  = 0x0000005b,
    CB_PERF_SEL_CC_CACHE_FLUSH__GFX11                  = 0x0000005c,
    CB_PERF_SEL_CC_CACHE_SECTORS_FLUSHED__GFX11        = 0x0000005d,
    CB_PERF_SEL_CC_CACHE_WA_TO_RMW_CONVERSION__GFX11   = 0x0000005e,
    CB_PERF_SEL_CC_CACHE_QBLOCKS_FLUSHED__GFX11        = 0x0000005f,
    CB_PERF_SEL_CC_CACHE_DIRTY_QBLOCKS_FLUSHED__GFX11  = 0x00000060,
    CB_PERF_SEL_CC_CACHE_READS_SAVED_DUE_TO_DCC__GFX11 = 0x00000061,
    CB_PERF_SEL_CCC_IN_EVICT_HAZARD_STALL__GFX11       = 0x00000062,
    CB_PERF_SEL_CCC_COLOR_RESOURCE_PANIC__GFX11        = 0x00000063,
    CB_PERF_SEL_CCC_FMASK_RESOURCE_PANIC__GFX11        = 0x00000064,
    CB_PERF_SEL_CCC_FREE_WAYS_PANIC__GFX11             = 0x00000065,
    CB_PERF_SEL_CCC_SKID_FIFO_FULL__GFX11              = 0x00000066,
    CB_PERF_SEL_CCC_SKID_FIFO_STALL__GFX11             = 0x00000067,
    CB_PERF_SEL_CCC_COLOR_RESOURCE_STALL__GFX11        = 0x00000068,
    CB_PERF_SEL_CCC_FMASK_RESOURCE_STALL__GFX11        = 0x00000069,
    CB_PERF_SEL_CCC_FREE_WAYS_STALL__GFX11             = 0x0000006a,
    CB_PERF_SEL_CCC_KEY_XFR_RETURN_STALL__GFX11        = 0x0000006b,
    CB_PERF_SEL_BE_SRCFIFO_FULL__GFX11                 = 0x0000006e,
    CB_PERF_SEL_BE_RDLATFIFO_FULL__GFX11               = 0x0000006f,
    CB_PERF_SEL_RDLAT_FIFO_QUAD_RESIDENCY_STALL__GFX11 = 0x00000070,
    CB_PERF_SEL_CC_QUADFRAG_VALID_READY__GFX11         = 0x00000071,
    CB_PERF_SEL_CC_QUADFRAG_VALID_READYB__GFX11        = 0x00000072,
    CB_PERF_SEL_CC_QUADFRAG_VALIDB_READY__GFX11        = 0x00000073,
    CB_PERF_SEL_CC_QUADFRAG_VALIDB_READYB__GFX11       = 0x00000074,
    CB_PERF_SEL_CC_MA_WRITE_REQUEST_PARTIAL__GFX11     = 0x00000075,
    CB_PERF_SEL_CC_BB_BLEND_PIXEL_VALID_READY__GFX11   = 0x00000076,
    CB_PERF_SEL_CC_BB_BLEND_PIXEL_VALID_READYB__GFX11  = 0x00000077,
    CB_PERF_SEL_CC_BB_BLEND_PIXEL_VALIDB_READY__GFX11  = 0x00000078,
    CB_PERF_SEL_CC_BB_BLEND_PIXEL_VALIDB_READYB__GFX11 = 0x00000079,
    CB_PERF_SEL_READ_REQ_PIPE3_STALL__GFX11            = 0x00000080,
    CB_PERF_SEL_DCC_CACHE_TAG_HIT__GFX11               = 0x00000082,
    CB_PERF_SEL_DCC_CACHE_TAG_MISS__GFX11              = 0x00000083,
    CB_PERF_SEL_DCC_CACHE_SECTOR_MISS__GFX11           = 0x00000084,
    CB_PERF_SEL_DCC_CACHE_REEVICTION_STALL__GFX11      = 0x00000085,
    CB_PERF_SEL_DCC_CACHE_EVICT_NONZERO_INFLIGHT_STALL__GFX11 = 0x00000086,
    CB_PERF_SEL_DCC_CACHE_REPLACE_PENDING_EVICT_STALL__GFX11 = 0x00000087,
    CB_PERF_SEL_DCC_CACHE_INFLIGHT_COUNTER_MAXIMUM_STALL__GFX11 = 0x00000088,
    CB_PERF_SEL_DCC_CACHE_READ_OUTPUT_STALL__GFX11     = 0x00000089,
    CB_PERF_SEL_DCC_CACHE_WRITE_OUTPUT_STALL__GFX11    = 0x0000008a,
    CB_PERF_SEL_DCC_CACHE_ACK_OUTPUT_STALL__GFX11      = 0x0000008b,
    CB_PERF_SEL_DCC_CACHE_STALL__GFX11                 = 0x0000008c,
    CB_PERF_SEL_DCC_CACHE_FLUSH__GFX11                 = 0x0000008d,
    CB_PERF_SEL_DCC_CACHE_SECTORS_FLUSHED__GFX11       = 0x0000008e,
    CB_PERF_SEL_DCC_CACHE_DIRTY_SECTORS_FLUSHED__GFX11 = 0x0000008f,
    CB_PERF_SEL_DCC_CACHE_TAGS_FLUSHED__GFX11          = 0x00000090,
    CB_PERF_SEL_DCC_COMP_SECTORS_IN__GFX11             = 0x00000091,
    CB_PERF_SEL_KEYID_RDLAT_FIFO_FULL__GFX11           = 0x00000095,
    CB_PERF_SEL_RBP_EXPORT_8PIX_LIT_BOTH__GFX11        = 0x00000096,
    CB_PERF_SEL_RBP_EXPORT_8PIX_LIT_LEFT__GFX11        = 0x00000097,
    CB_PERF_SEL_RBP_EXPORT_8PIX_LIT_RIGHT__GFX11       = 0x00000098,
    CB_PERF_SEL_DCC_KEY_VALUE_CLEAR__GFX11             = 0x0000009b,
    CB_PERF_SEL_SMT_OUT_XFR_FEA__GFX11                 = 0x000000a0,
    CB_PERF_SEL_SMT_FULLY_ON_PROBE__GFX11              = 0x000000a1,
    CB_PERF_SEL_SMT_PARTIAL_ON_PROBE__GFX11            = 0x000000a2,
    CB_PERF_SEL_SMT_OUT_XFR__GFX11                     = 0x000000a3,
    CB_PERF_SEL_SMT_OUT_GP_XFR__GFX11                  = 0x000000a4,
    CB_PERF_SEL_SMT_IN_XFR__GFX11                      = 0x000000a5,
    CB_PERF_SEL_SMT_CACHE_PROBE__GFX11                 = 0x000000a6,
    CB_PERF_SEL_SMT_MISS_ON_PROBE__GFX11               = 0x000000a7,
    CB_PERF_SEL_SMT_HIT_ON_PROBE__GFX11                = 0x000000a8,
    CB_PERF_SEL_SMT_IS_STALLED__GFX11                  = 0x000000a9,
    CB_PERF_SEL_SMT_QUAD_PTR_FIFO_IS_FULL__GFX11       = 0x000000aa,
    CB_PERF_SEL_SMT_TOTAL_OVERWRITTEN_SECTORS__GFX11   = 0x000000ab,
    CB_PERF_SEL_BLEND_QUAD_DST_READ_COULD_HAVE_BEEN_OPTIMIZED__GFX11 = 0x000000b4,
    CB_PERF_SEL_BLEND_QUAD_BLENDING_COULD_HAVE_BEEN_BYPASSED__GFX11 = 0x000000b5,
    CB_PERF_SEL_BLEND_QUAD_COULD_HAVE_BEEN_DISCARDED__GFX11 = 0x000000b6,
    CB_PERF_SEL_BLEND_OPT_PIXELS_RESULT_EQ_DEST__GFX11 = 0x000000b7,
    CB_PERF_SEL_BLEND_STALL_AT_OUTPUT__GFX11           = 0x000000b8,
    CB_PERF_SEL_BLEND_STALL_ON_CACHE_ACCESS__GFX11     = 0x000000b9,
    CB_PERF_SEL_BLEND_COLLISION_DUE_TO_CACHE_WRITE__GFX11 = 0x000000ba,
    CB_PERF_SEL_BLEND_RAW_HAZARD_STALL__GFX11          = 0x000000bb,
    CB_PERF_SEL_BE_CS_FILLRATE_1X2__GFX11              = 0x000000be,
    CB_PERF_SEL_BE_CS_FILLRATE_2X1__GFX11              = 0x000000bf,
    CB_PERF_SEL_BE_CS_FILLRATE_2X2__GFX11              = 0x000000c0,
    CB_PERF_SEL_CC_DCC_COMPRESS_RATIO_2TO1__GFX11      = 0x000000c8,
    CB_PERF_SEL_CC_DCC_COMPRESS_RATIO_2TO2__GFX11      = 0x000000c9,
    CB_PERF_SEL_CC_DCC_COMPRESS_RATIO_4TO1__GFX11      = 0x000000ca,
    CB_PERF_SEL_CC_DCC_COMPRESS_RATIO_4TO2__GFX11      = 0x000000cb,
    CB_PERF_SEL_CC_DCC_COMPRESS_RATIO_4TO3__GFX11      = 0x000000cc,
    CB_PERF_SEL_CC_DCC_COMPRESS_RATIO_4TO4__GFX11      = 0x000000cd,
    CB_PERF_SEL_CC_DCC_COMPRESS_RATIO_6TO1__GFX11      = 0x000000ce,
    CB_PERF_SEL_CC_DCC_COMPRESS_RATIO_6TO2__GFX11      = 0x000000cf,
    CB_PERF_SEL_CC_DCC_COMPRESS_RATIO_6TO3__GFX11      = 0x000000d0,
    CB_PERF_SEL_CC_DCC_COMPRESS_RATIO_6TO4__GFX11      = 0x000000d1,
    CB_PERF_SEL_CC_DCC_COMPRESS_RATIO_6TO5__GFX11      = 0x000000d2,
    CB_PERF_SEL_CC_DCC_COMPRESS_RATIO_6TO6__GFX11      = 0x000000d3,
    CB_PERF_SEL_CC_DCC_COMPRESS_RATIO_8TO1__GFX11      = 0x000000d4,
    CB_PERF_SEL_CC_DCC_COMPRESS_RATIO_8TO2__GFX11      = 0x000000d5,
    CB_PERF_SEL_CC_DCC_COMPRESS_RATIO_8TO3__GFX11      = 0x000000d6,
    CB_PERF_SEL_CC_DCC_COMPRESS_RATIO_8TO4__GFX11      = 0x000000d7,
    CB_PERF_SEL_CC_DCC_COMPRESS_RATIO_8TO5__GFX11      = 0x000000d8,
    CB_PERF_SEL_CC_DCC_COMPRESS_RATIO_8TO6__GFX11      = 0x000000d9,
    CB_PERF_SEL_CC_DCC_COMPRESS_RATIO_8TO7__GFX11      = 0x000000da,
    CB_PERF_SEL_CC_DCC_COMPRESS_RATIO_8TO8__GFX11      = 0x000000db,
    CB_PERF_SEL_CC_FDCC_COMPRESS_RATIO_4TO1_1MAXFRAGS__GFX11 = 0x000000dc,
    CB_PERF_SEL_CC_FDCC_COMPRESS_RATIO_4TO2_1MAXFRAGS__GFX11 = 0x000000dd,
    CB_PERF_SEL_CC_FDCC_COMPRESS_RATIO_4TO3_1MAXFRAGS__GFX11 = 0x000000de,
    CB_PERF_SEL_CC_FDCC_COMPRESS_RATIO_4TO4_1MAXFRAGS__GFX11 = 0x000000df,
    CB_PERF_SEL_CC_FDCC_COMPRESS_RATIO_4TO1_2MAXFRAGS__GFX11 = 0x000000e0,
    CB_PERF_SEL_CC_FDCC_COMPRESS_RATIO_4TO2_2MAXFRAGS__GFX11 = 0x000000e1,
    CB_PERF_SEL_CC_FDCC_COMPRESS_RATIO_4TO3_2MAXFRAGS__GFX11 = 0x000000e2,
    CB_PERF_SEL_CC_FDCC_COMPRESS_RATIO_4TO4_2MAXFRAGS__GFX11 = 0x000000e3,
    CB_PERF_SEL_CC_FDCC_COMPRESS_RATIO_4TO1_3MAXFRAGS__GFX11 = 0x000000e4,
    CB_PERF_SEL_CC_FDCC_COMPRESS_RATIO_4TO2_3MAXFRAGS__GFX11 = 0x000000e5,
    CB_PERF_SEL_CC_FDCC_COMPRESS_RATIO_4TO3_3MAXFRAGS__GFX11 = 0x000000e6,
    CB_PERF_SEL_CC_FDCC_COMPRESS_RATIO_4TO4_3MAXFRAGS__GFX11 = 0x000000e7,
    CB_PERF_SEL_CC_FDCC_COMPRESS_RATIO_4TO1_4MAXFRAGS__GFX11 = 0x000000e8,
    CB_PERF_SEL_CC_FDCC_COMPRESS_RATIO_4TO2_4MAXFRAGS__GFX11 = 0x000000e9,
    CB_PERF_SEL_CC_FDCC_COMPRESS_RATIO_4TO3_4MAXFRAGS__GFX11 = 0x000000ea,
    CB_PERF_SEL_CC_FDCC_COMPRESS_RATIO_4TO4_4MAXFRAGS__GFX11 = 0x000000eb,
    CB_PERF_SEL_CC_FDCC_COMPRESS_1X1__GFX11            = 0x000000ec,
    CB_PERF_SEL_CC_FDCC_COMPRESS_1X2__GFX11            = 0x000000ed,
    CB_PERF_SEL_CC_FDCC_COMPRESS_2X1__GFX11            = 0x000000ee,
    CB_PERF_SEL_CC_FDCC_COMPRESS_2X2__GFX11            = 0x000000ef,
    CB_PERF_SEL_FORMAT_IS_32_R__GFX11                  = 0x000000fa,
    CB_PERF_SEL_FORMAT_IS_32_AR__GFX11                 = 0x000000fb,
    CB_PERF_SEL_FORMAT_IS_32_GR__GFX11                 = 0x000000fc,
    CB_PERF_SEL_FORMAT_IS_32_ABGR__GFX11               = 0x000000fd,
    CB_PERF_SEL_FORMAT_IS_FP16_ABGR__GFX11             = 0x000000fe,
    CB_PERF_SEL_FORMAT_IS_SIGNED16_ABGR__GFX11         = 0x000000ff,
    CB_PERF_SEL_FORMAT_IS_UNSIGNED16_ABGR__GFX11       = 0x00000100,
    CB_PERF_SEL_FORMAT_IS_32BPP_8PIX__GFX11            = 0x00000101,
    CB_PERF_SEL_FORMAT_IS_16_16_UNSIGNED_8PIX__GFX11   = 0x00000102,
    CB_PERF_SEL_FORMAT_IS_16_16_SIGNED_8PIX__GFX11     = 0x00000103,
    CB_PERF_SEL_FORMAT_IS_16_16_FLOAT_8PIX__GFX11      = 0x00000104,
    CB_PERF_SEL_EXPORT_ADDED_1_FRAGMENT__GFX11         = 0x00000105,
    CB_PERF_SEL_EXPORT_ADDED_2_FRAGMENTS__GFX11        = 0x00000106,
    CB_PERF_SEL_EXPORT_ADDED_3_FRAGMENTS__GFX11        = 0x00000107,
    CB_PERF_SEL_EXPORT_ADDED_4_FRAGMENTS__GFX11        = 0x00000108,
    CB_PERF_SEL_EXPORT_ADDED_5_FRAGMENTS__GFX11        = 0x00000109,
    CB_PERF_SEL_EXPORT_ADDED_6_FRAGMENTS__GFX11        = 0x0000010a,
    CB_PERF_SEL_EXPORT_ADDED_7_FRAGMENTS__GFX11        = 0x0000010b,
    CB_PERF_SEL_EXPORT_BLEND_OPT_DONT_READ_DST__GFX11  = 0x0000010c,
    CB_PERF_SEL_EXPORT_BLEND_OPT_BLEND_BYPASS__GFX11   = 0x0000010d,
    CB_PERF_SEL_EXPORT_BLEND_OPT_DISCARD_PIXELS__GFX11 = 0x0000010e,
    CB_PERF_SEL_EXPORT_HAS_1_FRAGMENT_BEFORE_UPDATE__GFX11 = 0x0000010f,
    CB_PERF_SEL_EXPORT_HAS_1_FRAGMENT_AFTER_UPDATE__GFX11 = 0x00000110,
    CB_PERF_SEL_EXPORT_HAS_2_FRAGMENTS_BEFORE_UPDATE__GFX11 = 0x00000111,
    CB_PERF_SEL_EXPORT_HAS_2_FRAGMENTS_AFTER_UPDATE__GFX11 = 0x00000112,
    CB_PERF_SEL_EXPORT_HAS_3_FRAGMENTS_BEFORE_UPDATE__GFX11 = 0x00000113,
    CB_PERF_SEL_EXPORT_HAS_3_FRAGMENTS_AFTER_UPDATE__GFX11 = 0x00000114,
    CB_PERF_SEL_EXPORT_HAS_4_FRAGMENTS_BEFORE_UPDATE__GFX11 = 0x00000115,
    CB_PERF_SEL_EXPORT_HAS_4_FRAGMENTS_AFTER_UPDATE__GFX11 = 0x00000116,
    CB_PERF_SEL_EXPORT_HAS_5_FRAGMENTS_BEFORE_UPDATE__GFX11 = 0x00000117,
    CB_PERF_SEL_EXPORT_HAS_5_FRAGMENTS_AFTER_UPDATE__GFX11 = 0x00000118,
    CB_PERF_SEL_EXPORT_HAS_6_FRAGMENTS_BEFORE_UPDATE__GFX11 = 0x00000119,
    CB_PERF_SEL_EXPORT_HAS_6_FRAGMENTS_AFTER_UPDATE__GFX11 = 0x0000011a,
    CB_PERF_SEL_EXPORT_HAS_7_FRAGMENTS_BEFORE_UPDATE__GFX11 = 0x0000011b,
    CB_PERF_SEL_EXPORT_HAS_7_FRAGMENTS_AFTER_UPDATE__GFX11 = 0x0000011c,
    CB_PERF_SEL_EXPORT_HAS_8_FRAGMENTS_BEFORE_UPDATE__GFX11 = 0x0000011d,
    CB_PERF_SEL_EXPORT_HAS_8_FRAGMENTS_AFTER_UPDATE__GFX11 = 0x0000011e,
    CB_PERF_SEL_EXPORT_READS_FRAGMENT_0__GFX11         = 0x0000011f,
    CB_PERF_SEL_EXPORT_READS_FRAGMENT_1__GFX11         = 0x00000120,
    CB_PERF_SEL_EXPORT_READS_FRAGMENT_2__GFX11         = 0x00000121,
    CB_PERF_SEL_EXPORT_READS_FRAGMENT_3__GFX11         = 0x00000122,
    CB_PERF_SEL_EXPORT_READS_FRAGMENT_4__GFX11         = 0x00000123,
    CB_PERF_SEL_EXPORT_READS_FRAGMENT_5__GFX11         = 0x00000124,
    CB_PERF_SEL_EXPORT_READS_FRAGMENT_6__GFX11         = 0x00000125,
    CB_PERF_SEL_EXPORT_READS_FRAGMENT_7__GFX11         = 0x00000126,
    CB_PERF_SEL_EXPORT_REMOVED_1_FRAGMENT__GFX11       = 0x00000127,
    CB_PERF_SEL_EXPORT_REMOVED_2_FRAGMENTS__GFX11      = 0x00000128,
    CB_PERF_SEL_EXPORT_REMOVED_3_FRAGMENTS__GFX11      = 0x00000129,
    CB_PERF_SEL_EXPORT_REMOVED_4_FRAGMENTS__GFX11      = 0x0000012a,
    CB_PERF_SEL_EXPORT_REMOVED_5_FRAGMENTS__GFX11      = 0x0000012b,
    CB_PERF_SEL_EXPORT_REMOVED_6_FRAGMENTS__GFX11      = 0x0000012c,
    CB_PERF_SEL_EXPORT_REMOVED_7_FRAGMENTS__GFX11      = 0x0000012d,
    CB_PERF_SEL_EXPORT_WRITES_FRAGMENT_0__GFX11        = 0x0000012e,
    CB_PERF_SEL_EXPORT_WRITES_FRAGMENT_1__GFX11        = 0x0000012f,
    CB_PERF_SEL_EXPORT_WRITES_FRAGMENT_2__GFX11        = 0x00000130,
    CB_PERF_SEL_EXPORT_WRITES_FRAGMENT_3__GFX11        = 0x00000131,
    CB_PERF_SEL_EXPORT_WRITES_FRAGMENT_4__GFX11        = 0x00000132,
    CB_PERF_SEL_EXPORT_WRITES_FRAGMENT_5__GFX11        = 0x00000133,
    CB_PERF_SEL_EXPORT_WRITES_FRAGMENT_6__GFX11        = 0x00000134,
    CB_PERF_SEL_EXPORT_WRITES_FRAGMENT_7__GFX11        = 0x00000135,
    CB_PERF_SEL_EXPORT_KILLED_BY_COLOR_INVALID__GFX11  = 0x00000136,
    CB_PERF_SEL_EXPORT_KILLED_BY_DISCARD_PIXEL__GFX11  = 0x00000137,
    CB_PERF_SEL_EXPORT_KILLED_BY_NULL_SAMPLE_MASK__GFX11 = 0x00000138,
    CB_PERF_SEL_EXPORT_KILLED_BY_NULL_TARGET_SHADER_MASK__GFX11 = 0x00000139,
    CB_PERF_SEL_READ_REQ_PIPE1_STALL__GFX110           = 0x00000081,
    CB_PERF_SEL_READ_REQ_PIPE1_STALL__STX1             = 0x00000081,
#if CHIP_HDR_STRIX_HALO
    CB_PERF_SEL_READ_REQ_PIPE1_STALL__STXH             = 0x0000007f,
    CB_PERF_SEL_READ_REQ_PIPE7_STALL__STXH             = 0x00000081,
#endif
} CBPerfSel;

constexpr unsigned int MaxCBPerfSelGfx10               = CB_PERF_SEL_CC_CACHE_SECTOR_HIT__GFX10;
constexpr unsigned int MaxCBPerfSelGfx11               = CB_PERF_SEL_EXPORT_KILLED_BY_NULL_TARGET_SHADER_MASK__GFX11;

typedef enum CBRamList {
    CB_DCG_CCC_CAS_TAG_ARRAY                           = 0x00000000,
    CB_DCG_CCC_CAS_FRAG_PTR                            = 0x00000001,
    CB_DCG_CCC_CAS_COLOR_PTR                           = 0x00000002,
    CB_DCG_CCC_CAS_SURF_PARAM                          = 0x00000003,
    CB_DCG_CCC_CAS_KEYID                               = 0x00000004,
    CB_DCG_BACKEND_RDLAT_FIFO                          = 0x00000005,
    CB_DCG_FRONTEND_RDLAT_FIFO                         = 0x00000006,
    CB_DCG_SRC_FIFO                                    = 0x00000007,
    CB_DCG_COLOR_STORE                                 = 0x00000008,
    CB_DCG_COLOR_STORE_DIRTY_BYTE                      = 0x00000009,
    CB_DCG_FMASK_CACHE_STORE                           = 0x0000000a,
    CB_DCG_READ_SKID_FIFO                              = 0x0000000b,
    CB_DCG_QUAD_PTR_FIFO                               = 0x0000000c,
    CB_DCG_OUTPUT_FIFO                                 = 0x0000000d,
    CB_DCG_DCC_CACHE                                   = 0x0000000e,
    CB_DCG_DCC_DIRTY_BITS                              = 0x0000000f,
    CB_DCG_FOP_MAX_FRAG                                = 0x00000010,
    CB_DCG_QBLOCK_ALLOC                                = 0x00000011,
    CB_DCG_DCC_COMP_RAM                                = 0x00000012,
    CB_DCG_DCC_DECOMP_RAM                              = 0x00000013,
} CBRamList;

typedef enum CHA_PERF_SEL {
    CHA_PERF_SEL_BUSY                                  = 0x00000000,
    CHA_PERF_SEL_STALL_CHC0                            = 0x00000001,
    CHA_PERF_SEL_STALL_CHC1                            = 0x00000002,
    CHA_PERF_SEL_STALL_CHC2                            = 0x00000003,
    CHA_PERF_SEL_STALL_CHC3                            = 0x00000004,
    CHA_PERF_SEL_REQUEST_CHC0__APU103                  = 0x00000005,
    CHA_PERF_SEL_REQUEST_CHC1__APU103                  = 0x00000006,
    CHA_PERF_SEL_REQUEST_CHC2__APU103                  = 0x00000007,
    CHA_PERF_SEL_REQUEST_CHC3__APU103                  = 0x00000008,
    CHA_PERF_SEL_MEM_32B_WDS_CHC0__APU103              = 0x00000009,
    CHA_PERF_SEL_MEM_32B_WDS_CHC1__APU103              = 0x0000000a,
    CHA_PERF_SEL_MEM_32B_WDS_CHC2__APU103              = 0x0000000b,
    CHA_PERF_SEL_MEM_32B_WDS_CHC3__APU103              = 0x0000000c,
    CHA_PERF_SEL_IO_32B_WDS_CHC0__APU103               = 0x0000000d,
    CHA_PERF_SEL_IO_32B_WDS_CHC1__APU103               = 0x0000000e,
    CHA_PERF_SEL_IO_32B_WDS_CHC2__APU103               = 0x0000000f,
    CHA_PERF_SEL_IO_32B_WDS_CHC3__APU103               = 0x00000010,
    CHA_PERF_SEL_MEM_BURST_COUNT_CHC0__APU103          = 0x00000011,
    CHA_PERF_SEL_MEM_BURST_COUNT_CHC1__APU103          = 0x00000012,
    CHA_PERF_SEL_MEM_BURST_COUNT_CHC2__APU103          = 0x00000013,
    CHA_PERF_SEL_MEM_BURST_COUNT_CHC3__APU103          = 0x00000014,
    CHA_PERF_SEL_IO_BURST_COUNT_CHC0__APU103           = 0x00000015,
    CHA_PERF_SEL_IO_BURST_COUNT_CHC1__APU103           = 0x00000016,
    CHA_PERF_SEL_IO_BURST_COUNT_CHC2__APU103           = 0x00000017,
    CHA_PERF_SEL_IO_BURST_COUNT_CHC3__APU103           = 0x00000018,
    CHA_PERF_SEL_ARB_REQUESTS__APU103                  = 0x00000019,
    CHA_PERF_SEL_REQ_INFLIGHT_LEVEL__APU103            = 0x0000001a,
    CHA_PERF_SEL_STALL_RET_CONFLICT_CHC0__APU103       = 0x0000001b,
    CHA_PERF_SEL_STALL_RET_CONFLICT_CHC1__APU103       = 0x0000001c,
    CHA_PERF_SEL_STALL_RET_CONFLICT_CHC2__APU103       = 0x0000001d,
    CHA_PERF_SEL_STALL_RET_CONFLICT_CHC3__APU103       = 0x0000001e,
    CHA_PERF_SEL_CYCLE__APU103                         = 0x0000001f,
    CHA_PERF_SEL_REQUEST_CHC0__APU11                   = 0x00000005,
    CHA_PERF_SEL_REQUEST_CHC1__APU11                   = 0x00000006,
    CHA_PERF_SEL_REQUEST_CHC2__APU11                   = 0x00000007,
    CHA_PERF_SEL_REQUEST_CHC3__APU11                   = 0x00000008,
    CHA_PERF_SEL_MEM_32B_WDS_CHC0__APU11               = 0x00000009,
    CHA_PERF_SEL_MEM_32B_WDS_CHC1__APU11               = 0x0000000a,
    CHA_PERF_SEL_MEM_32B_WDS_CHC2__APU11               = 0x0000000b,
    CHA_PERF_SEL_MEM_32B_WDS_CHC3__APU11               = 0x0000000c,
    CHA_PERF_SEL_IO_32B_WDS_CHC0__APU11                = 0x0000000d,
    CHA_PERF_SEL_IO_32B_WDS_CHC1__APU11                = 0x0000000e,
    CHA_PERF_SEL_IO_32B_WDS_CHC2__APU11                = 0x0000000f,
    CHA_PERF_SEL_IO_32B_WDS_CHC3__APU11                = 0x00000010,
    CHA_PERF_SEL_MEM_BURST_COUNT_CHC0__APU11           = 0x00000011,
    CHA_PERF_SEL_MEM_BURST_COUNT_CHC1__APU11           = 0x00000012,
    CHA_PERF_SEL_MEM_BURST_COUNT_CHC2__APU11           = 0x00000013,
    CHA_PERF_SEL_MEM_BURST_COUNT_CHC3__APU11           = 0x00000014,
    CHA_PERF_SEL_IO_BURST_COUNT_CHC0__APU11            = 0x00000015,
    CHA_PERF_SEL_IO_BURST_COUNT_CHC1__APU11            = 0x00000016,
    CHA_PERF_SEL_IO_BURST_COUNT_CHC2__APU11            = 0x00000017,
    CHA_PERF_SEL_IO_BURST_COUNT_CHC3__APU11            = 0x00000018,
    CHA_PERF_SEL_ARB_REQUESTS__APU11                   = 0x00000019,
    CHA_PERF_SEL_REQ_INFLIGHT_LEVEL__APU11             = 0x0000001a,
    CHA_PERF_SEL_STALL_RET_CONFLICT_CHC0__APU11        = 0x0000001b,
    CHA_PERF_SEL_STALL_RET_CONFLICT_CHC1__APU11        = 0x0000001c,
    CHA_PERF_SEL_STALL_RET_CONFLICT_CHC2__APU11        = 0x0000001d,
    CHA_PERF_SEL_STALL_RET_CONFLICT_CHC3__APU11        = 0x0000001e,
    CHA_PERF_SEL_CYCLE__APU11                          = 0x0000001f,
    CHA_PERF_SEL_STALL_CHC4__GFX101                    = 0x00000005,
    CHA_PERF_SEL_STALL_CHC5__GFX101                    = 0x00000006,
    CHA_PERF_SEL_REQUEST_CHC0__GFX101                  = 0x00000007,
    CHA_PERF_SEL_REQUEST_CHC1__GFX101                  = 0x00000008,
    CHA_PERF_SEL_REQUEST_CHC2__GFX101                  = 0x00000009,
    CHA_PERF_SEL_REQUEST_CHC3__GFX101                  = 0x0000000a,
    CHA_PERF_SEL_REQUEST_CHC4__GFX101                  = 0x0000000b,
    CHA_PERF_SEL_MEM_32B_WDS_CHC0__GFX101              = 0x0000000c,
    CHA_PERF_SEL_MEM_32B_WDS_CHC1__GFX101              = 0x0000000d,
    CHA_PERF_SEL_MEM_32B_WDS_CHC2__GFX101              = 0x0000000e,
    CHA_PERF_SEL_MEM_32B_WDS_CHC3__GFX101              = 0x0000000f,
    CHA_PERF_SEL_MEM_32B_WDS_CHC4__GFX101              = 0x00000010,
    CHA_PERF_SEL_IO_32B_WDS_CHC0__GFX101               = 0x00000011,
    CHA_PERF_SEL_IO_32B_WDS_CHC1__GFX101               = 0x00000012,
    CHA_PERF_SEL_IO_32B_WDS_CHC2__GFX101               = 0x00000013,
    CHA_PERF_SEL_IO_32B_WDS_CHC3__GFX101               = 0x00000014,
    CHA_PERF_SEL_IO_32B_WDS_CHC4__GFX101               = 0x00000015,
    CHA_PERF_SEL_MEM_BURST_COUNT_CHC0__GFX101          = 0x00000016,
    CHA_PERF_SEL_MEM_BURST_COUNT_CHC1__GFX101          = 0x00000017,
    CHA_PERF_SEL_MEM_BURST_COUNT_CHC2__GFX101          = 0x00000018,
    CHA_PERF_SEL_MEM_BURST_COUNT_CHC3__GFX101          = 0x00000019,
    CHA_PERF_SEL_MEM_BURST_COUNT_CHC4__GFX101          = 0x0000001a,
    CHA_PERF_SEL_IO_BURST_COUNT_CHC0__GFX101           = 0x0000001b,
    CHA_PERF_SEL_IO_BURST_COUNT_CHC1__GFX101           = 0x0000001c,
    CHA_PERF_SEL_IO_BURST_COUNT_CHC2__GFX101           = 0x0000001d,
    CHA_PERF_SEL_IO_BURST_COUNT_CHC3__GFX101           = 0x0000001e,
    CHA_PERF_SEL_IO_BURST_COUNT_CHC4__GFX101           = 0x0000001f,
    CHA_PERF_SEL_ARB_REQUESTS__GFX101                  = 0x00000020,
    CHA_PERF_SEL_REQ_ARB_LEVEL_CHC0__GFX101            = 0x00000021,
    CHA_PERF_SEL_REQ_ARB_LEVEL_CHC1__GFX101            = 0x00000022,
    CHA_PERF_SEL_REQ_ARB_LEVEL_CHC2__GFX101            = 0x00000023,
    CHA_PERF_SEL_REQ_ARB_LEVEL_CHC3__GFX101            = 0x00000024,
    CHA_PERF_SEL_REQ_ARB_LEVEL_CHC4__GFX101            = 0x00000025,
    CHA_PERF_SEL_REQ_INFLIGHT_LEVEL__GFX101            = 0x00000026,
    CHA_PERF_SEL_STALL_RET_CONFLICT_CHC0__GFX101       = 0x00000027,
    CHA_PERF_SEL_STALL_RET_CONFLICT_CHC1__GFX101       = 0x00000028,
    CHA_PERF_SEL_STALL_RET_CONFLICT_CHC2__GFX101       = 0x00000029,
    CHA_PERF_SEL_STALL_RET_CONFLICT_CHC3__GFX101       = 0x0000002a,
    CHA_PERF_SEL_STALL_RET_CONFLICT_CHC4__GFX101       = 0x0000002b,
    CHA_PERF_SEL_CYCLE__GFX101                         = 0x0000002c,
    CHA_PERF_SEL_STALL_CHC4__NV2X                      = 0x00000005,
    CHA_PERF_SEL_STALL_CHC5__NV2X                      = 0x00000006,
    CHA_PERF_SEL_REQUEST_CHC0__NV2X                    = 0x00000007,
    CHA_PERF_SEL_REQUEST_CHC1__NV2X                    = 0x00000008,
    CHA_PERF_SEL_REQUEST_CHC2__NV2X                    = 0x00000009,
    CHA_PERF_SEL_REQUEST_CHC3__NV2X                    = 0x0000000a,
    CHA_PERF_SEL_REQUEST_CHC4__NV2X                    = 0x0000000b,
    CHA_PERF_SEL_MEM_32B_WDS_CHC0__NV2X                = 0x0000000c,
    CHA_PERF_SEL_MEM_32B_WDS_CHC1__NV2X                = 0x0000000d,
    CHA_PERF_SEL_MEM_32B_WDS_CHC2__NV2X                = 0x0000000e,
    CHA_PERF_SEL_MEM_32B_WDS_CHC3__NV2X                = 0x0000000f,
    CHA_PERF_SEL_MEM_32B_WDS_CHC4__NV2X                = 0x00000010,
    CHA_PERF_SEL_IO_32B_WDS_CHC0__NV2X                 = 0x00000011,
    CHA_PERF_SEL_IO_32B_WDS_CHC1__NV2X                 = 0x00000012,
    CHA_PERF_SEL_IO_32B_WDS_CHC2__NV2X                 = 0x00000013,
    CHA_PERF_SEL_IO_32B_WDS_CHC3__NV2X                 = 0x00000014,
    CHA_PERF_SEL_IO_32B_WDS_CHC4__NV2X                 = 0x00000015,
    CHA_PERF_SEL_MEM_BURST_COUNT_CHC0__NV2X            = 0x00000016,
    CHA_PERF_SEL_MEM_BURST_COUNT_CHC1__NV2X            = 0x00000017,
    CHA_PERF_SEL_MEM_BURST_COUNT_CHC2__NV2X            = 0x00000018,
    CHA_PERF_SEL_MEM_BURST_COUNT_CHC3__NV2X            = 0x00000019,
    CHA_PERF_SEL_MEM_BURST_COUNT_CHC4__NV2X            = 0x0000001a,
    CHA_PERF_SEL_IO_BURST_COUNT_CHC0__NV2X             = 0x0000001b,
    CHA_PERF_SEL_IO_BURST_COUNT_CHC1__NV2X             = 0x0000001c,
    CHA_PERF_SEL_IO_BURST_COUNT_CHC2__NV2X             = 0x0000001d,
    CHA_PERF_SEL_IO_BURST_COUNT_CHC3__NV2X             = 0x0000001e,
    CHA_PERF_SEL_IO_BURST_COUNT_CHC4__NV2X             = 0x0000001f,
    CHA_PERF_SEL_ARB_REQUESTS__NV2X                    = 0x00000020,
    CHA_PERF_SEL_REQ_INFLIGHT_LEVEL__NV2X              = 0x00000021,
    CHA_PERF_SEL_STALL_RET_CONFLICT_CHC0__NV2X         = 0x00000022,
    CHA_PERF_SEL_STALL_RET_CONFLICT_CHC1__NV2X         = 0x00000023,
    CHA_PERF_SEL_STALL_RET_CONFLICT_CHC2__NV2X         = 0x00000024,
    CHA_PERF_SEL_STALL_RET_CONFLICT_CHC3__NV2X         = 0x00000025,
    CHA_PERF_SEL_STALL_RET_CONFLICT_CHC4__NV2X         = 0x00000026,
    CHA_PERF_SEL_CYCLE__NV2X                           = 0x00000027,
    CHA_PERF_SEL_STALL_CHC4__NV3X                      = 0x00000005,
    CHA_PERF_SEL_STALL_CHC5__NV3X                      = 0x00000006,
    CHA_PERF_SEL_REQUEST_CHC0__NV3X                    = 0x00000007,
    CHA_PERF_SEL_REQUEST_CHC1__NV3X                    = 0x00000008,
    CHA_PERF_SEL_REQUEST_CHC2__NV3X                    = 0x00000009,
    CHA_PERF_SEL_REQUEST_CHC3__NV3X                    = 0x0000000a,
    CHA_PERF_SEL_REQUEST_CHC4__NV3X                    = 0x0000000b,
    CHA_PERF_SEL_MEM_32B_WDS_CHC0__NV3X                = 0x0000000c,
    CHA_PERF_SEL_MEM_32B_WDS_CHC1__NV3X                = 0x0000000d,
    CHA_PERF_SEL_MEM_32B_WDS_CHC2__NV3X                = 0x0000000e,
    CHA_PERF_SEL_MEM_32B_WDS_CHC3__NV3X                = 0x0000000f,
    CHA_PERF_SEL_MEM_32B_WDS_CHC4__NV3X                = 0x00000010,
    CHA_PERF_SEL_IO_32B_WDS_CHC0__NV3X                 = 0x00000011,
    CHA_PERF_SEL_IO_32B_WDS_CHC1__NV3X                 = 0x00000012,
    CHA_PERF_SEL_IO_32B_WDS_CHC2__NV3X                 = 0x00000013,
    CHA_PERF_SEL_IO_32B_WDS_CHC3__NV3X                 = 0x00000014,
    CHA_PERF_SEL_IO_32B_WDS_CHC4__NV3X                 = 0x00000015,
    CHA_PERF_SEL_MEM_BURST_COUNT_CHC0__NV3X            = 0x00000016,
    CHA_PERF_SEL_MEM_BURST_COUNT_CHC1__NV3X            = 0x00000017,
    CHA_PERF_SEL_MEM_BURST_COUNT_CHC2__NV3X            = 0x00000018,
    CHA_PERF_SEL_MEM_BURST_COUNT_CHC3__NV3X            = 0x00000019,
    CHA_PERF_SEL_MEM_BURST_COUNT_CHC4__NV3X            = 0x0000001a,
    CHA_PERF_SEL_IO_BURST_COUNT_CHC0__NV3X             = 0x0000001b,
    CHA_PERF_SEL_IO_BURST_COUNT_CHC1__NV3X             = 0x0000001c,
    CHA_PERF_SEL_IO_BURST_COUNT_CHC2__NV3X             = 0x0000001d,
    CHA_PERF_SEL_IO_BURST_COUNT_CHC3__NV3X             = 0x0000001e,
    CHA_PERF_SEL_IO_BURST_COUNT_CHC4__NV3X             = 0x0000001f,
    CHA_PERF_SEL_ARB_REQUESTS__NV3X                    = 0x00000020,
    CHA_PERF_SEL_REQ_INFLIGHT_LEVEL__NV3X              = 0x00000021,
    CHA_PERF_SEL_STALL_RET_CONFLICT_CHC0__NV3X         = 0x00000022,
    CHA_PERF_SEL_STALL_RET_CONFLICT_CHC1__NV3X         = 0x00000023,
    CHA_PERF_SEL_STALL_RET_CONFLICT_CHC2__NV3X         = 0x00000024,
    CHA_PERF_SEL_STALL_RET_CONFLICT_CHC3__NV3X         = 0x00000025,
    CHA_PERF_SEL_STALL_RET_CONFLICT_CHC4__NV3X         = 0x00000026,
    CHA_PERF_SEL_CYCLE__NV3X                           = 0x00000027,
} CHA_PERF_SEL;

constexpr unsigned int MaxChaPerfSelGfx101             = CHA_PERF_SEL_CYCLE__GFX101;
constexpr unsigned int MaxChaPerfSelNv2x               = CHA_PERF_SEL_CYCLE__NV2X;
constexpr unsigned int MaxChaPerfSelNv3x               = CHA_PERF_SEL_CYCLE__NV3X;
constexpr unsigned int MaxChaPerfSelApu11              = CHA_PERF_SEL_CYCLE__APU11;
constexpr unsigned int MaxChaPerfSelApu103             = CHA_PERF_SEL_CYCLE__APU103;

typedef enum CHCG_PERF_SEL {
    CHCG_PERF_SEL_CYCLE__GFX101                        = 0x00000000,
    CHCG_PERF_SEL_BUSY__GFX101                         = 0x00000001,
    CHCG_PERF_SEL_STARVE__GFX101                       = 0x00000002,
    CHCG_PERF_SEL_ARB_RET_LEVEL__GFX101                = 0x00000003,
    CHCG_PERF_SEL_GL2_REQ_READ_LATENCY__GFX101         = 0x00000004,
    CHCG_PERF_SEL_GL2_REQ_WRITE_LATENCY__GFX101        = 0x00000005,
    CHCG_PERF_SEL_REQ__GFX101                          = 0x00000006,
    CHCG_PERF_SEL_REQ_ATOMIC_WITH_RET__GFX101          = 0x00000007,
    CHCG_PERF_SEL_REQ_ATOMIC_WITHOUT_RET__GFX101       = 0x00000008,
    CHCG_PERF_SEL_REQ_NOP_ACK__GFX101                  = 0x00000009,
    CHCG_PERF_SEL_REQ_NOP_RTN0__GFX101                 = 0x0000000a,
    CHCG_PERF_SEL_REQ_READ__GFX101                     = 0x0000000b,
    CHCG_PERF_SEL_REQ_READ_128B__GFX101                = 0x0000000c,
    CHCG_PERF_SEL_REQ_READ_32B__GFX101                 = 0x0000000d,
    CHCG_PERF_SEL_REQ_READ_64B__GFX101                 = 0x0000000e,
    CHCG_PERF_SEL_REQ_WRITE__GFX101                    = 0x0000000f,
    CHCG_PERF_SEL_REQ_WRITE_32B__GFX101                = 0x00000010,
    CHCG_PERF_SEL_REQ_WRITE_64B__GFX101                = 0x00000011,
    CHCG_PERF_SEL_STALL_GUS_GL1__GFX101                = 0x00000012,
    CHCG_PERF_SEL_STALL_BUFFER_FULL__GFX101            = 0x00000013,
    CHCG_PERF_SEL_REQ_CLIENT0__GFX101                  = 0x00000014,
    CHCG_PERF_SEL_REQ_CLIENT1__GFX101                  = 0x00000015,
    CHCG_PERF_SEL_REQ_CLIENT2__GFX101                  = 0x00000016,
    CHCG_PERF_SEL_REQ_CLIENT3__GFX101                  = 0x00000017,
    CHCG_PERF_SEL_REQ_CLIENT4__GFX101                  = 0x00000018,
    CHCG_PERF_SEL_REQ_CLIENT5__GFX101                  = 0x00000019,
    CHCG_PERF_SEL_REQ_CLIENT6__GFX101                  = 0x0000001a,
    CHCG_PERF_SEL_REQ_CLIENT7__GFX101                  = 0x0000001b,
    CHCG_PERF_SEL_REQ_CLIENT8__GFX101                  = 0x0000001c,
    CHCG_PERF_SEL_REQ_CLIENT9__GFX101                  = 0x0000001d,
    CHCG_PERF_SEL_REQ_CLIENT10__GFX101                 = 0x0000001e,
    CHCG_PERF_SEL_REQ_CLIENT11__GFX101                 = 0x0000001f,
    CHCG_PERF_SEL_REQ_CLIENT12__GFX101                 = 0x00000020,
    CHCG_PERF_SEL_REQ_CLIENT13__GFX101                 = 0x00000021,
    CHCG_PERF_SEL_REQ_CLIENT14__GFX101                 = 0x00000022,
    CHCG_PERF_SEL_CYCLE__NV2X                          = 0x00000000,
    CHCG_PERF_SEL_BUSY__NV2X                           = 0x00000001,
    CHCG_PERF_SEL_STARVE__NV2X                         = 0x00000002,
    CHCG_PERF_SEL_ARB_RET_LEVEL__NV2X                  = 0x00000003,
    CHCG_PERF_SEL_GL2_REQ_READ_LATENCY__NV2X           = 0x00000004,
    CHCG_PERF_SEL_GL2_REQ_WRITE_LATENCY__NV2X          = 0x00000005,
    CHCG_PERF_SEL_REQ__NV2X                            = 0x00000006,
    CHCG_PERF_SEL_REQ_ATOMIC_WITH_RET__NV2X            = 0x00000007,
    CHCG_PERF_SEL_REQ_ATOMIC_WITHOUT_RET__NV2X         = 0x00000008,
    CHCG_PERF_SEL_REQ_NOP_ACK__NV2X                    = 0x00000009,
    CHCG_PERF_SEL_REQ_NOP_RTN0__NV2X                   = 0x0000000a,
    CHCG_PERF_SEL_REQ_READ__NV2X                       = 0x0000000b,
    CHCG_PERF_SEL_REQ_READ_128B__NV2X                  = 0x0000000c,
    CHCG_PERF_SEL_REQ_READ_32B__NV2X                   = 0x0000000d,
    CHCG_PERF_SEL_REQ_READ_64B__NV2X                   = 0x0000000e,
    CHCG_PERF_SEL_REQ_WRITE__NV2X                      = 0x0000000f,
    CHCG_PERF_SEL_REQ_WRITE_32B__NV2X                  = 0x00000010,
    CHCG_PERF_SEL_REQ_WRITE_64B__NV2X                  = 0x00000011,
    CHCG_PERF_SEL_STALL_GUS_GL1__NV2X                  = 0x00000012,
    CHCG_PERF_SEL_STALL_BUFFER_FULL__NV2X              = 0x00000013,
    CHCG_PERF_SEL_REQ_CLIENT0__NV2X                    = 0x00000014,
    CHCG_PERF_SEL_REQ_CLIENT1__NV2X                    = 0x00000015,
    CHCG_PERF_SEL_REQ_CLIENT2__NV2X                    = 0x00000016,
    CHCG_PERF_SEL_REQ_CLIENT3__NV2X                    = 0x00000017,
    CHCG_PERF_SEL_REQ_CLIENT4__NV2X                    = 0x00000018,
    CHCG_PERF_SEL_REQ_CLIENT5__NV2X                    = 0x00000019,
    CHCG_PERF_SEL_REQ_CLIENT6__NV2X                    = 0x0000001a,
    CHCG_PERF_SEL_REQ_CLIENT7__NV2X                    = 0x0000001b,
    CHCG_PERF_SEL_REQ_CLIENT8__NV2X                    = 0x0000001c,
    CHCG_PERF_SEL_REQ_CLIENT9__NV2X                    = 0x0000001d,
    CHCG_PERF_SEL_REQ_CLIENT10__NV2X                   = 0x0000001e,
    CHCG_PERF_SEL_REQ_CLIENT11__NV2X                   = 0x0000001f,
    CHCG_PERF_SEL_REQ_CLIENT12__NV2X                   = 0x00000020,
    CHCG_PERF_SEL_REQ_CLIENT13__NV2X                   = 0x00000021,
    CHCG_PERF_SEL_REQ_CLIENT14__NV2X                   = 0x00000022,
    CHCG_PERF_SEL_REQ_CLIENT15__NV2X                   = 0x00000023,
    CHCG_PERF_SEL_REQ_CLIENT16__NV2X                   = 0x00000024,
    CHCG_PERF_SEL_REQ_CLIENT17__NV2X                   = 0x00000025,
    CHCG_PERF_SEL_REQ_CLIENT18__NV2X                   = 0x00000026,
    CHCG_PERF_SEL_REQ_CLIENT19__NV2X                   = 0x00000027,
    CHCG_PERF_SEL_CYCLE__NV3X                          = 0x00000000,
    CHCG_PERF_SEL_BUSY__NV3X                           = 0x00000001,
    CHCG_PERF_SEL_STARVE__NV3X                         = 0x00000002,
    CHCG_PERF_SEL_ARB_RET_LEVEL__NV3X                  = 0x00000003,
    CHCG_PERF_SEL_GL2_REQ_READ_LATENCY__NV3X           = 0x00000004,
    CHCG_PERF_SEL_GL2_REQ_WRITE_LATENCY__NV3X          = 0x00000005,
    CHCG_PERF_SEL_REQ__NV3X                            = 0x00000006,
    CHCG_PERF_SEL_REQ_ATOMIC_WITH_RET__NV3X            = 0x00000007,
    CHCG_PERF_SEL_REQ_ATOMIC_WITHOUT_RET__NV3X         = 0x00000008,
    CHCG_PERF_SEL_REQ_NOP_ACK__NV3X                    = 0x00000009,
    CHCG_PERF_SEL_REQ_NOP_RTN0__NV3X                   = 0x0000000a,
    CHCG_PERF_SEL_REQ_READ__NV3X                       = 0x0000000b,
    CHCG_PERF_SEL_REQ_READ_128B__NV3X                  = 0x0000000c,
    CHCG_PERF_SEL_REQ_READ_32B__NV3X                   = 0x0000000d,
    CHCG_PERF_SEL_REQ_READ_64B__NV3X                   = 0x0000000e,
    CHCG_PERF_SEL_REQ_WRITE__NV3X                      = 0x0000000f,
    CHCG_PERF_SEL_REQ_WRITE_32B__NV3X                  = 0x00000010,
    CHCG_PERF_SEL_REQ_WRITE_64B__NV3X                  = 0x00000011,
    CHCG_PERF_SEL_STALL_GUS_GL1__NV3X                  = 0x00000012,
    CHCG_PERF_SEL_STALL_BUFFER_FULL__NV3X              = 0x00000013,
    CHCG_PERF_SEL_REQ_CLIENT0__NV3X                    = 0x00000014,
    CHCG_PERF_SEL_REQ_CLIENT1__NV3X                    = 0x00000015,
    CHCG_PERF_SEL_REQ_CLIENT2__NV3X                    = 0x00000016,
    CHCG_PERF_SEL_REQ_CLIENT3__NV3X                    = 0x00000017,
    CHCG_PERF_SEL_REQ_CLIENT4__NV3X                    = 0x00000018,
    CHCG_PERF_SEL_REQ_CLIENT5__NV3X                    = 0x00000019,
    CHCG_PERF_SEL_REQ_CLIENT6__NV3X                    = 0x0000001a,
    CHCG_PERF_SEL_REQ_CLIENT7__NV3X                    = 0x0000001b,
    CHCG_PERF_SEL_REQ_CLIENT8__NV3X                    = 0x0000001c,
    CHCG_PERF_SEL_REQ_CLIENT9__NV3X                    = 0x0000001d,
    CHCG_PERF_SEL_REQ_CLIENT10__NV3X                   = 0x0000001e,
    CHCG_PERF_SEL_REQ_CLIENT11__NV3X                   = 0x0000001f,
    CHCG_PERF_SEL_REQ_CLIENT12__NV3X                   = 0x00000020,
    CHCG_PERF_SEL_REQ_CLIENT13__NV3X                   = 0x00000021,
    CHCG_PERF_SEL_REQ_CLIENT14__NV3X                   = 0x00000022,
    CHCG_PERF_SEL_REQ_CLIENT15__NV3X                   = 0x00000023,
    CHCG_PERF_SEL_REQ_CLIENT16__NV3X                   = 0x00000024,
    CHCG_PERF_SEL_REQ_CLIENT17__NV3X                   = 0x00000025,
    CHCG_PERF_SEL_REQ_CLIENT18__NV3X                   = 0x00000026,
    CHCG_PERF_SEL_REQ_CLIENT19__NV3X                   = 0x00000027,
    CHCG_PERF_SEL_REQ_CLIENT20__NV3X                   = 0x00000028,
    CHCG_PERF_SEL_REQ_CLIENT21__NV3X                   = 0x00000029,
    CHCG_PERF_SEL_REQ_CLIENT22__NV3X                   = 0x0000002a,
    CHCG_PERF_SEL_REQ_CLIENT23__NV3X                   = 0x0000002b,
} CHCG_PERF_SEL;

constexpr unsigned int MaxChcgPerfSelGfx101            = CHCG_PERF_SEL_REQ_CLIENT14__GFX101;
constexpr unsigned int MaxChcgPerfSelNv2x              = CHCG_PERF_SEL_REQ_CLIENT19__NV2X;
constexpr unsigned int MaxChcgPerfSelNv3x              = CHCG_PERF_SEL_REQ_CLIENT23__NV3X;

typedef enum CHC_PERF_SEL {
    CHC_PERF_SEL_CYCLE                                 = 0x00000000,
    CHC_PERF_SEL_BUSY                                  = 0x00000001,
    CHC_PERF_SEL_STARVE                                = 0x00000002,
    CHC_PERF_SEL_ARB_RET_LEVEL                         = 0x00000003,
    CHC_PERF_SEL_GL2_REQ_READ_LATENCY                  = 0x00000004,
    CHC_PERF_SEL_GL2_REQ_WRITE_LATENCY                 = 0x00000005,
    CHC_PERF_SEL_REQ                                   = 0x00000006,
    CHC_PERF_SEL_REQ_ATOMIC_WITH_RET                   = 0x00000007,
    CHC_PERF_SEL_REQ_ATOMIC_WITHOUT_RET                = 0x00000008,
    CHC_PERF_SEL_REQ_NOP_ACK                           = 0x00000009,
    CHC_PERF_SEL_REQ_NOP_RTN0                          = 0x0000000a,
    CHC_PERF_SEL_REQ_READ                              = 0x0000000b,
    CHC_PERF_SEL_REQ_READ_128B                         = 0x0000000c,
    CHC_PERF_SEL_REQ_READ_32B                          = 0x0000000d,
    CHC_PERF_SEL_REQ_READ_64B                          = 0x0000000e,
    CHC_PERF_SEL_REQ_WRITE                             = 0x0000000f,
    CHC_PERF_SEL_REQ_WRITE_32B                         = 0x00000010,
    CHC_PERF_SEL_REQ_WRITE_64B                         = 0x00000011,
    CHC_PERF_SEL_STALL_GL2_GL1                         = 0x00000012,
    CHC_PERF_SEL_STALL_BUFFER_FULL                     = 0x00000013,
    CHC_PERF_SEL_REQ_CLIENT0                           = 0x00000014,
    CHC_PERF_SEL_REQ_CLIENT1                           = 0x00000015,
    CHC_PERF_SEL_REQ_CLIENT2                           = 0x00000016,
    CHC_PERF_SEL_REQ_CLIENT3                           = 0x00000017,
    CHC_PERF_SEL_REQ_CLIENT4                           = 0x00000018,
    CHC_PERF_SEL_REQ_CLIENT5                           = 0x00000019,
    CHC_PERF_SEL_REQ_CLIENT6                           = 0x0000001a,
    CHC_PERF_SEL_REQ_CLIENT7                           = 0x0000001b,
    CHC_PERF_SEL_REQ_CLIENT8                           = 0x0000001c,
    CHC_PERF_SEL_REQ_CLIENT9                           = 0x0000001d,
    CHC_PERF_SEL_REQ_CLIENT10                          = 0x0000001e,
    CHC_PERF_SEL_REQ_CLIENT11                          = 0x0000001f,
    CHC_PERF_SEL_REQ_CLIENT12                          = 0x00000020,
    CHC_PERF_SEL_REQ_CLIENT13                          = 0x00000021,
    CHC_PERF_SEL_REQ_CLIENT14                          = 0x00000022,
    CHC_PERF_SEL_REQ_CLIENT15__GFX103PLUS              = 0x00000023,
    CHC_PERF_SEL_REQ_CLIENT16__GFX103PLUS              = 0x00000024,
    CHC_PERF_SEL_REQ_CLIENT17__GFX103PLUS              = 0x00000025,
    CHC_PERF_SEL_REQ_CLIENT18__GFX103PLUS              = 0x00000026,
    CHC_PERF_SEL_REQ_CLIENT19__GFX103PLUS              = 0x00000027,
    CHC_PERF_SEL_REQ_CLIENT20__GFX11                   = 0x00000028,
    CHC_PERF_SEL_REQ_CLIENT21__GFX11                   = 0x00000029,
    CHC_PERF_SEL_REQ_CLIENT22__GFX11                   = 0x0000002a,
    CHC_PERF_SEL_REQ_CLIENT23__GFX11                   = 0x0000002b,
} CHC_PERF_SEL;

constexpr unsigned int MaxChcPerfSelGfx101             = CHC_PERF_SEL_REQ_CLIENT14;
constexpr unsigned int MaxChcPerfSelGfx10Vrs           = CHC_PERF_SEL_REQ_CLIENT19__GFX103PLUS;
constexpr unsigned int MaxChcPerfSelGfx11              = CHC_PERF_SEL_REQ_CLIENT23__GFX11;

typedef enum CmaskAddr {
    CMASK_ADDR_TILED                                   = 0x00000000,
    CMASK_ADDR_LINEAR                                  = 0x00000001,
    CMASK_ADDR_COMPATIBLE                              = 0x00000002,
} CmaskAddr;

typedef enum CmaskCode {
    CMASK_CLR00_F0                                     = 0x00000000,
    CMASK_CLR00_F1                                     = 0x00000001,
    CMASK_CLR00_F2                                     = 0x00000002,
    CMASK_CLR00_FX                                     = 0x00000003,
    CMASK_CLR01_F0                                     = 0x00000004,
    CMASK_CLR01_F1                                     = 0x00000005,
    CMASK_CLR01_F2                                     = 0x00000006,
    CMASK_CLR01_FX                                     = 0x00000007,
    CMASK_CLR10_F0                                     = 0x00000008,
    CMASK_CLR10_F1                                     = 0x00000009,
    CMASK_CLR10_F2                                     = 0x0000000a,
    CMASK_CLR10_FX                                     = 0x0000000b,
    CMASK_CLR11_F0                                     = 0x0000000c,
    CMASK_CLR11_F1                                     = 0x0000000d,
    CMASK_CLR11_F2                                     = 0x0000000e,
    CMASK_CLR11_FX                                     = 0x0000000f,
} CmaskCode;

typedef enum CmaskMode {
    CMASK_CLEAR_NONE                                   = 0x00000000,
    CMASK_CLEAR_ONE                                    = 0x00000001,
    CMASK_CLEAR_ALL                                    = 0x00000002,
    CMASK_ANY_EXPANDED                                 = 0x00000003,
    CMASK_ALPHA0_FRAG1                                 = 0x00000004,
    CMASK_ALPHA0_FRAG2                                 = 0x00000005,
    CMASK_ALPHA0_FRAG4                                 = 0x00000006,
    CMASK_ALPHA0_FRAGS                                 = 0x00000007,
    CMASK_ALPHA1_FRAG1                                 = 0x00000008,
    CMASK_ALPHA1_FRAG2                                 = 0x00000009,
    CMASK_ALPHA1_FRAG4                                 = 0x0000000a,
    CMASK_ALPHA1_FRAGS                                 = 0x0000000b,
    CMASK_ALPHAX_FRAG1                                 = 0x0000000c,
    CMASK_ALPHAX_FRAG2                                 = 0x0000000d,
    CMASK_ALPHAX_FRAG4                                 = 0x0000000e,
    CMASK_ALPHAX_FRAGS                                 = 0x0000000f,
} CmaskMode;

typedef enum ColorArray {
    ARRAY_2D_ALT_COLOR                                 = 0x00000000,
    ARRAY_2D_COLOR                                     = 0x00000001,
    ARRAY_3D_SLICE_COLOR                               = 0x00000003,
} ColorArray;

typedef enum ColorFormat {
    COLOR_INVALID                                      = 0x00000000,
    COLOR_8                                            = 0x00000001,
    COLOR_16                                           = 0x00000002,
    COLOR_8_8                                          = 0x00000003,
    COLOR_32                                           = 0x00000004,
    COLOR_16_16                                        = 0x00000005,
    COLOR_10_11_11                                     = 0x00000006,
    COLOR_11_11_10                                     = 0x00000007,
    COLOR_10_10_10_2                                   = 0x00000008,
    COLOR_2_10_10_10                                   = 0x00000009,
    COLOR_8_8_8_8                                      = 0x0000000a,
    COLOR_32_32                                        = 0x0000000b,
    COLOR_16_16_16_16                                  = 0x0000000c,
    COLOR_RESERVED_13                                  = 0x0000000d,
    COLOR_32_32_32_32                                  = 0x0000000e,
    COLOR_RESERVED_15                                  = 0x0000000f,
    COLOR_5_6_5                                        = 0x00000010,
    COLOR_1_5_5_5                                      = 0x00000011,
    COLOR_5_5_5_1                                      = 0x00000012,
    COLOR_4_4_4_4                                      = 0x00000013,
    COLOR_8_24                                         = 0x00000014,
    COLOR_24_8                                         = 0x00000015,
    COLOR_X24_8_32_FLOAT                               = 0x00000016,
    COLOR_RESERVED_23                                  = 0x00000017,
    COLOR_RESERVED_25                                  = 0x00000019,
    COLOR_RESERVED_26                                  = 0x0000001a,
    COLOR_RESERVED_27                                  = 0x0000001b,
    COLOR_RESERVED_28                                  = 0x0000001c,
    COLOR_RESERVED_29                                  = 0x0000001d,
    COLOR_2_10_10_10_6E4                               = 0x0000001f,
    COLOR_RESERVED_24__GFX101                          = 0x00000018,
    COLOR_RESERVED_30__GFX101                          = 0x0000001e,
    COLOR_5_9_9_9__GFX103PLUS                          = 0x00000018,
    COLOR_2_10_10_10_7E3__GFX103PLUS                   = 0x0000001e,
} ColorFormat;

typedef enum ColorTransform {
    DCC_CT_AUTO                                        = 0x00000000,
    DCC_CT_NONE                                        = 0x00000001,
    ABGR_TO_A_BG_G_RB                                  = 0x00000002,
    BGRA_TO_BG_G_RB_A                                  = 0x00000003,
} ColorTransform;

typedef enum CombFunc {
    COMB_DST_PLUS_SRC                                  = 0x00000000,
    COMB_SRC_MINUS_DST                                 = 0x00000001,
    COMB_MIN_DST_SRC                                   = 0x00000002,
    COMB_MAX_DST_SRC                                   = 0x00000003,
    COMB_DST_MINUS_SRC                                 = 0x00000004,
} CombFunc;

typedef enum CompareFrag {
    FRAG_NEVER                                         = 0x00000000,
    FRAG_LESS                                          = 0x00000001,
    FRAG_EQUAL                                         = 0x00000002,
    FRAG_LEQUAL                                        = 0x00000003,
    FRAG_GREATER                                       = 0x00000004,
    FRAG_NOTEQUAL                                      = 0x00000005,
    FRAG_GEQUAL                                        = 0x00000006,
    FRAG_ALWAYS                                        = 0x00000007,
} CompareFrag;

typedef enum CompareRef {
    REF_NEVER                                          = 0x00000000,
    REF_LESS                                           = 0x00000001,
    REF_EQUAL                                          = 0x00000002,
    REF_LEQUAL                                         = 0x00000003,
    REF_GREATER                                        = 0x00000004,
    REF_NOTEQUAL                                       = 0x00000005,
    REF_GEQUAL                                         = 0x00000006,
    REF_ALWAYS                                         = 0x00000007,
} CompareRef;

typedef enum ConservativeZExport {
    EXPORT_ANY_Z                                       = 0x00000000,
    EXPORT_LESS_THAN_Z                                 = 0x00000001,
    EXPORT_GREATER_THAN_Z                              = 0x00000002,
    EXPORT_RESERVED                                    = 0x00000003,
} ConservativeZExport;

typedef enum CovToShaderSel {
    INPUT_COVERAGE                                     = 0x00000000,
    INPUT_INNER_COVERAGE                               = 0x00000001,
    INPUT_DEPTH_COVERAGE                               = 0x00000002,
    RAW                                                = 0x00000003,
} CovToShaderSel;

typedef enum CPC_LATENCY_STATS_SEL {
    CPC_LATENCY_STATS_SEL_XACK_MAX                     = 0x00000000,
    CPC_LATENCY_STATS_SEL_XACK_MIN                     = 0x00000001,
    CPC_LATENCY_STATS_SEL_XACK_LAST                    = 0x00000002,
    CPC_LATENCY_STATS_SEL_XNACK_MAX                    = 0x00000003,
    CPC_LATENCY_STATS_SEL_XNACK_MIN                    = 0x00000004,
    CPC_LATENCY_STATS_SEL_XNACK_LAST                   = 0x00000005,
    CPC_LATENCY_STATS_SEL_INVAL_MAX__GFX10             = 0x00000006,
    CPC_LATENCY_STATS_SEL_INVAL_MIN__GFX10             = 0x00000007,
    CPC_LATENCY_STATS_SEL_INVAL_LAST__GFX10            = 0x00000008,
    CPC_LATENCY_STATS_SEL_INVAL_MAX__NV31              = 0x00000006,
    CPC_LATENCY_STATS_SEL_INVAL_MIN__NV31              = 0x00000007,
    CPC_LATENCY_STATS_SEL_INVAL_LAST__NV31             = 0x00000008,
    CPC_LATENCY_STATS_SEL_WRITE_MAX__NV32              = 0x00000006,
    CPC_LATENCY_STATS_SEL_WRITE_MIN__NV32              = 0x00000007,
    CPC_LATENCY_STATS_SEL_WRITE_LAST__NV32             = 0x00000008,
    CPC_LATENCY_STATS_SEL_READ_MAX__NV32               = 0x00000009,
    CPC_LATENCY_STATS_SEL_READ_MIN__NV32               = 0x0000000a,
    CPC_LATENCY_STATS_SEL_READ_LAST__NV32              = 0x0000000b,
    CPC_LATENCY_STATS_SEL_ATOMIC_MAX__NV32             = 0x0000000c,
    CPC_LATENCY_STATS_SEL_ATOMIC_MIN__NV32             = 0x0000000d,
    CPC_LATENCY_STATS_SEL_ATOMIC_LAST__NV32            = 0x0000000e,
    CPC_LATENCY_STATS_SEL_INVAL_MAX__NV32              = 0x0000000f,
    CPC_LATENCY_STATS_SEL_INVAL_MIN__NV32              = 0x00000010,
    CPC_LATENCY_STATS_SEL_INVAL_LAST__NV32             = 0x00000011,
    CPC_LATENCY_STATS_SEL_INVAL_MAX__NV33              = 0x00000006,
    CPC_LATENCY_STATS_SEL_INVAL_MIN__NV33              = 0x00000007,
    CPC_LATENCY_STATS_SEL_INVAL_LAST__NV33             = 0x00000008,
    CPC_LATENCY_STATS_SEL_INVAL_MAX__PHX1              = 0x00000006,
    CPC_LATENCY_STATS_SEL_INVAL_MIN__PHX1              = 0x00000007,
    CPC_LATENCY_STATS_SEL_INVAL_LAST__PHX1             = 0x00000008,
    CPC_LATENCY_STATS_SEL_INVAL_MAX__PHX2              = 0x00000006,
    CPC_LATENCY_STATS_SEL_INVAL_MIN__PHX2              = 0x00000007,
    CPC_LATENCY_STATS_SEL_INVAL_LAST__PHX2             = 0x00000008,
    CPC_LATENCY_STATS_SEL_INVAL_MAX__STX1              = 0x00000006,
    CPC_LATENCY_STATS_SEL_INVAL_MIN__STX1              = 0x00000007,
    CPC_LATENCY_STATS_SEL_INVAL_LAST__STX1             = 0x00000008,
#if CHIP_HDR_STRIX_HALO
    CPC_LATENCY_STATS_SEL_WRITE_MAX__STXH              = 0x00000006,
    CPC_LATENCY_STATS_SEL_WRITE_MIN__STXH              = 0x00000007,
    CPC_LATENCY_STATS_SEL_WRITE_LAST__STXH             = 0x00000008,
    CPC_LATENCY_STATS_SEL_READ_MAX__STXH               = 0x00000009,
    CPC_LATENCY_STATS_SEL_READ_MIN__STXH               = 0x0000000a,
    CPC_LATENCY_STATS_SEL_READ_LAST__STXH              = 0x0000000b,
    CPC_LATENCY_STATS_SEL_ATOMIC_MAX__STXH             = 0x0000000c,
    CPC_LATENCY_STATS_SEL_ATOMIC_MIN__STXH             = 0x0000000d,
    CPC_LATENCY_STATS_SEL_ATOMIC_LAST__STXH            = 0x0000000e,
    CPC_LATENCY_STATS_SEL_INVAL_MAX__STXH              = 0x0000000f,
    CPC_LATENCY_STATS_SEL_INVAL_MIN__STXH              = 0x00000010,
    CPC_LATENCY_STATS_SEL_INVAL_LAST__STXH             = 0x00000011,
#endif
} CPC_LATENCY_STATS_SEL;

#if   CHIP_HDR_STRIX_HALO
typedef enum CPC_PERFCOUNTWINDOW_SEL {
#if CHIP_HDR_STRIX_HALO
    CPC_PERFWINDOW_SEL_MES__STXH                       = 0x00000000,
    CPC_PERFWINDOW_SEL_MEC1__STXH                      = 0x00000001,
    CPC_PERFWINDOW_SEL_MEC2__STXH                      = 0x00000002,
    CPC_PERFWINDOW_SEL_SR__STXH                        = 0x00000003,
    CPC_PERFWINDOW_SEL_CPC_IC__STXH                    = 0x00000004,
#endif
} CPC_PERFCOUNTWINDOW_SEL;
#endif

typedef enum CPC_PERFCOUNT_SEL {
    CPC_PERF_SEL_ALWAYS_COUNT                          = 0x00000000,
    CPC_PERF_SEL_RCIU_STALL_WAIT_ON_FREE               = 0x00000001,
    CPC_PERF_SEL_RCIU_STALL_PRIV_VIOLATION             = 0x00000002,
    CPC_PERF_SEL_TCIU_STALL_WAIT_ON_FREE               = 0x00000005,
    CPC_PERF_SEL_ME1_STALL_WAIT_ON_RCIU_READY          = 0x00000006,
    CPC_PERF_SEL_ME1_STALL_WAIT_ON_RCIU_READY_PERF     = 0x00000007,
    CPC_PERF_SEL_ME1_STALL_WAIT_ON_RCIU_READ           = 0x00000008,
    CPC_PERF_SEL_ME1_STALL_ON_DATA_FROM_ROQ            = 0x0000000b,
    CPC_PERF_SEL_ME1_STALL_ON_DATA_FROM_ROQ_PERF       = 0x0000000c,
    CPC_PERF_SEL_ME1_BUSY_FOR_PACKET_DECODE            = 0x0000000d,
    CPC_PERF_SEL_ME2_STALL_WAIT_ON_RCIU_READY          = 0x0000000e,
    CPC_PERF_SEL_ME2_STALL_WAIT_ON_RCIU_READY_PERF     = 0x0000000f,
    CPC_PERF_SEL_ME2_STALL_WAIT_ON_RCIU_READ           = 0x00000010,
    CPC_PERF_SEL_ME2_STALL_ON_DATA_FROM_ROQ            = 0x00000013,
    CPC_PERF_SEL_ME2_STALL_ON_DATA_FROM_ROQ_PERF       = 0x00000014,
    CPC_PERF_SEL_ME2_BUSY_FOR_PACKET_DECODE            = 0x00000015,
    CPC_PERF_SEL_UTCL2IU_STALL_WAIT_ON_FREE            = 0x00000016,
    CPC_PERF_SEL_UTCL2IU_STALL_WAIT_ON_TAGS            = 0x00000017,
    CPC_PERF_SEL_UTCL1_STALL_ON_TRANSLATION            = 0x00000018,
    CPC_PERF_SEL_CPC_STAT_BUSY                         = 0x00000019,
    CPC_PERF_SEL_CPC_STAT_IDLE                         = 0x0000001a,
    CPC_PERF_SEL_CPC_STAT_STALL                        = 0x0000001b,
    CPC_PERF_SEL_CPC_TCIU_BUSY                         = 0x0000001c,
    CPC_PERF_SEL_CPC_TCIU_IDLE                         = 0x0000001d,
    CPC_PERF_SEL_CPC_UTCL2IU_BUSY                      = 0x0000001e,
    CPC_PERF_SEL_CPC_UTCL2IU_IDLE                      = 0x0000001f,
    CPC_PERF_SEL_CPC_UTCL2IU_STALL                     = 0x00000020,
    CPC_PERF_SEL_ME1_DC0_SPI_BUSY                      = 0x00000021,
    CPC_PERF_SEL_ME2_DC1_SPI_BUSY                      = 0x00000022,
    CPC_PERF_SEL_CPC_GCRIU_BUSY                        = 0x00000023,
    CPC_PERF_SEL_CPC_GCRIU_IDLE                        = 0x00000024,
    CPC_PERF_SEL_CPC_GCRIU_STALL                       = 0x00000025,
    CPC_PERF_SEL_GCRIU_STALL_WAIT_ON_FREE              = 0x00000026,
    CPC_PERF_SEL_ME1_STALL_WAIT_ON_TCIU_READ           = 0x00000027,
    CPC_PERF_SEL_ME2_STALL_WAIT_ON_TCIU_READ           = 0x00000028,
    CPC_PERF_SEL_CPC_UTCL2IU_XACK                      = 0x00000029,
    CPC_PERF_SEL_CPC_UTCL2IU_XNACK                     = 0x0000002a,
    CPC_PERF_SEL_MEC_INSTR_CACHE_HIT                   = 0x0000002b,
    CPC_PERF_SEL_MEC_INSTR_CACHE_MISS                  = 0x0000002c,
    CPC_PERF_SEL_MES_THREAD0                           = 0x0000002d,
    CPC_PERF_SEL_MES_THREAD1                           = 0x0000002e,
    CPC_PERF_SEL_MIU_STALL_ON_RDREQ_FREE__GFX10        = 0x00000003,
    CPC_PERF_SEL_MIU_STALL_ON_WRREQ_FREE__GFX10        = 0x00000004,
    CPC_PERF_SEL_ME1_STALL_WAIT_ON_GUS_READ__GFX10     = 0x00000009,
    CPC_PERF_SEL_ME1_STALL_WAIT_ON_GUS_WRITE__GFX10    = 0x0000000a,
    CPC_PERF_SEL_ME2_STALL_WAIT_ON_GUS_READ__GFX10     = 0x00000011,
    CPC_PERF_SEL_ME2_STALL_WAIT_ON_GUS_WRITE__GFX10    = 0x00000012,
    CPC_PERF_SEL_ME1_STALL_WAIT_ON_MEM_READ__GFX11     = 0x00000009,
    CPC_PERF_SEL_ME1_STALL_WAIT_ON_MEM_WRITE__GFX11    = 0x0000000a,
    CPC_PERF_SEL_ME2_STALL_WAIT_ON_MEM_READ__GFX11     = 0x00000011,
    CPC_PERF_SEL_ME2_STALL_WAIT_ON_MEM_WRITE__GFX11    = 0x00000012,
    CPC_PERF_SEL_TCIU_STALL_WAIT_ON_TAGS__GFX11        = 0x0000002f,
    CPC_PERF_SEL_TCIU_WRITE_REQUEST_SENT__GFX11        = 0x00000030,
    CPC_PERF_SEL_TCIU_READ_REQUEST_SENT__GFX11         = 0x00000031,
    CPC_PERF_SEL_GUS_WRITE_REQUEST_SENT__GFX11         = 0x00000032,
    CPC_PERF_SEL_GUS_READ_REQUEST_SENT__GFX11          = 0x00000033,
    CPC_PERF_SEL_MEC_THREAD0__GFX11                    = 0x00000034,
    CPC_PERF_SEL_MEC_THREAD1__GFX11                    = 0x00000035,
    CPC_PERF_SEL_MEC_THREAD2__GFX11                    = 0x00000036,
    CPC_PERF_SEL_MEC_THREAD3__GFX11                    = 0x00000037,
} CPC_PERFCOUNT_SEL;

constexpr unsigned int MaxCpcPerfcountSelGfx10         = CPC_PERF_SEL_MES_THREAD1;
constexpr unsigned int MaxCpcPerfcountSelGfx11         = CPC_PERF_SEL_MEC_THREAD3__GFX11;

typedef enum CPF_LATENCY_STATS_SEL {
    CPF_LATENCY_STATS_SEL_XACK_MAX                     = 0x00000000,
    CPF_LATENCY_STATS_SEL_XACK_MIN                     = 0x00000001,
    CPF_LATENCY_STATS_SEL_XACK_LAST                    = 0x00000002,
    CPF_LATENCY_STATS_SEL_XNACK_MAX                    = 0x00000003,
    CPF_LATENCY_STATS_SEL_XNACK_MIN                    = 0x00000004,
    CPF_LATENCY_STATS_SEL_XNACK_LAST                   = 0x00000005,
    CPF_LATENCY_STATS_SEL_READ_MAX                     = 0x00000006,
    CPF_LATENCY_STATS_SEL_READ_MIN                     = 0x00000007,
    CPF_LATENCY_STATS_SEL_READ_LAST                    = 0x00000008,
    CPF_LATENCY_STATS_SEL_INVAL_MAX                    = 0x00000009,
    CPF_LATENCY_STATS_SEL_INVAL_MIN                    = 0x0000000a,
    CPF_LATENCY_STATS_SEL_INVAL_LAST                   = 0x0000000b,
} CPF_LATENCY_STATS_SEL;

typedef enum CPF_PERFCOUNTWINDOW_SEL {
    CPF_PERFWINDOW_SEL_CSF                             = 0x00000000,
    CPF_PERFWINDOW_SEL_HQD1                            = 0x00000001,
    CPF_PERFWINDOW_SEL_HQD2                            = 0x00000002,
    CPF_PERFWINDOW_SEL_RDMA__GFX10                     = 0x00000003,
    CPF_PERFWINDOW_SEL_RWPP__GFX10                     = 0x00000004,
    CPF_PERFWINDOW_SEL_RDMA__GFX110                    = 0x00000003,
    CPF_PERFWINDOW_SEL_RWPP__GFX110                    = 0x00000004,
    CPF_PERFWINDOW_SEL_RDMA__STX1                      = 0x00000003,
    CPF_PERFWINDOW_SEL_RWPP__STX1                      = 0x00000004,
#if CHIP_HDR_STRIX_HALO
    CPF_PERFWINDOW_SEL_HQD3__STXH                      = 0x00000003,
    CPF_PERFWINDOW_SEL_RDMA__STXH                      = 0x00000004,
    CPF_PERFWINDOW_SEL_RWPP__STXH                      = 0x00000005,
#endif
} CPF_PERFCOUNTWINDOW_SEL;

typedef enum CPF_PERFCOUNT_SEL {
    CPF_PERF_SEL_ALWAYS_COUNT                          = 0x00000000,
    CPF_PERF_SEL_TCIU_STALLED_WAITING_ON_FREE          = 0x00000002,
    CPF_PERF_SEL_TCIU_STALLED_WAITING_ON_TAGS          = 0x00000003,
    CPF_PERF_SEL_CSF_BUSY_FOR_FETCHING_RING            = 0x00000004,
    CPF_PERF_SEL_CSF_BUSY_FOR_FETCHING_IB1             = 0x00000005,
    CPF_PERF_SEL_CSF_BUSY_FOR_FETCHING_IB2             = 0x00000006,
    CPF_PERF_SEL_CSF_STATE_FIFO_NOT_RTR                = 0x0000000a,
    CPF_PERF_SEL_CSF_FETCHING_CMD_BUFFERS              = 0x0000000b,
    CPF_PERF_SEL_GRBM_DWORDS_SENT                      = 0x0000000c,
    CPF_PERF_SEL_DYNAMIC_CLOCK_VALID                   = 0x0000000d,
    CPF_PERF_SEL_REGISTER_CLOCK_VALID                  = 0x0000000e,
    CPF_PERF_SEL_GUS_WRITE_REQUEST_SENT                = 0x0000000f,
    CPF_PERF_SEL_GUS_READ_REQUEST_SENT                 = 0x00000010,
    CPF_PERF_SEL_UTCL2IU_STALL_WAIT_ON_FREE            = 0x00000011,
    CPF_PERF_SEL_UTCL2IU_STALL_WAIT_ON_TAGS            = 0x00000012,
    CPF_PERF_SEL_GFX_UTCL1_STALL_ON_TRANSLATION        = 0x00000013,
    CPF_PERF_SEL_CMP_UTCL1_STALL_ON_TRANSLATION        = 0x00000014,
    CPF_PERF_SEL_RCIU_STALL_WAIT_ON_FREE               = 0x00000015,
    CPF_PERF_SEL_TCIU_WRITE_REQUEST_SENT               = 0x00000016,
    CPF_PERF_SEL_TCIU_READ_REQUEST_SENT                = 0x00000017,
    CPF_PERF_SEL_CPF_STAT_BUSY                         = 0x00000018,
    CPF_PERF_SEL_CPF_STAT_IDLE                         = 0x00000019,
    CPF_PERF_SEL_CPF_STAT_STALL                        = 0x0000001a,
    CPF_PERF_SEL_CPF_TCIU_BUSY                         = 0x0000001b,
    CPF_PERF_SEL_CPF_TCIU_IDLE                         = 0x0000001c,
    CPF_PERF_SEL_CPF_TCIU_STALL                        = 0x0000001d,
    CPF_PERF_SEL_CPF_UTCL2IU_BUSY                      = 0x0000001e,
    CPF_PERF_SEL_CPF_UTCL2IU_IDLE                      = 0x0000001f,
    CPF_PERF_SEL_CPF_UTCL2IU_STALL                     = 0x00000020,
    CPF_PERF_SEL_CPF_GCRIU_BUSY                        = 0x00000021,
    CPF_PERF_SEL_CPF_GCRIU_IDLE                        = 0x00000022,
    CPF_PERF_SEL_CPF_GCRIU_STALL                       = 0x00000023,
    CPF_PERF_SEL_GCRIU_STALL_WAIT_ON_FREE              = 0x00000024,
    CPF_PERF_SEL_CSF_BUSY_FOR_FETCHING_DB              = 0x00000025,
    CPF_PERF_SEL_CPF_UTCL2IU_XACK                      = 0x00000026,
    CPF_PERF_SEL_CPF_UTCL2IU_XNACK                     = 0x00000027,
    CPF_PERF_SEL_MIU_STALLED_WAITING_RDREQ_FREE__GFX10 = 0x00000001,
    CPF_PERF_SEL_MIU_BUSY_FOR_OUTSTANDING_TAGS__GFX10  = 0x00000008,
    CPF_PERF_SEL_CSF_RTS_MIU_NOT_RTR__GFX10            = 0x00000009,
    CPF_PERF_SEL_CSF_BUSY_FOR_FECTHINC_STATE__GFX101   = 0x00000007,
    CPF_PERF_SEL_CSF_BUSY_FOR_FETCHING_STATE__GFX103PLUS = 0x00000007,
    CPF_PERF_SEL_CP_SDMA_MNGR_DMA_REQ__GFX11           = 0x00000028,
    CPF_PERF_SEL_CP_SDMA_MNGR_DMA_DONE__GFX11          = 0x00000029,
    CPF_PERF_SEL_CP_SDMA_MNGR_LATENCY__GFX11           = 0x0000002a,
    CPF_PERF_SEL_CP_SDMA_MNGR_SDMABUSY__GFX11          = 0x0000002b,
} CPF_PERFCOUNT_SEL;

constexpr unsigned int MaxCpfPerfcountSelGfx10         = CPF_PERF_SEL_CPF_UTCL2IU_XNACK;
constexpr unsigned int MaxCpfPerfcountSelGfx11         = CPF_PERF_SEL_CP_SDMA_MNGR_SDMABUSY__GFX11;

typedef enum CPF_SCRATCH_REG_ATOMIC_OP {
    CPF_SCRATCH_REG_ATOMIC_ADD                         = 0x00000000,
    CPF_SCRATCH_REG_ATOMIC_SUB                         = 0x00000001,
    CPF_SCRATCH_REG_ATOMIC_OR                          = 0x00000002,
    CPF_SCRATCH_REG_ATOMIC_AND                         = 0x00000003,
    CPF_SCRATCH_REG_ATOMIC_NOT                         = 0x00000004,
    CPF_SCRATCH_REG_ATOMIC_MIN                         = 0x00000005,
    CPF_SCRATCH_REG_ATOMIC_MAX                         = 0x00000006,
    CPF_SCRATCH_REG_ATOMIC_CMPSWAP                     = 0x00000007,
} CPF_SCRATCH_REG_ATOMIC_OP;

typedef enum CPG_LATENCY_STATS_SEL {
    CPG_LATENCY_STATS_SEL_XACK_MAX                     = 0x00000000,
    CPG_LATENCY_STATS_SEL_XACK_MIN                     = 0x00000001,
    CPG_LATENCY_STATS_SEL_XACK_LAST                    = 0x00000002,
    CPG_LATENCY_STATS_SEL_XNACK_MAX                    = 0x00000003,
    CPG_LATENCY_STATS_SEL_XNACK_MIN                    = 0x00000004,
    CPG_LATENCY_STATS_SEL_XNACK_LAST                   = 0x00000005,
    CPG_LATENCY_STATS_SEL_WRITE_MAX                    = 0x00000006,
    CPG_LATENCY_STATS_SEL_WRITE_MIN                    = 0x00000007,
    CPG_LATENCY_STATS_SEL_WRITE_LAST                   = 0x00000008,
    CPG_LATENCY_STATS_SEL_READ_MAX                     = 0x00000009,
    CPG_LATENCY_STATS_SEL_READ_MIN                     = 0x0000000a,
    CPG_LATENCY_STATS_SEL_READ_LAST                    = 0x0000000b,
    CPG_LATENCY_STATS_SEL_ATOMIC_MAX                   = 0x0000000c,
    CPG_LATENCY_STATS_SEL_ATOMIC_MIN                   = 0x0000000d,
    CPG_LATENCY_STATS_SEL_ATOMIC_LAST                  = 0x0000000e,
    CPG_LATENCY_STATS_SEL_INVAL_MAX                    = 0x0000000f,
    CPG_LATENCY_STATS_SEL_INVAL_MIN                    = 0x00000010,
    CPG_LATENCY_STATS_SEL_INVAL_LAST                   = 0x00000011,
} CPG_LATENCY_STATS_SEL;

typedef enum CPG_PERFCOUNTWINDOW_SEL {
    CPG_PERFWINDOW_SEL_PFP                             = 0x00000000,
    CPG_PERFWINDOW_SEL_ME                              = 0x00000001,
    CPG_PERFWINDOW_SEL_CE                              = 0x00000002,
    CPG_PERFWINDOW_SEL_MES__GFX10                      = 0x00000003,
    CPG_PERFWINDOW_SEL_MEC1__GFX10                     = 0x00000004,
    CPG_PERFWINDOW_SEL_MEC2__GFX10                     = 0x00000005,
    CPG_PERFWINDOW_SEL_DFY__GFX10                      = 0x00000006,
    CPG_PERFWINDOW_SEL_DMA__GFX10                      = 0x00000007,
    CPG_PERFWINDOW_SEL_SHADOW__GFX10                   = 0x00000008,
    CPG_PERFWINDOW_SEL_RB__GFX10                       = 0x00000009,
    CPG_PERFWINDOW_SEL_CEDMA__GFX10                    = 0x0000000a,
    CPG_PERFWINDOW_SEL_PRT_HDR_RPTR__GFX10             = 0x0000000b,
    CPG_PERFWINDOW_SEL_PRT_SMP_RPTR__GFX10             = 0x0000000c,
    CPG_PERFWINDOW_SEL_PQ1__GFX10                      = 0x0000000d,
    CPG_PERFWINDOW_SEL_PQ2__GFX10                      = 0x0000000e,
    CPG_PERFWINDOW_SEL_PQ3__GFX10                      = 0x0000000f,
    CPG_PERFWINDOW_SEL_MEMWR__GFX10                    = 0x00000010,
    CPG_PERFWINDOW_SEL_MEMRD__GFX10                    = 0x00000011,
    CPG_PERFWINDOW_SEL_VGT0__GFX10                     = 0x00000012,
    CPG_PERFWINDOW_SEL_VGT1__GFX10                     = 0x00000013,
    CPG_PERFWINDOW_SEL_APPEND__GFX10                   = 0x00000014,
    CPG_PERFWINDOW_SEL_QURD__GFX10                     = 0x00000015,
    CPG_PERFWINDOW_SEL_DDID__GFX10                     = 0x00000016,
    CPG_PERFWINDOW_SEL_SR__GFX10                       = 0x00000017,
    CPG_PERFWINDOW_SEL_QU_EOP__GFX10                   = 0x00000018,
    CPG_PERFWINDOW_SEL_QU_STRM__GFX10                  = 0x00000019,
    CPG_PERFWINDOW_SEL_QU_PIPE__GFX10                  = 0x0000001a,
    CPG_PERFWINDOW_SEL_RESERVED1__GFX10                = 0x0000001b,
    CPG_PERFWINDOW_SEL_CPC_IC__GFX10                   = 0x0000001c,
    CPG_PERFWINDOW_SEL_RESERVED2__GFX10                = 0x0000001d,
    CPG_PERFWINDOW_SEL_CPG_IC__GFX10                   = 0x0000001e,
    CPG_PERFWINDOW_SEL_MES__GFX110                     = 0x00000003,
    CPG_PERFWINDOW_SEL_MEC1__GFX110                    = 0x00000004,
    CPG_PERFWINDOW_SEL_MEC2__GFX110                    = 0x00000005,
    CPG_PERFWINDOW_SEL_DFY__GFX110                     = 0x00000006,
    CPG_PERFWINDOW_SEL_DMA__GFX110                     = 0x00000007,
    CPG_PERFWINDOW_SEL_SHADOW__GFX110                  = 0x00000008,
    CPG_PERFWINDOW_SEL_RB__GFX110                      = 0x00000009,
    CPG_PERFWINDOW_SEL_CEDMA__GFX110                   = 0x0000000a,
    CPG_PERFWINDOW_SEL_PRT_HDR_RPTR__GFX110            = 0x0000000b,
    CPG_PERFWINDOW_SEL_PRT_SMP_RPTR__GFX110            = 0x0000000c,
    CPG_PERFWINDOW_SEL_PQ1__GFX110                     = 0x0000000d,
    CPG_PERFWINDOW_SEL_PQ2__GFX110                     = 0x0000000e,
    CPG_PERFWINDOW_SEL_PQ3__GFX110                     = 0x0000000f,
    CPG_PERFWINDOW_SEL_MEMWR__GFX110                   = 0x00000010,
    CPG_PERFWINDOW_SEL_MEMRD__GFX110                   = 0x00000011,
    CPG_PERFWINDOW_SEL_VGT0__GFX110                    = 0x00000012,
    CPG_PERFWINDOW_SEL_VGT1__GFX110                    = 0x00000013,
    CPG_PERFWINDOW_SEL_APPEND__GFX110                  = 0x00000014,
    CPG_PERFWINDOW_SEL_QURD__GFX110                    = 0x00000015,
    CPG_PERFWINDOW_SEL_DDID__GFX110                    = 0x00000016,
    CPG_PERFWINDOW_SEL_SR__GFX110                      = 0x00000017,
    CPG_PERFWINDOW_SEL_QU_EOP__GFX110                  = 0x00000018,
    CPG_PERFWINDOW_SEL_QU_STRM__GFX110                 = 0x00000019,
    CPG_PERFWINDOW_SEL_QU_PIPE__GFX110                 = 0x0000001a,
    CPG_PERFWINDOW_SEL_RESERVED1__GFX110               = 0x0000001b,
    CPG_PERFWINDOW_SEL_CPC_IC__GFX110                  = 0x0000001c,
    CPG_PERFWINDOW_SEL_RESERVED2__GFX110               = 0x0000001d,
    CPG_PERFWINDOW_SEL_CPG_IC__GFX110                  = 0x0000001e,
    CPG_PERFWINDOW_SEL_MES__STX1                       = 0x00000003,
    CPG_PERFWINDOW_SEL_MEC1__STX1                      = 0x00000004,
    CPG_PERFWINDOW_SEL_MEC2__STX1                      = 0x00000005,
    CPG_PERFWINDOW_SEL_DFY__STX1                       = 0x00000006,
    CPG_PERFWINDOW_SEL_DMA__STX1                       = 0x00000007,
    CPG_PERFWINDOW_SEL_SHADOW__STX1                    = 0x00000008,
    CPG_PERFWINDOW_SEL_RB__STX1                        = 0x00000009,
    CPG_PERFWINDOW_SEL_CEDMA__STX1                     = 0x0000000a,
    CPG_PERFWINDOW_SEL_PRT_HDR_RPTR__STX1              = 0x0000000b,
    CPG_PERFWINDOW_SEL_PRT_SMP_RPTR__STX1              = 0x0000000c,
    CPG_PERFWINDOW_SEL_PQ1__STX1                       = 0x0000000d,
    CPG_PERFWINDOW_SEL_PQ2__STX1                       = 0x0000000e,
    CPG_PERFWINDOW_SEL_PQ3__STX1                       = 0x0000000f,
    CPG_PERFWINDOW_SEL_MEMWR__STX1                     = 0x00000010,
    CPG_PERFWINDOW_SEL_MEMRD__STX1                     = 0x00000011,
    CPG_PERFWINDOW_SEL_VGT0__STX1                      = 0x00000012,
    CPG_PERFWINDOW_SEL_VGT1__STX1                      = 0x00000013,
    CPG_PERFWINDOW_SEL_APPEND__STX1                    = 0x00000014,
    CPG_PERFWINDOW_SEL_QURD__STX1                      = 0x00000015,
    CPG_PERFWINDOW_SEL_DDID__STX1                      = 0x00000016,
    CPG_PERFWINDOW_SEL_SR__STX1                        = 0x00000017,
    CPG_PERFWINDOW_SEL_QU_EOP__STX1                    = 0x00000018,
    CPG_PERFWINDOW_SEL_QU_STRM__STX1                   = 0x00000019,
    CPG_PERFWINDOW_SEL_QU_PIPE__STX1                   = 0x0000001a,
    CPG_PERFWINDOW_SEL_RESERVED1__STX1                 = 0x0000001b,
    CPG_PERFWINDOW_SEL_CPC_IC__STX1                    = 0x0000001c,
    CPG_PERFWINDOW_SEL_RESERVED2__STX1                 = 0x0000001d,
    CPG_PERFWINDOW_SEL_CPG_IC__STX1                    = 0x0000001e,
#if CHIP_HDR_STRIX_HALO
    CPG_PERFWINDOW_SEL_DFY__STXH                       = 0x00000003,
    CPG_PERFWINDOW_SEL_DMA__STXH                       = 0x00000004,
    CPG_PERFWINDOW_SEL_SHADOW__STXH                    = 0x00000005,
    CPG_PERFWINDOW_SEL_RB__STXH                        = 0x00000006,
    CPG_PERFWINDOW_SEL_CEDMA__STXH                     = 0x00000007,
    CPG_PERFWINDOW_SEL_PRT_HDR_RPTR__STXH              = 0x00000008,
    CPG_PERFWINDOW_SEL_PRT_SMP_RPTR__STXH              = 0x00000009,
    CPG_PERFWINDOW_SEL_PQ1__STXH                       = 0x0000000a,
    CPG_PERFWINDOW_SEL_PQ2__STXH                       = 0x0000000b,
    CPG_PERFWINDOW_SEL_PQ3__STXH                       = 0x0000000c,
    CPG_PERFWINDOW_SEL_MEMWR__STXH                     = 0x0000000d,
    CPG_PERFWINDOW_SEL_MEMRD__STXH                     = 0x0000000e,
    CPG_PERFWINDOW_SEL_VGT0__STXH                      = 0x0000000f,
    CPG_PERFWINDOW_SEL_VGT1__STXH                      = 0x00000010,
    CPG_PERFWINDOW_SEL_APPEND__STXH                    = 0x00000011,
    CPG_PERFWINDOW_SEL_QURD__STXH                      = 0x00000012,
    CPG_PERFWINDOW_SEL_DDID__STXH                      = 0x00000013,
    CPG_PERFWINDOW_SEL_QU_EOP__STXH                    = 0x00000014,
    CPG_PERFWINDOW_SEL_QU_STRM__STXH                   = 0x00000015,
    CPG_PERFWINDOW_SEL_QU_PIPE__STXH                   = 0x00000016,
    CPG_PERFWINDOW_SEL_SD__STXH                        = 0x00000017,
    CPG_PERFWINDOW_SEL_CPG_IC__STXH                    = 0x00000018,
    CPG_PERFWINDOW_SEL_RESERVED1__STXH                 = 0x00000019,
#endif
} CPG_PERFCOUNTWINDOW_SEL;

typedef enum CPG_PERFCOUNT_SEL {
    CPG_PERF_SEL_ALWAYS_COUNT                          = 0x00000000,
    CPG_PERF_SEL_RBIU_FIFO_FULL                        = 0x00000001,
    CPG_PERF_SEL_CP_GRBM_DWORDS_SENT                   = 0x00000004,
    CPG_PERF_SEL_ME_PARSER_BUSY                        = 0x00000005,
    CPG_PERF_SEL_COUNT_TYPE0_PACKETS                   = 0x00000006,
    CPG_PERF_SEL_COUNT_TYPE3_PACKETS                   = 0x00000007,
    CPG_PERF_SEL_CP_GRBM_OUT_OF_CREDITS                = 0x00000009,
    CPG_PERF_SEL_CP_PFP_GRBM_OUT_OF_CREDITS            = 0x0000000a,
    CPG_PERF_SEL_CP_GDS_GRBM_OUT_OF_CREDITS            = 0x0000000b,
    CPG_PERF_SEL_RCIU_STALLED_ON_ME_READ               = 0x0000000c,
    CPG_PERF_SEL_RCIU_STALLED_ON_DMA_READ              = 0x0000000d,
    CPG_PERF_SEL_SSU_STALLED_ON_ACTIVE_CNTX            = 0x0000000e,
    CPG_PERF_SEL_SSU_STALLED_ON_CLEAN_SIGNALS          = 0x0000000f,
    CPG_PERF_SEL_QU_STALLED_ON_EOP_DONE_PULSE          = 0x00000010,
    CPG_PERF_SEL_QU_STALLED_ON_EOP_DONE_WR_CONFIRM     = 0x00000011,
    CPG_PERF_SEL_PFP_STALLED_ON_CSF_READY              = 0x00000012,
    CPG_PERF_SEL_PFP_STALLED_ON_MEQ_READY              = 0x00000013,
    CPG_PERF_SEL_PFP_STALLED_ON_RCIU_READY             = 0x00000014,
    CPG_PERF_SEL_PFP_STALLED_FOR_DATA_FROM_ROQ         = 0x00000015,
    CPG_PERF_SEL_ME_STALLED_FOR_DATA_FROM_PFP          = 0x00000016,
    CPG_PERF_SEL_ME_STALLED_FOR_DATA_FROM_STQ          = 0x00000017,
    CPG_PERF_SEL_ME_STALLED_ON_NO_AVAIL_GFX_CNTX       = 0x00000018,
    CPG_PERF_SEL_ME_STALLED_WRITING_TO_RCIU            = 0x00000019,
    CPG_PERF_SEL_ME_STALLED_WRITING_CONSTANTS          = 0x0000001a,
    CPG_PERF_SEL_ME_STALLED_ON_PARTIAL_FLUSH           = 0x0000001b,
    CPG_PERF_SEL_ME_WAIT_ON_CE_COUNTER                 = 0x0000001c,
    CPG_PERF_SEL_ME_WAIT_ON_AVAIL_BUFFER               = 0x0000001d,
    CPG_PERF_SEL_LOAD_STALLED_ON_SET_COHERENCY         = 0x0000001f,
    CPG_PERF_SEL_DYNAMIC_CLK_VALID                     = 0x00000020,
    CPG_PERF_SEL_REGISTER_CLK_VALID                    = 0x00000021,
    CPG_PERF_SEL_GUS_WRITE_REQUEST_SENT                = 0x00000022,
    CPG_PERF_SEL_GUS_READ_REQUEST_SENT                 = 0x00000023,
    CPG_PERF_SEL_CE_STALL_RAM_DUMP                     = 0x00000024,
    CPG_PERF_SEL_CE_STALL_RAM_WRITE                    = 0x00000025,
    CPG_PERF_SEL_CE_STALL_ON_INC_FIFO                  = 0x00000026,
    CPG_PERF_SEL_CE_STALL_ON_WR_RAM_FIFO               = 0x00000027,
    CPG_PERF_SEL_CE_STALL_ON_DATA_FROM_ROQ             = 0x00000029,
    CPG_PERF_SEL_CE_STALL_ON_CE_BUFFER_FLAG            = 0x0000002a,
    CPG_PERF_SEL_CE_STALL_ON_DE_COUNTER                = 0x0000002b,
    CPG_PERF_SEL_TCIU_STALL_WAIT_ON_FREE               = 0x0000002c,
    CPG_PERF_SEL_TCIU_STALL_WAIT_ON_TAGS               = 0x0000002d,
    CPG_PERF_SEL_UTCL2IU_STALL_WAIT_ON_FREE            = 0x0000002e,
    CPG_PERF_SEL_UTCL2IU_STALL_WAIT_ON_TAGS            = 0x0000002f,
    CPG_PERF_SEL_UTCL1_STALL_ON_TRANSLATION            = 0x00000030,
    CPG_PERF_SEL_TCIU_WRITE_REQUEST_SENT               = 0x00000031,
    CPG_PERF_SEL_TCIU_READ_REQUEST_SENT                = 0x00000032,
    CPG_PERF_SEL_CPG_STAT_BUSY                         = 0x00000033,
    CPG_PERF_SEL_CPG_STAT_IDLE                         = 0x00000034,
    CPG_PERF_SEL_CPG_STAT_STALL                        = 0x00000035,
    CPG_PERF_SEL_CPG_TCIU_BUSY                         = 0x00000036,
    CPG_PERF_SEL_CPG_TCIU_IDLE                         = 0x00000037,
    CPG_PERF_SEL_CPG_TCIU_STALL                        = 0x00000038,
    CPG_PERF_SEL_CPG_UTCL2IU_BUSY                      = 0x00000039,
    CPG_PERF_SEL_CPG_UTCL2IU_IDLE                      = 0x0000003a,
    CPG_PERF_SEL_CPG_UTCL2IU_STALL                     = 0x0000003b,
    CPG_PERF_SEL_CPG_GCRIU_BUSY                        = 0x0000003c,
    CPG_PERF_SEL_CPG_GCRIU_IDLE                        = 0x0000003d,
    CPG_PERF_SEL_CPG_GCRIU_STALL                       = 0x0000003e,
    CPG_PERF_SEL_GCRIU_STALL_WAIT_ON_FREE              = 0x0000003f,
    CPG_PERF_SEL_ALL_GFX_PIPES_BUSY                    = 0x00000040,
    CPG_PERF_SEL_CPG_UTCL2IU_XACK                      = 0x00000041,
    CPG_PERF_SEL_CPG_UTCL2IU_XNACK                     = 0x00000042,
    CPG_PERF_SEL_PFP_STALLED_ON_MEQ_DDID_READY         = 0x00000043,
    CPG_PERF_SEL_PFP_INSTR_CACHE_HIT                   = 0x00000044,
    CPG_PERF_SEL_PFP_INSTR_CACHE_MISS                  = 0x00000045,
    CPG_PERF_SEL_CE_INSTR_CACHE_HIT                    = 0x00000046,
    CPG_PERF_SEL_CE_INSTR_CACHE_MISS                   = 0x00000047,
    CPG_PERF_SEL_ME_INSTR_CACHE_HIT                    = 0x00000048,
    CPG_PERF_SEL_ME_INSTR_CACHE_MISS                   = 0x00000049,
    CPG_PERF_SEL_PFP_PACKET_FILTER_HIT_IB1             = 0x0000004a,
    CPG_PERF_SEL_PFP_PACKET_FILTER_MISS_IB1            = 0x0000004b,
    CPG_PERF_SEL_PFP_PACKET_FILTER_HIT_IB2             = 0x0000004c,
    CPG_PERF_SEL_PFP_PACKET_FILTER_MISS_IB2            = 0x0000004d,
    CPG_PERF_SEL_DMA_BUSY                              = 0x0000004e,
    CPG_PERF_SEL_DMA_STARVED                           = 0x0000004f,
    CPG_PERF_SEL_DMA_STALLED                           = 0x00000050,
    CPG_PERF_SEL_DMA_FETCHER_STALLED_ON_ROQ_FULL       = 0x00000051,
    CPG_PERF_SEL_CSF_RTS_BUT_MIU_NOT_RTR__GFX10        = 0x00000002,
    CPG_PERF_SEL_CSF_ST_BASE_SIZE_FIFO_FULL__GFX10     = 0x00000003,
    CPG_PERF_SEL_CSF_FETCHING_CMD_BUFFERS__GFX10       = 0x00000008,
    CPG_PERF_SEL_SEMAPHORE_BUSY_POLLING_FOR_PASS__GFX10 = 0x0000001e,
    CPG_PERF_SEL_CE_STALL_ON_DATA_FROM_MIU__GFX10      = 0x00000028,
    CPG_PERF_SEL_PFP_PWS_STALLED0__GFX11               = 0x00000052,
    CPG_PERF_SEL_ME_PWS_STALLED0__GFX11                = 0x00000053,
    CPG_PERF_SEL_PFP_VGTDMA_INDR_STRUCT_BYPASS0__GFX11 = 0x00000054,
    CPG_PERF_SEL_PFP_VGTDMA_INDR_STRUCT_NOT_BYPASS0__GFX11 = 0x00000055,
    CPG_PERF_SEL_PFP_VGTDMA_DB_ROQ_DATA_STALL0__GFX11  = 0x00000056,
    CPG_PERF_SEL_PFP_PWS_STALLED1__GFX11               = 0x00000057,
    CPG_PERF_SEL_ME_PWS_STALLED1__GFX11                = 0x00000058,
    CPG_PERF_SEL_PFP_VGTDMA_INDR_STRUCT_BYPASS1__GFX11 = 0x00000059,
    CPG_PERF_SEL_PFP_VGTDMA_INDR_STRUCT_NOT_BYPASS1__GFX11 = 0x0000005a,
    CPG_PERF_SEL_PFP_VGTDMA_DB_ROQ_DATA_STALL1__GFX11  = 0x0000005b,
} CPG_PERFCOUNT_SEL;

constexpr unsigned int MaxCpgPerfcountSelGfx10         = CPG_PERF_SEL_DMA_FETCHER_STALLED_ON_ROQ_FULL;
constexpr unsigned int MaxCpgPerfcountSelGfx11         = CPG_PERF_SEL_PFP_VGTDMA_DB_ROQ_DATA_STALL1__GFX11;

typedef enum CP_PERFMON_ENABLE_MODE {
    CP_PERFMON_ENABLE_MODE_ALWAYS_COUNT                = 0x00000000,
    CP_PERFMON_ENABLE_MODE_RESERVED_1                  = 0x00000001,
    CP_PERFMON_ENABLE_MODE_COUNT_CONTEXT_TRUE          = 0x00000002,
    CP_PERFMON_ENABLE_MODE_COUNT_CONTEXT_FALSE         = 0x00000003,
} CP_PERFMON_ENABLE_MODE;

typedef enum CP_PERFMON_STATE {
    CP_PERFMON_STATE_DISABLE_AND_RESET                 = 0x00000000,
    CP_PERFMON_STATE_START_COUNTING                    = 0x00000001,
    CP_PERFMON_STATE_STOP_COUNTING                     = 0x00000002,
    CP_PERFMON_STATE_RESERVED_3                        = 0x00000003,
    CP_PERFMON_STATE_DISABLE_AND_RESET_PHANTOM         = 0x00000004,
    CP_PERFMON_STATE_COUNT_AND_DUMP_PHANTOM            = 0x00000005,
} CP_PERFMON_STATE;

typedef enum DbMemArbWatermarks {
    TRANSFERRED_64_BYTES                               = 0x00000000,
    TRANSFERRED_128_BYTES                              = 0x00000001,
    TRANSFERRED_256_BYTES                              = 0x00000002,
    TRANSFERRED_512_BYTES                              = 0x00000003,
    TRANSFERRED_1024_BYTES                             = 0x00000004,
    TRANSFERRED_2048_BYTES                             = 0x00000005,
    TRANSFERRED_4096_BYTES                             = 0x00000006,
    TRANSFERRED_8192_BYTES                             = 0x00000007,
} DbMemArbWatermarks;

typedef enum DbPRTFaultBehavior {
    FAULT_ZERO                                         = 0x00000000,
    FAULT_ONE                                          = 0x00000001,
    FAULT_FAIL                                         = 0x00000002,
    FAULT_PASS                                         = 0x00000003,
} DbPRTFaultBehavior;

typedef enum DbPSLControl {
    PSLC_AUTO                                          = 0x00000000,
    PSLC_ON_HANG_ONLY                                  = 0x00000001,
    PSLC_ASAP                                          = 0x00000002,
    PSLC_COUNTDOWN                                     = 0x00000003,
} DbPSLControl;

typedef enum DepthArray {
    ARRAY_2D_ALT_DEPTH                                 = 0x00000000,
    ARRAY_2D_DEPTH                                     = 0x00000001,
} DepthArray;

typedef enum DepthFormat {
    DEPTH_INVALID                                      = 0x00000000,
    DEPTH_16                                           = 0x00000001,
    DEPTH_X8_24                                        = 0x00000002,
    DEPTH_8_24                                         = 0x00000003,
    DEPTH_X8_24_FLOAT                                  = 0x00000004,
    DEPTH_8_24_FLOAT                                   = 0x00000005,
    DEPTH_32_FLOAT                                     = 0x00000006,
    DEPTH_X24_8_32_FLOAT                               = 0x00000007,
} DepthFormat;

typedef enum DF_CS_PERF_SEL {
    DF_CS_PERF_SEL_CS_CSQ_OCC                          = 0x00000000,
    DF_CS_PERF_SEL_CS_CSQ_WAIT                         = 0x00000001,
    DF_CS_PERF_SEL_CS_REQA                             = 0x00000002,
    DF_CS_PERF_SEL_CS_REQB                             = 0x00000003,
    DF_CS_PERF_SEL_CS_REQC                             = 0x00000004,
    DF_CS_PERF_SEL_CS_UMC_RSP                          = 0x00000005,
    DF_CS_PERF_SEL_CS_RSP                              = 0x00000006,
    DF_CS_PERF_SEL_CS_UMC_REQ                          = 0x00000007,
    DF_CS_PERF_SEL_CS_SPECDRAMRD                       = 0x00000008,
    DF_CS_PERF_SEL_CS_PICKER                           = 0x00000009,
    DF_CS_PERF_SEL_CS_ENCRYPTION                       = 0x0000000a,
    DF_CS_PERF_SEL_CS_PRB                              = 0x0000000c,
    DF_CS_PERF_SEL_CS_PRB_ACT0                         = 0x0000000d,
    DF_CS_PERF_SEL_CS_PRB_ACT1                         = 0x0000000e,
    DF_CS_PERF_SEL_CS_BYP                              = 0x0000000f,
    DF_CS_PERF_SEL_CS_PIPE                             = 0x00000010,
    DF_CS_PERF_SEL_CS_BWMON                            = 0x00000011,
    DF_CS_PERF_SEL_CS_RSPBW                            = 0x00000012,
    DF_CS_PERF_SEL_CS_WIDGET                           = 0x00000014,
    DF_CS_PERF_SEL_CS_DAT_CAC                          = 0x00000016,
    DF_CS_PERF_SEL_CS_UMC_SPM                          = 0x0000001a,
    DF_CS_PERF_SEL_CS_MSTR_REQA                        = 0x0000001b,
    DF_CS_PERF_SEL_CS_DMR                              = 0x0000001d,
    DF_CS_PERF_SEL_CS_DATA_BW                          = 0x0000001f,
    DF_CS_PERF_SEL_CS_PRB_FTI_AVG_LAT_TRANS_CNT        = 0x0000002e,
    DF_CS_PERF_SEL_CS_PRB_FTI_AVG_LAT_CYCLE_CNT        = 0x0000002f,
    DF_CS_PERF_SEL_CS_SDP_AVG_LAT_TRANS_CNT            = 0x00000030,
    DF_CS_PERF_SEL_CS_SDP_AVG_LAT_CYCLE_CNT            = 0x00000031,
    DF_CS_PERF_SEL_CS_SDP_LAT_HIST_GT50                = 0x00000032,
    DF_CS_PERF_SEL_CS_SDP_LAT_HIST_GT100               = 0x00000033,
    DF_CS_PERF_SEL_CS_SDP_LAT_HIST_GT150               = 0x00000034,
    DF_CS_PERF_SEL_CS_SDP_LAT_HIST_GT200               = 0x00000035,
    DF_CS_PERF_SEL_CS_SDP_LAT_HIST_GT500               = 0x00000036,
    DF_CS_PERF_SEL_CS_SDP_LAT_HIST_GT1000              = 0x00000037,
    DF_CS_PERF_SEL_CS_FTI_AVG_LAT_TRANS_CNT            = 0x00000038,
    DF_CS_PERF_SEL_CS_FTI_AVG_LAT_CYCLE_CNT            = 0x00000039,
    DF_CS_PERF_SEL_CS_FTI_LAT_HIST_GT50                = 0x0000003a,
    DF_CS_PERF_SEL_CS_FTI_LAT_HIST_GT100               = 0x0000003b,
    DF_CS_PERF_SEL_CS_FTI_LAT_HIST_GT150               = 0x0000003c,
    DF_CS_PERF_SEL_CS_FTI_LAT_HIST_GT200               = 0x0000003d,
    DF_CS_PERF_SEL_CS_FTI_LAT_HIST_GT500               = 0x0000003e,
    DF_CS_PERF_SEL_CS_FTI_LAT_HIST_GT1000              = 0x0000003f,
} DF_CS_PERF_SEL;

constexpr unsigned int MaxDfCsPerfSel                  = DF_CS_PERF_SEL_CS_FTI_LAT_HIST_GT1000;

typedef enum DF_MALL_PERF_SEL {
    DF_MALL_PERF_SEL_ML_MTQ_OCC__GFX11                 = 0x00000000,
    DF_MALL_PERF_SEL_ML_MRS_OCC__GFX11                 = 0x00000001,
    DF_MALL_PERF_SEL_ML_REQ0__GFX11                    = 0x00000002,
    DF_MALL_PERF_SEL_ML_OPTYPES__GFX11                 = 0x00000003,
    DF_MALL_PERF_SEL_ML_CACHE_LKUP_RSP0__GFX11         = 0x00000004,
    DF_MALL_PERF_SEL_ML_CACHE_LKUP_RSP1__GFX11         = 0x00000005,
    DF_MALL_PERF_SEL_ML_CACHE_TAG_PICK_STALL__GFX11    = 0x00000006,
    DF_MALL_PERF_SEL_ML_UMC_PICK_STALL__GFX11          = 0x00000007,
    DF_MALL_PERF_SEL_ML_MISC0__GFX11                   = 0x00000008,
    DF_MALL_PERF_SEL_ML_MISC1__GFX11                   = 0x00000009,
    DF_MALL_PERF_SEL_ML_CACHE_LKUP_RSP2__GFX11         = 0x0000000a,
    DF_MALL_PERF_SEL_ML_CACHE_LKUP_RSP3__GFX11         = 0x0000000b,
    DF_MALL_PERF_SEL_ML_REQ1__GFX11                    = 0x0000000c,
    DF_MALL_PERF_SEL_ML_UMC_SPM__GFX11                 = 0x0000000f,
    DF_MALL_PERF_SEL_MALL_SDP_AVG_LAT_TRANS_CNT__GFX11 = 0x00000030,
    DF_MALL_PERF_SEL_MALL_SDP_AVG_LAT_CYCLE_CNT__GFX11 = 0x00000031,
    DF_MALL_PERF_SEL_MALL_SDP_LAT_HIST_GT50__GFX11     = 0x00000032,
    DF_MALL_PERF_SEL_MALL_SDP_LAT_HIST_GT100__GFX11    = 0x00000033,
    DF_MALL_PERF_SEL_MALL_SDP_LAT_HIST_GT150__GFX11    = 0x00000034,
    DF_MALL_PERF_SEL_MALL_SDP_LAT_HIST_GT200__GFX11    = 0x00000035,
    DF_MALL_PERF_SEL_MALL_SDP_LAT_HIST_GT500__GFX11    = 0x00000036,
    DF_MALL_PERF_SEL_MALL_SDP_LAT_HIST_GT1000__GFX11   = 0x00000037,
    DF_MALL_PERF_SEL_ML_MTQ_OCC__NV2X                  = 0x00000000,
    DF_MALL_PERF_SEL_ML_MRS_OCC__NV2X                  = 0x00000001,
    DF_MALL_PERF_SEL_ML_REQ__NV2X                      = 0x00000002,
    DF_MALL_PERF_SEL_ML_OPTYPES__NV2X                  = 0x00000003,
    DF_MALL_PERF_SEL_ML_CACHE_LKUP_RSP0__NV2X          = 0x00000004,
    DF_MALL_PERF_SEL_ML_CACHE_LKUP_RSP1__NV2X          = 0x00000005,
    DF_MALL_PERF_SEL_ML_CACHE_TAG_PICK_STALL__NV2X     = 0x00000006,
    DF_MALL_PERF_SEL_ML_UMC_PICK_STALL__NV2X           = 0x00000007,
    DF_MALL_PERF_SEL_ML_MISC0__NV2X                    = 0x00000008,
    DF_MALL_PERF_SEL_ML_MISC1__NV2X                    = 0x00000009,
    DF_MALL_PERF_SEL_MALL_SDP_AVG_LAT_TRANS_CNT__NV2X  = 0x00000030,
    DF_MALL_PERF_SEL_MALL_SDP_AVG_LAT_CYCLE_CNT__NV2X  = 0x00000031,
    DF_MALL_PERF_SEL_MALL_SDP_LAT_HIST_GT50__NV2X      = 0x00000032,
    DF_MALL_PERF_SEL_MALL_SDP_LAT_HIST_GT100__NV2X     = 0x00000033,
    DF_MALL_PERF_SEL_MALL_SDP_LAT_HIST_GT150__NV2X     = 0x00000034,
    DF_MALL_PERF_SEL_MALL_SDP_LAT_HIST_GT200__NV2X     = 0x00000035,
    DF_MALL_PERF_SEL_MALL_SDP_LAT_HIST_GT500__NV2X     = 0x00000036,
    DF_MALL_PERF_SEL_MALL_SDP_LAT_HIST_GT1000__NV2X    = 0x00000037,
} DF_MALL_PERF_SEL;

constexpr unsigned int MaxDfMallPerfSelGfx11           = DF_MALL_PERF_SEL_MALL_SDP_LAT_HIST_GT1000__GFX11;
constexpr unsigned int MaxDfMallPerfSelNv2x            = DF_MALL_PERF_SEL_MALL_SDP_LAT_HIST_GT1000__NV2X;

typedef enum ForceControl {
    FORCE_OFF                                          = 0x00000000,
    FORCE_ENABLE                                       = 0x00000001,
    FORCE_DISABLE                                      = 0x00000002,
    FORCE_RESERVED                                     = 0x00000003,
} ForceControl;

typedef enum GCRPerfSel {
    GCR_PERF_SEL_NONE                                  = 0x00000000,
    GCR_PERF_SEL_SDMA0_ALL_REQ                         = 0x00000001,
    GCR_PERF_SEL_SDMA0_GL2_RANGE_REQ                   = 0x00000002,
    GCR_PERF_SEL_SDMA0_GL2_RANGE_LT16K_REQ             = 0x00000003,
    GCR_PERF_SEL_SDMA0_GL2_RANGE_16K_REQ               = 0x00000004,
    GCR_PERF_SEL_SDMA0_GL2_RANGE_GT16K_REQ             = 0x00000005,
    GCR_PERF_SEL_SDMA0_GL2_ALL_REQ                     = 0x00000006,
    GCR_PERF_SEL_SDMA0_GL1_RANGE_REQ                   = 0x00000007,
    GCR_PERF_SEL_SDMA0_GL1_RANGE_LT16K_REQ             = 0x00000008,
    GCR_PERF_SEL_SDMA0_GL1_RANGE_16K_REQ               = 0x00000009,
    GCR_PERF_SEL_SDMA0_GL1_RANGE_GT16K_REQ             = 0x0000000a,
    GCR_PERF_SEL_SDMA0_GL1_ALL_REQ                     = 0x0000000b,
    GCR_PERF_SEL_SDMA0_METADATA_REQ                    = 0x0000000c,
    GCR_PERF_SEL_SDMA0_SQC_DATA_REQ                    = 0x0000000d,
    GCR_PERF_SEL_SDMA0_SQC_INST_REQ                    = 0x0000000e,
    GCR_PERF_SEL_SDMA0_TCP_REQ                         = 0x0000000f,
    GCR_PERF_SEL_CPF_ALL_REQ                           = 0x00000041,
    GCR_PERF_SEL_CPF_GL2_RANGE_REQ                     = 0x00000042,
    GCR_PERF_SEL_CPF_GL2_RANGE_LT16K_REQ               = 0x00000043,
    GCR_PERF_SEL_CPF_GL2_RANGE_16K_REQ                 = 0x00000044,
    GCR_PERF_SEL_CPF_GL2_RANGE_GT16K_REQ               = 0x00000045,
    GCR_PERF_SEL_CPF_GL2_ALL_REQ                       = 0x00000046,
    GCR_PERF_SEL_CPF_GL1_RANGE_REQ                     = 0x00000047,
    GCR_PERF_SEL_CPF_GL1_RANGE_LT16K_REQ               = 0x00000048,
    GCR_PERF_SEL_CPF_GL1_RANGE_16K_REQ                 = 0x00000049,
    GCR_PERF_SEL_CPF_GL1_RANGE_GT16K_REQ               = 0x0000004a,
    GCR_PERF_SEL_CPF_GL1_ALL_REQ                       = 0x0000004b,
    GCR_PERF_SEL_CPF_METADATA_REQ                      = 0x0000004c,
    GCR_PERF_SEL_CPF_SQC_DATA_REQ                      = 0x0000004d,
    GCR_PERF_SEL_CPF_SQC_INST_REQ                      = 0x0000004e,
    GCR_PERF_SEL_CPF_TCP_REQ                           = 0x0000004f,
    GCR_PERF_SEL_VIRT_REQ                              = 0x00000051,
    GCR_PERF_SEL_PHY_REQ                               = 0x00000052,
    GCR_PERF_SEL_TLB_SHOOTDOWN_HEAVY_REQ               = 0x00000053,
    GCR_PERF_SEL_TLB_SHOOTDOWN_LIGHT_REQ               = 0x00000054,
    GCR_PERF_SEL_ALL_REQ                               = 0x00000055,
    GCR_PERF_SEL_CLK_FOR_PHY_OUTSTANDING_REQ           = 0x00000056,
    GCR_PERF_SEL_CLK_FOR_VIRT_OUTSTANDING_REQ          = 0x00000057,
    GCR_PERF_SEL_CLK_FOR_ALL_OUTSTANDING_REQ           = 0x00000058,
    GCR_PERF_SEL_UTCL2_REQ                             = 0x00000059,
    GCR_PERF_SEL_UTCL2_RET                             = 0x0000005a,
    GCR_PERF_SEL_UTCL2_OUT_OF_CREDIT_EVENT             = 0x0000005b,
    GCR_PERF_SEL_UTCL2_INFLIGHT_REQ                    = 0x0000005c,
    GCR_PERF_SEL_UTCL2_FILTERED_RET                    = 0x0000005d,
    GCR_PERF_SEL_RLC_ALL_REQ__APU103                   = 0x00000011,
    GCR_PERF_SEL_RLC_GL2_RANGE_REQ__APU103             = 0x00000012,
    GCR_PERF_SEL_RLC_GL2_RANGE_LT16K_REQ__APU103       = 0x00000013,
    GCR_PERF_SEL_RLC_GL2_RANGE_16K_REQ__APU103         = 0x00000014,
    GCR_PERF_SEL_RLC_GL2_RANGE_GT16K_REQ__APU103       = 0x00000015,
    GCR_PERF_SEL_RLC_GL2_ALL_REQ__APU103               = 0x00000016,
    GCR_PERF_SEL_RLC_GL1_RANGE_REQ__APU103             = 0x00000017,
    GCR_PERF_SEL_RLC_GL1_RANGE_LT16K_REQ__APU103       = 0x00000018,
    GCR_PERF_SEL_RLC_GL1_RANGE_16K_REQ__APU103         = 0x00000019,
    GCR_PERF_SEL_RLC_GL1_RANGE_GT16K_REQ__APU103       = 0x0000001a,
    GCR_PERF_SEL_RLC_GL1_ALL_REQ__APU103               = 0x0000001b,
    GCR_PERF_SEL_RLC_METADATA_REQ__APU103              = 0x0000001c,
    GCR_PERF_SEL_RLC_SQC_DATA_REQ__APU103              = 0x0000001d,
    GCR_PERF_SEL_RLC_SQC_INST_REQ__APU103              = 0x0000001e,
    GCR_PERF_SEL_RLC_TCP_REQ__APU103                   = 0x0000001f,
    GCR_PERF_SEL_RLC_TCP_TLB_SHOOTDOWN_REQ__APU103     = 0x00000020,
    GCR_PERF_SEL_PIO_ALL_REQ__APU103                   = 0x0000005e,
    GCR_PERF_SEL_PIO_GL2_RANGE_REQ__APU103             = 0x0000005f,
    GCR_PERF_SEL_PIO_GL2_RANGE_LT16K_REQ__APU103       = 0x00000060,
    GCR_PERF_SEL_PIO_GL2_RANGE_16K_REQ__APU103         = 0x00000061,
    GCR_PERF_SEL_PIO_GL2_RANGE_GT16K_REQ__APU103       = 0x00000062,
    GCR_PERF_SEL_PIO_GL2_ALL_REQ__APU103               = 0x00000063,
    GCR_PERF_SEL_PIO_GL1_RANGE_REQ__APU103             = 0x00000064,
    GCR_PERF_SEL_PIO_GL1_RANGE_LT16K_REQ__APU103       = 0x00000065,
    GCR_PERF_SEL_PIO_GL1_RANGE_16K_REQ__APU103         = 0x00000066,
    GCR_PERF_SEL_PIO_GL1_RANGE_GT16K_REQ__APU103       = 0x00000067,
    GCR_PERF_SEL_PIO_GL1_ALL_REQ__APU103               = 0x00000068,
    GCR_PERF_SEL_PIO_METADATA_REQ__APU103              = 0x00000069,
    GCR_PERF_SEL_PIO_SQC_DATA_REQ__APU103              = 0x0000006a,
    GCR_PERF_SEL_PIO_SQC_INST_REQ__APU103              = 0x0000006b,
    GCR_PERF_SEL_PIO_TCP_REQ__APU103                   = 0x0000006c,
    GCR_PERF_SEL_PIO_TCP_TLB_SHOOTDOWN_REQ__APU103     = 0x0000006d,
    GCR_PERF_SEL_RLC_ALL_REQ__APU11                    = 0x00000011,
    GCR_PERF_SEL_RLC_GL2_RANGE_REQ__APU11              = 0x00000012,
    GCR_PERF_SEL_RLC_GL2_RANGE_LT16K_REQ__APU11        = 0x00000013,
    GCR_PERF_SEL_RLC_GL2_RANGE_16K_REQ__APU11          = 0x00000014,
    GCR_PERF_SEL_RLC_GL2_RANGE_GT16K_REQ__APU11        = 0x00000015,
    GCR_PERF_SEL_RLC_GL2_ALL_REQ__APU11                = 0x00000016,
    GCR_PERF_SEL_RLC_GL1_RANGE_REQ__APU11              = 0x00000017,
    GCR_PERF_SEL_RLC_GL1_RANGE_LT16K_REQ__APU11        = 0x00000018,
    GCR_PERF_SEL_RLC_GL1_RANGE_16K_REQ__APU11          = 0x00000019,
    GCR_PERF_SEL_RLC_GL1_RANGE_GT16K_REQ__APU11        = 0x0000001a,
    GCR_PERF_SEL_RLC_GL1_ALL_REQ__APU11                = 0x0000001b,
    GCR_PERF_SEL_RLC_METADATA_REQ__APU11               = 0x0000001c,
    GCR_PERF_SEL_RLC_SQC_DATA_REQ__APU11               = 0x0000001d,
    GCR_PERF_SEL_RLC_SQC_INST_REQ__APU11               = 0x0000001e,
    GCR_PERF_SEL_RLC_TCP_REQ__APU11                    = 0x0000001f,
    GCR_PERF_SEL_RLC_GL1_TLB_SHOOTDOWN_REQ__APU11      = 0x00000020,
    GCR_PERF_SEL_PM_ALL_REQ__APU11                     = 0x0000005e,
    GCR_PERF_SEL_PM_GL2_RANGE_REQ__APU11               = 0x0000005f,
    GCR_PERF_SEL_PM_GL2_RANGE_LT16K_REQ__APU11         = 0x00000060,
    GCR_PERF_SEL_PM_GL2_RANGE_16K_REQ__APU11           = 0x00000061,
    GCR_PERF_SEL_PM_GL2_RANGE_GT16K_REQ__APU11         = 0x00000062,
    GCR_PERF_SEL_PM_GL2_ALL_REQ__APU11                 = 0x00000063,
    GCR_PERF_SEL_PM_GL1_RANGE_REQ__APU11               = 0x00000064,
    GCR_PERF_SEL_PM_GL1_RANGE_LT16K_REQ__APU11         = 0x00000065,
    GCR_PERF_SEL_PM_GL1_RANGE_16K_REQ__APU11           = 0x00000066,
    GCR_PERF_SEL_PM_GL1_RANGE_GT16K_REQ__APU11         = 0x00000067,
    GCR_PERF_SEL_PM_GL1_ALL_REQ__APU11                 = 0x00000068,
    GCR_PERF_SEL_PM_METADATA_REQ__APU11                = 0x00000069,
    GCR_PERF_SEL_PM_SQC_DATA_REQ__APU11                = 0x0000006a,
    GCR_PERF_SEL_PM_SQC_INST_REQ__APU11                = 0x0000006b,
    GCR_PERF_SEL_PM_TCP_REQ__APU11                     = 0x0000006c,
    GCR_PERF_SEL_PM_GL1_TLB_SHOOTDOWN_REQ__APU11       = 0x0000006d,
    GCR_PERF_SEL_PIO_ALL_REQ__APU11                    = 0x0000006e,
    GCR_PERF_SEL_PIO_GL2_RANGE_REQ__APU11              = 0x0000006f,
    GCR_PERF_SEL_PIO_GL2_RANGE_LT16K_REQ__APU11        = 0x00000070,
    GCR_PERF_SEL_PIO_GL2_RANGE_16K_REQ__APU11          = 0x00000071,
    GCR_PERF_SEL_PIO_GL2_RANGE_GT16K_REQ__APU11        = 0x00000072,
    GCR_PERF_SEL_PIO_GL2_ALL_REQ__APU11                = 0x00000073,
    GCR_PERF_SEL_PIO_GL1_RANGE_REQ__APU11              = 0x00000074,
    GCR_PERF_SEL_PIO_GL1_RANGE_LT16K_REQ__APU11        = 0x00000075,
    GCR_PERF_SEL_PIO_GL1_RANGE_16K_REQ__APU11          = 0x00000076,
    GCR_PERF_SEL_PIO_GL1_RANGE_GT16K_REQ__APU11        = 0x00000077,
    GCR_PERF_SEL_PIO_GL1_ALL_REQ__APU11                = 0x00000078,
    GCR_PERF_SEL_PIO_METADATA_REQ__APU11               = 0x00000079,
    GCR_PERF_SEL_PIO_SQC_DATA_REQ__APU11               = 0x0000007a,
    GCR_PERF_SEL_PIO_SQC_INST_REQ__APU11               = 0x0000007b,
    GCR_PERF_SEL_PIO_TCP_REQ__APU11                    = 0x0000007c,
    GCR_PERF_SEL_PIO_GL1_TLB_SHOOTDOWN_REQ__APU11      = 0x0000007d,
    GCR_PERF_SEL_SDMA0_TCP_TLB_SHOOTDOWN_REQ__GFX10    = 0x00000010,
    GCR_PERF_SEL_CPF_TCP_TLB_SHOOTDOWN_REQ__GFX10      = 0x00000050,
    GCR_PERF_SEL_SDMA1_ALL_REQ__GFX101                 = 0x00000011,
    GCR_PERF_SEL_SDMA1_GL2_RANGE_REQ__GFX101           = 0x00000012,
    GCR_PERF_SEL_SDMA1_GL2_RANGE_LT16K_REQ__GFX101     = 0x00000013,
    GCR_PERF_SEL_SDMA1_GL2_RANGE_16K_REQ__GFX101       = 0x00000014,
    GCR_PERF_SEL_SDMA1_GL2_RANGE_GT16K_REQ__GFX101     = 0x00000015,
    GCR_PERF_SEL_SDMA1_GL2_ALL_REQ__GFX101             = 0x00000016,
    GCR_PERF_SEL_SDMA1_GL1_RANGE_REQ__GFX101           = 0x00000017,
    GCR_PERF_SEL_SDMA1_GL1_RANGE_LT16K_REQ__GFX101     = 0x00000018,
    GCR_PERF_SEL_SDMA1_GL1_RANGE_16K_REQ__GFX101       = 0x00000019,
    GCR_PERF_SEL_SDMA1_GL1_RANGE_GT16K_REQ__GFX101     = 0x0000001a,
    GCR_PERF_SEL_SDMA1_GL1_ALL_REQ__GFX101             = 0x0000001b,
    GCR_PERF_SEL_SDMA1_METADATA_REQ__GFX101            = 0x0000001c,
    GCR_PERF_SEL_SDMA1_SQC_DATA_REQ__GFX101            = 0x0000001d,
    GCR_PERF_SEL_SDMA1_SQC_INST_REQ__GFX101            = 0x0000001e,
    GCR_PERF_SEL_SDMA1_TCP_REQ__GFX101                 = 0x0000001f,
    GCR_PERF_SEL_SDMA1_TCP_TLB_SHOOTDOWN_REQ__GFX101   = 0x00000020,
    GCR_PERF_SEL_CPG_ALL_REQ__GFX101                   = 0x00000021,
    GCR_PERF_SEL_CPG_GL2_RANGE_REQ__GFX101             = 0x00000022,
    GCR_PERF_SEL_CPG_GL2_RANGE_LT16K_REQ__GFX101       = 0x00000023,
    GCR_PERF_SEL_CPG_GL2_RANGE_16K_REQ__GFX101         = 0x00000024,
    GCR_PERF_SEL_CPG_GL2_RANGE_GT16K_REQ__GFX101       = 0x00000025,
    GCR_PERF_SEL_CPG_GL2_ALL_REQ__GFX101               = 0x00000026,
    GCR_PERF_SEL_CPG_GL1_RANGE_REQ__GFX101             = 0x00000027,
    GCR_PERF_SEL_CPG_GL1_RANGE_LT16K_REQ__GFX101       = 0x00000028,
    GCR_PERF_SEL_CPG_GL1_RANGE_16K_REQ__GFX101         = 0x00000029,
    GCR_PERF_SEL_CPG_GL1_RANGE_GT16K_REQ__GFX101       = 0x0000002a,
    GCR_PERF_SEL_CPG_GL1_ALL_REQ__GFX101               = 0x0000002b,
    GCR_PERF_SEL_CPG_METADATA_REQ__GFX101              = 0x0000002c,
    GCR_PERF_SEL_CPG_SQC_DATA_REQ__GFX101              = 0x0000002d,
    GCR_PERF_SEL_CPG_SQC_INST_REQ__GFX101              = 0x0000002e,
    GCR_PERF_SEL_CPG_TCP_REQ__GFX101                   = 0x0000002f,
    GCR_PERF_SEL_CPG_TCP_TLB_SHOOTDOWN_REQ__GFX101     = 0x00000030,
    GCR_PERF_SEL_CPC_ALL_REQ__GFX101                   = 0x00000031,
    GCR_PERF_SEL_CPC_GL2_RANGE_REQ__GFX101             = 0x00000032,
    GCR_PERF_SEL_CPC_GL2_RANGE_LT16K_REQ__GFX101       = 0x00000033,
    GCR_PERF_SEL_CPC_GL2_RANGE_16K_REQ__GFX101         = 0x00000034,
    GCR_PERF_SEL_CPC_GL2_RANGE_GT16K_REQ__GFX101       = 0x00000035,
    GCR_PERF_SEL_CPC_GL2_ALL_REQ__GFX101               = 0x00000036,
    GCR_PERF_SEL_CPC_GL1_RANGE_REQ__GFX101             = 0x00000037,
    GCR_PERF_SEL_CPC_GL1_RANGE_LT16K_REQ__GFX101       = 0x00000038,
    GCR_PERF_SEL_CPC_GL1_RANGE_16K_REQ__GFX101         = 0x00000039,
    GCR_PERF_SEL_CPC_GL1_RANGE_GT16K_REQ__GFX101       = 0x0000003a,
    GCR_PERF_SEL_CPC_GL1_ALL_REQ__GFX101               = 0x0000003b,
    GCR_PERF_SEL_CPC_METADATA_REQ__GFX101              = 0x0000003c,
    GCR_PERF_SEL_CPC_SQC_DATA_REQ__GFX101              = 0x0000003d,
    GCR_PERF_SEL_CPC_SQC_INST_REQ__GFX101              = 0x0000003e,
    GCR_PERF_SEL_CPC_TCP_REQ__GFX101                   = 0x0000003f,
    GCR_PERF_SEL_CPC_TCP_TLB_SHOOTDOWN_REQ__GFX101     = 0x00000040,
    GCR_PERF_SEL_CPC_ALL_REQ__GFX103PLUS               = 0x00000021,
    GCR_PERF_SEL_CPC_GL2_RANGE_REQ__GFX103PLUS         = 0x00000022,
    GCR_PERF_SEL_CPC_GL2_RANGE_LT16K_REQ__GFX103PLUS   = 0x00000023,
    GCR_PERF_SEL_CPC_GL2_RANGE_16K_REQ__GFX103PLUS     = 0x00000024,
    GCR_PERF_SEL_CPC_GL2_RANGE_GT16K_REQ__GFX103PLUS   = 0x00000025,
    GCR_PERF_SEL_CPC_GL2_ALL_REQ__GFX103PLUS           = 0x00000026,
    GCR_PERF_SEL_CPC_GL1_RANGE_REQ__GFX103PLUS         = 0x00000027,
    GCR_PERF_SEL_CPC_GL1_RANGE_LT16K_REQ__GFX103PLUS   = 0x00000028,
    GCR_PERF_SEL_CPC_GL1_RANGE_16K_REQ__GFX103PLUS     = 0x00000029,
    GCR_PERF_SEL_CPC_GL1_RANGE_GT16K_REQ__GFX103PLUS   = 0x0000002a,
    GCR_PERF_SEL_CPC_GL1_ALL_REQ__GFX103PLUS           = 0x0000002b,
    GCR_PERF_SEL_CPC_METADATA_REQ__GFX103PLUS          = 0x0000002c,
    GCR_PERF_SEL_CPC_SQC_DATA_REQ__GFX103PLUS          = 0x0000002d,
    GCR_PERF_SEL_CPC_SQC_INST_REQ__GFX103PLUS          = 0x0000002e,
    GCR_PERF_SEL_CPC_TCP_REQ__GFX103PLUS               = 0x0000002f,
    GCR_PERF_SEL_CPG_ALL_REQ__GFX103PLUS               = 0x00000031,
    GCR_PERF_SEL_CPG_GL2_RANGE_REQ__GFX103PLUS         = 0x00000032,
    GCR_PERF_SEL_CPG_GL2_RANGE_LT16K_REQ__GFX103PLUS   = 0x00000033,
    GCR_PERF_SEL_CPG_GL2_RANGE_16K_REQ__GFX103PLUS     = 0x00000034,
    GCR_PERF_SEL_CPG_GL2_RANGE_GT16K_REQ__GFX103PLUS   = 0x00000035,
    GCR_PERF_SEL_CPG_GL2_ALL_REQ__GFX103PLUS           = 0x00000036,
    GCR_PERF_SEL_CPG_GL1_RANGE_REQ__GFX103PLUS         = 0x00000037,
    GCR_PERF_SEL_CPG_GL1_RANGE_LT16K_REQ__GFX103PLUS   = 0x00000038,
    GCR_PERF_SEL_CPG_GL1_RANGE_16K_REQ__GFX103PLUS     = 0x00000039,
    GCR_PERF_SEL_CPG_GL1_RANGE_GT16K_REQ__GFX103PLUS   = 0x0000003a,
    GCR_PERF_SEL_CPG_GL1_ALL_REQ__GFX103PLUS           = 0x0000003b,
    GCR_PERF_SEL_CPG_METADATA_REQ__GFX103PLUS          = 0x0000003c,
    GCR_PERF_SEL_CPG_SQC_DATA_REQ__GFX103PLUS          = 0x0000003d,
    GCR_PERF_SEL_CPG_SQC_INST_REQ__GFX103PLUS          = 0x0000003e,
    GCR_PERF_SEL_CPG_TCP_REQ__GFX103PLUS               = 0x0000003f,
    GCR_PERF_SEL_CPC_TCP_TLB_SHOOTDOWN_REQ__GFX10VRS   = 0x00000030,
    GCR_PERF_SEL_CPG_TCP_TLB_SHOOTDOWN_REQ__GFX10VRS   = 0x00000040,
    GCR_PERF_SEL_SDMA0_GL1_TLB_SHOOTDOWN_REQ__GFX11    = 0x00000010,
    GCR_PERF_SEL_CPC_GL1_TLB_SHOOTDOWN_REQ__GFX11      = 0x00000030,
    GCR_PERF_SEL_CPG_GL1_TLB_SHOOTDOWN_REQ__GFX11      = 0x00000040,
    GCR_PERF_SEL_CPF_GL1_TLB_SHOOTDOWN_REQ__GFX11      = 0x00000050,
    GCR_PERF_SEL_SDMA2_ALL_REQ__NV21                   = 0x0000005e,
    GCR_PERF_SEL_SDMA2_GL2_RANGE_REQ__NV21             = 0x0000005f,
    GCR_PERF_SEL_SDMA2_GL2_RANGE_LT16K_REQ__NV21       = 0x00000060,
    GCR_PERF_SEL_SDMA2_GL2_RANGE_16K_REQ__NV21         = 0x00000061,
    GCR_PERF_SEL_SDMA2_GL2_RANGE_GT16K_REQ__NV21       = 0x00000062,
    GCR_PERF_SEL_SDMA2_GL2_ALL_REQ__NV21               = 0x00000063,
    GCR_PERF_SEL_SDMA2_GL1_RANGE_REQ__NV21             = 0x00000064,
    GCR_PERF_SEL_SDMA2_GL1_RANGE_LT16K_REQ__NV21       = 0x00000065,
    GCR_PERF_SEL_SDMA2_GL1_RANGE_16K_REQ__NV21         = 0x00000066,
    GCR_PERF_SEL_SDMA2_GL1_RANGE_GT16K_REQ__NV21       = 0x00000067,
    GCR_PERF_SEL_SDMA2_GL1_ALL_REQ__NV21               = 0x00000068,
    GCR_PERF_SEL_SDMA2_METADATA_REQ__NV21              = 0x00000069,
    GCR_PERF_SEL_SDMA2_SQC_DATA_REQ__NV21              = 0x0000006a,
    GCR_PERF_SEL_SDMA2_SQC_INST_REQ__NV21              = 0x0000006b,
    GCR_PERF_SEL_SDMA2_TCP_REQ__NV21                   = 0x0000006c,
    GCR_PERF_SEL_SDMA2_TCP_TLB_SHOOTDOWN_REQ__NV21     = 0x0000006d,
    GCR_PERF_SEL_SDMA3_ALL_REQ__NV21                   = 0x0000006e,
    GCR_PERF_SEL_SDMA3_GL2_RANGE_REQ__NV21             = 0x0000006f,
    GCR_PERF_SEL_SDMA3_GL2_RANGE_LT16K_REQ__NV21       = 0x00000070,
    GCR_PERF_SEL_SDMA3_GL2_RANGE_16K_REQ__NV21         = 0x00000071,
    GCR_PERF_SEL_SDMA3_GL2_RANGE_GT16K_REQ__NV21       = 0x00000072,
    GCR_PERF_SEL_SDMA3_GL2_ALL_REQ__NV21               = 0x00000073,
    GCR_PERF_SEL_SDMA3_GL1_RANGE_REQ__NV21             = 0x00000074,
    GCR_PERF_SEL_SDMA3_GL1_RANGE_LT16K_REQ__NV21       = 0x00000075,
    GCR_PERF_SEL_SDMA3_GL1_RANGE_16K_REQ__NV21         = 0x00000076,
    GCR_PERF_SEL_SDMA3_GL1_RANGE_GT16K_REQ__NV21       = 0x00000077,
    GCR_PERF_SEL_SDMA3_GL1_ALL_REQ__NV21               = 0x00000078,
    GCR_PERF_SEL_SDMA3_METADATA_REQ__NV21              = 0x00000079,
    GCR_PERF_SEL_SDMA3_SQC_DATA_REQ__NV21              = 0x0000007a,
    GCR_PERF_SEL_SDMA3_SQC_INST_REQ__NV21              = 0x0000007b,
    GCR_PERF_SEL_SDMA3_TCP_REQ__NV21                   = 0x0000007c,
    GCR_PERF_SEL_SDMA3_TCP_TLB_SHOOTDOWN_REQ__NV21     = 0x0000007d,
    GCR_PERF_SEL_PIO_ALL_REQ__NV21                     = 0x0000007e,
    GCR_PERF_SEL_PIO_GL2_RANGE_REQ__NV21               = 0x0000007f,
    GCR_PERF_SEL_PIO_GL2_RANGE_LT16K_REQ__NV21         = 0x00000080,
    GCR_PERF_SEL_PIO_GL2_RANGE_16K_REQ__NV21           = 0x00000081,
    GCR_PERF_SEL_PIO_GL2_RANGE_GT16K_REQ__NV21         = 0x00000082,
    GCR_PERF_SEL_PIO_GL2_ALL_REQ__NV21                 = 0x00000083,
    GCR_PERF_SEL_PIO_GL1_RANGE_REQ__NV21               = 0x00000084,
    GCR_PERF_SEL_PIO_GL1_RANGE_LT16K_REQ__NV21         = 0x00000085,
    GCR_PERF_SEL_PIO_GL1_RANGE_16K_REQ__NV21           = 0x00000086,
    GCR_PERF_SEL_PIO_GL1_RANGE_GT16K_REQ__NV21         = 0x00000087,
    GCR_PERF_SEL_PIO_GL1_ALL_REQ__NV21                 = 0x00000088,
    GCR_PERF_SEL_PIO_METADATA_REQ__NV21                = 0x00000089,
    GCR_PERF_SEL_PIO_SQC_DATA_REQ__NV21                = 0x0000008a,
    GCR_PERF_SEL_PIO_SQC_INST_REQ__NV21                = 0x0000008b,
    GCR_PERF_SEL_PIO_TCP_REQ__NV21                     = 0x0000008c,
    GCR_PERF_SEL_PIO_TCP_TLB_SHOOTDOWN_REQ__NV21       = 0x0000008d,
    GCR_PERF_SEL_PIO_ALL_REQ__NV22                     = 0x0000005e,
    GCR_PERF_SEL_PIO_GL2_RANGE_REQ__NV22               = 0x0000005f,
    GCR_PERF_SEL_PIO_GL2_RANGE_LT16K_REQ__NV22         = 0x00000060,
    GCR_PERF_SEL_PIO_GL2_RANGE_16K_REQ__NV22           = 0x00000061,
    GCR_PERF_SEL_PIO_GL2_RANGE_GT16K_REQ__NV22         = 0x00000062,
    GCR_PERF_SEL_PIO_GL2_ALL_REQ__NV22                 = 0x00000063,
    GCR_PERF_SEL_PIO_GL1_RANGE_REQ__NV22               = 0x00000064,
    GCR_PERF_SEL_PIO_GL1_RANGE_LT16K_REQ__NV22         = 0x00000065,
    GCR_PERF_SEL_PIO_GL1_RANGE_16K_REQ__NV22           = 0x00000066,
    GCR_PERF_SEL_PIO_GL1_RANGE_GT16K_REQ__NV22         = 0x00000067,
    GCR_PERF_SEL_PIO_GL1_ALL_REQ__NV22                 = 0x00000068,
    GCR_PERF_SEL_PIO_METADATA_REQ__NV22                = 0x00000069,
    GCR_PERF_SEL_PIO_SQC_DATA_REQ__NV22                = 0x0000006a,
    GCR_PERF_SEL_PIO_SQC_INST_REQ__NV22                = 0x0000006b,
    GCR_PERF_SEL_PIO_TCP_REQ__NV22                     = 0x0000006c,
    GCR_PERF_SEL_PIO_TCP_TLB_SHOOTDOWN_REQ__NV22       = 0x0000006d,
    GCR_PERF_SEL_PIO_ALL_REQ__NV23                     = 0x0000005e,
    GCR_PERF_SEL_PIO_GL2_RANGE_REQ__NV23               = 0x0000005f,
    GCR_PERF_SEL_PIO_GL2_RANGE_LT16K_REQ__NV23         = 0x00000060,
    GCR_PERF_SEL_PIO_GL2_RANGE_16K_REQ__NV23           = 0x00000061,
    GCR_PERF_SEL_PIO_GL2_RANGE_GT16K_REQ__NV23         = 0x00000062,
    GCR_PERF_SEL_PIO_GL2_ALL_REQ__NV23                 = 0x00000063,
    GCR_PERF_SEL_PIO_GL1_RANGE_REQ__NV23               = 0x00000064,
    GCR_PERF_SEL_PIO_GL1_RANGE_LT16K_REQ__NV23         = 0x00000065,
    GCR_PERF_SEL_PIO_GL1_RANGE_16K_REQ__NV23           = 0x00000066,
    GCR_PERF_SEL_PIO_GL1_RANGE_GT16K_REQ__NV23         = 0x00000067,
    GCR_PERF_SEL_PIO_GL1_ALL_REQ__NV23                 = 0x00000068,
    GCR_PERF_SEL_PIO_METADATA_REQ__NV23                = 0x00000069,
    GCR_PERF_SEL_PIO_SQC_DATA_REQ__NV23                = 0x0000006a,
    GCR_PERF_SEL_PIO_SQC_INST_REQ__NV23                = 0x0000006b,
    GCR_PERF_SEL_PIO_TCP_REQ__NV23                     = 0x0000006c,
    GCR_PERF_SEL_PIO_TCP_TLB_SHOOTDOWN_REQ__NV23       = 0x0000006d,
    GCR_PERF_SEL_PIO_ALL_REQ__NV24                     = 0x0000005e,
    GCR_PERF_SEL_PIO_GL2_RANGE_REQ__NV24               = 0x0000005f,
    GCR_PERF_SEL_PIO_GL2_RANGE_LT16K_REQ__NV24         = 0x00000060,
    GCR_PERF_SEL_PIO_GL2_RANGE_16K_REQ__NV24           = 0x00000061,
    GCR_PERF_SEL_PIO_GL2_RANGE_GT16K_REQ__NV24         = 0x00000062,
    GCR_PERF_SEL_PIO_GL2_ALL_REQ__NV24                 = 0x00000063,
    GCR_PERF_SEL_PIO_GL1_RANGE_REQ__NV24               = 0x00000064,
    GCR_PERF_SEL_PIO_GL1_RANGE_LT16K_REQ__NV24         = 0x00000065,
    GCR_PERF_SEL_PIO_GL1_RANGE_16K_REQ__NV24           = 0x00000066,
    GCR_PERF_SEL_PIO_GL1_RANGE_GT16K_REQ__NV24         = 0x00000067,
    GCR_PERF_SEL_PIO_GL1_ALL_REQ__NV24                 = 0x00000068,
    GCR_PERF_SEL_PIO_METADATA_REQ__NV24                = 0x00000069,
    GCR_PERF_SEL_PIO_SQC_DATA_REQ__NV24                = 0x0000006a,
    GCR_PERF_SEL_PIO_SQC_INST_REQ__NV24                = 0x0000006b,
    GCR_PERF_SEL_PIO_TCP_REQ__NV24                     = 0x0000006c,
    GCR_PERF_SEL_PIO_TCP_TLB_SHOOTDOWN_REQ__NV24       = 0x0000006d,
    GCR_PERF_SEL_SDMA1_ALL_REQ__NV2X                   = 0x00000011,
    GCR_PERF_SEL_SDMA1_GL2_RANGE_REQ__NV2X             = 0x00000012,
    GCR_PERF_SEL_SDMA1_GL2_RANGE_LT16K_REQ__NV2X       = 0x00000013,
    GCR_PERF_SEL_SDMA1_GL2_RANGE_16K_REQ__NV2X         = 0x00000014,
    GCR_PERF_SEL_SDMA1_GL2_RANGE_GT16K_REQ__NV2X       = 0x00000015,
    GCR_PERF_SEL_SDMA1_GL2_ALL_REQ__NV2X               = 0x00000016,
    GCR_PERF_SEL_SDMA1_GL1_RANGE_REQ__NV2X             = 0x00000017,
    GCR_PERF_SEL_SDMA1_GL1_RANGE_LT16K_REQ__NV2X       = 0x00000018,
    GCR_PERF_SEL_SDMA1_GL1_RANGE_16K_REQ__NV2X         = 0x00000019,
    GCR_PERF_SEL_SDMA1_GL1_RANGE_GT16K_REQ__NV2X       = 0x0000001a,
    GCR_PERF_SEL_SDMA1_GL1_ALL_REQ__NV2X               = 0x0000001b,
    GCR_PERF_SEL_SDMA1_METADATA_REQ__NV2X              = 0x0000001c,
    GCR_PERF_SEL_SDMA1_SQC_DATA_REQ__NV2X              = 0x0000001d,
    GCR_PERF_SEL_SDMA1_SQC_INST_REQ__NV2X              = 0x0000001e,
    GCR_PERF_SEL_SDMA1_TCP_REQ__NV2X                   = 0x0000001f,
    GCR_PERF_SEL_SDMA1_TCP_TLB_SHOOTDOWN_REQ__NV2X     = 0x00000020,
    GCR_PERF_SEL_SDMA1_ALL_REQ__NV3X                   = 0x00000011,
    GCR_PERF_SEL_SDMA1_GL2_RANGE_REQ__NV3X             = 0x00000012,
    GCR_PERF_SEL_SDMA1_GL2_RANGE_LT16K_REQ__NV3X       = 0x00000013,
    GCR_PERF_SEL_SDMA1_GL2_RANGE_16K_REQ__NV3X         = 0x00000014,
    GCR_PERF_SEL_SDMA1_GL2_RANGE_GT16K_REQ__NV3X       = 0x00000015,
    GCR_PERF_SEL_SDMA1_GL2_ALL_REQ__NV3X               = 0x00000016,
    GCR_PERF_SEL_SDMA1_GL1_RANGE_REQ__NV3X             = 0x00000017,
    GCR_PERF_SEL_SDMA1_GL1_RANGE_LT16K_REQ__NV3X       = 0x00000018,
    GCR_PERF_SEL_SDMA1_GL1_RANGE_16K_REQ__NV3X         = 0x00000019,
    GCR_PERF_SEL_SDMA1_GL1_RANGE_GT16K_REQ__NV3X       = 0x0000001a,
    GCR_PERF_SEL_SDMA1_GL1_ALL_REQ__NV3X               = 0x0000001b,
    GCR_PERF_SEL_SDMA1_METADATA_REQ__NV3X              = 0x0000001c,
    GCR_PERF_SEL_SDMA1_SQC_DATA_REQ__NV3X              = 0x0000001d,
    GCR_PERF_SEL_SDMA1_SQC_INST_REQ__NV3X              = 0x0000001e,
    GCR_PERF_SEL_SDMA1_TCP_REQ__NV3X                   = 0x0000001f,
    GCR_PERF_SEL_SDMA1_GL1_TLB_SHOOTDOWN_REQ__NV3X     = 0x00000020,
    GCR_PERF_SEL_PMM_ABIT_NUM_FLUSH__NV3X              = 0x0000005e,
    GCR_PERF_SEL_PMM_ABIT_FLUSH_ONGOING__NV3X          = 0x0000005f,
    GCR_PERF_SEL_PMM_NUM_INTERRUPT__NV3X               = 0x00000060,
    GCR_PERF_SEL_PMM_STALL_PMM_IH_CREDITS__NV3X        = 0x00000061,
    GCR_PERF_SEL_PMM_INTERRUPT_READY_TO_SEND__NV3X     = 0x00000062,
    GCR_PERF_SEL_PMM_ABIT_TIMER_FLUSH__NV3X            = 0x00000063,
    GCR_PERF_SEL_PMM_ABIT_FORCE_FLUSH__NV3X            = 0x00000064,
    GCR_PERF_SEL_PMM_ABIT_FLUSH_INTERRUPT__NV3X        = 0x00000065,
    GCR_PERF_SEL_PMM_ALOG_INTERRUPT__NV3X              = 0x00000066,
    GCR_PERF_SEL_PMM_MAM_FLUSH_REQ__NV3X               = 0x00000067,
    GCR_PERF_SEL_PMM_MAM_FLUSH_RESP__NV3X              = 0x00000068,
    GCR_PERF_SEL_PMM_RLC_CGCG_REQ__NV3X                = 0x00000069,
    GCR_PERF_SEL_PMM_RLC_CGCG_RESP__NV3X               = 0x0000006a,
    GCR_PERF_SEL_RLC_ALL_REQ__NV3X                     = 0x0000006b,
    GCR_PERF_SEL_RLC_GL2_RANGE_REQ__NV3X               = 0x0000006c,
    GCR_PERF_SEL_RLC_GL2_RANGE_LT16K_REQ__NV3X         = 0x0000006d,
    GCR_PERF_SEL_RLC_GL2_RANGE_16K_REQ__NV3X           = 0x0000006e,
    GCR_PERF_SEL_RLC_GL2_RANGE_GT16K_REQ__NV3X         = 0x0000006f,
    GCR_PERF_SEL_RLC_GL2_ALL_REQ__NV3X                 = 0x00000070,
    GCR_PERF_SEL_RLC_GL1_RANGE_REQ__NV3X               = 0x00000071,
    GCR_PERF_SEL_RLC_GL1_RANGE_LT16K_REQ__NV3X         = 0x00000072,
    GCR_PERF_SEL_RLC_GL1_RANGE_16K_REQ__NV3X           = 0x00000073,
    GCR_PERF_SEL_RLC_GL1_RANGE_GT16K_REQ__NV3X         = 0x00000074,
    GCR_PERF_SEL_RLC_GL1_ALL_REQ__NV3X                 = 0x00000075,
    GCR_PERF_SEL_RLC_METADATA_REQ__NV3X                = 0x00000076,
    GCR_PERF_SEL_RLC_SQC_DATA_REQ__NV3X                = 0x00000077,
    GCR_PERF_SEL_RLC_SQC_INST_REQ__NV3X                = 0x00000078,
    GCR_PERF_SEL_RLC_TCP_REQ__NV3X                     = 0x00000079,
    GCR_PERF_SEL_RLC_GL1_TLB_SHOOTDOWN_REQ__NV3X       = 0x0000007a,
    GCR_PERF_SEL_PM_ALL_REQ__NV3X                      = 0x0000007b,
    GCR_PERF_SEL_PM_GL2_RANGE_REQ__NV3X                = 0x0000007c,
    GCR_PERF_SEL_PM_GL2_RANGE_LT16K_REQ__NV3X          = 0x0000007d,
    GCR_PERF_SEL_PM_GL2_RANGE_16K_REQ__NV3X            = 0x0000007e,
    GCR_PERF_SEL_PM_GL2_RANGE_GT16K_REQ__NV3X          = 0x0000007f,
    GCR_PERF_SEL_PM_GL2_ALL_REQ__NV3X                  = 0x00000080,
    GCR_PERF_SEL_PM_GL1_RANGE_REQ__NV3X                = 0x00000081,
    GCR_PERF_SEL_PM_GL1_RANGE_LT16K_REQ__NV3X          = 0x00000082,
    GCR_PERF_SEL_PM_GL1_RANGE_16K_REQ__NV3X            = 0x00000083,
    GCR_PERF_SEL_PM_GL1_RANGE_GT16K_REQ__NV3X          = 0x00000084,
    GCR_PERF_SEL_PM_GL1_ALL_REQ__NV3X                  = 0x00000085,
    GCR_PERF_SEL_PM_METADATA_REQ__NV3X                 = 0x00000086,
    GCR_PERF_SEL_PM_SQC_DATA_REQ__NV3X                 = 0x00000087,
    GCR_PERF_SEL_PM_SQC_INST_REQ__NV3X                 = 0x00000088,
    GCR_PERF_SEL_PM_TCP_REQ__NV3X                      = 0x00000089,
    GCR_PERF_SEL_PM_GL1_TLB_SHOOTDOWN_REQ__NV3X        = 0x0000008a,
    GCR_PERF_SEL_PIO_ALL_REQ__NV3X                     = 0x0000008b,
    GCR_PERF_SEL_PIO_GL2_RANGE_REQ__NV3X               = 0x0000008c,
    GCR_PERF_SEL_PIO_GL2_RANGE_LT16K_REQ__NV3X         = 0x0000008d,
    GCR_PERF_SEL_PIO_GL2_RANGE_16K_REQ__NV3X           = 0x0000008e,
    GCR_PERF_SEL_PIO_GL2_RANGE_GT16K_REQ__NV3X         = 0x0000008f,
    GCR_PERF_SEL_PIO_GL2_ALL_REQ__NV3X                 = 0x00000090,
    GCR_PERF_SEL_PIO_GL1_RANGE_REQ__NV3X               = 0x00000091,
    GCR_PERF_SEL_PIO_GL1_RANGE_LT16K_REQ__NV3X         = 0x00000092,
    GCR_PERF_SEL_PIO_GL1_RANGE_16K_REQ__NV3X           = 0x00000093,
    GCR_PERF_SEL_PIO_GL1_RANGE_GT16K_REQ__NV3X         = 0x00000094,
    GCR_PERF_SEL_PIO_GL1_ALL_REQ__NV3X                 = 0x00000095,
    GCR_PERF_SEL_PIO_METADATA_REQ__NV3X                = 0x00000096,
    GCR_PERF_SEL_PIO_SQC_DATA_REQ__NV3X                = 0x00000097,
    GCR_PERF_SEL_PIO_SQC_INST_REQ__NV3X                = 0x00000098,
    GCR_PERF_SEL_PIO_TCP_REQ__NV3X                     = 0x00000099,
    GCR_PERF_SEL_PIO_GL1_TLB_SHOOTDOWN_REQ__NV3X       = 0x0000009a,
} GCRPerfSel;

constexpr unsigned int MaxGCRPerfSelGfx101             = GCR_PERF_SEL_UTCL2_FILTERED_RET;
constexpr unsigned int MaxGCRPerfSelNv21               = GCR_PERF_SEL_PIO_TCP_TLB_SHOOTDOWN_REQ__NV21;
constexpr unsigned int MaxGCRPerfSelApu103             = GCR_PERF_SEL_PIO_TCP_TLB_SHOOTDOWN_REQ__APU103;
constexpr unsigned int MaxGCRPerfSelNv24               = GCR_PERF_SEL_PIO_TCP_TLB_SHOOTDOWN_REQ__NV24;
constexpr unsigned int MaxGCRPerfSelNv23               = GCR_PERF_SEL_PIO_TCP_TLB_SHOOTDOWN_REQ__NV23;
constexpr unsigned int MaxGCRPerfSelNv22               = GCR_PERF_SEL_PIO_TCP_TLB_SHOOTDOWN_REQ__NV22;
constexpr unsigned int MaxGCRPerfSelNv3x               = GCR_PERF_SEL_PIO_GL1_TLB_SHOOTDOWN_REQ__NV3X;
constexpr unsigned int MaxGCRPerfSelApu11              = GCR_PERF_SEL_PIO_GL1_TLB_SHOOTDOWN_REQ__APU11;

typedef enum GCVML2_PERF_SEL {
    GCVML2_PERF_SEL_EVENT_0                            = 0x00000000,
    GCVML2_PERF_SEL_EVENT_1                            = 0x00000001,
    GCVML2_PERF_SEL_EVENT_2                            = 0x00000002,
    GCVML2_PERF_SEL_EVENT_3                            = 0x00000003,
    GCVML2_PERF_SEL_EVENT_4                            = 0x00000004,
    GCVML2_PERF_SEL_EVENT_5                            = 0x00000005,
    GCVML2_PERF_SEL_EVENT_6                            = 0x00000006,
    GCVML2_PERF_SEL_EVENT_7                            = 0x00000007,
    GCVML2_PERF_SEL_EVENT_8                            = 0x00000008,
    GCVML2_PERF_SEL_EVENT_9                            = 0x00000009,
    GCVML2_PERF_SEL_EVENT_10                           = 0x0000000a,
    GCVML2_PERF_SEL_EVENT_11                           = 0x0000000b,
    GCVML2_PERF_SEL_EVENT_12                           = 0x0000000c,
    GCVML2_PERF_SEL_EVENT_13                           = 0x0000000d,
    GCVML2_PERF_SEL_EVENT_14                           = 0x0000000e,
    GCVML2_PERF_SEL_EVENT_15                           = 0x0000000f,
    GCVML2_PERF_SEL_EVENT_16                           = 0x00000010,
    GCVML2_PERF_SEL_EVENT_17                           = 0x00000011,
    GCVML2_PERF_SEL_EVENT_18                           = 0x00000012,
    GCVML2_PERF_SEL_EVENT_19                           = 0x00000013,
    GCVML2_PERF_SEL_EVENT_20                           = 0x00000014,
    GCVML2_PERF_SEL_EVENT_21                           = 0x00000015,
    GCVML2_PERF_SEL_EVENT_22                           = 0x00000016,
    GCVML2_PERF_SEL_EVENT_23                           = 0x00000017,
    GCVML2_PERF_SEL_EVENT_24                           = 0x00000018,
    GCVML2_PERF_SEL_EVENT_25                           = 0x00000019,
    GCVML2_PERF_SEL_EVENT_26                           = 0x0000001a,
    GCVML2_PERF_SEL_EVENT_27                           = 0x0000001b,
    GCVML2_PERF_SEL_EVENT_28                           = 0x0000001c,
    GCVML2_PERF_SEL_EVENT_29                           = 0x0000001d,
    GCVML2_PERF_SEL_EVENT_30                           = 0x0000001e,
    GCVML2_PERF_SEL_EVENT_31                           = 0x0000001f,
    GCVML2_PERF_SEL_EVENT_32                           = 0x00000020,
    GCVML2_PERF_SEL_EVENT_33                           = 0x00000021,
    GCVML2_PERF_SEL_EVENT_34                           = 0x00000022,
    GCVML2_PERF_SEL_EVENT_35                           = 0x00000023,
    GCVML2_PERF_SEL_EVENT_36                           = 0x00000024,
    GCVML2_PERF_SEL_EVENT_37                           = 0x00000025,
    GCVML2_PERF_SEL_EVENT_38                           = 0x00000026,
    GCVML2_PERF_SEL_EVENT_39                           = 0x00000027,
    GCVML2_PERF_SEL_EVENT_40                           = 0x00000028,
    GCVML2_PERF_SEL_EVENT_41                           = 0x00000029,
    GCVML2_PERF_SEL_EVENT_42                           = 0x0000002a,
    GCVML2_PERF_SEL_EVENT_43                           = 0x0000002b,
    GCVML2_PERF_SEL_EVENT_44                           = 0x0000002c,
    GCVML2_PERF_SEL_EVENT_45                           = 0x0000002d,
    GCVML2_PERF_SEL_EVENT_46                           = 0x0000002e,
    GCVML2_PERF_SEL_EVENT_47                           = 0x0000002f,
    GCVML2_PERF_SEL_EVENT_48                           = 0x00000030,
    GCVML2_PERF_SEL_EVENT_49                           = 0x00000031,
    GCVML2_PERF_SEL_EVENT_50                           = 0x00000032,
    GCVML2_PERF_SEL_EVENT_51                           = 0x00000033,
    GCVML2_PERF_SEL_EVENT_52                           = 0x00000034,
    GCVML2_PERF_SEL_EVENT_53                           = 0x00000035,
    GCVML2_PERF_SEL_EVENT_54                           = 0x00000036,
    GCVML2_PERF_SEL_EVENT_55                           = 0x00000037,
    GCVML2_PERF_SEL_EVENT_56                           = 0x00000038,
    GCVML2_PERF_SEL_EVENT_57                           = 0x00000039,
    GCVML2_PERF_SEL_EVENT_58                           = 0x0000003a,
    GCVML2_PERF_SEL_EVENT_59                           = 0x0000003b,
    GCVML2_PERF_SEL_EVENT_60                           = 0x0000003c,
    GCVML2_PERF_SEL_EVENT_61                           = 0x0000003d,
    GCVML2_PERF_SEL_EVENT_62                           = 0x0000003e,
    GCVML2_PERF_SEL_EVENT_63                           = 0x0000003f,
    GCVML2_PERF_SEL_EVENT_64                           = 0x00000040,
    GCVML2_PERF_SEL_EVENT_65                           = 0x00000041,
    GCVML2_PERF_SEL_EVENT_66                           = 0x00000042,
    GCVML2_PERF_SEL_EVENT_67                           = 0x00000043,
    GCVML2_PERF_SEL_EVENT_68                           = 0x00000044,
    GCVML2_PERF_SEL_EVENT_69                           = 0x00000045,
    GCVML2_PERF_SEL_EVENT_70                           = 0x00000046,
    GCVML2_PERF_SEL_EVENT_71                           = 0x00000047,
    GCVML2_PERF_SEL_EVENT_72                           = 0x00000048,
    GCVML2_PERF_SEL_EVENT_73                           = 0x00000049,
    GCVML2_PERF_SEL_EVENT_74                           = 0x0000004a,
    GCVML2_PERF_SEL_EVENT_75                           = 0x0000004b,
    GCVML2_PERF_SEL_EVENT_76                           = 0x0000004c,
    GCVML2_PERF_SEL_EVENT_77                           = 0x0000004d,
    GCVML2_PERF_SEL_EVENT_78                           = 0x0000004e,
    GCVML2_PERF_SEL_EVENT_79                           = 0x0000004f,
    GCVML2_PERF_SEL_EVENT_80                           = 0x00000050,
    GCVML2_PERF_SEL_EVENT_81                           = 0x00000051,
    GCVML2_PERF_SEL_EVENT_82                           = 0x00000052,
    GCVML2_PERF_SEL_EVENT_83                           = 0x00000053,
    GCVML2_PERF_SEL_EVENT_84                           = 0x00000054,
    GCVML2_PERF_SEL_EVENT_85                           = 0x00000055,
    GCVML2_PERF_SEL_EVENT_86                           = 0x00000056,
    GCVML2_PERF_SEL_EVENT_87                           = 0x00000057,
    GCVML2_PERF_SEL_EVENT_88                           = 0x00000058,
    GCVML2_PERF_SEL_EVENT_89                           = 0x00000059,
    GCVML2_PERF_SEL_EVENT_90                           = 0x0000005a,
} GCVML2_PERF_SEL;

constexpr unsigned int MaxGcvml2PerfSel                = GCVML2_PERF_SEL_EVENT_90;

typedef enum GDS_PERFCOUNT_SELECT {
    GDS_PERF_SEL_DS_ADDR_CONFL__GFX10                  = 0x00000000,
    GDS_PERF_SEL_DS_BANK_CONFL__GFX10                  = 0x00000001,
    GDS_PERF_SEL_WBUF_FLUSH__GFX10                     = 0x00000002,
    GDS_PERF_SEL_WR_COMP__GFX10                        = 0x00000003,
    GDS_PERF_SEL_WBUF_WR__GFX10                        = 0x00000004,
    GDS_PERF_SEL_RBUF_HIT__GFX10                       = 0x00000005,
    GDS_PERF_SEL_RBUF_MISS__GFX10                      = 0x00000006,
    GDS_PERF_SEL_SE0_SH0_NORET__GFX10                  = 0x00000007,
    GDS_PERF_SEL_SE0_SH0_RET__GFX10                    = 0x00000008,
    GDS_PERF_SEL_SE0_SH0_ORD_CNT__GFX10                = 0x00000009,
    GDS_PERF_SEL_SE0_SH0_2COMP_REQ__GFX10              = 0x0000000a,
    GDS_PERF_SEL_SE0_SH0_ORD_WAVE_VALID__GFX10         = 0x0000000b,
    GDS_PERF_SEL_SE0_SH0_GDS_DATA_VALID__GFX10         = 0x0000000c,
    GDS_PERF_SEL_SE0_SH0_GDS_STALL_BY_ORD__GFX10       = 0x0000000d,
    GDS_PERF_SEL_SE0_SH0_GDS_WR_OP__GFX10              = 0x0000000e,
    GDS_PERF_SEL_SE0_SH0_GDS_RD_OP__GFX10              = 0x0000000f,
    GDS_PERF_SEL_SE0_SH0_GDS_ATOM_OP__GFX10            = 0x00000010,
    GDS_PERF_SEL_SE0_SH0_GDS_REL_OP__GFX10             = 0x00000011,
    GDS_PERF_SEL_SE0_SH0_GDS_CMPXCH_OP__GFX10          = 0x00000012,
    GDS_PERF_SEL_SE0_SH0_GDS_BYTE_OP__GFX10            = 0x00000013,
    GDS_PERF_SEL_SE0_SH0_GDS_SHORT_OP__GFX10           = 0x00000014,
    GDS_PERF_SEL_SE0_SH1_NORET__GFX10                  = 0x00000015,
    GDS_PERF_SEL_SE0_SH1_RET__GFX10                    = 0x00000016,
    GDS_PERF_SEL_SE0_SH1_ORD_CNT__GFX10                = 0x00000017,
    GDS_PERF_SEL_SE0_SH1_2COMP_REQ__GFX10              = 0x00000018,
    GDS_PERF_SEL_SE0_SH1_ORD_WAVE_VALID__GFX10         = 0x00000019,
    GDS_PERF_SEL_SE0_SH1_GDS_DATA_VALID__GFX10         = 0x0000001a,
    GDS_PERF_SEL_SE0_SH1_GDS_STALL_BY_ORD__GFX10       = 0x0000001b,
    GDS_PERF_SEL_SE0_SH1_GDS_WR_OP__GFX10              = 0x0000001c,
    GDS_PERF_SEL_SE0_SH1_GDS_RD_OP__GFX10              = 0x0000001d,
    GDS_PERF_SEL_SE0_SH1_GDS_ATOM_OP__GFX10            = 0x0000001e,
    GDS_PERF_SEL_SE0_SH1_GDS_REL_OP__GFX10             = 0x0000001f,
    GDS_PERF_SEL_SE0_SH1_GDS_CMPXCH_OP__GFX10          = 0x00000020,
    GDS_PERF_SEL_SE0_SH1_GDS_BYTE_OP__GFX10            = 0x00000021,
    GDS_PERF_SEL_SE0_SH1_GDS_SHORT_OP__GFX10           = 0x00000022,
    GDS_PERF_SEL_SE1_SH0_NORET__GFX10                  = 0x00000023,
    GDS_PERF_SEL_SE1_SH0_RET__GFX10                    = 0x00000024,
    GDS_PERF_SEL_SE1_SH0_ORD_CNT__GFX10                = 0x00000025,
    GDS_PERF_SEL_SE1_SH0_2COMP_REQ__GFX10              = 0x00000026,
    GDS_PERF_SEL_SE1_SH0_ORD_WAVE_VALID__GFX10         = 0x00000027,
    GDS_PERF_SEL_SE1_SH0_GDS_DATA_VALID__GFX10         = 0x00000028,
    GDS_PERF_SEL_SE1_SH0_GDS_STALL_BY_ORD__GFX10       = 0x00000029,
    GDS_PERF_SEL_SE1_SH0_GDS_WR_OP__GFX10              = 0x0000002a,
    GDS_PERF_SEL_SE1_SH0_GDS_RD_OP__GFX10              = 0x0000002b,
    GDS_PERF_SEL_SE1_SH0_GDS_ATOM_OP__GFX10            = 0x0000002c,
    GDS_PERF_SEL_SE1_SH0_GDS_REL_OP__GFX10             = 0x0000002d,
    GDS_PERF_SEL_SE1_SH0_GDS_CMPXCH_OP__GFX10          = 0x0000002e,
    GDS_PERF_SEL_SE1_SH0_GDS_BYTE_OP__GFX10            = 0x0000002f,
    GDS_PERF_SEL_SE1_SH0_GDS_SHORT_OP__GFX10           = 0x00000030,
    GDS_PERF_SEL_SE1_SH1_NORET__GFX10                  = 0x00000031,
    GDS_PERF_SEL_SE1_SH1_RET__GFX10                    = 0x00000032,
    GDS_PERF_SEL_SE1_SH1_ORD_CNT__GFX10                = 0x00000033,
    GDS_PERF_SEL_SE1_SH1_2COMP_REQ__GFX10              = 0x00000034,
    GDS_PERF_SEL_SE1_SH1_ORD_WAVE_VALID__GFX10         = 0x00000035,
    GDS_PERF_SEL_SE1_SH1_GDS_DATA_VALID__GFX10         = 0x00000036,
    GDS_PERF_SEL_SE1_SH1_GDS_STALL_BY_ORD__GFX10       = 0x00000037,
    GDS_PERF_SEL_SE1_SH1_GDS_WR_OP__GFX10              = 0x00000038,
    GDS_PERF_SEL_SE1_SH1_GDS_RD_OP__GFX10              = 0x00000039,
    GDS_PERF_SEL_SE1_SH1_GDS_ATOM_OP__GFX10            = 0x0000003a,
    GDS_PERF_SEL_SE1_SH1_GDS_REL_OP__GFX10             = 0x0000003b,
    GDS_PERF_SEL_SE1_SH1_GDS_CMPXCH_OP__GFX10          = 0x0000003c,
    GDS_PERF_SEL_SE1_SH1_GDS_BYTE_OP__GFX10            = 0x0000003d,
    GDS_PERF_SEL_SE1_SH1_GDS_SHORT_OP__GFX10           = 0x0000003e,
    GDS_PERF_SEL_SE2_SH0_NORET__GFX10                  = 0x0000003f,
    GDS_PERF_SEL_SE2_SH0_RET__GFX10                    = 0x00000040,
    GDS_PERF_SEL_SE2_SH0_ORD_CNT__GFX10                = 0x00000041,
    GDS_PERF_SEL_SE2_SH0_2COMP_REQ__GFX10              = 0x00000042,
    GDS_PERF_SEL_SE2_SH0_ORD_WAVE_VALID__GFX10         = 0x00000043,
    GDS_PERF_SEL_SE2_SH0_GDS_DATA_VALID__GFX10         = 0x00000044,
    GDS_PERF_SEL_SE2_SH0_GDS_STALL_BY_ORD__GFX10       = 0x00000045,
    GDS_PERF_SEL_SE2_SH0_GDS_WR_OP__GFX10              = 0x00000046,
    GDS_PERF_SEL_SE2_SH0_GDS_RD_OP__GFX10              = 0x00000047,
    GDS_PERF_SEL_SE2_SH0_GDS_ATOM_OP__GFX10            = 0x00000048,
    GDS_PERF_SEL_SE2_SH0_GDS_REL_OP__GFX10             = 0x00000049,
    GDS_PERF_SEL_SE2_SH0_GDS_CMPXCH_OP__GFX10          = 0x0000004a,
    GDS_PERF_SEL_SE2_SH0_GDS_BYTE_OP__GFX10            = 0x0000004b,
    GDS_PERF_SEL_SE2_SH0_GDS_SHORT_OP__GFX10           = 0x0000004c,
    GDS_PERF_SEL_SE2_SH1_NORET__GFX10                  = 0x0000004d,
    GDS_PERF_SEL_SE2_SH1_RET__GFX10                    = 0x0000004e,
    GDS_PERF_SEL_SE2_SH1_ORD_CNT__GFX10                = 0x0000004f,
    GDS_PERF_SEL_SE2_SH1_2COMP_REQ__GFX10              = 0x00000050,
    GDS_PERF_SEL_SE2_SH1_ORD_WAVE_VALID__GFX10         = 0x00000051,
    GDS_PERF_SEL_SE2_SH1_GDS_DATA_VALID__GFX10         = 0x00000052,
    GDS_PERF_SEL_SE2_SH1_GDS_STALL_BY_ORD__GFX10       = 0x00000053,
    GDS_PERF_SEL_SE2_SH1_GDS_WR_OP__GFX10              = 0x00000054,
    GDS_PERF_SEL_SE2_SH1_GDS_RD_OP__GFX10              = 0x00000055,
    GDS_PERF_SEL_SE2_SH1_GDS_ATOM_OP__GFX10            = 0x00000056,
    GDS_PERF_SEL_SE2_SH1_GDS_REL_OP__GFX10             = 0x00000057,
    GDS_PERF_SEL_SE2_SH1_GDS_CMPXCH_OP__GFX10          = 0x00000058,
    GDS_PERF_SEL_SE2_SH1_GDS_BYTE_OP__GFX10            = 0x00000059,
    GDS_PERF_SEL_SE2_SH1_GDS_SHORT_OP__GFX10           = 0x0000005a,
    GDS_PERF_SEL_SE3_SH0_NORET__GFX10                  = 0x0000005b,
    GDS_PERF_SEL_SE3_SH0_RET__GFX10                    = 0x0000005c,
    GDS_PERF_SEL_SE3_SH0_ORD_CNT__GFX10                = 0x0000005d,
    GDS_PERF_SEL_SE3_SH0_2COMP_REQ__GFX10              = 0x0000005e,
    GDS_PERF_SEL_SE3_SH0_ORD_WAVE_VALID__GFX10         = 0x0000005f,
    GDS_PERF_SEL_SE3_SH0_GDS_DATA_VALID__GFX10         = 0x00000060,
    GDS_PERF_SEL_SE3_SH0_GDS_STALL_BY_ORD__GFX10       = 0x00000061,
    GDS_PERF_SEL_SE3_SH0_GDS_WR_OP__GFX10              = 0x00000062,
    GDS_PERF_SEL_SE3_SH0_GDS_RD_OP__GFX10              = 0x00000063,
    GDS_PERF_SEL_SE3_SH0_GDS_ATOM_OP__GFX10            = 0x00000064,
    GDS_PERF_SEL_SE3_SH0_GDS_REL_OP__GFX10             = 0x00000065,
    GDS_PERF_SEL_SE3_SH0_GDS_CMPXCH_OP__GFX10          = 0x00000066,
    GDS_PERF_SEL_SE3_SH0_GDS_BYTE_OP__GFX10            = 0x00000067,
    GDS_PERF_SEL_SE3_SH0_GDS_SHORT_OP__GFX10           = 0x00000068,
    GDS_PERF_SEL_SE3_SH1_NORET__GFX10                  = 0x00000069,
    GDS_PERF_SEL_SE3_SH1_RET__GFX10                    = 0x0000006a,
    GDS_PERF_SEL_SE3_SH1_ORD_CNT__GFX10                = 0x0000006b,
    GDS_PERF_SEL_SE3_SH1_2COMP_REQ__GFX10              = 0x0000006c,
    GDS_PERF_SEL_SE3_SH1_ORD_WAVE_VALID__GFX10         = 0x0000006d,
    GDS_PERF_SEL_SE3_SH1_GDS_DATA_VALID__GFX10         = 0x0000006e,
    GDS_PERF_SEL_SE3_SH1_GDS_STALL_BY_ORD__GFX10       = 0x0000006f,
    GDS_PERF_SEL_SE3_SH1_GDS_WR_OP__GFX10              = 0x00000070,
    GDS_PERF_SEL_SE3_SH1_GDS_RD_OP__GFX10              = 0x00000071,
    GDS_PERF_SEL_SE3_SH1_GDS_ATOM_OP__GFX10            = 0x00000072,
    GDS_PERF_SEL_SE3_SH1_GDS_REL_OP__GFX10             = 0x00000073,
    GDS_PERF_SEL_SE3_SH1_GDS_CMPXCH_OP__GFX10          = 0x00000074,
    GDS_PERF_SEL_SE3_SH1_GDS_BYTE_OP__GFX10            = 0x00000075,
    GDS_PERF_SEL_SE3_SH1_GDS_SHORT_OP__GFX10           = 0x00000076,
    GDS_PERF_SEL_GWS_RELEASED__GFX10                   = 0x00000077,
    GDS_PERF_SEL_GWS_BYPASS__GFX10                     = 0x00000078,
    GDS_PERF_SEL_WR_COMP__GFX11                        = 0x00000000,
    GDS_PERF_SEL_WBUF_WR__GFX11                        = 0x00000001,
    GDS_PERF_SEL_SE0_NORET__GFX11                      = 0x00000002,
    GDS_PERF_SEL_SE0_RET__GFX11                        = 0x00000003,
    GDS_PERF_SEL_SE0_ORD_CNT__GFX11                    = 0x00000004,
    GDS_PERF_SEL_SE0_2COMP_REQ__GFX11                  = 0x00000005,
    GDS_PERF_SEL_SE0_ORD_WAVE_VALID__GFX11             = 0x00000006,
    GDS_PERF_SEL_SE0_GDS_STALL_BY_ORD__GFX11           = 0x00000007,
    GDS_PERF_SEL_SE0_GDS_WR_OP__GFX11                  = 0x00000008,
    GDS_PERF_SEL_SE0_GDS_RD_OP__GFX11                  = 0x00000009,
    GDS_PERF_SEL_SE0_GDS_ATOM_OP__GFX11                = 0x0000000a,
    GDS_PERF_SEL_SE0_GDS_REL_OP__GFX11                 = 0x0000000b,
    GDS_PERF_SEL_SE0_GDS_CMPXCH_OP__GFX11              = 0x0000000c,
    GDS_PERF_SEL_SE0_GDS_BYTE_OP__GFX11                = 0x0000000d,
    GDS_PERF_SEL_SE0_GDS_SHORT_OP__GFX11               = 0x0000000e,
    GDS_PERF_SEL_SE1_NORET__GFX11                      = 0x0000000f,
    GDS_PERF_SEL_SE1_RET__GFX11                        = 0x00000010,
    GDS_PERF_SEL_SE1_ORD_CNT__GFX11                    = 0x00000011,
    GDS_PERF_SEL_SE1_2COMP_REQ__GFX11                  = 0x00000012,
    GDS_PERF_SEL_SE1_ORD_WAVE_VALID__GFX11             = 0x00000013,
    GDS_PERF_SEL_SE1_GDS_STALL_BY_ORD__GFX11           = 0x00000014,
    GDS_PERF_SEL_SE1_GDS_WR_OP__GFX11                  = 0x00000015,
    GDS_PERF_SEL_SE1_GDS_RD_OP__GFX11                  = 0x00000016,
    GDS_PERF_SEL_SE1_GDS_ATOM_OP__GFX11                = 0x00000017,
    GDS_PERF_SEL_SE1_GDS_REL_OP__GFX11                 = 0x00000018,
    GDS_PERF_SEL_SE1_GDS_CMPXCH_OP__GFX11              = 0x00000019,
    GDS_PERF_SEL_SE1_GDS_BYTE_OP__GFX11                = 0x0000001a,
    GDS_PERF_SEL_SE1_GDS_SHORT_OP__GFX11               = 0x0000001b,
    GDS_PERF_SEL_SE2_NORET__GFX11                      = 0x0000001c,
    GDS_PERF_SEL_SE2_RET__GFX11                        = 0x0000001d,
    GDS_PERF_SEL_SE2_ORD_CNT__GFX11                    = 0x0000001e,
    GDS_PERF_SEL_SE2_2COMP_REQ__GFX11                  = 0x0000001f,
    GDS_PERF_SEL_SE2_ORD_WAVE_VALID__GFX11             = 0x00000020,
    GDS_PERF_SEL_SE2_GDS_STALL_BY_ORD__GFX11           = 0x00000021,
    GDS_PERF_SEL_SE2_GDS_WR_OP__GFX11                  = 0x00000022,
    GDS_PERF_SEL_SE2_GDS_RD_OP__GFX11                  = 0x00000023,
    GDS_PERF_SEL_SE2_GDS_ATOM_OP__GFX11                = 0x00000024,
    GDS_PERF_SEL_SE2_GDS_REL_OP__GFX11                 = 0x00000025,
    GDS_PERF_SEL_SE2_GDS_CMPXCH_OP__GFX11              = 0x00000026,
    GDS_PERF_SEL_SE2_GDS_BYTE_OP__GFX11                = 0x00000027,
    GDS_PERF_SEL_SE2_GDS_SHORT_OP__GFX11               = 0x00000028,
    GDS_PERF_SEL_SE3_NORET__GFX11                      = 0x00000029,
    GDS_PERF_SEL_SE3_RET__GFX11                        = 0x0000002a,
    GDS_PERF_SEL_SE3_ORD_CNT__GFX11                    = 0x0000002b,
    GDS_PERF_SEL_SE3_2COMP_REQ__GFX11                  = 0x0000002c,
    GDS_PERF_SEL_SE3_ORD_WAVE_VALID__GFX11             = 0x0000002d,
    GDS_PERF_SEL_SE3_GDS_STALL_BY_ORD__GFX11           = 0x0000002e,
    GDS_PERF_SEL_SE3_GDS_WR_OP__GFX11                  = 0x0000002f,
    GDS_PERF_SEL_SE3_GDS_RD_OP__GFX11                  = 0x00000030,
    GDS_PERF_SEL_SE3_GDS_ATOM_OP__GFX11                = 0x00000031,
    GDS_PERF_SEL_SE3_GDS_REL_OP__GFX11                 = 0x00000032,
    GDS_PERF_SEL_SE3_GDS_CMPXCH_OP__GFX11              = 0x00000033,
    GDS_PERF_SEL_SE3_GDS_BYTE_OP__GFX11                = 0x00000034,
    GDS_PERF_SEL_SE3_GDS_SHORT_OP__GFX11               = 0x00000035,
    GDS_PERF_SEL_SE4_NORET__GFX11                      = 0x00000036,
    GDS_PERF_SEL_SE4_RET__GFX11                        = 0x00000037,
    GDS_PERF_SEL_SE4_ORD_CNT__GFX11                    = 0x00000038,
    GDS_PERF_SEL_SE4_2COMP_REQ__GFX11                  = 0x00000039,
    GDS_PERF_SEL_SE4_ORD_WAVE_VALID__GFX11             = 0x0000003a,
    GDS_PERF_SEL_SE4_GDS_STALL_BY_ORD__GFX11           = 0x0000003b,
    GDS_PERF_SEL_SE4_GDS_WR_OP__GFX11                  = 0x0000003c,
    GDS_PERF_SEL_SE4_GDS_RD_OP__GFX11                  = 0x0000003d,
    GDS_PERF_SEL_SE4_GDS_ATOM_OP__GFX11                = 0x0000003e,
    GDS_PERF_SEL_SE4_GDS_REL_OP__GFX11                 = 0x0000003f,
    GDS_PERF_SEL_SE4_GDS_CMPXCH_OP__GFX11              = 0x00000040,
    GDS_PERF_SEL_SE4_GDS_BYTE_OP__GFX11                = 0x00000041,
    GDS_PERF_SEL_SE4_GDS_SHORT_OP__GFX11               = 0x00000042,
    GDS_PERF_SEL_SE5_NORET__GFX11                      = 0x00000043,
    GDS_PERF_SEL_SE5_RET__GFX11                        = 0x00000044,
    GDS_PERF_SEL_SE5_ORD_CNT__GFX11                    = 0x00000045,
    GDS_PERF_SEL_SE5_2COMP_REQ__GFX11                  = 0x00000046,
    GDS_PERF_SEL_SE5_ORD_WAVE_VALID__GFX11             = 0x00000047,
    GDS_PERF_SEL_SE5_GDS_STALL_BY_ORD__GFX11           = 0x00000048,
    GDS_PERF_SEL_SE5_GDS_WR_OP__GFX11                  = 0x00000049,
    GDS_PERF_SEL_SE5_GDS_RD_OP__GFX11                  = 0x0000004a,
    GDS_PERF_SEL_SE5_GDS_ATOM_OP__GFX11                = 0x0000004b,
    GDS_PERF_SEL_SE5_GDS_REL_OP__GFX11                 = 0x0000004c,
    GDS_PERF_SEL_SE5_GDS_CMPXCH_OP__GFX11              = 0x0000004d,
    GDS_PERF_SEL_SE5_GDS_BYTE_OP__GFX11                = 0x0000004e,
    GDS_PERF_SEL_SE5_GDS_SHORT_OP__GFX11               = 0x0000004f,
    GDS_PERF_SEL_SE6_NORET__GFX11                      = 0x00000050,
    GDS_PERF_SEL_SE6_RET__GFX11                        = 0x00000051,
    GDS_PERF_SEL_SE6_ORD_CNT__GFX11                    = 0x00000052,
    GDS_PERF_SEL_SE6_2COMP_REQ__GFX11                  = 0x00000053,
    GDS_PERF_SEL_SE6_ORD_WAVE_VALID__GFX11             = 0x00000054,
    GDS_PERF_SEL_SE6_GDS_STALL_BY_ORD__GFX11           = 0x00000055,
    GDS_PERF_SEL_SE6_GDS_WR_OP__GFX11                  = 0x00000056,
    GDS_PERF_SEL_SE6_GDS_RD_OP__GFX11                  = 0x00000057,
    GDS_PERF_SEL_SE6_GDS_ATOM_OP__GFX11                = 0x00000058,
    GDS_PERF_SEL_SE6_GDS_REL_OP__GFX11                 = 0x00000059,
    GDS_PERF_SEL_SE6_GDS_CMPXCH_OP__GFX11              = 0x0000005a,
    GDS_PERF_SEL_SE6_GDS_BYTE_OP__GFX11                = 0x0000005b,
    GDS_PERF_SEL_SE6_GDS_SHORT_OP__GFX11               = 0x0000005c,
    GDS_PERF_SEL_SE7_NORET__GFX11                      = 0x0000005d,
    GDS_PERF_SEL_SE7_RET__GFX11                        = 0x0000005e,
    GDS_PERF_SEL_SE7_ORD_CNT__GFX11                    = 0x0000005f,
    GDS_PERF_SEL_SE7_2COMP_REQ__GFX11                  = 0x00000060,
    GDS_PERF_SEL_SE7_ORD_WAVE_VALID__GFX11             = 0x00000061,
    GDS_PERF_SEL_SE7_GDS_STALL_BY_ORD__GFX11           = 0x00000062,
    GDS_PERF_SEL_SE7_GDS_WR_OP__GFX11                  = 0x00000063,
    GDS_PERF_SEL_SE7_GDS_RD_OP__GFX11                  = 0x00000064,
    GDS_PERF_SEL_SE7_GDS_ATOM_OP__GFX11                = 0x00000065,
    GDS_PERF_SEL_SE7_GDS_REL_OP__GFX11                 = 0x00000066,
    GDS_PERF_SEL_SE7_GDS_CMPXCH_OP__GFX11              = 0x00000067,
    GDS_PERF_SEL_SE7_GDS_BYTE_OP__GFX11                = 0x00000068,
    GDS_PERF_SEL_SE7_GDS_SHORT_OP__GFX11               = 0x00000069,
    GDS_PERF_SEL_GWS_RELEASED__GFX11                   = 0x0000006a,
    GDS_PERF_SEL_GWS_BYPASS__GFX11                     = 0x0000006b,
    GDS_PERF_SEL_SE0_GS_ORD_CNT_NOT_REL_NOT_DONE__GFX11 = 0x0000006c,
    GDS_PERF_SEL_SE0_GS_ORD_CNT_REL_DONE__GFX11        = 0x0000006d,
    GDS_PERF_SEL_SE0_ADD_GS_REG_OP__GFX11              = 0x0000006e,
    GDS_PERF_SEL_SE0_SUB_GS_REG_OP__GFX11              = 0x0000006f,
    GDS_PERF_SEL_SE0_GS_WAVE_ID_VALID__GFX11           = 0x00000070,
    GDS_PERF_SEL_SE1_GS_ORD_CNT_NOT_REL_NOT_DONE__GFX11 = 0x00000071,
    GDS_PERF_SEL_SE1_GS_ORD_CNT_REL_DONE__GFX11        = 0x00000072,
    GDS_PERF_SEL_SE1_ADD_GS_REG_OP__GFX11              = 0x00000073,
    GDS_PERF_SEL_SE1_SUB_GS_REG_OP__GFX11              = 0x00000074,
    GDS_PERF_SEL_SE1_GS_WAVE_ID_VALID__GFX11           = 0x00000075,
    GDS_PERF_SEL_SE2_GS_ORD_CNT_NOT_REL_NOT_DONE__GFX11 = 0x00000076,
    GDS_PERF_SEL_SE2_GS_ORD_CNT_REL_DONE__GFX11        = 0x00000077,
    GDS_PERF_SEL_SE2_ADD_GS_REG_OP__GFX11              = 0x00000078,
    GDS_PERF_SEL_SE2_SUB_GS_REG_OP__GFX11              = 0x00000079,
    GDS_PERF_SEL_SE2_GS_WAVE_ID_VALID__GFX11           = 0x0000007a,
    GDS_PERF_SEL_SE3_GS_ORD_CNT_NOT_REL_NOT_DONE__GFX11 = 0x0000007b,
    GDS_PERF_SEL_SE3_GS_ORD_CNT_REL_DONE__GFX11        = 0x0000007c,
    GDS_PERF_SEL_SE3_ADD_GS_REG_OP__GFX11              = 0x0000007d,
    GDS_PERF_SEL_SE3_SUB_GS_REG_OP__GFX11              = 0x0000007e,
    GDS_PERF_SEL_SE3_GS_WAVE_ID_VALID__GFX11           = 0x0000007f,
    GDS_PERF_SEL_SE4_GS_ORD_CNT_NOT_REL_NOT_DONE__GFX11 = 0x00000080,
    GDS_PERF_SEL_SE4_GS_ORD_CNT_REL_DONE__GFX11        = 0x00000081,
    GDS_PERF_SEL_SE4_ADD_GS_REG_OP__GFX11              = 0x00000082,
    GDS_PERF_SEL_SE4_SUB_GS_REG_OP__GFX11              = 0x00000083,
    GDS_PERF_SEL_SE4_GS_WAVE_ID_VALID__GFX11           = 0x00000084,
    GDS_PERF_SEL_SE5_GS_ORD_CNT_NOT_REL_NOT_DONE__GFX11 = 0x00000085,
    GDS_PERF_SEL_SE5_GS_ORD_CNT_REL_DONE__GFX11        = 0x00000086,
    GDS_PERF_SEL_SE5_ADD_GS_REG_OP__GFX11              = 0x00000087,
    GDS_PERF_SEL_SE5_SUB_GS_REG_OP__GFX11              = 0x00000088,
    GDS_PERF_SEL_SE5_GS_WAVE_ID_VALID__GFX11           = 0x00000089,
    GDS_PERF_SEL_SE6_GS_ORD_CNT_NOT_REL_NOT_DONE__GFX11 = 0x0000008a,
    GDS_PERF_SEL_SE6_GS_ORD_CNT_REL_DONE__GFX11        = 0x0000008b,
    GDS_PERF_SEL_SE6_ADD_GS_REG_OP__GFX11              = 0x0000008c,
    GDS_PERF_SEL_SE6_SUB_GS_REG_OP__GFX11              = 0x0000008d,
    GDS_PERF_SEL_SE6_GS_WAVE_ID_VALID__GFX11           = 0x0000008e,
    GDS_PERF_SEL_SE7_GS_ORD_CNT_NOT_REL_NOT_DONE__GFX11 = 0x0000008f,
    GDS_PERF_SEL_SE7_GS_ORD_CNT_REL_DONE__GFX11        = 0x00000090,
    GDS_PERF_SEL_SE7_ADD_GS_REG_OP__GFX11              = 0x00000091,
    GDS_PERF_SEL_SE7_SUB_GS_REG_OP__GFX11              = 0x00000092,
    GDS_PERF_SEL_SE7_GS_WAVE_ID_VALID__GFX11           = 0x00000093,
} GDS_PERFCOUNT_SELECT;

constexpr unsigned int MaxGdsPerfcountSelectGfx10      = GDS_PERF_SEL_GWS_BYPASS__GFX10;
constexpr unsigned int MaxGdsPerfcountSelectGfx11      = GDS_PERF_SEL_SE7_GS_WAVE_ID_VALID__GFX11;

typedef enum GE1_PERFCOUNT_SELECT {
    ge1_assembler_busy                                 = 0x00000000,
    ge1_assembler_stalled                              = 0x00000001,
    ge1_dma_busy                                       = 0x00000002,
    ge1_dma_lat_bin_0                                  = 0x00000003,
    ge1_dma_lat_bin_1                                  = 0x00000004,
    ge1_dma_lat_bin_2                                  = 0x00000005,
    ge1_dma_lat_bin_3                                  = 0x00000006,
    ge1_dma_lat_bin_4                                  = 0x00000007,
    ge1_dma_lat_bin_5                                  = 0x00000008,
    ge1_dma_lat_bin_6                                  = 0x00000009,
    ge1_dma_lat_bin_7                                  = 0x0000000a,
    ge1_dma_return_cl0                                 = 0x0000000b,
    ge1_dma_return_cl1                                 = 0x0000000c,
    ge1_dma_utcl1_consecutive_retry_event              = 0x0000000d,
    ge1_dma_utcl1_request_event                        = 0x0000000e,
    ge1_dma_utcl1_retry_event                          = 0x0000000f,
    ge1_dma_utcl1_stall_event                          = 0x00000010,
    ge1_dma_utcl1_stall_utcl2_event                    = 0x00000011,
    ge1_dma_utcl1_translation_hit_event                = 0x00000012,
    ge1_dma_utcl1_translation_miss_event               = 0x00000013,
    ge1_assembler_dma_starved                          = 0x00000014,
    ge1_rbiu_di_fifo_stalled_p0                        = 0x00000015,
    ge1_rbiu_di_fifo_starved_p0                        = 0x00000016,
    ge1_rbiu_dr_fifo_stalled_p0                        = 0x00000017,
    ge1_rbiu_dr_fifo_starved_p0                        = 0x00000018,
    ge1_sclk_reg_vld                                   = 0x00000019,
    ge1_stat_busy                                      = 0x0000001a,
    ge1_stat_no_dma_busy                               = 0x0000001b,
    ge1_pipe0_to_pipe1                                 = 0x0000001c,
    ge1_pipe1_to_pipe0                                 = 0x0000001d,
    ge1_dma_return_size_cl0                            = 0x0000001e,
    ge1_dma_return_size_cl1                            = 0x0000001f,
    ge1_sclk_input_vld                                 = 0x00000021,
    ge1_prim_group_limit_hit                           = 0x00000022,
    ge1_rbiu_di_fifo_stalled_p1                        = 0x00000024,
    ge1_rbiu_di_fifo_starved_p1                        = 0x00000025,
    ge1_rbiu_dr_fifo_stalled_p1                        = 0x00000026,
    ge1_rbiu_dr_fifo_starved_p1                        = 0x00000027,
    ge1_bypass_fifo_full__GFX10VRS                     = 0x00000020,
    ge1_vert_group_limit_hit__GFX10VRS                 = 0x00000023,
    ge1_small_draws_one_instance__GFX11                = 0x00000020,
} GE1_PERFCOUNT_SELECT;

constexpr unsigned int MaxGe1PerfcountSelect           = ge1_rbiu_dr_fifo_starved_p1;

typedef enum GE2_DIST_PERFCOUNT_SELECT {
    ge_dist_hs_done                                    = 0x00000000,
    ge_dist_hs_done_latency_se0                        = 0x00000001,
    ge_dist_hs_done_latency_se1                        = 0x00000002,
    ge_dist_hs_done_latency_se2                        = 0x00000003,
    ge_dist_hs_done_latency_se3                        = 0x00000004,
    ge_dist_inside_tf_bin_0__GFX10VRS                  = 0x00000005,
    ge_dist_inside_tf_bin_1__GFX10VRS                  = 0x00000006,
    ge_dist_inside_tf_bin_2__GFX10VRS                  = 0x00000007,
    ge_dist_inside_tf_bin_3__GFX10VRS                  = 0x00000008,
    ge_dist_inside_tf_bin_4__GFX10VRS                  = 0x00000009,
    ge_dist_inside_tf_bin_5__GFX10VRS                  = 0x0000000a,
    ge_dist_inside_tf_bin_6__GFX10VRS                  = 0x0000000b,
    ge_dist_inside_tf_bin_7__GFX10VRS                  = 0x0000000c,
    ge_dist_inside_tf_bin_8__GFX10VRS                  = 0x0000000d,
    ge_dist_null_patch__GFX10VRS                       = 0x0000000e,
    ge_dist_sclk_core_vld__GFX10VRS                    = 0x0000000f,
    ge_dist_sclk_wd_te11_vld__GFX10VRS                 = 0x00000010,
    ge_dist_tfreq_lat_bin_0__GFX10VRS                  = 0x00000011,
    ge_dist_tfreq_lat_bin_1__GFX10VRS                  = 0x00000012,
    ge_dist_tfreq_lat_bin_2__GFX10VRS                  = 0x00000013,
    ge_dist_tfreq_lat_bin_3__GFX10VRS                  = 0x00000014,
    ge_dist_tfreq_lat_bin_4__GFX10VRS                  = 0x00000015,
    ge_dist_tfreq_lat_bin_5__GFX10VRS                  = 0x00000016,
    ge_dist_tfreq_lat_bin_6__GFX10VRS                  = 0x00000017,
    ge_dist_tfreq_lat_bin_7__GFX10VRS                  = 0x00000018,
    ge_dist_tfreq_utcl1_consecutive_retry_event__GFX10VRS = 0x00000019,
    ge_dist_tfreq_utcl1_request_event__GFX10VRS        = 0x0000001a,
    ge_dist_tfreq_utcl1_retry_event__GFX10VRS          = 0x0000001b,
    ge_dist_tfreq_utcl1_stall_event__GFX10VRS          = 0x0000001c,
    ge_dist_tfreq_utcl1_stall_utcl2_event__GFX10VRS    = 0x0000001d,
    ge_dist_tfreq_utcl1_translation_hit_event__GFX10VRS = 0x0000001e,
    ge_dist_tfreq_utcl1_translation_miss_event__GFX10VRS = 0x0000001f,
    ge_dist_vs_pc_stall__GFX10VRS                      = 0x00000020,
    ge_dist_pc_feorder_fifo_full__GFX10VRS             = 0x00000021,
    ge_dist_pc_ge_manager_busy__GFX10VRS               = 0x00000022,
    ge_dist_pc_req_stall_se0__GFX10VRS                 = 0x00000023,
    ge_dist_pc_req_stall_se1__GFX10VRS                 = 0x00000024,
    ge_dist_pc_req_stall_se2__GFX10VRS                 = 0x00000025,
    ge_dist_pc_req_stall_se3__GFX10VRS                 = 0x00000026,
    ge_dist_pc_space_zero__GFX10VRS                    = 0x00000027,
    ge_dist_sclk_input_vld__GFX10VRS                   = 0x00000028,
    ge_dist_reserved__GFX10VRS                         = 0x00000029,
    ge_dist_wd_te11_busy__GFX10VRS                     = 0x0000002a,
    ge_dist_te11_starved__GFX10VRS                     = 0x0000002b,
    ge_dist_switch_mode_stall__GFX10VRS                = 0x0000002c,
    ge_all_tf_eq__GFX10VRS                             = 0x0000002d,
    ge_all_tf2__GFX10VRS                               = 0x0000002e,
    ge_all_tf3__GFX10VRS                               = 0x0000002f,
    ge_all_tf4__GFX10VRS                               = 0x00000030,
    ge_all_tf5__GFX10VRS                               = 0x00000031,
    ge_all_tf6__GFX10VRS                               = 0x00000032,
    ge_se0_te11_starved_on_hs_done__GFX10VRS           = 0x00000033,
    ge_se1_te11_starved_on_hs_done__GFX10VRS           = 0x00000034,
    ge_se2_te11_starved_on_hs_done__GFX10VRS           = 0x00000035,
    ge_se3_te11_starved_on_hs_done__GFX10VRS           = 0x00000036,
    ge_dist_op_fifo_full_starve__GFX10VRS              = 0x00000037,
    ge_dist_hs_done_se0__GFX10VRS                      = 0x00000038,
    ge_dist_hs_done_se1__GFX10VRS                      = 0x00000039,
    ge_dist_hs_done_se2__GFX10VRS                      = 0x0000003a,
    ge_dist_hs_done_se3__GFX10VRS                      = 0x0000003b,
    ge_dist_hs_done_latency__GFX10VRS                  = 0x0000003c,
    ge_dist_distributer_busy__GFX10VRS                 = 0x0000003d,
    ge_tf_ret_data_stalling_hs_done__GFX10VRS          = 0x0000003e,
    ge_dist_hs_done_latency_se4__GFX11                 = 0x00000005,
    ge_dist_hs_done_latency_se5__GFX11                 = 0x00000006,
    ge_dist_hs_done_latency_se6__GFX11                 = 0x00000007,
    ge_dist_hs_done_latency_se7__GFX11                 = 0x00000008,
    ge_dist_inside_tf_bin_0__GFX11                     = 0x00000009,
    ge_dist_inside_tf_bin_1__GFX11                     = 0x0000000a,
    ge_dist_inside_tf_bin_2__GFX11                     = 0x0000000b,
    ge_dist_inside_tf_bin_3__GFX11                     = 0x0000000c,
    ge_dist_inside_tf_bin_4__GFX11                     = 0x0000000d,
    ge_dist_inside_tf_bin_5__GFX11                     = 0x0000000e,
    ge_dist_inside_tf_bin_6__GFX11                     = 0x0000000f,
    ge_dist_inside_tf_bin_7__GFX11                     = 0x00000010,
    ge_dist_inside_tf_bin_8__GFX11                     = 0x00000011,
    ge_dist_null_patch__GFX11                          = 0x00000012,
    ge_dist_sclk_core_vld__GFX11                       = 0x00000013,
    ge_dist_sclk_wd_te11_vld__GFX11                    = 0x00000014,
    ge_dist_tfreq_lat_bin_0__GFX11                     = 0x00000015,
    ge_dist_tfreq_lat_bin_1__GFX11                     = 0x00000016,
    ge_dist_tfreq_lat_bin_2__GFX11                     = 0x00000017,
    ge_dist_tfreq_lat_bin_3__GFX11                     = 0x00000018,
    ge_dist_tfreq_lat_bin_4__GFX11                     = 0x00000019,
    ge_dist_tfreq_lat_bin_5__GFX11                     = 0x0000001a,
    ge_dist_tfreq_lat_bin_6__GFX11                     = 0x0000001b,
    ge_dist_tfreq_lat_bin_7__GFX11                     = 0x0000001c,
    ge_dist_tfreq_utcl1_consecutive_retry_event__GFX11 = 0x0000001d,
    ge_dist_tfreq_utcl1_request_event__GFX11           = 0x0000001e,
    ge_dist_tfreq_utcl1_retry_event__GFX11             = 0x0000001f,
    ge_dist_tfreq_utcl1_stall_event__GFX11             = 0x00000020,
    ge_dist_tfreq_utcl1_stall_utcl2_event__GFX11       = 0x00000021,
    ge_dist_tfreq_utcl1_translation_hit_event__GFX11   = 0x00000022,
    ge_dist_tfreq_utcl1_translation_miss_event__GFX11  = 0x00000023,
    ge_dist_pc_feorder_fifo_full__GFX11                = 0x00000024,
    ge_dist_pc_ge_manager_busy__GFX11                  = 0x00000025,
    ge_dist_sclk_input_vld__GFX11                      = 0x00000026,
    ge_dist_wd_te11_busy__GFX11                        = 0x00000027,
    ge_dist_te11_starved__GFX11                        = 0x00000028,
    ge_dist_switch_mode_stall__GFX11                   = 0x00000029,
    ge_all_tf_eq__GFX11                                = 0x0000002a,
    ge_all_tf2__GFX11                                  = 0x0000002b,
    ge_all_tf3__GFX11                                  = 0x0000002c,
    ge_all_tf4__GFX11                                  = 0x0000002d,
    ge_all_tf5__GFX11                                  = 0x0000002e,
    ge_all_tf6__GFX11                                  = 0x0000002f,
    ge_se0_te11_starved_on_hs_done__GFX11              = 0x00000030,
    ge_se1_te11_starved_on_hs_done__GFX11              = 0x00000031,
    ge_se2_te11_starved_on_hs_done__GFX11              = 0x00000032,
    ge_se3_te11_starved_on_hs_done__GFX11              = 0x00000033,
    ge_se4_te11_starved_on_hs_done__GFX11              = 0x00000034,
    ge_se5_te11_starved_on_hs_done__GFX11              = 0x00000035,
    ge_se6_te11_starved_on_hs_done__GFX11              = 0x00000036,
    ge_se7_te11_starved_on_hs_done__GFX11              = 0x00000037,
    ge_dist_op_fifo_full_starve__GFX11                 = 0x00000038,
    ge_dist_hs_done_se0__GFX11                         = 0x00000039,
    ge_dist_hs_done_se1__GFX11                         = 0x0000003a,
    ge_dist_hs_done_se2__GFX11                         = 0x0000003b,
    ge_dist_hs_done_se3__GFX11                         = 0x0000003c,
    ge_dist_hs_done_se4__GFX11                         = 0x0000003d,
    ge_dist_hs_done_se5__GFX11                         = 0x0000003e,
    ge_dist_hs_done_se6__GFX11                         = 0x0000003f,
    ge_dist_hs_done_se7__GFX11                         = 0x00000040,
    ge_dist_hs_done_latency__GFX11                     = 0x00000041,
    ge_dist_distributer_busy__GFX11                    = 0x00000042,
    ge_tf_ret_data_stalling_hs_done__GFX11             = 0x00000043,
    ge_num_of_no_dist_patches__GFX11                   = 0x00000044,
    ge_num_of_donut_dist_patches__GFX11                = 0x00000045,
    ge_num_of_patch_dist_patches__GFX11                = 0x00000046,
    ge_num_of_se_switches_due_to_patch_accum__GFX11    = 0x00000047,
    ge_num_of_se_switches_due_to_donut__GFX11          = 0x00000048,
    ge_num_of_se_switches_due_to_trap__GFX11           = 0x00000049,
    ge_num_of_hs_dealloc_events__GFX11                 = 0x0000004a,
    ge_agm_gcr_req__GFX11                              = 0x0000004b,
    ge_agm_gcr_tag_stall__GFX11                        = 0x0000004c,
    ge_agm_gcr_crd_stall__GFX11                        = 0x0000004d,
    ge_agm_gcr_stall__GFX11                            = 0x0000004e,
    ge_agm_gcr_latency__GFX11                          = 0x0000004f,
    ge_distclk_vld__GFX11                              = 0x00000050,
    ge_dist_indx_fifos_full_and_empty__GFX11           = 0x00000051,
    ge_hs_done_all_tf0_se0__GFX11                      = 0x00000052,
    ge_hs_done_all_tf0_se1__GFX11                      = 0x00000053,
    ge_hs_done_all_tf0_se2__GFX11                      = 0x00000054,
    ge_hs_done_all_tf0_se3__GFX11                      = 0x00000055,
    ge_hs_done_all_tf0_se4__GFX11                      = 0x00000056,
    ge_hs_done_all_tf0_se5__GFX11                      = 0x00000057,
    ge_hs_done_all_tf0_se6__GFX11                      = 0x00000058,
    ge_hs_done_all_tf0_se7__GFX11                      = 0x00000059,
    ge_hs_done_all_tf1_se0__GFX11                      = 0x0000005a,
    ge_hs_done_all_tf1_se1__GFX11                      = 0x0000005b,
    ge_hs_done_all_tf1_se2__GFX11                      = 0x0000005c,
    ge_hs_done_all_tf1_se3__GFX11                      = 0x0000005d,
    ge_hs_done_all_tf1_se4__GFX11                      = 0x0000005e,
    ge_hs_done_all_tf1_se5__GFX11                      = 0x0000005f,
    ge_hs_done_all_tf1_se6__GFX11                      = 0x00000060,
    ge_hs_done_all_tf1_se7__GFX11                      = 0x00000061,
    ge_agm_gcr_req_outstanding__GFX11                  = 0x00000062,
    ge_agm_gcr_req_amount__GFX11                       = 0x00000063,
    ge_agm_gcr_combine__GFX11                          = 0x00000064,
} GE2_DIST_PERFCOUNT_SELECT;

constexpr unsigned int MaxGe2DistPerfcountSelectGfx10Vrs = ge_tf_ret_data_stalling_hs_done__GFX10VRS;
constexpr unsigned int MaxGe2DistPerfcountSelectGfx11  = ge_agm_gcr_combine__GFX11;

typedef enum GE2_SE_PERFCOUNT_SELECT {
    ge_se_cm_reading_stalled__GFX10VRS                 = 0x00000000,
    ge_se_cm_stalled_by_gog__GFX10VRS                  = 0x00000001,
    ge_se_cm_stalled_by_gsfetch_done__GFX10VRS         = 0x00000002,
    ge_se_ds_cache_hits__GFX10VRS                      = 0x00000003,
    ge_se_ds_prims__GFX10VRS                           = 0x00000004,
    ge_se_es_done__GFX10VRS                            = 0x00000005,
    ge_se_es_done_latency__GFX10VRS                    = 0x00000006,
    ge_se_es_flush__GFX10VRS                           = 0x00000007,
    ge_se_es_ring_high_water_mark__GFX10VRS            = 0x00000008,
    ge_se_es_thread_groups__GFX10VRS                   = 0x00000009,
    ge_se_esthread_stalled_es_rb_full__GFX10VRS        = 0x0000000a,
    ge_se_esthread_stalled_spi_bp__GFX10VRS            = 0x0000000b,
    ge_se_esvert_stalled_gs_event__GFX10VRS            = 0x0000000c,
    ge_se_esvert_stalled_gs_tbl__GFX10VRS              = 0x0000000d,
    ge_se_esvert_stalled_gsprim__GFX10VRS              = 0x0000000e,
    ge_se_gog_busy__GFX10VRS                           = 0x0000000f,
    ge_se_gog_out_indx_stalled__GFX10VRS               = 0x00000010,
    ge_se_gog_out_prim_stalled__GFX10VRS               = 0x00000011,
    ge_se_gog_vs_tbl_stalled__GFX10VRS                 = 0x00000012,
    ge_se_gs_cache_hits__GFX10VRS                      = 0x00000013,
    ge_se_gs_counters_avail_stalled__GFX10VRS          = 0x00000014,
    ge_se_gs_done__GFX10VRS                            = 0x00000015,
    ge_se_gs_done_latency__GFX10VRS                    = 0x00000016,
    ge_se_gs_issue_rtr_stalled__GFX10VRS               = 0x00000017,
    ge_se_gs_rb_space_avail_stalled__GFX10VRS          = 0x00000018,
    ge_se_gs_ring_high_water_mark__GFX10VRS            = 0x00000019,
    ge_se_gsprim_stalled_es_tbl__GFX10VRS              = 0x0000001a,
    ge_se_gsprim_stalled_esvert__GFX10VRS              = 0x0000001b,
    ge_se_gsprim_stalled_gs_event__GFX10VRS            = 0x0000001c,
    ge_se_gsprim_stalled_gs_tbl__GFX10VRS              = 0x0000001d,
    ge_se_gsthread_stalled__GFX10VRS                   = 0x0000001e,
    ge_se_hs_tfm_stall__GFX10VRS                       = 0x0000001f,
    ge_se_hs_tgs_active_high_water_mark__GFX10VRS      = 0x00000020,
    ge_se_hs_thread_groups__GFX10VRS                   = 0x00000021,
    ge_se_pa0_clipp_eop__GFX10VRS                      = 0x00000022,
    ge_se_pa0_clipp_eopg__GFX10VRS                     = 0x00000023,
    ge_se_pa0_clipp_is_event__GFX10VRS                 = 0x00000024,
    ge_se_pa0_clipp_new_vtx_vect__GFX10VRS             = 0x00000025,
    ge_se_pa0_clipp_null_prim__GFX10VRS                = 0x00000026,
    ge_se_pa0_clipp_send__GFX10VRS                     = 0x00000027,
    ge_se_pa0_clipp_send_not_event__GFX10VRS           = 0x00000028,
    ge_se_pa0_clipp_stalled__GFX10VRS                  = 0x00000029,
    ge_se_pa0_clipp_starved_busy__GFX10VRS             = 0x0000002a,
    ge_se_pa0_clipp_starved_after_work__GFX10VRS       = 0x0000002b,
    ge_se_pa0_clipp_valid_prim__GFX10VRS               = 0x0000002c,
    ge_se_pa0_clips_send__GFX10VRS                     = 0x0000002d,
    ge_se_pa0_clips_stalled__GFX10VRS                  = 0x0000002e,
    ge_se_pa0_clipv_send__GFX10VRS                     = 0x0000002f,
    ge_se_pa0_clipv_stalled__GFX10VRS                  = 0x00000030,
    ge_se_pa1_clipp_eop__GFX10VRS                      = 0x00000031,
    ge_se_pa1_clipp_eopg__GFX10VRS                     = 0x00000032,
    ge_se_pa1_clipp_is_event__GFX10VRS                 = 0x00000033,
    ge_se_pa1_clipp_new_vtx_vect__GFX10VRS             = 0x00000034,
    ge_se_pa1_clipp_null_prim__GFX10VRS                = 0x00000035,
    ge_se_pa1_clipp_send__GFX10VRS                     = 0x00000036,
    ge_se_pa1_clipp_send_not_event__GFX10VRS           = 0x00000037,
    ge_se_pa1_clipp_stalled__GFX10VRS                  = 0x00000038,
    ge_se_pa1_clipp_starved_busy__GFX10VRS             = 0x00000039,
    ge_se_pa1_clipp_starved_after_work__GFX10VRS       = 0x0000003a,
    ge_se_pa1_clipp_valid_prim__GFX10VRS               = 0x0000003b,
    ge_se_pa1_clips_send__GFX10VRS                     = 0x0000003c,
    ge_se_pa1_clips_stalled__GFX10VRS                  = 0x0000003d,
    ge_se_pa1_clipv_send__GFX10VRS                     = 0x0000003e,
    ge_se_pa1_clipv_stalled__GFX10VRS                  = 0x0000003f,
    ge_se_reused_es_indices__GFX10VRS                  = 0x00000040,
    ge_se_reused_vs_indices__GFX10VRS                  = 0x00000041,
    ge_se_sclk_gs_vld__GFX10VRS                        = 0x00000042,
    ge_se_sclk_ngg_vld__GFX10VRS                       = 0x00000043,
    ge_se_sclk_te11_vld__GFX10VRS                      = 0x00000044,
    ge_se_sclk_vr_vld__GFX10VRS                        = 0x00000045,
    ge_se_spi_esvert_eov__GFX10VRS                     = 0x00000046,
    ge_se_spi_esvert_stalled__GFX10VRS                 = 0x00000047,
    ge_se_spi_esvert_starved_busy__GFX10VRS            = 0x00000048,
    ge_se_spi_esvert_valid__GFX10VRS                   = 0x00000049,
    ge_se_spi_eswave_is_event__GFX10VRS                = 0x0000004a,
    ge_se_spi_eswave_send__GFX10VRS                    = 0x0000004b,
    ge_se_spi_gsprim_cont__GFX10VRS                    = 0x0000004c,
    ge_se_spi_gsprim_eov__GFX10VRS                     = 0x0000004d,
    ge_se_spi_gsprim_stalled__GFX10VRS                 = 0x0000004e,
    ge_se_spi_gsprim_starved_busy__GFX10VRS            = 0x0000004f,
    ge_se_spi_gsprim_valid__GFX10VRS                   = 0x00000050,
    ge_se_spi_gssubgrp_is_event__GFX10VRS              = 0x00000051,
    ge_se_spi_gssubgrp_send__GFX10VRS                  = 0x00000052,
    ge_se_spi_hsvert_eov__GFX10VRS                     = 0x00000053,
    ge_se_spi_hsvert_stalled__GFX10VRS                 = 0x00000054,
    ge_se_spi_hsvert_starved_busy__GFX10VRS            = 0x00000055,
    ge_se_spi_hsvert_valid__GFX10VRS                   = 0x00000056,
    ge_se_spi_hswave_is_event__GFX10VRS                = 0x00000057,
    ge_se_spi_hswave_send__GFX10VRS                    = 0x00000058,
    ge_se_spi_lsvert_eov__GFX10VRS                     = 0x00000059,
    ge_se_spi_lsvert_stalled__GFX10VRS                 = 0x0000005a,
    ge_se_spi_lsvert_starved_busy__GFX10VRS            = 0x0000005b,
    ge_se_spi_lsvert_valid__GFX10VRS                   = 0x0000005c,
    ge_se_spi_hsvert_fifo_full_stall__GFX10VRS         = 0x0000005d,
    ge_se_spi_tgrp_fifo_stall__GFX10VRS                = 0x0000005e,
    ge_se_spi_vsvert_eov__GFX10VRS                     = 0x0000005f,
    ge_se_spi_vsvert_send__GFX10VRS                    = 0x00000060,
    ge_se_spi_vsvert_stalled__GFX10VRS                 = 0x00000061,
    ge_se_spi_vsvert_starved_busy__GFX10VRS            = 0x00000062,
    ge_se_spi_vswave_is_event__GFX10VRS                = 0x00000063,
    ge_se_spi_vswave_send__GFX10VRS                    = 0x00000064,
    ge_spi_hsgrp_spi_stall__GFX10VRS                   = 0x00000065,
    ge_se_rcm_indicies_hit__GFX10VRS                   = 0x00000066,
    ge_se_vs_cache_hits__GFX10VRS                      = 0x00000067,
    ge_se_vs_done__GFX10VRS                            = 0x00000068,
    ge_se_vs_table_high_water_mark__GFX10VRS           = 0x00000069,
    ge_se_vs_thread_groups__GFX10VRS                   = 0x0000006a,
    ge_se_vsvert_api_send__GFX10VRS                    = 0x0000006b,
    ge_se_vsvert_ds_send__GFX10VRS                     = 0x0000006c,
    ge_se_wait_for_es_done_stalled__GFX10VRS           = 0x0000006d,
    ge_se_waveid_stalled__GFX10VRS                     = 0x0000006e,
    ge_se_spi_vsvert_valid__GFX10VRS                   = 0x0000006f,
    ge_se_spi_gssubgrp_event_window_active__GFX10VRS   = 0x00000070,
    ge_se_hs_input_stall__GFX10VRS                     = 0x00000071,
    vgt_se_pa0_clipv_starved_busy__GFX10VRS            = 0x00000072,
    vgt_se_pa0_clipv_firstvert__GFX10VRS               = 0x00000073,
    vgt_se_pa0_clips_starved_busy__GFX10VRS            = 0x00000074,
    vgt_se_pa1_clipv_starved_busy__GFX10VRS            = 0x00000075,
    vgt_se_pa1_clipv_firstvert__GFX10VRS               = 0x00000076,
    vgt_se_pa1_clips_starved_busy__GFX10VRS            = 0x00000077,
    ge_se_sending_vert_or_prim__GFX10VRS               = 0x00000078,
    ge_se_sclk_input_vld__GFX10VRS                     = 0x00000079,
    ge_spi_lswave_fifo_full_stall__GFX10VRS            = 0x0000007a,
    ge_spi_hswave_fifo_full_stall__GFX10VRS            = 0x0000007b,
    ge_hs_tif_stall__GFX10VRS                          = 0x0000007c,
    ge_csb_spi_bp__GFX10VRS                            = 0x0000007d,
    ge_ngg_starving_for_pc_grant__GFX10VRS             = 0x0000007e,
    ge_vs_starving_for_pc_grant__GFX10VRS              = 0x0000007f,
    ge_se_strmout_stalled__GFX10VRS                    = 0x00000080,
    ge_pa0_csb_eop__GFX10VRS                           = 0x00000081,
    ge_pa1_csb_eop__GFX10VRS                           = 0x00000082,
    ge_ngg_starved_idle__GFX10VRS                      = 0x00000083,
    ge_gsprim_send__GFX10VRS                           = 0x00000084,
    ge_esvert_send__GFX10VRS                           = 0x00000085,
    ge_ngg_starved_after_work__GFX10VRS                = 0x00000086,
    ge_ngg_subgrp_fifo_stall__GFX10VRS                 = 0x00000087,
    ge_ngg_ord_id_req_stall__GFX10VRS                  = 0x00000088,
    ge_ngg_indx_bus_stall__GFX10VRS                    = 0x00000089,
    ge_hs_stall_tfmm_fifo_full__GFX10VRS               = 0x0000008a,
    ge_se_ds_prims__GFX11                              = 0x00000000,
    ge_se_es_thread_groups__GFX11                      = 0x00000001,
    ge_se_esvert_stalled_gsprim__GFX11                 = 0x00000002,
    ge_se_hs_tfm_stall__GFX11                          = 0x00000003,
    ge_se_hs_tgs_active_high_water_mark__GFX11         = 0x00000004,
    ge_se_hs_thread_groups__GFX11                      = 0x00000005,
    ge_se_reused_es_indices__GFX11                     = 0x00000006,
    ge_se_sclk_ngg_vld__GFX11                          = 0x00000007,
    ge_se_sclk_te11_vld__GFX11                         = 0x00000008,
    ge_se_spi_esvert_eov__GFX11                        = 0x00000009,
    ge_se_spi_esvert_stalled__GFX11                    = 0x0000000a,
    ge_se_spi_esvert_starved_busy__GFX11               = 0x0000000b,
    ge_se_spi_esvert_valid__GFX11                      = 0x0000000c,
    ge_se_spi_gsprim_cont__GFX11                       = 0x0000000d,
    ge_se_spi_gsprim_eov__GFX11                        = 0x0000000e,
    ge_se_spi_gsprim_stalled__GFX11                    = 0x0000000f,
    ge_se_spi_gsprim_starved_busy__GFX11               = 0x00000010,
    ge_se_spi_gsprim_valid__GFX11                      = 0x00000011,
    ge_se_spi_gssubgrp_is_event__GFX11                 = 0x00000012,
    ge_se_spi_gssubgrp_send__GFX11                     = 0x00000013,
    ge_se_spi_hsvert_eov__GFX11                        = 0x00000014,
    ge_se_spi_hsvert_stalled__GFX11                    = 0x00000015,
    ge_se_spi_hsvert_starved_busy__GFX11               = 0x00000016,
    ge_se_spi_hsvert_valid__GFX11                      = 0x00000017,
    ge_se_spi_hsgrp_is_event__GFX11                    = 0x00000018,
    ge_se_spi_hsgrp_send__GFX11                        = 0x00000019,
    ge_se_spi_lsvert_eov__GFX11                        = 0x0000001a,
    ge_se_spi_lsvert_stalled__GFX11                    = 0x0000001b,
    ge_se_spi_lsvert_starved_busy__GFX11               = 0x0000001c,
    ge_se_spi_lsvert_valid__GFX11                      = 0x0000001d,
    ge_se_spi_hsvert_fifo_full_stall__GFX11            = 0x0000001e,
    ge_se_spi_tgrp_fifo_stall__GFX11                   = 0x0000001f,
    ge_spi_hsgrp_spi_stall__GFX11                      = 0x00000020,
    ge_se_spi_gssubgrp_event_window_active__GFX11      = 0x00000021,
    ge_se_hs_input_stall__GFX11                        = 0x00000022,
    ge_se_sending_vert_or_prim__GFX11                  = 0x00000023,
    ge_se_sclk_input_vld__GFX11                        = 0x00000024,
    ge_spi_lswave_fifo_full_stall__GFX11               = 0x00000025,
    ge_spi_hswave_fifo_full_stall__GFX11               = 0x00000026,
    ge_hs_tif_stall__GFX11                             = 0x00000027,
    ge_csb_spi_bp__GFX11                               = 0x00000028,
    ge_ngg_starving_for_wave_id__GFX11                 = 0x00000029,
    ge_pa0_csb_eop__GFX11                              = 0x0000002a,
    ge_ngg_starved_idle__GFX11                         = 0x0000002b,
    ge_gsprim_send__GFX11                              = 0x0000002c,
    ge_esvert_send__GFX11                              = 0x0000002d,
    ge_ngg_starved_after_work__GFX11                   = 0x0000002e,
    ge_ngg_subgrp_fifo_stall__GFX11                    = 0x0000002f,
    ge_ngg_ord_id_req_stall__GFX11                     = 0x00000030,
    ge_ngg_indx_bus_stall__GFX11                       = 0x00000031,
    ge_hs_stall_tfmm_fifo_full__GFX11                  = 0x00000032,
    ge_gs_issue_rtr_stalled__GFX11                     = 0x00000033,
    ge_gsprim_stalled_esvert__GFX11                    = 0x00000034,
    ge_gsthread_stalled__GFX11                         = 0x00000035,
    ge_ngg_attr_grp_alloc__GFX11                       = 0x00000036,
    ge_ngg_attr_discard_alloc__GFX11                   = 0x00000037,
    ge_ngg_pc_space_not_avail__GFX11                   = 0x00000038,
    ge_ngg_agm_req_stall__GFX11                        = 0x00000039,
    ge_ngg_spi_esvert_partial_eov__GFX11               = 0x0000003a,
    ge_ngg_spi_gsprim_partial_eov__GFX11               = 0x0000003b,
    ge_spi_gsgrp_valid__GFX11                          = 0x0000003c,
    ge_ngg_attr_grp_latency__GFX11                     = 0x0000003d,
    ge_ngg_reuse_prim_limit_hit__GFX11                 = 0x0000003e,
    ge_ngg_reuse_vert_limit_hit__GFX11                 = 0x0000003f,
    ge_te11_con_stall__GFX11                           = 0x00000040,
    ge_te11_compactor_starved__GFX11                   = 0x00000041,
    ge_ngg_stall_tess_off_tess_on__GFX11               = 0x00000042,
    ge_ngg_stall_tess_on_tess_off__GFX11               = 0x00000043,
    ge_merged_lses_vert_stalled__GFX11                 = 0x00000044,
    ge_merged_hsgs_vert_stalled__GFX11                 = 0x00000045,
    ge_merged_hsgs_grp_stalled__GFX11                  = 0x00000046,
    ge_merge_lses_fifo_blocked__GFX11                  = 0x00000047,
    ge_merge_hsgs_fifo_blocked__GFX11                  = 0x00000048,
    ge_merge_lses_vert_switch__GFX11                   = 0x00000049,
    ge_merge_hsgs_vert_switch__GFX11                   = 0x0000004a,
    ge_merge_hsgs_grp_switch__GFX11                    = 0x0000004b,
    ge_merge_gsgrp_rdy_pending_verts__GFX11            = 0x0000004c,
    ge_merge_hsgrp_rdy_pending_verts__GFX11            = 0x0000004d,
    ge_se_ds_cache_hits__GFX11                         = 0x0000004e,
    ge_se_api_vs_verts__GFX11                          = 0x0000004f,
    ge_se_api_ds_verts__GFX11                          = 0x00000050,
    ge_se_combined_busy__GFX11                         = 0x00000051,
    ge_spi_lsvert_send__GFX11                          = 0x00000052,
    ge_spi_hsvert_send__GFX11                          = 0x00000053,
    ge_ngg_attr_grp_wasted__GFX11                      = 0x00000054,
    ge_spi_gssubgrp_stalled__GFX11                     = 0x00000055,
    ge_ngg_attr_null_dealloc__GFX11                    = 0x00000056,
    ge_ngg_busy_base__GFX11                            = 0x00000057,
} GE2_SE_PERFCOUNT_SELECT;

constexpr unsigned int MaxGe2SePerfcountSelectGfx10Vrs = ge_hs_stall_tfmm_fifo_full__GFX10VRS;
constexpr unsigned int MaxGe2SePerfcountSelectGfx11    = ge_ngg_busy_base__GFX11;

typedef enum GE_PERFCOUNT_SELECT {
    ge_assembler_busy                                  = 0x00000000,
    ge_assembler_stalled                               = 0x00000001,
    ge_cm_reading_stalled                              = 0x00000002,
    ge_cm_stalled_by_gog                               = 0x00000003,
    ge_cm_stalled_by_gsfetch_done                      = 0x00000004,
    ge_dma_busy                                        = 0x00000005,
    ge_dma_lat_bin_0                                   = 0x00000006,
    ge_dma_lat_bin_1                                   = 0x00000007,
    ge_dma_lat_bin_2                                   = 0x00000008,
    ge_dma_lat_bin_3                                   = 0x00000009,
    ge_dma_lat_bin_4                                   = 0x0000000a,
    ge_dma_lat_bin_5                                   = 0x0000000b,
    ge_dma_lat_bin_6                                   = 0x0000000c,
    ge_dma_lat_bin_7                                   = 0x0000000d,
    ge_dma_return_cl0                                  = 0x0000000e,
    ge_dma_return_cl1                                  = 0x0000000f,
    ge_dma_utcl1_consecutive_retry_event               = 0x00000010,
    ge_dma_utcl1_request_event                         = 0x00000011,
    ge_dma_utcl1_retry_event                           = 0x00000012,
    ge_dma_utcl1_stall_event                           = 0x00000013,
    ge_dma_utcl1_stall_utcl2_event                     = 0x00000014,
    ge_dma_utcl1_translation_hit_event                 = 0x00000015,
    ge_dma_utcl1_translation_miss_event                = 0x00000016,
    ge_ds_cache_hits                                   = 0x00000017,
    ge_ds_prims                                        = 0x00000018,
    ge_es_done                                         = 0x00000019,
    ge_es_done_latency                                 = 0x0000001a,
    ge_es_flush                                        = 0x0000001b,
    ge_es_ring_high_water_mark                         = 0x0000001c,
    ge_es_thread_groups                                = 0x0000001d,
    ge_esthread_stalled_es_rb_full                     = 0x0000001e,
    ge_esthread_stalled_spi_bp                         = 0x0000001f,
    ge_esvert_stalled_gs_event                         = 0x00000020,
    ge_esvert_stalled_gs_tbl                           = 0x00000021,
    ge_esvert_stalled_gsprim                           = 0x00000022,
    ge_assembler_dma_starved                           = 0x00000023,
    ge_gog_busy                                        = 0x00000024,
    ge_gog_out_indx_stalled                            = 0x00000025,
    ge_gog_out_prim_stalled                            = 0x00000026,
    ge_gog_vs_tbl_stalled                              = 0x00000027,
    ge_gs_cache_hits                                   = 0x00000028,
    ge_gs_counters_avail_stalled                       = 0x00000029,
    ge_gs_done                                         = 0x0000002a,
    ge_gs_done_latency                                 = 0x0000002b,
    ge_gs_issue_rtr_stalled                            = 0x0000002c,
    ge_gs_rb_space_avail_stalled                       = 0x0000002d,
    ge_gs_ring_high_water_mark                         = 0x0000002e,
    ge_gsprim_stalled_es_tbl                           = 0x0000002f,
    ge_gsprim_stalled_esvert                           = 0x00000030,
    ge_gsprim_stalled_gs_event                         = 0x00000031,
    ge_gsprim_stalled_gs_tbl                           = 0x00000032,
    ge_gsthread_stalled                                = 0x00000033,
    ge_hs_done                                         = 0x00000034,
    ge_hs_done_latency                                 = 0x00000035,
    ge_hs_done_se0                                     = 0x00000036,
    ge_hs_done_se1                                     = 0x00000037,
    ge_hs_done_se2_reserved                            = 0x00000038,
    ge_hs_done_se3_reserved                            = 0x00000039,
    ge_hs_tfm_stall                                    = 0x0000003a,
    ge_hs_tgs_active_high_water_mark                   = 0x0000003b,
    ge_hs_thread_groups                                = 0x0000003c,
    ge_inside_tf_bin_0                                 = 0x0000003d,
    ge_inside_tf_bin_1                                 = 0x0000003e,
    ge_inside_tf_bin_2                                 = 0x0000003f,
    ge_inside_tf_bin_3                                 = 0x00000040,
    ge_inside_tf_bin_4                                 = 0x00000041,
    ge_inside_tf_bin_5                                 = 0x00000042,
    ge_inside_tf_bin_6                                 = 0x00000043,
    ge_inside_tf_bin_7                                 = 0x00000044,
    ge_inside_tf_bin_8                                 = 0x00000045,
    ge_ls_done                                         = 0x00000046,
    ge_ls_done_latency                                 = 0x00000047,
    ge_null_patch                                      = 0x00000048,
    ge_se0pa0_clipp_eop                                = 0x00000049,
    ge_se0pa0_clipp_eopg                               = 0x0000004a,
    ge_se0pa0_clipp_is_event                           = 0x0000004b,
    ge_se0pa0_clipp_new_vtx_vect                       = 0x0000004c,
    ge_se0pa0_clipp_null_prim                          = 0x0000004d,
    ge_se0pa0_clipp_send                               = 0x0000004e,
    ge_se0pa0_clipp_send_not_event                     = 0x0000004f,
    ge_se0pa0_clipp_stalled                            = 0x00000050,
    ge_se0pa0_clipp_starved_busy                       = 0x00000051,
    ge_se0pa0_clipp_starved_after_work                 = 0x00000052,
    ge_se0pa0_clipp_valid_prim                         = 0x00000053,
    ge_se0pa0_clips_send                               = 0x00000054,
    ge_se0pa0_clips_stalled                            = 0x00000055,
    ge_se0pa0_clipv_send                               = 0x00000056,
    ge_se0pa0_clipv_stalled                            = 0x00000057,
    ge_se0pa1_clipp_eop                                = 0x00000058,
    ge_se0pa1_clipp_eopg                               = 0x00000059,
    ge_se0pa1_clipp_is_event                           = 0x0000005a,
    ge_se0pa1_clipp_new_vtx_vect                       = 0x0000005b,
    ge_se0pa1_clipp_null_prim                          = 0x0000005c,
    ge_se0pa1_clipp_send                               = 0x0000005d,
    ge_se0pa1_clipp_send_not_event                     = 0x0000005e,
    ge_se0pa1_clipp_stalled                            = 0x0000005f,
    ge_se0pa1_clipp_starved_busy                       = 0x00000060,
    ge_se0pa1_clipp_starved_after_work                 = 0x00000061,
    ge_se0pa1_clipp_valid_prim                         = 0x00000062,
    ge_se0pa1_clips_send                               = 0x00000063,
    ge_se0pa1_clips_stalled                            = 0x00000064,
    ge_se0pa1_clipv_send                               = 0x00000065,
    ge_se0pa1_clipv_stalled                            = 0x00000066,
    ge_se1pa0_clipp_eop                                = 0x00000067,
    ge_se1pa0_clipp_eopg                               = 0x00000068,
    ge_se1pa0_clipp_is_event                           = 0x00000069,
    ge_se1pa0_clipp_new_vtx_vect                       = 0x0000006a,
    ge_se1pa0_clipp_null_prim                          = 0x0000006b,
    ge_se1pa0_clipp_send                               = 0x0000006c,
    ge_se1pa0_clipp_send_not_event                     = 0x0000006d,
    ge_se1pa0_clipp_stalled                            = 0x0000006e,
    ge_se1pa0_clipp_starved_busy                       = 0x0000006f,
    ge_se1pa0_clipp_starved_after_work                 = 0x00000070,
    ge_se1pa0_clipp_valid_prim                         = 0x00000071,
    ge_se1pa0_clips_send                               = 0x00000072,
    ge_se1pa0_clips_stalled                            = 0x00000073,
    ge_se1pa0_clipv_send                               = 0x00000074,
    ge_se1pa0_clipv_stalled                            = 0x00000075,
    ge_se1pa1_clipp_eop                                = 0x00000076,
    ge_se1pa1_clipp_eopg                               = 0x00000077,
    ge_se1pa1_clipp_is_event                           = 0x00000078,
    ge_se1pa1_clipp_new_vtx_vect                       = 0x00000079,
    ge_se1pa1_clipp_null_prim                          = 0x0000007a,
    ge_se1pa1_clipp_send                               = 0x0000007b,
    ge_se1pa1_clipp_send_not_event                     = 0x0000007c,
    ge_se1pa1_clipp_stalled                            = 0x0000007d,
    ge_se1pa1_clipp_starved_busy                       = 0x0000007e,
    ge_se1pa1_clipp_starved_after_work                 = 0x0000007f,
    ge_se1pa1_clipp_valid_prim                         = 0x00000080,
    ge_se1pa1_clips_send                               = 0x00000081,
    ge_se1pa1_clips_stalled                            = 0x00000082,
    ge_se1pa1_clipv_send                               = 0x00000083,
    ge_se1pa1_clipv_stalled                            = 0x00000084,
    ge_se2pa0_clipp_eop                                = 0x00000085,
    ge_se2pa0_clipp_eopg                               = 0x00000086,
    ge_se2pa0_clipp_is_event                           = 0x00000087,
    ge_se2pa0_clipp_new_vtx_vect                       = 0x00000088,
    ge_se2pa0_clipp_null_prim                          = 0x00000089,
    ge_se2pa0_clipp_send                               = 0x0000008a,
    ge_se2pa0_clipp_send_not_event                     = 0x0000008b,
    ge_se2pa0_clipp_stalled                            = 0x0000008c,
    ge_se2pa0_clipp_starved_busy                       = 0x0000008d,
    ge_se2pa0_clipp_starved_after_work                 = 0x0000008e,
    ge_se2pa0_clipp_valid_prim                         = 0x0000008f,
    ge_se2pa0_clips_send                               = 0x00000090,
    ge_se2pa0_clips_stalled                            = 0x00000091,
    ge_se2pa0_clipv_send                               = 0x00000092,
    ge_se2pa0_clipv_stalled                            = 0x00000093,
    ge_se2pa1_clipp_eop                                = 0x00000094,
    ge_se2pa1_clipp_eopg                               = 0x00000095,
    ge_se2pa1_clipp_is_event                           = 0x00000096,
    ge_se2pa1_clipp_new_vtx_vect                       = 0x00000097,
    ge_se2pa1_clipp_null_prim                          = 0x00000098,
    ge_se2pa1_clipp_send                               = 0x00000099,
    ge_se2pa1_clipp_send_not_event                     = 0x0000009a,
    ge_se2pa1_clipp_stalled                            = 0x0000009b,
    ge_se2pa1_clipp_starved_busy                       = 0x0000009c,
    ge_se2pa1_clipp_starved_after_work                 = 0x0000009d,
    ge_se2pa1_clipp_valid_prim                         = 0x0000009e,
    ge_se2pa1_clips_send                               = 0x0000009f,
    ge_se2pa1_clips_stalled                            = 0x000000a0,
    ge_se2pa1_clipv_send                               = 0x000000a1,
    ge_se2pa1_clipv_stalled                            = 0x000000a2,
    ge_se3pa0_clipp_eop                                = 0x000000a3,
    ge_se3pa0_clipp_eopg                               = 0x000000a4,
    ge_se3pa0_clipp_is_event                           = 0x000000a5,
    ge_se3pa0_clipp_new_vtx_vect                       = 0x000000a6,
    ge_se3pa0_clipp_null_prim                          = 0x000000a7,
    ge_se3pa0_clipp_send                               = 0x000000a8,
    ge_se3pa0_clipp_send_not_event                     = 0x000000a9,
    ge_se3pa0_clipp_stalled                            = 0x000000aa,
    ge_se3pa0_clipp_starved_busy                       = 0x000000ab,
    ge_se3pa0_clipp_starved_after_work                 = 0x000000ac,
    ge_se3pa0_clipp_valid_prim                         = 0x000000ad,
    ge_se3pa0_clips_send                               = 0x000000ae,
    ge_se3pa0_clips_stalled                            = 0x000000af,
    ge_se3pa0_clipv_send                               = 0x000000b0,
    ge_se3pa0_clipv_stalled                            = 0x000000b1,
    ge_se3pa1_clipp_eop                                = 0x000000b2,
    ge_se3pa1_clipp_eopg                               = 0x000000b3,
    ge_se3pa1_clipp_is_event                           = 0x000000b4,
    ge_se3pa1_clipp_new_vtx_vect                       = 0x000000b5,
    ge_se3pa1_clipp_null_prim                          = 0x000000b6,
    ge_se3pa1_clipp_send                               = 0x000000b7,
    ge_se3pa1_clipp_send_not_event                     = 0x000000b8,
    ge_se3pa1_clipp_stalled                            = 0x000000b9,
    ge_se3pa1_clipp_starved_busy                       = 0x000000ba,
    ge_se3pa1_clipp_starved_after_work                 = 0x000000bb,
    ge_se3pa1_clipp_valid_prim                         = 0x000000bc,
    ge_se3pa1_clips_send                               = 0x000000bd,
    ge_se3pa1_clips_stalled                            = 0x000000be,
    ge_se3pa1_clipv_send                               = 0x000000bf,
    ge_se3pa1_clipv_stalled                            = 0x000000c0,
    ge_rbiu_di_fifo_stalled                            = 0x000000c1,
    ge_rbiu_di_fifo_starved                            = 0x000000c2,
    ge_rbiu_dr_fifo_stalled                            = 0x000000c3,
    ge_rbiu_dr_fifo_starved                            = 0x000000c4,
    ge_reused_es_indices                               = 0x000000c5,
    ge_reused_vs_indices                               = 0x000000c6,
    ge_sclk_core_vld                                   = 0x000000c7,
    ge_sclk_gs_vld                                     = 0x000000c8,
    ge_sclk_input_vld                                  = 0x000000c9,
    ge_sclk_leg_gs_arb_vld                             = 0x000000ca,
    ge_sclk_ngg_vld                                    = 0x000000cb,
    ge_sclk_reg_vld                                    = 0x000000cc,
    ge_sclk_te11_vld                                   = 0x000000cd,
    ge_sclk_vr_vld                                     = 0x000000ce,
    ge_sclk_wd_te11_vld                                = 0x000000cf,
    ge_spi_esvert_eov                                  = 0x000000d0,
    ge_spi_esvert_stalled                              = 0x000000d1,
    ge_spi_esvert_starved_busy                         = 0x000000d2,
    ge_spi_esvert_valid                                = 0x000000d3,
    ge_spi_eswave_is_event                             = 0x000000d4,
    ge_spi_eswave_send                                 = 0x000000d5,
    ge_se0spi_gsprim_cont                              = 0x000000d6,
    ge_se1spi_gsprim_cont                              = 0x000000d7,
    ge_se2spi_gsprim_cont                              = 0x000000d8,
    ge_se3spi_gsprim_cont                              = 0x000000d9,
    ge_spi_gsprim_eov                                  = 0x000000da,
    ge_spi_gsprim_stalled                              = 0x000000db,
    ge_spi_gsprim_starved_busy                         = 0x000000dc,
    ge_spi_gsprim_valid                                = 0x000000dd,
    ge_spi_gssubgrp_is_event                           = 0x000000de,
    ge_spi_gssubgrp_send                               = 0x000000df,
    ge_spi_hsvert_eov                                  = 0x000000e0,
    ge_spi_hsvert_stalled                              = 0x000000e1,
    ge_spi_hsvert_starved_busy                         = 0x000000e2,
    ge_spi_hsvert_valid                                = 0x000000e3,
    ge_spi_hswave_is_event                             = 0x000000e4,
    ge_spi_hswave_send                                 = 0x000000e5,
    ge_spi_lsvert_eov                                  = 0x000000e6,
    ge_spi_lsvert_stalled                              = 0x000000e7,
    ge_spi_lsvert_starved_busy                         = 0x000000e8,
    ge_spi_lsvert_valid                                = 0x000000e9,
    ge_spi_lswave_is_event                             = 0x000000ea,
    ge_spi_lswave_send                                 = 0x000000eb,
    ge_spi_vsvert_eov                                  = 0x000000ec,
    ge_spi_vsvert_send                                 = 0x000000ed,
    ge_spi_vsvert_stalled                              = 0x000000ee,
    ge_spi_vsvert_starved_busy                         = 0x000000ef,
    ge_spi_vswave_is_event                             = 0x000000f0,
    ge_spi_vswave_send                                 = 0x000000f1,
    ge_starved_on_hs_done                              = 0x000000f2,
    ge_stat_busy                                       = 0x000000f3,
    ge_stat_combined_busy                              = 0x000000f4,
    ge_stat_no_dma_busy                                = 0x000000f5,
    ge_strmout_stalled                                 = 0x000000f6,
    ge_te11_busy                                       = 0x000000f7,
    ge_te11_starved                                    = 0x000000f8,
    ge_tfreq_lat_bin_0                                 = 0x000000f9,
    ge_tfreq_lat_bin_1                                 = 0x000000fa,
    ge_tfreq_lat_bin_2                                 = 0x000000fb,
    ge_tfreq_lat_bin_3                                 = 0x000000fc,
    ge_tfreq_lat_bin_4                                 = 0x000000fd,
    ge_tfreq_lat_bin_5                                 = 0x000000fe,
    ge_tfreq_lat_bin_6                                 = 0x000000ff,
    ge_tfreq_lat_bin_7                                 = 0x00000100,
    ge_tfreq_utcl1_consecutive_retry_event             = 0x00000101,
    ge_tfreq_utcl1_request_event                       = 0x00000102,
    ge_tfreq_utcl1_retry_event                         = 0x00000103,
    ge_tfreq_utcl1_stall_event                         = 0x00000104,
    ge_tfreq_utcl1_stall_utcl2_event                   = 0x00000105,
    ge_tfreq_utcl1_translation_hit_event               = 0x00000106,
    ge_tfreq_utcl1_translation_miss_event              = 0x00000107,
    ge_ls_thread_group                                 = 0x00000108,
    ge_rcm_indicies_hit                                = 0x00000109,
    ge_vs_cache_hits                                   = 0x0000010a,
    ge_vs_done                                         = 0x0000010b,
    ge_vs_pc_stall                                     = 0x0000010c,
    ge_vs_table_high_water_mark                        = 0x0000010d,
    ge_vs_thread_groups                                = 0x0000010e,
    ge_vsvert_api_send                                 = 0x0000010f,
    ge_vsvert_ds_send                                  = 0x00000110,
    ge_wait_for_es_done_stalled                        = 0x00000111,
    ge_waveid_stalled                                  = 0x00000112,
    ge_spi_vsvert_valid                                = 0x00000113,
    pc_feorder_fifo_full                               = 0x00000114,
    pc_ge_manager_busy                                 = 0x00000115,
    pc_req_stall_se0                                   = 0x00000116,
    pc_req_stall_se1                                   = 0x00000117,
    pc_req_stall_se2                                   = 0x00000118,
    pc_req_stall_se3                                   = 0x00000119,
    ge_pipe0_to_pipe1                                  = 0x0000011a,
    ge_pipe1_to_pipe0                                  = 0x0000011b,
    ge_dma_return_size_cl0                             = 0x0000011c,
    ge_dma_return_size_cl1                             = 0x0000011d,
    ge_spi_gssubgrp_event_window_active                = 0x0000011e,
    ge_bypass_fifo_full                                = 0x0000011f,
    ge_hs_input_stall0                                 = 0x00000120,
    ge_hs_input_stall1                                 = 0x00000121,
    ge_pc_space_zero                                   = 0x00000122,
    vgt_se0pa0_clipv_starved_busy                      = 0x00000123,
    vgt_se0pa1_clipv_starved_busy                      = 0x00000124,
    vgt_se1pa0_clipv_starved_busy                      = 0x00000125,
    vgt_se1pa1_clipv_starved_busy                      = 0x00000126,
    vgt_se2pa0_clipv_starved_busy                      = 0x00000127,
    vgt_se2pa1_clipv_starved_busy                      = 0x00000128,
    vgt_se3pa0_clipv_starved_busy                      = 0x00000129,
    vgt_se3pa1_clipv_starved_busy                      = 0x0000012a,
    vgt_se0pa0_clipv_firstvert                         = 0x0000012b,
    vgt_se0pa1_clipv_firstvert                         = 0x0000012c,
    vgt_se1pa0_clipv_firstvert                         = 0x0000012d,
    vgt_se1pa1_clipv_firstvert                         = 0x0000012e,
    vgt_se2pa0_clipv_firstvert                         = 0x0000012f,
    vgt_se2pa1_clipv_firstvert                         = 0x00000130,
    vgt_se3pa0_clipv_firstvert                         = 0x00000131,
    vgt_se3pa1_clipv_firstvert                         = 0x00000132,
    vgt_se0pa0_clips_starved_busy                      = 0x00000133,
    vgt_se0pa1_clips_starved_busy                      = 0x00000134,
    vgt_se1pa0_clips_starved_busy                      = 0x00000135,
    vgt_se1pa1_clips_starved_busy                      = 0x00000136,
    vgt_se2pa0_clips_starved_busy                      = 0x00000137,
    vgt_se2pa1_clips_starved_busy                      = 0x00000138,
    vgt_se3pa0_clips_starved_busy                      = 0x00000139,
    vgt_se3pa1_clips_starved_busy                      = 0x0000013a,
} GE_PERFCOUNT_SELECT;

constexpr unsigned int MaxGePerfcountSelect            = vgt_se3pa1_clips_starved_busy;

typedef enum GL0V_CACHE_POLICIES {
    GL0V_CACHE_POLICY_MISS_LRU                         = 0x00000000,
    GL0V_CACHE_POLICY_MISS_EVICT                       = 0x00000001,
    GL0V_CACHE_POLICY_HIT_LRU                          = 0x00000002,
    GL0V_CACHE_POLICY_HIT_EVICT                        = 0x00000003,
} GL0V_CACHE_POLICIES;

typedef enum GL1A_PERF_SEL {
    GL1A_PERF_SEL_BUSY                                 = 0x00000000,
    GL1A_PERF_SEL_STALL_GL1C0                          = 0x00000001,
    GL1A_PERF_SEL_STALL_GL1C1                          = 0x00000002,
    GL1A_PERF_SEL_STALL_GL1C2                          = 0x00000003,
    GL1A_PERF_SEL_STALL_GL1C3                          = 0x00000004,
    GL1A_PERF_SEL_REQUEST_GL1C0                        = 0x00000005,
    GL1A_PERF_SEL_REQUEST_GL1C1                        = 0x00000006,
    GL1A_PERF_SEL_REQUEST_GL1C2                        = 0x00000007,
    GL1A_PERF_SEL_REQUEST_GL1C3                        = 0x00000008,
    GL1A_PERF_SEL_MEM_32B_WDS_GL1C0__GFX101            = 0x00000009,
    GL1A_PERF_SEL_MEM_32B_WDS_GL1C1__GFX101            = 0x0000000a,
    GL1A_PERF_SEL_MEM_32B_WDS_GL1C2__GFX101            = 0x0000000b,
    GL1A_PERF_SEL_MEM_32B_WDS_GL1C3__GFX101            = 0x0000000c,
    GL1A_PERF_SEL_IO_32B_WDS_GL1C0__GFX101             = 0x0000000d,
    GL1A_PERF_SEL_IO_32B_WDS_GL1C1__GFX101             = 0x0000000e,
    GL1A_PERF_SEL_IO_32B_WDS_GL1C2__GFX101             = 0x0000000f,
    GL1A_PERF_SEL_IO_32B_WDS_GL1C3__GFX101             = 0x00000010,
    GL1A_PERF_SEL_MEM_BURST_COUNT_GL1C0__GFX101        = 0x00000011,
    GL1A_PERF_SEL_MEM_BURST_COUNT_GL1C1__GFX101        = 0x00000012,
    GL1A_PERF_SEL_MEM_BURST_COUNT_GL1C2__GFX101        = 0x00000013,
    GL1A_PERF_SEL_MEM_BURST_COUNT_GL1C3__GFX101        = 0x00000014,
    GL1A_PERF_SEL_IO_BURST_COUNT_GL1C0__GFX101         = 0x00000015,
    GL1A_PERF_SEL_IO_BURST_COUNT_GL1C1__GFX101         = 0x00000016,
    GL1A_PERF_SEL_IO_BURST_COUNT_GL1C2__GFX101         = 0x00000017,
    GL1A_PERF_SEL_IO_BURST_COUNT_GL1C3__GFX101         = 0x00000018,
    GL1A_PERF_SEL_ARB_REQUESTS__GFX101                 = 0x00000019,
    GL1A_PERF_SEL_REQ_ARB_LEVEL_GL1C0__GFX101          = 0x0000001a,
    GL1A_PERF_SEL_REQ_ARB_LEVEL_GL1C1__GFX101          = 0x0000001b,
    GL1A_PERF_SEL_REQ_ARB_LEVEL_GL1C2__GFX101          = 0x0000001c,
    GL1A_PERF_SEL_REQ_ARB_LEVEL_GL1C3__GFX101          = 0x0000001d,
    GL1A_PERF_SEL_REQ_INFLIGHT_LEVEL__GFX101           = 0x0000001e,
    GL1A_PERF_SEL_STALL_RET_CONFLICT_GL1C0__GFX101     = 0x0000001f,
    GL1A_PERF_SEL_STALL_RET_CONFLICT_GL1C1__GFX101     = 0x00000020,
    GL1A_PERF_SEL_STALL_RET_CONFLICT_GL1C2__GFX101     = 0x00000021,
    GL1A_PERF_SEL_STALL_RET_CONFLICT_GL1C3__GFX101     = 0x00000022,
    GL1A_PERF_SEL_CYCLE__GFX101                        = 0x00000023,
    GL1A_PERF_SEL_WDS_32B_GL1C0__GFX103PLUS            = 0x00000009,
    GL1A_PERF_SEL_WDS_32B_GL1C1__GFX103PLUS            = 0x0000000a,
    GL1A_PERF_SEL_WDS_32B_GL1C2__GFX103PLUS            = 0x0000000b,
    GL1A_PERF_SEL_WDS_32B_GL1C3__GFX103PLUS            = 0x0000000c,
    GL1A_PERF_SEL_BURST_COUNT_GL1C0__GFX103PLUS        = 0x0000000d,
    GL1A_PERF_SEL_BURST_COUNT_GL1C1__GFX103PLUS        = 0x0000000e,
    GL1A_PERF_SEL_BURST_COUNT_GL1C2__GFX103PLUS        = 0x0000000f,
    GL1A_PERF_SEL_BURST_COUNT_GL1C3__GFX103PLUS        = 0x00000010,
    GL1A_PERF_SEL_ARB_REQUESTS__GFX103PLUS             = 0x00000011,
    GL1A_PERF_SEL_REQ_INFLIGHT_LEVEL__GFX103PLUS       = 0x00000012,
    GL1A_PERF_SEL_STALL_RET_CONFLICT_GL1C0__GFX103PLUS = 0x00000013,
    GL1A_PERF_SEL_STALL_RET_CONFLICT_GL1C1__GFX103PLUS = 0x00000014,
    GL1A_PERF_SEL_STALL_RET_CONFLICT_GL1C2__GFX103PLUS = 0x00000015,
    GL1A_PERF_SEL_STALL_RET_CONFLICT_GL1C3__GFX103PLUS = 0x00000016,
    GL1A_PERF_SEL_CYCLE__GFX103PLUS                    = 0x00000017,
} GL1A_PERF_SEL;

constexpr unsigned int MaxGl1aPerfSelGfx101            = GL1A_PERF_SEL_CYCLE__GFX101;
constexpr unsigned int MaxGl1aPerfSelGfx103Plus        = GL1A_PERF_SEL_CYCLE__GFX103PLUS;

typedef enum GL1C_PERF_SEL {
    GL1C_PERF_SEL_CYCLE                                = 0x00000000,
    GL1C_PERF_SEL_BUSY                                 = 0x00000001,
    GL1C_PERF_SEL_STARVE                               = 0x00000002,
    GL1C_PERF_SEL_ARB_RET_LEVEL                        = 0x00000003,
    GL1C_PERF_SEL_GL2_REQ_READ                         = 0x00000004,
    GL1C_PERF_SEL_GL2_REQ_READ_128B                    = 0x00000005,
    GL1C_PERF_SEL_GL2_REQ_READ_32B                     = 0x00000006,
    GL1C_PERF_SEL_GL2_REQ_READ_64B                     = 0x00000007,
    GL1C_PERF_SEL_GL2_REQ_READ_LATENCY                 = 0x00000008,
    GL1C_PERF_SEL_GL2_REQ_WRITE                        = 0x00000009,
    GL1C_PERF_SEL_GL2_REQ_WRITE_32B                    = 0x0000000a,
    GL1C_PERF_SEL_GL2_REQ_WRITE_64B                    = 0x0000000b,
    GL1C_PERF_SEL_GL2_REQ_WRITE_LATENCY                = 0x0000000c,
    GL1C_PERF_SEL_GL2_REQ_PREFETCH                     = 0x0000000d,
    GL1C_PERF_SEL_REQ                                  = 0x0000000e,
    GL1C_PERF_SEL_REQ_ATOMIC_WITH_RET                  = 0x0000000f,
    GL1C_PERF_SEL_REQ_ATOMIC_WITHOUT_RET               = 0x00000010,
    GL1C_PERF_SEL_REQ_SHADER_INV                       = 0x00000011,
    GL1C_PERF_SEL_REQ_MISS                             = 0x00000012,
    GL1C_PERF_SEL_REQ_NOP_ACK                          = 0x00000013,
    GL1C_PERF_SEL_REQ_NOP_RTN0                         = 0x00000014,
    GL1C_PERF_SEL_REQ_READ                             = 0x00000015,
    GL1C_PERF_SEL_REQ_READ_128B                        = 0x00000016,
    GL1C_PERF_SEL_REQ_READ_32B                         = 0x00000017,
    GL1C_PERF_SEL_REQ_READ_64B                         = 0x00000018,
    GL1C_PERF_SEL_REQ_READ_POLICY_HIT_EVICT            = 0x00000019,
    GL1C_PERF_SEL_REQ_READ_POLICY_HIT_LRU              = 0x0000001a,
    GL1C_PERF_SEL_REQ_READ_POLICY_MISS_EVICT           = 0x0000001b,
    GL1C_PERF_SEL_REQ_WRITE                            = 0x0000001c,
    GL1C_PERF_SEL_REQ_WRITE_32B                        = 0x0000001d,
    GL1C_PERF_SEL_REQ_WRITE_64B                        = 0x0000001e,
    GL1C_PERF_SEL_STALL_GL2_GL1                        = 0x0000001f,
    GL1C_PERF_SEL_STALL_LFIFO_FULL                     = 0x00000020,
    GL1C_PERF_SEL_STALL_NO_AVAILABLE_ACK_ALLOC         = 0x00000021,
    GL1C_PERF_SEL_STALL_NOTHING_REPLACEABLE            = 0x00000022,
    GL1C_PERF_SEL_STALL_GCR_INV                        = 0x00000023,
    GL1C_PERF_SEL_REQ_CLIENT0__GFX101                  = 0x00000024,
    GL1C_PERF_SEL_REQ_CLIENT1__GFX101                  = 0x00000025,
    GL1C_PERF_SEL_REQ_CLIENT2__GFX101                  = 0x00000026,
    GL1C_PERF_SEL_REQ_CLIENT3__GFX101                  = 0x00000027,
    GL1C_PERF_SEL_REQ_CLIENT4__GFX101                  = 0x00000028,
    GL1C_PERF_SEL_REQ_CLIENT5__GFX101                  = 0x00000029,
    GL1C_PERF_SEL_REQ_CLIENT6__GFX101                  = 0x0000002a,
    GL1C_PERF_SEL_REQ_CLIENT7__GFX101                  = 0x0000002b,
    GL1C_PERF_SEL_REQ_CLIENT8__GFX101                  = 0x0000002c,
    GL1C_PERF_SEL_REQ_CLIENT9__GFX101                  = 0x0000002d,
    GL1C_PERF_SEL_REQ_CLIENT10__GFX101                 = 0x0000002e,
    GL1C_PERF_SEL_REQ_CLIENT11__GFX101                 = 0x0000002f,
    GL1C_PERF_SEL_REQ_CLIENT12__GFX101                 = 0x00000030,
    GL1C_PERF_SEL_REQ_CLIENT13__GFX101                 = 0x00000031,
    GL1C_PERF_SEL_REQ_CLIENT14__GFX101                 = 0x00000032,
    GL1C_PERF_SEL_REQ_CLIENT15__GFX101                 = 0x00000033,
    GL1C_PERF_SEL_REQ_CLIENT16__GFX101                 = 0x00000034,
    GL1C_PERF_SEL_REQ_CLIENT17__GFX101                 = 0x00000035,
    GL1C_PERF_SEL_REQ_CLIENT18__GFX101                 = 0x00000036,
    GL1C_PERF_SEL_REQ_CLIENT19__GFX101                 = 0x00000037,
    GL1C_PERF_SEL_REQ_CLIENT20__GFX101                 = 0x00000038,
    GL1C_PERF_SEL_REQ_CLIENT21__GFX101                 = 0x00000039,
    GL1C_PERF_SEL_REQ_CLIENT22__GFX101                 = 0x0000003a,
    GL1C_PERF_SEL_REQ_CLIENT23__GFX101                 = 0x0000003b,
    GL1C_PERF_SEL_REQ_CLIENT24__GFX101                 = 0x0000003c,
    GL1C_PERF_SEL_REQ_CLIENT25__GFX101                 = 0x0000003d,
    GL1C_PERF_SEL_REQ_CLIENT26__GFX101                 = 0x0000003e,
    GL1C_PERF_SEL_REQ_CLIENT27__GFX101                 = 0x0000003f,
    GL1C_PERF_SEL_STALL_VM__GFX103PLUS                 = 0x00000024,
    GL1C_PERF_SEL_REQ_CLIENT0__GFX103PLUS              = 0x00000025,
    GL1C_PERF_SEL_REQ_CLIENT1__GFX103PLUS              = 0x00000026,
    GL1C_PERF_SEL_REQ_CLIENT2__GFX103PLUS              = 0x00000027,
    GL1C_PERF_SEL_REQ_CLIENT3__GFX103PLUS              = 0x00000028,
    GL1C_PERF_SEL_REQ_CLIENT4__GFX103PLUS              = 0x00000029,
    GL1C_PERF_SEL_REQ_CLIENT5__GFX103PLUS              = 0x0000002a,
    GL1C_PERF_SEL_REQ_CLIENT6__GFX103PLUS              = 0x0000002b,
    GL1C_PERF_SEL_REQ_CLIENT7__GFX103PLUS              = 0x0000002c,
    GL1C_PERF_SEL_REQ_CLIENT8__GFX103PLUS              = 0x0000002d,
    GL1C_PERF_SEL_REQ_CLIENT9__GFX103PLUS              = 0x0000002e,
    GL1C_PERF_SEL_REQ_CLIENT10__GFX103PLUS             = 0x0000002f,
    GL1C_PERF_SEL_REQ_CLIENT11__GFX103PLUS             = 0x00000030,
    GL1C_PERF_SEL_REQ_CLIENT12__GFX103PLUS             = 0x00000031,
    GL1C_PERF_SEL_REQ_CLIENT13__GFX103PLUS             = 0x00000032,
    GL1C_PERF_SEL_REQ_CLIENT14__GFX103PLUS             = 0x00000033,
    GL1C_PERF_SEL_REQ_CLIENT15__GFX103PLUS             = 0x00000034,
    GL1C_PERF_SEL_REQ_CLIENT16__GFX103PLUS             = 0x00000035,
    GL1C_PERF_SEL_REQ_CLIENT17__GFX103PLUS             = 0x00000036,
    GL1C_PERF_SEL_REQ_CLIENT18__GFX103PLUS             = 0x00000037,
    GL1C_PERF_SEL_REQ_CLIENT19__GFX103PLUS             = 0x00000038,
    GL1C_PERF_SEL_REQ_CLIENT20__GFX103PLUS             = 0x00000039,
    GL1C_PERF_SEL_REQ_CLIENT21__GFX103PLUS             = 0x0000003a,
    GL1C_PERF_SEL_REQ_CLIENT22__GFX103PLUS             = 0x0000003b,
    GL1C_PERF_SEL_REQ_CLIENT23__GFX103PLUS             = 0x0000003c,
    GL1C_PERF_SEL_REQ_CLIENT24__GFX103PLUS             = 0x0000003d,
    GL1C_PERF_SEL_REQ_CLIENT25__GFX103PLUS             = 0x0000003e,
    GL1C_PERF_SEL_REQ_CLIENT26__GFX103PLUS             = 0x0000003f,
    GL1C_PERF_SEL_REQ_CLIENT27__GFX103PLUS             = 0x00000040,
    GL1C_PERF_SEL_UTCL0_REQUEST__GFX103PLUS            = 0x00000041,
    GL1C_PERF_SEL_UTCL0_TRANSLATION_HIT__GFX103PLUS    = 0x00000042,
    GL1C_PERF_SEL_UTCL0_TRANSLATION_MISS__GFX103PLUS   = 0x00000043,
    GL1C_PERF_SEL_UTCL0_PERMISSION_MISS__GFX103PLUS    = 0x00000044,
    GL1C_PERF_SEL_UTCL0_LFIFO_FULL__GFX10VRS           = 0x00000045,
    GL1C_PERF_SEL_UTCL0_STALL_INFLIGHT_MAX__GFX10VRS   = 0x00000046,
    GL1C_PERF_SEL_UTCL0_STALL_LFIFO_NOT_RES__GFX10VRS  = 0x00000047,
    GL1C_PERF_SEL_UTCL0_STALL_LRU_INFLIGHT__GFX10VRS   = 0x00000048,
    GL1C_PERF_SEL_UTCL0_STALL_MISSFIFO_FULL__GFX10VRS  = 0x00000049,
    GL1C_PERF_SEL_UTCL0_STALL_MULTI_MISS__GFX10VRS     = 0x0000004a,
    GL1C_PERF_SEL_UTCL0_STALL_UTCL1_REQ_OUT_OF_CREDITS__GFX10VRS = 0x0000004b,
    GL1C_PERF_SEL_UTCL0_UTCL1_PERM_FAULT__GFX10VRS     = 0x0000004c,
    GL1C_PERF_SEL_CLIENT_UTCL0_INFLIGHT__GFX10VRS      = 0x0000004d,
    GL1C_PERF_SEL_UTCL0_UTCL1_INFLIGHT__GFX10VRS       = 0x0000004e,
    GL1C_PERF_SEL_UTCL0_INTERNAL_RETRY_REQ__GFX10VRS   = 0x0000004f,
    GL1C_PERF_SEL_UTCL0_UTCL1_XNACK_RETRY_FAULT__GFX10VRS = 0x00000050,
    GL1C_PERF_SEL_UTCL0_UTCL1_XNACK_PRT_FAULT__GFX10VRS = 0x00000051,
    GL1C_PERF_SEL_UTCL0_UTCL1_XNACK_NO_RETRY_FAULT__GFX10VRS = 0x00000052,
    GL1C_PERF_SEL_UTCL0_MISS_UNDER_MISS__GFX11         = 0x00000045,
    GL1C_PERF_SEL_UTCL0_LFIFO_FULL__GFX11              = 0x00000046,
    GL1C_PERF_SEL_UTCL0_STALL_INFLIGHT_MAX__GFX11      = 0x00000047,
    GL1C_PERF_SEL_UTCL0_STALL_LFIFO_NOT_RES__GFX11     = 0x00000048,
    GL1C_PERF_SEL_UTCL0_STALL_LRU_INFLIGHT__GFX11      = 0x00000049,
    GL1C_PERF_SEL_UTCL0_STALL_MISSFIFO_FULL__GFX11     = 0x0000004a,
    GL1C_PERF_SEL_UTCL0_STALL_MULTI_MISS__GFX11        = 0x0000004b,
    GL1C_PERF_SEL_UTCL0_STALL_UTCL1_REQ_OUT_OF_CREDITS__GFX11 = 0x0000004c,
    GL1C_PERF_SEL_UTCL0_UTCL1_PERM_FAULT__GFX11        = 0x0000004d,
    GL1C_PERF_SEL_CLIENT_UTCL0_INFLIGHT__GFX11         = 0x0000004e,
    GL1C_PERF_SEL_UTCL0_UTCL1_INFLIGHT__GFX11          = 0x0000004f,
    GL1C_PERF_SEL_UTCL0_INTERNAL_RETRY_REQ__GFX11      = 0x00000050,
    GL1C_PERF_SEL_UTCL0_UTCL1_XNACK_RETRY_FAULT__GFX11 = 0x00000051,
    GL1C_PERF_SEL_UTCL0_UTCL1_XNACK_PRT_FAULT__GFX11   = 0x00000052,
    GL1C_PERF_SEL_UTCL0_UTCL1_XNACK_NO_RETRY_FAULT__GFX11 = 0x00000053,
} GL1C_PERF_SEL;

constexpr unsigned int MaxGl1cPerfSelGfx101            = GL1C_PERF_SEL_REQ_CLIENT27__GFX101;
constexpr unsigned int MaxGl1cPerfSelGfx10Vrs          = GL1C_PERF_SEL_UTCL0_UTCL1_XNACK_NO_RETRY_FAULT__GFX10VRS;
constexpr unsigned int MaxGl1cPerfSelGfx11             = GL1C_PERF_SEL_UTCL0_UTCL1_XNACK_NO_RETRY_FAULT__GFX11;

typedef enum GL1_CACHE_POLICIES {
    GL1_CACHE_POLICY_MISS_LRU                          = 0x00000000,
    GL1_CACHE_POLICY_MISS_EVICT                        = 0x00000001,
    GL1_CACHE_POLICY_HIT_LRU                           = 0x00000002,
    GL1_CACHE_POLICY_HIT_EVICT                         = 0x00000003,
} GL1_CACHE_POLICIES;

typedef enum GL1_CACHE_STORE_POLICIES {
    GL1_CACHE_STORE_POLICY_BYPASS                      = 0x00000000,
} GL1_CACHE_STORE_POLICIES;

typedef enum GL2A_PERF_SEL {
    GL2A_PERF_SEL_NONE                                 = 0x00000000,
    GL2A_PERF_SEL_CYCLE                                = 0x00000001,
    GL2A_PERF_SEL_BUSY                                 = 0x00000002,
    GL2A_PERF_SEL_REQ_GL2C0                            = 0x00000003,
    GL2A_PERF_SEL_REQ_GL2C1                            = 0x00000004,
    GL2A_PERF_SEL_REQ_GL2C2                            = 0x00000005,
    GL2A_PERF_SEL_REQ_GL2C3                            = 0x00000006,
    GL2A_PERF_SEL_REQ_GL2C4                            = 0x00000007,
    GL2A_PERF_SEL_REQ_GL2C5                            = 0x00000008,
    GL2A_PERF_SEL_REQ_GL2C6                            = 0x00000009,
    GL2A_PERF_SEL_REQ_GL2C7                            = 0x0000000a,
    GL2A_PERF_SEL_REQ_HI_PRIO_GL2C0                    = 0x0000000b,
    GL2A_PERF_SEL_REQ_HI_PRIO_GL2C1                    = 0x0000000c,
    GL2A_PERF_SEL_REQ_HI_PRIO_GL2C2                    = 0x0000000d,
    GL2A_PERF_SEL_REQ_HI_PRIO_GL2C3                    = 0x0000000e,
    GL2A_PERF_SEL_REQ_HI_PRIO_GL2C4                    = 0x0000000f,
    GL2A_PERF_SEL_REQ_HI_PRIO_GL2C5                    = 0x00000010,
    GL2A_PERF_SEL_REQ_HI_PRIO_GL2C6                    = 0x00000011,
    GL2A_PERF_SEL_REQ_HI_PRIO_GL2C7                    = 0x00000012,
    GL2A_PERF_SEL_REQ_BURST_GL2C0                      = 0x00000013,
    GL2A_PERF_SEL_REQ_BURST_GL2C1                      = 0x00000014,
    GL2A_PERF_SEL_REQ_BURST_GL2C2                      = 0x00000015,
    GL2A_PERF_SEL_REQ_BURST_GL2C3                      = 0x00000016,
    GL2A_PERF_SEL_REQ_BURST_GL2C4                      = 0x00000017,
    GL2A_PERF_SEL_REQ_BURST_GL2C5                      = 0x00000018,
    GL2A_PERF_SEL_REQ_BURST_GL2C6                      = 0x00000019,
    GL2A_PERF_SEL_REQ_BURST_GL2C7                      = 0x0000001a,
    GL2A_PERF_SEL_REQ_STALL_GL2C0                      = 0x0000001b,
    GL2A_PERF_SEL_REQ_STALL_GL2C1                      = 0x0000001c,
    GL2A_PERF_SEL_REQ_STALL_GL2C2                      = 0x0000001d,
    GL2A_PERF_SEL_REQ_STALL_GL2C3                      = 0x0000001e,
    GL2A_PERF_SEL_REQ_STALL_GL2C4                      = 0x0000001f,
    GL2A_PERF_SEL_REQ_STALL_GL2C5                      = 0x00000020,
    GL2A_PERF_SEL_REQ_STALL_GL2C6                      = 0x00000021,
    GL2A_PERF_SEL_REQ_STALL_GL2C7                      = 0x00000022,
    GL2A_PERF_SEL_RTN_STALL_GL2C0                      = 0x00000023,
    GL2A_PERF_SEL_RTN_STALL_GL2C1                      = 0x00000024,
    GL2A_PERF_SEL_RTN_STALL_GL2C2                      = 0x00000025,
    GL2A_PERF_SEL_RTN_STALL_GL2C3                      = 0x00000026,
    GL2A_PERF_SEL_RTN_STALL_GL2C4                      = 0x00000027,
    GL2A_PERF_SEL_RTN_STALL_GL2C5                      = 0x00000028,
    GL2A_PERF_SEL_RTN_STALL_GL2C6                      = 0x00000029,
    GL2A_PERF_SEL_RTN_STALL_GL2C7                      = 0x0000002a,
    GL2A_PERF_SEL_RTN_CLIENT0                          = 0x0000002b,
    GL2A_PERF_SEL_RTN_CLIENT1                          = 0x0000002c,
    GL2A_PERF_SEL_RTN_CLIENT2                          = 0x0000002d,
    GL2A_PERF_SEL_RTN_CLIENT3                          = 0x0000002e,
    GL2A_PERF_SEL_RTN_CLIENT4                          = 0x0000002f,
    GL2A_PERF_SEL_RTN_CLIENT5                          = 0x00000030,
    GL2A_PERF_SEL_RTN_CLIENT6                          = 0x00000031,
    GL2A_PERF_SEL_RTN_CLIENT7                          = 0x00000032,
    GL2A_PERF_SEL_RTN_CLIENT8                          = 0x00000033,
    GL2A_PERF_SEL_RTN_CLIENT9                          = 0x00000034,
    GL2A_PERF_SEL_RTN_CLIENT10                         = 0x00000035,
    GL2A_PERF_SEL_RTN_CLIENT11                         = 0x00000036,
    GL2A_PERF_SEL_RTN_CLIENT12                         = 0x00000037,
    GL2A_PERF_SEL_RTN_CLIENT13                         = 0x00000038,
    GL2A_PERF_SEL_RTN_CLIENT14                         = 0x00000039,
    GL2A_PERF_SEL_RTN_CLIENT15                         = 0x0000003a,
    GL2A_PERF_SEL_RTN_ARB_COLLISION_CLIENT0            = 0x0000003b,
    GL2A_PERF_SEL_RTN_ARB_COLLISION_CLIENT1            = 0x0000003c,
    GL2A_PERF_SEL_RTN_ARB_COLLISION_CLIENT2            = 0x0000003d,
    GL2A_PERF_SEL_RTN_ARB_COLLISION_CLIENT3            = 0x0000003e,
    GL2A_PERF_SEL_RTN_ARB_COLLISION_CLIENT4            = 0x0000003f,
    GL2A_PERF_SEL_RTN_ARB_COLLISION_CLIENT5            = 0x00000040,
    GL2A_PERF_SEL_RTN_ARB_COLLISION_CLIENT6            = 0x00000041,
    GL2A_PERF_SEL_RTN_ARB_COLLISION_CLIENT7            = 0x00000042,
    GL2A_PERF_SEL_RTN_ARB_COLLISION_CLIENT8            = 0x00000043,
    GL2A_PERF_SEL_RTN_ARB_COLLISION_CLIENT9            = 0x00000044,
    GL2A_PERF_SEL_RTN_ARB_COLLISION_CLIENT10           = 0x00000045,
    GL2A_PERF_SEL_RTN_ARB_COLLISION_CLIENT11           = 0x00000046,
    GL2A_PERF_SEL_RTN_ARB_COLLISION_CLIENT12           = 0x00000047,
    GL2A_PERF_SEL_RTN_ARB_COLLISION_CLIENT13           = 0x00000048,
    GL2A_PERF_SEL_RTN_ARB_COLLISION_CLIENT14           = 0x00000049,
    GL2A_PERF_SEL_RTN_ARB_COLLISION_CLIENT15           = 0x0000004a,
    GL2A_PERF_SEL_REQ_BURST_CLIENT0                    = 0x0000004b,
    GL2A_PERF_SEL_REQ_BURST_CLIENT1                    = 0x0000004c,
    GL2A_PERF_SEL_REQ_BURST_CLIENT2                    = 0x0000004d,
    GL2A_PERF_SEL_REQ_BURST_CLIENT3                    = 0x0000004e,
    GL2A_PERF_SEL_REQ_BURST_CLIENT4                    = 0x0000004f,
    GL2A_PERF_SEL_REQ_BURST_CLIENT5                    = 0x00000050,
    GL2A_PERF_SEL_REQ_BURST_CLIENT6                    = 0x00000051,
    GL2A_PERF_SEL_REQ_BURST_CLIENT7                    = 0x00000052,
    GL2A_PERF_SEL_REQ_BURST_CLIENT8                    = 0x00000053,
    GL2A_PERF_SEL_REQ_BURST_CLIENT9                    = 0x00000054,
    GL2A_PERF_SEL_REQ_BURST_CLIENT10                   = 0x00000055,
    GL2A_PERF_SEL_REQ_BURST_CLIENT11                   = 0x00000056,
    GL2A_PERF_SEL_REQ_BURST_CLIENT12                   = 0x00000057,
    GL2A_PERF_SEL_REQ_BURST_CLIENT13                   = 0x00000058,
    GL2A_PERF_SEL_REQ_BURST_CLIENT14                   = 0x00000059,
    GL2A_PERF_SEL_REQ_BURST_CLIENT15                   = 0x0000005a,
    GL2A_PERF_SEL_RTN_CREDIT_STALL_CLIENT0__GFX11      = 0x0000005b,
    GL2A_PERF_SEL_RTN_CREDIT_STALL_CLIENT1__GFX11      = 0x0000005c,
    GL2A_PERF_SEL_RTN_CREDIT_STALL_CLIENT2__GFX11      = 0x0000005d,
    GL2A_PERF_SEL_RTN_CREDIT_STALL_CLIENT3__GFX11      = 0x0000005e,
    GL2A_PERF_SEL_RTN_CREDIT_STALL_CLIENT4__GFX11      = 0x0000005f,
    GL2A_PERF_SEL_RTN_CREDIT_STALL_CLIENT5__GFX11      = 0x00000060,
    GL2A_PERF_SEL_RTN_CREDIT_STALL_CLIENT6__GFX11      = 0x00000061,
    GL2A_PERF_SEL_RTN_CREDIT_STALL_CLIENT7__GFX11      = 0x00000062,
    GL2A_PERF_SEL_RTN_CREDIT_STALL_CLIENT8__GFX11      = 0x00000063,
    GL2A_PERF_SEL_RTN_CREDIT_STALL_CLIENT9__GFX11      = 0x00000064,
    GL2A_PERF_SEL_RTN_CREDIT_STALL_CLIENT10__GFX11     = 0x00000065,
    GL2A_PERF_SEL_RTN_CREDIT_STALL_CLIENT11__GFX11     = 0x00000067,
    GL2A_PERF_SEL_RTN_CREDIT_STALL_CLIENT12__GFX11     = 0x00000068,
    GL2A_PERF_SEL_RTN_CREDIT_STALL_CLIENT13__GFX11     = 0x00000069,
    GL2A_PERF_SEL_RTN_CREDIT_STALL_CLIENT14__GFX11     = 0x0000006a,
    GL2A_PERF_SEL_RTN_CREDIT_STALL_CLIENT15__GFX11     = 0x0000006b,
} GL2A_PERF_SEL;

constexpr unsigned int MaxGl2aPerfSelGfx10             = GL2A_PERF_SEL_REQ_BURST_CLIENT15;
constexpr unsigned int MaxGl2aPerfSelGfx11             = GL2A_PERF_SEL_RTN_CREDIT_STALL_CLIENT15__GFX11;

typedef enum GL2C_PERF_SEL {
    GL2C_PERF_SEL_NONE                                 = 0x00000000,
    GL2C_PERF_SEL_CYCLE                                = 0x00000001,
    GL2C_PERF_SEL_BUSY                                 = 0x00000002,
    GL2C_PERF_SEL_REQ                                  = 0x00000003,
    GL2C_PERF_SEL_VOL_REQ                              = 0x00000004,
    GL2C_PERF_SEL_HIGH_PRIORITY_REQ                    = 0x00000005,
    GL2C_PERF_SEL_READ                                 = 0x00000006,
    GL2C_PERF_SEL_WRITE                                = 0x00000007,
    GL2C_PERF_SEL_ATOMIC                               = 0x00000008,
    GL2C_PERF_SEL_NOP_ACK                              = 0x00000009,
    GL2C_PERF_SEL_NOP_RTN0                             = 0x0000000a,
    GL2C_PERF_SEL_PROBE                                = 0x0000000b,
    GL2C_PERF_SEL_PROBE_ALL                            = 0x0000000c,
    GL2C_PERF_SEL_INTERNAL_PROBE                       = 0x0000000d,
    GL2C_PERF_SEL_COMPRESSED_READ_REQ                  = 0x0000000e,
    GL2C_PERF_SEL_METADATA_READ_REQ                    = 0x0000000f,
    GL2C_PERF_SEL_CLIENT0_REQ                          = 0x00000010,
    GL2C_PERF_SEL_CLIENT1_REQ                          = 0x00000011,
    GL2C_PERF_SEL_CLIENT2_REQ                          = 0x00000012,
    GL2C_PERF_SEL_CLIENT3_REQ                          = 0x00000013,
    GL2C_PERF_SEL_CLIENT4_REQ                          = 0x00000014,
    GL2C_PERF_SEL_CLIENT5_REQ                          = 0x00000015,
    GL2C_PERF_SEL_CLIENT6_REQ                          = 0x00000016,
    GL2C_PERF_SEL_CLIENT7_REQ                          = 0x00000017,
    GL2C_PERF_SEL_C_RW_S_REQ__GFX101                   = 0x00000018,
    GL2C_PERF_SEL_C_RW_US_REQ__GFX101                  = 0x00000019,
    GL2C_PERF_SEL_C_RO_S_REQ__GFX101                   = 0x0000001a,
    GL2C_PERF_SEL_C_RO_US_REQ__GFX101                  = 0x0000001b,
    GL2C_PERF_SEL_UC_REQ__GFX101                       = 0x0000001c,
    GL2C_PERF_SEL_LRU_REQ__GFX101                      = 0x0000001d,
    GL2C_PERF_SEL_STREAM_REQ__GFX101                   = 0x0000001e,
    GL2C_PERF_SEL_BYPASS_REQ__GFX101                   = 0x0000001f,
    GL2C_PERF_SEL_NOA_REQ__GFX101                      = 0x00000020,
    GL2C_PERF_SEL_SHARED_REQ__GFX101                   = 0x00000021,
    GL2C_PERF_SEL_HIT__GFX101                          = 0x00000022,
    GL2C_PERF_SEL_MISS__GFX101                         = 0x00000023,
    GL2C_PERF_SEL_FULL_HIT__GFX101                     = 0x00000024,
    GL2C_PERF_SEL_PARTIAL_32B_HIT__GFX101              = 0x00000025,
    GL2C_PERF_SEL_PARTIAL_64B_HIT__GFX101              = 0x00000026,
    GL2C_PERF_SEL_PARTIAL_96B_HIT__GFX101              = 0x00000027,
    GL2C_PERF_SEL_DEWRITE_ALLOCATE_HIT__GFX101         = 0x00000028,
    GL2C_PERF_SEL_FULLY_WRITTEN_HIT__GFX101            = 0x00000029,
    GL2C_PERF_SEL_UNCACHED_WRITE__GFX101               = 0x0000002a,
    GL2C_PERF_SEL_WRITEBACK__GFX101                    = 0x0000002b,
    GL2C_PERF_SEL_NORMAL_WRITEBACK__GFX101             = 0x0000002c,
    GL2C_PERF_SEL_EVICT__GFX101                        = 0x0000002d,
    GL2C_PERF_SEL_NORMAL_EVICT__GFX101                 = 0x0000002e,
    GL2C_PERF_SEL_PROBE_EVICT__GFX101                  = 0x0000002f,
    GL2C_PERF_SEL_REQ_TO_MISS_QUEUE__GFX101            = 0x00000030,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT0__GFX101     = 0x00000031,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT1__GFX101     = 0x00000032,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT2__GFX101     = 0x00000033,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT3__GFX101     = 0x00000034,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT4__GFX101     = 0x00000035,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT5__GFX101     = 0x00000036,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT6__GFX101     = 0x00000037,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT7__GFX101     = 0x00000038,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT8__GFX101     = 0x00000039,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT9__GFX101     = 0x0000003a,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT10__GFX101    = 0x0000003b,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT11__GFX101    = 0x0000003c,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT12__GFX101    = 0x0000003d,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT13__GFX101    = 0x0000003e,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT14__GFX101    = 0x0000003f,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT15__GFX101    = 0x00000040,
    GL2C_PERF_SEL_READ_32_REQ__GFX101                  = 0x00000041,
    GL2C_PERF_SEL_READ_64_REQ__GFX101                  = 0x00000042,
    GL2C_PERF_SEL_READ_128_REQ__GFX101                 = 0x00000043,
    GL2C_PERF_SEL_WRITE_32_REQ__GFX101                 = 0x00000044,
    GL2C_PERF_SEL_WRITE_64_REQ__GFX101                 = 0x00000045,
    GL2C_PERF_SEL_COMPRESSED_READ_0_REQ__GFX101        = 0x00000046,
    GL2C_PERF_SEL_COMPRESSED_READ_32_REQ__GFX101       = 0x00000047,
    GL2C_PERF_SEL_COMPRESSED_READ_64_REQ__GFX101       = 0x00000048,
    GL2C_PERF_SEL_COMPRESSED_READ_96_REQ__GFX101       = 0x00000049,
    GL2C_PERF_SEL_COMPRESSED_READ_128_REQ__GFX101      = 0x0000004a,
    GL2C_PERF_SEL_MC_WRREQ__GFX101                     = 0x0000004b,
    GL2C_PERF_SEL_EA_WRREQ_64B__GFX101                 = 0x0000004c,
    GL2C_PERF_SEL_EA_WRREQ_PROBE_COMMAND__GFX101       = 0x0000004d,
    GL2C_PERF_SEL_EA_WR_UNCACHED_32B__GFX101           = 0x0000004e,
    GL2C_PERF_SEL_MC_WRREQ_STALL__GFX101               = 0x0000004f,
    GL2C_PERF_SEL_EA_WRREQ_IO_CREDIT_STALL__GFX101     = 0x00000050,
    GL2C_PERF_SEL_EA_WRREQ_GMI_CREDIT_STALL__GFX101    = 0x00000051,
    GL2C_PERF_SEL_EA_WRREQ_DRAM_CREDIT_STALL__GFX101   = 0x00000052,
    GL2C_PERF_SEL_TOO_MANY_EA_WRREQS_STALL__GFX101     = 0x00000053,
    GL2C_PERF_SEL_MC_WRREQ_LEVEL__GFX101               = 0x00000054,
    GL2C_PERF_SEL_EA_ATOMIC__GFX101                    = 0x00000055,
    GL2C_PERF_SEL_EA_ATOMIC_LEVEL__GFX101              = 0x00000056,
    GL2C_PERF_SEL_MC_RDREQ__GFX101                     = 0x00000057,
    GL2C_PERF_SEL_EA_RDREQ_SPLIT__GFX101               = 0x00000058,
    GL2C_PERF_SEL_EA_RDREQ_32B__GFX101                 = 0x00000059,
    GL2C_PERF_SEL_EA_RDREQ_64B__GFX101                 = 0x0000005a,
    GL2C_PERF_SEL_EA_RDREQ_96B__GFX101                 = 0x0000005b,
    GL2C_PERF_SEL_EA_RDREQ_128B__GFX101                = 0x0000005c,
    GL2C_PERF_SEL_EA_RD_UNCACHED_32B__GFX101           = 0x0000005d,
    GL2C_PERF_SEL_EA_RD_MDC_32B__GFX101                = 0x0000005e,
    GL2C_PERF_SEL_EA_RD_COMPRESSED_32B__GFX101         = 0x0000005f,
    GL2C_PERF_SEL_EA_RDREQ_IO_CREDIT_STALL__GFX101     = 0x00000060,
    GL2C_PERF_SEL_EA_RDREQ_GMI_CREDIT_STALL__GFX101    = 0x00000061,
    GL2C_PERF_SEL_EA_RDREQ_DRAM_CREDIT_STALL__GFX101   = 0x00000062,
    GL2C_PERF_SEL_MC_RDREQ_LEVEL__GFX101               = 0x00000063,
    GL2C_PERF_SEL_EA_RDREQ_DRAM__GFX101                = 0x00000064,
    GL2C_PERF_SEL_EA_WRREQ_DRAM__GFX101                = 0x00000065,
    GL2C_PERF_SEL_EA_RDREQ_DRAM_32B__GFX101            = 0x00000066,
    GL2C_PERF_SEL_EA_WRREQ_DRAM_32B__GFX101            = 0x00000067,
    GL2C_PERF_SEL_ONION_READ__GFX101                   = 0x00000068,
    GL2C_PERF_SEL_ONION_WRITE__GFX101                  = 0x00000069,
    GL2C_PERF_SEL_IO_READ__GFX101                      = 0x0000006a,
    GL2C_PERF_SEL_IO_WRITE__GFX101                     = 0x0000006b,
    GL2C_PERF_SEL_GARLIC_READ__GFX101                  = 0x0000006c,
    GL2C_PERF_SEL_GARLIC_WRITE__GFX101                 = 0x0000006d,
    GL2C_PERF_SEL_EA_OUTSTANDING__GFX101               = 0x0000006e,
    GL2C_PERF_SEL_LATENCY_FIFO_FULL__GFX101            = 0x0000006f,
    GL2C_PERF_SEL_SRC_FIFO_FULL__GFX101                = 0x00000070,
    GL2C_PERF_SEL_TAG_STALL__GFX101                    = 0x00000071,
    GL2C_PERF_SEL_TAG_WRITEBACK_FIFO_FULL_STALL__GFX101 = 0x00000072,
    GL2C_PERF_SEL_TAG_MISS_NOTHING_REPLACEABLE_STALL__GFX101 = 0x00000073,
    GL2C_PERF_SEL_TAG_UNCACHED_WRITE_ATOMIC_FIFO_FULL_STALL__GFX101 = 0x00000074,
    GL2C_PERF_SEL_TAG_NO_UNCACHED_WRITE_ATOMIC_ENTRIES_STALL__GFX101 = 0x00000075,
    GL2C_PERF_SEL_TAG_PROBE_STALL__GFX101              = 0x00000076,
    GL2C_PERF_SEL_TAG_PROBE_FILTER_STALL__GFX101       = 0x00000077,
    GL2C_PERF_SEL_TAG_PROBE_FIFO_FULL_STALL__GFX101    = 0x00000078,
    GL2C_PERF_SEL_TAG_READ_DST_STALL__GFX101           = 0x00000079,
    GL2C_PERF_SEL_READ_RETURN_TIMEOUT__GFX101          = 0x0000007a,
    GL2C_PERF_SEL_WRITEBACK_READ_TIMEOUT__GFX101       = 0x0000007b,
    GL2C_PERF_SEL_READ_RETURN_FULL_BUBBLE__GFX101      = 0x0000007c,
    GL2C_PERF_SEL_BUBBLE__GFX101                       = 0x0000007d,
    GL2C_PERF_SEL_IB_REQ__GFX101                       = 0x0000007e,
    GL2C_PERF_SEL_IB_STALL__GFX101                     = 0x0000007f,
    GL2C_PERF_SEL_IB_TAG_STALL__GFX101                 = 0x00000080,
    GL2C_PERF_SEL_IB_CM_STALL__GFX101                  = 0x00000081,
    GL2C_PERF_SEL_RETURN_ACK__GFX101                   = 0x00000082,
    GL2C_PERF_SEL_RETURN_DATA__GFX101                  = 0x00000083,
    GL2C_PERF_SEL_EA_RDRET_NACK__GFX101                = 0x00000084,
    GL2C_PERF_SEL_EA_WRRET_NACK__GFX101                = 0x00000085,
    GL2C_PERF_SEL_GL2A_LEVEL__GFX101                   = 0x00000086,
    GL2C_PERF_SEL_PROBE_FILTER_DISABLE_TRANSITION__GFX101 = 0x00000087,
    GL2C_PERF_SEL_PROBE_FILTER_DISABLED__GFX101        = 0x00000088,
    GL2C_PERF_SEL_ALL_TC_OP_WB_OR_INV_START__GFX101    = 0x00000089,
    GL2C_PERF_SEL_ALL_TC_OP_WB_OR_INV_VOL_START__GFX101 = 0x0000008a,
    GL2C_PERF_SEL_GCR_INV__GFX101                      = 0x0000008b,
    GL2C_PERF_SEL_GCR_WB__GFX101                       = 0x0000008c,
    GL2C_PERF_SEL_GCR_DISCARD__GFX101                  = 0x0000008d,
    GL2C_PERF_SEL_GCR_RANGE__GFX101                    = 0x0000008e,
    GL2C_PERF_SEL_GCR_ALL__GFX101                      = 0x0000008f,
    GL2C_PERF_SEL_GCR_VOL__GFX101                      = 0x00000090,
    GL2C_PERF_SEL_GCR_UNSHARED__GFX101                 = 0x00000091,
    GL2C_PERF_SEL_GCR_MDC_INV__GFX101                  = 0x00000092,
    GL2C_PERF_SEL_GCR_GL2_INV_ALL__GFX101              = 0x00000093,
    GL2C_PERF_SEL_GCR_GL2_WB_ALL__GFX101               = 0x00000094,
    GL2C_PERF_SEL_GCR_MDC_INV_ALL__GFX101              = 0x00000095,
    GL2C_PERF_SEL_GCR_GL2_INV_RANGE__GFX101            = 0x00000096,
    GL2C_PERF_SEL_GCR_GL2_WB_RANGE__GFX101             = 0x00000097,
    GL2C_PERF_SEL_GCR_GL2_WB_INV_RANGE__GFX101         = 0x00000098,
    GL2C_PERF_SEL_GCR_MDC_INV_RANGE__GFX101            = 0x00000099,
    GL2C_PERF_SEL_ALL_GCR_INV_EVICT__GFX101            = 0x0000009a,
    GL2C_PERF_SEL_ALL_GCR_INV_VOL_EVICT__GFX101        = 0x0000009b,
    GL2C_PERF_SEL_ALL_GCR_WB_OR_INV_CYCLE__GFX101      = 0x0000009c,
    GL2C_PERF_SEL_ALL_GCR_WB_OR_INV_VOL_CYCLE__GFX101  = 0x0000009d,
    GL2C_PERF_SEL_ALL_GCR_WB_WRITEBACK__GFX101         = 0x0000009e,
    GL2C_PERF_SEL_GCR_INVL2_VOL_CYCLE__GFX101          = 0x0000009f,
    GL2C_PERF_SEL_GCR_INVL2_VOL_EVICT__GFX101          = 0x000000a0,
    GL2C_PERF_SEL_GCR_INVL2_VOL_START__GFX101          = 0x000000a1,
    GL2C_PERF_SEL_GCR_WBL2_VOL_CYCLE__GFX101           = 0x000000a2,
    GL2C_PERF_SEL_GCR_WBL2_VOL_EVICT__GFX101           = 0x000000a3,
    GL2C_PERF_SEL_GCR_WBL2_VOL_START__GFX101           = 0x000000a4,
    GL2C_PERF_SEL_GCR_WBINVL2_CYCLE__GFX101            = 0x000000a5,
    GL2C_PERF_SEL_GCR_WBINVL2_EVICT__GFX101            = 0x000000a6,
    GL2C_PERF_SEL_GCR_WBINVL2_START__GFX101            = 0x000000a7,
    GL2C_PERF_SEL_MDC_INV_METADATA__GFX101             = 0x000000a8,
    GL2C_PERF_SEL_MDC_REQ__GFX101                      = 0x000000a9,
    GL2C_PERF_SEL_MDC_LEVEL__GFX101                    = 0x000000aa,
    GL2C_PERF_SEL_MDC_TAG_HIT__GFX101                  = 0x000000ab,
    GL2C_PERF_SEL_MDC_SECTOR_HIT__GFX101               = 0x000000ac,
    GL2C_PERF_SEL_MDC_SECTOR_MISS__GFX101              = 0x000000ad,
    GL2C_PERF_SEL_MDC_TAG_STALL__GFX101                = 0x000000ae,
    GL2C_PERF_SEL_MDC_TAG_REPLACEMENT_LINE_IN_USE_STALL__GFX101 = 0x000000af,
    GL2C_PERF_SEL_MDC_TAG_DESECTORIZATION_FIFO_FULL_STALL__GFX101 = 0x000000b0,
    GL2C_PERF_SEL_MDC_TAG_WAITING_FOR_INVALIDATE_COMPLETION_STALL__GFX101 = 0x000000b1,
    GL2C_PERF_SEL_CM_CHANNEL0_REQ__GFX101              = 0x000000b2,
    GL2C_PERF_SEL_CM_CHANNEL1_REQ__GFX101              = 0x000000b3,
    GL2C_PERF_SEL_CM_CHANNEL2_REQ__GFX101              = 0x000000b4,
    GL2C_PERF_SEL_CM_CHANNEL3_REQ__GFX101              = 0x000000b5,
    GL2C_PERF_SEL_CM_CHANNEL4_REQ__GFX101              = 0x000000b6,
    GL2C_PERF_SEL_CM_CHANNEL5_REQ__GFX101              = 0x000000b7,
    GL2C_PERF_SEL_CM_CHANNEL6_REQ__GFX101              = 0x000000b8,
    GL2C_PERF_SEL_CM_CHANNEL7_REQ__GFX101              = 0x000000b9,
    GL2C_PERF_SEL_CM_CHANNEL8_REQ__GFX101              = 0x000000ba,
    GL2C_PERF_SEL_CM_CHANNEL9_REQ__GFX101              = 0x000000bb,
    GL2C_PERF_SEL_CM_CHANNEL10_REQ__GFX101             = 0x000000bc,
    GL2C_PERF_SEL_CM_CHANNEL11_REQ__GFX101             = 0x000000bd,
    GL2C_PERF_SEL_CM_CHANNEL12_REQ__GFX101             = 0x000000be,
    GL2C_PERF_SEL_CM_CHANNEL13_REQ__GFX101             = 0x000000bf,
    GL2C_PERF_SEL_CM_CHANNEL14_REQ__GFX101             = 0x000000c0,
    GL2C_PERF_SEL_CM_CHANNEL15_REQ__GFX101             = 0x000000c1,
    GL2C_PERF_SEL_CM_CHANNEL16_REQ__GFX101             = 0x000000c2,
    GL2C_PERF_SEL_CM_CHANNEL17_REQ__GFX101             = 0x000000c3,
    GL2C_PERF_SEL_CM_CHANNEL18_REQ__GFX101             = 0x000000c4,
    GL2C_PERF_SEL_CM_CHANNEL19_REQ__GFX101             = 0x000000c5,
    GL2C_PERF_SEL_CM_CHANNEL20_REQ__GFX101             = 0x000000c6,
    GL2C_PERF_SEL_CM_CHANNEL21_REQ__GFX101             = 0x000000c7,
    GL2C_PERF_SEL_CM_CHANNEL22_REQ__GFX101             = 0x000000c8,
    GL2C_PERF_SEL_CM_CHANNEL23_REQ__GFX101             = 0x000000c9,
    GL2C_PERF_SEL_CM_CHANNEL24_REQ__GFX101             = 0x000000ca,
    GL2C_PERF_SEL_CM_CHANNEL25_REQ__GFX101             = 0x000000cb,
    GL2C_PERF_SEL_CM_CHANNEL26_REQ__GFX101             = 0x000000cc,
    GL2C_PERF_SEL_CM_CHANNEL27_REQ__GFX101             = 0x000000cd,
    GL2C_PERF_SEL_CM_CHANNEL28_REQ__GFX101             = 0x000000ce,
    GL2C_PERF_SEL_CM_CHANNEL29_REQ__GFX101             = 0x000000cf,
    GL2C_PERF_SEL_CM_CHANNEL30_REQ__GFX101             = 0x000000d0,
    GL2C_PERF_SEL_CM_CHANNEL31_REQ__GFX101             = 0x000000d1,
    GL2C_PERF_SEL_CM_COMP_ATOMIC_COLOR_REQ__GFX101     = 0x000000d2,
    GL2C_PERF_SEL_CM_COMP_ATOMIC_DEPTH16_REQ__GFX101   = 0x000000d3,
    GL2C_PERF_SEL_CM_COMP_ATOMIC_DEPTH32_REQ__GFX101   = 0x000000d4,
    GL2C_PERF_SEL_CM_COMP_WRITE_COLOR_REQ__GFX101      = 0x000000d5,
    GL2C_PERF_SEL_CM_COMP_WRITE_DEPTH16_REQ__GFX101    = 0x000000d6,
    GL2C_PERF_SEL_CM_COMP_WRITE_DEPTH32_REQ__GFX101    = 0x000000d7,
    GL2C_PERF_SEL_CM_COMP_WRITE_STENCIL_REQ__GFX101    = 0x000000d8,
    GL2C_PERF_SEL_CM_COMP_READ_REQ__GFX101             = 0x000000d9,
    GL2C_PERF_SEL_CM_READ_BACK_REQ__GFX101             = 0x000000da,
    GL2C_PERF_SEL_CM_METADATA_WR_REQ__GFX101           = 0x000000db,
    GL2C_PERF_SEL_CM_WR_ACK_REQ__GFX101                = 0x000000dc,
    GL2C_PERF_SEL_CM_NO_ACK_REQ__GFX101                = 0x000000dd,
    GL2C_PERF_SEL_CM_NOOP_REQ__GFX101                  = 0x000000de,
    GL2C_PERF_SEL_CM_COMP_COLOR_EN_REQ__GFX101         = 0x000000df,
    GL2C_PERF_SEL_CM_COMP_COLOR_DIS_REQ__GFX101        = 0x000000e0,
    GL2C_PERF_SEL_CM_COMP_STENCIL_REQ__GFX101          = 0x000000e1,
    GL2C_PERF_SEL_CM_COMP_DEPTH16_REQ__GFX101          = 0x000000e2,
    GL2C_PERF_SEL_CM_COMP_DEPTH32_REQ__GFX101          = 0x000000e3,
    GL2C_PERF_SEL_CM_COLOR_32B_WR_REQ__GFX101          = 0x000000e4,
    GL2C_PERF_SEL_CM_COLOR_64B_WR_REQ__GFX101          = 0x000000e5,
    GL2C_PERF_SEL_CM_FULL_WRITE_REQ__GFX101            = 0x000000e6,
    GL2C_PERF_SEL_CM_RVF_FULL__GFX101                  = 0x000000e7,
    GL2C_PERF_SEL_CM_SDR_FULL__GFX101                  = 0x000000e8,
    GL2C_PERF_SEL_CM_MERGE_BUF_FULL__GFX101            = 0x000000e9,
    GL2C_PERF_SEL_CM_DCC_STALL__GFX101                 = 0x000000ea,
    GL2C_PERF_SEL_CLIENT8_REQ__GFX103PLUS              = 0x00000018,
    GL2C_PERF_SEL_CLIENT9_REQ__GFX103PLUS              = 0x00000019,
    GL2C_PERF_SEL_CLIENT10_REQ__GFX103PLUS             = 0x0000001a,
    GL2C_PERF_SEL_CLIENT11_REQ__GFX103PLUS             = 0x0000001b,
    GL2C_PERF_SEL_CLIENT12_REQ__GFX103PLUS             = 0x0000001c,
    GL2C_PERF_SEL_CLIENT13_REQ__GFX103PLUS             = 0x0000001d,
    GL2C_PERF_SEL_CLIENT14_REQ__GFX103PLUS             = 0x0000001e,
    GL2C_PERF_SEL_CLIENT15_REQ__GFX103PLUS             = 0x0000001f,
    GL2C_PERF_SEL_C_RW_S_REQ__GFX103PLUS               = 0x00000020,
    GL2C_PERF_SEL_C_RW_US_REQ__GFX103PLUS              = 0x00000021,
    GL2C_PERF_SEL_C_RO_S_REQ__GFX103PLUS               = 0x00000022,
    GL2C_PERF_SEL_C_RO_US_REQ__GFX103PLUS              = 0x00000023,
    GL2C_PERF_SEL_UC_REQ__GFX103PLUS                   = 0x00000024,
    GL2C_PERF_SEL_LRU_REQ__GFX103PLUS                  = 0x00000025,
    GL2C_PERF_SEL_STREAM_REQ__GFX103PLUS               = 0x00000026,
    GL2C_PERF_SEL_BYPASS_REQ__GFX103PLUS               = 0x00000027,
    GL2C_PERF_SEL_NOA_REQ__GFX103PLUS                  = 0x00000028,
    GL2C_PERF_SEL_SHARED_REQ__GFX103PLUS               = 0x00000029,
    GL2C_PERF_SEL_HIT__GFX103PLUS                      = 0x0000002a,
    GL2C_PERF_SEL_MISS__GFX103PLUS                     = 0x0000002b,
    GL2C_PERF_SEL_FULL_HIT__GFX103PLUS                 = 0x0000002c,
    GL2C_PERF_SEL_PARTIAL_32B_HIT__GFX103PLUS          = 0x0000002d,
    GL2C_PERF_SEL_PARTIAL_64B_HIT__GFX103PLUS          = 0x0000002e,
    GL2C_PERF_SEL_PARTIAL_96B_HIT__GFX103PLUS          = 0x0000002f,
    GL2C_PERF_SEL_DEWRITE_ALLOCATE_HIT__GFX103PLUS     = 0x00000030,
    GL2C_PERF_SEL_FULLY_WRITTEN_HIT__GFX103PLUS        = 0x00000031,
    GL2C_PERF_SEL_UNCACHED_WRITE__GFX103PLUS           = 0x00000032,
    GL2C_PERF_SEL_WRITEBACK__GFX103PLUS                = 0x00000033,
    GL2C_PERF_SEL_NORMAL_WRITEBACK__GFX103PLUS         = 0x00000034,
    GL2C_PERF_SEL_EVICT__GFX103PLUS                    = 0x00000035,
    GL2C_PERF_SEL_NORMAL_EVICT__GFX103PLUS             = 0x00000036,
    GL2C_PERF_SEL_PROBE_EVICT__GFX103PLUS              = 0x00000037,
    GL2C_PERF_SEL_REQ_TO_MISS_QUEUE__GFX103PLUS        = 0x00000038,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT0__GFX103PLUS = 0x00000039,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT1__GFX103PLUS = 0x0000003a,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT2__GFX103PLUS = 0x0000003b,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT3__GFX103PLUS = 0x0000003c,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT4__GFX103PLUS = 0x0000003d,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT5__GFX103PLUS = 0x0000003e,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT6__GFX103PLUS = 0x0000003f,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT7__GFX103PLUS = 0x00000040,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT8__GFX103PLUS = 0x00000041,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT9__GFX103PLUS = 0x00000042,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT10__GFX103PLUS = 0x00000043,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT11__GFX103PLUS = 0x00000044,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT12__GFX103PLUS = 0x00000045,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT13__GFX103PLUS = 0x00000046,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT14__GFX103PLUS = 0x00000047,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT15__GFX103PLUS = 0x00000048,
    GL2C_PERF_SEL_READ_32_REQ__GFX103PLUS              = 0x00000049,
    GL2C_PERF_SEL_READ_64_REQ__GFX103PLUS              = 0x0000004a,
    GL2C_PERF_SEL_READ_128_REQ__GFX103PLUS             = 0x0000004b,
    GL2C_PERF_SEL_WRITE_32_REQ__GFX103PLUS             = 0x0000004c,
    GL2C_PERF_SEL_WRITE_64_REQ__GFX103PLUS             = 0x0000004d,
    GL2C_PERF_SEL_COMPRESSED_READ_0_REQ__GFX103PLUS    = 0x0000004e,
    GL2C_PERF_SEL_COMPRESSED_READ_32_REQ__GFX103PLUS   = 0x0000004f,
    GL2C_PERF_SEL_COMPRESSED_READ_64_REQ__GFX103PLUS   = 0x00000050,
    GL2C_PERF_SEL_COMPRESSED_READ_96_REQ__GFX103PLUS   = 0x00000051,
    GL2C_PERF_SEL_COMPRESSED_READ_128_REQ__GFX103PLUS  = 0x00000052,
    GL2C_PERF_SEL_MC_WRREQ__GFX103PLUS                 = 0x00000053,
    GL2C_PERF_SEL_EA_WRREQ_SNOOP__GFX103PLUS           = 0x00000054,
    GL2C_PERF_SEL_EA_WRREQ_64B__GFX103PLUS             = 0x00000055,
    GL2C_PERF_SEL_EA_WRREQ_PROBE_COMMAND__GFX103PLUS   = 0x00000056,
    GL2C_PERF_SEL_EA_WR_UNCACHED_32B__GFX103PLUS       = 0x00000057,
    GL2C_PERF_SEL_MC_WRREQ_STALL__GFX103PLUS           = 0x00000058,
    GL2C_PERF_SEL_EA_WRREQ_IO_CREDIT_STALL__GFX103PLUS = 0x00000059,
    GL2C_PERF_SEL_EA_WRREQ_GMI_CREDIT_STALL__GFX103PLUS = 0x0000005a,
    GL2C_PERF_SEL_EA_WRREQ_DRAM_CREDIT_STALL__GFX103PLUS = 0x0000005b,
    GL2C_PERF_SEL_TOO_MANY_EA_WRREQS_STALL__GFX103PLUS = 0x0000005c,
    GL2C_PERF_SEL_MC_WRREQ_LEVEL__GFX103PLUS           = 0x0000005d,
    GL2C_PERF_SEL_EA_ATOMIC__GFX103PLUS                = 0x0000005e,
    GL2C_PERF_SEL_EA_ATOMIC_LEVEL__GFX103PLUS          = 0x0000005f,
    GL2C_PERF_SEL_MC_RDREQ__GFX103PLUS                 = 0x00000060,
    GL2C_PERF_SEL_EA_RDREQ_SNOOP__GFX103PLUS           = 0x00000061,
    GL2C_PERF_SEL_EA_RDREQ_SPLIT__GFX103PLUS           = 0x00000062,
    GL2C_PERF_SEL_EA_RDREQ_32B__GFX103PLUS             = 0x00000063,
    GL2C_PERF_SEL_EA_RDREQ_64B__GFX103PLUS             = 0x00000064,
    GL2C_PERF_SEL_EA_RDREQ_96B__GFX103PLUS             = 0x00000065,
    GL2C_PERF_SEL_EA_RDREQ_128B__GFX103PLUS            = 0x00000066,
    GL2C_PERF_SEL_EA_RD_UNCACHED_32B__GFX103PLUS       = 0x00000067,
    GL2C_PERF_SEL_EA_RD_MDC_32B__GFX103PLUS            = 0x00000068,
    GL2C_PERF_SEL_EA_RD_COMPRESSED_32B__GFX103PLUS     = 0x00000069,
    GL2C_PERF_SEL_EA_RDREQ_IO_CREDIT_STALL__GFX103PLUS = 0x0000006a,
    GL2C_PERF_SEL_EA_RDREQ_GMI_CREDIT_STALL__GFX103PLUS = 0x0000006b,
    GL2C_PERF_SEL_EA_RDREQ_DRAM_CREDIT_STALL__GFX103PLUS = 0x0000006c,
    GL2C_PERF_SEL_MC_RDREQ_LEVEL__GFX103PLUS           = 0x0000006d,
    GL2C_PERF_SEL_EA_RDREQ_DRAM__GFX103PLUS            = 0x0000006e,
    GL2C_PERF_SEL_EA_WRREQ_DRAM__GFX103PLUS            = 0x0000006f,
    GL2C_PERF_SEL_EA_RDREQ_DRAM_32B__GFX103PLUS        = 0x00000070,
    GL2C_PERF_SEL_EA_WRREQ_DRAM_32B__GFX103PLUS        = 0x00000071,
    GL2C_PERF_SEL_ONION_READ__GFX103PLUS               = 0x00000072,
    GL2C_PERF_SEL_ONION_WRITE__GFX103PLUS              = 0x00000073,
    GL2C_PERF_SEL_IO_READ__GFX103PLUS                  = 0x00000074,
    GL2C_PERF_SEL_IO_WRITE__GFX103PLUS                 = 0x00000075,
    GL2C_PERF_SEL_GARLIC_READ__GFX103PLUS              = 0x00000076,
    GL2C_PERF_SEL_GARLIC_WRITE__GFX103PLUS             = 0x00000077,
    GL2C_PERF_SEL_EA_OUTSTANDING__GFX103PLUS           = 0x00000078,
    GL2C_PERF_SEL_LATENCY_FIFO_FULL__GFX103PLUS        = 0x00000079,
    GL2C_PERF_SEL_SRC_FIFO_FULL__GFX103PLUS            = 0x0000007a,
    GL2C_PERF_SEL_TAG_STALL__GFX103PLUS                = 0x0000007b,
    GL2C_PERF_SEL_TAG_WRITEBACK_FIFO_FULL_STALL__GFX103PLUS = 0x0000007c,
    GL2C_PERF_SEL_TAG_MISS_NOTHING_REPLACEABLE_STALL__GFX103PLUS = 0x0000007d,
    GL2C_PERF_SEL_TAG_UNCACHED_WRITE_ATOMIC_FIFO_FULL_STALL__GFX103PLUS = 0x0000007e,
    GL2C_PERF_SEL_TAG_NO_UNCACHED_WRITE_ATOMIC_ENTRIES_STALL__GFX103PLUS = 0x0000007f,
    GL2C_PERF_SEL_TAG_PROBE_STALL__GFX103PLUS          = 0x00000080,
    GL2C_PERF_SEL_TAG_PROBE_FILTER_STALL__GFX103PLUS   = 0x00000081,
    GL2C_PERF_SEL_TAG_PROBE_FIFO_FULL_STALL__GFX103PLUS = 0x00000082,
    GL2C_PERF_SEL_TAG_READ_DST_STALL__GFX103PLUS       = 0x00000083,
    GL2C_PERF_SEL_READ_RETURN_TIMEOUT__GFX103PLUS      = 0x00000084,
    GL2C_PERF_SEL_WRITEBACK_READ_TIMEOUT__GFX103PLUS   = 0x00000085,
    GL2C_PERF_SEL_READ_RETURN_FULL_BUBBLE__GFX103PLUS  = 0x00000086,
    GL2C_PERF_SEL_BUBBLE__GFX103PLUS                   = 0x00000087,
    GL2C_PERF_SEL_IB_REQ__GFX103PLUS                   = 0x00000088,
    GL2C_PERF_SEL_IB_STALL__GFX103PLUS                 = 0x00000089,
    GL2C_PERF_SEL_IB_TAG_STALL__GFX103PLUS             = 0x0000008a,
    GL2C_PERF_SEL_IB_CM_STALL__GFX103PLUS              = 0x0000008b,
    GL2C_PERF_SEL_RETURN_ACK__GFX103PLUS               = 0x0000008c,
    GL2C_PERF_SEL_RETURN_DATA__GFX103PLUS              = 0x0000008d,
    GL2C_PERF_SEL_EA_RDRET_NACK__GFX10VRS              = 0x0000008e,
    GL2C_PERF_SEL_EA_WRRET_NACK__GFX10VRS              = 0x0000008f,
    GL2C_PERF_SEL_GL2A_LEVEL__GFX10VRS                 = 0x00000090,
    GL2C_PERF_SEL_PROBE_FILTER_DISABLE_TRANSITION__GFX10VRS = 0x00000091,
    GL2C_PERF_SEL_PROBE_FILTER_DISABLED__GFX10VRS      = 0x00000092,
    GL2C_PERF_SEL_ALL_TC_OP_WB_OR_INV_START__GFX10VRS  = 0x00000093,
    GL2C_PERF_SEL_ALL_TC_OP_WB_OR_INV_VOL_START__GFX10VRS = 0x00000094,
    GL2C_PERF_SEL_GCR_INV__GFX10VRS                    = 0x00000095,
    GL2C_PERF_SEL_GCR_WB__GFX10VRS                     = 0x00000096,
    GL2C_PERF_SEL_GCR_DISCARD__GFX10VRS                = 0x00000097,
    GL2C_PERF_SEL_GCR_RANGE__GFX10VRS                  = 0x00000098,
    GL2C_PERF_SEL_GCR_ALL__GFX10VRS                    = 0x00000099,
    GL2C_PERF_SEL_GCR_VOL__GFX10VRS                    = 0x0000009a,
    GL2C_PERF_SEL_GCR_UNSHARED__GFX10VRS               = 0x0000009b,
    GL2C_PERF_SEL_GCR_MDC_INV__GFX10VRS                = 0x0000009c,
    GL2C_PERF_SEL_GCR_GL2_INV_ALL__GFX10VRS            = 0x0000009d,
    GL2C_PERF_SEL_GCR_GL2_WB_ALL__GFX10VRS             = 0x0000009e,
    GL2C_PERF_SEL_GCR_MDC_INV_ALL__GFX10VRS            = 0x0000009f,
    GL2C_PERF_SEL_GCR_GL2_INV_RANGE__GFX10VRS          = 0x000000a0,
    GL2C_PERF_SEL_GCR_GL2_WB_RANGE__GFX10VRS           = 0x000000a1,
    GL2C_PERF_SEL_GCR_GL2_WB_INV_RANGE__GFX10VRS       = 0x000000a2,
    GL2C_PERF_SEL_GCR_MDC_INV_RANGE__GFX10VRS          = 0x000000a3,
    GL2C_PERF_SEL_ALL_GCR_INV_EVICT__GFX10VRS          = 0x000000a4,
    GL2C_PERF_SEL_ALL_GCR_INV_VOL_EVICT__GFX10VRS      = 0x000000a5,
    GL2C_PERF_SEL_ALL_GCR_WB_OR_INV_CYCLE__GFX10VRS    = 0x000000a6,
    GL2C_PERF_SEL_ALL_GCR_WB_OR_INV_VOL_CYCLE__GFX10VRS = 0x000000a7,
    GL2C_PERF_SEL_ALL_GCR_WB_WRITEBACK__GFX10VRS       = 0x000000a8,
    GL2C_PERF_SEL_GCR_INVL2_VOL_CYCLE__GFX10VRS        = 0x000000a9,
    GL2C_PERF_SEL_GCR_INVL2_VOL_EVICT__GFX10VRS        = 0x000000aa,
    GL2C_PERF_SEL_GCR_INVL2_VOL_START__GFX10VRS        = 0x000000ab,
    GL2C_PERF_SEL_GCR_WBL2_VOL_CYCLE__GFX10VRS         = 0x000000ac,
    GL2C_PERF_SEL_GCR_WBL2_VOL_START__GFX10VRS         = 0x000000ad,
    GL2C_PERF_SEL_GCR_WBINVL2_CYCLE__GFX10VRS          = 0x000000ae,
    GL2C_PERF_SEL_GCR_WBINVL2_EVICT__GFX10VRS          = 0x000000af,
    GL2C_PERF_SEL_GCR_WBINVL2_START__GFX10VRS          = 0x000000b0,
    GL2C_PERF_SEL_MDC_INV_METADATA__GFX10VRS           = 0x000000b1,
    GL2C_PERF_SEL_MDC_REQ__GFX10VRS                    = 0x000000b2,
    GL2C_PERF_SEL_MDC_LEVEL__GFX10VRS                  = 0x000000b3,
    GL2C_PERF_SEL_MDC_TAG_HIT__GFX10VRS                = 0x000000b4,
    GL2C_PERF_SEL_MDC_SECTOR_HIT__GFX10VRS             = 0x000000b5,
    GL2C_PERF_SEL_MDC_SECTOR_MISS__GFX10VRS            = 0x000000b6,
    GL2C_PERF_SEL_MDC_TAG_STALL__GFX10VRS              = 0x000000b7,
    GL2C_PERF_SEL_MDC_TAG_REPLACEMENT_LINE_IN_USE_STALL__GFX10VRS = 0x000000b8,
    GL2C_PERF_SEL_MDC_TAG_DESECTORIZATION_FIFO_FULL_STALL__GFX10VRS = 0x000000b9,
    GL2C_PERF_SEL_MDC_TAG_WAITING_FOR_INVALIDATE_COMPLETION_STALL__GFX10VRS = 0x000000ba,
    GL2C_PERF_SEL_CM_CHANNEL0_REQ__GFX10VRS            = 0x000000bb,
    GL2C_PERF_SEL_CM_CHANNEL1_REQ__GFX10VRS            = 0x000000bc,
    GL2C_PERF_SEL_CM_CHANNEL2_REQ__GFX10VRS            = 0x000000bd,
    GL2C_PERF_SEL_CM_CHANNEL3_REQ__GFX10VRS            = 0x000000be,
    GL2C_PERF_SEL_CM_CHANNEL4_REQ__GFX10VRS            = 0x000000bf,
    GL2C_PERF_SEL_CM_CHANNEL5_REQ__GFX10VRS            = 0x000000c0,
    GL2C_PERF_SEL_CM_CHANNEL6_REQ__GFX10VRS            = 0x000000c1,
    GL2C_PERF_SEL_CM_CHANNEL7_REQ__GFX10VRS            = 0x000000c2,
    GL2C_PERF_SEL_CM_CHANNEL8_REQ__GFX10VRS            = 0x000000c3,
    GL2C_PERF_SEL_CM_CHANNEL9_REQ__GFX10VRS            = 0x000000c4,
    GL2C_PERF_SEL_CM_CHANNEL10_REQ__GFX10VRS           = 0x000000c5,
    GL2C_PERF_SEL_CM_CHANNEL11_REQ__GFX10VRS           = 0x000000c6,
    GL2C_PERF_SEL_CM_CHANNEL12_REQ__GFX10VRS           = 0x000000c7,
    GL2C_PERF_SEL_CM_CHANNEL13_REQ__GFX10VRS           = 0x000000c8,
    GL2C_PERF_SEL_CM_CHANNEL14_REQ__GFX10VRS           = 0x000000c9,
    GL2C_PERF_SEL_CM_CHANNEL15_REQ__GFX10VRS           = 0x000000ca,
    GL2C_PERF_SEL_CM_CHANNEL16_REQ__GFX10VRS           = 0x000000cb,
    GL2C_PERF_SEL_CM_CHANNEL17_REQ__GFX10VRS           = 0x000000cc,
    GL2C_PERF_SEL_CM_CHANNEL18_REQ__GFX10VRS           = 0x000000cd,
    GL2C_PERF_SEL_CM_CHANNEL19_REQ__GFX10VRS           = 0x000000ce,
    GL2C_PERF_SEL_CM_CHANNEL20_REQ__GFX10VRS           = 0x000000cf,
    GL2C_PERF_SEL_CM_CHANNEL21_REQ__GFX10VRS           = 0x000000d0,
    GL2C_PERF_SEL_CM_CHANNEL22_REQ__GFX10VRS           = 0x000000d1,
    GL2C_PERF_SEL_CM_CHANNEL23_REQ__GFX10VRS           = 0x000000d2,
    GL2C_PERF_SEL_CM_CHANNEL24_REQ__GFX10VRS           = 0x000000d3,
    GL2C_PERF_SEL_CM_CHANNEL25_REQ__GFX10VRS           = 0x000000d4,
    GL2C_PERF_SEL_CM_CHANNEL26_REQ__GFX10VRS           = 0x000000d5,
    GL2C_PERF_SEL_CM_CHANNEL27_REQ__GFX10VRS           = 0x000000d6,
    GL2C_PERF_SEL_CM_CHANNEL28_REQ__GFX10VRS           = 0x000000d7,
    GL2C_PERF_SEL_CM_CHANNEL29_REQ__GFX10VRS           = 0x000000d8,
    GL2C_PERF_SEL_CM_CHANNEL30_REQ__GFX10VRS           = 0x000000d9,
    GL2C_PERF_SEL_CM_CHANNEL31_REQ__GFX10VRS           = 0x000000da,
    GL2C_PERF_SEL_CM_COMP_ATOMIC_COLOR_REQ__GFX10VRS   = 0x000000db,
    GL2C_PERF_SEL_CM_COMP_ATOMIC_DEPTH16_REQ__GFX10VRS = 0x000000dc,
    GL2C_PERF_SEL_CM_COMP_ATOMIC_DEPTH32_REQ__GFX10VRS = 0x000000dd,
    GL2C_PERF_SEL_CM_COMP_ATOMIC_STENCIL_REQ__GFX10VRS = 0x000000de,
    GL2C_PERF_SEL_CM_COMP_WRITE_COLOR_REQ__GFX10VRS    = 0x000000df,
    GL2C_PERF_SEL_CM_COMP_WRITE_DEPTH16_REQ__GFX10VRS  = 0x000000e0,
    GL2C_PERF_SEL_CM_COMP_WRITE_DEPTH32_REQ__GFX10VRS  = 0x000000e1,
    GL2C_PERF_SEL_CM_COMP_WRITE_STENCIL_REQ__GFX10VRS  = 0x000000e2,
    GL2C_PERF_SEL_CM_COMP_READ_REQ__GFX10VRS           = 0x000000e3,
    GL2C_PERF_SEL_CM_READ_BACK_REQ__GFX10VRS           = 0x000000e4,
    GL2C_PERF_SEL_CM_METADATA_WR_REQ__GFX10VRS         = 0x000000e5,
    GL2C_PERF_SEL_CM_WR_ACK_REQ__GFX10VRS              = 0x000000e6,
    GL2C_PERF_SEL_CM_NO_ACK_REQ__GFX10VRS              = 0x000000e7,
    GL2C_PERF_SEL_CM_NOOP_REQ__GFX10VRS                = 0x000000e8,
    GL2C_PERF_SEL_CM_COMP_COLOR_EN_REQ__GFX10VRS       = 0x000000e9,
    GL2C_PERF_SEL_CM_COMP_COLOR_DIS_REQ__GFX10VRS      = 0x000000ea,
    GL2C_PERF_SEL_CM_COMP_STENCIL_REQ__GFX10VRS        = 0x000000eb,
    GL2C_PERF_SEL_CM_COMP_DEPTH16_REQ__GFX10VRS        = 0x000000ec,
    GL2C_PERF_SEL_CM_COMP_DEPTH32_REQ__GFX10VRS        = 0x000000ed,
    GL2C_PERF_SEL_CM_COMP_RB_SKIP_REQ__GFX10VRS        = 0x000000ee,
    GL2C_PERF_SEL_CM_COLOR_32B_WR_REQ__GFX10VRS        = 0x000000ef,
    GL2C_PERF_SEL_CM_COLOR_64B_WR_REQ__GFX10VRS        = 0x000000f0,
    GL2C_PERF_SEL_CM_FULL_WRITE_REQ__GFX10VRS          = 0x000000f1,
    GL2C_PERF_SEL_CM_RVF_FULL__GFX10VRS                = 0x000000f2,
    GL2C_PERF_SEL_CM_SDR_FULL__GFX10VRS                = 0x000000f3,
    GL2C_PERF_SEL_CM_MERGE_BUF_FULL__GFX10VRS          = 0x000000f4,
    GL2C_PERF_SEL_CM_DCC_STALL__GFX10VRS               = 0x000000f5,
    GL2C_PERF_SEL_CM_DCC_IN_XFC__GFX10VRS              = 0x000000f6,
    GL2C_PERF_SEL_CM_DCC_OUT_XFC__GFX10VRS             = 0x000000f7,
    GL2C_PERF_SEL_CM_DCC_OUT_1x1__GFX10VRS             = 0x000000f8,
    GL2C_PERF_SEL_CM_DCC_OUT_1x2__GFX10VRS             = 0x000000f9,
    GL2C_PERF_SEL_CM_DCC_OUT_2x1__GFX10VRS             = 0x000000fa,
    GL2C_PERF_SEL_CM_DCC_OUT_2x2__GFX10VRS             = 0x000000fb,
    GL2C_PERF_SEL_CM_DCC_OUT_UNCOMP__GFX10VRS          = 0x000000fc,
    GL2C_PERF_SEL_CM_DCC_OUT_CONST__GFX10VRS           = 0x000000fd,
    GL2C_PERF_SEL_EA_RDRET_NACK__GFX110                = 0x0000008e,
    GL2C_PERF_SEL_EA_WRRET_NACK__GFX110                = 0x0000008f,
    GL2C_PERF_SEL_GL2A_LEVEL__GFX110                   = 0x00000090,
    GL2C_PERF_SEL_PROBE_FILTER_DISABLE_TRANSITION__GFX110 = 0x00000091,
    GL2C_PERF_SEL_PROBE_FILTER_DISABLED__GFX110        = 0x00000092,
    GL2C_PERF_SEL_ALL_TC_OP_WB_OR_INV_START__GFX110    = 0x00000093,
    GL2C_PERF_SEL_ALL_TC_OP_WB_OR_INV_VOL_START__GFX110 = 0x00000094,
    GL2C_PERF_SEL_GCR_INV__GFX110                      = 0x00000095,
    GL2C_PERF_SEL_GCR_WB__GFX110                       = 0x00000096,
    GL2C_PERF_SEL_GCR_DISCARD__GFX110                  = 0x00000097,
    GL2C_PERF_SEL_GCR_RANGE__GFX110                    = 0x00000098,
    GL2C_PERF_SEL_GCR_ALL__GFX110                      = 0x00000099,
    GL2C_PERF_SEL_GCR_VOL__GFX110                      = 0x0000009a,
    GL2C_PERF_SEL_GCR_UNSHARED__GFX110                 = 0x0000009b,
    GL2C_PERF_SEL_GCR_MDC_INV__GFX110                  = 0x0000009c,
    GL2C_PERF_SEL_GCR_GL2_INV_ALL__GFX110              = 0x0000009d,
    GL2C_PERF_SEL_GCR_GL2_WB_ALL__GFX110               = 0x0000009e,
    GL2C_PERF_SEL_GCR_MDC_INV_ALL__GFX110              = 0x0000009f,
    GL2C_PERF_SEL_GCR_GL2_INV_RANGE__GFX110            = 0x000000a0,
    GL2C_PERF_SEL_GCR_GL2_WB_RANGE__GFX110             = 0x000000a1,
    GL2C_PERF_SEL_GCR_GL2_WB_INV_RANGE__GFX110         = 0x000000a2,
    GL2C_PERF_SEL_GCR_MDC_INV_RANGE__GFX110            = 0x000000a3,
    GL2C_PERF_SEL_ALL_GCR_INV_EVICT__GFX110            = 0x000000a4,
    GL2C_PERF_SEL_ALL_GCR_INV_VOL_EVICT__GFX110        = 0x000000a5,
    GL2C_PERF_SEL_ALL_GCR_WB_OR_INV_CYCLE__GFX110      = 0x000000a6,
    GL2C_PERF_SEL_ALL_GCR_WB_OR_INV_VOL_CYCLE__GFX110  = 0x000000a7,
    GL2C_PERF_SEL_ALL_GCR_WB_WRITEBACK__GFX110         = 0x000000a8,
    GL2C_PERF_SEL_GCR_INVL2_VOL_CYCLE__GFX110          = 0x000000a9,
    GL2C_PERF_SEL_GCR_INVL2_VOL_EVICT__GFX110          = 0x000000aa,
    GL2C_PERF_SEL_GCR_INVL2_VOL_START__GFX110          = 0x000000ab,
    GL2C_PERF_SEL_GCR_WBL2_VOL_CYCLE__GFX110           = 0x000000ac,
    GL2C_PERF_SEL_GCR_WBL2_VOL_START__GFX110           = 0x000000ad,
    GL2C_PERF_SEL_GCR_WBINVL2_CYCLE__GFX110            = 0x000000ae,
    GL2C_PERF_SEL_GCR_WBINVL2_EVICT__GFX110            = 0x000000af,
    GL2C_PERF_SEL_GCR_WBINVL2_START__GFX110            = 0x000000b0,
    GL2C_PERF_SEL_MDC_INV_METADATA__GFX110             = 0x000000b1,
    GL2C_PERF_SEL_MDC_REQ__GFX110                      = 0x000000b2,
    GL2C_PERF_SEL_MDC_LEVEL__GFX110                    = 0x000000b3,
    GL2C_PERF_SEL_MDC_TAG_HIT__GFX110                  = 0x000000b4,
    GL2C_PERF_SEL_MDC_SECTOR_HIT__GFX110               = 0x000000b5,
    GL2C_PERF_SEL_MDC_SECTOR_MISS__GFX110              = 0x000000b6,
    GL2C_PERF_SEL_MDC_TAG_STALL__GFX110                = 0x000000b7,
    GL2C_PERF_SEL_MDC_TAG_REPLACEMENT_LINE_IN_USE_STALL__GFX110 = 0x000000b8,
    GL2C_PERF_SEL_MDC_TAG_DESECTORIZATION_FIFO_FULL_STALL__GFX110 = 0x000000b9,
    GL2C_PERF_SEL_MDC_TAG_WAITING_FOR_INVALIDATE_COMPLETION_STALL__GFX110 = 0x000000ba,
    GL2C_PERF_SEL_CM_CHANNEL0_REQ__GFX110              = 0x000000bb,
    GL2C_PERF_SEL_CM_CHANNEL1_REQ__GFX110              = 0x000000bc,
    GL2C_PERF_SEL_CM_CHANNEL2_REQ__GFX110              = 0x000000bd,
    GL2C_PERF_SEL_CM_CHANNEL3_REQ__GFX110              = 0x000000be,
    GL2C_PERF_SEL_CM_CHANNEL4_REQ__GFX110              = 0x000000bf,
    GL2C_PERF_SEL_CM_CHANNEL5_REQ__GFX110              = 0x000000c0,
    GL2C_PERF_SEL_CM_CHANNEL6_REQ__GFX110              = 0x000000c1,
    GL2C_PERF_SEL_CM_CHANNEL7_REQ__GFX110              = 0x000000c2,
    GL2C_PERF_SEL_CM_CHANNEL8_REQ__GFX110              = 0x000000c3,
    GL2C_PERF_SEL_CM_CHANNEL9_REQ__GFX110              = 0x000000c4,
    GL2C_PERF_SEL_CM_CHANNEL10_REQ__GFX110             = 0x000000c5,
    GL2C_PERF_SEL_CM_CHANNEL11_REQ__GFX110             = 0x000000c6,
    GL2C_PERF_SEL_CM_CHANNEL12_REQ__GFX110             = 0x000000c7,
    GL2C_PERF_SEL_CM_CHANNEL13_REQ__GFX110             = 0x000000c8,
    GL2C_PERF_SEL_CM_CHANNEL14_REQ__GFX110             = 0x000000c9,
    GL2C_PERF_SEL_CM_CHANNEL15_REQ__GFX110             = 0x000000ca,
    GL2C_PERF_SEL_CM_CHANNEL16_REQ__GFX110             = 0x000000cb,
    GL2C_PERF_SEL_CM_CHANNEL17_REQ__GFX110             = 0x000000cc,
    GL2C_PERF_SEL_CM_CHANNEL18_REQ__GFX110             = 0x000000cd,
    GL2C_PERF_SEL_CM_CHANNEL19_REQ__GFX110             = 0x000000ce,
    GL2C_PERF_SEL_CM_CHANNEL20_REQ__GFX110             = 0x000000cf,
    GL2C_PERF_SEL_CM_CHANNEL21_REQ__GFX110             = 0x000000d0,
    GL2C_PERF_SEL_CM_CHANNEL22_REQ__GFX110             = 0x000000d1,
    GL2C_PERF_SEL_CM_CHANNEL23_REQ__GFX110             = 0x000000d2,
    GL2C_PERF_SEL_CM_CHANNEL24_REQ__GFX110             = 0x000000d3,
    GL2C_PERF_SEL_CM_CHANNEL25_REQ__GFX110             = 0x000000d4,
    GL2C_PERF_SEL_CM_CHANNEL26_REQ__GFX110             = 0x000000d5,
    GL2C_PERF_SEL_CM_CHANNEL27_REQ__GFX110             = 0x000000d6,
    GL2C_PERF_SEL_CM_CHANNEL28_REQ__GFX110             = 0x000000d7,
    GL2C_PERF_SEL_CM_CHANNEL29_REQ__GFX110             = 0x000000d8,
    GL2C_PERF_SEL_CM_CHANNEL30_REQ__GFX110             = 0x000000d9,
    GL2C_PERF_SEL_CM_CHANNEL31_REQ__GFX110             = 0x000000da,
    GL2C_PERF_SEL_CM_COMP_ATOMIC_COLOR_REQ__GFX110     = 0x000000db,
    GL2C_PERF_SEL_CM_COMP_ATOMIC_DEPTH16_REQ__GFX110   = 0x000000dc,
    GL2C_PERF_SEL_CM_COMP_ATOMIC_DEPTH32_REQ__GFX110   = 0x000000dd,
    GL2C_PERF_SEL_CM_COMP_ATOMIC_STENCIL_REQ__GFX110   = 0x000000de,
    GL2C_PERF_SEL_CM_COMP_WRITE_COLOR_REQ__GFX110      = 0x000000df,
    GL2C_PERF_SEL_CM_COMP_WRITE_DEPTH16_REQ__GFX110    = 0x000000e0,
    GL2C_PERF_SEL_CM_COMP_WRITE_DEPTH32_REQ__GFX110    = 0x000000e1,
    GL2C_PERF_SEL_CM_COMP_WRITE_STENCIL_REQ__GFX110    = 0x000000e2,
    GL2C_PERF_SEL_CM_COMP_READ_REQ__GFX110             = 0x000000e3,
    GL2C_PERF_SEL_CM_READ_BACK_REQ__GFX110             = 0x000000e4,
    GL2C_PERF_SEL_CM_METADATA_WR_REQ__GFX110           = 0x000000e5,
    GL2C_PERF_SEL_CM_WR_ACK_REQ__GFX110                = 0x000000e6,
    GL2C_PERF_SEL_CM_NO_ACK_REQ__GFX110                = 0x000000e7,
    GL2C_PERF_SEL_CM_NOOP_REQ__GFX110                  = 0x000000e8,
    GL2C_PERF_SEL_CM_COMP_COLOR_EN_REQ__GFX110         = 0x000000e9,
    GL2C_PERF_SEL_CM_COMP_COLOR_DIS_REQ__GFX110        = 0x000000ea,
    GL2C_PERF_SEL_CM_COMP_STENCIL_REQ__GFX110          = 0x000000eb,
    GL2C_PERF_SEL_CM_COMP_DEPTH16_REQ__GFX110          = 0x000000ec,
    GL2C_PERF_SEL_CM_COMP_DEPTH32_REQ__GFX110          = 0x000000ed,
    GL2C_PERF_SEL_CM_COMP_RB_SKIP_REQ__GFX110          = 0x000000ee,
    GL2C_PERF_SEL_CM_COLOR_32B_WR_REQ__GFX110          = 0x000000ef,
    GL2C_PERF_SEL_CM_COLOR_64B_WR_REQ__GFX110          = 0x000000f0,
    GL2C_PERF_SEL_CM_FULL_WRITE_REQ__GFX110            = 0x000000f1,
    GL2C_PERF_SEL_CM_RVF_FULL__GFX110                  = 0x000000f2,
    GL2C_PERF_SEL_CM_SDR_FULL__GFX110                  = 0x000000f3,
    GL2C_PERF_SEL_CM_MERGE_BUF_FULL__GFX110            = 0x000000f4,
    GL2C_PERF_SEL_CM_DCC_STALL__GFX110                 = 0x000000f5,
    GL2C_PERF_SEL_CM_DCC_IN_XFC__GFX110                = 0x000000f6,
    GL2C_PERF_SEL_CM_DCC_OUT_XFC__GFX110               = 0x000000f7,
    GL2C_PERF_SEL_CM_DCC_OUT_1x1__GFX110               = 0x000000f8,
    GL2C_PERF_SEL_CM_DCC_OUT_1x2__GFX110               = 0x000000f9,
    GL2C_PERF_SEL_CM_DCC_OUT_2x1__GFX110               = 0x000000fa,
    GL2C_PERF_SEL_CM_DCC_OUT_2x2__GFX110               = 0x000000fb,
    GL2C_PERF_SEL_CM_DCC_OUT_UNCOMP__GFX110            = 0x000000fc,
    GL2C_PERF_SEL_CM_DCC_OUT_CONST2SINGLE__GFX110      = 0x000000fd,
    GL2C_PERF_SEL_CM_DCC_OUT_CONST2CLEAR__GFX110       = 0x000000fe,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT16__GFX110    = 0x000000ff,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT17__GFX110    = 0x00000100,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT18__GFX110    = 0x00000101,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT19__GFX110    = 0x00000102,
    GL2C_PERF_SEL_EA_RDRET_NACK__STX1                  = 0x0000008e,
    GL2C_PERF_SEL_EA_WRRET_NACK__STX1                  = 0x0000008f,
    GL2C_PERF_SEL_GL2A_LEVEL__STX1                     = 0x00000090,
    GL2C_PERF_SEL_PROBE_FILTER_DISABLE_TRANSITION__STX1 = 0x00000091,
    GL2C_PERF_SEL_PROBE_FILTER_DISABLED__STX1          = 0x00000092,
    GL2C_PERF_SEL_ALL_TC_OP_WB_OR_INV_START__STX1      = 0x00000093,
    GL2C_PERF_SEL_ALL_TC_OP_WB_OR_INV_VOL_START__STX1  = 0x00000094,
    GL2C_PERF_SEL_GCR_INV__STX1                        = 0x00000095,
    GL2C_PERF_SEL_GCR_WB__STX1                         = 0x00000096,
    GL2C_PERF_SEL_GCR_DISCARD__STX1                    = 0x00000097,
    GL2C_PERF_SEL_GCR_RANGE__STX1                      = 0x00000098,
    GL2C_PERF_SEL_GCR_ALL__STX1                        = 0x00000099,
    GL2C_PERF_SEL_GCR_VOL__STX1                        = 0x0000009a,
    GL2C_PERF_SEL_GCR_UNSHARED__STX1                   = 0x0000009b,
    GL2C_PERF_SEL_GCR_MDC_INV__STX1                    = 0x0000009c,
    GL2C_PERF_SEL_GCR_GL2_INV_ALL__STX1                = 0x0000009d,
    GL2C_PERF_SEL_GCR_GL2_WB_ALL__STX1                 = 0x0000009e,
    GL2C_PERF_SEL_GCR_MDC_INV_ALL__STX1                = 0x0000009f,
    GL2C_PERF_SEL_GCR_GL2_INV_RANGE__STX1              = 0x000000a0,
    GL2C_PERF_SEL_GCR_GL2_WB_RANGE__STX1               = 0x000000a1,
    GL2C_PERF_SEL_GCR_GL2_WB_INV_RANGE__STX1           = 0x000000a2,
    GL2C_PERF_SEL_GCR_MDC_INV_RANGE__STX1              = 0x000000a3,
    GL2C_PERF_SEL_ALL_GCR_INV_EVICT__STX1              = 0x000000a4,
    GL2C_PERF_SEL_ALL_GCR_INV_VOL_EVICT__STX1          = 0x000000a5,
    GL2C_PERF_SEL_ALL_GCR_WB_OR_INV_CYCLE__STX1        = 0x000000a6,
    GL2C_PERF_SEL_ALL_GCR_WB_OR_INV_VOL_CYCLE__STX1    = 0x000000a7,
    GL2C_PERF_SEL_ALL_GCR_WB_WRITEBACK__STX1           = 0x000000a8,
    GL2C_PERF_SEL_GCR_INVL2_VOL_CYCLE__STX1            = 0x000000a9,
    GL2C_PERF_SEL_GCR_INVL2_VOL_EVICT__STX1            = 0x000000aa,
    GL2C_PERF_SEL_GCR_INVL2_VOL_START__STX1            = 0x000000ab,
    GL2C_PERF_SEL_GCR_WBL2_VOL_CYCLE__STX1             = 0x000000ac,
    GL2C_PERF_SEL_GCR_WBL2_VOL_START__STX1             = 0x000000ad,
    GL2C_PERF_SEL_GCR_WBINVL2_CYCLE__STX1              = 0x000000ae,
    GL2C_PERF_SEL_GCR_WBINVL2_EVICT__STX1              = 0x000000af,
    GL2C_PERF_SEL_GCR_WBINVL2_START__STX1              = 0x000000b0,
    GL2C_PERF_SEL_MDC_INV_METADATA__STX1               = 0x000000b1,
    GL2C_PERF_SEL_MDC_REQ__STX1                        = 0x000000b2,
    GL2C_PERF_SEL_MDC_LEVEL__STX1                      = 0x000000b3,
    GL2C_PERF_SEL_MDC_TAG_HIT__STX1                    = 0x000000b4,
    GL2C_PERF_SEL_MDC_SECTOR_HIT__STX1                 = 0x000000b5,
    GL2C_PERF_SEL_MDC_SECTOR_MISS__STX1                = 0x000000b6,
    GL2C_PERF_SEL_MDC_TAG_STALL__STX1                  = 0x000000b7,
    GL2C_PERF_SEL_MDC_TAG_REPLACEMENT_LINE_IN_USE_STALL__STX1 = 0x000000b8,
    GL2C_PERF_SEL_MDC_TAG_DESECTORIZATION_FIFO_FULL_STALL__STX1 = 0x000000b9,
    GL2C_PERF_SEL_MDC_TAG_WAITING_FOR_INVALIDATE_COMPLETION_STALL__STX1 = 0x000000ba,
    GL2C_PERF_SEL_CM_CHANNEL0_REQ__STX1                = 0x000000bb,
    GL2C_PERF_SEL_CM_CHANNEL1_REQ__STX1                = 0x000000bc,
    GL2C_PERF_SEL_CM_CHANNEL2_REQ__STX1                = 0x000000bd,
    GL2C_PERF_SEL_CM_CHANNEL3_REQ__STX1                = 0x000000be,
    GL2C_PERF_SEL_CM_CHANNEL4_REQ__STX1                = 0x000000bf,
    GL2C_PERF_SEL_CM_CHANNEL5_REQ__STX1                = 0x000000c0,
    GL2C_PERF_SEL_CM_CHANNEL6_REQ__STX1                = 0x000000c1,
    GL2C_PERF_SEL_CM_CHANNEL7_REQ__STX1                = 0x000000c2,
    GL2C_PERF_SEL_CM_CHANNEL8_REQ__STX1                = 0x000000c3,
    GL2C_PERF_SEL_CM_CHANNEL9_REQ__STX1                = 0x000000c4,
    GL2C_PERF_SEL_CM_CHANNEL10_REQ__STX1               = 0x000000c5,
    GL2C_PERF_SEL_CM_CHANNEL11_REQ__STX1               = 0x000000c6,
    GL2C_PERF_SEL_CM_CHANNEL12_REQ__STX1               = 0x000000c7,
    GL2C_PERF_SEL_CM_CHANNEL13_REQ__STX1               = 0x000000c8,
    GL2C_PERF_SEL_CM_CHANNEL14_REQ__STX1               = 0x000000c9,
    GL2C_PERF_SEL_CM_CHANNEL15_REQ__STX1               = 0x000000ca,
    GL2C_PERF_SEL_CM_CHANNEL16_REQ__STX1               = 0x000000cb,
    GL2C_PERF_SEL_CM_CHANNEL17_REQ__STX1               = 0x000000cc,
    GL2C_PERF_SEL_CM_CHANNEL18_REQ__STX1               = 0x000000cd,
    GL2C_PERF_SEL_CM_CHANNEL19_REQ__STX1               = 0x000000ce,
    GL2C_PERF_SEL_CM_CHANNEL20_REQ__STX1               = 0x000000cf,
    GL2C_PERF_SEL_CM_CHANNEL21_REQ__STX1               = 0x000000d0,
    GL2C_PERF_SEL_CM_CHANNEL22_REQ__STX1               = 0x000000d1,
    GL2C_PERF_SEL_CM_CHANNEL23_REQ__STX1               = 0x000000d2,
    GL2C_PERF_SEL_CM_CHANNEL24_REQ__STX1               = 0x000000d3,
    GL2C_PERF_SEL_CM_CHANNEL25_REQ__STX1               = 0x000000d4,
    GL2C_PERF_SEL_CM_CHANNEL26_REQ__STX1               = 0x000000d5,
    GL2C_PERF_SEL_CM_CHANNEL27_REQ__STX1               = 0x000000d6,
    GL2C_PERF_SEL_CM_CHANNEL28_REQ__STX1               = 0x000000d7,
    GL2C_PERF_SEL_CM_CHANNEL29_REQ__STX1               = 0x000000d8,
    GL2C_PERF_SEL_CM_CHANNEL30_REQ__STX1               = 0x000000d9,
    GL2C_PERF_SEL_CM_CHANNEL31_REQ__STX1               = 0x000000da,
    GL2C_PERF_SEL_CM_COMP_ATOMIC_COLOR_REQ__STX1       = 0x000000db,
    GL2C_PERF_SEL_CM_COMP_ATOMIC_DEPTH16_REQ__STX1     = 0x000000dc,
    GL2C_PERF_SEL_CM_COMP_ATOMIC_DEPTH32_REQ__STX1     = 0x000000dd,
    GL2C_PERF_SEL_CM_COMP_ATOMIC_STENCIL_REQ__STX1     = 0x000000de,
    GL2C_PERF_SEL_CM_COMP_WRITE_COLOR_REQ__STX1        = 0x000000df,
    GL2C_PERF_SEL_CM_COMP_WRITE_DEPTH16_REQ__STX1      = 0x000000e0,
    GL2C_PERF_SEL_CM_COMP_WRITE_DEPTH32_REQ__STX1      = 0x000000e1,
    GL2C_PERF_SEL_CM_COMP_WRITE_STENCIL_REQ__STX1      = 0x000000e2,
    GL2C_PERF_SEL_CM_COMP_READ_REQ__STX1               = 0x000000e3,
    GL2C_PERF_SEL_CM_READ_BACK_REQ__STX1               = 0x000000e4,
    GL2C_PERF_SEL_CM_METADATA_WR_REQ__STX1             = 0x000000e5,
    GL2C_PERF_SEL_CM_WR_ACK_REQ__STX1                  = 0x000000e6,
    GL2C_PERF_SEL_CM_NO_ACK_REQ__STX1                  = 0x000000e7,
    GL2C_PERF_SEL_CM_NOOP_REQ__STX1                    = 0x000000e8,
    GL2C_PERF_SEL_CM_COMP_COLOR_EN_REQ__STX1           = 0x000000e9,
    GL2C_PERF_SEL_CM_COMP_COLOR_DIS_REQ__STX1          = 0x000000ea,
    GL2C_PERF_SEL_CM_COMP_STENCIL_REQ__STX1            = 0x000000eb,
    GL2C_PERF_SEL_CM_COMP_DEPTH16_REQ__STX1            = 0x000000ec,
    GL2C_PERF_SEL_CM_COMP_DEPTH32_REQ__STX1            = 0x000000ed,
    GL2C_PERF_SEL_CM_COMP_RB_SKIP_REQ__STX1            = 0x000000ee,
    GL2C_PERF_SEL_CM_COLOR_32B_WR_REQ__STX1            = 0x000000ef,
    GL2C_PERF_SEL_CM_COLOR_64B_WR_REQ__STX1            = 0x000000f0,
    GL2C_PERF_SEL_CM_FULL_WRITE_REQ__STX1              = 0x000000f1,
    GL2C_PERF_SEL_CM_RVF_FULL__STX1                    = 0x000000f2,
    GL2C_PERF_SEL_CM_SDR_FULL__STX1                    = 0x000000f3,
    GL2C_PERF_SEL_CM_MERGE_BUF_FULL__STX1              = 0x000000f4,
    GL2C_PERF_SEL_CM_DCC_STALL__STX1                   = 0x000000f5,
    GL2C_PERF_SEL_CM_DCC_IN_XFC__STX1                  = 0x000000f6,
    GL2C_PERF_SEL_CM_DCC_OUT_XFC__STX1                 = 0x000000f7,
    GL2C_PERF_SEL_CM_DCC_OUT_1x1__STX1                 = 0x000000f8,
    GL2C_PERF_SEL_CM_DCC_OUT_1x2__STX1                 = 0x000000f9,
    GL2C_PERF_SEL_CM_DCC_OUT_2x1__STX1                 = 0x000000fa,
    GL2C_PERF_SEL_CM_DCC_OUT_2x2__STX1                 = 0x000000fb,
    GL2C_PERF_SEL_CM_DCC_OUT_UNCOMP__STX1              = 0x000000fc,
    GL2C_PERF_SEL_CM_DCC_OUT_CONST2SINGLE__STX1        = 0x000000fd,
    GL2C_PERF_SEL_CM_DCC_OUT_CONST2CLEAR__STX1         = 0x000000fe,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT16__STX1      = 0x000000ff,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT17__STX1      = 0x00000100,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT18__STX1      = 0x00000101,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT19__STX1      = 0x00000102,
#if CHIP_HDR_STRIX_HALO
    GL2C_PERF_SEL_MDC_MULTIPHASE_REQ__STXH             = 0x0000008e,
    GL2C_PERF_SEL_COMPRESSED_MULTIPHASE_READ_96_REQ__STXH = 0x0000008f,
    GL2C_PERF_SEL_COMPRESSED_MULTIPHASE_READ_128_REQ__STXH = 0x00000090,
    GL2C_PERF_SEL_COMPRESSED_MULTIPHASE_READ_160_REQ__STXH = 0x00000091,
    GL2C_PERF_SEL_COMPRESSED_MULTIPHASE_READ_192_REQ__STXH = 0x00000092,
    GL2C_PERF_SEL_COMPRESSED_MULTIPHASE_READ_224_REQ__STXH = 0x00000093,
    GL2C_PERF_SEL_MDC_MULTIPHASE_CLIENT_REQ__STXH      = 0x00000094,
    GL2C_PERF_SEL_COMPRESSED_MULTIPHASE_CLIENT_READ_96_REQ__STXH = 0x00000095,
    GL2C_PERF_SEL_COMPRESSED_MULTIPHASE_CLIENT_READ_128_REQ__STXH = 0x00000096,
    GL2C_PERF_SEL_COMPRESSED_MULTIPHASE_CLIENT_READ_160_REQ__STXH = 0x00000097,
    GL2C_PERF_SEL_COMPRESSED_MULTIPHASE_CLIENT_READ_192_REQ__STXH = 0x00000098,
    GL2C_PERF_SEL_COMPRESSED_MULTIPHASE_CLIENT_READ_224_REQ__STXH = 0x00000099,
    GL2C_PERF_SEL_EA_RDRET_NACK__STXH                  = 0x0000009a,
    GL2C_PERF_SEL_EA_WRRET_NACK__STXH                  = 0x0000009b,
    GL2C_PERF_SEL_GL2A_LEVEL__STXH                     = 0x0000009c,
    GL2C_PERF_SEL_PROBE_FILTER_DISABLE_TRANSITION__STXH = 0x0000009d,
    GL2C_PERF_SEL_PROBE_FILTER_DISABLED__STXH          = 0x0000009e,
    GL2C_PERF_SEL_ALL_TC_OP_WB_OR_INV_START__STXH      = 0x0000009f,
    GL2C_PERF_SEL_ALL_TC_OP_WB_OR_INV_VOL_START__STXH  = 0x000000a0,
    GL2C_PERF_SEL_GCR_INV__STXH                        = 0x000000a1,
    GL2C_PERF_SEL_GCR_WB__STXH                         = 0x000000a2,
    GL2C_PERF_SEL_GCR_DISCARD__STXH                    = 0x000000a3,
    GL2C_PERF_SEL_GCR_RANGE__STXH                      = 0x000000a4,
    GL2C_PERF_SEL_GCR_ALL__STXH                        = 0x000000a5,
    GL2C_PERF_SEL_GCR_VOL__STXH                        = 0x000000a6,
    GL2C_PERF_SEL_GCR_UNSHARED__STXH                   = 0x000000a7,
    GL2C_PERF_SEL_GCR_MDC_INV__STXH                    = 0x000000a8,
    GL2C_PERF_SEL_GCR_GL2_INV_ALL__STXH                = 0x000000a9,
    GL2C_PERF_SEL_GCR_GL2_WB_ALL__STXH                 = 0x000000aa,
    GL2C_PERF_SEL_GCR_MDC_INV_ALL__STXH                = 0x000000ab,
    GL2C_PERF_SEL_GCR_GL2_INV_RANGE__STXH              = 0x000000ac,
    GL2C_PERF_SEL_GCR_GL2_WB_RANGE__STXH               = 0x000000ad,
    GL2C_PERF_SEL_GCR_GL2_WB_INV_RANGE__STXH           = 0x000000ae,
    GL2C_PERF_SEL_GCR_MDC_INV_RANGE__STXH              = 0x000000af,
    GL2C_PERF_SEL_ALL_GCR_INV_EVICT__STXH              = 0x000000b0,
    GL2C_PERF_SEL_ALL_GCR_INV_VOL_EVICT__STXH          = 0x000000b1,
    GL2C_PERF_SEL_ALL_GCR_WB_OR_INV_CYCLE__STXH        = 0x000000b2,
    GL2C_PERF_SEL_ALL_GCR_WB_OR_INV_VOL_CYCLE__STXH    = 0x000000b3,
    GL2C_PERF_SEL_ALL_GCR_WB_WRITEBACK__STXH           = 0x000000b4,
    GL2C_PERF_SEL_GCR_INVL2_VOL_CYCLE__STXH            = 0x000000b5,
    GL2C_PERF_SEL_GCR_INVL2_VOL_EVICT__STXH            = 0x000000b6,
    GL2C_PERF_SEL_GCR_INVL2_VOL_START__STXH            = 0x000000b7,
    GL2C_PERF_SEL_GCR_WBL2_VOL_CYCLE__STXH             = 0x000000b8,
    GL2C_PERF_SEL_GCR_WBL2_VOL_START__STXH             = 0x000000b9,
    GL2C_PERF_SEL_GCR_WBINVL2_CYCLE__STXH              = 0x000000ba,
    GL2C_PERF_SEL_GCR_WBINVL2_EVICT__STXH              = 0x000000bb,
    GL2C_PERF_SEL_GCR_WBINVL2_START__STXH              = 0x000000bc,
    GL2C_PERF_SEL_MDC_INV_METADATA__STXH               = 0x000000bd,
    GL2C_PERF_SEL_MDC_REQ__STXH                        = 0x000000be,
    GL2C_PERF_SEL_MDC_LEVEL__STXH                      = 0x000000bf,
    GL2C_PERF_SEL_MDC_TAG_HIT__STXH                    = 0x000000c0,
    GL2C_PERF_SEL_MDC_SECTOR_HIT__STXH                 = 0x000000c1,
    GL2C_PERF_SEL_MDC_SECTOR_MISS__STXH                = 0x000000c2,
    GL2C_PERF_SEL_MDC_TAG_STALL__STXH                  = 0x000000c3,
    GL2C_PERF_SEL_MDC_TAG_REPLACEMENT_LINE_IN_USE_STALL__STXH = 0x000000c4,
    GL2C_PERF_SEL_MDC_TAG_DESECTORIZATION_FIFO_FULL_STALL__STXH = 0x000000c5,
    GL2C_PERF_SEL_MDC_TAG_WAITING_FOR_INVALIDATE_COMPLETION_STALL__STXH = 0x000000c6,
    GL2C_PERF_SEL_CM_CHANNEL0_REQ__STXH                = 0x000000c7,
    GL2C_PERF_SEL_CM_CHANNEL1_REQ__STXH                = 0x000000c8,
    GL2C_PERF_SEL_CM_CHANNEL2_REQ__STXH                = 0x000000c9,
    GL2C_PERF_SEL_CM_CHANNEL3_REQ__STXH                = 0x000000ca,
    GL2C_PERF_SEL_CM_CHANNEL4_REQ__STXH                = 0x000000cb,
    GL2C_PERF_SEL_CM_CHANNEL5_REQ__STXH                = 0x000000cc,
    GL2C_PERF_SEL_CM_CHANNEL6_REQ__STXH                = 0x000000cd,
    GL2C_PERF_SEL_CM_CHANNEL7_REQ__STXH                = 0x000000ce,
    GL2C_PERF_SEL_CM_CHANNEL8_REQ__STXH                = 0x000000cf,
    GL2C_PERF_SEL_CM_CHANNEL9_REQ__STXH                = 0x000000d0,
    GL2C_PERF_SEL_CM_CHANNEL10_REQ__STXH               = 0x000000d1,
    GL2C_PERF_SEL_CM_CHANNEL11_REQ__STXH               = 0x000000d2,
    GL2C_PERF_SEL_CM_CHANNEL12_REQ__STXH               = 0x000000d3,
    GL2C_PERF_SEL_CM_CHANNEL13_REQ__STXH               = 0x000000d4,
    GL2C_PERF_SEL_CM_CHANNEL14_REQ__STXH               = 0x000000d5,
    GL2C_PERF_SEL_CM_CHANNEL15_REQ__STXH               = 0x000000d6,
    GL2C_PERF_SEL_CM_CHANNEL16_REQ__STXH               = 0x000000d7,
    GL2C_PERF_SEL_CM_CHANNEL17_REQ__STXH               = 0x000000d8,
    GL2C_PERF_SEL_CM_CHANNEL18_REQ__STXH               = 0x000000d9,
    GL2C_PERF_SEL_CM_CHANNEL19_REQ__STXH               = 0x000000da,
    GL2C_PERF_SEL_CM_CHANNEL20_REQ__STXH               = 0x000000db,
    GL2C_PERF_SEL_CM_CHANNEL21_REQ__STXH               = 0x000000dc,
    GL2C_PERF_SEL_CM_CHANNEL22_REQ__STXH               = 0x000000dd,
    GL2C_PERF_SEL_CM_CHANNEL23_REQ__STXH               = 0x000000de,
    GL2C_PERF_SEL_CM_CHANNEL24_REQ__STXH               = 0x000000df,
    GL2C_PERF_SEL_CM_CHANNEL25_REQ__STXH               = 0x000000e0,
    GL2C_PERF_SEL_CM_CHANNEL26_REQ__STXH               = 0x000000e1,
    GL2C_PERF_SEL_CM_CHANNEL27_REQ__STXH               = 0x000000e2,
    GL2C_PERF_SEL_CM_CHANNEL28_REQ__STXH               = 0x000000e3,
    GL2C_PERF_SEL_CM_CHANNEL29_REQ__STXH               = 0x000000e4,
    GL2C_PERF_SEL_CM_CHANNEL30_REQ__STXH               = 0x000000e5,
    GL2C_PERF_SEL_CM_CHANNEL31_REQ__STXH               = 0x000000e6,
    GL2C_PERF_SEL_CM_COMP_ATOMIC_COLOR_REQ__STXH       = 0x000000e7,
    GL2C_PERF_SEL_CM_COMP_ATOMIC_DEPTH16_REQ__STXH     = 0x000000e8,
    GL2C_PERF_SEL_CM_COMP_ATOMIC_DEPTH32_REQ__STXH     = 0x000000e9,
    GL2C_PERF_SEL_CM_COMP_ATOMIC_STENCIL_REQ__STXH     = 0x000000ea,
    GL2C_PERF_SEL_CM_COMP_WRITE_COLOR_REQ__STXH        = 0x000000eb,
    GL2C_PERF_SEL_CM_COMP_WRITE_DEPTH16_REQ__STXH      = 0x000000ec,
    GL2C_PERF_SEL_CM_COMP_WRITE_DEPTH32_REQ__STXH      = 0x000000ed,
    GL2C_PERF_SEL_CM_COMP_WRITE_STENCIL_REQ__STXH      = 0x000000ee,
    GL2C_PERF_SEL_CM_COMP_READ_REQ__STXH               = 0x000000ef,
    GL2C_PERF_SEL_CM_READ_BACK_REQ__STXH               = 0x000000f0,
    GL2C_PERF_SEL_CM_METADATA_WR_REQ__STXH             = 0x000000f1,
    GL2C_PERF_SEL_CM_WR_ACK_REQ__STXH                  = 0x000000f2,
    GL2C_PERF_SEL_CM_NO_ACK_REQ__STXH                  = 0x000000f3,
    GL2C_PERF_SEL_CM_NOOP_REQ__STXH                    = 0x000000f4,
    GL2C_PERF_SEL_CM_COMP_COLOR_EN_REQ__STXH           = 0x000000f5,
    GL2C_PERF_SEL_CM_COMP_COLOR_DIS_REQ__STXH          = 0x000000f6,
    GL2C_PERF_SEL_CM_COMP_STENCIL_REQ__STXH            = 0x000000f7,
    GL2C_PERF_SEL_CM_COMP_DEPTH16_REQ__STXH            = 0x000000f8,
    GL2C_PERF_SEL_CM_COMP_DEPTH32_REQ__STXH            = 0x000000f9,
    GL2C_PERF_SEL_CM_COMP_RB_SKIP_REQ__STXH            = 0x000000fa,
    GL2C_PERF_SEL_CM_COLOR_32B_WR_REQ__STXH            = 0x000000fb,
    GL2C_PERF_SEL_CM_COLOR_64B_WR_REQ__STXH            = 0x000000fc,
    GL2C_PERF_SEL_CM_FULL_WRITE_REQ__STXH              = 0x000000fd,
    GL2C_PERF_SEL_CM_RVF_FULL__STXH                    = 0x000000fe,
    GL2C_PERF_SEL_CM_SDR_FULL__STXH                    = 0x000000ff,
    GL2C_PERF_SEL_CM_MERGE_BUF_FULL__STXH              = 0x00000100,
    GL2C_PERF_SEL_CM_DCC_STALL__STXH                   = 0x00000101,
    GL2C_PERF_SEL_CM_DCC_IN_XFC__STXH                  = 0x00000102,
    GL2C_PERF_SEL_CM_DCC_OUT_XFC__STXH                 = 0x00000103,
    GL2C_PERF_SEL_CM_DCC_OUT_1x1__STXH                 = 0x00000104,
    GL2C_PERF_SEL_CM_DCC_OUT_1x2__STXH                 = 0x00000105,
    GL2C_PERF_SEL_CM_DCC_OUT_2x1__STXH                 = 0x00000106,
    GL2C_PERF_SEL_CM_DCC_OUT_2x2__STXH                 = 0x00000107,
    GL2C_PERF_SEL_CM_DCC_OUT_UNCOMP__STXH              = 0x00000108,
    GL2C_PERF_SEL_CM_DCC_OUT_CONST2SINGLE__STXH        = 0x00000109,
    GL2C_PERF_SEL_CM_DCC_OUT_CONST2CLEAR__STXH         = 0x0000010a,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT16__STXH      = 0x0000010b,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT17__STXH      = 0x0000010c,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT18__STXH      = 0x0000010d,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT19__STXH      = 0x0000010e,
#endif
} GL2C_PERF_SEL;

constexpr unsigned int MaxGl2cPerfSelGfx101            = GL2C_PERF_SEL_CM_DCC_STALL__GFX101;
constexpr unsigned int MaxGl2cPerfSelGfx10Vrs          = GL2C_PERF_SEL_CM_DCC_OUT_CONST__GFX10VRS;
constexpr unsigned int MaxGl2cPerfSelGfx110            = GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT19__GFX110;
constexpr unsigned int MaxGl2cPerfSelStx1              = GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT19__STX1;
#if CHIP_HDR_STRIX_HALO
constexpr unsigned int MaxGl2cPerfSelStxH              = GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT19__STXH;
#endif

typedef enum GL2_CACHE_POLICIES {
    GL2_CACHE_POLICY_LRU                               = 0x00000000,
    GL2_CACHE_POLICY_STREAM                            = 0x00000001,
    GL2_CACHE_POLICY_NOA                               = 0x00000002,
    GL2_CACHE_POLICY_BYPASS                            = 0x00000003,
} GL2_CACHE_POLICIES;

typedef enum GRBM_PERF_SEL {
    GRBM_PERF_SEL_COUNT                                = 0x00000000,
    GRBM_PERF_SEL_USER_DEFINED                         = 0x00000001,
    GRBM_PERF_SEL_GUI_ACTIVE                           = 0x00000002,
    GRBM_PERF_SEL_CP_BUSY                              = 0x00000003,
    GRBM_PERF_SEL_CP_COHER_BUSY                        = 0x00000004,
    GRBM_PERF_SEL_CP_DMA_BUSY                          = 0x00000005,
    GRBM_PERF_SEL_CB_BUSY                              = 0x00000006,
    GRBM_PERF_SEL_DB_BUSY                              = 0x00000007,
    GRBM_PERF_SEL_PA_BUSY                              = 0x00000008,
    GRBM_PERF_SEL_SC_BUSY                              = 0x00000009,
    GRBM_PERF_SEL_SPI_BUSY                             = 0x0000000b,
    GRBM_PERF_SEL_SX_BUSY                              = 0x0000000c,
    GRBM_PERF_SEL_TA_BUSY                              = 0x0000000d,
    GRBM_PERF_SEL_CB_CLEAN                             = 0x0000000e,
    GRBM_PERF_SEL_DB_CLEAN                             = 0x0000000f,
    GRBM_PERF_SEL_GDS_BUSY                             = 0x00000019,
    GRBM_PERF_SEL_BCI_BUSY                             = 0x0000001a,
    GRBM_PERF_SEL_RLC_BUSY                             = 0x0000001b,
    GRBM_PERF_SEL_TCP_BUSY                             = 0x0000001c,
    GRBM_PERF_SEL_CPG_BUSY                             = 0x0000001d,
    GRBM_PERF_SEL_CPC_BUSY                             = 0x0000001e,
    GRBM_PERF_SEL_CPF_BUSY                             = 0x0000001f,
    GRBM_PERF_SEL_GE_BUSY                              = 0x00000020,
    GRBM_PERF_SEL_GE_NO_DMA_BUSY                       = 0x00000021,
    GRBM_PERF_SEL_UTCL2_BUSY                           = 0x00000022,
    GRBM_PERF_SEL_EA_BUSY                              = 0x00000023,
    GRBM_PERF_SEL_RMI_BUSY                             = 0x00000024,
    GRBM_PERF_SEL_CPAXI_BUSY                           = 0x00000025,
    GRBM_PERF_SEL_UTCL1_BUSY                           = 0x00000027,
    GRBM_PERF_SEL_GL2CC_BUSY                           = 0x00000028,
    GRBM_PERF_SEL_SDMA_BUSY                            = 0x00000029,
    GRBM_PERF_SEL_CH_BUSY                              = 0x0000002a,
    GRBM_PERF_SEL_PH_BUSY                              = 0x0000002b,
    GRBM_PERF_SEL_PMM_BUSY                             = 0x0000002c,
    GRBM_PERF_SEL_GUS_BUSY                             = 0x0000002d,
    GRBM_PERF_SEL_GL1CC_BUSY                           = 0x0000002e,
    GRBM_PERF_SEL_RESERVED_6__GFX10                    = 0x0000000a,
    GRBM_PERF_SEL_RESERVED_5__GFX10                    = 0x00000010,
    GRBM_PERF_SEL_RESERVED_9__GFX10                    = 0x00000011,
    GRBM_PERF_SEL_RESERVED_4__GFX10                    = 0x00000012,
    GRBM_PERF_SEL_RESERVED_3__GFX10                    = 0x00000013,
    GRBM_PERF_SEL_RESERVED_2__GFX10                    = 0x00000014,
    GRBM_PERF_SEL_RESERVED_1__GFX10                    = 0x00000015,
    GRBM_PERF_SEL_RESERVED_0__GFX10                    = 0x00000016,
    GRBM_PERF_SEL_RESERVED_8__GFX10                    = 0x00000017,
    GRBM_PERF_SEL_RESERVED_7__GFX10                    = 0x00000018,
    GRBM_PERF_SEL_ANY_ACTIVE_F_BUSY__GFX11             = 0x0000002f,
    GRBM_PERF_SEL_GL1H_BUSY__GFX11                     = 0x00000030,
    GRBM_PERF_SEL_PC_BUSY__GFX11                       = 0x00000031,
    GRBM_PERF_SEL_RSVD_BUSY__RAPHAEL                   = 0x00000026,
} GRBM_PERF_SEL;

constexpr unsigned int MaxGrbmPerfSelGfx10             = GRBM_PERF_SEL_GL1CC_BUSY;
constexpr unsigned int MaxGrbmPerfSelGfx11             = GRBM_PERF_SEL_PC_BUSY__GFX11;

typedef enum GRBM_SE0_PERF_SEL {
    GRBM_SE0_PERF_SEL_COUNT                            = 0x00000000,
    GRBM_SE0_PERF_SEL_USER_DEFINED                     = 0x00000001,
    GRBM_SE0_PERF_SEL_CB_BUSY                          = 0x00000002,
    GRBM_SE0_PERF_SEL_DB_BUSY                          = 0x00000003,
    GRBM_SE0_PERF_SEL_SC_BUSY                          = 0x00000004,
    GRBM_SE0_PERF_SEL_SPI_BUSY                         = 0x00000006,
    GRBM_SE0_PERF_SEL_SX_BUSY                          = 0x00000007,
    GRBM_SE0_PERF_SEL_TA_BUSY                          = 0x00000008,
    GRBM_SE0_PERF_SEL_CB_CLEAN                         = 0x00000009,
    GRBM_SE0_PERF_SEL_DB_CLEAN                         = 0x0000000a,
    GRBM_SE0_PERF_SEL_PA_BUSY                          = 0x0000000c,
    GRBM_SE0_PERF_SEL_BCI_BUSY                         = 0x0000000e,
    GRBM_SE0_PERF_SEL_RMI_BUSY                         = 0x0000000f,
    GRBM_SE0_PERF_SEL_UTCL1_BUSY                       = 0x00000010,
    GRBM_SE0_PERF_SEL_TCP_BUSY                         = 0x00000011,
    GRBM_SE0_PERF_SEL_GL1CC_BUSY                       = 0x00000012,
    GRBM_SE0_PERF_SEL_RESERVED_1__GFX10                = 0x00000005,
    GRBM_SE0_PERF_SEL_RESERVED_0__GFX10                = 0x0000000b,
    GRBM_SE0_PERF_SEL_RESERVED_2__GFX10                = 0x0000000d,
    GRBM_SE0_PERF_SEL_GL1H_BUSY__GFX11                 = 0x00000013,
    GRBM_SE0_PERF_SEL_PC_BUSY__GFX11                   = 0x00000014,
} GRBM_SE0_PERF_SEL;

constexpr unsigned int MaxGrbmSe0PerfSelGfx10          = GRBM_SE0_PERF_SEL_GL1CC_BUSY;
constexpr unsigned int MaxGrbmSe0PerfSelGfx11          = GRBM_SE0_PERF_SEL_PC_BUSY__GFX11;

typedef enum GRBM_SE1_PERF_SEL {
    GRBM_SE1_PERF_SEL_COUNT                            = 0x00000000,
    GRBM_SE1_PERF_SEL_USER_DEFINED                     = 0x00000001,
    GRBM_SE1_PERF_SEL_CB_BUSY                          = 0x00000002,
    GRBM_SE1_PERF_SEL_DB_BUSY                          = 0x00000003,
    GRBM_SE1_PERF_SEL_SC_BUSY                          = 0x00000004,
    GRBM_SE1_PERF_SEL_SPI_BUSY                         = 0x00000006,
    GRBM_SE1_PERF_SEL_SX_BUSY                          = 0x00000007,
    GRBM_SE1_PERF_SEL_TA_BUSY                          = 0x00000008,
    GRBM_SE1_PERF_SEL_CB_CLEAN                         = 0x00000009,
    GRBM_SE1_PERF_SEL_DB_CLEAN                         = 0x0000000a,
    GRBM_SE1_PERF_SEL_PA_BUSY                          = 0x0000000c,
    GRBM_SE1_PERF_SEL_BCI_BUSY                         = 0x0000000e,
    GRBM_SE1_PERF_SEL_RMI_BUSY                         = 0x0000000f,
    GRBM_SE1_PERF_SEL_UTCL1_BUSY                       = 0x00000010,
    GRBM_SE1_PERF_SEL_TCP_BUSY                         = 0x00000011,
    GRBM_SE1_PERF_SEL_GL1CC_BUSY                       = 0x00000012,
    GRBM_SE1_PERF_SEL_RESERVED_1__GFX10                = 0x00000005,
    GRBM_SE1_PERF_SEL_RESERVED_0__GFX10                = 0x0000000b,
    GRBM_SE1_PERF_SEL_RESERVED_2__GFX10                = 0x0000000d,
    GRBM_SE1_PERF_SEL_GL1H_BUSY__GFX11                 = 0x00000013,
    GRBM_SE1_PERF_SEL_PC_BUSY__GFX11                   = 0x00000014,
} GRBM_SE1_PERF_SEL;

constexpr unsigned int MaxGrbmSe1PerfSelGfx10          = GRBM_SE1_PERF_SEL_GL1CC_BUSY;
constexpr unsigned int MaxGrbmSe1PerfSelGfx11          = GRBM_SE1_PERF_SEL_PC_BUSY__GFX11;

typedef enum GRBM_SE2_PERF_SEL {
    GRBM_SE2_PERF_SEL_COUNT                            = 0x00000000,
    GRBM_SE2_PERF_SEL_USER_DEFINED                     = 0x00000001,
    GRBM_SE2_PERF_SEL_CB_BUSY                          = 0x00000002,
    GRBM_SE2_PERF_SEL_DB_BUSY                          = 0x00000003,
    GRBM_SE2_PERF_SEL_SC_BUSY                          = 0x00000004,
    GRBM_SE2_PERF_SEL_SPI_BUSY                         = 0x00000006,
    GRBM_SE2_PERF_SEL_SX_BUSY                          = 0x00000007,
    GRBM_SE2_PERF_SEL_TA_BUSY                          = 0x00000008,
    GRBM_SE2_PERF_SEL_CB_CLEAN                         = 0x00000009,
    GRBM_SE2_PERF_SEL_DB_CLEAN                         = 0x0000000a,
    GRBM_SE2_PERF_SEL_PA_BUSY                          = 0x0000000c,
    GRBM_SE2_PERF_SEL_BCI_BUSY                         = 0x0000000e,
    GRBM_SE2_PERF_SEL_RMI_BUSY                         = 0x0000000f,
    GRBM_SE2_PERF_SEL_UTCL1_BUSY                       = 0x00000010,
    GRBM_SE2_PERF_SEL_TCP_BUSY                         = 0x00000011,
    GRBM_SE2_PERF_SEL_GL1CC_BUSY                       = 0x00000012,
    GRBM_SE2_PERF_SEL_RESERVED_1__GFX10                = 0x00000005,
    GRBM_SE2_PERF_SEL_RESERVED_0__GFX10                = 0x0000000b,
    GRBM_SE2_PERF_SEL_RESERVED_2__GFX10                = 0x0000000d,
    GRBM_SE2_PERF_SEL_GL1H_BUSY__GFX11                 = 0x00000013,
    GRBM_SE2_PERF_SEL_PC_BUSY__GFX11                   = 0x00000014,
} GRBM_SE2_PERF_SEL;

constexpr unsigned int MaxGrbmSe2PerfSelGfx10          = GRBM_SE2_PERF_SEL_GL1CC_BUSY;
constexpr unsigned int MaxGrbmSe2PerfSelGfx11          = GRBM_SE2_PERF_SEL_PC_BUSY__GFX11;

typedef enum GRBM_SE3_PERF_SEL {
    GRBM_SE3_PERF_SEL_COUNT                            = 0x00000000,
    GRBM_SE3_PERF_SEL_USER_DEFINED                     = 0x00000001,
    GRBM_SE3_PERF_SEL_CB_BUSY                          = 0x00000002,
    GRBM_SE3_PERF_SEL_DB_BUSY                          = 0x00000003,
    GRBM_SE3_PERF_SEL_SC_BUSY                          = 0x00000004,
    GRBM_SE3_PERF_SEL_SPI_BUSY                         = 0x00000006,
    GRBM_SE3_PERF_SEL_SX_BUSY                          = 0x00000007,
    GRBM_SE3_PERF_SEL_TA_BUSY                          = 0x00000008,
    GRBM_SE3_PERF_SEL_CB_CLEAN                         = 0x00000009,
    GRBM_SE3_PERF_SEL_DB_CLEAN                         = 0x0000000a,
    GRBM_SE3_PERF_SEL_PA_BUSY                          = 0x0000000c,
    GRBM_SE3_PERF_SEL_BCI_BUSY                         = 0x0000000e,
    GRBM_SE3_PERF_SEL_RMI_BUSY                         = 0x0000000f,
    GRBM_SE3_PERF_SEL_UTCL1_BUSY                       = 0x00000010,
    GRBM_SE3_PERF_SEL_TCP_BUSY                         = 0x00000011,
    GRBM_SE3_PERF_SEL_GL1CC_BUSY                       = 0x00000012,
    GRBM_SE3_PERF_SEL_RESERVED_1__GFX10                = 0x00000005,
    GRBM_SE3_PERF_SEL_RESERVED_0__GFX10                = 0x0000000b,
    GRBM_SE3_PERF_SEL_RESERVED_2__GFX10                = 0x0000000d,
    GRBM_SE3_PERF_SEL_GL1H_BUSY__GFX11                 = 0x00000013,
    GRBM_SE3_PERF_SEL_PC_BUSY__GFX11                   = 0x00000014,
} GRBM_SE3_PERF_SEL;

constexpr unsigned int MaxGrbmSe3PerfSelGfx10          = GRBM_SE3_PERF_SEL_GL1CC_BUSY;
constexpr unsigned int MaxGrbmSe3PerfSelGfx11          = GRBM_SE3_PERF_SEL_PC_BUSY__GFX11;

typedef enum GRBM_SE4_PERF_SEL {
    GRBM_SE4_PERF_SEL_COUNT                            = 0x00000000,
    GRBM_SE4_PERF_SEL_USER_DEFINED                     = 0x00000001,
    GRBM_SE4_PERF_SEL_CB_BUSY                          = 0x00000002,
    GRBM_SE4_PERF_SEL_DB_BUSY                          = 0x00000003,
    GRBM_SE4_PERF_SEL_SC_BUSY                          = 0x00000004,
    GRBM_SE4_PERF_SEL_SPI_BUSY                         = 0x00000006,
    GRBM_SE4_PERF_SEL_SX_BUSY                          = 0x00000007,
    GRBM_SE4_PERF_SEL_TA_BUSY                          = 0x00000008,
    GRBM_SE4_PERF_SEL_CB_CLEAN                         = 0x00000009,
    GRBM_SE4_PERF_SEL_DB_CLEAN                         = 0x0000000a,
    GRBM_SE4_PERF_SEL_PA_BUSY                          = 0x0000000c,
    GRBM_SE4_PERF_SEL_BCI_BUSY                         = 0x0000000e,
    GRBM_SE4_PERF_SEL_RMI_BUSY                         = 0x0000000f,
    GRBM_SE4_PERF_SEL_UTCL1_BUSY                       = 0x00000010,
    GRBM_SE4_PERF_SEL_TCP_BUSY                         = 0x00000011,
    GRBM_SE4_PERF_SEL_GL1CC_BUSY                       = 0x00000012,
    GRBM_SE4_PERF_SEL_GL1H_BUSY                        = 0x00000013,
    GRBM_SE4_PERF_SEL_PC_BUSY                          = 0x00000014,
} GRBM_SE4_PERF_SEL;

constexpr unsigned int MaxGrbmSe4PerfSel               = GRBM_SE4_PERF_SEL_PC_BUSY;

typedef enum GRBM_SE5_PERF_SEL {
    GRBM_SE5_PERF_SEL_COUNT                            = 0x00000000,
    GRBM_SE5_PERF_SEL_USER_DEFINED                     = 0x00000001,
    GRBM_SE5_PERF_SEL_CB_BUSY                          = 0x00000002,
    GRBM_SE5_PERF_SEL_DB_BUSY                          = 0x00000003,
    GRBM_SE5_PERF_SEL_SC_BUSY                          = 0x00000004,
    GRBM_SE5_PERF_SEL_SPI_BUSY                         = 0x00000006,
    GRBM_SE5_PERF_SEL_SX_BUSY                          = 0x00000007,
    GRBM_SE5_PERF_SEL_TA_BUSY                          = 0x00000008,
    GRBM_SE5_PERF_SEL_CB_CLEAN                         = 0x00000009,
    GRBM_SE5_PERF_SEL_DB_CLEAN                         = 0x0000000a,
    GRBM_SE5_PERF_SEL_PA_BUSY                          = 0x0000000c,
    GRBM_SE5_PERF_SEL_BCI_BUSY                         = 0x0000000e,
    GRBM_SE5_PERF_SEL_RMI_BUSY                         = 0x0000000f,
    GRBM_SE5_PERF_SEL_UTCL1_BUSY                       = 0x00000010,
    GRBM_SE5_PERF_SEL_TCP_BUSY                         = 0x00000011,
    GRBM_SE5_PERF_SEL_GL1CC_BUSY                       = 0x00000012,
    GRBM_SE5_PERF_SEL_GL1H_BUSY                        = 0x00000013,
    GRBM_SE5_PERF_SEL_PC_BUSY                          = 0x00000014,
} GRBM_SE5_PERF_SEL;

constexpr unsigned int MaxGrbmSe5PerfSel               = GRBM_SE5_PERF_SEL_PC_BUSY;

typedef enum GRBM_SE6_PERF_SEL {
    GRBM_SE6_PERF_SEL_COUNT                            = 0x00000000,
    GRBM_SE6_PERF_SEL_USER_DEFINED                     = 0x00000001,
    GRBM_SE6_PERF_SEL_CB_BUSY                          = 0x00000002,
    GRBM_SE6_PERF_SEL_DB_BUSY                          = 0x00000003,
    GRBM_SE6_PERF_SEL_SC_BUSY                          = 0x00000004,
    GRBM_SE6_PERF_SEL_SPI_BUSY                         = 0x00000006,
    GRBM_SE6_PERF_SEL_SX_BUSY                          = 0x00000007,
    GRBM_SE6_PERF_SEL_TA_BUSY                          = 0x00000008,
    GRBM_SE6_PERF_SEL_CB_CLEAN                         = 0x00000009,
    GRBM_SE6_PERF_SEL_DB_CLEAN                         = 0x0000000a,
    GRBM_SE6_PERF_SEL_PA_BUSY                          = 0x0000000c,
    GRBM_SE6_PERF_SEL_BCI_BUSY                         = 0x0000000e,
    GRBM_SE6_PERF_SEL_RMI_BUSY                         = 0x0000000f,
    GRBM_SE6_PERF_SEL_UTCL1_BUSY                       = 0x00000010,
    GRBM_SE6_PERF_SEL_TCP_BUSY                         = 0x00000011,
    GRBM_SE6_PERF_SEL_GL1CC_BUSY                       = 0x00000012,
    GRBM_SE6_PERF_SEL_GL1H_BUSY                        = 0x00000013,
    GRBM_SE6_PERF_SEL_PC_BUSY                          = 0x00000014,
} GRBM_SE6_PERF_SEL;

constexpr unsigned int MaxGrbmSe6PerfSel               = GRBM_SE6_PERF_SEL_PC_BUSY;

typedef enum GRBM_SE7_PERF_SEL {
    GRBM_SE7_PERF_SEL_COUNT                            = 0x00000000,
    GRBM_SE7_PERF_SEL_USER_DEFINED                     = 0x00000001,
    GRBM_SE7_PERF_SEL_CB_BUSY                          = 0x00000002,
    GRBM_SE7_PERF_SEL_DB_BUSY                          = 0x00000003,
    GRBM_SE7_PERF_SEL_SC_BUSY                          = 0x00000004,
    GRBM_SE7_PERF_SEL_SPI_BUSY                         = 0x00000006,
    GRBM_SE7_PERF_SEL_SX_BUSY                          = 0x00000007,
    GRBM_SE7_PERF_SEL_TA_BUSY                          = 0x00000008,
    GRBM_SE7_PERF_SEL_CB_CLEAN                         = 0x00000009,
    GRBM_SE7_PERF_SEL_DB_CLEAN                         = 0x0000000a,
    GRBM_SE7_PERF_SEL_PA_BUSY                          = 0x0000000c,
    GRBM_SE7_PERF_SEL_BCI_BUSY                         = 0x0000000e,
    GRBM_SE7_PERF_SEL_RMI_BUSY                         = 0x0000000f,
    GRBM_SE7_PERF_SEL_UTCL1_BUSY                       = 0x00000010,
    GRBM_SE7_PERF_SEL_TCP_BUSY                         = 0x00000011,
    GRBM_SE7_PERF_SEL_GL1CC_BUSY                       = 0x00000012,
    GRBM_SE7_PERF_SEL_GL1H_BUSY                        = 0x00000013,
    GRBM_SE7_PERF_SEL_PC_BUSY                          = 0x00000014,
} GRBM_SE7_PERF_SEL;

constexpr unsigned int MaxGrbmSe7PerfSel               = GRBM_SE7_PERF_SEL_PC_BUSY;

typedef enum IMG_DATA_FORMAT {
    IMG_DATA_FORMAT_INVALID                            = 0x00000000,
    IMG_DATA_FORMAT_8                                  = 0x00000001,
    IMG_DATA_FORMAT_16                                 = 0x00000002,
    IMG_DATA_FORMAT_8_8                                = 0x00000003,
    IMG_DATA_FORMAT_32                                 = 0x00000004,
    IMG_DATA_FORMAT_16_16                              = 0x00000005,
    IMG_DATA_FORMAT_10_11_11                           = 0x00000006,
    IMG_DATA_FORMAT_11_11_10                           = 0x00000007,
    IMG_DATA_FORMAT_10_10_10_2                         = 0x00000008,
    IMG_DATA_FORMAT_2_10_10_10                         = 0x00000009,
    IMG_DATA_FORMAT_8_8_8_8                            = 0x0000000a,
    IMG_DATA_FORMAT_32_32                              = 0x0000000b,
    IMG_DATA_FORMAT_16_16_16_16                        = 0x0000000c,
    IMG_DATA_FORMAT_32_32_32                           = 0x0000000d,
    IMG_DATA_FORMAT_32_32_32_32                        = 0x0000000e,
    IMG_DATA_FORMAT_RESERVED_15                        = 0x0000000f,
    IMG_DATA_FORMAT_5_6_5                              = 0x00000010,
    IMG_DATA_FORMAT_1_5_5_5                            = 0x00000011,
    IMG_DATA_FORMAT_5_5_5_1                            = 0x00000012,
    IMG_DATA_FORMAT_4_4_4_4                            = 0x00000013,
    IMG_DATA_FORMAT_8_24                               = 0x00000014,
    IMG_DATA_FORMAT_24_8                               = 0x00000015,
    IMG_DATA_FORMAT_X24_8_32                           = 0x00000016,
    IMG_DATA_FORMAT_RESERVED_23                        = 0x00000017,
    IMG_DATA_FORMAT_6E4                                = 0x0000001f,
    IMG_DATA_FORMAT_RESERVED_42                        = 0x0000002a,
    IMG_DATA_FORMAT_RESERVED_43                        = 0x0000002b,
    IMG_DATA_FORMAT_RESERVED_62                        = 0x0000003e,
    IMG_DATA_FORMAT_DXT3A__APU103                      = 0x00000018,
    IMG_DATA_FORMAT_DXT3A_AS_1_1_1_1__APU103           = 0x00000019,
    IMG_DATA_FORMAT_DXT5A__APU103                      = 0x0000001a,
    IMG_DATA_FORMAT_DXN__APU103                        = 0x0000001b,
    IMG_DATA_FORMAT_CTX1__APU103                       = 0x0000001c,
    IMG_DATA_FORMAT_RESERVED_86__APU103                = 0x00000056,
    IMG_DATA_FORMAT_RESERVED_87__APU103                = 0x00000057,
    IMG_DATA_FORMAT_RESERVED_88__APU103                = 0x00000058,
    IMG_DATA_FORMAT_GB_GR__GFX10                       = 0x00000020,
    IMG_DATA_FORMAT_BG_RG__GFX10                       = 0x00000021,
    IMG_DATA_FORMAT_5_9_9_9__GFX10                     = 0x00000022,
    IMG_DATA_FORMAT_BC1__GFX10                         = 0x00000023,
    IMG_DATA_FORMAT_BC2__GFX10                         = 0x00000024,
    IMG_DATA_FORMAT_BC3__GFX10                         = 0x00000025,
    IMG_DATA_FORMAT_BC4__GFX10                         = 0x00000026,
    IMG_DATA_FORMAT_BC5__GFX10                         = 0x00000027,
    IMG_DATA_FORMAT_BC6__GFX10                         = 0x00000028,
    IMG_DATA_FORMAT_BC7__GFX10                         = 0x00000029,
    IMG_DATA_FORMAT_FMASK8_S2_F1__GFX10                = 0x0000002c,
    IMG_DATA_FORMAT_FMASK8_S4_F1__GFX10                = 0x0000002d,
    IMG_DATA_FORMAT_FMASK8_S8_F1__GFX10                = 0x0000002e,
    IMG_DATA_FORMAT_FMASK8_S2_F2__GFX10                = 0x0000002f,
    IMG_DATA_FORMAT_FMASK8_S4_F2__GFX10                = 0x00000030,
    IMG_DATA_FORMAT_FMASK8_S4_F4__GFX10                = 0x00000031,
    IMG_DATA_FORMAT_FMASK16_S16_F1__GFX10              = 0x00000032,
    IMG_DATA_FORMAT_FMASK16_S8_F2__GFX10               = 0x00000033,
    IMG_DATA_FORMAT_FMASK32_S16_F2__GFX10              = 0x00000034,
    IMG_DATA_FORMAT_FMASK32_S8_F4__GFX10               = 0x00000035,
    IMG_DATA_FORMAT_FMASK32_S8_F8__GFX10               = 0x00000036,
    IMG_DATA_FORMAT_FMASK64_S16_F4__GFX10              = 0x00000037,
    IMG_DATA_FORMAT_FMASK64_S16_F8__GFX10              = 0x00000038,
    IMG_DATA_FORMAT_4_4__GFX10                         = 0x00000039,
    IMG_DATA_FORMAT_6_5_5__GFX10                       = 0x0000003a,
    IMG_DATA_FORMAT_1__GFX10                           = 0x0000003b,
    IMG_DATA_FORMAT_1_REVERSED__GFX10                  = 0x0000003c,
    IMG_DATA_FORMAT_32_AS_32_32_32_32__GFX10           = 0x0000003f,
    IMG_DATA_FORMAT_ETC2_RGB__GFX10                    = 0x00000040,
    IMG_DATA_FORMAT_ETC2_RGBA__GFX10                   = 0x00000041,
    IMG_DATA_FORMAT_ETC2_R__GFX10                      = 0x00000042,
    IMG_DATA_FORMAT_ETC2_RG__GFX10                     = 0x00000043,
    IMG_DATA_FORMAT_ETC2_RGBA1__GFX10                  = 0x00000044,
    IMG_DATA_FORMAT_ASTC_2D_LDR__GFX10                 = 0x00000045,
    IMG_DATA_FORMAT_ASTC_2D_HDR__GFX10                 = 0x00000046,
    IMG_DATA_FORMAT_ASTC_2D_LDR_SRGB__GFX10            = 0x00000047,
    IMG_DATA_FORMAT_ASTC_3D_LDR__GFX10                 = 0x00000048,
    IMG_DATA_FORMAT_ASTC_3D_HDR__GFX10                 = 0x00000049,
    IMG_DATA_FORMAT_ASTC_3D_LDR_SRGB__GFX10            = 0x0000004a,
    IMG_DATA_FORMAT_RESERVED_75__GFX10                 = 0x0000004b,
    IMG_DATA_FORMAT_MM_8__GFX10                        = 0x0000004c,
    IMG_DATA_FORMAT_MM_8_8__GFX10                      = 0x0000004d,
    IMG_DATA_FORMAT_MM_8_8_8_8__GFX10                  = 0x0000004e,
    IMG_DATA_FORMAT_MM_VYUY8__GFX10                    = 0x0000004f,
    IMG_DATA_FORMAT_MM_10_11_11__GFX10                 = 0x00000050,
    IMG_DATA_FORMAT_MM_2_10_10_10__GFX10               = 0x00000051,
    IMG_DATA_FORMAT_MM_16_16_16_16__GFX10              = 0x00000052,
    IMG_DATA_FORMAT_MM_10_IN_16__GFX10                 = 0x00000053,
    IMG_DATA_FORMAT_MM_10_IN_16_16__GFX10              = 0x00000054,
    IMG_DATA_FORMAT_MM_10_IN_16_16_16_16__GFX10        = 0x00000055,
    IMG_DATA_FORMAT_RESERVED_89__GFX10                 = 0x00000059,
    IMG_DATA_FORMAT_RESERVED_90__GFX10                 = 0x0000005a,
    IMG_DATA_FORMAT_RESERVED_91__GFX10                 = 0x0000005b,
    IMG_DATA_FORMAT_RESERVED_92__GFX10                 = 0x0000005c,
    IMG_DATA_FORMAT_RESERVED_93__GFX10                 = 0x0000005d,
    IMG_DATA_FORMAT_RESERVED_94__GFX10                 = 0x0000005e,
    IMG_DATA_FORMAT_RESERVED_95__GFX10                 = 0x0000005f,
    IMG_DATA_FORMAT_RESERVED_96__GFX10                 = 0x00000060,
    IMG_DATA_FORMAT_RESERVED_97__GFX10                 = 0x00000061,
    IMG_DATA_FORMAT_RESERVED_98__GFX10                 = 0x00000062,
    IMG_DATA_FORMAT_RESERVED_99__GFX10                 = 0x00000063,
    IMG_DATA_FORMAT_RESERVED_100__GFX10                = 0x00000064,
    IMG_DATA_FORMAT_RESERVED_101__GFX10                = 0x00000065,
    IMG_DATA_FORMAT_RESERVED_102__GFX10                = 0x00000066,
    IMG_DATA_FORMAT_RESERVED_103__GFX10                = 0x00000067,
    IMG_DATA_FORMAT_RESERVED_104__GFX10                = 0x00000068,
    IMG_DATA_FORMAT_RESERVED_105__GFX10                = 0x00000069,
    IMG_DATA_FORMAT_RESERVED_106__GFX10                = 0x0000006a,
    IMG_DATA_FORMAT_RESERVED_107__GFX10                = 0x0000006b,
    IMG_DATA_FORMAT_RESERVED_108__GFX10                = 0x0000006c,
    IMG_DATA_FORMAT_RESERVED_109__GFX10                = 0x0000006d,
    IMG_DATA_FORMAT_RESERVED_110__GFX10                = 0x0000006e,
    IMG_DATA_FORMAT_RESERVED_111__GFX10                = 0x0000006f,
    IMG_DATA_FORMAT_RESERVED_112__GFX10                = 0x00000070,
    IMG_DATA_FORMAT_RESERVED_113__GFX10                = 0x00000071,
    IMG_DATA_FORMAT_RESERVED_114__GFX10                = 0x00000072,
    IMG_DATA_FORMAT_RESERVED_115__GFX10                = 0x00000073,
    IMG_DATA_FORMAT_RESERVED_116__GFX10                = 0x00000074,
    IMG_DATA_FORMAT_RESERVED_117__GFX10                = 0x00000075,
    IMG_DATA_FORMAT_RESERVED_118__GFX10                = 0x00000076,
    IMG_DATA_FORMAT_RESERVED_119__GFX10                = 0x00000077,
    IMG_DATA_FORMAT_RESERVED_120__GFX10                = 0x00000078,
    IMG_DATA_FORMAT_RESERVED_121__GFX10                = 0x00000079,
    IMG_DATA_FORMAT_RESERVED_122__GFX10                = 0x0000007a,
    IMG_DATA_FORMAT_RESERVED_123__GFX10                = 0x0000007b,
    IMG_DATA_FORMAT_RESERVED_124__GFX10                = 0x0000007c,
    IMG_DATA_FORMAT_RESERVED_125__GFX10                = 0x0000007d,
    IMG_DATA_FORMAT_RESERVED_126__GFX10                = 0x0000007e,
    IMG_DATA_FORMAT_RESERVED_127__GFX10                = 0x0000007f,
    IMG_DATA_FORMAT_RESERVED_24__GFX101                = 0x00000018,
    IMG_DATA_FORMAT_RESERVED_25__GFX101                = 0x00000019,
    IMG_DATA_FORMAT_RESERVED_26__GFX101                = 0x0000001a,
    IMG_DATA_FORMAT_RESERVED_27__GFX101                = 0x0000001b,
    IMG_DATA_FORMAT_RESERVED_28__GFX101                = 0x0000001c,
    IMG_DATA_FORMAT_RESERVED_29__GFX101                = 0x0000001d,
    IMG_DATA_FORMAT_RESERVED_30__GFX101                = 0x0000001e,
    IMG_DATA_FORMAT_RESERVED_61__GFX101                = 0x0000003d,
    IMG_DATA_FORMAT_RESERVED_86__GFX101                = 0x00000056,
    IMG_DATA_FORMAT_RESERVED_87__GFX101                = 0x00000057,
    IMG_DATA_FORMAT_RESERVED_88__GFX101                = 0x00000058,
    IMG_DATA_FORMAT_7E3__GFX103PLUS                    = 0x0000001e,
    IMG_DATA_FORMAT_YCBCR__GFX10VRS                    = 0x0000001d,
    IMG_DATA_FORMAT_LOD_5P3_USCALED__GFX10VRS          = 0x0000003d,
    IMG_DATA_FORMAT_5_9_9_9__GFX11                     = 0x00000018,
    IMG_DATA_FORMAT_GB_GR__GFX11                       = 0x00000019,
    IMG_DATA_FORMAT_BG_RG__GFX11                       = 0x0000001a,
    IMG_DATA_FORMAT_4_4__GFX11                         = 0x0000001b,
    IMG_DATA_FORMAT_1__GFX11                           = 0x0000001c,
    IMG_DATA_FORMAT_1_REVERSED__GFX11                  = 0x0000001d,
    IMG_DATA_FORMAT_BC1__GFX11                         = 0x00000020,
    IMG_DATA_FORMAT_BC2__GFX11                         = 0x00000021,
    IMG_DATA_FORMAT_BC3__GFX11                         = 0x00000022,
    IMG_DATA_FORMAT_BC4__GFX11                         = 0x00000023,
    IMG_DATA_FORMAT_BC5__GFX11                         = 0x00000024,
    IMG_DATA_FORMAT_BC6__GFX11                         = 0x00000025,
    IMG_DATA_FORMAT_BC7__GFX11                         = 0x00000026,
    IMG_DATA_FORMAT_RESERVED_39__GFX11                 = 0x00000027,
    IMG_DATA_FORMAT_RESERVED_40__GFX11                 = 0x00000028,
    IMG_DATA_FORMAT_RESERVED_41__GFX11                 = 0x00000029,
    IMG_DATA_FORMAT_RESERVED_44__GFX11                 = 0x0000002c,
    IMG_DATA_FORMAT_RESERVED_45__GFX11                 = 0x0000002d,
    IMG_DATA_FORMAT_RESERVED_46__GFX11                 = 0x0000002e,
    IMG_DATA_FORMAT_YCBCR__GFX11                       = 0x0000002f,
    IMG_DATA_FORMAT_LOD_5P3_USCALED__GFX11             = 0x00000030,
    IMG_DATA_FORMAT_MM_8__GFX11                        = 0x00000031,
    IMG_DATA_FORMAT_MM_8_8__GFX11                      = 0x00000032,
    IMG_DATA_FORMAT_MM_8_8_8_8__GFX11                  = 0x00000033,
    IMG_DATA_FORMAT_MM_VYUY8__GFX11                    = 0x00000034,
    IMG_DATA_FORMAT_MM_10_11_11__GFX11                 = 0x00000035,
    IMG_DATA_FORMAT_MM_2_10_10_10__GFX11               = 0x00000036,
    IMG_DATA_FORMAT_MM_10_IN_16__GFX11                 = 0x00000037,
    IMG_DATA_FORMAT_MM_10_IN_16_16__GFX11              = 0x00000038,
    IMG_DATA_FORMAT_MM_10_IN_16_16_16_16__GFX11        = 0x00000039,
    IMG_DATA_FORMAT_MM_12_IN_16__GFX11                 = 0x0000003a,
    IMG_DATA_FORMAT_MM_12_IN_16_16__GFX11              = 0x0000003b,
    IMG_DATA_FORMAT_MM_12_IN_16_16_16_16__GFX11        = 0x0000003c,
    IMG_DATA_FORMAT_MM_16_16_16_16__GFX11              = 0x0000003d,
    IMG_DATA_FORMAT_RESERVED_63__GFX11                 = 0x0000003f,
    IMG_DATA_FORMAT_RESERVED_24__NV2X                  = 0x00000018,
    IMG_DATA_FORMAT_RESERVED_25__NV2X                  = 0x00000019,
    IMG_DATA_FORMAT_RESERVED_26__NV2X                  = 0x0000001a,
    IMG_DATA_FORMAT_RESERVED_27__NV2X                  = 0x0000001b,
    IMG_DATA_FORMAT_RESERVED_28__NV2X                  = 0x0000001c,
    IMG_DATA_FORMAT_MM_12_IN_16__NV2X                  = 0x00000056,
    IMG_DATA_FORMAT_MM_12_IN_16_16__NV2X               = 0x00000057,
    IMG_DATA_FORMAT_MM_12_IN_16_16_16_16__NV2X         = 0x00000058,
} IMG_DATA_FORMAT;

typedef enum IMG_FMT {
    IMG_FMT_INVALID                                    = 0x00000000,
    IMG_FMT_8_UNORM                                    = 0x00000001,
    IMG_FMT_8_SNORM                                    = 0x00000002,
    IMG_FMT_8_USCALED                                  = 0x00000003,
    IMG_FMT_8_SSCALED                                  = 0x00000004,
    IMG_FMT_8_UINT                                     = 0x00000005,
    IMG_FMT_8_SINT                                     = 0x00000006,
    IMG_FMT_16_UNORM                                   = 0x00000007,
    IMG_FMT_16_SNORM                                   = 0x00000008,
    IMG_FMT_16_USCALED                                 = 0x00000009,
    IMG_FMT_16_SSCALED                                 = 0x0000000a,
    IMG_FMT_16_UINT                                    = 0x0000000b,
    IMG_FMT_16_SINT                                    = 0x0000000c,
    IMG_FMT_16_FLOAT                                   = 0x0000000d,
    IMG_FMT_8_8_UNORM                                  = 0x0000000e,
    IMG_FMT_8_8_SNORM                                  = 0x0000000f,
    IMG_FMT_8_8_USCALED                                = 0x00000010,
    IMG_FMT_8_8_SSCALED                                = 0x00000011,
    IMG_FMT_8_8_UINT                                   = 0x00000012,
    IMG_FMT_8_8_SINT                                   = 0x00000013,
    IMG_FMT_32_UINT                                    = 0x00000014,
    IMG_FMT_32_SINT                                    = 0x00000015,
    IMG_FMT_32_FLOAT                                   = 0x00000016,
    IMG_FMT_16_16_UNORM                                = 0x00000017,
    IMG_FMT_16_16_SNORM                                = 0x00000018,
    IMG_FMT_16_16_USCALED                              = 0x00000019,
    IMG_FMT_16_16_SSCALED                              = 0x0000001a,
    IMG_FMT_16_16_UINT                                 = 0x0000001b,
    IMG_FMT_16_16_SINT                                 = 0x0000001c,
    IMG_FMT_16_16_FLOAT                                = 0x0000001d,
    IMG_FMT_RESERVED_96                                = 0x00000060,
    IMG_FMT_RESERVED_97                                = 0x00000061,
    IMG_FMT_RESERVED_98                                = 0x00000062,
    IMG_FMT_RESERVED_99                                = 0x00000063,
    IMG_FMT_RESERVED_100                               = 0x00000064,
    IMG_FMT_RESERVED_101                               = 0x00000065,
    IMG_FMT_RESERVED_102                               = 0x00000066,
    IMG_FMT_RESERVED_103                               = 0x00000067,
    IMG_FMT_RESERVED_104                               = 0x00000068,
    IMG_FMT_RESERVED_105                               = 0x00000069,
    IMG_FMT_RESERVED_106                               = 0x0000006a,
    IMG_FMT_RESERVED_107                               = 0x0000006b,
    IMG_FMT_RESERVED_108                               = 0x0000006c,
    IMG_FMT_RESERVED_123                               = 0x0000007b,
    IMG_FMT_RESERVED_124                               = 0x0000007c,
    IMG_FMT_RESERVED_125                               = 0x0000007d,
    IMG_FMT_RESERVED_126                               = 0x0000007e,
    IMG_FMT_RESERVED_127                               = 0x0000007f,
    IMG_FMT_RESERVED_155                               = 0x0000009b,
    IMG_FMT_RESERVED_288__APU103                       = 0x00000120,
    IMG_FMT_RESERVED_289__APU103                       = 0x00000121,
    IMG_FMT_RESERVED_290__APU103                       = 0x00000122,
    IMG_FMT_RESERVED_291__APU103                       = 0x00000123,
    IMG_FMT_RESERVED_292__APU103                       = 0x00000124,
    IMG_FMT_RESERVED_293__APU103                       = 0x00000125,
    IMG_FMT_10_11_11_UNORM__GFX10                      = 0x0000001e,
    IMG_FMT_10_11_11_SNORM__GFX10                      = 0x0000001f,
    IMG_FMT_10_11_11_USCALED__GFX10                    = 0x00000020,
    IMG_FMT_10_11_11_SSCALED__GFX10                    = 0x00000021,
    IMG_FMT_10_11_11_UINT__GFX10                       = 0x00000022,
    IMG_FMT_10_11_11_SINT__GFX10                       = 0x00000023,
    IMG_FMT_10_11_11_FLOAT__GFX10                      = 0x00000024,
    IMG_FMT_11_11_10_UNORM__GFX10                      = 0x00000025,
    IMG_FMT_11_11_10_SNORM__GFX10                      = 0x00000026,
    IMG_FMT_11_11_10_USCALED__GFX10                    = 0x00000027,
    IMG_FMT_11_11_10_SSCALED__GFX10                    = 0x00000028,
    IMG_FMT_11_11_10_UINT__GFX10                       = 0x00000029,
    IMG_FMT_11_11_10_SINT__GFX10                       = 0x0000002a,
    IMG_FMT_11_11_10_FLOAT__GFX10                      = 0x0000002b,
    IMG_FMT_10_10_10_2_UNORM__GFX10                    = 0x0000002c,
    IMG_FMT_10_10_10_2_SNORM__GFX10                    = 0x0000002d,
    IMG_FMT_10_10_10_2_USCALED__GFX10                  = 0x0000002e,
    IMG_FMT_10_10_10_2_SSCALED__GFX10                  = 0x0000002f,
    IMG_FMT_10_10_10_2_UINT__GFX10                     = 0x00000030,
    IMG_FMT_10_10_10_2_SINT__GFX10                     = 0x00000031,
    IMG_FMT_2_10_10_10_UNORM__GFX10                    = 0x00000032,
    IMG_FMT_2_10_10_10_SNORM__GFX10                    = 0x00000033,
    IMG_FMT_2_10_10_10_USCALED__GFX10                  = 0x00000034,
    IMG_FMT_2_10_10_10_SSCALED__GFX10                  = 0x00000035,
    IMG_FMT_2_10_10_10_UINT__GFX10                     = 0x00000036,
    IMG_FMT_2_10_10_10_SINT__GFX10                     = 0x00000037,
    IMG_FMT_8_8_8_8_UNORM__GFX10                       = 0x00000038,
    IMG_FMT_8_8_8_8_SNORM__GFX10                       = 0x00000039,
    IMG_FMT_8_8_8_8_USCALED__GFX10                     = 0x0000003a,
    IMG_FMT_8_8_8_8_SSCALED__GFX10                     = 0x0000003b,
    IMG_FMT_8_8_8_8_UINT__GFX10                        = 0x0000003c,
    IMG_FMT_8_8_8_8_SINT__GFX10                        = 0x0000003d,
    IMG_FMT_32_32_UINT__GFX10                          = 0x0000003e,
    IMG_FMT_32_32_SINT__GFX10                          = 0x0000003f,
    IMG_FMT_32_32_FLOAT__GFX10                         = 0x00000040,
    IMG_FMT_16_16_16_16_UNORM__GFX10                   = 0x00000041,
    IMG_FMT_16_16_16_16_SNORM__GFX10                   = 0x00000042,
    IMG_FMT_16_16_16_16_USCALED__GFX10                 = 0x00000043,
    IMG_FMT_16_16_16_16_SSCALED__GFX10                 = 0x00000044,
    IMG_FMT_16_16_16_16_UINT__GFX10                    = 0x00000045,
    IMG_FMT_16_16_16_16_SINT__GFX10                    = 0x00000046,
    IMG_FMT_16_16_16_16_FLOAT__GFX10                   = 0x00000047,
    IMG_FMT_32_32_32_UINT__GFX10                       = 0x00000048,
    IMG_FMT_32_32_32_SINT__GFX10                       = 0x00000049,
    IMG_FMT_32_32_32_FLOAT__GFX10                      = 0x0000004a,
    IMG_FMT_32_32_32_32_UINT__GFX10                    = 0x0000004b,
    IMG_FMT_32_32_32_32_SINT__GFX10                    = 0x0000004c,
    IMG_FMT_32_32_32_32_FLOAT__GFX10                   = 0x0000004d,
    IMG_FMT_RESERVED_78__GFX10                         = 0x0000004e,
    IMG_FMT_RESERVED_79__GFX10                         = 0x0000004f,
    IMG_FMT_RESERVED_80__GFX10                         = 0x00000050,
    IMG_FMT_RESERVED_81__GFX10                         = 0x00000051,
    IMG_FMT_RESERVED_82__GFX10                         = 0x00000052,
    IMG_FMT_RESERVED_83__GFX10                         = 0x00000053,
    IMG_FMT_RESERVED_84__GFX10                         = 0x00000054,
    IMG_FMT_RESERVED_85__GFX10                         = 0x00000055,
    IMG_FMT_RESERVED_86__GFX10                         = 0x00000056,
    IMG_FMT_RESERVED_87__GFX10                         = 0x00000057,
    IMG_FMT_RESERVED_88__GFX10                         = 0x00000058,
    IMG_FMT_RESERVED_89__GFX10                         = 0x00000059,
    IMG_FMT_RESERVED_90__GFX10                         = 0x0000005a,
    IMG_FMT_RESERVED_91__GFX10                         = 0x0000005b,
    IMG_FMT_RESERVED_92__GFX10                         = 0x0000005c,
    IMG_FMT_RESERVED_93__GFX10                         = 0x0000005d,
    IMG_FMT_RESERVED_94__GFX10                         = 0x0000005e,
    IMG_FMT_RESERVED_95__GFX10                         = 0x0000005f,
    IMG_FMT_RESERVED_109__GFX10                        = 0x0000006d,
    IMG_FMT_RESERVED_110__GFX10                        = 0x0000006e,
    IMG_FMT_RESERVED_111__GFX10                        = 0x0000006f,
    IMG_FMT_RESERVED_112__GFX10                        = 0x00000070,
    IMG_FMT_RESERVED_113__GFX10                        = 0x00000071,
    IMG_FMT_RESERVED_114__GFX10                        = 0x00000072,
    IMG_FMT_RESERVED_115__GFX10                        = 0x00000073,
    IMG_FMT_RESERVED_116__GFX10                        = 0x00000074,
    IMG_FMT_RESERVED_117__GFX10                        = 0x00000075,
    IMG_FMT_RESERVED_118__GFX10                        = 0x00000076,
    IMG_FMT_RESERVED_119__GFX10                        = 0x00000077,
    IMG_FMT_RESERVED_120__GFX10                        = 0x00000078,
    IMG_FMT_RESERVED_121__GFX10                        = 0x00000079,
    IMG_FMT_RESERVED_122__GFX10                        = 0x0000007a,
    IMG_FMT_8_SRGB__GFX10                              = 0x00000080,
    IMG_FMT_8_8_SRGB__GFX10                            = 0x00000081,
    IMG_FMT_8_8_8_8_SRGB__GFX10                        = 0x00000082,
    IMG_FMT_6E4_FLOAT__GFX10                           = 0x00000083,
    IMG_FMT_5_9_9_9_FLOAT__GFX10                       = 0x00000084,
    IMG_FMT_5_6_5_UNORM__GFX10                         = 0x00000085,
    IMG_FMT_1_5_5_5_UNORM__GFX10                       = 0x00000086,
    IMG_FMT_5_5_5_1_UNORM__GFX10                       = 0x00000087,
    IMG_FMT_4_4_4_4_UNORM__GFX10                       = 0x00000088,
    IMG_FMT_4_4_UNORM__GFX10                           = 0x00000089,
    IMG_FMT_1_UNORM__GFX10                             = 0x0000008a,
    IMG_FMT_1_REVERSED_UNORM__GFX10                    = 0x0000008b,
    IMG_FMT_32_FLOAT_CLAMP__GFX10                      = 0x0000008c,
    IMG_FMT_8_24_UNORM__GFX10                          = 0x0000008d,
    IMG_FMT_8_24_UINT__GFX10                           = 0x0000008e,
    IMG_FMT_24_8_UNORM__GFX10                          = 0x0000008f,
    IMG_FMT_24_8_UINT__GFX10                           = 0x00000090,
    IMG_FMT_X24_8_32_UINT__GFX10                       = 0x00000091,
    IMG_FMT_X24_8_32_FLOAT__GFX10                      = 0x00000092,
    IMG_FMT_GB_GR_UNORM__GFX10                         = 0x00000093,
    IMG_FMT_GB_GR_SNORM__GFX10                         = 0x00000094,
    IMG_FMT_GB_GR_UINT__GFX10                          = 0x00000095,
    IMG_FMT_GB_GR_SRGB__GFX10                          = 0x00000096,
    IMG_FMT_BG_RG_UNORM__GFX10                         = 0x00000097,
    IMG_FMT_BG_RG_SNORM__GFX10                         = 0x00000098,
    IMG_FMT_BG_RG_UINT__GFX10                          = 0x00000099,
    IMG_FMT_BG_RG_SRGB__GFX10                          = 0x0000009a,
    IMG_FMT_FMASK8_S2_F1__GFX10                        = 0x0000009c,
    IMG_FMT_FMASK8_S4_F1__GFX10                        = 0x0000009d,
    IMG_FMT_FMASK8_S8_F1__GFX10                        = 0x0000009e,
    IMG_FMT_FMASK8_S2_F2__GFX10                        = 0x0000009f,
    IMG_FMT_FMASK8_S4_F2__GFX10                        = 0x000000a0,
    IMG_FMT_FMASK8_S4_F4__GFX10                        = 0x000000a1,
    IMG_FMT_FMASK16_S16_F1__GFX10                      = 0x000000a2,
    IMG_FMT_FMASK16_S8_F2__GFX10                       = 0x000000a3,
    IMG_FMT_FMASK32_S16_F2__GFX10                      = 0x000000a4,
    IMG_FMT_FMASK32_S8_F4__GFX10                       = 0x000000a5,
    IMG_FMT_FMASK32_S8_F8__GFX10                       = 0x000000a6,
    IMG_FMT_FMASK64_S16_F4__GFX10                      = 0x000000a7,
    IMG_FMT_FMASK64_S16_F8__GFX10                      = 0x000000a8,
    IMG_FMT_BC1_UNORM__GFX10                           = 0x000000a9,
    IMG_FMT_BC1_SRGB__GFX10                            = 0x000000aa,
    IMG_FMT_BC2_UNORM__GFX10                           = 0x000000ab,
    IMG_FMT_BC2_SRGB__GFX10                            = 0x000000ac,
    IMG_FMT_BC3_UNORM__GFX10                           = 0x000000ad,
    IMG_FMT_BC3_SRGB__GFX10                            = 0x000000ae,
    IMG_FMT_BC4_UNORM__GFX10                           = 0x000000af,
    IMG_FMT_BC4_SNORM__GFX10                           = 0x000000b0,
    IMG_FMT_BC5_UNORM__GFX10                           = 0x000000b1,
    IMG_FMT_BC5_SNORM__GFX10                           = 0x000000b2,
    IMG_FMT_BC6_UFLOAT__GFX10                          = 0x000000b3,
    IMG_FMT_BC6_SFLOAT__GFX10                          = 0x000000b4,
    IMG_FMT_BC7_UNORM__GFX10                           = 0x000000b5,
    IMG_FMT_BC7_SRGB__GFX10                            = 0x000000b6,
    IMG_FMT_ETC2_RGB_UNORM__GFX10                      = 0x000000b7,
    IMG_FMT_ETC2_RGB_SRGB__GFX10                       = 0x000000b8,
    IMG_FMT_ETC2_RGBA_UNORM__GFX10                     = 0x000000b9,
    IMG_FMT_ETC2_RGBA_SRGB__GFX10                      = 0x000000ba,
    IMG_FMT_ETC2_R_UNORM__GFX10                        = 0x000000bb,
    IMG_FMT_ETC2_R_SNORM__GFX10                        = 0x000000bc,
    IMG_FMT_ETC2_RG_UNORM__GFX10                       = 0x000000bd,
    IMG_FMT_ETC2_RG_SNORM__GFX10                       = 0x000000be,
    IMG_FMT_ETC2_RGBA1_UNORM__GFX10                    = 0x000000bf,
    IMG_FMT_ETC2_RGBA1_SRGB__GFX10                     = 0x000000c0,
    IMG_FMT_ASTC_2D_LDR_4X4__GFX10                     = 0x000000c1,
    IMG_FMT_ASTC_2D_LDR_5X4__GFX10                     = 0x000000c2,
    IMG_FMT_ASTC_2D_LDR_5X5__GFX10                     = 0x000000c3,
    IMG_FMT_ASTC_2D_LDR_6X5__GFX10                     = 0x000000c4,
    IMG_FMT_ASTC_2D_LDR_6X6__GFX10                     = 0x000000c5,
    IMG_FMT_ASTC_2D_LDR_8X5__GFX10                     = 0x000000c6,
    IMG_FMT_ASTC_2D_LDR_8X6__GFX10                     = 0x000000c7,
    IMG_FMT_ASTC_2D_LDR_8X8__GFX10                     = 0x000000c8,
    IMG_FMT_ASTC_2D_LDR_10X5__GFX10                    = 0x000000c9,
    IMG_FMT_ASTC_2D_LDR_10X6__GFX10                    = 0x000000ca,
    IMG_FMT_ASTC_2D_LDR_10X8__GFX10                    = 0x000000cb,
    IMG_FMT_ASTC_2D_LDR_10X10__GFX10                   = 0x000000cc,
    IMG_FMT_ASTC_2D_LDR_12X10__GFX10                   = 0x000000cd,
    IMG_FMT_ASTC_2D_LDR_12X12__GFX10                   = 0x000000ce,
    IMG_FMT_ASTC_2D_HDR_4X4__GFX10                     = 0x000000cf,
    IMG_FMT_ASTC_2D_HDR_5X4__GFX10                     = 0x000000d0,
    IMG_FMT_ASTC_2D_HDR_5X5__GFX10                     = 0x000000d1,
    IMG_FMT_ASTC_2D_HDR_6X5__GFX10                     = 0x000000d2,
    IMG_FMT_ASTC_2D_HDR_6X6__GFX10                     = 0x000000d3,
    IMG_FMT_ASTC_2D_HDR_8X5__GFX10                     = 0x000000d4,
    IMG_FMT_ASTC_2D_HDR_8X6__GFX10                     = 0x000000d5,
    IMG_FMT_ASTC_2D_HDR_8X8__GFX10                     = 0x000000d6,
    IMG_FMT_ASTC_2D_HDR_10X5__GFX10                    = 0x000000d7,
    IMG_FMT_ASTC_2D_HDR_10X6__GFX10                    = 0x000000d8,
    IMG_FMT_ASTC_2D_HDR_10X8__GFX10                    = 0x000000d9,
    IMG_FMT_ASTC_2D_HDR_10X10__GFX10                   = 0x000000da,
    IMG_FMT_ASTC_2D_HDR_12X10__GFX10                   = 0x000000db,
    IMG_FMT_ASTC_2D_HDR_12X12__GFX10                   = 0x000000dc,
    IMG_FMT_ASTC_2D_LDR_SRGB_4X4__GFX10                = 0x000000dd,
    IMG_FMT_ASTC_2D_LDR_SRGB_5X4__GFX10                = 0x000000de,
    IMG_FMT_ASTC_2D_LDR_SRGB_5X5__GFX10                = 0x000000df,
    IMG_FMT_ASTC_2D_LDR_SRGB_6X5__GFX10                = 0x000000e0,
    IMG_FMT_ASTC_2D_LDR_SRGB_6X6__GFX10                = 0x000000e1,
    IMG_FMT_ASTC_2D_LDR_SRGB_8X5__GFX10                = 0x000000e2,
    IMG_FMT_ASTC_2D_LDR_SRGB_8X6__GFX10                = 0x000000e3,
    IMG_FMT_ASTC_2D_LDR_SRGB_8X8__GFX10                = 0x000000e4,
    IMG_FMT_ASTC_2D_LDR_SRGB_10X5__GFX10               = 0x000000e5,
    IMG_FMT_ASTC_2D_LDR_SRGB_10X6__GFX10               = 0x000000e6,
    IMG_FMT_ASTC_2D_LDR_SRGB_10X8__GFX10               = 0x000000e7,
    IMG_FMT_ASTC_2D_LDR_SRGB_10X10__GFX10              = 0x000000e8,
    IMG_FMT_ASTC_2D_LDR_SRGB_12X10__GFX10              = 0x000000e9,
    IMG_FMT_ASTC_2D_LDR_SRGB_12X12__GFX10              = 0x000000ea,
    IMG_FMT_ASTC_3D_LDR_3X3X3__GFX10                   = 0x000000eb,
    IMG_FMT_ASTC_3D_LDR_4X3X3__GFX10                   = 0x000000ec,
    IMG_FMT_ASTC_3D_LDR_4X4X3__GFX10                   = 0x000000ed,
    IMG_FMT_ASTC_3D_LDR_4X4X4__GFX10                   = 0x000000ee,
    IMG_FMT_ASTC_3D_LDR_5X4X4__GFX10                   = 0x000000ef,
    IMG_FMT_ASTC_3D_LDR_5X5X4__GFX10                   = 0x000000f0,
    IMG_FMT_ASTC_3D_LDR_5X5X5__GFX10                   = 0x000000f1,
    IMG_FMT_ASTC_3D_LDR_6X5X5__GFX10                   = 0x000000f2,
    IMG_FMT_ASTC_3D_LDR_6X6X5__GFX10                   = 0x000000f3,
    IMG_FMT_ASTC_3D_LDR_6X6X6__GFX10                   = 0x000000f4,
    IMG_FMT_ASTC_3D_HDR_3X3X3__GFX10                   = 0x000000f5,
    IMG_FMT_ASTC_3D_HDR_4X3X3__GFX10                   = 0x000000f6,
    IMG_FMT_ASTC_3D_HDR_4X4X3__GFX10                   = 0x000000f7,
    IMG_FMT_ASTC_3D_HDR_4X4X4__GFX10                   = 0x000000f8,
    IMG_FMT_ASTC_3D_HDR_5X4X4__GFX10                   = 0x000000f9,
    IMG_FMT_ASTC_3D_HDR_5X5X4__GFX10                   = 0x000000fa,
    IMG_FMT_ASTC_3D_HDR_5X5X5__GFX10                   = 0x000000fb,
    IMG_FMT_ASTC_3D_HDR_6X5X5__GFX10                   = 0x000000fc,
    IMG_FMT_ASTC_3D_HDR_6X6X5__GFX10                   = 0x000000fd,
    IMG_FMT_ASTC_3D_HDR_6X6X6__GFX10                   = 0x000000fe,
    IMG_FMT_ASTC_3D_LDR_SRGB_3X3X3__GFX10              = 0x000000ff,
    IMG_FMT_ASTC_3D_LDR_SRGB_4X3X3__GFX10              = 0x00000100,
    IMG_FMT_ASTC_3D_LDR_SRGB_4X4X3__GFX10              = 0x00000101,
    IMG_FMT_ASTC_3D_LDR_SRGB_4X4X4__GFX10              = 0x00000102,
    IMG_FMT_ASTC_3D_LDR_SRGB_5X4X4__GFX10              = 0x00000103,
    IMG_FMT_ASTC_3D_LDR_SRGB_5X5X4__GFX10              = 0x00000104,
    IMG_FMT_ASTC_3D_LDR_SRGB_5X5X5__GFX10              = 0x00000105,
    IMG_FMT_ASTC_3D_LDR_SRGB_6X5X5__GFX10              = 0x00000106,
    IMG_FMT_ASTC_3D_LDR_SRGB_6X6X5__GFX10              = 0x00000107,
    IMG_FMT_ASTC_3D_LDR_SRGB_6X6X6__GFX10              = 0x00000108,
    IMG_FMT_MM_8_UNORM__GFX10                          = 0x00000109,
    IMG_FMT_MM_8_UINT__GFX10                           = 0x0000010a,
    IMG_FMT_MM_8_8_UNORM__GFX10                        = 0x0000010b,
    IMG_FMT_MM_8_8_UINT__GFX10                         = 0x0000010c,
    IMG_FMT_MM_8_8_8_8_UNORM__GFX10                    = 0x0000010d,
    IMG_FMT_MM_8_8_8_8_UINT__GFX10                     = 0x0000010e,
    IMG_FMT_MM_VYUY8_UNORM__GFX10                      = 0x0000010f,
    IMG_FMT_MM_VYUY8_UINT__GFX10                       = 0x00000110,
    IMG_FMT_MM_10_11_11_UNORM__GFX10                   = 0x00000111,
    IMG_FMT_MM_10_11_11_UINT__GFX10                    = 0x00000112,
    IMG_FMT_MM_2_10_10_10_UNORM__GFX10                 = 0x00000113,
    IMG_FMT_MM_2_10_10_10_UINT__GFX10                  = 0x00000114,
    IMG_FMT_MM_16_16_16_16_UNORM__GFX10                = 0x00000115,
    IMG_FMT_MM_16_16_16_16_UINT__GFX10                 = 0x00000116,
    IMG_FMT_MM_10_IN_16_UNORM__GFX10                   = 0x00000117,
    IMG_FMT_MM_10_IN_16_UINT__GFX10                    = 0x00000118,
    IMG_FMT_MM_10_IN_16_16_UNORM__GFX10                = 0x00000119,
    IMG_FMT_MM_10_IN_16_16_UINT__GFX10                 = 0x0000011a,
    IMG_FMT_MM_10_IN_16_16_16_16_UNORM__GFX10          = 0x0000011b,
    IMG_FMT_MM_10_IN_16_16_16_16_UINT__GFX10           = 0x0000011c,
    IMG_FMT_RESERVED_294__GFX10                        = 0x00000126,
    IMG_FMT_RESERVED_295__GFX10                        = 0x00000127,
    IMG_FMT_RESERVED_296__GFX10                        = 0x00000128,
    IMG_FMT_RESERVED_297__GFX10                        = 0x00000129,
    IMG_FMT_RESERVED_298__GFX10                        = 0x0000012a,
    IMG_FMT_RESERVED_299__GFX10                        = 0x0000012b,
    IMG_FMT_RESERVED_300__GFX10                        = 0x0000012c,
    IMG_FMT_RESERVED_301__GFX10                        = 0x0000012d,
    IMG_FMT_RESERVED_302__GFX10                        = 0x0000012e,
    IMG_FMT_RESERVED_303__GFX10                        = 0x0000012f,
    IMG_FMT_RESERVED_304__GFX10                        = 0x00000130,
    IMG_FMT_RESERVED_305__GFX10                        = 0x00000131,
    IMG_FMT_RESERVED_306__GFX10                        = 0x00000132,
    IMG_FMT_RESERVED_307__GFX10                        = 0x00000133,
    IMG_FMT_RESERVED_308__GFX10                        = 0x00000134,
    IMG_FMT_RESERVED_309__GFX10                        = 0x00000135,
    IMG_FMT_RESERVED_310__GFX10                        = 0x00000136,
    IMG_FMT_RESERVED_311__GFX10                        = 0x00000137,
    IMG_FMT_RESERVED_312__GFX10                        = 0x00000138,
    IMG_FMT_RESERVED_313__GFX10                        = 0x00000139,
    IMG_FMT_RESERVED_314__GFX10                        = 0x0000013a,
    IMG_FMT_RESERVED_315__GFX10                        = 0x0000013b,
    IMG_FMT_RESERVED_316__GFX10                        = 0x0000013c,
    IMG_FMT_RESERVED_317__GFX10                        = 0x0000013d,
    IMG_FMT_RESERVED_318__GFX10                        = 0x0000013e,
    IMG_FMT_RESERVED_319__GFX10                        = 0x0000013f,
    IMG_FMT_RESERVED_320__GFX10                        = 0x00000140,
    IMG_FMT_RESERVED_321__GFX10                        = 0x00000141,
    IMG_FMT_RESERVED_322__GFX10                        = 0x00000142,
    IMG_FMT_RESERVED_323__GFX10                        = 0x00000143,
    IMG_FMT_RESERVED_324__GFX10                        = 0x00000144,
    IMG_FMT_RESERVED_325__GFX10                        = 0x00000145,
    IMG_FMT_RESERVED_326__GFX10                        = 0x00000146,
    IMG_FMT_RESERVED_327__GFX10                        = 0x00000147,
    IMG_FMT_RESERVED_328__GFX10                        = 0x00000148,
    IMG_FMT_RESERVED_329__GFX10                        = 0x00000149,
    IMG_FMT_RESERVED_330__GFX10                        = 0x0000014a,
    IMG_FMT_RESERVED_331__GFX10                        = 0x0000014b,
    IMG_FMT_RESERVED_332__GFX10                        = 0x0000014c,
    IMG_FMT_RESERVED_333__GFX10                        = 0x0000014d,
    IMG_FMT_RESERVED_334__GFX10                        = 0x0000014e,
    IMG_FMT_RESERVED_335__GFX10                        = 0x0000014f,
    IMG_FMT_RESERVED_336__GFX10                        = 0x00000150,
    IMG_FMT_RESERVED_337__GFX10                        = 0x00000151,
    IMG_FMT_RESERVED_338__GFX10                        = 0x00000152,
    IMG_FMT_RESERVED_339__GFX10                        = 0x00000153,
    IMG_FMT_RESERVED_340__GFX10                        = 0x00000154,
    IMG_FMT_RESERVED_341__GFX10                        = 0x00000155,
    IMG_FMT_RESERVED_342__GFX10                        = 0x00000156,
    IMG_FMT_RESERVED_343__GFX10                        = 0x00000157,
    IMG_FMT_RESERVED_344__GFX10                        = 0x00000158,
    IMG_FMT_RESERVED_345__GFX10                        = 0x00000159,
    IMG_FMT_RESERVED_346__GFX10                        = 0x0000015a,
    IMG_FMT_RESERVED_347__GFX10                        = 0x0000015b,
    IMG_FMT_RESERVED_348__GFX10                        = 0x0000015c,
    IMG_FMT_RESERVED_349__GFX10                        = 0x0000015d,
    IMG_FMT_RESERVED_350__GFX10                        = 0x0000015e,
    IMG_FMT_RESERVED_351__GFX10                        = 0x0000015f,
    IMG_FMT_RESERVED_352__GFX10                        = 0x00000160,
    IMG_FMT_RESERVED_353__GFX10                        = 0x00000161,
    IMG_FMT_RESERVED_354__GFX10                        = 0x00000162,
    IMG_FMT_RESERVED_355__GFX10                        = 0x00000163,
    IMG_FMT_RESERVED_356__GFX10                        = 0x00000164,
    IMG_FMT_RESERVED_357__GFX10                        = 0x00000165,
    IMG_FMT_RESERVED_358__GFX10                        = 0x00000166,
    IMG_FMT_RESERVED_359__GFX10                        = 0x00000167,
    IMG_FMT_RESERVED_360__GFX10                        = 0x00000168,
    IMG_FMT_RESERVED_361__GFX10                        = 0x00000169,
    IMG_FMT_RESERVED_362__GFX10                        = 0x0000016a,
    IMG_FMT_RESERVED_363__GFX10                        = 0x0000016b,
    IMG_FMT_RESERVED_364__GFX10                        = 0x0000016c,
    IMG_FMT_RESERVED_365__GFX10                        = 0x0000016d,
    IMG_FMT_RESERVED_366__GFX10                        = 0x0000016e,
    IMG_FMT_RESERVED_367__GFX10                        = 0x0000016f,
    IMG_FMT_RESERVED_368__GFX10                        = 0x00000170,
    IMG_FMT_RESERVED_369__GFX10                        = 0x00000171,
    IMG_FMT_RESERVED_370__GFX10                        = 0x00000172,
    IMG_FMT_RESERVED_371__GFX10                        = 0x00000173,
    IMG_FMT_RESERVED_372__GFX10                        = 0x00000174,
    IMG_FMT_RESERVED_373__GFX10                        = 0x00000175,
    IMG_FMT_RESERVED_374__GFX10                        = 0x00000176,
    IMG_FMT_RESERVED_375__GFX10                        = 0x00000177,
    IMG_FMT_RESERVED_376__GFX10                        = 0x00000178,
    IMG_FMT_RESERVED_377__GFX10                        = 0x00000179,
    IMG_FMT_RESERVED_378__GFX10                        = 0x0000017a,
    IMG_FMT_RESERVED_379__GFX10                        = 0x0000017b,
    IMG_FMT_RESERVED_380__GFX10                        = 0x0000017c,
    IMG_FMT_RESERVED_381__GFX10                        = 0x0000017d,
    IMG_FMT_RESERVED_382__GFX10                        = 0x0000017e,
    IMG_FMT_RESERVED_383__GFX10                        = 0x0000017f,
    IMG_FMT_RESERVED_384__GFX10                        = 0x00000180,
    IMG_FMT_RESERVED_385__GFX10                        = 0x00000181,
    IMG_FMT_RESERVED_386__GFX10                        = 0x00000182,
    IMG_FMT_RESERVED_387__GFX10                        = 0x00000183,
    IMG_FMT_RESERVED_388__GFX10                        = 0x00000184,
    IMG_FMT_RESERVED_389__GFX10                        = 0x00000185,
    IMG_FMT_RESERVED_390__GFX10                        = 0x00000186,
    IMG_FMT_RESERVED_391__GFX10                        = 0x00000187,
    IMG_FMT_RESERVED_392__GFX10                        = 0x00000188,
    IMG_FMT_RESERVED_393__GFX10                        = 0x00000189,
    IMG_FMT_RESERVED_394__GFX10                        = 0x0000018a,
    IMG_FMT_RESERVED_395__GFX10                        = 0x0000018b,
    IMG_FMT_RESERVED_396__GFX10                        = 0x0000018c,
    IMG_FMT_RESERVED_397__GFX10                        = 0x0000018d,
    IMG_FMT_RESERVED_398__GFX10                        = 0x0000018e,
    IMG_FMT_RESERVED_399__GFX10                        = 0x0000018f,
    IMG_FMT_RESERVED_400__GFX10                        = 0x00000190,
    IMG_FMT_RESERVED_401__GFX10                        = 0x00000191,
    IMG_FMT_RESERVED_402__GFX10                        = 0x00000192,
    IMG_FMT_RESERVED_403__GFX10                        = 0x00000193,
    IMG_FMT_RESERVED_404__GFX10                        = 0x00000194,
    IMG_FMT_RESERVED_405__GFX10                        = 0x00000195,
    IMG_FMT_RESERVED_406__GFX10                        = 0x00000196,
    IMG_FMT_RESERVED_407__GFX10                        = 0x00000197,
    IMG_FMT_RESERVED_408__GFX10                        = 0x00000198,
    IMG_FMT_RESERVED_409__GFX10                        = 0x00000199,
    IMG_FMT_RESERVED_410__GFX10                        = 0x0000019a,
    IMG_FMT_RESERVED_411__GFX10                        = 0x0000019b,
    IMG_FMT_RESERVED_412__GFX10                        = 0x0000019c,
    IMG_FMT_RESERVED_413__GFX10                        = 0x0000019d,
    IMG_FMT_RESERVED_414__GFX10                        = 0x0000019e,
    IMG_FMT_RESERVED_415__GFX10                        = 0x0000019f,
    IMG_FMT_RESERVED_416__GFX10                        = 0x000001a0,
    IMG_FMT_RESERVED_417__GFX10                        = 0x000001a1,
    IMG_FMT_RESERVED_418__GFX10                        = 0x000001a2,
    IMG_FMT_RESERVED_419__GFX10                        = 0x000001a3,
    IMG_FMT_RESERVED_420__GFX10                        = 0x000001a4,
    IMG_FMT_RESERVED_421__GFX10                        = 0x000001a5,
    IMG_FMT_RESERVED_422__GFX10                        = 0x000001a6,
    IMG_FMT_RESERVED_423__GFX10                        = 0x000001a7,
    IMG_FMT_RESERVED_424__GFX10                        = 0x000001a8,
    IMG_FMT_RESERVED_425__GFX10                        = 0x000001a9,
    IMG_FMT_RESERVED_426__GFX10                        = 0x000001aa,
    IMG_FMT_RESERVED_427__GFX10                        = 0x000001ab,
    IMG_FMT_RESERVED_428__GFX10                        = 0x000001ac,
    IMG_FMT_RESERVED_429__GFX10                        = 0x000001ad,
    IMG_FMT_RESERVED_430__GFX10                        = 0x000001ae,
    IMG_FMT_RESERVED_431__GFX10                        = 0x000001af,
    IMG_FMT_RESERVED_432__GFX10                        = 0x000001b0,
    IMG_FMT_RESERVED_433__GFX10                        = 0x000001b1,
    IMG_FMT_RESERVED_434__GFX10                        = 0x000001b2,
    IMG_FMT_RESERVED_435__GFX10                        = 0x000001b3,
    IMG_FMT_RESERVED_436__GFX10                        = 0x000001b4,
    IMG_FMT_RESERVED_437__GFX10                        = 0x000001b5,
    IMG_FMT_RESERVED_438__GFX10                        = 0x000001b6,
    IMG_FMT_RESERVED_439__GFX10                        = 0x000001b7,
    IMG_FMT_RESERVED_440__GFX10                        = 0x000001b8,
    IMG_FMT_RESERVED_441__GFX10                        = 0x000001b9,
    IMG_FMT_RESERVED_442__GFX10                        = 0x000001ba,
    IMG_FMT_RESERVED_443__GFX10                        = 0x000001bb,
    IMG_FMT_RESERVED_444__GFX10                        = 0x000001bc,
    IMG_FMT_RESERVED_445__GFX10                        = 0x000001bd,
    IMG_FMT_RESERVED_446__GFX10                        = 0x000001be,
    IMG_FMT_RESERVED_447__GFX10                        = 0x000001bf,
    IMG_FMT_RESERVED_448__GFX10                        = 0x000001c0,
    IMG_FMT_RESERVED_449__GFX10                        = 0x000001c1,
    IMG_FMT_RESERVED_450__GFX10                        = 0x000001c2,
    IMG_FMT_RESERVED_451__GFX10                        = 0x000001c3,
    IMG_FMT_RESERVED_452__GFX10                        = 0x000001c4,
    IMG_FMT_RESERVED_453__GFX10                        = 0x000001c5,
    IMG_FMT_RESERVED_454__GFX10                        = 0x000001c6,
    IMG_FMT_RESERVED_455__GFX10                        = 0x000001c7,
    IMG_FMT_RESERVED_456__GFX10                        = 0x000001c8,
    IMG_FMT_RESERVED_457__GFX10                        = 0x000001c9,
    IMG_FMT_RESERVED_458__GFX10                        = 0x000001ca,
    IMG_FMT_RESERVED_459__GFX10                        = 0x000001cb,
    IMG_FMT_RESERVED_460__GFX10                        = 0x000001cc,
    IMG_FMT_RESERVED_461__GFX10                        = 0x000001cd,
    IMG_FMT_RESERVED_462__GFX10                        = 0x000001ce,
    IMG_FMT_RESERVED_463__GFX10                        = 0x000001cf,
    IMG_FMT_RESERVED_464__GFX10                        = 0x000001d0,
    IMG_FMT_RESERVED_465__GFX10                        = 0x000001d1,
    IMG_FMT_RESERVED_466__GFX10                        = 0x000001d2,
    IMG_FMT_RESERVED_467__GFX10                        = 0x000001d3,
    IMG_FMT_RESERVED_468__GFX10                        = 0x000001d4,
    IMG_FMT_RESERVED_469__GFX10                        = 0x000001d5,
    IMG_FMT_RESERVED_470__GFX10                        = 0x000001d6,
    IMG_FMT_RESERVED_471__GFX10                        = 0x000001d7,
    IMG_FMT_RESERVED_472__GFX10                        = 0x000001d8,
    IMG_FMT_RESERVED_473__GFX10                        = 0x000001d9,
    IMG_FMT_RESERVED_474__GFX10                        = 0x000001da,
    IMG_FMT_RESERVED_475__GFX10                        = 0x000001db,
    IMG_FMT_RESERVED_476__GFX10                        = 0x000001dc,
    IMG_FMT_RESERVED_477__GFX10                        = 0x000001dd,
    IMG_FMT_RESERVED_478__GFX10                        = 0x000001de,
    IMG_FMT_RESERVED_479__GFX10                        = 0x000001df,
    IMG_FMT_RESERVED_480__GFX10                        = 0x000001e0,
    IMG_FMT_RESERVED_481__GFX10                        = 0x000001e1,
    IMG_FMT_RESERVED_482__GFX10                        = 0x000001e2,
    IMG_FMT_RESERVED_483__GFX10                        = 0x000001e3,
    IMG_FMT_RESERVED_484__GFX10                        = 0x000001e4,
    IMG_FMT_RESERVED_485__GFX10                        = 0x000001e5,
    IMG_FMT_RESERVED_486__GFX10                        = 0x000001e6,
    IMG_FMT_RESERVED_487__GFX10                        = 0x000001e7,
    IMG_FMT_RESERVED_488__GFX10                        = 0x000001e8,
    IMG_FMT_RESERVED_489__GFX10                        = 0x000001e9,
    IMG_FMT_RESERVED_490__GFX10                        = 0x000001ea,
    IMG_FMT_RESERVED_491__GFX10                        = 0x000001eb,
    IMG_FMT_RESERVED_492__GFX10                        = 0x000001ec,
    IMG_FMT_RESERVED_493__GFX10                        = 0x000001ed,
    IMG_FMT_RESERVED_494__GFX10                        = 0x000001ee,
    IMG_FMT_RESERVED_495__GFX10                        = 0x000001ef,
    IMG_FMT_RESERVED_496__GFX10                        = 0x000001f0,
    IMG_FMT_RESERVED_497__GFX10                        = 0x000001f1,
    IMG_FMT_RESERVED_498__GFX10                        = 0x000001f2,
    IMG_FMT_RESERVED_499__GFX10                        = 0x000001f3,
    IMG_FMT_RESERVED_500__GFX10                        = 0x000001f4,
    IMG_FMT_RESERVED_501__GFX10                        = 0x000001f5,
    IMG_FMT_RESERVED_502__GFX10                        = 0x000001f6,
    IMG_FMT_RESERVED_503__GFX10                        = 0x000001f7,
    IMG_FMT_RESERVED_504__GFX10                        = 0x000001f8,
    IMG_FMT_RESERVED_505__GFX10                        = 0x000001f9,
    IMG_FMT_RESERVED_506__GFX10                        = 0x000001fa,
    IMG_FMT_RESERVED_507__GFX10                        = 0x000001fb,
    IMG_FMT_RESERVED_508__GFX10                        = 0x000001fc,
    IMG_FMT_RESERVED_509__GFX10                        = 0x000001fd,
    IMG_FMT_RESERVED_510__GFX10                        = 0x000001fe,
    IMG_FMT_RESERVED_511__GFX10                        = 0x000001ff,
    IMG_FMT_RESERVED_285__GFX101                       = 0x0000011d,
    IMG_FMT_RESERVED_286__GFX101                       = 0x0000011e,
    IMG_FMT_RESERVED_287__GFX101                       = 0x0000011f,
    IMG_FMT_RESERVED_288__GFX101                       = 0x00000120,
    IMG_FMT_RESERVED_289__GFX101                       = 0x00000121,
    IMG_FMT_RESERVED_290__GFX101                       = 0x00000122,
    IMG_FMT_RESERVED_291__GFX101                       = 0x00000123,
    IMG_FMT_RESERVED_292__GFX101                       = 0x00000124,
    IMG_FMT_RESERVED_293__GFX101                       = 0x00000125,
    IMG_FMT_7E3_FLOAT__GFX10VRS                        = 0x0000011d,
    IMG_FMT_YCBCR_UNORM__GFX10VRS                      = 0x0000011e,
    IMG_FMT_YCBCR_SRGB__GFX10VRS                       = 0x0000011f,
    IMG_FMT_10_11_11_FLOAT__GFX11                      = 0x0000001e,
    IMG_FMT_11_11_10_FLOAT__GFX11                      = 0x0000001f,
    IMG_FMT_10_10_10_2_UNORM__GFX11                    = 0x00000020,
    IMG_FMT_10_10_10_2_SNORM__GFX11                    = 0x00000021,
    IMG_FMT_10_10_10_2_UINT__GFX11                     = 0x00000022,
    IMG_FMT_10_10_10_2_SINT__GFX11                     = 0x00000023,
    IMG_FMT_2_10_10_10_UNORM__GFX11                    = 0x00000024,
    IMG_FMT_2_10_10_10_SNORM__GFX11                    = 0x00000025,
    IMG_FMT_2_10_10_10_USCALED__GFX11                  = 0x00000026,
    IMG_FMT_2_10_10_10_SSCALED__GFX11                  = 0x00000027,
    IMG_FMT_2_10_10_10_UINT__GFX11                     = 0x00000028,
    IMG_FMT_2_10_10_10_SINT__GFX11                     = 0x00000029,
    IMG_FMT_8_8_8_8_UNORM__GFX11                       = 0x0000002a,
    IMG_FMT_8_8_8_8_SNORM__GFX11                       = 0x0000002b,
    IMG_FMT_8_8_8_8_USCALED__GFX11                     = 0x0000002c,
    IMG_FMT_8_8_8_8_SSCALED__GFX11                     = 0x0000002d,
    IMG_FMT_8_8_8_8_UINT__GFX11                        = 0x0000002e,
    IMG_FMT_8_8_8_8_SINT__GFX11                        = 0x0000002f,
    IMG_FMT_32_32_UINT__GFX11                          = 0x00000030,
    IMG_FMT_32_32_SINT__GFX11                          = 0x00000031,
    IMG_FMT_32_32_FLOAT__GFX11                         = 0x00000032,
    IMG_FMT_16_16_16_16_UNORM__GFX11                   = 0x00000033,
    IMG_FMT_16_16_16_16_SNORM__GFX11                   = 0x00000034,
    IMG_FMT_16_16_16_16_USCALED__GFX11                 = 0x00000035,
    IMG_FMT_16_16_16_16_SSCALED__GFX11                 = 0x00000036,
    IMG_FMT_16_16_16_16_UINT__GFX11                    = 0x00000037,
    IMG_FMT_16_16_16_16_SINT__GFX11                    = 0x00000038,
    IMG_FMT_16_16_16_16_FLOAT__GFX11                   = 0x00000039,
    IMG_FMT_32_32_32_UINT__GFX11                       = 0x0000003a,
    IMG_FMT_32_32_32_SINT__GFX11                       = 0x0000003b,
    IMG_FMT_32_32_32_FLOAT__GFX11                      = 0x0000003c,
    IMG_FMT_32_32_32_32_UINT__GFX11                    = 0x0000003d,
    IMG_FMT_32_32_32_32_SINT__GFX11                    = 0x0000003e,
    IMG_FMT_32_32_32_32_FLOAT__GFX11                   = 0x0000003f,
    IMG_FMT_8_SRGB__GFX11                              = 0x00000040,
    IMG_FMT_8_8_SRGB__GFX11                            = 0x00000041,
    IMG_FMT_8_8_8_8_SRGB__GFX11                        = 0x00000042,
    IMG_FMT_5_9_9_9_FLOAT__GFX11                       = 0x00000043,
    IMG_FMT_5_6_5_UNORM__GFX11                         = 0x00000044,
    IMG_FMT_1_5_5_5_UNORM__GFX11                       = 0x00000045,
    IMG_FMT_5_5_5_1_UNORM__GFX11                       = 0x00000046,
    IMG_FMT_4_4_4_4_UNORM__GFX11                       = 0x00000047,
    IMG_FMT_4_4_UNORM__GFX11                           = 0x00000048,
    IMG_FMT_1_UNORM__GFX11                             = 0x00000049,
    IMG_FMT_1_REVERSED_UNORM__GFX11                    = 0x0000004a,
    IMG_FMT_32_FLOAT_CLAMP__GFX11                      = 0x0000004b,
    IMG_FMT_8_24_UNORM__GFX11                          = 0x0000004c,
    IMG_FMT_8_24_UINT__GFX11                           = 0x0000004d,
    IMG_FMT_24_8_UNORM__GFX11                          = 0x0000004e,
    IMG_FMT_24_8_UINT__GFX11                           = 0x0000004f,
    IMG_FMT_X24_8_32_UINT__GFX11                       = 0x00000050,
    IMG_FMT_X24_8_32_FLOAT__GFX11                      = 0x00000051,
    IMG_FMT_GB_GR_UNORM__GFX11                         = 0x00000052,
    IMG_FMT_GB_GR_SNORM__GFX11                         = 0x00000053,
    IMG_FMT_GB_GR_UINT__GFX11                          = 0x00000054,
    IMG_FMT_GB_GR_SRGB__GFX11                          = 0x00000055,
    IMG_FMT_BG_RG_UNORM__GFX11                         = 0x00000056,
    IMG_FMT_BG_RG_SNORM__GFX11                         = 0x00000057,
    IMG_FMT_BG_RG_UINT__GFX11                          = 0x00000058,
    IMG_FMT_BG_RG_SRGB__GFX11                          = 0x00000059,
    IMG_FMT_MM_10_IN_16_UNORM__GFX11                   = 0x0000005a,
    IMG_FMT_MM_10_IN_16_UINT__GFX11                    = 0x0000005b,
    IMG_FMT_MM_10_IN_16_16_UNORM__GFX11                = 0x0000005c,
    IMG_FMT_MM_10_IN_16_16_UINT__GFX11                 = 0x0000005d,
    IMG_FMT_MM_10_IN_16_16_16_16_UNORM__GFX11          = 0x0000005e,
    IMG_FMT_MM_10_IN_16_16_16_16_UINT__GFX11           = 0x0000005f,
    IMG_FMT_BC1_UNORM__GFX11                           = 0x0000006d,
    IMG_FMT_BC1_SRGB__GFX11                            = 0x0000006e,
    IMG_FMT_BC2_UNORM__GFX11                           = 0x0000006f,
    IMG_FMT_BC2_SRGB__GFX11                            = 0x00000070,
    IMG_FMT_BC3_UNORM__GFX11                           = 0x00000071,
    IMG_FMT_BC3_SRGB__GFX11                            = 0x00000072,
    IMG_FMT_BC4_UNORM__GFX11                           = 0x00000073,
    IMG_FMT_BC4_SNORM__GFX11                           = 0x00000074,
    IMG_FMT_BC5_UNORM__GFX11                           = 0x00000075,
    IMG_FMT_BC5_SNORM__GFX11                           = 0x00000076,
    IMG_FMT_BC6_UFLOAT__GFX11                          = 0x00000077,
    IMG_FMT_BC6_SFLOAT__GFX11                          = 0x00000078,
    IMG_FMT_BC7_UNORM__GFX11                           = 0x00000079,
    IMG_FMT_BC7_SRGB__GFX11                            = 0x0000007a,
    IMG_FMT_RESERVED_128__GFX11                        = 0x00000080,
    IMG_FMT_RESERVED_129__GFX11                        = 0x00000081,
    IMG_FMT_RESERVED_130__GFX11                        = 0x00000082,
    IMG_FMT_RESERVED_131__GFX11                        = 0x00000083,
    IMG_FMT_RESERVED_132__GFX11                        = 0x00000084,
    IMG_FMT_RESERVED_133__GFX11                        = 0x00000085,
    IMG_FMT_RESERVED_134__GFX11                        = 0x00000086,
    IMG_FMT_RESERVED_135__GFX11                        = 0x00000087,
    IMG_FMT_RESERVED_136__GFX11                        = 0x00000088,
    IMG_FMT_RESERVED_137__GFX11                        = 0x00000089,
    IMG_FMT_RESERVED_138__GFX11                        = 0x0000008a,
    IMG_FMT_RESERVED_139__GFX11                        = 0x0000008b,
    IMG_FMT_RESERVED_140__GFX11                        = 0x0000008c,
    IMG_FMT_RESERVED_141__GFX11                        = 0x0000008d,
    IMG_FMT_RESERVED_142__GFX11                        = 0x0000008e,
    IMG_FMT_RESERVED_143__GFX11                        = 0x0000008f,
    IMG_FMT_RESERVED_144__GFX11                        = 0x00000090,
    IMG_FMT_RESERVED_145__GFX11                        = 0x00000091,
    IMG_FMT_RESERVED_146__GFX11                        = 0x00000092,
    IMG_FMT_RESERVED_147__GFX11                        = 0x00000093,
    IMG_FMT_RESERVED_148__GFX11                        = 0x00000094,
    IMG_FMT_RESERVED_149__GFX11                        = 0x00000095,
    IMG_FMT_RESERVED_150__GFX11                        = 0x00000096,
    IMG_FMT_RESERVED_151__GFX11                        = 0x00000097,
    IMG_FMT_RESERVED_152__GFX11                        = 0x00000098,
    IMG_FMT_RESERVED_153__GFX11                        = 0x00000099,
    IMG_FMT_RESERVED_154__GFX11                        = 0x0000009a,
    IMG_FMT_RESERVED_156__GFX11                        = 0x0000009c,
    IMG_FMT_RESERVED_157__GFX11                        = 0x0000009d,
    IMG_FMT_RESERVED_158__GFX11                        = 0x0000009e,
    IMG_FMT_RESERVED_159__GFX11                        = 0x0000009f,
    IMG_FMT_RESERVED_160__GFX11                        = 0x000000a0,
    IMG_FMT_RESERVED_161__GFX11                        = 0x000000a1,
    IMG_FMT_RESERVED_162__GFX11                        = 0x000000a2,
    IMG_FMT_RESERVED_163__GFX11                        = 0x000000a3,
    IMG_FMT_RESERVED_164__GFX11                        = 0x000000a4,
    IMG_FMT_RESERVED_165__GFX11                        = 0x000000a5,
    IMG_FMT_RESERVED_166__GFX11                        = 0x000000a6,
    IMG_FMT_RESERVED_167__GFX11                        = 0x000000a7,
    IMG_FMT_RESERVED_168__GFX11                        = 0x000000a8,
    IMG_FMT_RESERVED_169__GFX11                        = 0x000000a9,
    IMG_FMT_RESERVED_170__GFX11                        = 0x000000aa,
    IMG_FMT_RESERVED_171__GFX11                        = 0x000000ab,
    IMG_FMT_RESERVED_172__GFX11                        = 0x000000ac,
    IMG_FMT_RESERVED_173__GFX11                        = 0x000000ad,
    IMG_FMT_RESERVED_174__GFX11                        = 0x000000ae,
    IMG_FMT_RESERVED_175__GFX11                        = 0x000000af,
    IMG_FMT_RESERVED_176__GFX11                        = 0x000000b0,
    IMG_FMT_RESERVED_177__GFX11                        = 0x000000b1,
    IMG_FMT_RESERVED_178__GFX11                        = 0x000000b2,
    IMG_FMT_RESERVED_179__GFX11                        = 0x000000b3,
    IMG_FMT_RESERVED_180__GFX11                        = 0x000000b4,
    IMG_FMT_RESERVED_181__GFX11                        = 0x000000b5,
    IMG_FMT_RESERVED_182__GFX11                        = 0x000000b6,
    IMG_FMT_RESERVED_183__GFX11                        = 0x000000b7,
    IMG_FMT_RESERVED_184__GFX11                        = 0x000000b8,
    IMG_FMT_RESERVED_185__GFX11                        = 0x000000b9,
    IMG_FMT_RESERVED_186__GFX11                        = 0x000000ba,
    IMG_FMT_RESERVED_187__GFX11                        = 0x000000bb,
    IMG_FMT_RESERVED_188__GFX11                        = 0x000000bc,
    IMG_FMT_RESERVED_189__GFX11                        = 0x000000bd,
    IMG_FMT_RESERVED_190__GFX11                        = 0x000000be,
    IMG_FMT_RESERVED_191__GFX11                        = 0x000000bf,
    IMG_FMT_RESERVED_192__GFX11                        = 0x000000c0,
    IMG_FMT_RESERVED_193__GFX11                        = 0x000000c1,
    IMG_FMT_RESERVED_194__GFX11                        = 0x000000c2,
    IMG_FMT_RESERVED_195__GFX11                        = 0x000000c3,
    IMG_FMT_RESERVED_196__GFX11                        = 0x000000c4,
    IMG_FMT_RESERVED_197__GFX11                        = 0x000000c5,
    IMG_FMT_RESERVED_198__GFX11                        = 0x000000c6,
    IMG_FMT_RESERVED_199__GFX11                        = 0x000000c7,
    IMG_FMT_RESERVED_200__GFX11                        = 0x000000c8,
    IMG_FMT_RESERVED_201__GFX11                        = 0x000000c9,
    IMG_FMT_RESERVED_202__GFX11                        = 0x000000ca,
    IMG_FMT_RESERVED_203__GFX11                        = 0x000000cb,
    IMG_FMT_RESERVED_204__GFX11                        = 0x000000cc,
    IMG_FMT_YCBCR_UNORM__GFX11                         = 0x000000cd,
    IMG_FMT_YCBCR_SRGB__GFX11                          = 0x000000ce,
    IMG_FMT_MM_8_UNORM__GFX11                          = 0x000000cf,
    IMG_FMT_MM_8_UINT__GFX11                           = 0x000000d0,
    IMG_FMT_MM_8_8_UNORM__GFX11                        = 0x000000d1,
    IMG_FMT_MM_8_8_UINT__GFX11                         = 0x000000d2,
    IMG_FMT_MM_8_8_8_8_UNORM__GFX11                    = 0x000000d3,
    IMG_FMT_MM_8_8_8_8_UINT__GFX11                     = 0x000000d4,
    IMG_FMT_MM_VYUY8_UNORM__GFX11                      = 0x000000d5,
    IMG_FMT_MM_VYUY8_UINT__GFX11                       = 0x000000d6,
    IMG_FMT_MM_10_11_11_UNORM__GFX11                   = 0x000000d7,
    IMG_FMT_MM_10_11_11_UINT__GFX11                    = 0x000000d8,
    IMG_FMT_MM_2_10_10_10_UNORM__GFX11                 = 0x000000d9,
    IMG_FMT_MM_2_10_10_10_UINT__GFX11                  = 0x000000da,
    IMG_FMT_MM_16_16_16_16_UNORM__GFX11                = 0x000000db,
    IMG_FMT_MM_16_16_16_16_UINT__GFX11                 = 0x000000dc,
    IMG_FMT_MM_12_IN_16_UNORM__GFX11                   = 0x000000dd,
    IMG_FMT_MM_12_IN_16_UINT__GFX11                    = 0x000000de,
    IMG_FMT_MM_12_IN_16_16_UNORM__GFX11                = 0x000000df,
    IMG_FMT_MM_12_IN_16_16_UINT__GFX11                 = 0x000000e0,
    IMG_FMT_MM_12_IN_16_16_16_16_UNORM__GFX11          = 0x000000e1,
    IMG_FMT_MM_12_IN_16_16_16_16_UINT__GFX11           = 0x000000e2,
    IMG_FMT_6E4_FLOAT__GFX11                           = 0x000000e3,
    IMG_FMT_7E3_FLOAT__GFX11                           = 0x000000e4,
    IMG_FMT_RESERVED_229__GFX11                        = 0x000000e5,
    IMG_FMT_RESERVED_230__GFX11                        = 0x000000e6,
    IMG_FMT_RESERVED_231__GFX11                        = 0x000000e7,
    IMG_FMT_RESERVED_232__GFX11                        = 0x000000e8,
    IMG_FMT_RESERVED_233__GFX11                        = 0x000000e9,
    IMG_FMT_RESERVED_234__GFX11                        = 0x000000ea,
    IMG_FMT_RESERVED_235__GFX11                        = 0x000000eb,
    IMG_FMT_RESERVED_236__GFX11                        = 0x000000ec,
    IMG_FMT_RESERVED_237__GFX11                        = 0x000000ed,
    IMG_FMT_RESERVED_238__GFX11                        = 0x000000ee,
    IMG_FMT_RESERVED_239__GFX11                        = 0x000000ef,
    IMG_FMT_RESERVED_240__GFX11                        = 0x000000f0,
    IMG_FMT_RESERVED_241__GFX11                        = 0x000000f1,
    IMG_FMT_RESERVED_242__GFX11                        = 0x000000f2,
    IMG_FMT_RESERVED_243__GFX11                        = 0x000000f3,
    IMG_FMT_RESERVED_244__GFX11                        = 0x000000f4,
    IMG_FMT_RESERVED_245__GFX11                        = 0x000000f5,
    IMG_FMT_RESERVED_246__GFX11                        = 0x000000f6,
    IMG_FMT_RESERVED_247__GFX11                        = 0x000000f7,
    IMG_FMT_RESERVED_248__GFX11                        = 0x000000f8,
    IMG_FMT_RESERVED_249__GFX11                        = 0x000000f9,
    IMG_FMT_RESERVED_250__GFX11                        = 0x000000fa,
    IMG_FMT_RESERVED_251__GFX11                        = 0x000000fb,
    IMG_FMT_RESERVED_252__GFX11                        = 0x000000fc,
    IMG_FMT_RESERVED_253__GFX11                        = 0x000000fd,
    IMG_FMT_RESERVED_254__GFX11                        = 0x000000fe,
    IMG_FMT_RESERVED_255__GFX11                        = 0x000000ff,
    IMG_FMT_MM_12_IN_16_UNORM__NV2X                    = 0x00000120,
    IMG_FMT_MM_12_IN_16_UINT__NV2X                     = 0x00000121,
    IMG_FMT_MM_12_IN_16_16_UNORM__NV2X                 = 0x00000122,
    IMG_FMT_MM_12_IN_16_16_UINT__NV2X                  = 0x00000123,
    IMG_FMT_MM_12_IN_16_16_16_16_UNORM__NV2X           = 0x00000124,
    IMG_FMT_MM_12_IN_16_16_16_16_UINT__NV2X            = 0x00000125,
} IMG_FMT;

typedef enum IMG_NUM_FORMAT {
    IMG_NUM_FORMAT_UNORM                               = 0x00000000,
    IMG_NUM_FORMAT_SNORM                               = 0x00000001,
    IMG_NUM_FORMAT_USCALED                             = 0x00000002,
    IMG_NUM_FORMAT_SSCALED                             = 0x00000003,
    IMG_NUM_FORMAT_UINT                                = 0x00000004,
    IMG_NUM_FORMAT_SINT                                = 0x00000005,
    IMG_NUM_FORMAT_FLOAT                               = 0x00000007,
    IMG_NUM_FORMAT_SNORM_NZ__GFX10                     = 0x00000006,
    IMG_NUM_FORMAT_RESERVED_8__GFX10                   = 0x00000008,
    IMG_NUM_FORMAT_SRGB__GFX10                         = 0x00000009,
    IMG_NUM_FORMAT_UBNORM__GFX10                       = 0x0000000a,
    IMG_NUM_FORMAT_UBNORM_NZ__GFX10                    = 0x0000000b,
    IMG_NUM_FORMAT_UBINT__GFX10                        = 0x0000000c,
    IMG_NUM_FORMAT_UBSCALED__GFX10                     = 0x0000000d,
    IMG_NUM_FORMAT_RESERVED_14__GFX10                  = 0x0000000e,
    IMG_NUM_FORMAT_RESERVED_15__GFX10                  = 0x0000000f,
    IMG_NUM_FORMAT_SRGB__GFX11                         = 0x00000006,
} IMG_NUM_FORMAT;

typedef enum IMG_NUM_FORMAT_ASTC_2D {
    IMG_NUM_FORMAT_ASTC_2D_4X4                         = 0x00000000,
    IMG_NUM_FORMAT_ASTC_2D_5X4                         = 0x00000001,
    IMG_NUM_FORMAT_ASTC_2D_5X5                         = 0x00000002,
    IMG_NUM_FORMAT_ASTC_2D_6X5                         = 0x00000003,
    IMG_NUM_FORMAT_ASTC_2D_6X6                         = 0x00000004,
    IMG_NUM_FORMAT_ASTC_2D_8X5                         = 0x00000005,
    IMG_NUM_FORMAT_ASTC_2D_8X6                         = 0x00000006,
    IMG_NUM_FORMAT_ASTC_2D_8X8                         = 0x00000007,
    IMG_NUM_FORMAT_ASTC_2D_10X5                        = 0x00000008,
    IMG_NUM_FORMAT_ASTC_2D_10X6                        = 0x00000009,
    IMG_NUM_FORMAT_ASTC_2D_10X8                        = 0x0000000a,
    IMG_NUM_FORMAT_ASTC_2D_10X10                       = 0x0000000b,
    IMG_NUM_FORMAT_ASTC_2D_12X10                       = 0x0000000c,
    IMG_NUM_FORMAT_ASTC_2D_12X12                       = 0x0000000d,
    IMG_NUM_FORMAT_ASTC_2D_RESERVED_14                 = 0x0000000e,
    IMG_NUM_FORMAT_ASTC_2D_RESERVED_15                 = 0x0000000f,
} IMG_NUM_FORMAT_ASTC_2D;

typedef enum IMG_NUM_FORMAT_ASTC_3D {
    IMG_NUM_FORMAT_ASTC_3D_3X3X3                       = 0x00000000,
    IMG_NUM_FORMAT_ASTC_3D_4X3X3                       = 0x00000001,
    IMG_NUM_FORMAT_ASTC_3D_4X4X3                       = 0x00000002,
    IMG_NUM_FORMAT_ASTC_3D_4X4X4                       = 0x00000003,
    IMG_NUM_FORMAT_ASTC_3D_5X4X4                       = 0x00000004,
    IMG_NUM_FORMAT_ASTC_3D_5X5X4                       = 0x00000005,
    IMG_NUM_FORMAT_ASTC_3D_5X5X5                       = 0x00000006,
    IMG_NUM_FORMAT_ASTC_3D_6X5X5                       = 0x00000007,
    IMG_NUM_FORMAT_ASTC_3D_6X6X5                       = 0x00000008,
    IMG_NUM_FORMAT_ASTC_3D_6X6X6                       = 0x00000009,
    IMG_NUM_FORMAT_ASTC_3D_RESERVED_10                 = 0x0000000a,
    IMG_NUM_FORMAT_ASTC_3D_RESERVED_11                 = 0x0000000b,
    IMG_NUM_FORMAT_ASTC_3D_RESERVED_12                 = 0x0000000c,
    IMG_NUM_FORMAT_ASTC_3D_RESERVED_13                 = 0x0000000d,
    IMG_NUM_FORMAT_ASTC_3D_RESERVED_14                 = 0x0000000e,
    IMG_NUM_FORMAT_ASTC_3D_RESERVED_15                 = 0x0000000f,
} IMG_NUM_FORMAT_ASTC_3D;

typedef enum IMG_NUM_FORMAT_FMASK {
    IMG_NUM_FORMAT_FMASK_8_2_1                         = 0x00000000,
    IMG_NUM_FORMAT_FMASK_8_4_1                         = 0x00000001,
    IMG_NUM_FORMAT_FMASK_8_8_1                         = 0x00000002,
    IMG_NUM_FORMAT_FMASK_8_2_2                         = 0x00000003,
    IMG_NUM_FORMAT_FMASK_8_4_2                         = 0x00000004,
    IMG_NUM_FORMAT_FMASK_8_4_4                         = 0x00000005,
    IMG_NUM_FORMAT_FMASK_16_16_1                       = 0x00000006,
    IMG_NUM_FORMAT_FMASK_16_8_2                        = 0x00000007,
    IMG_NUM_FORMAT_FMASK_32_16_2                       = 0x00000008,
    IMG_NUM_FORMAT_FMASK_32_8_4                        = 0x00000009,
    IMG_NUM_FORMAT_FMASK_32_8_8                        = 0x0000000a,
    IMG_NUM_FORMAT_FMASK_64_16_4                       = 0x0000000b,
    IMG_NUM_FORMAT_FMASK_64_16_8                       = 0x0000000c,
    IMG_NUM_FORMAT_FMASK_RESERVED_13                   = 0x0000000d,
    IMG_NUM_FORMAT_FMASK_RESERVED_14                   = 0x0000000e,
    IMG_NUM_FORMAT_FMASK_RESERVED_15                   = 0x0000000f,
} IMG_NUM_FORMAT_FMASK;

typedef enum IMG_NUM_FORMAT_N_IN_16 {
    IMG_NUM_FORMAT_N_IN_16_RESERVED_0                  = 0x00000000,
    IMG_NUM_FORMAT_N_IN_16_UNORM_10                    = 0x00000001,
    IMG_NUM_FORMAT_N_IN_16_UNORM_9                     = 0x00000002,
    IMG_NUM_FORMAT_N_IN_16_RESERVED_3                  = 0x00000003,
    IMG_NUM_FORMAT_N_IN_16_UINT_10                     = 0x00000004,
    IMG_NUM_FORMAT_N_IN_16_UINT_9                      = 0x00000005,
    IMG_NUM_FORMAT_N_IN_16_RESERVED_6                  = 0x00000006,
    IMG_NUM_FORMAT_N_IN_16_UNORM_UINT_10               = 0x00000007,
    IMG_NUM_FORMAT_N_IN_16_UNORM_UINT_9                = 0x00000008,
    IMG_NUM_FORMAT_N_IN_16_RESERVED_9                  = 0x00000009,
    IMG_NUM_FORMAT_N_IN_16_RESERVED_10                 = 0x0000000a,
    IMG_NUM_FORMAT_N_IN_16_RESERVED_11                 = 0x0000000b,
    IMG_NUM_FORMAT_N_IN_16_RESERVED_12                 = 0x0000000c,
    IMG_NUM_FORMAT_N_IN_16_RESERVED_13                 = 0x0000000d,
    IMG_NUM_FORMAT_N_IN_16_RESERVED_14                 = 0x0000000e,
    IMG_NUM_FORMAT_N_IN_16_RESERVED_15                 = 0x0000000f,
} IMG_NUM_FORMAT_N_IN_16;

typedef enum MTYPE {
    MTYPE_C_RW_US                                      = 0x00000000,
    MTYPE_RESERVED_1                                   = 0x00000001,
    MTYPE_C_RO_S                                       = 0x00000002,
    MTYPE_UC                                           = 0x00000003,
    MTYPE_C_RW_S                                       = 0x00000004,
    MTYPE_RESERVED_5                                   = 0x00000005,
    MTYPE_C_RO_US                                      = 0x00000006,
    MTYPE_RESERVED_7                                   = 0x00000007,
} MTYPE;

typedef enum OreoMode {
    OMODE_BLEND                                        = 0x00000000,
    OMODE_O_THEN_B                                     = 0x00000001,
    OMODE_P_THEN_O_THEN_B                              = 0x00000002,
    OMODE_RESERVED_3                                   = 0x00000003,
} OreoMode;

typedef enum PC_PERFCNT_SEL {
    PC_PERF_SC_PC_PTR_SEND0                            = 0x00000000,
    PC_PERF_SC_PC_PTR_VALID0                           = 0x00000001,
    PC_PERF_SC_FPOSG0                                  = 0x00000002,
    PC_PERF_SC_FPOSG_WAIT0                             = 0x00000003,
    PC_PERF_SC_WAIT_SYNC0                              = 0x00000004,
    PC_PERF_SC_PQ_FREEZE0                              = 0x00000005,
    PC_PERF_SC_PC_PTR_SEND1                            = 0x00000006,
    PC_PERF_SC_PC_PTR_VALID1                           = 0x00000007,
    PC_PERF_SC_FPOSG1                                  = 0x00000008,
    PC_PERF_SC_FPOSG_WAIT1                             = 0x00000009,
    PC_PERF_SC_WAIT_SYNC1                              = 0x0000000a,
    PC_PERF_SC_PQ_FREEZE1                              = 0x0000000b,
    PC_PERF_SC_PC_PTR_SEND2                            = 0x0000000c,
    PC_PERF_SC_PC_PTR_VALID2                           = 0x0000000d,
    PC_PERF_SC_FPOSG2                                  = 0x0000000e,
    PC_PERF_SC_FPOSG_WAIT2                             = 0x0000000f,
    PC_PERF_SC_WAIT_SYNC2                              = 0x00000010,
    PC_PERF_SC_PQ_FREEZE2                              = 0x00000011,
    PC_PERF_SC_PC_PTR_SEND3                            = 0x00000012,
    PC_PERF_SC_PC_PTR_VALID3                           = 0x00000013,
    PC_PERF_SC_FPOSG3                                  = 0x00000014,
    PC_PERF_SC_FPOSG_WAIT3                             = 0x00000015,
    PC_PERF_SC_WAIT_SYNC3                              = 0x00000016,
    PC_PERF_SC_PQ_FREEZE3                              = 0x00000017,
    PC_PERF_SC_MW_FREEZE                               = 0x00000018,
    PC_PERF_SC_NUM_PROBES                              = 0x00000019,
    PC_PERF_SC_NUM_WAVES                               = 0x0000001a,
    PC_PERF_SC_NUM_SPLIT_WAVES                         = 0x0000001b,
    PC_PERF_GE_GSDONE                                  = 0x0000001c,
    PC_PERF_PC_SPI_PROBE_FREEZE                        = 0x0000001d,
    PC_PERF_MW_RTN_ADDR_FREEZE                         = 0x0000001e,
    PC_PERF_MW_PROBE_CNT_FREEZE                        = 0x0000001f,
    PC_PERF_MW_GL1H_REQ_FREEZE                         = 0x00000020,
    PC_PERF_MW_GL1H_NUM_REQS                           = 0x00000021,
    PC_PERF_MW_DLINE_ALLOC                             = 0x00000022,
    PC_PERF_MW_DLINE_DEALLOC                           = 0x00000023,
    PC_PERF_MW_TAGLINE_ALLOC                           = 0x00000024,
    PC_PERF_MW_TAGLINE_DEALLOC                         = 0x00000025,
    PC_PERF_PC_MEM_BANK_CONF0                          = 0x00000026,
    PC_PERF_PC_MEM_BANK_CONF1                          = 0x00000027,
    PC_PERF_PC_LDS_VERTEX_REUSE0                       = 0x00000028,
    PC_PERF_PC_LDS_CNTL_VALID0                         = 0x00000029,
    PC_PERF_PC_LDS_VERTEX_REUSE1                       = 0x0000002a,
    PC_PERF_PC_LDS_CNTL_VALID1                         = 0x0000002b,
    PC_PERF_GRBM_BUSY                                  = 0x0000002c,
    PC_PERF_GL1H_REQ_ROUTE_ID0__APU11                  = 0x0000002d,
    PC_PERF_GL1H_REQ_ROUTE_ID1__APU11                  = 0x0000002e,
    PC_PERF_GL1H_REQ_ROUTE_ID0__NV33                   = 0x0000002d,
    PC_PERF_GL1H_REQ_ROUTE_ID1__NV33                   = 0x0000002e,
} PC_PERFCNT_SEL;

constexpr unsigned int MaxPcPerfcntSelNv32             = PC_PERF_GRBM_BUSY;
constexpr unsigned int MaxPcPerfcntSelNv31             = PC_PERF_GRBM_BUSY;
constexpr unsigned int MaxPcPerfcntSelApu11            = PC_PERF_GL1H_REQ_ROUTE_ID1__APU11;
constexpr unsigned int MaxPcPerfcntSelNv33             = PC_PERF_GL1H_REQ_ROUTE_ID1__NV33;

typedef enum PerfCounter_Vals {
    DB_PERF_SEL_SC_DB_tile_sends                       = 0x00000000,
    DB_PERF_SEL_SC_DB_tile_busy                        = 0x00000001,
    DB_PERF_SEL_SC_DB_tile_stalls                      = 0x00000002,
    DB_PERF_SEL_SC_DB_tile_events                      = 0x00000003,
    DB_PERF_SEL_SC_DB_tile_tiles                       = 0x00000004,
    DB_PERF_SEL_SC_DB_tile_covered                     = 0x00000005,
    DB_PERF_SEL_hiz_tc_read_starved                    = 0x00000006,
    DB_PERF_SEL_hiz_tc_write_stall                     = 0x00000007,
    DB_PERF_SEL_hiz_tile_culled                        = 0x00000008,
    DB_PERF_SEL_his_tile_culled                        = 0x00000009,
    DB_PERF_SEL_DB_SC_tile_sends                       = 0x0000000a,
    DB_PERF_SEL_DB_SC_tile_busy                        = 0x0000000b,
    DB_PERF_SEL_DB_SC_tile_stalls                      = 0x0000000c,
    DB_PERF_SEL_DB_SC_tile_df_stalls                   = 0x0000000d,
    DB_PERF_SEL_DB_SC_tile_tiles                       = 0x0000000e,
    DB_PERF_SEL_DB_SC_tile_culled                      = 0x0000000f,
    DB_PERF_SEL_DB_SC_tile_hier_kill                   = 0x00000010,
    DB_PERF_SEL_DB_SC_tile_fast_ops                    = 0x00000011,
    DB_PERF_SEL_DB_SC_tile_no_ops                      = 0x00000012,
    DB_PERF_SEL_DB_SC_tile_tile_rate                   = 0x00000013,
    DB_PERF_SEL_DB_SC_tile_ssaa_kill                   = 0x00000014,
    DB_PERF_SEL_DB_SC_tile_fast_z_ops                  = 0x00000015,
    DB_PERF_SEL_DB_SC_tile_fast_stencil_ops            = 0x00000016,
    DB_PERF_SEL_SC_DB_quad_sends                       = 0x00000017,
    DB_PERF_SEL_SC_DB_quad_busy                        = 0x00000018,
    DB_PERF_SEL_SC_DB_quad_squads                      = 0x00000019,
    DB_PERF_SEL_SC_DB_quad_tiles                       = 0x0000001a,
    DB_PERF_SEL_SC_DB_quad_pixels                      = 0x0000001b,
    DB_PERF_SEL_SC_DB_quad_killed_tiles                = 0x0000001c,
    DB_PERF_SEL_DB_SC_quad_sends                       = 0x0000001d,
    DB_PERF_SEL_DB_SC_quad_busy                        = 0x0000001e,
    DB_PERF_SEL_DB_SC_quad_stalls                      = 0x0000001f,
    DB_PERF_SEL_DB_SC_quad_tiles                       = 0x00000020,
    DB_PERF_SEL_DB_SC_quad_lit_quad                    = 0x00000021,
    DB_PERF_SEL_SX_DB_quad_sends                       = 0x00000025,
    DB_PERF_SEL_SX_DB_quad_busy                        = 0x00000026,
    DB_PERF_SEL_SX_DB_quad_stalls                      = 0x00000027,
    DB_PERF_SEL_SX_DB_quad_quads                       = 0x00000028,
    DB_PERF_SEL_SX_DB_quad_pixels                      = 0x00000029,
    DB_PERF_SEL_SX_DB_quad_exports                     = 0x0000002a,
    DB_PERF_SEL_SH_quads_outstanding_sum               = 0x0000002b,
    DB_PERF_SEL_tile_rd_sends                          = 0x00000030,
    DB_PERF_SEL_mi_tile_rd_outstanding_sum             = 0x00000031,
    DB_PERF_SEL_quad_rd_sends                          = 0x00000032,
    DB_PERF_SEL_quad_rd_busy                           = 0x00000033,
    DB_PERF_SEL_quad_rd_mi_stall                       = 0x00000034,
    DB_PERF_SEL_quad_rd_rw_collision                   = 0x00000035,
    DB_PERF_SEL_quad_rd_tag_stall                      = 0x00000036,
    DB_PERF_SEL_quad_rd_32byte_reqs                    = 0x00000037,
    DB_PERF_SEL_quad_rd_panic                          = 0x00000038,
    DB_PERF_SEL_mi_quad_rd_outstanding_sum             = 0x00000039,
    DB_PERF_SEL_quad_rdret_sends                       = 0x0000003a,
    DB_PERF_SEL_quad_rdret_busy                        = 0x0000003b,
    DB_PERF_SEL_tile_wr_sends                          = 0x0000003c,
    DB_PERF_SEL_tile_wr_acks                           = 0x0000003d,
    DB_PERF_SEL_mi_tile_wr_outstanding_sum             = 0x0000003e,
    DB_PERF_SEL_quad_wr_sends                          = 0x0000003f,
    DB_PERF_SEL_quad_wr_busy                           = 0x00000040,
    DB_PERF_SEL_quad_wr_mi_stall                       = 0x00000041,
    DB_PERF_SEL_quad_wr_coherency_stall                = 0x00000042,
    DB_PERF_SEL_quad_wr_acks                           = 0x00000043,
    DB_PERF_SEL_mi_quad_wr_outstanding_sum             = 0x00000044,
    DB_PERF_SEL_Tile_Cache_misses                      = 0x00000045,
    DB_PERF_SEL_Tile_Cache_hits                        = 0x00000046,
    DB_PERF_SEL_Tile_Cache_flushes                     = 0x00000047,
    DB_PERF_SEL_Tile_Cache_surface_stall               = 0x00000048,
    DB_PERF_SEL_Tile_Cache_starves                     = 0x00000049,
    DB_PERF_SEL_Tile_Cache_mem_return_starve           = 0x0000004a,
    DB_PERF_SEL_tcp_dispatcher_reads                   = 0x0000004b,
    DB_PERF_SEL_tcp_prefetcher_reads                   = 0x0000004c,
    DB_PERF_SEL_tcp_preloader_reads                    = 0x0000004d,
    DB_PERF_SEL_tcp_dispatcher_flushes                 = 0x0000004e,
    DB_PERF_SEL_tcp_prefetcher_flushes                 = 0x0000004f,
    DB_PERF_SEL_tcp_preloader_flushes                  = 0x00000050,
    DB_PERF_SEL_Depth_Tile_Cache_sends                 = 0x00000051,
    DB_PERF_SEL_Depth_Tile_Cache_busy                  = 0x00000052,
    DB_PERF_SEL_Depth_Tile_Cache_starves               = 0x00000053,
    DB_PERF_SEL_Depth_Tile_Cache_dtile_locked          = 0x00000054,
    DB_PERF_SEL_Depth_Tile_Cache_alloc_stall           = 0x00000055,
    DB_PERF_SEL_Depth_Tile_Cache_misses                = 0x00000056,
    DB_PERF_SEL_Depth_Tile_Cache_hits                  = 0x00000057,
    DB_PERF_SEL_Depth_Tile_Cache_flushes               = 0x00000058,
    DB_PERF_SEL_Depth_Tile_Cache_noop_tile             = 0x00000059,
    DB_PERF_SEL_Depth_Tile_Cache_detailed_noop         = 0x0000005a,
    DB_PERF_SEL_Depth_Tile_Cache_event                 = 0x0000005b,
    DB_PERF_SEL_Depth_Tile_Cache_tile_frees            = 0x0000005c,
    DB_PERF_SEL_Depth_Tile_Cache_data_frees            = 0x0000005d,
    DB_PERF_SEL_Depth_Tile_Cache_mem_return_starve     = 0x0000005e,
    DB_PERF_SEL_Stencil_Cache_misses                   = 0x0000005f,
    DB_PERF_SEL_Stencil_Cache_hits                     = 0x00000060,
    DB_PERF_SEL_Stencil_Cache_flushes                  = 0x00000061,
    DB_PERF_SEL_Stencil_Cache_starves                  = 0x00000062,
    DB_PERF_SEL_Stencil_Cache_frees                    = 0x00000063,
    DB_PERF_SEL_Z_Cache_separate_Z_misses              = 0x00000064,
    DB_PERF_SEL_Z_Cache_separate_Z_hits                = 0x00000065,
    DB_PERF_SEL_Z_Cache_separate_Z_flushes             = 0x00000066,
    DB_PERF_SEL_Z_Cache_separate_Z_starves             = 0x00000067,
    DB_PERF_SEL_Z_Cache_pmask_misses                   = 0x00000068,
    DB_PERF_SEL_Z_Cache_pmask_hits                     = 0x00000069,
    DB_PERF_SEL_Z_Cache_pmask_flushes                  = 0x0000006a,
    DB_PERF_SEL_Z_Cache_pmask_starves                  = 0x0000006b,
    DB_PERF_SEL_Z_Cache_frees                          = 0x0000006c,
    DB_PERF_SEL_Plane_Cache_misses                     = 0x0000006d,
    DB_PERF_SEL_Plane_Cache_hits                       = 0x0000006e,
    DB_PERF_SEL_Plane_Cache_flushes                    = 0x0000006f,
    DB_PERF_SEL_Plane_Cache_starves                    = 0x00000070,
    DB_PERF_SEL_Plane_Cache_frees                      = 0x00000071,
    DB_PERF_SEL_flush_expanded_stencil                 = 0x00000072,
    DB_PERF_SEL_flush_compressed_stencil               = 0x00000073,
    DB_PERF_SEL_flush_single_stencil                   = 0x00000074,
    DB_PERF_SEL_planes_flushed                         = 0x00000075,
    DB_PERF_SEL_flush_1plane                           = 0x00000076,
    DB_PERF_SEL_flush_2plane                           = 0x00000077,
    DB_PERF_SEL_flush_3plane                           = 0x00000078,
    DB_PERF_SEL_flush_4plane                           = 0x00000079,
    DB_PERF_SEL_flush_5plane                           = 0x0000007a,
    DB_PERF_SEL_flush_6plane                           = 0x0000007b,
    DB_PERF_SEL_flush_7plane                           = 0x0000007c,
    DB_PERF_SEL_flush_8plane                           = 0x0000007d,
    DB_PERF_SEL_flush_9plane                           = 0x0000007e,
    DB_PERF_SEL_flush_10plane                          = 0x0000007f,
    DB_PERF_SEL_flush_11plane                          = 0x00000080,
    DB_PERF_SEL_flush_12plane                          = 0x00000081,
    DB_PERF_SEL_flush_13plane                          = 0x00000082,
    DB_PERF_SEL_flush_14plane                          = 0x00000083,
    DB_PERF_SEL_flush_15plane                          = 0x00000084,
    DB_PERF_SEL_flush_16plane                          = 0x00000085,
    DB_PERF_SEL_flush_expanded_z                       = 0x00000086,
    DB_PERF_SEL_earlyZ_waiting_for_postZ_done          = 0x00000087,
    DB_PERF_SEL_reZ_waiting_for_postZ_done             = 0x00000088,
    DB_PERF_SEL_dk_tile_sends                          = 0x00000089,
    DB_PERF_SEL_dk_tile_busy                           = 0x0000008a,
    DB_PERF_SEL_dk_tile_quad_starves                   = 0x0000008b,
    DB_PERF_SEL_dk_tile_stalls                         = 0x0000008c,
    DB_PERF_SEL_dk_squad_sends                         = 0x0000008d,
    DB_PERF_SEL_dk_squad_busy                          = 0x0000008e,
    DB_PERF_SEL_dk_squad_stalls                        = 0x0000008f,
    DB_PERF_SEL_Op_Pipe_Busy                           = 0x00000090,
    DB_PERF_SEL_Op_Pipe_MC_Read_stall                  = 0x00000091,
    DB_PERF_SEL_qc_busy                                = 0x00000092,
    DB_PERF_SEL_qc_xfc                                 = 0x00000093,
    DB_PERF_SEL_qc_conflicts                           = 0x00000094,
    DB_PERF_SEL_qc_full_stall                          = 0x00000095,
    DB_PERF_SEL_qc_in_preZ_tile_stalls_postZ           = 0x00000096,
    DB_PERF_SEL_qc_in_postZ_tile_stalls_preZ           = 0x00000097,
    DB_PERF_SEL_tsc_insert_summarize_stall             = 0x00000098,
    DB_PERF_SEL_tl_busy                                = 0x00000099,
    DB_PERF_SEL_tl_dtc_read_starved                    = 0x0000009a,
    DB_PERF_SEL_tl_z_fetch_stall                       = 0x0000009b,
    DB_PERF_SEL_tl_stencil_stall                       = 0x0000009c,
    DB_PERF_SEL_tl_z_decompress_stall                  = 0x0000009d,
    DB_PERF_SEL_tl_stencil_locked_stall                = 0x0000009e,
    DB_PERF_SEL_tl_events                              = 0x0000009f,
    DB_PERF_SEL_tl_summarize_squads                    = 0x000000a0,
    DB_PERF_SEL_tl_flush_expand_squads                 = 0x000000a1,
    DB_PERF_SEL_tl_expand_squads                       = 0x000000a2,
    DB_PERF_SEL_tl_preZ_squads                         = 0x000000a3,
    DB_PERF_SEL_tl_postZ_squads                        = 0x000000a4,
    DB_PERF_SEL_tl_preZ_noop_squads                    = 0x000000a5,
    DB_PERF_SEL_tl_postZ_noop_squads                   = 0x000000a6,
    DB_PERF_SEL_tl_tile_ops                            = 0x000000a7,
    DB_PERF_SEL_tl_in_xfc                              = 0x000000a8,
    DB_PERF_SEL_tl_in_single_stencil_expand_stall      = 0x000000a9,
    DB_PERF_SEL_tl_in_fast_z_stall                     = 0x000000aa,
    DB_PERF_SEL_tl_out_xfc                             = 0x000000ab,
    DB_PERF_SEL_tl_out_squads                          = 0x000000ac,
    DB_PERF_SEL_zf_plane_multicycle                    = 0x000000ad,
    DB_PERF_SEL_PostZ_Samples_passing_Z                = 0x000000ae,
    DB_PERF_SEL_PostZ_Samples_failing_Z                = 0x000000af,
    DB_PERF_SEL_PostZ_Samples_failing_S                = 0x000000b0,
    DB_PERF_SEL_PreZ_Samples_passing_Z                 = 0x000000b1,
    DB_PERF_SEL_PreZ_Samples_failing_Z                 = 0x000000b2,
    DB_PERF_SEL_PreZ_Samples_failing_S                 = 0x000000b3,
    DB_PERF_SEL_ts_tc_update_stall                     = 0x000000b4,
    DB_PERF_SEL_sc_kick_start                          = 0x000000b5,
    DB_PERF_SEL_sc_kick_end                            = 0x000000b6,
    DB_PERF_SEL_clock_reg_active                       = 0x000000b7,
    DB_PERF_SEL_clock_main_active                      = 0x000000b8,
    DB_PERF_SEL_clock_mem_export_active                = 0x000000b9,
    DB_PERF_SEL_esr_ps_out_busy                        = 0x000000ba,
    DB_PERF_SEL_esr_ps_lqf_busy                        = 0x000000bb,
    DB_PERF_SEL_esr_ps_lqf_stall                       = 0x000000bc,
    DB_PERF_SEL_etr_out_send                           = 0x000000bd,
    DB_PERF_SEL_etr_out_busy                           = 0x000000be,
    DB_PERF_SEL_etr_out_ltile_probe_fifo_full_stall    = 0x000000bf,
    DB_PERF_SEL_etr_out_esr_stall                      = 0x000000c1,
    DB_PERF_SEL_esr_eot_fwd_busy                       = 0x000000c4,
    DB_PERF_SEL_esr_eot_fwd_holding_squad              = 0x000000c5,
    DB_PERF_SEL_esr_eot_fwd_forward                    = 0x000000c6,
    DB_PERF_SEL_esr_sqq_zi_busy                        = 0x000000c7,
    DB_PERF_SEL_esr_sqq_zi_stall                       = 0x000000c8,
    DB_PERF_SEL_postzl_sq_pt_busy                      = 0x000000c9,
    DB_PERF_SEL_postzl_sq_pt_stall                     = 0x000000ca,
    DB_PERF_SEL_postzl_se_busy                         = 0x000000cb,
    DB_PERF_SEL_postzl_se_stall                        = 0x000000cc,
    DB_PERF_SEL_postzl_partial_launch                  = 0x000000cd,
    DB_PERF_SEL_postzl_full_launch                     = 0x000000ce,
    DB_PERF_SEL_postzl_partial_waiting                 = 0x000000cf,
    DB_PERF_SEL_postzl_tile_mem_stall                  = 0x000000d0,
    DB_PERF_SEL_postzl_tile_init_stall                 = 0x000000d1,
    DB_PERF_SEL_prezl_tile_mem_stall                   = 0x000000d2,
    DB_PERF_SEL_prezl_tile_init_stall                  = 0x000000d3,
    DB_PERF_SEL_dtt_sm_clash_stall                     = 0x000000d4,
    DB_PERF_SEL_dtt_sm_slot_stall                      = 0x000000d5,
    DB_PERF_SEL_dtt_sm_miss_stall                      = 0x000000d6,
    DB_PERF_SEL_mi_rdreq_busy                          = 0x000000d7,
    DB_PERF_SEL_mi_rdreq_stall                         = 0x000000d8,
    DB_PERF_SEL_mi_wrreq_busy                          = 0x000000d9,
    DB_PERF_SEL_mi_wrreq_stall                         = 0x000000da,
    DB_PERF_SEL_recomp_tile_to_1zplane_no_fastop       = 0x000000db,
    DB_PERF_SEL_dkg_tile_rate_tile                     = 0x000000dc,
    DB_PERF_SEL_prezl_src_in_sends                     = 0x000000dd,
    DB_PERF_SEL_prezl_src_in_stall                     = 0x000000de,
    DB_PERF_SEL_prezl_src_in_squads                    = 0x000000df,
    DB_PERF_SEL_prezl_src_in_squads_unrolled           = 0x000000e0,
    DB_PERF_SEL_prezl_src_in_tile_rate                 = 0x000000e1,
    DB_PERF_SEL_prezl_src_in_tile_rate_unrolled        = 0x000000e2,
    DB_PERF_SEL_prezl_src_out_stall                    = 0x000000e3,
    DB_PERF_SEL_postzl_src_in_sends                    = 0x000000e4,
    DB_PERF_SEL_postzl_src_in_stall                    = 0x000000e5,
    DB_PERF_SEL_postzl_src_in_squads                   = 0x000000e6,
    DB_PERF_SEL_postzl_src_in_squads_unrolled          = 0x000000e7,
    DB_PERF_SEL_postzl_src_in_tile_rate                = 0x000000e8,
    DB_PERF_SEL_postzl_src_in_tile_rate_unrolled       = 0x000000e9,
    DB_PERF_SEL_postzl_src_out_stall                   = 0x000000ea,
    DB_PERF_SEL_esr_ps_src_in_sends                    = 0x000000eb,
    DB_PERF_SEL_esr_ps_src_in_stall                    = 0x000000ec,
    DB_PERF_SEL_esr_ps_src_in_squads                   = 0x000000ed,
    DB_PERF_SEL_esr_ps_src_in_squads_unrolled          = 0x000000ee,
    DB_PERF_SEL_esr_ps_src_in_tile_rate                = 0x000000ef,
    DB_PERF_SEL_esr_ps_src_in_tile_rate_unrolled       = 0x000000f0,
    DB_PERF_SEL_esr_ps_src_in_tile_rate_unrolled_to_pixel_rate = 0x000000f1,
    DB_PERF_SEL_esr_ps_src_out_stall                   = 0x000000f2,
    DB_PERF_SEL_depth_bounds_tile_culled               = 0x000000f3,
    DB_PERF_SEL_PreZ_Samples_failing_DB                = 0x000000f4,
    DB_PERF_SEL_PostZ_Samples_failing_DB               = 0x000000f5,
    DB_PERF_SEL_flush_compressed                       = 0x000000f6,
    DB_PERF_SEL_flush_plane_le4                        = 0x000000f7,
    DB_PERF_SEL_tiles_z_fully_summarized               = 0x000000f8,
    DB_PERF_SEL_tiles_stencil_fully_summarized         = 0x000000f9,
    DB_PERF_SEL_tiles_z_clear_on_expclear              = 0x000000fa,
    DB_PERF_SEL_tiles_s_clear_on_expclear              = 0x000000fb,
    DB_PERF_SEL_tiles_decomp_on_expclear               = 0x000000fc,
    DB_PERF_SEL_tiles_compressed_to_decompressed       = 0x000000fd,
    DB_PERF_SEL_Op_Pipe_Prez_Busy                      = 0x000000fe,
    DB_PERF_SEL_Op_Pipe_Postz_Busy                     = 0x000000ff,
    DB_PERF_SEL_di_dt_stall                            = 0x00000100,
    DB_PERF_SEL_DB_SC_s_tile_rate                      = 0x00000102,
    DB_PERF_SEL_DB_SC_c_tile_rate                      = 0x00000103,
    DB_PERF_SEL_DB_SC_z_tile_rate                      = 0x00000104,
    DB_PERF_SEL_DB_CB_tile_sends__GFX10                = 0x00000022,
    DB_PERF_SEL_DB_CB_tile_busy__GFX10                 = 0x00000023,
    DB_PERF_SEL_DB_CB_tile_stalls__GFX10               = 0x00000024,
    DB_PERF_SEL_DB_CB_lquad_sends__GFX10               = 0x0000002c,
    DB_PERF_SEL_DB_CB_lquad_busy__GFX10                = 0x0000002d,
    DB_PERF_SEL_DB_CB_lquad_stalls__GFX10              = 0x0000002e,
    DB_PERF_SEL_DB_CB_lquad_quads__GFX10               = 0x0000002f,
    DB_PERF_SEL_etr_out_cb_tile_stall__GFX10           = 0x000000c0,
    DB_PERF_SEL_esr_ps_sqq_busy__GFX10                 = 0x000000c2,
    DB_PERF_SEL_esr_ps_sqq_stall__GFX10                = 0x000000c3,
    Spare_261__GFX10                                   = 0x00000105,
    DB_PERF_SEL_DB_CB_lquad_export_quads__GFX10        = 0x00000106,
    DB_PERF_SEL_DB_CB_lquad_double_format__GFX10       = 0x00000107,
    DB_PERF_SEL_DB_CB_lquad_fast_format__GFX10         = 0x00000108,
    DB_PERF_SEL_DB_CB_lquad_slow_format__GFX10         = 0x00000109,
    DB_PERF_SEL_CB_DB_rdreq_sends__GFX10               = 0x0000010a,
    DB_PERF_SEL_CB_DB_rdreq_prt_sends__GFX10           = 0x0000010b,
    DB_PERF_SEL_CB_DB_wrreq_sends__GFX10               = 0x0000010c,
    DB_PERF_SEL_CB_DB_wrreq_prt_sends__GFX10           = 0x0000010d,
    DB_PERF_SEL_DB_CB_rdret_ack__GFX10                 = 0x0000010e,
    DB_PERF_SEL_DB_CB_rdret_nack__GFX10                = 0x0000010f,
    DB_PERF_SEL_DB_CB_wrret_ack__GFX10                 = 0x00000110,
    DB_PERF_SEL_DB_CB_wrret_nack__GFX10                = 0x00000111,
    Spare_274__GFX10                                   = 0x00000112,
    DB_PERF_SEL_DFSM_Stall_opmode_change__GFX10        = 0x00000113,
    DB_PERF_SEL_DFSM_Stall_cam_fifo__GFX10             = 0x00000114,
    DB_PERF_SEL_DFSM_Stall_bypass_fifo__GFX10          = 0x00000115,
    DB_PERF_SEL_DFSM_Stall_retained_tile_fifo__GFX10   = 0x00000116,
    DB_PERF_SEL_DFSM_Stall_control_fifo__GFX10         = 0x00000117,
    DB_PERF_SEL_DFSM_Stall_overflow_counter__GFX10     = 0x00000118,
    DB_PERF_SEL_DFSM_Stall_pops_stall_overflow__GFX10  = 0x00000119,
    DB_PERF_SEL_DFSM_Stall_pops_stall_self_flush__GFX10 = 0x0000011a,
    DB_PERF_SEL_DFSM_Stall_middle_output__GFX10        = 0x0000011b,
    DB_PERF_SEL_DFSM_Stall_stalling_general__GFX10     = 0x0000011c,
    Spare_285__GFX10                                   = 0x0000011d,
    Spare_286__GFX10                                   = 0x0000011e,
    DB_PERF_SEL_DFSM_prez_killed_squad__GFX10          = 0x0000011f,
    DB_PERF_SEL_DFSM_squads_in__GFX10                  = 0x00000120,
    DB_PERF_SEL_DFSM_full_cleared_squads_out__GFX10    = 0x00000121,
    DB_PERF_SEL_DFSM_quads_in__GFX10                   = 0x00000122,
    DB_PERF_SEL_DFSM_fully_cleared_quads_out__GFX10    = 0x00000123,
    DB_PERF_SEL_DFSM_lit_pixels_in__GFX10              = 0x00000124,
    DB_PERF_SEL_DFSM_fully_cleared_pixels_out__GFX10   = 0x00000125,
    DB_PERF_SEL_DFSM_lit_samples_in__GFX10             = 0x00000126,
    DB_PERF_SEL_DFSM_lit_samples_out__GFX10            = 0x00000127,
    DB_PERF_SEL_DFSM_evicted_tiles_above_watermark__GFX10 = 0x00000128,
    DB_PERF_SEL_DFSM_cant_accept_squads_but_not_stalled_by_downstream__GFX10 = 0x00000129,
    DB_PERF_SEL_DFSM_stalled_by_downstream__GFX10      = 0x0000012a,
    DB_PERF_SEL_DFSM_evicted_squads_above_watermark__GFX10 = 0x0000012b,
    DB_PERF_SEL_DFSM_collisions_due_to_POPS_overflow__GFX10 = 0x0000012c,
    DB_PERF_SEL_DFSM_collisions_detected_within_POPS_FIFO__GFX10 = 0x0000012d,
    DB_PERF_SEL_DFSM_evicted_squads_due_to_prim_watermark__GFX10 = 0x0000012e,
    DB_PERF_SEL_MI_tile_req_wrack_counter_stall__GFX10 = 0x0000012f,
    DB_PERF_SEL_MI_quad_req_wrack_counter_stall__GFX10 = 0x00000130,
    DB_PERF_SEL_MI_zpc_req_wrack_counter_stall__GFX10  = 0x00000131,
    DB_PERF_SEL_MI_psd_req_wrack_counter_stall__GFX10  = 0x00000132,
    DB_PERF_SEL_unmapped_z_tile_culled__GFX10          = 0x00000133,
    DB_PERF_SEL_DB_CB_tile_is_event_FLUSH_AND_INV_DB_DATA_TS__GFX10 = 0x00000134,
    DB_PERF_SEL_DB_CB_tile_is_event_FLUSH_AND_INV_CB_PIXEL_DATA__GFX10 = 0x00000135,
    DB_PERF_SEL_DB_CB_tile_is_event_BOTTOM_OF_PIPE_TS__GFX10 = 0x00000136,
    DB_PERF_SEL_DB_CB_tile_waiting_for_perfcounter_stop_event__GFX10 = 0x00000137,
    DB_PERF_SEL_DB_CB_lquad_fmt_32bpp_8pix__GFX10      = 0x00000138,
    DB_PERF_SEL_DB_CB_lquad_fmt_16_16_unsigned_8pix__GFX10 = 0x00000139,
    DB_PERF_SEL_DB_CB_lquad_fmt_16_16_signed_8pix__GFX10 = 0x0000013a,
    DB_PERF_SEL_DB_CB_lquad_fmt_16_16_float_8pix__GFX10 = 0x0000013b,
    DB_PERF_SEL_DB_CB_lquad_num_pixels_need_blending__GFX10 = 0x0000013c,
    DB_PERF_SEL_DB_CB_context_dones__GFX10             = 0x0000013d,
    DB_PERF_SEL_DB_CB_eop_dones__GFX10                 = 0x0000013e,
    DB_PERF_SEL_SX_DB_quad_all_pixels_killed__GFX10    = 0x0000013f,
    DB_PERF_SEL_SX_DB_quad_all_pixels_enabled__GFX10   = 0x00000140,
    DB_PERF_SEL_SX_DB_quad_need_blending_and_dst_read__GFX10 = 0x00000141,
    DB_PERF_SEL_SC_DB_tile_backface__GFX10             = 0x00000142,
    DB_PERF_SEL_SC_DB_quad_quads__GFX10                = 0x00000143,
    DB_PERF_SEL_DB_SC_quad_quads_with_1_pixel__GFX10   = 0x00000144,
    DB_PERF_SEL_DB_SC_quad_quads_with_2_pixels__GFX10  = 0x00000145,
    DB_PERF_SEL_DB_SC_quad_quads_with_3_pixels__GFX10  = 0x00000146,
    DB_PERF_SEL_DB_SC_quad_quads_with_4_pixels__GFX10  = 0x00000147,
    DB_PERF_SEL_DFSM_Flush_flushabit__GFX10            = 0x00000148,
    DB_PERF_SEL_DFSM_Flush_flushabit_camcoord_fifo__GFX10 = 0x00000149,
    DB_PERF_SEL_DFSM_Flush_flushabit_passthrough__GFX10 = 0x0000014a,
    DB_PERF_SEL_DFSM_Flush_flushabit_forceflush__GFX10 = 0x0000014b,
    DB_PERF_SEL_DFSM_Flush_flushabit_nearlyfull__GFX10 = 0x0000014c,
    DB_PERF_SEL_DFSM_Flush_flushabit_primitivesinflightwatermark__GFX10 = 0x0000014d,
    DB_PERF_SEL_DFSM_Flush_flushabit_punch_stalling__GFX10 = 0x0000014e,
    DB_PERF_SEL_DFSM_Flush_flushabit_retainedtilefifo_watermark__GFX10 = 0x0000014f,
    DB_PERF_SEL_DFSM_Flush_flushabit_tilesinflightwatermark__GFX10 = 0x00000150,
    DB_PERF_SEL_DFSM_Flush_flushall__GFX10             = 0x00000151,
    DB_PERF_SEL_DFSM_Flush_flushall_dfsmflush__GFX10   = 0x00000152,
    DB_PERF_SEL_DFSM_Flush_flushall_opmodechange__GFX10 = 0x00000153,
    DB_PERF_SEL_DFSM_Flush_flushall_sampleratechange__GFX10 = 0x00000154,
    DB_PERF_SEL_DFSM_Flush_flushall_watchdog__GFX10    = 0x00000155,
    DB_PERF_SEL_DB_SC_quad_double_quad__GFX10          = 0x00000156,
    DB_PERF_SEL_SX_DB_quad_export_quads__GFX10         = 0x00000157,
    DB_PERF_SEL_SX_DB_quad_double_format__GFX10        = 0x00000158,
    DB_PERF_SEL_SX_DB_quad_fast_format__GFX10          = 0x00000159,
    DB_PERF_SEL_SX_DB_quad_slow_format__GFX10          = 0x0000015a,
    DB_PERF_SEL_quad_rd_sends_unc__GFX10               = 0x0000015b,
    DB_PERF_SEL_quad_rd_mi_stall_unc__GFX10            = 0x0000015c,
    DB_PERF_SEL_DFSM_OutputPunch__GFX10                = 0x0000015d,
    DB_PERF_SEL_DFSM_OutputPops__GFX10                 = 0x0000015e,
    DB_PERF_SEL_DFSM_OutputFifo__GFX10                 = 0x0000015f,
    DB_PERF_SEL_DFSM_StallOpmodeChange__GFX10          = 0x00000160,
    DB_PERF_SEL_DFSM_StallCAMFifoFull__GFX10           = 0x00000161,
    DB_PERF_SEL_DFSM_StallBypassFifoFull__GFX10        = 0x00000162,
    DB_PERF_SEL_DFSM_StallRetainedTileFifoFull__GFX10  = 0x00000163,
    DB_PERF_SEL_DFSM_StallControlFifoFull__GFX10       = 0x00000164,
    DB_PERF_SEL_DFSM_StallControlCountFull__GFX10      = 0x00000165,
    DB_PERF_SEL_DFSM_StallOverflowMaximum__GFX10       = 0x00000166,
    DB_PERF_SEL_DFSM_StallPopsStallOverflow__GFX10     = 0x00000167,
    DB_PERF_SEL_DFSM_StallPopsStallSelfStall__GFX10    = 0x00000168,
    DB_PERF_SEL_DFSM_StallCamSlotFlush__GFX10          = 0x00000169,
    DB_PERF_SEL_DFSM_StallOutput__GFX10                = 0x0000016a,
    DB_PERF_SEL_DFSM_WatchdogTrigger__GFX10            = 0x0000016b,
    DB_PERF_SEL_DFSM_StallOnPOPSStall__GFX10           = 0x0000016c,
    DB_PERF_SEL_SC_DB_tile_tiles_pipe0__GFX10          = 0x0000016d,
    DB_PERF_SEL_SC_DB_tile_tiles_pipe1__GFX10          = 0x0000016e,
    DB_PERF_SEL_SC_DB_quad_quads_pipe0__GFX10          = 0x0000016f,
    DB_PERF_SEL_SC_DB_quad_quads_pipe1__GFX10          = 0x00000170,
    DB_PERF_SEL_FG_LOB_FWDR_TIMEOUT_hits__GFX10        = 0x00000171,
    DB_PERF_SEL_DB_SC_quad_lit_quad_pre_invoke__GFX101 = 0x00000101,
    DB_PERF_SEL_DB_SC_quad_lit_quad_pre_kill__GFX10VRS = 0x00000101,
    DB_PERF_SEL_noz_waiting_for_postz_done__GFX10VRS   = 0x00000172,
    DB_PERF_SEL_DB_CB_lquad_quads_vrs_rate_1x1__GFX10VRS = 0x00000173,
    DB_PERF_SEL_DB_CB_lquad_quads_vrs_rate_2x1__GFX10VRS = 0x00000174,
    DB_PERF_SEL_DB_CB_lquad_quads_vrs_rate_1x2__GFX10VRS = 0x00000175,
    RMI_rd_tile_32byte_req__GFX10VRS                   = 0x00000176,
    RMI_rd_z_32byte_req__GFX10VRS                      = 0x00000177,
    RMI_rd_s_32byte_req__GFX10VRS                      = 0x00000178,
    RMI_wr_tile_32byte_req__GFX10VRS                   = 0x00000179,
    RMI_wr_z_32byte_req__GFX10VRS                      = 0x0000017a,
    RMI_wr_s_32byte_req__GFX10VRS                      = 0x0000017b,
    RMI_wr_psdzpc_32byte_req__GFX10VRS                 = 0x0000017c,
    RMI_rd_tile_32byte_ret__GFX10VRS                   = 0x0000017d,
    RMI_rd_z_32byte_ret__GFX10VRS                      = 0x0000017e,
    RMI_rd_s_32byte_ret__GFX10VRS                      = 0x0000017f,
    RMI_wr_tile_32byte_ack__GFX10VRS                   = 0x00000180,
    RMI_wr_z_32byte_ack__GFX10VRS                      = 0x00000181,
    RMI_wr_s_32byte_ack__GFX10VRS                      = 0x00000182,
    RMI_wr_psdzpc_32byte_ack__GFX10VRS                 = 0x00000183,
    DB_PERF_SEL_DB_CB_lquad_quads_vrs_rate_2x2__GFX10VRS = 0x00000184,
    DB_PERF_SEL_prez_ps_invoked_pixel_cnt__GFX10VRS    = 0x00000185,
    DB_PERF_SEL_postz_ps_invoked_pixel_cnt__GFX10VRS   = 0x00000186,
    DB_PERF_SEL_DB_CB_export_events__GFX11             = 0x00000022,
    DB_PERF_SEL_DB_CB_export_sends__GFX11              = 0x0000002c,
    DB_PERF_SEL_DB_CB_export_busy__GFX11               = 0x0000002d,
    DB_PERF_SEL_DB_CB_export_stalls__GFX11             = 0x0000002e,
    DB_PERF_SEL_DB_CB_export_quads__GFX11              = 0x0000002f,
    DB_PERF_SEL_esr_ps_vic_busy__GFX11                 = 0x000000c2,
    DB_PERF_SEL_esr_ps_vic_stall__GFX11                = 0x000000c3,
    DB_PERF_SEL_DB_CB_export_export_quads__GFX11       = 0x00000105,
    DB_PERF_SEL_DB_CB_export_double_format__GFX11      = 0x00000106,
    DB_PERF_SEL_DB_CB_export_fast_format__GFX11        = 0x00000107,
    DB_PERF_SEL_DB_CB_export_slow_format__GFX11        = 0x00000108,
    DB_PERF_SEL_CB_DB_rdreq_sends__GFX11               = 0x00000109,
    DB_PERF_SEL_CB_DB_rdreq_prt_sends__GFX11           = 0x0000010a,
    DB_PERF_SEL_CB_DB_wrreq_sends__GFX11               = 0x0000010b,
    DB_PERF_SEL_CB_DB_wrreq_prt_sends__GFX11           = 0x0000010c,
    DB_PERF_SEL_DB_CB_rdret_ack__GFX11                 = 0x0000010d,
    DB_PERF_SEL_DB_CB_rdret_nack__GFX11                = 0x0000010e,
    DB_PERF_SEL_DB_CB_wrret_ack__GFX11                 = 0x0000010f,
    DB_PERF_SEL_DB_CB_wrret_nack__GFX11                = 0x00000110,
    DB_PERF_SEL_MI_tile_req_wrack_counter_stall__GFX11 = 0x00000111,
    DB_PERF_SEL_MI_quad_req_wrack_counter_stall__GFX11 = 0x00000112,
    DB_PERF_SEL_MI_zpc_req_wrack_counter_stall__GFX11  = 0x00000113,
    DB_PERF_SEL_MI_psd_req_wrack_counter_stall__GFX11  = 0x00000114,
    DB_PERF_SEL_unmapped_z_tile_culled__GFX11          = 0x00000115,
    DB_PERF_SEL_DB_CB_export_is_event_FLUSH_AND_INV_DB_DATA_TS__GFX11 = 0x00000116,
    DB_PERF_SEL_DB_CB_export_is_event_FLUSH_AND_INV_CB_PIXEL_DATA__GFX11 = 0x00000117,
    DB_PERF_SEL_DB_CB_export_is_event_BOTTOM_OF_PIPE_TS__GFX11 = 0x00000118,
    DB_PERF_SEL_DB_CB_export_waiting_for_perfcounter_stop_event__GFX11 = 0x00000119,
    DB_PERF_SEL_DB_CB_export_fmt_32bpp_8pix__GFX11     = 0x0000011a,
    DB_PERF_SEL_DB_CB_export_fmt_16_16_unsigned_8pix__GFX11 = 0x0000011b,
    DB_PERF_SEL_DB_CB_export_fmt_16_16_signed_8pix__GFX11 = 0x0000011c,
    DB_PERF_SEL_DB_CB_export_fmt_16_16_float_8pix__GFX11 = 0x0000011d,
    DB_PERF_SEL_DB_CB_export_num_pixels_need_blending__GFX11 = 0x0000011e,
    DB_PERF_SEL_DB_CB_context_dones__GFX11             = 0x0000011f,
    DB_PERF_SEL_DB_CB_eop_dones__GFX11                 = 0x00000120,
    DB_PERF_SEL_SX_DB_quad_all_pixels_killed__GFX11    = 0x00000121,
    DB_PERF_SEL_SX_DB_quad_all_pixels_enabled__GFX11   = 0x00000122,
    DB_PERF_SEL_SX_DB_quad_need_blending_and_dst_read__GFX11 = 0x00000123,
    DB_PERF_SEL_SC_DB_tile_backface__GFX11             = 0x00000124,
    DB_PERF_SEL_SC_DB_quad_quads__GFX11                = 0x00000125,
    DB_PERF_SEL_DB_SC_quad_quads_with_1_pixel__GFX11   = 0x00000126,
    DB_PERF_SEL_DB_SC_quad_quads_with_2_pixels__GFX11  = 0x00000127,
    DB_PERF_SEL_DB_SC_quad_quads_with_3_pixels__GFX11  = 0x00000128,
    DB_PERF_SEL_DB_SC_quad_quads_with_4_pixels__GFX11  = 0x00000129,
    DB_PERF_SEL_DB_SC_quad_double_quad__GFX11          = 0x0000012a,
    DB_PERF_SEL_SX_DB_quad_export_quads__GFX11         = 0x0000012b,
    DB_PERF_SEL_SX_DB_quad_double_format__GFX11        = 0x0000012c,
    DB_PERF_SEL_SX_DB_quad_fast_format__GFX11          = 0x0000012d,
    DB_PERF_SEL_SX_DB_quad_slow_format__GFX11          = 0x0000012e,
    DB_PERF_SEL_quad_rd_sends_unc__GFX11               = 0x0000012f,
    DB_PERF_SEL_quad_rd_mi_stall_unc__GFX11            = 0x00000130,
    DB_PERF_SEL_SC_DB_tile_tiles_pipe0__GFX11          = 0x00000131,
    DB_PERF_SEL_SC_DB_tile_tiles_pipe1__GFX11          = 0x00000132,
    DB_PERF_SEL_SC_DB_quad_quads_pipe0__GFX11          = 0x00000133,
    DB_PERF_SEL_SC_DB_quad_quads_pipe1__GFX11          = 0x00000134,
    DB_PERF_SEL_PERF_fg_lob_fwdr_timeout_hits__GFX11   = 0x00000135,
    DB_PERF_SEL_noz_waiting_for_postz_done__GFX11      = 0x00000136,
    DB_PERF_SEL_DB_CB_export_quads_vrs_rate_1x1__GFX11 = 0x00000137,
    DB_PERF_SEL_DB_CB_export_quads_vrs_rate_2x1__GFX11 = 0x00000138,
    DB_PERF_SEL_DB_CB_export_quads_vrs_rate_1x2__GFX11 = 0x00000139,
    DB_PERF_SEL_DB_CB_export_quads_vrs_rate_2x2__GFX11 = 0x0000013a,
    DB_PERF_SEL_RMI_rd_tile_32byte_req__GFX11          = 0x0000013b,
    DB_PERF_SEL_RMI_rd_z_32byte_req__GFX11             = 0x0000013c,
    DB_PERF_SEL_RMI_rd_s_32byte_req__GFX11             = 0x0000013d,
    DB_PERF_SEL_RMI_wr_tile_32byte_req__GFX11          = 0x0000013e,
    DB_PERF_SEL_RMI_wr_z_32byte_req__GFX11             = 0x0000013f,
    DB_PERF_SEL_RMI_wr_s_32byte_req__GFX11             = 0x00000140,
    DB_PERF_SEL_RMI_wr_psdzpc_32byte_req__GFX11        = 0x00000141,
    DB_PERF_SEL_RMI_rd_tile_32byte_ret__GFX11          = 0x00000142,
    DB_PERF_SEL_RMI_rd_z_32byte_ret__GFX11             = 0x00000143,
    DB_PERF_SEL_RMI_rd_s_32byte_ret__GFX11             = 0x00000144,
    DB_PERF_SEL_RMI_wr_tile_32byte_ack__GFX11          = 0x00000145,
    DB_PERF_SEL_RMI_wr_z_32byte_ack__GFX11             = 0x00000146,
    DB_PERF_SEL_RMI_wr_s_32byte_ack__GFX11             = 0x00000147,
    DB_PERF_SEL_RMI_wr_psdzpc_32byte_ack__GFX11        = 0x00000148,
    DB_PERF_SEL_esr_vic_sqq_busy__GFX11                = 0x00000149,
    DB_PERF_SEL_esr_vic_sqq_stall__GFX11               = 0x0000014a,
    DB_PERF_SEL_esr_psi_vic_tile_rate__GFX11           = 0x0000014b,
    DB_PERF_SEL_esr_vic_footprint_match_2x2__GFX11     = 0x0000014c,
    DB_PERF_SEL_esr_vic_footprint_match_2x1__GFX11     = 0x0000014d,
    DB_PERF_SEL_esr_vic_footprint_match_1x2__GFX11     = 0x0000014e,
    DB_PERF_SEL_DB_SC_quad_num_null_2x2_coarse_pixels__GFX11 = 0x0000014f,
    DB_PERF_SEL_DB_SC_quad_num_null_2x1_coarse_pixels__GFX11 = 0x00000150,
    DB_PERF_SEL_DB_SC_quad_num_null_1x2_coarse_pixels__GFX11 = 0x00000151,
    DB_PERF_SEL_hi_z_s_checker_force_coarse_vrs_1x1__GFX11 = 0x00000152,
    DB_PERF_SEL_hi_z_s_checker_force_ssaa_vrs_1x1__GFX11 = 0x00000153,
    DB_PERF_SEL_esr_ps_woc_1squadIn_2squadOut__GFX11   = 0x00000154,
    DB_PERF_SEL_esr_ps_woc_2squadIn_1squadOut__GFX11   = 0x00000155,
    DB_PERF_SEL_prez_ps_invoked_pixel_cnt__GFX11       = 0x00000156,
    DB_PERF_SEL_postz_ps_invoked_pixel_cnt__GFX11      = 0x00000157,
    DB_PERF_SEL_ts_events_pws_enable__GFX11            = 0x00000158,
    DB_PERF_SEL_ps_events_pws_enable__GFX11            = 0x00000159,
    DB_PERF_SEL_cs_events_pws_enable__GFX11            = 0x0000015a,
    DB_PERF_SEL_DB_SC_quad_noz_tiles__GFX11            = 0x0000015b,
    DB_PERF_SEL_DB_SC_quad_lit_noz_quad__GFX11         = 0x0000015c,
    DB_PERF_SEL_DB_SC_quad_conflicts__GFX11            = 0x0000015d,
    DB_PERF_SEL_SC_DB_quad_vrs_1x1__GFX11              = 0x0000015e,
    DB_PERF_SEL_SC_DB_quad_vrs_1x2__GFX11              = 0x0000015f,
    DB_PERF_SEL_SC_DB_quad_vrs_2x1__GFX11              = 0x00000160,
    DB_PERF_SEL_SC_DB_quad_vrs_2x2__GFX11              = 0x00000161,
    DB_PERF_SEL_SC_DB_quad_vrs_2x_ssaa__GFX11          = 0x00000162,
    DB_PERF_SEL_SC_DB_quad_vrs_4x_ssaa__GFX11          = 0x00000163,
    DB_PERF_SEL_SC_DB_quad_vrs_8x_ssaa__GFX11          = 0x00000164,
    DB_PERF_SEL_SC_DB_wave_sends__GFX11                = 0x00000165,
    DB_PERF_SEL_SC_DB_wave_busy__GFX11                 = 0x00000166,
    DB_PERF_SEL_SC_DB_wave_quads__GFX11                = 0x00000167,
    DB_PERF_SEL_SC_DB_wave_id_wrapped__GFX11           = 0x00000168,
    DB_PERF_SEL_DB_SC_wave_sends__GFX11                = 0x00000169,
    DB_PERF_SEL_DB_SC_wave_busy__GFX11                 = 0x0000016a,
    DB_PERF_SEL_DB_SC_wave_stalls__GFX11               = 0x0000016b,
    DB_PERF_SEL_DB_SC_wave_conflict__GFX11             = 0x0000016c,
    DB_PERF_SEL_DB_SC_wave_hard_conflict__GFX11        = 0x0000016d,
    DB_PERF_SEL_DB_SC_wave_id_wrapped__GFX11           = 0x0000016e,
    DB_PERF_SEL_SX_DB_quad_waves__GFX11                = 0x0000016f,
    DB_PERF_SEL_pws_stall__GFX11                       = 0x00000170,
    DB_PERF_SEL_pws_liveness_stall_dtt_tag__GFX11      = 0x00000171,
    DB_PERF_SEL_pws_liveness_stall_tcp_cache_mgr__GFX11 = 0x00000172,
    DB_PERF_SEL_OREO_TT_load__GFX11                    = 0x00000173,
    DB_PERF_SEL_OREO_TT_read__GFX11                    = 0x00000174,
    DB_PERF_SEL_OREO_TT_stalls__GFX11                  = 0x00000175,
    DB_PERF_SEL_OREO_ST_load__GFX11                    = 0x00000176,
    DB_PERF_SEL_OREO_ST_read__GFX11                    = 0x00000177,
    DB_PERF_SEL_OREO_ST_stalls__GFX11                  = 0x00000178,
    DB_PERF_SEL_OREO_WT_load__GFX11                    = 0x00000179,
    DB_PERF_SEL_OREO_WT_read__GFX11                    = 0x0000017a,
    DB_PERF_SEL_OREO_SB_misses__GFX11                  = 0x0000017b,
    DB_PERF_SEL_OREO_SB_hits__GFX11                    = 0x0000017c,
    DB_PERF_SEL_OREO_SB_evicts__GFX11                  = 0x0000017d,
    DB_PERF_SEL_OREO_SB_stalls__GFX11                  = 0x0000017e,
    DB_PERF_SEL_OREO_Events_load__GFX11                = 0x0000017f,
    DB_PERF_SEL_OREO_Events_transition__GFX11          = 0x00000180,
    DB_PERF_SEL_OREO_Events_non_transition__GFX11      = 0x00000181,
    DB_PERF_SEL_OREO_Events_delayed__GFX11             = 0x00000182,
    DB_PERF_SEL_OREO_Events_stalls__GFX11              = 0x00000183,
#if CHIP_HDR_STRIX_HALO
    DB_PERF_SEL_SC_DB_lso_pre_pass_tiles__STXH         = 0x00000184,
    DB_PERF_SEL_SC_DB_lso_color_pass_tiles__STXH       = 0x00000185,
    DB_PERF_SEL_DB_SC_lso_color_pass_killed_tiles__STXH = 0x00000186,
    DB_PERF_dc_zpp_passed_samples_cnt__STXH            = 0x00000187,
    DB_PERF_dc_zpp_failed_samples_cnt__STXH            = 0x00000188,
#endif
} PerfCounter_Vals;

constexpr unsigned int MaxPerfcounterValsGfx101        = DB_PERF_SEL_FG_LOB_FWDR_TIMEOUT_hits__GFX10;
constexpr unsigned int MaxPerfcounterValsGfx10Vrs      = DB_PERF_SEL_postz_ps_invoked_pixel_cnt__GFX10VRS;
constexpr unsigned int MaxPerfcounterValsGfx110        = DB_PERF_SEL_OREO_Events_stalls__GFX11;
constexpr unsigned int MaxPerfcounterValsStx1          = DB_PERF_SEL_OREO_Events_stalls__GFX11;
#if CHIP_HDR_STRIX_HALO
constexpr unsigned int MaxPerfcounterValsStxH          = DB_PERF_dc_zpp_failed_samples_cnt__STXH;
#endif

typedef enum PERFMON_CNTOFF_AND_OR {
    PERFMON_CNTOFF_OR__GFX101                          = 0x00000000,
    PERFMON_CNTOFF_AND__GFX101                         = 0x00000001,
    PERFMON_CNTOFF_OR__GFX110                          = 0x00000000,
    PERFMON_CNTOFF_AND__GFX110                         = 0x00000001,
    PERFMON_CNTOFF_OR__NV2X                            = 0x00000000,
    PERFMON_CNTOFF_AND__NV2X                           = 0x00000001,
    PERFMON_CNTOFF_OR__RAPHAEL                         = 0x00000000,
    PERFMON_CNTOFF_AND__RAPHAEL                        = 0x00000001,
    PERFMON_CNTOFF_OR__REMBRANDT                       = 0x00000000,
    PERFMON_CNTOFF_AND__REMBRANDT                      = 0x00000001,
    PERFMON_CNTOFF_OR__STX1                            = 0x00000000,
    PERFMON_CNTOFF_AND__STX1                           = 0x00000001,
#if CHIP_HDR_STRIX_HALO
    PERFMON_CNTOFF_OR__STXH                            = 0x00000000,
    PERFMON_CNTOFF_AND__STXH                           = 0x00000001,
#endif
} PERFMON_CNTOFF_AND_OR;

typedef enum PERFMON_CNTOFF_INT_EN {
    PERFMON_CNTOFF_INT_DISABLE__GFX101                 = 0x00000000,
    PERFMON_CNTOFF_INT_ENABLE__GFX101                  = 0x00000001,
    PERFMON_CNTOFF_INT_DISABLE__GFX110                 = 0x00000000,
    PERFMON_CNTOFF_INT_ENABLE__GFX110                  = 0x00000001,
    PERFMON_CNTOFF_INT_DISABLE__NV2X                   = 0x00000000,
    PERFMON_CNTOFF_INT_ENABLE__NV2X                    = 0x00000001,
    PERFMON_CNTOFF_INT_DISABLE__RAPHAEL                = 0x00000000,
    PERFMON_CNTOFF_INT_ENABLE__RAPHAEL                 = 0x00000001,
    PERFMON_CNTOFF_INT_DISABLE__REMBRANDT              = 0x00000000,
    PERFMON_CNTOFF_INT_ENABLE__REMBRANDT               = 0x00000001,
    PERFMON_CNTOFF_INT_DISABLE__STX1                   = 0x00000000,
    PERFMON_CNTOFF_INT_ENABLE__STX1                    = 0x00000001,
#if CHIP_HDR_STRIX_HALO
    PERFMON_CNTOFF_INT_DISABLE__STXH                   = 0x00000000,
    PERFMON_CNTOFF_INT_ENABLE__STXH                    = 0x00000001,
#endif
} PERFMON_CNTOFF_INT_EN;

typedef enum PERFMON_CNTOFF_INT_TYPE {
    PERFMON_CNTOFF_INT_TYPE_LEVEL__GFX101              = 0x00000000,
    PERFMON_CNTOFF_INT_TYPE_PULSE__GFX101              = 0x00000001,
    PERFMON_CNTOFF_INT_TYPE_LEVEL__GFX110              = 0x00000000,
    PERFMON_CNTOFF_INT_TYPE_PULSE__GFX110              = 0x00000001,
    PERFMON_CNTOFF_INT_TYPE_LEVEL__NV2X                = 0x00000000,
    PERFMON_CNTOFF_INT_TYPE_PULSE__NV2X                = 0x00000001,
    PERFMON_CNTOFF_INT_TYPE_LEVEL__RAPHAEL             = 0x00000000,
    PERFMON_CNTOFF_INT_TYPE_PULSE__RAPHAEL             = 0x00000001,
    PERFMON_CNTOFF_INT_TYPE_LEVEL__REMBRANDT           = 0x00000000,
    PERFMON_CNTOFF_INT_TYPE_PULSE__REMBRANDT           = 0x00000001,
    PERFMON_CNTOFF_INT_TYPE_LEVEL__STX1                = 0x00000000,
    PERFMON_CNTOFF_INT_TYPE_PULSE__STX1                = 0x00000001,
#if CHIP_HDR_STRIX_HALO
    PERFMON_CNTOFF_INT_TYPE_LEVEL__STXH                = 0x00000000,
    PERFMON_CNTOFF_INT_TYPE_PULSE__STXH                = 0x00000001,
#endif
} PERFMON_CNTOFF_INT_TYPE;

typedef enum PERFMON_COUNTER_MODE {
    PERFMON_COUNTER_MODE_ACCUM                         = 0x00000000,
    PERFMON_COUNTER_MODE_ACTIVE_CYCLES                 = 0x00000001,
    PERFMON_COUNTER_MODE_MAX                           = 0x00000002,
    PERFMON_COUNTER_MODE_DIRTY                         = 0x00000003,
    PERFMON_COUNTER_MODE_SAMPLE                        = 0x00000004,
    PERFMON_COUNTER_MODE_CYCLES_SINCE_FIRST_EVENT      = 0x00000005,
    PERFMON_COUNTER_MODE_CYCLES_SINCE_LAST_EVENT       = 0x00000006,
    PERFMON_COUNTER_MODE_CYCLES_GE_HI                  = 0x00000007,
    PERFMON_COUNTER_MODE_CYCLES_EQ_HI                  = 0x00000008,
    PERFMON_COUNTER_MODE_INACTIVE_CYCLES               = 0x00000009,
    PERFMON_COUNTER_MODE_RESERVED                      = 0x0000000f,
} PERFMON_COUNTER_MODE;

typedef enum PERFMON_SPM_MODE {
    PERFMON_SPM_MODE_OFF                               = 0x00000000,
    PERFMON_SPM_MODE_16BIT_CLAMP                       = 0x00000001,
    PERFMON_SPM_MODE_16BIT_NO_CLAMP                    = 0x00000002,
    PERFMON_SPM_MODE_32BIT_CLAMP                       = 0x00000003,
    PERFMON_SPM_MODE_32BIT_NO_CLAMP                    = 0x00000004,
    PERFMON_SPM_MODE_RESERVED_5                        = 0x00000005,
    PERFMON_SPM_MODE_RESERVED_6                        = 0x00000006,
    PERFMON_SPM_MODE_RESERVED_7                        = 0x00000007,
    PERFMON_SPM_MODE_TEST_MODE_0                       = 0x00000008,
    PERFMON_SPM_MODE_TEST_MODE_1                       = 0x00000009,
    PERFMON_SPM_MODE_TEST_MODE_2                       = 0x0000000a,
} PERFMON_SPM_MODE;

typedef enum PERFMON_STATE {
    PERFMON_STATE_RESET__GFX101                        = 0x00000000,
    PERFMON_STATE_START__GFX101                        = 0x00000001,
    PERFMON_STATE_FREEZE__GFX101                       = 0x00000002,
    PERFMON_STATE_HW__GFX101                           = 0x00000003,
    PERFMON_STATE_RESET__GFX110                        = 0x00000000,
    PERFMON_STATE_START__GFX110                        = 0x00000001,
    PERFMON_STATE_FREEZE__GFX110                       = 0x00000002,
    PERFMON_STATE_HW__GFX110                           = 0x00000003,
    PERFMON_STATE_RESET__NV2X                          = 0x00000000,
    PERFMON_STATE_START__NV2X                          = 0x00000001,
    PERFMON_STATE_FREEZE__NV2X                         = 0x00000002,
    PERFMON_STATE_HW__NV2X                             = 0x00000003,
    PERFMON_STATE_RESET__RAPHAEL                       = 0x00000000,
    PERFMON_STATE_START__RAPHAEL                       = 0x00000001,
    PERFMON_STATE_FREEZE__RAPHAEL                      = 0x00000002,
    PERFMON_STATE_HW__RAPHAEL                          = 0x00000003,
    PERFMON_STATE_RESET__REMBRANDT                     = 0x00000000,
    PERFMON_STATE_START__REMBRANDT                     = 0x00000001,
    PERFMON_STATE_FREEZE__REMBRANDT                    = 0x00000002,
    PERFMON_STATE_HW__REMBRANDT                        = 0x00000003,
    PERFMON_STATE_RESET__STX1                          = 0x00000000,
    PERFMON_STATE_START__STX1                          = 0x00000001,
    PERFMON_STATE_FREEZE__STX1                         = 0x00000002,
    PERFMON_STATE_HW__STX1                             = 0x00000003,
#if CHIP_HDR_STRIX_HALO
    PERFMON_STATE_RESET__STXH                          = 0x00000000,
    PERFMON_STATE_START__STXH                          = 0x00000001,
    PERFMON_STATE_FREEZE__STXH                         = 0x00000002,
    PERFMON_STATE_HW__STXH                             = 0x00000003,
#endif
} PERFMON_STATE;

typedef enum PH_PERFCNT_SEL {
    PH_PERF_SEL_SC0_SRPS_WINDOW_VALID                  = 0x00000000,
    PH_PERF_SEL_SC0_ARB_XFC_ALL_EVENT_OR_PRIM_CYCLES   = 0x00000001,
    PH_PERF_SEL_SC0_ARB_XFC_ONLY_PRIM_CYCLES           = 0x00000002,
    PH_PERF_SEL_SC0_ARB_XFC_ONLY_ONE_INC_PER_PRIM      = 0x00000003,
    PH_PERF_SEL_SC0_ARB_STALLED_FROM_BELOW             = 0x00000004,
    PH_PERF_SEL_SC0_ARB_STARVED_FROM_ABOVE             = 0x00000005,
    PH_PERF_SEL_SC0_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_NOT_EMPTY = 0x00000006,
    PH_PERF_SEL_SC0_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_FULL = 0x00000007,
    PH_PERF_SEL_SC0_ARB_BUSY                           = 0x00000008,
    PH_PERF_SEL_SC0_ARB_PA_BUSY_SOP                    = 0x00000009,
    PH_PERF_SEL_SC0_ARB_EOP_POP_SYNC_POP               = 0x0000000a,
    PH_PERF_SEL_SC0_ARB_EVENT_SYNC_POP                 = 0x0000000b,
    PH_PERF_SEL_SC0_PS_ENG_MULTICYCLE_BUBBLE           = 0x0000000c,
    PH_PERF_SEL_SC0_EOP_SYNC_WINDOW                    = 0x0000000d,
    PH_PERF_SEL_SC0_BUSY_PROCESSING_MULTICYCLE_PRIM    = 0x0000000e,
    PH_PERF_SEL_SC0_BUSY_CNT_NOT_ZERO                  = 0x0000000f,
    PH_PERF_SEL_SC0_SEND                               = 0x00000010,
    PH_PERF_SEL_SC0_CREDIT_AT_ZERO_WITH_PENDING_SEND   = 0x00000011,
    PH_PERF_SEL_SC0_CREDIT_AT_MAX                      = 0x00000012,
    PH_PERF_SEL_SC0_CREDIT_AT_MAX_NO_PENDING_SEND      = 0x00000013,
    PH_PERF_SEL_SC0_PA0_DATA_FIFO_RD                   = 0x00000018,
    PH_PERF_SEL_SC0_PA0_DATA_FIFO_WE                   = 0x00000019,
    PH_PERF_SEL_SC0_PA0_FIFO_EMPTY                     = 0x0000001a,
    PH_PERF_SEL_SC0_PA0_FIFO_FULL                      = 0x0000001b,
    PH_PERF_SEL_SC0_PA0_NULL_WE                        = 0x0000001c,
    PH_PERF_SEL_SC0_PA0_EVENT_WE                       = 0x0000001d,
    PH_PERF_SEL_SC0_PA0_FPOV_WE                        = 0x0000001e,
    PH_PERF_SEL_SC0_PA0_EOP_WE                         = 0x00000020,
    PH_PERF_SEL_SC0_PA0_DATA_FIFO_EOP_RD               = 0x00000021,
    PH_PERF_SEL_SC0_PA0_EOPG_WE                        = 0x00000022,
    PH_PERF_SEL_SC0_PA1_DATA_FIFO_RD                   = 0x00000024,
    PH_PERF_SEL_SC0_PA1_DATA_FIFO_WE                   = 0x00000025,
    PH_PERF_SEL_SC0_PA1_FIFO_EMPTY                     = 0x00000026,
    PH_PERF_SEL_SC0_PA1_FIFO_FULL                      = 0x00000027,
    PH_PERF_SEL_SC0_PA1_NULL_WE                        = 0x00000028,
    PH_PERF_SEL_SC0_PA1_EVENT_WE                       = 0x00000029,
    PH_PERF_SEL_SC0_PA1_FPOV_WE                        = 0x0000002a,
    PH_PERF_SEL_SC0_PA1_EOP_WE                         = 0x0000002c,
    PH_PERF_SEL_SC0_PA1_DATA_FIFO_EOP_RD               = 0x0000002d,
    PH_PERF_SEL_SC0_PA1_EOPG_WE                        = 0x0000002e,
    PH_PERF_SEL_SC0_PA2_DATA_FIFO_RD                   = 0x00000030,
    PH_PERF_SEL_SC0_PA2_DATA_FIFO_WE                   = 0x00000031,
    PH_PERF_SEL_SC0_PA2_FIFO_EMPTY                     = 0x00000032,
    PH_PERF_SEL_SC0_PA2_FIFO_FULL                      = 0x00000033,
    PH_PERF_SEL_SC0_PA2_NULL_WE                        = 0x00000034,
    PH_PERF_SEL_SC0_PA2_EVENT_WE                       = 0x00000035,
    PH_PERF_SEL_SC0_PA2_FPOV_WE                        = 0x00000036,
    PH_PERF_SEL_SC0_PA2_EOP_WE                         = 0x00000038,
    PH_PERF_SEL_SC0_PA2_DATA_FIFO_EOP_RD               = 0x00000039,
    PH_PERF_SEL_SC0_PA2_EOPG_WE                        = 0x0000003a,
    PH_PERF_SEL_SC0_PA3_DATA_FIFO_RD                   = 0x0000003c,
    PH_PERF_SEL_SC0_PA3_DATA_FIFO_WE                   = 0x0000003d,
    PH_PERF_SEL_SC0_PA3_FIFO_EMPTY                     = 0x0000003e,
    PH_PERF_SEL_SC0_PA3_FIFO_FULL                      = 0x0000003f,
    PH_PERF_SEL_SC0_PA3_NULL_WE                        = 0x00000040,
    PH_PERF_SEL_SC0_PA3_EVENT_WE                       = 0x00000041,
    PH_PERF_SEL_SC0_PA3_FPOV_WE                        = 0x00000042,
    PH_PERF_SEL_SC0_PA3_EOP_WE                         = 0x00000044,
    PH_PERF_SEL_SC0_PA3_DATA_FIFO_EOP_RD               = 0x00000045,
    PH_PERF_SEL_SC0_PA3_EOPG_WE                        = 0x00000046,
    PH_PERF_SEL_SC0_PA4_DATA_FIFO_RD                   = 0x00000048,
    PH_PERF_SEL_SC0_PA4_DATA_FIFO_WE                   = 0x00000049,
    PH_PERF_SEL_SC0_PA4_FIFO_EMPTY                     = 0x0000004a,
    PH_PERF_SEL_SC0_PA4_FIFO_FULL                      = 0x0000004b,
    PH_PERF_SEL_SC0_PA4_NULL_WE                        = 0x0000004c,
    PH_PERF_SEL_SC0_PA4_EVENT_WE                       = 0x0000004d,
    PH_PERF_SEL_SC0_PA4_FPOV_WE                        = 0x0000004e,
    PH_PERF_SEL_SC0_PA4_EOP_WE                         = 0x00000050,
    PH_PERF_SEL_SC0_PA4_DATA_FIFO_EOP_RD               = 0x00000051,
    PH_PERF_SEL_SC0_PA4_EOPG_WE                        = 0x00000052,
    PH_PERF_SEL_SC0_PA5_DATA_FIFO_RD                   = 0x00000054,
    PH_PERF_SEL_SC0_PA5_DATA_FIFO_WE                   = 0x00000055,
    PH_PERF_SEL_SC0_PA5_FIFO_EMPTY                     = 0x00000056,
    PH_PERF_SEL_SC0_PA5_FIFO_FULL                      = 0x00000057,
    PH_PERF_SEL_SC0_PA5_NULL_WE                        = 0x00000058,
    PH_PERF_SEL_SC0_PA5_EVENT_WE                       = 0x00000059,
    PH_PERF_SEL_SC0_PA5_FPOV_WE                        = 0x0000005a,
    PH_PERF_SEL_SC0_PA5_EOP_WE                         = 0x0000005c,
    PH_PERF_SEL_SC0_PA5_DATA_FIFO_EOP_RD               = 0x0000005d,
    PH_PERF_SEL_SC0_PA5_EOPG_WE                        = 0x0000005e,
    PH_PERF_SEL_SC0_PA6_DATA_FIFO_RD                   = 0x00000060,
    PH_PERF_SEL_SC0_PA6_DATA_FIFO_WE                   = 0x00000061,
    PH_PERF_SEL_SC0_PA6_FIFO_EMPTY                     = 0x00000062,
    PH_PERF_SEL_SC0_PA6_FIFO_FULL                      = 0x00000063,
    PH_PERF_SEL_SC0_PA6_NULL_WE                        = 0x00000064,
    PH_PERF_SEL_SC0_PA6_EVENT_WE                       = 0x00000065,
    PH_PERF_SEL_SC0_PA6_FPOV_WE                        = 0x00000066,
    PH_PERF_SEL_SC0_PA6_EOP_WE                         = 0x00000068,
    PH_PERF_SEL_SC0_PA6_DATA_FIFO_EOP_RD               = 0x00000069,
    PH_PERF_SEL_SC0_PA6_EOPG_WE                        = 0x0000006a,
    PH_PERF_SEL_SC0_PA7_DATA_FIFO_RD                   = 0x0000006c,
    PH_PERF_SEL_SC0_PA7_DATA_FIFO_WE                   = 0x0000006d,
    PH_PERF_SEL_SC0_PA7_FIFO_EMPTY                     = 0x0000006e,
    PH_PERF_SEL_SC0_PA7_FIFO_FULL                      = 0x0000006f,
    PH_PERF_SEL_SC0_PA7_NULL_WE                        = 0x00000070,
    PH_PERF_SEL_SC0_PA7_EVENT_WE                       = 0x00000071,
    PH_PERF_SEL_SC0_PA7_FPOV_WE                        = 0x00000072,
    PH_PERF_SEL_SC0_PA7_EOP_WE                         = 0x00000074,
    PH_PERF_SEL_SC0_PA7_DATA_FIFO_EOP_RD               = 0x00000075,
    PH_PERF_SEL_SC0_PA7_EOPG_WE                        = 0x00000076,
    PH_PERF_SEL_SC1_SRPS_WINDOW_VALID                  = 0x00000078,
    PH_PERF_SEL_SC1_ARB_XFC_ALL_EVENT_OR_PRIM_CYCLES   = 0x00000079,
    PH_PERF_SEL_SC1_ARB_XFC_ONLY_PRIM_CYCLES           = 0x0000007a,
    PH_PERF_SEL_SC1_ARB_XFC_ONLY_ONE_INC_PER_PRIM      = 0x0000007b,
    PH_PERF_SEL_SC1_ARB_STALLED_FROM_BELOW             = 0x0000007c,
    PH_PERF_SEL_SC1_ARB_STARVED_FROM_ABOVE             = 0x0000007d,
    PH_PERF_SEL_SC1_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_NOT_EMPTY = 0x0000007e,
    PH_PERF_SEL_SC1_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_FULL = 0x0000007f,
    PH_PERF_SEL_SC1_ARB_BUSY                           = 0x00000080,
    PH_PERF_SEL_SC1_ARB_PA_BUSY_SOP                    = 0x00000081,
    PH_PERF_SEL_SC1_ARB_EOP_POP_SYNC_POP               = 0x00000082,
    PH_PERF_SEL_SC1_ARB_EVENT_SYNC_POP                 = 0x00000083,
    PH_PERF_SEL_SC1_PS_ENG_MULTICYCLE_BUBBLE           = 0x00000084,
    PH_PERF_SEL_SC1_EOP_SYNC_WINDOW                    = 0x00000085,
    PH_PERF_SEL_SC1_BUSY_PROCESSING_MULTICYCLE_PRIM    = 0x00000086,
    PH_PERF_SEL_SC1_BUSY_CNT_NOT_ZERO                  = 0x00000087,
    PH_PERF_SEL_SC1_SEND                               = 0x00000088,
    PH_PERF_SEL_SC1_CREDIT_AT_ZERO_WITH_PENDING_SEND   = 0x00000089,
    PH_PERF_SEL_SC1_CREDIT_AT_MAX                      = 0x0000008a,
    PH_PERF_SEL_SC1_CREDIT_AT_MAX_NO_PENDING_SEND      = 0x0000008b,
    PH_PERF_SEL_SC1_PA0_DATA_FIFO_RD                   = 0x00000090,
    PH_PERF_SEL_SC1_PA0_DATA_FIFO_WE                   = 0x00000091,
    PH_PERF_SEL_SC1_PA0_FIFO_EMPTY                     = 0x00000092,
    PH_PERF_SEL_SC1_PA0_FIFO_FULL                      = 0x00000093,
    PH_PERF_SEL_SC1_PA0_NULL_WE                        = 0x00000094,
    PH_PERF_SEL_SC1_PA0_EVENT_WE                       = 0x00000095,
    PH_PERF_SEL_SC1_PA0_FPOV_WE                        = 0x00000096,
    PH_PERF_SEL_SC1_PA0_EOP_WE                         = 0x00000098,
    PH_PERF_SEL_SC1_PA0_DATA_FIFO_EOP_RD               = 0x00000099,
    PH_PERF_SEL_SC1_PA0_EOPG_WE                        = 0x0000009a,
    PH_PERF_SEL_SC1_PA1_DATA_FIFO_RD                   = 0x0000009c,
    PH_PERF_SEL_SC1_PA1_DATA_FIFO_WE                   = 0x0000009d,
    PH_PERF_SEL_SC1_PA1_FIFO_EMPTY                     = 0x0000009e,
    PH_PERF_SEL_SC1_PA1_FIFO_FULL                      = 0x0000009f,
    PH_PERF_SEL_SC1_PA1_NULL_WE                        = 0x000000a0,
    PH_PERF_SEL_SC1_PA1_EVENT_WE                       = 0x000000a1,
    PH_PERF_SEL_SC1_PA1_FPOV_WE                        = 0x000000a2,
    PH_PERF_SEL_SC1_PA1_EOP_WE                         = 0x000000a4,
    PH_PERF_SEL_SC1_PA1_DATA_FIFO_EOP_RD               = 0x000000a5,
    PH_PERF_SEL_SC1_PA1_EOPG_WE                        = 0x000000a6,
    PH_PERF_SEL_SC1_PA2_DATA_FIFO_RD                   = 0x000000a8,
    PH_PERF_SEL_SC1_PA2_DATA_FIFO_WE                   = 0x000000a9,
    PH_PERF_SEL_SC1_PA2_FIFO_EMPTY                     = 0x000000aa,
    PH_PERF_SEL_SC1_PA2_FIFO_FULL                      = 0x000000ab,
    PH_PERF_SEL_SC1_PA2_NULL_WE                        = 0x000000ac,
    PH_PERF_SEL_SC1_PA2_EVENT_WE                       = 0x000000ad,
    PH_PERF_SEL_SC1_PA2_FPOV_WE                        = 0x000000ae,
    PH_PERF_SEL_SC1_PA2_EOP_WE                         = 0x000000b0,
    PH_PERF_SEL_SC1_PA2_DATA_FIFO_EOP_RD               = 0x000000b1,
    PH_PERF_SEL_SC1_PA2_EOPG_WE                        = 0x000000b2,
    PH_PERF_SEL_SC1_PA3_DATA_FIFO_RD                   = 0x000000b4,
    PH_PERF_SEL_SC1_PA3_DATA_FIFO_WE                   = 0x000000b5,
    PH_PERF_SEL_SC1_PA3_FIFO_EMPTY                     = 0x000000b6,
    PH_PERF_SEL_SC1_PA3_FIFO_FULL                      = 0x000000b7,
    PH_PERF_SEL_SC1_PA3_NULL_WE                        = 0x000000b8,
    PH_PERF_SEL_SC1_PA3_EVENT_WE                       = 0x000000b9,
    PH_PERF_SEL_SC1_PA3_FPOV_WE                        = 0x000000ba,
    PH_PERF_SEL_SC1_PA3_EOP_WE                         = 0x000000bc,
    PH_PERF_SEL_SC1_PA3_DATA_FIFO_EOP_RD               = 0x000000bd,
    PH_PERF_SEL_SC1_PA3_EOPG_WE                        = 0x000000be,
    PH_PERF_SEL_SC1_PA4_DATA_FIFO_RD                   = 0x000000c0,
    PH_PERF_SEL_SC1_PA4_DATA_FIFO_WE                   = 0x000000c1,
    PH_PERF_SEL_SC1_PA4_FIFO_EMPTY                     = 0x000000c2,
    PH_PERF_SEL_SC1_PA4_FIFO_FULL                      = 0x000000c3,
    PH_PERF_SEL_SC1_PA4_NULL_WE                        = 0x000000c4,
    PH_PERF_SEL_SC1_PA4_EVENT_WE                       = 0x000000c5,
    PH_PERF_SEL_SC1_PA4_FPOV_WE                        = 0x000000c6,
    PH_PERF_SEL_SC1_PA4_EOP_WE                         = 0x000000c8,
    PH_PERF_SEL_SC1_PA4_DATA_FIFO_EOP_RD               = 0x000000c9,
    PH_PERF_SEL_SC1_PA4_EOPG_WE                        = 0x000000ca,
    PH_PERF_SEL_SC1_PA5_DATA_FIFO_RD                   = 0x000000cc,
    PH_PERF_SEL_SC1_PA5_DATA_FIFO_WE                   = 0x000000cd,
    PH_PERF_SEL_SC1_PA5_FIFO_EMPTY                     = 0x000000ce,
    PH_PERF_SEL_SC1_PA5_FIFO_FULL                      = 0x000000cf,
    PH_PERF_SEL_SC1_PA5_NULL_WE                        = 0x000000d0,
    PH_PERF_SEL_SC1_PA5_EVENT_WE                       = 0x000000d1,
    PH_PERF_SEL_SC1_PA5_FPOV_WE                        = 0x000000d2,
    PH_PERF_SEL_SC1_PA5_EOP_WE                         = 0x000000d4,
    PH_PERF_SEL_SC1_PA5_DATA_FIFO_EOP_RD               = 0x000000d5,
    PH_PERF_SEL_SC1_PA5_EOPG_WE                        = 0x000000d6,
    PH_PERF_SEL_SC1_PA6_DATA_FIFO_RD                   = 0x000000d8,
    PH_PERF_SEL_SC1_PA6_DATA_FIFO_WE                   = 0x000000d9,
    PH_PERF_SEL_SC1_PA6_FIFO_EMPTY                     = 0x000000da,
    PH_PERF_SEL_SC1_PA6_FIFO_FULL                      = 0x000000db,
    PH_PERF_SEL_SC1_PA6_NULL_WE                        = 0x000000dc,
    PH_PERF_SEL_SC1_PA6_EVENT_WE                       = 0x000000dd,
    PH_PERF_SEL_SC1_PA6_FPOV_WE                        = 0x000000de,
    PH_PERF_SEL_SC1_PA6_EOP_WE                         = 0x000000e0,
    PH_PERF_SEL_SC1_PA6_DATA_FIFO_EOP_RD               = 0x000000e1,
    PH_PERF_SEL_SC1_PA6_EOPG_WE                        = 0x000000e2,
    PH_PERF_SEL_SC1_PA7_DATA_FIFO_RD                   = 0x000000e4,
    PH_PERF_SEL_SC1_PA7_DATA_FIFO_WE                   = 0x000000e5,
    PH_PERF_SEL_SC1_PA7_FIFO_EMPTY                     = 0x000000e6,
    PH_PERF_SEL_SC1_PA7_FIFO_FULL                      = 0x000000e7,
    PH_PERF_SEL_SC1_PA7_NULL_WE                        = 0x000000e8,
    PH_PERF_SEL_SC1_PA7_EVENT_WE                       = 0x000000e9,
    PH_PERF_SEL_SC1_PA7_FPOV_WE                        = 0x000000ea,
    PH_PERF_SEL_SC1_PA7_EOP_WE                         = 0x000000ec,
    PH_PERF_SEL_SC1_PA7_DATA_FIFO_EOP_RD               = 0x000000ed,
    PH_PERF_SEL_SC1_PA7_EOPG_WE                        = 0x000000ee,
    PH_PERF_SEL_SC2_SRPS_WINDOW_VALID                  = 0x000000f0,
    PH_PERF_SEL_SC2_ARB_XFC_ALL_EVENT_OR_PRIM_CYCLES   = 0x000000f1,
    PH_PERF_SEL_SC2_ARB_XFC_ONLY_PRIM_CYCLES           = 0x000000f2,
    PH_PERF_SEL_SC2_ARB_XFC_ONLY_ONE_INC_PER_PRIM      = 0x000000f3,
    PH_PERF_SEL_SC2_ARB_STALLED_FROM_BELOW             = 0x000000f4,
    PH_PERF_SEL_SC2_ARB_STARVED_FROM_ABOVE             = 0x000000f5,
    PH_PERF_SEL_SC2_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_NOT_EMPTY = 0x000000f6,
    PH_PERF_SEL_SC2_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_FULL = 0x000000f7,
    PH_PERF_SEL_SC2_ARB_BUSY                           = 0x000000f8,
    PH_PERF_SEL_SC2_ARB_PA_BUSY_SOP                    = 0x000000f9,
    PH_PERF_SEL_SC2_ARB_EOP_POP_SYNC_POP               = 0x000000fa,
    PH_PERF_SEL_SC2_ARB_EVENT_SYNC_POP                 = 0x000000fb,
    PH_PERF_SEL_SC2_PS_ENG_MULTICYCLE_BUBBLE           = 0x000000fc,
    PH_PERF_SEL_SC2_EOP_SYNC_WINDOW                    = 0x000000fd,
    PH_PERF_SEL_SC2_BUSY_PROCESSING_MULTICYCLE_PRIM    = 0x000000fe,
    PH_PERF_SEL_SC2_BUSY_CNT_NOT_ZERO                  = 0x000000ff,
    PH_PERF_SEL_SC2_SEND                               = 0x00000100,
    PH_PERF_SEL_SC2_CREDIT_AT_ZERO_WITH_PENDING_SEND   = 0x00000101,
    PH_PERF_SEL_SC2_CREDIT_AT_MAX                      = 0x00000102,
    PH_PERF_SEL_SC2_CREDIT_AT_MAX_NO_PENDING_SEND      = 0x00000103,
    PH_PERF_SEL_SC2_PA0_DATA_FIFO_RD                   = 0x00000108,
    PH_PERF_SEL_SC2_PA0_DATA_FIFO_WE                   = 0x00000109,
    PH_PERF_SEL_SC2_PA0_FIFO_EMPTY                     = 0x0000010a,
    PH_PERF_SEL_SC2_PA0_FIFO_FULL                      = 0x0000010b,
    PH_PERF_SEL_SC2_PA0_NULL_WE                        = 0x0000010c,
    PH_PERF_SEL_SC2_PA0_EVENT_WE                       = 0x0000010d,
    PH_PERF_SEL_SC2_PA0_FPOV_WE                        = 0x0000010e,
    PH_PERF_SEL_SC2_PA0_EOP_WE                         = 0x00000110,
    PH_PERF_SEL_SC2_PA0_DATA_FIFO_EOP_RD               = 0x00000111,
    PH_PERF_SEL_SC2_PA0_EOPG_WE                        = 0x00000112,
    PH_PERF_SEL_SC2_PA1_DATA_FIFO_RD                   = 0x00000114,
    PH_PERF_SEL_SC2_PA1_DATA_FIFO_WE                   = 0x00000115,
    PH_PERF_SEL_SC2_PA1_FIFO_EMPTY                     = 0x00000116,
    PH_PERF_SEL_SC2_PA1_FIFO_FULL                      = 0x00000117,
    PH_PERF_SEL_SC2_PA1_NULL_WE                        = 0x00000118,
    PH_PERF_SEL_SC2_PA1_EVENT_WE                       = 0x00000119,
    PH_PERF_SEL_SC2_PA1_FPOV_WE                        = 0x0000011a,
    PH_PERF_SEL_SC2_PA1_EOP_WE                         = 0x0000011c,
    PH_PERF_SEL_SC2_PA1_DATA_FIFO_EOP_RD               = 0x0000011d,
    PH_PERF_SEL_SC2_PA1_EOPG_WE                        = 0x0000011e,
    PH_PERF_SEL_SC2_PA2_DATA_FIFO_RD                   = 0x00000120,
    PH_PERF_SEL_SC2_PA2_DATA_FIFO_WE                   = 0x00000121,
    PH_PERF_SEL_SC2_PA2_FIFO_EMPTY                     = 0x00000122,
    PH_PERF_SEL_SC2_PA2_FIFO_FULL                      = 0x00000123,
    PH_PERF_SEL_SC2_PA2_NULL_WE                        = 0x00000124,
    PH_PERF_SEL_SC2_PA2_EVENT_WE                       = 0x00000125,
    PH_PERF_SEL_SC2_PA2_FPOV_WE                        = 0x00000126,
    PH_PERF_SEL_SC2_PA2_EOP_WE                         = 0x00000128,
    PH_PERF_SEL_SC2_PA2_DATA_FIFO_EOP_RD               = 0x00000129,
    PH_PERF_SEL_SC2_PA2_EOPG_WE                        = 0x0000012a,
    PH_PERF_SEL_SC2_PA3_DATA_FIFO_RD                   = 0x0000012c,
    PH_PERF_SEL_SC2_PA3_DATA_FIFO_WE                   = 0x0000012d,
    PH_PERF_SEL_SC2_PA3_FIFO_EMPTY                     = 0x0000012e,
    PH_PERF_SEL_SC2_PA3_FIFO_FULL                      = 0x0000012f,
    PH_PERF_SEL_SC2_PA3_NULL_WE                        = 0x00000130,
    PH_PERF_SEL_SC2_PA3_EVENT_WE                       = 0x00000131,
    PH_PERF_SEL_SC2_PA3_FPOV_WE                        = 0x00000132,
    PH_PERF_SEL_SC2_PA3_EOP_WE                         = 0x00000134,
    PH_PERF_SEL_SC2_PA3_DATA_FIFO_EOP_RD               = 0x00000135,
    PH_PERF_SEL_SC2_PA3_EOPG_WE                        = 0x00000136,
    PH_PERF_SEL_SC2_PA4_DATA_FIFO_RD                   = 0x00000138,
    PH_PERF_SEL_SC2_PA4_DATA_FIFO_WE                   = 0x00000139,
    PH_PERF_SEL_SC2_PA4_FIFO_EMPTY                     = 0x0000013a,
    PH_PERF_SEL_SC2_PA4_FIFO_FULL                      = 0x0000013b,
    PH_PERF_SEL_SC2_PA4_NULL_WE                        = 0x0000013c,
    PH_PERF_SEL_SC2_PA4_EVENT_WE                       = 0x0000013d,
    PH_PERF_SEL_SC2_PA4_FPOV_WE                        = 0x0000013e,
    PH_PERF_SEL_SC2_PA4_EOP_WE                         = 0x00000140,
    PH_PERF_SEL_SC2_PA4_DATA_FIFO_EOP_RD               = 0x00000141,
    PH_PERF_SEL_SC2_PA4_EOPG_WE                        = 0x00000142,
    PH_PERF_SEL_SC2_PA5_DATA_FIFO_RD                   = 0x00000144,
    PH_PERF_SEL_SC2_PA5_DATA_FIFO_WE                   = 0x00000145,
    PH_PERF_SEL_SC2_PA5_FIFO_EMPTY                     = 0x00000146,
    PH_PERF_SEL_SC2_PA5_FIFO_FULL                      = 0x00000147,
    PH_PERF_SEL_SC2_PA5_NULL_WE                        = 0x00000148,
    PH_PERF_SEL_SC2_PA5_EVENT_WE                       = 0x00000149,
    PH_PERF_SEL_SC2_PA5_FPOV_WE                        = 0x0000014a,
    PH_PERF_SEL_SC2_PA5_EOP_WE                         = 0x0000014c,
    PH_PERF_SEL_SC2_PA5_DATA_FIFO_EOP_RD               = 0x0000014d,
    PH_PERF_SEL_SC2_PA5_EOPG_WE                        = 0x0000014e,
    PH_PERF_SEL_SC2_PA6_DATA_FIFO_RD                   = 0x00000150,
    PH_PERF_SEL_SC2_PA6_DATA_FIFO_WE                   = 0x00000151,
    PH_PERF_SEL_SC2_PA6_FIFO_EMPTY                     = 0x00000152,
    PH_PERF_SEL_SC2_PA6_FIFO_FULL                      = 0x00000153,
    PH_PERF_SEL_SC2_PA6_NULL_WE                        = 0x00000154,
    PH_PERF_SEL_SC2_PA6_EVENT_WE                       = 0x00000155,
    PH_PERF_SEL_SC2_PA6_FPOV_WE                        = 0x00000156,
    PH_PERF_SEL_SC2_PA6_EOP_WE                         = 0x00000158,
    PH_PERF_SEL_SC2_PA6_DATA_FIFO_EOP_RD               = 0x00000159,
    PH_PERF_SEL_SC2_PA6_EOPG_WE                        = 0x0000015a,
    PH_PERF_SEL_SC2_PA7_DATA_FIFO_RD                   = 0x0000015c,
    PH_PERF_SEL_SC2_PA7_DATA_FIFO_WE                   = 0x0000015d,
    PH_PERF_SEL_SC2_PA7_FIFO_EMPTY                     = 0x0000015e,
    PH_PERF_SEL_SC2_PA7_FIFO_FULL                      = 0x0000015f,
    PH_PERF_SEL_SC2_PA7_NULL_WE                        = 0x00000160,
    PH_PERF_SEL_SC2_PA7_EVENT_WE                       = 0x00000161,
    PH_PERF_SEL_SC2_PA7_FPOV_WE                        = 0x00000162,
    PH_PERF_SEL_SC2_PA7_EOP_WE                         = 0x00000164,
    PH_PERF_SEL_SC2_PA7_DATA_FIFO_EOP_RD               = 0x00000165,
    PH_PERF_SEL_SC2_PA7_EOPG_WE                        = 0x00000166,
    PH_PERF_SEL_SC3_SRPS_WINDOW_VALID                  = 0x00000168,
    PH_PERF_SEL_SC3_ARB_XFC_ALL_EVENT_OR_PRIM_CYCLES   = 0x00000169,
    PH_PERF_SEL_SC3_ARB_XFC_ONLY_PRIM_CYCLES           = 0x0000016a,
    PH_PERF_SEL_SC3_ARB_XFC_ONLY_ONE_INC_PER_PRIM      = 0x0000016b,
    PH_PERF_SEL_SC3_ARB_STALLED_FROM_BELOW             = 0x0000016c,
    PH_PERF_SEL_SC3_ARB_STARVED_FROM_ABOVE             = 0x0000016d,
    PH_PERF_SEL_SC3_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_NOT_EMPTY = 0x0000016e,
    PH_PERF_SEL_SC3_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_FULL = 0x0000016f,
    PH_PERF_SEL_SC3_ARB_BUSY                           = 0x00000170,
    PH_PERF_SEL_SC3_ARB_PA_BUSY_SOP                    = 0x00000171,
    PH_PERF_SEL_SC3_ARB_EOP_POP_SYNC_POP               = 0x00000172,
    PH_PERF_SEL_SC3_ARB_EVENT_SYNC_POP                 = 0x00000173,
    PH_PERF_SEL_SC3_PS_ENG_MULTICYCLE_BUBBLE           = 0x00000174,
    PH_PERF_SEL_SC3_EOP_SYNC_WINDOW                    = 0x00000175,
    PH_PERF_SEL_SC3_BUSY_PROCESSING_MULTICYCLE_PRIM    = 0x00000176,
    PH_PERF_SEL_SC3_BUSY_CNT_NOT_ZERO                  = 0x00000177,
    PH_PERF_SEL_SC3_SEND                               = 0x00000178,
    PH_PERF_SEL_SC3_CREDIT_AT_ZERO_WITH_PENDING_SEND   = 0x00000179,
    PH_PERF_SEL_SC3_CREDIT_AT_MAX                      = 0x0000017a,
    PH_PERF_SEL_SC3_CREDIT_AT_MAX_NO_PENDING_SEND      = 0x0000017b,
    PH_PERF_SEL_SC3_PA0_DATA_FIFO_RD                   = 0x00000180,
    PH_PERF_SEL_SC3_PA0_DATA_FIFO_WE                   = 0x00000181,
    PH_PERF_SEL_SC3_PA0_FIFO_EMPTY                     = 0x00000182,
    PH_PERF_SEL_SC3_PA0_FIFO_FULL                      = 0x00000183,
    PH_PERF_SEL_SC3_PA0_NULL_WE                        = 0x00000184,
    PH_PERF_SEL_SC3_PA0_EVENT_WE                       = 0x00000185,
    PH_PERF_SEL_SC3_PA0_FPOV_WE                        = 0x00000186,
    PH_PERF_SEL_SC3_PA0_EOP_WE                         = 0x00000188,
    PH_PERF_SEL_SC3_PA0_DATA_FIFO_EOP_RD               = 0x00000189,
    PH_PERF_SEL_SC3_PA0_EOPG_WE                        = 0x0000018a,
    PH_PERF_SEL_SC3_PA1_DATA_FIFO_RD                   = 0x0000018c,
    PH_PERF_SEL_SC3_PA1_DATA_FIFO_WE                   = 0x0000018d,
    PH_PERF_SEL_SC3_PA1_FIFO_EMPTY                     = 0x0000018e,
    PH_PERF_SEL_SC3_PA1_FIFO_FULL                      = 0x0000018f,
    PH_PERF_SEL_SC3_PA1_NULL_WE                        = 0x00000190,
    PH_PERF_SEL_SC3_PA1_EVENT_WE                       = 0x00000191,
    PH_PERF_SEL_SC3_PA1_FPOV_WE                        = 0x00000192,
    PH_PERF_SEL_SC3_PA1_EOP_WE                         = 0x00000194,
    PH_PERF_SEL_SC3_PA1_DATA_FIFO_EOP_RD               = 0x00000195,
    PH_PERF_SEL_SC3_PA1_EOPG_WE                        = 0x00000196,
    PH_PERF_SEL_SC3_PA2_DATA_FIFO_RD                   = 0x00000198,
    PH_PERF_SEL_SC3_PA2_DATA_FIFO_WE                   = 0x00000199,
    PH_PERF_SEL_SC3_PA2_FIFO_EMPTY                     = 0x0000019a,
    PH_PERF_SEL_SC3_PA2_FIFO_FULL                      = 0x0000019b,
    PH_PERF_SEL_SC3_PA2_NULL_WE                        = 0x0000019c,
    PH_PERF_SEL_SC3_PA2_EVENT_WE                       = 0x0000019d,
    PH_PERF_SEL_SC3_PA2_FPOV_WE                        = 0x0000019e,
    PH_PERF_SEL_SC3_PA2_EOP_WE                         = 0x000001a0,
    PH_PERF_SEL_SC3_PA2_DATA_FIFO_EOP_RD               = 0x000001a1,
    PH_PERF_SEL_SC3_PA2_EOPG_WE                        = 0x000001a2,
    PH_PERF_SEL_SC3_PA3_DATA_FIFO_RD                   = 0x000001a4,
    PH_PERF_SEL_SC3_PA3_DATA_FIFO_WE                   = 0x000001a5,
    PH_PERF_SEL_SC3_PA3_FIFO_EMPTY                     = 0x000001a6,
    PH_PERF_SEL_SC3_PA3_FIFO_FULL                      = 0x000001a7,
    PH_PERF_SEL_SC3_PA3_NULL_WE                        = 0x000001a8,
    PH_PERF_SEL_SC3_PA3_EVENT_WE                       = 0x000001a9,
    PH_PERF_SEL_SC3_PA3_FPOV_WE                        = 0x000001aa,
    PH_PERF_SEL_SC3_PA3_EOP_WE                         = 0x000001ac,
    PH_PERF_SEL_SC3_PA3_DATA_FIFO_EOP_RD               = 0x000001ad,
    PH_PERF_SEL_SC3_PA3_EOPG_WE                        = 0x000001ae,
    PH_PERF_SEL_SC3_PA4_DATA_FIFO_RD                   = 0x000001b0,
    PH_PERF_SEL_SC3_PA4_DATA_FIFO_WE                   = 0x000001b1,
    PH_PERF_SEL_SC3_PA4_FIFO_EMPTY                     = 0x000001b2,
    PH_PERF_SEL_SC3_PA4_FIFO_FULL                      = 0x000001b3,
    PH_PERF_SEL_SC3_PA4_NULL_WE                        = 0x000001b4,
    PH_PERF_SEL_SC3_PA4_EVENT_WE                       = 0x000001b5,
    PH_PERF_SEL_SC3_PA4_FPOV_WE                        = 0x000001b6,
    PH_PERF_SEL_SC3_PA4_EOP_WE                         = 0x000001b8,
    PH_PERF_SEL_SC3_PA4_DATA_FIFO_EOP_RD               = 0x000001b9,
    PH_PERF_SEL_SC3_PA4_EOPG_WE                        = 0x000001ba,
    PH_PERF_SEL_SC3_PA5_DATA_FIFO_RD                   = 0x000001bc,
    PH_PERF_SEL_SC3_PA5_DATA_FIFO_WE                   = 0x000001bd,
    PH_PERF_SEL_SC3_PA5_FIFO_EMPTY                     = 0x000001be,
    PH_PERF_SEL_SC3_PA5_FIFO_FULL                      = 0x000001bf,
    PH_PERF_SEL_SC3_PA5_NULL_WE                        = 0x000001c0,
    PH_PERF_SEL_SC3_PA5_EVENT_WE                       = 0x000001c1,
    PH_PERF_SEL_SC3_PA5_FPOV_WE                        = 0x000001c2,
    PH_PERF_SEL_SC3_PA5_EOP_WE                         = 0x000001c4,
    PH_PERF_SEL_SC3_PA5_DATA_FIFO_EOP_RD               = 0x000001c5,
    PH_PERF_SEL_SC3_PA5_EOPG_WE                        = 0x000001c6,
    PH_PERF_SEL_SC3_PA6_DATA_FIFO_RD                   = 0x000001c8,
    PH_PERF_SEL_SC3_PA6_DATA_FIFO_WE                   = 0x000001c9,
    PH_PERF_SEL_SC3_PA6_FIFO_EMPTY                     = 0x000001ca,
    PH_PERF_SEL_SC3_PA6_FIFO_FULL                      = 0x000001cb,
    PH_PERF_SEL_SC3_PA6_NULL_WE                        = 0x000001cc,
    PH_PERF_SEL_SC3_PA6_EVENT_WE                       = 0x000001cd,
    PH_PERF_SEL_SC3_PA6_FPOV_WE                        = 0x000001ce,
    PH_PERF_SEL_SC3_PA6_EOP_WE                         = 0x000001d0,
    PH_PERF_SEL_SC3_PA6_DATA_FIFO_EOP_RD               = 0x000001d1,
    PH_PERF_SEL_SC3_PA6_EOPG_WE                        = 0x000001d2,
    PH_PERF_SEL_SC3_PA7_DATA_FIFO_RD                   = 0x000001d4,
    PH_PERF_SEL_SC3_PA7_DATA_FIFO_WE                   = 0x000001d5,
    PH_PERF_SEL_SC3_PA7_FIFO_EMPTY                     = 0x000001d6,
    PH_PERF_SEL_SC3_PA7_FIFO_FULL                      = 0x000001d7,
    PH_PERF_SEL_SC3_PA7_NULL_WE                        = 0x000001d8,
    PH_PERF_SEL_SC3_PA7_EVENT_WE                       = 0x000001d9,
    PH_PERF_SEL_SC3_PA7_FPOV_WE                        = 0x000001da,
    PH_PERF_SEL_SC3_PA7_EOP_WE                         = 0x000001dc,
    PH_PERF_SEL_SC3_PA7_DATA_FIFO_EOP_RD               = 0x000001dd,
    PH_PERF_SEL_SC3_PA7_EOPG_WE                        = 0x000001de,
    PH_PERF_SEL_SC4_SRPS_WINDOW_VALID                  = 0x000001e0,
    PH_PERF_SEL_SC4_ARB_XFC_ALL_EVENT_OR_PRIM_CYCLES   = 0x000001e1,
    PH_PERF_SEL_SC4_ARB_XFC_ONLY_PRIM_CYCLES           = 0x000001e2,
    PH_PERF_SEL_SC4_ARB_XFC_ONLY_ONE_INC_PER_PRIM      = 0x000001e3,
    PH_PERF_SEL_SC4_ARB_STALLED_FROM_BELOW             = 0x000001e4,
    PH_PERF_SEL_SC4_ARB_STARVED_FROM_ABOVE             = 0x000001e5,
    PH_PERF_SEL_SC4_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_NOT_EMPTY = 0x000001e6,
    PH_PERF_SEL_SC4_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_FULL = 0x000001e7,
    PH_PERF_SEL_SC4_ARB_BUSY                           = 0x000001e8,
    PH_PERF_SEL_SC4_ARB_PA_BUSY_SOP                    = 0x000001e9,
    PH_PERF_SEL_SC4_ARB_EOP_POP_SYNC_POP               = 0x000001ea,
    PH_PERF_SEL_SC4_ARB_EVENT_SYNC_POP                 = 0x000001eb,
    PH_PERF_SEL_SC4_PS_ENG_MULTICYCLE_BUBBLE           = 0x000001ec,
    PH_PERF_SEL_SC4_EOP_SYNC_WINDOW                    = 0x000001ed,
    PH_PERF_SEL_SC4_BUSY_PROCESSING_MULTICYCLE_PRIM    = 0x000001ee,
    PH_PERF_SEL_SC4_BUSY_CNT_NOT_ZERO                  = 0x000001ef,
    PH_PERF_SEL_SC4_SEND                               = 0x000001f0,
    PH_PERF_SEL_SC4_CREDIT_AT_ZERO_WITH_PENDING_SEND   = 0x000001f1,
    PH_PERF_SEL_SC4_CREDIT_AT_MAX                      = 0x000001f2,
    PH_PERF_SEL_SC4_CREDIT_AT_MAX_NO_PENDING_SEND      = 0x000001f3,
    PH_PERF_SEL_SC4_PA0_DATA_FIFO_RD                   = 0x000001f8,
    PH_PERF_SEL_SC4_PA0_DATA_FIFO_WE                   = 0x000001f9,
    PH_PERF_SEL_SC4_PA0_FIFO_EMPTY                     = 0x000001fa,
    PH_PERF_SEL_SC4_PA0_FIFO_FULL                      = 0x000001fb,
    PH_PERF_SEL_SC4_PA0_NULL_WE                        = 0x000001fc,
    PH_PERF_SEL_SC4_PA0_EVENT_WE                       = 0x000001fd,
    PH_PERF_SEL_SC4_PA0_FPOV_WE                        = 0x000001fe,
    PH_PERF_SEL_SC4_PA0_EOP_WE                         = 0x00000200,
    PH_PERF_SEL_SC4_PA0_DATA_FIFO_EOP_RD               = 0x00000201,
    PH_PERF_SEL_SC4_PA0_EOPG_WE                        = 0x00000202,
    PH_PERF_SEL_SC4_PA1_DATA_FIFO_RD                   = 0x00000204,
    PH_PERF_SEL_SC4_PA1_DATA_FIFO_WE                   = 0x00000205,
    PH_PERF_SEL_SC4_PA1_FIFO_EMPTY                     = 0x00000206,
    PH_PERF_SEL_SC4_PA1_FIFO_FULL                      = 0x00000207,
    PH_PERF_SEL_SC4_PA1_NULL_WE                        = 0x00000208,
    PH_PERF_SEL_SC4_PA1_EVENT_WE                       = 0x00000209,
    PH_PERF_SEL_SC4_PA1_FPOV_WE                        = 0x0000020a,
    PH_PERF_SEL_SC4_PA1_EOP_WE                         = 0x0000020c,
    PH_PERF_SEL_SC4_PA1_DATA_FIFO_EOP_RD               = 0x0000020d,
    PH_PERF_SEL_SC4_PA1_EOPG_WE                        = 0x0000020e,
    PH_PERF_SEL_SC4_PA2_DATA_FIFO_RD                   = 0x00000210,
    PH_PERF_SEL_SC4_PA2_DATA_FIFO_WE                   = 0x00000211,
    PH_PERF_SEL_SC4_PA2_FIFO_EMPTY                     = 0x00000212,
    PH_PERF_SEL_SC4_PA2_FIFO_FULL                      = 0x00000213,
    PH_PERF_SEL_SC4_PA2_NULL_WE                        = 0x00000214,
    PH_PERF_SEL_SC4_PA2_EVENT_WE                       = 0x00000215,
    PH_PERF_SEL_SC4_PA2_FPOV_WE                        = 0x00000216,
    PH_PERF_SEL_SC4_PA2_EOP_WE                         = 0x00000218,
    PH_PERF_SEL_SC4_PA2_DATA_FIFO_EOP_RD               = 0x00000219,
    PH_PERF_SEL_SC4_PA2_EOPG_WE                        = 0x0000021a,
    PH_PERF_SEL_SC4_PA3_DATA_FIFO_RD                   = 0x0000021c,
    PH_PERF_SEL_SC4_PA3_DATA_FIFO_WE                   = 0x0000021d,
    PH_PERF_SEL_SC4_PA3_FIFO_EMPTY                     = 0x0000021e,
    PH_PERF_SEL_SC4_PA3_FIFO_FULL                      = 0x0000021f,
    PH_PERF_SEL_SC4_PA3_NULL_WE                        = 0x00000220,
    PH_PERF_SEL_SC4_PA3_EVENT_WE                       = 0x00000221,
    PH_PERF_SEL_SC4_PA3_FPOV_WE                        = 0x00000222,
    PH_PERF_SEL_SC4_PA3_EOP_WE                         = 0x00000224,
    PH_PERF_SEL_SC4_PA3_DATA_FIFO_EOP_RD               = 0x00000225,
    PH_PERF_SEL_SC4_PA3_EOPG_WE                        = 0x00000226,
    PH_PERF_SEL_SC4_PA4_DATA_FIFO_RD                   = 0x00000228,
    PH_PERF_SEL_SC4_PA4_DATA_FIFO_WE                   = 0x00000229,
    PH_PERF_SEL_SC4_PA4_FIFO_EMPTY                     = 0x0000022a,
    PH_PERF_SEL_SC4_PA4_FIFO_FULL                      = 0x0000022b,
    PH_PERF_SEL_SC4_PA4_NULL_WE                        = 0x0000022c,
    PH_PERF_SEL_SC4_PA4_EVENT_WE                       = 0x0000022d,
    PH_PERF_SEL_SC4_PA4_FPOV_WE                        = 0x0000022e,
    PH_PERF_SEL_SC4_PA4_EOP_WE                         = 0x00000230,
    PH_PERF_SEL_SC4_PA4_DATA_FIFO_EOP_RD               = 0x00000231,
    PH_PERF_SEL_SC4_PA4_EOPG_WE                        = 0x00000232,
    PH_PERF_SEL_SC4_PA5_DATA_FIFO_RD                   = 0x00000234,
    PH_PERF_SEL_SC4_PA5_DATA_FIFO_WE                   = 0x00000235,
    PH_PERF_SEL_SC4_PA5_FIFO_EMPTY                     = 0x00000236,
    PH_PERF_SEL_SC4_PA5_FIFO_FULL                      = 0x00000237,
    PH_PERF_SEL_SC4_PA5_NULL_WE                        = 0x00000238,
    PH_PERF_SEL_SC4_PA5_EVENT_WE                       = 0x00000239,
    PH_PERF_SEL_SC4_PA5_FPOV_WE                        = 0x0000023a,
    PH_PERF_SEL_SC4_PA5_EOP_WE                         = 0x0000023c,
    PH_PERF_SEL_SC4_PA5_DATA_FIFO_EOP_RD               = 0x0000023d,
    PH_PERF_SEL_SC4_PA5_EOPG_WE                        = 0x0000023e,
    PH_PERF_SEL_SC4_PA6_DATA_FIFO_RD                   = 0x00000240,
    PH_PERF_SEL_SC4_PA6_DATA_FIFO_WE                   = 0x00000241,
    PH_PERF_SEL_SC4_PA6_FIFO_EMPTY                     = 0x00000242,
    PH_PERF_SEL_SC4_PA6_FIFO_FULL                      = 0x00000243,
    PH_PERF_SEL_SC4_PA6_NULL_WE                        = 0x00000244,
    PH_PERF_SEL_SC4_PA6_EVENT_WE                       = 0x00000245,
    PH_PERF_SEL_SC4_PA6_FPOV_WE                        = 0x00000246,
    PH_PERF_SEL_SC4_PA6_EOP_WE                         = 0x00000248,
    PH_PERF_SEL_SC4_PA6_DATA_FIFO_EOP_RD               = 0x00000249,
    PH_PERF_SEL_SC4_PA6_EOPG_WE                        = 0x0000024a,
    PH_PERF_SEL_SC4_PA7_DATA_FIFO_RD                   = 0x0000024c,
    PH_PERF_SEL_SC4_PA7_DATA_FIFO_WE                   = 0x0000024d,
    PH_PERF_SEL_SC4_PA7_FIFO_EMPTY                     = 0x0000024e,
    PH_PERF_SEL_SC4_PA7_FIFO_FULL                      = 0x0000024f,
    PH_PERF_SEL_SC4_PA7_NULL_WE                        = 0x00000250,
    PH_PERF_SEL_SC4_PA7_EVENT_WE                       = 0x00000251,
    PH_PERF_SEL_SC4_PA7_FPOV_WE                        = 0x00000252,
    PH_PERF_SEL_SC4_PA7_EOP_WE                         = 0x00000254,
    PH_PERF_SEL_SC4_PA7_DATA_FIFO_EOP_RD               = 0x00000255,
    PH_PERF_SEL_SC4_PA7_EOPG_WE                        = 0x00000256,
    PH_PERF_SEL_SC5_SRPS_WINDOW_VALID                  = 0x00000258,
    PH_PERF_SEL_SC5_ARB_XFC_ALL_EVENT_OR_PRIM_CYCLES   = 0x00000259,
    PH_PERF_SEL_SC5_ARB_XFC_ONLY_PRIM_CYCLES           = 0x0000025a,
    PH_PERF_SEL_SC5_ARB_XFC_ONLY_ONE_INC_PER_PRIM      = 0x0000025b,
    PH_PERF_SEL_SC5_ARB_STALLED_FROM_BELOW             = 0x0000025c,
    PH_PERF_SEL_SC5_ARB_STARVED_FROM_ABOVE             = 0x0000025d,
    PH_PERF_SEL_SC5_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_NOT_EMPTY = 0x0000025e,
    PH_PERF_SEL_SC5_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_FULL = 0x0000025f,
    PH_PERF_SEL_SC5_ARB_BUSY                           = 0x00000260,
    PH_PERF_SEL_SC5_ARB_PA_BUSY_SOP                    = 0x00000261,
    PH_PERF_SEL_SC5_ARB_EOP_POP_SYNC_POP               = 0x00000262,
    PH_PERF_SEL_SC5_ARB_EVENT_SYNC_POP                 = 0x00000263,
    PH_PERF_SEL_SC5_PS_ENG_MULTICYCLE_BUBBLE           = 0x00000264,
    PH_PERF_SEL_SC5_EOP_SYNC_WINDOW                    = 0x00000265,
    PH_PERF_SEL_SC5_BUSY_PROCESSING_MULTICYCLE_PRIM    = 0x00000266,
    PH_PERF_SEL_SC5_BUSY_CNT_NOT_ZERO                  = 0x00000267,
    PH_PERF_SEL_SC5_SEND                               = 0x00000268,
    PH_PERF_SEL_SC5_CREDIT_AT_ZERO_WITH_PENDING_SEND   = 0x00000269,
    PH_PERF_SEL_SC5_CREDIT_AT_MAX                      = 0x0000026a,
    PH_PERF_SEL_SC5_CREDIT_AT_MAX_NO_PENDING_SEND      = 0x0000026b,
    PH_PERF_SEL_SC5_PA0_DATA_FIFO_RD                   = 0x00000270,
    PH_PERF_SEL_SC5_PA0_DATA_FIFO_WE                   = 0x00000271,
    PH_PERF_SEL_SC5_PA0_FIFO_EMPTY                     = 0x00000272,
    PH_PERF_SEL_SC5_PA0_FIFO_FULL                      = 0x00000273,
    PH_PERF_SEL_SC5_PA0_NULL_WE                        = 0x00000274,
    PH_PERF_SEL_SC5_PA0_EVENT_WE                       = 0x00000275,
    PH_PERF_SEL_SC5_PA0_FPOV_WE                        = 0x00000276,
    PH_PERF_SEL_SC5_PA0_EOP_WE                         = 0x00000278,
    PH_PERF_SEL_SC5_PA0_DATA_FIFO_EOP_RD               = 0x00000279,
    PH_PERF_SEL_SC5_PA0_EOPG_WE                        = 0x0000027a,
    PH_PERF_SEL_SC5_PA1_DATA_FIFO_RD                   = 0x0000027c,
    PH_PERF_SEL_SC5_PA1_DATA_FIFO_WE                   = 0x0000027d,
    PH_PERF_SEL_SC5_PA1_FIFO_EMPTY                     = 0x0000027e,
    PH_PERF_SEL_SC5_PA1_FIFO_FULL                      = 0x0000027f,
    PH_PERF_SEL_SC5_PA1_NULL_WE                        = 0x00000280,
    PH_PERF_SEL_SC5_PA1_EVENT_WE                       = 0x00000281,
    PH_PERF_SEL_SC5_PA1_FPOV_WE                        = 0x00000282,
    PH_PERF_SEL_SC5_PA1_EOP_WE                         = 0x00000284,
    PH_PERF_SEL_SC5_PA1_DATA_FIFO_EOP_RD               = 0x00000285,
    PH_PERF_SEL_SC5_PA1_EOPG_WE                        = 0x00000286,
    PH_PERF_SEL_SC5_PA2_DATA_FIFO_RD                   = 0x00000288,
    PH_PERF_SEL_SC5_PA2_DATA_FIFO_WE                   = 0x00000289,
    PH_PERF_SEL_SC5_PA2_FIFO_EMPTY                     = 0x0000028a,
    PH_PERF_SEL_SC5_PA2_FIFO_FULL                      = 0x0000028b,
    PH_PERF_SEL_SC5_PA2_NULL_WE                        = 0x0000028c,
    PH_PERF_SEL_SC5_PA2_EVENT_WE                       = 0x0000028d,
    PH_PERF_SEL_SC5_PA2_FPOV_WE                        = 0x0000028e,
    PH_PERF_SEL_SC5_PA2_EOP_WE                         = 0x00000290,
    PH_PERF_SEL_SC5_PA2_DATA_FIFO_EOP_RD               = 0x00000291,
    PH_PERF_SEL_SC5_PA2_EOPG_WE                        = 0x00000292,
    PH_PERF_SEL_SC5_PA3_DATA_FIFO_RD                   = 0x00000294,
    PH_PERF_SEL_SC5_PA3_DATA_FIFO_WE                   = 0x00000295,
    PH_PERF_SEL_SC5_PA3_FIFO_EMPTY                     = 0x00000296,
    PH_PERF_SEL_SC5_PA3_FIFO_FULL                      = 0x00000297,
    PH_PERF_SEL_SC5_PA3_NULL_WE                        = 0x00000298,
    PH_PERF_SEL_SC5_PA3_EVENT_WE                       = 0x00000299,
    PH_PERF_SEL_SC5_PA3_FPOV_WE                        = 0x0000029a,
    PH_PERF_SEL_SC5_PA3_EOP_WE                         = 0x0000029c,
    PH_PERF_SEL_SC5_PA3_DATA_FIFO_EOP_RD               = 0x0000029d,
    PH_PERF_SEL_SC5_PA3_EOPG_WE                        = 0x0000029e,
    PH_PERF_SEL_SC5_PA4_DATA_FIFO_RD                   = 0x000002a0,
    PH_PERF_SEL_SC5_PA4_DATA_FIFO_WE                   = 0x000002a1,
    PH_PERF_SEL_SC5_PA4_FIFO_EMPTY                     = 0x000002a2,
    PH_PERF_SEL_SC5_PA4_FIFO_FULL                      = 0x000002a3,
    PH_PERF_SEL_SC5_PA4_NULL_WE                        = 0x000002a4,
    PH_PERF_SEL_SC5_PA4_EVENT_WE                       = 0x000002a5,
    PH_PERF_SEL_SC5_PA4_FPOV_WE                        = 0x000002a6,
    PH_PERF_SEL_SC5_PA4_EOP_WE                         = 0x000002a8,
    PH_PERF_SEL_SC5_PA4_DATA_FIFO_EOP_RD               = 0x000002a9,
    PH_PERF_SEL_SC5_PA4_EOPG_WE                        = 0x000002aa,
    PH_PERF_SEL_SC5_PA5_DATA_FIFO_RD                   = 0x000002ac,
    PH_PERF_SEL_SC5_PA5_DATA_FIFO_WE                   = 0x000002ad,
    PH_PERF_SEL_SC5_PA5_FIFO_EMPTY                     = 0x000002ae,
    PH_PERF_SEL_SC5_PA5_FIFO_FULL                      = 0x000002af,
    PH_PERF_SEL_SC5_PA5_NULL_WE                        = 0x000002b0,
    PH_PERF_SEL_SC5_PA5_EVENT_WE                       = 0x000002b1,
    PH_PERF_SEL_SC5_PA5_FPOV_WE                        = 0x000002b2,
    PH_PERF_SEL_SC5_PA5_EOP_WE                         = 0x000002b4,
    PH_PERF_SEL_SC5_PA5_DATA_FIFO_EOP_RD               = 0x000002b5,
    PH_PERF_SEL_SC5_PA5_EOPG_WE                        = 0x000002b6,
    PH_PERF_SEL_SC5_PA6_DATA_FIFO_RD                   = 0x000002b8,
    PH_PERF_SEL_SC5_PA6_DATA_FIFO_WE                   = 0x000002b9,
    PH_PERF_SEL_SC5_PA6_FIFO_EMPTY                     = 0x000002ba,
    PH_PERF_SEL_SC5_PA6_FIFO_FULL                      = 0x000002bb,
    PH_PERF_SEL_SC5_PA6_NULL_WE                        = 0x000002bc,
    PH_PERF_SEL_SC5_PA6_EVENT_WE                       = 0x000002bd,
    PH_PERF_SEL_SC5_PA6_FPOV_WE                        = 0x000002be,
    PH_PERF_SEL_SC5_PA6_EOP_WE                         = 0x000002c0,
    PH_PERF_SEL_SC5_PA6_DATA_FIFO_EOP_RD               = 0x000002c1,
    PH_PERF_SEL_SC5_PA6_EOPG_WE                        = 0x000002c2,
    PH_PERF_SEL_SC5_PA7_DATA_FIFO_RD                   = 0x000002c4,
    PH_PERF_SEL_SC5_PA7_DATA_FIFO_WE                   = 0x000002c5,
    PH_PERF_SEL_SC5_PA7_FIFO_EMPTY                     = 0x000002c6,
    PH_PERF_SEL_SC5_PA7_FIFO_FULL                      = 0x000002c7,
    PH_PERF_SEL_SC5_PA7_NULL_WE                        = 0x000002c8,
    PH_PERF_SEL_SC5_PA7_EVENT_WE                       = 0x000002c9,
    PH_PERF_SEL_SC5_PA7_FPOV_WE                        = 0x000002ca,
    PH_PERF_SEL_SC5_PA7_EOP_WE                         = 0x000002cc,
    PH_PERF_SEL_SC5_PA7_DATA_FIFO_EOP_RD               = 0x000002cd,
    PH_PERF_SEL_SC5_PA7_EOPG_WE                        = 0x000002ce,
    PH_PERF_SEL_SC6_SRPS_WINDOW_VALID                  = 0x000002d0,
    PH_PERF_SEL_SC6_ARB_XFC_ALL_EVENT_OR_PRIM_CYCLES   = 0x000002d1,
    PH_PERF_SEL_SC6_ARB_XFC_ONLY_PRIM_CYCLES           = 0x000002d2,
    PH_PERF_SEL_SC6_ARB_XFC_ONLY_ONE_INC_PER_PRIM      = 0x000002d3,
    PH_PERF_SEL_SC6_ARB_STALLED_FROM_BELOW             = 0x000002d4,
    PH_PERF_SEL_SC6_ARB_STARVED_FROM_ABOVE             = 0x000002d5,
    PH_PERF_SEL_SC6_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_NOT_EMPTY = 0x000002d6,
    PH_PERF_SEL_SC6_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_FULL = 0x000002d7,
    PH_PERF_SEL_SC6_ARB_BUSY                           = 0x000002d8,
    PH_PERF_SEL_SC6_ARB_PA_BUSY_SOP                    = 0x000002d9,
    PH_PERF_SEL_SC6_ARB_EOP_POP_SYNC_POP               = 0x000002da,
    PH_PERF_SEL_SC6_ARB_EVENT_SYNC_POP                 = 0x000002db,
    PH_PERF_SEL_SC6_PS_ENG_MULTICYCLE_BUBBLE           = 0x000002dc,
    PH_PERF_SEL_SC6_EOP_SYNC_WINDOW                    = 0x000002dd,
    PH_PERF_SEL_SC6_BUSY_PROCESSING_MULTICYCLE_PRIM    = 0x000002de,
    PH_PERF_SEL_SC6_BUSY_CNT_NOT_ZERO                  = 0x000002df,
    PH_PERF_SEL_SC6_SEND                               = 0x000002e0,
    PH_PERF_SEL_SC6_CREDIT_AT_ZERO_WITH_PENDING_SEND   = 0x000002e1,
    PH_PERF_SEL_SC6_CREDIT_AT_MAX                      = 0x000002e2,
    PH_PERF_SEL_SC6_CREDIT_AT_MAX_NO_PENDING_SEND      = 0x000002e3,
    PH_PERF_SEL_SC6_PA0_DATA_FIFO_RD                   = 0x000002e8,
    PH_PERF_SEL_SC6_PA0_DATA_FIFO_WE                   = 0x000002e9,
    PH_PERF_SEL_SC6_PA0_FIFO_EMPTY                     = 0x000002ea,
    PH_PERF_SEL_SC6_PA0_FIFO_FULL                      = 0x000002eb,
    PH_PERF_SEL_SC6_PA0_NULL_WE                        = 0x000002ec,
    PH_PERF_SEL_SC6_PA0_EVENT_WE                       = 0x000002ed,
    PH_PERF_SEL_SC6_PA0_FPOV_WE                        = 0x000002ee,
    PH_PERF_SEL_SC6_PA0_EOP_WE                         = 0x000002f0,
    PH_PERF_SEL_SC6_PA0_DATA_FIFO_EOP_RD               = 0x000002f1,
    PH_PERF_SEL_SC6_PA0_EOPG_WE                        = 0x000002f2,
    PH_PERF_SEL_SC6_PA1_DATA_FIFO_RD                   = 0x000002f4,
    PH_PERF_SEL_SC6_PA1_DATA_FIFO_WE                   = 0x000002f5,
    PH_PERF_SEL_SC6_PA1_FIFO_EMPTY                     = 0x000002f6,
    PH_PERF_SEL_SC6_PA1_FIFO_FULL                      = 0x000002f7,
    PH_PERF_SEL_SC6_PA1_NULL_WE                        = 0x000002f8,
    PH_PERF_SEL_SC6_PA1_EVENT_WE                       = 0x000002f9,
    PH_PERF_SEL_SC6_PA1_FPOV_WE                        = 0x000002fa,
    PH_PERF_SEL_SC6_PA1_EOP_WE                         = 0x000002fc,
    PH_PERF_SEL_SC6_PA1_DATA_FIFO_EOP_RD               = 0x000002fd,
    PH_PERF_SEL_SC6_PA1_EOPG_WE                        = 0x000002fe,
    PH_PERF_SEL_SC6_PA2_DATA_FIFO_RD                   = 0x00000300,
    PH_PERF_SEL_SC6_PA2_DATA_FIFO_WE                   = 0x00000301,
    PH_PERF_SEL_SC6_PA2_FIFO_EMPTY                     = 0x00000302,
    PH_PERF_SEL_SC6_PA2_FIFO_FULL                      = 0x00000303,
    PH_PERF_SEL_SC6_PA2_NULL_WE                        = 0x00000304,
    PH_PERF_SEL_SC6_PA2_EVENT_WE                       = 0x00000305,
    PH_PERF_SEL_SC6_PA2_FPOV_WE                        = 0x00000306,
    PH_PERF_SEL_SC6_PA2_EOP_WE                         = 0x00000308,
    PH_PERF_SEL_SC6_PA2_DATA_FIFO_EOP_RD               = 0x00000309,
    PH_PERF_SEL_SC6_PA2_EOPG_WE                        = 0x0000030a,
    PH_PERF_SEL_SC6_PA3_DATA_FIFO_RD                   = 0x0000030c,
    PH_PERF_SEL_SC6_PA3_DATA_FIFO_WE                   = 0x0000030d,
    PH_PERF_SEL_SC6_PA3_FIFO_EMPTY                     = 0x0000030e,
    PH_PERF_SEL_SC6_PA3_FIFO_FULL                      = 0x0000030f,
    PH_PERF_SEL_SC6_PA3_NULL_WE                        = 0x00000310,
    PH_PERF_SEL_SC6_PA3_EVENT_WE                       = 0x00000311,
    PH_PERF_SEL_SC6_PA3_FPOV_WE                        = 0x00000312,
    PH_PERF_SEL_SC6_PA3_EOP_WE                         = 0x00000314,
    PH_PERF_SEL_SC6_PA3_DATA_FIFO_EOP_RD               = 0x00000315,
    PH_PERF_SEL_SC6_PA3_EOPG_WE                        = 0x00000316,
    PH_PERF_SEL_SC6_PA4_DATA_FIFO_RD                   = 0x00000318,
    PH_PERF_SEL_SC6_PA4_DATA_FIFO_WE                   = 0x00000319,
    PH_PERF_SEL_SC6_PA4_FIFO_EMPTY                     = 0x0000031a,
    PH_PERF_SEL_SC6_PA4_FIFO_FULL                      = 0x0000031b,
    PH_PERF_SEL_SC6_PA4_NULL_WE                        = 0x0000031c,
    PH_PERF_SEL_SC6_PA4_EVENT_WE                       = 0x0000031d,
    PH_PERF_SEL_SC6_PA4_FPOV_WE                        = 0x0000031e,
    PH_PERF_SEL_SC6_PA4_EOP_WE                         = 0x00000320,
    PH_PERF_SEL_SC6_PA4_DATA_FIFO_EOP_RD               = 0x00000321,
    PH_PERF_SEL_SC6_PA4_EOPG_WE                        = 0x00000322,
    PH_PERF_SEL_SC6_PA5_DATA_FIFO_RD                   = 0x00000324,
    PH_PERF_SEL_SC6_PA5_DATA_FIFO_WE                   = 0x00000325,
    PH_PERF_SEL_SC6_PA5_FIFO_EMPTY                     = 0x00000326,
    PH_PERF_SEL_SC6_PA5_FIFO_FULL                      = 0x00000327,
    PH_PERF_SEL_SC6_PA5_NULL_WE                        = 0x00000328,
    PH_PERF_SEL_SC6_PA5_EVENT_WE                       = 0x00000329,
    PH_PERF_SEL_SC6_PA5_FPOV_WE                        = 0x0000032a,
    PH_PERF_SEL_SC6_PA5_EOP_WE                         = 0x0000032c,
    PH_PERF_SEL_SC6_PA5_DATA_FIFO_EOP_RD               = 0x0000032d,
    PH_PERF_SEL_SC6_PA5_EOPG_WE                        = 0x0000032e,
    PH_PERF_SEL_SC6_PA6_DATA_FIFO_RD                   = 0x00000330,
    PH_PERF_SEL_SC6_PA6_DATA_FIFO_WE                   = 0x00000331,
    PH_PERF_SEL_SC6_PA6_FIFO_EMPTY                     = 0x00000332,
    PH_PERF_SEL_SC6_PA6_FIFO_FULL                      = 0x00000333,
    PH_PERF_SEL_SC6_PA6_NULL_WE                        = 0x00000334,
    PH_PERF_SEL_SC6_PA6_EVENT_WE                       = 0x00000335,
    PH_PERF_SEL_SC6_PA6_FPOV_WE                        = 0x00000336,
    PH_PERF_SEL_SC6_PA6_EOP_WE                         = 0x00000338,
    PH_PERF_SEL_SC6_PA6_DATA_FIFO_EOP_RD               = 0x00000339,
    PH_PERF_SEL_SC6_PA6_EOPG_WE                        = 0x0000033a,
    PH_PERF_SEL_SC6_PA7_DATA_FIFO_RD                   = 0x0000033c,
    PH_PERF_SEL_SC6_PA7_DATA_FIFO_WE                   = 0x0000033d,
    PH_PERF_SEL_SC6_PA7_FIFO_EMPTY                     = 0x0000033e,
    PH_PERF_SEL_SC6_PA7_FIFO_FULL                      = 0x0000033f,
    PH_PERF_SEL_SC6_PA7_NULL_WE                        = 0x00000340,
    PH_PERF_SEL_SC6_PA7_EVENT_WE                       = 0x00000341,
    PH_PERF_SEL_SC6_PA7_FPOV_WE                        = 0x00000342,
    PH_PERF_SEL_SC6_PA7_EOP_WE                         = 0x00000344,
    PH_PERF_SEL_SC6_PA7_DATA_FIFO_EOP_RD               = 0x00000345,
    PH_PERF_SEL_SC6_PA7_EOPG_WE                        = 0x00000346,
    PH_PERF_SEL_SC7_SRPS_WINDOW_VALID                  = 0x00000348,
    PH_PERF_SEL_SC7_ARB_XFC_ALL_EVENT_OR_PRIM_CYCLES   = 0x00000349,
    PH_PERF_SEL_SC7_ARB_XFC_ONLY_PRIM_CYCLES           = 0x0000034a,
    PH_PERF_SEL_SC7_ARB_XFC_ONLY_ONE_INC_PER_PRIM      = 0x0000034b,
    PH_PERF_SEL_SC7_ARB_STALLED_FROM_BELOW             = 0x0000034c,
    PH_PERF_SEL_SC7_ARB_STARVED_FROM_ABOVE             = 0x0000034d,
    PH_PERF_SEL_SC7_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_NOT_EMPTY = 0x0000034e,
    PH_PERF_SEL_SC7_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_FULL = 0x0000034f,
    PH_PERF_SEL_SC7_ARB_BUSY                           = 0x00000350,
    PH_PERF_SEL_SC7_ARB_PA_BUSY_SOP                    = 0x00000351,
    PH_PERF_SEL_SC7_ARB_EOP_POP_SYNC_POP               = 0x00000352,
    PH_PERF_SEL_SC7_ARB_EVENT_SYNC_POP                 = 0x00000353,
    PH_PERF_SEL_SC7_PS_ENG_MULTICYCLE_BUBBLE           = 0x00000354,
    PH_PERF_SEL_SC7_EOP_SYNC_WINDOW                    = 0x00000355,
    PH_PERF_SEL_SC7_BUSY_PROCESSING_MULTICYCLE_PRIM    = 0x00000356,
    PH_PERF_SEL_SC7_BUSY_CNT_NOT_ZERO                  = 0x00000357,
    PH_PERF_SEL_SC7_SEND                               = 0x00000358,
    PH_PERF_SEL_SC7_CREDIT_AT_ZERO_WITH_PENDING_SEND   = 0x00000359,
    PH_PERF_SEL_SC7_CREDIT_AT_MAX                      = 0x0000035a,
    PH_PERF_SEL_SC7_CREDIT_AT_MAX_NO_PENDING_SEND      = 0x0000035b,
    PH_PERF_SEL_SC7_PA0_DATA_FIFO_RD                   = 0x00000360,
    PH_PERF_SEL_SC7_PA0_DATA_FIFO_WE                   = 0x00000361,
    PH_PERF_SEL_SC7_PA0_FIFO_EMPTY                     = 0x00000362,
    PH_PERF_SEL_SC7_PA0_FIFO_FULL                      = 0x00000363,
    PH_PERF_SEL_SC7_PA0_NULL_WE                        = 0x00000364,
    PH_PERF_SEL_SC7_PA0_EVENT_WE                       = 0x00000365,
    PH_PERF_SEL_SC7_PA0_FPOV_WE                        = 0x00000366,
    PH_PERF_SEL_SC7_PA0_EOP_WE                         = 0x00000368,
    PH_PERF_SEL_SC7_PA0_DATA_FIFO_EOP_RD               = 0x00000369,
    PH_PERF_SEL_SC7_PA0_EOPG_WE                        = 0x0000036a,
    PH_PERF_SEL_SC7_PA1_DATA_FIFO_RD                   = 0x0000036c,
    PH_PERF_SEL_SC7_PA1_DATA_FIFO_WE                   = 0x0000036d,
    PH_PERF_SEL_SC7_PA1_FIFO_EMPTY                     = 0x0000036e,
    PH_PERF_SEL_SC7_PA1_FIFO_FULL                      = 0x0000036f,
    PH_PERF_SEL_SC7_PA1_NULL_WE                        = 0x00000370,
    PH_PERF_SEL_SC7_PA1_EVENT_WE                       = 0x00000371,
    PH_PERF_SEL_SC7_PA1_FPOV_WE                        = 0x00000372,
    PH_PERF_SEL_SC7_PA1_EOP_WE                         = 0x00000374,
    PH_PERF_SEL_SC7_PA1_DATA_FIFO_EOP_RD               = 0x00000375,
    PH_PERF_SEL_SC7_PA1_EOPG_WE                        = 0x00000376,
    PH_PERF_SEL_SC7_PA2_DATA_FIFO_RD                   = 0x00000378,
    PH_PERF_SEL_SC7_PA2_DATA_FIFO_WE                   = 0x00000379,
    PH_PERF_SEL_SC7_PA2_FIFO_EMPTY                     = 0x0000037a,
    PH_PERF_SEL_SC7_PA2_FIFO_FULL                      = 0x0000037b,
    PH_PERF_SEL_SC7_PA2_NULL_WE                        = 0x0000037c,
    PH_PERF_SEL_SC7_PA2_EVENT_WE                       = 0x0000037d,
    PH_PERF_SEL_SC7_PA2_FPOV_WE                        = 0x0000037e,
    PH_PERF_SEL_SC7_PA2_EOP_WE                         = 0x00000380,
    PH_PERF_SEL_SC7_PA2_DATA_FIFO_EOP_RD               = 0x00000381,
    PH_PERF_SEL_SC7_PA2_EOPG_WE                        = 0x00000382,
    PH_PERF_SEL_SC7_PA3_DATA_FIFO_RD                   = 0x00000384,
    PH_PERF_SEL_SC7_PA3_DATA_FIFO_WE                   = 0x00000385,
    PH_PERF_SEL_SC7_PA3_FIFO_EMPTY                     = 0x00000386,
    PH_PERF_SEL_SC7_PA3_FIFO_FULL                      = 0x00000387,
    PH_PERF_SEL_SC7_PA3_NULL_WE                        = 0x00000388,
    PH_PERF_SEL_SC7_PA3_EVENT_WE                       = 0x00000389,
    PH_PERF_SEL_SC7_PA3_FPOV_WE                        = 0x0000038a,
    PH_PERF_SEL_SC7_PA3_EOP_WE                         = 0x0000038c,
    PH_PERF_SEL_SC7_PA3_DATA_FIFO_EOP_RD               = 0x0000038d,
    PH_PERF_SEL_SC7_PA3_EOPG_WE                        = 0x0000038e,
    PH_PERF_SEL_SC7_PA4_DATA_FIFO_RD                   = 0x00000390,
    PH_PERF_SEL_SC7_PA4_DATA_FIFO_WE                   = 0x00000391,
    PH_PERF_SEL_SC7_PA4_FIFO_EMPTY                     = 0x00000392,
    PH_PERF_SEL_SC7_PA4_FIFO_FULL                      = 0x00000393,
    PH_PERF_SEL_SC7_PA4_NULL_WE                        = 0x00000394,
    PH_PERF_SEL_SC7_PA4_EVENT_WE                       = 0x00000395,
    PH_PERF_SEL_SC7_PA4_FPOV_WE                        = 0x00000396,
    PH_PERF_SEL_SC7_PA4_EOP_WE                         = 0x00000398,
    PH_PERF_SEL_SC7_PA4_DATA_FIFO_EOP_RD               = 0x00000399,
    PH_PERF_SEL_SC7_PA4_EOPG_WE                        = 0x0000039a,
    PH_PERF_SEL_SC7_PA5_DATA_FIFO_RD                   = 0x0000039c,
    PH_PERF_SEL_SC7_PA5_DATA_FIFO_WE                   = 0x0000039d,
    PH_PERF_SEL_SC7_PA5_FIFO_EMPTY                     = 0x0000039e,
    PH_PERF_SEL_SC7_PA5_FIFO_FULL                      = 0x0000039f,
    PH_PERF_SEL_SC7_PA5_NULL_WE                        = 0x000003a0,
    PH_PERF_SEL_SC7_PA5_EVENT_WE                       = 0x000003a1,
    PH_PERF_SEL_SC7_PA5_FPOV_WE                        = 0x000003a2,
    PH_PERF_SEL_SC7_PA5_EOP_WE                         = 0x000003a4,
    PH_PERF_SEL_SC7_PA5_DATA_FIFO_EOP_RD               = 0x000003a5,
    PH_PERF_SEL_SC7_PA5_EOPG_WE                        = 0x000003a6,
    PH_PERF_SEL_SC7_PA6_DATA_FIFO_RD                   = 0x000003a8,
    PH_PERF_SEL_SC7_PA6_DATA_FIFO_WE                   = 0x000003a9,
    PH_PERF_SEL_SC7_PA6_FIFO_EMPTY                     = 0x000003aa,
    PH_PERF_SEL_SC7_PA6_FIFO_FULL                      = 0x000003ab,
    PH_PERF_SEL_SC7_PA6_NULL_WE                        = 0x000003ac,
    PH_PERF_SEL_SC7_PA6_EVENT_WE                       = 0x000003ad,
    PH_PERF_SEL_SC7_PA6_FPOV_WE                        = 0x000003ae,
    PH_PERF_SEL_SC7_PA6_EOP_WE                         = 0x000003b0,
    PH_PERF_SEL_SC7_PA6_DATA_FIFO_EOP_RD               = 0x000003b1,
    PH_PERF_SEL_SC7_PA6_EOPG_WE                        = 0x000003b2,
    PH_PERF_SEL_SC7_PA7_DATA_FIFO_RD                   = 0x000003b4,
    PH_PERF_SEL_SC7_PA7_DATA_FIFO_WE                   = 0x000003b5,
    PH_PERF_SEL_SC7_PA7_FIFO_EMPTY                     = 0x000003b6,
    PH_PERF_SEL_SC7_PA7_FIFO_FULL                      = 0x000003b7,
    PH_PERF_SEL_SC7_PA7_NULL_WE                        = 0x000003b8,
    PH_PERF_SEL_SC7_PA7_EVENT_WE                       = 0x000003b9,
    PH_PERF_SEL_SC7_PA7_FPOV_WE                        = 0x000003ba,
    PH_PERF_SEL_SC7_PA7_EOP_WE                         = 0x000003bc,
    PH_PERF_SEL_SC7_PA7_DATA_FIFO_EOP_RD               = 0x000003bd,
    PH_PERF_SEL_SC7_PA7_EOPG_WE                        = 0x000003be,
    PH_PERF_SEL_SC0_PA0_LPOV_WE__GFX10                 = 0x0000001f,
    PH_PERF_SEL_SC0_PA0_DEALLOC_4_0_RD__GFX10          = 0x00000023,
    PH_PERF_SEL_SC0_PA1_LPOV_WE__GFX10                 = 0x0000002b,
    PH_PERF_SEL_SC0_PA1_DEALLOC_4_0_RD__GFX10          = 0x0000002f,
    PH_PERF_SEL_SC0_PA2_LPOV_WE__GFX10                 = 0x00000037,
    PH_PERF_SEL_SC0_PA2_DEALLOC_4_0_RD__GFX10          = 0x0000003b,
    PH_PERF_SEL_SC0_PA3_LPOV_WE__GFX10                 = 0x00000043,
    PH_PERF_SEL_SC0_PA3_DEALLOC_4_0_RD__GFX10          = 0x00000047,
    PH_PERF_SEL_SC0_PA4_LPOV_WE__GFX10                 = 0x0000004f,
    PH_PERF_SEL_SC0_PA4_DEALLOC_4_0_RD__GFX10          = 0x00000053,
    PH_PERF_SEL_SC0_PA5_LPOV_WE__GFX10                 = 0x0000005b,
    PH_PERF_SEL_SC0_PA5_DEALLOC_4_0_RD__GFX10          = 0x0000005f,
    PH_PERF_SEL_SC0_PA6_LPOV_WE__GFX10                 = 0x00000067,
    PH_PERF_SEL_SC0_PA6_DEALLOC_4_0_RD__GFX10          = 0x0000006b,
    PH_PERF_SEL_SC0_PA7_LPOV_WE__GFX10                 = 0x00000073,
    PH_PERF_SEL_SC0_PA7_DEALLOC_4_0_RD__GFX10          = 0x00000077,
    PH_PERF_SEL_SC1_PA0_LPOV_WE__GFX10                 = 0x00000097,
    PH_PERF_SEL_SC1_PA0_DEALLOC_4_0_RD__GFX10          = 0x0000009b,
    PH_PERF_SEL_SC1_PA1_LPOV_WE__GFX10                 = 0x000000a3,
    PH_PERF_SEL_SC1_PA1_DEALLOC_4_0_RD__GFX10          = 0x000000a7,
    PH_PERF_SEL_SC1_PA2_LPOV_WE__GFX10                 = 0x000000af,
    PH_PERF_SEL_SC1_PA2_DEALLOC_4_0_RD__GFX10          = 0x000000b3,
    PH_PERF_SEL_SC1_PA3_LPOV_WE__GFX10                 = 0x000000bb,
    PH_PERF_SEL_SC1_PA3_DEALLOC_4_0_RD__GFX10          = 0x000000bf,
    PH_PERF_SEL_SC1_PA4_LPOV_WE__GFX10                 = 0x000000c7,
    PH_PERF_SEL_SC1_PA4_DEALLOC_4_0_RD__GFX10          = 0x000000cb,
    PH_PERF_SEL_SC1_PA5_LPOV_WE__GFX10                 = 0x000000d3,
    PH_PERF_SEL_SC1_PA5_DEALLOC_4_0_RD__GFX10          = 0x000000d7,
    PH_PERF_SEL_SC1_PA6_LPOV_WE__GFX10                 = 0x000000df,
    PH_PERF_SEL_SC1_PA6_DEALLOC_4_0_RD__GFX10          = 0x000000e3,
    PH_PERF_SEL_SC1_PA7_LPOV_WE__GFX10                 = 0x000000eb,
    PH_PERF_SEL_SC1_PA7_DEALLOC_4_0_RD__GFX10          = 0x000000ef,
    PH_PERF_SEL_SC2_PA0_LPOV_WE__GFX10                 = 0x0000010f,
    PH_PERF_SEL_SC2_PA0_DEALLOC_4_0_RD__GFX10          = 0x00000113,
    PH_PERF_SEL_SC2_PA1_LPOV_WE__GFX10                 = 0x0000011b,
    PH_PERF_SEL_SC2_PA1_DEALLOC_4_0_RD__GFX10          = 0x0000011f,
    PH_PERF_SEL_SC2_PA2_LPOV_WE__GFX10                 = 0x00000127,
    PH_PERF_SEL_SC2_PA2_DEALLOC_4_0_RD__GFX10          = 0x0000012b,
    PH_PERF_SEL_SC2_PA3_LPOV_WE__GFX10                 = 0x00000133,
    PH_PERF_SEL_SC2_PA3_DEALLOC_4_0_RD__GFX10          = 0x00000137,
    PH_PERF_SEL_SC2_PA4_LPOV_WE__GFX10                 = 0x0000013f,
    PH_PERF_SEL_SC2_PA4_DEALLOC_4_0_RD__GFX10          = 0x00000143,
    PH_PERF_SEL_SC2_PA5_LPOV_WE__GFX10                 = 0x0000014b,
    PH_PERF_SEL_SC2_PA5_DEALLOC_4_0_RD__GFX10          = 0x0000014f,
    PH_PERF_SEL_SC2_PA6_LPOV_WE__GFX10                 = 0x00000157,
    PH_PERF_SEL_SC2_PA6_DEALLOC_4_0_RD__GFX10          = 0x0000015b,
    PH_PERF_SEL_SC2_PA7_LPOV_WE__GFX10                 = 0x00000163,
    PH_PERF_SEL_SC2_PA7_DEALLOC_4_0_RD__GFX10          = 0x00000167,
    PH_PERF_SEL_SC3_PA0_LPOV_WE__GFX10                 = 0x00000187,
    PH_PERF_SEL_SC3_PA0_DEALLOC_4_0_RD__GFX10          = 0x0000018b,
    PH_PERF_SEL_SC3_PA1_LPOV_WE__GFX10                 = 0x00000193,
    PH_PERF_SEL_SC3_PA1_DEALLOC_4_0_RD__GFX10          = 0x00000197,
    PH_PERF_SEL_SC3_PA2_LPOV_WE__GFX10                 = 0x0000019f,
    PH_PERF_SEL_SC3_PA2_DEALLOC_4_0_RD__GFX10          = 0x000001a3,
    PH_PERF_SEL_SC3_PA3_LPOV_WE__GFX10                 = 0x000001ab,
    PH_PERF_SEL_SC3_PA3_DEALLOC_4_0_RD__GFX10          = 0x000001af,
    PH_PERF_SEL_SC3_PA4_LPOV_WE__GFX10                 = 0x000001b7,
    PH_PERF_SEL_SC3_PA4_DEALLOC_4_0_RD__GFX10          = 0x000001bb,
    PH_PERF_SEL_SC3_PA5_LPOV_WE__GFX10                 = 0x000001c3,
    PH_PERF_SEL_SC3_PA5_DEALLOC_4_0_RD__GFX10          = 0x000001c7,
    PH_PERF_SEL_SC3_PA6_LPOV_WE__GFX10                 = 0x000001cf,
    PH_PERF_SEL_SC3_PA6_DEALLOC_4_0_RD__GFX10          = 0x000001d3,
    PH_PERF_SEL_SC3_PA7_LPOV_WE__GFX10                 = 0x000001db,
    PH_PERF_SEL_SC3_PA7_DEALLOC_4_0_RD__GFX10          = 0x000001df,
    PH_PERF_SEL_SC4_PA0_LPOV_WE__GFX10                 = 0x000001ff,
    PH_PERF_SEL_SC4_PA0_DEALLOC_4_0_RD__GFX10          = 0x00000203,
    PH_PERF_SEL_SC4_PA1_LPOV_WE__GFX10                 = 0x0000020b,
    PH_PERF_SEL_SC4_PA1_DEALLOC_4_0_RD__GFX10          = 0x0000020f,
    PH_PERF_SEL_SC4_PA2_LPOV_WE__GFX10                 = 0x00000217,
    PH_PERF_SEL_SC4_PA2_DEALLOC_4_0_RD__GFX10          = 0x0000021b,
    PH_PERF_SEL_SC4_PA3_LPOV_WE__GFX10                 = 0x00000223,
    PH_PERF_SEL_SC4_PA3_DEALLOC_4_0_RD__GFX10          = 0x00000227,
    PH_PERF_SEL_SC4_PA4_LPOV_WE__GFX10                 = 0x0000022f,
    PH_PERF_SEL_SC4_PA4_DEALLOC_4_0_RD__GFX10          = 0x00000233,
    PH_PERF_SEL_SC4_PA5_LPOV_WE__GFX10                 = 0x0000023b,
    PH_PERF_SEL_SC4_PA5_DEALLOC_4_0_RD__GFX10          = 0x0000023f,
    PH_PERF_SEL_SC4_PA6_LPOV_WE__GFX10                 = 0x00000247,
    PH_PERF_SEL_SC4_PA6_DEALLOC_4_0_RD__GFX10          = 0x0000024b,
    PH_PERF_SEL_SC4_PA7_LPOV_WE__GFX10                 = 0x00000253,
    PH_PERF_SEL_SC4_PA7_DEALLOC_4_0_RD__GFX10          = 0x00000257,
    PH_PERF_SEL_SC5_PA0_LPOV_WE__GFX10                 = 0x00000277,
    PH_PERF_SEL_SC5_PA0_DEALLOC_4_0_RD__GFX10          = 0x0000027b,
    PH_PERF_SEL_SC5_PA1_LPOV_WE__GFX10                 = 0x00000283,
    PH_PERF_SEL_SC5_PA1_DEALLOC_4_0_RD__GFX10          = 0x00000287,
    PH_PERF_SEL_SC5_PA2_LPOV_WE__GFX10                 = 0x0000028f,
    PH_PERF_SEL_SC5_PA2_DEALLOC_4_0_RD__GFX10          = 0x00000293,
    PH_PERF_SEL_SC5_PA3_LPOV_WE__GFX10                 = 0x0000029b,
    PH_PERF_SEL_SC5_PA3_DEALLOC_4_0_RD__GFX10          = 0x0000029f,
    PH_PERF_SEL_SC5_PA4_LPOV_WE__GFX10                 = 0x000002a7,
    PH_PERF_SEL_SC5_PA4_DEALLOC_4_0_RD__GFX10          = 0x000002ab,
    PH_PERF_SEL_SC5_PA5_LPOV_WE__GFX10                 = 0x000002b3,
    PH_PERF_SEL_SC5_PA5_DEALLOC_4_0_RD__GFX10          = 0x000002b7,
    PH_PERF_SEL_SC5_PA6_LPOV_WE__GFX10                 = 0x000002bf,
    PH_PERF_SEL_SC5_PA6_DEALLOC_4_0_RD__GFX10          = 0x000002c3,
    PH_PERF_SEL_SC5_PA7_LPOV_WE__GFX10                 = 0x000002cb,
    PH_PERF_SEL_SC5_PA7_DEALLOC_4_0_RD__GFX10          = 0x000002cf,
    PH_PERF_SEL_SC6_PA0_LPOV_WE__GFX10                 = 0x000002ef,
    PH_PERF_SEL_SC6_PA0_DEALLOC_4_0_RD__GFX10          = 0x000002f3,
    PH_PERF_SEL_SC6_PA1_LPOV_WE__GFX10                 = 0x000002fb,
    PH_PERF_SEL_SC6_PA1_DEALLOC_4_0_RD__GFX10          = 0x000002ff,
    PH_PERF_SEL_SC6_PA2_LPOV_WE__GFX10                 = 0x00000307,
    PH_PERF_SEL_SC6_PA2_DEALLOC_4_0_RD__GFX10          = 0x0000030b,
    PH_PERF_SEL_SC6_PA3_LPOV_WE__GFX10                 = 0x00000313,
    PH_PERF_SEL_SC6_PA3_DEALLOC_4_0_RD__GFX10          = 0x00000317,
    PH_PERF_SEL_SC6_PA4_LPOV_WE__GFX10                 = 0x0000031f,
    PH_PERF_SEL_SC6_PA4_DEALLOC_4_0_RD__GFX10          = 0x00000323,
    PH_PERF_SEL_SC6_PA5_LPOV_WE__GFX10                 = 0x0000032b,
    PH_PERF_SEL_SC6_PA5_DEALLOC_4_0_RD__GFX10          = 0x0000032f,
    PH_PERF_SEL_SC6_PA6_LPOV_WE__GFX10                 = 0x00000337,
    PH_PERF_SEL_SC6_PA6_DEALLOC_4_0_RD__GFX10          = 0x0000033b,
    PH_PERF_SEL_SC6_PA7_LPOV_WE__GFX10                 = 0x00000343,
    PH_PERF_SEL_SC6_PA7_DEALLOC_4_0_RD__GFX10          = 0x00000347,
    PH_PERF_SEL_SC7_PA0_LPOV_WE__GFX10                 = 0x00000367,
    PH_PERF_SEL_SC7_PA0_DEALLOC_4_0_RD__GFX10          = 0x0000036b,
    PH_PERF_SEL_SC7_PA1_LPOV_WE__GFX10                 = 0x00000373,
    PH_PERF_SEL_SC7_PA1_DEALLOC_4_0_RD__GFX10          = 0x00000377,
    PH_PERF_SEL_SC7_PA2_LPOV_WE__GFX10                 = 0x0000037f,
    PH_PERF_SEL_SC7_PA2_DEALLOC_4_0_RD__GFX10          = 0x00000383,
    PH_PERF_SEL_SC7_PA3_LPOV_WE__GFX10                 = 0x0000038b,
    PH_PERF_SEL_SC7_PA3_DEALLOC_4_0_RD__GFX10          = 0x0000038f,
    PH_PERF_SEL_SC7_PA4_LPOV_WE__GFX10                 = 0x00000397,
    PH_PERF_SEL_SC7_PA4_DEALLOC_4_0_RD__GFX10          = 0x0000039b,
    PH_PERF_SEL_SC7_PA5_LPOV_WE__GFX10                 = 0x000003a3,
    PH_PERF_SEL_SC7_PA5_DEALLOC_4_0_RD__GFX10          = 0x000003a7,
    PH_PERF_SEL_SC7_PA6_LPOV_WE__GFX10                 = 0x000003af,
    PH_PERF_SEL_SC7_PA6_DEALLOC_4_0_RD__GFX10          = 0x000003b3,
    PH_PERF_SEL_SC7_PA7_LPOV_WE__GFX10                 = 0x000003bb,
    PH_PERF_SEL_SC7_PA7_DEALLOC_4_0_RD__GFX10          = 0x000003bf,
    PH_PERF_SEL_SC0_GFX_PIPE_EVENT_PROVOKED_TRANSITION__GFX101 = 0x00000014,
    PH_PERF_SEL_SC0_GFX_PIPE_PRIM_PROVOKED_TRANSITION__GFX101 = 0x00000015,
    PH_PERF_SEL_SC0_GFX_PIPE0_TO_1_TRANSITION__GFX101  = 0x00000016,
    PH_PERF_SEL_SC0_GFX_PIPE1_TO_0_TRANSITION__GFX101  = 0x00000017,
    PH_PERF_SEL_SC1_GFX_PIPE_EVENT_PROVOKED_TRANSITION__GFX101 = 0x0000008c,
    PH_PERF_SEL_SC1_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__GFX101 = 0x0000008d,
    PH_PERF_SEL_SC1_GFX_PIPE0_TO_1_TRANSITION__GFX101  = 0x0000008e,
    PH_PERF_SEL_SC1_GFX_PIPE1_TO_0_TRANSITION__GFX101  = 0x0000008f,
    PH_PERF_SEL_SC2_GFX_PIPE_EVENT_PROVOKED_TRANSITION__GFX101 = 0x00000104,
    PH_PERF_SEL_SC2_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__GFX101 = 0x00000105,
    PH_PERF_SEL_SC2_GFX_PIPE0_TO_1_TRANSITION__GFX101  = 0x00000106,
    PH_PERF_SEL_SC2_GFX_PIPE1_TO_0_TRANSITION__GFX101  = 0x00000107,
    PH_PERF_SEL_SC3_GFX_PIPE_EVENT_PROVOKED_TRANSITION__GFX101 = 0x0000017c,
    PH_PERF_SEL_SC3_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__GFX101 = 0x0000017d,
    PH_PERF_SEL_SC3_GFX_PIPE0_TO_1_TRANSITION__GFX101  = 0x0000017e,
    PH_PERF_SEL_SC3_GFX_PIPE1_TO_0_TRANSITION__GFX101  = 0x0000017f,
    PH_PERF_SEL_SC4_GFX_PIPE_EVENT_PROVOKED_TRANSITION__GFX101 = 0x000001f4,
    PH_PERF_SEL_SC4_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__GFX101 = 0x000001f5,
    PH_PERF_SEL_SC4_GFX_PIPE0_TO_1_TRANSITION__GFX101  = 0x000001f6,
    PH_PERF_SEL_SC4_GFX_PIPE1_TO_0_TRANSITION__GFX101  = 0x000001f7,
    PH_PERF_SEL_SC5_GFX_PIPE_EVENT_PROVOKED_TRANSITION__GFX101 = 0x0000026c,
    PH_PERF_SEL_SC5_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__GFX101 = 0x0000026d,
    PH_PERF_SEL_SC5_GFX_PIPE0_TO_1_TRANSITION__GFX101  = 0x0000026e,
    PH_PERF_SEL_SC5_GFX_PIPE1_TO_0_TRANSITION__GFX101  = 0x0000026f,
    PH_PERF_SEL_SC6_GFX_PIPE_EVENT_PROVOKED_TRANSITION__GFX101 = 0x000002e4,
    PH_PERF_SEL_SC6_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__GFX101 = 0x000002e5,
    PH_PERF_SEL_SC6_GFX_PIPE0_TO_1_TRANSITION__GFX101  = 0x000002e6,
    PH_PERF_SEL_SC6_GFX_PIPE1_TO_0_TRANSITION__GFX101  = 0x000002e7,
    PH_PERF_SEL_SC7_GFX_PIPE_EVENT_PROVOKED_TRANSITION__GFX101 = 0x0000035c,
    PH_PERF_SEL_SC7_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__GFX101 = 0x0000035d,
    PH_PERF_SEL_SC7_GFX_PIPE0_TO_1_TRANSITION__GFX101  = 0x0000035e,
    PH_PERF_SEL_SC7_GFX_PIPE1_TO_0_TRANSITION__GFX101  = 0x0000035f,
    PH_PERF_SEL_1_SC_ARB_STALLED_FROM_BELOW__GFX103PLUS = 0x000003c0,
    PH_PERF_SEL_2_SC_ARB_STALLED_FROM_BELOW__GFX103PLUS = 0x000003c1,
    PH_PERF_SEL_3_SC_ARB_STALLED_FROM_BELOW__GFX103PLUS = 0x000003c2,
    PH_PERF_SEL_4_SC_ARB_STALLED_FROM_BELOW__GFX103PLUS = 0x000003c3,
    PH_PERF_SEL_5_SC_ARB_STALLED_FROM_BELOW__GFX103PLUS = 0x000003c4,
    PH_PERF_SEL_6_SC_ARB_STALLED_FROM_BELOW__GFX103PLUS = 0x000003c5,
    PH_PERF_SEL_7_SC_ARB_STALLED_FROM_BELOW__GFX103PLUS = 0x000003c6,
    PH_PERF_SEL_8_SC_ARB_STALLED_FROM_BELOW__GFX103PLUS = 0x000003c7,
    PH_PERF_SEL_1_SC_ARB_STARVED_FROM_ABOVE__GFX103PLUS = 0x000003c8,
    PH_PERF_SEL_2_SC_ARB_STARVED_FROM_ABOVE__GFX103PLUS = 0x000003c9,
    PH_PERF_SEL_3_SC_ARB_STARVED_FROM_ABOVE__GFX103PLUS = 0x000003ca,
    PH_PERF_SEL_4_SC_ARB_STARVED_FROM_ABOVE__GFX103PLUS = 0x000003cb,
    PH_PERF_SEL_5_SC_ARB_STARVED_FROM_ABOVE__GFX103PLUS = 0x000003cc,
    PH_PERF_SEL_6_SC_ARB_STARVED_FROM_ABOVE__GFX103PLUS = 0x000003cd,
    PH_PERF_SEL_7_SC_ARB_STARVED_FROM_ABOVE__GFX103PLUS = 0x000003ce,
    PH_PERF_SEL_8_SC_ARB_STARVED_FROM_ABOVE__GFX103PLUS = 0x000003cf,
    PH_PERF_SEL_1_SC_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_NOT_EMPTY__GFX103PLUS = 0x000003d0,
    PH_PERF_SEL_2_SC_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_NOT_EMPTY__GFX103PLUS = 0x000003d1,
    PH_PERF_SEL_3_SC_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_NOT_EMPTY__GFX103PLUS = 0x000003d2,
    PH_PERF_SEL_4_SC_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_NOT_EMPTY__GFX103PLUS = 0x000003d3,
    PH_PERF_SEL_5_SC_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_NOT_EMPTY__GFX103PLUS = 0x000003d4,
    PH_PERF_SEL_6_SC_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_NOT_EMPTY__GFX103PLUS = 0x000003d5,
    PH_PERF_SEL_7_SC_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_NOT_EMPTY__GFX103PLUS = 0x000003d6,
    PH_PERF_SEL_8_SC_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_NOT_EMPTY__GFX103PLUS = 0x000003d7,
    PH_PERF_SEL_1_SC_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_FULL__GFX103PLUS = 0x000003d8,
    PH_PERF_SEL_2_SC_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_FULL__GFX103PLUS = 0x000003d9,
    PH_PERF_SEL_3_SC_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_FULL__GFX103PLUS = 0x000003da,
    PH_PERF_SEL_4_SC_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_FULL__GFX103PLUS = 0x000003db,
    PH_PERF_SEL_5_SC_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_FULL__GFX103PLUS = 0x000003dc,
    PH_PERF_SEL_6_SC_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_FULL__GFX103PLUS = 0x000003dd,
    PH_PERF_SEL_7_SC_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_FULL__GFX103PLUS = 0x000003de,
    PH_PERF_SEL_8_SC_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_FULL__GFX103PLUS = 0x000003df,
    PH_PERF_SEL_SC0_GFX_PIPE0_TO_1_TRANSITION__GFX11   = 0x00000014,
    PH_PERF_SEL_SC0_GFX_PIPE1_TO_0_TRANSITION__GFX11   = 0x00000015,
    PH_PERF_SEL_SC0_GFX_PIPE_PRIM_PROVOKED_TRANSITION__GFX11 = 0x00000016,
    PH_PERF_SEL_SC0_GFX_PIPE_EVENT_PROVOKED_TRANSITION__GFX11 = 0x00000017,
    PH_PERF_SEL_SC0_PA0_FPOP_WE__GFX11                 = 0x0000001f,
    PH_PERF_SEL_SC0_PA0_DEALLOC_WE__GFX11              = 0x00000023,
    PH_PERF_SEL_SC0_PA1_FPOP_WE__GFX11                 = 0x0000002b,
    PH_PERF_SEL_SC0_PA1_DEALLOC_WE__GFX11              = 0x0000002f,
    PH_PERF_SEL_SC0_PA2_FPOP_WE__GFX11                 = 0x00000037,
    PH_PERF_SEL_SC0_PA2_DEALLOC_WE__GFX11              = 0x0000003b,
    PH_PERF_SEL_SC0_PA3_FPOP_WE__GFX11                 = 0x00000043,
    PH_PERF_SEL_SC0_PA3_DEALLOC_WE__GFX11              = 0x00000047,
    PH_PERF_SEL_SC0_PA4_FPOP_WE__GFX11                 = 0x0000004f,
    PH_PERF_SEL_SC0_PA4_DEALLOC_WE__GFX11              = 0x00000053,
    PH_PERF_SEL_SC0_PA5_FPOP_WE__GFX11                 = 0x0000005b,
    PH_PERF_SEL_SC0_PA5_DEALLOC_WE__GFX11              = 0x0000005f,
    PH_PERF_SEL_SC0_PA6_FPOP_WE__GFX11                 = 0x00000067,
    PH_PERF_SEL_SC0_PA6_DEALLOC_WE__GFX11              = 0x0000006b,
    PH_PERF_SEL_SC0_PA7_FPOP_WE__GFX11                 = 0x00000073,
    PH_PERF_SEL_SC0_PA7_DEALLOC_WE__GFX11              = 0x00000077,
    PH_PERF_SEL_SC1_GFX_PIPE0_TO_1_TRANSITION__GFX11   = 0x0000008c,
    PH_PERF_SEL_SC1_GFX_PIPE1_TO_0_TRANSITION__GFX11   = 0x0000008d,
    PH_PERF_SEL_SC1_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__GFX11 = 0x0000008e,
    PH_PERF_SEL_SC1_GFX_PIPE_EVENT_PROVOKED_TRANSITION__GFX11 = 0x0000008f,
    PH_PERF_SEL_SC1_PA0_FPOP_WE__GFX11                 = 0x00000097,
    PH_PERF_SEL_SC1_PA0_DEALLOC_WE__GFX11              = 0x0000009b,
    PH_PERF_SEL_SC1_PA1_FPOP_WE__GFX11                 = 0x000000a3,
    PH_PERF_SEL_SC1_PA1_DEALLOC_WE__GFX11              = 0x000000a7,
    PH_PERF_SEL_SC1_PA2_FPOP_WE__GFX11                 = 0x000000af,
    PH_PERF_SEL_SC1_PA2_DEALLOC_WE__GFX11              = 0x000000b3,
    PH_PERF_SEL_SC1_PA3_FPOP_WE__GFX11                 = 0x000000bb,
    PH_PERF_SEL_SC1_PA3_DEALLOC_WE__GFX11              = 0x000000bf,
    PH_PERF_SEL_SC1_PA4_FPOP_WE__GFX11                 = 0x000000c7,
    PH_PERF_SEL_SC1_PA4_DEALLOC_WE__GFX11              = 0x000000cb,
    PH_PERF_SEL_SC1_PA5_FPOP_WE__GFX11                 = 0x000000d3,
    PH_PERF_SEL_SC1_PA5_DEALLOC_WE__GFX11              = 0x000000d7,
    PH_PERF_SEL_SC1_PA6_FPOP_WE__GFX11                 = 0x000000df,
    PH_PERF_SEL_SC1_PA6_DEALLOC_WE__GFX11              = 0x000000e3,
    PH_PERF_SEL_SC1_PA7_FPOP_WE__GFX11                 = 0x000000eb,
    PH_PERF_SEL_SC1_PA7_DEALLOC_WE__GFX11              = 0x000000ef,
    PH_PERF_SEL_SC2_GFX_PIPE0_TO_1_TRANSITION__GFX11   = 0x00000104,
    PH_PERF_SEL_SC2_GFX_PIPE1_TO_0_TRANSITION__GFX11   = 0x00000105,
    PH_PERF_SEL_SC2_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__GFX11 = 0x00000106,
    PH_PERF_SEL_SC2_GFX_PIPE_EVENT_PROVOKED_TRANSITION__GFX11 = 0x00000107,
    PH_PERF_SEL_SC2_PA0_FPOP_WE__GFX11                 = 0x0000010f,
    PH_PERF_SEL_SC2_PA0_DEALLOC_WE__GFX11              = 0x00000113,
    PH_PERF_SEL_SC2_PA1_FPOP_WE__GFX11                 = 0x0000011b,
    PH_PERF_SEL_SC2_PA1_DEALLOC_WE__GFX11              = 0x0000011f,
    PH_PERF_SEL_SC2_PA2_FPOP_WE__GFX11                 = 0x00000127,
    PH_PERF_SEL_SC2_PA2_DEALLOC_WE__GFX11              = 0x0000012b,
    PH_PERF_SEL_SC2_PA3_FPOP_WE__GFX11                 = 0x00000133,
    PH_PERF_SEL_SC2_PA3_DEALLOC_WE__GFX11              = 0x00000137,
    PH_PERF_SEL_SC2_PA4_FPOP_WE__GFX11                 = 0x0000013f,
    PH_PERF_SEL_SC2_PA4_DEALLOC_WE__GFX11              = 0x00000143,
    PH_PERF_SEL_SC2_PA5_FPOP_WE__GFX11                 = 0x0000014b,
    PH_PERF_SEL_SC2_PA5_DEALLOC_WE__GFX11              = 0x0000014f,
    PH_PERF_SEL_SC2_PA6_FPOP_WE__GFX11                 = 0x00000157,
    PH_PERF_SEL_SC2_PA6_DEALLOC_WE__GFX11              = 0x0000015b,
    PH_PERF_SEL_SC2_PA7_FPOP_WE__GFX11                 = 0x00000163,
    PH_PERF_SEL_SC2_PA7_DEALLOC_WE__GFX11              = 0x00000167,
    PH_PERF_SEL_SC3_GFX_PIPE0_TO_1_TRANSITION__GFX11   = 0x0000017c,
    PH_PERF_SEL_SC3_GFX_PIPE1_TO_0_TRANSITION__GFX11   = 0x0000017d,
    PH_PERF_SEL_SC3_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__GFX11 = 0x0000017e,
    PH_PERF_SEL_SC3_GFX_PIPE_EVENT_PROVOKED_TRANSITION__GFX11 = 0x0000017f,
    PH_PERF_SEL_SC3_PA0_FPOP_WE__GFX11                 = 0x00000187,
    PH_PERF_SEL_SC3_PA0_DEALLOC_WE__GFX11              = 0x0000018b,
    PH_PERF_SEL_SC3_PA1_FPOP_WE__GFX11                 = 0x00000193,
    PH_PERF_SEL_SC3_PA1_DEALLOC_WE__GFX11              = 0x00000197,
    PH_PERF_SEL_SC3_PA2_FPOP_WE__GFX11                 = 0x0000019f,
    PH_PERF_SEL_SC3_PA2_DEALLOC_WE__GFX11              = 0x000001a3,
    PH_PERF_SEL_SC3_PA3_FPOP_WE__GFX11                 = 0x000001ab,
    PH_PERF_SEL_SC3_PA3_DEALLOC_WE__GFX11              = 0x000001af,
    PH_PERF_SEL_SC3_PA4_FPOP_WE__GFX11                 = 0x000001b7,
    PH_PERF_SEL_SC3_PA4_DEALLOC_WE__GFX11              = 0x000001bb,
    PH_PERF_SEL_SC3_PA5_FPOP_WE__GFX11                 = 0x000001c3,
    PH_PERF_SEL_SC3_PA5_DEALLOC_WE__GFX11              = 0x000001c7,
    PH_PERF_SEL_SC3_PA6_FPOP_WE__GFX11                 = 0x000001cf,
    PH_PERF_SEL_SC3_PA6_DEALLOC_WE__GFX11              = 0x000001d3,
    PH_PERF_SEL_SC3_PA7_FPOP_WE__GFX11                 = 0x000001db,
    PH_PERF_SEL_SC3_PA7_DEALLOC_WE__GFX11              = 0x000001df,
    PH_PERF_SEL_SC4_GFX_PIPE0_TO_1_TRANSITION__GFX11   = 0x000001f4,
    PH_PERF_SEL_SC4_GFX_PIPE1_TO_0_TRANSITION__GFX11   = 0x000001f5,
    PH_PERF_SEL_SC4_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__GFX11 = 0x000001f6,
    PH_PERF_SEL_SC4_GFX_PIPE_EVENT_PROVOKED_TRANSITION__GFX11 = 0x000001f7,
    PH_PERF_SEL_SC4_PA0_FPOP_WE__GFX11                 = 0x000001ff,
    PH_PERF_SEL_SC4_PA0_DEALLOC_WE__GFX11              = 0x00000203,
    PH_PERF_SEL_SC4_PA1_FPOP_WE__GFX11                 = 0x0000020b,
    PH_PERF_SEL_SC4_PA1_DEALLOC_WE__GFX11              = 0x0000020f,
    PH_PERF_SEL_SC4_PA2_FPOP_WE__GFX11                 = 0x00000217,
    PH_PERF_SEL_SC4_PA2_DEALLOC_WE__GFX11              = 0x0000021b,
    PH_PERF_SEL_SC4_PA3_FPOP_WE__GFX11                 = 0x00000223,
    PH_PERF_SEL_SC4_PA3_DEALLOC_WE__GFX11              = 0x00000227,
    PH_PERF_SEL_SC4_PA4_FPOP_WE__GFX11                 = 0x0000022f,
    PH_PERF_SEL_SC4_PA4_DEALLOC_WE__GFX11              = 0x00000233,
    PH_PERF_SEL_SC4_PA5_FPOP_WE__GFX11                 = 0x0000023b,
    PH_PERF_SEL_SC4_PA5_DEALLOC_WE__GFX11              = 0x0000023f,
    PH_PERF_SEL_SC4_PA6_FPOP_WE__GFX11                 = 0x00000247,
    PH_PERF_SEL_SC4_PA6_DEALLOC_WE__GFX11              = 0x0000024b,
    PH_PERF_SEL_SC4_PA7_FPOP_WE__GFX11                 = 0x00000253,
    PH_PERF_SEL_SC4_PA7_DEALLOC_WE__GFX11              = 0x00000257,
    PH_PERF_SEL_SC5_GFX_PIPE0_TO_1_TRANSITION__GFX11   = 0x0000026c,
    PH_PERF_SEL_SC5_GFX_PIPE1_TO_0_TRANSITION__GFX11   = 0x0000026d,
    PH_PERF_SEL_SC5_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__GFX11 = 0x0000026e,
    PH_PERF_SEL_SC5_GFX_PIPE_EVENT_PROVOKED_TRANSITION__GFX11 = 0x0000026f,
    PH_PERF_SEL_SC5_PA0_FPOP_WE__GFX11                 = 0x00000277,
    PH_PERF_SEL_SC5_PA0_DEALLOC_WE__GFX11              = 0x0000027b,
    PH_PERF_SEL_SC5_PA1_FPOP_WE__GFX11                 = 0x00000283,
    PH_PERF_SEL_SC5_PA1_DEALLOC_WE__GFX11              = 0x00000287,
    PH_PERF_SEL_SC5_PA2_FPOP_WE__GFX11                 = 0x0000028f,
    PH_PERF_SEL_SC5_PA2_DEALLOC_WE__GFX11              = 0x00000293,
    PH_PERF_SEL_SC5_PA3_FPOP_WE__GFX11                 = 0x0000029b,
    PH_PERF_SEL_SC5_PA3_DEALLOC_WE__GFX11              = 0x0000029f,
    PH_PERF_SEL_SC5_PA4_FPOP_WE__GFX11                 = 0x000002a7,
    PH_PERF_SEL_SC5_PA4_DEALLOC_WE__GFX11              = 0x000002ab,
    PH_PERF_SEL_SC5_PA5_FPOP_WE__GFX11                 = 0x000002b3,
    PH_PERF_SEL_SC5_PA5_DEALLOC_WE__GFX11              = 0x000002b7,
    PH_PERF_SEL_SC5_PA6_FPOP_WE__GFX11                 = 0x000002bf,
    PH_PERF_SEL_SC5_PA6_DEALLOC_WE__GFX11              = 0x000002c3,
    PH_PERF_SEL_SC5_PA7_FPOP_WE__GFX11                 = 0x000002cb,
    PH_PERF_SEL_SC5_PA7_DEALLOC_WE__GFX11              = 0x000002cf,
    PH_PERF_SEL_SC6_GFX_PIPE0_TO_1_TRANSITION__GFX11   = 0x000002e4,
    PH_PERF_SEL_SC6_GFX_PIPE1_TO_0_TRANSITION__GFX11   = 0x000002e5,
    PH_PERF_SEL_SC6_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__GFX11 = 0x000002e6,
    PH_PERF_SEL_SC6_GFX_PIPE_EVENT_PROVOKED_TRANSITION__GFX11 = 0x000002e7,
    PH_PERF_SEL_SC6_PA0_FPOP_WE__GFX11                 = 0x000002ef,
    PH_PERF_SEL_SC6_PA0_DEALLOC_WE__GFX11              = 0x000002f3,
    PH_PERF_SEL_SC6_PA1_FPOP_WE__GFX11                 = 0x000002fb,
    PH_PERF_SEL_SC6_PA1_DEALLOC_WE__GFX11              = 0x000002ff,
    PH_PERF_SEL_SC6_PA2_FPOP_WE__GFX11                 = 0x00000307,
    PH_PERF_SEL_SC6_PA2_DEALLOC_WE__GFX11              = 0x0000030b,
    PH_PERF_SEL_SC6_PA3_FPOP_WE__GFX11                 = 0x00000313,
    PH_PERF_SEL_SC6_PA3_DEALLOC_WE__GFX11              = 0x00000317,
    PH_PERF_SEL_SC6_PA4_FPOP_WE__GFX11                 = 0x0000031f,
    PH_PERF_SEL_SC6_PA4_DEALLOC_WE__GFX11              = 0x00000323,
    PH_PERF_SEL_SC6_PA5_FPOP_WE__GFX11                 = 0x0000032b,
    PH_PERF_SEL_SC6_PA5_DEALLOC_WE__GFX11              = 0x0000032f,
    PH_PERF_SEL_SC6_PA6_FPOP_WE__GFX11                 = 0x00000337,
    PH_PERF_SEL_SC6_PA6_DEALLOC_WE__GFX11              = 0x0000033b,
    PH_PERF_SEL_SC6_PA7_FPOP_WE__GFX11                 = 0x00000343,
    PH_PERF_SEL_SC6_PA7_DEALLOC_WE__GFX11              = 0x00000347,
    PH_PERF_SEL_SC7_GFX_PIPE0_TO_1_TRANSITION__GFX11   = 0x0000035c,
    PH_PERF_SEL_SC7_GFX_PIPE1_TO_0_TRANSITION__GFX11   = 0x0000035d,
    PH_PERF_SEL_SC7_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__GFX11 = 0x0000035e,
    PH_PERF_SEL_SC7_GFX_PIPE_EVENT_PROVOKED_TRANSITION__GFX11 = 0x0000035f,
    PH_PERF_SEL_SC7_PA0_FPOP_WE__GFX11                 = 0x00000367,
    PH_PERF_SEL_SC7_PA0_DEALLOC_WE__GFX11              = 0x0000036b,
    PH_PERF_SEL_SC7_PA1_FPOP_WE__GFX11                 = 0x00000373,
    PH_PERF_SEL_SC7_PA1_DEALLOC_WE__GFX11              = 0x00000377,
    PH_PERF_SEL_SC7_PA2_FPOP_WE__GFX11                 = 0x0000037f,
    PH_PERF_SEL_SC7_PA2_DEALLOC_WE__GFX11              = 0x00000383,
    PH_PERF_SEL_SC7_PA3_FPOP_WE__GFX11                 = 0x0000038b,
    PH_PERF_SEL_SC7_PA3_DEALLOC_WE__GFX11              = 0x0000038f,
    PH_PERF_SEL_SC7_PA4_FPOP_WE__GFX11                 = 0x00000397,
    PH_PERF_SEL_SC7_PA4_DEALLOC_WE__GFX11              = 0x0000039b,
    PH_PERF_SEL_SC7_PA5_FPOP_WE__GFX11                 = 0x000003a3,
    PH_PERF_SEL_SC7_PA5_DEALLOC_WE__GFX11              = 0x000003a7,
    PH_PERF_SEL_SC7_PA6_FPOP_WE__GFX11                 = 0x000003af,
    PH_PERF_SEL_SC7_PA6_DEALLOC_WE__GFX11              = 0x000003b3,
    PH_PERF_SEL_SC7_PA7_FPOP_WE__GFX11                 = 0x000003bb,
    PH_PERF_SEL_SC7_PA7_DEALLOC_WE__GFX11              = 0x000003bf,
    PH_PERF_SC0_FIFO_STATUS_0__GFX11                   = 0x000003e0,
    PH_PERF_SC0_FIFO_STATUS_1__GFX11                   = 0x000003e1,
    PH_PERF_SC0_FIFO_STATUS_2__GFX11                   = 0x000003e2,
    PH_PERF_SC0_FIFO_STATUS_3__GFX11                   = 0x000003e3,
    PH_PERF_SC1_FIFO_STATUS_0__GFX11                   = 0x000003e4,
    PH_PERF_SC1_FIFO_STATUS_1__GFX11                   = 0x000003e5,
    PH_PERF_SC1_FIFO_STATUS_2__GFX11                   = 0x000003e6,
    PH_PERF_SC1_FIFO_STATUS_3__GFX11                   = 0x000003e7,
    PH_PERF_SC2_FIFO_STATUS_0__GFX11                   = 0x000003e8,
    PH_PERF_SC2_FIFO_STATUS_1__GFX11                   = 0x000003e9,
    PH_PERF_SC2_FIFO_STATUS_2__GFX11                   = 0x000003ea,
    PH_PERF_SC2_FIFO_STATUS_3__GFX11                   = 0x000003eb,
    PH_PERF_SC3_FIFO_STATUS_0__GFX11                   = 0x000003ec,
    PH_PERF_SC3_FIFO_STATUS_1__GFX11                   = 0x000003ed,
    PH_PERF_SC3_FIFO_STATUS_2__GFX11                   = 0x000003ee,
    PH_PERF_SC3_FIFO_STATUS_3__GFX11                   = 0x000003ef,
    PH_PERF_SC4_FIFO_STATUS_0__GFX11                   = 0x000003f0,
    PH_PERF_SC4_FIFO_STATUS_1__GFX11                   = 0x000003f1,
    PH_PERF_SC4_FIFO_STATUS_2__GFX11                   = 0x000003f2,
    PH_PERF_SC4_FIFO_STATUS_3__GFX11                   = 0x000003f3,
    PH_PERF_SC5_FIFO_STATUS_0__GFX11                   = 0x000003f4,
    PH_PERF_SC5_FIFO_STATUS_1__GFX11                   = 0x000003f5,
    PH_PERF_SC5_FIFO_STATUS_2__GFX11                   = 0x000003f6,
    PH_PERF_SC5_FIFO_STATUS_3__GFX11                   = 0x000003f7,
    PH_PERF_SC6_FIFO_STATUS_0__GFX11                   = 0x000003f8,
    PH_PERF_SC6_FIFO_STATUS_1__GFX11                   = 0x000003f9,
    PH_PERF_SC6_FIFO_STATUS_2__GFX11                   = 0x000003fa,
    PH_PERF_SC6_FIFO_STATUS_3__GFX11                   = 0x000003fb,
    PH_PERF_SC7_FIFO_STATUS_0__GFX11                   = 0x000003fc,
    PH_PERF_SC7_FIFO_STATUS_1__GFX11                   = 0x000003fd,
    PH_PERF_SC7_FIFO_STATUS_2__GFX11                   = 0x000003fe,
    PH_PERF_SC7_FIFO_STATUS_3__GFX11                   = 0x000003ff,
    PH_PERF_SEL_SC0_GFX_PIPE_EVENT_PROVOKED_TRANSITION__NV21 = 0x00000014,
    PH_PERF_SEL_SC0_GFX_PIPE_PRIM_PROVOKED_TRANSITION__NV21 = 0x00000015,
    PH_PERF_SEL_SC0_GFX_PIPE0_TO_1_TRANSITION__NV21    = 0x00000016,
    PH_PERF_SEL_SC0_GFX_PIPE1_TO_0_TRANSITION__NV21    = 0x00000017,
    PH_PERF_SEL_SC1_GFX_PIPE_EVENT_PROVOKED_TRANSITION__NV21 = 0x0000008c,
    PH_PERF_SEL_SC1_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__NV21 = 0x0000008d,
    PH_PERF_SEL_SC1_GFX_PIPE0_TO_1_TRANSITION__NV21    = 0x0000008e,
    PH_PERF_SEL_SC1_GFX_PIPE1_TO_0_TRANSITION__NV21    = 0x0000008f,
    PH_PERF_SEL_SC2_GFX_PIPE_EVENT_PROVOKED_TRANSITION__NV21 = 0x00000104,
    PH_PERF_SEL_SC2_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__NV21 = 0x00000105,
    PH_PERF_SEL_SC2_GFX_PIPE0_TO_1_TRANSITION__NV21    = 0x00000106,
    PH_PERF_SEL_SC2_GFX_PIPE1_TO_0_TRANSITION__NV21    = 0x00000107,
    PH_PERF_SEL_SC3_GFX_PIPE_EVENT_PROVOKED_TRANSITION__NV21 = 0x0000017c,
    PH_PERF_SEL_SC3_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__NV21 = 0x0000017d,
    PH_PERF_SEL_SC3_GFX_PIPE0_TO_1_TRANSITION__NV21    = 0x0000017e,
    PH_PERF_SEL_SC3_GFX_PIPE1_TO_0_TRANSITION__NV21    = 0x0000017f,
    PH_PERF_SEL_SC4_GFX_PIPE_EVENT_PROVOKED_TRANSITION__NV21 = 0x000001f4,
    PH_PERF_SEL_SC4_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__NV21 = 0x000001f5,
    PH_PERF_SEL_SC4_GFX_PIPE0_TO_1_TRANSITION__NV21    = 0x000001f6,
    PH_PERF_SEL_SC4_GFX_PIPE1_TO_0_TRANSITION__NV21    = 0x000001f7,
    PH_PERF_SEL_SC5_GFX_PIPE_EVENT_PROVOKED_TRANSITION__NV21 = 0x0000026c,
    PH_PERF_SEL_SC5_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__NV21 = 0x0000026d,
    PH_PERF_SEL_SC5_GFX_PIPE0_TO_1_TRANSITION__NV21    = 0x0000026e,
    PH_PERF_SEL_SC5_GFX_PIPE1_TO_0_TRANSITION__NV21    = 0x0000026f,
    PH_PERF_SEL_SC6_GFX_PIPE_EVENT_PROVOKED_TRANSITION__NV21 = 0x000002e4,
    PH_PERF_SEL_SC6_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__NV21 = 0x000002e5,
    PH_PERF_SEL_SC6_GFX_PIPE0_TO_1_TRANSITION__NV21    = 0x000002e6,
    PH_PERF_SEL_SC6_GFX_PIPE1_TO_0_TRANSITION__NV21    = 0x000002e7,
    PH_PERF_SEL_SC7_GFX_PIPE_EVENT_PROVOKED_TRANSITION__NV21 = 0x0000035c,
    PH_PERF_SEL_SC7_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__NV21 = 0x0000035d,
    PH_PERF_SEL_SC7_GFX_PIPE0_TO_1_TRANSITION__NV21    = 0x0000035e,
    PH_PERF_SEL_SC7_GFX_PIPE1_TO_0_TRANSITION__NV21    = 0x0000035f,
    PH_PERF_SEL_SC0_GFX_PIPE0_TO_1_TRANSITION__NV22    = 0x00000014,
    PH_PERF_SEL_SC0_GFX_PIPE1_TO_0_TRANSITION__NV22    = 0x00000015,
    PH_PERF_SEL_SC0_GFX_PIPE_PRIM_PROVOKED_TRANSITION__NV22 = 0x00000016,
    PH_PERF_SEL_SC0_GFX_PIPE_EVENT_PROVOKED_TRANSITION__NV22 = 0x00000017,
    PH_PERF_SEL_SC1_GFX_PIPE0_TO_1_TRANSITION__NV22    = 0x0000008c,
    PH_PERF_SEL_SC1_GFX_PIPE1_TO_0_TRANSITION__NV22    = 0x0000008d,
    PH_PERF_SEL_SC1_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__NV22 = 0x0000008e,
    PH_PERF_SEL_SC1_GFX_PIPE_EVENT_PROVOKED_TRANSITION__NV22 = 0x0000008f,
    PH_PERF_SEL_SC2_GFX_PIPE0_TO_1_TRANSITION__NV22    = 0x00000104,
    PH_PERF_SEL_SC2_GFX_PIPE1_TO_0_TRANSITION__NV22    = 0x00000105,
    PH_PERF_SEL_SC2_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__NV22 = 0x00000106,
    PH_PERF_SEL_SC2_GFX_PIPE_EVENT_PROVOKED_TRANSITION__NV22 = 0x00000107,
    PH_PERF_SEL_SC3_GFX_PIPE0_TO_1_TRANSITION__NV22    = 0x0000017c,
    PH_PERF_SEL_SC3_GFX_PIPE1_TO_0_TRANSITION__NV22    = 0x0000017d,
    PH_PERF_SEL_SC3_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__NV22 = 0x0000017e,
    PH_PERF_SEL_SC3_GFX_PIPE_EVENT_PROVOKED_TRANSITION__NV22 = 0x0000017f,
    PH_PERF_SEL_SC4_GFX_PIPE0_TO_1_TRANSITION__NV22    = 0x000001f4,
    PH_PERF_SEL_SC4_GFX_PIPE1_TO_0_TRANSITION__NV22    = 0x000001f5,
    PH_PERF_SEL_SC4_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__NV22 = 0x000001f6,
    PH_PERF_SEL_SC4_GFX_PIPE_EVENT_PROVOKED_TRANSITION__NV22 = 0x000001f7,
    PH_PERF_SEL_SC5_GFX_PIPE0_TO_1_TRANSITION__NV22    = 0x0000026c,
    PH_PERF_SEL_SC5_GFX_PIPE1_TO_0_TRANSITION__NV22    = 0x0000026d,
    PH_PERF_SEL_SC5_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__NV22 = 0x0000026e,
    PH_PERF_SEL_SC5_GFX_PIPE_EVENT_PROVOKED_TRANSITION__NV22 = 0x0000026f,
    PH_PERF_SEL_SC6_GFX_PIPE0_TO_1_TRANSITION__NV22    = 0x000002e4,
    PH_PERF_SEL_SC6_GFX_PIPE1_TO_0_TRANSITION__NV22    = 0x000002e5,
    PH_PERF_SEL_SC6_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__NV22 = 0x000002e6,
    PH_PERF_SEL_SC6_GFX_PIPE_EVENT_PROVOKED_TRANSITION__NV22 = 0x000002e7,
    PH_PERF_SEL_SC7_GFX_PIPE0_TO_1_TRANSITION__NV22    = 0x0000035c,
    PH_PERF_SEL_SC7_GFX_PIPE1_TO_0_TRANSITION__NV22    = 0x0000035d,
    PH_PERF_SEL_SC7_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__NV22 = 0x0000035e,
    PH_PERF_SEL_SC7_GFX_PIPE_EVENT_PROVOKED_TRANSITION__NV22 = 0x0000035f,
    PH_PERF_SEL_SC0_GFX_PIPE0_TO_1_TRANSITION__NV23    = 0x00000014,
    PH_PERF_SEL_SC0_GFX_PIPE1_TO_0_TRANSITION__NV23    = 0x00000015,
    PH_PERF_SEL_SC0_GFX_PIPE_PRIM_PROVOKED_TRANSITION__NV23 = 0x00000016,
    PH_PERF_SEL_SC0_GFX_PIPE_EVENT_PROVOKED_TRANSITION__NV23 = 0x00000017,
    PH_PERF_SEL_SC1_GFX_PIPE0_TO_1_TRANSITION__NV23    = 0x0000008c,
    PH_PERF_SEL_SC1_GFX_PIPE1_TO_0_TRANSITION__NV23    = 0x0000008d,
    PH_PERF_SEL_SC1_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__NV23 = 0x0000008e,
    PH_PERF_SEL_SC1_GFX_PIPE_EVENT_PROVOKED_TRANSITION__NV23 = 0x0000008f,
    PH_PERF_SEL_SC2_GFX_PIPE0_TO_1_TRANSITION__NV23    = 0x00000104,
    PH_PERF_SEL_SC2_GFX_PIPE1_TO_0_TRANSITION__NV23    = 0x00000105,
    PH_PERF_SEL_SC2_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__NV23 = 0x00000106,
    PH_PERF_SEL_SC2_GFX_PIPE_EVENT_PROVOKED_TRANSITION__NV23 = 0x00000107,
    PH_PERF_SEL_SC3_GFX_PIPE0_TO_1_TRANSITION__NV23    = 0x0000017c,
    PH_PERF_SEL_SC3_GFX_PIPE1_TO_0_TRANSITION__NV23    = 0x0000017d,
    PH_PERF_SEL_SC3_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__NV23 = 0x0000017e,
    PH_PERF_SEL_SC3_GFX_PIPE_EVENT_PROVOKED_TRANSITION__NV23 = 0x0000017f,
    PH_PERF_SEL_SC4_GFX_PIPE0_TO_1_TRANSITION__NV23    = 0x000001f4,
    PH_PERF_SEL_SC4_GFX_PIPE1_TO_0_TRANSITION__NV23    = 0x000001f5,
    PH_PERF_SEL_SC4_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__NV23 = 0x000001f6,
    PH_PERF_SEL_SC4_GFX_PIPE_EVENT_PROVOKED_TRANSITION__NV23 = 0x000001f7,
    PH_PERF_SEL_SC5_GFX_PIPE0_TO_1_TRANSITION__NV23    = 0x0000026c,
    PH_PERF_SEL_SC5_GFX_PIPE1_TO_0_TRANSITION__NV23    = 0x0000026d,
    PH_PERF_SEL_SC5_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__NV23 = 0x0000026e,
    PH_PERF_SEL_SC5_GFX_PIPE_EVENT_PROVOKED_TRANSITION__NV23 = 0x0000026f,
    PH_PERF_SEL_SC6_GFX_PIPE0_TO_1_TRANSITION__NV23    = 0x000002e4,
    PH_PERF_SEL_SC6_GFX_PIPE1_TO_0_TRANSITION__NV23    = 0x000002e5,
    PH_PERF_SEL_SC6_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__NV23 = 0x000002e6,
    PH_PERF_SEL_SC6_GFX_PIPE_EVENT_PROVOKED_TRANSITION__NV23 = 0x000002e7,
    PH_PERF_SEL_SC7_GFX_PIPE0_TO_1_TRANSITION__NV23    = 0x0000035c,
    PH_PERF_SEL_SC7_GFX_PIPE1_TO_0_TRANSITION__NV23    = 0x0000035d,
    PH_PERF_SEL_SC7_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__NV23 = 0x0000035e,
    PH_PERF_SEL_SC7_GFX_PIPE_EVENT_PROVOKED_TRANSITION__NV23 = 0x0000035f,
    PH_PERF_SEL_SC0_GFX_PIPE0_TO_1_TRANSITION__NV24    = 0x00000014,
    PH_PERF_SEL_SC0_GFX_PIPE1_TO_0_TRANSITION__NV24    = 0x00000015,
    PH_PERF_SEL_SC0_GFX_PIPE_PRIM_PROVOKED_TRANSITION__NV24 = 0x00000016,
    PH_PERF_SEL_SC0_GFX_PIPE_EVENT_PROVOKED_TRANSITION__NV24 = 0x00000017,
    PH_PERF_SEL_SC1_GFX_PIPE0_TO_1_TRANSITION__NV24    = 0x0000008c,
    PH_PERF_SEL_SC1_GFX_PIPE1_TO_0_TRANSITION__NV24    = 0x0000008d,
    PH_PERF_SEL_SC1_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__NV24 = 0x0000008e,
    PH_PERF_SEL_SC1_GFX_PIPE_EVENT_PROVOKED_TRANSITION__NV24 = 0x0000008f,
    PH_PERF_SEL_SC2_GFX_PIPE0_TO_1_TRANSITION__NV24    = 0x00000104,
    PH_PERF_SEL_SC2_GFX_PIPE1_TO_0_TRANSITION__NV24    = 0x00000105,
    PH_PERF_SEL_SC2_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__NV24 = 0x00000106,
    PH_PERF_SEL_SC2_GFX_PIPE_EVENT_PROVOKED_TRANSITION__NV24 = 0x00000107,
    PH_PERF_SEL_SC3_GFX_PIPE0_TO_1_TRANSITION__NV24    = 0x0000017c,
    PH_PERF_SEL_SC3_GFX_PIPE1_TO_0_TRANSITION__NV24    = 0x0000017d,
    PH_PERF_SEL_SC3_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__NV24 = 0x0000017e,
    PH_PERF_SEL_SC3_GFX_PIPE_EVENT_PROVOKED_TRANSITION__NV24 = 0x0000017f,
    PH_PERF_SEL_SC4_GFX_PIPE0_TO_1_TRANSITION__NV24    = 0x000001f4,
    PH_PERF_SEL_SC4_GFX_PIPE1_TO_0_TRANSITION__NV24    = 0x000001f5,
    PH_PERF_SEL_SC4_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__NV24 = 0x000001f6,
    PH_PERF_SEL_SC4_GFX_PIPE_EVENT_PROVOKED_TRANSITION__NV24 = 0x000001f7,
    PH_PERF_SEL_SC5_GFX_PIPE0_TO_1_TRANSITION__NV24    = 0x0000026c,
    PH_PERF_SEL_SC5_GFX_PIPE1_TO_0_TRANSITION__NV24    = 0x0000026d,
    PH_PERF_SEL_SC5_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__NV24 = 0x0000026e,
    PH_PERF_SEL_SC5_GFX_PIPE_EVENT_PROVOKED_TRANSITION__NV24 = 0x0000026f,
    PH_PERF_SEL_SC6_GFX_PIPE0_TO_1_TRANSITION__NV24    = 0x000002e4,
    PH_PERF_SEL_SC6_GFX_PIPE1_TO_0_TRANSITION__NV24    = 0x000002e5,
    PH_PERF_SEL_SC6_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__NV24 = 0x000002e6,
    PH_PERF_SEL_SC6_GFX_PIPE_EVENT_PROVOKED_TRANSITION__NV24 = 0x000002e7,
    PH_PERF_SEL_SC7_GFX_PIPE0_TO_1_TRANSITION__NV24    = 0x0000035c,
    PH_PERF_SEL_SC7_GFX_PIPE1_TO_0_TRANSITION__NV24    = 0x0000035d,
    PH_PERF_SEL_SC7_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__NV24 = 0x0000035e,
    PH_PERF_SEL_SC7_GFX_PIPE_EVENT_PROVOKED_TRANSITION__NV24 = 0x0000035f,
    PH_PERF_SEL_SC0_GFX_PIPE0_TO_1_TRANSITION__RAPHAEL = 0x00000014,
    PH_PERF_SEL_SC0_GFX_PIPE1_TO_0_TRANSITION__RAPHAEL = 0x00000015,
    PH_PERF_SEL_SC0_GFX_PIPE_PRIM_PROVOKED_TRANSITION__RAPHAEL = 0x00000016,
    PH_PERF_SEL_SC0_GFX_PIPE_EVENT_PROVOKED_TRANSITION__RAPHAEL = 0x00000017,
    PH_PERF_SEL_SC1_GFX_PIPE0_TO_1_TRANSITION__RAPHAEL = 0x0000008c,
    PH_PERF_SEL_SC1_GFX_PIPE1_TO_0_TRANSITION__RAPHAEL = 0x0000008d,
    PH_PERF_SEL_SC1_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__RAPHAEL = 0x0000008e,
    PH_PERF_SEL_SC1_GFX_PIPE_EVENT_PROVOKED_TRANSITION__RAPHAEL = 0x0000008f,
    PH_PERF_SEL_SC2_GFX_PIPE0_TO_1_TRANSITION__RAPHAEL = 0x00000104,
    PH_PERF_SEL_SC2_GFX_PIPE1_TO_0_TRANSITION__RAPHAEL = 0x00000105,
    PH_PERF_SEL_SC2_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__RAPHAEL = 0x00000106,
    PH_PERF_SEL_SC2_GFX_PIPE_EVENT_PROVOKED_TRANSITION__RAPHAEL = 0x00000107,
    PH_PERF_SEL_SC3_GFX_PIPE0_TO_1_TRANSITION__RAPHAEL = 0x0000017c,
    PH_PERF_SEL_SC3_GFX_PIPE1_TO_0_TRANSITION__RAPHAEL = 0x0000017d,
    PH_PERF_SEL_SC3_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__RAPHAEL = 0x0000017e,
    PH_PERF_SEL_SC3_GFX_PIPE_EVENT_PROVOKED_TRANSITION__RAPHAEL = 0x0000017f,
    PH_PERF_SEL_SC4_GFX_PIPE0_TO_1_TRANSITION__RAPHAEL = 0x000001f4,
    PH_PERF_SEL_SC4_GFX_PIPE1_TO_0_TRANSITION__RAPHAEL = 0x000001f5,
    PH_PERF_SEL_SC4_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__RAPHAEL = 0x000001f6,
    PH_PERF_SEL_SC4_GFX_PIPE_EVENT_PROVOKED_TRANSITION__RAPHAEL = 0x000001f7,
    PH_PERF_SEL_SC5_GFX_PIPE0_TO_1_TRANSITION__RAPHAEL = 0x0000026c,
    PH_PERF_SEL_SC5_GFX_PIPE1_TO_0_TRANSITION__RAPHAEL = 0x0000026d,
    PH_PERF_SEL_SC5_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__RAPHAEL = 0x0000026e,
    PH_PERF_SEL_SC5_GFX_PIPE_EVENT_PROVOKED_TRANSITION__RAPHAEL = 0x0000026f,
    PH_PERF_SEL_SC6_GFX_PIPE0_TO_1_TRANSITION__RAPHAEL = 0x000002e4,
    PH_PERF_SEL_SC6_GFX_PIPE1_TO_0_TRANSITION__RAPHAEL = 0x000002e5,
    PH_PERF_SEL_SC6_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__RAPHAEL = 0x000002e6,
    PH_PERF_SEL_SC6_GFX_PIPE_EVENT_PROVOKED_TRANSITION__RAPHAEL = 0x000002e7,
    PH_PERF_SEL_SC7_GFX_PIPE0_TO_1_TRANSITION__RAPHAEL = 0x0000035c,
    PH_PERF_SEL_SC7_GFX_PIPE1_TO_0_TRANSITION__RAPHAEL = 0x0000035d,
    PH_PERF_SEL_SC7_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__RAPHAEL = 0x0000035e,
    PH_PERF_SEL_SC7_GFX_PIPE_EVENT_PROVOKED_TRANSITION__RAPHAEL = 0x0000035f,
    PH_PERF_SEL_SC0_GFX_PIPE0_TO_1_TRANSITION__REMBRANDT = 0x00000014,
    PH_PERF_SEL_SC0_GFX_PIPE1_TO_0_TRANSITION__REMBRANDT = 0x00000015,
    PH_PERF_SEL_SC0_GFX_PIPE_PRIM_PROVOKED_TRANSITION__REMBRANDT = 0x00000016,
    PH_PERF_SEL_SC0_GFX_PIPE_EVENT_PROVOKED_TRANSITION__REMBRANDT = 0x00000017,
    PH_PERF_SEL_SC1_GFX_PIPE0_TO_1_TRANSITION__REMBRANDT = 0x0000008c,
    PH_PERF_SEL_SC1_GFX_PIPE1_TO_0_TRANSITION__REMBRANDT = 0x0000008d,
    PH_PERF_SEL_SC1_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__REMBRANDT = 0x0000008e,
    PH_PERF_SEL_SC1_GFX_PIPE_EVENT_PROVOKED_TRANSITION__REMBRANDT = 0x0000008f,
    PH_PERF_SEL_SC2_GFX_PIPE0_TO_1_TRANSITION__REMBRANDT = 0x00000104,
    PH_PERF_SEL_SC2_GFX_PIPE1_TO_0_TRANSITION__REMBRANDT = 0x00000105,
    PH_PERF_SEL_SC2_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__REMBRANDT = 0x00000106,
    PH_PERF_SEL_SC2_GFX_PIPE_EVENT_PROVOKED_TRANSITION__REMBRANDT = 0x00000107,
    PH_PERF_SEL_SC3_GFX_PIPE0_TO_1_TRANSITION__REMBRANDT = 0x0000017c,
    PH_PERF_SEL_SC3_GFX_PIPE1_TO_0_TRANSITION__REMBRANDT = 0x0000017d,
    PH_PERF_SEL_SC3_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__REMBRANDT = 0x0000017e,
    PH_PERF_SEL_SC3_GFX_PIPE_EVENT_PROVOKED_TRANSITION__REMBRANDT = 0x0000017f,
    PH_PERF_SEL_SC4_GFX_PIPE0_TO_1_TRANSITION__REMBRANDT = 0x000001f4,
    PH_PERF_SEL_SC4_GFX_PIPE1_TO_0_TRANSITION__REMBRANDT = 0x000001f5,
    PH_PERF_SEL_SC4_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__REMBRANDT = 0x000001f6,
    PH_PERF_SEL_SC4_GFX_PIPE_EVENT_PROVOKED_TRANSITION__REMBRANDT = 0x000001f7,
    PH_PERF_SEL_SC5_GFX_PIPE0_TO_1_TRANSITION__REMBRANDT = 0x0000026c,
    PH_PERF_SEL_SC5_GFX_PIPE1_TO_0_TRANSITION__REMBRANDT = 0x0000026d,
    PH_PERF_SEL_SC5_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__REMBRANDT = 0x0000026e,
    PH_PERF_SEL_SC5_GFX_PIPE_EVENT_PROVOKED_TRANSITION__REMBRANDT = 0x0000026f,
    PH_PERF_SEL_SC6_GFX_PIPE0_TO_1_TRANSITION__REMBRANDT = 0x000002e4,
    PH_PERF_SEL_SC6_GFX_PIPE1_TO_0_TRANSITION__REMBRANDT = 0x000002e5,
    PH_PERF_SEL_SC6_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__REMBRANDT = 0x000002e6,
    PH_PERF_SEL_SC6_GFX_PIPE_EVENT_PROVOKED_TRANSITION__REMBRANDT = 0x000002e7,
    PH_PERF_SEL_SC7_GFX_PIPE0_TO_1_TRANSITION__REMBRANDT = 0x0000035c,
    PH_PERF_SEL_SC7_GFX_PIPE1_TO_0_TRANSITION__REMBRANDT = 0x0000035d,
    PH_PERF_SEL_SC7_GFX_PIPE_EOP_PRIM_PROVOKED_TRANSITION__REMBRANDT = 0x0000035e,
    PH_PERF_SEL_SC7_GFX_PIPE_EVENT_PROVOKED_TRANSITION__REMBRANDT = 0x0000035f,
} PH_PERFCNT_SEL;

constexpr unsigned int MaxPhPerfcntSelGfx101           = PH_PERF_SEL_SC7_PA7_DEALLOC_4_0_RD__GFX10;
constexpr unsigned int MaxPhPerfcntSelGfx10Vrs         = PH_PERF_SEL_8_SC_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_FULL__GFX103PLUS;
constexpr unsigned int MaxPhPerfcntSelGfx11            = PH_PERF_SC7_FIFO_STATUS_3__GFX11;

typedef enum PipeConfig {
    ADDR_SURF_P2                                       = 0x00000000,
    ADDR_SURF_P2_RESERVED0                             = 0x00000001,
    ADDR_SURF_P2_RESERVED1                             = 0x00000002,
    ADDR_SURF_P2_RESERVED2                             = 0x00000003,
    ADDR_SURF_P4_8x16                                  = 0x00000004,
    ADDR_SURF_P4_16x16                                 = 0x00000005,
    ADDR_SURF_P4_16x32                                 = 0x00000006,
    ADDR_SURF_P4_32x32                                 = 0x00000007,
    ADDR_SURF_P8_16x16_8x16                            = 0x00000008,
    ADDR_SURF_P8_16x32_8x16                            = 0x00000009,
    ADDR_SURF_P8_32x32_8x16                            = 0x0000000a,
    ADDR_SURF_P8_16x32_16x16                           = 0x0000000b,
    ADDR_SURF_P8_32x32_16x16                           = 0x0000000c,
    ADDR_SURF_P8_32x32_16x32                           = 0x0000000d,
    ADDR_SURF_P8_32x64_32x32                           = 0x0000000e,
    ADDR_SURF_P8_RESERVED0                             = 0x0000000f,
    ADDR_SURF_P16_32x32_8x16                           = 0x00000010,
    ADDR_SURF_P16_32x32_16x16                          = 0x00000011,
    ADDR_SURF_P16                                      = 0x00000012,
} PipeConfig;

typedef enum PixelPipeCounterId {
    PIXEL_PIPE_OCCLUSION_COUNT_0                       = 0x00000000,
    PIXEL_PIPE_OCCLUSION_COUNT_1                       = 0x00000001,
    PIXEL_PIPE_OCCLUSION_COUNT_2                       = 0x00000002,
    PIXEL_PIPE_OCCLUSION_COUNT_3                       = 0x00000003,
    PIXEL_PIPE_SCREEN_MIN_EXTENTS_0                    = 0x00000004,
    PIXEL_PIPE_SCREEN_MAX_EXTENTS_0                    = 0x00000005,
    PIXEL_PIPE_SCREEN_MIN_EXTENTS_1                    = 0x00000006,
    PIXEL_PIPE_SCREEN_MAX_EXTENTS_1                    = 0x00000007,
} PixelPipeCounterId;

typedef enum PixelPipeStride {
    PIXEL_PIPE_STRIDE_32_BITS                          = 0x00000000,
    PIXEL_PIPE_STRIDE_64_BITS                          = 0x00000001,
    PIXEL_PIPE_STRIDE_128_BITS                         = 0x00000002,
    PIXEL_PIPE_STRIDE_256_BITS                         = 0x00000003,
} PixelPipeStride;

typedef enum QuadExportFormat {
    EXPORT_UNUSED                                      = 0x00000000,
    EXPORT_32_R                                        = 0x00000001,
    EXPORT_32_GR                                       = 0x00000002,
    EXPORT_32_AR                                       = 0x00000003,
    EXPORT_FP16_ABGR                                   = 0x00000004,
    EXPORT_UNSIGNED16_ABGR                             = 0x00000005,
    EXPORT_SIGNED16_ABGR                               = 0x00000006,
    EXPORT_32_ABGR                                     = 0x00000007,
    EXPORT_32BPP_8PIX                                  = 0x00000008,
    EXPORT_16_16_UNSIGNED_8PIX                         = 0x00000009,
    EXPORT_16_16_SIGNED_8PIX                           = 0x0000000a,
    EXPORT_16_16_FLOAT_8PIX                            = 0x0000000b,
} QuadExportFormat;

typedef enum QuadExportFormatOld {
    EXPORT_4P_32BPC_ABGR                               = 0x00000000,
    EXPORT_4P_16BPC_ABGR                               = 0x00000001,
    EXPORT_4P_32BPC_GR                                 = 0x00000002,
    EXPORT_4P_32BPC_AR                                 = 0x00000003,
    EXPORT_2P_32BPC_ABGR                               = 0x00000004,
    EXPORT_8P_32BPC_R                                  = 0x00000005,
} QuadExportFormatOld;

typedef enum ReadPolicy {
    CACHE_LRU_RD                                       = 0x00000000,
    CACHE_STREAM_RD                                    = 0x00000001,
    CACHE_NOA                                          = 0x00000002,
    RESERVED_RDPOLICY                                  = 0x00000003,
} ReadPolicy;

typedef enum RLC_PERFCOUNTER_SEL {
    RLC_PERF_SEL_POWER_FEATURE_0                       = 0x00000000,
    RLC_PERF_SEL_POWER_FEATURE_1                       = 0x00000001,
    RLC_PERF_SEL_CP_INTERRUPT                          = 0x00000002,
    RLC_PERF_SEL_GRBM_INTERRUPT                        = 0x00000003,
    RLC_PERF_SEL_SPM_INTERRUPT                         = 0x00000004,
    RLC_PERF_SEL_IH_INTERRUPT                          = 0x00000005,
    RLC_PERF_SEL_SERDES_COMMAND_WRITE                  = 0x00000006,
} RLC_PERFCOUNTER_SEL;

constexpr unsigned int MaxRlcPerfcounterSel            = RLC_PERF_SEL_SERDES_COMMAND_WRITE;

typedef enum RLC_PERFMON_STATE {
    RLC_PERFMON_STATE_RESET                            = 0x00000000,
    RLC_PERFMON_STATE_ENABLE                           = 0x00000001,
    RLC_PERFMON_STATE_DISABLE                          = 0x00000002,
    RLC_PERFMON_STATE_RESERVED_3                       = 0x00000003,
    RLC_PERFMON_STATE_RESERVED_4                       = 0x00000004,
    RLC_PERFMON_STATE_RESERVED_5                       = 0x00000005,
    RLC_PERFMON_STATE_RESERVED_6                       = 0x00000006,
    RLC_PERFMON_STATE_ROLLOVER                         = 0x00000007,
} RLC_PERFMON_STATE;

typedef enum RMIPerfSel {
    RMI_PERF_SEL_NONE                                  = 0x00000000,
    RMI_PERF_SEL_BUSY                                  = 0x00000001,
    RMI_PERF_SEL_REG_CLK_VLD                           = 0x00000002,
    RMI_PERF_SEL_DYN_CLK_CMN_VLD                       = 0x00000003,
    RMI_PERF_SEL_DYN_CLK_RB_VLD                        = 0x00000004,
    RMI_PERF_SEL_DYN_CLK_PERF_VLD                      = 0x00000005,
    RMI_PERF_SEL_PERF_WINDOW                           = 0x00000006,
    RMI_PERF_SEL_EVENT_SEND                            = 0x00000007,
    RMI_PERF_SEL_RMI_INVALIDATION_ATC_REQ_VMID0__GFX10 = 0x00000008,
    RMI_PERF_SEL_RMI_INVALIDATION_ATC_REQ_VMID1__GFX10 = 0x00000009,
    RMI_PERF_SEL_RMI_INVALIDATION_ATC_REQ_VMID2__GFX10 = 0x0000000a,
    RMI_PERF_SEL_RMI_INVALIDATION_ATC_REQ_VMID3__GFX10 = 0x0000000b,
    RMI_PERF_SEL_RMI_INVALIDATION_ATC_REQ_VMID4__GFX10 = 0x0000000c,
    RMI_PERF_SEL_RMI_INVALIDATION_ATC_REQ_VMID5__GFX10 = 0x0000000d,
    RMI_PERF_SEL_RMI_INVALIDATION_ATC_REQ_VMID6__GFX10 = 0x0000000e,
    RMI_PERF_SEL_RMI_INVALIDATION_ATC_REQ_VMID7__GFX10 = 0x0000000f,
    RMI_PERF_SEL_RMI_INVALIDATION_ATC_REQ_VMID8__GFX10 = 0x00000010,
    RMI_PERF_SEL_RMI_INVALIDATION_ATC_REQ_VMID9__GFX10 = 0x00000011,
    RMI_PERF_SEL_RMI_INVALIDATION_ATC_REQ_VMID10__GFX10 = 0x00000012,
    RMI_PERF_SEL_RMI_INVALIDATION_ATC_REQ_VMID11__GFX10 = 0x00000013,
    RMI_PERF_SEL_RMI_INVALIDATION_ATC_REQ_VMID12__GFX10 = 0x00000014,
    RMI_PERF_SEL_RMI_INVALIDATION_ATC_REQ_VMID13__GFX10 = 0x00000015,
    RMI_PERF_SEL_RMI_INVALIDATION_ATC_REQ_VMID14__GFX10 = 0x00000016,
    RMI_PERF_SEL_RMI_INVALIDATION_ATC_REQ_VMID15__GFX10 = 0x00000017,
    RMI_PERF_SEL_RMI_INVALIDATION_ATC_REQ_VMID_ALL__GFX10 = 0x00000018,
    RMI_PERF_SEL_RMI_INVALIDATION_REQ_START_FINISH_VMID0__GFX10 = 0x00000019,
    RMI_PERF_SEL_RMI_INVALIDATION_REQ_START_FINISH_VMID1__GFX10 = 0x0000001a,
    RMI_PERF_SEL_RMI_INVALIDATION_REQ_START_FINISH_VMID2__GFX10 = 0x0000001b,
    RMI_PERF_SEL_RMI_INVALIDATION_REQ_START_FINISH_VMID3__GFX10 = 0x0000001c,
    RMI_PERF_SEL_RMI_INVALIDATION_REQ_START_FINISH_VMID4__GFX10 = 0x0000001d,
    RMI_PERF_SEL_RMI_INVALIDATION_REQ_START_FINISH_VMID5__GFX10 = 0x0000001e,
    RMI_PERF_SEL_RMI_INVALIDATION_REQ_START_FINISH_VMID6__GFX10 = 0x0000001f,
    RMI_PERF_SEL_RMI_INVALIDATION_REQ_START_FINISH_VMID7__GFX10 = 0x00000020,
    RMI_PERF_SEL_RMI_INVALIDATION_REQ_START_FINISH_VMID8__GFX10 = 0x00000021,
    RMI_PERF_SEL_RMI_INVALIDATION_REQ_START_FINISH_VMID9__GFX10 = 0x00000022,
    RMI_PERF_SEL_RMI_INVALIDATION_REQ_START_FINISH_VMID10__GFX10 = 0x00000023,
    RMI_PERF_SEL_RMI_INVALIDATION_REQ_START_FINISH_VMID11__GFX10 = 0x00000024,
    RMI_PERF_SEL_RMI_INVALIDATION_REQ_START_FINISH_VMID12__GFX10 = 0x00000025,
    RMI_PERF_SEL_RMI_INVALIDATION_REQ_START_FINISH_VMID13__GFX10 = 0x00000026,
    RMI_PERF_SEL_RMI_INVALIDATION_REQ_START_FINISH_VMID14__GFX10 = 0x00000027,
    RMI_PERF_SEL_RMI_INVALIDATION_REQ_START_FINISH_VMID15__GFX10 = 0x00000028,
    RMI_PERF_SEL_RMI_INVALIDATION_REQ_START_FINISH_VMID_ALL__GFX10 = 0x00000029,
    RMI_PERF_SEL_UTCL1_TRANSLATION_MISS__GFX10         = 0x0000002a,
    RMI_PERF_SEL_UTCL1_PERMISSION_MISS__GFX10          = 0x0000002b,
    RMI_PERF_SEL_UTCL1_TRANSLATION_HIT__GFX10          = 0x0000002c,
    RMI_PERF_SEL_UTCL1_REQUEST__GFX10                  = 0x0000002d,
    RMI_PERF_SEL_UTCL1_STALL_INFLIGHT_MAX__GFX10       = 0x0000002e,
    RMI_PERF_SEL_UTCL1_STALL_LRU_INFLIGHT__GFX10       = 0x0000002f,
    RMI_PERF_SEL_UTCL1_LFIFO_FULL__GFX10               = 0x00000030,
    RMI_PERF_SEL_UTCL1_STALL_LFIFO_NOT_RES__GFX10      = 0x00000031,
    RMI_PERF_SEL_UTCL1_STALL_UTCL2_REQ_OUT_OF_CREDITS__GFX10 = 0x00000032,
    RMI_PERF_SEL_UTCL1_STALL_MISSFIFO_FULL__GFX10      = 0x00000033,
    RMI_PERF_SEL_UTCL1_HIT_FIFO_FULL__GFX10            = 0x00000034,
    RMI_PERF_SEL_UTCL1_STALL_MULTI_MISS__GFX10         = 0x00000035,
    RMI_PERF_SEL_RB_RMI_WRREQ_ALL_CID__GFX10           = 0x00000036,
    RMI_PERF_SEL_RB_RMI_WRREQ_TO_WRRET_BUSY__GFX10     = 0x00000037,
    RMI_PERF_SEL_RB_RMI_WRREQ_CID0__GFX10              = 0x00000038,
    RMI_PERF_SEL_RB_RMI_WRREQ_CID1__GFX10              = 0x00000039,
    RMI_PERF_SEL_RB_RMI_WRREQ_CID2__GFX10              = 0x0000003a,
    RMI_PERF_SEL_RB_RMI_WRREQ_CID3__GFX10              = 0x0000003b,
    RMI_PERF_SEL_RB_RMI_WRREQ_CID4__GFX10              = 0x0000003c,
    RMI_PERF_SEL_RB_RMI_WRREQ_CID5__GFX10              = 0x0000003d,
    RMI_PERF_SEL_RB_RMI_WRREQ_CID6__GFX10              = 0x0000003e,
    RMI_PERF_SEL_RB_RMI_WRREQ_CID7__GFX10              = 0x0000003f,
    RMI_PERF_SEL_RB_RMI_32BWRREQ_INFLIGHT_ALL_ORONE_CID__GFX10 = 0x00000040,
    RMI_PERF_SEL_RB_RMI_WRREQ_BURST_LENGTH_ALL_ORONE_CID__GFX10 = 0x00000041,
    RMI_PERF_SEL_RB_RMI_WRREQ_BURST_ALL_ORONE_CID__GFX10 = 0x00000042,
    RMI_PERF_SEL_RB_RMI_WRREQ_RESIDENCY__GFX10         = 0x00000043,
    RMI_PERF_SEL_RMI_RB_WRRET_VALID_ALL_CID__GFX10     = 0x00000044,
    RMI_PERF_SEL_RMI_RB_WRRET_VALID_CID0__GFX10        = 0x00000045,
    RMI_PERF_SEL_RMI_RB_WRRET_VALID_CID1__GFX10        = 0x00000046,
    RMI_PERF_SEL_RMI_RB_WRRET_VALID_CID2__GFX10        = 0x00000047,
    RMI_PERF_SEL_RMI_RB_WRRET_VALID_CID3__GFX10        = 0x00000048,
    RMI_PERF_SEL_RMI_RB_WRRET_VALID_CID4__GFX10        = 0x00000049,
    RMI_PERF_SEL_RMI_RB_WRRET_VALID_CID5__GFX10        = 0x0000004a,
    RMI_PERF_SEL_RMI_RB_WRRET_VALID_CID6__GFX10        = 0x0000004b,
    RMI_PERF_SEL_RMI_RB_WRRET_VALID_CID7__GFX10        = 0x0000004c,
    RMI_PERF_SEL_RMI_RB_WRRET_VALID_NACK0__GFX10       = 0x0000004d,
    RMI_PERF_SEL_RMI_RB_WRRET_VALID_NACK1__GFX10       = 0x0000004e,
    RMI_PERF_SEL_RMI_RB_WRRET_VALID_NACK2__GFX10       = 0x0000004f,
    RMI_PERF_SEL_RMI_RB_WRRET_VALID_NACK3__GFX10       = 0x00000050,
    RMI_PERF_SEL_RB_RMI_32BRDREQ_ALL_CID__GFX10        = 0x00000051,
    RMI_PERF_SEL_RB_RMI_RDREQ_ALL_CID__GFX10           = 0x00000052,
    RMI_PERF_SEL_RB_RMI_RDREQ_TO_RDRET_BUSY__GFX10     = 0x00000053,
    RMI_PERF_SEL_RB_RMI_32BRDREQ_CID0__GFX10           = 0x00000054,
    RMI_PERF_SEL_RB_RMI_32BRDREQ_CID1__GFX10           = 0x00000055,
    RMI_PERF_SEL_RB_RMI_32BRDREQ_CID2__GFX10           = 0x00000056,
    RMI_PERF_SEL_RB_RMI_32BRDREQ_CID3__GFX10           = 0x00000057,
    RMI_PERF_SEL_RB_RMI_32BRDREQ_CID4__GFX10           = 0x00000058,
    RMI_PERF_SEL_RB_RMI_32BRDREQ_CID5__GFX10           = 0x00000059,
    RMI_PERF_SEL_RB_RMI_32BRDREQ_CID6__GFX10           = 0x0000005a,
    RMI_PERF_SEL_RB_RMI_32BRDREQ_CID7__GFX10           = 0x0000005b,
    RMI_PERF_SEL_RB_RMI_RDREQ_CID0__GFX10              = 0x0000005c,
    RMI_PERF_SEL_RB_RMI_RDREQ_CID1__GFX10              = 0x0000005d,
    RMI_PERF_SEL_RB_RMI_RDREQ_CID2__GFX10              = 0x0000005e,
    RMI_PERF_SEL_RB_RMI_RDREQ_CID3__GFX10              = 0x0000005f,
    RMI_PERF_SEL_RB_RMI_RDREQ_CID4__GFX10              = 0x00000060,
    RMI_PERF_SEL_RB_RMI_RDREQ_CID5__GFX10              = 0x00000061,
    RMI_PERF_SEL_RB_RMI_RDREQ_CID6__GFX10              = 0x00000062,
    RMI_PERF_SEL_RB_RMI_RDREQ_CID7__GFX10              = 0x00000063,
    RMI_PERF_SEL_RB_RMI_32BRDREQ_INFLIGHT_ALL_ORONE_CID__GFX10 = 0x00000064,
    RMI_PERF_SEL_RB_RMI_RDREQ_BURST_LENGTH_ALL_ORONE_CID__GFX10 = 0x00000065,
    RMI_PERF_SEL_RB_RMI_RDREQ_BURST_ALL_ORONE_CID__GFX10 = 0x00000066,
    RMI_PERF_SEL_RB_RMI_RDREQ_RESIDENCY__GFX10         = 0x00000067,
    RMI_PERF_SEL_RMI_RB_32BRDRET_VALID_ALL_CID__GFX10  = 0x00000068,
    RMI_PERF_SEL_RMI_RB_32BRDRET_VALID_CID0__GFX10     = 0x00000069,
    RMI_PERF_SEL_RMI_RB_32BRDRET_VALID_CID1__GFX10     = 0x0000006a,
    RMI_PERF_SEL_RMI_RB_32BRDRET_VALID_CID2__GFX10     = 0x0000006b,
    RMI_PERF_SEL_RMI_RB_32BRDRET_VALID_CID3__GFX10     = 0x0000006c,
    RMI_PERF_SEL_RMI_RB_32BRDRET_VALID_CID4__GFX10     = 0x0000006d,
    RMI_PERF_SEL_RMI_RB_32BRDRET_VALID_CID5__GFX10     = 0x0000006e,
    RMI_PERF_SEL_RMI_RB_32BRDRET_VALID_CID6__GFX10     = 0x0000006f,
    RMI_PERF_SEL_RMI_RB_32BRDRET_VALID_CID7__GFX10     = 0x00000070,
    RMI_PERF_SEL_RMI_RB_32BRDRET_VALID_NACK0__GFX10    = 0x00000071,
    RMI_PERF_SEL_RMI_RB_32BRDRET_VALID_NACK1__GFX10    = 0x00000072,
    RMI_PERF_SEL_RMI_RB_32BRDRET_VALID_NACK2__GFX10    = 0x00000073,
    RMI_PERF_SEL_RMI_RB_32BRDRET_VALID_NACK3__GFX10    = 0x00000074,
    RMI_PERF_SEL_RB_RMI_WR_FIFO_MAX__GFX10             = 0x00000075,
    RMI_PERF_SEL_RB_RMI_WR_FIFO_EMPTY__GFX10           = 0x00000076,
    RMI_PERF_SEL_RB_RMI_WR_IDLE__GFX10                 = 0x00000077,
    RMI_PERF_SEL_RB_RMI_WR_STARVE__GFX10               = 0x00000078,
    RMI_PERF_SEL_RB_RMI_WR_STALL__GFX10                = 0x00000079,
    RMI_PERF_SEL_RB_RMI_WR_BUSY__GFX10                 = 0x0000007a,
    RMI_PERF_SEL_RB_RMI_WR_INTF_BUSY__GFX10            = 0x0000007b,
    RMI_PERF_SEL_RB_RMI_RD_FIFO_MAX__GFX10             = 0x0000007c,
    RMI_PERF_SEL_RB_RMI_RD_FIFO_EMPTY__GFX10           = 0x0000007d,
    RMI_PERF_SEL_RB_RMI_RD_IDLE__GFX10                 = 0x0000007e,
    RMI_PERF_SEL_RB_RMI_RD_STARVE__GFX10               = 0x0000007f,
    RMI_PERF_SEL_RB_RMI_RD_STALL__GFX10                = 0x00000080,
    RMI_PERF_SEL_RB_RMI_RD_BUSY__GFX10                 = 0x00000081,
    RMI_PERF_SEL_RB_RMI_RD_INTF_BUSY__GFX10            = 0x00000082,
    RMI_PERF_SEL_RMI_TC_64BWRREQ_ALL_ORONE_CID__GFX10  = 0x00000083,
    RMI_PERF_SEL_RMI_TC_64BRDREQ_ALL_ORONE_CID__GFX10  = 0x00000084,
    RMI_PERF_SEL_RMI_TC_WRREQ_ALL_CID__GFX10           = 0x00000085,
    RMI_PERF_SEL_RMI_TC_REQ_BUSY__GFX10                = 0x00000086,
    RMI_PERF_SEL_RMI_TC_WRREQ_CID0__GFX10              = 0x00000087,
    RMI_PERF_SEL_RMI_TC_WRREQ_CID1__GFX10              = 0x00000088,
    RMI_PERF_SEL_RMI_TC_WRREQ_CID2__GFX10              = 0x00000089,
    RMI_PERF_SEL_RMI_TC_WRREQ_CID3__GFX10              = 0x0000008a,
    RMI_PERF_SEL_RMI_TC_WRREQ_CID4__GFX10              = 0x0000008b,
    RMI_PERF_SEL_RMI_TC_WRREQ_CID5__GFX10              = 0x0000008c,
    RMI_PERF_SEL_RMI_TC_WRREQ_CID6__GFX10              = 0x0000008d,
    RMI_PERF_SEL_RMI_TC_WRREQ_CID7__GFX10              = 0x0000008e,
    RMI_PERF_SEL_RMI_TC_WRREQ_INFLIGHT_ALL_CID__GFX10  = 0x0000008f,
    RMI_PERF_SEL_TC_RMI_WRRET_VALID_ALL_CID__GFX10     = 0x00000090,
    RMI_PERF_SEL_RMI_TC_RDREQ_ALL_CID__GFX10           = 0x00000091,
    RMI_PERF_SEL_RMI_TC_RDREQ_CID0__GFX10              = 0x00000092,
    RMI_PERF_SEL_RMI_TC_RDREQ_CID1__GFX10              = 0x00000093,
    RMI_PERF_SEL_RMI_TC_RDREQ_CID2__GFX10              = 0x00000094,
    RMI_PERF_SEL_RMI_TC_RDREQ_CID3__GFX10              = 0x00000095,
    RMI_PERF_SEL_RMI_TC_RDREQ_CID4__GFX10              = 0x00000096,
    RMI_PERF_SEL_RMI_TC_RDREQ_CID5__GFX10              = 0x00000097,
    RMI_PERF_SEL_RMI_TC_RDREQ_CID6__GFX10              = 0x00000098,
    RMI_PERF_SEL_RMI_TC_RDREQ_CID7__GFX10              = 0x00000099,
    RMI_PERF_SEL_RMI_TC_STALL_RDREQ__GFX10             = 0x0000009a,
    RMI_PERF_SEL_RMI_TC_STALL_WRREQ__GFX10             = 0x0000009b,
    RMI_PERF_SEL_RMI_TC_STALL_ALLREQ__GFX10            = 0x0000009c,
    RMI_PERF_SEL_RMI_TC_CREDIT_FULL_NO_PENDING_SEND__GFX10 = 0x0000009d,
    RMI_PERF_SEL_RMI_TC_CREDIT_ZERO_PENDING_SEND__GFX10 = 0x0000009e,
    RMI_PERF_SEL_RMI_TC_RDREQ_INFLIGHT_ALL_CID__GFX10  = 0x0000009f,
    RMI_PERF_SEL_TC_RMI_RDRET_VALID_ALL_CID__GFX10     = 0x000000a0,
    RMI_PERF_SEL_UTCL1_BUSY__GFX10                     = 0x000000a1,
    RMI_PERF_SEL_RMI_UTC_REQ__GFX10                    = 0x000000a2,
    RMI_PERF_SEL_RMI_UTC_BUSY__GFX10                   = 0x000000a3,
    RMI_PERF_SEL_UTCL1_UTCL2_REQ__GFX10                = 0x000000a4,
    RMI_PERF_SEL_LEVEL_ADD_UTCL1_TO_UTCL2__GFX10       = 0x000000a5,
    RMI_PERF_SEL_PROBE_UTCL1_XNACK_RETRY__GFX10        = 0x000000a6,
    RMI_PERF_SEL_PROBE_UTCL1_ALL_FAULT__GFX10          = 0x000000a7,
    RMI_PERF_SEL_PROBE_UTCL1_PRT_FAULT__GFX10          = 0x000000a8,
    RMI_PERF_SEL_PROBE_UTCL1_VMID_BYPASS__GFX10        = 0x000000a9,
    RMI_PERF_SEL_PROBE_UTCL1_XNACK_NORETRY_FAULT__GFX10 = 0x000000aa,
    RMI_PERF_SEL_XNACK_FIFO_NUM_USED__GFX10            = 0x000000ab,
    RMI_PERF_SEL_LAT_FIFO_NUM_USED__GFX10              = 0x000000ac,
    RMI_PERF_SEL_LAT_FIFO_BLOCKING_REQ__GFX10          = 0x000000ad,
    RMI_PERF_SEL_LAT_FIFO_NONBLOCKING_REQ__GFX10       = 0x000000ae,
    RMI_PERF_SEL_XNACK_FIFO_FULL__GFX10                = 0x000000af,
    RMI_PERF_SEL_XNACK_FIFO_BUSY__GFX10                = 0x000000b0,
    RMI_PERF_SEL_LAT_FIFO_FULL__GFX10                  = 0x000000b1,
    RMI_PERF_SEL_SKID_FIFO_DEPTH__GFX10                = 0x000000b2,
    RMI_PERF_SEL_TCIW_INFLIGHT_COUNT__GFX10            = 0x000000b3,
    RMI_PERF_SEL_PRT_FIFO_NUM_USED__GFX10              = 0x000000b4,
    RMI_PERF_SEL_PRT_FIFO_REQ__GFX10                   = 0x000000b5,
    RMI_PERF_SEL_PRT_FIFO_BUSY__GFX10                  = 0x000000b6,
    RMI_PERF_SEL_TCIW_REQ__GFX10                       = 0x000000b7,
    RMI_PERF_SEL_TCIW_BUSY__GFX10                      = 0x000000b8,
    RMI_PERF_SEL_SKID_FIFO_REQ__GFX10                  = 0x000000b9,
    RMI_PERF_SEL_SKID_FIFO_BUSY__GFX10                 = 0x000000ba,
    RMI_PERF_SEL_DEMUX_TCIW_RESIDENCY_NACK0__GFX10     = 0x000000bb,
    RMI_PERF_SEL_DEMUX_TCIW_RESIDENCY_NACK1__GFX10     = 0x000000bc,
    RMI_PERF_SEL_DEMUX_TCIW_RESIDENCY_NACK2__GFX10     = 0x000000bd,
    RMI_PERF_SEL_DEMUX_TCIW_RESIDENCY_NACK3__GFX10     = 0x000000be,
    RMI_PERF_SEL_XBAR_PROBEGEN_RTS_RTR__GFX10          = 0x000000bf,
    RMI_PERF_SEL_XBAR_PROBEGEN_RTSB_RTR__GFX10         = 0x000000c0,
    RMI_PERF_SEL_XBAR_PROBEGEN_RTS_RTRB__GFX10         = 0x000000c1,
    RMI_PERF_SEL_XBAR_PROBEGEN_RTSB_RTRB__GFX10        = 0x000000c2,
    RMI_PERF_SEL_DEMUX_TCIW_FORMATTER_RTS_RTR__GFX10   = 0x000000c3,
    RMI_PERF_SEL_DEMUX_TCIW_FORMATTER_RTSB_RTR__GFX10  = 0x000000c4,
    RMI_PERF_SEL_DEMUX_TCIW_FORMATTER_RTS_RTRB__GFX10  = 0x000000c5,
    RMI_PERF_SEL_DEMUX_TCIW_FORMATTER_RTSB_RTRB__GFX10 = 0x000000c6,
    RMI_PERF_SEL_WRREQCONSUMER_XBAR_WRREQ_RTS_RTR__GFX10 = 0x000000c7,
    RMI_PERF_SEL_WRREQCONSUMER_XBAR_WRREQ_RTSB_RTR__GFX10 = 0x000000c8,
    RMI_PERF_SEL_WRREQCONSUMER_XBAR_WRREQ_RTS_RTRB__GFX10 = 0x000000c9,
    RMI_PERF_SEL_WRREQCONSUMER_XBAR_WRREQ_RTSB_RTRB__GFX10 = 0x000000ca,
    RMI_PERF_SEL_RDREQCONSUMER_XBAR_RDREQ_RTS_RTR__GFX10 = 0x000000cb,
    RMI_PERF_SEL_RDREQCONSUMER_XBAR_RDREQ_RTSB_RTR__GFX10 = 0x000000cc,
    RMI_PERF_SEL_RDREQCONSUMER_XBAR_RDREQ_RTS_RTRB__GFX10 = 0x000000cd,
    RMI_PERF_SEL_RDREQCONSUMER_XBAR_RDREQ_RTSB_RTRB__GFX10 = 0x000000ce,
    RMI_PERF_SEL_POP_DEMUX_RTS_RTR__GFX10              = 0x000000cf,
    RMI_PERF_SEL_POP_DEMUX_RTSB_RTR__GFX10             = 0x000000d0,
    RMI_PERF_SEL_POP_DEMUX_RTS_RTRB__GFX10             = 0x000000d1,
    RMI_PERF_SEL_POP_DEMUX_RTSB_RTRB__GFX10            = 0x000000d2,
    RMI_PERF_SEL_PROBEGEN_UTC_RTS_RTR__GFX10           = 0x000000d3,
    RMI_PERF_SEL_LEVEL_ADD_RMI_TO_UTC__GFX10           = 0x000000d4,
    RMI_PERF_SEL_PROBEGEN_UTC_RTSB_RTR__GFX10          = 0x000000d5,
    RMI_PERF_SEL_PROBEGEN_UTC_RTS_RTRB__GFX10          = 0x000000d6,
    RMI_PERF_SEL_PROBEGEN_UTC_RTSB_RTRB__GFX10         = 0x000000d7,
    RMI_PERF_SEL_UTC_POP_RTS_RTR__GFX10                = 0x000000d8,
    RMI_PERF_SEL_UTC_POP_RTSB_RTR__GFX10               = 0x000000d9,
    RMI_PERF_SEL_UTC_POP_RTS_RTRB__GFX10               = 0x000000da,
    RMI_PERF_SEL_UTC_POP_RTSB_RTRB__GFX10              = 0x000000db,
    RMI_PERF_SEL_POP_XNACK_RTS_RTR__GFX10              = 0x000000dc,
    RMI_PERF_SEL_POP_XNACK_RTSB_RTR__GFX10             = 0x000000dd,
    RMI_PERF_SEL_POP_XNACK_RTS_RTRB__GFX10             = 0x000000de,
    RMI_PERF_SEL_POP_XNACK_RTSB_RTRB__GFX10            = 0x000000df,
    RMI_PERF_SEL_XNACK_PROBEGEN_RTS_RTR__GFX10         = 0x000000e0,
    RMI_PERF_SEL_XNACK_PROBEGEN_RTSB_RTR__GFX10        = 0x000000e1,
    RMI_PERF_SEL_XNACK_PROBEGEN_RTS_RTRB__GFX10        = 0x000000e2,
    RMI_PERF_SEL_XNACK_PROBEGEN_RTSB_RTRB__GFX10       = 0x000000e3,
    RMI_PERF_SEL_PRTFIFO_RTNFORMATTER_RTS_RTR__GFX10   = 0x000000e4,
    RMI_PERF_SEL_PRTFIFO_RTNFORMATTER_RTSB_RTR__GFX10  = 0x000000e5,
    RMI_PERF_SEL_PRTFIFO_RTNFORMATTER_RTS_RTRB__GFX10  = 0x000000e6,
    RMI_PERF_SEL_PRTFIFO_RTNFORMATTER_RTSB_RTRB__GFX10 = 0x000000e7,
    RMI_PERF_SEL_SKID_FIFO_IN_RTS__GFX10               = 0x000000e8,
    RMI_PERF_SEL_SKID_FIFO_IN_RTSB__GFX10              = 0x000000e9,
    RMI_PERF_SEL_SKID_FIFO_OUT_RTS__GFX10              = 0x000000ea,
    RMI_PERF_SEL_SKID_FIFO_OUT_RTSB__GFX10             = 0x000000eb,
    RMI_PERF_SEL_XBAR_PROBEGEN_READ_RTS_RTR__GFX10     = 0x000000ec,
    RMI_PERF_SEL_XBAR_PROBEGEN_WRITE_RTS_RTR__GFX10    = 0x000000ed,
    RMI_PERF_SEL_XBAR_PROBEGEN_IN0_RTS_RTR__GFX10      = 0x000000ee,
    RMI_PERF_SEL_XBAR_PROBEGEN_IN1_RTS_RTR__GFX10      = 0x000000ef,
    RMI_PERF_SEL_XBAR_PROBEGEN_CB_RTS_RTR__GFX10       = 0x000000f0,
    RMI_PERF_SEL_XBAR_PROBEGEN_DB_RTS_RTR__GFX10       = 0x000000f1,
    RMI_PERF_SEL_REORDER_FIFO_REQ__GFX10               = 0x000000f2,
    RMI_PERF_SEL_REORDER_FIFO_BUSY__GFX10              = 0x000000f3,
    RMI_PERF_SEL_RMI_RB_EARLY_WRACK_ALL_CID__GFX10     = 0x000000f4,
    RMI_PERF_SEL_RMI_RB_EARLY_WRACK_CID0__GFX10        = 0x000000f5,
    RMI_PERF_SEL_RMI_RB_EARLY_WRACK_CID1__GFX10        = 0x000000f6,
    RMI_PERF_SEL_RMI_RB_EARLY_WRACK_CID2__GFX10        = 0x000000f7,
    RMI_PERF_SEL_RMI_RB_EARLY_WRACK_CID3__GFX10        = 0x000000f8,
    RMI_PERF_SEL_RMI_RB_EARLY_WRACK_CID4__GFX10        = 0x000000f9,
    RMI_PERF_SEL_RMI_RB_EARLY_WRACK_CID5__GFX10        = 0x000000fa,
    RMI_PERF_SEL_RMI_RB_EARLY_WRACK_CID6__GFX10        = 0x000000fb,
    RMI_PERF_SEL_RMI_RB_EARLY_WRACK_CID7__GFX10        = 0x000000fc,
    RMI_PERF_SEL_RMI_RB_EARLY_WRACK_NACK0__GFX10       = 0x000000fd,
    RMI_PERF_SEL_RMI_RB_EARLY_WRACK_NACK1__GFX10       = 0x000000fe,
    RMI_PERF_SEL_RMI_RB_EARLY_WRACK_NACK2__GFX10       = 0x000000ff,
    RMI_PERF_SEL_RMI_RB_EARLY_WRACK_NACK3__GFX10       = 0x00000100,
    RMI_PERF_SEL_UTCL0_UTCL1_PERM_FAULT__GFX10         = 0x00000101,
    RMI_PERF_SEL_RB_RMI_WRREQ_ALL_CID__GFX11           = 0x00000008,
    RMI_PERF_SEL_RB_RMI_WRREQ_TO_WRRET_BUSY__GFX11     = 0x00000009,
    RMI_PERF_SEL_RB_RMI_WRREQ_CID0__GFX11              = 0x0000000a,
    RMI_PERF_SEL_RB_RMI_WRREQ_CID1__GFX11              = 0x0000000b,
    RMI_PERF_SEL_RB_RMI_WRREQ_CID2__GFX11              = 0x0000000c,
    RMI_PERF_SEL_RB_RMI_WRREQ_CID3__GFX11              = 0x0000000d,
    RMI_PERF_SEL_RB_RMI_WRREQ_CID4__GFX11              = 0x0000000e,
    RMI_PERF_SEL_RB_RMI_WRREQ_CID5__GFX11              = 0x0000000f,
    RMI_PERF_SEL_RB_RMI_WRREQ_CID6__GFX11              = 0x00000010,
    RMI_PERF_SEL_RB_RMI_WRREQ_CID7__GFX11              = 0x00000011,
    RMI_PERF_SEL_RB_RMI_32BWRREQ_INFLIGHT_ALL_ORONE_CID__GFX11 = 0x00000012,
    RMI_PERF_SEL_RB_RMI_WRREQ_BURST_LENGTH_ALL_ORONE_CID__GFX11 = 0x00000013,
    RMI_PERF_SEL_RB_RMI_WRREQ_BURST_ALL_ORONE_CID__GFX11 = 0x00000014,
    RMI_PERF_SEL_RB_RMI_WRREQ_RESIDENCY__GFX11         = 0x00000015,
    RMI_PERF_SEL_RMI_RB_WRRET_VALID_ALL_CID__GFX11     = 0x00000016,
    RMI_PERF_SEL_RMI_RB_WRRET_VALID_CID0__GFX11        = 0x00000017,
    RMI_PERF_SEL_RMI_RB_WRRET_VALID_CID1__GFX11        = 0x00000018,
    RMI_PERF_SEL_RMI_RB_WRRET_VALID_CID2__GFX11        = 0x00000019,
    RMI_PERF_SEL_RMI_RB_WRRET_VALID_CID3__GFX11        = 0x0000001a,
    RMI_PERF_SEL_RMI_RB_WRRET_VALID_CID4__GFX11        = 0x0000001b,
    RMI_PERF_SEL_RMI_RB_WRRET_VALID_CID5__GFX11        = 0x0000001c,
    RMI_PERF_SEL_RMI_RB_WRRET_VALID_CID6__GFX11        = 0x0000001d,
    RMI_PERF_SEL_RMI_RB_WRRET_VALID_CID7__GFX11        = 0x0000001e,
    RMI_PERF_SEL_RMI_RB_WRRET_VALID_NACK0__GFX11       = 0x0000001f,
    RMI_PERF_SEL_RMI_RB_WRRET_VALID_NACK1__GFX11       = 0x00000020,
    RMI_PERF_SEL_RMI_RB_WRRET_VALID_NACK2__GFX11       = 0x00000021,
    RMI_PERF_SEL_RMI_RB_WRRET_VALID_NACK3__GFX11       = 0x00000022,
    RMI_PERF_SEL_RB_RMI_32BRDREQ_ALL_CID__GFX11        = 0x00000023,
    RMI_PERF_SEL_RB_RMI_RDREQ_ALL_CID__GFX11           = 0x00000024,
    RMI_PERF_SEL_RB_RMI_RDREQ_TO_RDRET_BUSY__GFX11     = 0x00000025,
    RMI_PERF_SEL_RB_RMI_32BRDREQ_CID0__GFX11           = 0x00000026,
    RMI_PERF_SEL_RB_RMI_32BRDREQ_CID1__GFX11           = 0x00000027,
    RMI_PERF_SEL_RB_RMI_32BRDREQ_CID2__GFX11           = 0x00000028,
    RMI_PERF_SEL_RB_RMI_32BRDREQ_CID3__GFX11           = 0x00000029,
    RMI_PERF_SEL_RB_RMI_32BRDREQ_CID4__GFX11           = 0x0000002a,
    RMI_PERF_SEL_RB_RMI_32BRDREQ_CID5__GFX11           = 0x0000002b,
    RMI_PERF_SEL_RB_RMI_32BRDREQ_CID6__GFX11           = 0x0000002c,
    RMI_PERF_SEL_RB_RMI_32BRDREQ_CID7__GFX11           = 0x0000002d,
    RMI_PERF_SEL_RB_RMI_RDREQ_CID0__GFX11              = 0x0000002e,
    RMI_PERF_SEL_RB_RMI_RDREQ_CID1__GFX11              = 0x0000002f,
    RMI_PERF_SEL_RB_RMI_RDREQ_CID2__GFX11              = 0x00000030,
    RMI_PERF_SEL_RB_RMI_RDREQ_CID3__GFX11              = 0x00000031,
    RMI_PERF_SEL_RB_RMI_RDREQ_CID4__GFX11              = 0x00000032,
    RMI_PERF_SEL_RB_RMI_RDREQ_CID5__GFX11              = 0x00000033,
    RMI_PERF_SEL_RB_RMI_RDREQ_CID6__GFX11              = 0x00000034,
    RMI_PERF_SEL_RB_RMI_RDREQ_CID7__GFX11              = 0x00000035,
    RMI_PERF_SEL_RB_RMI_32BRDREQ_INFLIGHT_ALL_ORONE_CID__GFX11 = 0x00000036,
    RMI_PERF_SEL_RB_RMI_RDREQ_BURST_LENGTH_ALL_ORONE_CID__GFX11 = 0x00000037,
    RMI_PERF_SEL_RB_RMI_RDREQ_BURST_ALL_ORONE_CID__GFX11 = 0x00000038,
    RMI_PERF_SEL_RB_RMI_RDREQ_RESIDENCY__GFX11         = 0x00000039,
    RMI_PERF_SEL_RMI_RB_32BRDRET_VALID_ALL_CID__GFX11  = 0x0000003a,
    RMI_PERF_SEL_RMI_RB_32BRDRET_VALID_CID0__GFX11     = 0x0000003b,
    RMI_PERF_SEL_RMI_RB_32BRDRET_VALID_CID1__GFX11     = 0x0000003c,
    RMI_PERF_SEL_RMI_RB_32BRDRET_VALID_CID2__GFX11     = 0x0000003d,
    RMI_PERF_SEL_RMI_RB_32BRDRET_VALID_CID3__GFX11     = 0x0000003e,
    RMI_PERF_SEL_RMI_RB_32BRDRET_VALID_CID4__GFX11     = 0x0000003f,
    RMI_PERF_SEL_RMI_RB_32BRDRET_VALID_CID5__GFX11     = 0x00000040,
    RMI_PERF_SEL_RMI_RB_32BRDRET_VALID_CID6__GFX11     = 0x00000041,
    RMI_PERF_SEL_RMI_RB_32BRDRET_VALID_CID7__GFX11     = 0x00000042,
    RMI_PERF_SEL_RMI_RB_32BRDRET_VALID_NACK0__GFX11    = 0x00000043,
    RMI_PERF_SEL_RMI_RB_32BRDRET_VALID_NACK1__GFX11    = 0x00000044,
    RMI_PERF_SEL_RMI_RB_32BRDRET_VALID_NACK2__GFX11    = 0x00000045,
    RMI_PERF_SEL_RMI_RB_32BRDRET_VALID_NACK3__GFX11    = 0x00000046,
    RMI_PERF_SEL_RB_RMI_WR_FIFO_MAX__GFX11             = 0x00000047,
    RMI_PERF_SEL_RB_RMI_WR_FIFO_EMPTY__GFX11           = 0x00000048,
    RMI_PERF_SEL_RB_RMI_WR_IDLE__GFX11                 = 0x00000049,
    RMI_PERF_SEL_RB_RMI_WR_STARVE__GFX11               = 0x0000004a,
    RMI_PERF_SEL_RB_RMI_WR_STALL__GFX11                = 0x0000004b,
    RMI_PERF_SEL_RB_RMI_WR_BUSY__GFX11                 = 0x0000004c,
    RMI_PERF_SEL_RB_RMI_WR_INTF_BUSY__GFX11            = 0x0000004d,
    RMI_PERF_SEL_RB_RMI_RD_FIFO_MAX__GFX11             = 0x0000004e,
    RMI_PERF_SEL_RB_RMI_RD_FIFO_EMPTY__GFX11           = 0x0000004f,
    RMI_PERF_SEL_RB_RMI_RD_IDLE__GFX11                 = 0x00000050,
    RMI_PERF_SEL_RB_RMI_RD_STARVE__GFX11               = 0x00000051,
    RMI_PERF_SEL_RB_RMI_RD_STALL__GFX11                = 0x00000052,
    RMI_PERF_SEL_RB_RMI_RD_BUSY__GFX11                 = 0x00000053,
    RMI_PERF_SEL_RB_RMI_RD_INTF_BUSY__GFX11            = 0x00000054,
    RMI_PERF_SEL_RMI_TC_64BWRREQ_ALL_ORONE_CID__GFX11  = 0x00000055,
    RMI_PERF_SEL_RMI_TC_64BRDREQ_ALL_ORONE_CID__GFX11  = 0x00000056,
    RMI_PERF_SEL_RMI_TC_WRREQ_ALL_CID__GFX11           = 0x00000057,
    RMI_PERF_SEL_RMI_TC_REQ_BUSY__GFX11                = 0x00000058,
    RMI_PERF_SEL_RMI_TC_WRREQ_CID0__GFX11              = 0x00000059,
    RMI_PERF_SEL_RMI_TC_WRREQ_CID1__GFX11              = 0x0000005a,
    RMI_PERF_SEL_RMI_TC_WRREQ_CID2__GFX11              = 0x0000005b,
    RMI_PERF_SEL_RMI_TC_WRREQ_CID3__GFX11              = 0x0000005c,
    RMI_PERF_SEL_RMI_TC_WRREQ_CID4__GFX11              = 0x0000005d,
    RMI_PERF_SEL_RMI_TC_WRREQ_CID5__GFX11              = 0x0000005e,
    RMI_PERF_SEL_RMI_TC_WRREQ_CID6__GFX11              = 0x0000005f,
    RMI_PERF_SEL_RMI_TC_WRREQ_CID7__GFX11              = 0x00000060,
    RMI_PERF_SEL_RMI_TC_WRREQ_INFLIGHT_ALL_CID__GFX11  = 0x00000061,
    RMI_PERF_SEL_TC_RMI_WRRET_VALID_ALL_CID__GFX11     = 0x00000062,
    RMI_PERF_SEL_RMI_TC_RDREQ_ALL_CID__GFX11           = 0x00000063,
    RMI_PERF_SEL_RMI_TC_RDREQ_CID0__GFX11              = 0x00000064,
    RMI_PERF_SEL_RMI_TC_RDREQ_CID1__GFX11              = 0x00000065,
    RMI_PERF_SEL_RMI_TC_RDREQ_CID2__GFX11              = 0x00000066,
    RMI_PERF_SEL_RMI_TC_RDREQ_CID3__GFX11              = 0x00000067,
    RMI_PERF_SEL_RMI_TC_RDREQ_CID4__GFX11              = 0x00000068,
    RMI_PERF_SEL_RMI_TC_RDREQ_CID5__GFX11              = 0x00000069,
    RMI_PERF_SEL_RMI_TC_RDREQ_CID6__GFX11              = 0x0000006a,
    RMI_PERF_SEL_RMI_TC_RDREQ_CID7__GFX11              = 0x0000006b,
    RMI_PERF_SEL_RMI_TC_STALL_RDREQ__GFX11             = 0x0000006c,
    RMI_PERF_SEL_RMI_TC_STALL_WRREQ__GFX11             = 0x0000006d,
    RMI_PERF_SEL_RMI_TC_STALL_ALLREQ__GFX11            = 0x0000006e,
    RMI_PERF_SEL_RMI_TC_CREDIT_FULL_NO_PENDING_SEND__GFX11 = 0x0000006f,
    RMI_PERF_SEL_RMI_TC_CREDIT_ZERO_PENDING_SEND__GFX11 = 0x00000070,
    RMI_PERF_SEL_RMI_TC_RDREQ_INFLIGHT_ALL_CID__GFX11  = 0x00000071,
    RMI_PERF_SEL_TC_RMI_RDRET_VALID_ALL_CID__GFX11     = 0x00000072,
    RMI_PERF_SEL_TCIW_INFLIGHT_COUNT__GFX11            = 0x00000073,
    RMI_PERF_SEL_TCIW_REQ__GFX11                       = 0x00000074,
    RMI_PERF_SEL_TCIW_BUSY__GFX11                      = 0x00000075,
    RMI_PERF_SEL_DEMUX_TCIW_FORMATTER_RTS_RTR__GFX11   = 0x00000076,
    RMI_PERF_SEL_DEMUX_TCIW_FORMATTER_RTSB_RTR__GFX11  = 0x00000077,
    RMI_PERF_SEL_DEMUX_TCIW_FORMATTER_RTS_RTRB__GFX11  = 0x00000078,
    RMI_PERF_SEL_DEMUX_TCIW_FORMATTER_RTSB_RTRB__GFX11 = 0x00000079,
    RMI_PERF_SEL_REORDER_FIFO_REQ__GFX11               = 0x0000007a,
    RMI_PERF_SEL_REORDER_FIFO_BUSY__GFX11              = 0x0000007b,
    RMI_PERF_SEL_RMI_RB_EARLY_WRACK_ALL_CID__GFX11     = 0x0000007c,
    RMI_PERF_SEL_RMI_RB_EARLY_WRACK_CID0__GFX11        = 0x0000007d,
    RMI_PERF_SEL_RMI_RB_EARLY_WRACK_CID1__GFX11        = 0x0000007e,
    RMI_PERF_SEL_RMI_RB_EARLY_WRACK_CID2__GFX11        = 0x0000007f,
    RMI_PERF_SEL_RMI_RB_EARLY_WRACK_CID3__GFX11        = 0x00000080,
    RMI_PERF_SEL_RMI_RB_EARLY_WRACK_CID4__GFX11        = 0x00000081,
    RMI_PERF_SEL_RMI_RB_EARLY_WRACK_CID5__GFX11        = 0x00000082,
    RMI_PERF_SEL_RMI_RB_EARLY_WRACK_CID6__GFX11        = 0x00000083,
    RMI_PERF_SEL_RMI_RB_EARLY_WRACK_CID7__GFX11        = 0x00000084,
    RMI_PERF_SEL_CONSUMER_PROBEGEN_READ_RTS_RTR__GFX11 = 0x00000085,
    RMI_PERF_SEL_CONSUMER_PROBEGEN_WRITE_RTS_RTR__GFX11 = 0x00000086,
    RMI_PERF_SEL_CONSUMER_PROBEGEN_IN0_RTS_RTR__GFX11  = 0x00000087,
    RMI_PERF_SEL_CONSUMER_PROBEGEN_IN1_RTS_RTR__GFX11  = 0x00000088,
    RMI_PERF_SEL_CONSUMER_PROBEGEN_CB_RTS_RTR__GFX11   = 0x00000089,
    RMI_PERF_SEL_CONSUMER_PROBEGEN_DB_RTS_RTR__GFX11   = 0x0000008a,
} RMIPerfSel;

constexpr unsigned int MaxRMIPerfSelGfx10              = RMI_PERF_SEL_UTCL0_UTCL1_PERM_FAULT__GFX10;
constexpr unsigned int MaxRMIPerfSelGfx11              = RMI_PERF_SEL_CONSUMER_PROBEGEN_DB_RTS_RTR__GFX11;

typedef enum RoundMode {
    ROUND_BY_HALF                                      = 0x00000000,
    ROUND_TRUNCATE                                     = 0x00000001,
} RoundMode;

typedef enum SC_PERFCNT_SEL {
    SC_SRPS_WINDOW_VALID                               = 0x00000000,
    SC_PSSW_WINDOW_VALID                               = 0x00000001,
    SC_TPQZ_WINDOW_VALID                               = 0x00000002,
    SC_QZQP_WINDOW_VALID                               = 0x00000003,
    SC_TRPK_WINDOW_VALID                               = 0x00000004,
    SC_SRPS_WINDOW_VALID_BUSY                          = 0x00000005,
    SC_PSSW_WINDOW_VALID_BUSY                          = 0x00000006,
    SC_TPQZ_WINDOW_VALID_BUSY                          = 0x00000007,
    SC_QZQP_WINDOW_VALID_BUSY                          = 0x00000008,
    SC_TRPK_WINDOW_VALID_BUSY                          = 0x00000009,
    SC_STARVED_BY_PA                                   = 0x0000000a,
    SC_STALLED_BY_PRIMFIFO                             = 0x0000000b,
    SC_STALLED_BY_DB_TILE                              = 0x0000000c,
    SC_STARVED_BY_DB_TILE                              = 0x0000000d,
    SC_STALLED_BY_TILEORDERFIFO                        = 0x0000000e,
    SC_STALLED_BY_TILEFIFO                             = 0x0000000f,
    SC_STALLED_BY_DB_QUAD                              = 0x00000010,
    SC_STARVED_BY_DB_QUAD                              = 0x00000011,
    SC_STALLED_BY_QUADFIFO                             = 0x00000012,
    SC_STALLED_BY_BCI                                  = 0x00000013,
    SC_STALLED_BY_SPI                                  = 0x00000014,
    SC_SCISSOR_DISCARD                                 = 0x00000015,
    SC_BB_DISCARD                                      = 0x00000016,
    SC_SUPERTILE_COUNT                                 = 0x00000017,
    SC_SUPERTILE_PER_PRIM_H0                           = 0x00000018,
    SC_SUPERTILE_PER_PRIM_H1                           = 0x00000019,
    SC_SUPERTILE_PER_PRIM_H2                           = 0x0000001a,
    SC_SUPERTILE_PER_PRIM_H3                           = 0x0000001b,
    SC_SUPERTILE_PER_PRIM_H4                           = 0x0000001c,
    SC_SUPERTILE_PER_PRIM_H5                           = 0x0000001d,
    SC_SUPERTILE_PER_PRIM_H6                           = 0x0000001e,
    SC_SUPERTILE_PER_PRIM_H7                           = 0x0000001f,
    SC_SUPERTILE_PER_PRIM_H8                           = 0x00000020,
    SC_SUPERTILE_PER_PRIM_H9                           = 0x00000021,
    SC_SUPERTILE_PER_PRIM_H10                          = 0x00000022,
    SC_SUPERTILE_PER_PRIM_H11                          = 0x00000023,
    SC_SUPERTILE_PER_PRIM_H12                          = 0x00000024,
    SC_SUPERTILE_PER_PRIM_H13                          = 0x00000025,
    SC_SUPERTILE_PER_PRIM_H14                          = 0x00000026,
    SC_SUPERTILE_PER_PRIM_H15                          = 0x00000027,
    SC_SUPERTILE_PER_PRIM_H16                          = 0x00000028,
    SC_TILE_PER_PRIM_H0                                = 0x00000029,
    SC_TILE_PER_PRIM_H1                                = 0x0000002a,
    SC_TILE_PER_PRIM_H2                                = 0x0000002b,
    SC_TILE_PER_PRIM_H3                                = 0x0000002c,
    SC_TILE_PER_PRIM_H4                                = 0x0000002d,
    SC_TILE_PER_PRIM_H5                                = 0x0000002e,
    SC_TILE_PER_PRIM_H6                                = 0x0000002f,
    SC_TILE_PER_PRIM_H7                                = 0x00000030,
    SC_TILE_PER_PRIM_H8                                = 0x00000031,
    SC_TILE_PER_PRIM_H9                                = 0x00000032,
    SC_TILE_PER_PRIM_H10                               = 0x00000033,
    SC_TILE_PER_PRIM_H11                               = 0x00000034,
    SC_TILE_PER_PRIM_H12                               = 0x00000035,
    SC_TILE_PER_PRIM_H13                               = 0x00000036,
    SC_TILE_PER_PRIM_H14                               = 0x00000037,
    SC_TILE_PER_PRIM_H15                               = 0x00000038,
    SC_TILE_PER_PRIM_H16                               = 0x00000039,
    SC_TILE_PER_SUPERTILE_H0                           = 0x0000003a,
    SC_TILE_PER_SUPERTILE_H1                           = 0x0000003b,
    SC_TILE_PER_SUPERTILE_H2                           = 0x0000003c,
    SC_TILE_PER_SUPERTILE_H3                           = 0x0000003d,
    SC_TILE_PER_SUPERTILE_H4                           = 0x0000003e,
    SC_TILE_PER_SUPERTILE_H5                           = 0x0000003f,
    SC_TILE_PER_SUPERTILE_H6                           = 0x00000040,
    SC_TILE_PER_SUPERTILE_H7                           = 0x00000041,
    SC_TILE_PER_SUPERTILE_H8                           = 0x00000042,
    SC_TILE_PER_SUPERTILE_H9                           = 0x00000043,
    SC_TILE_PER_SUPERTILE_H10                          = 0x00000044,
    SC_TILE_PER_SUPERTILE_H11                          = 0x00000045,
    SC_TILE_PER_SUPERTILE_H12                          = 0x00000046,
    SC_TILE_PER_SUPERTILE_H13                          = 0x00000047,
    SC_TILE_PER_SUPERTILE_H14                          = 0x00000048,
    SC_TILE_PER_SUPERTILE_H15                          = 0x00000049,
    SC_TILE_PER_SUPERTILE_H16                          = 0x0000004a,
    SC_TILE_PICKED_H1                                  = 0x0000004b,
    SC_QZ0_TILE_COUNT                                  = 0x0000004f,
    SC_QZ0_TILE_COVERED_COUNT                          = 0x00000053,
    SC_QZ0_TILE_NOT_COVERED_COUNT                      = 0x00000057,
    SC_QZ0_QUAD_PER_TILE_H0                            = 0x0000005b,
    SC_QZ0_QUAD_PER_TILE_H1                            = 0x0000005c,
    SC_QZ0_QUAD_PER_TILE_H2                            = 0x0000005d,
    SC_QZ0_QUAD_PER_TILE_H3                            = 0x0000005e,
    SC_QZ0_QUAD_PER_TILE_H4                            = 0x0000005f,
    SC_QZ0_QUAD_PER_TILE_H5                            = 0x00000060,
    SC_QZ0_QUAD_PER_TILE_H6                            = 0x00000061,
    SC_QZ0_QUAD_PER_TILE_H7                            = 0x00000062,
    SC_QZ0_QUAD_PER_TILE_H8                            = 0x00000063,
    SC_QZ0_QUAD_PER_TILE_H9                            = 0x00000064,
    SC_QZ0_QUAD_PER_TILE_H10                           = 0x00000065,
    SC_QZ0_QUAD_PER_TILE_H11                           = 0x00000066,
    SC_QZ0_QUAD_PER_TILE_H12                           = 0x00000067,
    SC_QZ0_QUAD_PER_TILE_H13                           = 0x00000068,
    SC_QZ0_QUAD_PER_TILE_H14                           = 0x00000069,
    SC_QZ0_QUAD_PER_TILE_H15                           = 0x0000006a,
    SC_QZ0_QUAD_PER_TILE_H16                           = 0x0000006b,
    SC_QZ0_QUAD_COUNT                                  = 0x0000009f,
    SC_P0_HIZ_TILE_COUNT                               = 0x000000a3,
    SC_P0_HIZ_QUAD_PER_TILE_H0                         = 0x000000a7,
    SC_P0_HIZ_QUAD_PER_TILE_H1                         = 0x000000a8,
    SC_P0_HIZ_QUAD_PER_TILE_H2                         = 0x000000a9,
    SC_P0_HIZ_QUAD_PER_TILE_H3                         = 0x000000aa,
    SC_P0_HIZ_QUAD_PER_TILE_H4                         = 0x000000ab,
    SC_P0_HIZ_QUAD_PER_TILE_H5                         = 0x000000ac,
    SC_P0_HIZ_QUAD_PER_TILE_H6                         = 0x000000ad,
    SC_P0_HIZ_QUAD_PER_TILE_H7                         = 0x000000ae,
    SC_P0_HIZ_QUAD_PER_TILE_H8                         = 0x000000af,
    SC_P0_HIZ_QUAD_PER_TILE_H9                         = 0x000000b0,
    SC_P0_HIZ_QUAD_PER_TILE_H10                        = 0x000000b1,
    SC_P0_HIZ_QUAD_PER_TILE_H11                        = 0x000000b2,
    SC_P0_HIZ_QUAD_PER_TILE_H12                        = 0x000000b3,
    SC_P0_HIZ_QUAD_PER_TILE_H13                        = 0x000000b4,
    SC_P0_HIZ_QUAD_PER_TILE_H14                        = 0x000000b5,
    SC_P0_HIZ_QUAD_PER_TILE_H15                        = 0x000000b6,
    SC_P0_HIZ_QUAD_PER_TILE_H16                        = 0x000000b7,
    SC_P0_HIZ_QUAD_COUNT                               = 0x000000eb,
    SC_P0_DETAIL_QUAD_COUNT                            = 0x000000ef,
    SC_P0_DETAIL_QUAD_WITH_1_PIX                       = 0x000000f3,
    SC_P0_DETAIL_QUAD_WITH_2_PIX                       = 0x000000f4,
    SC_P0_DETAIL_QUAD_WITH_3_PIX                       = 0x000000f5,
    SC_P0_DETAIL_QUAD_WITH_4_PIX                       = 0x000000f6,
    SC_EARLYZ_QUAD_COUNT                               = 0x00000103,
    SC_EARLYZ_QUAD_WITH_1_PIX                          = 0x00000104,
    SC_EARLYZ_QUAD_WITH_2_PIX                          = 0x00000105,
    SC_EARLYZ_QUAD_WITH_3_PIX                          = 0x00000106,
    SC_EARLYZ_QUAD_WITH_4_PIX                          = 0x00000107,
    SC_PKR_QUAD_PER_ROW_H1                             = 0x00000108,
    SC_PKR_QUAD_PER_ROW_H2                             = 0x00000109,
    SC_PKR_4X2_QUAD_SPLIT                              = 0x0000010a,
    SC_PKR_4X2_FILL_QUAD                               = 0x0000010b,
    SC_PKR_END_OF_VECTOR                               = 0x0000010c,
    SC_PKR_CONTROL_XFER                                = 0x0000010d,
    SC_PKR_DBHANG_FORCE_EOV                            = 0x0000010e,
    SC_REG_SCLK_BUSY                                   = 0x0000010f,
    SC_GRP0_DYN_SCLK_BUSY                              = 0x00000110,
    SC_GRP1_DYN_SCLK_BUSY                              = 0x00000111,
    SC_GRP2_DYN_SCLK_BUSY                              = 0x00000112,
    SC_GRP3_DYN_SCLK_BUSY                              = 0x00000113,
    SC_GRP4_DYN_SCLK_BUSY                              = 0x00000114,
    SC_PA0_SC_DATA_FIFO_RD                             = 0x00000115,
    SC_PA0_SC_DATA_FIFO_WE                             = 0x00000116,
    SC_PS_ARB_XFC_ALL_EVENT_OR_PRIM_CYCLES             = 0x00000119,
    SC_PS_ARB_XFC_ONLY_PRIM_CYCLES                     = 0x0000011a,
    SC_PS_ARB_XFC_ONLY_ONE_INC_PER_PRIM                = 0x0000011b,
    SC_PS_ARB_STALLED_FROM_BELOW                       = 0x0000011c,
    SC_PS_ARB_STARVED_FROM_ABOVE                       = 0x0000011d,
    SC_PS_ARB_SC_BUSY                                  = 0x0000011e,
    SC_PS_ARB_PA_SC_BUSY                               = 0x0000011f,
    SC_PA0_SC_EOP_WE                                   = 0x0000012c,
    SC_PA0_SC_EVENT_WE                                 = 0x0000012e,
    SC_PS_TS_EVENT_FIFO_PUSH                           = 0x00000157,
    SC_PS_TS_EVENT_FIFO_POP                            = 0x00000158,
    SC_PS_CTX_DONE_FIFO_PUSH                           = 0x00000159,
    SC_PS_CTX_DONE_FIFO_POP                            = 0x0000015a,
    SC_PA0_SC_NULL_WE                                  = 0x0000015d,
    SC_PA0_SC_NULL_DEALLOC_WE                          = 0x0000015e,
    SC_PA0_SC_DATA_FIFO_EOP_RD                         = 0x00000160,
    SC_PS_PA0_SC_FIFO_EMPTY                            = 0x00000175,
    SC_PS_PA0_SC_FIFO_FULL                             = 0x00000176,
    SC_BUSY_CNT_NOT_ZERO                               = 0x00000182,
    SC_BM_BUSY                                         = 0x00000183,
    SC_BACKEND_BUSY                                    = 0x00000184,
    SC_SCF_SCB_INTERFACE_BUSY                          = 0x00000185,
    SC_SCB_BUSY                                        = 0x00000186,
    SC_STARVED_BY_PA_WITH_UNSELECTED_PA_NOT_EMPTY      = 0x00000187,
    SC_STARVED_BY_PA_WITH_UNSELECTED_PA_FULL           = 0x00000188,
    SC_PBB_BIN_HIST_NUM_PRIMS                          = 0x00000189,
    SC_PBB_BATCH_HIST_NUM_PRIMS                        = 0x0000018a,
    SC_PBB_BIN_HIST_NUM_CONTEXTS                       = 0x0000018b,
    SC_PBB_BATCH_HIST_NUM_CONTEXTS                     = 0x0000018c,
    SC_PBB_BIN_HIST_NUM_PERSISTENT_STATES              = 0x0000018d,
    SC_PBB_BATCH_HIST_NUM_PERSISTENT_STATES            = 0x0000018e,
    SC_PBB_BATCH_HIST_NUM_PS_WAVE_BREAKS               = 0x0000018f,
    SC_PBB_BATCH_HIST_NUM_TRIV_REJECTED_PRIMS          = 0x00000190,
    SC_PBB_BATCH_HIST_NUM_ROWS_PER_PRIM                = 0x00000191,
    SC_PBB_BATCH_HIST_NUM_COLUMNS_PER_ROW              = 0x00000192,
    SC_PBB_BUSY                                        = 0x00000193,
    SC_PBB_BUSY_AND_NO_SENDS                           = 0x00000194,
    SC_PBB_STALLS_PA_DUE_TO_NO_TILES                   = 0x00000195,
    SC_PBB_NUM_BINS                                    = 0x00000196,
    SC_PBB_END_OF_BIN                                  = 0x00000197,
    SC_PBB_END_OF_BATCH                                = 0x00000198,
    SC_PBB_PRIMBIN_PROCESSED                           = 0x00000199,
    SC_PBB_PRIM_ADDED_TO_BATCH                         = 0x0000019a,
    SC_PBB_NONBINNED_PRIM                              = 0x0000019b,
    SC_PBB_TOTAL_REAL_PRIMS_OUT_OF_PBB                 = 0x0000019c,
    SC_PBB_TOTAL_NULL_PRIMS_OUT_OF_PBB                 = 0x0000019d,
    SC_PBB_IDLE_CLK_DUE_TO_ROW_TO_COLUMN_TRANSITION    = 0x0000019e,
    SC_PBB_IDLE_CLK_DUE_TO_FALSE_POSITIVE_ON_ROW       = 0x0000019f,
    SC_PBB_IDLE_CLK_DUE_TO_FALSE_POSITIVE_ON_COLUMN    = 0x000001a0,
    SC_PBB_BATCH_BREAK_DUE_TO_PERSISTENT_STATE         = 0x000001a1,
    SC_PBB_BATCH_BREAK_DUE_TO_CONTEXT_STATE            = 0x000001a2,
    SC_PBB_BATCH_BREAK_DUE_TO_PRIM                     = 0x000001a3,
    SC_PBB_BATCH_BREAK_DUE_TO_PC_STORAGE               = 0x000001a4,
    SC_PBB_BATCH_BREAK_DUE_TO_EVENT                    = 0x000001a5,
    SC_PBB_BATCH_BREAK_DUE_TO_FPOV_LIMIT               = 0x000001a6,
    SC_GRP5_DYN_SCLK_BUSY                              = 0x000001b5,
    SC_GRP6_DYN_SCLK_BUSY                              = 0x000001b6,
    SC_GRP7_DYN_SCLK_BUSY                              = 0x000001b7,
    SC_GRP8_DYN_SCLK_BUSY                              = 0x000001b8,
    SC_GRP9_DYN_SCLK_BUSY                              = 0x000001b9,
    SC_PS_TO_BE_SCLK_GATE_STALL                        = 0x000001ba,
    SC_PA_TO_PBB_SCLK_GATE_STALL_STALL                 = 0x000001bb,
    SC_PK_BUSY                                         = 0x000001bc,
    SC_PK_MAX_DEALLOC_FORCE_EOV                        = 0x000001bd,
    SC_PK_DEALLOC_WAVE_BREAK                           = 0x000001be,
    SC_SPI_SEND                                        = 0x000001bf,
    SC_SPI_CREDIT_AT_ZERO_WITH_PENDING_SEND            = 0x000001c0,
    SC_SPI_CREDIT_AT_MAX                               = 0x000001c1,
    SC_SPI_CREDIT_AT_MAX_NO_PENDING_SEND               = 0x000001c2,
    SC_BCI_SEND                                        = 0x000001c3,
    SC_BCI_CREDIT_AT_ZERO_WITH_PENDING_SEND            = 0x000001c4,
    SC_BCI_CREDIT_AT_MAX                               = 0x000001c5,
    SC_BCI_CREDIT_AT_MAX_NO_PENDING_SEND               = 0x000001c6,
    SC_SPIBC_FULL_FREEZE                               = 0x000001c7,
    SC_PW_BM_PASS_EMPTY_PRIM                           = 0x000001c8,
    SC_SUPERTILE_COUNT_EXCLUDE_PASS_EMPTY_PRIM         = 0x000001c9,
    SC_SUPERTILE_PER_PRIM_EXCLUDE_PASS_EMPTY_PRIM_H0   = 0x000001ca,
    SC_SUPERTILE_PER_PRIM_EXCLUDE_PASS_EMPTY_PRIM_H1   = 0x000001cb,
    SC_SUPERTILE_PER_PRIM_EXCLUDE_PASS_EMPTY_PRIM_H2   = 0x000001cc,
    SC_SUPERTILE_PER_PRIM_EXCLUDE_PASS_EMPTY_PRIM_H3   = 0x000001cd,
    SC_SUPERTILE_PER_PRIM_EXCLUDE_PASS_EMPTY_PRIM_H4   = 0x000001ce,
    SC_SUPERTILE_PER_PRIM_EXCLUDE_PASS_EMPTY_PRIM_H5   = 0x000001cf,
    SC_SUPERTILE_PER_PRIM_EXCLUDE_PASS_EMPTY_PRIM_H6   = 0x000001d0,
    SC_SUPERTILE_PER_PRIM_EXCLUDE_PASS_EMPTY_PRIM_H7   = 0x000001d1,
    SC_SUPERTILE_PER_PRIM_EXCLUDE_PASS_EMPTY_PRIM_H8   = 0x000001d2,
    SC_SUPERTILE_PER_PRIM_EXCLUDE_PASS_EMPTY_PRIM_H9   = 0x000001d3,
    SC_SUPERTILE_PER_PRIM_EXCLUDE_PASS_EMPTY_PRIM_H10  = 0x000001d4,
    SC_SUPERTILE_PER_PRIM_EXCLUDE_PASS_EMPTY_PRIM_H11  = 0x000001d5,
    SC_SUPERTILE_PER_PRIM_EXCLUDE_PASS_EMPTY_PRIM_H12  = 0x000001d6,
    SC_SUPERTILE_PER_PRIM_EXCLUDE_PASS_EMPTY_PRIM_H13  = 0x000001d7,
    SC_SUPERTILE_PER_PRIM_EXCLUDE_PASS_EMPTY_PRIM_H14  = 0x000001d8,
    SC_SUPERTILE_PER_PRIM_EXCLUDE_PASS_EMPTY_PRIM_H15  = 0x000001d9,
    SC_SUPERTILE_PER_PRIM_EXCLUDE_PASS_EMPTY_PRIM_H16  = 0x000001da,
    SC_DB0_TILE_INTERFACE_BUSY                         = 0x000001db,
    SC_DB0_TILE_INTERFACE_SEND                         = 0x000001dc,
    SC_DB0_TILE_INTERFACE_SEND_EVENT                   = 0x000001dd,
    SC_DB0_TILE_INTERFACE_CREDIT_AT_ZERO_WITH_PENDING_SEND = 0x000001e0,
    SC_DB0_TILE_INTERFACE_CREDIT_AT_MAX                = 0x000001e1,
    SC_DB0_TILE_INTERFACE_CREDIT_AT_MAX_WITH_NO_PENDING_SEND = 0x000001e2,
    SC_BACKEND_PRIM_FIFO_FULL                          = 0x000001eb,
    SC_PBB_BATCH_BREAK_DUE_TO_NONBINNED_BATCH          = 0x000001ed,
    SC_PBB_BATCH_BREAK_DUE_TO_DEBUG_DATA_PER_DRAW_DISPATCH = 0x000001ee,
    SC_PBB_BATCH_BREAK_DUE_TO_OVERRIDE_REGISTER_PERSISTENT = 0x000001ef,
    SC_PBB_BATCH_BREAK_DUE_TO_OVERRIDE_REGISTER_CONTEXT = 0x000001f0,
    SC_PBB_BATCH_BREAK_DUE_TO_OVERRIDE_REGISTER_FPOV   = 0x000001f1,
    SC_PBB_BATCH_BREAK_DUE_TO_NEW_SC_MODE              = 0x000001f2,
    SC_PBB_BATCH_BREAK_DUE_TO_BINNING_MODE_CHANGE      = 0x000001f3,
    SC_PBB_BATCH_BREAK_DUE_TO_PIPELINE_EVENT_COUNT     = 0x000001f4,
    SC_PBB_BATCH_BREAK_DUE_TO_PIPE_RESET               = 0x000001f5,
    SC_PBB_BATCH_BREAK_DUE_TO_GFX_PIPE_CHANGE          = 0x000001f6,
    SC_STALLED_BY_DB0_TILEFIFO                         = 0x000001f7,
    SC_DB0_QUAD_INTF_SEND                              = 0x000001f8,
    SC_DB0_QUAD_INTF_BUSY                              = 0x000001f9,
    SC_DB0_QUAD_INTF_STALLED_BY_DB                     = 0x000001fa,
    SC_DB0_QUAD_INTF_CREDIT_AT_MAX                     = 0x000001fb,
    SC_DB0_QUAD_INTF_IDLE                              = 0x000001fc,
    SC_PKR_WAVE_BREAK_OUTSIDE_REGION                   = 0x00000203,
    SC_PKR_WAVE_BREAK_FULL_TILE                        = 0x00000204,
    SC_FSR_WALKED                                      = 0x00000205,
    SC_PBB_EMPTY_INPUT_CYCLE_WHEN_BATCH_OPEN           = 0x00000206,
    SC_PBB_BATCH_BREAK_DUE_TO_NULL_PRIM_BREAK_BATCH_LIMIT = 0x00000207,
    SC_DB0_WE_STALLED_BY_RSLT_FIFO_FULL                = 0x00000208,
    SC_DB0_WE_TILE_MASK_RETURN_FIFO_FULL_WITH_WE_RSLT_FIFO_STALL = 0x00000209,
    SC_DB0_TILE_MASK_FIFO_FULL                         = 0x0000020a,
    SC_PS_PM_PBB_TO_PSE_FIFO_WE_STALL_BY_PFF_PW_FULL   = 0x0000020e,
    SC_PS_PM_PBB_TO_PSE_FIFO_WE_STALL_BY_ZFF_PW_FULL   = 0x0000020f,
    SC_PS_PM_PBB_TO_PSE_FIFO_WE_STALL_BY_PBB_TO_PSE_FIFO_FULL = 0x00000210,
    SC_PS_PM_PFF_PW_FULL                               = 0x00000211,
    SC_PS_PM_ZFF_PW_FULL                               = 0x00000212,
    SC_PS_PM_PBB_TO_PSE_FIFO_FULL                      = 0x00000213,
    SC_PK_PM_4X2_SPLIT_WAVE_BRK_1H                     = 0x00000217,
    SC_PK_PM_PKR_FILL_4X2_WAVE_BRK_1H                  = 0x00000218,
    SC_PK_PM_SPLIT_OR_FILL_4X2_WAVE_BRK_1H             = 0x00000219,
    SC_PK_PM_END_OF_VECTOR_WAVE_BRK_1H                 = 0x0000021a,
    SC_PK_PM_CTL_ONLY_CMD_WAVE_BRK_1H                  = 0x0000021c,
    SC_PK_PM_AVOID_DEALLOC_ADD_WAVE_BRK_1H             = 0x0000021d,
    SC_PK_PM_FD_CONFLICT_WAVE_BRK_1H                   = 0x0000021e,
    SC_PK_PM_FORCE_PARTIAL_FOR_DEALLOC_WAVE_BRK_1H     = 0x0000021f,
    SC_PK_PM_AE_CONFLICT_WAVE_BRK_1H                   = 0x00000220,
    SC_PK_PM_EOP_OR_LAD_WAVE_BRK_1H                    = 0x00000221,
    SC_PK_PM_FULL_TILE_WAVE_BRK_1H                     = 0x00000222,
    SC_PK_PM_MAX_DEALLOC_FORCE_EOV_WAVE_BRK_1H         = 0x00000224,
    SC_PK_PM_WAVE_BREAK_OUTSIDE_REGION_WAVE_BRK_1H     = 0x00000225,
    SC_PK_PM_MAX_CLK_CNT_FORCE_EOV_WAVE_BRK_1H         = 0x00000226,
    SC_PK_PM_MAX_REZ_CNT_FORCE_EOV_WAVE_BRK_1H         = 0x00000227,
    SC_VRC_REPROBE_XFR__APU11                          = 0x00000299,
    SC_VRC_REPROBE_FULL__APU11                         = 0x0000029a,
    SC_TILE_PICKED_H2__GFX10                           = 0x0000004c,
    SC_TILE_PICKED_H3__GFX10                           = 0x0000004d,
    SC_TILE_PICKED_H4__GFX10                           = 0x0000004e,
    SC_QZ1_TILE_COUNT__GFX10                           = 0x00000050,
    SC_QZ2_TILE_COUNT__GFX10                           = 0x00000051,
    SC_QZ3_TILE_COUNT__GFX10                           = 0x00000052,
    SC_QZ1_TILE_COVERED_COUNT__GFX10                   = 0x00000054,
    SC_QZ2_TILE_COVERED_COUNT__GFX10                   = 0x00000055,
    SC_QZ3_TILE_COVERED_COUNT__GFX10                   = 0x00000056,
    SC_QZ1_TILE_NOT_COVERED_COUNT__GFX10               = 0x00000058,
    SC_QZ2_TILE_NOT_COVERED_COUNT__GFX10               = 0x00000059,
    SC_QZ3_TILE_NOT_COVERED_COUNT__GFX10               = 0x0000005a,
    SC_QZ1_QUAD_PER_TILE_H0__GFX10                     = 0x0000006c,
    SC_QZ1_QUAD_PER_TILE_H1__GFX10                     = 0x0000006d,
    SC_QZ1_QUAD_PER_TILE_H2__GFX10                     = 0x0000006e,
    SC_QZ1_QUAD_PER_TILE_H3__GFX10                     = 0x0000006f,
    SC_QZ1_QUAD_PER_TILE_H4__GFX10                     = 0x00000070,
    SC_QZ1_QUAD_PER_TILE_H5__GFX10                     = 0x00000071,
    SC_QZ1_QUAD_PER_TILE_H6__GFX10                     = 0x00000072,
    SC_QZ1_QUAD_PER_TILE_H7__GFX10                     = 0x00000073,
    SC_QZ1_QUAD_PER_TILE_H8__GFX10                     = 0x00000074,
    SC_QZ1_QUAD_PER_TILE_H9__GFX10                     = 0x00000075,
    SC_QZ1_QUAD_PER_TILE_H10__GFX10                    = 0x00000076,
    SC_QZ1_QUAD_PER_TILE_H11__GFX10                    = 0x00000077,
    SC_QZ1_QUAD_PER_TILE_H12__GFX10                    = 0x00000078,
    SC_QZ1_QUAD_PER_TILE_H13__GFX10                    = 0x00000079,
    SC_QZ1_QUAD_PER_TILE_H14__GFX10                    = 0x0000007a,
    SC_QZ1_QUAD_PER_TILE_H15__GFX10                    = 0x0000007b,
    SC_QZ1_QUAD_PER_TILE_H16__GFX10                    = 0x0000007c,
    SC_QZ2_QUAD_PER_TILE_H0__GFX10                     = 0x0000007d,
    SC_QZ2_QUAD_PER_TILE_H1__GFX10                     = 0x0000007e,
    SC_QZ2_QUAD_PER_TILE_H2__GFX10                     = 0x0000007f,
    SC_QZ2_QUAD_PER_TILE_H3__GFX10                     = 0x00000080,
    SC_QZ2_QUAD_PER_TILE_H4__GFX10                     = 0x00000081,
    SC_QZ2_QUAD_PER_TILE_H5__GFX10                     = 0x00000082,
    SC_QZ2_QUAD_PER_TILE_H6__GFX10                     = 0x00000083,
    SC_QZ2_QUAD_PER_TILE_H7__GFX10                     = 0x00000084,
    SC_QZ2_QUAD_PER_TILE_H8__GFX10                     = 0x00000085,
    SC_QZ2_QUAD_PER_TILE_H9__GFX10                     = 0x00000086,
    SC_QZ2_QUAD_PER_TILE_H10__GFX10                    = 0x00000087,
    SC_QZ2_QUAD_PER_TILE_H11__GFX10                    = 0x00000088,
    SC_QZ2_QUAD_PER_TILE_H12__GFX10                    = 0x00000089,
    SC_QZ2_QUAD_PER_TILE_H13__GFX10                    = 0x0000008a,
    SC_QZ2_QUAD_PER_TILE_H14__GFX10                    = 0x0000008b,
    SC_QZ2_QUAD_PER_TILE_H15__GFX10                    = 0x0000008c,
    SC_QZ2_QUAD_PER_TILE_H16__GFX10                    = 0x0000008d,
    SC_QZ3_QUAD_PER_TILE_H0__GFX10                     = 0x0000008e,
    SC_QZ3_QUAD_PER_TILE_H1__GFX10                     = 0x0000008f,
    SC_QZ3_QUAD_PER_TILE_H2__GFX10                     = 0x00000090,
    SC_QZ3_QUAD_PER_TILE_H3__GFX10                     = 0x00000091,
    SC_QZ3_QUAD_PER_TILE_H4__GFX10                     = 0x00000092,
    SC_QZ3_QUAD_PER_TILE_H5__GFX10                     = 0x00000093,
    SC_QZ3_QUAD_PER_TILE_H6__GFX10                     = 0x00000094,
    SC_QZ3_QUAD_PER_TILE_H7__GFX10                     = 0x00000095,
    SC_QZ3_QUAD_PER_TILE_H8__GFX10                     = 0x00000096,
    SC_QZ3_QUAD_PER_TILE_H9__GFX10                     = 0x00000097,
    SC_QZ3_QUAD_PER_TILE_H10__GFX10                    = 0x00000098,
    SC_QZ3_QUAD_PER_TILE_H11__GFX10                    = 0x00000099,
    SC_QZ3_QUAD_PER_TILE_H12__GFX10                    = 0x0000009a,
    SC_QZ3_QUAD_PER_TILE_H13__GFX10                    = 0x0000009b,
    SC_QZ3_QUAD_PER_TILE_H14__GFX10                    = 0x0000009c,
    SC_QZ3_QUAD_PER_TILE_H15__GFX10                    = 0x0000009d,
    SC_QZ3_QUAD_PER_TILE_H16__GFX10                    = 0x0000009e,
    SC_QZ1_QUAD_COUNT__GFX10                           = 0x000000a0,
    SC_QZ2_QUAD_COUNT__GFX10                           = 0x000000a1,
    SC_QZ3_QUAD_COUNT__GFX10                           = 0x000000a2,
    SC_P1_HIZ_TILE_COUNT__GFX10                        = 0x000000a4,
    SC_P2_HIZ_TILE_COUNT__GFX10                        = 0x000000a5,
    SC_P3_HIZ_TILE_COUNT__GFX10                        = 0x000000a6,
    SC_P1_HIZ_QUAD_PER_TILE_H0__GFX10                  = 0x000000b8,
    SC_P1_HIZ_QUAD_PER_TILE_H1__GFX10                  = 0x000000b9,
    SC_P1_HIZ_QUAD_PER_TILE_H2__GFX10                  = 0x000000ba,
    SC_P1_HIZ_QUAD_PER_TILE_H3__GFX10                  = 0x000000bb,
    SC_P1_HIZ_QUAD_PER_TILE_H4__GFX10                  = 0x000000bc,
    SC_P1_HIZ_QUAD_PER_TILE_H5__GFX10                  = 0x000000bd,
    SC_P1_HIZ_QUAD_PER_TILE_H6__GFX10                  = 0x000000be,
    SC_P1_HIZ_QUAD_PER_TILE_H7__GFX10                  = 0x000000bf,
    SC_P1_HIZ_QUAD_PER_TILE_H8__GFX10                  = 0x000000c0,
    SC_P1_HIZ_QUAD_PER_TILE_H9__GFX10                  = 0x000000c1,
    SC_P1_HIZ_QUAD_PER_TILE_H10__GFX10                 = 0x000000c2,
    SC_P1_HIZ_QUAD_PER_TILE_H11__GFX10                 = 0x000000c3,
    SC_P1_HIZ_QUAD_PER_TILE_H12__GFX10                 = 0x000000c4,
    SC_P1_HIZ_QUAD_PER_TILE_H13__GFX10                 = 0x000000c5,
    SC_P1_HIZ_QUAD_PER_TILE_H14__GFX10                 = 0x000000c6,
    SC_P1_HIZ_QUAD_PER_TILE_H15__GFX10                 = 0x000000c7,
    SC_P1_HIZ_QUAD_PER_TILE_H16__GFX10                 = 0x000000c8,
    SC_P2_HIZ_QUAD_PER_TILE_H0__GFX10                  = 0x000000c9,
    SC_P2_HIZ_QUAD_PER_TILE_H1__GFX10                  = 0x000000ca,
    SC_P2_HIZ_QUAD_PER_TILE_H2__GFX10                  = 0x000000cb,
    SC_P2_HIZ_QUAD_PER_TILE_H3__GFX10                  = 0x000000cc,
    SC_P2_HIZ_QUAD_PER_TILE_H4__GFX10                  = 0x000000cd,
    SC_P2_HIZ_QUAD_PER_TILE_H5__GFX10                  = 0x000000ce,
    SC_P2_HIZ_QUAD_PER_TILE_H6__GFX10                  = 0x000000cf,
    SC_P2_HIZ_QUAD_PER_TILE_H7__GFX10                  = 0x000000d0,
    SC_P2_HIZ_QUAD_PER_TILE_H8__GFX10                  = 0x000000d1,
    SC_P2_HIZ_QUAD_PER_TILE_H9__GFX10                  = 0x000000d2,
    SC_P2_HIZ_QUAD_PER_TILE_H10__GFX10                 = 0x000000d3,
    SC_P2_HIZ_QUAD_PER_TILE_H11__GFX10                 = 0x000000d4,
    SC_P2_HIZ_QUAD_PER_TILE_H12__GFX10                 = 0x000000d5,
    SC_P2_HIZ_QUAD_PER_TILE_H13__GFX10                 = 0x000000d6,
    SC_P2_HIZ_QUAD_PER_TILE_H14__GFX10                 = 0x000000d7,
    SC_P2_HIZ_QUAD_PER_TILE_H15__GFX10                 = 0x000000d8,
    SC_P2_HIZ_QUAD_PER_TILE_H16__GFX10                 = 0x000000d9,
    SC_P3_HIZ_QUAD_PER_TILE_H0__GFX10                  = 0x000000da,
    SC_P3_HIZ_QUAD_PER_TILE_H1__GFX10                  = 0x000000db,
    SC_P3_HIZ_QUAD_PER_TILE_H2__GFX10                  = 0x000000dc,
    SC_P3_HIZ_QUAD_PER_TILE_H3__GFX10                  = 0x000000dd,
    SC_P3_HIZ_QUAD_PER_TILE_H4__GFX10                  = 0x000000de,
    SC_P3_HIZ_QUAD_PER_TILE_H5__GFX10                  = 0x000000df,
    SC_P3_HIZ_QUAD_PER_TILE_H6__GFX10                  = 0x000000e0,
    SC_P3_HIZ_QUAD_PER_TILE_H7__GFX10                  = 0x000000e1,
    SC_P3_HIZ_QUAD_PER_TILE_H8__GFX10                  = 0x000000e2,
    SC_P3_HIZ_QUAD_PER_TILE_H9__GFX10                  = 0x000000e3,
    SC_P3_HIZ_QUAD_PER_TILE_H10__GFX10                 = 0x000000e4,
    SC_P3_HIZ_QUAD_PER_TILE_H11__GFX10                 = 0x000000e5,
    SC_P3_HIZ_QUAD_PER_TILE_H12__GFX10                 = 0x000000e6,
    SC_P3_HIZ_QUAD_PER_TILE_H13__GFX10                 = 0x000000e7,
    SC_P3_HIZ_QUAD_PER_TILE_H14__GFX10                 = 0x000000e8,
    SC_P3_HIZ_QUAD_PER_TILE_H15__GFX10                 = 0x000000e9,
    SC_P3_HIZ_QUAD_PER_TILE_H16__GFX10                 = 0x000000ea,
    SC_P1_HIZ_QUAD_COUNT__GFX10                        = 0x000000ec,
    SC_P2_HIZ_QUAD_COUNT__GFX10                        = 0x000000ed,
    SC_P3_HIZ_QUAD_COUNT__GFX10                        = 0x000000ee,
    SC_P1_DETAIL_QUAD_COUNT__GFX10                     = 0x000000f0,
    SC_P2_DETAIL_QUAD_COUNT__GFX10                     = 0x000000f1,
    SC_P3_DETAIL_QUAD_COUNT__GFX10                     = 0x000000f2,
    SC_P1_DETAIL_QUAD_WITH_1_PIX__GFX10                = 0x000000f7,
    SC_P1_DETAIL_QUAD_WITH_2_PIX__GFX10                = 0x000000f8,
    SC_P1_DETAIL_QUAD_WITH_3_PIX__GFX10                = 0x000000f9,
    SC_P1_DETAIL_QUAD_WITH_4_PIX__GFX10                = 0x000000fa,
    SC_P2_DETAIL_QUAD_WITH_1_PIX__GFX10                = 0x000000fb,
    SC_P2_DETAIL_QUAD_WITH_2_PIX__GFX10                = 0x000000fc,
    SC_P2_DETAIL_QUAD_WITH_3_PIX__GFX10                = 0x000000fd,
    SC_P2_DETAIL_QUAD_WITH_4_PIX__GFX10                = 0x000000fe,
    SC_P3_DETAIL_QUAD_WITH_1_PIX__GFX10                = 0x000000ff,
    SC_P3_DETAIL_QUAD_WITH_2_PIX__GFX10                = 0x00000100,
    SC_P3_DETAIL_QUAD_WITH_3_PIX__GFX10                = 0x00000101,
    SC_P3_DETAIL_QUAD_WITH_4_PIX__GFX10                = 0x00000102,
    SC_PA1_SC_DATA_FIFO_RD__GFX10                      = 0x00000117,
    SC_PA1_SC_DATA_FIFO_WE__GFX10                      = 0x00000118,
    SC_PA2_SC_DATA_FIFO_RD__GFX10                      = 0x00000120,
    SC_PA2_SC_DATA_FIFO_WE__GFX10                      = 0x00000121,
    SC_PA3_SC_DATA_FIFO_RD__GFX10                      = 0x00000122,
    SC_PA3_SC_DATA_FIFO_WE__GFX10                      = 0x00000123,
    SC_PA_SC_DEALLOC_0_0_WE__GFX10                     = 0x00000124,
    SC_PA_SC_DEALLOC_0_1_WE__GFX10                     = 0x00000125,
    SC_PA_SC_DEALLOC_1_0_WE__GFX10                     = 0x00000126,
    SC_PA_SC_DEALLOC_1_1_WE__GFX10                     = 0x00000127,
    SC_PA_SC_DEALLOC_2_0_WE__GFX10                     = 0x00000128,
    SC_PA_SC_DEALLOC_2_1_WE__GFX10                     = 0x00000129,
    SC_PA_SC_DEALLOC_3_0_WE__GFX10                     = 0x0000012a,
    SC_PA_SC_DEALLOC_3_1_WE__GFX10                     = 0x0000012b,
    SC_PA0_SC_EOPG_WE__GFX10                           = 0x0000012d,
    SC_PA1_SC_EOP_WE__GFX10                            = 0x0000012f,
    SC_PA1_SC_EOPG_WE__GFX10                           = 0x00000130,
    SC_PA1_SC_EVENT_WE__GFX10                          = 0x00000131,
    SC_PA2_SC_EOP_WE__GFX10                            = 0x00000132,
    SC_PA2_SC_EOPG_WE__GFX10                           = 0x00000133,
    SC_PA2_SC_EVENT_WE__GFX10                          = 0x00000134,
    SC_PA3_SC_EOP_WE__GFX10                            = 0x00000135,
    SC_PA3_SC_EOPG_WE__GFX10                           = 0x00000136,
    SC_PA3_SC_EVENT_WE__GFX10                          = 0x00000137,
    SC_PS_ARB_OOO_THRESHOLD_SWITCH_TO_DESIRED_FIFO__GFX10 = 0x00000138,
    SC_PS_ARB_OOO_FIFO_EMPTY_SWITCH__GFX10             = 0x00000139,
    SC_PS_ARB_NULL_PRIM_BUBBLE_POP__GFX10              = 0x0000013a,
    SC_PS_ARB_EOP_POP_SYNC_POP__GFX10                  = 0x0000013b,
    SC_PS_ARB_EVENT_SYNC_POP__GFX10                    = 0x0000013c,
    SC_PA0_SC_FPOV_WE__GFX10                           = 0x0000013e,
    SC_PA1_SC_FPOV_WE__GFX10                           = 0x0000013f,
    SC_PA2_SC_FPOV_WE__GFX10                           = 0x00000140,
    SC_PA3_SC_FPOV_WE__GFX10                           = 0x00000141,
    SC_PA0_SC_LPOV_WE__GFX10                           = 0x00000142,
    SC_PA1_SC_LPOV_WE__GFX10                           = 0x00000143,
    SC_PA2_SC_LPOV_WE__GFX10                           = 0x00000144,
    SC_PA3_SC_LPOV_WE__GFX10                           = 0x00000145,
    SC_MULTICYCLE_BUBBLE_FREEZE__GFX10                 = 0x0000015b,
    SC_EOP_SYNC_WINDOW__GFX10                          = 0x0000015c,
    SC_PA0_SC_DATA_FIFO_EOPG_RD__GFX10                 = 0x0000015f,
    SC_PA0_SC_DEALLOC_0_RD__GFX10                      = 0x00000161,
    SC_PA0_SC_DEALLOC_1_RD__GFX10                      = 0x00000162,
    SC_PA1_SC_DATA_FIFO_EOPG_RD__GFX10                 = 0x00000163,
    SC_PA1_SC_DATA_FIFO_EOP_RD__GFX10                  = 0x00000164,
    SC_PA1_SC_DEALLOC_0_RD__GFX10                      = 0x00000165,
    SC_PA1_SC_DEALLOC_1_RD__GFX10                      = 0x00000166,
    SC_PA1_SC_NULL_WE__GFX10                           = 0x00000167,
    SC_PA1_SC_NULL_DEALLOC_WE__GFX10                   = 0x00000168,
    SC_PA2_SC_DATA_FIFO_EOPG_RD__GFX10                 = 0x00000169,
    SC_PA2_SC_DATA_FIFO_EOP_RD__GFX10                  = 0x0000016a,
    SC_PA2_SC_DEALLOC_0_RD__GFX10                      = 0x0000016b,
    SC_PA2_SC_DEALLOC_1_RD__GFX10                      = 0x0000016c,
    SC_PA2_SC_NULL_WE__GFX10                           = 0x0000016d,
    SC_PA2_SC_NULL_DEALLOC_WE__GFX10                   = 0x0000016e,
    SC_PA3_SC_DATA_FIFO_EOPG_RD__GFX10                 = 0x0000016f,
    SC_PA3_SC_DATA_FIFO_EOP_RD__GFX10                  = 0x00000170,
    SC_PA3_SC_DEALLOC_0_RD__GFX10                      = 0x00000171,
    SC_PA3_SC_DEALLOC_1_RD__GFX10                      = 0x00000172,
    SC_PA3_SC_NULL_WE__GFX10                           = 0x00000173,
    SC_PA3_SC_NULL_DEALLOC_WE__GFX10                   = 0x00000174,
    SC_RESERVED_0__GFX10                               = 0x00000177,
    SC_PS_PA1_SC_FIFO_EMPTY__GFX10                     = 0x00000178,
    SC_PS_PA1_SC_FIFO_FULL__GFX10                      = 0x00000179,
    SC_RESERVED_1__GFX10                               = 0x0000017a,
    SC_PS_PA2_SC_FIFO_EMPTY__GFX10                     = 0x0000017b,
    SC_PS_PA2_SC_FIFO_FULL__GFX10                      = 0x0000017c,
    SC_RESERVED_2__GFX10                               = 0x0000017d,
    SC_PS_PA3_SC_FIFO_EMPTY__GFX10                     = 0x0000017e,
    SC_PS_PA3_SC_FIFO_FULL__GFX10                      = 0x0000017f,
    SC_RESERVED_3__GFX10                               = 0x00000180,
    SC_BUSY_PROCESSING_MULTICYCLE_PRIM__GFX10          = 0x00000181,
    SC_POPS_INTRA_WAVE_OVERLAPS__GFX10                 = 0x000001a7,
    SC_POPS_FORCE_EOV__GFX10                           = 0x000001a8,
    SC_PKR_QUAD_OVLP_NOT_FOUND_IN_WAVE_TABLE_AND_WAVES_SINCE_OVLP_SET_TO_MAX__GFX10 = 0x000001a9,
    SC_PKR_QUAD_OVLP_NOT_FOUND_IN_WAVE_TABLE_AND_NO_CHANGE_TO_WAVES_SINCE_OVLP__GFX10 = 0x000001aa,
    SC_PKR_QUAD_OVLP_FOUND_IN_WAVE_TABLE__GFX10        = 0x000001ab,
    SC_FULL_FULL_QUAD__GFX10                           = 0x000001ac,
    SC_FULL_HALF_QUAD__GFX10                           = 0x000001ad,
    SC_FULL_QTR_QUAD__GFX10                            = 0x000001ae,
    SC_HALF_FULL_QUAD__GFX10                           = 0x000001af,
    SC_HALF_HALF_QUAD__GFX10                           = 0x000001b0,
    SC_HALF_QTR_QUAD__GFX10                            = 0x000001b1,
    SC_QTR_FULL_QUAD__GFX10                            = 0x000001b2,
    SC_QTR_HALF_QUAD__GFX10                            = 0x000001b3,
    SC_QTR_QTR_QUAD__GFX10                             = 0x000001b4,
    SC_DB0_TILE_INTERFACE_SEND_SOP_ONLY_EVENT__GFX10   = 0x000001de,
    SC_DB0_TILE_INTERFACE_SEND_SOP__GFX10              = 0x000001df,
    SC_DB1_TILE_INTERFACE_BUSY__GFX10                  = 0x000001e3,
    SC_DB1_TILE_INTERFACE_SEND__GFX10                  = 0x000001e4,
    SC_DB1_TILE_INTERFACE_SEND_EVENT__GFX10            = 0x000001e5,
    SC_DB1_TILE_INTERFACE_SEND_SOP_ONLY_EVENT__GFX10   = 0x000001e6,
    SC_DB1_TILE_INTERFACE_SEND_SOP__GFX10              = 0x000001e7,
    SC_DB1_TILE_INTERFACE_CREDIT_AT_ZERO_WITH_PENDING_SEND__GFX10 = 0x000001e8,
    SC_DB1_TILE_INTERFACE_CREDIT_AT_MAX__GFX10         = 0x000001e9,
    SC_DB1_TILE_INTERFACE_CREDIT_AT_MAX_WITH_NO_PENDING_SEND__GFX10 = 0x000001ea,
    SC_PBB_BATCH_BREAK_DUE_TO_TIMEOUT_COUNTER__GFX10   = 0x000001ec,
    SC_DB1_QUAD_INTF_SEND__GFX10                       = 0x000001fd,
    SC_STALLED_BY_DB1_TILEFIFO__GFX10                  = 0x000001fe,
    SC_DB1_QUAD_INTF_BUSY__GFX10                       = 0x000001ff,
    SC_DB1_QUAD_INTF_STALLED_BY_DB__GFX10              = 0x00000200,
    SC_DB1_QUAD_INTF_CREDIT_AT_MAX__GFX10              = 0x00000201,
    SC_DB1_QUAD_INTF_IDLE__GFX10                       = 0x00000202,
    SC_DB1_WE_STALLED_BY_RSLT_FIFO_FULL__GFX10         = 0x0000020b,
    SC_DB1_WE_TILE_MASK_RETURN_FIFO_FULL_WITH_WE_RSLT_FIFO_STALL__GFX10 = 0x0000020c,
    SC_DB1_TILE_MASK_FIFO_FULL__GFX10                  = 0x0000020d,
    SC_PK_PM_QD1_FD_CONFLICT_WAVE_BRK_1H__GFX10        = 0x00000214,
    SC_PK_PM_QD1_FORCE_PARTIAL_FOR_DEALLOC_WAVE_BRK_1H__GFX10 = 0x00000215,
    SC_PK_PM_QD1_AVOID_DEALLOC_ADD_WAVE_BRK_1H__GFX10  = 0x00000216,
    SC_PK_PM_LAST_AND_DEALLOC_WAVE_BRK_1H__GFX10       = 0x0000021b,
    SC_PK_PM_POPS_FORCE_EOV_WAVE_BRK_1H__GFX10         = 0x00000223,
    SC_SC_PS_ENG_MULTICYCLE_BUBBLE__GFX101             = 0x0000013d,
    SC_SC_SPI_DEALLOC_0_0__GFX101                      = 0x00000146,
    SC_SC_SPI_DEALLOC_0_1__GFX101                      = 0x00000147,
    SC_SC_SPI_DEALLOC_0_2__GFX101                      = 0x00000148,
    SC_SC_SPI_DEALLOC_1_0__GFX101                      = 0x00000149,
    SC_SC_SPI_DEALLOC_1_1__GFX101                      = 0x0000014a,
    SC_SC_SPI_DEALLOC_1_2__GFX101                      = 0x0000014b,
    SC_SC_SPI_DEALLOC_2_0__GFX101                      = 0x0000014c,
    SC_SC_SPI_DEALLOC_2_1__GFX101                      = 0x0000014d,
    SC_SC_SPI_DEALLOC_2_2__GFX101                      = 0x0000014e,
    SC_SC_SPI_DEALLOC_3_0__GFX101                      = 0x0000014f,
    SC_SC_SPI_DEALLOC_3_1__GFX101                      = 0x00000150,
    SC_SC_SPI_DEALLOC_3_2__GFX101                      = 0x00000151,
    SC_SC_SPI_FPOV_0__GFX101                           = 0x00000152,
    SC_SC_SPI_FPOV_1__GFX101                           = 0x00000153,
    SC_SC_SPI_FPOV_2__GFX101                           = 0x00000154,
    SC_SC_SPI_FPOV_3__GFX101                           = 0x00000155,
    SC_SC_SPI_EVENT__GFX101                            = 0x00000156,
    SC_SPI_EVENT__GFX103PLUS                           = 0x00000156,
    SC_PK_PM_VRS_RATE_X_00_Y_00_QUAD__GFX103PLUS       = 0x00000228,
    SC_PK_PM_VRS_RATE_X_00_Y_01_QUAD__GFX103PLUS       = 0x00000229,
    SC_PK_PM_VRS_RATE_X_00_Y_10_QUAD__GFX103PLUS       = 0x0000022a,
    SC_PK_PM_VRS_RATE_X_00_Y_11_QUAD__GFX103PLUS       = 0x0000022b,
    SC_PK_PM_VRS_RATE_X_01_Y_00_QUAD__GFX103PLUS       = 0x0000022c,
    SC_PK_PM_VRS_RATE_X_01_Y_01_QUAD__GFX103PLUS       = 0x0000022d,
    SC_PK_PM_VRS_RATE_X_01_Y_10_QUAD__GFX103PLUS       = 0x0000022e,
    SC_PK_PM_VRS_RATE_X_01_Y_11_QUAD__GFX103PLUS       = 0x0000022f,
    SC_PK_PM_VRS_RATE_X_10_Y_00_QUAD__GFX103PLUS       = 0x00000230,
    SC_PK_PM_VRS_RATE_X_10_Y_01_QUAD__GFX103PLUS       = 0x00000231,
    SC_PK_PM_VRS_RATE_X_10_Y_10_QUAD__GFX103PLUS       = 0x00000232,
    SC_PK_PM_VRS_RATE_X_10_Y_11_QUAD__GFX103PLUS       = 0x00000233,
    SC_PK_PM_VRS_RATE_X_11_Y_00_QUAD__GFX103PLUS       = 0x00000234,
    SC_PK_PM_VRS_RATE_X_11_Y_01_QUAD__GFX103PLUS       = 0x00000235,
    SC_PK_PM_VRS_RATE_X_11_Y_10_QUAD__GFX103PLUS       = 0x00000236,
    SC_PK_PM_VRS_RATE_X_11_Y_11_QUAD__GFX103PLUS       = 0x00000237,
    SC_PBB_RESERVED__GFX103PLUS                        = 0x00000239,
    SC_BM_BE0_STALLED__GFX103PLUS                      = 0x0000023a,
    SC_BM_BE1_STALLED__GFX103PLUS                      = 0x0000023b,
    SC_BM_BE2_STALLED__GFX103PLUS                      = 0x0000023c,
    SC_BM_BE3_STALLED__GFX103PLUS                      = 0x0000023d,
    SC_BM_MULTI_ACCUM_1_BE_STALLED__GFX103PLUS         = 0x0000023e,
    SC_BM_MULTI_ACCUM_2_BE_STALLED__GFX103PLUS         = 0x0000023f,
    SC_BM_MULTI_ACCUM_3_BE_STALLED__GFX103PLUS         = 0x00000240,
    SC_BM_MULTI_ACCUM_4_BE_STALLED__GFX103PLUS         = 0x00000241,
    SC_PS_ENG_MULTICYCLE_BUBBLE__GFX10VRS              = 0x0000013d,
    SC_SPI_DEALLOC_0_0__GFX10VRS                       = 0x00000146,
    SC_SPI_DEALLOC_0_1__GFX10VRS                       = 0x00000147,
    SC_SPI_DEALLOC_0_2__GFX10VRS                       = 0x00000148,
    SC_SPI_DEALLOC_1_0__GFX10VRS                       = 0x00000149,
    SC_SPI_DEALLOC_1_1__GFX10VRS                       = 0x0000014a,
    SC_SPI_DEALLOC_1_2__GFX10VRS                       = 0x0000014b,
    SC_SPI_DEALLOC_2_0__GFX10VRS                       = 0x0000014c,
    SC_SPI_DEALLOC_2_1__GFX10VRS                       = 0x0000014d,
    SC_SPI_DEALLOC_2_2__GFX10VRS                       = 0x0000014e,
    SC_SPI_DEALLOC_3_0__GFX10VRS                       = 0x0000014f,
    SC_SPI_DEALLOC_3_1__GFX10VRS                       = 0x00000150,
    SC_SPI_DEALLOC_3_2__GFX10VRS                       = 0x00000151,
    SC_SPI_FPOV_0__GFX10VRS                            = 0x00000152,
    SC_SPI_FPOV_1__GFX10VRS                            = 0x00000153,
    SC_SPI_FPOV_2__GFX10VRS                            = 0x00000154,
    SC_SPI_FPOV_3__GFX10VRS                            = 0x00000155,
    SC_PBB_BATCH_BREAK_DUE_TO_PIPELINE_MODE_CHANGE__GFX10VRS = 0x00000238,
    SC_PERF_SEL_RESERVED_76__GFX11                     = 0x0000004c,
    SC_PERF_SEL_RESERVED_77__GFX11                     = 0x0000004d,
    SC_PERF_SEL_RESERVED_78__GFX11                     = 0x0000004e,
    SC_PERF_SEL_RESERVED_80__GFX11                     = 0x00000050,
    SC_PERF_SEL_RESERVED_81__GFX11                     = 0x00000051,
    SC_PERF_SEL_RESERVED_82__GFX11                     = 0x00000052,
    SC_PERF_SEL_RESERVED_84__GFX11                     = 0x00000054,
    SC_PERF_SEL_RESERVED_85__GFX11                     = 0x00000055,
    SC_PERF_SEL_RESERVED_86__GFX11                     = 0x00000056,
    SC_PERF_SEL_RESERVED_88__GFX11                     = 0x00000058,
    SC_PERF_SEL_RESERVED_89__GFX11                     = 0x00000059,
    SC_PERF_SEL_RESERVED_90__GFX11                     = 0x0000005a,
    SC_PERF_SEL_RESERVED_108__GFX11                    = 0x0000006c,
    SC_PERF_SEL_RESERVED_109__GFX11                    = 0x0000006d,
    SC_PERF_SEL_RESERVED_110__GFX11                    = 0x0000006e,
    SC_PERF_SEL_RESERVED_111__GFX11                    = 0x0000006f,
    SC_PERF_SEL_RESERVED_112__GFX11                    = 0x00000070,
    SC_PERF_SEL_RESERVED_113__GFX11                    = 0x00000071,
    SC_PERF_SEL_RESERVED_114__GFX11                    = 0x00000072,
    SC_PERF_SEL_RESERVED_115__GFX11                    = 0x00000073,
    SC_PERF_SEL_RESERVED_116__GFX11                    = 0x00000074,
    SC_PERF_SEL_RESERVED_117__GFX11                    = 0x00000075,
    SC_PERF_SEL_RESERVED_118__GFX11                    = 0x00000076,
    SC_PERF_SEL_RESERVED_119__GFX11                    = 0x00000077,
    SC_PERF_SEL_RESERVED_120__GFX11                    = 0x00000078,
    SC_PERF_SEL_RESERVED_121__GFX11                    = 0x00000079,
    SC_PERF_SEL_RESERVED_122__GFX11                    = 0x0000007a,
    SC_PERF_SEL_RESERVED_123__GFX11                    = 0x0000007b,
    SC_PERF_SEL_RESERVED_124__GFX11                    = 0x0000007c,
    SC_PERF_SEL_RESERVED_125__GFX11                    = 0x0000007d,
    SC_PERF_SEL_RESERVED_126__GFX11                    = 0x0000007e,
    SC_PERF_SEL_RESERVED_127__GFX11                    = 0x0000007f,
    SC_PERF_SEL_RESERVED_128__GFX11                    = 0x00000080,
    SC_PERF_SEL_RESERVED_129__GFX11                    = 0x00000081,
    SC_PERF_SEL_RESERVED_130__GFX11                    = 0x00000082,
    SC_PERF_SEL_RESERVED_131__GFX11                    = 0x00000083,
    SC_PERF_SEL_RESERVED_132__GFX11                    = 0x00000084,
    SC_PERF_SEL_RESERVED_133__GFX11                    = 0x00000085,
    SC_PERF_SEL_RESERVED_134__GFX11                    = 0x00000086,
    SC_PERF_SEL_RESERVED_135__GFX11                    = 0x00000087,
    SC_PERF_SEL_RESERVED_136__GFX11                    = 0x00000088,
    SC_PERF_SEL_RESERVED_137__GFX11                    = 0x00000089,
    SC_PERF_SEL_RESERVED_138__GFX11                    = 0x0000008a,
    SC_PERF_SEL_RESERVED_139__GFX11                    = 0x0000008b,
    SC_PERF_SEL_RESERVED_140__GFX11                    = 0x0000008c,
    SC_PERF_SEL_RESERVED_141__GFX11                    = 0x0000008d,
    SC_PERF_SEL_RESERVED_142__GFX11                    = 0x0000008e,
    SC_PERF_SEL_RESERVED_143__GFX11                    = 0x0000008f,
    SC_PERF_SEL_RESERVED_144__GFX11                    = 0x00000090,
    SC_PERF_SEL_RESERVED_145__GFX11                    = 0x00000091,
    SC_PERF_SEL_RESERVED_146__GFX11                    = 0x00000092,
    SC_PERF_SEL_RESERVED_147__GFX11                    = 0x00000093,
    SC_PERF_SEL_RESERVED_148__GFX11                    = 0x00000094,
    SC_PERF_SEL_RESERVED_149__GFX11                    = 0x00000095,
    SC_PERF_SEL_RESERVED_150__GFX11                    = 0x00000096,
    SC_PERF_SEL_RESERVED_151__GFX11                    = 0x00000097,
    SC_PERF_SEL_RESERVED_152__GFX11                    = 0x00000098,
    SC_PERF_SEL_RESERVED_153__GFX11                    = 0x00000099,
    SC_PERF_SEL_RESERVED_154__GFX11                    = 0x0000009a,
    SC_PERF_SEL_RESERVED_155__GFX11                    = 0x0000009b,
    SC_PERF_SEL_RESERVED_156__GFX11                    = 0x0000009c,
    SC_PERF_SEL_RESERVED_157__GFX11                    = 0x0000009d,
    SC_PERF_SEL_RESERVED_158__GFX11                    = 0x0000009e,
    SC_PERF_SEL_RESERVED_160__GFX11                    = 0x000000a0,
    SC_PERF_SEL_RESERVED_161__GFX11                    = 0x000000a1,
    SC_PERF_SEL_RESERVED_162__GFX11                    = 0x000000a2,
    SC_PERF_SEL_RESERVED_164__GFX11                    = 0x000000a4,
    SC_PERF_SEL_RESERVED_165__GFX11                    = 0x000000a5,
    SC_PERF_SEL_RESERVED_166__GFX11                    = 0x000000a6,
    SC_PERF_SEL_RESERVED_184__GFX11                    = 0x000000b8,
    SC_PERF_SEL_RESERVED_185__GFX11                    = 0x000000b9,
    SC_PERF_SEL_RESERVED_186__GFX11                    = 0x000000ba,
    SC_PERF_SEL_RESERVED_187__GFX11                    = 0x000000bb,
    SC_PERF_SEL_RESERVED_188__GFX11                    = 0x000000bc,
    SC_PERF_SEL_RESERVED_189__GFX11                    = 0x000000bd,
    SC_PERF_SEL_RESERVED_190__GFX11                    = 0x000000be,
    SC_PERF_SEL_RESERVED_191__GFX11                    = 0x000000bf,
    SC_PERF_SEL_RESERVED_192__GFX11                    = 0x000000c0,
    SC_PERF_SEL_RESERVED_193__GFX11                    = 0x000000c1,
    SC_PERF_SEL_RESERVED_194__GFX11                    = 0x000000c2,
    SC_PERF_SEL_RESERVED_195__GFX11                    = 0x000000c3,
    SC_PERF_SEL_RESERVED_196__GFX11                    = 0x000000c4,
    SC_PERF_SEL_RESERVED_197__GFX11                    = 0x000000c5,
    SC_PERF_SEL_RESERVED_198__GFX11                    = 0x000000c6,
    SC_PERF_SEL_RESERVED_199__GFX11                    = 0x000000c7,
    SC_PERF_SEL_RESERVED_200__GFX11                    = 0x000000c8,
    SC_PERF_SEL_RESERVED_201__GFX11                    = 0x000000c9,
    SC_PERF_SEL_RESERVED_202__GFX11                    = 0x000000ca,
    SC_PERF_SEL_RESERVED_203__GFX11                    = 0x000000cb,
    SC_PERF_SEL_RESERVED_204__GFX11                    = 0x000000cc,
    SC_PERF_SEL_RESERVED_205__GFX11                    = 0x000000cd,
    SC_PERF_SEL_RESERVED_206__GFX11                    = 0x000000ce,
    SC_PERF_SEL_RESERVED_207__GFX11                    = 0x000000cf,
    SC_PERF_SEL_RESERVED_208__GFX11                    = 0x000000d0,
    SC_PERF_SEL_RESERVED_209__GFX11                    = 0x000000d1,
    SC_PERF_SEL_RESERVED_210__GFX11                    = 0x000000d2,
    SC_PERF_SEL_RESERVED_211__GFX11                    = 0x000000d3,
    SC_PERF_SEL_RESERVED_212__GFX11                    = 0x000000d4,
    SC_PERF_SEL_RESERVED_213__GFX11                    = 0x000000d5,
    SC_PERF_SEL_RESERVED_214__GFX11                    = 0x000000d6,
    SC_PERF_SEL_RESERVED_215__GFX11                    = 0x000000d7,
    SC_PERF_SEL_RESERVED_216__GFX11                    = 0x000000d8,
    SC_PERF_SEL_RESERVED_217__GFX11                    = 0x000000d9,
    SC_PERF_SEL_RESERVED_218__GFX11                    = 0x000000da,
    SC_PERF_SEL_RESERVED_219__GFX11                    = 0x000000db,
    SC_PERF_SEL_RESERVED_220__GFX11                    = 0x000000dc,
    SC_PERF_SEL_RESERVED_221__GFX11                    = 0x000000dd,
    SC_PERF_SEL_RESERVED_222__GFX11                    = 0x000000de,
    SC_PERF_SEL_RESERVED_223__GFX11                    = 0x000000df,
    SC_PERF_SEL_RESERVED_224__GFX11                    = 0x000000e0,
    SC_PERF_SEL_RESERVED_225__GFX11                    = 0x000000e1,
    SC_PERF_SEL_RESERVED_226__GFX11                    = 0x000000e2,
    SC_PERF_SEL_RESERVED_227__GFX11                    = 0x000000e3,
    SC_PERF_SEL_RESERVED_228__GFX11                    = 0x000000e4,
    SC_PERF_SEL_RESERVED_229__GFX11                    = 0x000000e5,
    SC_PERF_SEL_RESERVED_230__GFX11                    = 0x000000e6,
    SC_PERF_SEL_RESERVED_231__GFX11                    = 0x000000e7,
    SC_PERF_SEL_RESERVED_232__GFX11                    = 0x000000e8,
    SC_PERF_SEL_RESERVED_233__GFX11                    = 0x000000e9,
    SC_PERF_SEL_RESERVED_234__GFX11                    = 0x000000ea,
    SC_PERF_SEL_RESERVED_236__GFX11                    = 0x000000ec,
    SC_PERF_SEL_RESERVED_237__GFX11                    = 0x000000ed,
    SC_PERF_SEL_RESERVED_238__GFX11                    = 0x000000ee,
    SC_PERF_SEL_RESERVED_240__GFX11                    = 0x000000f0,
    SC_PERF_SEL_RESERVED_241__GFX11                    = 0x000000f1,
    SC_PERF_SEL_RESERVED_242__GFX11                    = 0x000000f2,
    SC_PERF_SEL_RESERVED_247__GFX11                    = 0x000000f7,
    SC_PERF_SEL_RESERVED_248__GFX11                    = 0x000000f8,
    SC_PERF_SEL_RESERVED_249__GFX11                    = 0x000000f9,
    SC_PERF_SEL_RESERVED_250__GFX11                    = 0x000000fa,
    SC_PERF_SEL_RESERVED_251__GFX11                    = 0x000000fb,
    SC_PERF_SEL_RESERVED_252__GFX11                    = 0x000000fc,
    SC_PERF_SEL_RESERVED_253__GFX11                    = 0x000000fd,
    SC_PERF_SEL_RESERVED_254__GFX11                    = 0x000000fe,
    SC_PERF_SEL_RESERVED_255__GFX11                    = 0x000000ff,
    SC_PERF_SEL_RESERVED_256__GFX11                    = 0x00000100,
    SC_PERF_SEL_RESERVED_257__GFX11                    = 0x00000101,
    SC_PERF_SEL_RESERVED_258__GFX11                    = 0x00000102,
    SC_PERF_SEL_RESERVED_279__GFX11                    = 0x00000117,
    SC_PERF_SEL_RESERVED_280__GFX11                    = 0x00000118,
    SC_PERF_SEL_RESERVED_288__GFX11                    = 0x00000120,
    SC_PERF_SEL_RESERVED_289__GFX11                    = 0x00000121,
    SC_PERF_SEL_RESERVED_290__GFX11                    = 0x00000122,
    SC_PERF_SEL_RESERVED_291__GFX11                    = 0x00000123,
    SC_PA_SC_DEALLOC_2_0_WE__GFX11                     = 0x00000124,
    SC_PERF_SEL_RESERVED_293__GFX11                    = 0x00000125,
    SC_PERF_SEL_RESERVED_294__GFX11                    = 0x00000126,
    SC_PERF_SEL_RESERVED_295__GFX11                    = 0x00000127,
    SC_PERF_SEL_RESERVED_296__GFX11                    = 0x00000128,
    SC_PERF_SEL_RESERVED_297__GFX11                    = 0x00000129,
    SC_PERF_SEL_RESERVED_298__GFX11                    = 0x0000012a,
    SC_PERF_SEL_RESERVED_299__GFX11                    = 0x0000012b,
    SC_PERF_SEL_RESERVED_301__GFX11                    = 0x0000012d,
    SC_PERF_SEL_RESERVED_303__GFX11                    = 0x0000012f,
    SC_PERF_SEL_RESERVED_304__GFX11                    = 0x00000130,
    SC_PERF_SEL_RESERVED_305__GFX11                    = 0x00000131,
    SC_PERF_SEL_RESERVED_306__GFX11                    = 0x00000132,
    SC_PERF_SEL_RESERVED_307__GFX11                    = 0x00000133,
    SC_PERF_SEL_RESERVED_308__GFX11                    = 0x00000134,
    SC_PERF_SEL_RESERVED_309__GFX11                    = 0x00000135,
    SC_PERF_SEL_RESERVED_310__GFX11                    = 0x00000136,
    SC_PERF_SEL_RESERVED_311__GFX11                    = 0x00000137,
    SC_PERF_SEL_RESERVED_312__GFX11                    = 0x00000138,
    SC_PERF_SEL_RESERVED_313__GFX11                    = 0x00000139,
    SC_PERF_SEL_RESERVED_314__GFX11                    = 0x0000013a,
    SC_PERF_SEL_RESERVED_315__GFX11                    = 0x0000013b,
    SC_PERF_SEL_RESERVED_316__GFX11                    = 0x0000013c,
    SC_PERF_SEL_RESERVED_317__GFX11                    = 0x0000013d,
    SC_PA_SC_FPOV_WE__GFX11                            = 0x0000013e,
    SC_PERF_SEL_RESERVED_319__GFX11                    = 0x0000013f,
    SC_PERF_SEL_RESERVED_320__GFX11                    = 0x00000140,
    SC_PERF_SEL_RESERVED_321__GFX11                    = 0x00000141,
    SC_PERF_SEL_RESERVED_322__GFX11                    = 0x00000142,
    SC_PERF_SEL_RESERVED_323__GFX11                    = 0x00000143,
    SC_PERF_SEL_RESERVED_324__GFX11                    = 0x00000144,
    SC_PERF_SEL_RESERVED_325__GFX11                    = 0x00000145,
    SC_SPI_DEALLOC_4_0__GFX11                          = 0x00000146,
    SC_SPI_DEALLOC_7_5__GFX11                          = 0x00000147,
    SC_PERF_SEL_RESERVED_328__GFX11                    = 0x00000148,
    SC_PERF_SEL_RESERVED_329__GFX11                    = 0x00000149,
    SC_PERF_SEL_RESERVED_330__GFX11                    = 0x0000014a,
    SC_PERF_SEL_RESERVED_331__GFX11                    = 0x0000014b,
    SC_PERF_SEL_RESERVED_332__GFX11                    = 0x0000014c,
    SC_PERF_SEL_RESERVED_333__GFX11                    = 0x0000014d,
    SC_PERF_SEL_RESERVED_334__GFX11                    = 0x0000014e,
    SC_PERF_SEL_RESERVED_335__GFX11                    = 0x0000014f,
    SC_PERF_SEL_RESERVED_336__GFX11                    = 0x00000150,
    SC_PERF_SEL_RESERVED_337__GFX11                    = 0x00000151,
    SC_SPI_FPOV_4_0__GFX11                             = 0x00000152,
    SC_SPI_FPOV_7_5__GFX11                             = 0x00000153,
    SC_PERF_SEL_RESERVED_340__GFX11                    = 0x00000154,
    SC_PERF_SEL_RESERVED_341__GFX11                    = 0x00000155,
    SC_PERF_SEL_RESERVED_347__GFX11                    = 0x0000015b,
    SC_PERF_SEL_RESERVED_348__GFX11                    = 0x0000015c,
    SC_PERF_SEL_RESERVED_351__GFX11                    = 0x0000015f,
    SC_PA0_SC_DEALLOC_2_0_RD__GFX11                    = 0x00000161,
    SC_PERF_SEL_RESERVED_354__GFX11                    = 0x00000162,
    SC_PERF_SEL_RESERVED_355__GFX11                    = 0x00000163,
    SC_PERF_SEL_RESERVED_356__GFX11                    = 0x00000164,
    SC_PERF_SEL_RESERVED_357__GFX11                    = 0x00000165,
    SC_PERF_SEL_RESERVED_358__GFX11                    = 0x00000166,
    SC_PERF_SEL_RESERVED_359__GFX11                    = 0x00000167,
    SC_PERF_SEL_RESERVED_360__GFX11                    = 0x00000168,
    SC_PERF_SEL_RESERVED_361__GFX11                    = 0x00000169,
    SC_PERF_SEL_RESERVED_362__GFX11                    = 0x0000016a,
    SC_PERF_SEL_RESERVED_363__GFX11                    = 0x0000016b,
    SC_PERF_SEL_RESERVED_364__GFX11                    = 0x0000016c,
    SC_PERF_SEL_RESERVED_365__GFX11                    = 0x0000016d,
    SC_PERF_SEL_RESERVED_366__GFX11                    = 0x0000016e,
    SC_PERF_SEL_RESERVED_367__GFX11                    = 0x0000016f,
    SC_PERF_SEL_RESERVED_368__GFX11                    = 0x00000170,
    SC_PERF_SEL_RESERVED_369__GFX11                    = 0x00000171,
    SC_PERF_SEL_RESERVED_370__GFX11                    = 0x00000172,
    SC_PERF_SEL_RESERVED_371__GFX11                    = 0x00000173,
    SC_PERF_SEL_RESERVED_372__GFX11                    = 0x00000174,
    SC_PERF_SEL_RESERVED_375__GFX11                    = 0x00000177,
    SC_PERF_SEL_RESERVED_376__GFX11                    = 0x00000178,
    SC_PERF_SEL_RESERVED_377__GFX11                    = 0x00000179,
    SC_PERF_SEL_RESERVED_378__GFX11                    = 0x0000017a,
    SC_PERF_SEL_RESERVED_379__GFX11                    = 0x0000017b,
    SC_PERF_SEL_RESERVED_380__GFX11                    = 0x0000017c,
    SC_PERF_SEL_RESERVED_381__GFX11                    = 0x0000017d,
    SC_PERF_SEL_RESERVED_382__GFX11                    = 0x0000017e,
    SC_PERF_SEL_RESERVED_383__GFX11                    = 0x0000017f,
    SC_PERF_SEL_RESERVED_384__GFX11                    = 0x00000180,
    SC_PERF_SEL_RESERVED_385__GFX11                    = 0x00000181,
    SC_PERF_SEL_RESERVED_423__GFX11                    = 0x000001a7,
    SC_PERF_SEL_RESERVED_424__GFX11                    = 0x000001a8,
    SC_PERF_SEL_RESERVED_425__GFX11                    = 0x000001a9,
    SC_PERF_SEL_RESERVED_426__GFX11                    = 0x000001aa,
    SC_PERF_SEL_RESERVED_427__GFX11                    = 0x000001ab,
    SC_PERF_SEL_RESERVED_428__GFX11                    = 0x000001ac,
    SC_PERF_SEL_RESERVED_429__GFX11                    = 0x000001ad,
    SC_PERF_SEL_RESERVED_430__GFX11                    = 0x000001ae,
    SC_PERF_SEL_RESERVED_431__GFX11                    = 0x000001af,
    SC_PERF_SEL_RESERVED_432__GFX11                    = 0x000001b0,
    SC_PERF_SEL_RESERVED_433__GFX11                    = 0x000001b1,
    SC_PERF_SEL_RESERVED_434__GFX11                    = 0x000001b2,
    SC_PERF_SEL_RESERVED_435__GFX11                    = 0x000001b3,
    SC_PERF_SEL_RESERVED_436__GFX11                    = 0x000001b4,
    SC_PERF_SEL_RESERVED_478__GFX11                    = 0x000001de,
    SC_PERF_SEL_RESERVED_479__GFX11                    = 0x000001df,
    SC_PERF_SEL_RESERVED_483__GFX11                    = 0x000001e3,
    SC_PERF_SEL_RESERVED_484__GFX11                    = 0x000001e4,
    SC_PERF_SEL_RESERVED_485__GFX11                    = 0x000001e5,
    SC_PERF_SEL_RESERVED_486__GFX11                    = 0x000001e6,
    SC_PERF_SEL_RESERVED_487__GFX11                    = 0x000001e7,
    SC_PERF_SEL_RESERVED_488__GFX11                    = 0x000001e8,
    SC_PERF_SEL_RESERVED_489__GFX11                    = 0x000001e9,
    SC_PERF_SEL_RESERVED_490__GFX11                    = 0x000001ea,
    SC_PERF_SEL_RESERVED_509__GFX11                    = 0x000001fd,
    SC_PERF_SEL_RESERVED_510__GFX11                    = 0x000001fe,
    SC_PERF_SEL_RESERVED_511__GFX11                    = 0x000001ff,
    SC_PERF_SEL_RESERVED_512__GFX11                    = 0x00000200,
    SC_PERF_SEL_RESERVED_513__GFX11                    = 0x00000201,
    SC_PERF_SEL_RESERVED_514__GFX11                    = 0x00000202,
    SC_PERF_SEL_RESERVED_523__GFX11                    = 0x0000020b,
    SC_PERF_SEL_RESERVED_524__GFX11                    = 0x0000020c,
    SC_PERF_SEL_RESERVED_525__GFX11                    = 0x0000020d,
    SC_PERF_SEL_RESERVED_532__GFX11                    = 0x00000214,
    SC_PERF_SEL_RESERVED_533__GFX11                    = 0x00000215,
    SC_PERF_SEL_RESERVED_534__GFX11                    = 0x00000216,
    SC_PERF_SEL_RESERVED_539__GFX11                    = 0x0000021b,
    SC_PK_PM_OREO_CONFLICT_QUAD_FORCE_EOV_WAVE_BRK_1H__GFX11 = 0x00000223,
    SC_PERF_SEL_RESERVED_568__GFX11                    = 0x00000238,
    SC_PBB_READ_PH0__GFX11                             = 0x00000242,
    SC_PBB_READ_DEALLOC_4_0__GFX11                     = 0x00000243,
    SC_PBB_READ_DEALLOC_7_5__GFX11                     = 0x00000244,
    SC_PBB_READ_FPOG_4_0__GFX11                        = 0x00000245,
    SC_PBB_READ_FPOG_7_5__GFX11                        = 0x00000246,
    SC_VRC_SECTOR_HIT__GFX11                           = 0x00000247,
    SC_VRC_TAG_MISS__GFX11                             = 0x00000248,
    SC_VRC_SECTOR_MISS__GFX11                          = 0x00000249,
    SC_VRC_LRU_EVICT_STALL__GFX11                      = 0x0000024a,
    SC_VRC_LRU_EVICT_SCHEDULED_EVICT_STALL__GFX11      = 0x0000024b,
    SC_VRC_LRU_EVICT_PENDING_EVICT_STALL__GFX11        = 0x0000024c,
    SC_VRC_REEVICTION_STALL__GFX11                     = 0x0000024d,
    SC_VRC_EVICT_NONZERO_INFLIGHT_STALL__GFX11         = 0x0000024e,
    SC_VRC_REPLACE_SCHEDULED_EVICT_STALL__GFX11        = 0x0000024f,
    SC_VRC_REPLACE_PENDING_EVICT_STALL__GFX11          = 0x00000250,
    SC_VRC_REPLACE_FLUSH_IN_PROGRESS_STALL__GFX11      = 0x00000251,
    SC_VRC_INFLIGHT_COUNTER_MAXIMUM_STALL__GFX11       = 0x00000252,
    SC_VRC_READ_OUTPUT_STALL__GFX11                    = 0x00000253,
    SC_VRC_WRITE_OUTPUT_STALL__GFX11                   = 0x00000254,
    SC_VRC_ACK_OUTPUT_STALL__GFX11                     = 0x00000255,
    SC_VRC_FLUSH_EVICT_STALL__GFX11                    = 0x00000256,
    SC_VRC_FLUSH_REFLUSH_STALL__GFX11                  = 0x00000257,
    SC_VRC_FLUSH_FIP_HIT_STALL__GFX11                  = 0x00000258,
    SC_VRC_FLUSH_WRREQ_DRAIN_STALL__GFX11              = 0x00000259,
    SC_VRC_FLUSH_DONE_STALL__GFX11                     = 0x0000025a,
    SC_VRC_FLUSH_STALL__GFX11                          = 0x0000025b,
    SC_VRC_STALL__GFX11                                = 0x0000025c,
    SC_VRC_SECTORS_FLUSHED__GFX11                      = 0x0000025d,
    SC_VRC_DIRTY_SECTORS_FLUSHED__GFX11                = 0x0000025e,
    SC_VRC_TAGS_FLUSHED__GFX11                         = 0x0000025f,
    SC_VRC_VRF_REQ__GFX11                              = 0x00000260,
    SC_VRC_VRF_EVENT__GFX11                            = 0x00000261,
    SC_VRC_VRF_STALLED__GFX11                          = 0x00000262,
    SC_VRC_PROBE_ACK_TILES__GFX11                      = 0x00000263,
    SC_VRC_GL1H_RD_REQ__GFX11                          = 0x00000264,
    SC_VRC_GL1H_WR_REQ__GFX11                          = 0x00000265,
    SC_VRC_GL1H_SRC_XFR__GFX11                         = 0x00000266,
    SC_VRC_GL1H_RD_RET_0__GFX11                        = 0x00000267,
    SC_VRC_GL1H_RD_RET_1__GFX11                        = 0x00000268,
    SC_VRC_GL1H_WR_ACK_0__GFX11                        = 0x00000269,
    SC_VRC_GL1H_WR_ACK_1__GFX11                        = 0x0000026a,
    SC_VRC_GL1H_RD_XNACK_0__GFX11                      = 0x0000026b,
    SC_VRC_GL1H_RD_XNACK_1__GFX11                      = 0x0000026c,
    SC_VRC_GL1H_WR_XNACK_0__GFX11                      = 0x0000026d,
    SC_VRC_GL1H_WR_XNACK_1__GFX11                      = 0x0000026e,
    SC_VRC_GL1H_REQ_STALLED__GFX11                     = 0x0000026f,
    SC_VRC_GL1H_SRC_STALLED__GFX11                     = 0x00000270,
    SC_VRC_RATEMEM_WE_CNT__GFX11                       = 0x00000271,
    SC_VRC_RATEMEM_RE_CNT__GFX11                       = 0x00000272,
    SC_VRC_HINTMEM_WE_CNT__GFX11                       = 0x00000273,
    SC_VRC_HINTMEM_RE_CNT__GFX11                       = 0x00000274,
    SC_VRC_BUSY__GFX11                                 = 0x00000275,
    SC_GL1H_BUSY__GFX11                                = 0x00000276,
    SC_BE_VRS_RD_REQ__GFX11                            = 0x00000277,
    SC_BE_VRS_RD_REQ_STALLED__GFX11                    = 0x00000278,
    SC_BE_VRS_RD_REQ_HIT__GFX11                        = 0x00000279,
    SC_BE_VRS_RD_RET__GFX11                            = 0x0000027a,
    SC_BE_VRS_RD_RET_STALLED__GFX11                    = 0x0000027b,
    SC_BE_VRS_FB_RET__GFX11                            = 0x0000027c,
    SC_BE_VRS_FB_RET_STALLED__GFX11                    = 0x0000027d,
    SC_BE_VRS_FB_RET_HIT__GFX11                        = 0x0000027e,
    SC_VRS_BE_BUSY__GFX11                              = 0x0000027f,
    SC_PWS_CS_EVENTS_PWS_ENABLE__GFX11                 = 0x00000280,
    SC_PWS_PS_EVENTS_PWS_ENABLE__GFX11                 = 0x00000281,
    SC_PWS_TS_EVENTS_PWS_ENABLE__GFX11                 = 0x00000282,
    SC_PWS_STALLED__GFX11                              = 0x00000283,
    SC_PWS_P0_CS_SYNC_COMPLETE__GFX11                  = 0x00000284,
    SC_PWS_P0_PS_SYNC_COMPLETE__GFX11                  = 0x00000285,
    SC_PWS_P0_TS_SYNC_COMPLETE__GFX11                  = 0x00000286,
    SC_PWS_P1_CS_SYNC_COMPLETE__GFX11                  = 0x00000287,
    SC_PWS_P1_PS_SYNC_COMPLETE__GFX11                  = 0x00000288,
    SC_PWS_P1_TS_SYNC_COMPLETE__GFX11                  = 0x00000289,
    SC_PKR_PC_NO_CREDITS__GFX11                        = 0x0000028a,
    SC_PKR_PC_STALLED__GFX11                           = 0x0000028b,
    SC_PKR_PC_SEND__GFX11                              = 0x0000028c,
    SC_PKR_PC_SEND_PRIM_VALID_1__GFX11                 = 0x0000028d,
    SC_PKR_PC_SEND_PRIM_VALID_0__GFX11                 = 0x0000028e,
    SC_PKR_PC_SEND_TRUE_PRIM__GFX11                    = 0x0000028f,
    SC_PKR_PC_SEND_EOV__GFX11                          = 0x00000290,
    SC_PKR_PC_SEND_EVENT__GFX11                        = 0x00000291,
    SC_PKR_DB_WAVE_STALL__GFX11                        = 0x00000292,
    SC_PKR_PSINVOC_SEDC_FIFO_FULL__GFX11               = 0x00000293,
    SC_PKR_OREO_STALLED_BY_NO_VALID_WAIVE_ID__GFX11    = 0x00000294,
    SC_PKR_SPI_QUAD_COUNT__GFX11                       = 0x00000295,
    SC_PKR_DB_OREO_WAVE_QUAD_COUNT__GFX11              = 0x00000296,
    SC_PKR_BCI_QUAD_NEW_PRIM__GFX11                    = 0x00000297,
    SC_SPI_WAVE_STALLED_BY_SPI__GFX11                  = 0x00000298,
    SC_PBB_BATCH_BREAK_DUE_TO_TIMEOUT_COUNTER__GFX110  = 0x000001ec,
    SC_VRC_REPROBE_XFR__NV33                           = 0x00000299,
    SC_VRC_REPROBE_FULL__NV33                          = 0x0000029a,
    SC_PBB_BATCH_BREAK_DUE_TO_TIMEOUT_COUNTER__STX1    = 0x000001ec,
#if CHIP_HDR_STRIX_HALO
    SC_PERF_SEL_RESERVED_492__STXH                     = 0x000001ec,
    SC_PBB_BATCH_BREAK_DUE_TO_LSO_ENABLE__STXH         = 0x0000029b,
    SC_PBB_BATCH_BREAK_DUE_TO_INPUT_TIMEOUT_COUNTER__STXH = 0x0000029c,
    SC_PBB_BATCH_BREAK_DUE_TO_TIMEOUT_COUNTER__STXH    = 0x0000029d,
    SC_PBB_BATCH_BREAK_DUE_TO_ZPP_ENABLE__STXH         = 0x0000029e,
    SC_PBB_BATCH_BREAK_DUE_TO_ZPP_ZFUNC__STXH          = 0x0000029f,
    SC_PBB_BATCH_BREAK_DUE_TO_ZPP_CBMASK__STXH         = 0x000002a0,
    SC_PBB_BATCH_BREAK_DUE_TO_ZPP_OPTIMIZATION__STXH   = 0x000002a1,
    SC_PBB_BATCH_BREAK_DUE_TO_NULLPRIM_LIMIT__STXH     = 0x000002a2,
#endif
} SC_PERFCNT_SEL;

constexpr unsigned int MaxScPerfcntSelGfx101           = SC_PK_PM_MAX_REZ_CNT_FORCE_EOV_WAVE_BRK_1H;
constexpr unsigned int MaxScPerfcntSelGfx10Vrs         = SC_BM_MULTI_ACCUM_4_BE_STALLED__GFX103PLUS;
constexpr unsigned int MaxScPerfcntSelNv32             = SC_SPI_WAVE_STALLED_BY_SPI__GFX11;
constexpr unsigned int MaxScPerfcntSelNv31             = SC_SPI_WAVE_STALLED_BY_SPI__GFX11;
constexpr unsigned int MaxScPerfcntSelStx1             = SC_VRC_REPROBE_FULL__APU11;
constexpr unsigned int MaxScPerfcntSelPhx2             = SC_VRC_REPROBE_FULL__APU11;
constexpr unsigned int MaxScPerfcntSelPhx1             = SC_VRC_REPROBE_FULL__APU11;
constexpr unsigned int MaxScPerfcntSelNv33             = SC_VRC_REPROBE_FULL__NV33;
#if CHIP_HDR_STRIX_HALO
constexpr unsigned int MaxScPerfcntSelStxH             = SC_PBB_BATCH_BREAK_DUE_TO_NULLPRIM_LIMIT__STXH;
#endif

typedef enum SDMA_PERFMON_SEL {
    SDMA_PERFMON_SEL_CYCLE                             = 0x00000000,
    SDMA_PERFMON_SEL_IDLE                              = 0x00000001,
    SDMA_PERFMON_SEL_REG_IDLE                          = 0x00000002,
    SDMA_PERFMON_SEL_RB_EMPTY                          = 0x00000003,
    SDMA_PERFMON_SEL_RB_FULL                           = 0x00000004,
    SDMA_PERFMON_SEL_RB_WPTR_WRAP                      = 0x00000005,
    SDMA_PERFMON_SEL_RB_RPTR_WRAP                      = 0x00000006,
    SDMA_PERFMON_SEL_RB_WPTR_POLL_READ                 = 0x00000007,
    SDMA_PERFMON_SEL_RB_RPTR_WB                        = 0x00000008,
    SDMA_PERFMON_SEL_RB_CMD_IDLE                       = 0x00000009,
    SDMA_PERFMON_SEL_RB_CMD_FULL                       = 0x0000000a,
    SDMA_PERFMON_SEL_IB_CMD_IDLE                       = 0x0000000b,
    SDMA_PERFMON_SEL_IB_CMD_FULL                       = 0x0000000c,
    SDMA_PERFMON_SEL_EX_IDLE                           = 0x0000000d,
    SDMA_PERFMON_SEL_SRBM_REG_SEND                     = 0x0000000e,
    SDMA_PERFMON_SEL_EX_IDLE_POLL_TIMER_EXPIRE         = 0x0000000f,
    SDMA_PERFMON_SEL_WR_BA_RTR                         = 0x00000010,
    SDMA_PERFMON_SEL_MC_WR_IDLE                        = 0x00000011,
    SDMA_PERFMON_SEL_MC_WR_COUNT                       = 0x00000012,
    SDMA_PERFMON_SEL_RD_BA_RTR                         = 0x00000013,
    SDMA_PERFMON_SEL_MC_RD_IDLE                        = 0x00000014,
    SDMA_PERFMON_SEL_MC_RD_COUNT                       = 0x00000015,
    SDMA_PERFMON_SEL_MC_RD_RET_STALL                   = 0x00000016,
    SDMA_PERFMON_SEL_MC_RD_NO_POLL_IDLE                = 0x00000017,
    SDMA_PERFMON_SEL_DRM_IDLE                          = 0x00000018,
    SDMA_PERFMON_SEL_DRM_REQ_STALL                     = 0x00000019,
    SDMA_PERFMON_SEL_SEM_IDLE                          = 0x0000001a,
    SDMA_PERFMON_SEL_SEM_REQ_STALL                     = 0x0000001b,
    SDMA_PERFMON_SEL_SEM_REQ_COUNT                     = 0x0000001c,
    SDMA_PERFMON_SEL_SEM_RESP_INCOMPLETE               = 0x0000001d,
    SDMA_PERFMON_SEL_SEM_RESP_FAIL                     = 0x0000001e,
    SDMA_PERFMON_SEL_SEM_RESP_PASS                     = 0x0000001f,
    SDMA_PERFMON_SEL_INT_IDLE                          = 0x00000020,
    SDMA_PERFMON_SEL_INT_REQ_STALL                     = 0x00000021,
    SDMA_PERFMON_SEL_INT_REQ_COUNT                     = 0x00000022,
    SDMA_PERFMON_SEL_INT_RESP_ACCEPTED                 = 0x00000023,
    SDMA_PERFMON_SEL_INT_RESP_RETRY                    = 0x00000024,
    SDMA_PERFMON_SEL_NUM_PACKET                        = 0x00000025,
    SDMA_PERFMON_SEL_DRM1_REQ_STALL                    = 0x00000026,
    SDMA_PERFMON_SEL_CE_WREQ_IDLE                      = 0x00000027,
    SDMA_PERFMON_SEL_CE_WR_IDLE                        = 0x00000028,
    SDMA_PERFMON_SEL_CE_SPLIT_IDLE                     = 0x00000029,
    SDMA_PERFMON_SEL_CE_RREQ_IDLE                      = 0x0000002a,
    SDMA_PERFMON_SEL_CE_OUT_IDLE                       = 0x0000002b,
    SDMA_PERFMON_SEL_CE_IN_IDLE                        = 0x0000002c,
    SDMA_PERFMON_SEL_CE_DST_IDLE                       = 0x0000002d,
    SDMA_PERFMON_SEL_CE_DRM_IDLE                       = 0x0000002e,
    SDMA_PERFMON_SEL_CE_DRM1_IDLE                      = 0x0000002f,
    SDMA_PERFMON_SEL_CE_AFIFO_FULL                     = 0x00000030,
    SDMA_PERFMON_SEL_DUMMY_0                           = 0x00000031,
    SDMA_PERFMON_SEL_DUMMY_1                           = 0x00000032,
    SDMA_PERFMON_SEL_CE_INFO_FULL                      = 0x00000033,
    SDMA_PERFMON_SEL_CE_INFO1_FULL                     = 0x00000034,
    SDMA_PERFMON_SEL_CE_RD_STALL                       = 0x00000035,
    SDMA_PERFMON_SEL_CE_WR_STALL                       = 0x00000036,
    SDMA_PERFMON_SEL_QUEUE0_SELECT                     = 0x00000037,
    SDMA_PERFMON_SEL_QUEUE1_SELECT                     = 0x00000038,
    SDMA_PERFMON_SEL_QUEUE2_SELECT                     = 0x00000039,
    SDMA_PERFMON_SEL_QUEUE3_SELECT                     = 0x0000003a,
    SDMA_PERFMON_SEL_CTX_CHANGE                        = 0x0000003b,
    SDMA_PERFMON_SEL_CTX_CHANGE_EXPIRED                = 0x0000003c,
    SDMA_PERFMON_SEL_CTX_CHANGE_EXCEPTION              = 0x0000003d,
    SDMA_PERFMON_SEL_DOORBELL                          = 0x0000003e,
    SDMA_PERFMON_SEL_F32_L1_WR_VLD                     = 0x0000003f,
    SDMA_PERFMON_SEL_CE_L1_WR_VLD                      = 0x00000040,
    SDMA_PERFMON_SEL_CPF_SDMA_INVREQ                   = 0x00000041,
    SDMA_PERFMON_SEL_SDMA_CPF_INVACK                   = 0x00000042,
    SDMA_PERFMON_SEL_UTCL2_SDMA_INVREQ                 = 0x00000043,
    SDMA_PERFMON_SEL_SDMA_UTCL2_INVACK                 = 0x00000044,
    SDMA_PERFMON_SEL_UTCL2_SDMA_INVREQ_ALL             = 0x00000045,
    SDMA_PERFMON_SEL_SDMA_UTCL2_INVACK_ALL             = 0x00000046,
    SDMA_PERFMON_SEL_UTCL2_RET_XNACK                   = 0x00000047,
    SDMA_PERFMON_SEL_UTCL2_RET_ACK                     = 0x00000048,
    SDMA_PERFMON_SEL_UTCL2_FREE                        = 0x00000049,
    SDMA_PERFMON_SEL_SDMA_UTCL2_SEND                   = 0x0000004a,
    SDMA_PERFMON_SEL_DMA_L1_WR_SEND                    = 0x0000004b,
    SDMA_PERFMON_SEL_DMA_L1_RD_SEND                    = 0x0000004c,
    SDMA_PERFMON_SEL_DMA_MC_WR_SEND                    = 0x0000004d,
    SDMA_PERFMON_SEL_DMA_MC_RD_SEND                    = 0x0000004e,
    SDMA_PERFMON_SEL_GPUVM_INV_HIGH                    = 0x0000004f,
    SDMA_PERFMON_SEL_GPUVM_INV_LOW                     = 0x00000050,
    SDMA_PERFMON_SEL_L1_WRL2_IDLE                      = 0x00000051,
    SDMA_PERFMON_SEL_L1_RDL2_IDLE                      = 0x00000052,
    SDMA_PERFMON_SEL_L1_WRMC_IDLE                      = 0x00000053,
    SDMA_PERFMON_SEL_L1_RDMC_IDLE                      = 0x00000054,
    SDMA_PERFMON_SEL_L1_WR_INV_IDLE                    = 0x00000055,
    SDMA_PERFMON_SEL_L1_RD_INV_IDLE                    = 0x00000056,
    SDMA_PERFMON_SEL_META_L2_REQ_SEND                  = 0x00000057,
    SDMA_PERFMON_SEL_L2_META_RET_VLD                   = 0x00000058,
    SDMA_PERFMON_SEL_SDMA_UTCL2_RD_SEND                = 0x00000059,
    SDMA_PERFMON_SEL_UTCL2_SDMA_RD_RTN                 = 0x0000005a,
    SDMA_PERFMON_SEL_SDMA_UTCL2_WR_SEND                = 0x0000005b,
    SDMA_PERFMON_SEL_UTCL2_SDMA_WR_RTN                 = 0x0000005c,
    SDMA_PERFMON_SEL_META_REQ_SEND                     = 0x0000005d,
    SDMA_PERFMON_SEL_META_RTN_VLD                      = 0x0000005e,
    SDMA_PERFMON_SEL_TLBI_SEND                         = 0x0000005f,
    SDMA_PERFMON_SEL_TLBI_RTN                          = 0x00000060,
    SDMA_PERFMON_SEL_GCR_SEND                          = 0x00000061,
    SDMA_PERFMON_SEL_GCR_RTN                           = 0x00000062,
    SDMA_PERFMON_SEL_UTCL1_TAG_DELAY_COUNTER           = 0x00000063,
    SDMA_PERFMON_SEL_MMHUB_TAG_DELAY_COUNTER           = 0x00000064,
} SDMA_PERFMON_SEL;

constexpr unsigned int MaxSdmaPerfmonSel               = SDMA_PERFMON_SEL_MMHUB_TAG_DELAY_COUNTER;

typedef enum SDMA_PERF_SEL {
    SDMA_PERF_SEL_CYCLE                                = 0x00000000,
    SDMA_PERF_SEL_IDLE                                 = 0x00000001,
    SDMA_PERF_SEL_REG_IDLE                             = 0x00000002,
    SDMA_PERF_SEL_RB_EMPTY                             = 0x00000003,
    SDMA_PERF_SEL_RB_FULL                              = 0x00000004,
    SDMA_PERF_SEL_RB_WPTR_WRAP                         = 0x00000005,
    SDMA_PERF_SEL_RB_RPTR_WRAP                         = 0x00000006,
    SDMA_PERF_SEL_RB_WPTR_POLL_READ                    = 0x00000007,
    SDMA_PERF_SEL_RB_RPTR_WB                           = 0x00000008,
    SDMA_PERF_SEL_RB_CMD_IDLE                          = 0x00000009,
    SDMA_PERF_SEL_RB_CMD_FULL                          = 0x0000000a,
    SDMA_PERF_SEL_IB_CMD_IDLE                          = 0x0000000b,
    SDMA_PERF_SEL_IB_CMD_FULL                          = 0x0000000c,
    SDMA_PERF_SEL_EX_IDLE                              = 0x0000000d,
    SDMA_PERF_SEL_SRBM_REG_SEND                        = 0x0000000e,
    SDMA_PERF_SEL_EX_IDLE_POLL_TIMER_EXPIRE            = 0x0000000f,
    SDMA_PERF_SEL_MC_WR_IDLE                           = 0x00000010,
    SDMA_PERF_SEL_MC_WR_COUNT                          = 0x00000011,
    SDMA_PERF_SEL_MC_RD_IDLE                           = 0x00000012,
    SDMA_PERF_SEL_MC_RD_COUNT                          = 0x00000013,
    SDMA_PERF_SEL_MC_RD_RET_STALL                      = 0x00000014,
    SDMA_PERF_SEL_MC_RD_NO_POLL_IDLE                   = 0x00000015,
    SDMA_PERF_SEL_DRM_IDLE                             = 0x00000016,
    SDMA_PERF_SEL_DRM_REQ_STALL                        = 0x00000017,
    SDMA_PERF_SEL_SEM_IDLE                             = 0x00000018,
    SDMA_PERF_SEL_SEM_REQ_STALL                        = 0x00000019,
    SDMA_PERF_SEL_SEM_REQ_COUNT                        = 0x0000001a,
    SDMA_PERF_SEL_SEM_RESP_INCOMPLETE                  = 0x0000001b,
    SDMA_PERF_SEL_SEM_RESP_FAIL                        = 0x0000001c,
    SDMA_PERF_SEL_SEM_RESP_PASS                        = 0x0000001d,
    SDMA_PERF_SEL_INT_IDLE                             = 0x0000001e,
    SDMA_PERF_SEL_INT_REQ_STALL                        = 0x0000001f,
    SDMA_PERF_SEL_INT_REQ_COUNT                        = 0x00000020,
    SDMA_PERF_SEL_INT_RESP_ACCEPTED                    = 0x00000021,
    SDMA_PERF_SEL_INT_RESP_RETRY                       = 0x00000022,
    SDMA_PERF_SEL_NUM_PACKET                           = 0x00000023,
    SDMA_PERF_SEL_DRM1_REQ_STALL                       = 0x00000024,
    SDMA_PERF_SEL_CE_WREQ_IDLE                         = 0x00000025,
    SDMA_PERF_SEL_CE_WR_IDLE                           = 0x00000026,
    SDMA_PERF_SEL_CE_SPLIT_IDLE                        = 0x00000027,
    SDMA_PERF_SEL_CE_RREQ_IDLE                         = 0x00000028,
    SDMA_PERF_SEL_CE_OUT_IDLE                          = 0x00000029,
    SDMA_PERF_SEL_CE_IN_IDLE                           = 0x0000002a,
    SDMA_PERF_SEL_CE_DST_IDLE                          = 0x0000002b,
    SDMA_PERF_SEL_CE_DRM_IDLE                          = 0x0000002c,
    SDMA_PERF_SEL_CE_DRM1_IDLE                         = 0x0000002d,
    SDMA_PERF_SEL_CE_AFIFO_FULL                        = 0x0000002e,
    SDMA_PERF_SEL_CE_INFO_FULL                         = 0x00000031,
    SDMA_PERF_SEL_CE_INFO1_FULL                        = 0x00000032,
    SDMA_PERF_SEL_CE_RD_STALL                          = 0x00000033,
    SDMA_PERF_SEL_CE_WR_STALL                          = 0x00000034,
    SDMA_PERF_SEL_CTX_CHANGE                           = 0x00000039,
    SDMA_PERF_SEL_CTX_CHANGE_EXPIRED                   = 0x0000003a,
    SDMA_PERF_SEL_CTX_CHANGE_EXCEPTION                 = 0x0000003b,
    SDMA_PERF_SEL_DOORBELL                             = 0x0000003c,
    SDMA_PERF_SEL_RD_BA_RTR                            = 0x0000003d,
    SDMA_PERF_SEL_WR_BA_RTR                            = 0x0000003e,
    SDMA_PERF_SEL_F32_L1_WR_VLD                        = 0x0000003f,
    SDMA_PERF_SEL_CE_L1_WR_VLD                         = 0x00000040,
    SDMA_PERF_SEL_CPF_SDMA_INVREQ                      = 0x00000041,
    SDMA_PERF_SEL_SDMA_CPF_INVACK                      = 0x00000042,
    SDMA_PERF_SEL_UTCL2_SDMA_INVREQ                    = 0x00000043,
    SDMA_PERF_SEL_SDMA_UTCL2_INVACK                    = 0x00000044,
    SDMA_PERF_SEL_UTCL2_SDMA_INVREQ_ALL                = 0x00000045,
    SDMA_PERF_SEL_SDMA_UTCL2_INVACK_ALL                = 0x00000046,
    SDMA_PERF_SEL_UTCL2_RET_XNACK                      = 0x00000047,
    SDMA_PERF_SEL_UTCL2_RET_ACK                        = 0x00000048,
    SDMA_PERF_SEL_UTCL2_FREE                           = 0x00000049,
    SDMA_PERF_SEL_SDMA_UTCL2_SEND                      = 0x0000004a,
    SDMA_PERF_SEL_DMA_L1_WR_SEND                       = 0x0000004b,
    SDMA_PERF_SEL_DMA_L1_RD_SEND                       = 0x0000004c,
    SDMA_PERF_SEL_DMA_MC_WR_SEND                       = 0x0000004d,
    SDMA_PERF_SEL_DMA_MC_RD_SEND                       = 0x0000004e,
    SDMA_PERF_SEL_L1_WRL2_IDLE                         = 0x00000051,
    SDMA_PERF_SEL_L1_RDL2_IDLE                         = 0x00000052,
    SDMA_PERF_SEL_L1_WRMC_IDLE                         = 0x00000053,
    SDMA_PERF_SEL_L1_RDMC_IDLE                         = 0x00000054,
    SDMA_PERF_SEL_L1_WR_INV_IDLE                       = 0x00000055,
    SDMA_PERF_SEL_L1_RD_INV_IDLE                       = 0x00000056,
    SDMA_PERF_SEL_META_L2_REQ_SEND                     = 0x00000057,
    SDMA_PERF_SEL_L2_META_RET_VLD                      = 0x00000058,
    SDMA_PERF_SEL_SDMA_UTCL2_RD_SEND                   = 0x00000059,
    SDMA_PERF_SEL_UTCL2_SDMA_RD_RTN                    = 0x0000005a,
    SDMA_PERF_SEL_SDMA_UTCL2_WR_SEND                   = 0x0000005b,
    SDMA_PERF_SEL_UTCL2_SDMA_WR_RTN                    = 0x0000005c,
    SDMA_PERF_SEL_META_REQ_SEND                        = 0x0000005d,
    SDMA_PERF_SEL_META_RTN_VLD                         = 0x0000005e,
    SDMA_PERF_SEL_TLBI_SEND                            = 0x0000005f,
    SDMA_PERF_SEL_TLBI_RTN                             = 0x00000060,
    SDMA_PERF_SEL_GCR_SEND                             = 0x00000061,
    SDMA_PERF_SEL_GCR_RTN                              = 0x00000062,
    SDMA_PERF_SEL_CE_DRM_FULL__GFX10                   = 0x0000002f,
    SDMA_PERF_SEL_CE_DRM1_FULL__GFX10                  = 0x00000030,
    SDMA_PERF_SEL_GFX_SELECT__GFX10                    = 0x00000035,
    SDMA_PERF_SEL_RLC0_SELECT__GFX10                   = 0x00000036,
    SDMA_PERF_SEL_RLC1_SELECT__GFX10                   = 0x00000037,
    SDMA_PERF_SEL_PAGE_SELECT__GFX10                   = 0x00000038,
    SDMA_PERF_SEL_UTCL1_TAG_DELAY_COUNTER__GFX10       = 0x00000063,
    SDMA_PERF_SEL_MMHUB_TAG_DELAY_COUNTER__GFX10       = 0x00000064,
    SDMA_PERF_SEL_GPUVM_INVREQ_HIGH__GFX101            = 0x0000004f,
    SDMA_PERF_SEL_GPUVM_INVREQ_LOW__GFX101             = 0x00000050,
    SDMA_PERF_SEL_GPUVM_INV_HIGH__GFX103PLUS           = 0x0000004f,
    SDMA_PERF_SEL_GPUVM_INV_LOW__GFX103PLUS            = 0x00000050,
    SDMA_PERF_SEL_CE_CH_WR_REQ__GFX10VRS               = 0x00000065,
    SDMA_PERF_SEL_CE_CH_WR_RET__GFX10VRS               = 0x00000066,
    SDMA_PERF_SEL_F32_CH_WR_REQ__GFX10VRS              = 0x00000067,
    SDMA_PERF_SEL_F32_CH_WR_RET__GFX10VRS              = 0x00000068,
    SDMA_PERF_SEL_CE_OR_F32_CH_RD_REQ__GFX10VRS        = 0x00000069,
    SDMA_PERF_SEL_CE_OR_F32_CH_RD_RET__GFX10VRS        = 0x0000006a,
    SDMA_PERF_SEL_RB_CH_RD_REQ__GFX10VRS               = 0x0000006b,
    SDMA_PERF_SEL_RB_CH_RD_RET__GFX10VRS               = 0x0000006c,
    SDMA_PERF_SEL_IB_CH_RD_REQ__GFX10VRS               = 0x0000006d,
    SDMA_PERF_SEL_IB_CH_RD_RET__GFX10VRS               = 0x0000006e,
    SDMA_PERF_SEL_WPTR_CH_RD_REQ__GFX10VRS             = 0x0000006f,
    SDMA_PERF_SEL_WPTR_CH_RD_RET__GFX10VRS             = 0x00000070,
    SDMA_PERF_SEL_UTCL1_UTCL2_REQ__GFX10VRS            = 0x00000071,
    SDMA_PERF_SEL_UTCL1_UTCL2_RET__GFX10VRS            = 0x00000072,
    SDMA_PERF_SEL_CMD_OP_MATCH__GFX10VRS               = 0x00000073,
    SDMA_PERF_SEL_CMD_OP_START__GFX10VRS               = 0x00000074,
    SDMA_PERF_SEL_CMD_OP_END__GFX10VRS                 = 0x00000075,
    SDMA_PERF_SEL_CE_BUSY__GFX10VRS                    = 0x00000076,
    SDMA_PERF_SEL_CE_BUSY_START__GFX10VRS              = 0x00000077,
    SDMA_PERF_SEL_CE_BUSY_END__GFX10VRS                = 0x00000078,
    SDMA_PERF_SEL_F32_PERFCNT_TRIGGER__GFX10VRS        = 0x00000079,
    SDMA_PERF_SEL_F32_PERFCNT_TRIGGER_START__GFX10VRS  = 0x0000007a,
    SDMA_PERF_SEL_F32_PERFCNT_TRIGGER_END__GFX10VRS    = 0x0000007b,
    SDMA_PERF_SEL_CE_CH_WRREQ_SEND__GFX10VRS           = 0x0000007c,
    SDMA_PERF_SEL_CH_CE_WRRET_VALID__GFX10VRS          = 0x0000007d,
    SDMA_PERF_SEL_CE_CH_RDREQ_SEND__GFX10VRS           = 0x0000007e,
    SDMA_PERF_SEL_CH_CE_RDRET_VALID__GFX10VRS          = 0x0000007f,
    SDMA_PERF_SEL_DUMMY_0__GFX11                       = 0x0000002f,
    SDMA_PERF_SEL_DUMMY_1__GFX11                       = 0x00000030,
    SDMA_PERF_SEL_QUEUE0_SELECT__GFX11                 = 0x00000035,
    SDMA_PERF_SEL_QUEUE1_SELECT__GFX11                 = 0x00000036,
    SDMA_PERF_SEL_QUEUE2_SELECT__GFX11                 = 0x00000037,
    SDMA_PERF_SEL_QUEUE3_SELECT__GFX11                 = 0x00000038,
    SDMA_PERF_SEL_CGCG_FENCE__GFX11                    = 0x00000063,
    SDMA_PERF_SEL_CE_CH_WR_REQ__GFX11                  = 0x00000064,
    SDMA_PERF_SEL_CE_CH_WR_RET__GFX11                  = 0x00000065,
    SDMA_PERF_SEL_F32_CH_WR_REQ__GFX11                 = 0x00000066,
    SDMA_PERF_SEL_F32_CH_WR_RET__GFX11                 = 0x00000067,
    SDMA_PERF_SEL_CE_OR_F32_CH_RD_REQ__GFX11           = 0x00000068,
    SDMA_PERF_SEL_CE_OR_F32_CH_RD_RET__GFX11           = 0x00000069,
    SDMA_PERF_SEL_RB_CH_RD_REQ__GFX11                  = 0x0000006a,
    SDMA_PERF_SEL_RB_CH_RD_RET__GFX11                  = 0x0000006b,
    SDMA_PERF_SEL_IB_CH_RD_REQ__GFX11                  = 0x0000006c,
    SDMA_PERF_SEL_IB_CH_RD_RET__GFX11                  = 0x0000006d,
    SDMA_PERF_SEL_WPTR_CH_RD_REQ__GFX11                = 0x0000006e,
    SDMA_PERF_SEL_WPTR_CH_RD_RET__GFX11                = 0x0000006f,
    SDMA_PERF_SEL_UTCL1_UTCL2_REQ__GFX11               = 0x00000070,
    SDMA_PERF_SEL_UTCL1_UTCL2_RET__GFX11               = 0x00000071,
    SDMA_PERF_SEL_CMD_OP_MATCH__GFX11                  = 0x00000072,
    SDMA_PERF_SEL_CMD_OP_START__GFX11                  = 0x00000073,
    SDMA_PERF_SEL_CMD_OP_END__GFX11                    = 0x00000074,
    SDMA_PERF_SEL_CE_BUSY__GFX11                       = 0x00000075,
    SDMA_PERF_SEL_CE_BUSY_START__GFX11                 = 0x00000076,
    SDMA_PERF_SEL_CE_BUSY_END__GFX11                   = 0x00000077,
    SDMA_PERF_SEL_F32_PERFCNT_TRIGGER__GFX11           = 0x00000078,
    SDMA_PERF_SEL_F32_PERFCNT_TRIGGER_START__GFX11     = 0x00000079,
    SDMA_PERF_SEL_F32_PERFCNT_TRIGGER_END__GFX11       = 0x0000007a,
    SDMA_PERF_SEL_CE_CH_WRREQ_SEND__GFX11              = 0x0000007b,
    SDMA_PERF_SEL_CH_CE_WRRET_VALID__GFX11             = 0x0000007c,
    SDMA_PERF_SEL_CE_CH_RDREQ_SEND__GFX11              = 0x0000007d,
    SDMA_PERF_SEL_CH_CE_RDRET_VALID__GFX11             = 0x0000007e,
    SDMA_PERF_SEL_QUEUE4_SELECT__GFX11                 = 0x0000007f,
    SDMA_PERF_SEL_QUEUE5_SELECT__GFX11                 = 0x00000080,
    SDMA_PERF_SEL_QUEUE6_SELECT__GFX11                 = 0x00000081,
    SDMA_PERF_SEL_QUEUE7_SELECT__GFX11                 = 0x00000082,
} SDMA_PERF_SEL;

constexpr unsigned int MaxSdmaPerfSelGfx101            = SDMA_PERF_SEL_MMHUB_TAG_DELAY_COUNTER__GFX10;
constexpr unsigned int MaxSdmaPerfSelGfx10Vrs          = SDMA_PERF_SEL_CH_CE_RDRET_VALID__GFX10VRS;
constexpr unsigned int MaxSdmaPerfSelGfx11             = SDMA_PERF_SEL_QUEUE7_SELECT__GFX11;

typedef enum SH_MEM_ADDRESS_MODE {
    SH_MEM_ADDRESS_MODE_64                             = 0x00000000,
    SH_MEM_ADDRESS_MODE_32                             = 0x00000001,
} SH_MEM_ADDRESS_MODE;

typedef enum SH_MEM_ALIGNMENT_MODE {
    SH_MEM_ALIGNMENT_MODE_DWORD                        = 0x00000000,
    SH_MEM_ALIGNMENT_MODE_DWORD_STRICT                 = 0x00000001,
    SH_MEM_ALIGNMENT_MODE_STRICT                       = 0x00000002,
    SH_MEM_ALIGNMENT_MODE_UNALIGNED                    = 0x00000003,
} SH_MEM_ALIGNMENT_MODE;

typedef enum SH_MEM_RETRY_MODE {
    SH_MEM_RETRY_MODE_ALL                              = 0x00000000,
    SH_MEM_RETRY_MODE_WRITEATOMIC                      = 0x00000001,
    SH_MEM_RETRY_MODE_NONE                             = 0x00000002,
} SH_MEM_RETRY_MODE;

typedef enum SPI_FOG_MODE {
    SPI_FOG_NONE                                       = 0x00000000,
    SPI_FOG_EXP                                        = 0x00000001,
    SPI_FOG_EXP2                                       = 0x00000002,
    SPI_FOG_LINEAR                                     = 0x00000003,
} SPI_FOG_MODE;

typedef enum SPI_LB_WAVES_SELECT {
    HS_GS                                              = 0x00000000,
    CS_NA                                              = 0x00000002,
    SPI_LB_WAVES_RSVD                                  = 0x00000003,
    VS_PS__GFX10                                       = 0x00000001,
    PS__GFX11                                          = 0x00000001,
} SPI_LB_WAVES_SELECT;

typedef enum SPI_PERFCNT_SEL {
    SPI_PERF_GS_GRP_LIFETIME_SAMPLE__APU11             = 0x00000012,
    SPI_PERF_VS_WINDOW_VALID__GFX10                    = 0x00000000,
    SPI_PERF_VS_BUSY__GFX10                            = 0x00000001,
    SPI_PERF_VS_FIRST_WAVE__GFX10                      = 0x00000002,
    SPI_PERF_VS_LAST_WAVE__GFX10                       = 0x00000003,
    SPI_PERF_VS_LSHS_DEALLOC__GFX10                    = 0x00000004,
    SPI_PERF_VS_POS0_STALL__GFX10                      = 0x00000005,
    SPI_PERF_VS_POS1_STALL__GFX10                      = 0x00000006,
    SPI_PERF_VS_CRAWLER_STALL__GFX10                   = 0x00000007,
    SPI_PERF_VS_EVENT_WAVE__GFX10                      = 0x00000008,
    SPI_PERF_VS_WAVE__GFX10                            = 0x00000009,
    SPI_PERF_VS_PERS_UPD_FULL0__GFX10                  = 0x0000000a,
    SPI_PERF_VS_PERS_UPD_FULL1__GFX10                  = 0x0000000b,
    SPI_PERF_VS_LATE_ALLOC_FULL__GFX10                 = 0x0000000c,
    SPI_PERF_VS_FIRST_SUBGRP__GFX10                    = 0x0000000d,
    SPI_PERF_VS_LAST_SUBGRP__GFX10                     = 0x0000000e,
    SPI_PERF_VS_ALLOC_CNT__GFX10                       = 0x0000000f,
    SPI_PERF_VS_LATE_ALLOC_ACCUM__GFX10                = 0x00000010,
    SPI_PERF_GS_WINDOW_VALID__GFX10                    = 0x00000011,
    SPI_PERF_GS_BUSY__GFX10                            = 0x00000012,
    SPI_PERF_GS_CRAWLER_STALL__GFX10                   = 0x00000013,
    SPI_PERF_GS_EVENT_WAVE__GFX10                      = 0x00000014,
    SPI_PERF_GS_WAVE__GFX10                            = 0x00000015,
    SPI_PERF_GS_PERS_UPD_FULL0__GFX10                  = 0x00000016,
    SPI_PERF_GS_PERS_UPD_FULL1__GFX10                  = 0x00000017,
    SPI_PERF_GS_FIRST_SUBGRP__GFX10                    = 0x00000018,
    SPI_PERF_GS_LAST_SUBGRP__GFX10                     = 0x00000019,
    SPI_PERF_GS_HS_DEALLOC__GFX10                      = 0x0000001a,
    SPI_PERF_GS_NGG_SE_LATE_ALLOC_LIMIT__GFX10         = 0x0000001b,
    SPI_PERF_GS_GRP_FIFO_FULL__GFX10                   = 0x0000001c,
    SPI_PERF_GS_POS0_STALL__GFX10                      = 0x0000001d,
    SPI_PERF_GS_POS1_STALL__GFX10                      = 0x0000001e,
    SPI_PERF_GS_INDX0_STALL__GFX10                     = 0x0000001f,
    SPI_PERF_GS_INDX1_STALL__GFX10                     = 0x00000020,
    SPI_PERF_HS_WINDOW_VALID__GFX10                    = 0x00000021,
    SPI_PERF_HS_BUSY__GFX10                            = 0x00000022,
    SPI_PERF_HS_CRAWLER_STALL__GFX10                   = 0x00000023,
    SPI_PERF_HS_FIRST_WAVE__GFX10                      = 0x00000024,
    SPI_PERF_HS_LAST_WAVE__GFX10                       = 0x00000025,
    SPI_PERF_HS_LSHS_DEALLOC__GFX10                    = 0x00000026,
    SPI_PERF_HS_OFFCHIP_LDS_STALL__GFX10               = 0x00000027,
    SPI_PERF_HS_EVENT_WAVE__GFX10                      = 0x00000028,
    SPI_PERF_HS_WAVE__GFX10                            = 0x00000029,
    SPI_PERF_HS_PERS_UPD_FULL0__GFX10                  = 0x0000002a,
    SPI_PERF_HS_PERS_UPD_FULL1__GFX10                  = 0x0000002b,
    SPI_PERF_CSGN_WINDOW_VALID__GFX10                  = 0x0000002c,
    SPI_PERF_CSGN_BUSY__GFX10                          = 0x0000002d,
    SPI_PERF_CSGN_NUM_THREADGROUPS__GFX10              = 0x0000002e,
    SPI_PERF_CSGN_CRAWLER_STALL__GFX10                 = 0x0000002f,
    SPI_PERF_CSGN_EVENT_WAVE__GFX10                    = 0x00000030,
    SPI_PERF_CSGN_WAVE__GFX10                          = 0x00000031,
    SPI_PERF_CSN_WINDOW_VALID__GFX10                   = 0x00000032,
    SPI_PERF_CSN_BUSY__GFX10                           = 0x00000033,
    SPI_PERF_CSN_NUM_THREADGROUPS__GFX10               = 0x00000034,
    SPI_PERF_CSN_CRAWLER_STALL__GFX10                  = 0x00000035,
    SPI_PERF_CSN_EVENT_WAVE__GFX10                     = 0x00000036,
    SPI_PERF_CSN_WAVE__GFX10                           = 0x00000037,
    SPI_PERF_PS0_WINDOW_VALID__GFX10                   = 0x00000038,
    SPI_PERF_PS1_WINDOW_VALID__GFX10                   = 0x00000039,
    SPI_PERF_PS2_WINDOW_VALID__GFX10                   = 0x0000003a,
    SPI_PERF_PS3_WINDOW_VALID__GFX10                   = 0x0000003b,
    SPI_PERF_PS0_BUSY__GFX10                           = 0x0000003c,
    SPI_PERF_PS1_BUSY__GFX10                           = 0x0000003d,
    SPI_PERF_PS2_BUSY__GFX10                           = 0x0000003e,
    SPI_PERF_PS3_BUSY__GFX10                           = 0x0000003f,
    SPI_PERF_PS0_ACTIVE__GFX10                         = 0x00000040,
    SPI_PERF_PS1_ACTIVE__GFX10                         = 0x00000041,
    SPI_PERF_PS2_ACTIVE__GFX10                         = 0x00000042,
    SPI_PERF_PS3_ACTIVE__GFX10                         = 0x00000043,
    SPI_PERF_PS0_DEALLOC__GFX10                        = 0x00000044,
    SPI_PERF_PS1_DEALLOC__GFX10                        = 0x00000045,
    SPI_PERF_PS2_DEALLOC__GFX10                        = 0x00000046,
    SPI_PERF_PS3_DEALLOC__GFX10                        = 0x00000047,
    SPI_PERF_PS0_FPOS_STALL__GFX10                     = 0x00000048,
    SPI_PERF_PS1_FPOS_STALL__GFX10                     = 0x00000049,
    SPI_PERF_PS2_FPOS_STALL__GFX10                     = 0x0000004a,
    SPI_PERF_PS3_FPOS_STALL__GFX10                     = 0x0000004b,
    SPI_PERF_PS0_EVENT_WAVE__GFX10                     = 0x0000004c,
    SPI_PERF_PS1_EVENT_WAVE__GFX10                     = 0x0000004d,
    SPI_PERF_PS2_EVENT_WAVE__GFX10                     = 0x0000004e,
    SPI_PERF_PS3_EVENT_WAVE__GFX10                     = 0x0000004f,
    SPI_PERF_PS0_WAVE__GFX10                           = 0x00000050,
    SPI_PERF_PS1_WAVE__GFX10                           = 0x00000051,
    SPI_PERF_PS2_WAVE__GFX10                           = 0x00000052,
    SPI_PERF_PS3_WAVE__GFX10                           = 0x00000053,
    SPI_PERF_PS0_OPT_WAVE__GFX10                       = 0x00000054,
    SPI_PERF_PS1_OPT_WAVE__GFX10                       = 0x00000055,
    SPI_PERF_PS2_OPT_WAVE__GFX10                       = 0x00000056,
    SPI_PERF_PS3_OPT_WAVE__GFX10                       = 0x00000057,
    SPI_PERF_PS0_PASS_BIN0__GFX10                      = 0x00000058,
    SPI_PERF_PS1_PASS_BIN0__GFX10                      = 0x00000059,
    SPI_PERF_PS2_PASS_BIN0__GFX10                      = 0x0000005a,
    SPI_PERF_PS3_PASS_BIN0__GFX10                      = 0x0000005b,
    SPI_PERF_PS0_PASS_BIN1__GFX10                      = 0x0000005c,
    SPI_PERF_PS1_PASS_BIN1__GFX10                      = 0x0000005d,
    SPI_PERF_PS2_PASS_BIN1__GFX10                      = 0x0000005e,
    SPI_PERF_PS3_PASS_BIN1__GFX10                      = 0x0000005f,
    SPI_PERF_PS0_FPOS__GFX10                           = 0x00000060,
    SPI_PERF_PS1_FPOS__GFX10                           = 0x00000061,
    SPI_PERF_PS2_FPOS__GFX10                           = 0x00000062,
    SPI_PERF_PS3_FPOS__GFX10                           = 0x00000063,
    SPI_PERF_PS0_PRIM_BIN0__GFX10                      = 0x00000064,
    SPI_PERF_PS1_PRIM_BIN0__GFX10                      = 0x00000065,
    SPI_PERF_PS2_PRIM_BIN0__GFX10                      = 0x00000066,
    SPI_PERF_PS3_PRIM_BIN0__GFX10                      = 0x00000067,
    SPI_PERF_PS0_PRIM_BIN1__GFX10                      = 0x00000068,
    SPI_PERF_PS1_PRIM_BIN1__GFX10                      = 0x00000069,
    SPI_PERF_PS2_PRIM_BIN1__GFX10                      = 0x0000006a,
    SPI_PERF_PS3_PRIM_BIN1__GFX10                      = 0x0000006b,
    SPI_PERF_PS0_CNF_BIN2__GFX10                       = 0x0000006c,
    SPI_PERF_PS1_CNF_BIN2__GFX10                       = 0x0000006d,
    SPI_PERF_PS2_CNF_BIN2__GFX10                       = 0x0000006e,
    SPI_PERF_PS3_CNF_BIN2__GFX10                       = 0x0000006f,
    SPI_PERF_PS0_CNF_BIN3__GFX10                       = 0x00000070,
    SPI_PERF_PS1_CNF_BIN3__GFX10                       = 0x00000071,
    SPI_PERF_PS2_CNF_BIN3__GFX10                       = 0x00000072,
    SPI_PERF_PS3_CNF_BIN3__GFX10                       = 0x00000073,
    SPI_PERF_PS0_CRAWLER_STALL__GFX10                  = 0x00000074,
    SPI_PERF_PS1_CRAWLER_STALL__GFX10                  = 0x00000075,
    SPI_PERF_PS2_CRAWLER_STALL__GFX10                  = 0x00000076,
    SPI_PERF_PS3_CRAWLER_STALL__GFX10                  = 0x00000077,
    SPI_PERF_PS0_LDS_RES_FULL__GFX10                   = 0x00000078,
    SPI_PERF_PS1_LDS_RES_FULL__GFX10                   = 0x00000079,
    SPI_PERF_PS2_LDS_RES_FULL__GFX10                   = 0x0000007a,
    SPI_PERF_PS3_LDS_RES_FULL__GFX10                   = 0x0000007b,
    SPI_PERF_PS0_POPS_WAVE_SENT__GFX10                 = 0x0000007c,
    SPI_PERF_PS1_POPS_WAVE_SENT__GFX10                 = 0x0000007d,
    SPI_PERF_PS2_POPS_WAVE_SENT__GFX10                 = 0x0000007e,
    SPI_PERF_PS3_POPS_WAVE_SENT__GFX10                 = 0x0000007f,
    SPI_PERF_PS0_POPS_WAVE_EXIT__GFX10                 = 0x00000080,
    SPI_PERF_PS1_POPS_WAVE_EXIT__GFX10                 = 0x00000081,
    SPI_PERF_PS2_POPS_WAVE_EXIT__GFX10                 = 0x00000082,
    SPI_PERF_PS3_POPS_WAVE_EXIT__GFX10                 = 0x00000083,
    SPI_PERF_PS_PERS_UPD_FULL0__GFX10                  = 0x00000084,
    SPI_PERF_PS_PERS_UPD_FULL1__GFX10                  = 0x00000085,
    SPI_PERF_PS0_INTRA_PRIM_CNFLCT__GFX10              = 0x00000086,
    SPI_PERF_PS1_INTRA_PRIM_CNFLCT__GFX10              = 0x00000087,
    SPI_PERF_PS2_INTRA_PRIM_CNFLCT__GFX10              = 0x00000088,
    SPI_PERF_PS3_INTRA_PRIM_CNFLCT__GFX10              = 0x00000089,
    SPI_PERF_LDS0_PC_VALID__GFX10                      = 0x0000008a,
    SPI_PERF_LDS1_PC_VALID__GFX10                      = 0x0000008b,
    SPI_PERF_RA_PIPE_REQ_BIN2__GFX10                   = 0x0000008c,
    SPI_PERF_RA_TASK_REQ_BIN3__GFX10                   = 0x0000008d,
    SPI_PERF_RA_WR_CTL_FULL__GFX10                     = 0x0000008e,
    SPI_PERF_RA_REQ_NO_ALLOC__GFX10                    = 0x0000008f,
    SPI_PERF_RA_REQ_NO_ALLOC_PS__GFX10                 = 0x00000090,
    SPI_PERF_RA_REQ_NO_ALLOC_VS__GFX10                 = 0x00000091,
    SPI_PERF_RA_REQ_NO_ALLOC_GS__GFX10                 = 0x00000093,
    SPI_PERF_RA_REQ_NO_ALLOC_HS__GFX10                 = 0x00000095,
    SPI_PERF_RA_REQ_NO_ALLOC_CSG__GFX10                = 0x00000096,
    SPI_PERF_RA_REQ_NO_ALLOC_CSN__GFX10                = 0x00000097,
    SPI_PERF_RA_RES_STALL_PS__GFX10                    = 0x00000098,
    SPI_PERF_RA_RES_STALL_VS__GFX10                    = 0x00000099,
    SPI_PERF_RA_RES_STALL_GS__GFX10                    = 0x0000009b,
    SPI_PERF_RA_RES_STALL_HS__GFX10                    = 0x0000009d,
    SPI_PERF_RA_RES_STALL_CSG__GFX10                   = 0x0000009e,
    SPI_PERF_RA_RES_STALL_CSN__GFX10                   = 0x0000009f,
    SPI_PERF_RA_TMP_STALL_PS__GFX10                    = 0x000000a0,
    SPI_PERF_RA_TMP_STALL_VS__GFX10                    = 0x000000a1,
    SPI_PERF_RA_TMP_STALL_GS__GFX10                    = 0x000000a3,
    SPI_PERF_RA_TMP_STALL_HS__GFX10                    = 0x000000a5,
    SPI_PERF_RA_TMP_STALL_CSG__GFX10                   = 0x000000a6,
    SPI_PERF_RA_TMP_STALL_CSN__GFX10                   = 0x000000a7,
    SPI_PERF_RA_WAVE_SIMD_FULL_PS__GFX10               = 0x000000a8,
    SPI_PERF_RA_WAVE_SIMD_FULL_VS__GFX10               = 0x000000a9,
    SPI_PERF_RA_WAVE_SIMD_FULL_GS__GFX10               = 0x000000ab,
    SPI_PERF_RA_WAVE_SIMD_FULL_HS__GFX10               = 0x000000ad,
    SPI_PERF_RA_WAVE_SIMD_FULL_CSG__GFX10              = 0x000000ae,
    SPI_PERF_RA_WAVE_SIMD_FULL_CSN__GFX10              = 0x000000af,
    SPI_PERF_RA_VGPR_SIMD_FULL_PS__GFX10               = 0x000000b0,
    SPI_PERF_RA_VGPR_SIMD_FULL_VS__GFX10               = 0x000000b1,
    SPI_PERF_RA_VGPR_SIMD_FULL_GS__GFX10               = 0x000000b3,
    SPI_PERF_RA_VGPR_SIMD_FULL_HS__GFX10               = 0x000000b4,
    SPI_PERF_RA_VGPR_SIMD_FULL_CSG__GFX10              = 0x000000b6,
    SPI_PERF_RA_VGPR_SIMD_FULL_CSN__GFX10              = 0x000000b7,
    SPI_PERF_RA_LDS_CU_FULL_PS__GFX10                  = 0x000000c0,
    SPI_PERF_RA_LDS_CU_FULL_HS__GFX10                  = 0x000000c2,
    SPI_PERF_RA_LDS_CU_FULL_GS__GFX10                  = 0x000000c4,
    SPI_PERF_RA_LDS_CU_FULL_CSG__GFX10                 = 0x000000c5,
    SPI_PERF_RA_LDS_CU_FULL_CSN__GFX10                 = 0x000000c6,
    SPI_PERF_RA_BAR_CU_FULL_HS__GFX10                  = 0x000000c7,
    SPI_PERF_RA_BAR_CU_FULL_CSG__GFX10                 = 0x000000c8,
    SPI_PERF_RA_BAR_CU_FULL_CSN__GFX10                 = 0x000000c9,
    SPI_PERF_RA_BULKY_CU_FULL_CSG__GFX10               = 0x000000ca,
    SPI_PERF_RA_BULKY_CU_FULL_CSN__GFX10               = 0x000000cb,
    SPI_PERF_RA_TGLIM_CU_FULL_CSG__GFX10               = 0x000000cc,
    SPI_PERF_RA_TGLIM_CU_FULL_CSN__GFX10               = 0x000000cd,
    SPI_PERF_RA_WVLIM_STALL_PS__GFX10                  = 0x000000ce,
    SPI_PERF_RA_WVLIM_STALL_VS__GFX10                  = 0x000000cf,
    SPI_PERF_RA_WVLIM_STALL_GS__GFX10                  = 0x000000d1,
    SPI_PERF_RA_WVLIM_STALL_HS__GFX10                  = 0x000000d3,
    SPI_PERF_RA_WVLIM_STALL_CSG__GFX10                 = 0x000000d4,
    SPI_PERF_RA_WVLIM_STALL_CSN__GFX10                 = 0x000000d5,
    SPI_PERF_RA_VS_LOCK__GFX10                         = 0x000000d6,
    SPI_PERF_RA_PS_LOCK__GFX10                         = 0x000000d7,
    SPI_PERF_RA_GS_LOCK__GFX10                         = 0x000000d9,
    SPI_PERF_RA_HS_LOCK__GFX10                         = 0x000000db,
    SPI_PERF_RA_CSG_LOCK__GFX10                        = 0x000000dc,
    SPI_PERF_RA_CSN_LOCK__GFX10                        = 0x000000dd,
    SPI_PERF_RA_RSV_UPD__GFX10                         = 0x000000de,
    SPI_PERF_RA_PRE_ALLOC_STALL__GFX10                 = 0x000000df,
    SPI_PERF_RA_GFX_UNDER_TUNNEL__GFX10                = 0x000000e0,
    SPI_PERF_RA_CSC_UNDER_TUNNEL__GFX10                = 0x000000e1,
    SPI_PERF_RA_WVALLOC_STALL__GFX10                   = 0x000000e2,
    SPI_PERF_RA_ACCUM0_SIMD_FULL_PS__GFX10             = 0x000000e3,
    SPI_PERF_RA_ACCUM1_SIMD_FULL_PS__GFX10             = 0x000000e4,
    SPI_PERF_RA_ACCUM2_SIMD_FULL_PS__GFX10             = 0x000000e5,
    SPI_PERF_RA_ACCUM3_SIMD_FULL_PS__GFX10             = 0x000000e6,
    SPI_PERF_RA_ACCUM0_SIMD_FULL_VS__GFX10             = 0x000000e7,
    SPI_PERF_RA_ACCUM1_SIMD_FULL_VS__GFX10             = 0x000000e8,
    SPI_PERF_RA_ACCUM2_SIMD_FULL_VS__GFX10             = 0x000000e9,
    SPI_PERF_RA_ACCUM3_SIMD_FULL_VS__GFX10             = 0x000000ea,
    SPI_PERF_RA_ACCUM0_SIMD_FULL_GS__GFX10             = 0x000000eb,
    SPI_PERF_RA_ACCUM1_SIMD_FULL_GS__GFX10             = 0x000000ec,
    SPI_PERF_RA_ACCUM2_SIMD_FULL_GS__GFX10             = 0x000000ed,
    SPI_PERF_RA_ACCUM3_SIMD_FULL_GS__GFX10             = 0x000000ee,
    SPI_PERF_RA_ACCUM0_SIMD_FULL_HS__GFX10             = 0x000000ef,
    SPI_PERF_RA_ACCUM1_SIMD_FULL_HS__GFX10             = 0x000000f0,
    SPI_PERF_RA_ACCUM2_SIMD_FULL_HS__GFX10             = 0x000000f1,
    SPI_PERF_RA_ACCUM3_SIMD_FULL_HS__GFX10             = 0x000000f2,
    SPI_PERF_RA_ACCUM0_SIMD_FULL_CSG__GFX10            = 0x000000f3,
    SPI_PERF_RA_ACCUM1_SIMD_FULL_CSG__GFX10            = 0x000000f4,
    SPI_PERF_RA_ACCUM2_SIMD_FULL_CSG__GFX10            = 0x000000f5,
    SPI_PERF_RA_ACCUM3_SIMD_FULL_CSG__GFX10            = 0x000000f6,
    SPI_PERF_RA_ACCUM0_SIMD_FULL_CSN__GFX10            = 0x000000f7,
    SPI_PERF_RA_ACCUM1_SIMD_FULL_CSN__GFX10            = 0x000000f8,
    SPI_PERF_RA_ACCUM2_SIMD_FULL_CSN__GFX10            = 0x000000f9,
    SPI_PERF_RA_ACCUM3_SIMD_FULL_CSN__GFX10            = 0x000000fa,
    SPI_PERF_EXP_ARB_COL_CNT__GFX10                    = 0x000000fb,
    SPI_PERF_EXP_ARB_PAR_CNT__GFX10                    = 0x000000fc,
    SPI_PERF_EXP_ARB_POS_CNT__GFX10                    = 0x000000fd,
    SPI_PERF_EXP_ARB_GDS_CNT__GFX10                    = 0x000000fe,
    SPI_PERF_NUM_PS_COL_SA0SQ0_EXPORTS__GFX10          = 0x000000ff,
    SPI_PERF_NUM_PS_COL_SA0SQ1_EXPORTS__GFX10          = 0x00000100,
    SPI_PERF_NUM_PS_COL_SA1SQ0_EXPORTS__GFX10          = 0x00000101,
    SPI_PERF_NUM_PS_COL_SA1SQ1_EXPORTS__GFX10          = 0x00000102,
    SPI_PERF_NUM_VS_POS_SA0SQ0_EXPORTS__GFX10          = 0x00000103,
    SPI_PERF_NUM_VS_POS_SA0SQ1_EXPORTS__GFX10          = 0x00000104,
    SPI_PERF_NUM_VS_POS_SA1SQ0_EXPORTS__GFX10          = 0x00000105,
    SPI_PERF_NUM_VS_POS_SA1SQ1_EXPORTS__GFX10          = 0x00000106,
    SPI_PERF_NUM_VS_PARAM_SA0SQ0_EXPORTS__GFX10        = 0x00000107,
    SPI_PERF_NUM_VS_PARAM_SA0SQ1_EXPORTS__GFX10        = 0x00000108,
    SPI_PERF_NUM_VS_PARAM_SA1SQ0_EXPORTS__GFX10        = 0x00000109,
    SPI_PERF_NUM_VS_PARAM_SA1SQ1_EXPORTS__GFX10        = 0x0000010a,
    SPI_PERF_NUM_VS_GDS_SA0SQ0_EXPORTS__GFX10          = 0x0000010b,
    SPI_PERF_NUM_VS_GDS_SA0SQ1_EXPORTS__GFX10          = 0x0000010c,
    SPI_PERF_NUM_VS_GDS_SA1SQ0_EXPORTS__GFX10          = 0x0000010d,
    SPI_PERF_NUM_VS_GDS_SA1SQ1_EXPORTS__GFX10          = 0x0000010e,
    SPI_PERF_NUM_EXPGRANT_EXPORTS__GFX10               = 0x0000010f,
    SPI_PERF_CLKGATE_BUSY_STALL__GFX10                 = 0x00000110,
    SPI_PERF_CLKGATE_ACTIVE_STALL__GFX10               = 0x00000111,
    SPI_PERF_CLKGATE_ALL_CLOCKS_ON__GFX10              = 0x00000112,
    SPI_PERF_CLKGATE_CGTT_DYN_ON__GFX10                = 0x00000113,
    SPI_PERF_CLKGATE_CGTT_REG_ON__GFX10                = 0x00000114,
    SPI_PERF_PIX_ALLOC_PEND_CNT__GFX10                 = 0x00000115,
    SPI_PERF_PIX_ALLOC_SCB0_STALL__GFX10               = 0x00000116,
    SPI_PERF_PIX_ALLOC_SCB1_STALL__GFX10               = 0x00000117,
    SPI_PERF_PIX_ALLOC_SCB2_STALL__GFX10               = 0x00000118,
    SPI_PERF_PIX_ALLOC_SCB3_STALL__GFX10               = 0x00000119,
    SPI_PERF_PIX_ALLOC_DB0_STALL__GFX10                = 0x0000011a,
    SPI_PERF_PIX_ALLOC_DB1_STALL__GFX10                = 0x0000011b,
    SPI_PERF_PIX_ALLOC_DB2_STALL__GFX10                = 0x0000011c,
    SPI_PERF_PIX_ALLOC_DB3_STALL__GFX10                = 0x0000011d,
    SPI_PERF_PIX_ALLOC_DB4_STALL__GFX10                = 0x0000011e,
    SPI_PERF_PIX_ALLOC_DB5_STALL__GFX10                = 0x0000011f,
    SPI_PERF_PIX_ALLOC_DB6_STALL__GFX10                = 0x00000120,
    SPI_PERF_PIX_ALLOC_DB7_STALL__GFX10                = 0x00000121,
    SPI_PERF_GS_NGG_SE_SEND_GS_ALLOC__GFX10            = 0x00000122,
    SPI_PERF_SWC_GS_WR__GFX10                          = 0x00000126,
    SPI_PERF_SWC_HS_WR__GFX10                          = 0x00000128,
    SPI_PERF_SWC_CSGN_WR__GFX10                        = 0x00000129,
    SPI_PERF_SWC_CSN_WR__GFX10                         = 0x0000012a,
    SPI_PERF_VWC_PS_WR__GFX10                          = 0x0000012b,
    SPI_PERF_VWC_VS_WR__GFX10                          = 0x0000012c,
    SPI_PERF_VWC_GS_WR__GFX10                          = 0x0000012e,
    SPI_PERF_VWC_HS_WR__GFX10                          = 0x00000130,
    SPI_PERF_VWC_CSGN_WR__GFX10                        = 0x00000131,
    SPI_PERF_VWC_CSN_WR__GFX10                         = 0x00000132,
    SPI_PERF_RA_REQ_NO_ALLOC_ES__GFX101                = 0x00000092,
    SPI_PERF_RA_REQ_NO_ALLOC_LS__GFX101                = 0x00000094,
    SPI_PERF_RA_RES_STALL_ES__GFX101                   = 0x0000009a,
    SPI_PERF_RA_RES_STALL_LS__GFX101                   = 0x0000009c,
    SPI_PERF_RA_TMP_STALL_ES__GFX101                   = 0x000000a2,
    SPI_PERF_RA_TMP_STALL_LS__GFX101                   = 0x000000a4,
    SPI_PERF_RA_WAVE_SIMD_FULL_ES__GFX101              = 0x000000aa,
    SPI_PERF_RA_WAVE_SIMD_FULL_LS__GFX101              = 0x000000ac,
    SPI_PERF_RA_VGPR_SIMD_FULL_ES__GFX101              = 0x000000b2,
    SPI_PERF_RA_VGPR_SIMD_FULL_LS__GFX101              = 0x000000b5,
    SPI_PERF_RA_SGPR_SIMD_FULL_PS__GFX101              = 0x000000b8,
    SPI_PERF_RA_SGPR_SIMD_FULL_VS__GFX101              = 0x000000b9,
    SPI_PERF_RA_SGPR_SIMD_FULL_ES__GFX101              = 0x000000ba,
    SPI_PERF_RA_SGPR_SIMD_FULL_GS__GFX101              = 0x000000bb,
    SPI_PERF_RA_SGPR_SIMD_FULL_LS__GFX101              = 0x000000bc,
    SPI_PERF_RA_SGPR_SIMD_FULL_HS__GFX101              = 0x000000bd,
    SPI_PERF_RA_SGPR_SIMD_FULL_CSG__GFX101             = 0x000000be,
    SPI_PERF_RA_SGPR_SIMD_FULL_CSN__GFX101             = 0x000000bf,
    SPI_PERF_RA_LDS_CU_FULL_LS__GFX101                 = 0x000000c1,
    SPI_PERF_RA_LDS_CU_FULL_ES__GFX101                 = 0x000000c3,
    SPI_PERF_RA_WVLIM_STALL_ES__GFX101                 = 0x000000d0,
    SPI_PERF_RA_WVLIM_STALL_LS__GFX101                 = 0x000000d2,
    SPI_PERF_RA_ES_LOCK__GFX101                        = 0x000000d8,
    SPI_PERF_RA_LS_LOCK__GFX101                        = 0x000000da,
    SPI_PERF_SWC_PS_WR__GFX101                         = 0x00000123,
    SPI_PERF_SWC_VS_WR__GFX101                         = 0x00000124,
    SPI_PERF_SWC_ES_WR__GFX101                         = 0x00000125,
    SPI_PERF_SWC_LS_WR__GFX101                         = 0x00000127,
    SPI_PERF_VWC_ES_WR__GFX101                         = 0x0000012d,
    SPI_PERF_VWC_LS_WR__GFX101                         = 0x0000012f,
    SPI_PERF_ES_WINDOW_VALID__GFX101                   = 0x00000133,
    SPI_PERF_ES_BUSY__GFX101                           = 0x00000134,
    SPI_PERF_ES_CRAWLER_STALL__GFX101                  = 0x00000135,
    SPI_PERF_ES_FIRST_WAVE__GFX101                     = 0x00000136,
    SPI_PERF_ES_LAST_WAVE__GFX101                      = 0x00000137,
    SPI_PERF_ES_LSHS_DEALLOC__GFX101                   = 0x00000138,
    SPI_PERF_ES_EVENT_WAVE__GFX101                     = 0x00000139,
    SPI_PERF_ES_WAVE__GFX101                           = 0x0000013a,
    SPI_PERF_ES_PERS_UPD_FULL0__GFX101                 = 0x0000013b,
    SPI_PERF_ES_PERS_UPD_FULL1__GFX101                 = 0x0000013c,
    SPI_PERF_ES_FIRST_SUBGRP__GFX101                   = 0x0000013d,
    SPI_PERF_ES_LAST_SUBGRP__GFX101                    = 0x0000013e,
    SPI_PERF_LS_WINDOW_VALID__GFX101                   = 0x0000013f,
    SPI_PERF_LS_BUSY__GFX101                           = 0x00000140,
    SPI_PERF_LS_CRAWLER_STALL__GFX101                  = 0x00000141,
    SPI_PERF_LS_FIRST_WAVE__GFX101                     = 0x00000142,
    SPI_PERF_LS_LAST_WAVE__GFX101                      = 0x00000143,
    SPI_PERF_LS_OFFCHIP_LDS_STALL__GFX101              = 0x00000144,
    SPI_PERF_LS_EVENT_WAVE__GFX101                     = 0x00000145,
    SPI_PERF_LS_WAVE__GFX101                           = 0x00000146,
    SPI_PERF_LS_PERS_UPD_FULL0__GFX101                 = 0x00000147,
    SPI_PERF_LS_PERS_UPD_FULL1__GFX101                 = 0x00000148,
    SPI_PERF_EXP_THROT_UPSTEP__GFX10VRS                = 0x00000149,
    SPI_PERF_EXP_THROT_DOWNSTEP__GFX10VRS              = 0x0000014a,
    SPI_PERF_EXP_THROT_CAUSALITY_DETECTED__GFX10VRS    = 0x0000014b,
    SPI_PERF_GS_WINDOW_VALID__GFX11                    = 0x00000001,
    SPI_PERF_GS_BUSY__GFX11                            = 0x00000002,
    SPI_PERF_GS_CRAWLER_STALL__GFX11                   = 0x00000003,
    SPI_PERF_GS_EVENT_WAVE__GFX11                      = 0x00000004,
    SPI_PERF_GS_WAVE__GFX11                            = 0x00000005,
    SPI_PERF_GS_PERS_UPD_FULL0__GFX11                  = 0x00000006,
    SPI_PERF_GS_PERS_UPD_FULL1__GFX11                  = 0x00000007,
    SPI_PERF_GS_FIRST_SUBGRP__GFX11                    = 0x00000008,
    SPI_PERF_GS_HS_DEALLOC__GFX11                      = 0x00000009,
    SPI_PERF_GS_NGG_SE_LATE_ALLOC_LIMIT__GFX11         = 0x0000000a,
    SPI_PERF_GS_POS0_STALL__GFX11                      = 0x0000000b,
    SPI_PERF_GS_POS1_STALL__GFX11                      = 0x0000000c,
    SPI_PERF_GS_INDX0_STALL__GFX11                     = 0x0000000d,
    SPI_PERF_GS_INDX1_STALL__GFX11                     = 0x0000000e,
    SPI_PERF_GS_PWS_STALL__GFX11                       = 0x0000000f,
    SPI_PERF_GS_GRP_LIFETIME__GFX11                    = 0x00000010,
    SPI_PERF_GS_WAVE_IN_FLIGHT__GFX11                  = 0x00000011,
    SPI_PERF_HS_WINDOW_VALID__GFX11                    = 0x00000015,
    SPI_PERF_HS_BUSY__GFX11                            = 0x00000016,
    SPI_PERF_HS_CRAWLER_STALL__GFX11                   = 0x00000017,
    SPI_PERF_HS_FIRST_WAVE__GFX11                      = 0x00000018,
    SPI_PERF_HS_OFFCHIP_LDS_STALL__GFX11               = 0x00000019,
    SPI_PERF_HS_EVENT_WAVE__GFX11                      = 0x0000001a,
    SPI_PERF_HS_WAVE__GFX11                            = 0x0000001b,
    SPI_PERF_HS_PERS_UPD_FULL0__GFX11                  = 0x0000001c,
    SPI_PERF_HS_PERS_UPD_FULL1__GFX11                  = 0x0000001d,
    SPI_PERF_HS_PWS_STALL__GFX11                       = 0x0000001e,
    SPI_PERF_HS_WAVE_IN_FLIGHT__GFX11                  = 0x0000001f,
    SPI_PERF_CSGN_WINDOW_VALID__GFX11                  = 0x00000025,
    SPI_PERF_CSGN_BUSY__GFX11                          = 0x00000026,
    SPI_PERF_CSGN_NUM_THREADGROUPS__GFX11              = 0x00000027,
    SPI_PERF_CSGN_CRAWLER_STALL__GFX11                 = 0x00000028,
    SPI_PERF_CSGN_EVENT_WAVE__GFX11                    = 0x00000029,
    SPI_PERF_CSGN_WAVE__GFX11                          = 0x0000002a,
    SPI_PERF_CSGN_PWS_STALL__GFX11                     = 0x0000002b,
    SPI_PERF_CSGN_WAVE_IN_FLIGHT__GFX11                = 0x0000002c,
    SPI_PERF_CSN_WINDOW_VALID__GFX11                   = 0x0000002d,
    SPI_PERF_CSN_BUSY__GFX11                           = 0x0000002e,
    SPI_PERF_CSN_NUM_THREADGROUPS__GFX11               = 0x0000002f,
    SPI_PERF_CSN_CRAWLER_STALL__GFX11                  = 0x00000030,
    SPI_PERF_CSN_EVENT_WAVE__GFX11                     = 0x00000031,
    SPI_PERF_CSN_WAVE__GFX11                           = 0x00000032,
    SPI_PERF_CSN_WAVE_IN_FLIGHT__GFX11                 = 0x00000033,
    SPI_PERF_PS0_WINDOW_VALID__GFX11                   = 0x00000035,
    SPI_PERF_PS1_WINDOW_VALID__GFX11                   = 0x00000036,
    SPI_PERF_PS2_WINDOW_VALID__GFX11                   = 0x00000037,
    SPI_PERF_PS3_WINDOW_VALID__GFX11                   = 0x00000038,
    SPI_PERF_PS0_BUSY__GFX11                           = 0x00000039,
    SPI_PERF_PS1_BUSY__GFX11                           = 0x0000003a,
    SPI_PERF_PS2_BUSY__GFX11                           = 0x0000003b,
    SPI_PERF_PS3_BUSY__GFX11                           = 0x0000003c,
    SPI_PERF_PS0_ACTIVE__GFX11                         = 0x0000003d,
    SPI_PERF_PS1_ACTIVE__GFX11                         = 0x0000003e,
    SPI_PERF_PS2_ACTIVE__GFX11                         = 0x0000003f,
    SPI_PERF_PS3_ACTIVE__GFX11                         = 0x00000040,
    SPI_PERF_PS0_DEALLOC__GFX11                        = 0x00000041,
    SPI_PERF_PS1_DEALLOC__GFX11                        = 0x00000042,
    SPI_PERF_PS2_DEALLOC__GFX11                        = 0x00000043,
    SPI_PERF_PS3_DEALLOC__GFX11                        = 0x00000044,
    SPI_PERF_PS0_EVENT_WAVE__GFX11                     = 0x00000045,
    SPI_PERF_PS1_EVENT_WAVE__GFX11                     = 0x00000046,
    SPI_PERF_PS2_EVENT_WAVE__GFX11                     = 0x00000047,
    SPI_PERF_PS3_EVENT_WAVE__GFX11                     = 0x00000048,
    SPI_PERF_PS0_WAVE__GFX11                           = 0x00000049,
    SPI_PERF_PS1_WAVE__GFX11                           = 0x0000004a,
    SPI_PERF_PS2_WAVE__GFX11                           = 0x0000004b,
    SPI_PERF_PS3_WAVE__GFX11                           = 0x0000004c,
    SPI_PERF_PS0_OPT_WAVE__GFX11                       = 0x0000004d,
    SPI_PERF_PS1_OPT_WAVE__GFX11                       = 0x0000004e,
    SPI_PERF_PS2_OPT_WAVE__GFX11                       = 0x0000004f,
    SPI_PERF_PS3_OPT_WAVE__GFX11                       = 0x00000050,
    SPI_PERF_PS0_PRIM_BIN0__GFX11                      = 0x00000051,
    SPI_PERF_PS1_PRIM_BIN0__GFX11                      = 0x00000052,
    SPI_PERF_PS2_PRIM_BIN0__GFX11                      = 0x00000053,
    SPI_PERF_PS3_PRIM_BIN0__GFX11                      = 0x00000054,
    SPI_PERF_PS0_PRIM_BIN1__GFX11                      = 0x00000055,
    SPI_PERF_PS1_PRIM_BIN1__GFX11                      = 0x00000056,
    SPI_PERF_PS2_PRIM_BIN1__GFX11                      = 0x00000057,
    SPI_PERF_PS3_PRIM_BIN1__GFX11                      = 0x00000058,
    SPI_PERF_PS0_CRAWLER_STALL__GFX11                  = 0x00000059,
    SPI_PERF_PS1_CRAWLER_STALL__GFX11                  = 0x0000005a,
    SPI_PERF_PS2_CRAWLER_STALL__GFX11                  = 0x0000005b,
    SPI_PERF_PS3_CRAWLER_STALL__GFX11                  = 0x0000005c,
    SPI_PERF_PS_PERS_UPD_FULL0__GFX11                  = 0x0000005d,
    SPI_PERF_PS_PERS_UPD_FULL1__GFX11                  = 0x0000005e,
    SPI_PERF_PS0_2_WAVE_GROUPS__GFX11                  = 0x0000005f,
    SPI_PERF_PS1_2_WAVE_GROUPS__GFX11                  = 0x00000060,
    SPI_PERF_PS2_2_WAVE_GROUPS__GFX11                  = 0x00000061,
    SPI_PERF_PS3_2_WAVE_GROUPS__GFX11                  = 0x00000062,
    SPI_PERF_PS0_WAVE_GROUP_CLOCK_DELAY__GFX11         = 0x00000063,
    SPI_PERF_PS1_WAVE_GROUP_CLOCK_DELAY__GFX11         = 0x00000064,
    SPI_PERF_PS2_WAVE_GROUP_CLOCK_DELAY__GFX11         = 0x00000065,
    SPI_PERF_PS3_WAVE_GROUP_CLOCK_DELAY__GFX11         = 0x00000066,
    SPI_PERF_PS0_WAVE_GROUP_TIMEOUTS__GFX11            = 0x00000067,
    SPI_PERF_PS1_WAVE_GROUP_TIMEOUTS__GFX11            = 0x00000068,
    SPI_PERF_PS2_WAVE_GROUP_TIMEOUTS__GFX11            = 0x00000069,
    SPI_PERF_PS3_WAVE_GROUP_TIMEOUTS__GFX11            = 0x0000006a,
    SPI_PERF_PS_PWS_STALL__GFX11                       = 0x0000006b,
    SPI_PERF_PS0_LDS_DONE_FULL__GFX11                  = 0x0000006c,
    SPI_PERF_PS1_LDS_DONE_FULL__GFX11                  = 0x0000006d,
    SPI_PERF_PS2_LDS_DONE_FULL__GFX11                  = 0x0000006e,
    SPI_PERF_PS3_LDS_DONE_FULL__GFX11                  = 0x0000006f,
    SPI_PERF_PS0_DEALLOC_FULL__GFX11                   = 0x00000070,
    SPI_PERF_PS1_DEALLOC_FULL__GFX11                   = 0x00000071,
    SPI_PERF_PS2_DEALLOC_FULL__GFX11                   = 0x00000072,
    SPI_PERF_PS3_DEALLOC_FULL__GFX11                   = 0x00000073,
    SPI_PERF_PS0_WAVE_IN_FLIGHT__GFX11                 = 0x00000074,
    SPI_PERF_PS1_WAVE_IN_FLIGHT__GFX11                 = 0x00000075,
    SPI_PERF_PS2_WAVE_IN_FLIGHT__GFX11                 = 0x00000076,
    SPI_PERF_PS3_WAVE_IN_FLIGHT__GFX11                 = 0x00000077,
    SPI_PERF_RA_GS_LDS_OCCUPANCY__GFX11                = 0x00000085,
    SPI_PERF_RA_GS_VGPR_OCCUPANCY__GFX11               = 0x00000086,
    SPI_PERF_RA_PS_LDS_OCCUPANCY__GFX11                = 0x00000087,
    SPI_PERF_RA_PS_VGPR_OCCUPANCY__GFX11               = 0x00000088,
    SPI_PERF_RA_SPI_THROTTLE__GFX11                    = 0x00000089,
    SPI_PERF_RA_PH_THROTTLE__GFX11                     = 0x0000008a,
    SPI_PERF_RA_PC_PROBE_STALL_PS__GFX11               = 0x0000008b,
    SPI_PERF_RA_PC_PSWAVE_STALL_PS__GFX11              = 0x0000008c,
    SPI_PERF_RA_PIPE_REQ_BIN2__GFX11                   = 0x0000008d,
    SPI_PERF_RA_TASK_REQ_BIN3__GFX11                   = 0x0000008e,
    SPI_PERF_RA_WR_CTL_FULL__GFX11                     = 0x0000008f,
    SPI_PERF_RA_REQ_NO_ALLOC__GFX11                    = 0x00000090,
    SPI_PERF_RA_REQ_NO_ALLOC_PS__GFX11                 = 0x00000091,
    SPI_PERF_RA_REQ_NO_ALLOC_GS__GFX11                 = 0x00000092,
    SPI_PERF_RA_REQ_NO_ALLOC_HS__GFX11                 = 0x00000093,
    SPI_PERF_RA_REQ_NO_ALLOC_CSG__GFX11                = 0x00000094,
    SPI_PERF_RA_REQ_NO_ALLOC_CSN__GFX11                = 0x00000095,
    SPI_PERF_RA_RES_STALL_PS__GFX11                    = 0x00000096,
    SPI_PERF_RA_RES_STALL_GS__GFX11                    = 0x00000097,
    SPI_PERF_RA_RES_STALL_HS__GFX11                    = 0x00000098,
    SPI_PERF_RA_RES_STALL_CSG__GFX11                   = 0x00000099,
    SPI_PERF_RA_RES_STALL_CSN__GFX11                   = 0x0000009a,
    SPI_PERF_RA_TMP_STALL_PS__GFX11                    = 0x0000009b,
    SPI_PERF_RA_TMP_STALL_GS__GFX11                    = 0x0000009c,
    SPI_PERF_RA_TMP_STALL_HS__GFX11                    = 0x0000009d,
    SPI_PERF_RA_TMP_STALL_CSG__GFX11                   = 0x0000009e,
    SPI_PERF_RA_TMP_STALL_CSN__GFX11                   = 0x0000009f,
    SPI_PERF_RA_WAVE_SIMD_FULL_PS__GFX11               = 0x000000a0,
    SPI_PERF_RA_WAVE_SIMD_FULL_GS__GFX11               = 0x000000a1,
    SPI_PERF_RA_WAVE_SIMD_FULL_HS__GFX11               = 0x000000a2,
    SPI_PERF_RA_WAVE_SIMD_FULL_CSG__GFX11              = 0x000000a3,
    SPI_PERF_RA_WAVE_SIMD_FULL_CSN__GFX11              = 0x000000a4,
    SPI_PERF_RA_VGPR_SIMD_FULL_PS__GFX11               = 0x000000a5,
    SPI_PERF_RA_VGPR_SIMD_FULL_GS__GFX11               = 0x000000a6,
    SPI_PERF_RA_VGPR_SIMD_FULL_HS__GFX11               = 0x000000a7,
    SPI_PERF_RA_VGPR_SIMD_FULL_CSG__GFX11              = 0x000000a8,
    SPI_PERF_RA_VGPR_SIMD_FULL_CSN__GFX11              = 0x000000a9,
    SPI_PERF_RA_LDS_CU_FULL_PS__GFX11                  = 0x000000aa,
    SPI_PERF_RA_LDS_CU_FULL_HS__GFX11                  = 0x000000ab,
    SPI_PERF_RA_LDS_CU_FULL_GS__GFX11                  = 0x000000ac,
    SPI_PERF_RA_LDS_CU_FULL_CSG__GFX11                 = 0x000000ad,
    SPI_PERF_RA_LDS_CU_FULL_CSN__GFX11                 = 0x000000ae,
    SPI_PERF_RA_BAR_CU_FULL_PS__GFX11                  = 0x000000af,
    SPI_PERF_RA_BAR_CU_FULL_GS__GFX11                  = 0x000000b0,
    SPI_PERF_RA_BAR_CU_FULL_HS__GFX11                  = 0x000000b1,
    SPI_PERF_RA_BAR_CU_FULL_CSG__GFX11                 = 0x000000b2,
    SPI_PERF_RA_BAR_CU_FULL_CSN__GFX11                 = 0x000000b3,
    SPI_PERF_RA_BULKY_CU_FULL_CSG__GFX11               = 0x000000b4,
    SPI_PERF_RA_BULKY_CU_FULL_CSN__GFX11               = 0x000000b5,
    SPI_PERF_RA_TGLIM_CU_FULL_CSG__GFX11               = 0x000000b6,
    SPI_PERF_RA_TGLIM_CU_FULL_CSN__GFX11               = 0x000000b7,
    SPI_PERF_RA_WVLIM_STALL_PS__GFX11                  = 0x000000b8,
    SPI_PERF_RA_WVLIM_STALL_GS__GFX11                  = 0x000000b9,
    SPI_PERF_RA_WVLIM_STALL_HS__GFX11                  = 0x000000ba,
    SPI_PERF_RA_WVLIM_STALL_CSG__GFX11                 = 0x000000bb,
    SPI_PERF_RA_WVLIM_STALL_CSN__GFX11                 = 0x000000bc,
    SPI_PERF_RA_GS_LOCK__GFX11                         = 0x000000bd,
    SPI_PERF_RA_HS_LOCK__GFX11                         = 0x000000be,
    SPI_PERF_RA_CSG_LOCK__GFX11                        = 0x000000bf,
    SPI_PERF_RA_CSN_LOCK__GFX11                        = 0x000000c0,
    SPI_PERF_RA_RSV_UPD__GFX11                         = 0x000000c1,
    SPI_PERF_RA_PRE_ALLOC_STALL__GFX11                 = 0x000000c2,
    SPI_PERF_RA_GFX_UNDER_TUNNEL__GFX11                = 0x000000c3,
    SPI_PERF_RA_CSC_UNDER_TUNNEL__GFX11                = 0x000000c4,
    SPI_PERF_RA_WVALLOC_STALL__GFX11                   = 0x000000c5,
    SPI_PERF_RA_ACCUM0_SIMD_FULL_PS__GFX11             = 0x000000c6,
    SPI_PERF_RA_ACCUM1_SIMD_FULL_PS__GFX11             = 0x000000c7,
    SPI_PERF_RA_ACCUM2_SIMD_FULL_PS__GFX11             = 0x000000c8,
    SPI_PERF_RA_ACCUM3_SIMD_FULL_PS__GFX11             = 0x000000c9,
    SPI_PERF_RA_ACCUM0_SIMD_FULL_GS__GFX11             = 0x000000ca,
    SPI_PERF_RA_ACCUM1_SIMD_FULL_GS__GFX11             = 0x000000cb,
    SPI_PERF_RA_ACCUM2_SIMD_FULL_GS__GFX11             = 0x000000cc,
    SPI_PERF_RA_ACCUM3_SIMD_FULL_GS__GFX11             = 0x000000cd,
    SPI_PERF_RA_ACCUM0_SIMD_FULL_HS__GFX11             = 0x000000ce,
    SPI_PERF_RA_ACCUM1_SIMD_FULL_HS__GFX11             = 0x000000cf,
    SPI_PERF_RA_ACCUM2_SIMD_FULL_HS__GFX11             = 0x000000d0,
    SPI_PERF_RA_ACCUM3_SIMD_FULL_HS__GFX11             = 0x000000d1,
    SPI_PERF_RA_ACCUM0_SIMD_FULL_CSG__GFX11            = 0x000000d2,
    SPI_PERF_RA_ACCUM1_SIMD_FULL_CSG__GFX11            = 0x000000d3,
    SPI_PERF_RA_ACCUM2_SIMD_FULL_CSG__GFX11            = 0x000000d4,
    SPI_PERF_RA_ACCUM3_SIMD_FULL_CSG__GFX11            = 0x000000d5,
    SPI_PERF_RA_ACCUM0_SIMD_FULL_CSN__GFX11            = 0x000000d6,
    SPI_PERF_RA_ACCUM1_SIMD_FULL_CSN__GFX11            = 0x000000d7,
    SPI_PERF_RA_ACCUM2_SIMD_FULL_CSN__GFX11            = 0x000000d8,
    SPI_PERF_RA_ACCUM3_SIMD_FULL_CSN__GFX11            = 0x000000d9,
    SPI_PERF_EXP_ARB_COL_CNT__GFX11                    = 0x000000da,
    SPI_PERF_EXP_ARB_POS_CNT__GFX11                    = 0x000000db,
    SPI_PERF_EXP_ARB_GDS_CNT__GFX11                    = 0x000000dc,
    SPI_PERF_EXP_ARB_IDX_CNT__GFX11                    = 0x000000dd,
    SPI_PERF_EXP_WITH_CONFLICT__GFX11                  = 0x000000de,
    SPI_PERF_EXP_WITH_CONFLICT_CLEAR__GFX11            = 0x000000df,
    SPI_PERF_GS_EXP_DONE__GFX11                        = 0x000000e0,
    SPI_PERF_PS_EXP_DONE__GFX11                        = 0x000000e1,
    SPI_PERF_PS_EXP_ARB_CONFLICT__GFX11                = 0x000000e2,
    SPI_PERF_GS_SCBD_IDX_CLEANUP__GFX11                = 0x000000e3,
    SPI_PERF_GS_SCBD_POS_CLEANUP__GFX11                = 0x000000e4,
    SPI_PERF_PS_EXP_ALLOC__GFX11                       = 0x000000e5,
    SPI_PERF_PS0_WAVEID_STARVED__GFX11                 = 0x000000e6,
    SPI_PERF_PS1_WAVEID_STARVED__GFX11                 = 0x000000e7,
    SPI_PERF_PS2_WAVEID_STARVED__GFX11                 = 0x000000e8,
    SPI_PERF_PS3_WAVEID_STARVED__GFX11                 = 0x000000e9,
    SPI_PERF_PS0_EXP_ALLOC_WITH_CONFLICT__GFX11        = 0x000000ea,
    SPI_PERF_PS1_EXP_ALLOC_WITH_CONFLICT__GFX11        = 0x000000eb,
    SPI_PERF_PS2_EXP_ALLOC_WITH_CONFLICT__GFX11        = 0x000000ec,
    SPI_PERF_PS3_EXP_ALLOC_WITH_CONFLICT__GFX11        = 0x000000ed,
    SPI_PERF_NUM_PS_COL_SA0SQ0_EXPORTS__GFX11          = 0x000000ee,
    SPI_PERF_NUM_PS_COL_SA0SQ1_EXPORTS__GFX11          = 0x000000ef,
    SPI_PERF_NUM_PS_COL_SA1SQ0_EXPORTS__GFX11          = 0x000000f0,
    SPI_PERF_NUM_PS_COL_SA1SQ1_EXPORTS__GFX11          = 0x000000f1,
    SPI_PERF_NUM_POS_SA0SQ0_EXPORTS__GFX11             = 0x000000f2,
    SPI_PERF_NUM_POS_SA0SQ1_EXPORTS__GFX11             = 0x000000f3,
    SPI_PERF_NUM_POS_SA1SQ0_EXPORTS__GFX11             = 0x000000f4,
    SPI_PERF_NUM_POS_SA1SQ1_EXPORTS__GFX11             = 0x000000f5,
    SPI_PERF_NUM_GDS_SA0SQ0_EXPORTS__GFX11             = 0x000000f6,
    SPI_PERF_NUM_GDS_SA0SQ1_EXPORTS__GFX11             = 0x000000f7,
    SPI_PERF_NUM_GDS_SA1SQ0_EXPORTS__GFX11             = 0x000000f8,
    SPI_PERF_NUM_GDS_SA1SQ1_EXPORTS__GFX11             = 0x000000f9,
    SPI_PERF_NUM_EXPGRANT_EXPORTS__GFX11               = 0x000000fa,
    SPI_PERF_GS_ALLOC_IDX__GFX11                       = 0x000000fb,
    SPI_PERF_GS_ALLOC_POS__GFX11                       = 0x000000fc,
    SPI_PERF_PIX_ALLOC_PEND_CNT__GFX11                 = 0x000000fd,
    SPI_PERF_EXPORT_SCB0_STALL__GFX11                  = 0x000000fe,
    SPI_PERF_EXPORT_SCB1_STALL__GFX11                  = 0x000000ff,
    SPI_PERF_EXPORT_SCB2_STALL__GFX11                  = 0x00000100,
    SPI_PERF_EXPORT_SCB3_STALL__GFX11                  = 0x00000101,
    SPI_PERF_EXPORT_DB0_STALL__GFX11                   = 0x00000102,
    SPI_PERF_EXPORT_DB1_STALL__GFX11                   = 0x00000103,
    SPI_PERF_EXPORT_DB2_STALL__GFX11                   = 0x00000104,
    SPI_PERF_EXPORT_DB3_STALL__GFX11                   = 0x00000105,
    SPI_PERF_EXPORT_DB4_STALL__GFX11                   = 0x00000106,
    SPI_PERF_EXPORT_DB5_STALL__GFX11                   = 0x00000107,
    SPI_PERF_EXPORT_DB6_STALL__GFX11                   = 0x00000108,
    SPI_PERF_EXPORT_DB7_STALL__GFX11                   = 0x00000109,
    SPI_PERF_GS_NGG_SE_SEND_GS_ALLOC__GFX11            = 0x0000010a,
    SPI_PERF_GS_NGG_STALL_MSG_VAL__GFX11               = 0x0000010b,
    SPI_PERF_SWC_PS_WR__GFX11                          = 0x0000010c,
    SPI_PERF_SWC_GS_WR__GFX11                          = 0x0000010d,
    SPI_PERF_SWC_HS_WR__GFX11                          = 0x0000010e,
    SPI_PERF_SWC_CSGN_WR__GFX11                        = 0x0000010f,
    SPI_PERF_SWC_CSN_WR__GFX11                         = 0x00000110,
    SPI_PERF_VWC_PS_WR__GFX11                          = 0x00000111,
    SPI_PERF_VWC_ES_WR__GFX11                          = 0x00000112,
    SPI_PERF_VWC_GS_WR__GFX11                          = 0x00000113,
    SPI_PERF_VWC_LS_WR__GFX11                          = 0x00000114,
    SPI_PERF_VWC_HS_WR__GFX11                          = 0x00000115,
    SPI_PERF_VWC_CSGN_WR__GFX11                        = 0x00000116,
    SPI_PERF_VWC_CSN_WR__GFX11                         = 0x00000117,
    SPI_PERF_EXP_THROT_UPSTEP__GFX11                   = 0x00000118,
    SPI_PERF_EXP_THROT_DOWNSTEP__GFX11                 = 0x00000119,
    SPI_PERF_EXP_THROT_CAUSALITY_DETECTED__GFX11       = 0x0000011a,
    SPI_PERF_BUSY__GFX11                               = 0x0000011b,
    SPI_PERF_RA_REQ_NO_ALLOC_ES__NV21                  = 0x00000092,
    SPI_PERF_RA_REQ_NO_ALLOC_LS__NV21                  = 0x00000094,
    SPI_PERF_RA_RES_STALL_ES__NV21                     = 0x0000009a,
    SPI_PERF_RA_RES_STALL_LS__NV21                     = 0x0000009c,
    SPI_PERF_RA_TMP_STALL_ES__NV21                     = 0x000000a2,
    SPI_PERF_RA_TMP_STALL_LS__NV21                     = 0x000000a4,
    SPI_PERF_RA_WAVE_SIMD_FULL_ES__NV21                = 0x000000aa,
    SPI_PERF_RA_WAVE_SIMD_FULL_LS__NV21                = 0x000000ac,
    SPI_PERF_RA_VGPR_SIMD_FULL_ES__NV21                = 0x000000b2,
    SPI_PERF_RA_VGPR_SIMD_FULL_LS__NV21                = 0x000000b5,
    SPI_PERF_RA_SGPR_SIMD_FULL_PS__NV21                = 0x000000b8,
    SPI_PERF_RA_SGPR_SIMD_FULL_VS__NV21                = 0x000000b9,
    SPI_PERF_RA_SGPR_SIMD_FULL_ES__NV21                = 0x000000ba,
    SPI_PERF_RA_SGPR_SIMD_FULL_GS__NV21                = 0x000000bb,
    SPI_PERF_RA_SGPR_SIMD_FULL_LS__NV21                = 0x000000bc,
    SPI_PERF_RA_SGPR_SIMD_FULL_HS__NV21                = 0x000000bd,
    SPI_PERF_RA_SGPR_SIMD_FULL_CSG__NV21               = 0x000000be,
    SPI_PERF_RA_SGPR_SIMD_FULL_CSN__NV21               = 0x000000bf,
    SPI_PERF_RA_LDS_CU_FULL_LS__NV21                   = 0x000000c1,
    SPI_PERF_RA_LDS_CU_FULL_ES__NV21                   = 0x000000c3,
    SPI_PERF_RA_WVLIM_STALL_ES__NV21                   = 0x000000d0,
    SPI_PERF_RA_WVLIM_STALL_LS__NV21                   = 0x000000d2,
    SPI_PERF_RA_ES_LOCK__NV21                          = 0x000000d8,
    SPI_PERF_RA_LS_LOCK__NV21                          = 0x000000da,
    SPI_PERF_SWC_PS_WR__NV21                           = 0x00000123,
    SPI_PERF_SWC_VS_WR__NV21                           = 0x00000124,
    SPI_PERF_SWC_ES_WR__NV21                           = 0x00000125,
    SPI_PERF_SWC_LS_WR__NV21                           = 0x00000127,
    SPI_PERF_VWC_ES_WR__NV21                           = 0x0000012d,
    SPI_PERF_VWC_LS_WR__NV21                           = 0x0000012f,
    SPI_PERF_ES_WINDOW_VALID__NV21                     = 0x00000133,
    SPI_PERF_ES_BUSY__NV21                             = 0x00000134,
    SPI_PERF_ES_CRAWLER_STALL__NV21                    = 0x00000135,
    SPI_PERF_ES_FIRST_WAVE__NV21                       = 0x00000136,
    SPI_PERF_ES_LAST_WAVE__NV21                        = 0x00000137,
    SPI_PERF_ES_LSHS_DEALLOC__NV21                     = 0x00000138,
    SPI_PERF_ES_EVENT_WAVE__NV21                       = 0x00000139,
    SPI_PERF_ES_WAVE__NV21                             = 0x0000013a,
    SPI_PERF_ES_PERS_UPD_FULL0__NV21                   = 0x0000013b,
    SPI_PERF_ES_PERS_UPD_FULL1__NV21                   = 0x0000013c,
    SPI_PERF_ES_FIRST_SUBGRP__NV21                     = 0x0000013d,
    SPI_PERF_ES_LAST_SUBGRP__NV21                      = 0x0000013e,
    SPI_PERF_LS_WINDOW_VALID__NV21                     = 0x0000013f,
    SPI_PERF_LS_BUSY__NV21                             = 0x00000140,
    SPI_PERF_LS_CRAWLER_STALL__NV21                    = 0x00000141,
    SPI_PERF_LS_FIRST_WAVE__NV21                       = 0x00000142,
    SPI_PERF_LS_LAST_WAVE__NV21                        = 0x00000143,
    SPI_PERF_LS_OFFCHIP_LDS_STALL__NV21                = 0x00000144,
    SPI_PERF_LS_EVENT_WAVE__NV21                       = 0x00000145,
    SPI_PERF_LS_WAVE__NV21                             = 0x00000146,
    SPI_PERF_LS_PERS_UPD_FULL0__NV21                   = 0x00000147,
    SPI_PERF_LS_PERS_UPD_FULL1__NV21                   = 0x00000148,
    SPI_PERF_RA_REQ_NO_ALLOC_ES__NV22                  = 0x00000092,
    SPI_PERF_RA_REQ_NO_ALLOC_LS__NV22                  = 0x00000094,
    SPI_PERF_RA_RES_STALL_ES__NV22                     = 0x0000009a,
    SPI_PERF_RA_RES_STALL_LS__NV22                     = 0x0000009c,
    SPI_PERF_RA_TMP_STALL_ES__NV22                     = 0x000000a2,
    SPI_PERF_RA_TMP_STALL_LS__NV22                     = 0x000000a4,
    SPI_PERF_RA_WAVE_SIMD_FULL_ES__NV22                = 0x000000aa,
    SPI_PERF_RA_WAVE_SIMD_FULL_LS__NV22                = 0x000000ac,
    SPI_PERF_RA_VGPR_SIMD_FULL_ES__NV22                = 0x000000b2,
    SPI_PERF_RA_VGPR_SIMD_FULL_LS__NV22                = 0x000000b5,
    SPI_PERF_RA_SGPR_SIMD_FULL_PS__NV22                = 0x000000b8,
    SPI_PERF_RA_SGPR_SIMD_FULL_VS__NV22                = 0x000000b9,
    SPI_PERF_RA_SGPR_SIMD_FULL_ES__NV22                = 0x000000ba,
    SPI_PERF_RA_SGPR_SIMD_FULL_GS__NV22                = 0x000000bb,
    SPI_PERF_RA_SGPR_SIMD_FULL_LS__NV22                = 0x000000bc,
    SPI_PERF_RA_SGPR_SIMD_FULL_HS__NV22                = 0x000000bd,
    SPI_PERF_RA_SGPR_SIMD_FULL_CSG__NV22               = 0x000000be,
    SPI_PERF_RA_SGPR_SIMD_FULL_CSN__NV22               = 0x000000bf,
    SPI_PERF_RA_LDS_CU_FULL_LS__NV22                   = 0x000000c1,
    SPI_PERF_RA_LDS_CU_FULL_ES__NV22                   = 0x000000c3,
    SPI_PERF_RA_WVLIM_STALL_ES__NV22                   = 0x000000d0,
    SPI_PERF_RA_WVLIM_STALL_LS__NV22                   = 0x000000d2,
    SPI_PERF_RA_ES_LOCK__NV22                          = 0x000000d8,
    SPI_PERF_RA_LS_LOCK__NV22                          = 0x000000da,
    SPI_PERF_SWC_PS_WR__NV22                           = 0x00000123,
    SPI_PERF_SWC_VS_WR__NV22                           = 0x00000124,
    SPI_PERF_SWC_ES_WR__NV22                           = 0x00000125,
    SPI_PERF_SWC_LS_WR__NV22                           = 0x00000127,
    SPI_PERF_VWC_ES_WR__NV22                           = 0x0000012d,
    SPI_PERF_VWC_LS_WR__NV22                           = 0x0000012f,
    SPI_PERF_ES_WINDOW_VALID__NV22                     = 0x00000133,
    SPI_PERF_ES_BUSY__NV22                             = 0x00000134,
    SPI_PERF_ES_CRAWLER_STALL__NV22                    = 0x00000135,
    SPI_PERF_ES_FIRST_WAVE__NV22                       = 0x00000136,
    SPI_PERF_ES_LAST_WAVE__NV22                        = 0x00000137,
    SPI_PERF_ES_LSHS_DEALLOC__NV22                     = 0x00000138,
    SPI_PERF_ES_EVENT_WAVE__NV22                       = 0x00000139,
    SPI_PERF_ES_WAVE__NV22                             = 0x0000013a,
    SPI_PERF_ES_PERS_UPD_FULL0__NV22                   = 0x0000013b,
    SPI_PERF_ES_PERS_UPD_FULL1__NV22                   = 0x0000013c,
    SPI_PERF_ES_FIRST_SUBGRP__NV22                     = 0x0000013d,
    SPI_PERF_ES_LAST_SUBGRP__NV22                      = 0x0000013e,
    SPI_PERF_LS_WINDOW_VALID__NV22                     = 0x0000013f,
    SPI_PERF_LS_BUSY__NV22                             = 0x00000140,
    SPI_PERF_LS_CRAWLER_STALL__NV22                    = 0x00000141,
    SPI_PERF_LS_FIRST_WAVE__NV22                       = 0x00000142,
    SPI_PERF_LS_LAST_WAVE__NV22                        = 0x00000143,
    SPI_PERF_LS_OFFCHIP_LDS_STALL__NV22                = 0x00000144,
    SPI_PERF_LS_EVENT_WAVE__NV22                       = 0x00000145,
    SPI_PERF_LS_WAVE__NV22                             = 0x00000146,
    SPI_PERF_LS_PERS_UPD_FULL0__NV22                   = 0x00000147,
    SPI_PERF_LS_PERS_UPD_FULL1__NV22                   = 0x00000148,
    SPI_PERF_GS_NGG_STALL_MSG_VAL__NV23                = 0x00000123,
    SPI_PERF_SWC_PS_WR__NV23                           = 0x00000124,
    SPI_PERF_SWC_VS_WR__NV23                           = 0x00000125,
    SPI_PERF_GS_NGG_STALL_MSG_VAL__NV24                = 0x00000123,
    SPI_PERF_SWC_PS_WR__NV24                           = 0x00000124,
    SPI_PERF_SWC_VS_WR__NV24                           = 0x00000125,
    SPI_PERF_GS_GRP_LIFETIME_SAMPLE__NV33              = 0x00000012,
    SPI_PERF_GS_NGG_STALL_MSG_VAL__RAPHAEL             = 0x00000123,
    SPI_PERF_SWC_PS_WR__RAPHAEL                        = 0x00000124,
    SPI_PERF_SWC_VS_WR__RAPHAEL                        = 0x00000125,
    SPI_PERF_GS_NGG_STALL_MSG_VAL__REMBRANDT           = 0x00000123,
    SPI_PERF_SWC_PS_WR__REMBRANDT                      = 0x00000124,
    SPI_PERF_SWC_VS_WR__REMBRANDT                      = 0x00000125,
} SPI_PERFCNT_SEL;

constexpr unsigned int MaxSpiPerfcntSelGfx101          = SPI_PERF_LS_PERS_UPD_FULL1__GFX101;
constexpr unsigned int MaxSpiPerfcntSelGfx10Vrs        = SPI_PERF_EXP_THROT_CAUSALITY_DETECTED__GFX10VRS;
constexpr unsigned int MaxSpiPerfcntSelGfx11           = SPI_PERF_BUSY__GFX11;

typedef enum SPI_PNT_SPRITE_OVERRIDE {
    SPI_PNT_SPRITE_SEL_0                               = 0x00000000,
    SPI_PNT_SPRITE_SEL_1                               = 0x00000001,
    SPI_PNT_SPRITE_SEL_S                               = 0x00000002,
    SPI_PNT_SPRITE_SEL_T                               = 0x00000003,
    SPI_PNT_SPRITE_SEL_NONE                            = 0x00000004,
} SPI_PNT_SPRITE_OVERRIDE;

typedef enum SPI_PS_LDS_GROUP_SIZE {
    SPI_PS_LDS_GROUP_1                                 = 0x00000000,
    SPI_PS_LDS_GROUP_2                                 = 0x00000001,
    SPI_PS_LDS_GROUP_4                                 = 0x00000002,
} SPI_PS_LDS_GROUP_SIZE;

typedef enum SPI_SAMPLE_CNTL {
    CENTROIDS_ONLY                                     = 0x00000000,
    CENTERS_ONLY                                       = 0x00000001,
    CENTROIDS_AND_CENTERS                              = 0x00000002,
    UNDEF                                              = 0x00000003,
} SPI_SAMPLE_CNTL;

typedef enum SPI_SHADER_EX_FORMAT {
    SPI_SHADER_ZERO                                    = 0x00000000,
    SPI_SHADER_32_R                                    = 0x00000001,
    SPI_SHADER_32_GR                                   = 0x00000002,
    SPI_SHADER_32_AR                                   = 0x00000003,
    SPI_SHADER_FP16_ABGR                               = 0x00000004,
    SPI_SHADER_UNORM16_ABGR                            = 0x00000005,
    SPI_SHADER_SNORM16_ABGR                            = 0x00000006,
    SPI_SHADER_UINT16_ABGR                             = 0x00000007,
    SPI_SHADER_SINT16_ABGR                             = 0x00000008,
    SPI_SHADER_32_ABGR                                 = 0x00000009,
} SPI_SHADER_EX_FORMAT;

typedef enum SPI_SHADER_FORMAT {
    SPI_SHADER_NONE                                    = 0x00000000,
    SPI_SHADER_1COMP                                   = 0x00000001,
    SPI_SHADER_2COMP                                   = 0x00000002,
    SPI_SHADER_4COMPRESS                               = 0x00000003,
    SPI_SHADER_4COMP                                   = 0x00000004,
} SPI_SHADER_FORMAT;

typedef enum SPM_PERFMON_STATE {
    STRM_PERFMON_STATE_DISABLE_AND_RESET               = 0x00000000,
    STRM_PERFMON_STATE_START_COUNTING                  = 0x00000001,
    STRM_PERFMON_STATE_STOP_COUNTING                   = 0x00000002,
    STRM_PERFMON_STATE_RESERVED_3                      = 0x00000003,
    STRM_PERFMON_STATE_DISABLE_AND_RESET_PHANTOM       = 0x00000004,
    STRM_PERFMON_STATE_COUNT_AND_DUMP_PHANTOM          = 0x00000005,
} SPM_PERFMON_STATE;

typedef enum SQG_PERF_SEL {
    SQG_PERF_SEL_NONE                                  = 0x00000000,
    SQG_PERF_SEL_MSG_BUS_BUSY                          = 0x00000001,
    SQG_PERF_SEL_EXP_REQ0_BUS_BUSY                     = 0x00000002,
    SQG_PERF_SEL_EXP_REQ1_BUS_BUSY                     = 0x00000003,
    SQG_PERF_SEL_EXP_BUS0_BUSY                         = 0x00000004,
    SQG_PERF_SEL_EXP_BUS1_BUSY                         = 0x00000005,
    SQG_PERF_SEL_TTRACE_REQS                           = 0x00000006,
    SQG_PERF_SEL_TTRACE_INFLIGHT_REQS                  = 0x00000007,
    SQG_PERF_SEL_TTRACE_STALL                          = 0x00000008,
    SQG_PERF_SEL_TTRACE_LOST_PACKETS                   = 0x00000009,
    SQG_PERF_SEL_WAVES_INITIAL_PREFETCH                = 0x0000000a,
    SQG_PERF_SEL_EVENTS                                = 0x0000000b,
    SQG_PERF_SEL_WAVES_RESTORED                        = 0x0000000c,
    SQG_PERF_SEL_WAVES_SAVED                           = 0x0000000d,
    SQG_PERF_SEL_ACCUM_PREV                            = 0x0000000e,
    SQG_PERF_SEL_CYCLES                                = 0x0000000f,
    SQG_PERF_SEL_BUSY_CYCLES                           = 0x00000010,
    SQG_PERF_SEL_WAVE_CYCLES                           = 0x00000011,
    SQG_PERF_SEL_MSG                                   = 0x00000012,
    SQG_PERF_SEL_MSG_INTERRUPT                         = 0x00000013,
    SQG_PERF_SEL_WAVES                                 = 0x00000014,
    SQG_PERF_SEL_WAVES_32                              = 0x00000015,
    SQG_PERF_SEL_WAVES_64                              = 0x00000016,
    SQG_PERF_SEL_LEVEL_WAVES                           = 0x00000017,
    SQG_PERF_SEL_ITEMS                                 = 0x00000018,
    SQG_PERF_SEL_WAVE32_ITEMS                          = 0x00000019,
    SQG_PERF_SEL_WAVE64_ITEMS                          = 0x0000001a,
    SQG_PERF_SEL_PS_QUADS                              = 0x0000001b,
    SQG_PERF_SEL_WAVES_EQ_64                           = 0x0000001c,
    SQG_PERF_SEL_WAVES_EQ_32                           = 0x0000001d,
    SQG_PERF_SEL_WAVES_LT_64                           = 0x0000001e,
    SQG_PERF_SEL_WAVES_LT_48                           = 0x0000001f,
    SQG_PERF_SEL_WAVES_LT_32                           = 0x00000020,
    SQG_PERF_SEL_WAVES_LT_16                           = 0x00000021,
    SQG_PERF_SEL_WAVES_STARTED                         = 0x00000022,
    SQG_PERF_SEL_WAVES_ENDED                           = 0x00000023,
    SQG_PERF_SEL_DUMMY_LAST                            = 0x00000024,
} SQG_PERF_SEL;

constexpr unsigned int MaxSqgPerfSel                   = SQG_PERF_SEL_DUMMY_LAST;

typedef enum SQ_IMG_FILTER_TYPE {
    SQ_IMG_FILTER_MODE_BLEND                           = 0x00000000,
    SQ_IMG_FILTER_MODE_MIN                             = 0x00000001,
    SQ_IMG_FILTER_MODE_MAX                             = 0x00000002,
} SQ_IMG_FILTER_TYPE;

typedef enum SQ_LLC_CTL {
    SQ_LLC_0                                           = 0x00000000,
    SQ_LLC_1                                           = 0x00000001,
    SQ_LLC_RSVD_2                                      = 0x00000002,
    SQ_LLC_BYPASS                                      = 0x00000003,
} SQ_LLC_CTL;

typedef enum SQ_OOB_SELECT {
    SQ_OOB_INDEX_AND_OFFSET                            = 0x00000000,
    SQ_OOB_INDEX_ONLY                                  = 0x00000001,
    SQ_OOB_NUM_RECORDS_0                               = 0x00000002,
    SQ_OOB_COMPLETE                                    = 0x00000003,
} SQ_OOB_SELECT;

typedef enum SQ_PERF_SEL {
    SQ_PERF_SEL_NONE                                   = 0x00000000,
    SQ_PERF_SEL_ACCUM_PREV                             = 0x00000001,
    SQ_PERF_SEL_CYCLES                                 = 0x00000002,
    SQ_PERF_SEL_BUSY_CYCLES                            = 0x00000003,
    SQ_PERF_SEL_WAVES                                  = 0x00000004,
    SQ_PERF_SEL_WAVES_32                               = 0x00000005,
    SQ_PERF_SEL_WAVES_64                               = 0x00000006,
    SQ_PERF_SEL_LEVEL_WAVES                            = 0x00000007,
    SQ_PERF_SEL_ITEMS                                  = 0x00000008,
    SQ_PERF_SEL_WAVE32_ITEMS                           = 0x00000009,
    SQ_PERF_SEL_WAVE64_ITEMS                           = 0x0000000a,
    SQ_PERF_SEL_EVENTS                                 = 0x0000000c,
    SQ_PERF_SEL_MSG_INTERRUPT                          = 0x00000016,
    SQ_PERF_SEL_DUMMY_LAST                             = 0x000000ff,
    SQ_PERF_SEL_QUADS__GFX10                           = 0x0000000b,
    SQ_PERF_SEL_WAVES_EQ_64__GFX10                     = 0x0000000d,
    SQ_PERF_SEL_WAVES_LT_64__GFX10                     = 0x0000000e,
    SQ_PERF_SEL_WAVES_LT_48__GFX10                     = 0x0000000f,
    SQ_PERF_SEL_WAVES_LT_32__GFX10                     = 0x00000010,
    SQ_PERF_SEL_WAVES_LT_16__GFX10                     = 0x00000011,
    SQ_PERF_SEL_WAVES_RESTORED__GFX10                  = 0x00000012,
    SQ_PERF_SEL_WAVES_SAVED__GFX10                     = 0x00000013,
    SQ_PERF_SEL_MSG__GFX10                             = 0x00000014,
    SQ_PERF_SEL_MSG_GSCNT__GFX10                       = 0x00000015,
    SQ_PERF_SEL_Reserved_1__GFX10                      = 0x00000017,
    SQ_PERF_SEL_Reserved_2__GFX10                      = 0x00000018,
    SQ_PERF_SEL_Reserved_3__GFX10                      = 0x00000019,
    SQ_PERF_SEL_WAVE_CYCLES__GFX10                     = 0x0000001a,
    SQ_PERF_SEL_WAVE_READY__GFX10                      = 0x0000001b,
    SQ_PERF_SEL_WAIT_INST_ANY__GFX10                   = 0x0000001c,
    SQ_PERF_SEL_WAIT_INST_VALU__GFX10                  = 0x0000001d,
    SQ_PERF_SEL_WAIT_INST_SCA__GFX10                   = 0x0000001e,
    SQ_PERF_SEL_WAIT_INST_LDS__GFX10                   = 0x0000001f,
    SQ_PERF_SEL_WAIT_INST_TEX__GFX10                   = 0x00000020,
    SQ_PERF_SEL_WAIT_INST_FLAT__GFX10                  = 0x00000021,
    SQ_PERF_SEL_WAIT_INST_VMEM__GFX10                  = 0x00000022,
    SQ_PERF_SEL_WAIT_INST_EXP_GDS__GFX10               = 0x00000023,
    SQ_PERF_SEL_WAIT_INST_BR_MSG__GFX10                = 0x00000024,
    SQ_PERF_SEL_WAIT_ANY__GFX10                        = 0x00000025,
    SQ_PERF_SEL_WAIT_CNT_ANY__GFX10                    = 0x00000026,
    SQ_PERF_SEL_WAIT_CNT_VMVS__GFX10                   = 0x00000027,
    SQ_PERF_SEL_WAIT_CNT_LGKM__GFX10                   = 0x00000028,
    SQ_PERF_SEL_WAIT_CNT_EXP__GFX10                    = 0x00000029,
    SQ_PERF_SEL_WAIT_TTRACE__GFX10                     = 0x0000002a,
    SQ_PERF_SEL_WAIT_IFETCH__GFX10                     = 0x0000002b,
    SQ_PERF_SEL_WAIT_BARRIER__GFX10                    = 0x0000002c,
    SQ_PERF_SEL_WAIT_EXP_ALLOC__GFX10                  = 0x0000002d,
    SQ_PERF_SEL_WAIT_SLEEP__GFX10                      = 0x0000002e,
    SQ_PERF_SEL_WAIT_OTHER__GFX10                      = 0x00000030,
    SQ_PERF_SEL_INSTS_ALL__GFX10                       = 0x00000031,
    SQ_PERF_SEL_INSTS_BRANCH__GFX10                    = 0x00000032,
    SQ_PERF_SEL_INSTS_CBRANCH_NOT_TAKEN__GFX10         = 0x00000033,
    SQ_PERF_SEL_INSTS_CBRANCH_TAKEN__GFX10             = 0x00000034,
    SQ_PERF_SEL_INSTS_CBRANCH_TAKEN_HIT_IS__GFX10      = 0x00000035,
    SQ_PERF_SEL_INSTS_EXP_GDS__GFX10                   = 0x00000036,
    SQ_PERF_SEL_INSTS_GDS__GFX10                       = 0x00000037,
    SQ_PERF_SEL_INSTS_EXP__GFX10                       = 0x00000038,
    SQ_PERF_SEL_INSTS_FLAT__GFX10                      = 0x00000039,
    SQ_PERF_SEL_Reserved_4__GFX10                      = 0x0000003a,
    SQ_PERF_SEL_INSTS_LDS__GFX10                       = 0x0000003b,
    SQ_PERF_SEL_INSTS_SALU__GFX10                      = 0x0000003c,
    SQ_PERF_SEL_INSTS_SMEM__GFX10                      = 0x0000003d,
    SQ_PERF_SEL_INSTS_SMEM_NORM__GFX10                 = 0x0000003e,
    SQ_PERF_SEL_INSTS_SENDMSG__GFX10                   = 0x0000003f,
    SQ_PERF_SEL_INSTS_VALU__GFX10                      = 0x00000040,
    SQ_PERF_SEL_Reserved_17__GFX10                     = 0x00000041,
    SQ_PERF_SEL_INSTS_VALU_TRANS32__GFX10              = 0x00000042,
    SQ_PERF_SEL_INSTS_VALU_NO_COEXEC__GFX10            = 0x00000043,
    SQ_PERF_SEL_INSTS_TEX__GFX10                       = 0x00000044,
    SQ_PERF_SEL_INSTS_TEX_LOAD__GFX10                  = 0x00000045,
    SQ_PERF_SEL_INSTS_TEX_STORE__GFX10                 = 0x00000046,
    SQ_PERF_SEL_INSTS_WAVE32__GFX10                    = 0x00000047,
    SQ_PERF_SEL_INSTS_WAVE32_FLAT__GFX10               = 0x00000048,
    SQ_PERF_SEL_Reserved_5__GFX10                      = 0x00000049,
    SQ_PERF_SEL_INSTS_WAVE32_LDS__GFX10                = 0x0000004a,
    SQ_PERF_SEL_INSTS_WAVE32_VALU__GFX10               = 0x0000004b,
    SQ_PERF_SEL_INSTS_WAVE32_EXP_GDS__GFX10            = 0x0000004c,
    SQ_PERF_SEL_INSTS_WAVE32_VALU_TRANS32__GFX10       = 0x0000004d,
    SQ_PERF_SEL_INSTS_WAVE32_VALU_NO_COEXEC__GFX10     = 0x0000004e,
    SQ_PERF_SEL_INSTS_WAVE32_TEX__GFX10                = 0x0000004f,
    SQ_PERF_SEL_INSTS_WAVE32_TEX_LOAD__GFX10           = 0x00000050,
    SQ_PERF_SEL_INSTS_WAVE32_TEX_STORE__GFX10          = 0x00000051,
    SQ_PERF_SEL_ITEM_CYCLES_VALU__GFX10                = 0x00000052,
    SQ_PERF_SEL_VALU_READWRITELANE_CYCLES__GFX10       = 0x00000053,
    SQ_PERF_SEL_WAVE32_INSTS__GFX10                    = 0x00000054,
    SQ_PERF_SEL_WAVE64_INSTS__GFX10                    = 0x00000055,
    SQ_PERF_SEL_Reserved_18__GFX10                     = 0x00000056,
    SQ_PERF_SEL_INSTS_VALU_EXEC_SKIPPED__GFX10         = 0x00000057,
    SQ_PERF_SEL_WAVE64_HALF_SKIP__GFX10                = 0x00000058,
    SQ_PERF_SEL_Reserved_20__GFX10                     = 0x0000005b,
    SQ_PERF_SEL_INSTS_VALU_LDS_DIRECT_RD__GFX10        = 0x0000005f,
    SQ_PERF_SEL_INSTS_VALU_VINTRP_OP__GFX10            = 0x00000060,
    SQ_PERF_SEL_INST_LEVEL_EXP__GFX10                  = 0x00000061,
    SQ_PERF_SEL_INST_LEVEL_GDS__GFX10                  = 0x00000062,
    SQ_PERF_SEL_INST_LEVEL_LDS__GFX10                  = 0x00000063,
    SQ_PERF_SEL_INST_LEVEL_SMEM__GFX10                 = 0x00000064,
    SQ_PERF_SEL_INST_LEVEL_TEX_LOAD__GFX10             = 0x00000065,
    SQ_PERF_SEL_INST_LEVEL_TEX_STORE__GFX10            = 0x00000066,
    SQ_PERF_SEL_IFETCH_REQS__GFX10                     = 0x00000067,
    SQ_PERF_SEL_IFETCH_LEVEL__GFX10                    = 0x00000068,
    SQ_PERF_SEL_Reserved_6__GFX10                      = 0x0000006a,
    SQ_PERF_SEL_Reserved_7__GFX10                      = 0x0000006b,
    SQ_PERF_SEL_LDS_DIRECT_CMD_FIFO_FULL_STALL__GFX10  = 0x0000006c,
    SQ_PERF_SEL_VALU_SGATHER_STALL__GFX10              = 0x0000006d,
    SQ_PERF_SEL_VALU_FWD_BUFFER_FULL_STALL__GFX10      = 0x0000006e,
    SQ_PERF_SEL_VALU_SGPR_RD_FIFO_FULL_STALL__GFX10    = 0x0000006f,
    SQ_PERF_SEL_VALU_SGATHER_FULL_STALL__GFX10         = 0x00000070,
    SQ_PERF_SEL_SALU_SGATHER_STALL__GFX10              = 0x00000071,
    SQ_PERF_SEL_SALU_SGPR_RD_FIFO_FULL_STALL__GFX10    = 0x00000072,
    SQ_PERF_SEL_SALU_GATHER_FULL_STALL__GFX10          = 0x00000073,
    SQ_PERF_SEL_SMEM_DCACHE_FIFO_FULL_STALL__GFX10     = 0x00000074,
    SQ_PERF_SEL_INST_CYCLES_VALU__GFX10                = 0x00000075,
    SQ_PERF_SEL_INST_CYCLES_VALU_TRANS32__GFX10        = 0x00000076,
    SQ_PERF_SEL_INST_CYCLES_VALU_NO_COEXEC__GFX10      = 0x00000077,
    SQ_PERF_SEL_INST_CYCLES_VMEM__GFX10                = 0x00000078,
    SQ_PERF_SEL_INST_CYCLES_VMEM_LOAD__GFX10           = 0x00000079,
    SQ_PERF_SEL_INST_CYCLES_VMEM_STORE__GFX10          = 0x0000007a,
    SQ_PERF_SEL_INST_CYCLES_LDS__GFX10                 = 0x0000007b,
    SQ_PERF_SEL_INST_CYCLES_TEX__GFX10                 = 0x0000007c,
    SQ_PERF_SEL_INST_CYCLES_FLAT__GFX10                = 0x0000007d,
    SQ_PERF_SEL_INST_CYCLES_EXP_GDS__GFX10             = 0x0000007e,
    SQ_PERF_SEL_INST_CYCLES_EXP__GFX10                 = 0x0000007f,
    SQ_PERF_SEL_INST_CYCLES_GDS__GFX10                 = 0x00000080,
    SQ_PERF_SEL_VALU_STARVE__GFX10                     = 0x00000081,
    SQ_PERF_SEL_VMEM_ARB_FIFO_FULL__GFX10              = 0x00000082,
    SQ_PERF_SEL_MSG_FIFO_FULL_STALL__GFX10             = 0x00000083,
    SQ_PERF_SEL_EXP_REQ_FIFO_FULL__GFX10               = 0x00000084,
    SQ_PERF_SEL_Reserved_11__GFX10                     = 0x00000085,
    SQ_PERF_SEL_Reserved_12__GFX10                     = 0x00000086,
    SQ_PERF_SEL_Reserved_13__GFX10                     = 0x00000087,
    SQ_PERF_SEL_Reserved_14__GFX10                     = 0x00000088,
    SQ_PERF_SEL_VMEM_BUS_ACTIVE__GFX10                 = 0x00000089,
    SQ_PERF_SEL_VMEM_BUS_STALL__GFX10                  = 0x0000008a,
    SQ_PERF_SEL_VMEM_BUS_STALL_TA_ADDR_FIFO_FULL__GFX10 = 0x0000008b,
    SQ_PERF_SEL_VMEM_BUS_STALL_TA_CMD_FIFO_FULL__GFX10 = 0x0000008c,
    SQ_PERF_SEL_VMEM_BUS_STALL_LDS_ADDR_FIFO_FULL__GFX10 = 0x0000008d,
    SQ_PERF_SEL_VMEM_BUS_STALL_LDS_CMD_FIFO_FULL__GFX10 = 0x0000008e,
    SQ_PERF_SEL_VMEM_STARVE_TA_ADDR_EMPTY__GFX10       = 0x0000008f,
    SQ_PERF_SEL_VMEM_STARVE_LDS_ADDR_EMPTY__GFX10      = 0x00000090,
    SQ_PERF_SEL_Reserved_15__GFX10                     = 0x00000091,
    SQ_PERF_SEL_SALU_PIPE_STALL__GFX10                 = 0x00000092,
    SQ_PERF_SEL_SMEM_DCACHE_RETURN_CYCLES__GFX10       = 0x00000093,
    SQ_PERF_SEL_Reserved_21__GFX10                     = 0x00000094,
    SQ_PERF_SEL_MSG_BUS_BUSY__GFX10                    = 0x00000095,
    SQ_PERF_SEL_EXP_REQ_BUS_STALL__GFX10               = 0x00000096,
    SQ_PERF_SEL_EXP_REQ0_BUS_BUSY__GFX10               = 0x00000097,
    SQ_PERF_SEL_EXP_REQ1_BUS_BUSY__GFX10               = 0x00000098,
    SQ_PERF_SEL_EXP_BUS0_BUSY__GFX10                   = 0x00000099,
    SQ_PERF_SEL_EXP_BUS1_BUSY__GFX10                   = 0x0000009a,
    SQ_PERF_SEL_Reserved_19__GFX10                     = 0x0000009b,
    SQ_PERF_SEL_INST_CACHE_REQ_STALL__GFX10            = 0x0000009c,
    SQ_PERF_SEL_USER0__GFX10                           = 0x000000a0,
    SQ_PERF_SEL_USER1__GFX10                           = 0x000000a1,
    SQ_PERF_SEL_USER2__GFX10                           = 0x000000a2,
    SQ_PERF_SEL_USER3__GFX10                           = 0x000000a3,
    SQ_PERF_SEL_USER4__GFX10                           = 0x000000a4,
    SQ_PERF_SEL_USER5__GFX10                           = 0x000000a5,
    SQ_PERF_SEL_USER6__GFX10                           = 0x000000a6,
    SQ_PERF_SEL_USER7__GFX10                           = 0x000000a7,
    SQ_PERF_SEL_USER8__GFX10                           = 0x000000a8,
    SQ_PERF_SEL_USER9__GFX10                           = 0x000000a9,
    SQ_PERF_SEL_USER10__GFX10                          = 0x000000aa,
    SQ_PERF_SEL_USER11__GFX10                          = 0x000000ab,
    SQ_PERF_SEL_USER12__GFX10                          = 0x000000ac,
    SQ_PERF_SEL_USER13__GFX10                          = 0x000000ad,
    SQ_PERF_SEL_USER14__GFX10                          = 0x000000ae,
    SQ_PERF_SEL_USER15__GFX10                          = 0x000000af,
    SQ_PERF_SEL_USER_LEVEL0__GFX10                     = 0x000000b0,
    SQ_PERF_SEL_USER_LEVEL1__GFX10                     = 0x000000b1,
    SQ_PERF_SEL_USER_LEVEL2__GFX10                     = 0x000000b2,
    SQ_PERF_SEL_USER_LEVEL3__GFX10                     = 0x000000b3,
    SQ_PERF_SEL_USER_LEVEL4__GFX10                     = 0x000000b4,
    SQ_PERF_SEL_USER_LEVEL5__GFX10                     = 0x000000b5,
    SQ_PERF_SEL_USER_LEVEL6__GFX10                     = 0x000000b6,
    SQ_PERF_SEL_USER_LEVEL7__GFX10                     = 0x000000b7,
    SQ_PERF_SEL_USER_LEVEL8__GFX10                     = 0x000000b8,
    SQ_PERF_SEL_USER_LEVEL9__GFX10                     = 0x000000b9,
    SQ_PERF_SEL_USER_LEVEL10__GFX10                    = 0x000000ba,
    SQ_PERF_SEL_USER_LEVEL11__GFX10                    = 0x000000bb,
    SQ_PERF_SEL_USER_LEVEL12__GFX10                    = 0x000000bc,
    SQ_PERF_SEL_USER_LEVEL13__GFX10                    = 0x000000bd,
    SQ_PERF_SEL_USER_LEVEL14__GFX10                    = 0x000000be,
    SQ_PERF_SEL_USER_LEVEL15__GFX10                    = 0x000000bf,
    SQ_PERF_SEL_VALU_RETURN_SDST__GFX10                = 0x000000c0,
    SQ_PERF_SEL_DUMMY_END__GFX10                       = 0x000000c3,
    SQG_PERF_SEL_UTCL0_TRANSLATION_MISS__GFX10         = 0x00000100,
    SQG_PERF_SEL_UTCL0_PERMISSION_MISS__GFX10          = 0x00000101,
    SQG_PERF_SEL_UTCL0_TRANSLATION_HIT__GFX10          = 0x00000102,
    SQG_PERF_SEL_UTCL0_REQUEST__GFX10                  = 0x00000103,
    SQG_PERF_SEL_UTCL0_STALL_MISSFIFO_FULL__GFX10      = 0x00000104,
    SQG_PERF_SEL_UTCL0_STALL_INFLIGHT_MAX__GFX10       = 0x00000105,
    SQG_PERF_SEL_UTCL0_STALL_LRU_INFLIGHT__GFX10       = 0x00000106,
    SQG_PERF_SEL_UTCL0_LFIFO_FULL__GFX10               = 0x00000107,
    SQG_PERF_SEL_UTCL0_STALL_LFIFO_NOT_RES__GFX10      = 0x00000108,
    SQG_PERF_SEL_UTCL0_STALL_UTCL1_REQ_OUT_OF_CREDITS__GFX10 = 0x00000109,
    SQG_PERF_SEL_UTCL0_HIT_FIFO_FULL__GFX10            = 0x0000010a,
    SQG_PERF_SEL_UTCL0_UTCL1_REQ__GFX10                = 0x0000010b,
    SQG_PERF_SEL_TTRACE_REQS__GFX10                    = 0x0000010e,
    SQG_PERF_SEL_TTRACE_INFLIGHT_REQS__GFX10           = 0x0000010f,
    SQG_PERF_SEL_TTRACE_STALL__GFX10                   = 0x00000110,
    SQG_PERF_SEL_TTRACE_LOST_PACKETS__GFX10            = 0x00000111,
    SQG_PERF_SEL_DUMMY_LAST__GFX10                     = 0x00000112,
    SQC_PERF_SEL_LDS_BANK_CONFLICT__GFX10              = 0x0000011d,
    SQC_PERF_SEL_LDS_ADDR_CONFLICT__GFX10              = 0x0000011e,
    SQC_PERF_SEL_LDS_UNALIGNED_STALL__GFX10            = 0x0000011f,
    SQC_PERF_SEL_LDS_MEM_VIOLATIONS__GFX10             = 0x00000120,
    SQC_PERF_SEL_LDS_ATOMIC_RETURN__GFX10              = 0x00000121,
    SQC_PERF_SEL_LDS_IDX_ACTIVE__GFX10                 = 0x00000122,
    SQC_PERF_SEL_LDS_ADDR_STALL__GFX10                 = 0x00000123,
    SQC_PERF_SEL_LDS_ADDR_ACTIVE__GFX10                = 0x00000124,
    SQC_PERF_SEL_LDS_DIRECT_FIFO_FULL_STALL__GFX10     = 0x00000125,
    SQC_PERF_SEL_LDS_PC_LDS_WRITE_STALL_TD__GFX10      = 0x00000126,
    SQC_PERF_SEL_LDS_SPI_VGPR_WRITE_STALL_TD__GFX10    = 0x00000127,
    SQC_PERF_SEL_LDS_LDS_VGPR_WRITE_STALL__GFX10       = 0x00000128,
    SQC_PERF_SEL_LDS_FP_ADD_CYCLES__GFX10              = 0x00000129,
    SQC_PERF_SEL_ICACHE_BUSY_CYCLES__GFX10             = 0x0000012a,
    SQC_PERF_SEL_ICACHE_REQ__GFX10                     = 0x0000012b,
    SQC_PERF_SEL_ICACHE_HITS__GFX10                    = 0x0000012c,
    SQC_PERF_SEL_ICACHE_MISSES__GFX10                  = 0x0000012d,
    SQC_PERF_SEL_ICACHE_MISSES_DUPLICATE__GFX10        = 0x0000012e,
    SQC_PERF_SEL_ICACHE_INVAL_INST__GFX10              = 0x0000012f,
    SQC_PERF_SEL_ICACHE_INVAL_ASYNC__GFX10             = 0x00000130,
    SQC_PERF_SEL_ICACHE_INFLIGHT_LEVEL__GFX10          = 0x00000131,
    SQC_PERF_SEL_DCACHE_INFLIGHT_LEVEL__GFX10          = 0x00000132,
    SQC_PERF_SEL_TC_INFLIGHT_LEVEL__GFX10              = 0x00000133,
    SQC_PERF_SEL_ICACHE_TC_INFLIGHT_LEVEL__GFX10       = 0x00000134,
    SQC_PERF_SEL_DCACHE_TC_INFLIGHT_LEVEL__GFX10       = 0x00000135,
    SQC_PERF_SEL_ICACHE_INPUT_VALID_READY__GFX10       = 0x00000136,
    SQC_PERF_SEL_ICACHE_INPUT_VALID_READYB__GFX10      = 0x00000137,
    SQC_PERF_SEL_ICACHE_INPUT_VALIDB__GFX10            = 0x00000138,
    SQC_PERF_SEL_DCACHE_INPUT_VALID_READY__GFX10       = 0x00000139,
    SQC_PERF_SEL_DCACHE_INPUT_VALID_READYB__GFX10      = 0x0000013a,
    SQC_PERF_SEL_DCACHE_INPUT_VALIDB__GFX10            = 0x0000013b,
    SQC_PERF_SEL_TC_REQ__GFX10                         = 0x0000013c,
    SQC_PERF_SEL_TC_INST_REQ__GFX10                    = 0x0000013d,
    SQC_PERF_SEL_TC_DATA_READ_REQ__GFX10               = 0x0000013e,
    SQC_PERF_SEL_TC_STALL__GFX10                       = 0x00000141,
    SQC_PERF_SEL_TC_STARVE__GFX10                      = 0x00000142,
    SQC_PERF_SEL_ICACHE_INPUT_STALL_ARB_NO_GRANT__GFX10 = 0x00000143,
    SQC_PERF_SEL_ICACHE_INPUT_STALL_BANK_READYB__GFX10 = 0x00000144,
    SQC_PERF_SEL_ICACHE_CACHE_STALLED__GFX10           = 0x00000145,
    SQC_PERF_SEL_ICACHE_CACHE_STALL_INFLIGHT_NONZERO__GFX10 = 0x00000146,
    SQC_PERF_SEL_ICACHE_CACHE_STALL_INFLIGHT_MAX__GFX10 = 0x00000147,
    SQC_PERF_SEL_ICACHE_CACHE_STALL_OUTPUT__GFX10      = 0x00000148,
    SQC_PERF_SEL_ICACHE_CACHE_STALL_OUTPUT_MISS_FIFO__GFX10 = 0x00000149,
    SQC_PERF_SEL_ICACHE_CACHE_STALL_OUTPUT_HIT_FIFO__GFX10 = 0x0000014a,
    SQC_PERF_SEL_ICACHE_CACHE_STALL_OUTPUT_TC_IF__GFX10 = 0x0000014b,
    SQC_PERF_SEL_ICACHE_STALL_OUTXBAR_ARB_NO_GRANT__GFX10 = 0x0000014c,
    SQC_PERF_SEL_DCACHE_BUSY_CYCLES__GFX10             = 0x0000014d,
    SQC_PERF_SEL_DCACHE_REQ__GFX10                     = 0x0000014e,
    SQC_PERF_SEL_DCACHE_HITS__GFX10                    = 0x0000014f,
    SQC_PERF_SEL_DCACHE_MISSES__GFX10                  = 0x00000150,
    SQC_PERF_SEL_DCACHE_MISSES_DUPLICATE__GFX10        = 0x00000151,
    SQC_PERF_SEL_DCACHE_INVAL_INST__GFX10              = 0x00000152,
    SQC_PERF_SEL_DCACHE_INVAL_ASYNC__GFX10             = 0x00000153,
    SQC_PERF_SEL_DCACHE_HIT_LRU_READ__GFX10            = 0x00000154,
    SQC_PERF_SEL_DCACHE_INPUT_STALL_ARB_NO_GRANT__GFX10 = 0x0000015a,
    SQC_PERF_SEL_DCACHE_INPUT_STALL_BANK_READYB__GFX10 = 0x0000015b,
    SQC_PERF_SEL_DCACHE_CACHE_STALLED__GFX10           = 0x0000015c,
    SQC_PERF_SEL_DCACHE_CACHE_STALL_INFLIGHT_MAX__GFX10 = 0x0000015d,
    SQC_PERF_SEL_DCACHE_CACHE_STALL_OUTPUT__GFX10      = 0x0000015e,
    SQC_PERF_SEL_DCACHE_CACHE_STALL_EVICT__GFX10       = 0x0000015f,
    SQC_PERF_SEL_DCACHE_CACHE_STALL_UNORDERED__GFX10   = 0x00000160,
    SQC_PERF_SEL_DCACHE_CACHE_STALL_ALLOC_UNAVAILABLE__GFX10 = 0x00000161,
    SQC_PERF_SEL_DCACHE_CACHE_STALL_FORCE_EVICT__GFX10 = 0x00000162,
    SQC_PERF_SEL_DCACHE_CACHE_STALL_MULTI_FLUSH__GFX10 = 0x00000163,
    SQC_PERF_SEL_DCACHE_CACHE_STALL_FLUSH_DONE__GFX10  = 0x00000164,
    SQC_PERF_SEL_DCACHE_CACHE_STALL_OUTPUT_MISS_FIFO__GFX10 = 0x00000165,
    SQC_PERF_SEL_DCACHE_CACHE_STALL_OUTPUT_HIT_FIFO__GFX10 = 0x00000166,
    SQC_PERF_SEL_DCACHE_CACHE_STALL_OUTPUT_TC_IF__GFX10 = 0x00000167,
    SQC_PERF_SEL_DCACHE_STALL_OUTXBAR_ARB_NO_GRANT__GFX10 = 0x00000168,
    SQC_PERF_SEL_DCACHE_REQ_READ_1__GFX10              = 0x00000169,
    SQC_PERF_SEL_DCACHE_REQ_READ_2__GFX10              = 0x0000016a,
    SQC_PERF_SEL_DCACHE_REQ_READ_4__GFX10              = 0x0000016b,
    SQC_PERF_SEL_DCACHE_REQ_READ_8__GFX10              = 0x0000016c,
    SQC_PERF_SEL_DCACHE_REQ_READ_16__GFX10             = 0x0000016d,
    SQC_PERF_SEL_DCACHE_REQ_TIME__GFX10                = 0x0000016e,
    SP_PERF_SEL_DCACHE_WB_CNT__GFX10                   = 0x000001c0,
    SP_PERF_SEL_DCACHE_STALL__GFX10                    = 0x000001c1,
    SP_PERF_SEL_DCACHE_HIT__GFX10                      = 0x000001c2,
    SP_PERF_SEL_DCACHE_HIT_DIRTY__GFX10                = 0x000001c3,
    SP_PERF_SEL_DCACHE_WB_CONF_W_LDS__GFX10            = 0x000001c4,
    SP_PERF_SEL_DCACHE_WB_CONF_W_TD__GFX10             = 0x000001c5,
    SP_PERF_SEL_DCACHE_WB_CONF_W_SPI__GFX10            = 0x000001c6,
    SP_PERF_SEL_DCACHE_WB_CONF_W_EXP_VMEM__GFX10       = 0x000001c7,
    SP_PERF_SEL_DCACHE_EVEN_DIRTY_CNT__GFX10           = 0x000001c8,
    SP_PERF_SEL_DCACHE_ODD_DIRTY_CNT__GFX10            = 0x000001c9,
    SP_PERF_SEL_DCACHE_INVALIDATE_BY_VEC__GFX10        = 0x000001ca,
    SP_PERF_SEL_GATHER_SRCA_FWD__GFX10                 = 0x000001cb,
    SP_PERF_SEL_GATHER_SRCB_FWD__GFX10                 = 0x000001cc,
    SP_PERF_SEL_GATHER_SRCC_FWD__GFX10                 = 0x000001cd,
    SP_PERF_SEL_GATHER_SRCA_REDUNDANT__GFX10           = 0x000001ce,
    SP_PERF_SEL_GATHER_SRCB_REDUNDANT__GFX10           = 0x000001cf,
    SP_PERF_SEL_GATHER_SRCC_REDUNDANT__GFX10           = 0x000001d0,
    SP_PERF_SEL_GATHER_SRCA_DIRECT__GFX10              = 0x000001d1,
    SP_PERF_SEL_GATHER_SRCB_DIRECT__GFX10              = 0x000001d2,
    SP_PERF_SEL_GATHER_SRCC_DIRECT__GFX10              = 0x000001d3,
    SP_PERF_SEL_VGPR_VALU_BANK0_RD__GFX10              = 0x000001d4,
    SP_PERF_SEL_VGPR_VALU_BANK1_RD__GFX10              = 0x000001d5,
    SP_PERF_SEL_VGPR_VALU_BANK2_RD__GFX10              = 0x000001d6,
    SP_PERF_SEL_VGPR_VALU_BANK3_RD__GFX10              = 0x000001d7,
    SP_PERF_SEL_VGPR_VMEM_BANK_RD__GFX10               = 0x000001d8,
    SP_PERF_SEL_VGPR_EXP_BANK_RD__GFX10                = 0x000001d9,
    SP_PERF_SEL_VGPR_TDDATA_WR__GFX10                  = 0x000001da,
    SP_PERF_SEL_VGPR_LDSDATA_WR__GFX10                 = 0x000001db,
    SP_PERF_SEL_PB_STALL__GFX10                        = 0x000001dc,
    SP_PERF_SEL_PB_SEND__GFX10                         = 0x000001dd,
    SP_PERF_SEL_LDS_DIRECT_FIFO_FULL__GFX10            = 0x000001de,
    SP_PERF_SEL_LDS_DIRECT_FIFO_SEND__GFX10            = 0x000001df,
    SP_PERF_SEL_SQ_SP_CONST_FIFO_FULL__GFX10           = 0x000001e0,
    SP_PERF_SEL_SQ_SP_CONST_FIFO_SEND__GFX10           = 0x000001e1,
    SP_PERF_SEL_SGPR_FULL__GFX10                       = 0x000001e2,
    SP_PERF_SEL_VALU_CONFLICT_WITH_EXP__GFX10          = 0x000001e3,
    SP_PERF_SEL_VALU_CONFLICT_WITH_VMEM_LDS__GFX10     = 0x000001e4,
    SP_PERF_SEL_VALU_CONFLICT_WITH_VMEM_NON_LDS__GFX10 = 0x000001e5,
    SP_PERF_SEL_VALU_STALL_DUE_CONST_NOT_READY__GFX10  = 0x000001e6,
    SP_PERF_SEL_VALU_STALL_DUE_VDST_FWD__GFX10         = 0x000001e7,
    SP_PERF_SEL_VALU_STALL_DUE_SDST_FWD__GFX10         = 0x000001e8,
    SP_PERF_SEL_VALU_STALL_DUE_LDS_DIR_NOT_READY__GFX10 = 0x000001e9,
    SP_PERF_SEL_VALU_STALL_DUE_VGPR_NOT_READY__GFX10   = 0x000001ea,
    SP_PERF_SEL_VALU_STALL_DUE_DST_STALL__GFX10        = 0x000001eb,
    SP_PERF_SEL_VALU_STALL_DUE_DST_CACHE_WRITE_CONF__GFX10 = 0x000001ec,
    SP_PERF_SEL_VALU_STALL__GFX10                      = 0x000001ed,
    SP_PERF_SEL_PERF_MEM_RD_CNT__GFX10                 = 0x000001ee,
    SP_PERF_SEL_PERF_MEM_WR_CNT__GFX10                 = 0x000001ef,
    SP_PERF_SEL_PERF_THREAD_REDUNDANT__GFX10           = 0x000001f0,
    SP_PERF_SEL_PERF_OP_W_1_PASS__GFX10                = 0x000001f1,
    SP_PERF_SEL_PERF_OP_W_2_PASS__GFX10                = 0x000001f2,
    SP_PERF_SEL_PERF_OP_W_4_PASS__GFX10                = 0x000001f3,
    SP_PERF_SEL_PERF_OP_W_16_PASS__GFX10               = 0x000001f4,
    SP_PERF_SEL_PERF_COEXEC__GFX10                     = 0x000001f5,
    SP_PERF_SEL_PERF_ACTIVE_THREAD__GFX10              = 0x000001f6,
    SP_PERF_SEL_PERF_ALL_ACTIVE__GFX10                 = 0x000001f7,
    SP_PERF_SEL_PERF_ZERO_P_ZERO__GFX10                = 0x000001f8,
    SP_PERF_SEL_PERF_TRANS_OP__GFX10                   = 0x000001f9,
    SP_PERF_SEL_PERF_OP_W_MAD__GFX10                   = 0x000001fa,
    SP_PERF_SEL_PERF_OP_W_MUL__GFX10                   = 0x000001fb,
    SP_PERF_SEL_PERF_OP_W_ADD__GFX10                   = 0x000001fc,
    SP_PERF_SEL_DUMMY_LAST__GFX10                      = 0x000001ff,
    SQ_PERF_SEL_WAIT_SLEEP_XNACK__GFX101               = 0x0000002f,
    SQ_PERF_SEL_INSTS_TEX_REPLAY__GFX101               = 0x00000059,
    SQ_PERF_SEL_INSTS_SMEM_REPLAY__GFX101              = 0x0000005a,
    SQ_PERF_SEL_INSTS_FLAT_REPLAY__GFX101              = 0x0000005c,
    SQ_PERF_SEL_TA_XNACK_ALL__GFX101                   = 0x0000005d,
    SQ_PERF_SEL_TA_XNACK_FIRST__GFX101                 = 0x0000005e,
    SQ_PERF_SEL_IFETCH_XNACK__GFX101                   = 0x00000069,
    SQ_PERF_SEL_MIXED_SUBSEQUENT_ISSUES_VALU__GFX101   = 0x0000009d,
    SQ_PERF_SEL_MIXED_SUBSEQUENT_ISSUES_SALU__GFX101   = 0x0000009e,
    SQ_PERF_SEL_MIXED_SUBSEQUENT_ISSUES_VMEM__GFX101   = 0x0000009f,
    SQ_PERF_SEL_VMEM_SECOND_TRY_USED__GFX101           = 0x000000c1,
    SQ_PERF_SEL_VMEM_SECOND_TRY_STALL__GFX101          = 0x000000c2,
    SQG_PERF_SEL_TLB_SHOOTDOWN__GFX101                 = 0x0000010c,
    SQG_PERF_SEL_TLB_SHOOTDOWN_CYCLES__GFX101          = 0x0000010d,
    SQC_PERF_SEL_POWER_VALU__GFX101                    = 0x00000113,
    SQC_PERF_SEL_POWER_VALU0__GFX101                   = 0x00000114,
    SQC_PERF_SEL_POWER_VALU1__GFX101                   = 0x00000115,
    SQC_PERF_SEL_POWER_VALU2__GFX101                   = 0x00000116,
    SQC_PERF_SEL_POWER_GPR_RD__GFX101                  = 0x00000117,
    SQC_PERF_SEL_POWER_GPR_WR__GFX101                  = 0x00000118,
    SQC_PERF_SEL_POWER_LDS_BUSY__GFX101                = 0x00000119,
    SQC_PERF_SEL_POWER_ALU_BUSY__GFX101                = 0x0000011a,
    SQC_PERF_SEL_POWER_TEX_BUSY__GFX101                = 0x0000011b,
    SQC_PERF_SEL_PT_POWER_STALL__GFX101                = 0x0000011c,
    SQC_PERF_SEL_TC_DATA_WRITE_REQ__GFX101             = 0x0000013f,
    SQC_PERF_SEL_TC_DATA_ATOMIC_REQ__GFX101            = 0x00000140,
    SQC_PERF_SEL_DCACHE_WC_LRU_WRITE__GFX101           = 0x00000155,
    SQC_PERF_SEL_DCACHE_WT_EVICT_WRITE__GFX101         = 0x00000156,
    SQC_PERF_SEL_DCACHE_ATOMIC__GFX101                 = 0x00000157,
    SQC_PERF_SEL_DCACHE_WB_INST__GFX101                = 0x00000158,
    SQC_PERF_SEL_DCACHE_WB_ASYNC__GFX101               = 0x00000159,
    SQC_PERF_SEL_DCACHE_REQ_WRITE_1__GFX101            = 0x0000016f,
    SQC_PERF_SEL_DCACHE_REQ_WRITE_2__GFX101            = 0x00000170,
    SQC_PERF_SEL_DCACHE_REQ_WRITE_4__GFX101            = 0x00000171,
    SQC_PERF_SEL_DCACHE_REQ_ATC_PROBE__GFX101          = 0x00000172,
    SQC_PERF_SEL_SQ_DCACHE_REQS__GFX101                = 0x00000173,
    SQC_PERF_SEL_DCACHE_FLAT_REQ__GFX101               = 0x00000174,
    SQC_PERF_SEL_DCACHE_NONFLAT_REQ__GFX101            = 0x00000175,
    SQC_PERF_SEL_ICACHE_UTCL0_TRANSLATION_MISS__GFX101 = 0x00000176,
    SQC_PERF_SEL_ICACHE_UTCL0_PERMISSION_MISS__GFX101  = 0x00000177,
    SQC_PERF_SEL_ICACHE_UTCL0_TRANSLATION_HIT__GFX101  = 0x00000178,
    SQC_PERF_SEL_ICACHE_UTCL0_REQUEST__GFX101          = 0x00000179,
    SQC_PERF_SEL_ICACHE_UTCL0_XNACK__GFX101            = 0x0000017a,
    SQC_PERF_SEL_ICACHE_UTCL0_STALL_INFLIGHT_MAX__GFX101 = 0x0000017b,
    SQC_PERF_SEL_ICACHE_UTCL0_STALL_LRU_INFLIGHT__GFX101 = 0x0000017c,
    SQC_PERF_SEL_ICACHE_UTCL0_LFIFO_FULL__GFX101       = 0x0000017d,
    SQC_PERF_SEL_ICACHE_UTCL0_STALL_LFIFO_NOT_RES__GFX101 = 0x0000017e,
    SQC_PERF_SEL_ICACHE_UTCL0_STALL_UTCL1_REQ_OUT_OF_CREDITS__GFX101 = 0x0000017f,
    SQC_PERF_SEL_ICACHE_UTCL0_UTCL1_INFLIGHT__GFX101   = 0x00000180,
    SQC_PERF_SEL_ICACHE_UTCL0_STALL_MISSFIFO_FULL__GFX101 = 0x00000181,
    SQC_PERF_SEL_DCACHE_UTCL0_TRANSLATION_MISS__GFX101 = 0x00000182,
    SQC_PERF_SEL_DCACHE_UTCL0_PERMISSION_MISS__GFX101  = 0x00000183,
    SQC_PERF_SEL_DCACHE_UTCL0_TRANSLATION_HIT__GFX101  = 0x00000184,
    SQC_PERF_SEL_DCACHE_UTCL0_REQUEST__GFX101          = 0x00000185,
    SQC_PERF_SEL_DCACHE_UTCL0_XNACK__GFX101            = 0x00000186,
    SQC_PERF_SEL_DCACHE_UTCL0_STALL_INFLIGHT_MAX__GFX101 = 0x00000187,
    SQC_PERF_SEL_DCACHE_UTCL0_STALL_LRU_INFLIGHT__GFX101 = 0x00000188,
    SQC_PERF_SEL_DCACHE_UTCL0_LFIFO_FULL__GFX101       = 0x00000189,
    SQC_PERF_SEL_DCACHE_UTCL0_STALL_LFIFO_NOT_RES__GFX101 = 0x0000018a,
    SQC_PERF_SEL_DCACHE_UTCL0_STALL_UTCL1_REQ_OUT_OF_CREDITS__GFX101 = 0x0000018b,
    SQC_PERF_SEL_DCACHE_UTCL0_UTCL1_INFLIGHT__GFX101   = 0x0000018c,
    SQC_PERF_SEL_DCACHE_UTCL0_STALL_MISSFIFO_FULL__GFX101 = 0x0000018d,
    SQC_PERF_SEL_DCACHE_UTCL0_STALL_MULTI_MISS__GFX101 = 0x0000018e,
    SQC_PERF_SEL_DCACHE_UTCL0_HIT_FIFO_FULL__GFX101    = 0x0000018f,
    SQC_PERF_SEL_ICACHE_UTCL0_INFLIGHT_LEVEL__GFX101   = 0x00000190,
    SQC_PERF_SEL_ICACHE_UTCL0_ALL_REQ__GFX101          = 0x00000191,
    SQC_PERF_SEL_ICACHE_UTCL1_INFLIGHT_LEVEL__GFX101   = 0x00000192,
    SQC_PERF_SEL_ICACHE_UTCL1_ALL_REQ__GFX101          = 0x00000193,
    SQC_PERF_SEL_ICACHE_UTCL0_UTCL1_PERM_FAULT__GFX101 = 0x00000194,
    SQC_PERF_SEL_DCACHE_UTCL0_INFLIGHT_LEVEL__GFX101   = 0x00000195,
    SQC_PERF_SEL_DCACHE_UTCL0_ALL_REQ__GFX101          = 0x00000196,
    SQC_PERF_SEL_DCACHE_UTCL1_INFLIGHT_LEVEL__GFX101   = 0x00000197,
    SQC_PERF_SEL_DCACHE_UTCL1_ALL_REQ__GFX101          = 0x00000198,
    SQC_PERF_SEL_DCACHE_UTCL0_UTCL1_PERM_FAULT__GFX101 = 0x00000199,
    SQC_PERF_SEL_ICACHE_GCR__GFX101                    = 0x0000019a,
    SQC_PERF_SEL_ICACHE_GCR_HITS__GFX101               = 0x0000019b,
    SQC_PERF_SEL_DCACHE_GCR__GFX101                    = 0x0000019c,
    SQC_PERF_SEL_DCACHE_GCR_HITS__GFX101               = 0x0000019d,
    SQC_PERF_SEL_ICACHE_GCR_INVALIDATE__GFX101         = 0x0000019e,
    SQC_PERF_SEL_DCACHE_GCR_INVALIDATE__GFX101         = 0x0000019f,
    SQC_PERF_SEL_DCACHE_GCR_WRITEBACK__GFX101          = 0x000001a0,
    SQC_PERF_SEL_DCACHE_SPI_RETURN_STALL__GFX101       = 0x000001a1,
    SQC_PERF_SEL_DUMMY_LAST__GFX101                    = 0x000001a2,
    SQ_PERF_SEL_Reserved_0x02f__GFX10VRS               = 0x0000002f,
    SQ_PERF_SEL_Reserved_0x059__GFX10VRS               = 0x00000059,
    SQ_PERF_SEL_Reserved_0x05a__GFX10VRS               = 0x0000005a,
    SQ_PERF_SEL_Reserved_0x05c__GFX10VRS               = 0x0000005c,
    SQ_PERF_SEL_Reserved_0x05d__GFX10VRS               = 0x0000005d,
    SQ_PERF_SEL_Reserved_0x05e__GFX10VRS               = 0x0000005e,
    SQ_PERF_SEL_Reserved_0x069__GFX10VRS               = 0x00000069,
    SQ_PERF_SEL_Reserved_22__GFX10VRS                  = 0x0000009d,
    SQ_PERF_SEL_Reserved_23__GFX10VRS                  = 0x0000009e,
    SQ_PERF_SEL_Reserved_24__GFX10VRS                  = 0x0000009f,
    SQ_PERF_SEL_VMEM_VGPR_READ_STALLED_BY_EXPORT__GFX10VRS = 0x000000c1,
    SQ_PERF_SEL_INSTS_VALU_TRANS__GFX10VRS             = 0x000000c2,
    SQG_PERF_SEL_Reserved_0__GFX10VRS                  = 0x0000010c,
    SQG_PERF_SEL_TTRACE_LOST_PACKETS_NO_CH_CREDITS__GFX10VRS = 0x0000010d,
    SQC_PERF_SEL_POWER_Reserved_0__GFX10VRS            = 0x00000113,
    SQC_PERF_SEL_POWER_Reserved_1__GFX10VRS            = 0x00000114,
    SQC_PERF_SEL_POWER_Reserved_2__GFX10VRS            = 0x00000115,
    SQC_PERF_SEL_POWER_Reserved_3__GFX10VRS            = 0x00000116,
    SQC_PERF_SEL_POWER_Reserved_4__GFX10VRS            = 0x00000117,
    SQC_PERF_SEL_POWER_Reserved_5__GFX10VRS            = 0x00000118,
    SQC_PERF_SEL_POWER_Reserved_6__GFX10VRS            = 0x00000119,
    SQC_PERF_SEL_POWER_Reserved_7__GFX10VRS            = 0x0000011a,
    SQC_PERF_SEL_POWER_Reserved_8__GFX10VRS            = 0x0000011b,
    SQC_PERF_SEL_POWER_Reserved_9__GFX10VRS            = 0x0000011c,
    SQC_PERF_SEL_Reserved_0x13f__GFX10VRS              = 0x0000013f,
    SQC_PERF_SEL_Reserved_0x140__GFX10VRS              = 0x00000140,
    SQC_PERF_SEL_Reserved_0x155__GFX10VRS              = 0x00000155,
    SQC_PERF_SEL_Reserved_0x156__GFX10VRS              = 0x00000156,
    SQC_PERF_SEL_Reserved_0x157__GFX10VRS              = 0x00000157,
    SQC_PERF_SEL_Reserved_0x158__GFX10VRS              = 0x00000158,
    SQC_PERF_SEL_Reserved_0x159__GFX10VRS              = 0x00000159,
    SQC_PERF_SEL_Reserved_0x16e__GFX10VRS              = 0x0000016f,
    SQC_PERF_SEL_Reserved_0x16f__GFX10VRS              = 0x00000170,
    SQC_PERF_SEL_Reserved_0x170__GFX10VRS              = 0x00000171,
    SQC_PERF_SEL_Reserved_0x171__GFX10VRS              = 0x00000172,
    SQC_PERF_SEL_DCACHE_REQ_ATC_PROBE__GFX10VRS        = 0x00000173,
    SQC_PERF_SEL_SQ_DCACHE_REQS__GFX10VRS              = 0x00000174,
    SQC_PERF_SEL_DCACHE_FLAT_REQ__GFX10VRS             = 0x00000175,
    SQC_PERF_SEL_DCACHE_NONFLAT_REQ__GFX10VRS          = 0x00000176,
    SQC_PERF_SEL_Reserved_0x176__GFX10VRS              = 0x00000177,
    SQC_PERF_SEL_Reserved_0x177__GFX10VRS              = 0x00000178,
    SQC_PERF_SEL_Reserved_0x178__GFX10VRS              = 0x00000179,
    SQC_PERF_SEL_Reserved_0x179__GFX10VRS              = 0x0000017a,
    SQC_PERF_SEL_Reserved_0x17a__GFX10VRS              = 0x0000017b,
    SQC_PERF_SEL_Reserved_0x17b__GFX10VRS              = 0x0000017c,
    SQC_PERF_SEL_Reserved_0x17c__GFX10VRS              = 0x0000017d,
    SQC_PERF_SEL_Reserved_0x17d__GFX10VRS              = 0x0000017e,
    SQC_PERF_SEL_Reserved_0x17e__GFX10VRS              = 0x0000017f,
    SQC_PERF_SEL_Reserved_0x17f__GFX10VRS              = 0x00000180,
    SQC_PERF_SEL_Reserved_0x180__GFX10VRS              = 0x00000181,
    SQC_PERF_SEL_Reserved_0x181__GFX10VRS              = 0x00000182,
    SQC_PERF_SEL_Reserved_0x182__GFX10VRS              = 0x00000183,
    SQC_PERF_SEL_Reserved_0x183__GFX10VRS              = 0x00000184,
    SQC_PERF_SEL_Reserved_0x184__GFX10VRS              = 0x00000185,
    SQC_PERF_SEL_Reserved_0x185__GFX10VRS              = 0x00000186,
    SQC_PERF_SEL_Reserved_0x186__GFX10VRS              = 0x00000187,
    SQC_PERF_SEL_Reserved_0x187__GFX10VRS              = 0x00000188,
    SQC_PERF_SEL_Reserved_0x188__GFX10VRS              = 0x00000189,
    SQC_PERF_SEL_Reserved_0x189__GFX10VRS              = 0x0000018a,
    SQC_PERF_SEL_Reserved_0x18a__GFX10VRS              = 0x0000018b,
    SQC_PERF_SEL_Reserved_0x18b__GFX10VRS              = 0x0000018c,
    SQC_PERF_SEL_Reserved_0x18c__GFX10VRS              = 0x0000018d,
    SQC_PERF_SEL_Reserved_0x18d__GFX10VRS              = 0x0000018e,
    SQC_PERF_SEL_Reserved_0x18e__GFX10VRS              = 0x0000018f,
    SQC_PERF_SEL_Reserved_0x18f__GFX10VRS              = 0x00000190,
    SQC_PERF_SEL_Reserved_0x190__GFX10VRS              = 0x00000191,
    SQC_PERF_SEL_Reserved_0x191__GFX10VRS              = 0x00000192,
    SQC_PERF_SEL_Reserved_0x192__GFX10VRS              = 0x00000193,
    SQC_PERF_SEL_Reserved_0x193__GFX10VRS              = 0x00000194,
    SQC_PERF_SEL_Reserved_0x194__GFX10VRS              = 0x00000195,
    SQC_PERF_SEL_Reserved_0x195__GFX10VRS              = 0x00000196,
    SQC_PERF_SEL_Reserved_0x196__GFX10VRS              = 0x00000197,
    SQC_PERF_SEL_Reserved_0x197__GFX10VRS              = 0x00000198,
    SQC_PERF_SEL_Reserved_0x198__GFX10VRS              = 0x00000199,
    SQC_PERF_SEL_Reserved_0x199__GFX10VRS              = 0x0000019a,
    SQC_PERF_SEL_ICACHE_GCR__GFX10VRS                  = 0x0000019b,
    SQC_PERF_SEL_ICACHE_GCR_HITS__GFX10VRS             = 0x0000019c,
    SQC_PERF_SEL_DCACHE_GCR__GFX10VRS                  = 0x0000019d,
    SQC_PERF_SEL_DCACHE_GCR_HITS__GFX10VRS             = 0x0000019e,
    SQC_PERF_SEL_ICACHE_GCR_INVALIDATE__GFX10VRS       = 0x0000019f,
    SQC_PERF_SEL_DCACHE_GCR_INVALIDATE__GFX10VRS       = 0x000001a0,
    SQC_PERF_SEL_Reserved_0x1a0__GFX10VRS              = 0x000001a1,
    SQC_PERF_SEL_DCACHE_SPI_RETURN_STALL__GFX10VRS     = 0x000001a2,
    SQC_PERF_SEL_DUMMY_LAST__GFX10VRS                  = 0x000001a3,
    SQ_PERF_SEL_PS_QUADS__GFX11                        = 0x0000000b,
    SQ_PERF_SEL_WAVES_EQ_32__GFX11                     = 0x0000000d,
    SQ_PERF_SEL_WAVES_EQ_64__GFX11                     = 0x0000000e,
    SQ_PERF_SEL_WAVES_LT_64__GFX11                     = 0x0000000f,
    SQ_PERF_SEL_WAVES_LT_48__GFX11                     = 0x00000010,
    SQ_PERF_SEL_WAVES_LT_32__GFX11                     = 0x00000011,
    SQ_PERF_SEL_WAVES_LT_16__GFX11                     = 0x00000012,
    SQ_PERF_SEL_WAVES_RESTORED__GFX11                  = 0x00000013,
    SQ_PERF_SEL_WAVES_SAVED__GFX11                     = 0x00000014,
    SQ_PERF_SEL_MSG__GFX11                             = 0x00000015,
    SQ_PERF_SEL_WAVES_INITIAL_PREFETCH__GFX11          = 0x00000017,
    SQ_PERF_SEL_WAVE_CYCLES__GFX11                     = 0x00000018,
    SQ_PERF_SEL_WAVE_READY__GFX11                      = 0x00000019,
    SQ_PERF_SEL_WAIT_INST_ANY__GFX11                   = 0x0000001a,
    SQ_PERF_SEL_WAIT_INST_VALU__GFX11                  = 0x0000001b,
    SQ_PERF_SEL_WAIT_INST_SCA__GFX11                   = 0x0000001c,
    SQ_PERF_SEL_WAIT_INST_LDS__GFX11                   = 0x0000001d,
    SQ_PERF_SEL_WAIT_INST_TEX__GFX11                   = 0x0000001e,
    SQ_PERF_SEL_WAIT_INST_FLAT__GFX11                  = 0x0000001f,
    SQ_PERF_SEL_WAIT_INST_VMEM__GFX11                  = 0x00000020,
    SQ_PERF_SEL_WAIT_INST_EXP_GDS__GFX11               = 0x00000021,
    SQ_PERF_SEL_WAIT_INST_BR_MSG__GFX11                = 0x00000022,
    SQ_PERF_SEL_WAIT_ANY__GFX11                        = 0x00000023,
    SQ_PERF_SEL_WAIT_CNT_ANY__GFX11                    = 0x00000024,
    SQ_PERF_SEL_WAIT_CNT_VMVS__GFX11                   = 0x00000025,
    SQ_PERF_SEL_WAIT_CNT_LGKM__GFX11                   = 0x00000026,
    SQ_PERF_SEL_WAIT_CNT_EXP__GFX11                    = 0x00000027,
    SQ_PERF_SEL_WAIT_TTRACE__GFX11                     = 0x00000028,
    SQ_PERF_SEL_WAIT_IFETCH__GFX11                     = 0x00000029,
    SQ_PERF_SEL_WAIT_BARRIER__GFX11                    = 0x0000002a,
    SQ_PERF_SEL_WAIT_EXP_ALLOC__GFX11                  = 0x0000002b,
    SQ_PERF_SEL_WAIT_SLEEP__GFX11                      = 0x0000002c,
    SQ_PERF_SEL_WAIT_DELAY_ALU__GFX11                  = 0x0000002d,
    SQ_PERF_SEL_WAIT_DEPCTR__GFX11                     = 0x0000002e,
    SQ_PERF_SEL_WAIT_OTHER__GFX11                      = 0x0000002f,
    SQ_PERF_SEL_INSTS_ALL__GFX11                       = 0x00000030,
    SQ_PERF_SEL_INSTS_BRANCH__GFX11                    = 0x00000031,
    SQ_PERF_SEL_INSTS_CBRANCH_NOT_TAKEN__GFX11         = 0x00000032,
    SQ_PERF_SEL_INSTS_CBRANCH_TAKEN__GFX11             = 0x00000033,
    SQ_PERF_SEL_INSTS_CBRANCH_TAKEN_HIT_IS__GFX11      = 0x00000034,
    SQ_PERF_SEL_INSTS_EXP_GDS__GFX11                   = 0x00000035,
    SQ_PERF_SEL_INSTS_GDS__GFX11                       = 0x00000036,
    SQ_PERF_SEL_INSTS_EXP__GFX11                       = 0x00000037,
    SQ_PERF_SEL_INSTS_FLAT__GFX11                      = 0x00000038,
    SQ_PERF_SEL_INSTS_LDS__GFX11                       = 0x00000039,
    SQ_PERF_SEL_INSTS_SALU__GFX11                      = 0x0000003a,
    SQ_PERF_SEL_INSTS_SMEM__GFX11                      = 0x0000003b,
    SQ_PERF_SEL_INSTS_SMEM_NORM__GFX11                 = 0x0000003c,
    SQ_PERF_SEL_INSTS_SENDMSG__GFX11                   = 0x0000003d,
    SQ_PERF_SEL_INSTS_VALU__GFX11                      = 0x0000003e,
    SQ_PERF_SEL_INSTS_VALU_TRANS32__GFX11              = 0x0000003f,
    SQ_PERF_SEL_INSTS_VALU_NO_COEXEC__GFX11            = 0x00000040,
    SQ_PERF_SEL_INSTS_TEX__GFX11                       = 0x00000041,
    SQ_PERF_SEL_INSTS_TEX_LOAD__GFX11                  = 0x00000042,
    SQ_PERF_SEL_INSTS_TEX_STORE__GFX11                 = 0x00000043,
    SQ_PERF_SEL_INSTS_DELAY_ALU__GFX11                 = 0x00000044,
    SQ_PERF_SEL_INSTS_INTERNAL__GFX11                  = 0x00000045,
    SQ_PERF_SEL_INSTS_WAVE32__GFX11                    = 0x00000046,
    SQ_PERF_SEL_INSTS_WAVE32_FLAT__GFX11               = 0x00000047,
    SQ_PERF_SEL_INSTS_WAVE32_LDS__GFX11                = 0x00000048,
    SQ_PERF_SEL_INSTS_WAVE32_VALU__GFX11               = 0x00000049,
    SQ_PERF_SEL_WAVE32_INSTS_EXP_GDS__GFX11            = 0x0000004a,
    SQ_PERF_SEL_INSTS_WAVE32_VALU_TRANS32__GFX11       = 0x0000004b,
    SQ_PERF_SEL_INSTS_WAVE32_VALU_NO_COEXEC__GFX11     = 0x0000004c,
    SQ_PERF_SEL_INSTS_WAVE32_TEX__GFX11                = 0x0000004d,
    SQ_PERF_SEL_INSTS_WAVE32_TEX_LOAD__GFX11           = 0x0000004e,
    SQ_PERF_SEL_INSTS_WAVE32_TEX_STORE__GFX11          = 0x0000004f,
    SQ_PERF_SEL_ITEM_CYCLES_VALU__GFX11                = 0x00000050,
    SQ_PERF_SEL_VALU_READWRITELANE_CYCLES__GFX11       = 0x00000051,
    SQ_PERF_SEL_WAVE32_INSTS__GFX11                    = 0x00000052,
    SQ_PERF_SEL_WAVE64_INSTS__GFX11                    = 0x00000053,
    SQ_PERF_SEL_INSTS_VALU_EXEC_SKIPPED__GFX11         = 0x00000054,
    SQ_PERF_SEL_WAVE64_HALF_SKIP__GFX11                = 0x00000055,
    SQ_PERF_SEL_INST_LEVEL_EXP__GFX11                  = 0x00000056,
    SQ_PERF_SEL_INST_LEVEL_GDS__GFX11                  = 0x00000057,
    SQ_PERF_SEL_INST_LEVEL_LDS__GFX11                  = 0x00000058,
    SQ_PERF_SEL_INST_LEVEL_SMEM__GFX11                 = 0x00000059,
    SQ_PERF_SEL_INST_LEVEL_TEX_LOAD__GFX11             = 0x0000005a,
    SQ_PERF_SEL_INST_LEVEL_TEX_STORE__GFX11            = 0x0000005b,
    SQ_PERF_SEL_IFETCH_REQS__GFX11                     = 0x0000005c,
    SQ_PERF_SEL_LDS_DIRECT_CMD_FIFO_FULL_STALL__GFX11  = 0x0000005e,
    SQ_PERF_SEL_VALU_SGATHER_STALL__GFX11              = 0x0000005f,
    SQ_PERF_SEL_VALU_FWD_BUFFER_FULL_STALL__GFX11      = 0x00000060,
    SQ_PERF_SEL_VALU_SGPR_RD_FIFO_FULL_STALL__GFX11    = 0x00000061,
    SQ_PERF_SEL_VALU_SGATHER_FULL_STALL__GFX11         = 0x00000062,
    SQ_PERF_SEL_SALU_SGATHER_STALL__GFX11              = 0x00000063,
    SQ_PERF_SEL_SALU_SGPR_RD_FIFO_FULL_STALL__GFX11    = 0x00000064,
    SQ_PERF_SEL_SALU_GATHER_FULL_STALL__GFX11          = 0x00000065,
    SQ_PERF_SEL_SMEM_DCACHE_FIFO_FULL_STALL__GFX11     = 0x00000066,
    SQ_PERF_SEL_INST_CYCLES_VALU__GFX11                = 0x00000067,
    SQ_PERF_SEL_INST_CYCLES_VALU_TRANS32__GFX11        = 0x00000068,
    SQ_PERF_SEL_INST_CYCLES_VALU_NO_COEXEC__GFX11      = 0x00000069,
    SQ_PERF_SEL_INST_CYCLES_VMEM__GFX11                = 0x0000006a,
    SQ_PERF_SEL_INST_CYCLES_VMEM_LOAD__GFX11           = 0x0000006b,
    SQ_PERF_SEL_INST_CYCLES_VMEM_STORE__GFX11          = 0x0000006c,
    SQ_PERF_SEL_INST_CYCLES_LDS__GFX11                 = 0x0000006d,
    SQ_PERF_SEL_INST_CYCLES_TEX__GFX11                 = 0x0000006e,
    SQ_PERF_SEL_INST_CYCLES_FLAT__GFX11                = 0x0000006f,
    SQ_PERF_SEL_INST_CYCLES_EXP_GDS__GFX11             = 0x00000070,
    SQ_PERF_SEL_INST_CYCLES_EXP__GFX11                 = 0x00000071,
    SQ_PERF_SEL_INST_CYCLES_GDS__GFX11                 = 0x00000072,
    SQ_PERF_SEL_VALU_STARVE__GFX11                     = 0x00000073,
    SQ_PERF_SEL_VMEM_ARB_FIFO_FULL__GFX11              = 0x00000074,
    SQ_PERF_SEL_MSG_FIFO_FULL_STALL__GFX11             = 0x00000075,
    SQ_PERF_SEL_EXP_REQ_FIFO_FULL__GFX11               = 0x00000076,
    SQ_PERF_SEL_VMEM_BUS_ACTIVE__GFX11                 = 0x00000077,
    SQ_PERF_SEL_VMEM_BUS_STALL__GFX11                  = 0x00000078,
    SQ_PERF_SEL_VMEM_BUS_STALL_TA_ADDR_FIFO_FULL__GFX11 = 0x00000079,
    SQ_PERF_SEL_VMEM_BUS_STALL_TA_CMD_FIFO_FULL__GFX11 = 0x0000007a,
    SQ_PERF_SEL_VMEM_BUS_STALL_LDS_ADDR_FIFO_FULL__GFX11 = 0x0000007b,
    SQ_PERF_SEL_VMEM_BUS_STALL_LDS_CMD_FIFO_FULL__GFX11 = 0x0000007c,
    SQ_PERF_SEL_VMEM_STARVE_TA_ADDR_EMPTY__GFX11       = 0x0000007d,
    SQ_PERF_SEL_VMEM_STARVE_LDS_ADDR_EMPTY__GFX11      = 0x0000007e,
    SQ_PERF_SEL_SALU_PIPE_STALL__GFX11                 = 0x0000007f,
    SQ_PERF_SEL_SMEM_DCACHE_RETURN_CYCLES__GFX11       = 0x00000080,
    SQ_PERF_SEL_MSG_BUS_BUSY__GFX11                    = 0x00000081,
    SQ_PERF_SEL_EXP_REQ_BUS_STALL__GFX11               = 0x00000082,
    SQ_PERF_SEL_EXP_REQ0_BUS_BUSY__GFX11               = 0x00000083,
    SQ_PERF_SEL_EXP_REQ1_BUS_BUSY__GFX11               = 0x00000084,
    SQ_PERF_SEL_EXP_BUS0_BUSY__GFX11                   = 0x00000085,
    SQ_PERF_SEL_EXP_BUS1_BUSY__GFX11                   = 0x00000086,
    SQ_PERF_SEL_INST_CACHE_REQ_STALL__GFX11            = 0x00000087,
    SQ_PERF_SEL_USER0__GFX11                           = 0x00000088,
    SQ_PERF_SEL_USER1__GFX11                           = 0x00000089,
    SQ_PERF_SEL_USER2__GFX11                           = 0x0000008a,
    SQ_PERF_SEL_USER3__GFX11                           = 0x0000008b,
    SQ_PERF_SEL_USER4__GFX11                           = 0x0000008c,
    SQ_PERF_SEL_USER5__GFX11                           = 0x0000008d,
    SQ_PERF_SEL_USER6__GFX11                           = 0x0000008e,
    SQ_PERF_SEL_USER7__GFX11                           = 0x0000008f,
    SQ_PERF_SEL_USER8__GFX11                           = 0x00000090,
    SQ_PERF_SEL_USER9__GFX11                           = 0x00000091,
    SQ_PERF_SEL_USER10__GFX11                          = 0x00000092,
    SQ_PERF_SEL_USER11__GFX11                          = 0x00000093,
    SQ_PERF_SEL_USER12__GFX11                          = 0x00000094,
    SQ_PERF_SEL_USER13__GFX11                          = 0x00000095,
    SQ_PERF_SEL_USER14__GFX11                          = 0x00000096,
    SQ_PERF_SEL_USER15__GFX11                          = 0x00000097,
    SQ_PERF_SEL_USER_LEVEL0__GFX11                     = 0x00000098,
    SQ_PERF_SEL_USER_LEVEL1__GFX11                     = 0x00000099,
    SQ_PERF_SEL_USER_LEVEL2__GFX11                     = 0x0000009a,
    SQ_PERF_SEL_USER_LEVEL3__GFX11                     = 0x0000009b,
    SQ_PERF_SEL_USER_LEVEL4__GFX11                     = 0x0000009c,
    SQ_PERF_SEL_USER_LEVEL5__GFX11                     = 0x0000009d,
    SQ_PERF_SEL_USER_LEVEL6__GFX11                     = 0x0000009e,
    SQ_PERF_SEL_USER_LEVEL7__GFX11                     = 0x0000009f,
    SQ_PERF_SEL_USER_LEVEL8__GFX11                     = 0x000000a0,
    SQ_PERF_SEL_USER_LEVEL9__GFX11                     = 0x000000a1,
    SQ_PERF_SEL_USER_LEVEL10__GFX11                    = 0x000000a2,
    SQ_PERF_SEL_USER_LEVEL11__GFX11                    = 0x000000a3,
    SQ_PERF_SEL_USER_LEVEL12__GFX11                    = 0x000000a4,
    SQ_PERF_SEL_USER_LEVEL13__GFX11                    = 0x000000a5,
    SQ_PERF_SEL_USER_LEVEL14__GFX11                    = 0x000000a6,
    SQ_PERF_SEL_USER_LEVEL15__GFX11                    = 0x000000a7,
    SQ_PERF_SEL_VALU_RETURN_SDST__GFX11                = 0x000000a8,
    SQ_PERF_SEL_VMEM_VGPR_READ_STALLED_BY_EXPORT__GFX11 = 0x000000a9,
    SQ_PERF_SEL_INSTS_VALU_TRANS__GFX11                = 0x000000aa,
    SQ_PERF_SEL_INSTS_LDS_DIRECT_LOAD__GFX11           = 0x000000ab,
    SQ_PERF_SEL_INSTS_LDS_PARAM_LOAD__GFX11            = 0x000000ac,
    SQ_PERF_SEL_INSTS_WAVE32_LDS_PARAM_LOAD__GFX11     = 0x000000ad,
    SQ_PERF_SEL_INSTS_VALU_ONE_CYCLE_WAVE64__GFX11     = 0x000000ae,
    SQ_PERF_SEL_INSTS_VALU_VINTERP__GFX11              = 0x000000af,
    SQ_PERF_SEL_INSTS_VALU_WAVE32_VINTERP__GFX11       = 0x000000b0,
    SQ_PERF_SEL_OVERFLOW_PREV__GFX11                   = 0x000000b1,
    SQ_PERF_SEL_INSTS_DUAL_VALU_WAVE32__GFX11          = 0x000000b2,
    SQ_PERF_SEL_INSTS_VALU_1_PASS__GFX11               = 0x000000b3,
    SQ_PERF_SEL_INSTS_VALU_2_PASS__GFX11               = 0x000000b4,
    SQ_PERF_SEL_INSTS_VALU_4_PASS__GFX11               = 0x000000b5,
    SQ_PERF_SEL_INSTS_VALU_DP__GFX11                   = 0x000000b6,
    SQ_PERF_SEL_SP_CONST_CYCLES__GFX11                 = 0x000000b7,
    SQ_PERF_SEL_SP_CONST_STALL_CYCLES__GFX11           = 0x000000b8,
    SQ_PERF_SEL_ITEMS_VALU__GFX11                      = 0x000000b9,
    SQ_PERF_SEL_ITEMS_MAX_VALU__GFX11                  = 0x000000ba,
    SQ_PERF_SEL_ITEM_CYCLES_VMEM__GFX11                = 0x000000bb,
    SQ_PERF_SEL_INSTS_DELAY_ALU_COISSUE__GFX11         = 0x000000bc,
    SQ_PERF_SEL_INSTS_FLAT_LOAD__GFX11                 = 0x000000bd,
    SQ_PERF_SEL_INSTS_FLAT_STORE__GFX11                = 0x000000be,
    SQ_PERF_SEL_INSTS_VALU_ONE_CYCLE_WAVE64_16BIT__GFX11 = 0x000000bf,
    SQ_PERF_SEL_INSTS_VALU_ONE_CYCLE_WAVE64_32BIT__GFX11 = 0x000000c0,
    SQ_PERF_SEL_INSTS_NON_VALU_EXEC_SKIPPED__GFX11     = 0x000000c1,
    SQ_PERF_SEL_INSTS_BARRIER__GFX11                   = 0x000000c2,
    SQ_PERF_SEL_INSTS_WAKEUP__GFX11                    = 0x000000c3,
    SQ_PERF_SEL_DUMMY_END__GFX11                       = 0x000000c4,
    SQC_PERF_SEL_LDS_BANK_CONFLICT__GFX11              = 0x00000100,
    SQC_PERF_SEL_LDS_ADDR_CONFLICT__GFX11              = 0x00000101,
    SQC_PERF_SEL_LDS_UNALIGNED_STALL__GFX11            = 0x00000102,
    SQC_PERF_SEL_LDS_MEM_VIOLATIONS__GFX11             = 0x00000103,
    SQC_PERF_SEL_LDS_ATOMIC_RETURN__GFX11              = 0x00000104,
    SQC_PERF_SEL_LDS_IDX_ACTIVE__GFX11                 = 0x00000105,
    SQC_PERF_SEL_LDS_ADDR_STALL__GFX11                 = 0x00000106,
    SQC_PERF_SEL_LDS_ADDR_ACTIVE__GFX11                = 0x00000107,
    SQC_PERF_SEL_LDS_PC_LDS_WRITE_STALL_TD__GFX11      = 0x00000108,
    SQC_PERF_SEL_LDS_SPI_VGPR_WRITE_STALL_TD__GFX11    = 0x00000109,
    SQC_PERF_SEL_LDS_LDS_VGPR_WRITE_STALL__GFX11       = 0x0000010a,
    SQC_PERF_SEL_LDS_FP_ADD_CYCLES__GFX11              = 0x0000010b,
    SQC_PERF_SEL_ICACHE_BUSY_CYCLES__GFX11             = 0x0000010c,
    SQC_PERF_SEL_ICACHE_REQ__GFX11                     = 0x0000010d,
    SQC_PERF_SEL_ICACHE_HITS__GFX11                    = 0x0000010e,
    SQC_PERF_SEL_ICACHE_MISSES__GFX11                  = 0x0000010f,
    SQC_PERF_SEL_ICACHE_MISSES_DUPLICATE__GFX11        = 0x00000110,
    SQC_PERF_SEL_ICACHE_INVAL_INST__GFX11              = 0x00000111,
    SQC_PERF_SEL_ICACHE_INVAL_ASYNC__GFX11             = 0x00000112,
    SQC_PERF_SEL_ICACHE_INFLIGHT_LEVEL__GFX11          = 0x00000113,
    SQC_PERF_SEL_DCACHE_INFLIGHT_LEVEL__GFX11          = 0x00000114,
    SQC_PERF_SEL_TC_INFLIGHT_LEVEL__GFX11              = 0x00000115,
    SQC_PERF_SEL_ICACHE_TC_INFLIGHT_LEVEL__GFX11       = 0x00000116,
    SQC_PERF_SEL_DCACHE_TC_INFLIGHT_LEVEL__GFX11       = 0x00000117,
    SQC_PERF_SEL_ICACHE_INPUT_VALID_READYB__GFX11      = 0x00000118,
    SQC_PERF_SEL_DCACHE_INPUT_VALID_READYB__GFX11      = 0x00000119,
    SQC_PERF_SEL_TC_REQ__GFX11                         = 0x0000011a,
    SQC_PERF_SEL_TC_INST_REQ__GFX11                    = 0x0000011b,
    SQC_PERF_SEL_TC_DATA_READ_REQ__GFX11               = 0x0000011c,
    SQC_PERF_SEL_TC_STALL__GFX11                       = 0x0000011d,
    SQC_PERF_SEL_TC_STARVE__GFX11                      = 0x0000011e,
    SQC_PERF_SEL_ICACHE_INPUT_STALL_ARB_NO_GRANT__GFX11 = 0x0000011f,
    SQC_PERF_SEL_ICACHE_INPUT_STALL_BANK_READYB__GFX11 = 0x00000120,
    SQC_PERF_SEL_ICACHE_CACHE_STALLED__GFX11           = 0x00000121,
    SQC_PERF_SEL_ICACHE_CACHE_STALL_INFLIGHT_MAX__GFX11 = 0x00000122,
    SQC_PERF_SEL_ICACHE_STALL_OUTXBAR_ARB_NO_GRANT__GFX11 = 0x00000123,
    SQC_PERF_SEL_DCACHE_BUSY_CYCLES__GFX11             = 0x00000124,
    SQC_PERF_SEL_DCACHE_REQ__GFX11                     = 0x00000125,
    SQC_PERF_SEL_DCACHE_HITS__GFX11                    = 0x00000126,
    SQC_PERF_SEL_DCACHE_MISSES__GFX11                  = 0x00000127,
    SQC_PERF_SEL_DCACHE_MISSES_DUPLICATE__GFX11        = 0x00000128,
    SQC_PERF_SEL_DCACHE_INVAL_INST__GFX11              = 0x00000129,
    SQC_PERF_SEL_DCACHE_INVAL_ASYNC__GFX11             = 0x0000012a,
    SQC_PERF_SEL_DCACHE_HIT_LRU_READ__GFX11            = 0x0000012b,
    SQC_PERF_SEL_DCACHE_INPUT_STALL_ARB_NO_GRANT__GFX11 = 0x0000012c,
    SQC_PERF_SEL_DCACHE_INPUT_STALL_BANK_READYB__GFX11 = 0x0000012d,
    SQC_PERF_SEL_DCACHE_CACHE_STALLED__GFX11           = 0x0000012e,
    SQC_PERF_SEL_DCACHE_CACHE_STALL_INFLIGHT_MAX__GFX11 = 0x0000012f,
    SQC_PERF_SEL_DCACHE_CACHE_STALL_OUTPUT__GFX11      = 0x00000130,
    SQC_PERF_SEL_DCACHE_STALL_OUTXBAR_ARB_NO_GRANT__GFX11 = 0x00000131,
    SQC_PERF_SEL_DCACHE_REQ_READ_1__GFX11              = 0x00000132,
    SQC_PERF_SEL_DCACHE_REQ_READ_2__GFX11              = 0x00000133,
    SQC_PERF_SEL_DCACHE_REQ_READ_4__GFX11              = 0x00000134,
    SQC_PERF_SEL_DCACHE_REQ_READ_8__GFX11              = 0x00000135,
    SQC_PERF_SEL_DCACHE_REQ_READ_16__GFX11             = 0x00000136,
    SQC_PERF_SEL_DCACHE_REQ_ATC_PROBE__GFX11           = 0x00000137,
    SQC_PERF_SEL_SQ_DCACHE_REQS__GFX11                 = 0x00000138,
    SQC_PERF_SEL_DCACHE_FLAT_REQ__GFX11                = 0x00000139,
    SQC_PERF_SEL_TD_VGPR_BUSY__GFX11                   = 0x0000013a,
    SQC_PERF_SEL_LDS_VGPR_BUSY__GFX11                  = 0x0000013b,
    SQC_PERF_SEL_LDS_TD_VGPR_CONF_STALL__GFX11         = 0x0000013c,
    SQC_PERF_SEL_ICACHE_GCR__GFX11                     = 0x0000013d,
    SQC_PERF_SEL_ICACHE_GCR_HITS__GFX11                = 0x0000013e,
    SQC_PERF_SEL_DCACHE_GCR__GFX11                     = 0x0000013f,
    SQC_PERF_SEL_DCACHE_GCR_HITS__GFX11                = 0x00000140,
    SQC_PERF_SEL_ICACHE_GCR_INVALIDATE__GFX11          = 0x00000141,
    SQC_PERF_SEL_DCACHE_GCR_INVALIDATE__GFX11          = 0x00000142,
    SQC_PERF_SEL_DCACHE_SPI_RETURN_STALL__GFX11        = 0x00000143,
    SQC_PERF_SEL_DUMMY_LAST__GFX11                     = 0x00000144,
    SP_PERF_SEL_DST_BUF_ALLOC_STALL__GFX11             = 0x000001c0,
    SP_PERF_SEL_DST_BUF_WB_CONF_W_TD_LDS__GFX11        = 0x000001c1,
    SP_PERF_SEL_DST_BUF_WB_CONF_W_SPI__GFX11           = 0x000001c2,
    SP_PERF_SEL_DST_BUF_EVEN_DIRTY__GFX11              = 0x000001c3,
    SP_PERF_SEL_DST_BUF_ODD_DIRTY__GFX11               = 0x000001c4,
    SP_PERF_SEL_SRC_CACHE_HIT_B0__GFX11                = 0x000001c5,
    SP_PERF_SEL_SRC_CACHE_HIT_B1__GFX11                = 0x000001c6,
    SP_PERF_SEL_SRC_CACHE_HIT_B2__GFX11                = 0x000001c7,
    SP_PERF_SEL_SRC_CACHE_HIT_B3__GFX11                = 0x000001c8,
    SP_PERF_SEL_SRC_CACHE_PROBE_B0__GFX11              = 0x000001c9,
    SP_PERF_SEL_SRC_CACHE_PROBE_B1__GFX11              = 0x000001ca,
    SP_PERF_SEL_SRC_CACHE_PROBE_B2__GFX11              = 0x000001cb,
    SP_PERF_SEL_SRC_CACHE_PROBE_B3__GFX11              = 0x000001cc,
    SP_PERF_SEL_SRC_CACHE_VGPR_RD_B0__GFX11            = 0x000001cd,
    SP_PERF_SEL_SRC_CACHE_VGPR_RD_B1__GFX11            = 0x000001ce,
    SP_PERF_SEL_SRC_CACHE_VGPR_RD_B2__GFX11            = 0x000001cf,
    SP_PERF_SEL_SRC_CACHE_VGPR_RD_B3__GFX11            = 0x000001d0,
    SP_PERF_SEL_SRC_CACHE_RECYCLE_HIT_B0__GFX11        = 0x000001d1,
    SP_PERF_SEL_SRC_CACHE_RECYCLE_HIT_B1__GFX11        = 0x000001d2,
    SP_PERF_SEL_SRC_CACHE_RECYCLE_HIT_B2__GFX11        = 0x000001d3,
    SP_PERF_SEL_SRC_CACHE_RECYCLE_HIT_B3__GFX11        = 0x000001d4,
    SP_PERF_SEL_SRC_CACHE_PROBE_SUCCESS_B0__GFX11      = 0x000001d5,
    SP_PERF_SEL_SRC_CACHE_PROBE_SUCCESS_B1__GFX11      = 0x000001d6,
    SP_PERF_SEL_SRC_CACHE_PROBE_SUCCESS_B2__GFX11      = 0x000001d7,
    SP_PERF_SEL_SRC_CACHE_PROBE_SUCCESS_B3__GFX11      = 0x000001d8,
    SP_PERF_SEL_VALU_PENDING_QUEUE_STALL__GFX11        = 0x000001d9,
    SP_PERF_SEL_VALU_OPERAND__GFX11                    = 0x000001da,
    SP_PERF_SEL_VALU_VGPR_OPERAND__GFX11               = 0x000001db,
    SP_PERF_SEL_VALU_OPERAND_FROM_DST_BUF__GFX11       = 0x000001dc,
    SP_PERF_SEL_VALU_EXEC_MASK_CHANGE__GFX11           = 0x000001dd,
    SP_PERF_SEL_VALU_COEXEC_WITH_TRANS__GFX11          = 0x000001de,
    SP_PERF_SEL_VALU_SGPR_FWD_BUF_FULL__GFX11          = 0x000001df,
    SP_PERF_SEL_VALU_STALL__GFX11                      = 0x000001e0,
    SP_PERF_SEL_VALU_STALL_VGPR_NOT_READY__GFX11       = 0x000001e1,
    SP_PERF_SEL_VALU_STALL_SGPR_NOT_READY__GFX11       = 0x000001e2,
    SP_PERF_SEL_VALU_STALL_VDST_FWD__GFX11             = 0x000001e3,
    SP_PERF_SEL_VALU_STALL_SDST_FWD__GFX11             = 0x000001e4,
    SP_PERF_SEL_VALU_STALL_DST_STALL__GFX11            = 0x000001e5,
    SP_PERF_SEL_VALU_FAST_OP_STALL_VGPR_NOT_READY__GFX11 = 0x000001e6,
    SP_PERF_SEL_VGPR_VMEM_RD__GFX11                    = 0x000001e7,
    SP_PERF_SEL_VGPR_EXP_RD__GFX11                     = 0x000001e8,
    SP_PERF_SEL_VGPR_SPI_WR__GFX11                     = 0x000001e9,
    SP_PERF_SEL_VGPR_TDLDS_DATA_WR__GFX11              = 0x000001ea,
    SP_PERF_SEL_VGPR_WR__GFX11                         = 0x000001eb,
    SP_PERF_SEL_VGPR_RD__GFX11                         = 0x000001ec,
    SQ_PERF_SEL_NONE2__GFX11                           = 0x000001ff,
    SQ_PERF_SEL_IFETCH_LEVEL__GFX110                   = 0x0000005d,
    SP_PERF_SEL_DUMMY_LAST__GFX110                     = 0x000001ed,
    SQ_PERF_SEL_IFETCH_LEVEL__STX1                     = 0x0000005d,
    SP_PERF_SEL_DUMMY_LAST__STX1                       = 0x000001ed,
#if CHIP_HDR_STRIX_HALO
    SQ_PERF_SEL_RESERVED_93__STXH                      = 0x0000005d,
    SQ_PERF_SEL_IS_REQ__STXH                           = 0x000000c5,
    SQ_PERF_SEL_IS_HIT__STXH                           = 0x000000c6,
    SP_PERF_SEL_VGPR_WR_KILL__STXH                     = 0x000001ed,
    SP_PERF_SEL_DUMMY_LAST__STXH                       = 0x000001ee,
#endif
} SQ_PERF_SEL;

constexpr unsigned int MaxSqPerfSelGfx10               = SP_PERF_SEL_DUMMY_LAST__GFX10;
constexpr unsigned int MaxSqPerfSelGfx11               = SQ_PERF_SEL_NONE2__GFX11;

typedef enum SQ_RSRC_BUF_TYPE {
    SQ_RSRC_BUF                                        = 0x00000000,
    SQ_RSRC_BUF_RSVD_1                                 = 0x00000001,
    SQ_RSRC_BUF_RSVD_2                                 = 0x00000002,
    SQ_RSRC_BUF_RSVD_3                                 = 0x00000003,
} SQ_RSRC_BUF_TYPE;

typedef enum SQ_RSRC_FLAT_TYPE {
    SQ_RSRC_FLAT_RSVD_0                                = 0x00000000,
    SQ_RSRC_FLAT                                       = 0x00000001,
    SQ_RSRC_FLAT_RSVD_2                                = 0x00000002,
    SQ_RSRC_FLAT_RSVD_3                                = 0x00000003,
} SQ_RSRC_FLAT_TYPE;

typedef enum SQ_RSRC_IMG_TYPE {
    SQ_RSRC_IMG_RSVD_0                                 = 0x00000000,
    SQ_RSRC_IMG_RSVD_1                                 = 0x00000001,
    SQ_RSRC_IMG_RSVD_2                                 = 0x00000002,
    SQ_RSRC_IMG_RSVD_3                                 = 0x00000003,
    SQ_RSRC_IMG_RSVD_4                                 = 0x00000004,
    SQ_RSRC_IMG_RSVD_5                                 = 0x00000005,
    SQ_RSRC_IMG_RSVD_6                                 = 0x00000006,
    SQ_RSRC_IMG_RSVD_7                                 = 0x00000007,
    SQ_RSRC_IMG_1D                                     = 0x00000008,
    SQ_RSRC_IMG_2D                                     = 0x00000009,
    SQ_RSRC_IMG_3D                                     = 0x0000000a,
    SQ_RSRC_IMG_CUBE                                   = 0x0000000b,
    SQ_RSRC_IMG_1D_ARRAY                               = 0x0000000c,
    SQ_RSRC_IMG_2D_ARRAY                               = 0x0000000d,
    SQ_RSRC_IMG_2D_MSAA                                = 0x0000000e,
    SQ_RSRC_IMG_2D_MSAA_ARRAY                          = 0x0000000f,
} SQ_RSRC_IMG_TYPE;

typedef enum SQ_SEL_XYZW01 {
    SQ_SEL_0                                           = 0x00000000,
    SQ_SEL_1                                           = 0x00000001,
    SQ_SEL_N_BC_1                                      = 0x00000002,
    SQ_SEL_RESERVED_1                                  = 0x00000003,
    SQ_SEL_X                                           = 0x00000004,
    SQ_SEL_Y                                           = 0x00000005,
    SQ_SEL_Z                                           = 0x00000006,
    SQ_SEL_W                                           = 0x00000007,
} SQ_SEL_XYZW01;

typedef enum SQ_TEX_ANISO_RATIO {
    SQ_TEX_ANISO_RATIO_1                               = 0x00000000,
    SQ_TEX_ANISO_RATIO_2                               = 0x00000001,
    SQ_TEX_ANISO_RATIO_4                               = 0x00000002,
    SQ_TEX_ANISO_RATIO_8                               = 0x00000003,
    SQ_TEX_ANISO_RATIO_16                              = 0x00000004,
} SQ_TEX_ANISO_RATIO;

typedef enum SQ_TEX_BORDER_COLOR {
    SQ_TEX_BORDER_COLOR_TRANS_BLACK                    = 0x00000000,
    SQ_TEX_BORDER_COLOR_OPAQUE_BLACK                   = 0x00000001,
    SQ_TEX_BORDER_COLOR_OPAQUE_WHITE                   = 0x00000002,
    SQ_TEX_BORDER_COLOR_REGISTER                       = 0x00000003,
} SQ_TEX_BORDER_COLOR;

typedef enum SQ_TEX_CLAMP {
    SQ_TEX_WRAP                                        = 0x00000000,
    SQ_TEX_MIRROR                                      = 0x00000001,
    SQ_TEX_CLAMP_LAST_TEXEL                            = 0x00000002,
    SQ_TEX_MIRROR_ONCE_LAST_TEXEL                      = 0x00000003,
    SQ_TEX_CLAMP_HALF_BORDER                           = 0x00000004,
    SQ_TEX_MIRROR_ONCE_HALF_BORDER                     = 0x00000005,
    SQ_TEX_CLAMP_BORDER                                = 0x00000006,
    SQ_TEX_MIRROR_ONCE_BORDER                          = 0x00000007,
} SQ_TEX_CLAMP;

typedef enum SQ_TEX_DEPTH_COMPARE {
    SQ_TEX_DEPTH_COMPARE_NEVER                         = 0x00000000,
    SQ_TEX_DEPTH_COMPARE_LESS                          = 0x00000001,
    SQ_TEX_DEPTH_COMPARE_EQUAL                         = 0x00000002,
    SQ_TEX_DEPTH_COMPARE_LESSEQUAL                     = 0x00000003,
    SQ_TEX_DEPTH_COMPARE_GREATER                       = 0x00000004,
    SQ_TEX_DEPTH_COMPARE_NOTEQUAL                      = 0x00000005,
    SQ_TEX_DEPTH_COMPARE_GREATEREQUAL                  = 0x00000006,
    SQ_TEX_DEPTH_COMPARE_ALWAYS                        = 0x00000007,
} SQ_TEX_DEPTH_COMPARE;

typedef enum SQ_TEX_MIP_FILTER {
    SQ_TEX_MIP_FILTER_NONE                             = 0x00000000,
    SQ_TEX_MIP_FILTER_POINT                            = 0x00000001,
    SQ_TEX_MIP_FILTER_LINEAR                           = 0x00000002,
    SQ_TEX_MIP_FILTER_POINT_ANISO_ADJ                  = 0x00000003,
} SQ_TEX_MIP_FILTER;

typedef enum SQ_TEX_XY_FILTER {
    SQ_TEX_XY_FILTER_POINT                             = 0x00000000,
    SQ_TEX_XY_FILTER_BILINEAR                          = 0x00000001,
    SQ_TEX_XY_FILTER_ANISO_POINT                       = 0x00000002,
    SQ_TEX_XY_FILTER_ANISO_BILINEAR                    = 0x00000003,
} SQ_TEX_XY_FILTER;

typedef enum SQ_TEX_Z_FILTER {
    SQ_TEX_Z_FILTER_NONE                               = 0x00000000,
    SQ_TEX_Z_FILTER_POINT                              = 0x00000001,
    SQ_TEX_Z_FILTER_LINEAR                             = 0x00000002,
} SQ_TEX_Z_FILTER;

typedef enum SQ_TT_MODE {
    SQ_TT_MODE_OFF                                     = 0x00000000,
    SQ_TT_MODE_ON                                      = 0x00000001,
    SQ_TT_MODE_GLOBAL                                  = 0x00000002,
    SQ_TT_MODE_DETAIL                                  = 0x00000003,
} SQ_TT_MODE;

typedef enum SQ_TT_RT_FREQ {
    SQ_TT_RT_FREQ_NEVER                                = 0x00000000,
    SQ_TT_RT_FREQ_1024_CLK                             = 0x00000001,
    SQ_TT_RT_FREQ_4096_CLK                             = 0x00000002,
} SQ_TT_RT_FREQ;

typedef enum SQ_TT_TOKEN_MASK_INST_EXCLUDE {
    SQ_TT_INST_EXCLUDE_VMEM_OTHER_SIMD_BIT             = 0x00000001,
    SQ_TT_INST_EXCLUDE_EXPGNT234_BIT                   = 0x00000002,
} SQ_TT_TOKEN_MASK_INST_EXCLUDE;

typedef enum SQ_TT_TOKEN_MASK_INST_EXCLUDE_SHIFT {
    SQ_TT_INST_EXCLUDE_VMEM_OTHER_SIMD_SHIFT           = 0x00000000,
    SQ_TT_INST_EXCLUDE_EXPGNT234_SHIFT                 = 0x00000001,
} SQ_TT_TOKEN_MASK_INST_EXCLUDE_SHIFT;

typedef enum SQ_TT_TOKEN_MASK_REG_EXCLUDE {
    SQ_TT_REG_EXCLUDE_USER_DATA_BIT                    = 0x00000001,
    SQ_TT_REG_EXCLUDE_CP_ME_MC_RADDR_BIT               = 0x00000002,
    SQ_TT_REG_EXCLUDE_GRBM_COMPUTE_EXCLUDE_BIT         = 0x00000004,
} SQ_TT_TOKEN_MASK_REG_EXCLUDE;

typedef enum SQ_TT_TOKEN_MASK_REG_EXCLUDE_SHIFT {
    SQ_TT_REG_EXCLUDE_USER_DATA_SHIFT                  = 0x00000000,
    SQ_TT_REG_EXCLUDE_CP_ME_MC_RADDR_SHIFT             = 0x00000001,
    SQ_TT_REG_EXCLUDE_GRBM_COMPUTE_EXCLUDE_SHIFT       = 0x00000002,
} SQ_TT_TOKEN_MASK_REG_EXCLUDE_SHIFT;

typedef enum SQ_TT_TOKEN_MASK_REG_INCLUDE {
    SQ_TT_TOKEN_MASK_SQDEC_BIT                         = 0x00000001,
    SQ_TT_TOKEN_MASK_SHDEC_BIT                         = 0x00000002,
    SQ_TT_TOKEN_MASK_GFXUDEC_BIT                       = 0x00000004,
    SQ_TT_TOKEN_MASK_COMP_BIT                          = 0x00000008,
    SQ_TT_TOKEN_MASK_CONTEXT_BIT                       = 0x00000010,
    SQ_TT_TOKEN_MASK_CONFIG_BIT                        = 0x00000020,
    SQ_TT_TOKEN_MASK_OTHER_BIT__GFX10                  = 0x00000040,
    SQ_TT_TOKEN_MASK_READS_BIT__GFX10                  = 0x00000080,
    SQ_TT_TOKEN_MASK_ALL_BIT__GFX11                    = 0x00000040,
    SQ_TT_TOKEN_MASK_RSVD_BIT__GFX11                   = 0x00000080,
} SQ_TT_TOKEN_MASK_REG_INCLUDE;

typedef enum SQ_TT_TOKEN_MASK_REG_INCLUDE_SHIFT {
    SQ_TT_TOKEN_MASK_SQDEC_SHIFT                       = 0x00000000,
    SQ_TT_TOKEN_MASK_SHDEC_SHIFT                       = 0x00000001,
    SQ_TT_TOKEN_MASK_GFXUDEC_SHIFT                     = 0x00000002,
    SQ_TT_TOKEN_MASK_COMP_SHIFT                        = 0x00000003,
    SQ_TT_TOKEN_MASK_CONTEXT_SHIFT                     = 0x00000004,
    SQ_TT_TOKEN_MASK_CONFIG_SHIFT                      = 0x00000005,
    SQ_TT_TOKEN_MASK_OTHER_SHIFT__GFX10                = 0x00000006,
    SQ_TT_TOKEN_MASK_READS_SHIFT__GFX10                = 0x00000007,
    SQ_TT_TOKEN_MASK_ALL_SHIFT__GFX11                  = 0x00000006,
    SQ_TT_TOKEN_MASK_RSVD_SHIFT__GFX11                 = 0x00000007,
} SQ_TT_TOKEN_MASK_REG_INCLUDE_SHIFT;

typedef enum SQ_TT_TOKEN_MASK_TOKEN_EXCLUDE_SHIFT {
    SQ_TT_TOKEN_EXCLUDE_VMEMEXEC_SHIFT                 = 0x00000000,
    SQ_TT_TOKEN_EXCLUDE_ALUEXEC_SHIFT                  = 0x00000001,
    SQ_TT_TOKEN_EXCLUDE_VALUINST_SHIFT                 = 0x00000002,
    SQ_TT_TOKEN_EXCLUDE_WAVERDY_SHIFT                  = 0x00000003,
    SQ_TT_TOKEN_EXCLUDE_IMMEDIATE_SHIFT                = 0x00000005,
    SQ_TT_TOKEN_EXCLUDE_REG_SHIFT                      = 0x00000006,
    SQ_TT_TOKEN_EXCLUDE_EVENT_SHIFT                    = 0x00000007,
    SQ_TT_TOKEN_EXCLUDE_INST_SHIFT                     = 0x00000008,
    SQ_TT_TOKEN_EXCLUDE_UTILCTR_SHIFT                  = 0x00000009,
    SQ_TT_TOKEN_EXCLUDE_WAVEALLOC_SHIFT                = 0x0000000a,
    SQ_TT_TOKEN_EXCLUDE_PERF_SHIFT                     = 0x0000000b,
    SQ_TT_TOKEN_EXCLUDE_IMMED1_SHIFT__GFX101           = 0x00000004,
    SQ_TT_TOKEN_EXCLUDE_WAVESTARTEND_SHIFT__GFX103PLUS = 0x00000004,
} SQ_TT_TOKEN_MASK_TOKEN_EXCLUDE_SHIFT;

typedef enum SQ_TT_UTIL_TIMER {
    SQ_TT_UTIL_TIMER_100_CLK                           = 0x00000000,
    SQ_TT_UTIL_TIMER_250_CLK                           = 0x00000001,
} SQ_TT_UTIL_TIMER;

typedef enum SQ_TT_WAVESTART_MODE {
    SQ_TT_WAVESTART_MODE_SHORT                         = 0x00000000,
    SQ_TT_WAVESTART_MODE_ALLOC                         = 0x00000001,
    SQ_TT_WAVESTART_MODE_PBB_ID                        = 0x00000002,
} SQ_TT_WAVESTART_MODE;

typedef enum SQ_TT_WTYPE_INCLUDE {
    SQ_TT_WTYPE_INCLUDE_PS_BIT                         = 0x00000001,
    SQ_TT_WTYPE_INCLUDE_GS_BIT                         = 0x00000004,
    SQ_TT_WTYPE_INCLUDE_HS_BIT                         = 0x00000010,
    SQ_TT_WTYPE_INCLUDE_CS_BIT                         = 0x00000040,
    SQ_TT_WTYPE_INCLUDE_VS_BIT__GFX10                  = 0x00000002,
    SQ_TT_WTYPE_INCLUDE_ES_BIT__GFX10                  = 0x00000008,
    SQ_TT_WTYPE_INCLUDE_LS_BIT__GFX10                  = 0x00000020,
    SQ_TT_WTYPE_INCLUDE_RSVD0_BIT__GFX11               = 0x00000002,
    SQ_TT_WTYPE_INCLUDE_RSVD1_BIT__GFX11               = 0x00000008,
    SQ_TT_WTYPE_INCLUDE_RSVD2_BIT__GFX11               = 0x00000020,
} SQ_TT_WTYPE_INCLUDE;

typedef enum SQ_TT_WTYPE_INCLUDE_SHIFT {
    SQ_TT_WTYPE_INCLUDE_PS_SHIFT                       = 0x00000000,
    SQ_TT_WTYPE_INCLUDE_GS_SHIFT                       = 0x00000002,
    SQ_TT_WTYPE_INCLUDE_HS_SHIFT                       = 0x00000004,
    SQ_TT_WTYPE_INCLUDE_CS_SHIFT                       = 0x00000006,
    SQ_TT_WTYPE_INCLUDE_VS_SHIFT__GFX10                = 0x00000001,
    SQ_TT_WTYPE_INCLUDE_ES_SHIFT__GFX10                = 0x00000003,
    SQ_TT_WTYPE_INCLUDE_LS_SHIFT__GFX10                = 0x00000005,
    SQ_TT_WTYPE_INCLUDE_RSVD0_SHIFT__GFX11             = 0x00000001,
    SQ_TT_WTYPE_INCLUDE_RSVD1_SHIFT__GFX11             = 0x00000003,
    SQ_TT_WTYPE_INCLUDE_RSVD2_SHIFT__GFX11             = 0x00000005,
} SQ_TT_WTYPE_INCLUDE_SHIFT;

typedef enum StencilFormat {
    STENCIL_INVALID                                    = 0x00000000,
    STENCIL_8                                          = 0x00000001,
} StencilFormat;

typedef enum StencilOp {
    STENCIL_KEEP                                       = 0x00000000,
    STENCIL_ZERO                                       = 0x00000001,
    STENCIL_ONES                                       = 0x00000002,
    STENCIL_REPLACE_TEST                               = 0x00000003,
    STENCIL_REPLACE_OP                                 = 0x00000004,
    STENCIL_ADD_CLAMP                                  = 0x00000005,
    STENCIL_SUB_CLAMP                                  = 0x00000006,
    STENCIL_INVERT                                     = 0x00000007,
    STENCIL_ADD_WRAP                                   = 0x00000008,
    STENCIL_SUB_WRAP                                   = 0x00000009,
    STENCIL_AND                                        = 0x0000000a,
    STENCIL_OR                                         = 0x0000000b,
    STENCIL_XOR                                        = 0x0000000c,
    STENCIL_NAND                                       = 0x0000000d,
    STENCIL_NOR                                        = 0x0000000e,
    STENCIL_XNOR                                       = 0x0000000f,
} StencilOp;

typedef enum SurfaceArray {
    ARRAY_1D                                           = 0x00000000,
    ARRAY_2D                                           = 0x00000001,
    ARRAY_3D                                           = 0x00000002,
    ARRAY_3D_SLICE                                     = 0x00000003,
} SurfaceArray;

typedef enum SurfaceEndian {
    ENDIAN_NONE                                        = 0x00000000,
    ENDIAN_8IN16                                       = 0x00000001,
    ENDIAN_8IN32                                       = 0x00000002,
    ENDIAN_8IN64                                       = 0x00000003,
} SurfaceEndian;

typedef enum SurfaceFormat {
    FMT_INVALID                                        = 0x00000000,
    FMT_8                                              = 0x00000001,
    FMT_16                                             = 0x00000002,
    FMT_8_8                                            = 0x00000003,
    FMT_32                                             = 0x00000004,
    FMT_16_16                                          = 0x00000005,
    FMT_10_11_11                                       = 0x00000006,
    FMT_11_11_10                                       = 0x00000007,
    FMT_10_10_10_2                                     = 0x00000008,
    FMT_2_10_10_10                                     = 0x00000009,
    FMT_8_8_8_8                                        = 0x0000000a,
    FMT_32_32                                          = 0x0000000b,
    FMT_16_16_16_16                                    = 0x0000000c,
    FMT_32_32_32                                       = 0x0000000d,
    FMT_32_32_32_32                                    = 0x0000000e,
    FMT_RESERVED_4                                     = 0x0000000f,
    FMT_5_6_5                                          = 0x00000010,
    FMT_1_5_5_5                                        = 0x00000011,
    FMT_5_5_5_1                                        = 0x00000012,
    FMT_4_4_4_4                                        = 0x00000013,
    FMT_8_24                                           = 0x00000014,
    FMT_24_8                                           = 0x00000015,
    FMT_X24_8_32_FLOAT                                 = 0x00000016,
    FMT_RESERVED_33                                    = 0x00000017,
    FMT_11_11_10_FLOAT                                 = 0x00000018,
    FMT_16_FLOAT                                       = 0x00000019,
    FMT_32_FLOAT                                       = 0x0000001a,
    FMT_16_16_FLOAT                                    = 0x0000001b,
    FMT_8_24_FLOAT                                     = 0x0000001c,
    FMT_24_8_FLOAT                                     = 0x0000001d,
    FMT_32_32_FLOAT                                    = 0x0000001e,
    FMT_10_11_11_FLOAT                                 = 0x0000001f,
    FMT_16_16_16_16_FLOAT                              = 0x00000020,
    FMT_3_3_2                                          = 0x00000021,
    FMT_6_5_5                                          = 0x00000022,
    FMT_32_32_32_32_FLOAT                              = 0x00000023,
    FMT_RESERVED_36                                    = 0x00000024,
    FMT_1                                              = 0x00000025,
    FMT_1_REVERSED                                     = 0x00000026,
    FMT_GB_GR                                          = 0x00000027,
    FMT_BG_RG                                          = 0x00000028,
    FMT_32_AS_8                                        = 0x00000029,
    FMT_32_AS_8_8                                      = 0x0000002a,
    FMT_5_9_9_9_SHAREDEXP                              = 0x0000002b,
    FMT_8_8_8                                          = 0x0000002c,
    FMT_16_16_16                                       = 0x0000002d,
    FMT_16_16_16_FLOAT                                 = 0x0000002e,
    FMT_4_4                                            = 0x0000002f,
    FMT_32_32_32_FLOAT                                 = 0x00000030,
    FMT_BC1                                            = 0x00000031,
    FMT_BC2                                            = 0x00000032,
    FMT_BC3                                            = 0x00000033,
    FMT_BC4                                            = 0x00000034,
    FMT_BC5                                            = 0x00000035,
    FMT_BC6                                            = 0x00000036,
    FMT_BC7                                            = 0x00000037,
    FMT_32_AS_32_32_32_32                              = 0x00000038,
    FMT_APC3                                           = 0x00000039,
    FMT_APC4                                           = 0x0000003a,
    FMT_APC5                                           = 0x0000003b,
    FMT_APC6                                           = 0x0000003c,
    FMT_APC7                                           = 0x0000003d,
    FMT_CTX1                                           = 0x0000003e,
    FMT_RESERVED_63                                    = 0x0000003f,
} SurfaceFormat;

typedef enum SurfaceNumber {
    NUMBER_UNORM                                       = 0x00000000,
    NUMBER_SNORM                                       = 0x00000001,
    NUMBER_USCALED                                     = 0x00000002,
    NUMBER_SSCALED                                     = 0x00000003,
    NUMBER_UINT                                        = 0x00000004,
    NUMBER_SINT                                        = 0x00000005,
    NUMBER_SRGB                                        = 0x00000006,
    NUMBER_FLOAT                                       = 0x00000007,
} SurfaceNumber;

typedef enum SurfaceSwap {
    SWAP_STD                                           = 0x00000000,
    SWAP_ALT                                           = 0x00000001,
    SWAP_STD_REV                                       = 0x00000002,
    SWAP_ALT_REV                                       = 0x00000003,
} SurfaceSwap;

typedef enum SurfaceTiling {
    ARRAY_LINEAR                                       = 0x00000000,
    ARRAY_TILED                                        = 0x00000001,
} SurfaceTiling;

typedef enum SU_PERFCNT_SEL {
    PERF_PAPC_PASX_REQ                                 = 0x00000000,
    PERF_PAPC_PASX_VTX_KILL_DISCARD                    = 0x00000006,
    PERF_PAPC_PASX_VTX_NAN_DISCARD                     = 0x00000007,
    PERF_PAPC_CLPR_CULL_PRIM                           = 0x0000000e,
    PERF_PAPC_CLPR_VVUCP_CULL_PRIM                     = 0x0000000f,
    PERF_PAPC_CLPR_VV_CULL_PRIM                        = 0x00000010,
    PERF_PAPC_CLPR_UCP_CULL_PRIM                       = 0x00000011,
    PERF_PAPC_CLPR_VTX_KILL_CULL_PRIM                  = 0x00000012,
    PERF_PAPC_CLPR_VTX_NAN_CULL_PRIM                   = 0x00000013,
    PERF_PAPC_CLPR_CULL_TO_NULL_PRIM                   = 0x00000014,
    PERF_PAPC_CLPR_VVUCP_CLIP_PRIM                     = 0x00000015,
    PERF_PAPC_CLPR_VV_CLIP_PRIM                        = 0x00000016,
    PERF_PAPC_CLPR_UCP_CLIP_PRIM                       = 0x00000017,
    PERF_PAPC_CLPR_POINT_CLIP_CANDIDATE                = 0x00000018,
    PERF_PAPC_CLPR_CLIP_PLANE_CNT_1                    = 0x00000019,
    PERF_PAPC_CLPR_CLIP_PLANE_CNT_2                    = 0x0000001a,
    PERF_PAPC_CLPR_CLIP_PLANE_CNT_3                    = 0x0000001b,
    PERF_PAPC_CLPR_CLIP_PLANE_CNT_4                    = 0x0000001c,
    PERF_PAPC_CLPR_CLIP_PLANE_CNT_5_8                  = 0x0000001d,
    PERF_PAPC_CLPR_CLIP_PLANE_NEAR                     = 0x0000001f,
    PERF_PAPC_CLPR_CLIP_PLANE_FAR                      = 0x00000020,
    PERF_PAPC_CLPR_CLIP_PLANE_LEFT                     = 0x00000021,
    PERF_PAPC_CLPR_CLIP_PLANE_RIGHT                    = 0x00000022,
    PERF_PAPC_CLPR_CLIP_PLANE_TOP                      = 0x00000023,
    PERF_PAPC_CLPR_CLIP_PLANE_BOTTOM                   = 0x00000024,
    PERF_PAPC_CLPR_RASTER_KILL_CULL_PRIM               = 0x00000026,
    PERF_PAPC_CLSM_NULL_PRIM                           = 0x00000027,
    PERF_PAPC_CLSM_TOTALLY_VISIBLE_PRIM                = 0x00000028,
    PERF_PAPC_CLSM_CULL_TO_NULL_PRIM                   = 0x00000029,
    PERF_PAPC_CLSM_OUT_PRIM_CNT_1                      = 0x0000002a,
    PERF_PAPC_CLSM_OUT_PRIM_CNT_2                      = 0x0000002b,
    PERF_PAPC_CLSM_OUT_PRIM_CNT_3                      = 0x0000002c,
    PERF_PAPC_CLSM_OUT_PRIM_CNT_4                      = 0x0000002d,
    PERF_PAPC_CLSM_OUT_PRIM_CNT_5_8                    = 0x0000002e,
    PERF_PAPC_CLIPGA_VTE_KILL_PRIM                     = 0x00000030,
    PERF_PAPC_SU_INPUT_PRIM                            = 0x00000031,
    PERF_PAPC_SU_INPUT_CLIP_PRIM                       = 0x00000032,
    PERF_PAPC_SU_INPUT_NULL_PRIM                       = 0x00000033,
    PERF_PAPC_SU_INPUT_PRIM_DUAL                       = 0x00000034,
    PERF_PAPC_SU_INPUT_CLIP_PRIM_DUAL                  = 0x00000035,
    PERF_PAPC_SU_ZERO_AREA_CULL_PRIM                   = 0x00000036,
    PERF_PAPC_SU_BACK_FACE_CULL_PRIM                   = 0x00000037,
    PERF_PAPC_SU_FRONT_FACE_CULL_PRIM                  = 0x00000038,
    PERF_PAPC_SU_POLYMODE_FACE_CULL                    = 0x00000039,
    PERF_PAPC_SU_POLYMODE_BACK_CULL                    = 0x0000003a,
    PERF_PAPC_SU_POLYMODE_FRONT_CULL                   = 0x0000003b,
    PERF_PAPC_SU_POLYMODE_INVALID_FILL                 = 0x0000003c,
    PERF_PAPC_SU_OUTPUT_PRIM                           = 0x0000003d,
    PERF_PAPC_SU_OUTPUT_CLIP_PRIM                      = 0x0000003e,
    PERF_PAPC_SU_OUTPUT_NULL_PRIM                      = 0x0000003f,
    PERF_PAPC_SU_OUTPUT_EVENT_FLAG                     = 0x00000040,
    PERF_PAPC_SU_OUTPUT_FIRST_PRIM_SLOT                = 0x00000041,
    PERF_PAPC_SU_OUTPUT_END_OF_PACKET                  = 0x00000042,
    PERF_PAPC_SU_OUTPUT_POLYMODE_FACE                  = 0x00000043,
    PERF_PAPC_SU_OUTPUT_POLYMODE_BACK                  = 0x00000044,
    PERF_PAPC_SU_OUTPUT_POLYMODE_FRONT                 = 0x00000045,
    PERF_PAPC_SU_OUT_CLIP_POLYMODE_FACE                = 0x00000046,
    PERF_PAPC_SU_OUT_CLIP_POLYMODE_BACK                = 0x00000047,
    PERF_PAPC_SU_OUT_CLIP_POLYMODE_FRONT               = 0x00000048,
    PERF_PAPC_SU_OUTPUT_PRIM_DUAL                      = 0x00000049,
    PERF_PAPC_SU_OUTPUT_CLIP_PRIM_DUAL                 = 0x0000004a,
    PERF_PAPC_SU_OUTPUT_POLYMODE_DUAL                  = 0x0000004b,
    PERF_PAPC_SU_OUTPUT_CLIP_POLYMODE_DUAL             = 0x0000004c,
    PERF_PAPC_PASX_REQ_IDLE                            = 0x0000004d,
    PERF_PAPC_PASX_REQ_BUSY                            = 0x0000004e,
    PERF_PAPC_PASX_REQ_STALLED                         = 0x0000004f,
    PERF_PAPC_PASX_REC_IDLE                            = 0x00000050,
    PERF_PAPC_PASX_REC_BUSY                            = 0x00000051,
    PERF_PAPC_PASX_REC_STARVED_SX                      = 0x00000052,
    PERF_PAPC_PASX_REC_STALLED                         = 0x00000053,
    PERF_PAPC_PASX_REC_STALLED_POS_MEM                 = 0x00000054,
    PERF_PAPC_PASX_REC_STALLED_CCGSM_IN                = 0x00000055,
    PERF_PAPC_CCGSM_IDLE                               = 0x00000056,
    PERF_PAPC_CCGSM_BUSY                               = 0x00000057,
    PERF_PAPC_CCGSM_STALLED                            = 0x00000058,
    PERF_PAPC_CLPRIM_IDLE                              = 0x00000059,
    PERF_PAPC_CLPRIM_BUSY                              = 0x0000005a,
    PERF_PAPC_CLPRIM_STALLED                           = 0x0000005b,
    PERF_PAPC_CLPRIM_STARVED_CCGSM                     = 0x0000005c,
    PERF_PAPC_CLIPSM_IDLE                              = 0x0000005d,
    PERF_PAPC_CLIPSM_BUSY                              = 0x0000005e,
    PERF_PAPC_CLIPSM_WAIT_CLIP_VERT_ENGH               = 0x0000005f,
    PERF_PAPC_CLIPSM_WAIT_HIGH_PRI_SEQ                 = 0x00000060,
    PERF_PAPC_CLIPSM_WAIT_CLIPGA                       = 0x00000061,
    PERF_PAPC_CLIPSM_WAIT_AVAIL_VTE_CLIP               = 0x00000062,
    PERF_PAPC_CLIPSM_WAIT_CLIP_OUTSM                   = 0x00000063,
    PERF_PAPC_CLIPGA_IDLE                              = 0x00000064,
    PERF_PAPC_CLIPGA_BUSY                              = 0x00000065,
    PERF_PAPC_CLIPGA_STARVED_VTE_CLIP                  = 0x00000066,
    PERF_PAPC_CLIPGA_STALLED                           = 0x00000067,
    PERF_PAPC_CLIP_IDLE                                = 0x00000068,
    PERF_PAPC_CLIP_BUSY                                = 0x00000069,
    PERF_PAPC_SU_IDLE                                  = 0x0000006a,
    PERF_PAPC_SU_BUSY                                  = 0x0000006b,
    PERF_PAPC_SU_STARVED_CLIP                          = 0x0000006c,
    PERF_PAPC_SU_STALLED_SC                            = 0x0000006d,
    PERF_PAPC_CL_DYN_SCLK_VLD                          = 0x0000006e,
    PERF_PAPC_SU_DYN_SCLK_VLD                          = 0x0000006f,
    PERF_PAPC_PA_REG_SCLK_VLD                          = 0x00000070,
    PERF_PAPC_SU_SE0_PRIM_FILTER_CULL                  = 0x00000078,
    PERF_PAPC_SU_SE1_PRIM_FILTER_CULL                  = 0x00000079,
    PERF_PAPC_SU_SE0_OUTPUT_PRIM                       = 0x0000007b,
    PERF_PAPC_SU_SE1_OUTPUT_PRIM                       = 0x0000007c,
    PERF_PAPC_SU_SE0_OUTPUT_NULL_PRIM                  = 0x0000007e,
    PERF_PAPC_SU_SE1_OUTPUT_NULL_PRIM                  = 0x0000007f,
    PERF_PAPC_SU_SE0_STALLED_SC                        = 0x00000083,
    PERF_PAPC_SU_SE1_STALLED_SC                        = 0x00000084,
    PERF_PAPC_CLSM_CLIPPING_PRIM                       = 0x00000086,
    PERF_PAPC_SU_CULLED_PRIM                           = 0x00000087,
    PERF_PAPC_SU_OUTPUT_EOPG                           = 0x00000088,
    PERF_PAPC_SU_SE2_PRIM_FILTER_CULL                  = 0x00000089,
    PERF_PAPC_SU_SE3_PRIM_FILTER_CULL                  = 0x0000008a,
    PERF_PAPC_SU_SE2_OUTPUT_PRIM                       = 0x0000008b,
    PERF_PAPC_SU_SE3_OUTPUT_PRIM                       = 0x0000008c,
    PERF_PAPC_SU_SE2_OUTPUT_NULL_PRIM                  = 0x0000008d,
    PERF_PAPC_SU_SE3_OUTPUT_NULL_PRIM                  = 0x0000008e,
    PERF_PAPC_SU_SE2_STALLED_SC                        = 0x00000097,
    PERF_PAPC_SU_SE3_STALLED_SC                        = 0x00000098,
    PERF_SU_SMALL_PRIM_FILTER_CULL_CNT                 = 0x00000099,
    PERF_SMALL_PRIM_CULL_PRIM_1X1                      = 0x0000009a,
    PERF_SMALL_PRIM_CULL_PRIM_2X1                      = 0x0000009b,
    PERF_SMALL_PRIM_CULL_PRIM_1X2                      = 0x0000009c,
    PERF_SMALL_PRIM_CULL_PRIM_2X2                      = 0x0000009d,
    PERF_SMALL_PRIM_CULL_PRIM_3X1                      = 0x0000009e,
    PERF_SMALL_PRIM_CULL_PRIM_1X3                      = 0x0000009f,
    PERF_SMALL_PRIM_CULL_PRIM_3X2                      = 0x000000a0,
    PERF_SMALL_PRIM_CULL_PRIM_2X3                      = 0x000000a1,
    PERF_SMALL_PRIM_CULL_PRIM_NX1                      = 0x000000a2,
    PERF_SMALL_PRIM_CULL_PRIM_1XN                      = 0x000000a3,
    PERF_SMALL_PRIM_CULL_PRIM_NX2                      = 0x000000a4,
    PERF_SMALL_PRIM_CULL_PRIM_2XN                      = 0x000000a5,
    PERF_SC0_QUALIFIED_SEND_BUSY_EVENT                 = 0x000000a9,
    PERF_SC0_QUALIFIED_SEND_NOT_BUSY_EVENT             = 0x000000aa,
    PERF_SC1_QUALIFIED_SEND_BUSY_EVENT                 = 0x000000ab,
    PERF_SC1_QUALIFIED_SEND_NOT_BUSY_EVENT             = 0x000000ac,
    PERF_SC2_QUALIFIED_SEND_BUSY_EVENT                 = 0x000000ad,
    PERF_SC2_QUALIFIED_SEND_NOT_BUSY_EVENT             = 0x000000ae,
    PERF_SC3_QUALIFIED_SEND_BUSY_EVENT                 = 0x000000af,
    PERF_SC3_QUALIFIED_SEND_NOT_BUSY_EVENT             = 0x000000b0,
    PERF_PA_VERTEX_FIFO_FULL                           = 0x000000b1,
    PERF_PA_PRIMIC_TO_CLPRIM_FIFO_FULL                 = 0x000000b2,
    PERF_PA_FETCH_TO_PRIMIC_P_FIFO_FULL                = 0x000000b3,
    PERF_ENGG_CSB_MACHINE_IS_STARVED                   = 0x000000b7,
    PERF_ENGG_CSB_MACHINE_STALLED_BY_CSB_MEMORY        = 0x000000b8,
    PERF_ENGG_CSB_MACHINE_STALLED_BY_SPI               = 0x000000b9,
    PERF_ENGG_CSB_GE_INPUT_FIFO_FULL                   = 0x000000ba,
    PERF_ENGG_CSB_GE_INPUT_FIFO_POP_BIT                = 0x000000bd,
    PERF_ENGG_CSB_PRIM_COUNT_EQ0                       = 0x000000be,
    PERF_ENGG_CSB_NULL_SUBGROUP                        = 0x000000bf,
    PERF_ENGG_CSB_GE_SENDING_SUBGROUP                  = 0x000000c0,
    PERF_ENGG_CSB_GE_MEMORY_FULL                       = 0x000000c1,
    PERF_ENGG_CSB_GE_MEMORY_EMPTY                      = 0x000000c2,
    PERF_ENGG_CSB_SPI_MEMORY_FULL                      = 0x000000c3,
    PERF_ENGG_CSB_SPI_MEMORY_EMPTY                     = 0x000000c4,
    PERF_ENGG_INDEX_REQ_STARVED                        = 0x000000e5,
    PERF_ENGG_INDEX_REQ_IDLE_AND_STALLED_BY_REQ2RTN_FIFO_FULL = 0x000000e6,
    PERF_ENGG_INDEX_REQ_BUSY_AND_STALLED_BY_REQ2RTN_FIFO_FULL = 0x000000e7,
    PERF_ENGG_INDEX_REQ_STALLED_BY_SX_CREDITS          = 0x000000e8,
    PERF_ENGG_INDEX_RET_REQ2RTN_FIFO_FULL              = 0x000000e9,
    PERF_ENGG_INDEX_RET_REQ2RTN_FIFO_EMPTY             = 0x000000ea,
    PERF_ENGG_INDEX_RET_SX_RECEIVE_FIFO_FULL           = 0x000000eb,
    PERF_ENGG_INDEX_RET_SXRX_STARVED_BY_CSB            = 0x000000ec,
    PERF_ENGG_INDEX_RET_SXRX_STARVED_BY_PRIMS          = 0x000000ed,
    PERF_ENGG_INDEX_RET_SXRX_STALLED_BY_PRIM_INDICES_CSB_FIFO = 0x000000ee,
    PERF_ENGG_INDEX_RET_SXRX_STALLED_BY_PRIM_INDICES_FIFO = 0x000000ef,
    PERF_ENGG_INDEX_RET_SXRX_READING_EVENT             = 0x000000f0,
    PERF_ENGG_INDEX_RET_SXRX_READING_NULL_SUBGROUP     = 0x000000f1,
    PERF_ENGG_INDEX_RET_SXRX_READING_SUBGROUP_PRIMCOUNT_EQ0 = 0x000000f2,
    PERF_ENGG_INDEX_RET_SXRX_READING_QDWORD_0_VALID_PRIMS_NOPL = 0x000000f3,
    PERF_ENGG_INDEX_RET_SXRX_READING_QDWORD_1_VALID_PRIMS_NOPL = 0x000000f4,
    PERF_ENGG_INDEX_RET_SXRX_READING_QDWORD_2_VALID_PRIMS_NOPL = 0x000000f5,
    PERF_ENGG_INDEX_RET_SXRX_READING_QDWORD_3_VALID_PRIMS_NOPL = 0x000000f6,
    PERF_ENGG_INDEX_RET_SXRX_READING_QDWORD_4_VALID_PRIMS_NOPL = 0x000000f7,
    PERF_ENGG_INDEX_RET_SXRX_READING_QDWORD_0_VALID_PRIMS_PL = 0x000000f8,
    PERF_ENGG_INDEX_RET_SXRX_READING_QDWORD_1_VALID_PRIMS_PL = 0x000000f9,
    PERF_ENGG_INDEX_RET_SXRX_READING_QDWORD_2_VALID_PRIMS_PL = 0x000000fa,
    PERF_ENGG_INDEX_RET_SXRX_READING_QDWORD_3_VALID_PRIMS_PL = 0x000000fb,
    PERF_ENGG_INDEX_RET_SXRX_READING_QDWORD_4_VALID_PRIMS_PL = 0x000000fc,
    PERF_ENGG_INDEX_PRIM_IF_STALLED_BY_FULL_FETCH_TO_PRIMIC_P_FIFO = 0x00000102,
    PERF_ENGG_INDEX_PRIM_IF_STALLED_BY_FULL_FETCH_TO_PRIMIC_S_FIFO = 0x00000103,
    PERF_ENGG_INDEX_PRIM_IF_STARVED_BY_NO_CSB          = 0x00000104,
    PERF_ENGG_INDEX_PRIM_IF_STARVED_BY_NO_PRIM         = 0x00000105,
    PERF_ENGG_INDEX_PRIM_IF_FETCH_TO_PRIMIC_P_FIFO_WRITE = 0x00000106,
    PERF_ENGG_INDEX_PRIM_IF_FETCH_TO_PRIMIC_P_FIFO_NO_WRITE = 0x00000107,
    PERF_ENGG_POS_REQ_STARVED                          = 0x00000108,
    PERF_PAPC_PASX_DISABLE_PIPE__GFX10                 = 0x00000001,
    PERF_PAPC_PASX_FIRST_VECTOR__GFX10                 = 0x00000002,
    PERF_PAPC_PASX_SECOND_VECTOR__GFX10                = 0x00000003,
    PERF_PAPC_PASX_FIRST_DEAD__GFX10                   = 0x00000004,
    PERF_PAPC_PASX_SECOND_DEAD__GFX10                  = 0x00000005,
    PERF_PAPC_PA_INPUT_PRIM__GFX10                     = 0x00000008,
    PERF_PAPC_PA_INPUT_NULL_PRIM__GFX10                = 0x00000009,
    PERF_PAPC_PA_INPUT_EVENT_FLAG__GFX10               = 0x0000000a,
    PERF_PAPC_PA_INPUT_FIRST_PRIM_SLOT__GFX10          = 0x0000000b,
    PERF_PAPC_PA_INPUT_END_OF_PACKET__GFX10            = 0x0000000c,
    PERF_PAPC_PA_INPUT_EXTENDED_EVENT__GFX10           = 0x0000000d,
    PERF_PAPC_CLPR_CLIP_PLANE_CNT_9_12__GFX10          = 0x0000001e,
    PERF_PAPC_CLPR_GSC_KILL_CULL_PRIM__GFX10           = 0x00000025,
    PERF_PAPC_CLSM_OUT_PRIM_CNT_9_13__GFX10            = 0x0000002f,
    PERF_PAPC_SU_MULTI_GPU_PRIM_FILTER_CULL__GFX10     = 0x00000071,
    PERF_PAPC_PASX_SE0_REQ__GFX10                      = 0x00000072,
    PERF_PAPC_PASX_SE1_REQ__GFX10                      = 0x00000073,
    PERF_PAPC_PASX_SE0_FIRST_VECTOR__GFX10             = 0x00000074,
    PERF_PAPC_PASX_SE0_SECOND_VECTOR__GFX10            = 0x00000075,
    PERF_PAPC_PASX_SE1_FIRST_VECTOR__GFX10             = 0x00000076,
    PERF_PAPC_PASX_SE1_SECOND_VECTOR__GFX10            = 0x00000077,
    PERF_PAPC_SU_SE01_PRIM_FILTER_CULL__GFX10          = 0x0000007a,
    PERF_PAPC_SU_SE01_OUTPUT_PRIM__GFX10               = 0x0000007d,
    PERF_PAPC_SU_SE01_OUTPUT_NULL_PRIM__GFX10          = 0x00000080,
    PERF_PAPC_SU_SE0_OUTPUT_FIRST_PRIM_SLOT__GFX10     = 0x00000081,
    PERF_PAPC_SU_SE1_OUTPUT_FIRST_PRIM_SLOT__GFX10     = 0x00000082,
    PERF_PAPC_SU_SE01_STALLED_SC__GFX10                = 0x00000085,
    PERF_PAPC_SU_SE0_OUTPUT_END_OF_PACKET__GFX10       = 0x0000008f,
    PERF_PAPC_SU_SE1_OUTPUT_END_OF_PACKET__GFX10       = 0x00000090,
    PERF_PAPC_SU_SE2_OUTPUT_END_OF_PACKET__GFX10       = 0x00000091,
    PERF_PAPC_SU_SE3_OUTPUT_END_OF_PACKET__GFX10       = 0x00000092,
    PERF_PAPC_SU_SE0_OUTPUT_EOPG__GFX10                = 0x00000093,
    PERF_PAPC_SU_SE1_OUTPUT_EOPG__GFX10                = 0x00000094,
    PERF_PAPC_SU_SE2_OUTPUT_EOPG__GFX10                = 0x00000095,
    PERF_PAPC_SU_SE3_OUTPUT_EOPG__GFX10                = 0x00000096,
    PERF_SMALL_PRIM_CULL_PRIM_FULL_RES_EVENT__GFX10    = 0x000000a6,
    PERF_SMALL_PRIM_CULL_PRIM_HALF_RES_EVENT__GFX10    = 0x000000a7,
    PERF_SMALL_PRIM_CULL_PRIM_QUARTER_RES_EVENT__GFX10 = 0x000000a8,
    PERF_PA_FETCH_TO_SXIF_FIFO_FULL__GFX10             = 0x000000b4,
    PERF_ENGG_CSB_SPI_INPUT_FIFO_FULL__GFX10           = 0x000000bb,
    PERF_ENGG_CSB_DELAY_BIN00__GFX10                   = 0x000000c5,
    PERF_ENGG_CSB_DELAY_BIN01__GFX10                   = 0x000000c6,
    PERF_ENGG_CSB_DELAY_BIN02__GFX10                   = 0x000000c7,
    PERF_ENGG_CSB_DELAY_BIN03__GFX10                   = 0x000000c8,
    PERF_ENGG_CSB_DELAY_BIN04__GFX10                   = 0x000000c9,
    PERF_ENGG_CSB_DELAY_BIN05__GFX10                   = 0x000000ca,
    PERF_ENGG_CSB_DELAY_BIN06__GFX10                   = 0x000000cb,
    PERF_ENGG_CSB_DELAY_BIN07__GFX10                   = 0x000000cc,
    PERF_ENGG_CSB_DELAY_BIN08__GFX10                   = 0x000000cd,
    PERF_ENGG_CSB_DELAY_BIN09__GFX10                   = 0x000000ce,
    PERF_ENGG_CSB_DELAY_BIN10__GFX10                   = 0x000000cf,
    PERF_ENGG_CSB_DELAY_BIN11__GFX10                   = 0x000000d0,
    PERF_ENGG_CSB_DELAY_BIN12__GFX10                   = 0x000000d1,
    PERF_ENGG_CSB_DELAY_BIN13__GFX10                   = 0x000000d2,
    PERF_ENGG_CSB_DELAY_BIN14__GFX10                   = 0x000000d3,
    PERF_ENGG_CSB_DELAY_BIN15__GFX10                   = 0x000000d4,
    PERF_ENGG_CSB_SPI_DELAY_BIN00__GFX10               = 0x000000d5,
    PERF_ENGG_CSB_SPI_DELAY_BIN01__GFX10               = 0x000000d6,
    PERF_ENGG_CSB_SPI_DELAY_BIN02__GFX10               = 0x000000d7,
    PERF_ENGG_CSB_SPI_DELAY_BIN03__GFX10               = 0x000000d8,
    PERF_ENGG_CSB_SPI_DELAY_BIN04__GFX10               = 0x000000d9,
    PERF_ENGG_CSB_SPI_DELAY_BIN05__GFX10               = 0x000000da,
    PERF_ENGG_CSB_SPI_DELAY_BIN06__GFX10               = 0x000000db,
    PERF_ENGG_CSB_SPI_DELAY_BIN07__GFX10               = 0x000000dc,
    PERF_ENGG_CSB_SPI_DELAY_BIN08__GFX10               = 0x000000dd,
    PERF_ENGG_CSB_SPI_DELAY_BIN09__GFX10               = 0x000000de,
    PERF_ENGG_CSB_SPI_DELAY_BIN10__GFX10               = 0x000000df,
    PERF_ENGG_INDEX_RET_SXRX_READING_QDWORD_0_NULL_PRIMS__GFX10 = 0x000000fd,
    PERF_ENGG_INDEX_RET_SXRX_READING_QDWORD_1_NULL_PRIMS__GFX10 = 0x000000fe,
    PERF_ENGG_INDEX_RET_SXRX_READING_QDWORD_2_NULL_PRIMS__GFX10 = 0x000000ff,
    PERF_ENGG_INDEX_RET_SXRX_READING_QDWORD_3_NULL_PRIMS__GFX10 = 0x00000100,
    PERF_ENGG_INDEX_RET_SXRX_READING_QDWORD_4_NULL_PRIMS__GFX10 = 0x00000101,
    PERF_ENGG_CSB_SPI_DELAY_BIN11__GFX101              = 0x000000e0,
    PERF_ENGG_CSB_SPI_DELAY_BIN12__GFX101              = 0x000000e1,
    PERF_ENGG_CSB_SPI_DELAY_BIN13__GFX101              = 0x000000e2,
    PERF_ENGG_CSB_SPI_DELAY_BIN14__GFX101              = 0x000000e3,
    PERF_ENGG_CSB_SPI_DELAY_BIN15__GFX101              = 0x000000e4,
    PERF_ENGG_POS_REQ_STALLED_BY_FULL_CLIPV_FIFO__GFX101 = 0x00000109,
    PERF_ENGG_CSB_PAYLOAD_INPUT_FIFO_FULL__GFX103PLUS  = 0x000000bc,
    PERF_ENGG_INDEX_REQ_NULL_REQUEST__GFX103PLUS       = 0x000000e0,
    PERF_ENGG_INDEX_RET_SXRX_NULL_DROPPER_STALLED_BY_FULL_PRIM_FIFO__GFX103PLUS = 0x00000109,
    PERF_ENGG_BUSY__GFX103PLUS                         = 0x0000010a,
    PERF_CLIPSM_CULL_PRIMS_CNT__GFX103PLUS             = 0x0000010b,
    PERF_PH_SEND_1_SC__GFX103PLUS                      = 0x0000010c,
    PERF_PH_SEND_2_SC__GFX103PLUS                      = 0x0000010d,
    PERF_PH_SEND_3_SC__GFX103PLUS                      = 0x0000010e,
    PERF_PH_SEND_4_SC__GFX103PLUS                      = 0x0000010f,
    PERF_OUTPUT_PRIM_1_SC__GFX103PLUS                  = 0x00000110,
    PERF_OUTPUT_PRIM_2_SC__GFX103PLUS                  = 0x00000111,
    PERF_OUTPUT_PRIM_3_SC__GFX103PLUS                  = 0x00000112,
    PERF_OUTPUT_PRIM_4_SC__GFX103PLUS                  = 0x00000113,
    PERF_ENGG_INDEX_REQ_0_NEW_VERTS_THIS_PRIM__GFX10VRS = 0x000000e1,
    PERF_ENGG_INDEX_REQ_1_NEW_VERTS_THIS_PRIM__GFX10VRS = 0x000000e2,
    PERF_ENGG_INDEX_REQ_2_NEW_VERTS_THIS_PRIM__GFX10VRS = 0x000000e3,
    PERF_ENGG_INDEX_REQ_3_NEW_VERTS_THIS_PRIM__GFX10VRS = 0x000000e4,
    PERF_CLPR_INPUT_PRIM__GFX11                        = 0x00000008,
    PERF_CLPR_INPUT_NULL_PRIM__GFX11                   = 0x00000009,
    PERF_CLPR_INPUT_EVENT__GFX11                       = 0x0000000a,
    PERF_CLPR_INPUT_FIRST_OF_SUBGROUP__GFX11           = 0x0000000b,
    PERF_CLPR_INPUT_END_OF_PACKET__GFX11               = 0x0000000c,
    PERF_CLPR_INPUT_EXTENDED_EVENT__GFX11              = 0x0000000d,
    PERF_CLPR_CLIP_PLANE_CNT_9_PLUS__GFX11             = 0x0000001e,
    PERF_PAPC_CLSM_OUT_PRIM_CNT_9_PLUS__GFX11          = 0x0000002f,
    PERF_PAPC_SU_ALL_OUTPUT_PRIM__GFX11                = 0x0000007d,
    PERF_PAPC_SU_ALL_OUTPUT_NULL_PRIM__GFX11           = 0x00000080,
    PERF_PAPC_SU_ALL_STALLED_SC__GFX11                 = 0x00000085,
    PERF_ENGG_INDEX_RET_0_NEW_VERTS_THIS_PRIM__GFX11   = 0x000000e1,
    PERF_ENGG_INDEX_RET_1_NEW_VERTS_THIS_PRIM__GFX11   = 0x000000e2,
    PERF_ENGG_INDEX_RET_2_NEW_VERTS_THIS_PRIM__GFX11   = 0x000000e3,
    PERF_ENGG_INDEX_RET_3_NEW_VERTS_THIS_PRIM__GFX11   = 0x000000e4,
    PERF_PASX_POS_VECTOR__GFX11                        = 0x00000114,
    PERF_PASX_MISC_VECTOR__GFX11                       = 0x00000115,
    PERF_PASX_CCDIST0_VECTOR__GFX11                    = 0x00000116,
    PERF_PASX_CCDIST1_VECTOR__GFX11                    = 0x00000117,
    PERF_PASX_STEREO_POS_VECTOR__GFX11                 = 0x00000118,
    PERF_CLPR_INPUT_SEND__GFX11                        = 0x00000119,
    PERF_SU_INPUT_SEND__GFX11                          = 0x0000011a,
    PERF_SU_OUTPUT_SEND__GFX11                         = 0x0000011b,
    PERF_PAPC_SU_SE4_PRIM_FILTER_CULL__GFX11           = 0x0000011c,
    PERF_PAPC_SU_SE5_PRIM_FILTER_CULL__GFX11           = 0x0000011d,
    PERF_PAPC_SU_SE4_OUTPUT_PRIM__GFX11                = 0x0000011e,
    PERF_PAPC_SU_SE5_OUTPUT_PRIM__GFX11                = 0x0000011f,
    PERF_PAPC_SU_SE4_OUTPUT_NULL_PRIM__GFX11           = 0x00000120,
    PERF_PAPC_SU_SE5_OUTPUT_NULL_PRIM__GFX11           = 0x00000121,
    PERF_PAPC_SU_SE4_STALLED_SC__GFX11                 = 0x00000122,
    PERF_PAPC_SU_SE5_STALLED_SC__GFX11                 = 0x00000123,
    PERF_ENGG_INDEX_RET0_NEW_VERTS__GFX11              = 0x00000124,
    PERF_ENGG_INDEX_RET1_NEW_VERTS__GFX11              = 0x00000125,
    PERF_ENGG_INDEX_RET2_NEW_VERTS__GFX11              = 0x00000126,
    PERF_ENGG_INDEX_RET3_NEW_VERTS__GFX11              = 0x00000127,
    PERF_ENGG_INDEX_RET4_NEW_VERTS__GFX11              = 0x00000128,
    PERF_ENGG_INDEX_RET5_NEW_VERTS__GFX11              = 0x00000129,
    PERF_ENGG_INDEX_RET6_NEW_VERTS__GFX11              = 0x0000012a,
    PERF_ENGG_INDEX_RET7_NEW_VERTS__GFX11              = 0x0000012b,
    PERF_ENGG_INDEX_RET8_NEW_VERTS__GFX11              = 0x0000012c,
    PERF_ENGG_INDEX_RET9_NEW_VERTS__GFX11              = 0x0000012d,
    PERF_ENGG_INDEX_RET10_NEW_VERTS__GFX11             = 0x0000012e,
    PERF_ENGG_INDEX_RET11_NEW_VERTS__GFX11             = 0x0000012f,
    PERF_ENGG_INDEX_RET12_NEW_VERTS__GFX11             = 0x00000130,
    PERF_PH_SEND_5_SC__GFX11                           = 0x00000131,
    PERF_PH_SEND_6_SC__GFX11                           = 0x00000132,
    PERF_OUTPUT_PRIM_5_SC__GFX11                       = 0x00000133,
    PERF_OUTPUT_PRIM_6_SC__GFX11                       = 0x00000134,
    PERF_CLPR_BACK_PRIM__GFX11                         = 0x00000135,
    PERF_PA_BUSY__GFX11                                = 0x00000136,
} SU_PERFCNT_SEL;

constexpr unsigned int MaxSuPerfcntSelGfx101           = PERF_ENGG_POS_REQ_STALLED_BY_FULL_CLIPV_FIFO__GFX101;
constexpr unsigned int MaxSuPerfcntSelGfx10Vrs         = PERF_OUTPUT_PRIM_4_SC__GFX103PLUS;
constexpr unsigned int MaxSuPerfcntSelGfx11            = PERF_PA_BUSY__GFX11;

typedef enum SWIZZLE_MODE_ENUM {
    SW_LINEAR                                          = 0x00000000,
    SW_256B_S                                          = 0x00000001,
    SW_256B_D                                          = 0x00000002,
    SW_256B_R                                          = 0x00000003,
    SW_4KB_Z                                           = 0x00000004,
    SW_4KB_S                                           = 0x00000005,
    SW_4KB_D                                           = 0x00000006,
    SW_4KB_R                                           = 0x00000007,
    SW_64KB_Z                                          = 0x00000008,
    SW_64KB_S                                          = 0x00000009,
    SW_64KB_D                                          = 0x0000000a,
    SW_64KB_R                                          = 0x0000000b,
    SW_64KB_Z_T                                        = 0x00000010,
    SW_64KB_S_T                                        = 0x00000011,
    SW_64KB_D_T                                        = 0x00000012,
    SW_64KB_R_T                                        = 0x00000013,
    SW_4KB_Z_X                                         = 0x00000014,
    SW_4KB_S_X                                         = 0x00000015,
    SW_4KB_D_X                                         = 0x00000016,
    SW_4KB_R_X                                         = 0x00000017,
    SW_64KB_Z_X                                        = 0x00000018,
    SW_64KB_S_X                                        = 0x00000019,
    SW_64KB_D_X                                        = 0x0000001a,
    SW_64KB_R_X                                        = 0x0000001b,
    SW_VAR_Z__GFX10                                    = 0x0000000c,
    SW_VAR_S__GFX10                                    = 0x0000000d,
    SW_VAR_D__GFX10                                    = 0x0000000e,
    SW_VAR_R__GFX10                                    = 0x0000000f,
    SW_VAR_Z_X__GFX10                                  = 0x0000001c,
    SW_VAR_S_X__GFX10                                  = 0x0000001d,
    SW_VAR_D_X__GFX10                                  = 0x0000001e,
    SW_VAR_R_X__GFX10                                  = 0x0000001f,
    SW_256KB_Z__GFX11                                  = 0x0000000c,
    SW_256KB_S__GFX11                                  = 0x0000000d,
    SW_256KB_D__GFX11                                  = 0x0000000e,
    SW_256KB_R__GFX11                                  = 0x0000000f,
    SW_256KB_Z_X__GFX11                                = 0x0000001c,
    SW_256KB_S_X__GFX11                                = 0x0000001d,
    SW_256KB_D_X__GFX11                                = 0x0000001e,
    SW_256KB_R_X__GFX11                                = 0x0000001f,
} SWIZZLE_MODE_ENUM;

typedef enum SWIZZLE_TYPE_ENUM {
    SW_Z                                               = 0x00000000,
    SW_S                                               = 0x00000001,
    SW_D                                               = 0x00000002,
    SW_R                                               = 0x00000003,
    SW_L                                               = 0x00000004,
} SWIZZLE_TYPE_ENUM;

typedef enum SX_BLEND_OPT {
    BLEND_OPT_PRESERVE_NONE_IGNORE_ALL                 = 0x00000000,
    BLEND_OPT_PRESERVE_ALL_IGNORE_NONE                 = 0x00000001,
    BLEND_OPT_PRESERVE_C1_IGNORE_C0                    = 0x00000002,
    BLEND_OPT_PRESERVE_C0_IGNORE_C1                    = 0x00000003,
    BLEND_OPT_PRESERVE_A1_IGNORE_A0                    = 0x00000004,
    BLEND_OPT_PRESERVE_A0_IGNORE_A1                    = 0x00000005,
    BLEND_OPT_PRESERVE_NONE_IGNORE_A0                  = 0x00000006,
    BLEND_OPT_PRESERVE_NONE_IGNORE_NONE                = 0x00000007,
} SX_BLEND_OPT;

typedef enum SX_DOWNCONVERT_FORMAT {
    SX_RT_EXPORT_NO_CONVERSION                         = 0x00000000,
    SX_RT_EXPORT_32_R                                  = 0x00000001,
    SX_RT_EXPORT_32_A                                  = 0x00000002,
    SX_RT_EXPORT_10_11_11                              = 0x00000003,
    SX_RT_EXPORT_2_10_10_10                            = 0x00000004,
    SX_RT_EXPORT_8_8_8_8                               = 0x00000005,
    SX_RT_EXPORT_5_6_5                                 = 0x00000006,
    SX_RT_EXPORT_1_5_5_5                               = 0x00000007,
    SX_RT_EXPORT_4_4_4_4                               = 0x00000008,
    SX_RT_EXPORT_16_16_GR                              = 0x00000009,
    SX_RT_EXPORT_16_16_AR                              = 0x0000000a,
    SX_RT_EXPORT_9_9_9_E5__GFX103PLUS                  = 0x0000000b,
    SX_RT_EXPORT_2_10_10_10_7E3__GFX103PLUS            = 0x0000000c,
    SX_RT_EXPORT_2_10_10_10_6E4__GFX103PLUS            = 0x0000000d,
} SX_DOWNCONVERT_FORMAT;

typedef enum SX_OPT_COMB_FCN {
    OPT_COMB_NONE                                      = 0x00000000,
    OPT_COMB_ADD                                       = 0x00000001,
    OPT_COMB_SUBTRACT                                  = 0x00000002,
    OPT_COMB_MIN                                       = 0x00000003,
    OPT_COMB_MAX                                       = 0x00000004,
    OPT_COMB_REVSUBTRACT                               = 0x00000005,
    OPT_COMB_BLEND_DISABLED                            = 0x00000006,
    OPT_COMB_SAFE_ADD                                  = 0x00000007,
} SX_OPT_COMB_FCN;

typedef enum SX_PERFCOUNTER_VALS {
    SX_PERF_SEL_PA_IDLE_CYCLES                         = 0x00000000,
    SX_PERF_SEL_PA_REQ                                 = 0x00000001,
    SX_PERF_SEL_PA_POS                                 = 0x00000002,
    SX_PERF_SEL_CLOCK                                  = 0x00000003,
    SX_PERF_SEL_GATE_EN1                               = 0x00000004,
    SX_PERF_SEL_GATE_EN2                               = 0x00000005,
    SX_PERF_SEL_GATE_EN3                               = 0x00000006,
    SX_PERF_SEL_GATE_EN4                               = 0x00000007,
    SX_PERF_SEL_SH_POS_STARVE                          = 0x00000008,
    SX_PERF_SEL_SH_COLOR_STARVE                        = 0x00000009,
    SX_PERF_SEL_SH_POS_STALL                           = 0x0000000a,
    SX_PERF_SEL_SH_COLOR_STALL                         = 0x0000000b,
    SX_PERF_SEL_DB0_PIXELS                             = 0x0000000c,
    SX_PERF_SEL_DB0_HALF_QUADS                         = 0x0000000d,
    SX_PERF_SEL_DB0_PIXEL_STALL                        = 0x0000000e,
    SX_PERF_SEL_DB0_PIXEL_IDLE                         = 0x0000000f,
    SX_PERF_SEL_DB0_PRED_PIXELS                        = 0x00000010,
    SX_PERF_SEL_DB1_PIXELS                             = 0x00000011,
    SX_PERF_SEL_DB1_HALF_QUADS                         = 0x00000012,
    SX_PERF_SEL_DB1_PIXEL_STALL                        = 0x00000013,
    SX_PERF_SEL_DB1_PIXEL_IDLE                         = 0x00000014,
    SX_PERF_SEL_DB1_PRED_PIXELS                        = 0x00000015,
    SX_PERF_SEL_DB2_PIXELS                             = 0x00000016,
    SX_PERF_SEL_DB2_HALF_QUADS                         = 0x00000017,
    SX_PERF_SEL_DB2_PIXEL_STALL                        = 0x00000018,
    SX_PERF_SEL_DB2_PIXEL_IDLE                         = 0x00000019,
    SX_PERF_SEL_DB2_PRED_PIXELS                        = 0x0000001a,
    SX_PERF_SEL_DB3_PIXELS                             = 0x0000001b,
    SX_PERF_SEL_DB3_HALF_QUADS                         = 0x0000001c,
    SX_PERF_SEL_DB3_PIXEL_STALL                        = 0x0000001d,
    SX_PERF_SEL_DB3_PIXEL_IDLE                         = 0x0000001e,
    SX_PERF_SEL_DB3_PRED_PIXELS                        = 0x0000001f,
    SX_PERF_SEL_COL_BUSY                               = 0x00000020,
    SX_PERF_SEL_POS_BUSY                               = 0x00000021,
    SX_PERF_SEL_DB0_A2M_DISCARD_QUADS__GFX10           = 0x00000022,
    SX_PERF_SEL_DB0_MRT0_BLEND_BYPASS__GFX10           = 0x00000023,
    SX_PERF_SEL_DB0_MRT0_DONT_RD_DEST__GFX10           = 0x00000024,
    SX_PERF_SEL_DB0_MRT0_DISCARD_SRC__GFX10            = 0x00000025,
    SX_PERF_SEL_DB0_MRT0_SINGLE_QUADS__GFX10           = 0x00000026,
    SX_PERF_SEL_DB0_MRT0_DOUBLE_QUADS__GFX10           = 0x00000027,
    SX_PERF_SEL_DB0_MRT1_BLEND_BYPASS__GFX10           = 0x00000028,
    SX_PERF_SEL_DB0_MRT1_DONT_RD_DEST__GFX10           = 0x00000029,
    SX_PERF_SEL_DB0_MRT1_DISCARD_SRC__GFX10            = 0x0000002a,
    SX_PERF_SEL_DB0_MRT1_SINGLE_QUADS__GFX10           = 0x0000002b,
    SX_PERF_SEL_DB0_MRT1_DOUBLE_QUADS__GFX10           = 0x0000002c,
    SX_PERF_SEL_DB0_MRT2_BLEND_BYPASS__GFX10           = 0x0000002d,
    SX_PERF_SEL_DB0_MRT2_DONT_RD_DEST__GFX10           = 0x0000002e,
    SX_PERF_SEL_DB0_MRT2_DISCARD_SRC__GFX10            = 0x0000002f,
    SX_PERF_SEL_DB0_MRT2_SINGLE_QUADS__GFX10           = 0x00000030,
    SX_PERF_SEL_DB0_MRT2_DOUBLE_QUADS__GFX10           = 0x00000031,
    SX_PERF_SEL_DB0_MRT3_BLEND_BYPASS__GFX10           = 0x00000032,
    SX_PERF_SEL_DB0_MRT3_DONT_RD_DEST__GFX10           = 0x00000033,
    SX_PERF_SEL_DB0_MRT3_DISCARD_SRC__GFX10            = 0x00000034,
    SX_PERF_SEL_DB0_MRT3_SINGLE_QUADS__GFX10           = 0x00000035,
    SX_PERF_SEL_DB0_MRT3_DOUBLE_QUADS__GFX10           = 0x00000036,
    SX_PERF_SEL_DB0_MRT4_BLEND_BYPASS__GFX10           = 0x00000037,
    SX_PERF_SEL_DB0_MRT4_DONT_RD_DEST__GFX10           = 0x00000038,
    SX_PERF_SEL_DB0_MRT4_DISCARD_SRC__GFX10            = 0x00000039,
    SX_PERF_SEL_DB0_MRT4_SINGLE_QUADS__GFX10           = 0x0000003a,
    SX_PERF_SEL_DB0_MRT4_DOUBLE_QUADS__GFX10           = 0x0000003b,
    SX_PERF_SEL_DB0_MRT5_BLEND_BYPASS__GFX10           = 0x0000003c,
    SX_PERF_SEL_DB0_MRT5_DONT_RD_DEST__GFX10           = 0x0000003d,
    SX_PERF_SEL_DB0_MRT5_DISCARD_SRC__GFX10            = 0x0000003e,
    SX_PERF_SEL_DB0_MRT5_SINGLE_QUADS__GFX10           = 0x0000003f,
    SX_PERF_SEL_DB0_MRT5_DOUBLE_QUADS__GFX10           = 0x00000040,
    SX_PERF_SEL_DB0_MRT6_BLEND_BYPASS__GFX10           = 0x00000041,
    SX_PERF_SEL_DB0_MRT6_DONT_RD_DEST__GFX10           = 0x00000042,
    SX_PERF_SEL_DB0_MRT6_DISCARD_SRC__GFX10            = 0x00000043,
    SX_PERF_SEL_DB0_MRT6_SINGLE_QUADS__GFX10           = 0x00000044,
    SX_PERF_SEL_DB0_MRT6_DOUBLE_QUADS__GFX10           = 0x00000045,
    SX_PERF_SEL_DB0_MRT7_BLEND_BYPASS__GFX10           = 0x00000046,
    SX_PERF_SEL_DB0_MRT7_DONT_RD_DEST__GFX10           = 0x00000047,
    SX_PERF_SEL_DB0_MRT7_DISCARD_SRC__GFX10            = 0x00000048,
    SX_PERF_SEL_DB0_MRT7_SINGLE_QUADS__GFX10           = 0x00000049,
    SX_PERF_SEL_DB0_MRT7_DOUBLE_QUADS__GFX10           = 0x0000004a,
    SX_PERF_SEL_DB1_A2M_DISCARD_QUADS__GFX10           = 0x0000004b,
    SX_PERF_SEL_DB1_MRT0_BLEND_BYPASS__GFX10           = 0x0000004c,
    SX_PERF_SEL_DB1_MRT0_DONT_RD_DEST__GFX10           = 0x0000004d,
    SX_PERF_SEL_DB1_MRT0_DISCARD_SRC__GFX10            = 0x0000004e,
    SX_PERF_SEL_DB1_MRT0_SINGLE_QUADS__GFX10           = 0x0000004f,
    SX_PERF_SEL_DB1_MRT0_DOUBLE_QUADS__GFX10           = 0x00000050,
    SX_PERF_SEL_DB1_MRT1_BLEND_BYPASS__GFX10           = 0x00000051,
    SX_PERF_SEL_DB1_MRT1_DONT_RD_DEST__GFX10           = 0x00000052,
    SX_PERF_SEL_DB1_MRT1_DISCARD_SRC__GFX10            = 0x00000053,
    SX_PERF_SEL_DB1_MRT1_SINGLE_QUADS__GFX10           = 0x00000054,
    SX_PERF_SEL_DB1_MRT1_DOUBLE_QUADS__GFX10           = 0x00000055,
    SX_PERF_SEL_DB1_MRT2_BLEND_BYPASS__GFX10           = 0x00000056,
    SX_PERF_SEL_DB1_MRT2_DONT_RD_DEST__GFX10           = 0x00000057,
    SX_PERF_SEL_DB1_MRT2_DISCARD_SRC__GFX10            = 0x00000058,
    SX_PERF_SEL_DB1_MRT2_SINGLE_QUADS__GFX10           = 0x00000059,
    SX_PERF_SEL_DB1_MRT2_DOUBLE_QUADS__GFX10           = 0x0000005a,
    SX_PERF_SEL_DB1_MRT3_BLEND_BYPASS__GFX10           = 0x0000005b,
    SX_PERF_SEL_DB1_MRT3_DONT_RD_DEST__GFX10           = 0x0000005c,
    SX_PERF_SEL_DB1_MRT3_DISCARD_SRC__GFX10            = 0x0000005d,
    SX_PERF_SEL_DB1_MRT3_SINGLE_QUADS__GFX10           = 0x0000005e,
    SX_PERF_SEL_DB1_MRT3_DOUBLE_QUADS__GFX10           = 0x0000005f,
    SX_PERF_SEL_DB1_MRT4_BLEND_BYPASS__GFX10           = 0x00000060,
    SX_PERF_SEL_DB1_MRT4_DONT_RD_DEST__GFX10           = 0x00000061,
    SX_PERF_SEL_DB1_MRT4_DISCARD_SRC__GFX10            = 0x00000062,
    SX_PERF_SEL_DB1_MRT4_SINGLE_QUADS__GFX10           = 0x00000063,
    SX_PERF_SEL_DB1_MRT4_DOUBLE_QUADS__GFX10           = 0x00000064,
    SX_PERF_SEL_DB1_MRT5_BLEND_BYPASS__GFX10           = 0x00000065,
    SX_PERF_SEL_DB1_MRT5_DONT_RD_DEST__GFX10           = 0x00000066,
    SX_PERF_SEL_DB1_MRT5_DISCARD_SRC__GFX10            = 0x00000067,
    SX_PERF_SEL_DB1_MRT5_SINGLE_QUADS__GFX10           = 0x00000068,
    SX_PERF_SEL_DB1_MRT5_DOUBLE_QUADS__GFX10           = 0x00000069,
    SX_PERF_SEL_DB1_MRT6_BLEND_BYPASS__GFX10           = 0x0000006a,
    SX_PERF_SEL_DB1_MRT6_DONT_RD_DEST__GFX10           = 0x0000006b,
    SX_PERF_SEL_DB1_MRT6_DISCARD_SRC__GFX10            = 0x0000006c,
    SX_PERF_SEL_DB1_MRT6_SINGLE_QUADS__GFX10           = 0x0000006d,
    SX_PERF_SEL_DB1_MRT6_DOUBLE_QUADS__GFX10           = 0x0000006e,
    SX_PERF_SEL_DB1_MRT7_BLEND_BYPASS__GFX10           = 0x0000006f,
    SX_PERF_SEL_DB1_MRT7_DONT_RD_DEST__GFX10           = 0x00000070,
    SX_PERF_SEL_DB1_MRT7_DISCARD_SRC__GFX10            = 0x00000071,
    SX_PERF_SEL_DB1_MRT7_SINGLE_QUADS__GFX10           = 0x00000072,
    SX_PERF_SEL_DB1_MRT7_DOUBLE_QUADS__GFX10           = 0x00000073,
    SX_PERF_SEL_DB2_A2M_DISCARD_QUADS__GFX10           = 0x00000074,
    SX_PERF_SEL_DB2_MRT0_BLEND_BYPASS__GFX10           = 0x00000075,
    SX_PERF_SEL_DB2_MRT0_DONT_RD_DEST__GFX10           = 0x00000076,
    SX_PERF_SEL_DB2_MRT0_DISCARD_SRC__GFX10            = 0x00000077,
    SX_PERF_SEL_DB2_MRT0_SINGLE_QUADS__GFX10           = 0x00000078,
    SX_PERF_SEL_DB2_MRT0_DOUBLE_QUADS__GFX10           = 0x00000079,
    SX_PERF_SEL_DB2_MRT1_BLEND_BYPASS__GFX10           = 0x0000007a,
    SX_PERF_SEL_DB2_MRT1_DONT_RD_DEST__GFX10           = 0x0000007b,
    SX_PERF_SEL_DB2_MRT1_DISCARD_SRC__GFX10            = 0x0000007c,
    SX_PERF_SEL_DB2_MRT1_SINGLE_QUADS__GFX10           = 0x0000007d,
    SX_PERF_SEL_DB2_MRT1_DOUBLE_QUADS__GFX10           = 0x0000007e,
    SX_PERF_SEL_DB2_MRT2_BLEND_BYPASS__GFX10           = 0x0000007f,
    SX_PERF_SEL_DB2_MRT2_DONT_RD_DEST__GFX10           = 0x00000080,
    SX_PERF_SEL_DB2_MRT2_DISCARD_SRC__GFX10            = 0x00000081,
    SX_PERF_SEL_DB2_MRT2_SINGLE_QUADS__GFX10           = 0x00000082,
    SX_PERF_SEL_DB2_MRT2_DOUBLE_QUADS__GFX10           = 0x00000083,
    SX_PERF_SEL_DB2_MRT3_BLEND_BYPASS__GFX10           = 0x00000084,
    SX_PERF_SEL_DB2_MRT3_DONT_RD_DEST__GFX10           = 0x00000085,
    SX_PERF_SEL_DB2_MRT3_DISCARD_SRC__GFX10            = 0x00000086,
    SX_PERF_SEL_DB2_MRT3_SINGLE_QUADS__GFX10           = 0x00000087,
    SX_PERF_SEL_DB2_MRT3_DOUBLE_QUADS__GFX10           = 0x00000088,
    SX_PERF_SEL_DB2_MRT4_BLEND_BYPASS__GFX10           = 0x00000089,
    SX_PERF_SEL_DB2_MRT4_DONT_RD_DEST__GFX10           = 0x0000008a,
    SX_PERF_SEL_DB2_MRT4_DISCARD_SRC__GFX10            = 0x0000008b,
    SX_PERF_SEL_DB2_MRT4_SINGLE_QUADS__GFX10           = 0x0000008c,
    SX_PERF_SEL_DB2_MRT4_DOUBLE_QUADS__GFX10           = 0x0000008d,
    SX_PERF_SEL_DB2_MRT5_BLEND_BYPASS__GFX10           = 0x0000008e,
    SX_PERF_SEL_DB2_MRT5_DONT_RD_DEST__GFX10           = 0x0000008f,
    SX_PERF_SEL_DB2_MRT5_DISCARD_SRC__GFX10            = 0x00000090,
    SX_PERF_SEL_DB2_MRT5_SINGLE_QUADS__GFX10           = 0x00000091,
    SX_PERF_SEL_DB2_MRT5_DOUBLE_QUADS__GFX10           = 0x00000092,
    SX_PERF_SEL_DB2_MRT6_BLEND_BYPASS__GFX10           = 0x00000093,
    SX_PERF_SEL_DB2_MRT6_DONT_RD_DEST__GFX10           = 0x00000094,
    SX_PERF_SEL_DB2_MRT6_DISCARD_SRC__GFX10            = 0x00000095,
    SX_PERF_SEL_DB2_MRT6_SINGLE_QUADS__GFX10           = 0x00000096,
    SX_PERF_SEL_DB2_MRT6_DOUBLE_QUADS__GFX10           = 0x00000097,
    SX_PERF_SEL_DB2_MRT7_BLEND_BYPASS__GFX10           = 0x00000098,
    SX_PERF_SEL_DB2_MRT7_DONT_RD_DEST__GFX10           = 0x00000099,
    SX_PERF_SEL_DB2_MRT7_DISCARD_SRC__GFX10            = 0x0000009a,
    SX_PERF_SEL_DB2_MRT7_SINGLE_QUADS__GFX10           = 0x0000009b,
    SX_PERF_SEL_DB2_MRT7_DOUBLE_QUADS__GFX10           = 0x0000009c,
    SX_PERF_SEL_DB3_A2M_DISCARD_QUADS__GFX10           = 0x0000009d,
    SX_PERF_SEL_DB3_MRT0_BLEND_BYPASS__GFX10           = 0x0000009e,
    SX_PERF_SEL_DB3_MRT0_DONT_RD_DEST__GFX10           = 0x0000009f,
    SX_PERF_SEL_DB3_MRT0_DISCARD_SRC__GFX10            = 0x000000a0,
    SX_PERF_SEL_DB3_MRT0_SINGLE_QUADS__GFX10           = 0x000000a1,
    SX_PERF_SEL_DB3_MRT0_DOUBLE_QUADS__GFX10           = 0x000000a2,
    SX_PERF_SEL_DB3_MRT1_BLEND_BYPASS__GFX10           = 0x000000a3,
    SX_PERF_SEL_DB3_MRT1_DONT_RD_DEST__GFX10           = 0x000000a4,
    SX_PERF_SEL_DB3_MRT1_DISCARD_SRC__GFX10            = 0x000000a5,
    SX_PERF_SEL_DB3_MRT1_SINGLE_QUADS__GFX10           = 0x000000a6,
    SX_PERF_SEL_DB3_MRT1_DOUBLE_QUADS__GFX10           = 0x000000a7,
    SX_PERF_SEL_DB3_MRT2_BLEND_BYPASS__GFX10           = 0x000000a8,
    SX_PERF_SEL_DB3_MRT2_DONT_RD_DEST__GFX10           = 0x000000a9,
    SX_PERF_SEL_DB3_MRT2_DISCARD_SRC__GFX10            = 0x000000aa,
    SX_PERF_SEL_DB3_MRT2_SINGLE_QUADS__GFX10           = 0x000000ab,
    SX_PERF_SEL_DB3_MRT2_DOUBLE_QUADS__GFX10           = 0x000000ac,
    SX_PERF_SEL_DB3_MRT3_BLEND_BYPASS__GFX10           = 0x000000ad,
    SX_PERF_SEL_DB3_MRT3_DONT_RD_DEST__GFX10           = 0x000000ae,
    SX_PERF_SEL_DB3_MRT3_DISCARD_SRC__GFX10            = 0x000000af,
    SX_PERF_SEL_DB3_MRT3_SINGLE_QUADS__GFX10           = 0x000000b0,
    SX_PERF_SEL_DB3_MRT3_DOUBLE_QUADS__GFX10           = 0x000000b1,
    SX_PERF_SEL_DB3_MRT4_BLEND_BYPASS__GFX10           = 0x000000b2,
    SX_PERF_SEL_DB3_MRT4_DONT_RD_DEST__GFX10           = 0x000000b3,
    SX_PERF_SEL_DB3_MRT4_DISCARD_SRC__GFX10            = 0x000000b4,
    SX_PERF_SEL_DB3_MRT4_SINGLE_QUADS__GFX10           = 0x000000b5,
    SX_PERF_SEL_DB3_MRT4_DOUBLE_QUADS__GFX10           = 0x000000b6,
    SX_PERF_SEL_DB3_MRT5_BLEND_BYPASS__GFX10           = 0x000000b7,
    SX_PERF_SEL_DB3_MRT5_DONT_RD_DEST__GFX10           = 0x000000b8,
    SX_PERF_SEL_DB3_MRT5_DISCARD_SRC__GFX10            = 0x000000b9,
    SX_PERF_SEL_DB3_MRT5_SINGLE_QUADS__GFX10           = 0x000000ba,
    SX_PERF_SEL_DB3_MRT5_DOUBLE_QUADS__GFX10           = 0x000000bb,
    SX_PERF_SEL_DB3_MRT6_BLEND_BYPASS__GFX10           = 0x000000bc,
    SX_PERF_SEL_DB3_MRT6_DONT_RD_DEST__GFX10           = 0x000000bd,
    SX_PERF_SEL_DB3_MRT6_DISCARD_SRC__GFX10            = 0x000000be,
    SX_PERF_SEL_DB3_MRT6_SINGLE_QUADS__GFX10           = 0x000000bf,
    SX_PERF_SEL_DB3_MRT6_DOUBLE_QUADS__GFX10           = 0x000000c0,
    SX_PERF_SEL_DB3_MRT7_BLEND_BYPASS__GFX10           = 0x000000c1,
    SX_PERF_SEL_DB3_MRT7_DONT_RD_DEST__GFX10           = 0x000000c2,
    SX_PERF_SEL_DB3_MRT7_DISCARD_SRC__GFX10            = 0x000000c3,
    SX_PERF_SEL_DB3_MRT7_SINGLE_QUADS__GFX10           = 0x000000c4,
    SX_PERF_SEL_DB3_MRT7_DOUBLE_QUADS__GFX10           = 0x000000c5,
    SX_PERF_SEL_PA_REQ_LATENCY__GFX10                  = 0x000000c6,
    SX_PERF_SEL_POS_SCBD_STALL__GFX10                  = 0x000000c7,
    SX_PERF_SEL_COL_SCBD_STALL__GFX10                  = 0x000000c8,
    SX_PERF_SEL_CLOCK_DROP_STALL__GFX10                = 0x000000c9,
    SX_PERF_SEL_GATE_EN5__GFX10                        = 0x000000ca,
    SX_PERF_SEL_GATE_EN6__GFX10                        = 0x000000cb,
    SX_PERF_SEL_DB0_SIZE__GFX10                        = 0x000000cc,
    SX_PERF_SEL_DB1_SIZE__GFX10                        = 0x000000cd,
    SX_PERF_SEL_DB2_SIZE__GFX10                        = 0x000000ce,
    SX_PERF_SEL_DB3_SIZE__GFX10                        = 0x000000cf,
    SX_PERF_SEL_SPLITMODE__GFX10                       = 0x000000d0,
    SX_PERF_SEL_COL_SCBD0_STALL__GFX10                 = 0x000000d1,
    SX_PERF_SEL_COL_SCBD1_STALL__GFX10                 = 0x000000d2,
    SX_PERF_SEL_IDX_STALL_CYCLES__GFX10                = 0x000000d3,
    SX_PERF_SEL_IDX_IDLE_CYCLES__GFX10                 = 0x000000d4,
    SX_PERF_SEL_IDX_REQ__GFX10                         = 0x000000d5,
    SX_PERF_SEL_IDX_RET__GFX10                         = 0x000000d6,
    SX_PERF_SEL_IDX_REQ_LATENCY__GFX10                 = 0x000000d7,
    SX_PERF_SEL_IDX_SCBD_STALL__GFX10                  = 0x000000d8,
    SX_PERF_SEL_GATE_EN7__GFX10                        = 0x000000d9,
    SX_PERF_SEL_GATE_EN8__GFX10                        = 0x000000da,
    SX_PERF_SEL_SH_IDX_STARVE__GFX10                   = 0x000000db,
    SX_PERF_SEL_IDX_BUSY__GFX10                        = 0x000000dc,
    SX_PERF_SEL_RB0_STALL_DUE_TO_ORDERING__GFX10       = 0x000000dd,
    SX_PERF_SEL_RB1_STALL_DUE_TO_ORDERING__GFX10       = 0x000000de,
    SX_PERF_SEL_RB2_STALL_DUE_TO_ORDERING__GFX10       = 0x000000df,
    SX_PERF_SEL_RB3_STALL_DUE_TO_ORDERING__GFX10       = 0x000000e0,
    SX_PERF_SEL_DB0_MRT_BLEND_BYPASS__GFX11            = 0x00000022,
    SX_PERF_SEL_DB0_MRT_DONT_RD_DEST__GFX11            = 0x00000023,
    SX_PERF_SEL_DB0_MRT_DISCARD_SRC__GFX11             = 0x00000024,
    SX_PERF_SEL_DB0_MRT_SINGLE_QUADS__GFX11            = 0x00000025,
    SX_PERF_SEL_DB0_MRT_DOUBLE_QUADS__GFX11            = 0x00000026,
    SX_PERF_SEL_DB1_MRT_BLEND_BYPASS__GFX11            = 0x00000027,
    SX_PERF_SEL_DB1_MRT_DONT_RD_DEST__GFX11            = 0x00000028,
    SX_PERF_SEL_DB1_MRT_DISCARD_SRC__GFX11             = 0x00000029,
    SX_PERF_SEL_DB1_MRT_SINGLE_QUADS__GFX11            = 0x0000002a,
    SX_PERF_SEL_DB1_MRT_DOUBLE_QUADS__GFX11            = 0x0000002b,
    SX_PERF_SEL_DB2_MRT_BLEND_BYPASS__GFX11            = 0x0000002c,
    SX_PERF_SEL_DB2_MRT_DONT_RD_DEST__GFX11            = 0x0000002d,
    SX_PERF_SEL_DB2_MRT_DISCARD_SRC__GFX11             = 0x0000002e,
    SX_PERF_SEL_DB2_MRT_SINGLE_QUADS__GFX11            = 0x0000002f,
    SX_PERF_SEL_DB2_MRT_DOUBLE_QUADS__GFX11            = 0x00000030,
    SX_PERF_SEL_DB3_MRT_BLEND_BYPASS__GFX11            = 0x00000031,
    SX_PERF_SEL_DB3_MRT_DONT_RD_DEST__GFX11            = 0x00000032,
    SX_PERF_SEL_DB3_MRT_DISCARD_SRC__GFX11             = 0x00000033,
    SX_PERF_SEL_DB3_MRT_SINGLE_QUADS__GFX11            = 0x00000034,
    SX_PERF_SEL_DB3_MRT_DOUBLE_QUADS__GFX11            = 0x00000035,
    SX_PERF_SEL_PA_REQ_LATENCY__GFX11                  = 0x00000036,
    SX_PERF_SEL_POS_SCBD_STALL__GFX11                  = 0x00000037,
    SX_PERF_SEL_CLOCK_DROP_STALL__GFX11                = 0x00000038,
    SX_PERF_SEL_GATE_EN5__GFX11                        = 0x00000039,
    SX_PERF_SEL_GATE_EN6__GFX11                        = 0x0000003a,
    SX_PERF_SEL_DB0_SIZE__GFX11                        = 0x0000003b,
    SX_PERF_SEL_DB1_SIZE__GFX11                        = 0x0000003c,
    SX_PERF_SEL_DB2_SIZE__GFX11                        = 0x0000003d,
    SX_PERF_SEL_DB3_SIZE__GFX11                        = 0x0000003e,
    SX_PERF_SEL_IDX_STALL_CYCLES__GFX11                = 0x0000003f,
    SX_PERF_SEL_IDX_IDLE_CYCLES__GFX11                 = 0x00000040,
    SX_PERF_SEL_IDX_REQ__GFX11                         = 0x00000041,
    SX_PERF_SEL_IDX_RET__GFX11                         = 0x00000042,
    SX_PERF_SEL_IDX_REQ_LATENCY__GFX11                 = 0x00000043,
    SX_PERF_SEL_IDX_SCBD_STALL__GFX11                  = 0x00000044,
    SX_PERF_SEL_GATE_EN7__GFX11                        = 0x00000045,
    SX_PERF_SEL_GATE_EN8__GFX11                        = 0x00000046,
    SX_PERF_SEL_SH_IDX_STARVE__GFX11                   = 0x00000047,
    SX_PERF_SEL_IDX_BUSY__GFX11                        = 0x00000048,
    SX_PERF_SEL_PA_POS_BANK_CONF__GFX11                = 0x00000049,
    SX_PERF_SEL_DB0_END_OF_WAVE__GFX11                 = 0x0000004a,
    SX_PERF_SEL_DB0_4X2_DISCARD__GFX11                 = 0x0000004b,
    SX_PERF_SEL_DB1_END_OF_WAVE__GFX11                 = 0x0000004c,
    SX_PERF_SEL_DB1_4X2_DISCARD__GFX11                 = 0x0000004d,
    SX_PERF_SEL_DB2_END_OF_WAVE__GFX11                 = 0x0000004e,
    SX_PERF_SEL_DB2_4X2_DISCARD__GFX11                 = 0x0000004f,
    SX_PERF_SEL_DB3_END_OF_WAVE__GFX11                 = 0x00000050,
    SX_PERF_SEL_DB3_4X2_DISCARD__GFX11                 = 0x00000051,
} SX_PERFCOUNTER_VALS;

constexpr unsigned int MaxSxPerfcounterValsGfx10       = SX_PERF_SEL_RB3_STALL_DUE_TO_ORDERING__GFX10;
constexpr unsigned int MaxSxPerfcounterValsGfx11       = SX_PERF_SEL_DB3_4X2_DISCARD__GFX11;

typedef enum TA_PERFCOUNT_SEL {
    TA_PERF_SEL_NULL                                   = 0x00000000,
    TA_PERF_SEL_gradient_busy                          = 0x00000007,
    TA_PERF_SEL_gradient_fifo_busy                     = 0x00000008,
    TA_PERF_SEL_lod_busy                               = 0x00000009,
    TA_PERF_SEL_lod_fifo_busy                          = 0x0000000a,
    TA_PERF_SEL_addresser_busy                         = 0x0000000b,
    TA_PERF_SEL_addresser_fifo_busy                    = 0x0000000c,
    TA_PERF_SEL_aligner_busy                           = 0x0000000d,
    TA_PERF_SEL_write_path_busy                        = 0x0000000e,
    TA_PERF_SEL_ta_busy                                = 0x0000000f,
    TA_PERF_SEL_total_wavefronts                       = 0x00000020,
    TA_PERF_SEL_gradient_cycles                        = 0x00000021,
    TA_PERF_SEL_walker_cycles                          = 0x00000022,
    TA_PERF_SEL_aligner_cycles                         = 0x00000023,
    TA_PERF_SEL_image_wavefronts                       = 0x00000024,
    TA_PERF_SEL_image_read_wavefronts                  = 0x00000025,
    TA_PERF_SEL_image_atomic_wavefronts                = 0x00000027,
    TA_PERF_SEL_image_sampler_total_cycles             = 0x00000028,
    TA_PERF_SEL_image_nosampler_total_cycles           = 0x00000029,
    TA_PERF_SEL_flat_total_cycles                      = 0x0000002a,
    TA_PERF_SEL_buffer_wavefronts                      = 0x0000002c,
    TA_PERF_SEL_buffer_atomic_wavefronts               = 0x0000002f,
    TA_PERF_SEL_buffer_total_cycles                    = 0x00000031,
    TA_PERF_SEL_addr_stalled_by_tc_cycles              = 0x00000036,
    TA_PERF_SEL_addr_stalled_by_td_cycles              = 0x00000037,
    TA_PERF_SEL_addresser_stalled_by_aligner_only_cycles = 0x00000039,
    TA_PERF_SEL_addresser_stalled_cycles               = 0x0000003a,
    TA_PERF_SEL_aniso_stalled_by_addresser_only_cycles = 0x0000003b,
    TA_PERF_SEL_aniso_stalled_cycles                   = 0x0000003c,
    TA_PERF_SEL_deriv_stalled_by_aniso_only_cycles     = 0x0000003d,
    TA_PERF_SEL_deriv_stalled_cycles                   = 0x0000003e,
    TA_PERF_SEL_aniso_gt1_cycle_quads                  = 0x0000003f,
    TA_PERF_SEL_mipmap_lod_0_samples                   = 0x00000049,
    TA_PERF_SEL_mipmap_lod_1_samples                   = 0x0000004a,
    TA_PERF_SEL_mipmap_lod_2_samples                   = 0x0000004b,
    TA_PERF_SEL_mipmap_lod_3_samples                   = 0x0000004c,
    TA_PERF_SEL_mipmap_lod_4_samples                   = 0x0000004d,
    TA_PERF_SEL_mipmap_lod_5_samples                   = 0x0000004e,
    TA_PERF_SEL_mipmap_lod_6_samples                   = 0x0000004f,
    TA_PERF_SEL_mipmap_lod_7_samples                   = 0x00000050,
    TA_PERF_SEL_mipmap_lod_8_samples                   = 0x00000051,
    TA_PERF_SEL_mipmap_lod_9_samples                   = 0x00000052,
    TA_PERF_SEL_mipmap_lod_10_samples                  = 0x00000053,
    TA_PERF_SEL_mipmap_lod_11_samples                  = 0x00000054,
    TA_PERF_SEL_mipmap_lod_12_samples                  = 0x00000055,
    TA_PERF_SEL_mipmap_lod_13_samples                  = 0x00000056,
    TA_PERF_SEL_mipmap_lod_14_samples                  = 0x00000057,
    TA_PERF_SEL_mipmap_invalid_samples                 = 0x00000058,
    TA_PERF_SEL_aniso_1_cycle_quads                    = 0x00000059,
    TA_PERF_SEL_aniso_2_cycle_quads                    = 0x0000005a,
    TA_PERF_SEL_aniso_4_cycle_quads                    = 0x0000005b,
    TA_PERF_SEL_aniso_6_cycle_quads                    = 0x0000005c,
    TA_PERF_SEL_aniso_8_cycle_quads                    = 0x0000005d,
    TA_PERF_SEL_aniso_10_cycle_quads                   = 0x0000005e,
    TA_PERF_SEL_aniso_12_cycle_quads                   = 0x0000005f,
    TA_PERF_SEL_aniso_14_cycle_quads                   = 0x00000060,
    TA_PERF_SEL_aniso_16_cycle_quads                   = 0x00000061,
    TA_PERF_SEL_flat_wavefronts                        = 0x00000064,
    TA_PERF_SEL_flat_atomic_wavefronts                 = 0x00000067,
    TA_PERF_SEL_in_busy                                = 0x00000080,
    TA_PERF_SEL_in_fifos_busy                          = 0x00000081,
    TA_PERF_SEL_in_cfifo_busy                          = 0x00000082,
    TA_PERF_SEL_in_qfifo_busy                          = 0x00000083,
    TA_PERF_SEL_in_wfifo_busy                          = 0x00000084,
    TA_PERF_SEL_in_rfifo_busy                          = 0x00000085,
    TA_PERF_SEL_bf_busy                                = 0x00000086,
    TA_PERF_SEL_ns_busy                                = 0x00000087,
    TA_PERF_SEL_smp_busy_ns_idle                       = 0x00000088,
    TA_PERF_SEL_smp_idle_ns_busy                       = 0x00000089,
    TA_PERF_SEL_vmemcmd_cycles                         = 0x00000090,
    TA_PERF_SEL_vmemreq_cycles                         = 0x00000091,
    TA_PERF_SEL_in_waiting_on_req_cycles               = 0x00000092,
    TA_PERF_SEL_in_addr_cycles                         = 0x00000096,
    TA_PERF_SEL_in_data_cycles                         = 0x00000097,
    TA_PERF_SEL_point_sampled_quads                    = 0x000000a0,
    TA_PERF_SEL_RESERVED_31__GFX10                     = 0x0000001f,
    TA_PERF_SEL_RESERVED_48__GFX10                     = 0x00000030,
    TA_PERF_SEL_RESERVED_67__GFX10                     = 0x00000043,
    TA_PERF_SEL_RESERVED_112__GFX10                    = 0x00000070,
    TA_PERF_SEL_RESERVED_113__GFX10                    = 0x00000071,
    TA_PERF_SEL_RESERVED_138__GFX10                    = 0x0000008a,
    TA_PERF_SEL_RESERVED_139__GFX10                    = 0x0000008b,
    TA_PERF_SEL_RESERVED_140__GFX10                    = 0x0000008c,
    TA_PERF_SEL_RESERVED_141__GFX10                    = 0x0000008d,
    TA_PERF_SEL_RESERVED_142__GFX10                    = 0x0000008e,
    TA_PERF_SEL_RESERVED_143__GFX10                    = 0x0000008f,
    TA_PERF_SEL_RESERVED_149__GFX10                    = 0x00000095,
    TA_PERF_SEL_RESERVED_153__GFX10                    = 0x00000099,
    TA_PERF_SEL_RESERVED_161__GFX10                    = 0x000000a1,
    TA_PERF_SEL_RESERVED_166__GFX10                    = 0x000000a6,
    TA_PERF_SEL_RESERVED_167__GFX10                    = 0x000000a7,
    TA_PERF_SEL_RESERVED_168__GFX10                    = 0x000000a8,
    TA_PERF_SEL_RESERVED_169__GFX10                    = 0x000000a9,
    TA_PERF_SEL_RESERVED_170__GFX10                    = 0x000000aa,
    TA_PERF_SEL_RESERVED_171__GFX10                    = 0x000000ab,
    TA_PERF_SEL_RESERVED_172__GFX10                    = 0x000000ac,
    TA_PERF_SEL_RESERVED_222__GFX10                    = 0x000000de,
    TA_PERF_SEL_RESERVED_223__GFX10                    = 0x000000df,
    TA_PERF_SEL_RESERVED_1__GFX101                     = 0x00000001,
    TA_PERF_SEL_RESERVED_2__GFX101                     = 0x00000002,
    TA_PERF_SEL_RESERVED_3__GFX101                     = 0x00000003,
    TA_PERF_SEL_RESERVED_4__GFX101                     = 0x00000004,
    TA_PERF_SEL_RESERVED_5__GFX101                     = 0x00000005,
    TA_PERF_SEL_RESERVED_6__GFX101                     = 0x00000006,
    TA_PERF_SEL_RESERVED_16__GFX101                    = 0x00000010,
    TA_PERF_SEL_RESERVED_17__GFX101                    = 0x00000011,
    TA_PERF_SEL_RESERVED_18__GFX101                    = 0x00000012,
    TA_PERF_SEL_RESERVED_19__GFX101                    = 0x00000013,
    TA_PERF_SEL_RESERVED_20__GFX101                    = 0x00000014,
    TA_PERF_SEL_RESERVED_21__GFX101                    = 0x00000015,
    TA_PERF_SEL_RESERVED_22__GFX101                    = 0x00000016,
    TA_PERF_SEL_RESERVED_23__GFX101                    = 0x00000017,
    TA_PERF_SEL_RESERVED_24__GFX101                    = 0x00000018,
    TA_PERF_SEL_RESERVED_25__GFX101                    = 0x00000019,
    TA_PERF_SEL_RESERVED_26__GFX101                    = 0x0000001a,
    TA_PERF_SEL_RESERVED_27__GFX101                    = 0x0000001b,
    TA_PERF_SEL_RESERVED_28__GFX101                    = 0x0000001c,
    TA_PERF_SEL_RESERVED_29__GFX101                    = 0x0000001d,
    TA_PERF_SEL_RESERVED_30__GFX101                    = 0x0000001e,
    TA_PERF_SEL_image_write_wavefronts__GFX101         = 0x00000026,
    TA_PERF_SEL_RESERVED_43__GFX101                    = 0x0000002b,
    TA_PERF_SEL_buffer_read_wavefronts__GFX101         = 0x0000002d,
    TA_PERF_SEL_buffer_write_wavefronts__GFX101        = 0x0000002e,
    TA_PERF_SEL_RESERVED_50__GFX101                    = 0x00000032,
    TA_PERF_SEL_RESERVED_51__GFX101                    = 0x00000033,
    TA_PERF_SEL_RESERVED_52__GFX101                    = 0x00000034,
    TA_PERF_SEL_RESERVED_53__GFX101                    = 0x00000035,
    TA_PERF_SEL_RESERVED_56__GFX101                    = 0x00000038,
    TA_PERF_SEL_color_1_cycle_pixels__GFX101           = 0x00000040,
    TA_PERF_SEL_color_2_cycle_pixels__GFX101           = 0x00000041,
    TA_PERF_SEL_color_3_cycle_pixels__GFX101           = 0x00000042,
    TA_PERF_SEL_mip_1_cycle_pixels__GFX101             = 0x00000044,
    TA_PERF_SEL_mip_2_cycle_pixels__GFX101             = 0x00000045,
    TA_PERF_SEL_vol_1_cycle_pixels__GFX101             = 0x00000046,
    TA_PERF_SEL_vol_2_cycle_pixels__GFX101             = 0x00000047,
    TA_PERF_SEL_bilin_point_1_cycle_pixels__GFX101     = 0x00000048,
    TA_PERF_SEL_write_path_input_cycles__GFX101        = 0x00000062,
    TA_PERF_SEL_write_path_output_cycles__GFX101       = 0x00000063,
    TA_PERF_SEL_flat_read_wavefronts__GFX101           = 0x00000065,
    TA_PERF_SEL_flat_write_wavefronts__GFX101          = 0x00000066,
    TA_PERF_SEL_RESERVED_104__GFX101                   = 0x00000068,
    TA_PERF_SEL_reg_sclk_vld__GFX101                   = 0x00000069,
    TA_PERF_SEL_local_cg_dyn_sclk_grp0_en__GFX101      = 0x0000006a,
    TA_PERF_SEL_local_cg_dyn_sclk_grp1_en__GFX101      = 0x0000006b,
    TA_PERF_SEL_local_cg_dyn_sclk_grp1_mems_en__GFX101 = 0x0000006c,
    TA_PERF_SEL_local_cg_dyn_sclk_grp4_en__GFX101      = 0x0000006d,
    TA_PERF_SEL_local_cg_dyn_sclk_grp5_en__GFX101      = 0x0000006e,
    TA_PERF_SEL_xnack__GFX101                          = 0x0000006f,
    TA_PERF_SEL_RESERVED_114__GFX101                   = 0x00000072,
    TA_PERF_SEL_first_xnack__GFX101                    = 0x00000073,
    TA_PERF_SEL_RESERVED_116__GFX101                   = 0x00000074,
    TA_PERF_SEL_RESERVED_117__GFX101                   = 0x00000075,
    TA_PERF_SEL_RESERVED_118__GFX101                   = 0x00000076,
    TA_PERF_SEL_RESERVED_119__GFX101                   = 0x00000077,
    TA_PERF_SEL_RESERVED_120__GFX101                   = 0x00000078,
    TA_PERF_SEL_RESERVED_121__GFX101                   = 0x00000079,
    TA_PERF_SEL_RESERVED_122__GFX101                   = 0x0000007a,
    TA_PERF_SEL_RESERVED_123__GFX101                   = 0x0000007b,
    TA_PERF_SEL_RESERVED_124__GFX101                   = 0x0000007c,
    TA_PERF_SEL_RESERVED_125__GFX101                   = 0x0000007d,
    TA_PERF_SEL_RESERVED_126__GFX101                   = 0x0000007e,
    TA_PERF_SEL_RESERVED_127__GFX101                   = 0x0000007f,
    TA_PERF_SEL_in_waiting_on_cmd_cycles__GFX101       = 0x00000093,
    TA_PERF_SEL_in_waiting_on_any_cycles__GFX101       = 0x00000094,
    TA_PERF_SEL_addr_stalled_by_xnack_cycles__GFX101   = 0x00000098,
    TA_PERF_SEL_RESERVED_154__GFX101                   = 0x0000009a,
    TA_PERF_SEL_RESERVED_155__GFX101                   = 0x0000009b,
    TA_PERF_SEL_RESERVED_156__GFX101                   = 0x0000009c,
    TA_PERF_SEL_RESERVED_157__GFX101                   = 0x0000009d,
    TA_PERF_SEL_RESERVED_158__GFX101                   = 0x0000009e,
    TA_PERF_SEL_RESERVED_159__GFX101                   = 0x0000009f,
    TA_PERF_SEL_RESERVED_162__GFX101                   = 0x000000a2,
    TA_PERF_SEL_RESERVED_163__GFX101                   = 0x000000a3,
    TA_PERF_SEL_RESERVED_164__GFX101                   = 0x000000a4,
    TA_PERF_SEL_RESERVED_165__GFX101                   = 0x000000a5,
    TA_PERF_SEL_RESERVED_173__GFX101                   = 0x000000ad,
    TA_PERF_SEL_RESERVED_174__GFX101                   = 0x000000ae,
    TA_PERF_SEL_RESERVED_175__GFX101                   = 0x000000af,
    TA_PERF_SEL_RESERVED_176__GFX101                   = 0x000000b0,
    TA_PERF_SEL_RESERVED_177__GFX101                   = 0x000000b1,
    TA_PERF_SEL_RESERVED_178__GFX101                   = 0x000000b2,
    TA_PERF_SEL_RESERVED_179__GFX101                   = 0x000000b3,
    TA_PERF_SEL_RESERVED_180__GFX101                   = 0x000000b4,
    TA_PERF_SEL_RESERVED_181__GFX101                   = 0x000000b5,
    TA_PERF_SEL_RESERVED_182__GFX101                   = 0x000000b6,
    TA_PERF_SEL_RESERVED_183__GFX101                   = 0x000000b7,
    TA_PERF_SEL_RESERVED_184__GFX101                   = 0x000000b8,
    TA_PERF_SEL_RESERVED_185__GFX101                   = 0x000000b9,
    TA_PERF_SEL_RESERVED_186__GFX101                   = 0x000000ba,
    TA_PERF_SEL_RESERVED_187__GFX101                   = 0x000000bb,
    TA_PERF_SEL_RESERVED_188__GFX101                   = 0x000000bc,
    TA_PERF_SEL_RESERVED_189__GFX101                   = 0x000000bd,
    TA_PERF_SEL_RESERVED_190__GFX101                   = 0x000000be,
    TA_PERF_SEL_RESERVED_191__GFX101                   = 0x000000bf,
    TA_PERF_SEL_RESERVED_192__GFX101                   = 0x000000c0,
    TA_PERF_SEL_RESERVED_193__GFX101                   = 0x000000c1,
    TA_PERF_SEL_RESERVED_194__GFX101                   = 0x000000c2,
    TA_PERF_SEL_RESERVED_195__GFX101                   = 0x000000c3,
    TA_PERF_SEL_RESERVED_196__GFX101                   = 0x000000c4,
    TA_PERF_SEL_RESERVED_197__GFX101                   = 0x000000c5,
    TA_PERF_SEL_RESERVED_198__GFX101                   = 0x000000c6,
    TA_PERF_SEL_RESERVED_199__GFX101                   = 0x000000c7,
    TA_PERF_SEL_RESERVED_200__GFX101                   = 0x000000c8,
    TA_PERF_SEL_RESERVED_201__GFX101                   = 0x000000c9,
    TA_PERF_SEL_RESERVED_202__GFX101                   = 0x000000ca,
    TA_PERF_SEL_RESERVED_203__GFX101                   = 0x000000cb,
    TA_PERF_SEL_RESERVED_204__GFX101                   = 0x000000cc,
    TA_PERF_SEL_RESERVED_205__GFX101                   = 0x000000cd,
    TA_PERF_SEL_RESERVED_206__GFX101                   = 0x000000ce,
    TA_PERF_SEL_RESERVED_207__GFX101                   = 0x000000cf,
    TA_PERF_SEL_RESERVED_208__GFX101                   = 0x000000d0,
    TA_PERF_SEL_RESERVED_209__GFX101                   = 0x000000d1,
    TA_PERF_SEL_RESERVED_210__GFX101                   = 0x000000d2,
    TA_PERF_SEL_RESERVED_211__GFX101                   = 0x000000d3,
    TA_PERF_SEL_RESERVED_212__GFX101                   = 0x000000d4,
    TA_PERF_SEL_RESERVED_213__GFX101                   = 0x000000d5,
    TA_PERF_SEL_RESERVED_214__GFX101                   = 0x000000d6,
    TA_PERF_SEL_RESERVED_215__GFX101                   = 0x000000d7,
    TA_PERF_SEL_RESERVED_216__GFX101                   = 0x000000d8,
    TA_PERF_SEL_RESERVED_217__GFX101                   = 0x000000d9,
    TA_PERF_SEL_RESERVED_218__GFX101                   = 0x000000da,
    TA_PERF_SEL_RESERVED_219__GFX101                   = 0x000000db,
    TA_PERF_SEL_RESERVED_220__GFX101                   = 0x000000dc,
    TA_PERF_SEL_RESERVED_221__GFX101                   = 0x000000dd,
    TA_PERF_SEL_sample_path_cycles__GFX101             = 0x000000e0,
    TA_PERF_SEL_nosample_path_cycles__GFX101           = 0x000000e1,
    TA_PERF_SEL_image_sampler_has_offset_instructions__GFX103PLUS = 0x00000001,
    TA_PERF_SEL_image_sampler_has_bias_instructions__GFX103PLUS = 0x00000002,
    TA_PERF_SEL_image_sampler_has_reference_instructions__GFX103PLUS = 0x00000003,
    TA_PERF_SEL_image_sampler_has_ds_instructions__GFX103PLUS = 0x00000004,
    TA_PERF_SEL_image_sampler_has_dt_instructions__GFX103PLUS = 0x00000005,
    TA_PERF_SEL_image_sampler_has_dr_instructions__GFX103PLUS = 0x00000006,
    TA_PERF_SEL_image_sampler_1_input_vgpr_instructions__GFX103PLUS = 0x00000010,
    TA_PERF_SEL_image_sampler_2_input_vgpr_instructions__GFX103PLUS = 0x00000011,
    TA_PERF_SEL_image_sampler_3_input_vgpr_instructions__GFX103PLUS = 0x00000012,
    TA_PERF_SEL_image_sampler_4_input_vgpr_instructions__GFX103PLUS = 0x00000013,
    TA_PERF_SEL_image_sampler_5_input_vgpr_instructions__GFX103PLUS = 0x00000014,
    TA_PERF_SEL_image_sampler_6_input_vgpr_instructions__GFX103PLUS = 0x00000015,
    TA_PERF_SEL_image_sampler_7_input_vgpr_instructions__GFX103PLUS = 0x00000016,
    TA_PERF_SEL_image_sampler_8_input_vgpr_instructions__GFX103PLUS = 0x00000017,
    TA_PERF_SEL_image_sampler_9_input_vgpr_instructions__GFX103PLUS = 0x00000018,
    TA_PERF_SEL_image_sampler_10_input_vgpr_instructions__GFX103PLUS = 0x00000019,
    TA_PERF_SEL_image_sampler_11_input_vgpr_instructions__GFX103PLUS = 0x0000001a,
    TA_PERF_SEL_image_sampler_12_input_vgpr_instructions__GFX103PLUS = 0x0000001b,
    TA_PERF_SEL_image_sampler_has_t_instructions__GFX103PLUS = 0x0000001c,
    TA_PERF_SEL_image_sampler_has_r_instructions__GFX103PLUS = 0x0000001d,
    TA_PERF_SEL_image_sampler_has_q_instructions__GFX103PLUS = 0x0000001e,
    TA_PERF_SEL_image_store_wavefronts__GFX103PLUS     = 0x00000026,
    TA_PERF_SEL_bvh_total_cycles__GFX103PLUS           = 0x0000002b,
    TA_PERF_SEL_buffer_load_wavefronts__GFX103PLUS     = 0x0000002d,
    TA_PERF_SEL_buffer_store_wavefronts__GFX103PLUS    = 0x0000002e,
    TA_PERF_SEL_buffer_1_address_input_vgpr_instructions__GFX103PLUS = 0x00000032,
    TA_PERF_SEL_buffer_2_address_input_vgpr_instructions__GFX103PLUS = 0x00000033,
    TA_PERF_SEL_buffer_has_index_instructions__GFX103PLUS = 0x00000034,
    TA_PERF_SEL_buffer_has_offset_instructions__GFX103PLUS = 0x00000035,
    TA_PERF_SEL_image_sampler_wavefronts__GFX103PLUS   = 0x00000038,
    TA_PERF_SEL_color_1_cycle_quads__GFX103PLUS        = 0x00000040,
    TA_PERF_SEL_color_2_cycle_quads__GFX103PLUS        = 0x00000041,
    TA_PERF_SEL_color_3_cycle_quads__GFX103PLUS        = 0x00000042,
    TA_PERF_SEL_mip_1_cycle_quads__GFX103PLUS          = 0x00000044,
    TA_PERF_SEL_mip_2_cycle_quads__GFX103PLUS          = 0x00000045,
    TA_PERF_SEL_vol_1_cycle_quads__GFX103PLUS          = 0x00000046,
    TA_PERF_SEL_vol_2_cycle_quads__GFX103PLUS          = 0x00000047,
    TA_PERF_SEL_sampler_op_quads__GFX103PLUS           = 0x00000048,
    TA_PERF_SEL_store_write_data_input_cycles__GFX103PLUS = 0x00000062,
    TA_PERF_SEL_store_write_data_output_cycles__GFX103PLUS = 0x00000063,
    TA_PERF_SEL_flat_load_wavefronts__GFX103PLUS       = 0x00000065,
    TA_PERF_SEL_flat_store_wavefronts__GFX103PLUS      = 0x00000066,
    TA_PERF_SEL_flat_1_address_input_vgpr_instructions__GFX103PLUS = 0x00000068,
    TA_PERF_SEL_register_clk_valid_cycles__GFX103PLUS  = 0x00000069,
    TA_PERF_SEL_non_harvestable_clk_enabled_cycles__GFX103PLUS = 0x0000006a,
    TA_PERF_SEL_harvestable_clk_enabled_cycles__GFX103PLUS = 0x0000006b,
    TA_PERF_SEL_harvestable_register_clk_enabled_cycles__GFX103PLUS = 0x0000006c,
    TA_PERF_SEL_boundary_non_harvestable_clk_enabled_cycles__GFX103PLUS = 0x0000006d,
    TA_PERF_SEL_boundary_harvestable_clk_enabled_cycles__GFX103PLUS = 0x0000006e,
    TA_PERF_SEL_store_2_write_data_vgpr_instructions__GFX103PLUS = 0x00000072,
    TA_PERF_SEL_store_3_write_data_vgpr_instructions__GFX103PLUS = 0x00000073,
    TA_PERF_SEL_store_4_write_data_vgpr_instructions__GFX103PLUS = 0x00000074,
    TA_PERF_SEL_store_has_x_instructions__GFX103PLUS   = 0x00000075,
    TA_PERF_SEL_store_has_y_instructions__GFX103PLUS   = 0x00000076,
    TA_PERF_SEL_store_has_z_instructions__GFX103PLUS   = 0x00000077,
    TA_PERF_SEL_store_has_w_instructions__GFX103PLUS   = 0x00000078,
    TA_PERF_SEL_image_nosampler_has_t_instructions__GFX103PLUS = 0x00000079,
    TA_PERF_SEL_image_nosampler_has_r_instructions__GFX103PLUS = 0x0000007a,
    TA_PERF_SEL_image_nosampler_has_q_instructions__GFX103PLUS = 0x0000007b,
    TA_PERF_SEL_image_nosampler_1_address_input_vgpr_instructions__GFX103PLUS = 0x0000007c,
    TA_PERF_SEL_image_nosampler_2_address_input_vgpr_instructions__GFX103PLUS = 0x0000007d,
    TA_PERF_SEL_image_nosampler_3_address_input_vgpr_instructions__GFX103PLUS = 0x0000007e,
    TA_PERF_SEL_image_nosampler_4_address_input_vgpr_instructions__GFX103PLUS = 0x0000007f,
    TA_PERF_SEL_latency_ram_weights_written_cycles__GFX103PLUS = 0x0000009a,
    TA_PERF_SEL_latency_ram_ws_required_quads__GFX103PLUS = 0x0000009b,
    TA_PERF_SEL_latency_ram_whv_required_quads__GFX103PLUS = 0x0000009c,
    TA_PERF_SEL_latency_ram_ws_required_instructions__GFX103PLUS = 0x0000009d,
    TA_PERF_SEL_latency_ram_whv_required_instructions__GFX103PLUS = 0x0000009e,
    TA_PERF_SEL_latency_ram_ref_required_instructions__GFX103PLUS = 0x0000009f,
    TA_PERF_SEL_atomic_2_write_data_vgpr_instructions__GFX103PLUS = 0x000000a2,
    TA_PERF_SEL_atomic_4_write_data_vgpr_instructions__GFX103PLUS = 0x000000a3,
    TA_PERF_SEL_atomic_write_data_input_cycles__GFX103PLUS = 0x000000a4,
    TA_PERF_SEL_atomic_write_data_output_cycles__GFX103PLUS = 0x000000a5,
    TA_PERF_SEL_num_unlit_nodes_ta_opt__GFX103PLUS     = 0x000000ad,
    TA_PERF_SEL_num_nodes_invalidated_due_to_bad_input__GFX103PLUS = 0x000000ae,
    TA_PERF_SEL_num_nodes_invalidated_due_to_oob__GFX103PLUS = 0x000000af,
    TA_PERF_SEL_num_of_bvh_valid_first_tri__GFX103PLUS = 0x000000b0,
    TA_PERF_SEL_num_of_bvh_valid_second_tri__GFX103PLUS = 0x000000b1,
    TA_PERF_SEL_num_of_bvh_valid_third_tri__GFX103PLUS = 0x000000b2,
    TA_PERF_SEL_num_of_bvh_valid_fourth_tri__GFX103PLUS = 0x000000b3,
    TA_PERF_SEL_num_of_bvh_valid_fp16_box__GFX103PLUS  = 0x000000b4,
    TA_PERF_SEL_num_of_bvh_valid_fp32_box__GFX103PLUS  = 0x000000b5,
    TA_PERF_SEL_num_of_bvh_invalidated_first_tri__GFX103PLUS = 0x000000b6,
    TA_PERF_SEL_num_of_bvh_invalidated_second_tri__GFX103PLUS = 0x000000b7,
    TA_PERF_SEL_num_of_bvh_invalidated_third_tri__GFX103PLUS = 0x000000b8,
    TA_PERF_SEL_num_of_bvh_invalidated_fourth_tri__GFX103PLUS = 0x000000b9,
    TA_PERF_SEL_num_of_bvh_invalidated_fp16_box__GFX103PLUS = 0x000000ba,
    TA_PERF_SEL_num_of_bvh_invalidated_fp32_box__GFX103PLUS = 0x000000bb,
    TA_PERF_SEL_image_bvh_8_input_vgpr_instructions__GFX103PLUS = 0x000000bc,
    TA_PERF_SEL_image_bvh_9_input_vgpr_instructions__GFX103PLUS = 0x000000bd,
    TA_PERF_SEL_image_bvh_11_input_vgpr_instructions__GFX103PLUS = 0x000000be,
    TA_PERF_SEL_image_bvh_12_input_vgpr_instructions__GFX103PLUS = 0x000000bf,
    TA_PERF_SEL_image_sampler_1_op_burst__GFX103PLUS   = 0x000000c0,
    TA_PERF_SEL_image_sampler_2to3_op_burst__GFX103PLUS = 0x000000c1,
    TA_PERF_SEL_image_sampler_4to7_op_burst__GFX103PLUS = 0x000000c2,
    TA_PERF_SEL_image_sampler_ge8_op_burst__GFX103PLUS = 0x000000c3,
    TA_PERF_SEL_image_linked_1_op_burst__GFX103PLUS    = 0x000000c4,
    TA_PERF_SEL_image_linked_2to3_op_burst__GFX103PLUS = 0x000000c5,
    TA_PERF_SEL_image_linked_4to7_op_burst__GFX103PLUS = 0x000000c6,
    TA_PERF_SEL_image_linked_ge8_op_burst__GFX103PLUS  = 0x000000c7,
    TA_PERF_SEL_image_bvh_1_op_burst__GFX103PLUS       = 0x000000c8,
    TA_PERF_SEL_image_bvh_2to3_op_burst__GFX103PLUS    = 0x000000c9,
    TA_PERF_SEL_image_bvh_4to7_op_burst__GFX103PLUS    = 0x000000ca,
    TA_PERF_SEL_image_bvh_ge8_op_burst__GFX103PLUS     = 0x000000cb,
    TA_PERF_SEL_image_nosampler_1_op_burst__GFX103PLUS = 0x000000cc,
    TA_PERF_SEL_image_nosampler_2to3_op_burst__GFX103PLUS = 0x000000cd,
    TA_PERF_SEL_image_nosampler_4to31_op_burst__GFX103PLUS = 0x000000ce,
    TA_PERF_SEL_image_nosampler_ge32_op_burst__GFX103PLUS = 0x000000cf,
    TA_PERF_SEL_buffer_flat_1_op_burst__GFX103PLUS     = 0x000000d0,
    TA_PERF_SEL_buffer_flat_2to3_op_burst__GFX103PLUS  = 0x000000d1,
    TA_PERF_SEL_buffer_flat_4to31_op_burst__GFX103PLUS = 0x000000d2,
    TA_PERF_SEL_buffer_flat_ge32_op_burst__GFX103PLUS  = 0x000000d3,
    TA_PERF_SEL_write_1_op_burst__GFX103PLUS           = 0x000000d4,
    TA_PERF_SEL_write_2to3_op_burst__GFX103PLUS        = 0x000000d5,
    TA_PERF_SEL_write_4to31_op_burst__GFX103PLUS       = 0x000000d6,
    TA_PERF_SEL_write_ge32_op_burst__GFX103PLUS        = 0x000000d7,
    TA_PERF_SEL_ibubble_1_cycle_burst__GFX103PLUS      = 0x000000d8,
    TA_PERF_SEL_ibubble_2to3_cycle_burst__GFX103PLUS   = 0x000000d9,
    TA_PERF_SEL_ibubble_4to15_cycle_burst__GFX103PLUS  = 0x000000da,
    TA_PERF_SEL_ibubble_16to31_cycle_burst__GFX103PLUS = 0x000000db,
    TA_PERF_SEL_ibubble_32to63_cycle_burst__GFX103PLUS = 0x000000dc,
    TA_PERF_SEL_ibubble_ge64_cycle_burst__GFX103PLUS   = 0x000000dd,
    TA_PERF_SEL_sampler_clk_valid_cycles__GFX103PLUS   = 0x000000e0,
    TA_PERF_SEL_nonsampler_clk_valid_cycles__GFX103PLUS = 0x000000e1,
    TA_PERF_SEL_buffer_flat_clk_valid_cycles__GFX103PLUS = 0x000000e2,
    TA_PERF_SEL_write_data_clk_valid_cycles__GFX103PLUS = 0x000000e3,
    TA_PERF_SEL_gradient_clk_valid_cycles__GFX103PLUS  = 0x000000e4,
    TA_PERF_SEL_lod_aniso_clk_valid_cycles__GFX103PLUS = 0x000000e5,
    TA_PERF_SEL_sampler_addressing_clk_valid_cycles__GFX103PLUS = 0x000000e6,
    TA_PERF_SEL_sync_sampler_sstate_fifo_clk_valid_cycles__GFX103PLUS = 0x000000e7,
    TA_PERF_SEL_sync_sampler_cstate_fifo_clk_valid_cycles__GFX103PLUS = 0x000000e8,
    TA_PERF_SEL_sync_nonsampler_fifo_clk_valid_cycles__GFX103PLUS = 0x000000e9,
    TA_PERF_SEL_aligner_clk_valid_cycles__GFX103PLUS   = 0x000000ea,
    TA_PERF_SEL_tcreq_clk_valid_cycles__GFX103PLUS     = 0x000000eb,
    TA_PERF_SEL_RESERVED_111__GFX10VRS                 = 0x0000006f,
    TA_PERF_SEL_RESERVED_147__GFX10VRS                 = 0x00000093,
    TA_PERF_SEL_RESERVED_148__GFX10VRS                 = 0x00000094,
    TA_PERF_SEL_RESERVED_152__GFX10VRS                 = 0x00000098,
} TA_PERFCOUNT_SEL;

constexpr unsigned int MaxTaPerfcountSelGfx101         = TA_PERF_SEL_nosample_path_cycles__GFX101;
constexpr unsigned int MaxTaPerfcountSelGfx103Plus     = TA_PERF_SEL_tcreq_clk_valid_cycles__GFX103PLUS;

typedef enum TA_TC_ADDR_MODES {
    TA_TC_ADDR_MODE_DEFAULT                            = 0x00000000,
    TA_TC_ADDR_MODE_COMP0                              = 0x00000001,
    TA_TC_ADDR_MODE_COMP1                              = 0x00000002,
    TA_TC_ADDR_MODE_COMP2                              = 0x00000003,
    TA_TC_ADDR_MODE_COMP3                              = 0x00000004,
    TA_TC_ADDR_MODE_UNALIGNED                          = 0x00000005,
    TA_TC_ADDR_MODE_BORDER_COLOR                       = 0x00000006,
} TA_TC_ADDR_MODES;

typedef enum TA_TC_REQ_MODES {
    TA_TC_REQ_MODE_BORDER                              = 0x00000000,
    TA_TC_REQ_MODE_TEX2                                = 0x00000001,
    TA_TC_REQ_MODE_TEX1                                = 0x00000002,
    TA_TC_REQ_MODE_TEX0                                = 0x00000003,
    TA_TC_REQ_MODE_NORMAL                              = 0x00000004,
    TA_TC_REQ_MODE_DWORD                               = 0x00000005,
    TA_TC_REQ_MODE_BYTE                                = 0x00000006,
    TA_TC_REQ_MODE_BYTE_NV                             = 0x00000007,
} TA_TC_REQ_MODES;

typedef enum TCC_CACHE_POLICIES {
    TCC_CACHE_POLICY_LRU                               = 0x00000000,
    TCC_CACHE_POLICY_STREAM                            = 0x00000001,
} TCC_CACHE_POLICIES;

typedef enum TCC_MTYPE {
    MTYPE_NC                                           = 0x00000000,
    MTYPE_WC                                           = 0x00000001,
    MTYPE_CC                                           = 0x00000002,
} TCC_MTYPE;

typedef enum TCP_CACHE_POLICIES {
    TCP_CACHE_POLICY_MISS_LRU                          = 0x00000000,
    TCP_CACHE_POLICY_MISS_EVICT                        = 0x00000001,
    TCP_CACHE_POLICY_HIT_LRU                           = 0x00000002,
    TCP_CACHE_POLICY_HIT_EVICT                         = 0x00000003,
} TCP_CACHE_POLICIES;

typedef enum TCP_CACHE_STORE_POLICIES {
    TCP_CACHE_STORE_POLICY_WT_LRU                      = 0x00000000,
    TCP_CACHE_STORE_POLICY_WT_EVICT                    = 0x00000001,
} TCP_CACHE_STORE_POLICIES;

typedef enum TCP_DSM_DATA_SEL {
    TCP_DSM_DISABLE                                    = 0x00000000,
    TCP_DSM_SEL0                                       = 0x00000001,
    TCP_DSM_SEL1                                       = 0x00000002,
    TCP_DSM_SEL_BOTH                                   = 0x00000003,
} TCP_DSM_DATA_SEL;

typedef enum TCP_DSM_INJECT_SEL {
    TCP_DSM_INJECT_SEL0                                = 0x00000000,
    TCP_DSM_INJECT_SEL1                                = 0x00000001,
    TCP_DSM_INJECT_SEL2                                = 0x00000002,
    TCP_DSM_INJECT_SEL3                                = 0x00000003,
} TCP_DSM_INJECT_SEL;

typedef enum TCP_DSM_SINGLE_WRITE {
    TCP_DSM_SINGLE_WRITE_DIS                           = 0x00000000,
    TCP_DSM_SINGLE_WRITE_EN                            = 0x00000001,
} TCP_DSM_SINGLE_WRITE;

typedef enum TCP_OPCODE_TYPE {
    TCP_OPCODE_READ                                    = 0x00000000,
    TCP_OPCODE_WRITE                                   = 0x00000001,
    TCP_OPCODE_ATOMIC                                  = 0x00000002,
    TCP_OPCODE_INV                                     = 0x00000003,
    TCP_OPCODE_ATOMIC_CMPSWAP                          = 0x00000004,
    TCP_OPCODE_SAMPLER                                 = 0x00000005,
    TCP_OPCODE_LOAD                                    = 0x00000006,
    TCP_OPCODE_GATHERH                                 = 0x00000007,
} TCP_OPCODE_TYPE;

typedef enum TCP_PERFCOUNT_SELECT {
    TCP_PERF_SEL_GATE_EN1                              = 0x00000000,
    TCP_PERF_SEL_GATE_EN2                              = 0x00000001,
    TCP_PERF_SEL_TA_REQ                                = 0x00000002,
    TCP_PERF_SEL_TA_REQ_STATE_READ                     = 0x00000003,
    TCP_PERF_SEL_TA_REQ_READ                           = 0x00000004,
    TCP_PERF_SEL_TA_REQ_WRITE                          = 0x00000005,
    TCP_PERF_SEL_TA_REQ_ATOMIC_WITH_RET                = 0x00000006,
    TCP_PERF_SEL_TA_REQ_ATOMIC_WITHOUT_RET             = 0x00000007,
    TCP_PERF_SEL_TA_REQ_GL0_INV                        = 0x00000008,
    TCP_PERF_SEL_REQ                                   = 0x00000009,
    TCP_PERF_SEL_REQ_READ                              = 0x0000000a,
    TCP_PERF_SEL_REQ_READ_HIT_LRU                      = 0x0000000c,
    TCP_PERF_SEL_REQ_READ_MISS_EVICT                   = 0x0000000d,
    TCP_PERF_SEL_REQ_WRITE                             = 0x0000000e,
    TCP_PERF_SEL_REQ_WRITE_MISS_EVICT                  = 0x0000000f,
    TCP_PERF_SEL_REQ_READ_HIT_EVICT__GFX10             = 0x0000000b,
    TCP_PERF_SEL_REQ_WRITE_MISS_LRU__GFX10             = 0x00000010,
    TCP_PERF_SEL_REQ_NON_READ__GFX10                   = 0x00000011,
    TCP_PERF_SEL_REQ_MISS__GFX10                       = 0x00000012,
    TCP_PERF_SEL_REQ_TAGBANK0__GFX10                   = 0x00000013,
    TCP_PERF_SEL_REQ_TAGBANK1__GFX10                   = 0x00000014,
    TCP_PERF_SEL_REQ_TAGBANK2__GFX10                   = 0x00000015,
    TCP_PERF_SEL_REQ_TAGBANK3__GFX10                   = 0x00000016,
    TCP_PERF_SEL_REQ_MISS_TAGBANK0__GFX10              = 0x00000017,
    TCP_PERF_SEL_REQ_MISS_TAGBANK1__GFX10              = 0x00000018,
    TCP_PERF_SEL_REQ_MISS_TAGBANK2__GFX10              = 0x00000019,
    TCP_PERF_SEL_REQ_MISS_TAGBANK3__GFX10              = 0x0000001a,
    TCP_PERF_SEL_GL1_REQ_READ__GFX10                   = 0x0000001b,
    TCP_PERF_SEL_GL1_REQ_READ_128B__GFX10              = 0x0000001c,
    TCP_PERF_SEL_GL1_REQ_READ_64B__GFX10               = 0x0000001d,
    TCP_PERF_SEL_GL1_REQ_WRITE__GFX10                  = 0x0000001e,
    TCP_PERF_SEL_GL1_REQ_ATOMIC_WITH_RET__GFX10        = 0x0000001f,
    TCP_PERF_SEL_GL1_REQ_ATOMIC_WITHOUT_RET__GFX10     = 0x00000020,
    TCP_PERF_SEL_GL1_READ_LATENCY__GFX10               = 0x00000021,
    TCP_PERF_SEL_GL1_WRITE_LATENCY__GFX10              = 0x00000022,
    TCP_PERF_SEL_TCP_LATENCY__GFX10                    = 0x00000023,
    TCP_PERF_SEL_TCP_TA_REQ_STALL__GFX10               = 0x00000024,
    TCP_PERF_SEL_TA_TCP_REQ_STARVE__GFX10              = 0x00000025,
    TCP_PERF_SEL_TA_TCP_XNACK_STALL__GFX101            = 0x00000026,
    TCP_PERF_SEL_DATA_FIFO_STALL__GFX101               = 0x00000027,
    TCP_PERF_SEL_LOD_STALL__GFX101                     = 0x00000028,
    TCP_PERF_SEL_POWER_STALL__GFX101                   = 0x00000029,
    TCP_PERF_SEL_ALLOC_STALL__GFX101                   = 0x0000002a,
    TCP_PERF_SEL_UNORDERED_MTYPE_STALL__GFX101         = 0x0000002b,
    TCP_PERF_SEL_READ_TAGCONFLICT_STALL__GFX101        = 0x0000002c,
    TCP_PERF_SEL_WRITE_TAGCONFLICT_STALL__GFX101       = 0x0000002d,
    TCP_PERF_SEL_ATOMIC_TAGCONFLICT_STALL__GFX101      = 0x0000002e,
    TCP_PERF_SEL_LFIFO_STALL__GFX101                   = 0x0000002f,
    TCP_PERF_SEL_MEM_REQ_FIFO_STALL__GFX101            = 0x00000030,
    TCP_PERF_SEL_GL1_TCP_STALL__GFX101                 = 0x00000031,
    TCP_PERF_SEL_TCP_GL1_STARVE__GFX101                = 0x00000032,
    TCP_PERF_SEL_GL1_TCP_RDRET_STALL__GFX101           = 0x00000033,
    TCP_PERF_SEL_GL1_GRANT_READ_STALL__GFX101          = 0x00000034,
    TCP_PERF_SEL_GL1_PENDING_STALL__GFX101             = 0x00000035,
    TCP_PERF_SEL_OFIFO_INCOMPLETE_STALL__GFX101        = 0x00000036,
    TCP_PERF_SEL_OFIFO_AGE_ORDER_STALL__GFX101         = 0x00000037,
    TCP_PERF_SEL_TD_DATA_CYCLE_STALL__GFX101           = 0x00000038,
    TCP_PERF_SEL_COMP_TEX_LOAD_STALL__GFX101           = 0x00000039,
    TCP_PERF_SEL_READ_DATACONFLICT_STALL__GFX101       = 0x0000003a,
    TCP_PERF_SEL_WRITE_DATACONFLICT_STALL__GFX101      = 0x0000003b,
    TCP_PERF_SEL_TD_TCP_STALL__GFX101                  = 0x0000003c,
    TCP_PERF_SEL_UTCL0_REQUEST__GFX101                 = 0x0000003d,
    TCP_PERF_SEL_UTCL0_TRANSLATION_MISS__GFX101        = 0x0000003e,
    TCP_PERF_SEL_UTCL0_TRANSLATION_HIT__GFX101         = 0x0000003f,
    TCP_PERF_SEL_UTCL0_PERMISSION_MISS__GFX101         = 0x00000040,
    TCP_PERF_SEL_UTCL0_SERIALIZATION_STALL__GFX101     = 0x00000041,
    TCP_PERF_SEL_UTCL0_STALL_INFLIGHT_MAX__GFX101      = 0x00000042,
    TCP_PERF_SEL_UTCL0_STALL_LRU_INFLIGHT__GFX101      = 0x00000043,
    TCP_PERF_SEL_UTCL0_STALL_MULTI_MISS__GFX101        = 0x00000044,
    TCP_PERF_SEL_UTCL0_STALL_LFIFO_FULL__GFX101        = 0x00000045,
    TCP_PERF_SEL_UTCL0_STALL_MISSFIFO_FULL__GFX101     = 0x00000046,
    TCP_PERF_SEL_UTCL0_STALL_LFIFO_NOT_RES__GFX101     = 0x00000047,
    TCP_PERF_SEL_UTCL0_STALL_UTCL1_REQ_OUT_OF_CREDITS__GFX101 = 0x00000048,
    TCP_PERF_SEL_CLIENT_UTCL0_INFLIGHT__GFX101         = 0x00000049,
    TCP_PERF_SEL_UTCL0_UTCL1_INFLIGHT__GFX101          = 0x0000004a,
    TCP_PERF_SEL_UTCL0_UTCL1_PERM_FAULT__GFX101        = 0x0000004b,
    TCP_PERF_SEL_BACK_COMPAT_SWITCH__GFX101            = 0x0000004c,
    TCP_PERF_SEL_DATA_FIFO_STALL__GFX10VRS             = 0x00000026,
    TCP_PERF_SEL_LOD_STALL__GFX10VRS                   = 0x00000027,
    TCP_PERF_SEL_POWER_STALL__GFX10VRS                 = 0x00000028,
    TCP_PERF_SEL_ALLOC_STALL__GFX10VRS                 = 0x00000029,
    TCP_PERF_SEL_UNORDERED_MTYPE_STALL__GFX10VRS       = 0x0000002a,
    TCP_PERF_SEL_READ_TAGCONFLICT_STALL__GFX10VRS      = 0x0000002b,
    TCP_PERF_SEL_WRITE_TAGCONFLICT_STALL__GFX10VRS     = 0x0000002c,
    TCP_PERF_SEL_ATOMIC_TAGCONFLICT_STALL__GFX10VRS    = 0x0000002d,
    TCP_PERF_SEL_LFIFO_STALL__GFX10VRS                 = 0x0000002e,
    TCP_PERF_SEL_MEM_REQ_FIFO_STALL__GFX10VRS          = 0x0000002f,
    TCP_PERF_SEL_GL1_TCP_STALL__GFX10VRS               = 0x00000030,
    TCP_PERF_SEL_TCP_GL1_STARVE__GFX10VRS              = 0x00000031,
    TCP_PERF_SEL_GL1_TCP_RDRET_STALL__GFX10VRS         = 0x00000032,
    TCP_PERF_SEL_GL1_GRANT_READ_STALL__GFX10VRS        = 0x00000033,
    TCP_PERF_SEL_GL1_PENDING_STALL__GFX10VRS           = 0x00000034,
    TCP_PERF_SEL_OFIFO_INCOMPLETE_STALL__GFX10VRS      = 0x00000035,
    TCP_PERF_SEL_OFIFO_AGE_ORDER_STALL__GFX10VRS       = 0x00000036,
    TCP_PERF_SEL_TD_DATA_CYCLE_STALL__GFX10VRS         = 0x00000037,
    TCP_PERF_SEL_COMP_TEX_LOAD_STALL__GFX10VRS         = 0x00000038,
    TCP_PERF_SEL_READ_DATACONFLICT_STALL__GFX10VRS     = 0x00000039,
    TCP_PERF_SEL_WRITE_DATACONFLICT_STALL__GFX10VRS    = 0x0000003a,
    TCP_PERF_SEL_TD_TCP_STALL__GFX10VRS                = 0x0000003b,
    TCP_PERF_SEL_BACK_COMPAT_SWITCH__GFX10VRS          = 0x0000003c,
    TCP_PERF_SEL_REQ_NON_READ__GFX11                   = 0x00000010,
    TCP_PERF_SEL_REQ_MISS__GFX11                       = 0x00000011,
    TCP_PERF_SEL_REQ_TAGBANK0_SET0__GFX11              = 0x00000012,
    TCP_PERF_SEL_REQ_TAGBANK0_SET1__GFX11              = 0x00000013,
    TCP_PERF_SEL_REQ_TAGBANK1_SET0__GFX11              = 0x00000014,
    TCP_PERF_SEL_REQ_TAGBANK1_SET1__GFX11              = 0x00000015,
    TCP_PERF_SEL_REQ_TAGBANK2_SET0__GFX11              = 0x00000016,
    TCP_PERF_SEL_REQ_TAGBANK2_SET1__GFX11              = 0x00000017,
    TCP_PERF_SEL_REQ_TAGBANK3_SET0__GFX11              = 0x00000018,
    TCP_PERF_SEL_REQ_TAGBANK3_SET1__GFX11              = 0x00000019,
    TCP_PERF_SEL_REQ_MISS_TAGBANK0__GFX11              = 0x0000001a,
    TCP_PERF_SEL_REQ_MISS_TAGBANK1__GFX11              = 0x0000001b,
    TCP_PERF_SEL_REQ_MISS_TAGBANK2__GFX11              = 0x0000001c,
    TCP_PERF_SEL_REQ_MISS_TAGBANK3__GFX11              = 0x0000001d,
    TCP_PERF_SEL_GL1_REQ_READ__GFX11                   = 0x0000001e,
    TCP_PERF_SEL_GL1_REQ_READ_128B__GFX11              = 0x0000001f,
    TCP_PERF_SEL_GL1_REQ_READ_64B__GFX11               = 0x00000020,
    TCP_PERF_SEL_GL1_REQ_WRITE__GFX11                  = 0x00000021,
    TCP_PERF_SEL_GL1_REQ_ATOMIC_WITH_RET__GFX11        = 0x00000022,
    TCP_PERF_SEL_GL1_REQ_ATOMIC_WITHOUT_RET__GFX11     = 0x00000023,
    TCP_PERF_SEL_GL1_READ_LATENCY__GFX11               = 0x00000024,
    TCP_PERF_SEL_GL1_WRITE_LATENCY__GFX11              = 0x00000025,
    TCP_PERF_SEL_TCP_LATENCY__GFX11                    = 0x00000026,
    TCP_PERF_SEL_TCP_TA_REQ_STALL__GFX11               = 0x00000027,
    TCP_PERF_SEL_TA_TCP_REQ_STARVE__GFX11              = 0x00000028,
    TCP_PERF_SEL_DATA_FIFO_STALL__GFX11                = 0x00000029,
    TCP_PERF_SEL_LOD_STALL__GFX11                      = 0x0000002a,
    TCP_PERF_SEL_POWER_STALL__GFX11                    = 0x0000002b,
    TCP_PERF_SEL_ALLOC_STALL__GFX11                    = 0x0000002c,
    TCP_PERF_SEL_UNORDERED_MTYPE_STALL__GFX11          = 0x0000002d,
    TCP_PERF_SEL_READ_TAGCONFLICT_STALL__GFX11         = 0x0000002e,
    TCP_PERF_SEL_WRITE_TAGCONFLICT_STALL__GFX11        = 0x0000002f,
    TCP_PERF_SEL_ATOMIC_TAGCONFLICT_STALL__GFX11       = 0x00000030,
    TCP_PERF_SEL_LFIFO_STALL__GFX11                    = 0x00000031,
    TCP_PERF_SEL_MEM_REQ_FIFO_STALL__GFX11             = 0x00000032,
    TCP_PERF_SEL_GL1_TCP_BACK_PRESSURE__GFX11          = 0x00000033,
    TCP_PERF_SEL_GL1_TCP_RDRET_STALL__GFX11            = 0x00000034,
    TCP_PERF_SEL_GL1_GRANT_READ_STALL__GFX11           = 0x00000035,
    TCP_PERF_SEL_GL1_PENDING_STALL__GFX11              = 0x00000036,
    TCP_PERF_SEL_OFIFO_INCOMPLETE_STALL__GFX110        = 0x00000037,
    TCP_PERF_SEL_OFIFO_AGE_ORDER_STALL__GFX110         = 0x00000038,
    TCP_PERF_SEL_TD_DATA_CYCLE_STALL__GFX110           = 0x00000039,
    TCP_PERF_SEL_COMP_TEX_LOAD_STALL__GFX110           = 0x0000003a,
    TCP_PERF_SEL_READ_DATACONFLICT_STALL__GFX110       = 0x0000003b,
    TCP_PERF_SEL_WRITE_DATACONFLICT_STALL__GFX110      = 0x0000003c,
    TCP_PERF_SEL_TD_TCP_STALL__GFX110                  = 0x0000003d,
    TCP_PERF_SEL_TA_REQ_BUFFERNOP__GFX110              = 0x0000003e,
    TCP_PERF_SEL_WRITECOMBINE_ENDCLAUSE__GFX110        = 0x0000003f,
    TCP_PERF_SEL_TAGFAKE_EOW__GFX110                   = 0x00000040,
    TCP_PERF_SEL_REQ_TAG_MATCH_AND_NOT_VALID__GFX110   = 0x00000041,
    TCP_PERF_SEL_BURST_BIN_WRITECOMBINE_0__GFX110      = 0x00000042,
    TCP_PERF_SEL_BURST_BIN_WRITECOMBINE_1to2__GFX110   = 0x00000043,
    TCP_PERF_SEL_BURST_BIN_WRITECOMBINE_3to4__GFX110   = 0x00000044,
    TCP_PERF_SEL_BURST_BIN_WRITECOMBINE_5to8__GFX110   = 0x00000045,
    TCP_PERF_SEL_BURST_BIN_WRITECOMBINE_9to16__GFX110  = 0x00000046,
    TCP_PERF_SEL_BURST_BIN_WRITECOMBINE_gt16__GFX110   = 0x00000047,
    TCP_PERF_SEL_BURST_BIN_READHIT_0__GFX110           = 0x00000048,
    TCP_PERF_SEL_BURST_BIN_READHIT_1__GFX110           = 0x00000049,
    TCP_PERF_SEL_BURST_BIN_READHIT_2to4__GFX110        = 0x0000004a,
    TCP_PERF_SEL_BURST_BIN_READHIT_5to8__GFX110        = 0x0000004b,
    TCP_PERF_SEL_BURST_BIN_READHIT_9to16__GFX110       = 0x0000004c,
    TCP_PERF_SEL_BURST_BIN_READHIT_gt16__GFX110        = 0x0000004d,
    TCP_PERF_SEL_OFIFO_INCOMPLETE_STALL__STX1          = 0x00000037,
    TCP_PERF_SEL_OFIFO_AGE_ORDER_STALL__STX1           = 0x00000038,
    TCP_PERF_SEL_TD_DATA_CYCLE_STALL__STX1             = 0x00000039,
    TCP_PERF_SEL_COMP_TEX_LOAD_STALL__STX1             = 0x0000003a,
    TCP_PERF_SEL_READ_DATACONFLICT_STALL__STX1         = 0x0000003b,
    TCP_PERF_SEL_WRITE_DATACONFLICT_STALL__STX1        = 0x0000003c,
    TCP_PERF_SEL_TD_TCP_STALL__STX1                    = 0x0000003d,
    TCP_PERF_SEL_TA_REQ_BUFFERNOP__STX1                = 0x0000003e,
    TCP_PERF_SEL_WRITECOMBINE_ENDCLAUSE__STX1          = 0x0000003f,
    TCP_PERF_SEL_TAGFAKE_EOW__STX1                     = 0x00000040,
    TCP_PERF_SEL_REQ_TAG_MATCH_AND_NOT_VALID__STX1     = 0x00000041,
    TCP_PERF_SEL_BURST_BIN_WRITECOMBINE_0__STX1        = 0x00000042,
    TCP_PERF_SEL_BURST_BIN_WRITECOMBINE_1to2__STX1     = 0x00000043,
    TCP_PERF_SEL_BURST_BIN_WRITECOMBINE_3to4__STX1     = 0x00000044,
    TCP_PERF_SEL_BURST_BIN_WRITECOMBINE_5to8__STX1     = 0x00000045,
    TCP_PERF_SEL_BURST_BIN_WRITECOMBINE_9to16__STX1    = 0x00000046,
    TCP_PERF_SEL_BURST_BIN_WRITECOMBINE_gt16__STX1     = 0x00000047,
    TCP_PERF_SEL_BURST_BIN_READHIT_0__STX1             = 0x00000048,
    TCP_PERF_SEL_BURST_BIN_READHIT_1__STX1             = 0x00000049,
    TCP_PERF_SEL_BURST_BIN_READHIT_2to4__STX1          = 0x0000004a,
    TCP_PERF_SEL_BURST_BIN_READHIT_5to8__STX1          = 0x0000004b,
    TCP_PERF_SEL_BURST_BIN_READHIT_9to16__STX1         = 0x0000004c,
    TCP_PERF_SEL_BURST_BIN_READHIT_gt16__STX1          = 0x0000004d,
#if CHIP_HDR_STRIX_HALO
    TCP_PERF_SEL_TD_DATA_CYCLE_STALL__STXH             = 0x00000037,
    TCP_PERF_SEL_COMP_TEX_LOAD_STALL__STXH             = 0x00000038,
    TCP_PERF_SEL_READ_DATACONFLICT_STALL__STXH         = 0x00000039,
    TCP_PERF_SEL_WRITE_DATACONFLICT_STALL__STXH        = 0x0000003a,
    TCP_PERF_SEL_TD_TCP_STALL__STXH                    = 0x0000003b,
    TCP_PERF_SEL_TA_REQ_BUFFERNOP__STXH                = 0x0000003c,
    TCP_PERF_SEL_WRITECOMBINE_ENDCLAUSE__STXH          = 0x0000003d,
    TCP_PERF_SEL_TAGFAKE_EOW__STXH                     = 0x0000003e,
    TCP_PERF_SEL_REQ_TAG_MATCH_AND_NOT_VALID__STXH     = 0x0000003f,
    TCP_PERF_SEL_BURST_BIN_WRITECOMBINE_0__STXH        = 0x00000040,
    TCP_PERF_SEL_BURST_BIN_WRITECOMBINE_1to2__STXH     = 0x00000041,
    TCP_PERF_SEL_BURST_BIN_WRITECOMBINE_3to4__STXH     = 0x00000042,
    TCP_PERF_SEL_BURST_BIN_WRITECOMBINE_5to8__STXH     = 0x00000043,
    TCP_PERF_SEL_BURST_BIN_WRITECOMBINE_9to16__STXH    = 0x00000044,
    TCP_PERF_SEL_BURST_BIN_WRITECOMBINE_gt16__STXH     = 0x00000045,
    TCP_PERF_SEL_BURST_BIN_READHIT_0__STXH             = 0x00000046,
    TCP_PERF_SEL_BURST_BIN_READHIT_1__STXH             = 0x00000047,
    TCP_PERF_SEL_BURST_BIN_READHIT_2to4__STXH          = 0x00000048,
    TCP_PERF_SEL_BURST_BIN_READHIT_5to8__STXH          = 0x00000049,
    TCP_PERF_SEL_BURST_BIN_READHIT_9to16__STXH         = 0x0000004a,
    TCP_PERF_SEL_BURST_BIN_READHIT_gt16__STXH          = 0x0000004b,
    TCP_PERF_SEL_TA_TC_REQ_EN_SUM__STXH                = 0x0000004c,
    TCP_PERF_SEL_XBAR_REQ_EN_SUM__STXH                 = 0x0000004d,
    TCP_PERF_SEL_XBAR_REQ_UNIQUE_SUM__STXH             = 0x0000004e,
#endif
} TCP_PERFCOUNT_SELECT;

constexpr unsigned int MaxTcpPerfcountSelectGfx101     = TCP_PERF_SEL_BACK_COMPAT_SWITCH__GFX101;
constexpr unsigned int MaxTcpPerfcountSelectGfx10Vrs   = TCP_PERF_SEL_BACK_COMPAT_SWITCH__GFX10VRS;
constexpr unsigned int MaxTcpPerfcountSelectGfx110     = TCP_PERF_SEL_BURST_BIN_READHIT_gt16__GFX110;
constexpr unsigned int MaxTcpPerfcountSelectStx1       = TCP_PERF_SEL_BURST_BIN_READHIT_gt16__STX1;
#if CHIP_HDR_STRIX_HALO
constexpr unsigned int MaxTcpPerfcountSelectStxH       = TCP_PERF_SEL_XBAR_REQ_UNIQUE_SUM__STXH;
#endif

typedef enum TCP_WATCH_MODES {
    TCP_WATCH_MODE_READ                                = 0x00000000,
    TCP_WATCH_MODE_NONREAD                             = 0x00000001,
    TCP_WATCH_MODE_ATOMIC                              = 0x00000002,
    TCP_WATCH_MODE_ALL                                 = 0x00000003,
} TCP_WATCH_MODES;

typedef enum TC_EA_CID {
    TC_EA_CID_RT                                       = 0x00000000,
    TC_EA_CID_FMASK                                    = 0x00000001,
    TC_EA_CID_DCC                                      = 0x00000002,
    TC_EA_CID_TCPMETA                                  = 0x00000003,
    TC_EA_CID_Z                                        = 0x00000004,
    TC_EA_CID_STENCIL                                  = 0x00000005,
    TC_EA_CID_HTILE                                    = 0x00000006,
    TC_EA_CID_MISC                                     = 0x00000007,
    TC_EA_CID_TCP                                      = 0x00000008,
    TC_EA_CID_SQC                                      = 0x00000009,
    TC_EA_CID_CPF                                      = 0x0000000a,
    TC_EA_CID_CPG                                      = 0x0000000b,
    TC_EA_CID_IA                                       = 0x0000000c,
    TC_EA_CID_WD                                       = 0x0000000d,
    TC_EA_CID_PA                                       = 0x0000000e,
    TC_EA_CID_UTCL2_TPI                                = 0x0000000f,
} TC_EA_CID;

typedef enum TC_MICRO_TILE_MODE {
    MICRO_TILE_MODE_LINEAR                             = 0x00000000,
    MICRO_TILE_MODE_RENDER_TARGET                      = 0x00000001,
    MICRO_TILE_MODE_STD_2D                             = 0x00000002,
    MICRO_TILE_MODE_STD_3D                             = 0x00000003,
    MICRO_TILE_MODE_DISPLAY_2D                         = 0x00000004,
    MICRO_TILE_MODE_DISPLAY_3D                         = 0x00000005,
    MICRO_TILE_MODE_Z                                  = 0x00000006,
    MICRO_TILE_MODE_Z_VAR__GFX11                       = 0x00000007,
    MICRO_TILE_MODE_Z_VAR__NV2X                        = 0x00000007,
} TC_MICRO_TILE_MODE;

typedef enum TC_NACKS {
    TC_NACK_NO_FAULT                                   = 0x00000000,
    TC_NACK_PAGE_FAULT                                 = 0x00000001,
    TC_NACK_PROTECTION_FAULT                           = 0x00000002,
    TC_NACK_DATA_ERROR                                 = 0x00000003,
} TC_NACKS;

typedef enum TC_OP {
    TC_OP_READ                                         = 0x00000000,
    TC_OP_ATOMIC_FCMPSWAP_RTN_32                       = 0x00000001,
    TC_OP_ATOMIC_FMIN_RTN_32                           = 0x00000002,
    TC_OP_ATOMIC_FMAX_RTN_32                           = 0x00000003,
    TC_OP_RESERVED_FOP_RTN_32_0                        = 0x00000004,
    TC_OP_RESERVED_FOP_RTN_32_2                        = 0x00000006,
    TC_OP_ATOMIC_SWAP_RTN_32                           = 0x00000007,
    TC_OP_ATOMIC_CMPSWAP_RTN_32                        = 0x00000008,
    TC_OP_ATOMIC_FCMPSWAP_FLUSH_DENORM_RTN_32          = 0x00000009,
    TC_OP_ATOMIC_FMIN_FLUSH_DENORM_RTN_32              = 0x0000000a,
    TC_OP_ATOMIC_FMAX_FLUSH_DENORM_RTN_32              = 0x0000000b,
    TC_OP_PROBE_FILTER                                 = 0x0000000c,
    TC_OP_RESERVED_FOP_FLUSH_DENORM_RTN_32_2           = 0x0000000e,
    TC_OP_ATOMIC_ADD_RTN_32                            = 0x0000000f,
    TC_OP_ATOMIC_SUB_RTN_32                            = 0x00000010,
    TC_OP_ATOMIC_SMIN_RTN_32                           = 0x00000011,
    TC_OP_ATOMIC_UMIN_RTN_32                           = 0x00000012,
    TC_OP_ATOMIC_SMAX_RTN_32                           = 0x00000013,
    TC_OP_ATOMIC_UMAX_RTN_32                           = 0x00000014,
    TC_OP_ATOMIC_AND_RTN_32                            = 0x00000015,
    TC_OP_ATOMIC_OR_RTN_32                             = 0x00000016,
    TC_OP_ATOMIC_XOR_RTN_32                            = 0x00000017,
    TC_OP_ATOMIC_INC_RTN_32                            = 0x00000018,
    TC_OP_ATOMIC_DEC_RTN_32                            = 0x00000019,
    TC_OP_WBINVL1_VOL                                  = 0x0000001a,
    TC_OP_WBINVL1_SD                                   = 0x0000001b,
    TC_OP_RESERVED_NON_FLOAT_RTN_32_0                  = 0x0000001c,
    TC_OP_RESERVED_NON_FLOAT_RTN_32_1                  = 0x0000001d,
    TC_OP_RESERVED_NON_FLOAT_RTN_32_2                  = 0x0000001e,
    TC_OP_RESERVED_NON_FLOAT_RTN_32_3                  = 0x0000001f,
    TC_OP_WRITE                                        = 0x00000020,
    TC_OP_ATOMIC_FCMPSWAP_RTN_64                       = 0x00000021,
    TC_OP_ATOMIC_FMIN_RTN_64                           = 0x00000022,
    TC_OP_ATOMIC_FMAX_RTN_64                           = 0x00000023,
    TC_OP_RESERVED_FOP_RTN_64_0                        = 0x00000024,
    TC_OP_RESERVED_FOP_RTN_64_1                        = 0x00000025,
    TC_OP_RESERVED_FOP_RTN_64_2                        = 0x00000026,
    TC_OP_ATOMIC_SWAP_RTN_64                           = 0x00000027,
    TC_OP_ATOMIC_CMPSWAP_RTN_64                        = 0x00000028,
    TC_OP_ATOMIC_FCMPSWAP_FLUSH_DENORM_RTN_64          = 0x00000029,
    TC_OP_ATOMIC_FMIN_FLUSH_DENORM_RTN_64              = 0x0000002a,
    TC_OP_ATOMIC_FMAX_FLUSH_DENORM_RTN_64              = 0x0000002b,
    TC_OP_WBINVL2_SD                                   = 0x0000002c,
    TC_OP_RESERVED_FOP_FLUSH_DENORM_RTN_64_0           = 0x0000002d,
    TC_OP_RESERVED_FOP_FLUSH_DENORM_RTN_64_1           = 0x0000002e,
    TC_OP_ATOMIC_ADD_RTN_64                            = 0x0000002f,
    TC_OP_ATOMIC_SUB_RTN_64                            = 0x00000030,
    TC_OP_ATOMIC_SMIN_RTN_64                           = 0x00000031,
    TC_OP_ATOMIC_UMIN_RTN_64                           = 0x00000032,
    TC_OP_ATOMIC_SMAX_RTN_64                           = 0x00000033,
    TC_OP_ATOMIC_UMAX_RTN_64                           = 0x00000034,
    TC_OP_ATOMIC_AND_RTN_64                            = 0x00000035,
    TC_OP_ATOMIC_OR_RTN_64                             = 0x00000036,
    TC_OP_ATOMIC_XOR_RTN_64                            = 0x00000037,
    TC_OP_ATOMIC_INC_RTN_64                            = 0x00000038,
    TC_OP_ATOMIC_DEC_RTN_64                            = 0x00000039,
    TC_OP_WBL2_NC                                      = 0x0000003a,
    TC_OP_WBL2_WC                                      = 0x0000003b,
    TC_OP_RESERVED_NON_FLOAT_RTN_64_1                  = 0x0000003c,
    TC_OP_RESERVED_NON_FLOAT_RTN_64_2                  = 0x0000003d,
    TC_OP_RESERVED_NON_FLOAT_RTN_64_3                  = 0x0000003e,
    TC_OP_RESERVED_NON_FLOAT_RTN_64_4                  = 0x0000003f,
    TC_OP_WBINVL1                                      = 0x00000040,
    TC_OP_ATOMIC_FCMPSWAP_32                           = 0x00000041,
    TC_OP_ATOMIC_FMIN_32                               = 0x00000042,
    TC_OP_ATOMIC_FMAX_32                               = 0x00000043,
    TC_OP_RESERVED_FOP_32_0                            = 0x00000044,
    TC_OP_RESERVED_FOP_32_2                            = 0x00000046,
    TC_OP_ATOMIC_SWAP_32                               = 0x00000047,
    TC_OP_ATOMIC_CMPSWAP_32                            = 0x00000048,
    TC_OP_ATOMIC_FCMPSWAP_FLUSH_DENORM_32              = 0x00000049,
    TC_OP_ATOMIC_FMIN_FLUSH_DENORM_32                  = 0x0000004a,
    TC_OP_ATOMIC_FMAX_FLUSH_DENORM_32                  = 0x0000004b,
    TC_OP_INV_METADATA                                 = 0x0000004c,
    TC_OP_RESERVED_FOP_FLUSH_DENORM_32_2               = 0x0000004e,
    TC_OP_ATOMIC_ADD_32                                = 0x0000004f,
    TC_OP_ATOMIC_SUB_32                                = 0x00000050,
    TC_OP_ATOMIC_SMIN_32                               = 0x00000051,
    TC_OP_ATOMIC_UMIN_32                               = 0x00000052,
    TC_OP_ATOMIC_SMAX_32                               = 0x00000053,
    TC_OP_ATOMIC_UMAX_32                               = 0x00000054,
    TC_OP_ATOMIC_AND_32                                = 0x00000055,
    TC_OP_ATOMIC_OR_32                                 = 0x00000056,
    TC_OP_ATOMIC_XOR_32                                = 0x00000057,
    TC_OP_ATOMIC_INC_32                                = 0x00000058,
    TC_OP_ATOMIC_DEC_32                                = 0x00000059,
    TC_OP_INVL2_NC                                     = 0x0000005a,
    TC_OP_NOP_RTN0                                     = 0x0000005b,
    TC_OP_RESERVED_NON_FLOAT_32_1                      = 0x0000005c,
    TC_OP_RESERVED_NON_FLOAT_32_2                      = 0x0000005d,
    TC_OP_RESERVED_NON_FLOAT_32_3                      = 0x0000005e,
    TC_OP_RESERVED_NON_FLOAT_32_4                      = 0x0000005f,
    TC_OP_WBINVL2                                      = 0x00000060,
    TC_OP_ATOMIC_FCMPSWAP_64                           = 0x00000061,
    TC_OP_ATOMIC_FMIN_64                               = 0x00000062,
    TC_OP_ATOMIC_FMAX_64                               = 0x00000063,
    TC_OP_RESERVED_FOP_64_0                            = 0x00000064,
    TC_OP_RESERVED_FOP_64_1                            = 0x00000065,
    TC_OP_RESERVED_FOP_64_2                            = 0x00000066,
    TC_OP_ATOMIC_SWAP_64                               = 0x00000067,
    TC_OP_ATOMIC_CMPSWAP_64                            = 0x00000068,
    TC_OP_ATOMIC_FCMPSWAP_FLUSH_DENORM_64              = 0x00000069,
    TC_OP_ATOMIC_FMIN_FLUSH_DENORM_64                  = 0x0000006a,
    TC_OP_ATOMIC_FMAX_FLUSH_DENORM_64                  = 0x0000006b,
    TC_OP_RESERVED_FOP_FLUSH_DENORM_64_0               = 0x0000006c,
    TC_OP_RESERVED_FOP_FLUSH_DENORM_64_1               = 0x0000006d,
    TC_OP_RESERVED_FOP_FLUSH_DENORM_64_2               = 0x0000006e,
    TC_OP_ATOMIC_ADD_64                                = 0x0000006f,
    TC_OP_ATOMIC_SUB_64                                = 0x00000070,
    TC_OP_ATOMIC_SMIN_64                               = 0x00000071,
    TC_OP_ATOMIC_UMIN_64                               = 0x00000072,
    TC_OP_ATOMIC_SMAX_64                               = 0x00000073,
    TC_OP_ATOMIC_UMAX_64                               = 0x00000074,
    TC_OP_ATOMIC_AND_64                                = 0x00000075,
    TC_OP_ATOMIC_OR_64                                 = 0x00000076,
    TC_OP_ATOMIC_XOR_64                                = 0x00000077,
    TC_OP_ATOMIC_INC_64                                = 0x00000078,
    TC_OP_ATOMIC_DEC_64                                = 0x00000079,
    TC_OP_WBINVL2_NC                                   = 0x0000007a,
    TC_OP_NOP_ACK                                      = 0x0000007b,
    TC_OP_RESERVED_NON_FLOAT_64_1                      = 0x0000007c,
    TC_OP_RESERVED_NON_FLOAT_64_2                      = 0x0000007d,
    TC_OP_RESERVED_NON_FLOAT_64_3                      = 0x0000007e,
    TC_OP_RESERVED_NON_FLOAT_64_4                      = 0x0000007f,
    TC_OP_RESERVED_FOP_RTN_32_1__GFX10                 = 0x00000005,
    TC_OP_RESERVED_FOP_FLUSH_DENORM_RTN_32_1__GFX10    = 0x0000000d,
    TC_OP_RESERVED_FOP_32_1__GFX10                     = 0x00000045,
    TC_OP_RESERVED_FOP_FLUSH_DENORM_32_1__GFX10        = 0x0000004d,
    TC_OP_RESERVED_FADD_RTN_32__GFX11                  = 0x00000005,
    TC_OP_ATOMIC_FADD_FLUSH_DENORM_RTN_32__GFX11       = 0x0000000d,
    TC_OP_RESERVED_FADD_32__GFX11                      = 0x00000045,
    TC_OP_ATOMIC_FADD_FLUSH_DENORM_32__GFX11           = 0x0000004d,
} TC_OP;

typedef enum TC_OP_MASKS {
    TC_OP_MASK_FLUSH_DENROM                            = 0x00000008,
    TC_OP_MASK_64                                      = 0x00000020,
    TC_OP_MASK_NO_RTN                                  = 0x00000040,
} TC_OP_MASKS;

typedef enum TD_PERFCOUNT_SEL {
    TD_PERF_SEL_none                                   = 0x00000000,
    TD_PERF_SEL_td_busy                                = 0x00000001,
    TD_PERF_SEL_input_busy                             = 0x00000002,
    TD_PERF_SEL_sampler_lerp_busy                      = 0x00000003,
    TD_PERF_SEL_sampler_out_busy                       = 0x00000004,
    TD_PERF_SEL_nofilter_busy                          = 0x00000005,
    TD_PERF_SEL_sampler_sclk_on_nofilter_sclk_off__GFX101 = 0x00000006,
    TD_PERF_SEL_nofilter_sclk_on_sampler_sclk_off__GFX101 = 0x00000007,
    TD_PERF_SEL_core_state_ram_max_cnt__GFX101         = 0x00000008,
    TD_PERF_SEL_core_state_rams_read__GFX101           = 0x00000009,
    TD_PERF_SEL_weight_data_rams_read__GFX101          = 0x0000000a,
    TD_PERF_SEL_reference_data_rams_read__GFX101       = 0x0000000b,
    TD_PERF_SEL_tc_td_ram_fifo_full__GFX101            = 0x0000000c,
    TD_PERF_SEL_tc_td_ram_fifo_max_cnt__GFX101         = 0x0000000d,
    TD_PERF_SEL_tc_td_data_fifo_full__GFX101           = 0x0000000e,
    TD_PERF_SEL_input_state_fifo_full__GFX101          = 0x0000000f,
    TD_PERF_SEL_ta_data_stall__GFX101                  = 0x00000010,
    TD_PERF_SEL_tc_data_stall__GFX101                  = 0x00000011,
    TD_PERF_SEL_tc_ram_stall__GFX101                   = 0x00000012,
    TD_PERF_SEL_lds_stall__GFX101                      = 0x00000013,
    TD_PERF_SEL_sampler_pkr_full__GFX101               = 0x00000014,
    TD_PERF_SEL_nofilter_pkr_full__GFX101              = 0x00000015,
    TD_PERF_SEL_gather4_wavefront__GFX101              = 0x00000016,
    TD_PERF_SEL_gather4h_wavefront__GFX101             = 0x00000017,
    TD_PERF_SEL_gather4h_packed_wavefront__GFX101      = 0x00000018,
    TD_PERF_SEL_gather8h_packed_wavefront__GFX101      = 0x00000019,
    TD_PERF_SEL_sample_c_wavefront__GFX101             = 0x0000001a,
    TD_PERF_SEL_load_wavefront__GFX101                 = 0x0000001b,
    TD_PERF_SEL_ldfptr_wavefront__GFX101               = 0x0000001c,
    TD_PERF_SEL_RESERVED_29__GFX101                    = 0x0000001d,
    TD_PERF_SEL_write_ack_wavefront__GFX101            = 0x0000001e,
    TD_PERF_SEL_d16_en_wavefront__GFX101               = 0x0000001f,
    TD_PERF_SEL_bypassLerp_wavefront__GFX101           = 0x00000020,
    TD_PERF_SEL_min_max_filter_wavefront__GFX101       = 0x00000021,
    TD_PERF_SEL_one_comp_wavefront__GFX101             = 0x00000022,
    TD_PERF_SEL_two_comp_wavefront__GFX101             = 0x00000023,
    TD_PERF_SEL_three_comp_wavefront__GFX101           = 0x00000024,
    TD_PERF_SEL_four_comp_wavefront__GFX101            = 0x00000025,
    TD_PERF_SEL_user_defined_border__GFX101            = 0x00000026,
    TD_PERF_SEL_white_border__GFX101                   = 0x00000027,
    TD_PERF_SEL_opaque_black_border__GFX101            = 0x00000028,
    TD_PERF_SEL_lod_warn_from_ta__GFX101               = 0x00000029,
    TD_PERF_SEL_wavefront_dest_is_lds__GFX101          = 0x0000002a,
    TD_PERF_SEL_td_cycling_of_nofilter_instr__GFX101   = 0x0000002b,
    TD_PERF_SEL_tc_cycling_of_nofilter_instr__GFX101   = 0x0000002c,
    TD_PERF_SEL_out_of_order_instr__GFX101             = 0x0000002d,
    TD_PERF_SEL_total_num_instr__GFX101                = 0x0000002e,
    TD_PERF_SEL_mixmode_instruction__GFX101            = 0x0000002f,
    TD_PERF_SEL_mixmode_resource__GFX101               = 0x00000030,
    TD_PERF_SEL_status_packet__GFX101                  = 0x00000031,
    TD_PERF_SEL_address_cmd_poison__GFX101             = 0x00000032,
    TD_PERF_SEL_data_poison__GFX101                    = 0x00000033,
    TD_PERF_SEL_done_scoreboard_max_stored_cnt__GFX101 = 0x00000034,
    TD_PERF_SEL_done_scoreboard_max_waiting_cnt__GFX101 = 0x00000035,
    TD_PERF_SEL_done_scoreboard_not_empty__GFX101      = 0x00000036,
    TD_PERF_SEL_done_scoreboard_is_full__GFX101        = 0x00000037,
    TD_PERF_SEL_done_scoreboard_bp_due_to_ooo__GFX101  = 0x00000038,
    TD_PERF_SEL_done_scoreboard_bp_due_to_lds__GFX101  = 0x00000039,
    TD_PERF_SEL_nofilter_formatters_turned_on__GFX101  = 0x0000003a,
    TD_PERF_SEL_nofilter_popcount_dmask_gt_num_comp_of_fmt__GFX101 = 0x0000003b,
    TD_PERF_SEL_nofilter_popcount_dmask_lt_num_comp_of_fmt__GFX101 = 0x0000003c,
    TD_PERF_SEL_ray_tracing_bvh4_busy__GFX103PLUS      = 0x00000006,
    TD_PERF_SEL_sampler_core_sclk_en__GFX103PLUS       = 0x00000007,
    TD_PERF_SEL_ray_tracing_bvh4_box_sclk_en__GFX103PLUS = 0x00000018,
    TD_PERF_SEL_ray_tracing_bvh4_tri_sclk_en__GFX103PLUS = 0x00000019,
    TD_PERF_SEL_sampler_sclk_on_nofilter_sclk_off__GFX103PLUS = 0x0000001a,
    TD_PERF_SEL_nofilter_sclk_on_sampler_sclk_off__GFX103PLUS = 0x0000001b,
    TD_PERF_SEL_all_pipes_sclk_on_at_same_time__GFX103PLUS = 0x0000001c,
    TD_PERF_SEL_sampler_and_nofilter_sclk_on_bvh4_sclk_off__GFX103PLUS = 0x0000001d,
    TD_PERF_SEL_sampler_and_bvh4_sclk_on_nofilter_sclk_off__GFX103PLUS = 0x0000001e,
    TD_PERF_SEL_nofilter_and_bvh4_sclk_on_sampler_sclk_off__GFX103PLUS = 0x0000001f,
    TD_PERF_SEL_core_state_ram_max_cnt__GFX103PLUS     = 0x00000020,
    TD_PERF_SEL_core_state_rams_read__GFX103PLUS       = 0x00000021,
    TD_PERF_SEL_weight_data_rams_read__GFX103PLUS      = 0x00000022,
    TD_PERF_SEL_reference_data_rams_read__GFX103PLUS   = 0x00000023,
    TD_PERF_SEL_tc_td_ram_fifo_full__GFX103PLUS        = 0x00000024,
    TD_PERF_SEL_tc_td_ram_fifo_max_cnt__GFX103PLUS     = 0x00000025,
    TD_PERF_SEL_tc_td_data_fifo_full__GFX103PLUS       = 0x00000026,
    TD_PERF_SEL_input_state_fifo_full__GFX103PLUS      = 0x00000027,
    TD_PERF_SEL_ta_data_stall__GFX103PLUS              = 0x00000028,
    TD_PERF_SEL_tc_data_stall__GFX103PLUS              = 0x00000029,
    TD_PERF_SEL_tc_ram_stall__GFX103PLUS               = 0x0000002a,
    TD_PERF_SEL_lds_stall__GFX103PLUS                  = 0x0000002b,
    TD_PERF_SEL_sampler_pkr_full__GFX103PLUS           = 0x0000002c,
    TD_PERF_SEL_sampler_pkr_full_due_to_arb__GFX103PLUS = 0x0000002d,
    TD_PERF_SEL_nofilter_pkr_full__GFX103PLUS          = 0x0000002e,
    TD_PERF_SEL_nofilter_pkr_full_due_to_arb__GFX103PLUS = 0x0000002f,
    TD_PERF_SEL_ray_tracing_bvh4_pkr_full__GFX103PLUS  = 0x00000030,
    TD_PERF_SEL_ray_tracing_bvh4_pkr_full_due_to_arb__GFX103PLUS = 0x00000031,
    TD_PERF_SEL_gather4_instr__GFX103PLUS              = 0x00000032,
    TD_PERF_SEL_gather4h_instr__GFX103PLUS             = 0x00000033,
    TD_PERF_SEL_sample_instr__GFX103PLUS               = 0x00000036,
    TD_PERF_SEL_sample_c_instr__GFX103PLUS             = 0x00000037,
    TD_PERF_SEL_load_instr__GFX103PLUS                 = 0x00000038,
    TD_PERF_SEL_write_ack_instr__GFX103PLUS            = 0x0000003a,
    TD_PERF_SEL_d16_en_instr__GFX103PLUS               = 0x0000003b,
    TD_PERF_SEL_bypassLerp_instr__GFX103PLUS           = 0x0000003c,
    TD_PERF_SEL_min_max_filter_instr__GFX103PLUS       = 0x0000003d,
    TD_PERF_SEL_one_comp_return_instr__GFX103PLUS      = 0x0000003e,
    TD_PERF_SEL_two_comp_return_instr__GFX103PLUS      = 0x0000003f,
    TD_PERF_SEL_three_comp_return_instr__GFX103PLUS    = 0x00000040,
    TD_PERF_SEL_four_comp_return_instr__GFX103PLUS     = 0x00000041,
    TD_PERF_SEL_user_defined_border__GFX103PLUS        = 0x00000042,
    TD_PERF_SEL_white_border__GFX103PLUS               = 0x00000043,
    TD_PERF_SEL_opaque_black_border__GFX103PLUS        = 0x00000044,
    TD_PERF_SEL_lod_warn_from_ta__GFX103PLUS           = 0x00000045,
    TD_PERF_SEL_instruction_dest_is_lds__GFX103PLUS    = 0x00000046,
    TD_PERF_SEL_td_cycling_of_nofilter_instr_2cycles__GFX103PLUS = 0x00000047,
    TD_PERF_SEL_td_cycling_of_nofilter_instr_4cycles__GFX103PLUS = 0x00000048,
    TD_PERF_SEL_tc_cycling_of_nofilter_instr_2cycles__GFX103PLUS = 0x00000049,
    TD_PERF_SEL_tc_cycling_of_nofilter_instr_4cycles__GFX103PLUS = 0x0000004a,
    TD_PERF_SEL_out_of_order_instr__GFX103PLUS         = 0x0000004b,
    TD_PERF_SEL_total_num_instr__GFX103PLUS            = 0x0000004c,
    TD_PERF_SEL_total_num_instr_with_perf_wdw__GFX103PLUS = 0x0000004d,
    TD_PERF_SEL_total_num_sampler_instr__GFX103PLUS    = 0x0000004e,
    TD_PERF_SEL_total_num_sampler_instr_with_perf_wdw__GFX103PLUS = 0x0000004f,
    TD_PERF_SEL_total_num_nofilter_instr__GFX103PLUS   = 0x00000050,
    TD_PERF_SEL_total_num_nofilter_instr_with_perf_wdw__GFX103PLUS = 0x00000051,
    TD_PERF_SEL_total_num_ray_tracing_bvh4_instr__GFX103PLUS = 0x00000052,
    TD_PERF_SEL_total_num_ray_tracing_bvh4_instr_with_perf_wdw__GFX103PLUS = 0x00000053,
    TD_PERF_SEL_status_packet__GFX103PLUS              = 0x00000056,
    TD_PERF_SEL_done_scoreboard_max_stored_cnt__GFX103PLUS = 0x00000059,
    TD_PERF_SEL_done_scoreboard_max_waiting_cnt__GFX103PLUS = 0x0000005a,
    TD_PERF_SEL_done_scoreboard_not_empty__GFX103PLUS  = 0x0000005b,
    TD_PERF_SEL_done_scoreboard_is_full__GFX103PLUS    = 0x0000005c,
    TD_PERF_SEL_done_scoreboard_bp_due_to_ooo__GFX103PLUS = 0x0000005d,
    TD_PERF_SEL_done_scoreboard_bp_due_to_lds__GFX103PLUS = 0x0000005e,
    TD_PERF_SEL_nofilter_formatters_turned_on__GFX103PLUS = 0x0000005f,
    TD_PERF_SEL_nofilter_insert_extra_comps__GFX103PLUS = 0x00000060,
    TD_PERF_SEL_nofilter_popcount_dmask_gt_num_comp_of_fmt__GFX103PLUS = 0x00000061,
    TD_PERF_SEL_nofilter_popcount_dmask_lt_num_comp_of_fmt__GFX103PLUS = 0x00000062,
    TD_PERF_SEL_msaa_load_instr__GFX103PLUS            = 0x00000063,
    TD_PERF_SEL_blend_prt_with_prt_default_0__GFX103PLUS = 0x00000064,
    TD_PERF_SEL_blend_prt_with_prt_default_1__GFX103PLUS = 0x00000065,
    TD_PERF_SEL_resmap_instr__GFX103PLUS               = 0x00000066,
    TD_PERF_SEL_prt_ack_instr__GFX103PLUS              = 0x00000067,
    TD_PERF_SEL_resmap_with_volume_filtering__GFX103PLUS = 0x00000068,
    TD_PERF_SEL_resmap_with_aniso_filtering__GFX103PLUS = 0x00000069,
    TD_PERF_SEL_resmap_with_no_more_filtering__GFX103PLUS = 0x0000006a,
    TD_PERF_SEL_resmap_with_cubemap_corner__GFX103PLUS = 0x0000006b,
    TD_PERF_SEL_ray_tracing_bvh4_threads_per_instruction_is_0__GFX103PLUS = 0x0000006c,
    TD_PERF_SEL_ray_tracing_bvh4_threads_per_instruction_is_1__GFX103PLUS = 0x0000006d,
    TD_PERF_SEL_ray_tracing_bvh4_threads_per_instruction_is_2__GFX103PLUS = 0x0000006e,
    TD_PERF_SEL_ray_tracing_bvh4_threads_per_instruction_is_3to4__GFX103PLUS = 0x0000006f,
    TD_PERF_SEL_ray_tracing_bvh4_threads_per_instruction_is_5to8__GFX103PLUS = 0x00000070,
    TD_PERF_SEL_ray_tracing_bvh4_threads_per_instruction_is_9to16__GFX103PLUS = 0x00000071,
    TD_PERF_SEL_ray_tracing_bvh4_threads_per_instruction_is_17to31__GFX103PLUS = 0x00000072,
    TD_PERF_SEL_ray_tracing_bvh4_threads_per_instruction_is_32__GFX103PLUS = 0x00000073,
    TD_PERF_SEL_ray_tracing_bvh4_fp16_box_node__GFX103PLUS = 0x00000074,
    TD_PERF_SEL_ray_tracing_bvh4_fp32_box_node__GFX103PLUS = 0x00000075,
    TD_PERF_SEL_ray_tracing_bvh4_tri_node__GFX103PLUS  = 0x00000076,
    TD_PERF_SEL_ray_tracing_bvh4_dropped_box_node__GFX103PLUS = 0x00000077,
    TD_PERF_SEL_ray_tracing_bvh4_dropped_tri_node__GFX103PLUS = 0x00000078,
    TD_PERF_SEL_ray_tracing_bvh4_invalid_box_node__GFX103PLUS = 0x00000079,
    TD_PERF_SEL_ray_tracing_bvh4_invalid_tri_node__GFX103PLUS = 0x0000007a,
    TD_PERF_SEL_ray_tracing_bvh4_box_sort_en__GFX103PLUS = 0x0000007b,
    TD_PERF_SEL_ray_tracing_bvh4_box_grow_val_nonzero__GFX103PLUS = 0x0000007c,
    TD_PERF_SEL_ray_tracing_bvh4_num_box_with_inf_or_nan_vtx__GFX103PLUS = 0x0000007d,
    TD_PERF_SEL_ray_tracing_bvh4_num_tri_with_inf_or_nan_vtx__GFX103PLUS = 0x0000007e,
    TD_PERF_SEL_ray_tracing_bvh4_num_box_that_squashed_a_nan__GFX103PLUS = 0x0000007f,
    TD_PERF_SEL_ray_tracing_bvh4_num_box_misses__GFX103PLUS = 0x00000080,
    TD_PERF_SEL_ray_tracing_bvh4_num_tri_misses__GFX103PLUS = 0x00000081,
    TD_PERF_SEL_ray_tracing_bvh4_num_tri_tie_breakers__GFX103PLUS = 0x00000082,
    TD_PERF_SEL_burst_bin_preempting_nofilter_1__GFX103PLUS = 0x00000083,
    TD_PERF_SEL_burst_bin_preempting_nofilter_2to4__GFX103PLUS = 0x00000084,
    TD_PERF_SEL_burst_bin_preempting_nofilter_5to7__GFX103PLUS = 0x00000085,
    TD_PERF_SEL_burst_bin_preempting_nofilter_8to16__GFX103PLUS = 0x00000086,
    TD_PERF_SEL_burst_bin_preempting_nofilter_gt16__GFX103PLUS = 0x00000087,
    TD_PERF_SEL_burst_bin_sampler_1__GFX103PLUS        = 0x00000088,
    TD_PERF_SEL_burst_bin_sampler_2to8__GFX103PLUS     = 0x00000089,
    TD_PERF_SEL_burst_bin_sampler_9to16__GFX103PLUS    = 0x0000008a,
    TD_PERF_SEL_burst_bin_sampler_gt16__GFX103PLUS     = 0x0000008b,
    TD_PERF_SEL_burst_bin_gather_1__GFX103PLUS         = 0x0000008c,
    TD_PERF_SEL_burst_bin_gather_2to8__GFX103PLUS      = 0x0000008d,
    TD_PERF_SEL_burst_bin_gather_9to16__GFX103PLUS     = 0x0000008e,
    TD_PERF_SEL_burst_bin_gather_gt16__GFX103PLUS      = 0x0000008f,
    TD_PERF_SEL_burst_bin_nofilter_1__GFX103PLUS       = 0x00000090,
    TD_PERF_SEL_burst_bin_nofilter_2to4__GFX103PLUS    = 0x00000091,
    TD_PERF_SEL_burst_bin_nofilter_5to7__GFX103PLUS    = 0x00000092,
    TD_PERF_SEL_burst_bin_nofilter_8to16__GFX103PLUS   = 0x00000093,
    TD_PERF_SEL_burst_bin_nofilter_gt16__GFX103PLUS    = 0x00000094,
    TD_PERF_SEL_burst_bin_bvh4_1__GFX103PLUS           = 0x00000095,
    TD_PERF_SEL_burst_bin_bvh4_2to8__GFX103PLUS        = 0x00000096,
    TD_PERF_SEL_burst_bin_bvh4_9to16__GFX103PLUS       = 0x00000097,
    TD_PERF_SEL_burst_bin_bvh4_gt16__GFX103PLUS        = 0x00000098,
    TD_PERF_SEL_burst_bin_bvh4_box_nodes_1__GFX103PLUS = 0x00000099,
    TD_PERF_SEL_burst_bin_bvh4_box_nodes_2to4__GFX103PLUS = 0x0000009a,
    TD_PERF_SEL_burst_bin_bvh4_box_nodes_5to7__GFX103PLUS = 0x0000009b,
    TD_PERF_SEL_burst_bin_bvh4_box_nodes_8to16__GFX103PLUS = 0x0000009c,
    TD_PERF_SEL_burst_bin_bvh4_box_nodes_gt16__GFX103PLUS = 0x0000009d,
    TD_PERF_SEL_burst_bin_bvh4_tri_nodes_1__GFX103PLUS = 0x0000009e,
    TD_PERF_SEL_burst_bin_bvh4_tri_nodes_2to8__GFX103PLUS = 0x0000009f,
    TD_PERF_SEL_burst_bin_bvh4_tri_nodes_9to16__GFX103PLUS = 0x000000a0,
    TD_PERF_SEL_burst_bin_bvh4_tri_nodes_gt16__GFX103PLUS = 0x000000a1,
    TD_PERF_SEL_burst_bin_bvh4_dropped_nodes_1__GFX103PLUS = 0x000000a2,
    TD_PERF_SEL_burst_bin_bvh4_dropped_nodes_2to8__GFX103PLUS = 0x000000a3,
    TD_PERF_SEL_burst_bin_bvh4_dropped_nodes_9to16__GFX103PLUS = 0x000000a4,
    TD_PERF_SEL_burst_bin_bvh4_dropped_nodes_gt16__GFX103PLUS = 0x000000a5,
    TD_PERF_SEL_burst_bin_bvh4_invalid_nodes_1__GFX103PLUS = 0x000000a6,
    TD_PERF_SEL_burst_bin_bvh4_invalid_nodes_2to8__GFX103PLUS = 0x000000a7,
    TD_PERF_SEL_burst_bin_bvh4_invalid_nodes_9to16__GFX103PLUS = 0x000000a8,
    TD_PERF_SEL_burst_bin_bvh4_invalid_nodes_gt16__GFX103PLUS = 0x000000a9,
    TD_PERF_SEL_bubble_bin_ta_waiting_for_tc_data_0__GFX103PLUS = 0x000000aa,
    TD_PERF_SEL_bubble_bin_ta_waiting_for_tc_data_1__GFX103PLUS = 0x000000ab,
    TD_PERF_SEL_bubble_bin_ta_waiting_for_tc_data_2to31__GFX103PLUS = 0x000000ac,
    TD_PERF_SEL_bubble_bin_ta_waiting_for_tc_data_32to127__GFX103PLUS = 0x000000ad,
    TD_PERF_SEL_bubble_bin_ta_waiting_for_tc_data_128to511__GFX103PLUS = 0x000000ae,
    TD_PERF_SEL_bubble_bin_ta_waiting_for_tc_data_gt511__GFX103PLUS = 0x000000af,
    TD_PERF_SEL_bubble_bin_lds_stall_1to3__GFX103PLUS  = 0x000000b0,
    TD_PERF_SEL_bubble_bin_lds_stall_4to7__GFX103PLUS  = 0x000000b1,
    TD_PERF_SEL_bubble_bin_lds_stall_8to15__GFX103PLUS = 0x000000b2,
    TD_PERF_SEL_bubble_bin_lds_stall_gt15__GFX103PLUS  = 0x000000b3,
    TD_PERF_SEL_preempting_nofilter_max_cnt__GFX103PLUS = 0x000000b4,
    TD_PERF_SEL_sampler_lerp0_active__GFX103PLUS       = 0x000000b5,
    TD_PERF_SEL_sampler_lerp1_active__GFX103PLUS       = 0x000000b6,
    TD_PERF_SEL_sampler_lerp2_active__GFX103PLUS       = 0x000000b7,
    TD_PERF_SEL_sampler_lerp3_active__GFX103PLUS       = 0x000000b8,
    TD_PERF_SEL_sampler_bilerp_sclk_en__GFX10VRS       = 0x00000008,
    TD_PERF_SEL_sampler_bypass_sclk_en__GFX10VRS       = 0x00000009,
    TD_PERF_SEL_sampler_minmax_sclk_en__GFX10VRS       = 0x0000000a,
    TD_PERF_SEL_sampler_accum_sclk_en__GFX10VRS        = 0x0000000b,
    TD_PERF_SEL_sampler_format_flt_sclk_en__GFX10VRS   = 0x0000000c,
    TD_PERF_SEL_sampler_format_fxdpt_sclk_en__GFX10VRS = 0x0000000d,
    TD_PERF_SEL_sampler_out_sclk_en__GFX10VRS          = 0x0000000e,
    TD_PERF_SEL_nofilter_sclk_en__GFX10VRS             = 0x0000000f,
    TD_PERF_SEL_nofilter_d16_sclk_en__GFX10VRS         = 0x00000010,
    TD_PERF_SEL_RESERVED_17__GFX10VRS                  = 0x00000011,
    TD_PERF_SEL_RESERVED_18__GFX10VRS                  = 0x00000012,
    TD_PERF_SEL_RESERVED_19__GFX10VRS                  = 0x00000013,
    TD_PERF_SEL_RESERVED_20__GFX10VRS                  = 0x00000014,
    TD_PERF_SEL_RESERVED_21__GFX10VRS                  = 0x00000015,
    TD_PERF_SEL_RESERVED_22__GFX10VRS                  = 0x00000016,
    TD_PERF_SEL_ray_tracing_bvh4_sclk_en__GFX10VRS     = 0x00000017,
    TD_PERF_SEL_gather4h_packed_instr__GFX10VRS        = 0x00000034,
    TD_PERF_SEL_gather8h_packed_instr__GFX10VRS        = 0x00000035,
    TD_PERF_SEL_ldfptr_instr__GFX10VRS                 = 0x00000039,
    TD_PERF_SEL_mixmode_instr__GFX10VRS                = 0x00000054,
    TD_PERF_SEL_mixmode_resource__GFX10VRS             = 0x00000055,
    TD_PERF_SEL_address_cmd_poison__GFX10VRS           = 0x00000057,
    TD_PERF_SEL_data_poison__GFX10VRS                  = 0x00000058,
    TD_PERF_SEL_nofilter_total_num_comps_to_lds__GFX10VRS = 0x000000b9,
    TD_PERF_SEL_nofilter_byte_cycling_4cycles__GFX10VRS = 0x000000ba,
    TD_PERF_SEL_nofilter_byte_cycling_8cycles__GFX10VRS = 0x000000bb,
    TD_PERF_SEL_nofilter_byte_cycling_16cycles__GFX10VRS = 0x000000bc,
    TD_PERF_SEL_nofilter_dword_cycling_2cycles__GFX10VRS = 0x000000bd,
    TD_PERF_SEL_nofilter_dword_cycling_4cycles__GFX10VRS = 0x000000be,
    TD_PERF_SEL_input_bp_due_to_done_scoreboard_full__GFX10VRS = 0x000000bf,
    TD_PERF_SEL_sampler_preformatter_sclk_en__GFX11    = 0x00000008,
    TD_PERF_SEL_sampler_bilerp_sclk_en__GFX11          = 0x00000009,
    TD_PERF_SEL_sampler_bypass_sclk_en__GFX11          = 0x0000000a,
    TD_PERF_SEL_sampler_minmax_sclk_en__GFX11          = 0x0000000b,
    TD_PERF_SEL_sampler_format_flt_sclk_en__GFX11      = 0x0000000d,
    TD_PERF_SEL_sampler_format_fxdpt_sclk_en__GFX11    = 0x0000000e,
    TD_PERF_SEL_sampler_out_sclk_en__GFX11             = 0x0000000f,
    TD_PERF_SEL_nofilter_sclk_en__GFX11                = 0x00000010,
    TD_PERF_SEL_nofilter_d32_sclk_en__GFX11            = 0x00000011,
    TD_PERF_SEL_nofilter_d16_sclk_en__GFX11            = 0x00000012,
    TD_PERF_SEL_ray_tracing_bvh4_sclk_en__GFX11        = 0x00000016,
    TD_PERF_SEL_ray_tracing_bvh4_ip_sclk_en__GFX11     = 0x00000017,
    TD_PERF_SEL_sampler_accum_sclk_en__GFX110          = 0x0000000c,
    TD_PERF_SEL_mixmode_instr__GFX110                  = 0x00000054,
    TD_PERF_SEL_mixmode_resource__GFX110               = 0x00000055,
    TD_PERF_SEL_nofilter_total_num_comps_to_lds__GFX110 = 0x000000b9,
    TD_PERF_SEL_nofilter_byte_cycling_4cycles__GFX110  = 0x000000ba,
    TD_PERF_SEL_nofilter_byte_cycling_8cycles__GFX110  = 0x000000bb,
    TD_PERF_SEL_nofilter_byte_cycling_16cycles__GFX110 = 0x000000bc,
    TD_PERF_SEL_nofilter_dword_cycling_2cycles__GFX110 = 0x000000bd,
    TD_PERF_SEL_nofilter_dword_cycling_4cycles__GFX110 = 0x000000be,
    TD_PERF_SEL_input_bp_due_to_done_scoreboard_full__GFX110 = 0x000000bf,
    TD_PERF_SEL_ray_tracing_bvh4_instr_invld_thread_cnt__GFX110 = 0x000000c0,
    TD_PERF_SEL_ray_tracing_bvh4_box_sort_closest_child__GFX110 = 0x000000c1,
    TD_PERF_SEL_ray_tracing_bvh4_box_sort_largest_first__GFX110 = 0x000000c2,
    TD_PERF_SEL_ray_tracing_bvh4_box_sort_closest_midpoint__GFX110 = 0x000000c3,
    TD_PERF_SEL_store_preempts_a_load__GFX110          = 0x000000c4,
    TD_PERF_SEL_sampler_accum_sclk_en__STX1            = 0x0000000c,
    TD_PERF_SEL_mixmode_instr__STX1                    = 0x00000054,
    TD_PERF_SEL_mixmode_resource__STX1                 = 0x00000055,
    TD_PERF_SEL_nofilter_total_num_comps_to_lds__STX1  = 0x000000b9,
    TD_PERF_SEL_nofilter_byte_cycling_4cycles__STX1    = 0x000000ba,
    TD_PERF_SEL_nofilter_byte_cycling_8cycles__STX1    = 0x000000bb,
    TD_PERF_SEL_nofilter_byte_cycling_16cycles__STX1   = 0x000000bc,
    TD_PERF_SEL_nofilter_dword_cycling_2cycles__STX1   = 0x000000bd,
    TD_PERF_SEL_nofilter_dword_cycling_4cycles__STX1   = 0x000000be,
    TD_PERF_SEL_input_bp_due_to_done_scoreboard_full__STX1 = 0x000000bf,
    TD_PERF_SEL_ray_tracing_bvh4_instr_invld_thread_cnt__STX1 = 0x000000c0,
    TD_PERF_SEL_ray_tracing_bvh4_box_sort_closest_child__STX1 = 0x000000c1,
    TD_PERF_SEL_ray_tracing_bvh4_box_sort_largest_first__STX1 = 0x000000c2,
    TD_PERF_SEL_ray_tracing_bvh4_box_sort_closest_midpoint__STX1 = 0x000000c3,
    TD_PERF_SEL_store_preempts_a_load__STX1            = 0x000000c4,
#if CHIP_HDR_STRIX_HALO
    TD_PERF_SEL_sampler_post_blend_sclk_en__STXH       = 0x0000000c,
    TD_PERF_SEL_getlod_instr__STXH                     = 0x00000034,
    TD_PERF_SEL_ps_load_instr__STXH                    = 0x00000039,
    TD_PERF_SEL_sampler_lerp4_active__STXH             = 0x000000b9,
    TD_PERF_SEL_sampler_lerp5_active__STXH             = 0x000000ba,
    TD_PERF_SEL_sampler_lerp6_active__STXH             = 0x000000bb,
    TD_PERF_SEL_sampler_lerp7_active__STXH             = 0x000000bc,
    TD_PERF_SEL_nofilter_total_num_comps_to_lds__STXH  = 0x000000bd,
    TD_PERF_SEL_nofilter_byte_cycling_4cycles__STXH    = 0x000000be,
    TD_PERF_SEL_nofilter_byte_cycling_8cycles__STXH    = 0x000000bf,
    TD_PERF_SEL_nofilter_byte_cycling_16cycles__STXH   = 0x000000c0,
    TD_PERF_SEL_nofilter_dword_cycling_2cycles__STXH   = 0x000000c1,
    TD_PERF_SEL_nofilter_dword_cycling_4cycles__STXH   = 0x000000c2,
    TD_PERF_SEL_input_bp_due_to_done_scoreboard_full__STXH = 0x000000c3,
    TD_PERF_SEL_ray_tracing_bvh4_instr_invld_thread_cnt__STXH = 0x000000c4,
    TD_PERF_SEL_ray_tracing_bvh4_box_sort_closest_child__STXH = 0x000000c5,
    TD_PERF_SEL_ray_tracing_bvh4_box_sort_largest_first__STXH = 0x000000c6,
    TD_PERF_SEL_ray_tracing_bvh4_box_sort_closest_midpoint__STXH = 0x000000c7,
    TD_PERF_SEL_store_preempts_a_load__STXH            = 0x000000c8,
    TD_PERF_SEL_sample_2x_instr__STXH                  = 0x000000c9,
    TD_PERF_SEL_gather4_2x_instr__STXH                 = 0x000000ca,
    TD_PERF_SEL_gather4h_2x_instr__STXH                = 0x000000cb,
    TD_PERF_SEL_getlod_2x_instr__STXH                  = 0x000000cc,
    TD_PERF_SEL_resmap_2x_instr__STXH                  = 0x000000cd,
    TD_PERF_SEL_2x_sampler_op_with_1_unlit_quad__STXH  = 0x000000ce,
    TD_PERF_SEL_2x_sampler_op_with_both_quads_unlit__STXH = 0x000000cf,
#endif
} TD_PERFCOUNT_SEL;

constexpr unsigned int MaxTdPerfcountSelGfx101         = TD_PERF_SEL_nofilter_popcount_dmask_lt_num_comp_of_fmt__GFX101;
constexpr unsigned int MaxTdPerfcountSelGfx10Vrs       = TD_PERF_SEL_input_bp_due_to_done_scoreboard_full__GFX10VRS;
constexpr unsigned int MaxTdPerfcountSelGfx110         = TD_PERF_SEL_store_preempts_a_load__GFX110;
constexpr unsigned int MaxTdPerfcountSelStx1           = TD_PERF_SEL_store_preempts_a_load__STX1;
#if CHIP_HDR_STRIX_HALO
constexpr unsigned int MaxTdPerfcountSelStxH           = TD_PERF_SEL_2x_sampler_op_with_both_quads_unlit__STXH;
#endif

typedef enum TEX_BC_SWIZZLE {
    TEX_BC_Swizzle_XYZW                                = 0x00000000,
    TEX_BC_Swizzle_XWYZ                                = 0x00000001,
    TEX_BC_Swizzle_WZYX                                = 0x00000002,
    TEX_BC_Swizzle_WXYZ                                = 0x00000003,
    TEX_BC_Swizzle_ZYXW                                = 0x00000004,
    TEX_BC_Swizzle_YXWZ                                = 0x00000005,
} TEX_BC_SWIZZLE;

typedef enum TEX_BORDER_COLOR_TYPE {
    TEX_BorderColor_TransparentBlack                   = 0x00000000,
    TEX_BorderColor_OpaqueBlack                        = 0x00000001,
    TEX_BorderColor_OpaqueWhite                        = 0x00000002,
    TEX_BorderColor_Register                           = 0x00000003,
} TEX_BORDER_COLOR_TYPE;

typedef enum TEX_CHROMA_KEY {
    TEX_ChromaKey_Disabled                             = 0x00000000,
    TEX_ChromaKey_Kill                                 = 0x00000001,
    TEX_ChromaKey_Blend                                = 0x00000002,
    TEX_ChromaKey_RESERVED_3                           = 0x00000003,
} TEX_CHROMA_KEY;

typedef enum TEX_CLAMP {
    TEX_Clamp_Repeat                                   = 0x00000000,
    TEX_Clamp_Mirror                                   = 0x00000001,
    TEX_Clamp_ClampToLast                              = 0x00000002,
    TEX_Clamp_MirrorOnceToLast                         = 0x00000003,
    TEX_Clamp_ClampHalfToBorder                        = 0x00000004,
    TEX_Clamp_MirrorOnceHalfToBorder                   = 0x00000005,
    TEX_Clamp_ClampToBorder                            = 0x00000006,
    TEX_Clamp_MirrorOnceToBorder                       = 0x00000007,
} TEX_CLAMP;

typedef enum TEX_COORD_TYPE {
    TEX_CoordType_Unnormalized                         = 0x00000000,
    TEX_CoordType_Normalized                           = 0x00000001,
} TEX_COORD_TYPE;

typedef enum TEX_DEPTH_COMPARE_FUNCTION {
    TEX_DepthCompareFunction_Never                     = 0x00000000,
    TEX_DepthCompareFunction_Less                      = 0x00000001,
    TEX_DepthCompareFunction_Equal                     = 0x00000002,
    TEX_DepthCompareFunction_LessEqual                 = 0x00000003,
    TEX_DepthCompareFunction_Greater                   = 0x00000004,
    TEX_DepthCompareFunction_NotEqual                  = 0x00000005,
    TEX_DepthCompareFunction_GreaterEqual              = 0x00000006,
    TEX_DepthCompareFunction_Always                    = 0x00000007,
} TEX_DEPTH_COMPARE_FUNCTION;

typedef enum TEX_DIM {
    TEX_Dim_1D                                         = 0x00000000,
    TEX_Dim_2D                                         = 0x00000001,
    TEX_Dim_3D                                         = 0x00000002,
    TEX_Dim_CubeMap                                    = 0x00000003,
    TEX_Dim_1DArray                                    = 0x00000004,
    TEX_Dim_2DArray                                    = 0x00000005,
    TEX_Dim_2D_MSAA                                    = 0x00000006,
    TEX_Dim_2DArray_MSAA                               = 0x00000007,
} TEX_DIM;

typedef enum TEX_FORMAT_COMP {
    TEX_FormatComp_Unsigned                            = 0x00000000,
    TEX_FormatComp_Signed                              = 0x00000001,
    TEX_FormatComp_UnsignedBiased                      = 0x00000002,
    TEX_FormatComp_RESERVED_3                          = 0x00000003,
} TEX_FORMAT_COMP;

typedef enum TEX_MAX_ANISO_RATIO {
    TEX_MaxAnisoRatio_1to1                             = 0x00000000,
    TEX_MaxAnisoRatio_2to1                             = 0x00000001,
    TEX_MaxAnisoRatio_4to1                             = 0x00000002,
    TEX_MaxAnisoRatio_8to1                             = 0x00000003,
    TEX_MaxAnisoRatio_16to1                            = 0x00000004,
    TEX_MaxAnisoRatio_RESERVED_5                       = 0x00000005,
    TEX_MaxAnisoRatio_RESERVED_6                       = 0x00000006,
    TEX_MaxAnisoRatio_RESERVED_7                       = 0x00000007,
} TEX_MAX_ANISO_RATIO;

typedef enum TEX_MIP_FILTER {
    TEX_MipFilter_None                                 = 0x00000000,
    TEX_MipFilter_Point                                = 0x00000001,
    TEX_MipFilter_Linear                               = 0x00000002,
    TEX_MipFilter_Point_Aniso_Adj                      = 0x00000003,
} TEX_MIP_FILTER;

typedef enum TEX_REQUEST_SIZE {
    TEX_RequestSize_32B                                = 0x00000000,
    TEX_RequestSize_64B                                = 0x00000001,
    TEX_RequestSize_128B                               = 0x00000002,
    TEX_RequestSize_2X64B                              = 0x00000003,
} TEX_REQUEST_SIZE;

typedef enum TEX_SAMPLER_TYPE {
    TEX_SamplerType_Invalid                            = 0x00000000,
    TEX_SamplerType_Valid                              = 0x00000001,
} TEX_SAMPLER_TYPE;

typedef enum TEX_XY_FILTER {
    TEX_XYFilter_Point                                 = 0x00000000,
    TEX_XYFilter_Linear                                = 0x00000001,
    TEX_XYFilter_AnisoPoint                            = 0x00000002,
    TEX_XYFilter_AnisoLinear                           = 0x00000003,
} TEX_XY_FILTER;

typedef enum TEX_Z_FILTER {
    TEX_ZFilter_None                                   = 0x00000000,
    TEX_ZFilter_Point                                  = 0x00000001,
    TEX_ZFilter_Linear                                 = 0x00000002,
    TEX_ZFilter_RESERVED_3                             = 0x00000003,
} TEX_Z_FILTER;

typedef enum UMC_PERFCOUNT_SELECT {
    UMC_PERF_SEL_IncomingPh__GFX101                    = 0x00000001,
    UMC_PERF_SEL_IncomingPm__GFX101                    = 0x00000002,
    UMC_PERF_SEL_IncomingPc__GFX101                    = 0x00000003,
    UMC_PERF_SEL_IncomingPgtMiss__GFX101               = 0x00000004,
    UMC_PERF_SEL_ActCmd__GFX101                        = 0x00000005,
    UMC_PERF_SEL_PchgCmd__GFX101                       = 0x00000006,
    UMC_PERF_SEL_SpecPchgCmd__GFX101                   = 0x00000007,
    UMC_PERF_SEL_PchgAllCmd__GFX101                    = 0x00000008,
    UMC_PERF_SEL_OtherPchgCmd__GFX101                  = 0x00000009,
    UMC_PERF_SEL_CasCmd__GFX101                        = 0x0000000a,
    UMC_PERF_SEL_CasWithAP__GFX101                     = 0x0000000b,
    UMC_PERF_SEL_CasWithBurstCh__GFX101                = 0x0000000c,
    UMC_PERF_SEL_TurnRndRdWr__GFX101                   = 0x0000000d,
    UMC_PERF_SEL_Back2BackSame__GFX101                 = 0x0000000e,
    UMC_PERF_SEL_Back2BackDiffB__GFX101                = 0x0000000f,
    UMC_PERF_SEL_Back2BackDiffCidSameCS__GFX101        = 0x00000010,
    UMC_PERF_SEL_Back2BackDiffCSSameDimm__GFX101       = 0x00000011,
    UMC_PERF_SEL_Back2BackDiffDimm__GFX101             = 0x00000012,
    UMC_PERF_SEL_CmdSlotClks__GFX101                   = 0x00000013,
    UMC_PERF_SEL_DataSlotClks__GFX101                  = 0x00000014,
    UMC_PERF_SEL_PageTableOverflow__GFX101             = 0x00000015,
    UMC_PERF_SEL_CmdError__GFX101                      = 0x00000016,
    UMC_PERF_SEL_DataError__GFX101                     = 0x00000017,
    UMC_PERF_SEL_RetryParity__GFX101                   = 0x00000018,
    UMC_PERF_SEL_RetryCRC__GFX101                      = 0x00000019,
    UMC_PERF_SEL_BypassSat__GFX101                     = 0x0000001a,
    UMC_PERF_SEL_PrechagePwrDownClks__GFX101           = 0x0000001b,
    UMC_PERF_SEL_ThermalThrottleClks__GFX101           = 0x0000001c,
    UMC_PERF_SEL_DcqOccupancy_00__GFX101               = 0x0000001d,
    UMC_PERF_SEL_DcqOccupancy_25__GFX101               = 0x0000001e,
    UMC_PERF_SEL_DcqOccupancy_50__GFX101               = 0x0000001f,
    UMC_PERF_SEL_DcqOccupancy_75__GFX101               = 0x00000020,
    UMC_PERF_SEL_DcqOccupancy_90__GFX101               = 0x00000021,
    UMC_PERF_SEL_SelfRefreshClks__GFX101               = 0x00000022,
    UMC_PERF_SEL_PrechargePwrDownAllClks__GFX101       = 0x00000023,
    UMC_PERF_SEL_PendReqCnt__GFX101                    = 0x00000024,
    UMC_PERF_SEL_AutoRefCnt__GFX101                    = 0x00000025,
    UMC_PERF_SEL_BeqRdWrCmd__GFX101                    = 0x00000026,
    UMC_PERF_SEL_BeqEdcErr__GFX101                     = 0x00000027,
    UMC_PERF_SEL_Switch2Read__GFX101                   = 0x0000002a,
    UMC_PERF_SEL_Switch2Write__GFX101                  = 0x0000002b,
    UMC_PERF_SEL_ArbRefReqClks__GFX101                 = 0x0000002c,
    UMC_PERF_SEL_UrgRefStbkClks__GFX101                = 0x0000002d,
    UMC_PERF_SEL_RtCalEvtCnt__GFX101                   = 0x00000030,
    UMC_PERF_SEL_RtCalBlockClks__GFX101                = 0x00000031,
    UMC_PERF_SEL_RefreshType__GFX101                   = 0x00000032,
    UMC_PERF_SEL_NONE__GFX11                           = 0x00000000,
    UMC_PERF_SEL_SdpPh__GFX11                          = 0x00000001,
    UMC_PERF_SEL_SdpPm__GFX11                          = 0x00000002,
    UMC_PERF_SEL_SdpPc__GFX11                          = 0x00000003,
    UMC_PERF_SEL_ActCmd__GFX11                         = 0x00000005,
    UMC_PERF_SEL_PchgCmd__GFX11                        = 0x00000006,
    UMC_PERF_SEL_SpecPchgCmd__GFX11                    = 0x00000007,
    UMC_PERF_SEL_PchgAllCmd__GFX11                     = 0x00000008,
    UMC_PERF_SEL_OtherPchgCmd__GFX11                   = 0x00000009,
    UMC_PERF_SEL_CasCmd__GFX11                         = 0x0000000a,
    UMC_PERF_SEL_CasWithAP__GFX11                      = 0x0000000b,
    UMC_PERF_SEL_TurnRndRdWr__GFX11                    = 0x0000000d,
    UMC_PERF_SEL_CmdSlotClks__GFX11                    = 0x00000013,
    UMC_PERF_SEL_DataSlotClks__GFX11                   = 0x00000014,
    UMC_PERF_SEL_CmdError__GFX11                       = 0x00000016,
    UMC_PERF_SEL_DataError__GFX11                      = 0x00000017,
    UMC_PERF_SEL_RetryParity__GFX11                    = 0x00000018,
    UMC_PERF_SEL_RetryCRC__GFX11                       = 0x00000019,
    UMC_PERF_SEL_BypassSat__GFX11                      = 0x0000001a,
    UMC_PERF_SEL_PrechagePwrDownClks__GFX11            = 0x0000001b,
    UMC_PERF_SEL_ThermalThrottleClks__GFX11            = 0x0000001c,
    UMC_PERF_SEL_DcqOccupancy__GFX11                   = 0x0000001d,
    UMC_PERF_SEL_SelfRefreshClks__GFX11                = 0x00000022,
    UMC_PERF_SEL_PendReqCnt__GFX11                     = 0x00000024,
    UMC_PERF_SEL_AutoRefCnt__GFX11                     = 0x00000025,
    UMC_PERF_SEL_BeqRdWrCmd__GFX11                     = 0x00000026,
    UMC_PERF_SEL_BeqEdcErr__GFX11                      = 0x00000027,
    UMC_PERF_SEL_ArbRdWrPhase__GFX11                   = 0x0000002a,
    UMC_PERF_SEL_WriteCycle__GFX11                     = 0x0000002b,
    UMC_PERF_SEL_ArbRefReqClks__GFX11                  = 0x0000002c,
    UMC_PERF_SEL_UrgRefStBkClks__GFX11                 = 0x0000002d,
    UMC_PERF_SEL_RtCalEvtCnt__GFX11                    = 0x00000030,
    UMC_PERF_SEL_RtCalBlockClks__GFX11                 = 0x00000031,
    UMC_PERF_SEL_SdpCoherent__GFX11                    = 0x00000033,
    UMC_PERF_SEL_SdpChngPri__GFX11                     = 0x00000034,
    UMC_PERF_SEL_SdpCancel__GFX11                      = 0x00000035,
    UMC_PERF_SEL_SdpBurstLength__GFX11                 = 0x00000036,
    UMC_PERF_SEL_SdpBurstCnt__GFX11                    = 0x00000037,
    UMC_PERF_SEL_SdpGapLen__GFX11                      = 0x00000038,
    UMC_PERF_SEL_SdpGapCnt__GFX11                      = 0x00000039,
    UMC_PERF_SEL_CasCmdWSM__GFX11                      = 0x0000003a,
    UMC_PERF_SEL_CasCmdWDM__GFX11                      = 0x0000003b,
    UMC_PERF_SEL_CasBurstLength__GFX11                 = 0x0000003c,
    UMC_PERF_SEL_CasBurstCnt__GFX11                    = 0x0000003d,
    UMC_PERF_SEL_CasCmdCW__GFX11                       = 0x0000003e,
    UMC_PERF_SEL_TurnRndRdWr_noRd__GFX11               = 0x0000003f,
    UMC_PERF_SEL_TurnRndRdWr_RdThresh__GFX11           = 0x00000040,
    UMC_PERF_SEL_TurnRndWrRd_noWr__GFX11               = 0x00000041,
    UMC_PERF_SEL_TurnRndWrRd_WrThresh__GFX11           = 0x00000042,
    UMC_PERF_SEL_TurnRndWrRd_RdUrg__GFX11              = 0x00000043,
    UMC_PERF_SEL_PrechargePwrDownEntry__GFX11          = 0x00000044,
    UMC_PERF_SEL_ThermalThrottleEntry__GFX11           = 0x00000045,
    UMC_PERF_SEL_PgtActiveBanksCnt__GFX11              = 0x00000046,
    UMC_PERF_SEL_SelfRefreshEntry__GFX11               = 0x00000047,
    UMC_PERF_SEL_RdRspBusyNoCrdt__GFX11                = 0x00000048,
    UMC_PERF_SEL_RdstallRdRspBufNotRdy__GFX11          = 0x00000049,
    UMC_PERF_SEL_RdrspBusyVdciNotRdy__GFX11            = 0x0000004a,
    UMC_PERF_SEL_DfiLpReq__GFX11                       = 0x0000004b,
    UMC_PERF_SEL_DfiLpCyc__GFX11                       = 0x0000004c,
    UMC_PERF_SEL_BubbleOverall__GFX11                  = 0x0000004d,
    UMC_PERF_SEL_BeqEdcErrB0__GFX11                    = 0x0000004e,
    UMC_PERF_SEL_BeqEdcErrB1__GFX11                    = 0x0000004f,
    UMC_PERF_SEL_BeqEdcErrB2__GFX11                    = 0x00000050,
    UMC_PERF_SEL_BeqEdcErrB3__GFX11                    = 0x00000051,
    UMC_PERF_SEL_TempOverThresh__GFX11                 = 0x00000052,
    UMC_PERF_SEL_TempCnt__GFX11                        = 0x00000053,
    UMC_PERF_SEL_ReqFullEntry__GFX11                   = 0x00000054,
    UMC_PERF_SEL_NumReqs__GFX11                        = 0x00000055,
    UMC_PERF_SEL_CasWithDeAlloc__GFX11                 = 0x00000056,
    UMC_PERF_SEL_RfmCmd__GFX11                         = 0x00000057,
    UMC_PERF_SEL_SdpNewEntry__GFX11                    = 0x00000058,
    UMC_PERF_SEL_SdpHitEntry__GFX11                    = 0x00000059,
    UMC_PERF_SEL_PatrolScrub__GFX11                    = 0x0000005a,
    UMC_PERF_SEL_RedirectScrub__GFX11                  = 0x0000005b,
    UMC_PERF_SEL_PoisonScrub__GFX11                    = 0x0000005c,
    UMC_PERF_SEL_MemClear__GFX11                       = 0x0000005d,
    UMC_PERF_SEL_DroppedRedirectScrub__GFX11           = 0x0000005e,
    UMC_PERF_SEL_DroppedPoisonScrub__GFX11             = 0x0000005f,
    UMC_PERF_SEL_DroppedScrubWrite__GFX11              = 0x00000060,
    UMC_PERF_SEL_RfmBlockClk__GFX11                    = 0x00000061,
    UMC_PERF_SEL_NoRefOverThres__GFX11                 = 0x00000062,
    UMC_PERF_SEL_StoredRefOverThres__GFX11             = 0x00000063,
    UMC_PERF_SEL_PptRunning__GFX11                     = 0x00000064,
    UMC_PERF_SEL_PptTrig__GFX11                        = 0x00000065,
    UMC_PERF_SEL_PptMinWin__GFX11                      = 0x00000066,
    UMC_PERF_SEL_ClockCount__GFX11                     = 0x000000ff,
    UMC_PERF_SEL_NONE__NV2X                            = 0x00000000,
    UMC_PERF_SEL_SdpPh__NV2X                           = 0x00000001,
    UMC_PERF_SEL_SdpPm__NV2X                           = 0x00000002,
    UMC_PERF_SEL_SdpPc__NV2X                           = 0x00000003,
    UMC_PERF_SEL_ActCmd__NV2X                          = 0x00000005,
    UMC_PERF_SEL_PchgCmd__NV2X                         = 0x00000006,
    UMC_PERF_SEL_SpecPchgCmd__NV2X                     = 0x00000007,
    UMC_PERF_SEL_PchgAllCmd__NV2X                      = 0x00000008,
    UMC_PERF_SEL_OtherPchgCmd__NV2X                    = 0x00000009,
    UMC_PERF_SEL_CasCmd__NV2X                          = 0x0000000a,
    UMC_PERF_SEL_CasWithAP__NV2X                       = 0x0000000b,
    UMC_PERF_SEL_TurnRndRdWr__NV2X                     = 0x0000000d,
    UMC_PERF_SEL_CmdSlotClks__NV2X                     = 0x00000013,
    UMC_PERF_SEL_DataSlotClks__NV2X                    = 0x00000014,
    UMC_PERF_SEL_CmdError__NV2X                        = 0x00000016,
    UMC_PERF_SEL_DataError__NV2X                       = 0x00000017,
    UMC_PERF_SEL_RetryParity__NV2X                     = 0x00000018,
    UMC_PERF_SEL_RetryCRC__NV2X                        = 0x00000019,
    UMC_PERF_SEL_BypassSat__NV2X                       = 0x0000001a,
    UMC_PERF_SEL_PrechagePwrDownClks__NV2X             = 0x0000001b,
    UMC_PERF_SEL_ThermalThrottleClks__NV2X             = 0x0000001c,
    UMC_PERF_SEL_DcqOccupancy__NV2X                    = 0x0000001d,
    UMC_PERF_SEL_SelfRefreshClks__NV2X                 = 0x00000022,
    UMC_PERF_SEL_PendReqCnt__NV2X                      = 0x00000024,
    UMC_PERF_SEL_AutoRefCnt__NV2X                      = 0x00000025,
    UMC_PERF_SEL_BeqRdWrCmd__NV2X                      = 0x00000026,
    UMC_PERF_SEL_BeqEdcErr__NV2X                       = 0x00000027,
    UMC_PERF_SEL_ArbRdWrPhase__NV2X                    = 0x0000002a,
    UMC_PERF_SEL_WriteCycle__NV2X                      = 0x0000002b,
    UMC_PERF_SEL_ArbRefReqClks__NV2X                   = 0x0000002c,
    UMC_PERF_SEL_UrgRefStBkClks__NV2X                  = 0x0000002d,
    UMC_PERF_SEL_RtCalEvtCnt__NV2X                     = 0x00000030,
    UMC_PERF_SEL_RtCalBlockClks__NV2X                  = 0x00000031,
    UMC_PERF_SEL_SdpCoherent__NV2X                     = 0x00000033,
    UMC_PERF_SEL_SdpChngPri__NV2X                      = 0x00000034,
    UMC_PERF_SEL_SdpCancel__NV2X                       = 0x00000035,
    UMC_PERF_SEL_SdpBurstLength__NV2X                  = 0x00000036,
    UMC_PERF_SEL_SdpBurstCnt__NV2X                     = 0x00000037,
    UMC_PERF_SEL_SdpGapLen__NV2X                       = 0x00000038,
    UMC_PERF_SEL_SdpGapCnt__NV2X                       = 0x00000039,
    UMC_PERF_SEL_CasCmdWSM__NV2X                       = 0x0000003a,
    UMC_PERF_SEL_CasCmdWDM__NV2X                       = 0x0000003b,
    UMC_PERF_SEL_CasBurstLength__NV2X                  = 0x0000003c,
    UMC_PERF_SEL_CasBurstCnt__NV2X                     = 0x0000003d,
    UMC_PERF_SEL_CasCmdCW__NV2X                        = 0x0000003e,
    UMC_PERF_SEL_TurnRndRdWr_n__NV2X                   = 0x0000003f,
    UMC_PERF_SEL_TurnRndRdWr_RdThresh__NV2X            = 0x00000040,
    UMC_PERF_SEL_TurnRndWrRd_noWr__NV2X                = 0x00000041,
    UMC_PERF_SEL_TurnRndWrRd_WrThresh__NV2X            = 0x00000042,
    UMC_PERF_SEL_TurnRndWrRd_RdUrg__NV2X               = 0x00000043,
    UMC_PERF_SEL_PrechargePwrDownEntry__NV2X           = 0x00000044,
    UMC_PERF_SEL_ThermalThrottleEntry__NV2X            = 0x00000045,
    UMC_PERF_SEL_PgtActiveBanksCnt__NV2X               = 0x00000046,
    UMC_PERF_SEL_SelfRefreshEntry__NV2X                = 0x00000047,
    UMC_PERF_SEL_RdRspBusyNoCrdt__NV2X                 = 0x00000048,
    UMC_PERF_SEL_RdstallRdRspBufNotRdy__NV2X           = 0x00000049,
    UMC_PERF_SEL_RdrspBusyVdciNotRdy__NV2X             = 0x0000004a,
    UMC_PERF_SEL_DfiLpReq__NV2X                        = 0x0000004b,
    UMC_PERF_SEL_DfiLpCyc__NV2X                        = 0x0000004c,
    UMC_PERF_SEL_BubbleOverall__NV2X                   = 0x0000004d,
    UMC_PERF_SEL_BeqEdcErrB0__NV2X                     = 0x0000004e,
    UMC_PERF_SEL_BeqEdcErrB1__NV2X                     = 0x0000004f,
    UMC_PERF_SEL_TempOverThresh__NV2X                  = 0x00000052,
    UMC_PERF_SEL_TempCnt__NV2X                         = 0x00000053,
    UMC_PERF_SEL_CrcErr__NV33                          = 0x00000067,
    UMC_PERF_SEL_EccErr__NV33                          = 0x00000068,
    UMC_PERF_SEL_DroopCmdThrot__NV33                   = 0x00000069,
    UMC_PERF_SEL_DroopDetAtWin__NV33                   = 0x0000006a,
    UMC_PERF_SEL_DroopDetect__NV33                     = 0x0000006b,
} UMC_PERFCOUNT_SELECT;

constexpr unsigned int MaxUmcPerfcountSelectGfx101     = UMC_PERF_SEL_RefreshType__GFX101;
constexpr unsigned int MaxUmcPerfcountSelectNv2x       = UMC_PERF_SEL_TempCnt__NV2X;
constexpr unsigned int MaxUmcPerfcountSelectGfx11      = UMC_PERF_SEL_ClockCount__GFX11;

typedef enum UTCL1PerfSel {
    UTCL1_PERF_SEL_NONE                                = 0x00000000,
    UTCL1_PERF_SEL_REQS                                = 0x00000001,
    UTCL1_PERF_SEL_HITS                                = 0x00000002,
    UTCL1_PERF_SEL_MISSES                              = 0x00000003,
    UTCL1_PERF_SEL_BYPASS_REQS__GFX10                  = 0x00000004,
    UTCL1_PERF_SEL_HIT_INV_FILTER_REQS__GFX10          = 0x00000005,
    UTCL1_PERF_SEL_NUM_SMALLK_PAGES__GFX101            = 0x00000006,
    UTCL1_PERF_SEL_NUM_BIGK_PAGES__GFX101              = 0x00000007,
    UTCL1_PERF_SEL_TOTAL_UTCL2_REQS__GFX101            = 0x00000008,
    UTCL1_PERF_SEL_OUTSTANDING_UTCL2_REQS_ACCUM__GFX101 = 0x00000009,
    UTCL1_PERF_SEL_STALL_ON_UTCL2_CREDITS__GFX101      = 0x0000000a,
    UTCL1_PERF_SEL_STALL_MH_OFIFO_FULL__GFX101         = 0x0000000b,
    UTCL1_PERF_SEL_STALL_MH_CAM_FULL__GFX101           = 0x0000000c,
    UTCL1_PERF_SEL_NONRANGE_INV_REQS__GFX101           = 0x0000000d,
    UTCL1_PERF_SEL_RANGE_INV_REQS__GFX101              = 0x0000000e,
    UTCL1_PERF_SEL_SMALLK_PAGES__GFX10VRS              = 0x00000006,
    UTCL1_PERF_SEL_BIGK_PAGES__GFX10VRS                = 0x00000007,
    UTCL1_PERF_SEL_MH_RECENT_BUF_HIT__GFX10VRS         = 0x00000008,
    UTCL1_PERF_SEL_MH_DUPLICATE_DETECT__GFX10VRS       = 0x00000009,
    UTCL1_PERF_SEL_UTCL2_REQS__GFX10VRS                = 0x0000000a,
    UTCL1_PERF_SEL_UTCL2_RET_PERM_FAULT__GFX10VRS      = 0x0000000b,
    UTCL1_PERF_SEL_UTCL2_RET_PRT_FAULT__GFX10VRS       = 0x0000000c,
    UTCL1_PERF_SEL_UTCL2_RET_XNACK_RETRY__GFX10VRS     = 0x0000000d,
    UTCL1_PERF_SEL_UTCL2_RET_FAULT__GFX10VRS           = 0x0000000e,
    UTCL1_PERF_SEL_STALL_UTCL2_CREDITS__GFX10VRS       = 0x0000000f,
    UTCL1_PERF_SEL_STALL_MH_FULL__GFX10VRS             = 0x00000010,
    UTCL1_PERF_SEL_CP_INVREQS__GFX10VRS                = 0x00000011,
    UTCL1_PERF_SEL_UTCL2_UTCL1_INVREQS__GFX10VRS       = 0x00000012,
    UTCL1_PERF_SEL_RANGE_INVREQS__GFX10VRS             = 0x00000013,
    UTCL1_PERF_SEL_INV_ALL_VMID_INVREQS__GFX10VRS      = 0x00000014,
    UTCL1_PERF_SEL_MH_RECENT_BUF_HIT__GFX11            = 0x00000004,
    UTCL1_PERF_SEL_MH_DUPLICATE_DETECT__GFX11          = 0x00000005,
    UTCL1_PERF_SEL_UTCL2_REQS__GFX11                   = 0x00000006,
    UTCL1_PERF_SEL_UTCL2_RET_XNACK_RETRY__GFX11        = 0x00000007,
    UTCL1_PERF_SEL_UTCL2_RET_FAULT__GFX11              = 0x00000008,
    UTCL1_PERF_SEL_STALL_UTCL2_CREDITS__GFX11          = 0x00000009,
    UTCL1_PERF_SEL_STALL_MH_FULL__GFX11                = 0x0000000a,
    UTCL1_PERF_SEL_UTCL2_REQS_OUTSTANDING_ACCUM__GFX11 = 0x0000000b,
    UTCL1_PERF_SEL_UTCL2_RET_CNT__GFX11                = 0x0000000c,
    UTCL1_PERF_SEL_RTNS__GFX11                         = 0x0000000d,
    UTCL1_PERF_SEL_XLAT_REQ_BUSY__GFX11                = 0x0000000e,
    UTCL1_PERF_SEL_RANGE_INVREQS__GFX11                = 0x0000000f,
    UTCL1_PERF_SEL_INV_ALL_VMID_INVREQS__GFX11         = 0x00000010,
    UTCL1_PERF_SEL_BYPASS_REQS__GFX11                  = 0x00000011,
    UTCL1_PERF_SEL_HIT_INV_FILTER_REQS__GFX11          = 0x00000012,
    UTCL1_PERF_SEL_UTCL2_RET_PERM_FAULT__GFX11         = 0x00000013,
    UTCL1_PERF_SEL_UTCL2_RET_PRT_FAULT__GFX11          = 0x00000014,
    UTCL1_PERF_SEL_CP_INVREQS__GFX11                   = 0x00000015,
    UTCL1_PERF_SEL_UTCL2_UTCL1_INVREQS__GFX11          = 0x00000016,
    UTCL1_PERF_SEL_NUM_UTCL2_RTN_SIZE_4K_64K__GFX11    = 0x00000017,
    UTCL1_PERF_SEL_NUM_UTCL2_RTN_SIZE_64K_256K__GFX11  = 0x00000018,
    UTCL1_PERF_SEL_NUM_UTCL2_RTN_SIZE_256K_512K__GFX11 = 0x00000019,
    UTCL1_PERF_SEL_NUM_UTCL2_RTN_SIZE_512K_1M__GFX11   = 0x0000001a,
    UTCL1_PERF_SEL_NUM_UTCL2_RTN_SIZE_1M_2M__GFX11     = 0x0000001b,
    UTCL1_PERF_SEL_NUM_UTCL2_RTN_SIZE_2M_4M__GFX11     = 0x0000001c,
    UTCL1_PERF_SEL_NUM_UTCL2_RTN_SIZE_4M_8M__GFX11     = 0x0000001d,
    UTCL1_PERF_SEL_NUM_UTCL2_RTN_SIZE_8M_16M__GFX11    = 0x0000001e,
    UTCL1_PERF_SEL_NUM_UTCL2_RTN_SIZE_16M_32M__GFX11   = 0x0000001f,
    UTCL1_PERF_SEL_NUM_UTCL2_RTN_SIZE_32M_INF__GFX11   = 0x00000020,
    UTCL1_PERF_SEL_UTCL2_REQ_SQUASHED_NUM__GFX11       = 0x00000021,
    UTCL1_PERF_SEL_REQ_NUM_CACHE_CORE_0__GFX11         = 0x00000022,
    UTCL1_PERF_SEL_REQ_NUM_CACHE_CORE_1__GFX11         = 0x00000023,
    UTCL1_PERF_SEL_REQ_NUM_CACHE_CORE_2__GFX11         = 0x00000024,
    UTCL1_PERF_SEL_REQ_NUM_CACHE_CORE_3__GFX11         = 0x00000025,
    UTCL1_PERF_SEL_STALL_CYCLES_CACHE_CORE_0__GFX11    = 0x00000026,
    UTCL1_PERF_SEL_STALL_CYCLES_CACHE_CORE_1__GFX11    = 0x00000027,
    UTCL1_PERF_SEL_STALL_CYCLES_CACHE_CORE_2__GFX11    = 0x00000028,
    UTCL1_PERF_SEL_STALL_CYCLES_CACHE_CORE_3__GFX11    = 0x00000029,
    UTCL1_PERF_SEL_UTCL1_UTCL2_INVACKS__GFX11          = 0x0000002a,
    UTCL1_PERF_SEL_UTCL0_UTCL1_INVACKS__GFX11          = 0x0000002b,
    UTCL1_PERF_SEL_HITS_PG_SIZE_1__GFX11               = 0x0000002c,
    UTCL1_PERF_SEL_HITS_PG_SIZE_2__GFX11               = 0x0000002d,
    UTCL1_PERF_SEL_HITS_PG_SIZE_3__GFX11               = 0x0000002e,
    UTCL1_PERF_SEL_HITS_PG_SIZE_4__GFX11               = 0x0000002f,
    UTCL1_PERF_SEL_REQ_TO_MISS_HNDLR_0__GFX11          = 0x00000030,
    UTCL1_PERF_SEL_REQ_TO_MISS_HNDLR_1__GFX11          = 0x00000031,
    UTCL1_PERF_SEL_REQ_TO_MISS_HNDLR_2__GFX11          = 0x00000032,
    UTCL1_PERF_SEL_REQ_TO_MISS_HNDLR_3__GFX11          = 0x00000033,
    UTCL1_PERF_SEL_AVG_INV_LATENCY__GFX11              = 0x00000034,
    UTCL1_PERF_SEL_NUM_OF_CYCLES_RQ_EXISTS_TO_CC0__GFX11 = 0x00000035,
    UTCL1_PERF_SEL_NUM_OF_CYCLES_RQ_EXISTS_TO_CC1__GFX11 = 0x00000036,
    UTCL1_PERF_SEL_NUM_OF_CYCLES_RQ_EXISTS_TO_CC2__GFX11 = 0x00000037,
    UTCL1_PERF_SEL_NUM_OF_CYCLES_RQ_EXISTS_TO_CC3__GFX11 = 0x00000038,
    UTCL1_PERF_SEL_NUM_OF_CYCLES_W_COLLISION_CC0__GFX11 = 0x00000039,
    UTCL1_PERF_SEL_NUM_OF_CYCLES_W_COLLISION_CC1__GFX11 = 0x0000003a,
    UTCL1_PERF_SEL_NUM_OF_CYCLES_W_COLLISION_CC2__GFX11 = 0x0000003b,
    UTCL1_PERF_SEL_NUM_OF_CYCLES_W_COLLISION_CC3__GFX11 = 0x0000003c,
    UTCL1_PERF_SEL_ALOG_INTERRUPT__GFX11               = 0x0000003d,
    UTCL1_PERF_SEL_ALOG_INTERRUPT_DROPPED__GFX11       = 0x0000003e,
    UTCL1_PERF_SEL_ALOG_CACHE_REQ__GFX11               = 0x0000003f,
    UTCL1_PERF_SEL_ALOG_CACHE_HIT__GFX11               = 0x00000040,
    UTCL1_PERF_SEL_ALOG_STALL_PMM_CREDITS__GFX11       = 0x00000041,
    UTCL1_PERF_SEL_SMALL_PAGE_HITS__NV23               = 0x00000015,
    UTCL1_PERF_SEL_LARGE_PAGE_HITS__NV23               = 0x00000016,
    UTCL1_PERF_SEL_UTCL2_REQS_OUTSTANDING_ACCUM__NV23  = 0x00000017,
    UTCL1_PERF_SEL_SMALL_PAGE_HITS__NV24               = 0x00000015,
    UTCL1_PERF_SEL_LARGE_PAGE_HITS__NV24               = 0x00000016,
    UTCL1_PERF_SEL_UTCL2_REQS_OUTSTANDING_ACCUM__NV24  = 0x00000017,
    UTCL1_PERF_SEL_SMALL_PAGE_HITS__RAPHAEL            = 0x00000015,
    UTCL1_PERF_SEL_LARGE_PAGE_HITS__RAPHAEL            = 0x00000016,
    UTCL1_PERF_SEL_UTCL2_REQS_OUTSTANDING_ACCUM__RAPHAEL = 0x00000017,
    UTCL1_PERF_SEL_SMALL_PAGE_HITS__REMBRANDT          = 0x00000015,
    UTCL1_PERF_SEL_LARGE_PAGE_HITS__REMBRANDT          = 0x00000016,
    UTCL1_PERF_SEL_UTCL2_REQS_OUTSTANDING_ACCUM__REMBRANDT = 0x00000017,
} UTCL1PerfSel;

constexpr unsigned int MaxUTCL1PerfSelGfx101           = UTCL1_PERF_SEL_RANGE_INV_REQS__GFX101;
constexpr unsigned int MaxUTCL1PerfSelNv22             = UTCL1_PERF_SEL_INV_ALL_VMID_INVREQS__GFX10VRS;
constexpr unsigned int MaxUTCL1PerfSelNv21             = UTCL1_PERF_SEL_INV_ALL_VMID_INVREQS__GFX10VRS;
constexpr unsigned int MaxUTCL1PerfSelRaphael          = UTCL1_PERF_SEL_UTCL2_REQS_OUTSTANDING_ACCUM__RAPHAEL;
constexpr unsigned int MaxUTCL1PerfSelRembrandt        = UTCL1_PERF_SEL_UTCL2_REQS_OUTSTANDING_ACCUM__REMBRANDT;
constexpr unsigned int MaxUTCL1PerfSelNv24             = UTCL1_PERF_SEL_UTCL2_REQS_OUTSTANDING_ACCUM__NV24;
constexpr unsigned int MaxUTCL1PerfSelNv23             = UTCL1_PERF_SEL_UTCL2_REQS_OUTSTANDING_ACCUM__NV23;
constexpr unsigned int MaxUTCL1PerfSelGfx11            = UTCL1_PERF_SEL_ALOG_STALL_PMM_CREDITS__GFX11;

typedef enum VGT_CACHE_INVALID_MODE {
    VC_ONLY                                            = 0x00000000,
    TC_ONLY                                            = 0x00000001,
    VC_AND_TC                                          = 0x00000002,
} VGT_CACHE_INVALID_MODE;

typedef enum VGT_DETECT_ONE {
    ENABLE_TF1_OPT                                     = 0x00000000,
    DISABLE_TF1_OPT                                    = 0x00000001,
} VGT_DETECT_ONE;

typedef enum VGT_DETECT_ZERO {
    ENABLE_TF0_OPT                                     = 0x00000000,
    DISABLE_TF0_OPT                                    = 0x00000001,
} VGT_DETECT_ZERO;

typedef enum VGT_DIST_MODE {
    NO_DIST                                            = 0x00000000,
    PATCHES                                            = 0x00000001,
    DONUTS                                             = 0x00000002,
    TRAPEZOIDS                                         = 0x00000003,
} VGT_DIST_MODE;

typedef enum VGT_DI_INDEX_SIZE {
    DI_INDEX_SIZE_16_BIT                               = 0x00000000,
    DI_INDEX_SIZE_32_BIT                               = 0x00000001,
    DI_INDEX_SIZE_8_BIT                                = 0x00000002,
} VGT_DI_INDEX_SIZE;

typedef enum VGT_DI_MAJOR_MODE_SELECT {
    DI_MAJOR_MODE_0                                    = 0x00000000,
    DI_MAJOR_MODE_1                                    = 0x00000001,
} VGT_DI_MAJOR_MODE_SELECT;

typedef enum VGT_DI_PRIM_TYPE {
    DI_PT_NONE                                         = 0x00000000,
    DI_PT_POINTLIST                                    = 0x00000001,
    DI_PT_LINELIST                                     = 0x00000002,
    DI_PT_LINESTRIP                                    = 0x00000003,
    DI_PT_TRILIST                                      = 0x00000004,
    DI_PT_TRIFAN                                       = 0x00000005,
    DI_PT_TRISTRIP                                     = 0x00000006,
    DI_PT_2D_RECTANGLE                                 = 0x00000007,
    DI_PT_UNUSED_1                                     = 0x00000008,
    DI_PT_PATCH                                        = 0x00000009,
    DI_PT_LINELIST_ADJ                                 = 0x0000000a,
    DI_PT_LINESTRIP_ADJ                                = 0x0000000b,
    DI_PT_TRILIST_ADJ                                  = 0x0000000c,
    DI_PT_TRISTRIP_ADJ                                 = 0x0000000d,
    DI_PT_UNUSED_3                                     = 0x0000000e,
    DI_PT_UNUSED_4                                     = 0x0000000f,
    DI_PT_UNUSED_5                                     = 0x00000010,
    DI_PT_RECTLIST                                     = 0x00000011,
    DI_PT_LINELOOP                                     = 0x00000012,
    DI_PT_QUADLIST                                     = 0x00000013,
    DI_PT_QUADSTRIP                                    = 0x00000014,
    DI_PT_POLYGON                                      = 0x00000015,
} VGT_DI_PRIM_TYPE;

typedef enum VGT_DI_SOURCE_SELECT {
    DI_SRC_SEL_DMA                                     = 0x00000000,
    DI_SRC_SEL_IMMEDIATE                               = 0x00000001,
    DI_SRC_SEL_AUTO_INDEX                              = 0x00000002,
    DI_SRC_SEL_RESERVED                                = 0x00000003,
} VGT_DI_SOURCE_SELECT;

typedef enum VGT_DMA_BUF_TYPE {
    VGT_DMA_BUF_MEM                                    = 0x00000000,
    VGT_DMA_BUF_RING                                   = 0x00000001,
    VGT_DMA_BUF_SETUP                                  = 0x00000002,
    VGT_DMA_PTR_UPDATE                                 = 0x00000003,
} VGT_DMA_BUF_TYPE;

typedef enum VGT_DMA_SWAP_MODE {
    VGT_DMA_SWAP_NONE                                  = 0x00000000,
    VGT_DMA_SWAP_16_BIT                                = 0x00000001,
    VGT_DMA_SWAP_32_BIT                                = 0x00000002,
    VGT_DMA_SWAP_WORD                                  = 0x00000003,
} VGT_DMA_SWAP_MODE;

typedef enum VGT_EVENT_TYPE {
    Reserved_0x00                                      = 0x00000000,
    SAMPLE_STREAMOUTSTATS1                             = 0x00000001,
    SAMPLE_STREAMOUTSTATS2                             = 0x00000002,
    SAMPLE_STREAMOUTSTATS3                             = 0x00000003,
    CACHE_FLUSH_TS                                     = 0x00000004,
    CONTEXT_DONE                                       = 0x00000005,
    CACHE_FLUSH                                        = 0x00000006,
    CS_PARTIAL_FLUSH                                   = 0x00000007,
    VGT_STREAMOUT_SYNC                                 = 0x00000008,
    Reserved_0x09                                      = 0x00000009,
    VGT_STREAMOUT_RESET                                = 0x0000000a,
    END_OF_PIPE_INCR_DE                                = 0x0000000b,
    END_OF_PIPE_IB_END                                 = 0x0000000c,
    RST_PIX_CNT                                        = 0x0000000d,
    BREAK_BATCH                                        = 0x0000000e,
    VS_PARTIAL_FLUSH                                   = 0x0000000f,
    PS_PARTIAL_FLUSH                                   = 0x00000010,
    FLUSH_HS_OUTPUT                                    = 0x00000011,
    FLUSH_DFSM                                         = 0x00000012,
    RESET_TO_LOWEST_VGT                                = 0x00000013,
    CACHE_FLUSH_AND_INV_TS_EVENT                       = 0x00000014,
    CACHE_FLUSH_AND_INV_EVENT                          = 0x00000016,
    PERFCOUNTER_START                                  = 0x00000017,
    PERFCOUNTER_STOP                                   = 0x00000018,
    PIPELINESTAT_START                                 = 0x00000019,
    PIPELINESTAT_STOP                                  = 0x0000001a,
    PERFCOUNTER_SAMPLE                                 = 0x0000001b,
    FLUSH_ES_OUTPUT                                    = 0x0000001c,
    BIN_CONF_OVERRIDE_CHECK                            = 0x0000001d,
    SAMPLE_PIPELINESTAT                                = 0x0000001e,
    SO_VGTSTREAMOUT_FLUSH                              = 0x0000001f,
    SAMPLE_STREAMOUTSTATS                              = 0x00000020,
    RESET_VTX_CNT                                      = 0x00000021,
    BLOCK_CONTEXT_DONE                                 = 0x00000022,
    CS_CONTEXT_DONE                                    = 0x00000023,
    VGT_FLUSH                                          = 0x00000024,
    TGID_ROLLOVER                                      = 0x00000025,
    SQ_NON_EVENT                                       = 0x00000026,
    SC_SEND_DB_VPZ                                     = 0x00000027,
    BOTTOM_OF_PIPE_TS                                  = 0x00000028,
    FLUSH_SX_TS                                        = 0x00000029,
    DB_CACHE_FLUSH_AND_INV                             = 0x0000002a,
    FLUSH_AND_INV_DB_DATA_TS                           = 0x0000002b,
    FLUSH_AND_INV_DB_META                              = 0x0000002c,
    FLUSH_AND_INV_CB_DATA_TS                           = 0x0000002d,
    FLUSH_AND_INV_CB_META                              = 0x0000002e,
    CS_DONE                                            = 0x0000002f,
    PS_DONE                                            = 0x00000030,
    FLUSH_AND_INV_CB_PIXEL_DATA                        = 0x00000031,
    SX_CB_RAT_ACK_REQUEST                              = 0x00000032,
    THREAD_TRACE_START                                 = 0x00000033,
    THREAD_TRACE_STOP                                  = 0x00000034,
    THREAD_TRACE_MARKER                                = 0x00000035,
    THREAD_TRACE_DRAW                                  = 0x00000036,
    THREAD_TRACE_FINISH                                = 0x00000037,
    PIXEL_PIPE_STAT_CONTROL                            = 0x00000038,
    PIXEL_PIPE_STAT_DUMP                               = 0x00000039,
    PIXEL_PIPE_STAT_RESET                              = 0x0000003a,
    CONTEXT_SUSPEND                                    = 0x0000003b,
    OFFCHIP_HS_DEALLOC                                 = 0x0000003c,
    ENABLE_NGG_PIPELINE                                = 0x0000003d,
    DRAW_DONE                                          = 0x0000003f,
    ZPASS_DONE__GFX10                                  = 0x00000015,
    ENABLE_LEGACY_PIPELINE__GFX10                      = 0x0000003e,
    WAIT_SYNC__GFX11                                   = 0x00000015,
    ENABLE_PIPELINE_NOT_USED__GFX11                    = 0x0000003e,
} VGT_EVENT_TYPE;

typedef enum VGT_GROUP_CONV_SEL {
    VGT_GRP_INDEX_16                                   = 0x00000000,
    VGT_GRP_INDEX_32                                   = 0x00000001,
    VGT_GRP_UINT_16                                    = 0x00000002,
    VGT_GRP_UINT_32                                    = 0x00000003,
    VGT_GRP_SINT_16                                    = 0x00000004,
    VGT_GRP_SINT_32                                    = 0x00000005,
    VGT_GRP_FLOAT_32                                   = 0x00000006,
    VGT_GRP_AUTO_PRIM                                  = 0x00000007,
    VGT_GRP_FIX_1_23_TO_FLOAT                          = 0x00000008,
} VGT_GROUP_CONV_SEL;

typedef enum VGT_GRP_PRIM_ORDER {
    VGT_GRP_LIST                                       = 0x00000000,
    VGT_GRP_STRIP                                      = 0x00000001,
    VGT_GRP_FAN                                        = 0x00000002,
    VGT_GRP_LOOP                                       = 0x00000003,
    VGT_GRP_POLYGON                                    = 0x00000004,
} VGT_GRP_PRIM_ORDER;

typedef enum VGT_GRP_PRIM_TYPE {
    VGT_GRP_3D_POINT                                   = 0x00000000,
    VGT_GRP_3D_LINE                                    = 0x00000001,
    VGT_GRP_3D_TRI                                     = 0x00000002,
    VGT_GRP_3D_RECT                                    = 0x00000003,
    VGT_GRP_3D_QUAD                                    = 0x00000004,
    VGT_GRP_2D_COPY_RECT_V0                            = 0x00000005,
    VGT_GRP_2D_COPY_RECT_V1                            = 0x00000006,
    VGT_GRP_2D_COPY_RECT_V2                            = 0x00000007,
    VGT_GRP_2D_COPY_RECT_V3                            = 0x00000008,
    VGT_GRP_2D_FILL_RECT                               = 0x00000009,
    VGT_GRP_2D_LINE                                    = 0x0000000a,
    VGT_GRP_2D_TRI                                     = 0x0000000b,
    VGT_GRP_PRIM_INDEX_LINE                            = 0x0000000c,
    VGT_GRP_PRIM_INDEX_TRI                             = 0x0000000d,
    VGT_GRP_PRIM_INDEX_QUAD                            = 0x0000000e,
    VGT_GRP_3D_LINE_ADJ                                = 0x0000000f,
    VGT_GRP_3D_TRI_ADJ                                 = 0x00000010,
    VGT_GRP_3D_PATCH                                   = 0x00000011,
    VGT_GRP_2D_RECT                                    = 0x00000012,
} VGT_GRP_PRIM_TYPE;

typedef enum VGT_GS_CUT_MODE {
    GS_CUT_1024                                        = 0x00000000,
    GS_CUT_512                                         = 0x00000001,
    GS_CUT_256                                         = 0x00000002,
    GS_CUT_128                                         = 0x00000003,
} VGT_GS_CUT_MODE;

typedef enum VGT_GS_MODE_TYPE {
    GS_OFF                                             = 0x00000000,
    GS_SCENARIO_A                                      = 0x00000001,
    GS_SCENARIO_B                                      = 0x00000002,
    GS_SCENARIO_G                                      = 0x00000003,
    GS_SCENARIO_C                                      = 0x00000004,
    SPRITE_EN                                          = 0x00000005,
} VGT_GS_MODE_TYPE;

typedef enum VGT_GS_OUTPRIM_TYPE {
    POINTLIST                                          = 0x00000000,
    LINESTRIP                                          = 0x00000001,
    TRISTRIP                                           = 0x00000002,
    RECT_2D                                            = 0x00000003,
    RECTLIST                                           = 0x00000004,
} VGT_GS_OUTPRIM_TYPE;

typedef enum VGT_INDEX_TYPE_MODE {
    VGT_INDEX_16                                       = 0x00000000,
    VGT_INDEX_32                                       = 0x00000001,
    VGT_INDEX_8                                        = 0x00000002,
} VGT_INDEX_TYPE_MODE;

typedef enum VGT_OUTPATH_SELECT {
    VGT_OUTPATH_VTX_REUSE                              = 0x00000000,
    VGT_OUTPATH_GS_BLOCK                               = 0x00000001,
    VGT_OUTPATH_HS_BLOCK                               = 0x00000002,
    VGT_OUTPATH_PRIM_GEN                               = 0x00000003,
    VGT_OUTPATH_TE_PRIM_GEN                            = 0x00000004,
    VGT_OUTPATH_TE_GS_BLOCK                            = 0x00000005,
    VGT_OUTPATH_TE_OUTPUT                              = 0x00000006,
} VGT_OUTPATH_SELECT;

typedef enum VGT_OUT_PRIM_TYPE {
    VGT_OUT_POINT                                      = 0x00000000,
    VGT_OUT_LINE                                       = 0x00000001,
    VGT_OUT_TRI                                        = 0x00000002,
    VGT_OUT_RECT_V0__GFX10                             = 0x00000003,
    VGT_OUT_RECT_V1__GFX10                             = 0x00000004,
    VGT_OUT_RECT_V2__GFX10                             = 0x00000005,
    VGT_OUT_RECT_V3__GFX10                             = 0x00000006,
    VGT_OUT_2D_RECT__GFX10                             = 0x00000007,
    VGT_TE_QUAD__GFX10                                 = 0x00000008,
    VGT_TE_PRIM_INDEX_LINE__GFX10                      = 0x00000009,
    VGT_TE_PRIM_INDEX_TRI__GFX10                       = 0x0000000a,
    VGT_TE_PRIM_INDEX_QUAD__GFX10                      = 0x0000000b,
    VGT_OUT_LINE_ADJ__GFX10                            = 0x0000000c,
    VGT_OUT_TRI_ADJ__GFX10                             = 0x0000000d,
    VGT_OUT_PATCH__GFX10                               = 0x0000000e,
    VGT_OUT_2D_RECT__GFX11                             = 0x00000003,
    VGT_OUT_RECT_V0__GFX11                             = 0x00000004,
    VGT_OUT_DUMMY_1__GFX11                             = 0x00000005,
    VGT_OUT_DUMMY_2__GFX11                             = 0x00000006,
    VGT_OUT_DUMMY_3__GFX11                             = 0x00000007,
    VGT_OUT_PATCH__GFX11                               = 0x00000008,
    VGT_OUT_LINE_ADJ__GFX11                            = 0x00000009,
    VGT_OUT_TRI_ADJ__GFX11                             = 0x0000000a,
} VGT_OUT_PRIM_TYPE;

typedef enum VGT_RDREQ_POLICY {
    VGT_POLICY_LRU                                     = 0x00000000,
    VGT_POLICY_STREAM                                  = 0x00000001,
    VGT_POLICY_BYPASS                                  = 0x00000002,
} VGT_RDREQ_POLICY;

typedef enum VGT_STAGES_ES_EN {
    ES_STAGE_OFF                                       = 0x00000000,
    ES_STAGE_DS                                        = 0x00000001,
    ES_STAGE_REAL                                      = 0x00000002,
    RESERVED_ES                                        = 0x00000003,
} VGT_STAGES_ES_EN;

typedef enum VGT_STAGES_GS_EN {
    GS_STAGE_OFF                                       = 0x00000000,
    GS_STAGE_ON                                        = 0x00000001,
} VGT_STAGES_GS_EN;

typedef enum VGT_STAGES_HS_EN {
    HS_STAGE_OFF                                       = 0x00000000,
    HS_STAGE_ON                                        = 0x00000001,
} VGT_STAGES_HS_EN;

typedef enum VGT_STAGES_LS_EN {
    LS_STAGE_OFF                                       = 0x00000000,
    LS_STAGE_ON                                        = 0x00000001,
    CS_STAGE_ON                                        = 0x00000002,
    RESERVED_LS                                        = 0x00000003,
} VGT_STAGES_LS_EN;

typedef enum VGT_STAGES_VS_EN {
    VS_STAGE_REAL                                      = 0x00000000,
    VS_STAGE_DS                                        = 0x00000001,
    VS_STAGE_COPY_SHADER                               = 0x00000002,
    RESERVED_VS                                        = 0x00000003,
} VGT_STAGES_VS_EN;

typedef enum VGT_TESS_PARTITION {
    PART_INTEGER                                       = 0x00000000,
    PART_POW2                                          = 0x00000001,
    PART_FRAC_ODD                                      = 0x00000002,
    PART_FRAC_EVEN                                     = 0x00000003,
} VGT_TESS_PARTITION;

typedef enum VGT_TESS_TOPOLOGY {
    OUTPUT_POINT                                       = 0x00000000,
    OUTPUT_LINE                                        = 0x00000001,
    OUTPUT_TRIANGLE_CW                                 = 0x00000002,
    OUTPUT_TRIANGLE_CCW                                = 0x00000003,
} VGT_TESS_TOPOLOGY;

typedef enum VGT_TESS_TYPE {
    TESS_ISOLINE                                       = 0x00000000,
    TESS_TRIANGLE                                      = 0x00000001,
    TESS_QUAD                                          = 0x00000002,
} VGT_TESS_TYPE;

typedef enum VRSCombinerMode {
    VRS_COMB_MODE_PASSTHRU                             = 0x00000000,
    VRS_COMB_MODE_OVERRIDE                             = 0x00000001,
    VRS_COMB_MODE_MIN                                  = 0x00000002,
    VRS_COMB_MODE_MAX                                  = 0x00000003,
    VRS_COMB_MODE_SATURATE                             = 0x00000004,
} VRSCombinerMode;

typedef enum VRSCombinerModeSC {
    SC_VRS_COMB_MODE_PASSTHRU                          = 0x00000000,
    SC_VRS_COMB_MODE_OVERRIDE                          = 0x00000001,
    SC_VRS_COMB_MODE_MIN                               = 0x00000002,
    SC_VRS_COMB_MODE_MAX                               = 0x00000003,
    SC_VRS_COMB_MODE_SATURATE                          = 0x00000004,
} VRSCombinerModeSC;

typedef enum VRSHtileEncoding {
    VRS_HTILE_DISABLE                                  = 0x00000000,
    VRS_HTILE_2BIT_ENCODING                            = 0x00000001,
    VRS_HTILE_4BIT_ENCODING                            = 0x00000002,
} VRSHtileEncoding;

typedef enum VRSrate {
    VRS_SHADING_RATE_1X1                               = 0x00000000,
    VRS_SHADING_RATE_1X2                               = 0x00000001,
    VRS_SHADING_RATE_UNDEFINED0                        = 0x00000002,
    VRS_SHADING_RATE_UNDEFINED1                        = 0x00000003,
    VRS_SHADING_RATE_2X1                               = 0x00000004,
    VRS_SHADING_RATE_2X2                               = 0x00000005,
    VRS_SHADING_RATE_2X4                               = 0x00000006,
    VRS_SHADING_RATE_UNDEFINED2                        = 0x00000007,
    VRS_SHADING_RATE_UNDEFINED3                        = 0x00000008,
    VRS_SHADING_RATE_4X2                               = 0x00000009,
    VRS_SHADING_RATE_4X4                               = 0x0000000a,
    VRS_SHADING_RATE_UNDEFINED4                        = 0x0000000b,
    VRS_SHADING_RATE_16X_SSAA                          = 0x0000000c,
    VRS_SHADING_RATE_8X_SSAA                           = 0x0000000d,
    VRS_SHADING_RATE_4X_SSAA                           = 0x0000000e,
    VRS_SHADING_RATE_2X_SSAA                           = 0x0000000f,
} VRSrate;

typedef enum WD_IA_DRAW_REG_XFER {
    WD_IA_DRAW_REG_XFER_IA_MULTI_VGT_PARAM             = 0x00000000,
    WD_IA_DRAW_REG_XFER_VGT_MULTI_PRIM_IB_RESET_EN     = 0x00000001,
    WD_IA_DRAW_REG_XFER_VGT_INSTANCE_BASE_ID           = 0x00000002,
    WD_IA_DRAW_REG_XFER_GE_CNTL                        = 0x00000003,
    WD_IA_DRAW_REG_XFER_GE_USER_VGPR_EN                = 0x00000004,
    WD_IA_DRAW_REG_XFER_FL_MS_WG_DIM__GFX11            = 0x00000005,
    WD_IA_DRAW_REG_XFER_FL_MS_WG_DIM_1__GFX11          = 0x00000006,
    WD_IA_DRAW_REG_XFER_FL_MS_TG_SIZE__GFX11           = 0x00000007,
    WD_IA_DRAW_REG_XFER_FL_MS_EXP_ALLOC__GFX11         = 0x00000008,
    WD_IA_DRAW_REG_XFER_VGT_GS_OUT_PRIM_TYPE__GFX11    = 0x00000009,
} WD_IA_DRAW_REG_XFER;

typedef enum WD_IA_DRAW_SOURCE {
    WD_IA_DRAW_SOURCE_DMA                              = 0x00000000,
    WD_IA_DRAW_SOURCE_IMMD                             = 0x00000001,
    WD_IA_DRAW_SOURCE_AUTO                             = 0x00000002,
    WD_IA_DRAW_SOURCE_OPAQ                             = 0x00000003,
} WD_IA_DRAW_SOURCE;

typedef enum WD_IA_DRAW_TYPE {
    WD_IA_DRAW_TYPE_DI_MM0                             = 0x00000000,
    WD_IA_DRAW_TYPE_REG_XFER                           = 0x00000001,
    WD_IA_DRAW_TYPE_EVENT_INIT                         = 0x00000002,
    WD_IA_DRAW_TYPE_EVENT_ADDR                         = 0x00000003,
    WD_IA_DRAW_TYPE_MIN_INDX                           = 0x00000004,
    WD_IA_DRAW_TYPE_MAX_INDX                           = 0x00000005,
    WD_IA_DRAW_TYPE_INDX_OFF                           = 0x00000006,
    WD_IA_DRAW_TYPE_IMM_DATA                           = 0x00000007,
} WD_IA_DRAW_TYPE;

typedef enum WritePolicy {
    CACHE_LRU_WR                                       = 0x00000000,
    CACHE_STREAM                                       = 0x00000001,
    CACHE_NOA_WR                                       = 0x00000002,
    CACHE_BYPASS                                       = 0x00000003,
} WritePolicy;

typedef enum ZFormat {
    Z_INVALID                                          = 0x00000000,
    Z_16                                               = 0x00000001,
    Z_24                                               = 0x00000002,
    Z_32_FLOAT                                         = 0x00000003,
} ZFormat;

typedef enum ZLimitSumm {
    FORCE_SUMM_OFF                                     = 0x00000000,
    FORCE_SUMM_MINZ                                    = 0x00000001,
    FORCE_SUMM_MAXZ                                    = 0x00000002,
    FORCE_SUMM_BOTH                                    = 0x00000003,
} ZLimitSumm;

typedef enum ZModeForce {
    NO_FORCE                                           = 0x00000000,
    FORCE_EARLY_Z                                      = 0x00000001,
    FORCE_LATE_Z                                       = 0x00000002,
    FORCE_RE_Z                                         = 0x00000003,
} ZModeForce;

typedef enum ZOrder {
    LATE_Z                                             = 0x00000000,
    EARLY_Z_THEN_LATE_Z                                = 0x00000001,
    RE_Z                                               = 0x00000002,
    EARLY_Z_THEN_RE_Z                                  = 0x00000003,
} ZOrder;

typedef enum ZpassControl {
    ZPASS_DISABLE                                      = 0x00000000,
    ZPASS_SAMPLES                                      = 0x00000001,
    ZPASS_PIXELS                                       = 0x00000002,
} ZpassControl;

typedef enum ZSamplePosition {
    Z_SAMPLE_CENTER                                    = 0x00000000,
    Z_SAMPLE_CENTROID                                  = 0x00000001,
} ZSamplePosition;

constexpr unsigned int CONFIG_SPACE1_END                                  = 0x00002bff;
constexpr unsigned int CONFIG_SPACE1_START                                = 0x00002000;
constexpr unsigned int CONFIG_SPACE2_END                                  = 0x00009fff;
constexpr unsigned int CONFIG_SPACE2_START                                = 0x00003000;
constexpr unsigned int CONFIG_SPACE_END                                   = 0x00009fff;
constexpr unsigned int CONFIG_SPACE_START                                 = 0x00002000;
constexpr unsigned int CONTEXT_SPACE_START                                = 0x0000a000;
constexpr unsigned int CSCNTL_ADDR_WIDTH                                  = 0x00000007;
constexpr unsigned int CSCNTL_DATA_WIDTH                                  = 0x00000020;
constexpr unsigned int CSCNTL_TYPE_WIDTH                                  = 0x00000002;
constexpr unsigned int CSDATA_ADDR_WIDTH                                  = 0x00000007;
constexpr unsigned int CSDATA_DATA_WIDTH                                  = 0x00000020;
constexpr unsigned int CSDATA_TYPE_WIDTH                                  = 0x00000002;
constexpr unsigned int GB_TILING_CONFIG_MACROTABLE_SIZE                   = 0x00000010;
constexpr unsigned int GB_TILING_CONFIG_TABLE_SIZE                        = 0x00000020;
constexpr unsigned int GSTHREADID_SIZE                                    = 0x00000002;
constexpr unsigned int INST_ID_ECC_INTERRUPT_MSG                          = 0xfffffff0;
constexpr unsigned int INST_ID_HOST_REG_TRAP_MSG                          = 0xfffffffe;
constexpr unsigned int INST_ID_HW_TRAP                                    = 0xfffffff2;
constexpr unsigned int INST_ID_KILL_SEQ                                   = 0xfffffff3;
constexpr unsigned int INST_ID_PRIV_START                                 = 0x80000000;
constexpr unsigned int INST_ID_SPI_WREXEC                                 = 0xfffffff4;
constexpr unsigned int INST_ID_TTRACE_NEW_PC_MSG                          = 0xfffffff1;
constexpr unsigned int IQ_DEQUEUE_RETRY                                   = 0x00000004;
constexpr unsigned int IQ_INTR_TYPE_IB                                    = 0x00000001;
constexpr unsigned int IQ_INTR_TYPE_MQD                                   = 0x00000002;
constexpr unsigned int IQ_INTR_TYPE_PQ                                    = 0x00000000;
constexpr unsigned int IQ_OFFLOAD_RETRY                                   = 0x00000001;
constexpr unsigned int IQ_QUEUE_SLEEP                                     = 0x00000000;
constexpr unsigned int IQ_SCH_WAVE_MSG                                    = 0x00000002;
constexpr unsigned int IQ_SEM_REARM                                       = 0x00000003;
constexpr unsigned int PERSISTENT_SPACE_END                               = 0x00002fff;
constexpr unsigned int PERSISTENT_SPACE_START                             = 0x00002c00;
constexpr unsigned int SEM_ECC_ERROR                                      = 0x00000000;
constexpr unsigned int SEM_RESP_FAILED                                    = 0x00000002;
constexpr unsigned int SEM_RESP_PASSED                                    = 0x00000003;
constexpr unsigned int SEM_TRANS_ERROR                                    = 0x00000001;
constexpr unsigned int SIMM16_WAITCNT_DEPCTR_SA_SDST_SIZE                 = 0x00000001;
constexpr unsigned int SIMM16_WAITCNT_DEPCTR_SA_SDST_START                = 0x00000000;
constexpr unsigned int SIMM16_WAITCNT_DEPCTR_VA_SDST_SIZE                 = 0x00000003;
constexpr unsigned int SIMM16_WAITCNT_DEPCTR_VA_SDST_START                = 0x00000009;
constexpr unsigned int SIMM16_WAITCNT_DEPCTR_VA_SSRC_SIZE                 = 0x00000001;
constexpr unsigned int SIMM16_WAITCNT_DEPCTR_VA_SSRC_START                = 0x00000008;
constexpr unsigned int SIMM16_WAITCNT_DEPCTR_VA_VCC_SIZE                  = 0x00000001;
constexpr unsigned int SIMM16_WAITCNT_DEPCTR_VA_VCC_START                 = 0x00000001;
constexpr unsigned int SIMM16_WAITCNT_DEPCTR_VA_VDST_SIZE                 = 0x00000004;
constexpr unsigned int SIMM16_WAITCNT_DEPCTR_VA_VDST_START                = 0x0000000c;
constexpr unsigned int SIMM16_WAITCNT_DEPCTR_VM_VSRC_SIZE                 = 0x00000003;
constexpr unsigned int SIMM16_WAITCNT_DEPCTR_VM_VSRC_START                = 0x00000002;
constexpr unsigned int SIMM16_WAITCNT_EXP_CNT_SIZE                        = 0x00000003;
constexpr unsigned int SIMM16_WAITCNT_LGKM_CNT_SIZE                       = 0x00000006;
constexpr unsigned int SQDEC_BEGIN                                        = 0x00002300;
constexpr unsigned int SQDEC_END                                          = 0x000023ff;
constexpr unsigned int SQGFXUDEC_BEGIN                                    = 0x0000c330;
constexpr unsigned int SQGFXUDEC_END                                      = 0x0000c380;
constexpr unsigned int SQIND_GLOBAL_REGS_OFFSET                           = 0x00000000;
constexpr unsigned int SQIND_GLOBAL_REGS_SIZE                             = 0x00000008;
constexpr unsigned int SQIND_LOCAL_REGS_OFFSET                            = 0x00000008;
constexpr unsigned int SQIND_LOCAL_REGS_SIZE                              = 0x00000008;
constexpr unsigned int SQIND_WAVE_HWREGS_OFFSET                           = 0x00000100;
constexpr unsigned int SQIND_WAVE_HWREGS_SIZE                             = 0x00000100;
constexpr unsigned int SQIND_WAVE_SGPRS_OFFSET                            = 0x00000200;
constexpr unsigned int SQIND_WAVE_SGPRS_SIZE                              = 0x00000200;
constexpr unsigned int SQIND_WAVE_VGPRS_OFFSET                            = 0x00000400;
constexpr unsigned int SQIND_WAVE_VGPRS_SIZE                              = 0x00000400;
constexpr unsigned int SQPERFDDEC_BEGIN                                   = 0x0000d1c0;
constexpr unsigned int SQPERFDDEC_END                                     = 0x0000d240;
constexpr unsigned int SQPERFSDEC_BEGIN                                   = 0x0000d9c0;
constexpr unsigned int SQPERFSDEC_END                                     = 0x0000da40;
constexpr unsigned int SQPWRDEC_BEGIN                                     = 0x0000f08c;
constexpr unsigned int SQPWRDEC_END                                       = 0x0000f094;
constexpr unsigned int SQ_DISPATCHER_GFX_CNT_PER_RING                     = 0x00000008;
constexpr unsigned int SQ_DISPATCHER_GFX_MIN                              = 0x00000010;
constexpr unsigned int SQ_EDC_FUE_CNTL_LDS                                = 0x00000005;
constexpr unsigned int SQ_EDC_FUE_CNTL_SIMD0                              = 0x00000000;
constexpr unsigned int SQ_EDC_FUE_CNTL_SIMD1                              = 0x00000001;
constexpr unsigned int SQ_EDC_FUE_CNTL_SIMD2                              = 0x00000002;
constexpr unsigned int SQ_EDC_FUE_CNTL_SIMD3                              = 0x00000003;
constexpr unsigned int SQ_EDC_FUE_CNTL_SQ                                 = 0x00000004;
constexpr unsigned int SQ_EDC_FUE_CNTL_TA                                 = 0x00000007;
constexpr unsigned int SQ_EDC_FUE_CNTL_TCP                                = 0x00000008;
constexpr unsigned int SQ_EDC_FUE_CNTL_TD                                 = 0x00000006;
constexpr unsigned int SQ_EX_MODE_EXCP_ADDR_WATCH0                        = 0x00000007;
constexpr unsigned int SQ_EX_MODE_EXCP_DIV0                               = 0x00000002;
constexpr unsigned int SQ_EX_MODE_EXCP_HI_ADDR_WATCH1                     = 0x00000000;
constexpr unsigned int SQ_EX_MODE_EXCP_HI_ADDR_WATCH2                     = 0x00000001;
constexpr unsigned int SQ_EX_MODE_EXCP_HI_ADDR_WATCH3                     = 0x00000002;
constexpr unsigned int SQ_EX_MODE_EXCP_INEXACT                            = 0x00000005;
constexpr unsigned int SQ_EX_MODE_EXCP_INPUT_DENORM                       = 0x00000001;
constexpr unsigned int SQ_EX_MODE_EXCP_INT_DIV0                           = 0x00000006;
constexpr unsigned int SQ_EX_MODE_EXCP_INVALID                            = 0x00000000;
constexpr unsigned int SQ_EX_MODE_EXCP_MEM_VIOL                           = 0x00000008;
constexpr unsigned int SQ_EX_MODE_EXCP_OVERFLOW                           = 0x00000003;
constexpr unsigned int SQ_EX_MODE_EXCP_UNDERFLOW                          = 0x00000004;
constexpr unsigned int SQ_EX_MODE_EXCP_VALU_BASE                          = 0x00000000;
constexpr unsigned int SQ_EX_MODE_EXCP_VALU_SIZE                          = 0x00000007;
constexpr unsigned int SQ_GFXDEC_BEGIN                                    = 0x0000a000;
constexpr unsigned int SQ_GFXDEC_END                                      = 0x0000c000;
constexpr unsigned int SQ_GFXDEC_STATE_ID_SHIFT                           = 0x0000000a;
constexpr unsigned int SQ_MAX_PGM_SGPRS                                   = 0x00000068;
constexpr unsigned int SQ_MAX_PGM_VGPRS                                   = 0x00000100;
constexpr unsigned int SQ_WAVE_IB_DEP_LDS_DIR_SIZE                        = 0x00000003;
constexpr unsigned int SQ_WAVE_IB_DEP_SA_EXEC_SIZE                        = 0x00000002;
constexpr unsigned int SQ_WAVE_IB_DEP_SA_M0_SIZE                          = 0x00000001;
constexpr unsigned int SQ_WAVE_IB_DEP_SA_SDST_SIZE                        = 0x00000004;
constexpr unsigned int SQ_WAVE_IB_DEP_VA_EXEC_SIZE                        = 0x00000002;
constexpr unsigned int SQ_WAVE_IB_DEP_VA_SDST_SIZE                        = 0x00000004;
constexpr unsigned int SQ_WAVE_IB_DEP_VA_SSRC_SIZE                        = 0x00000003;
constexpr unsigned int SQ_WAVE_IB_DEP_VA_VCC_SIZE                         = 0x00000003;
constexpr unsigned int SQ_WAVE_IB_DEP_VM_VSRC_SIZE                        = 0x00000004;
constexpr unsigned int SQ_WAVE_TYPE_PS0                                   = 0x00000000;
constexpr unsigned int SRCID_NONSECURE_CP                                 = 0x00000001;
constexpr unsigned int SRCID_NONSECURE_CP_RCIU                            = 0x00000001;
constexpr unsigned int SRCID_RLC                                          = 0x00000000;
constexpr unsigned int SRCID_RLCV                                         = 0x00000006;
constexpr unsigned int SRCID_SECURE_CP                                    = 0x00000007;
constexpr unsigned int SRCID_SECURE_CP_RCIU                               = 0x00000007;
constexpr unsigned int UCONFIG_SPACE_END                                  = 0x0000ffff;
constexpr unsigned int UCONFIG_SPACE_START                                = 0x0000c000;
constexpr unsigned int VMID_SZ                                            = 0x00000004;

namespace Gfx10
{
    constexpr unsigned int CONTEXT_SPACE_END                                  = 0x0000bfff;
    constexpr unsigned int SIMM16_WAITCNT_EXP_CNT_START                       = 0x00000004;
    constexpr unsigned int SIMM16_WAITCNT_LGKM_CNT_START                      = 0x00000008;
    constexpr unsigned int SIMM16_WAITCNT_VM_CNT_HI_SIZE                      = 0x00000002;
    constexpr unsigned int SIMM16_WAITCNT_VM_CNT_HI_START                     = 0x0000000e;
    constexpr unsigned int SIMM16_WAITCNT_VM_CNT_SIZE                         = 0x00000004;
    constexpr unsigned int SIMM16_WAITCNT_VM_CNT_START                        = 0x00000000;
    constexpr unsigned int SQ_WAVE_IB_DEP_VA_VDST_SIZE                        = 0x00000004;
} // namespace Gfx10

namespace Gfx101
{
    constexpr unsigned int GB_TILE_MODE0_GEN0                                 = 0x90800310;
    constexpr unsigned int GB_TILE_MODE0_GEN1                                 = 0x90800310;
    constexpr unsigned int GB_TILE_MODE1_GEN0                                 = 0x90800b10;
    constexpr unsigned int GB_TILE_MODE1_GEN1                                 = 0x90800b10;
    constexpr unsigned int GB_TILE_MODE2_GEN0                                 = 0x90801310;
    constexpr unsigned int GB_TILE_MODE2_GEN1                                 = 0x90801310;
    constexpr unsigned int GB_TILE_MODE3_GEN0                                 = 0x90801b10;
    constexpr unsigned int GB_TILE_MODE3_GEN1                                 = 0x90801b10;
    constexpr unsigned int GB_TILE_MODE4_GEN0                                 = 0x90802310;
    constexpr unsigned int GB_TILE_MODE4_GEN1                                 = 0x90802310;
    constexpr unsigned int GB_TILE_MODE5_GEN0                                 = 0x90800308;
    constexpr unsigned int GB_TILE_MODE5_GEN1                                 = 0x90800308;
    constexpr unsigned int GB_TILE_MODE6_GEN0                                 = 0x90801318;
    constexpr unsigned int GB_TILE_MODE6_GEN1                                 = 0x90801318;
    constexpr unsigned int GB_TILE_MODE7_GEN0                                 = 0x90802318;
    constexpr unsigned int GB_TILE_MODE7_GEN1                                 = 0x90802318;
    constexpr unsigned int GB_TILE_MODE8_GEN0                                 = 0x90000304;
    constexpr unsigned int GB_TILE_MODE8_GEN1                                 = 0x90000304;
    constexpr unsigned int GB_TILE_MODE9_GEN0                                 = 0x90000308;
    constexpr unsigned int GB_TILE_MODE9_GEN1                                 = 0x90000308;
    constexpr unsigned int GB_TILE_MODE10_GEN0                                = 0x92000310;
    constexpr unsigned int GB_TILE_MODE10_GEN1                                = 0x92000310;
    constexpr unsigned int GB_TILE_MODE11_GEN0                                = 0x92000294;
    constexpr unsigned int GB_TILE_MODE11_GEN1                                = 0x92000294;
    constexpr unsigned int GB_TILE_MODE12_GEN0                                = 0x92000318;
    constexpr unsigned int GB_TILE_MODE12_GEN1                                = 0x92000318;
    constexpr unsigned int GB_TILE_MODE13_GEN0                                = 0x90400308;
    constexpr unsigned int GB_TILE_MODE13_GEN1                                = 0x90400308;
    constexpr unsigned int GB_TILE_MODE14_GEN0                                = 0x92400310;
    constexpr unsigned int GB_TILE_MODE14_GEN1                                = 0x92400310;
    constexpr unsigned int GB_TILE_MODE15_GEN0                                = 0x924002b0;
    constexpr unsigned int GB_TILE_MODE15_GEN1                                = 0x924002b0;
    constexpr unsigned int GB_TILE_MODE16_GEN0                                = 0x92400294;
    constexpr unsigned int GB_TILE_MODE16_GEN1                                = 0x92400294;
    constexpr unsigned int GB_TILE_MODE17_GEN0                                = 0x92400318;
    constexpr unsigned int GB_TILE_MODE17_GEN1                                = 0x92400318;
    constexpr unsigned int GB_TILE_MODE18_GEN0                                = 0x9240032c;
    constexpr unsigned int GB_TILE_MODE18_GEN1                                = 0x9240032c;
    constexpr unsigned int GB_TILE_MODE19_GEN0                                = 0x9100030c;
    constexpr unsigned int GB_TILE_MODE19_GEN1                                = 0x9100030c;
    constexpr unsigned int GB_TILE_MODE20_GEN0                                = 0x9100031c;
    constexpr unsigned int GB_TILE_MODE20_GEN1                                = 0x9100031c;
    constexpr unsigned int GB_TILE_MODE21_GEN0                                = 0x910002b4;
    constexpr unsigned int GB_TILE_MODE21_GEN1                                = 0x910002b4;
    constexpr unsigned int GB_TILE_MODE22_GEN0                                = 0x910002a4;
    constexpr unsigned int GB_TILE_MODE22_GEN1                                = 0x910002a4;
    constexpr unsigned int GB_TILE_MODE23_GEN0                                = 0x91000328;
    constexpr unsigned int GB_TILE_MODE23_GEN1                                = 0x91000328;
    constexpr unsigned int GB_TILE_MODE24_GEN0                                = 0x910002bc;
    constexpr unsigned int GB_TILE_MODE24_GEN1                                = 0x910002bc;
    constexpr unsigned int GB_TILE_MODE25_GEN0                                = 0x91000320;
    constexpr unsigned int GB_TILE_MODE25_GEN1                                = 0x91000320;
    constexpr unsigned int GB_TILE_MODE26_GEN0                                = 0x910002b8;
    constexpr unsigned int GB_TILE_MODE26_GEN1                                = 0x910002b8;
    constexpr unsigned int GB_TILE_MODE27_GEN0                                = 0x90c00308;
    constexpr unsigned int GB_TILE_MODE27_GEN1                                = 0x90c00308;
    constexpr unsigned int GB_TILE_MODE28_GEN0                                = 0x92c00310;
    constexpr unsigned int GB_TILE_MODE28_GEN1                                = 0x92c00310;
    constexpr unsigned int GB_TILE_MODE29_GEN0                                = 0x92c00294;
    constexpr unsigned int GB_TILE_MODE29_GEN1                                = 0x92c00294;
    constexpr unsigned int GB_TILE_MODE30_GEN0                                = 0x92c00318;
    constexpr unsigned int GB_TILE_MODE30_GEN1                                = 0x92c00318;
    constexpr unsigned int GB_TILE_MODE31_GEN0                                = 0x00000000;
    constexpr unsigned int GB_TILE_MODE31_GEN1                                = 0x00000000;
    constexpr unsigned int INST_ID_XNACK_OVERRIDE                             = 0xfffffff5;
    constexpr unsigned int IP_USB_PD_REVISION_ID                              = 0x00000000;
    constexpr unsigned int ROM_SIGNATURE                                      = 0x0000aa55;
} // namespace Gfx101

namespace Gfx103Plus
{
    constexpr unsigned int SIMM16_WAITCNT_DEPCTR_HOLD_CNT_SIZE                = 0x00000001;
    constexpr unsigned int SIMM16_WAITCNT_DEPCTR_HOLD_CNT_START               = 0x00000007;
    constexpr unsigned int SQ_WAVE_IB_DEP_HOLD_CNT_SIZE                       = 0x00000001;
} // namespace Gfx103Plus

namespace Gfx11
{
    constexpr unsigned int CONTEXT_SPACE_END                                  = 0x0000a3ff;
    constexpr unsigned int INST_ID_HW_TRAP_GET_TBA                            = 0xfffffff5;
    constexpr unsigned int PFVF_SQDEC_BEGIN                                   = 0x0000a9e0;
    constexpr unsigned int PFVF_SQDEC_END                                     = 0x0000a9ff;
    constexpr unsigned int SIMM16_WAITCNT_EXP_CNT_START                       = 0x00000000;
    constexpr unsigned int SIMM16_WAITCNT_LGKM_CNT_START                      = 0x00000004;
    constexpr unsigned int SIMM16_WAITCNT_VM_CNT_SIZE                         = 0x00000006;
    constexpr unsigned int SIMM16_WAITCNT_VM_CNT_START                        = 0x0000000a;
    constexpr unsigned int SIMM16_WAIT_EVENT_EXP_RDY_SIZE                     = 0x00000001;
    constexpr unsigned int SIMM16_WAIT_EVENT_EXP_RDY_START                    = 0x00000000;
    constexpr unsigned int SQ_WAVE_IB_DEP_VA_VDST_SIZE                        = 0x00000005;
} // namespace Gfx11

namespace Nv21
{
    constexpr unsigned int IP_I2C_M_REVISION_ID                               = 0x00000000;
    constexpr unsigned int IP_USB_PD_REVISION_ID                              = 0x00000000;
} // namespace Nv21

namespace Nv22
{
    constexpr unsigned int IP_I2C_M_REVISION_ID                               = 0x00000000;
    constexpr unsigned int IP_USB_PD_REVISION_ID                              = 0x00000000;
} // namespace Nv22

namespace Nv23
{
    constexpr unsigned int IP_I2C_M_REVISION_ID                               = 0x00000000;
    constexpr unsigned int IP_USB_PD_REVISION_ID                              = 0x00000000;
} // namespace Nv23

namespace Nv2x
{
    constexpr unsigned int ROM_SIGNATURE                                      = 0x0000aa55;
} // namespace Nv2x

namespace Nv31
{
    constexpr unsigned int IP_I2C_M_REVISION_ID                               = 0x00000000;
    constexpr unsigned int IP_USB_PD_REVISION_ID                              = 0x00000000;
} // namespace Nv31

namespace Nv32
{
    constexpr unsigned int IP_I2C_M_REVISION_ID                               = 0x00000000;
    constexpr unsigned int IP_USB_PD_REVISION_ID                              = 0x00000000;
} // namespace Nv32

namespace Nv3x
{
    constexpr unsigned int ROM_SIGNATURE                                      = 0x0000aa55;
} // namespace Nv3x

namespace Phx1
{
    constexpr unsigned int AL_REVISION_ID                                     = 0x00000012;
    constexpr unsigned int EMMC_CFG_REVISION_ID                               = 0x00000000;
    constexpr unsigned int EMMC_HC_REG_REVISION_ID                            = 0x00000000;
    constexpr unsigned int ENHIOMEMAPERTURE_REVISION_ID                       = 0x00000000;
    constexpr unsigned int ILA_REG_REVISION_ID                                = 0x00000000;
    constexpr unsigned int IP_AL2AHB_DEVICE_CFG_REVISION_ID                   = 0x00000000;
    constexpr unsigned int IP_AL2AHB_HARD_ADDR_HCLK_REVISION_ID               = 0x00000000;
    constexpr unsigned int IP_AL2AHB_HARD_ADDR_REVISION_ID                    = 0x00000000;
    constexpr unsigned int IP_AL2AHB_P2P_CFG_REVISION_ID                      = 0x00000000;
    constexpr unsigned int IP_ESPI_REG_REVISION_ID                            = 0x00000000;
    constexpr unsigned int IP_SDP_REG_REVISION_ID                             = 0x00000000;
    constexpr unsigned int IP_USB_PD_REVISION_ID                              = 0x00000000;
    constexpr unsigned int MMREG_REVISION_ID                                  = 0x00000000;
    constexpr unsigned int NUM_REQUESTORS                                     = 0x00000005;
    constexpr unsigned int REQID_ECHI2                                        = 0x00000004;
    constexpr unsigned int REQID_EHCI                                         = 0x00000001;
    constexpr unsigned int REQID_OCHI2                                        = 0x00000003;
    constexpr unsigned int REQID_OHCI                                         = 0x00000000;
    constexpr unsigned int REQID_SATA0                                        = 0x00000002;
    constexpr unsigned int SDB_REG_REVISION_ID                                = 0x00000000;
    constexpr unsigned int SD_PCI_CFG_REVISION_ID                             = 0x00000000;
} // namespace Phx1

namespace Phx2
{
    constexpr unsigned int AL_REVISION_ID                                     = 0x00000012;
    constexpr unsigned int EMMC_CFG_REVISION_ID                               = 0x00000000;
    constexpr unsigned int EMMC_HC_REG_REVISION_ID                            = 0x00000000;
    constexpr unsigned int ENHIOMEMAPERTURE_REVISION_ID                       = 0x00000000;
    constexpr unsigned int ILA_REG_REVISION_ID                                = 0x00000000;
    constexpr unsigned int IP_AL2AHB_DEVICE_CFG_REVISION_ID                   = 0x00000000;
    constexpr unsigned int IP_AL2AHB_HARD_ADDR_HCLK_REVISION_ID               = 0x00000000;
    constexpr unsigned int IP_AL2AHB_HARD_ADDR_REVISION_ID                    = 0x00000000;
    constexpr unsigned int IP_AL2AHB_P2P_CFG_REVISION_ID                      = 0x00000000;
    constexpr unsigned int IP_ESPI_REG_REVISION_ID                            = 0x00000000;
    constexpr unsigned int IP_SDP_REG_REVISION_ID                             = 0x00000000;
    constexpr unsigned int IP_USB_PD_REVISION_ID                              = 0x00000000;
    constexpr unsigned int MMREG_REVISION_ID                                  = 0x00000000;
    constexpr unsigned int NUM_REQUESTORS                                     = 0x00000005;
    constexpr unsigned int REQID_ECHI2                                        = 0x00000004;
    constexpr unsigned int REQID_EHCI                                         = 0x00000001;
    constexpr unsigned int REQID_OCHI2                                        = 0x00000003;
    constexpr unsigned int REQID_OHCI                                         = 0x00000000;
    constexpr unsigned int REQID_SATA0                                        = 0x00000002;
    constexpr unsigned int SDB_REG_REVISION_ID                                = 0x00000000;
    constexpr unsigned int SD_PCI_CFG_REVISION_ID                             = 0x00000000;
} // namespace Phx2

namespace Raphael
{
    constexpr unsigned int AL_REVISION_ID                                     = 0x00000012;
    constexpr unsigned int EMMC_CFG_REVISION_ID                               = 0x00000000;
    constexpr unsigned int EMMC_HC_REG_REVISION_ID                            = 0x00000000;
    constexpr unsigned int ENHIOMEMAPERTURE_REVISION_ID                       = 0x00000000;
    constexpr unsigned int ILA_REG_REVISION_ID                                = 0x00000000;
    constexpr unsigned int IP_AL2AHB_DEVICE_CFG_REVISION_ID                   = 0x00000000;
    constexpr unsigned int IP_AL2AHB_HARD_ADDR_HCLK_REVISION_ID               = 0x00000000;
    constexpr unsigned int IP_AL2AHB_HARD_ADDR_REVISION_ID                    = 0x00000000;
    constexpr unsigned int IP_AL2AHB_P2P_CFG_REVISION_ID                      = 0x00000000;
    constexpr unsigned int IP_ESPI_REG_REVISION_ID                            = 0x00000000;
    constexpr unsigned int IP_SDP_REG_REVISION_ID                             = 0x00000000;
    constexpr unsigned int IP_USB_PD_REVISION_ID                              = 0x00000000;
    constexpr unsigned int MMREG_REVISION_ID                                  = 0x00000000;
    constexpr unsigned int NUM_REQUESTORS                                     = 0x00000005;
    constexpr unsigned int REQID_ECHI2                                        = 0x00000004;
    constexpr unsigned int REQID_EHCI                                         = 0x00000001;
    constexpr unsigned int REQID_OCHI2                                        = 0x00000003;
    constexpr unsigned int REQID_OHCI                                         = 0x00000000;
    constexpr unsigned int REQID_SATA0                                        = 0x00000002;
    constexpr unsigned int SDB_REG_REVISION_ID                                = 0x00000000;
    constexpr unsigned int SD_PCI_CFG_REVISION_ID                             = 0x00000000;
} // namespace Raphael

namespace Rembrandt
{
    constexpr unsigned int AL_REVISION_ID                                     = 0x00000012;
    constexpr unsigned int EMMC_CFG_REVISION_ID                               = 0x00000000;
    constexpr unsigned int EMMC_HC_REG_REVISION_ID                            = 0x00000000;
    constexpr unsigned int ENHIOMEMAPERTURE_REVISION_ID                       = 0x00000000;
    constexpr unsigned int ILA_REG_REVISION_ID                                = 0x00000000;
    constexpr unsigned int IP_AL2AHB_DEVICE_CFG_REVISION_ID                   = 0x00000000;
    constexpr unsigned int IP_AL2AHB_HARD_ADDR_HCLK_REVISION_ID               = 0x00000000;
    constexpr unsigned int IP_AL2AHB_HARD_ADDR_REVISION_ID                    = 0x00000000;
    constexpr unsigned int IP_AL2AHB_P2P_CFG_REVISION_ID                      = 0x00000000;
    constexpr unsigned int IP_ESPI_REG_REVISION_ID                            = 0x00000000;
    constexpr unsigned int IP_SDP_REG_REVISION_ID                             = 0x00000000;
    constexpr unsigned int IP_USB_PD_REVISION_ID                              = 0x00000000;
    constexpr unsigned int MMREG_REVISION_ID                                  = 0x00000000;
    constexpr unsigned int NUM_REQUESTORS                                     = 0x00000005;
    constexpr unsigned int REQID_ECHI2                                        = 0x00000004;
    constexpr unsigned int REQID_EHCI                                         = 0x00000001;
    constexpr unsigned int REQID_OCHI2                                        = 0x00000003;
    constexpr unsigned int REQID_OHCI                                         = 0x00000000;
    constexpr unsigned int REQID_SATA0                                        = 0x00000002;
    constexpr unsigned int SATA_AHCI_REG_REVISION_ID                          = 0x00000000;
    constexpr unsigned int SATA_PCI_CFG_REVISION_ID                           = 0x00000000;
    constexpr unsigned int SATA_REVISION_ID                                   = 0x00000000;
    constexpr unsigned int sata_sgpio_reg_REVISION_ID                         = 0x00000000;
    constexpr unsigned int SDB_REG_REVISION_ID                                = 0x00000000;
    constexpr unsigned int SD_PCI_CFG_REVISION_ID                             = 0x00000000;
} // namespace Rembrandt

namespace Stx1
{
    constexpr unsigned int AL_REVISION_ID                                     = 0x00000012;
    constexpr unsigned int EMMC_CFG_REVISION_ID                               = 0x00000000;
    constexpr unsigned int EMMC_HC_REG_REVISION_ID                            = 0x00000000;
    constexpr unsigned int ENHIOMEMAPERTURE_REVISION_ID                       = 0x00000000;
    constexpr unsigned int ILA_REG_REVISION_ID                                = 0x00000000;
    constexpr unsigned int IP_AL2AHB_DEVICE_CFG_REVISION_ID                   = 0x00000000;
    constexpr unsigned int IP_AL2AHB_HARD_ADDR_HCLK_REVISION_ID               = 0x00000000;
    constexpr unsigned int IP_AL2AHB_HARD_ADDR_REVISION_ID                    = 0x00000000;
    constexpr unsigned int IP_AL2AHB_P2P_CFG_REVISION_ID                      = 0x00000000;
    constexpr unsigned int IP_ESPI_REG_REVISION_ID                            = 0x00000000;
    constexpr unsigned int IP_SDP_REG_REVISION_ID                             = 0x00000000;
    constexpr unsigned int IP_USB_PD_REVISION_ID                              = 0x00000000;
    constexpr unsigned int MMREG_REVISION_ID                                  = 0x00000000;
    constexpr unsigned int NUM_REQUESTORS                                     = 0x00000005;
    constexpr unsigned int REQID_ECHI2                                        = 0x00000004;
    constexpr unsigned int REQID_EHCI                                         = 0x00000001;
    constexpr unsigned int REQID_OCHI2                                        = 0x00000003;
    constexpr unsigned int REQID_OHCI                                         = 0x00000000;
    constexpr unsigned int REQID_SATA0                                        = 0x00000002;
    constexpr unsigned int SDB_REG_REVISION_ID                                = 0x00000000;
    constexpr unsigned int SD_PCI_CFG_REVISION_ID                             = 0x00000000;
} // namespace Stx1

#if CHIP_HDR_STRIX_HALO
namespace StxH
{
    constexpr unsigned int AL_REVISION_ID                                     = 0x00000012;
    constexpr unsigned int CC_GC_PRIM_CONFIG__RESET                           = 0x000fffa0;
    constexpr unsigned int CC_GC_SA_UNIT_DISABLE__RESET                       = 0x00fff000;
    constexpr unsigned int CC_GC_SHADER_ARRAY_CONFIG__RESET                   = 0xffe00000;
    constexpr unsigned int CC_RB_BACKEND_DISABLE__RESET                       = 0x00000000;
    constexpr unsigned int CGTS_TCC_DISABLE__RESET                            = 0x00000000;
    constexpr unsigned int EMMC_CFG_REVISION_ID                               = 0x00000000;
    constexpr unsigned int EMMC_HC_REG_REVISION_ID                            = 0x00000000;
    constexpr unsigned int ENHIOMEMAPERTURE_REVISION_ID                       = 0x00000000;
    constexpr unsigned int ILA_REG_REVISION_ID                                = 0x00000000;
    constexpr unsigned int IP_AL2AHB_DEVICE_CFG_REVISION_ID                   = 0x00000000;
    constexpr unsigned int IP_AL2AHB_HARD_ADDR_HCLK_REVISION_ID               = 0x00000000;
    constexpr unsigned int IP_AL2AHB_HARD_ADDR_REVISION_ID                    = 0x00000000;
    constexpr unsigned int IP_AL2AHB_P2P_CFG_REVISION_ID                      = 0x00000000;
    constexpr unsigned int IP_ESPI_REG_REVISION_ID                            = 0x00000000;
    constexpr unsigned int IP_SDP_REG_REVISION_ID                             = 0x00000000;
    constexpr unsigned int IP_TWDBG_REVISION_ID                               = 0x00000000;
    constexpr unsigned int IP_USB_PD_REVISION_ID                              = 0x00000000;
    constexpr unsigned int MMREG_REVISION_ID                                  = 0x00000000;
    constexpr unsigned int NUM_REQUESTORS                                     = 0x00000005;
    constexpr unsigned int REQID_ECHI2                                        = 0x00000004;
    constexpr unsigned int REQID_EHCI                                         = 0x00000001;
    constexpr unsigned int REQID_OCHI2                                        = 0x00000003;
    constexpr unsigned int REQID_OHCI                                         = 0x00000000;
    constexpr unsigned int REQID_SATA0                                        = 0x00000002;
    constexpr unsigned int SDB_REG_REVISION_ID                                = 0x00000000;
    constexpr unsigned int SD_PCI_CFG_REVISION_ID                             = 0x00000000;
} // namespace StxH
#endif

enum PerfCtrId
{
    CBPerfSelId                              = 0,
    ChaPerfSelId                             = 1,
    ChcPerfSelId                             = 2,
    ChcgPerfSelId                            = 3,
    CpcPerfcountSelId                        = 4,
    CpfPerfcountSelId                        = 5,
    CpgPerfcountSelId                        = 6,
    DfCsPerfSelId                            = 7,
    DfMallPerfSelId                          = 8,
    GCRPerfSelId                             = 9,
    Gcvml2PerfSelId                          = 10,
    GdsPerfcountSelectId                     = 11,
    Ge1PerfcountSelectId                     = 12,
    Ge2DistPerfcountSelectId                 = 13,
    Ge2SePerfcountSelectId                   = 14,
    GePerfcountSelectId                      = 15,
    Gl1aPerfSelId                            = 16,
    Gl1cPerfSelId                            = 17,
    Gl2aPerfSelId                            = 18,
    Gl2cPerfSelId                            = 19,
    GrbmPerfSelId                            = 20,
    GrbmSe0PerfSelId                         = 21,
    GrbmSe1PerfSelId                         = 22,
    GrbmSe2PerfSelId                         = 23,
    GrbmSe3PerfSelId                         = 24,
    GrbmSe4PerfSelId                         = 25,
    GrbmSe5PerfSelId                         = 26,
    GrbmSe6PerfSelId                         = 27,
    GrbmSe7PerfSelId                         = 28,
    PcPerfcntSelId                           = 29,
    PerfcounterValsId                        = 30,
    PhPerfcntSelId                           = 31,
    RMIPerfSelId                             = 32,
    RlcPerfcounterSelId                      = 33,
    ScPerfcntSelId                           = 34,
    SdmaPerfSelId                            = 35,
    SdmaPerfmonSelId                         = 36,
    SpiPerfcntSelId                          = 37,
    SqPerfSelId                              = 38,
    SqgPerfSelId                             = 39,
    SuPerfcntSelId                           = 40,
    SxPerfcounterValsId                      = 41,
    TaPerfcountSelId                         = 42,
    TcpPerfcountSelectId                     = 43,
    TdPerfcountSelId                         = 44,
    UTCL1PerfSelId                           = 45,
    UmcPerfcountSelectId                     = 46,
    MaxPerfCtrId                             = 47,
};

constexpr unsigned int Nv10MaxPerfEventIds[MaxPerfCtrId] =
{
    CB_PERF_SEL_CC_CACHE_SECTOR_HIT__GFX10,
    CHA_PERF_SEL_CYCLE__GFX101,
    CHC_PERF_SEL_REQ_CLIENT14,
    CHCG_PERF_SEL_REQ_CLIENT14__GFX101,
    CPC_PERF_SEL_MES_THREAD1,
    CPF_PERF_SEL_CPF_UTCL2IU_XNACK,
    CPG_PERF_SEL_DMA_FETCHER_STALLED_ON_ROQ_FULL,
    0,
    0,
    GCR_PERF_SEL_UTCL2_FILTERED_RET,
    0,
    GDS_PERF_SEL_GWS_BYPASS__GFX10,
    0,
    0,
    0,
    vgt_se3pa1_clips_starved_busy,
    GL1A_PERF_SEL_CYCLE__GFX101,
    GL1C_PERF_SEL_REQ_CLIENT27__GFX101,
    GL2A_PERF_SEL_REQ_BURST_CLIENT15,
    GL2C_PERF_SEL_CM_DCC_STALL__GFX101,
    GRBM_PERF_SEL_GL1CC_BUSY,
    GRBM_SE0_PERF_SEL_GL1CC_BUSY,
    GRBM_SE1_PERF_SEL_GL1CC_BUSY,
    GRBM_SE2_PERF_SEL_GL1CC_BUSY,
    GRBM_SE3_PERF_SEL_GL1CC_BUSY,
    0,
    0,
    0,
    0,
    0,
    DB_PERF_SEL_FG_LOB_FWDR_TIMEOUT_hits__GFX10,
    PH_PERF_SEL_SC7_PA7_DEALLOC_4_0_RD__GFX10,
    RMI_PERF_SEL_UTCL0_UTCL1_PERM_FAULT__GFX10,
    RLC_PERF_SEL_SERDES_COMMAND_WRITE,
    SC_PK_PM_MAX_REZ_CNT_FORCE_EOV_WAVE_BRK_1H,
    SDMA_PERF_SEL_MMHUB_TAG_DELAY_COUNTER__GFX10,
    0,
    SPI_PERF_LS_PERS_UPD_FULL1__GFX101,
    SP_PERF_SEL_DUMMY_LAST__GFX10,
    0,
    PERF_ENGG_POS_REQ_STALLED_BY_FULL_CLIPV_FIFO__GFX101,
    SX_PERF_SEL_RB3_STALL_DUE_TO_ORDERING__GFX10,
    TA_PERF_SEL_nosample_path_cycles__GFX101,
    TCP_PERF_SEL_BACK_COMPAT_SWITCH__GFX101,
    TD_PERF_SEL_nofilter_popcount_dmask_lt_num_comp_of_fmt__GFX101,
    UTCL1_PERF_SEL_RANGE_INV_REQS__GFX101,
    UMC_PERF_SEL_RefreshType__GFX101,
};

constexpr unsigned int Nv21MaxPerfEventIds[MaxPerfCtrId] =
{
    CB_PERF_SEL_CC_CACHE_SECTOR_HIT__GFX10,
    CHA_PERF_SEL_CYCLE__NV2X,
    CHC_PERF_SEL_REQ_CLIENT19__GFX103PLUS,
    CHCG_PERF_SEL_REQ_CLIENT19__NV2X,
    CPC_PERF_SEL_MES_THREAD1,
    CPF_PERF_SEL_CPF_UTCL2IU_XNACK,
    CPG_PERF_SEL_DMA_FETCHER_STALLED_ON_ROQ_FULL,
    0,
    DF_MALL_PERF_SEL_MALL_SDP_LAT_HIST_GT1000__NV2X,
    GCR_PERF_SEL_PIO_TCP_TLB_SHOOTDOWN_REQ__NV21,
    0,
    GDS_PERF_SEL_GWS_BYPASS__GFX10,
    ge1_rbiu_dr_fifo_starved_p1,
    ge_tf_ret_data_stalling_hs_done__GFX10VRS,
    ge_hs_stall_tfmm_fifo_full__GFX10VRS,
    0,
    GL1A_PERF_SEL_CYCLE__GFX103PLUS,
    GL1C_PERF_SEL_UTCL0_UTCL1_XNACK_NO_RETRY_FAULT__GFX10VRS,
    GL2A_PERF_SEL_REQ_BURST_CLIENT15,
    GL2C_PERF_SEL_CM_DCC_OUT_CONST__GFX10VRS,
    GRBM_PERF_SEL_GL1CC_BUSY,
    GRBM_SE0_PERF_SEL_GL1CC_BUSY,
    GRBM_SE1_PERF_SEL_GL1CC_BUSY,
    GRBM_SE2_PERF_SEL_GL1CC_BUSY,
    GRBM_SE3_PERF_SEL_GL1CC_BUSY,
    0,
    0,
    0,
    0,
    0,
    DB_PERF_SEL_postz_ps_invoked_pixel_cnt__GFX10VRS,
    PH_PERF_SEL_8_SC_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_FULL__GFX103PLUS,
    RMI_PERF_SEL_UTCL0_UTCL1_PERM_FAULT__GFX10,
    RLC_PERF_SEL_SERDES_COMMAND_WRITE,
    SC_BM_MULTI_ACCUM_4_BE_STALLED__GFX103PLUS,
    SDMA_PERF_SEL_CH_CE_RDRET_VALID__GFX10VRS,
    0,
    SPI_PERF_EXP_THROT_CAUSALITY_DETECTED__GFX10VRS,
    SP_PERF_SEL_DUMMY_LAST__GFX10,
    0,
    PERF_OUTPUT_PRIM_4_SC__GFX103PLUS,
    SX_PERF_SEL_RB3_STALL_DUE_TO_ORDERING__GFX10,
    TA_PERF_SEL_tcreq_clk_valid_cycles__GFX103PLUS,
    TCP_PERF_SEL_BACK_COMPAT_SWITCH__GFX10VRS,
    TD_PERF_SEL_input_bp_due_to_done_scoreboard_full__GFX10VRS,
    UTCL1_PERF_SEL_INV_ALL_VMID_INVREQS__GFX10VRS,
    UMC_PERF_SEL_TempCnt__NV2X,
};

constexpr unsigned int Nv22MaxPerfEventIds[MaxPerfCtrId] =
{
    CB_PERF_SEL_CC_CACHE_SECTOR_HIT__GFX10,
    CHA_PERF_SEL_CYCLE__NV2X,
    CHC_PERF_SEL_REQ_CLIENT19__GFX103PLUS,
    CHCG_PERF_SEL_REQ_CLIENT19__NV2X,
    CPC_PERF_SEL_MES_THREAD1,
    CPF_PERF_SEL_CPF_UTCL2IU_XNACK,
    CPG_PERF_SEL_DMA_FETCHER_STALLED_ON_ROQ_FULL,
    0,
    DF_MALL_PERF_SEL_MALL_SDP_LAT_HIST_GT1000__NV2X,
    GCR_PERF_SEL_PIO_TCP_TLB_SHOOTDOWN_REQ__NV22,
    0,
    GDS_PERF_SEL_GWS_BYPASS__GFX10,
    ge1_rbiu_dr_fifo_starved_p1,
    ge_tf_ret_data_stalling_hs_done__GFX10VRS,
    ge_hs_stall_tfmm_fifo_full__GFX10VRS,
    0,
    GL1A_PERF_SEL_CYCLE__GFX103PLUS,
    GL1C_PERF_SEL_UTCL0_UTCL1_XNACK_NO_RETRY_FAULT__GFX10VRS,
    GL2A_PERF_SEL_REQ_BURST_CLIENT15,
    GL2C_PERF_SEL_CM_DCC_OUT_CONST__GFX10VRS,
    GRBM_PERF_SEL_GL1CC_BUSY,
    GRBM_SE0_PERF_SEL_GL1CC_BUSY,
    GRBM_SE1_PERF_SEL_GL1CC_BUSY,
    GRBM_SE2_PERF_SEL_GL1CC_BUSY,
    GRBM_SE3_PERF_SEL_GL1CC_BUSY,
    0,
    0,
    0,
    0,
    0,
    DB_PERF_SEL_postz_ps_invoked_pixel_cnt__GFX10VRS,
    PH_PERF_SEL_8_SC_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_FULL__GFX103PLUS,
    RMI_PERF_SEL_UTCL0_UTCL1_PERM_FAULT__GFX10,
    RLC_PERF_SEL_SERDES_COMMAND_WRITE,
    SC_BM_MULTI_ACCUM_4_BE_STALLED__GFX103PLUS,
    SDMA_PERF_SEL_CH_CE_RDRET_VALID__GFX10VRS,
    0,
    SPI_PERF_EXP_THROT_CAUSALITY_DETECTED__GFX10VRS,
    SP_PERF_SEL_DUMMY_LAST__GFX10,
    0,
    PERF_OUTPUT_PRIM_4_SC__GFX103PLUS,
    SX_PERF_SEL_RB3_STALL_DUE_TO_ORDERING__GFX10,
    TA_PERF_SEL_tcreq_clk_valid_cycles__GFX103PLUS,
    TCP_PERF_SEL_BACK_COMPAT_SWITCH__GFX10VRS,
    TD_PERF_SEL_input_bp_due_to_done_scoreboard_full__GFX10VRS,
    UTCL1_PERF_SEL_INV_ALL_VMID_INVREQS__GFX10VRS,
    UMC_PERF_SEL_TempCnt__NV2X,
};

constexpr unsigned int Nv23MaxPerfEventIds[MaxPerfCtrId] =
{
    CB_PERF_SEL_CC_CACHE_SECTOR_HIT__GFX10,
    CHA_PERF_SEL_CYCLE__NV2X,
    CHC_PERF_SEL_REQ_CLIENT19__GFX103PLUS,
    CHCG_PERF_SEL_REQ_CLIENT19__NV2X,
    CPC_PERF_SEL_MES_THREAD1,
    CPF_PERF_SEL_CPF_UTCL2IU_XNACK,
    CPG_PERF_SEL_DMA_FETCHER_STALLED_ON_ROQ_FULL,
    0,
    DF_MALL_PERF_SEL_MALL_SDP_LAT_HIST_GT1000__NV2X,
    GCR_PERF_SEL_PIO_TCP_TLB_SHOOTDOWN_REQ__NV23,
    0,
    GDS_PERF_SEL_GWS_BYPASS__GFX10,
    ge1_rbiu_dr_fifo_starved_p1,
    ge_tf_ret_data_stalling_hs_done__GFX10VRS,
    ge_hs_stall_tfmm_fifo_full__GFX10VRS,
    0,
    GL1A_PERF_SEL_CYCLE__GFX103PLUS,
    GL1C_PERF_SEL_UTCL0_UTCL1_XNACK_NO_RETRY_FAULT__GFX10VRS,
    GL2A_PERF_SEL_REQ_BURST_CLIENT15,
    GL2C_PERF_SEL_CM_DCC_OUT_CONST__GFX10VRS,
    GRBM_PERF_SEL_GL1CC_BUSY,
    GRBM_SE0_PERF_SEL_GL1CC_BUSY,
    GRBM_SE1_PERF_SEL_GL1CC_BUSY,
    GRBM_SE2_PERF_SEL_GL1CC_BUSY,
    GRBM_SE3_PERF_SEL_GL1CC_BUSY,
    0,
    0,
    0,
    0,
    0,
    DB_PERF_SEL_postz_ps_invoked_pixel_cnt__GFX10VRS,
    PH_PERF_SEL_8_SC_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_FULL__GFX103PLUS,
    RMI_PERF_SEL_UTCL0_UTCL1_PERM_FAULT__GFX10,
    RLC_PERF_SEL_SERDES_COMMAND_WRITE,
    SC_BM_MULTI_ACCUM_4_BE_STALLED__GFX103PLUS,
    SDMA_PERF_SEL_CH_CE_RDRET_VALID__GFX10VRS,
    0,
    SPI_PERF_EXP_THROT_CAUSALITY_DETECTED__GFX10VRS,
    SP_PERF_SEL_DUMMY_LAST__GFX10,
    0,
    PERF_OUTPUT_PRIM_4_SC__GFX103PLUS,
    SX_PERF_SEL_RB3_STALL_DUE_TO_ORDERING__GFX10,
    TA_PERF_SEL_tcreq_clk_valid_cycles__GFX103PLUS,
    TCP_PERF_SEL_BACK_COMPAT_SWITCH__GFX10VRS,
    TD_PERF_SEL_input_bp_due_to_done_scoreboard_full__GFX10VRS,
    UTCL1_PERF_SEL_UTCL2_REQS_OUTSTANDING_ACCUM__NV23,
    UMC_PERF_SEL_TempCnt__NV2X,
};

constexpr unsigned int Nv24MaxPerfEventIds[MaxPerfCtrId] =
{
    CB_PERF_SEL_CC_CACHE_SECTOR_HIT__GFX10,
    CHA_PERF_SEL_CYCLE__NV2X,
    CHC_PERF_SEL_REQ_CLIENT19__GFX103PLUS,
    CHCG_PERF_SEL_REQ_CLIENT19__NV2X,
    CPC_PERF_SEL_MES_THREAD1,
    CPF_PERF_SEL_CPF_UTCL2IU_XNACK,
    CPG_PERF_SEL_DMA_FETCHER_STALLED_ON_ROQ_FULL,
    0,
    DF_MALL_PERF_SEL_MALL_SDP_LAT_HIST_GT1000__NV2X,
    GCR_PERF_SEL_PIO_TCP_TLB_SHOOTDOWN_REQ__NV24,
    0,
    GDS_PERF_SEL_GWS_BYPASS__GFX10,
    ge1_rbiu_dr_fifo_starved_p1,
    ge_tf_ret_data_stalling_hs_done__GFX10VRS,
    ge_hs_stall_tfmm_fifo_full__GFX10VRS,
    0,
    GL1A_PERF_SEL_CYCLE__GFX103PLUS,
    GL1C_PERF_SEL_UTCL0_UTCL1_XNACK_NO_RETRY_FAULT__GFX10VRS,
    GL2A_PERF_SEL_REQ_BURST_CLIENT15,
    GL2C_PERF_SEL_CM_DCC_OUT_CONST__GFX10VRS,
    GRBM_PERF_SEL_GL1CC_BUSY,
    GRBM_SE0_PERF_SEL_GL1CC_BUSY,
    GRBM_SE1_PERF_SEL_GL1CC_BUSY,
    GRBM_SE2_PERF_SEL_GL1CC_BUSY,
    GRBM_SE3_PERF_SEL_GL1CC_BUSY,
    0,
    0,
    0,
    0,
    0,
    DB_PERF_SEL_postz_ps_invoked_pixel_cnt__GFX10VRS,
    PH_PERF_SEL_8_SC_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_FULL__GFX103PLUS,
    RMI_PERF_SEL_UTCL0_UTCL1_PERM_FAULT__GFX10,
    RLC_PERF_SEL_SERDES_COMMAND_WRITE,
    SC_BM_MULTI_ACCUM_4_BE_STALLED__GFX103PLUS,
    SDMA_PERF_SEL_CH_CE_RDRET_VALID__GFX10VRS,
    0,
    SPI_PERF_EXP_THROT_CAUSALITY_DETECTED__GFX10VRS,
    SP_PERF_SEL_DUMMY_LAST__GFX10,
    0,
    PERF_OUTPUT_PRIM_4_SC__GFX103PLUS,
    SX_PERF_SEL_RB3_STALL_DUE_TO_ORDERING__GFX10,
    TA_PERF_SEL_tcreq_clk_valid_cycles__GFX103PLUS,
    TCP_PERF_SEL_BACK_COMPAT_SWITCH__GFX10VRS,
    TD_PERF_SEL_input_bp_due_to_done_scoreboard_full__GFX10VRS,
    UTCL1_PERF_SEL_UTCL2_REQS_OUTSTANDING_ACCUM__NV24,
    UMC_PERF_SEL_TempCnt__NV2X,
};

constexpr unsigned int RembrandtMaxPerfEventIds[MaxPerfCtrId] =
{
    CB_PERF_SEL_CC_CACHE_SECTOR_HIT__GFX10,
    CHA_PERF_SEL_CYCLE__APU103,
    CHC_PERF_SEL_REQ_CLIENT19__GFX103PLUS,
    0,
    CPC_PERF_SEL_MES_THREAD1,
    CPF_PERF_SEL_CPF_UTCL2IU_XNACK,
    CPG_PERF_SEL_DMA_FETCHER_STALLED_ON_ROQ_FULL,
    0,
    0,
    GCR_PERF_SEL_PIO_TCP_TLB_SHOOTDOWN_REQ__APU103,
    0,
    GDS_PERF_SEL_GWS_BYPASS__GFX10,
    ge1_rbiu_dr_fifo_starved_p1,
    ge_tf_ret_data_stalling_hs_done__GFX10VRS,
    ge_hs_stall_tfmm_fifo_full__GFX10VRS,
    0,
    GL1A_PERF_SEL_CYCLE__GFX103PLUS,
    GL1C_PERF_SEL_UTCL0_UTCL1_XNACK_NO_RETRY_FAULT__GFX10VRS,
    GL2A_PERF_SEL_REQ_BURST_CLIENT15,
    GL2C_PERF_SEL_CM_DCC_OUT_CONST__GFX10VRS,
    GRBM_PERF_SEL_GL1CC_BUSY,
    GRBM_SE0_PERF_SEL_GL1CC_BUSY,
    GRBM_SE1_PERF_SEL_GL1CC_BUSY,
    GRBM_SE2_PERF_SEL_GL1CC_BUSY,
    GRBM_SE3_PERF_SEL_GL1CC_BUSY,
    0,
    0,
    0,
    0,
    0,
    DB_PERF_SEL_postz_ps_invoked_pixel_cnt__GFX10VRS,
    PH_PERF_SEL_8_SC_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_FULL__GFX103PLUS,
    RMI_PERF_SEL_UTCL0_UTCL1_PERM_FAULT__GFX10,
    RLC_PERF_SEL_SERDES_COMMAND_WRITE,
    SC_BM_MULTI_ACCUM_4_BE_STALLED__GFX103PLUS,
    SDMA_PERF_SEL_CH_CE_RDRET_VALID__GFX10VRS,
    0,
    SPI_PERF_EXP_THROT_CAUSALITY_DETECTED__GFX10VRS,
    SP_PERF_SEL_DUMMY_LAST__GFX10,
    0,
    PERF_OUTPUT_PRIM_4_SC__GFX103PLUS,
    SX_PERF_SEL_RB3_STALL_DUE_TO_ORDERING__GFX10,
    TA_PERF_SEL_tcreq_clk_valid_cycles__GFX103PLUS,
    TCP_PERF_SEL_BACK_COMPAT_SWITCH__GFX10VRS,
    TD_PERF_SEL_input_bp_due_to_done_scoreboard_full__GFX10VRS,
    UTCL1_PERF_SEL_UTCL2_REQS_OUTSTANDING_ACCUM__REMBRANDT,
    0,
};

constexpr unsigned int Nv31MaxPerfEventIds[MaxPerfCtrId] =
{
    CB_PERF_SEL_EXPORT_KILLED_BY_NULL_TARGET_SHADER_MASK__GFX11,
    CHA_PERF_SEL_CYCLE__NV3X,
    CHC_PERF_SEL_REQ_CLIENT23__GFX11,
    CHCG_PERF_SEL_REQ_CLIENT23__NV3X,
    CPC_PERF_SEL_MEC_THREAD3__GFX11,
    CPF_PERF_SEL_CP_SDMA_MNGR_SDMABUSY__GFX11,
    CPG_PERF_SEL_PFP_VGTDMA_DB_ROQ_DATA_STALL1__GFX11,
    DF_CS_PERF_SEL_CS_FTI_LAT_HIST_GT1000,
    DF_MALL_PERF_SEL_MALL_SDP_LAT_HIST_GT1000__GFX11,
    GCR_PERF_SEL_PIO_GL1_TLB_SHOOTDOWN_REQ__NV3X,
    GCVML2_PERF_SEL_EVENT_90,
    GDS_PERF_SEL_SE7_GS_WAVE_ID_VALID__GFX11,
    ge1_rbiu_dr_fifo_starved_p1,
    ge_agm_gcr_combine__GFX11,
    ge_ngg_busy_base__GFX11,
    0,
    GL1A_PERF_SEL_CYCLE__GFX103PLUS,
    GL1C_PERF_SEL_UTCL0_UTCL1_XNACK_NO_RETRY_FAULT__GFX11,
    GL2A_PERF_SEL_RTN_CREDIT_STALL_CLIENT15__GFX11,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT19__GFX110,
    GRBM_PERF_SEL_PC_BUSY__GFX11,
    GRBM_SE0_PERF_SEL_PC_BUSY__GFX11,
    GRBM_SE1_PERF_SEL_PC_BUSY__GFX11,
    GRBM_SE2_PERF_SEL_PC_BUSY__GFX11,
    GRBM_SE3_PERF_SEL_PC_BUSY__GFX11,
    GRBM_SE4_PERF_SEL_PC_BUSY,
    GRBM_SE5_PERF_SEL_PC_BUSY,
    GRBM_SE6_PERF_SEL_PC_BUSY,
    GRBM_SE7_PERF_SEL_PC_BUSY,
    PC_PERF_GRBM_BUSY,
    DB_PERF_SEL_OREO_Events_stalls__GFX11,
    PH_PERF_SC7_FIFO_STATUS_3__GFX11,
    RMI_PERF_SEL_CONSUMER_PROBEGEN_DB_RTS_RTR__GFX11,
    RLC_PERF_SEL_SERDES_COMMAND_WRITE,
    SC_SPI_WAVE_STALLED_BY_SPI__GFX11,
    SDMA_PERF_SEL_QUEUE7_SELECT__GFX11,
    SDMA_PERFMON_SEL_MMHUB_TAG_DELAY_COUNTER,
    SPI_PERF_BUSY__GFX11,
    SQ_PERF_SEL_NONE2__GFX11,
    SQG_PERF_SEL_DUMMY_LAST,
    PERF_PA_BUSY__GFX11,
    SX_PERF_SEL_DB3_4X2_DISCARD__GFX11,
    TA_PERF_SEL_tcreq_clk_valid_cycles__GFX103PLUS,
    TCP_PERF_SEL_BURST_BIN_READHIT_gt16__GFX110,
    TD_PERF_SEL_store_preempts_a_load__GFX110,
    UTCL1_PERF_SEL_ALOG_STALL_PMM_CREDITS__GFX11,
    UMC_PERF_SEL_ClockCount__GFX11,
};

constexpr unsigned int Nv32MaxPerfEventIds[MaxPerfCtrId] =
{
    CB_PERF_SEL_EXPORT_KILLED_BY_NULL_TARGET_SHADER_MASK__GFX11,
    CHA_PERF_SEL_CYCLE__NV3X,
    CHC_PERF_SEL_REQ_CLIENT23__GFX11,
    CHCG_PERF_SEL_REQ_CLIENT23__NV3X,
    CPC_PERF_SEL_MEC_THREAD3__GFX11,
    CPF_PERF_SEL_CP_SDMA_MNGR_SDMABUSY__GFX11,
    CPG_PERF_SEL_PFP_VGTDMA_DB_ROQ_DATA_STALL1__GFX11,
    DF_CS_PERF_SEL_CS_FTI_LAT_HIST_GT1000,
    DF_MALL_PERF_SEL_MALL_SDP_LAT_HIST_GT1000__GFX11,
    GCR_PERF_SEL_PIO_GL1_TLB_SHOOTDOWN_REQ__NV3X,
    GCVML2_PERF_SEL_EVENT_90,
    GDS_PERF_SEL_SE7_GS_WAVE_ID_VALID__GFX11,
    ge1_rbiu_dr_fifo_starved_p1,
    ge_agm_gcr_combine__GFX11,
    ge_ngg_busy_base__GFX11,
    0,
    GL1A_PERF_SEL_CYCLE__GFX103PLUS,
    GL1C_PERF_SEL_UTCL0_UTCL1_XNACK_NO_RETRY_FAULT__GFX11,
    GL2A_PERF_SEL_RTN_CREDIT_STALL_CLIENT15__GFX11,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT19__GFX110,
    GRBM_PERF_SEL_PC_BUSY__GFX11,
    GRBM_SE0_PERF_SEL_PC_BUSY__GFX11,
    GRBM_SE1_PERF_SEL_PC_BUSY__GFX11,
    GRBM_SE2_PERF_SEL_PC_BUSY__GFX11,
    GRBM_SE3_PERF_SEL_PC_BUSY__GFX11,
    GRBM_SE4_PERF_SEL_PC_BUSY,
    GRBM_SE5_PERF_SEL_PC_BUSY,
    GRBM_SE6_PERF_SEL_PC_BUSY,
    GRBM_SE7_PERF_SEL_PC_BUSY,
    PC_PERF_GRBM_BUSY,
    DB_PERF_SEL_OREO_Events_stalls__GFX11,
    PH_PERF_SC7_FIFO_STATUS_3__GFX11,
    RMI_PERF_SEL_CONSUMER_PROBEGEN_DB_RTS_RTR__GFX11,
    RLC_PERF_SEL_SERDES_COMMAND_WRITE,
    SC_SPI_WAVE_STALLED_BY_SPI__GFX11,
    SDMA_PERF_SEL_QUEUE7_SELECT__GFX11,
    SDMA_PERFMON_SEL_MMHUB_TAG_DELAY_COUNTER,
    SPI_PERF_BUSY__GFX11,
    SQ_PERF_SEL_NONE2__GFX11,
    SQG_PERF_SEL_DUMMY_LAST,
    PERF_PA_BUSY__GFX11,
    SX_PERF_SEL_DB3_4X2_DISCARD__GFX11,
    TA_PERF_SEL_tcreq_clk_valid_cycles__GFX103PLUS,
    TCP_PERF_SEL_BURST_BIN_READHIT_gt16__GFX110,
    TD_PERF_SEL_store_preempts_a_load__GFX110,
    UTCL1_PERF_SEL_ALOG_STALL_PMM_CREDITS__GFX11,
    UMC_PERF_SEL_ClockCount__GFX11,
};

constexpr unsigned int Nv33MaxPerfEventIds[MaxPerfCtrId] =
{
    CB_PERF_SEL_EXPORT_KILLED_BY_NULL_TARGET_SHADER_MASK__GFX11,
    CHA_PERF_SEL_CYCLE__NV3X,
    CHC_PERF_SEL_REQ_CLIENT23__GFX11,
    CHCG_PERF_SEL_REQ_CLIENT23__NV3X,
    CPC_PERF_SEL_MEC_THREAD3__GFX11,
    CPF_PERF_SEL_CP_SDMA_MNGR_SDMABUSY__GFX11,
    CPG_PERF_SEL_PFP_VGTDMA_DB_ROQ_DATA_STALL1__GFX11,
    DF_CS_PERF_SEL_CS_FTI_LAT_HIST_GT1000,
    DF_MALL_PERF_SEL_MALL_SDP_LAT_HIST_GT1000__GFX11,
    GCR_PERF_SEL_PIO_GL1_TLB_SHOOTDOWN_REQ__NV3X,
    GCVML2_PERF_SEL_EVENT_90,
    GDS_PERF_SEL_SE7_GS_WAVE_ID_VALID__GFX11,
    ge1_rbiu_dr_fifo_starved_p1,
    ge_agm_gcr_combine__GFX11,
    ge_ngg_busy_base__GFX11,
    0,
    GL1A_PERF_SEL_CYCLE__GFX103PLUS,
    GL1C_PERF_SEL_UTCL0_UTCL1_XNACK_NO_RETRY_FAULT__GFX11,
    GL2A_PERF_SEL_RTN_CREDIT_STALL_CLIENT15__GFX11,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT19__GFX110,
    GRBM_PERF_SEL_PC_BUSY__GFX11,
    GRBM_SE0_PERF_SEL_PC_BUSY__GFX11,
    GRBM_SE1_PERF_SEL_PC_BUSY__GFX11,
    GRBM_SE2_PERF_SEL_PC_BUSY__GFX11,
    GRBM_SE3_PERF_SEL_PC_BUSY__GFX11,
    GRBM_SE4_PERF_SEL_PC_BUSY,
    GRBM_SE5_PERF_SEL_PC_BUSY,
    GRBM_SE6_PERF_SEL_PC_BUSY,
    GRBM_SE7_PERF_SEL_PC_BUSY,
    PC_PERF_GL1H_REQ_ROUTE_ID1__NV33,
    DB_PERF_SEL_OREO_Events_stalls__GFX11,
    PH_PERF_SC7_FIFO_STATUS_3__GFX11,
    RMI_PERF_SEL_CONSUMER_PROBEGEN_DB_RTS_RTR__GFX11,
    RLC_PERF_SEL_SERDES_COMMAND_WRITE,
    SC_VRC_REPROBE_FULL__NV33,
    SDMA_PERF_SEL_QUEUE7_SELECT__GFX11,
    SDMA_PERFMON_SEL_MMHUB_TAG_DELAY_COUNTER,
    SPI_PERF_BUSY__GFX11,
    SQ_PERF_SEL_NONE2__GFX11,
    SQG_PERF_SEL_DUMMY_LAST,
    PERF_PA_BUSY__GFX11,
    SX_PERF_SEL_DB3_4X2_DISCARD__GFX11,
    TA_PERF_SEL_tcreq_clk_valid_cycles__GFX103PLUS,
    TCP_PERF_SEL_BURST_BIN_READHIT_gt16__GFX110,
    TD_PERF_SEL_store_preempts_a_load__GFX110,
    UTCL1_PERF_SEL_ALOG_STALL_PMM_CREDITS__GFX11,
    UMC_PERF_SEL_ClockCount__GFX11,
};

constexpr unsigned int RaphaelMaxPerfEventIds[MaxPerfCtrId] =
{
    CB_PERF_SEL_CC_CACHE_SECTOR_HIT__GFX10,
    CHA_PERF_SEL_CYCLE__APU103,
    CHC_PERF_SEL_REQ_CLIENT19__GFX103PLUS,
    0,
    CPC_PERF_SEL_MES_THREAD1,
    CPF_PERF_SEL_CPF_UTCL2IU_XNACK,
    CPG_PERF_SEL_DMA_FETCHER_STALLED_ON_ROQ_FULL,
    0,
    0,
    GCR_PERF_SEL_PIO_TCP_TLB_SHOOTDOWN_REQ__APU103,
    0,
    GDS_PERF_SEL_GWS_BYPASS__GFX10,
    ge1_rbiu_dr_fifo_starved_p1,
    ge_tf_ret_data_stalling_hs_done__GFX10VRS,
    ge_hs_stall_tfmm_fifo_full__GFX10VRS,
    0,
    GL1A_PERF_SEL_CYCLE__GFX103PLUS,
    GL1C_PERF_SEL_UTCL0_UTCL1_XNACK_NO_RETRY_FAULT__GFX10VRS,
    GL2A_PERF_SEL_REQ_BURST_CLIENT15,
    GL2C_PERF_SEL_CM_DCC_OUT_CONST__GFX10VRS,
    GRBM_PERF_SEL_GL1CC_BUSY,
    GRBM_SE0_PERF_SEL_GL1CC_BUSY,
    GRBM_SE1_PERF_SEL_GL1CC_BUSY,
    GRBM_SE2_PERF_SEL_GL1CC_BUSY,
    GRBM_SE3_PERF_SEL_GL1CC_BUSY,
    0,
    0,
    0,
    0,
    0,
    DB_PERF_SEL_postz_ps_invoked_pixel_cnt__GFX10VRS,
    PH_PERF_SEL_8_SC_ARB_STARVED_FROM_ABOVE_WITH_UNSELECTED_FIFO_FULL__GFX103PLUS,
    RMI_PERF_SEL_UTCL0_UTCL1_PERM_FAULT__GFX10,
    RLC_PERF_SEL_SERDES_COMMAND_WRITE,
    SC_BM_MULTI_ACCUM_4_BE_STALLED__GFX103PLUS,
    SDMA_PERF_SEL_CH_CE_RDRET_VALID__GFX10VRS,
    0,
    SPI_PERF_EXP_THROT_CAUSALITY_DETECTED__GFX10VRS,
    SP_PERF_SEL_DUMMY_LAST__GFX10,
    0,
    PERF_OUTPUT_PRIM_4_SC__GFX103PLUS,
    SX_PERF_SEL_RB3_STALL_DUE_TO_ORDERING__GFX10,
    TA_PERF_SEL_tcreq_clk_valid_cycles__GFX103PLUS,
    TCP_PERF_SEL_BACK_COMPAT_SWITCH__GFX10VRS,
    TD_PERF_SEL_input_bp_due_to_done_scoreboard_full__GFX10VRS,
    UTCL1_PERF_SEL_UTCL2_REQS_OUTSTANDING_ACCUM__RAPHAEL,
    0,
};

constexpr unsigned int Phx1MaxPerfEventIds[MaxPerfCtrId] =
{
    CB_PERF_SEL_EXPORT_KILLED_BY_NULL_TARGET_SHADER_MASK__GFX11,
    CHA_PERF_SEL_CYCLE__APU11,
    CHC_PERF_SEL_REQ_CLIENT23__GFX11,
    0,
    CPC_PERF_SEL_MEC_THREAD3__GFX11,
    CPF_PERF_SEL_CP_SDMA_MNGR_SDMABUSY__GFX11,
    CPG_PERF_SEL_PFP_VGTDMA_DB_ROQ_DATA_STALL1__GFX11,
    DF_CS_PERF_SEL_CS_FTI_LAT_HIST_GT1000,
    DF_MALL_PERF_SEL_MALL_SDP_LAT_HIST_GT1000__GFX11,
    GCR_PERF_SEL_PIO_GL1_TLB_SHOOTDOWN_REQ__APU11,
    GCVML2_PERF_SEL_EVENT_90,
    GDS_PERF_SEL_SE7_GS_WAVE_ID_VALID__GFX11,
    ge1_rbiu_dr_fifo_starved_p1,
    ge_agm_gcr_combine__GFX11,
    ge_ngg_busy_base__GFX11,
    0,
    GL1A_PERF_SEL_CYCLE__GFX103PLUS,
    GL1C_PERF_SEL_UTCL0_UTCL1_XNACK_NO_RETRY_FAULT__GFX11,
    GL2A_PERF_SEL_RTN_CREDIT_STALL_CLIENT15__GFX11,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT19__GFX110,
    GRBM_PERF_SEL_PC_BUSY__GFX11,
    GRBM_SE0_PERF_SEL_PC_BUSY__GFX11,
    GRBM_SE1_PERF_SEL_PC_BUSY__GFX11,
    GRBM_SE2_PERF_SEL_PC_BUSY__GFX11,
    GRBM_SE3_PERF_SEL_PC_BUSY__GFX11,
    GRBM_SE4_PERF_SEL_PC_BUSY,
    GRBM_SE5_PERF_SEL_PC_BUSY,
    GRBM_SE6_PERF_SEL_PC_BUSY,
    GRBM_SE7_PERF_SEL_PC_BUSY,
    PC_PERF_GL1H_REQ_ROUTE_ID1__APU11,
    DB_PERF_SEL_OREO_Events_stalls__GFX11,
    PH_PERF_SC7_FIFO_STATUS_3__GFX11,
    RMI_PERF_SEL_CONSUMER_PROBEGEN_DB_RTS_RTR__GFX11,
    RLC_PERF_SEL_SERDES_COMMAND_WRITE,
    SC_VRC_REPROBE_FULL__APU11,
    SDMA_PERF_SEL_QUEUE7_SELECT__GFX11,
    SDMA_PERFMON_SEL_MMHUB_TAG_DELAY_COUNTER,
    SPI_PERF_BUSY__GFX11,
    SQ_PERF_SEL_NONE2__GFX11,
    SQG_PERF_SEL_DUMMY_LAST,
    PERF_PA_BUSY__GFX11,
    SX_PERF_SEL_DB3_4X2_DISCARD__GFX11,
    TA_PERF_SEL_tcreq_clk_valid_cycles__GFX103PLUS,
    TCP_PERF_SEL_BURST_BIN_READHIT_gt16__GFX110,
    TD_PERF_SEL_store_preempts_a_load__GFX110,
    UTCL1_PERF_SEL_ALOG_STALL_PMM_CREDITS__GFX11,
    UMC_PERF_SEL_ClockCount__GFX11,
};

constexpr unsigned int Phx2MaxPerfEventIds[MaxPerfCtrId] =
{
    CB_PERF_SEL_EXPORT_KILLED_BY_NULL_TARGET_SHADER_MASK__GFX11,
    CHA_PERF_SEL_CYCLE__APU11,
    CHC_PERF_SEL_REQ_CLIENT23__GFX11,
    0,
    CPC_PERF_SEL_MEC_THREAD3__GFX11,
    CPF_PERF_SEL_CP_SDMA_MNGR_SDMABUSY__GFX11,
    CPG_PERF_SEL_PFP_VGTDMA_DB_ROQ_DATA_STALL1__GFX11,
    DF_CS_PERF_SEL_CS_FTI_LAT_HIST_GT1000,
    DF_MALL_PERF_SEL_MALL_SDP_LAT_HIST_GT1000__GFX11,
    GCR_PERF_SEL_PIO_GL1_TLB_SHOOTDOWN_REQ__APU11,
    GCVML2_PERF_SEL_EVENT_90,
    GDS_PERF_SEL_SE7_GS_WAVE_ID_VALID__GFX11,
    ge1_rbiu_dr_fifo_starved_p1,
    ge_agm_gcr_combine__GFX11,
    ge_ngg_busy_base__GFX11,
    0,
    GL1A_PERF_SEL_CYCLE__GFX103PLUS,
    GL1C_PERF_SEL_UTCL0_UTCL1_XNACK_NO_RETRY_FAULT__GFX11,
    GL2A_PERF_SEL_RTN_CREDIT_STALL_CLIENT15__GFX11,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT19__GFX110,
    GRBM_PERF_SEL_PC_BUSY__GFX11,
    GRBM_SE0_PERF_SEL_PC_BUSY__GFX11,
    GRBM_SE1_PERF_SEL_PC_BUSY__GFX11,
    GRBM_SE2_PERF_SEL_PC_BUSY__GFX11,
    GRBM_SE3_PERF_SEL_PC_BUSY__GFX11,
    GRBM_SE4_PERF_SEL_PC_BUSY,
    GRBM_SE5_PERF_SEL_PC_BUSY,
    GRBM_SE6_PERF_SEL_PC_BUSY,
    GRBM_SE7_PERF_SEL_PC_BUSY,
    PC_PERF_GL1H_REQ_ROUTE_ID1__APU11,
    DB_PERF_SEL_OREO_Events_stalls__GFX11,
    PH_PERF_SC7_FIFO_STATUS_3__GFX11,
    RMI_PERF_SEL_CONSUMER_PROBEGEN_DB_RTS_RTR__GFX11,
    RLC_PERF_SEL_SERDES_COMMAND_WRITE,
    SC_VRC_REPROBE_FULL__APU11,
    SDMA_PERF_SEL_QUEUE7_SELECT__GFX11,
    SDMA_PERFMON_SEL_MMHUB_TAG_DELAY_COUNTER,
    SPI_PERF_BUSY__GFX11,
    SQ_PERF_SEL_NONE2__GFX11,
    SQG_PERF_SEL_DUMMY_LAST,
    PERF_PA_BUSY__GFX11,
    SX_PERF_SEL_DB3_4X2_DISCARD__GFX11,
    TA_PERF_SEL_tcreq_clk_valid_cycles__GFX103PLUS,
    TCP_PERF_SEL_BURST_BIN_READHIT_gt16__GFX110,
    TD_PERF_SEL_store_preempts_a_load__GFX110,
    UTCL1_PERF_SEL_ALOG_STALL_PMM_CREDITS__GFX11,
    UMC_PERF_SEL_ClockCount__GFX11,
};

constexpr unsigned int Stx1MaxPerfEventIds[MaxPerfCtrId] =
{
    CB_PERF_SEL_EXPORT_KILLED_BY_NULL_TARGET_SHADER_MASK__GFX11,
    CHA_PERF_SEL_CYCLE__APU11,
    CHC_PERF_SEL_REQ_CLIENT23__GFX11,
    0,
    CPC_PERF_SEL_MEC_THREAD3__GFX11,
    CPF_PERF_SEL_CP_SDMA_MNGR_SDMABUSY__GFX11,
    CPG_PERF_SEL_PFP_VGTDMA_DB_ROQ_DATA_STALL1__GFX11,
    DF_CS_PERF_SEL_CS_FTI_LAT_HIST_GT1000,
    DF_MALL_PERF_SEL_MALL_SDP_LAT_HIST_GT1000__GFX11,
    GCR_PERF_SEL_PIO_GL1_TLB_SHOOTDOWN_REQ__APU11,
    GCVML2_PERF_SEL_EVENT_90,
    GDS_PERF_SEL_SE7_GS_WAVE_ID_VALID__GFX11,
    ge1_rbiu_dr_fifo_starved_p1,
    ge_agm_gcr_combine__GFX11,
    ge_ngg_busy_base__GFX11,
    0,
    GL1A_PERF_SEL_CYCLE__GFX103PLUS,
    GL1C_PERF_SEL_UTCL0_UTCL1_XNACK_NO_RETRY_FAULT__GFX11,
    GL2A_PERF_SEL_RTN_CREDIT_STALL_CLIENT15__GFX11,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT19__STX1,
    GRBM_PERF_SEL_PC_BUSY__GFX11,
    GRBM_SE0_PERF_SEL_PC_BUSY__GFX11,
    GRBM_SE1_PERF_SEL_PC_BUSY__GFX11,
    GRBM_SE2_PERF_SEL_PC_BUSY__GFX11,
    GRBM_SE3_PERF_SEL_PC_BUSY__GFX11,
    GRBM_SE4_PERF_SEL_PC_BUSY,
    GRBM_SE5_PERF_SEL_PC_BUSY,
    GRBM_SE6_PERF_SEL_PC_BUSY,
    GRBM_SE7_PERF_SEL_PC_BUSY,
    PC_PERF_GL1H_REQ_ROUTE_ID1__APU11,
    DB_PERF_SEL_OREO_Events_stalls__GFX11,
    PH_PERF_SC7_FIFO_STATUS_3__GFX11,
    RMI_PERF_SEL_CONSUMER_PROBEGEN_DB_RTS_RTR__GFX11,
    RLC_PERF_SEL_SERDES_COMMAND_WRITE,
    SC_VRC_REPROBE_FULL__APU11,
    SDMA_PERF_SEL_QUEUE7_SELECT__GFX11,
    SDMA_PERFMON_SEL_MMHUB_TAG_DELAY_COUNTER,
    SPI_PERF_BUSY__GFX11,
    SQ_PERF_SEL_NONE2__GFX11,
    SQG_PERF_SEL_DUMMY_LAST,
    PERF_PA_BUSY__GFX11,
    SX_PERF_SEL_DB3_4X2_DISCARD__GFX11,
    TA_PERF_SEL_tcreq_clk_valid_cycles__GFX103PLUS,
    TCP_PERF_SEL_BURST_BIN_READHIT_gt16__STX1,
    TD_PERF_SEL_store_preempts_a_load__STX1,
    UTCL1_PERF_SEL_ALOG_STALL_PMM_CREDITS__GFX11,
    UMC_PERF_SEL_ClockCount__GFX11,
};

#if CHIP_HDR_STRIX_HALO
constexpr unsigned int StxHMaxPerfEventIds[MaxPerfCtrId] =
{
    CB_PERF_SEL_EXPORT_KILLED_BY_NULL_TARGET_SHADER_MASK__GFX11,
    CHA_PERF_SEL_CYCLE__APU11,
    CHC_PERF_SEL_REQ_CLIENT23__GFX11,
    0,
    CPC_PERF_SEL_MEC_THREAD3__GFX11,
    CPF_PERF_SEL_CP_SDMA_MNGR_SDMABUSY__GFX11,
    CPG_PERF_SEL_PFP_VGTDMA_DB_ROQ_DATA_STALL1__GFX11,
    DF_CS_PERF_SEL_CS_FTI_LAT_HIST_GT1000,
    DF_MALL_PERF_SEL_MALL_SDP_LAT_HIST_GT1000__GFX11,
    GCR_PERF_SEL_PIO_GL1_TLB_SHOOTDOWN_REQ__APU11,
    GCVML2_PERF_SEL_EVENT_90,
    GDS_PERF_SEL_SE7_GS_WAVE_ID_VALID__GFX11,
    ge1_rbiu_dr_fifo_starved_p1,
    ge_agm_gcr_combine__GFX11,
    ge_ngg_busy_base__GFX11,
    0,
    GL1A_PERF_SEL_CYCLE__GFX103PLUS,
    GL1C_PERF_SEL_UTCL0_UTCL1_XNACK_NO_RETRY_FAULT__GFX11,
    GL2A_PERF_SEL_RTN_CREDIT_STALL_CLIENT15__GFX11,
    GL2C_PERF_SEL_HIT_PASS_MISS_IN_CLIENT19__STXH,
    GRBM_PERF_SEL_PC_BUSY__GFX11,
    GRBM_SE0_PERF_SEL_PC_BUSY__GFX11,
    GRBM_SE1_PERF_SEL_PC_BUSY__GFX11,
    GRBM_SE2_PERF_SEL_PC_BUSY__GFX11,
    GRBM_SE3_PERF_SEL_PC_BUSY__GFX11,
    GRBM_SE4_PERF_SEL_PC_BUSY,
    GRBM_SE5_PERF_SEL_PC_BUSY,
    GRBM_SE6_PERF_SEL_PC_BUSY,
    GRBM_SE7_PERF_SEL_PC_BUSY,
    PC_PERF_GL1H_REQ_ROUTE_ID1__APU11,
    DB_PERF_dc_zpp_failed_samples_cnt__STXH,
    PH_PERF_SC7_FIFO_STATUS_3__GFX11,
    RMI_PERF_SEL_CONSUMER_PROBEGEN_DB_RTS_RTR__GFX11,
    RLC_PERF_SEL_SERDES_COMMAND_WRITE,
    SC_PBB_BATCH_BREAK_DUE_TO_NULLPRIM_LIMIT__STXH,
    SDMA_PERF_SEL_QUEUE7_SELECT__GFX11,
    SDMA_PERFMON_SEL_MMHUB_TAG_DELAY_COUNTER,
    SPI_PERF_BUSY__GFX11,
    SQ_PERF_SEL_NONE2__GFX11,
    SQG_PERF_SEL_DUMMY_LAST,
    PERF_PA_BUSY__GFX11,
    SX_PERF_SEL_DB3_4X2_DISCARD__GFX11,
    TA_PERF_SEL_tcreq_clk_valid_cycles__GFX103PLUS,
    TCP_PERF_SEL_XBAR_REQ_UNIQUE_SUM__STXH,
    TD_PERF_SEL_2x_sampler_op_with_both_quads_unlit__STXH,
    UTCL1_PERF_SEL_ALOG_STALL_PMM_CREDITS__GFX11,
    UMC_PERF_SEL_ClockCount__GFX11,
};
#endif

} // inline namespace Chip
} // namespace Gfx9
} // namespace Pal
